/*! 
 *  IDS Enterprise Components - v4.90.3
 *  Date: 2024-03-27T14:01:43.011Z
 *  Revision: 679a9f12b8e63632fba16aa5af7929e1ade33ca2
 *  
 *  
 *  Apache License
 *  Version 2.0, January 2004
 *  http://www.apache.org/licenses/
 *  
 *  TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *  
 *  1. Definitions.
 *  
 *  "License" shall mean the terms and conditions for use, reproduction,
 *  and distribution as defined by Sections 1 through 9 of this document.
 *  
 *  "Licensor" shall mean the copyright owner or entity authorized by
 *  the copyright owner that is granting the License.
 *  
 *  "Legal Entity" shall mean the union of the acting entity and all
 *  other entities that control, are controlled by, or are under common
 *  control with that entity. For the purposes of this definition,
 *  "control" means (i) the power, direct or indirect, to cause the
 *  direction or management of such entity, whether by contract or
 *  otherwise, or (ii) ownership of fifty percent (50%) or more of the
 *  outstanding shares, or (iii) beneficial ownership of such entity.
 *  
 *  "You" (or "Your") shall mean an individual or Legal Entity
 *  exercising permissions granted by this License.
 *  
 *  "Source" form shall mean the preferred form for making modifications,
 *  including but not limited to software source code, documentation
 *  source, and configuration files.
 *  
 *  "Object" form shall mean any form resulting from mechanical
 *  transformation or translation of a Source form, including but
 *  not limited to compiled object code, generated documentation,
 *  and conversions to other media types.
 *  
 *  "Work" shall mean the work of authorship, whether in Source or
 *  Object form, made available under the License, as indicated by a
 *  copyright notice that is included in or attached to the work
 *  (an example is provided in the Appendix below).
 *  
 *  "Derivative Works" shall mean any work, whether in Source or Object
 *  form, that is based on (or derived from) the Work and for which the
 *  editorial revisions, annotations, elaborations, or other modifications
 *  represent, as a whole, an original work of authorship. For the purposes
 *  of this License, Derivative Works shall not include works that remain
 *  separable from, or merely link (or bind by name) to the interfaces of,
 *  the Work and Derivative Works thereof.
 *  
 *  "Contribution" shall mean any work of authorship, including
 *  the original version of the Work and any modifications or additions
 *  to that Work or Derivative Works thereof, that is intentionally
 *  submitted to Licensor for inclusion in the Work by the copyright owner
 *  or by an individual or Legal Entity authorized to submit on behalf of
 *  the copyright owner. For the purposes of this definition, "submitted"
 *  means any form of electronic, verbal, or written communication sent
 *  to the Licensor or its representatives, including but not limited to
 *  communication on electronic mailing lists, source code control systems,
 *  and issue tracking systems that are managed by, or on behalf of, the
 *  Licensor for the purpose of discussing and improving the Work, but
 *  excluding communication that is conspicuously marked or otherwise
 *  designated in writing by the copyright owner as "Not a Contribution."
 *  
 *  "Contributor" shall mean Licensor and any individual or Legal Entity
 *  on behalf of whom a Contribution has been received by Licensor and
 *  subsequently incorporated within the Work.
 *  
 *  2. Grant of Copyright License. Subject to the terms and conditions of
 *  this License, each Contributor hereby grants to You a perpetual,
 *  worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *  copyright license to reproduce, prepare Derivative Works of,
 *  publicly display, publicly perform, sublicense, and distribute the
 *  Work and such Derivative Works in Source or Object form.
 *  
 *  3. Grant of Patent License. Subject to the terms and conditions of
 *  this License, each Contributor hereby grants to You a perpetual,
 *  worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *  (except as stated in this section) patent license to make, have made,
 *  use, offer to sell, sell, import, and otherwise transfer the Work,
 *  where such license applies only to those patent claims licensable
 *  by such Contributor that are necessarily infringed by their
 *  Contribution(s) alone or by combination of their Contribution(s)
 *  with the Work to which such Contribution(s) was submitted. If You
 *  institute patent litigation against any entity (including a
 *  cross-claim or counterclaim in a lawsuit) alleging that the Work
 *  or a Contribution incorporated within the Work constitutes direct
 *  or contributory patent infringement, then any patent licenses
 *  granted to You under this License for that Work shall terminate
 *  as of the date such litigation is filed.
 *  
 *  4. Redistribution. You may reproduce and distribute copies of the
 *  Work or Derivative Works thereof in any medium, with or without
 *  modifications, and in Source or Object form, provided that You
 *  meet the following conditions:
 *  
 *  (a) You must give any other recipients of the Work or
 *  Derivative Works a copy of this License; and
 *  
 *  (b) You must cause any modified files to carry prominent notices
 *  stating that You changed the files; and
 *  
 *  (c) You must retain, in the Source form of any Derivative Works
 *  that You distribute, all copyright, patent, trademark, and
 *  attribution notices from the Source form of the Work,
 *  excluding those notices that do not pertain to any part of
 *  the Derivative Works; and
 *  
 *  (d) If the Work includes a "NOTICE" text file as part of its
 *  distribution, then any Derivative Works that You distribute must
 *  include a readable copy of the attribution notices contained
 *  within such NOTICE file, excluding those notices that do not
 *  pertain to any part of the Derivative Works, in at least one
 *  of the following places: within a NOTICE text file distributed
 *  as part of the Derivative Works; within the Source form or
 *  documentation, if provided along with the Derivative Works; or,
 *  within a display generated by the Derivative Works, if and
 *  wherever such third-party notices normally appear. The contents
 *  of the NOTICE file are for informational purposes only and
 *  do not modify the License. You may add Your own attribution
 *  notices within Derivative Works that You distribute, alongside
 *  or as an addendum to the NOTICE text from the Work, provided
 *  that such additional attribution notices cannot be construed
 *  as modifying the License.
 *  
 *  You may add Your own copyright statement to Your modifications and
 *  may provide additional or different license terms and conditions
 *  for use, reproduction, or distribution of Your modifications, or
 *  for any such Derivative Works as a whole, provided Your use,
 *  reproduction, and distribution of the Work otherwise complies with
 *  the conditions stated in this License.
 *  
 *  5. Submission of Contributions. Unless You explicitly state otherwise,
 *  any Contribution intentionally submitted for inclusion in the Work
 *  by You to the Licensor shall be under the terms and conditions of
 *  this License, without any additional terms or conditions.
 *  Notwithstanding the above, nothing herein shall supersede or modify
 *  the terms of any separate license agreement you may have executed
 *  with Licensor regarding such Contributions.
 *  
 *  6. Trademarks. This License does not grant permission to use the trade
 *  names, trademarks, service marks, or product names of the Licensor,
 *  except as required for reasonable and customary use in describing the
 *  origin of the Work and reproducing the content of the NOTICE file.
 *  
 *  7. Disclaimer of Warranty. Unless required by applicable law or
 *  agreed to in writing, Licensor provides the Work (and each
 *  Contributor provides its Contributions) on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *  implied, including, without limitation, any warranties or conditions
 *  of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 *  PARTICULAR PURPOSE. You are solely responsible for determining the
 *  appropriateness of using or redistributing the Work and assume any
 *  risks associated with Your exercise of permissions under this License.
 *  
 *  8. Limitation of Liability. In no event and under no legal theory,
 *  whether in tort (including negligence), contract, or otherwise,
 *  unless required by applicable law (such as deliberate and grossly
 *  negligent acts) or agreed to in writing, shall any Contributor be
 *  liable to You for damages, including any direct, indirect, special,
 *  incidental, or consequential damages of any character arising as a
 *  result of this License or out of the use or inability to use the
 *  Work (including but not limited to damages for loss of goodwill,
 *  work stoppage, computer failure or malfunction, or any and all
 *  other commercial damages or losses), even if such Contributor
 *  has been advised of the possibility of such damages.
 *  
 *  9. Accepting Warranty or Additional Liability. While redistributing
 *  the Work or Derivative Works thereof, You may choose to offer,
 *  and charge a fee for, acceptance of support, warranty, indemnity,
 *  or other liability obligations and/or rights consistent with this
 *  License. However, in accepting such obligations, You may act only
 *  on Your own behalf and on Your sole responsibility, not on behalf
 *  of any other Contributor, and only if You agree to indemnify,
 *  defend, and hold each Contributor harmless for any liability
 *  incurred by, or claims asserted against, such Contributor by reason
 *  of your accepting any such warranty or additional liability.
 *  
 *  END OF TERMS AND CONDITIONS
 *  
 *  APPENDIX: How to apply the Apache License to your work.
 *  
 *  To apply the Apache License to your work, attach the following
 *  boilerplate notice, with the fields enclosed by brackets "[]"
 *  replaced with your own identifying information. (Don't include
 *  the brackets!)  The text should be enclosed in the appropriate
 *  comment syntax for the file format. We also recommend that a
 *  file or class name and description of purpose be included on the
 *  same "printed page" as the copyright notice for easier
 *  identification within third-party archives.
 *  
 *  Copyright 2018  Infor, Inc.
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  
 */
var Soho=function(t){"use strict";function e(t){}function i(t){}var n=Object.freeze({__proto__:null,enableConsoleLogging:!1,enableTimeLogging:!1,getComponents:function(t){var e=(t?$(t):$("body")).find("*"),i=[];return e.each((function(t,e){var n=$(e).data();n&&Object.keys(n).length&&i.push({data:n,element:e})})),i},log:function(t,e){},logTimeEnd:i,logTimeStart:e});Element.prototype.matches||(Element.prototype.matches=Element.prototype.matchesSelector||Element.prototype.mozMatchesSelector||Element.prototype.msMatchesSelector||Element.prototype.oMatchesSelector||Element.prototype.webkitMatchesSelector||function(t){for(var e=(this.document||this.ownerDocument).querySelectorAll(t),i=e.length;--i>=0&&e.item(i)!==this;);return i>-1}),[Element.prototype,CharacterData.prototype,DocumentType.prototype].forEach((function(t){t.hasOwnProperty("remove")||Object.defineProperty(t,"remove",{configurable:!0,enumerable:!0,writable:!0,value:function(){null!==this.parentNode&&this.parentNode.removeChild(this)}})})),!!window.MSInputMethodContext&&!!document.documentMode||(jQuery.event.special.touchstart={setup:function(t,e,i){e.includes("noPreventDefault")?this.addEventListener("touchstart",i,{passive:!1}):this.addEventListener("touchstart",i,{passive:!0})}},jQuery.event.special.touchmove={setup:function(t,e,i){this.addEventListener("touchmove",i,{passive:!0})}},jQuery.event.special.mousewheel={setup:function(t,e,i){this.addEventListener("mousewheel",i,{passive:!0})}});var s="4.90.3",a=s;function o(t,e){var i=Object.keys(t);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(t);e&&(n=n.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),i.push.apply(i,n)}return i}function r(t){for(var e=1;e<arguments.length;e++){var i=null!=arguments[e]?arguments[e]:{};e%2?o(Object(i),!0).forEach((function(e){c(t,e,i[e])})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(i)):o(Object(i)).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(i,e))}))}return t}function l(t){return l="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},l(t)}function c(t,e,i){return(e=function(t){var e=function(t,e){if("object"!=typeof t||null===t)return t;var i=t[Symbol.toPrimitive];if(void 0!==i){var n=i.call(t,e||"default");if("object"!=typeof n)return n;throw new TypeError("@@toPrimitive must return a primitive value.")}return("string"===e?String:Number)(t)}(t,"string");return"symbol"==typeof e?e:String(e)}(e))in t?Object.defineProperty(t,e,{value:i,enumerable:!0,configurable:!0,writable:!0}):t[e]=i,t}function d(t,e){return d=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(t,e){return t.__proto__=e,t},d(t,e)}function h(t,e,i){return h=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],(function(){}))),!0}catch(t){return!1}}()?Reflect.construct.bind():function(t,e,i){var n=[null];n.push.apply(n,e);var s=new(Function.bind.apply(t,n));return i&&d(s,i.prototype),s},h.apply(null,arguments)}function u(t){return function(t){if(Array.isArray(t))return p(t)}(t)||function(t){if("undefined"!=typeof Symbol&&null!=t[Symbol.iterator]||null!=t["@@iterator"])return Array.from(t)}(t)||function(t,e){if(!t)return;if("string"==typeof t)return p(t,e);var i=Object.prototype.toString.call(t).slice(8,-1);"Object"===i&&t.constructor&&(i=t.constructor.name);if("Map"===i||"Set"===i)return Array.from(t);if("Arguments"===i||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i))return p(t,e)}(t)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}function p(t,e){(null==e||e>t.length)&&(e=t.length);for(var i=0,n=new Array(e);i<e;i++)n[i]=t[i];return n}var g={stripHTML:function(t){var e=t;return e?e=e.replace(/<\/?[^>]+(>|$)/g,""):""},stripTags:function(t,e){if(!t)return"";if("number"==typeof t)return t;var i=("".concat(e||"").toLowerCase().match(/<[a-z][a-z0-9]*>/g)||[]).join(""),n=/<\/?([a-z][a-z0-9]*)\b[^>]*>/gi;return t.replace(/<!--[\s\S]*?-->|<\?(?:php)?[\s\S]*?\?>/gi,"").replace(n,(function(t,e){return i.indexOf("<"+e.toLowerCase()+">")>-1?t:""})).replace(n,(function(t,e){return i.indexOf("<"+e.toLowerCase()+">")>-1?t:""})).replace(n,(function(t,e){return i.indexOf("<"+e.toLowerCase()+">")>-1?t:""}))},sanitizeConsoleMethods:function(t){var e=new RegExp("console\\.(".concat(["assert","clear","count","debug","dirxml","dir","error","exception","groupCollapsed","groupEnd","group","info","log","markTimeline","profileEnd","profile","table","timeEnd","timeStamp","time","trace","warn"].join("|"),")((\\s+)?\\(([^)]+)\\);?)?"),"igm");return"string"!=typeof t?t:t.replace(e,"")},sanitizeHTML:function(t){if(!t)return"";var e=t.replace(/\bon\w+=\S+?(?=(>|&|<| |"))/g,"");return e=(e=(e=e.replace(/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/g,"")).replace(/<iframe\b[^<]*(?:(?!<\/iframe>)<[^<]*)*<\/iframe>/gi,"")).replace(/<iframe\b[^<]*(?:(?!>)<[^<]*)*>/gi,""),e=(e=this.sanitizeConsoleMethods(e)).replace(/<\/script>/g,"")},removeWhiteSpaceCharacters:function(t){if(!t)return"";var e=t[0].innerHTML;return e=e.replace(/\s+/g," ").trim(),t[0].innerHTML=e,t[0].innerHTML},ensureAlphaNumeric:function(t){return"number"==typeof t?t:this.stripTags(t).replace(/[^a-z0-9-]/gi,"","")},ensureAlphaNumericWithSpaces:function(t){return"number"==typeof t?t:this.stripTags(t).replace(/[^a-z0-9 ]/gi,"","")},toCamelCase:function(t){return"string"!=typeof t?t:(t=this.stripTags(t).replace(/[^a-z0-9 ]/gi,"","")).toLowerCase().replace(/[^a-zA-Z0-9]+(.)/g,(function(t,e){return e.toUpperCase()}))},escapeHTML:function(t){var e=t;if("string"==typeof e){var i={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&apos;","\\":"&bsol;"};return e.replace(/[&<>"'\\]/gi,(function(t){return i[t]}))}return e},unescapeHTML:function(t){if(""===t)return"";if("string"==typeof t){var e=function(e){return(t.match(e)||[""])[0]},i=(new DOMParser).parseFromString(t,"text/html");return"".concat(e(/^\s*|\\/)).concat(i.documentElement.textContent.trim()).concat(e(/\s*$|\\/))}return t},htmlEntities:function(t){return String(t).replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/\\/g,"&bsol;").replace(/"/g,"&quot;")},isUrlLocal:function(t){return!(""===t)&&("/"===t[0]&&(1===t.length||"/"!==t[1]&&"\\"!==t[1])||t.length>1&&"~"===t[0]&&"/"===t[1])||t.length>=1&&"#"===t[0]}},f={isValidElement:function(t){return t instanceof HTMLElement||t instanceof SVGElement},getAttributes:function(t){return t&&(t instanceof HTMLElement||t instanceof SVGElement)?t.attributes:{}},hasAnyClass:function(t){var e=t.className;return e&&e.length>0},hasClass:function(t,e){var i=!1;if(null==t||!(null==t?void 0:t.getAttribute))return i;if("classList"in t)i=t.classList.contains(e);else{var n=t.getAttribute("class");i=!!n&&-1!==n.split(/\s+/).indexOf(e)}return i},addClass:function(t){if(t){for(var e="",i=arguments.length,n=new Array(i>1?i-1:0),s=1;s<i;s++)n[s-1]=arguments[s];for(var a=0;a<n.length;a++)t.classList?t.classList.add(n[a]):f.hasClass(t,[a])||(e.length&&(e+=" "),e+=n[a]);e.length&&$(t).addClass(e)}},removeClass:function(t){if(t){for(var e="",i=arguments.length,n=new Array(i>1?i-1:0),s=1;s<i;s++)n[s-1]=arguments[s];for(var a=0;a<n.length;a++)t.classList?t.classList.remove(n[a]):f.hasClass(t,[a])||(e.length&&(e+=" "),e+=n[a]);e.length&&$(t).removeClass(e)}},isElement:function(t){return!!(t instanceof HTMLElement||t instanceof SVGElement||t instanceof $&&t.length)},getDimensions:function(t){if(!f.isElement(t))return{};if(t instanceof $){if(!t.length)return{};t=t[0]}var e=t.getBoundingClientRect(),i={};for(var n in e)isNaN(e[n])||(i[n]=e[n]);return i},prepend:function(t,e,i){var n=t;t instanceof $&&t.length&&(n=n[0]),(n instanceof HTMLElement||n instanceof SVGElement)&&n.insertAdjacentHTML("afterbegin",this.xssClean(e,i))},append:function(t,e,i){var n=t;t instanceof $&&t.length&&(n=n[0]),(n instanceof HTMLElement||n instanceof SVGElement)&&n.insertAdjacentHTML("beforeend",this.xssClean(e,i))},after:function(t,e,i){var n=t;t instanceof $&&t.length&&(n=n[0]),(n instanceof HTMLElement||n instanceof SVGElement)&&n.insertAdjacentHTML("afterend",this.xssClean(e,i))},remove:function(t){var e=t;t instanceof $&&t.length&&(e=e[0]),(e instanceof HTMLElement||e instanceof SVGElement)&&t.parentNode&&t.parentNode.removeChild(t)},setAttribute:function(t,e,i){var n=t;t instanceof $&&t.length&&(n=n[0]),(n instanceof HTMLElement||n instanceof SVGElement)&&n.setAttribute("attribute",i)},xssClean:function(t,e){var i=t;return e&&"*"!==e&&(i=g.stripTags(t,e)),"*"===e&&(i=g.sanitizeHTML(t)),i},html:function(t,e,i){var n=t;t instanceof $&&t.length&&(n=n[0]),(n instanceof HTMLElement||n instanceof SVGElement)&&(n.innerHTML=this.xssClean(e,i))},replaceHtml:function(t,e,i){var n=t,s=e;t instanceof $&&t.length&&(n=n[0]),e instanceof $&&t.length&&(s=e[0]),(n instanceof HTMLElement||n instanceof SVGElement)&&n.replaceChildren(this.xssClean(s,i))},parents:function(t,e,i){var n,s=[];if(!f.isValidElement(t))return s;for(;t.parentNode&&(!i||!s.length);)t=t.parentNode,(n=t)!==document&&n.matches(e)&&s.push(n);return i?s[0]:s},getNextSibling:function(t,e){t instanceof $&&t.length&&(t=t[0]);var i=t.nextElementSibling;if(!e)return i;for(;i;){if(i.matches(e))return i;i=i.nextElementSibling}},getPreviousSibling:function(t,e){t instanceof $&&t.length&&(t=t[0]);var i=t.previousElementSibling;if(!e)return i;for(;i;){if(i.matches(e))return i;i=i.previousElementSibling}},getSiblings:function(t){return t instanceof $&&t.length&&(t=t[0]),[].slice.call(t.parentNode.children).filter((function(e){return e!==t}))},getSimpleSelector:function(t){var e=t.tagName.toLowerCase(),i=t.id?"#".concat(t.id):"",n=t.className?".".concat(t.className.split(" ").join(".")):"";return"".concat(e).concat(i).concat(n)}},m={phone:320,slim:400,phablet:610,"phone-to-tablet":767,"wide-tablet":968,"tablet-to-desktop":1280,desktop:1024,"desktop-to-extralarge":1600},v=Object.keys(m);function b(t,e,i){var n;return function(){for(var s=arguments.length,a=new Array(s),o=0;o<s;o++)a[o]=arguments[o];var r=this;n?clearTimeout(n):i&&t.apply(r,a),n=setTimeout((function(){i||t.apply(r,a),n=null}),e||250)}}m.available=v,m.current=function(){if(!window.getComputedStyle)return"";var t=!!window.getComputedStyle&&window.getComputedStyle(document.body,":after");return t?(t.getPropertyValue?t.getPropertyValue("content"):"").replace(/"/g,""):""},m.isAbove=function(t){var e=m[t];return!!e&&$(window).width()>e-1},m.isBelow=function(t){var e=m[t];return!!e&&$(window).width()<e},m.compare=function(){this.last||(this.last="");var t=this.current();this.last!==t&&($("body").triggerHandler("breakpoint-change",[{previous:this.last,current:t}]),this.last=t)},m.isHidden=function(t){if(!t||!f.isElement(t))return!1;var e=t.classList;if(!e.length)return!1;if(e.contains("hidden"))return!0;var i={phonedown:"xs",phone:"sm",tablet:"md",desktop:"lg",extralarge:"xl"}[this.current()],n="hidden-".concat(i),s="visible-".concat(i,"-");return!!e.contains(n)||(e.toString().indexOf(s),!1)},$.fn.isHiddenAtBreakpoint=function(){return!!this.length&&m.isHidden($(this).first()[0])};var y="debouncedResize";$.fn[y]=function(t){return t?this.bind("resize",b(t)):this.trigger(y)};var w="environment",x={browser:{},features:{resizeObserver:"undefined"!=typeof ResizeObserver,touch:"ontouchstart"in window||navigator.MaxTouchPoints>0||navigator.msMaxTouchPoints>0},os:{},devicespecs:{},get rtl(){return"rtl"===$("html").attr("dir")},set:function(){$("html").attr("data-sohoxi-version",a),this.viewport=document.querySelector("meta[name=viewport]"),this.viewport&&this.viewport.setAttribute("content","width=device-width, initial-scale=1.0, user-scalable=0"),this.addBrowserClasses(),this.addGlobalResize(),this.addDeviceSpecs()},addBrowserClasses:function(){var t=navigator.userAgent||navigator.vendor||window.opera,e=navigator.platform,i=$("html"),n="";if(-1!==t.indexOf("Safari")&&-1===t.indexOf("Chrome")&&-1===t.indexOf("Android")&&(n+="is-safari ",this.browser.name="safari"),this.browser.isWKWebView=function(){return!1},"iP"===navigator.platform.substr(0,2)||x.browser.isIPad()){var s=/constructor/i.test(window.HTMLElement),a=!!window.indexedDB;(window.webkit&&window.webkit.messageHandlers||!s||a)&&(this.browser.name="wkwebview",n+="is-safari is-wkwebview ",this.browser.isWKWebView=function(){return!0})}if(-1!==t.indexOf("Chrome")&&(n+="is-chrome ",this.browser.name="chrome"),t.indexOf("Firefox")>0&&(n+="is-firefox ",this.browser.name="firefox"),t.match(/Edg\//)&&(n+="ie-edge ",this.browser.name="edge",this.browser.version=navigator.appVersion.indexOf("Edge/18")>-1?"18":"17",n+="ie-edge".concat(this.browser.version)),t.match(/Trident/)&&(n+="ie ",this.browser.name="ie"),(navigator.appVersion.indexOf("MSIE 8.0")>-1||t.indexOf("MSIE 8.0")>-1||8===document.documentMode)&&(n+="ie8 ",this.browser.version="8"),navigator.appVersion.indexOf("MSIE 9.0")>-1&&(n+="ie9 ",this.browser.version="9"),navigator.appVersion.indexOf("MSIE 10.0")>-1?(n+="ie10 ",this.browser.version="10"):t.match(/Trident\/7\./)&&(n+="ie11 ",this.browser.version="11"),/iPhone|iPod|iPad/.test(t)||x.browser.isIPad()){n+="ios ",this.os.name="ios";for(var o=["iPod","iPad","iPhone"],r=0;r<o.length;r++)new RegExp(o[r]).test(t)&&(n+="".concat(o[r].toLowerCase()," "),this.device=o[r])}["Macintosh","MacIntel","MacPPC","Mac68K"].indexOf(e)>-1&&!/Linux/.test(e)&&(n=n.replace("ios ",""),n+="is-mac ",this.os.name="mac"),/Android/.test(t)&&(n+="android ",this.os.name="android"),!this.os.name&&/Linux/.test(e)&&(this.os.name="linux"),i.addClass(n)},addDeviceSpecs:function(){var t,e,i,n=navigator.appVersion,s=navigator.userAgent,a=navigator.appName,o=" ".concat(parseFloat(navigator.appVersion)),r=parseInt(navigator.appVersion,10),l="";-1!==(e=s.indexOf("Opera"))&&(a="Opera",o=s.substring(e+6),-1!==(e=s.indexOf("Version"))&&(o=s.substring(e+8))),-1!==(e=s.indexOf("OPR"))?(a="Opera",o=s.substring(e+4)):-1!==(e=s.indexOf("Edg"))?(a="Microsoft Edge",o=s.substring(e+4)):-1!==(e=s.indexOf("MSIE"))?(a="Microsoft Internet Explorer",o=s.substring(e+5)):-1!==(e=s.indexOf("Chrome"))?(a="Chrome",o=s.substring(e+7),s.indexOf("Edg")>-1&&(l="Microsoft Edge"),Number(o.substring(0,3))>=108&&(o=o.substring(0,3))):-1!==(e=s.indexOf("Safari"))?(a="Safari",o=s.substring(e+7),-1!==(e=s.indexOf("Version"))&&(o=s.substring(e+8))):this.browser.isWKWebView()?(a="WKWebView",o="",r=""):-1!==(e=s.indexOf("Firefox"))?(a="Firefox",o=s.substring(e+8)):-1!==s.indexOf("Trident/")?(a="Microsoft Internet Explorer",o=s.substring(s.indexOf("rv:")+3)):(t=s.lastIndexOf(" ")+1)<(e=s.lastIndexOf("/"))&&(a=s.substring(t,e),o=s.substring(e+1),a.toLowerCase()===a.toUpperCase()&&(a=navigator.appName)),-1!==(i=o.indexOf(";"))&&(o=o.substring(0,i)),-1!==(i=o.indexOf(" "))&&(o=o.substring(0,i)),-1!==(i=o.indexOf(")"))&&(o=o.substring(0,i)),r=" ".concat(parseInt(o,10)),isNaN(r)&&(o=" ".concat(parseFloat(navigator.appVersion)),r=parseInt(navigator.appVersion,10));var c=/Mobile|mini|Fennec|Android|iP(ad|od|hone)/.test(n),d="-",h=[{s:"Windows 10",r:/(Windows 10.0|Windows NT 10.0)/},{s:"Windows 8.1",r:/(Windows 8.1|Windows NT 6.3)/},{s:"Windows 8",r:/(Windows 8|Windows NT 6.2)/},{s:"Windows 7",r:/(Windows 7|Windows NT 6.1)/},{s:"Android",r:/Android/},{s:"Open BSD",r:/OpenBSD/},{s:"Sun OS",r:/SunOS/},{s:"Linux",r:/(Linux|X11)/},{s:"iOS",r:/(iPhone|iPad|iPod)/},{s:"Mac OS X",r:/Mac OS X/},{s:"Mac OS",r:/(MacPPC|MacIntel|Mac_PowerPC|Macintosh)/},{s:"UNIX",r:/UNIX/}];for(var u in h){var p=h[u];if(p.r.test(s)){d=p.s;break}}var g="-";switch(/Windows/.test(d)&&(g=/Windows (.*)/.exec(d)[1]),d){case"Windows 10":g="",d="Windows";break;case"Mac OS X":g="",d="Mac OS";break;case"Android":g=/Android ([\.\_\d]+)/.exec(s)[1];break;case"iOS":g=/OS (\d+)_?(\d+)?/.exec(s),g="".concat(g[1],".").concat(g[2],".").concat(0|g[3])}x.browser.isIPad()&&(g=s.substr(s.indexOf("Version"),s.substr(s.indexOf("Version")).indexOf(" ")).replace("Version/",""),d="IOS");this.devicespecs={currentBrowser:a,browserVersion:o.trim(),browserMajorVersion:r,isMobile:c||x.browser.isIPad(),os:d,currentOSVersion:g,browserVersionName:l}},addGlobalResize:function(){$(window).debouncedResize((function(){$("body").triggerHandler("resize",[window]),m.compare()})),$(window).on("orientationchange load",(function(){return m.compare()}))},removeGlobalEvents:function(){$(window).off("scroll.".concat(w)),$("body").off(["focusin.".concat(w),"focusout.".concat(w)].join(" "))}};function C(t){return this.init(t)}function k(t,e,i){var n=$.Deferred();if(!f.isElement(t))return n.reject();if(t instanceof $&&t.length&&(t=t[0]),null==e)return n.resolve();if(isNaN(i)&&(i=0),e=Math.round(e),(i=Math.round(i))<0)return n.fail();if(0===i)return t.scrollLeft+=e,n.resolve();var s=Date.now(),a=s+i,o=t.scrollLeft,r=e;var l=t.scrollLeft;return setTimeout((function e(){if(t.scrollLeft===l){var i=Date.now(),c=function(t,e,i){if(i<=t)return 0;if(i>=e)return 1;var n=(i-t)/(e-t);return n*n*(3-2*n)}(s,a,i),d=Math.round(o+r*c);t.scrollLeft=d,i>=a?n.resolve():t.scrollLeft!==l||t.scrollLeft===d?(l=t.scrollLeft,setTimeout(e,0)):n.resolve()}else n.reject()}),0),n}function T(t,e){return(void 0!==window.requestAnimationFrame?window.requestAnimationFrame:setTimeout)(t,e)}x.browser.isEdge=function(){return"edge"===x.browser.name},x.browser.isIE11=function(){return"ie"===x.browser.name&&"11"===x.browser.version},x.browser.isSafari=function(){return"safari"===x.browser.name||"wkwebview"===x.browser.name},x.browser.isIE10=function(){return"ie"===x.browser.name&&"10"===x.browser.version},x.browser.isFirefox=function(){return"firefox"===x.browser.name},x.browser.isIPad=function(){return!!(navigator.userAgent.match(/(iPad)/)||"MacIntel"===navigator.platform&&void 0!==navigator.standalone)},x.set(),C.prototype={init:function(t){!this.element&&(t instanceof HTMLElement||t instanceof SVGElement)&&(this.element=t);var e=$(this.element),i=!1,n=!1,s=!1;if(e.is(".checkbox, .radio, .switch")){var a=e.next();a.is('[type="hidden"]')&&(a=a.next()),this.label=a[0],e.addClass("hide-focus").on("focusin.hide-focus",(function(t){i||n||s||(e.removeClass("hide-focus"),e.triggerHandler("hidefocusremove",[t])),i=!1,n=!0,s=!1})).on("focusout.hide-focus",(function(t){e.addClass("hide-focus"),s=a.is(s),i=!1,n=!1,e.triggerHandler("hidefocusadd",[t])})),a.on("mousedown.hide-focus",(function(t){s=this,i=!0,e.addClass("hide-focus"),e.triggerHandler("hidefocusadd",[t])}))}else{var o=function(t){i=!0,e.addClass("hide-focus"),e.triggerHandler("hidefocusadd",[t])},r=x.features.touch,l=e;e.hasClass("accordion-header")&&(l=e.find("a")),l.on("mousedown.hide-focus",(function(t){o(t)})),r&&l.on("touchstart.hide-focus",(function(t){o(t)})),e.addClass("hide-focus"),e.on("focusin.hide-focus",(function(t){i||(e.removeClass("hide-focus"),e.triggerHandler("hidefocusremove",[t])),i=!1,n=!0})).on("focusout.hide-focus",(function(t){e.addClass("hide-focus"),i=!1,n=!1,e.triggerHandler("hidefocusadd",[t])})),e.is(l)||(this.separateEventTarget=l)}return this},updated:function(){return this.teardown().init()},teardown:function(){var t,e;this.label&&$(this.label).off("mousedown.hide-focus");var i=["focusin.hide-focus","focusout.hide-focus","mousedown.hide-focus","touchstart.hide-focus"].join(" ");return $(this.element).off(i),null!==(t=this.separateEventTarget)&&void 0!==t&&(null==t?void 0:t.length)&&(this.separateEventTarget.off(i),this.separateEventTarget=null),null===(e=this.element)||void 0===e||null==e||e.classList.remove("hide-focus"),this},destroy:function(){this.teardown(),$.removeData(this.element,"hidefocus")}},$.fn.hideFocus=function(){return this.each((function(){var t=$.data(this,"hidefocus");t?t.updated():t=$.data(this,"hidefocus",new C(this))}))},$.fn.smoothScroll=function(t,e){return k(this,t,e)},$.fn.bindFirst=function(t,e){this.on(t,e),this.each((function(){var e=$._data(this,"events")[t.split(".")[0]],i=e.pop();e.splice(0,0,i)}))};var A=[];function S(t){return $.expr.filters.visible(t)&&!$(t).parents().addBack().filter((function(){return"hidden"===$.css(this,"visibility")})).length}$.fn.cssPropSupport=function(t){if(!t)return null;var e=$("<div></div>")[0],i=t.toString(),n=["Moz","Webkit","O","ms"],s=i.charAt(0).toUpperCase()+i.substr(1);if(t in e.style)return $(e).remove(),t;for(var a=0;a<n.length;a++){var o=n[a]+s;if(o in e.style)return $(e).remove(),o}return $(e).remove(),null},$.fn.transitionEndName=function(){return{WebkitTransition:"webkitTransitionEnd",MozTransition:"transitionend",MSTransition:"msTransitionEnd",OTransition:"oTransitionEnd",transition:"transitionend"}[$.fn.cssPropSupport("transition")]||null},$.extend($.expr[":"],{focusable:function(t){return function(t){var e,i,n,s=t.nodeName.toLowerCase(),a=!isNaN($.attr(t,"tabindex"));return"area"===s?(i=(e=t.parentNode).name,!(!t.href||!i||"map"!==e.nodeName.toLowerCase())&&!!(n=$("img[usemap=#".concat(i,"]"))[0])&&S(n)):!!S(t)&&(/input|select|textarea|button|object/.test(s)?!t.disabled:"a"===s&&void 0!==t.href||a)}(t,isNaN($.attr(t,"tabindex")))}}),$.fn.listEvents=function(){var t={};return this.each((function(){t=$._data(this,"events")})),t};var E={};function D(t){return"object"===l(t)||"function"==typeof t}function I(t){return"function"==typeof t?t():t}E.uniqueId=function(t,e,i,n){var s=t.id;if(s&&$("#".concat(s)).length<2)return s;i=i?"".concat(i,"-"):"",n=n?"-".concat(n):"",e=e||E.getArrayFromList(t.classList).join("-"),A[e]||(A[e]=1);var a="".concat(i).concat(e,"-").concat(A[e]).concat(n);return A[e]+=1,a},E.parseSettings=function(t,e){var i={};if(!t||!(t instanceof HTMLElement)&&!(t instanceof $)||t instanceof $&&!t.length)return i;t instanceof $&&(t=t[0]),e=e||"data-options";var n,s,a=t.getAttribute(e);if(!a||"string"!=typeof a||-1===a.indexOf("{"))return i;function o(t){return t.replace(/'/g,'"')}try{i=JSON.parse(o(a))}catch(t){i=JSON.parse((s=/({|,)(?:\s*)(?:')?([A-Za-z_$\.][A-Za-z0-9_ \-\.$]*)(?:')?(?:\s*):/g,n=(n=a).replace(s,'$1"$2":'),s=/:(?:\s*)(?!(true|false|null|undefined))([A-Za-z_$\.#][A-Za-z0-9_ \-\.$]*)/g,o(n=n.replace(s,':"$2"'))))}return i},E.parseOptions=function(t,e){return E.parseSettings(t,e)},$.fn.parseOptions=function(t,e){var i=[],n=this;return t instanceof HTMLElement||t instanceof SVGElement||t instanceof $?n=$(t):(e=t,t=void 0),n.each((function(t,n){i.push({element:this,options:E.parseOptions(n,e)})})),1===i.length?i[0].options:i},E.coerceToBoolean=function(t){return!/^(false|0)$/i.test(t)&&!!t},E.coerceSettingsToBoolean=function(t,e){var i,n;for(e&&Array.isArray(e)||Object.keys(t).forEach((function(t){e.push(t)})),i=0,n=e.length;i<n;i++)t[e[i]]=E.coerceToBoolean(t[e[i]]);return t},$.fn.timer=function(t,e){var i,n=$(this),s=10,a=0;function o(){i=setInterval((function(){a+=s,n.triggerHandler("update",[{counter:a}]),a>e&&(n.triggerHandler("timeout"),t.apply(arguments),clearInterval(i),a=0)}),s)}return o(),{event:this,cancel:function(){n.triggerHandler("cancel"),clearInterval(i),a=0},pause:function(){n.triggerHandler("pause"),clearInterval(i)},resume:function(){n.triggerHandler("resume"),o()}}},$.copyToClipboard=function(t){if(window.clipboardData&&window.clipboardData.setData)return window.clipboardData.setData("Text",t);if(document.queryCommandSupported&&document.queryCommandSupported("copy")){var e=document.createElement("textarea");e.textContent=t,e.style.position="fixed",document.body.appendChild(e),e.select();try{return document.execCommand("copy")}catch(t){return!1}finally{document.body.removeChild(e)}}},$.fn.clearable=function(t){var e=this;this.element=$(this);var i="clearable";this.xButton=this.element.find(".icon.close").first(),this.xButton&&this.xButton.length||(this.xButton=$.createIconElement({classes:"close is-empty",icon:"close"}).icon()),null!=t&&(null==t?void 0:t.tabbable)&&(this.xButton=$('<button type="button" class="btn-icon hide-focus close"></button>').append($.createIconElement({classes:"close",icon:"close"}).icon())),this.clear=function(){e.element.val("").trigger("change").focus().trigger("cleared"),e.checkContents()},this.handleKeydown=function(t){var i=t.key;("Enter"===i||t.altKey&&("Delete"===i||"Backspace"===i))&&(t.preventDefault(),e.clear())},this.checkContents=function(){var t=e.element.val();t&&t.length?this.xButton.removeClass("is-empty"):this.xButton.addClass("is-empty"),this.element.trigger("contents-checked")},this.xButton.insertAfter(e.element),this.xButton[0].tabIndex=null!=t&&(null==t?void 0:t.tabbable)?0:-1,this.xButton[0].setAttribute("focusable",!0),this.xButton.off(["click.".concat(i),"keydown.".concat(i)].join(" ")).on("click.clearable",this.clear).on("keydown.clearable",this.handleKeydown),null!=t&&(null==t?void 0:t.tabbable)&&this.xButton.hideFocus();var n=["blur.".concat(i),"change.".concat(i),"keyup.".concat(i)].join(" ");this.element.off(n).on(n,(function(){e.checkContents()})),this.checkContents()},E.actualChar=function(t){var e=t.which,i={188:"44",190:"46",191:"47",192:"96",220:"92",222:"39",221:"93",219:"91",173:"45",187:"61",186:"59",189:"45"},n={96:"~",49:"!",50:"@",51:"#",52:"$",53:"%",54:"^",55:"&",56:"*",57:"(",48:")",45:"_",61:"+",91:"{",93:"}",92:"|",59:":",37:"%",38:"&",39:'"',44:"<",46:">",47:"?"};return Object.prototype.hasOwnProperty.call(i,e)&&(e=i[e]),e>=96&&e<=105&&(e-=48),!t.shiftKey&&e>=65&&e<=90?String.fromCharCode(e+32):!t.shiftKey&&e>=37&&e<=40?"":t.shiftKey&&Object.prototype.hasOwnProperty.call(n,e)?n[e]:String.fromCharCode(e)},$.actualChar=function(t){return E.actualChar(t)},E.equals=function(t,e){return JSON.stringify(t)===JSON.stringify(e)},f.convertToHTMLElement=function(t){return t instanceof HTMLElement?t:t instanceof $?t=t.length?t[0]:void 0:void 0},f.focusableElems=function(t){var e=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[],i=['button:not([disabled]):not([tabindex="-1"])','[href]:not([disabled]):not([tabindex="-1"])','input:not([disabled]):not([tabindex="-1"])','div.dropdown:not(.is-disabled):not([tabindex="-1"])','div.multiselect:not(.is-disabled):not([tabindex="-1"])','textarea:not([disabled]):not([tabindex="-1"])','[focusable]:not([focusable="false"])','[tabindex]:not([tabindex="-1"])',"[contenteditable]","iframe"].concat(arguments.length>1&&void 0!==arguments[1]?arguments[1]:[]).filter((function(t){return-1===e.indexOf(t)}));return u(t.querySelectorAll(i.join(", "))).filter((function(t){return"use"!==t.tagName.toLowerCase()}))},E.isPlainObject=function(t){return!(!t||"[object Object]"!==Object.prototype.toString.call(t))&&(!Object.getPrototypeOf(t)||null!==t&&"object"===l(t)&&Object.getPrototypeOf(t)===Object.prototype)},E.mergeByPosition=function(t,e){var i,n=Math.max((null===(i=t)||void 0===i||null==i?void 0:i.length)||0,(null==e||null==e?void 0:e.length)||0);if(t||(t=[]),0===n)return[];for(var s=0;s<n;s++)void 0!==e[s]&&null!==e[s]&&(t[s]=e[s]);return t||[]},E.extend=function t(){var e=arguments[0]||{},i=!1,n=0;"[object Boolean]"===Object.prototype.toString.call(arguments[0])&&(i=arguments[0],e=Array.isArray(arguments[1])?[]:{},n++);for(var s=function(n){for(var s in n){if(n.hasOwnProperty(s))if(n[s]instanceof jQuery)e[s]=$(n[s]);else if(i&&"[object Object]"===Object.prototype.toString.call(n[s])){var a=n[s],o=E.isPlainObject(a),r=Array.isArray(a)?[]:{};e[s]=o?t(!0,r,e[s],a):a}else Array.isArray(n[s])?e[s]=E.mergeByPosition(e[s],n[s]):void 0!==n[s]&&(e[s]=void 0===n[s]&&void 0!==e[s]?e[s]:n[s]);n.hasOwnProperty(s)||e[s]||"[object Function]"!==Object.prototype.toString.call(n[s])||(e[s]=n[s])}};n<arguments.length;n++)s(arguments[n]);return e},E.fixSVGIcons=function t(e){if(("ie"===x.browser.name||"11"===x.browser.version)&&void 0!==e){var i="http://www.w3.org/1999/xlink";if(e instanceof $){if(!e.length)return;if(1!==e.length)return void e.each((function(e,i){t(i)}));e=e[0]}e instanceof NodeList?Array.prototype.forEach.call(e,(function(e){t(e)})):setTimeout((function(){for(var t=e.getElementsByTagName("use"),n=0;n<t.length;n++){var s=t[n].getAttributeNS(i,"href");t[n].setAttributeNS(i,"href","x"),t[n].setAttributeNS(i,"href",s)}}),1)}},E.getViewportSize=function(){return{width:Math.max(document.documentElement.clientWidth,window.innerWidth||0),height:Math.max(document.documentElement.clientHeight,window.innerHeight||0)}},E.getContainerScrollDistance=function(t){if(!f.isElement(t))return[];var e=[];return $(t).parents([".scrollable",".scrollable-x",".scrollable-y",".modal",".card-content",".widget-content",".tab-panel",".datagrid-content"].join(", ")).each((function(){var t=this;e.push({element:t,left:t.scrollLeft,top:t.scrollTop})})),document.body.classList.contains("no-scroll")||e.push({element:document.body,left:document.body.scrollLeft,top:document.body.scrollTop}),e},E.getHiddenSize=function(t,e){var i={dims:{width:0,height:0,innerWidth:0,innerHeight:0,outerWidth:0,outerHeight:0},parentElement:void 0,includeMargin:!1};if(!f.isElement(t))return i.dims;t=$(t),e=r(r({},i),e);var n=f.isElement(e.parentElement);n&&(t=t.clone().appendTo(e.parentElement));var s=e.dims,a=t.parents().add(t),o={transition:"none",webkitTransition:"none",mozTransition:"none",msTransition:"none",visibility:"hidden",display:"block"},l=[];return a.each((function(){var t=this,e={};Object.keys(o).forEach((function(i){t.style[i]&&(e[i]=t.style[i],t.style[i]=o[i])})),l.push(e)})),s.padding={bottom:t.css("padding-bottom"),left:t.css("padding-left"),right:t.css("padding-right"),top:t.css("padding-top")},s.width=t.width(),s.outerWidth=t.outerWidth(e.includeMargin),s.innerWidth=t.innerWidth(),s.scrollWidth=t[0].scrollWidth,s.height=t.height(),s.innerHeight=t.innerHeight(),s.outerHeight=t.outerHeight(e.includeMargin),s.scrollHeight=t[0].scrollHeight,a.each((function(t){var e=this,i=l[t];Object.keys(o).forEach((function(t){i[t]&&(e.style[t]=i[t])}))})),n&&t.remove(),s},$.fn.getHiddenSize=function(t){return E.getHiddenSize(this,t)},E.isString=function(t){return"string"==typeof t||t instanceof String},E.isNumber=function(t){return"number"==typeof t&&void 0===t.length&&!isNaN(t)},E.safeSetSelection=function(t,e,i){t instanceof HTMLInputElement&&-1!==["text","password","search","url","week","month"].indexOf(t.type)&&(e&&void 0===i&&(i=e),document.activeElement===t&&("android"===x.os.name?T((function(){t.setSelectionRange(e,i,"none")}),0):t.setSelectionRange(e,i,"none")))},E.mergeSettings=function(t,e,i){return e&&D(e)||(e=D(i)?i:{}),E.extend(!0,{},I(i),I(e),void 0!==t?E.parseSettings(t):{})},E.isHTML=function(t){return/(<([^>]+)>)/i.test(t)};var L={convertDelayToFPS:function(t){if(isNaN(t))throw new Error("provided delay value is not a number");return t/16.7},convertFPSToDelay:function(t){if(isNaN(t))throw new Error("provided delay value is not a number");return 16.7*t},isFinite:function(t){return"number"==typeof t&&(t==t&&t!==1/0&&t!==-1/0)}};E.forEach=function(t,e,i){for(var n=0;n<t.length;n++)e.call(i,t[n],n,t)},L.sign=function(t){return Math.sign?Math.sign(t):0===(t=+t)||isNaN(t)?t:t>0?1:-1},E.getArrayFromList=function(t){var e=Array.prototype.slice;return Function.prototype.call.bind(e)(t)},E.getScrollbarWidth=function(){var t=document.createElement("div");t.style.visibility="hidden",t.style.width="100px",document.body.appendChild(t);var e=t.offsetWidth;t.style.overflow="scroll";var i=document.createElement("div");i.style.width="100%",t.appendChild(i);var n=i.offsetWidth;return t.parentNode.removeChild(t),e-n},E.deepCopy=function(t){return E.extend(!0,Array.isArray(t)?[]:{},t)},E.isSubscribedTo=function(t,e,i,n){var s=$._data(t).events;for(var a in s)if(a===i&&(1!==s[a].length||s[a][0].namespace!==n))return!0;return!1},E.isInViewport=function(t){var e=t.getBoundingClientRect();return e.top>=0&&e.left>=0&&e.bottom<=(window.innerHeight||document.documentElement.clientHeight)&&e.right<=(window.innerWidth||document.documentElement.clientWidth)},E.getSiblingsHeight=function(t){return f.getSiblings(t).map((function(t){return t.offsetHeight})).reduce((function(t,e){return t+e}),0)},E.getParentAvailableHeight=function(t){return t.parentNode.offsetHeight-E.getSiblingsHeight(t)},E.clearSelection=function(){window.getSelection?window.getSelection().removeAllRanges():document.selection&&document.selection.empty()},E.toggleCompactMode=function(t){var e="form-layout-compact",i=t.classList.contains(e);i?t.classList.remove(e):t.classList.add(e);for(var n=t.querySelectorAll(".datagrid-container"),s=0;s<n.length;s++){var a=$(n[s]).data("datagrid");if(!a.rowHeight)return;i?a.rowHeight(a.oldRowHeight||"large"):(a.oldRowHeight=a.settings.rowHeight,a.rowHeight("extra-small"))}},E.addAttributes=function(t,e,i,n,s){if(i){var a=function(e,i){!t[0]||!s&&t[0].hasAttribute(e)||t.attr(e,i+(n?"-".concat(n.toLowerCase()):""))};if(Array.isArray(i))i.forEach((function(t){var i="function"==typeof t.value?t.value(e):t.value;a(t.name,i)}));else{var o="function"==typeof i.value?i.value(e):i.value;a(i.name,o)}}},E.stringAttributes=function(t,e,i){var n="";if(!e)return n;if(Array.isArray(e))return e.forEach((function(e){var s="function"==typeof e.value?e.value(t):e.value;n+=" ".concat(e.name,'="').concat(s+(i?"-".concat(i):""),'"')})),n;var s="function"==typeof e.value?e.value(t):e.value;return n+=" ".concat(e.name,'="').concat(s+(i?"-".concat(i.toLowerCase()):""),'"')},E.getAttribute=function(t,e,i){var n;return"string"==typeof e&&e.length&&Array.isArray(i)?(i.forEach((function(i){n||i.name!==e||(n="function"==typeof i.value?i.value(t):i.value)})),n):n},E.waitForTransitionEnd=function(t,e){return new Promise((function(i){t.addEventListener("transitionend",(function n(s){s.propertyName===e&&(t.removeEventListener("transitionend",n),i())}))}))};var P={isEmpty:function(t){return 0===Object.keys(t).length&&t.constructor===Object}},M={EMPTY:"",splice:function(t,e,i,n){return t.slice(0,e)+n+t.slice(e+Math.abs(i))},removeDuplicates:function(t){return t.split("").filter((function(t,e,i){return i.indexOf(t)===e})).join("")},containsHTML:function(t){return/<[a-z][\s\S]*>/i.test(t)},stripWhitespace:function(t){return t.replace(/\n/g,"").replace(/[\t ]+</g,"<").replace(/>[\t ]+</g,"><").replace(/>[\t ]+$/g,">")},capitalize:function(t){return t.charAt(0).toUpperCase()+t.slice(1)},toBoolean:function(t){var e=+t;return isNaN(e)?!!String(t).toLowerCase().replace(!1,""):!!e},textWidth:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:14;this.canvas=this.canvas||(this.canvas=document.createElement("canvas"));var i=this.canvas.getContext("2d");i.font="".concat(e,"px arial");var n=i.measureText(t);return Math.round(n.width)},padDate:function(t,e,i){return t+"0".concat(e+1).slice(-2)+"0".concat(i).slice(-2)}};M.textWidth=function(t,e,i){var n=document.createElement("canvas");if(!n)return 0;var s=n.getContext("2d");if(!s)return 0;s.font=i||"14px arial";var a=s.measureText(t);return Math.round(a.width+(e||0))},M.escapeRegExp=function(t){return t.replace(/[.*+?^${}()|[\]\\]/g,"\\$&")},M.count=function(t,e){return t.split(e).length-1},M.isValidURL=function(t){return!!new RegExp("^(https?:\\/\\/)?((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|((\\d{1,3}\\.){3}\\d{1,3}))(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*(\\?[;&a-z\\d%_.~+=-]*)?(\\#[-a-z\\d_]*)?$","i").test(t)};var B={truncate:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:2,i=t.toString().split(".");return i[1]=i[1]?i[1].substr(0,e):"","".concat(i[0],".").concat(i[1])},round:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:2;return B.toFixed(t,e)},fixTo:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:2;return e>10&&t.toString().indexOf(".")>-1?t.toLocaleString("en-US",{useGrouping:!1,minimumFractionDigits:e}):(+(Math.round(+(t+"e"+e))+"e"+-e)).toFixed(e)},toFixed:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:2,i=t.toString(),n=!1;"-"===i.substr(0,1)&&(n=!0,i=i.replace("-",""));var s="",a=i.split("."),o=[a[0].substr(0,10),a[0].substr(10),a[1]],r=o[0],l=o[1];if(s=this.fixTo((l||r)+(o[2]?".".concat(o[2]):""),e).toString(),l&&11===s.length&&"10000000000"===s&&(s="0000000000",r=(parseInt(r,10)+1).toString()),l&&"0"!==l.substr(0,1)&&(s=r+s),l&&"0"===l.substr(0,1))if(0===parseInt(l,10)){var c=s.split(".")[1];s="".concat(r).concat(l).concat(c?".".concat(c):"")}else s="".concat(r).concat(s);return n?"-".concat(s):s},decimalPlaces:function(t){return Math.floor(t)===t||-1===t.toString().indexOf(".")?0:t.toString().split(".")[1].length||0}},O={};function R(t){var e=Number(t).toString(16);return 1===e.length?"0".concat(e):e}O.hexToRgba=function(t,e){var i;return/^#([A-Fa-f0-9]{3}){1,2}$/.test(t)?(3===(i=t.substring(1).split("")).length&&(i=[i[0],i[0],i[1],i[1],i[2],i[2]]),i="0x".concat(i.join("")),"rgba(".concat([i>>16&255,i>>8&255,255&i].join(","),",").concat(e.toString(),")")):""},O.hexToRgb=function(t){var e=/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(t);return e?{r:parseInt(e[1],16),g:parseInt(e[2],16),b:parseInt(e[3],16)}:null},O.rgbToHex=function(t,e,i){return"object"!==l(t)||"number"!=typeof t.r&&"string"!=typeof t.r||(e=t.g,i=t.b,t=t.r),"#".concat(R(t)).concat(R(e)).concat(R(i))},O.rgbToHsl=function(t,e,i){"object"!==l(t)||"number"!=typeof t.r&&"string"!=typeof t.r||(e=t.g,i=t.b,t=t.r),t=Number(t),e=Number(e),i=Number(i),t/=255,e/=255,i/=255;var n=Math.min(t,e,i),s=Math.max(t,e,i),a=s-n,o=0,r=0,c=0;return o=0===a?0:s===t?(e-i)/a%6:s===e?(i-t)/a+2:(t-e)/a+4,(o=Math.round(60*o))<0&&(o+=360),c=(s+n)/2,0!==a&&(r=a/(1-Math.abs(2*c-1))),{h:o,s:r=+(100*r).toFixed(1),l:c=+(100*c).toFixed(1)}},O.hslToRgb=function(t,e,i){"object"!==l(t)||"number"!=typeof t.h&&"string"!=typeof t.h||(e=(t=t.h).s,i=t.l),t=Number(t),e=Number(e),i=Number(i),e/=100,i/=100;var n=(1-Math.abs(2*i-1))*e,s=n*(1-Math.abs(t/60%2-1)),a=i-n/2,o=0,r=0,c=0;return 0<=t&&t<60?(o=n,r=s,c=0):60<=t&&t<120?(o=s,r=n,c=0):120<=t&&t<180||180<=t&&t<240?(o=0,r=s,c=n):240<=t&&t<300?(o=s,r=0,c=n):300<=t&&t<360&&(o=n,r=0,c=s),{r:o=Math.round(255*(o+a)),g:r=Math.round(255*(r+a)),b:c=Math.round(255*(c+a))}},O.getLuminousColorShade=function(t,e){t=this.validateHex(t).substr(1),e=e||0;var i,n,s="#";for(n=0;n<3;n++)i=parseInt(t.substr(2*n,2),16),i=Math.round(Math.min(Math.max(0,i+i*e),255)).toString(16),s+="00".concat(i).substr(i.length);return s},O.validateHex=function(t){return(t=String(t).replace(/[^0-9a-f]/gi,"")).length<6&&(t=t[0]+t[0]+t[1]+t[1]+t[2]+t[2]),"#".concat(t)},O.getContrastColor=function(t,e,i){t=t?t.replace("#",""):"";var n=function(e){return parseInt(t.substr(e,2),16)};return(299*n(0)+587*n(2)+114*n(4))/1e3>=146?i||"black":e||"white"},O.getDesaturatedColor=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1;t=t?t.replace("#",""):"";var i=O.hexToRgb(t),n=.3086*i.r+.6094*i.g+.082*i.b;return i.r=Math.round(i.r*e+n*(1-e)),i.g=Math.round(i.g*e+n*(1-e)),i.b=Math.round(i.b*e+n*(1-e)),O.rgbToHex(i.r,i.g,i.b)},O.isLighter=function(t,e){var i=O.hexToRgb(t),n=O.hexToRgb(e),s=O.rgbToHsl(i),a=O.rgbToHsl(n);return s.l<=a.l};var z=[20,50,79,109,138,168,197,227,256,286,315,345,374,404,433,463,492,522,551,581,611,641,670,700,729,759,788,818,847,877,906,936,965,995,1024,1054,1083,1113,1142,1172,1201,1231,1260,1290,1320,1350,1379,1409,1438,1468,1497,1527,1556,1586,1615,1645,1674,1704,1733,1763,1792,1822,1851,1881,1910,1940,1969,1999,2028,2058,2087,2117,2146,2176,2205,2235,2264,2294,2323,2353,2383,2413,2442,2472,2501,2531,2560,2590,2619,2649,2678,2708,2737,2767,2796,2826,2855,2885,2914,2944,2973,3003,3032,3062,3091,3121,3150,3180,3209,3239,3268,3298,3327,3357,3386,3416,3446,3476,3505,3535,3564,3594,3623,3653,3682,3712,3741,3771,3800,3830,3859,3889,3918,3948,3977,4007,4036,4066,4095,4125,4155,4185,4214,4244,4273,4303,4332,4362,4391,4421,4450,4480,4509,4539,4568,4598,4627,4657,4686,4716,4745,4775,4804,4834,4863,4893,4922,4952,4981,5011,5040,5070,5099,5129,5158,5188,5218,5248,5277,5307,5336,5366,5395,5425,5454,5484,5513,5543,5572,5602,5631,5661,5690,5720,5749,5779,5808,5838,5867,5897,5926,5956,5985,6015,6044,6074,6103,6133,6162,6192,6221,6251,6281,6311,6340,6370,6399,6429,6458,6488,6517,6547,6576,6606,6635,6665,6694,6724,6753,6783,6812,6842,6871,6901,6930,6960,6989,7019,7048,7078,7107,7137,7166,7196,7225,7255,7284,7314,7344,7374,7403,7433,7462,7492,7521,7551,7580,7610,7639,7669,7698,7728,7757,7787,7816,7846,7875,7905,7934,7964,7993,8023,8053,8083,8112,8142,8171,8201,8230,8260,8289,8319,8348,8378,8407,8437,8466,8496,8525,8555,8584,8614,8643,8673,8702,8732,8761,8791,8821,8850,8880,8909,8938,8968,8997,9027,9056,9086,9115,9145,9175,9205,9234,9264,9293,9322,9352,9381,9410,9440,9470,9499,9529,9559,9589,9618,9648,9677,9706,9736,9765,9794,9824,9853,9883,9913,9943,9972,10002,10032,10061,10090,10120,10149,10178,10208,10237,10267,10297,10326,10356,10386,10415,10445,10474,10504,10533,10562,10592,10621,10651,10680,10710,10740,10770,10799,10829,10858,10888,10917,10947,10976,11005,11035,11064,11094,11124,11153,11183,11213,11242,11272,11301,11331,11360,11389,11419,11448,11478,11507,11537,11567,11596,11626,11655,11685,11715,11744,11774,11803,11832,11862,11891,11921,11950,11980,12010,12039,12069,12099,12128,12158,12187,12216,12246,12275,12304,12334,12364,12393,12423,12453,12483,12512,12542,12571,12600,12630,12659,12688,12718,12747,12777,12807,12837,12866,12896,12926,12955,12984,13014,13043,13072,13102,13131,13161,13191,13220,13250,13280,13310,13339,13368,13398,13427,13456,13486,13515,13545,13574,13604,13634,13664,13693,13723,13752,13782,13811,13840,13870,13899,13929,13958,13988,14018,14047,14077,14107,14136,14166,14195,14224,14254,14283,14313,14342,14372,14401,14431,14461,14490,14520,14550,14579,14609,14638,14667,14697,14726,14756,14785,14815,14844,14874,14904,14933,14963,14993,15021,15051,15081,15110,15140,15169,15199,15228,15258,15287,15317,15347,15377,15406,15436,15465,15494,15524,15553,15582,15612,15641,15671,15701,15731,15760,15790,15820,15849,15878,15908,15937,15966,15996,16025,16055,16085,16114,16144,16174,16204,16233,16262,16292,16321,16350,16380,16409,16439,16468,16498,16528,16558,16587,16617,16646,16676,16705,16734,16764,16793,16823,16852,16882,16912,16941,16971,17001,17030,17060,17089,17118,17148,17177,17207,17236,17266,17295,17325,17355,17384,17414,17444,17473,17502,17532,17561,17591,17620,17650,17679,17709,17738,17768,17798,17827,17857,17886,17916,17945,17975,18004,18034,18063,18093,18122,18152,18181,18211,18241,18270,18300,18330,18359,18388,18418,18447,18476,18506,18535,18565,18595,18625,18654,18684,18714,18743,18772,18802,18831,18860,18890,18919,18949,18979,19008,19038,19068,19098,19127,19156,19186,19215,19244,19274,19303,19333,19362,19392,19422,19452,19481,19511,19540,19570,19599,19628,19658,19687,19717,19746,19776,19806,19836,19865,19895,19924,19954,19983,20012,20042,20071,20101,20130,20160,20190,20219,20249,20279,20308,20338,20367,20396,20426,20455,20485,20514,20544,20573,20603,20633,20662,20692,20721,20751,20780,20810,20839,20869,20898,20928,20957,20987,21016,21046,21076,21105,21135,21164,21194,21223,21253,21282,21312,21341,21371,21400,21430,21459,21489,21519,21548,21578,21607,21637,21666,21696,21725,21754,21784,21813,21843,21873,21902,21932,21962,21991,22021,22050,22080,22109,22138,22168,22197,22227,22256,22286,22316,22346,22375,22405,22434,22464,22493,22522,22552,22581,22611,22640,22670,22700,22730,22759,22789,22818,22848,22877,22906,22936,22965,22994,23024,23054,23083,23113,23143,23173,23202,23232,23261,23290,23320,23349,23379,23408,23438,23467,23497,23527,23556,23586,23616,23645,23674,23704,23733,23763,23792,23822,23851,23881,23910,23940,23970,23999,24029,24058,24088,24117,24147,24176,24206,24235,24265,24294,24324,24353,24383,24413,24442,24472,24501,24531,24560,24590,24619,24648,24678,24707,24737,24767,24796,24826,24856,24885,24915,24944,24974,25003,25032,25062,25091,25121,25150,25180,25210,25240,25269,25299,25328,25358,25387,25416,25446,25475,25505,25534,25564,25594,25624,25653,25683,25712,25742,25771,25800,25830,25859,25888,25918,25948,25977,26007,26037,26067,26096,26126,26155,26184,26214,26243,26272,26302,26332,26361,26391,26421,26451,26480,26510,26539,26568,26598,26627,26656,26686,26715,26745,26775,26805,26834,26864,26893,26923,26952,26982,27011,27041,27070,27099,27129,27159,27188,27218,27248,27277,27307,27336,27366,27395,27425,27454,27484,27513,27542,27572,27602,27631,27661,27691,27720,27750,27779,27809,27838,27868,27897,27926,27956,27985,28015,28045,28074,28104,28134,28163,28193,28222,28252,28281,28310,28340,28369,28399,28428,28458,28488,28517,28547,28577,28607,28636,28665,28695,28724,28754,28783,28813,28843,28872,28901,28931,28960,28990,29019,29049,29078,29108,29137,29167,29196,29226,29255,29285,29315,29345,29375,29404,29434,29463,29492,29522,29551,29580,29610,29640,29669,29699,29729,29759,29788,29818,29847,29876,29906,29935,29964,29994,30023,30053,30082,30112,30141,30171,30200,30230,30259,30289,30318,30348,30378,30408,30437,30467,30496,30526,30555,30585,30614,30644,30673,30703,30732,30762,30791,30821,30850,30880,30909,30939,30968,30998,31027,31057,31086,31116,31145,31175,31204,31234,31263,31293,31322,31352,31381,31411,31441,31471,31500,31530,31559,31589,31618,31648,31676,31706,31736,31766,31795,31825,31854,31884,31913,31943,31972,32002,32031,32061,32090,32120,32150,32180,32209,32239,32268,32298,32327,32357,32386,32416,32445,32475,32504,32534,32563,32593,32622,32652,32681,32711,32740,32770,32799,32829,32858,32888,32917,32947,32976,33006,33035,33065,33094,33124,33153,33183,33213,33243,33272,33302,33331,33361,33390,33420,33450,33479,33509,33539,33568,33598,33627,33657,33686,33716,33745,33775,33804,33834,33863,33893,33922,33952,33981,34011,34040,34069,34099,34128,34158,34187,34217,34247,34277,34306,34336,34365,34395,34424,34454,34483,34512,34542,34571,34601,34631,34660,34690,34719,34749,34778,34808,34837,34867,34896,34926,34955,34985,35015,35044,35074,35103,35133,35162,35192,35222,35251,35280,35310,35340,35370,35399,35429,35458,35488,35517,35547,35576,35605,35635,35665,35694,35723,35753,35782,35811,35841,35871,35901,35930,35960,35989,36019,36048,36078,36107,36136,36166,36195,36225,36254,36284,36314,36343,36373,36403,36433,36462,36492,36521,36551,36580,36610,36639,36669,36698,36728,36757,36786,36816,36845,36875,36904,36934,36963,36993,37022,37052,37081,37111,37141,37170,37200,37229,37259,37288,37318,37347,37377,37406,37436,37465,37495,37524,37554,37584,37613,37643,37672,37701,37731,37760,37790,37819,37849,37878,37908,37938,37967,37997,38027,38056,38085,38115,38144,38174,38203,38233,38262,38292,38322,38351,38381,38410,38440,38469,38499,38528,38558,38587,38617,38646,38676,38705,38735,38764,38794,38823,38853,38882,38912,38941,38971,39001,39030,39059,39089,39118,39148,39178,39208,39237,39267,39297,39326,39355,39385,39414,39444,39473,39503,39532,39562,39592,39621,39650,39680,39709,39739,39768,39798,39827,39857,39886,39916,39946,39975,40005,40035,40064,40094,40123,40153,40182,40212,40241,40271,40300,40330,40359,40389,40418,40448,40477,40507,40536,40566,40595,40625,40655,40685,40714,40744,40773,40803,40832,40862,40892,40921,40951,40980,41009,41039,41068,41098,41127,41157,41186,41216,41245,41275,41304,41334,41364,41393,41422,41452,41481,41511,41540,41570,41599,41629,41658,41688,41718,41748,41777,41807,41836,41865,41894,41924,41953,41983,42012,42042,42072,42102,42131,42161,42190,42220,42249,42279,42308,42337,42367,42397,42426,42456,42485,42515,42545,42574,42604,42633,42662,42692,42721,42751,42780,42810,42839,42869,42899,42929,42958,42988,43017,43046,43076,43105,43135,43164,43194,43223,43253,43283,43312,43342,43371,43401,43430,43460,43489,43519,43548,43578,43607,43637,43666,43696,43726,43755,43785,43814,43844,43873,43903,43932,43962,43991,44021,44050,44080,44109,44139,44169,44198,44228,44258,44287,44317,44346,44375,44405,44434,44464,44493,44523,44553,44582,44612,44641,44671,44700,44730,44759,44788,44818,44847,44877,44906,44936,44966,44996,45025,45055,45084,45114,45143,45172,45202,45231,45261,45290,45320,45350,45380,45409,45439,45468,45498,45527,45556,45586,45615,45644,45674,45704,45733,45763,45793,45823,45852,45882,45911,45940,45970,45999,46028,46058,46088,46117,46147,46177,46206,46236,46265,46295,46324,46354,46383,46413,46442,46472,46501,46531,46560,46590,46620,46649,46679,46708,46738,46767,46797,46826,46856,46885,46915,46944,46974,47003,47033,47063,47092,47122,47151,47181,47210,47240,47269,47298,47328,47357,47387,47417,47446,47476,47506,47535,47565,47594,47624,47653,47682,47712,47741,47771,47800,47830,47860,47890,47919,47949,47978,48008,48037,48066,48096,48125,48155,48184,48214,48244,48273,48303,48333,48362,48392,48421,48450,48480,48509,48538,48568,48598,48627,48657,48687,48717,48746,48776,48805,48834,48864,48893,48922,48952,48982,49011,49041,49071,49100,49130,49160,49189,49218,49248,49277,49306,49336,49365,49395,49425,49455,49484,49514,49543,49573,49602,49632,49661,49690,49720,49749,49779,49809,49838,49868,49898,49927,49957,49986,50016,50045,50075,50104,50133,50163,50192,50222,50252,50281,50311,50340,50370,50400,50429,50459,50488,50518,50547,50576,50606,50635,50665,50694,50724,50754,50784,50813,50843,50872,50902,50931,50960,50990,51019,51049,51078,51108,51138,51167,51197,51227,51256,51286,51315,51345,51374,51403,51433,51462,51492,51522,51552,51582,51611,51641,51670,51699,51729,51758,51787,51816,51846,51876,51906,51936,51965,51995,52025,52054,52083,52113,52142,52171,52200,52230,52260,52290,52319,52349,52379,52408,52438,52467,52497,52526,52555,52585,52614,52644,52673,52703,52733,52762,52792,52822,52851,52881,52910,52939,52969,52998,53028,53057,53087,53116,53146,53176,53205,53235,53264,53294,53324,53353,53383,53412,53441,53471,53500,53530,53559,53589,53619,53648,53678,53708,53737,53767,53796,53825,53855,53884,53914,53943,53973,54003,54032,54062,54092,54121,54151,54180,54209,54239,54268,54297,54327,54357,54387,54416,54446,54476,54505,54535,54564,54593,54623,54652,54681,54711,54741,54770,54800,54830,54859,54889,54919,54948,54977,55007,55036,55066,55095,55125,55154,55184,55213,55243,55273,55302,55332,55361,55391,55420,55450,55479,55508,55538,55567,55597,55627,55657,55686,55716,55745,55775,55804,55834,55863,55892,55922,55951,55981,56011,56040,56070,56100,56129,56159,56188,56218,56247,56276,56306,56335,56365,56394,56424,56454,56483,56513,56543,56572,56601,56631,56660,56690,56719,56749,56778,56808,56837,56867,56897,56926,56956,56985,57015,57044,57074,57103,57133,57162,57192,57221,57251,57280,57310,57340,57369,57399,57429,57458,57487,57517,57546,57576,57605,57634,57664,57694,57723,57753,57783,57813,57842,57871,57901,57930,57959,57989,58018,58048,58077,58107,58137,58167,58196,58226,58255,58285,58314,58343,58373,58402,58432,58461,58491,58521,58551,58580,58610,58639,58669,58698,58727,58757,58786,58816,58845,58875,58905,58934,58964,58994,59023,59053,59082,59111,59141,59170,59200,59229,59259,59288,59318,59348,59377,59407,59436,59466,59495,59525,59554,59584,59613,59643,59672,59702,59731,59761,59791,59820,59850,59879,59909,59939,59968,59997,60027,60056,60086,60115,60145,60174,60204,60234,60264,60293,60323,60352,60381,60411,60440,60469,60499,60528,60558,60588,60618,60647,60677,60707,60736,60765,60795,60824,60853,60883,60912,60942,60972,61002,61031,61061,61090,61120,61149,61179,61208,61237,61267,61296,61326,61356,61385,61415,61445,61474,61504,61533,61563,61592,61621,61651,61680,61710,61739,61769,61799,61828,61858,61888,61917,61947,61976,62006,62035,62064,62094,62123,62153,62182,62212,62242,62271,62301,62331,62360,62390,62419,62448,62478,62507,62537,62566,62596,62625,62655,62685,62715,62744,62774,62803,62832,62862,62891,62921,62950,62980,63009,63039,63069,63099,63128,63157,63187,63216,63246,63275,63305,63334,63363,63393,63423,63453,63482,63512,63541,63571,63600,63630,63659,63689,63718,63747,63777,63807,63836,63866,63895,63925,63955,63984,64014,64043,64073,64102,64131,64161,64190,64220,64249,64279,64309,64339,64368,64398,64427,64457,64486,64515,64545,64574,64603,64633,64663,64692,64722,64752,64782,64811,64841,64870,64899,64929,64958,64987,65017,65047,65076,65106,65136,65166,65195,65225,65254,65283,65313,65342,65371,65401,65431,65460,65490,65520,65549,65579,65608,65638,65667,65697,65726,65755,65785,65815,65844,65874,65903,65933,65963,65992,66022,66051,66081,66110,66140,66169,66199,66228,66258,66287,66317,66346,66376,66405,66435,66465,66494,66524,66553,66583,66612,66641,66671,66700,66730,66760,66789,66819,66849,66878,66908,66937,66967,66996,67025,67055,67084,67114,67143,67173,67203,67233,67262,67292,67321,67351,67380,67409,67439,67468,67497,67527,67557,67587,67617,67646,67676,67705,67735,67764,67793,67823,67852,67882,67911,67941,67971,68e3,68030,68060,68089,68119,68148,68177,68207,68236,68266,68295,68325,68354,68384,68414,68443,68473,68502,68532,68561,68591,68620,68650,68679,68708,68738,68768,68797,68827,68857,68886,68916,68946,68975,69004,69034,69063,69092,69122,69152,69181,69211,69240,69270,69300,69330,69359,69388,69418,69447,69476,69506,69535,69565,69595,69624,69654,69684,69713,69743,69772,69802,69831,69861,69890,69919,69949,69978,70008,70038,70067,70097,70126,70156,70186,70215,70245,70274,70303,70333,70362,70392,70421,70451,70481,70510,70540,70570,70599,70629,70658,70687,70717,70746,70776,70805,70835,70864,70894,70924,70954,70983,71013,71042,71071,71101,71130,71159,71189,71218,71248,71278,71308,71337,71367,71397,71426,71455,71485,71514,71543,71573,71602,71632,71662,71691,71721,71751,71781,71810,71839,71869,71898,71927,71957,71986,72016,72046,72075,72105,72135,72164,72194,72223,72253,72282,72311,72341,72370,72400,72429,72459,72489,72518,72548,72577,72607,72637,72666,72695,72725,72754,72784,72813,72843,72872,72902,72931,72961,72991,73020,73050,73080,73109,73139,73168,73197,73227,73256,73286,73315,73345,73375,73404,73434,73464,73493,73523,73552,73581,73611,73640,73669,73699,73729,73758,73788,73818,73848,73877,73907,73936,73965,73995,74024,74053,74083,74113,74142,74172,74202,74231,74261,74291,74320,74349,74379,74408,74437,74467,74497,74526,74556,74585,74615,74645,74675,74704,74733,74763,74792,74822,74851,74881,74910,74940,74969,74999,75029,75058,75088,75117,75147,75176,75206,75235,75264,75294,75323,75353,75383,75412,75442,75472,75501,75531,75560,75590,75619,75648,75678,75707,75737,75766,75796,75826,75856,75885,75915,75944,75974,76003,76032,76062,76091,76121,76150,76180,76210,76239,76269,76299,76328,76358,76387,76416,76446,76475,76505,76534,76564,76593,76623,76653,76682,76712,76741,76771,76801,76830,76859,76889,76918,76948,76977,77007,77036,77066,77096,77125,77155,77185,77214,77243,77273,77302,77332,77361,77390,77420,77450,77479,77509,77539,77569,77598,77627,77657,77686,77715,77745,77774,77804,77833,77863,77893,77923,77952,77982,78011,78041,78070,78099,78129,78158,78188,78217,78247,78277,78307,78336,78366,78395,78425,78454,78483,78513,78542,78572,78601,78631,78661,78690,78720,78750,78779,78808,78838,78867,78897,78926,78956,78985,79015,79044,79074,79104,79133,79163,79192,79222,79251,79281,79310,79340,79369,79399,79428,79458,79487,79517,79546,79576,79606,79635,79665,79695,79724,79753,79783,79812,79841,79871,79900,79930,79960,79990],F="",N=!1;"object"===l(window.SohoConfig)&&("string"==typeof window.SohoConfig.culturesPath&&(F=window.SohoConfig.culturesPath),"boolean"==typeof window.SohoConfig.minifyCultures&&(N=window.SohoConfig.minifyCultures));var H={currentLocale:{name:"",data:{}},currentLanguage:{name:""},cultures:{},languages:{},dff:[],culturesPath:F,defaultLocales:[{lang:"af",default:"af-ZA"},{lang:"ar",default:"ar-EG"},{lang:"bg",default:"bg-BG"},{lang:"cs",default:"cs-CZ"},{lang:"da",default:"da-DK"},{lang:"de",default:"de-DE"},{lang:"el",default:"el-GR"},{lang:"en",default:"en-US"},{lang:"es",default:"es-ES"},{lang:"et",default:"et-EE"},{lang:"fi",default:"fi-FI"},{lang:"fr",default:"fr-FR"},{lang:"he",default:"he-IL"},{lang:"hi",default:"hi-IN"},{lang:"hr",default:"hr-HR"},{lang:"hu",default:"hu-HU"},{lang:"id",default:"id-ID"},{lang:"it",default:"it-IT"},{lang:"iw",default:"he-IL"},{lang:"ja",default:"ja-JP"},{lang:"ko",default:"ko-KR"},{lang:"lt",default:"lt-LT"},{lang:"lv",default:"lv-LV"},{lang:"ms",default:"ms-bn"},{lang:"nb",default:"no-NO"},{lang:"nn",default:"no-NO"},{lang:"nl",default:"nl-NL"},{lang:"no",default:"no-NO"},{lang:"pl",default:"pl-PL"},{lang:"pt",default:"pt-PT"},{lang:"ro",default:"ro-RO"},{lang:"ru",default:"ru-RU"},{lang:"sk",default:"sk-SK"},{lang:"sl",default:"sl-SI"},{lang:"sv",default:"sv-SE"},{lang:"th",default:"th-TH"},{lang:"tl",default:"tl-PH"},{lang:"tr",default:"tr-TR"},{lang:"uk",default:"uk-UA"},{lang:"vi",default:"vi-VN"},{lang:"zh",default:"zh-CN"}],supportedLocales:["af-ZA","ar-EG","ar-SA","bg-BG","cs-CZ","da-DK","de-DE","el-GR","en-AU","en-GB","en-IN","en-NZ","en-US","en-ZA","es-AR","es-ES","es-419","es-MX","es-US","et-EE","fi-FI","fr-CA","fr-FR","he-IL","hi-IN","hr-HR","hu-HU","id-ID","it-IT","ja-JP","ko-KR","lt-LT","lv-LV","ms-bn","ms-my","nb-NO","nn-NO","nl-NL","no-NO","pl-PL","pt-BR","pt-PT","ro-RO","ru-RU","sk-SK","sl-SI","sv-SE","th-TH","tl-PH","tr-TR","uk-UA","vi-VN","zh-CN","zh-Hans","zh-Hant","zh-TW"],translatedLocales:["fr-CA","fr-FR","pt-BR","pt-PT"],defaultLocale:"en-US",minify:N,updateLanguageTag:function(t){var e=$("html");2===t.length&&(t=this.defaultLocales.filter((function(e){return e.lang===t}))),e.attr("lang",t),this.isRTL()?e.attr("dir","rtl"):e.attr("dir","ltr"),H.flipIconsHorizontally(this.isRTL()),$("body").removeClass("busy-loading-locale")},getCulturesPath:function(){if(!this.culturesPath)for(var t=document.getElementsByTagName("script"),e=/sohoxi(.min){0,1}(.{0,1}[a-z0-9]*)\.js/,i=0;i<t.length;i++){var n=t[i].src,s=n.indexOf("?");if(""!==n&&s>-1&&(n=n.substr(0,s)),"sohoxi-script"===t[i].id)return"".concat(n.substring(0,n.lastIndexOf("/")),"/");n.match(e)&&(this.culturesPath="".concat(n.replace(e,""),"cultures/"))}return this.culturesPath},cultureInHead:function(){for(var t=!1,e=document.getElementsByTagName("script"),i=0;i<e.length;i++){e[i].src.indexOf("cultures")>-1&&(t=!0)}return t},correctLocale:function(t){"in-ID"===t&&(t="id-ID"),"iw"===t.substr(0,2)&&(t="he-IL");var e=t.split("-")[0];if(-1===this.supportedLocales.indexOf(t)){if((t=this.defaultLocales.filter((function(t){return t.lang===e})))&&t[0])return t[0].default;t=this.defaultLocale}return t},correctLanguage:function(t){var e;"en-"===(null===(e=t)||void 0===e||null==e?void 0:e.substr(0,3))&&(t=t.substr(0,2));var i=this.defaultLocales.filter((function(e){return e.lang===t}));return i&&i[0]?this.remapLanguage(t):(i=this.remapLanguage(t),i)},remapLanguage:function(t){var e=t;return"in"===t&&(e="id"),"iw"===t&&(e="he"),"nb"!==t&&"nn"!==t&&"nb-NO"!==t&&"nn-NO"!==t||(e="no"),this.translatedLocales.indexOf(t)>-1?this.translatedLocales[this.translatedLocales.indexOf(t)]:e.substr(0,2)},addCulture:function(t,e,i){var n=this.remapLanguage(t);if(this.cultures[t]=e,this.cultures[t].name=t,e.messages)this.languages[n]={name:n,direction:e.direction||(i?i.direction:""),nativeName:e.nativeName||(i?i.nativeName:""),messages:e.messages||(i?i.messages:{})},this.languages[t]={name:t,direction:e.direction||(i?i.direction:""),nativeName:e.nativeName||(i?i.nativeName:""),messages:e.messages||(i?i.messages:{})};else if(!e.messages){var s=this.parentLocale(t);s.default&&s.default!==t&&!this.cultures[s.default]&&this.appendLocaleScript(s.default)}},parentLocale:function(t){var e=t.substr(0,2),i=this.defaultLocales.filter((function(t){return t.lang===e}))[0]||[{default:"en-US"}];return this.translatedLocales.indexOf(t)>-1?{lang:"fr",default:"fr-CA"}:i},appendedLocales:[],appendLocaleScript:function(t,e,i,n){var s=this,a=document.createElement("script"),o=this.minify?".min":"";a.async=!1,this.appendedLocales.indexOf(t)>-1||(this.appendedLocales.push(t),a.src=n?"".concat(this.getCulturesPath()+n).concat(o,".js"):"".concat(this.getCulturesPath()+t).concat(o,".js"),a.onload=function(){e&&!i&&(s.setCurrentLocale(t,s.cultures[t]),s.dff[t].resolve(t)),i&&s.dff[i]&&(s.setCurrentLocale(t,s.cultures[t]),s.setCurrentLocale(i,s.cultures[i]),s.dff[i].resolve(i)),i&&s.dff[t]&&s.cultures[t]&&(s.setCurrentLocale(t,s.cultures[t]),s.dff[t].resolve(t)),e||i||!s.dff[t]||s.dff[t].resolve(t)},a.onerror=function(){s.dff[t]&&s.dff[t].reject()},"object"===l(window.SohoConfig)&&"string"==typeof window.SohoConfig.nonce&&a.setAttribute("nonce",window.SohoConfig.nonce),document.head.appendChild(a))},set:function(t){var e=this;if(t=this.correctLocale(t),this.dff[t]=$.Deferred(),""===t)return e.dff.resolve(),this.dff.promise();this.cultures["en-US"]||this.appendLocaleScript("en-US","en-US"===t);var i=!1,n=this.parentLocale(t);return n.default&&n.default!==t&&!this.cultures[n.default]&&(i=!0),i||!t||this.cultures[t]||this.currentLocale.name===t||"en-US"===t||(this.setCurrentLocale(t),this.appendLocaleScript(t,!0)),i&&("en-US"!==n.default&&this.appendLocaleScript(n.default,!1),this.appendLocaleScript(t,!1,n.default)),t&&e.currentLocale.data&&e.currentLocale.dataName===t&&e.dff[t].resolve(e.currentLocale.name),e.setCurrentLocale(t,e.cultures[t]),e.cultures[t]&&this.cultureInHead()&&e.dff[t].resolve(e.currentLocale.name),this.dff[t].promise()},getLocale:function(t,e){if(!t)return null;if(t=this.correctLocale(t),this.dff[t]=$.Deferred(),""===t){var i=$.Deferred();return i.resolve(),i.promise()}return t&&"en-US"!==t&&!this.cultures["en-US"]&&this.appendLocaleScript("en-US",!1),t&&!this.cultures[t]&&this.currentLocale.name!==t&&"en-US"!==t&&this.appendLocaleScript(t,!1,!1,e),t&&this.currentLocale.data&&this.currentLocale.dataName===t&&this.dff[t].resolve(t),this.cultures[t]&&this.cultureInHead()&&this.dff[t].resolve(t),this.dff[t].promise()},setLanguage:function(t){var e=this,i=this.currentLocale.name;t=this.correctLanguage(t),this.languages[t]||this.set(t).done((function(){e.set(i),e.setLanguage(t)}));var n=this.correctLocale(t);return this.languages[t]?(this.currentLanguage=this.languages[t],this.updateLanguageTag(t),this.dff[n]=$.Deferred(),this.dff[n].resolve()):(this.currentLanguage.name=t,this.dff[n])},setCurrentLocale:function(t,e){var i=this.remapLanguage(t);this.currentLocale.name=t;var n,s,a,o,r,c=void 0!==this.languages[i]&&void 0!==this.languages[t]&&this.languages[i].nativeName!==this.languages[t].nativeName?t:i;e&&(this.currentLocale.data=e,this.currentLocale.dataName=t,this.currentLanguage={},this.currentLanguage.name=i,this.languages[c]&&(this.currentLanguage=this.languages[c],this.currentLanguage.name=i,this.updateLanguageTag(t)),this.translatedLocales.indexOf(t)>-1&&(this.languages[c].direction=e.direction,this.languages[c].messages=e.messages,this.languages[c].name=i,this.languages[c].nativeName=e.nativeName,this.languages[t]={direction:e.direction,messages:e.messages,name:t,nativeName:e.nativeName}),"object"===("undefined"==typeof d3?"undefined":l(d3))&&d3.formatDefaultLocale({decimal:(null==e||null===(n=null==e?void 0:e.numbers)||void 0===n||null==n?void 0:n.decimal)||".",thousands:(null==e||null===(s=null==e?void 0:e.numbers)||void 0===s||null==s?void 0:s.group)||",",grouping:(null==e||null===(a=null==e?void 0:e.numbers)||void 0===a||null==a?void 0:a.groupSizes)||[3],percent:(null==e||null===(o=null==e?void 0:e.numbers)||void 0===o||null==o?void 0:o.percentSign)||"%",currency:null!=e&&(null==e?void 0:e.currencySign)?""===e.currencyFormat.split("¤")[0]?[e.currencySign,""]:["",e.currencySign]:["$",""],minus:(null==e||null===(r=null==e?void 0:e.numbers)||void 0===r||null==r?void 0:r.minusSign)||"-"}))},formatDate:function(t,e){e||(e={date:"short"});var i=this.useLocale(e);if(t){if("0000"===t||"000000"===t||"00000000"===t)return"";if("string"==typeof t&&/T|Z/g.test(t)&&(t=this.newDateObj(t)),!(t instanceof Date)&&"string"==typeof t){var n=H.parseDate(t,e);isNaN(n)&&"datetime"===e.date&&"-"===t.substr(4,1)&&"-"===t.substr(7,1)&&(n=new Date(t.substr(0,4),t.substr(5,2)-1,t.substr(8,2),t.substr(11,2),t.substr(14,2),t.substr(17,2))),t=n}if(!(t instanceof Date)&&"number"==typeof t)t=new Date(t);if(t){var s,a="",o=i.calendars?i.calendars[0]:null;e.date&&o&&(s=o.dateFormat[e.date]),e.pattern&&(s=e.pattern),"string"==typeof e&&""!==e&&(s=e),s||(s=o.dateFormat.short);var r=t instanceof Array?t[0]:t.getFullYear(),l=t instanceof Array?t[1]:t.getMonth(),c=t instanceof Array?t[2]:t.getDate(),d=t.getDay?t.getDay():"",h=t instanceof Array?t[3]:t.getHours(),u=t instanceof Array?t[4]:t.getMinutes(),p=t instanceof Array?t[5]:t.getSeconds(),g=t instanceof Array?t[6]:t.getMilliseconds();if(e.fromGregorian||e.toUmalqura){var f=this.gregorianToUmalqura(t);c=f[2],l=f[1],r=f[0]}if(e.toGregorian||e.fromUmalqura){var m=this.umalquraToGregorian(r,l,c);c=m.getDate(),l=m.getMonth(),r=m.getFullYear()}var v=(a=(a=(a=(a=(a=(s=(s=(s=(s=s.replace("de","nnnnn")).replace("ngày","nnnn")).replace("tháng","t1áng")).replace("den","nnn")).replace("dd",this.pad(c,2))).replace("d",c)).replace("yyyy",r)).replace("yy",r.toString().substr(2))).replace("y",r)).indexOf(" a")>-1||0===a.indexOf("a");if(v&&0===h&&(a=(a=a.replace("hh",12)).replace("h",12)),a=(a=(a=(a=(a=(a=(a=(a=(a=a.replace("hh",h>12?this.pad(h-12,2):this.pad(h,2))).replace("h",h>12?h-12:h)).replace("HH",this.pad(h,2))).replace("H",h)).replace("mm",this.pad(u,2))).replace("ss",this.pad(p,2))).replace("SSS",this.pad(g,3))).replace("MMMM",o?o.months.wide[l]:null)).replace("MMM",o?o.months.abbreviated[l]:null),-1===s.indexOf("MMM")&&(a=(a=a.replace("MM",this.pad(l+1,2))).replace("M",l+1)),v&&o&&(0===(a=a.replace(" a"," ".concat(h>=12?o.dayPeriods[1]:o.dayPeriods[0]))).indexOf("a")&&(a=a.replace("a"," ".concat(h>=12?o.dayPeriods[1]:o.dayPeriods[0]))),a=a.replace("EEEE",o.days.wide[d])),o&&(a=a.replace("EEEE",o.days.wide[d])),o&&(a=a.replace("EEE",o.days.abbreviated[d])),o&&(a=a.replace("EE",o.days.narrow[d])),(a=(a=(a=(a=a.replace("nnnnn","de")).replace("nnnn","ngày")).replace("t1áng","tháng")).replace("nnn","den")).indexOf("zz")>-1){var b=new Date,y=this.getTimeZone(b,"short"),w=this.getTimeZone(b,"long");a=(a=a.replace("zzzz",w)).replace("zz",y)}return(a=a.replace(/&lrm;|\u200E/gi," ")).trim()}}},newDateObj:function(t){var e=new Date(t);if("safari"===x.browser.name&&"string"==typeof t&&/T|Z/g.test(t)){var i=t.replace(/Z/,"").replace(/T|:/g,"-").split("-");i=i.map((function(t,e){return+(1===e?t-1:t)})),e=h(Date,u(i))}return e},formatHour:function(t,e){var i=this.calendar().dateFormat.timeSeparator,n=this.currentLocale.name;"object"===l(e)&&(n=e.locale||n,i=e.timeSeparator||this.calendar(n).dateFormat.timeSeparator),"string"==typeof t&&-1===t.indexOf(i)&&(i=":");var s=new Date;if("number"==typeof t){var a=t.toString().split(".");s.setHours(a[0]),s.setMinutes(a[1]?60*parseFloat("0.".concat(a[1])):0)}else{var o=t.split(i);s.setHours(o[0]),s.setMinutes(o[1]||0)}var r={date:"hour"};return null!=e&&(null==e?void 0:e.pattern)&&(r.pattern=e.pattern),this.formatDate(s,r)},formatHourRange:function(t,e,i){var n=this.currentLocale.name,s=this.calendar(n).dayPeriods,a=!1;"object"===l(i)&&(n=i.locale||n,s=this.calendar(n).dayPeriods);var o="".concat(H.formatHour(t,i)," - ").concat(H.formatHour(e,i));return null!=i&&(null==i?void 0:i.keepPeriod)||((o.indexOf(":00 AM -")>-1||o.indexOf(":00 PM -")>-1)&&(a=!0),M.count(o,s[0])>1&&(o=o.replace(s[0],"")),M.count(o,s[1])>1&&(o=o.replace(" ".concat(s[1]),""))),o=o.replace("  "," "),a&&(o=o.replace(":00 -"," -")),o},getTimeZone:function(t,e){var i=H.currentLocale.name||"en-US";if("ie"===x.browser.name&&"11"===x.browser.version)return t.toTimeString().match(new RegExp("[A-Z](?!.*[(])","g")).join("");var n=t.toLocaleDateString(i),s=t.toLocaleDateString(i,{timeZoneName:"long"===e?"long":"short"}),a=s.indexOf(n);return a>=0?(s.substring(0,a)+s.substring(a+n.length)).replace(/^[\s,.\-:;]+|[\s,.\-:;]+$/g,""):s},dateToTimeZone:function(t,e,i){return"ie"===x.browser.name&&"11"===x.browser.version?"".concat(t.toLocaleString(H.currentLocale.name)," ").concat(t.toTimeString().match(new RegExp("[A-Z](?!.*[(])","g")).join("")):t.toLocaleString(H.currentLocale.name,{timeZone:e,timeZoneName:i})},dateToUTC:function(t){return new Date(Date.UTC(t.getFullYear(),t.getMonth(),t.getDate(),t.getHours(),t.getMinutes(),t.getSeconds()))},toLocaleString:function(t,e,i,n){if("number"!=typeof t)return"";var s={locale:e||H.currentLocale.name,options:i||void 0},a=t.toLocaleString(s.locale,s.options);if(!/undefined|null/.test(l(n))){var o=this.getSeparator(s.locale,"group");"."!==o&&(a=a.replace(new RegExp(o,"g"),n.toString()))}return a},getSeparator:function(t,e){return Intl.NumberFormat(t).formatToParts(1000.1).find((function(t){return t.type===e})).value},convertNumberToEnglish:function(t){for(var e=["٠","١","٢","٣","٤","٥","٦","٧","٨","٩"],i=["०","१","२","३","४","५","६","७","८","९"],n=["零","壹","貳","叄","肆","伍","陸","柒","捌","玖"],s=["零","壹","贰","叁","肆","伍","陆","柒","捌","玖"],a=["〇","一","二","三","四","五","六","七","八","九"],o=0;o<=9;o++)t=(t=(t=(t=(t=(t=(t=t.replace(e[o],o)).replace("٬","")).replace(",","")).replace(i[o],o)).replace(n[o],o)).replace(s[o],o)).replace(a[o],o),0===o&&(t=t.replace("零",o));return parseFloat(t)},isValidDate:function(t){return"[object Date]"===Object.prototype.toString.call(t)&&!isNaN(t.getTime())},parseDate:function(t,e,i){if(t){var n=e,s=this.currentLocale.name,a=this.calendar();"object"===l(e)&&(s=e.locale||s,n=e.dateFormat||this.calendar(s).dateFormat[n.date]),"object"===l(e)&&e.pattern&&(n=e.dateFormat||e.pattern),"object"===l(e)&&e.calendarName&&e.locale&&(a=this.calendar(e.locale,e.language,e.calendarName)),n||(n=this.calendar(s).dateFormat.short);var o=t;if("0000"!==t&&"000000"!==t&&"00000000"!==t){var r,c;n.pattern&&(n=n.pattern);var d,h,u={},p=n.toLowerCase().indexOf("h")>-1,g=t.toLowerCase().indexOf("z")>-1,f=!!a.dayPeriods.filter((function(t){return t.indexOf(".")>-1})).length&&n.indexOf("a")>-1&&n.indexOf("ah")<0&&n.indexOf("H")<0,m=3===(n.match(/M/g)||[]).length&&a&&a.months&&a.months.abbreviated&&a.months.abbreviated.filter((function(t){return/\./.test(t)})).length;if(p){var v=H.getTimeZone(new Date,"short"),b=H.getTimeZone(new Date,"long");t=(t=t.replace(" ".concat(v),"")).replace(" ".concat(b),"");var y=m?/[T\s:-]/g:/[T\s:.-]/g;n=(n=n.replace(" zzzz","").replace(" zz","")).replace(y,"/").replace(/z/i,""),n=f?n.replace("//","/"):n,t=t.replace(f?/[T\s:-]/g:y,"/").replace(/z/i,"")}n=n.replace(" de "," "),t=t.replace(" de "," "),n=n.replace("ah","a/h"),t=t.replace("午","午/"),n=n.replace(",",""),t=t.replace(",","");var w=n.indexOf("Mdyyyy"),x=n.indexOf("dMyyyy"),C=-1,k=-1;if((w>-1||x>-1)&&(k=(C=w>-1?w:x>-1?x:0)+t.indexOf("/")>-1?t.indexOf("/"):t.length,t="".concat(t.substr(C,k-4),"/").concat(t.substr(k-4,t.length)),t="".concat(t.substr(C,t.indexOf("/")/2),"/").concat(t.substr(t.indexOf("/")/2,t.length))),w>-1&&(n=n.replace("Mdyyyy","M/d/yyyy")),x>-1&&(n=n.replace("dMyyyy","d/M/yyyy")),-1!==n.indexOf(" ")){var T=m?/[\s:]/g:/[\s:.]/g;n=n.replace(T,"/"),t=t.replace(T,"/")}if(n.indexOf("MMMM")>-1&&H.isRTL()&&n&&"MMMM/dd"!==n&&"dd/MMMM"!==n){var A=t.lastIndexOf("/");t=t.substr(0,A-1).replace("/"," ")+t.substr(A)}if(-1===n.indexOf(" ")&&-1===n.indexOf(".")&&-1===n.indexOf("/")&&-1===n.indexOf("-")){-1!==t.indexOf(" ")?t=t.split(" ").join(""):-1!==t.indexOf(".")?t=t.split(".").join(""):-1!==t.indexOf("/")?t=t.split("/").join(""):-1!==t.indexOf("-")&&(t=t.split("-").join(""));var S=n[0],E="",D="";for(d=0,h=n.length;d<h;d++)D+=n[d]!==S?"/".concat(t[d]):t[d],E+=n[d]!==S?"/".concat(n[d]):n[d],d>1&&(S=n[d]);t=D,n=E}r=n.split("/"),c=t.split("/"),1===r.length&&(r=n.split(".")),1===c.length&&(c=t.split(".")),1===r.length&&(r=n.split("-")),1===c.length&&(c=t.split("-")),1===r.length&&(r=n.split(" ")),1===c.length&&(c=t.split(" "));var I=this.getDatePart(r,c,"M","MM","MMM","MMMM"),L=this.getDatePart(r,c,"y","yy","yyyy"),P=!1,M=!1,B=a.dayPeriods[0],O=a.dayPeriods[1];for(f||(B=B.replace(/\./g,""),O=O.replace(/\./g,"")),d=0,h=c.length;d<h;d++){var R="".concat(r[d]),z=c[d],F=parseInt(z,10);P||(P=R.toLowerCase().indexOf("d")>-1);var N=void 0,q=void 0,V=void 0;switch(R){case"d":if(N=new Date(L,I,0).getDate(),F<1||F>31||F>N)return;u.day=z;break;case"dd":if(F<1||F>31||F<10&&"0"!==z.substr(0,1))return;u.day=z;break;case"M":if(F<1||F>12)return;u.month=z-1;break;case"MM":if(F<1||F>12||F<10&&"0"!==z.substr(0,1))return;u.month=z-1;break;case"MMM":q=this.calendar(s).months.abbreviated;for(var W=0;W<q.length;W++)o.indexOf(q[W])>-1&&(u.month=W);break;case"MMMM":V=this.calendar(s).months.wide;for(var j=0;j<V.length;j++)o.indexOf(V[j])>-1&&(u.month=j);break;case"yy":u.year=this.twoToFourDigitYear(z);break;case"y":case"yyyy":u.year=2===z.length?this.twoToFourDigitYear(z):z;break;case"h":if(F<0||F>12)return;u.h=M?u.h:z;break;case"hh":if(F<0||F>12)return;u.h=M?u.h:1===z.length?"0".concat(z):z;break;case"H":if(F<0||F>24)return;u.h=M?u.h:z;break;case"HH":if(F<0||F>24)return;u.h=M?u.h:1===z.length?"0".concat(z):z;break;case"ss":if(F<0||F>60){u.ss=0;break}u.ss=z;break;case"SSS":u.ms=z;break;case"mm":if(F<0||F>60){u.mm=0;break}u.mm=z;break;case"a":!u.h&&r[d+1]&&"h"===r[d+1].toLowerCase().substr(0,1)&&(u.h=c[d+1],M=!0),z.toLowerCase()!==B&&z.toUpperCase()!==B||(u.a="AM",u.h&&(12!==u.h&&"12"!==u.h||(u.h=0))),z.toLowerCase()!==O&&z.toUpperCase()!==O||(u.a="PM",u.h&&u.h<12&&(u.h=parseInt(u.h,10)+12))}}var U=function(t){return t%4==0&&t%100!=0||t%400==0};if(u.return=void 0,u.leapYear=U(u.year),!p||u.h||u.mm){if(!u.year&&0!==u.year&&!i){for(u.isUndefindedYear=!0,d=0,h=r.length;d<h;d++)if(r[d].indexOf("y")>-1&&void 0!==c[d]){u.isUndefindedYear=!1;break}if(u.isUndefindedYear){var G=29===parseInt(u.day,10)&&1===parseInt(u.month,10);if(u.year=G?function(t){for(var e="number"!=typeof t||isNaN(t)?(new Date).getFullYear():t,i=0;i<4&&!U(e);i++)e--;return e}():(new Date).getFullYear(),"islamic-umalqura"===a.name){var _=this.gregorianToUmalqura(new Date(u.year,0,1));u.year=_[0]}}else delete u.year}if(u.year&&2===u.year.length&&(u.year="20".concat(u.year)),u.year=$.trim(u.year),u.day=$.trim(u.day),(""===u.year||u.year&&2!=="".concat(u.year).length&&4!=="".concat(u.year).length)&&delete u.year,!u.month&&0!==u.month&&!i){for(u.isUndefindedMonth=!0,d=0,h=r.length;d<h;d++)if(r[d].indexOf("M")>-1&&void 0!==c[d]){u.isUndefindedMonth=!1;break}u.isUndefindedMonth&&(u.month=(new Date).getMonth())}if(!(u.day||0===u.day||i&&P)){for(u.isUndefindedDay=!0,d=0,h=r.length;d<h;d++)if(r[d].indexOf("d")>-1&&void 0!==c[d]){u.isUndefindedDay=!1;break}u.isUndefindedDay?u.day=1:delete u.day}return p?g?(void 0!==u.h&&(u.return=new Date(Date.UTC(u.year,u.month,u.day,u.h,u.mm))),void 0!==u.ss&&(u.return=new Date(Date.UTC(u.year,u.month,u.day,u.h,u.mm,u.ss))),void 0!==u.ms&&(u.return=new Date(Date.UTC(u.year,u.month,u.day,u.h,u.mm,u.ss,u.ms)))):(void 0!==u.h&&(u.return=new Date(u.year,u.month,u.day,u.h,u.mm)),void 0!==u.ss&&(u.return=new Date(u.year,u.month,u.day,u.h,u.mm,u.ss)),void 0!==u.ms&&(u.return=new Date(u.year,u.month,u.day,u.h,u.mm,u.ss,u.ms))):u.return=new Date(u.year,u.month,u.day),"islamic-umalqura"===a.name?[parseInt(u.year,10),parseInt(u.month,10),parseInt(u.day,10),parseInt(u.h||0,10),parseInt(u.mm||0,10),parseInt(u.ss||0,10),parseInt(u.ms||0,10)]:this.isValidDate(u.return)?u.return:void 0}}}},twoToFourDigitYear:function(t){return parseInt((t>39?"19":"20")+t,10)},getDatePart:function(t,e,i,n,s,a){var o=0;return $.each(e,(function(r){i!==t[r]&&n!==t[r]&&s!==t[r]&&a!==t[r]||(o=e[r])})),o},useLocale:function(t){var e=this.currentLocale.data;if(t&&t.locale&&this.cultures[t.locale]&&(e=this.cultures[t.locale]),t&&t.language&&this.languages[t.language]){var i=E.extend(!0,{},this.currentLocale.data);if(i.calendars)return i.calendars[0]=this.calendar(t.locale||this.currentLocale.name,t.language),i}return e.numbers||(e.numbers=this.numbers()),e},useLanguage:function(t){var e=this.languages[this.currentLanguage.name];if(t&&t.locale){var i=this.remapLanguage(t.locale);e=this.languages[i]}return t&&t.locale&&this.currentLanguage.name!==this.currentLocale.name&&this.languages[this.currentLanguage.name]&&(e=this.languages[this.currentLanguage.name]),t&&t.language&&this.languages[t.language]&&(e=this.languages[t.language]),e},formatNumber:function(t,e){var i,n,s,a=this.useLocale(e),o=e&&e.decimal?e.decimal:a.numbers.decimal,r=e&&void 0!==e.minimumFractionDigits?e.minimumFractionDigits:e&&e.style&&"currency"===e.style?2:e&&e.style&&"percent"===e.style?0:2,l=e&&void 0!==e.maximumFractionDigits?e.maximumFractionDigits:e&&e.style&&("currency"===e.style||"percent"===e.style)?2:e&&e.minimumFractionDigits?e.minimumFractionDigits:3;if(null!=t&&""!==t){if(e&&"integer"===e.style&&(l=0,r=0),e&&"currency"===e.style){var c=e&&e.currencySign?e.currencySign:a.currencySign,d=e&&e.currencyFormat?e.currencyFormat:a.currencyFormat;d||(d="¤#,##0.00"),n=d.replace("¤",c)}if(e&&"percent"===e.style){var h=a.numbers?a.numbers.percentSign:"%";s=(s=a.numbers?a.numbers.percentFormat:"### %").replace("¤",h)}"string"==typeof t&&!1!==(null==e||null==e?void 0:e.parseNumber)&&(t=H.parseNumber(t,e)),t.toString().indexOf("e")>-1&&(t=t.toFixed(l+1)),e&&"percent"===e.style&&(t=(100*t).toFixed(r));var u=this.truncateDecimals(t,r,l,e&&e.round).split("."),p=[3,3];a&&a.numbers&&a.numbers.groupSizes&&(p=a.numbers.groupSizes),e&&e.groupSizes&&(p=e.groupSizes);var g=e&&void 0!==e.group?e.group:a.numbers.group,f=this.expandNumber(u[0],p,g);u[0]=f,i=u.join(o);var m=a&&a.numbers&&a.numbers.minusSign?a.numbers.minusSign:"-",v=i.indexOf(m)>-1;i=i.replace(m,"");var b=function(t){return t.replace(/([.?*+^$[\]\\(){}|-])/g,"\\$1")},y="";return 0===r&&(y=new RegExp("(".concat(b(o),"[0-9]*?)0+$")),i=i.replace(y,"$1")),r>0&&(y=new RegExp("(".concat(b(o),".{").concat(r,"}[0-9]*?)0+$")),i=i.replace(y,"$1")),y=new RegExp("".concat(b(o),"$")),i=i.replace(y,""),e&&"currency"===e.style&&(i=n.replace("###",i)),e&&"percent"===e.style&&(i=s.replace("###",i)),v&&(i=m+i),i}},expandNumber:function(t,e,i){var n=t.length,s=!1;if("-"===t.substr(0,1)&&(n=(t=t.substr(1)).length,s=!0),n<=3)return(s?"-":"")+t;if(0===e[0])return(s?"-":"")+t;var a=t.substr(t.length-e[0]),o=t.substr(0,t.length-e[0]);if(0===e[1])return(s?"-":"")+o+(""===o?"":i)+a;var r=o.split("").reverse().join(""),l=new RegExp(".{1,".concat(e[1],"}"),"g");return(s?"-":"")+r.match(l).join(i).split("").reverse().join("")+i+a},truncateDecimals:function(t,e,i,n){var s=t;s=n?B.round(t,i):B.truncate(t,i);var a=B.decimalPlaces(s);return a<e&&(s=s.toString()+new Array(e-a+1).join("0")),s.toString()},parseNumber:function(t,e){var i,n=this.useLocale(e),s=n.numbers;if(!(i=t))return NaN;"number"==typeof t&&(i=i.toString());var a=e&&void 0!==e.group?e.group:s?s.group:",",o=e&&e.decimal?e.decimal:s?s.decimal:".",r=e&&e.percentSign?e.percentSign:s?s.percentSign:"%",l=e&&e.currencySign?e.currencySign:n.currencySign||"$",c=" "===a||""===a?new RegExp(/\s/g):new RegExp("\\".concat(a),"g");return(i=(i=(i=(i=(i=(i=i.replace(c,"")).replace(o,".")).replace(r,"")).replace(l,"")).replace("$","")).replace(" ","")).length>=18?i:parseFloat(i)},translate:function(t,e){var i=this.useLanguage(e),n=!1,s=!0;if("&nsbp;"===t)return"";if("boolean"==typeof e&&(n=e),"object"===l(e)&&void 0!==e.showAsUndefined&&(n=e.showAsUndefined),"object"===l(e)&&void 0!==e.showBrackets&&(s=e.showBrackets),void 0===i||void 0===i.messages)return n?void 0:"".concat(s?"[":"").concat(t).concat(s?"]":"");if(void 0===i.messages[t]){var a="en";return this.languages&&this.languages[a]&&this.languages[a].messages&&void 0!==this.languages[a].messages[t]?this.languages[a].messages[t].value:n?void 0:"".concat(s?"[":"").concat(t).concat(s?"]":"")}return i.messages[t].value},extendTranslations:function(t,e){if(this.languages[t]){var i=this.languages[t].messages;this.languages[t].messages=E.extend(!0,i,e)}},calendar:function(t,e,i){var n=[];if(this.currentLocale.data.calendars&&!t&&(n=this.currentLocale.data.calendars),e&&e.length>2&&(e=e.substr(0,2)),t&&this.cultures[t]&&(n=this.cultures[t].calendars),i&&n)for(var s=0;s<n.length;s++){var a=n[s];if(a.name===i)return a}if(!n[0])return{dateFormat:{separator:"/",timeSeparator:":",short:"M/d/yyyy",medium:"MMM d, yyyy",long:"MMMM d, yyyy",full:"EEEE, MMMM d, y",month:"MMMM d",year:"MMMM yyyy",timestamp:"h:mm:ss a",datetime:"M/d/yyyy h:mm a"},timeFormat:"HH:mm:ss",dayPeriods:["AM","PM"]};var o=E.extend(!0,{},n[0]);if(e&&t.substr(0,2)!==e){var r=this.defaultLocales.filter((function(t){return t.lang===e}));if(r[0]){var l=this.cultures[r[0].default];o.days=l.calendars[0].days,o.months=l.calendars[0].months,o.dayPeriods=l.calendars[0].dayPeriods}}return o},numbers:function(){return this.currentLocale.data.numbers?this.currentLocale.data.numbers:{percentSign:"%",percentFormat:"### %",minusSign:"-",decimal:".",group:","}},pad:function(t,e,i){return i=i||"0",(t+="").length>=e?t:new Array(e-t.length+1).join(i)+t},isRTL:function(){return!!this.currentLanguage&&"right-to-left"===this.currentLanguage.direction},isIslamic:function(t){var e;return"islamic-umalqura"===(null===(e=this.calendar(t))||void 0===e||null==e?void 0:e.name)},toUpperCase:function(t){return"function"==typeof this.currentLocale.data.toUpperCase?this.currentLocale.data.toUpperCase(t):t.toLocaleUpperCase()},toLowerCase:function(t){return"function"==typeof this.currentLocale.data.toLowerCase?this.currentLocale.data.toLowerCase(t):t.toString().toLocaleLowerCase()},capitalize:function(t){return this.toUpperCase(t.charAt(0))+t.slice(1)},capitalizeWords:function(t){for(var e=t.split(" "),i=0;i<e.length;i++)e[i]=this.capitalize(e[i]);return e.join(" ")},gregorianToUmalqura:function(t){if("function"!=typeof t.getMonth)return null;var e=function(t){for(var e=t-24e5+.5,i=0,n=0;n<z.length&&!(z[n]>e);n++)i++;var s=i+15292,a=Math.floor((s-1)/12);return{year:a+1,month:s-12*a-1,day:e-z[i-1]+1}}(function(t,e,i){t<0&&t++,e<3&&(e+=12,t--);var n=Math.floor(t/100),s=2-n+Math.floor(n/4);return Math.floor(365.25*(t+4716))+Math.floor(30.6001*(e+1))+i+s-1524.5}(t.getFullYear(),t.getMonth()+1,t.getDate()));return[e.year,e.month,e.day,t.getHours(),t.getMinutes(),t.getSeconds(),t.getMilliseconds()]},umalquraToGregorian:function(t,e,i){var n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:0,s=arguments.length>4&&void 0!==arguments[4]?arguments[4]:0,a=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0,o=arguments.length>6&&void 0!==arguments[6]?arguments[6]:0;Array.isArray(t)&&(e=t[1],i=t[2]||0,n=t[3]||0,s=t[4]||0,a=t[5]||0,o=t[6]||0,t=t[0]);var r=function(t){return"number"==typeof t&&!isNaN(t)};if(!r(t)||!r(e)||!r(i))return null;var l=function(t){var e=Math.floor(t+.5),i=Math.floor((e-1867216.25)/36524.25),n=(i=e+1+i-Math.floor(i/4))+1524,s=Math.floor((n-122.1)/365.25),a=Math.floor(365.25*s),o=Math.floor((n-a)/30.6001),r=o-(o>13.5?13:1),l=s-(r>2.5?4716:4715);return l<=0&&l--,{year:l,month:r-1,day:n-a-Math.floor(30.6001*o)}}(i+z[12*(t-1)+(e+1)-15292-1]-1+24e5-.5);return new Date(l.year,l.month,l.day,n,s,a,o)},flipIconsHorizontally:function(){var t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0],e=["attach","bottom-aligned","bullet-list","cancel","cart","collapse-app-tray","cut","document","drilldown","duplicate","expand-app-tray","export","first-page","folder","import","last-page","launch","left-align","left-text-align","left-arrow","new-document","next-page","number-list","paste","previous-page","quote","redo","refresh","right-align","right-arrow","right-text-align","save","search-folder","search-list","search","send","tack","tree-collapse","tree-expand","undo","unlocked","add-grid-record","add-grid-row","additional-help","bubble","bullet-steps","cascade","change-font","clear-screen","script","clockwise-90","close-cancel","close-save","contacts","copy-from","copy-mail","copy-url","counter-clockwise-90","create-report","delete-grid-record","delete-grid-row","display","employee-directory","export-2","export-to-pdf","generate-key","get-more-rows","group-selection","headphones","help","helper-list-select","history","invoice-released","language","logout","key","lasso","line-bar-chart","line-chart","new-expense-report","new-payment-request","new-time-sheet","new-travel-plan","no-attachment","no-comment","no-filter","overlay-line","pdf-file","phone","payment-request","pie-chart","queries","quick-access","refresh-current","restore-user","run-quick-access","save-close","save-new","search-results-history","select","send-submit","show-last-x-days","special-item","stacked","timesheet","unsubscribe","update-preview","zoom-100","zoom-in","zoom-out","caret-left","caret-right"];$("svg").each((function(){var i=$(this).getIconName();t&&i&&-1!==$.inArray(i,e)&&0===$(this).closest(".monthview").length?$(this).addClass("icon-rtl-rotate"):$(this).removeClass("icon-rtl-rotate")}))}};function q(t){if("function"!=typeof t){if("string"==typeof t)return t;throw new Error("".concat(l(t)," was provided where a function was expected."))}if("string"==typeof t.name)return"".concat(t.name,"()");var e=/^function\s+([\w$]+)\s*\(/.exec(t.toString());return e?"".concat(e[1],"()"):"(anonymous function)"}function V(t,e){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"";if("object"===("undefined"==typeof console?"undefined":l(console))){var n=q(t),s=q(e);i.length&&(i=" (".concat(i,")")),console.warn("IDS Enterprise".concat(i,': "').concat(s,'" is deprecated. Please use "').concat(n,'" instead.'))}}function W(t){"object"===("undefined"==typeof console?"undefined":l(console))&&console.warn('IDS Enterprise: "'.concat(t,'" is deprecated and will be later removed. Please adjust your code accordingly.'))}function j(t,e){var i=function(){V(t,e);for(var i=arguments.length,n=new Array(i),s=0;s<i;s++)n[s]=arguments[s];t.apply(this,n)};return i.prototype=t.prototype,i}function U(t){return this.element=$(t),this}U.prototype={get url(){return window.location.href.replace(window.location.hash,"")},get href(){return this.element[0].getAttribute("href")},getCurrentURL:function(){return V("url","getCurrentURL"),this.url},getBaseURL:function(t){return this.element.length&&this.href&&"/"!==this.href?t?0===t.indexOf("/")?t:(t=(-1===t.indexOf("#")?"#":"")+t,this.url+t):this.url:t||""}};var G=new U($("base[href]"));$.getBaseURL=function(t){return G.getBaseURL(t)};var _={noAutoStart:!1},Y={};function K(){return window.performance&&window.performance.now?window.performance.now():(new Date).getTime()}function X(t){if(this.id=t.id,this.duration=t.duration||-1,this.duration<1&&("string"!=typeof this.id||!this.id.length))throw new Error("cannot build a RenderLoopItem with no duration and no namespace");return this.updateDuration=t.updateDuration||1,this.setFuncs(t),this.paused=!1,this.elapsedTime=0,this.startTime=K(),this}function Q(t){return this.items=[],this.element=$("body"),this.settings=E.mergeSettings(null,t,_),!0!==this.settings.noAutoStart&&this.start(),this}"object"===l(window.SohoConfig)&&"object"===l(window.SohoConfig.renderLoop)&&(Y=E.extend({},_,window.SohoConfig.renderLoop)),X.prototype={setFuncs:function(t){if("function"!=typeof t.updateCallback&&"function"!=typeof t.timeoutCallback)throw new Error("cannot register callback to RenderLoop because callback is not a function");"function"==typeof t.updateCallback&&(this.updateCallback=t.updateCallback),"function"==typeof t.timeoutCallback&&(this.timeoutCallback=t.timeoutCallback)},pause:function(){this.paused=!0},resume:function(){this.paused=!1},destroy:function(t){t&&(this.noTimeout=!0),this.doRemoveOnNextTick=!0}},Q.prototype={start:function(){this.doLoop=!0,this.startTime=K();var t,e,i=this,n=K();!function s(){i.doLoop&&(t=K(),e=(t-n)/1e3,i.items.forEach((function(s){var a;if(s.doRemoveOnNextTick)i.remove(s);else if(s.paused||s.elapsedTime++,"number"==typeof s.duration&&s.duration>-1&&(s.startTime||(s.startTime=t),s.elapsedTime>=s.duration))s.destroy();else if("function"==typeof s.updateCallback){if(s.updateDuration&&s.updateDuration>1&&(isNaN(s.timeUntilNextUpdate)&&(s.timeUntilNextUpdate=s.updateDuration),s.timeUntilNextUpdate>0))return void--s.timeUntilNextUpdate;a=[s,{last:n,delta:e,now:t}],s.updateCallback.apply(null,a)}})),n=t,requestAnimationFrame(s))}()},stop:function(){this.doLoop=!1},totalDuration:function(){return K()-this.startTime},queue:function(){return this.items},buildRenderLoopItem:function(t,e,i,n){var s="string"!=typeof n||!n.length;if("string"==typeof i)if(s)n=i,i=-1,s=!1;else{var a=Number(i);isNaN(a)||(i=a)}else"number"!=typeof i&&(i=-1);return"string"==typeof n&&n.length||(n=""),new X({id:n,updateCallback:t,timeoutCallback:e,duration:i})},register:function(t,e,i,n){return t instanceof X||(t=this.buildRenderLoopItem(t,e,i,n)),this.items.push(t),t},unregister:function(t,e){if("function"!=typeof t&&"string"!=typeof t&&"string"!=typeof e)throw new Error("must provide either a callback function or a namespace string to remove an entry from the RenderLoop queue.");return"string"==typeof t&&(e=t,t=void 0),this.remove({cb:t,id:e})},getFromQueue:function(t,e){var i;return"string"==typeof callback&&(e=t,t=void 0),"function"==typeof callback?this.items.forEach((function(e){return"".concat(e.updateCallback)!=="".concat(t)||(i=e,!1)})):"string"==typeof e&&this.items.forEach((function(t){return t.id!==e||(i=t,!1)})),i},remove:function(t){var e;return t instanceof X?(e=t,this.items=this.items.filter((function(e){return e!==t}))):"function"==typeof t.updateCallback?this.items=this.items.filter((function(i){return"".concat(i.updateCallback)!=="".concat(t.updateCallback)||(e=i,!1)})):"string"==typeof t.id&&(this.items=this.items.filter((function(i){return i.id!==t.id||(e=i,!1)}))),"function"!=typeof e.timeoutCallback||e.noTimeout||e.timeoutCallback.apply(null,e),this.element.triggerHandler("remove.renderLoop",[e]),e},pause:function(t,e){if("function"!=typeof t&&"string"!=typeof t&&"string"!=typeof e)throw new Error("must provide either a callback function or a namespace string to pause an entry in the RenderLoop queue.");var i=this.getFromQueue(t,e);return i.pause(),i},resume:function(t,e){if("function"!=typeof t&&"string"!=typeof t&&"string"!=typeof e)throw new Error("must provide either a callback function or a namespace string to pause an entry in the RenderLoop queue.");var i=this.getFromQueue(t,e);return i.resume(),i}};var J=new Q(Y),Z="keyboard";function tt(){this.pressedKeys=new Map,this.init()}tt.prototype={init:function(){var t=this;$(document).on("keydown.".concat(Z),(function(e){t.press(e.key),t.announceKeys()})).on("keyup.".concat(Z),(function(e){t.unpress(e.key),t.announceKeys()}))},announceKeys:function(){var t=[];this.pressedKeys.forEach((function(e,i){t.push(i)})),$("body").triggerHandler("keys",[t])},press:function(t){return this.pressedKeys.set("".concat(t),!0)},unpress:function(t){return this.pressedKeys.delete("".concat(t))},isPressed:function(t){return this.pressedKeys.has("".concat(t))}};var et=new tt,it="modalmanager";function nt(t){return void 0!==t&&t.element instanceof $&&"function"==typeof t.destroy}function st(){return this.modals=[],this.handleEvents(),this}st.prototype={get modalFadeDuration(){return 10},get currentlyOpen(){return this.modals.filter((function(t){return t.visible}))},get currentlyActive(){var t;return this.modals.forEach((function(e){e.active&&(t=e)})),t},set currentlyActive(t){if(!nt(t))throw new Error("Cannot set the provided Modal API to currently displayed.");this.modals.forEach((function(e,i){e.active=$(e.element).is(t.element),e.active?e.element[0].style.zIndex="".concat(1020+i):e.visible?e.element[0].style.zIndex="".concat(920+i):e.element[0].style.zIndex=""})),this.refresh()},get last(){var t,e=this.currentlyOpen,i=e.length;return i&&(t=e[i-1]),t},render:function(){var t=document.createDocumentFragment(),e=document.createElement("div"),i=document.createElement("div");e.id="ids-modal-root",e.classList.add("modal-page-container"),e.setAttribute("aria-hidden",!0),t.appendChild(e),i.classList.add("overlay"),i.setAttribute("aria-hidden",!0),e.appendChild(i),document.body.appendChild(e),this.rootElem=e,this.overlayElem=i},refresh:function(){var t=this.currentlyActive;t||(t=this.activateLast()),this.checkOverlayVisibility(),t?this.showContainers(t):this.hideContainers(t)},showContainers:function(){this.rootElem.removeAttribute("aria-hidden"),this.overlayElem.removeAttribute("aria-hidden"),$("body")[0].classList.add("modal-engaged")},hideContainers:function(){this.rootElem.setAttribute("aria-hidden",!0),this.overlayElem.setAttribute("aria-hidden",!0),$("body")[0].classList.remove("modal-engaged")},register:function(t){if(!nt(t))throw new Error("The provided API is not a Modal API, and cannot be registered.");this.checkRootElements(),this.modals.filter((function(e){return $(e.element).is(t)}))>0||this.modals.push(t)},unregister:function(t){if(!nt(t))throw new Error("The provided API is not a Modal API, and cannot be unregistered.");this.checkRootElements(),this.modals=this.modals.filter((function(e){return!$(e.element).is(t.element)}))},checkOverlayVisibility:function(){var t=0;this.currentlyActive&&(t=this.currentlyActive.settings.overlayOpacity),this.overlayElem.style.background="rgba(0, 0, 0, ".concat(t?"".concat(t):"",")")},checkRootElements:function(){var t=document.querySelector("#ids-modal-root");if(t instanceof HTMLElement){this.rootElem=t;var e=t.querySelector(".overlay");e instanceof HTMLElement&&(this.overlayElem=e)}else this.rootElem instanceof HTMLElement?document.body.appendChild(this.rootElem):this.render()},close:function(t){var e,i=arguments.length>1&&void 0!==arguments[1]&&arguments[1];(e=t&&(t instanceof HTMLElement||t instanceof $&&t.length)?$(t).data("modal"):this.last)&&(i&&(e.isCancelled=!0),e.close())},closeAll:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.modals.forEach((function(e){t&&(e.isCancelled=!0),e.close(void 0,!0)})),this.refresh()},destroyAll:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.modals.forEach((function(e){(!t||e.capAPI)&&(e.close(void 0,!0),e.destroy())})),this.refresh()},closeLast:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.close(void 0,t)},activateLast:function(){var t=this.last;return t&&(this.currentlyActive=t),t},findById:function(t){if(t)return this.modals.filter((function(e){return e.id===t}))[0]||void 0},handleEvents:function(){var t=this;this.hasEstablishedEvents&&this.teardown(),$(document).on("keydown.".concat(it),(function(e){var i=$(e.target).parents(".modal");switch(e.which||e.keyCode){case 27:t.close(i,!0);break;case 9:t.setModalFocus(e.shiftKey?"last":"first",e)}}));var e=this;this.preRendered?this.checkRootElements():$(document).ready((function(){e.checkRootElements(),e.preRendered=!0})),this.hasEstablishedEvents=!0},setModalFocus:function(t,e){var i=this.currentlyActive;i&&(!i.isFocused||$(i.focusableElems.last).is(document.activeElement)&&"first"===t||$(i.focusableElems.first).is(document.activeElement)&&"last"===t)&&(e.preventDefault(),i.setFocus(t))},teardown:function(){$(document).off("keydown.".concat(it)),delete this.hasEstablishedEvents,this.preRendered&&(this.rootElem.parentNode.remove(this.rootElem),delete this.overlayElem,delete this.rootElem,delete this.preRendered)}};var at=new st;function ot(t){return void 0!==t&&t.element instanceof $&&"function"==typeof t.destroy}function rt(){return this.notifications=[],this}rt.prototype={register:function(t){if(!ot(t))throw new Error("The provided object has no Notification API, and cannot be registered.");this.notifications.filter((function(e){return $(e.element).is(t)}))>0||this.notifications.push(t)},unregister:function(t){if(!ot(t))throw new Error("The provided object has no Notification API, and cannot be registered.");var e=this.notifications.findIndex((function(e){return $(e.element).is(t)}));e>=0&&this.notifications.splice(e,1)},closeById:function(t){if(this.notifications.length>0){var e=this.notifications.findIndex((function(e){return e.settings.id===t}));e>=0&&(this.close(this.notifications[e]),this.notifications.splice(e,1))}},closeLatest:function(){this.notifications.length>0&&this.close(this.notifications.pop())},closeAll:function(){for(;this.notifications.length>0;)this.close(this.notifications.pop())},close:function(t){var e=t.element;t.close(),e&&this.notifications.length>0&&e.data("notification",this.notifications[0])}};var lt=new rt,ct={themeName:"theme-classic",palette:{amber:{10:{name:"ids-color-palette-amber-10",value:"#fbe9bf"},20:{name:"ids-color-palette-amber-20",value:"#f8e09c"},30:{name:"ids-color-palette-amber-30",value:"#f7d475"},40:{name:"ids-color-palette-amber-40",value:"#f4c951"},50:{name:"ids-color-palette-amber-50",value:"#f2bc41"},60:{name:"ids-color-palette-amber-60",value:"#efa836"},70:{name:"ids-color-palette-amber-70",value:"#ee9a36"},80:{name:"ids-color-palette-amber-80",value:"#e4882b"},90:{name:"ids-color-palette-amber-90",value:"#db7726"},100:{name:"ids-color-palette-amber-100",value:"#d66221"}},amethyst:{10:{name:"ids-color-palette-amethyst-10",value:"#ede3fc"},20:{name:"ids-color-palette-amethyst-20",value:"#daccec"},30:{name:"ids-color-palette-amethyst-30",value:"#c7b4db"},40:{name:"ids-color-palette-amethyst-40",value:"#b59eca"},50:{name:"ids-color-palette-amethyst-50",value:"#a38db7"},60:{name:"ids-color-palette-amethyst-60",value:"#9279a6"},70:{name:"ids-color-palette-amethyst-70",value:"#806594"},80:{name:"ids-color-palette-amethyst-80",value:"#6e5282"},90:{name:"ids-color-palette-amethyst-90",value:"#5d3e70"},100:{name:"ids-color-palette-amethyst-100",value:"#4b2a5e"}},azure:{10:{name:"ids-color-palette-azure-10",value:"#c8e9f4"},20:{name:"ids-color-palette-azure-20",value:"#addbeb"},30:{name:"ids-color-palette-azure-30",value:"#8dc9e6"},40:{name:"ids-color-palette-azure-40",value:"#69b5dd"},50:{name:"ids-color-palette-azure-50",value:"#54a1d3"},60:{name:"ids-color-palette-azure-60",value:"#368ac0"},70:{name:"ids-color-palette-azure-70",value:"#2578a9"},80:{name:"ids-color-palette-azure-80",value:"#1d5f8a"},90:{name:"ids-color-palette-azure-90",value:"#134d71"},100:{name:"ids-color-palette-azure-100",value:"#133c59"}},emerald:{10:{name:"ids-color-palette-emerald-10",value:"#d5f6c0"},20:{name:"ids-color-palette-emerald-20",value:"#c3e8ac"},30:{name:"ids-color-palette-emerald-30",value:"#afdc91"},40:{name:"ids-color-palette-emerald-40",value:"#9cce7c"},50:{name:"ids-color-palette-emerald-50",value:"#89bf65"},60:{name:"ids-color-palette-emerald-60",value:"#76b051"},70:{name:"ids-color-palette-emerald-70",value:"#66a140"},80:{name:"ids-color-palette-emerald-80",value:"#56932e"},90:{name:"ids-color-palette-emerald-90",value:"#488421"},100:{name:"ids-color-palette-emerald-100",value:"#397514"}},graphite:{10:{name:"ids-color-palette-graphite-10",value:"#f0f0f0"},20:{name:"ids-color-palette-graphite-20",value:"#d8d8d8"},30:{name:"ids-color-palette-graphite-30",value:"#bdbdbd"},40:{name:"ids-color-palette-graphite-40",value:"#999999"},50:{name:"ids-color-palette-graphite-50",value:"#737373"},60:{name:"ids-color-palette-graphite-60",value:"#5c5c5c"},70:{name:"ids-color-palette-graphite-70",value:"#454545"},80:{name:"ids-color-palette-graphite-80",value:"#383838"},90:{name:"ids-color-palette-graphite-90",value:"#292929"},100:{name:"ids-color-palette-graphite-100",value:"#1a1a1a"}},ruby:{10:{name:"ids-color-palette-ruby-10",value:"#f6caca"},20:{name:"ids-color-palette-ruby-20",value:"#eb9d9d"},30:{name:"ids-color-palette-ruby-30",value:"#de8181"},40:{name:"ids-color-palette-ruby-40",value:"#d26d6d"},50:{name:"ids-color-palette-ruby-50",value:"#c65f5f"},60:{name:"ids-color-palette-ruby-60",value:"#b94e4a"},70:{name:"ids-color-palette-ruby-70",value:"#ad4242"},80:{name:"ids-color-palette-ruby-80",value:"#a13030"},90:{name:"ids-color-palette-ruby-90",value:"#941e1e"},100:{name:"ids-color-palette-ruby-100",value:"#880e0e"}},slate:{10:{name:"ids-color-palette-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-slate-40",value:"#888B94"},50:{name:"ids-color-palette-slate-50",value:"#656871"},60:{name:"ids-color-palette-slate-60",value:"#50535A"},70:{name:"ids-color-palette-slate-70",value:"#414247"},80:{name:"ids-color-palette-slate-80",value:"#313236"},90:{name:"ids-color-palette-slate-90",value:"#212224"},100:{name:"ids-color-palette-slate-100",value:"#1C1819"}},"classic-slate":{10:{name:"ids-color-palette-classic-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-classic-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-classic-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-classic-slate-40",value:"#888B94"},50:{name:"ids-color-palette-classic-slate-50",value:"#656871"},60:{name:"ids-color-palette-classic-slate-60",value:"#50535A"},70:{name:"ids-color-palette-classic-slate-70",value:"#414247"},80:{name:"ids-color-palette-classic-slate-80",value:"#313236"},90:{name:"ids-color-palette-classic-slate-90",value:"#212224"},100:{name:"ids-color-palette-classic-slate-100",value:"#1C1819"}},turquoise:{10:{name:"ids-color-palette-turquoise-10",value:"#c0ede3"},20:{name:"ids-color-palette-turquoise-20",value:"#a9e1d6"},30:{name:"ids-color-palette-turquoise-30",value:"#8ed1c6"},40:{name:"ids-color-palette-turquoise-40",value:"#7cc0b5"},50:{name:"ids-color-palette-turquoise-50",value:"#69ada3"},60:{name:"ids-color-palette-turquoise-60",value:"#579e95"},70:{name:"ids-color-palette-turquoise-70",value:"#448d83"},80:{name:"ids-color-palette-turquoise-80",value:"#317c73"},90:{name:"ids-color-palette-turquoise-90",value:"#206b62"},100:{name:"ids-color-palette-turquoise-100",value:"#0e5b52"}},white:{name:"ids-color-palette-white",value:"#ffffff",paletteName:"#ffffff"},black:{name:"ids-color-palette-black",value:"#000000",paletteName:"#000000"}},status:{base:{name:"ids-color-status-base",value:"#2578a9",paletteName:"{ids.color.brand.primary.alt.value}"},caution:{name:"ids-color-status-caution",value:"#ffd726",paletteName:"#ffd726"},danger:{name:"ids-color-status-danger",value:"#e84f4f",paletteName:"#e84f4f"},error:{name:"ids-color-status-error",value:"#e84f4f",paletteName:"#e84f4f"},success:{name:"ids-color-status-success",value:"#80ce4d",paletteName:"#80ce4d"},warning:{name:"ids-color-status-warning",value:"#ff9426",paletteName:"#ff9426"},info:{name:"ids-color-status-info",value:"#2578a9",paletteName:"{ids.color.palette.azure.70.value}"},status01:{name:"ids-color-status-status-01",value:"#e84f4f",paletteName:"#e84f4f"},status02:{name:"ids-color-status-status-02",value:"#e84f4f",paletteName:"#e84f4f"},status03:{name:"ids-color-status-status-03",value:"#ffd726",paletteName:"#ffd726"},status04:{name:"ids-color-status-status-04",value:"#80ce4d",paletteName:"#80ce4d"},status05:{name:"ids-color-status-status-05",value:"#2578a9",paletteName:"{ids.color.palette.azure.70.value}"}},brand:{primary:{lighter:{name:"ids-color-brand-primary-lighter",value:"#69b5dd"},base:{name:"ids-color-brand-primary-base",value:"#368ac0"},alt:{name:"ids-color-brand-primary-alt",value:"#2578a9"},contrast:{name:"ids-color-brand-primary-contrast",value:"#ffffff"}},secondary:{lighter:{name:"ids-color-brand-secondary-lighter",value:"#d8d8d8"},base:{name:"ids-color-brand-secondary-base",value:"#bdbdbd"},alt:{name:"ids-color-brand-secondary-alt",value:"#999999"},contrast:{name:"ids-color-brand-secondary-contrast",value:"#383838"}}},components:{body:{primary:{background:{name:"body-color-primary-background",value:"#f0f0f0"},font:{name:"body-color-primary-font",value:"#1a1a1a"}},secondary:{background:{name:"body-color-secondary-background",value:"#ffffff"}}}}},dt={themeName:"theme-classic-dark",palette:{amber:{10:{name:"ids-color-palette-amber-10",value:"#fbe9bf"},20:{name:"ids-color-palette-amber-20",value:"#f8e09c"},30:{name:"ids-color-palette-amber-30",value:"#f7d475"},40:{name:"ids-color-palette-amber-40",value:"#f4c951"},50:{name:"ids-color-palette-amber-50",value:"#f2bc41"},60:{name:"ids-color-palette-amber-60",value:"#efa836"},70:{name:"ids-color-palette-amber-70",value:"#ee9a36"},80:{name:"ids-color-palette-amber-80",value:"#e4882b"},90:{name:"ids-color-palette-amber-90",value:"#db7726"},100:{name:"ids-color-palette-amber-100",value:"#d66221"}},amethyst:{10:{name:"ids-color-palette-amethyst-10",value:"#ede3fc"},20:{name:"ids-color-palette-amethyst-20",value:"#daccec"},30:{name:"ids-color-palette-amethyst-30",value:"#c7b4db"},40:{name:"ids-color-palette-amethyst-40",value:"#b59eca"},50:{name:"ids-color-palette-amethyst-50",value:"#a38db7"},60:{name:"ids-color-palette-amethyst-60",value:"#9279a6"},70:{name:"ids-color-palette-amethyst-70",value:"#806594"},80:{name:"ids-color-palette-amethyst-80",value:"#6e5282"},90:{name:"ids-color-palette-amethyst-90",value:"#5d3e70"},100:{name:"ids-color-palette-amethyst-100",value:"#4b2a5e"}},azure:{10:{name:"ids-color-palette-azure-10",value:"#c8e9f4"},20:{name:"ids-color-palette-azure-20",value:"#addbeb"},30:{name:"ids-color-palette-azure-30",value:"#8dc9e6"},40:{name:"ids-color-palette-azure-40",value:"#69b5dd"},50:{name:"ids-color-palette-azure-50",value:"#54a1d3"},60:{name:"ids-color-palette-azure-60",value:"#368ac0"},70:{name:"ids-color-palette-azure-70",value:"#2578a9"},80:{name:"ids-color-palette-azure-80",value:"#1d5f8a"},90:{name:"ids-color-palette-azure-90",value:"#134d71"},100:{name:"ids-color-palette-azure-100",value:"#133c59"}},emerald:{10:{name:"ids-color-palette-emerald-10",value:"#d5f6c0"},20:{name:"ids-color-palette-emerald-20",value:"#c3e8ac"},30:{name:"ids-color-palette-emerald-30",value:"#afdc91"},40:{name:"ids-color-palette-emerald-40",value:"#9cce7c"},50:{name:"ids-color-palette-emerald-50",value:"#89bf65"},60:{name:"ids-color-palette-emerald-60",value:"#76b051"},70:{name:"ids-color-palette-emerald-70",value:"#66a140"},80:{name:"ids-color-palette-emerald-80",value:"#56932e"},90:{name:"ids-color-palette-emerald-90",value:"#488421"},100:{name:"ids-color-palette-emerald-100",value:"#397514"}},graphite:{10:{name:"ids-color-palette-graphite-10",value:"#f0f0f0"},20:{name:"ids-color-palette-graphite-20",value:"#d8d8d8"},30:{name:"ids-color-palette-graphite-30",value:"#bdbdbd"},40:{name:"ids-color-palette-graphite-40",value:"#999999"},50:{name:"ids-color-palette-graphite-50",value:"#737373"},60:{name:"ids-color-palette-graphite-60",value:"#5c5c5c"},70:{name:"ids-color-palette-graphite-70",value:"#454545"},80:{name:"ids-color-palette-graphite-80",value:"#383838"},90:{name:"ids-color-palette-graphite-90",value:"#292929"},100:{name:"ids-color-palette-graphite-100",value:"#1a1a1a"}},ruby:{10:{name:"ids-color-palette-ruby-10",value:"#f6caca"},20:{name:"ids-color-palette-ruby-20",value:"#eb9d9d"},30:{name:"ids-color-palette-ruby-30",value:"#de8181"},40:{name:"ids-color-palette-ruby-40",value:"#d26d6d"},50:{name:"ids-color-palette-ruby-50",value:"#c65f5f"},60:{name:"ids-color-palette-ruby-60",value:"#b94e4a"},70:{name:"ids-color-palette-ruby-70",value:"#ad4242"},80:{name:"ids-color-palette-ruby-80",value:"#a13030"},90:{name:"ids-color-palette-ruby-90",value:"#941e1e"},100:{name:"ids-color-palette-ruby-100",value:"#880e0e"}},slate:{10:{name:"ids-color-palette-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-slate-40",value:"#888B94"},50:{name:"ids-color-palette-slate-50",value:"#656871"},60:{name:"ids-color-palette-slate-60",value:"#50535A"},70:{name:"ids-color-palette-slate-70",value:"#414247"},80:{name:"ids-color-palette-slate-80",value:"#313236"},90:{name:"ids-color-palette-slate-90",value:"#212224"},100:{name:"ids-color-palette-slate-100",value:"#1C1819"}},"classic-slate":{10:{name:"ids-color-palette-classic-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-classic-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-classic-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-classic-slate-40",value:"#888B94"},50:{name:"ids-color-palette-classic-slate-50",value:"#656871"},60:{name:"ids-color-palette-classic-slate-60",value:"#50535A"},70:{name:"ids-color-palette-classic-slate-70",value:"#414247"},80:{name:"ids-color-palette-classic-slate-80",value:"#313236"},90:{name:"ids-color-palette-classic-slate-90",value:"#212224"},100:{name:"ids-color-palette-classic-slate-100",value:"#1C1819"}},turquoise:{10:{name:"ids-color-palette-turquoise-10",value:"#c0ede3"},20:{name:"ids-color-palette-turquoise-20",value:"#a9e1d6"},30:{name:"ids-color-palette-turquoise-30",value:"#8ed1c6"},40:{name:"ids-color-palette-turquoise-40",value:"#7cc0b5"},50:{name:"ids-color-palette-turquoise-50",value:"#69ada3"},60:{name:"ids-color-palette-turquoise-60",value:"#579e95"},70:{name:"ids-color-palette-turquoise-70",value:"#448d83"},80:{name:"ids-color-palette-turquoise-80",value:"#317c73"},90:{name:"ids-color-palette-turquoise-90",value:"#206b62"},100:{name:"ids-color-palette-turquoise-100",value:"#0e5b52"}},white:{name:"ids-color-palette-white",value:"#ffffff",paletteName:"#ffffff"},black:{name:"ids-color-palette-black",value:"#000000",paletteName:"#000000"}},status:{base:{name:"ids-color-status-base",value:"#2578a9",paletteName:"{ids.color.brand.primary.alt.value}"},caution:{name:"ids-color-status-caution",value:"#ffd726",paletteName:"#ffd726"},danger:{name:"ids-color-status-danger",value:"#e84f4f",paletteName:"#e84f4f"},error:{name:"ids-color-status-error",value:"#e84f4f",paletteName:"#e84f4f"},success:{name:"ids-color-status-success",value:"#80ce4d",paletteName:"#80ce4d"},warning:{name:"ids-color-status-warning",value:"#ff9426",paletteName:"#ff9426"},info:{name:"ids-color-status-info",value:"#2578a9",paletteName:"{ids.color.palette.azure.70.value}"},status01:{name:"ids-color-status-status-01",value:"#e84f4f",paletteName:"#e84f4f"},status02:{name:"ids-color-status-status-02",value:"#e84f4f",paletteName:"#e84f4f"},status03:{name:"ids-color-status-status-03",value:"#ffd726",paletteName:"#ffd726"},status04:{name:"ids-color-status-status-04",value:"#80ce4d",paletteName:"#80ce4d"},status05:{name:"ids-color-status-status-05",value:"#2578a9",paletteName:"{ids.color.palette.azure.70.value}"}},brand:{primary:{lighter:{name:"ids-color-brand-primary-lighter",value:"#888B94"},base:{name:"ids-color-brand-primary-base",value:"#50535A"},alt:{name:"ids-color-brand-primary-alt",value:"#2578a9"},contrast:{name:"ids-color-brand-primary-contrast",value:"#ffffff"}},secondary:{lighter:{name:"ids-color-brand-secondary-lighter",value:"#888B94"},base:{name:"ids-color-brand-secondary-base",value:"#656871"},alt:{name:"ids-color-brand-secondary-alt",value:"#414247"},contrast:{name:"ids-color-brand-secondary-contrast",value:"#212224"}}},components:{body:{primary:{background:{name:"body-color-primary-background",value:"#313236"},font:{name:"body-color-primary-font",value:"#ffffff"}},secondary:{background:{name:"body-color-secondary-background",value:"#414247"}}}}},ht={themeName:"theme-classic-contrast",palette:{amber:{10:{name:"ids-color-palette-amber-10",value:"#fbe9bf"},20:{name:"ids-color-palette-amber-20",value:"#f8e09c"},30:{name:"ids-color-palette-amber-30",value:"#f7d475"},40:{name:"ids-color-palette-amber-40",value:"#f4c951"},50:{name:"ids-color-palette-amber-50",value:"#f2bc41"},60:{name:"ids-color-palette-amber-60",value:"#efa836"},70:{name:"ids-color-palette-amber-70",value:"#ee9a36"},80:{name:"ids-color-palette-amber-80",value:"#e4882b"},90:{name:"ids-color-palette-amber-90",value:"#db7726"},100:{name:"ids-color-palette-amber-100",value:"#d66221"}},amethyst:{10:{name:"ids-color-palette-amethyst-10",value:"#ede3fc"},20:{name:"ids-color-palette-amethyst-20",value:"#daccec"},30:{name:"ids-color-palette-amethyst-30",value:"#c7b4db"},40:{name:"ids-color-palette-amethyst-40",value:"#b59eca"},50:{name:"ids-color-palette-amethyst-50",value:"#a38db7"},60:{name:"ids-color-palette-amethyst-60",value:"#9279a6"},70:{name:"ids-color-palette-amethyst-70",value:"#806594"},80:{name:"ids-color-palette-amethyst-80",value:"#6e5282"},90:{name:"ids-color-palette-amethyst-90",value:"#5d3e70"},100:{name:"ids-color-palette-amethyst-100",value:"#4b2a5e"}},azure:{10:{name:"ids-color-palette-azure-10",value:"#c8e9f4"},20:{name:"ids-color-palette-azure-20",value:"#addbeb"},30:{name:"ids-color-palette-azure-30",value:"#8dc9e6"},40:{name:"ids-color-palette-azure-40",value:"#69b5dd"},50:{name:"ids-color-palette-azure-50",value:"#54a1d3"},60:{name:"ids-color-palette-azure-60",value:"#368ac0"},70:{name:"ids-color-palette-azure-70",value:"#2578a9"},80:{name:"ids-color-palette-azure-80",value:"#1d5f8a"},90:{name:"ids-color-palette-azure-90",value:"#134d71"},100:{name:"ids-color-palette-azure-100",value:"#133c59"}},emerald:{10:{name:"ids-color-palette-emerald-10",value:"#d5f6c0"},20:{name:"ids-color-palette-emerald-20",value:"#c3e8ac"},30:{name:"ids-color-palette-emerald-30",value:"#afdc91"},40:{name:"ids-color-palette-emerald-40",value:"#9cce7c"},50:{name:"ids-color-palette-emerald-50",value:"#89bf65"},60:{name:"ids-color-palette-emerald-60",value:"#76b051"},70:{name:"ids-color-palette-emerald-70",value:"#66a140"},80:{name:"ids-color-palette-emerald-80",value:"#56932e"},90:{name:"ids-color-palette-emerald-90",value:"#488421"},100:{name:"ids-color-palette-emerald-100",value:"#397514"}},graphite:{10:{name:"ids-color-palette-graphite-10",value:"#f0f0f0"},20:{name:"ids-color-palette-graphite-20",value:"#d8d8d8"},30:{name:"ids-color-palette-graphite-30",value:"#bdbdbd"},40:{name:"ids-color-palette-graphite-40",value:"#999999"},50:{name:"ids-color-palette-graphite-50",value:"#737373"},60:{name:"ids-color-palette-graphite-60",value:"#5c5c5c"},70:{name:"ids-color-palette-graphite-70",value:"#454545"},80:{name:"ids-color-palette-graphite-80",value:"#383838"},90:{name:"ids-color-palette-graphite-90",value:"#292929"},100:{name:"ids-color-palette-graphite-100",value:"#1a1a1a"}},ruby:{10:{name:"ids-color-palette-ruby-10",value:"#f6caca"},20:{name:"ids-color-palette-ruby-20",value:"#eb9d9d"},30:{name:"ids-color-palette-ruby-30",value:"#de8181"},40:{name:"ids-color-palette-ruby-40",value:"#d26d6d"},50:{name:"ids-color-palette-ruby-50",value:"#c65f5f"},60:{name:"ids-color-palette-ruby-60",value:"#b94e4a"},70:{name:"ids-color-palette-ruby-70",value:"#ad4242"},80:{name:"ids-color-palette-ruby-80",value:"#a13030"},90:{name:"ids-color-palette-ruby-90",value:"#941e1e"},100:{name:"ids-color-palette-ruby-100",value:"#880e0e"}},slate:{10:{name:"ids-color-palette-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-slate-40",value:"#888B94"},50:{name:"ids-color-palette-slate-50",value:"#656871"},60:{name:"ids-color-palette-slate-60",value:"#50535A"},70:{name:"ids-color-palette-slate-70",value:"#414247"},80:{name:"ids-color-palette-slate-80",value:"#313236"},90:{name:"ids-color-palette-slate-90",value:"#212224"},100:{name:"ids-color-palette-slate-100",value:"#1C1819"}},"classic-slate":{10:{name:"ids-color-palette-classic-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-classic-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-classic-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-classic-slate-40",value:"#888B94"},50:{name:"ids-color-palette-classic-slate-50",value:"#656871"},60:{name:"ids-color-palette-classic-slate-60",value:"#50535A"},70:{name:"ids-color-palette-classic-slate-70",value:"#414247"},80:{name:"ids-color-palette-classic-slate-80",value:"#313236"},90:{name:"ids-color-palette-classic-slate-90",value:"#212224"},100:{name:"ids-color-palette-classic-slate-100",value:"#1C1819"}},turquoise:{10:{name:"ids-color-palette-turquoise-10",value:"#c0ede3"},20:{name:"ids-color-palette-turquoise-20",value:"#a9e1d6"},30:{name:"ids-color-palette-turquoise-30",value:"#8ed1c6"},40:{name:"ids-color-palette-turquoise-40",value:"#7cc0b5"},50:{name:"ids-color-palette-turquoise-50",value:"#69ada3"},60:{name:"ids-color-palette-turquoise-60",value:"#579e95"},70:{name:"ids-color-palette-turquoise-70",value:"#448d83"},80:{name:"ids-color-palette-turquoise-80",value:"#317c73"},90:{name:"ids-color-palette-turquoise-90",value:"#206b62"},100:{name:"ids-color-palette-turquoise-100",value:"#0e5b52"}},white:{name:"ids-color-palette-white",value:"#ffffff",paletteName:"#ffffff"},black:{name:"ids-color-palette-black",value:"#000000",paletteName:"#000000"}},status:{base:{name:"ids-color-status-base",value:"#133c59",paletteName:"{ids.color.brand.primary.alt.value}"},caution:{name:"ids-color-status-caution",value:"#ffd726",paletteName:"#ffd726"},danger:{name:"ids-color-status-danger",value:"#e84f4f",paletteName:"#e84f4f"},error:{name:"ids-color-status-error",value:"#e84f4f",paletteName:"#e84f4f"},success:{name:"ids-color-status-success",value:"#80ce4d",paletteName:"#80ce4d"},warning:{name:"ids-color-status-warning",value:"#ff9426",paletteName:"#ff9426"},info:{name:"ids-color-status-info",value:"#2578a9",paletteName:"{ids.color.palette.azure.70.value}"},status01:{name:"ids-color-status-status-01",value:"#e84f4f",paletteName:"#e84f4f"},status02:{name:"ids-color-status-status-02",value:"#e84f4f",paletteName:"#e84f4f"},status03:{name:"ids-color-status-status-03",value:"#ffd726",paletteName:"#ffd726"},status04:{name:"ids-color-status-status-04",value:"#80ce4d",paletteName:"#80ce4d"},status05:{name:"ids-color-status-status-05",value:"#2578a9",paletteName:"{ids.color.palette.azure.70.value}"}},brand:{primary:{lighter:{name:"ids-color-brand-primary-lighter",value:"#69b5dd"},base:{name:"ids-color-brand-primary-base",value:"#134d71"},alt:{name:"ids-color-brand-primary-alt",value:"#133c59"},contrast:{name:"ids-color-brand-primary-contrast",value:"#ffffff"}},secondary:{lighter:{name:"ids-color-brand-secondary-lighter",value:"#d8d8d8"},base:{name:"ids-color-brand-secondary-base",value:"#bdbdbd"},alt:{name:"ids-color-brand-secondary-alt",value:"#999999"},contrast:{name:"ids-color-brand-secondary-contrast",value:"#383838"}}},components:{body:{primary:{background:{name:"body-color-primary-background",value:"#d8d8d8"},font:{name:"body-color-primary-font",value:"#000000"}},secondary:{background:{name:"body-color-secondary-background",value:"#d8d8d8"}}}}},ut={themeName:"theme-new",palette:{amber:{10:{name:"ids-color-palette-amber-10",value:"#FEF2E5"},20:{name:"ids-color-palette-amber-20",value:"#FDDFBD"},30:{name:"ids-color-palette-amber-30",value:"#FCC888"},40:{name:"ids-color-palette-amber-40",value:"#FBAF50"},50:{name:"ids-color-palette-amber-50",value:"#FA9601"},60:{name:"ids-color-palette-amber-60",value:"#F98300"},70:{name:"ids-color-palette-amber-70",value:"#DF6F00"},80:{name:"ids-color-palette-amber-80",value:"#CD6200"},90:{name:"ids-color-palette-amber-90",value:"#BB5500"},100:{name:"ids-color-palette-amber-100",value:"#A14100"}},amethyst:{10:{name:"ids-color-palette-amethyst-10",value:"#F1EBFC"},20:{name:"ids-color-palette-amethyst-20",value:"#DDCBF7"},30:{name:"ids-color-palette-amethyst-30",value:"#C2A1F1"},40:{name:"ids-color-palette-amethyst-40",value:"#A876EB"},50:{name:"ids-color-palette-amethyst-50",value:"#8D4BE5"},60:{name:"ids-color-palette-amethyst-60",value:"#7928E1"},70:{name:"ids-color-palette-amethyst-70",value:"#6C23C9"},80:{name:"ids-color-palette-amethyst-80",value:"#591DA8"},90:{name:"ids-color-palette-amethyst-90",value:"#4E1A91"},100:{name:"ids-color-palette-amethyst-100",value:"#3B1470"}},azure:{10:{name:"ids-color-palette-azure-10",value:"#E6F1FD"},20:{name:"ids-color-palette-azure-20",value:"#BEDCFA"},30:{name:"ids-color-palette-azure-30",value:"#8ABFF7"},40:{name:"ids-color-palette-azure-40",value:"#55A3F3"},50:{name:"ids-color-palette-azure-50",value:"#1C86EF"},60:{name:"ids-color-palette-azure-60",value:"#0072ED"},70:{name:"ids-color-palette-azure-70",value:"#0066D4"},80:{name:"ids-color-palette-azure-80",value:"#0054B1"},90:{name:"ids-color-palette-azure-90",value:"#004A99"},100:{name:"ids-color-palette-azure-100",value:"#003876"}},emerald:{10:{name:"ids-color-palette-emerald-10",value:"#EBF9F1"},20:{name:"ids-color-palette-emerald-20",value:"#CBEFDC"},30:{name:"ids-color-palette-emerald-30",value:"#A1E4BF"},40:{name:"ids-color-palette-emerald-40",value:"#78D8A3"},50:{name:"ids-color-palette-emerald-50",value:"#4DCC86"},60:{name:"ids-color-palette-emerald-60",value:"#2AC371"},70:{name:"ids-color-palette-emerald-70",value:"#25AF65"},80:{name:"ids-color-palette-emerald-80",value:"#1F9254"},90:{name:"ids-color-palette-emerald-90",value:"#1C7F49"},100:{name:"ids-color-palette-emerald-100",value:"#156138"}},graphite:{10:{name:"ids-color-palette-graphite-10",value:"#EEEEEE"},20:{name:"ids-color-palette-graphite-20",value:"#D3D3D3"},30:{name:"ids-color-palette-graphite-30",value:"#B1B1B1"},40:{name:"ids-color-palette-graphite-40",value:"#8F8F8F"},50:{name:"ids-color-palette-graphite-50",value:"#6C6C6C"},60:{name:"ids-color-palette-graphite-60",value:"#535353"},70:{name:"ids-color-palette-graphite-70",value:"#4A4A4A"},80:{name:"ids-color-palette-graphite-80",value:"#3E3E3E"},90:{name:"ids-color-palette-graphite-90",value:"#363636"},100:{name:"ids-color-palette-graphite-100",value:"#292929"}},ruby:{10:{name:"ids-color-palette-ruby-10",value:"#FBE7E8"},20:{name:"ids-color-palette-ruby-20",value:"#F5C3C4"},30:{name:"ids-color-palette-ruby-30",value:"#EE9496"},40:{name:"ids-color-palette-ruby-40",value:"#E66467"},50:{name:"ids-color-palette-ruby-50",value:"#DF3539"},60:{name:"ids-color-palette-ruby-60",value:"#DA1217"},70:{name:"ids-color-palette-ruby-70",value:"#C31014"},80:{name:"ids-color-palette-ruby-80",value:"#A30D11"},90:{name:"ids-color-palette-ruby-90",value:"#8D0B0E"},100:{name:"ids-color-palette-ruby-100",value:"#6C080B"}},slate:{10:{name:"ids-color-palette-slate-10",value:"#F5F5F5"},20:{name:"ids-color-palette-slate-20",value:"#E0E0E1"},30:{name:"ids-color-palette-slate-30",value:"#C5C5C9"},40:{name:"ids-color-palette-slate-40",value:"#BBBBBF"},50:{name:"ids-color-palette-slate-50",value:"#8E8E95"},60:{name:"ids-color-palette-slate-60",value:"#6F6F76"},70:{name:"ids-color-palette-slate-70",value:"#525257"},80:{name:"ids-color-palette-slate-80",value:"#3B3B3F"},90:{name:"ids-color-palette-slate-90",value:"#28282A"},100:{name:"ids-color-palette-slate-100",value:"#161618"}},"classic-slate":{10:{name:"ids-color-palette-classic-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-classic-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-classic-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-classic-slate-40",value:"#888B94"},50:{name:"ids-color-palette-classic-slate-50",value:"#656871"},60:{name:"ids-color-palette-classic-slate-60",value:"#50535A"},70:{name:"ids-color-palette-classic-slate-70",value:"#414247"},80:{name:"ids-color-palette-classic-slate-80",value:"#313236"},90:{name:"ids-color-palette-classic-slate-90",value:"#212224"},100:{name:"ids-color-palette-classic-slate-100",value:"#1c1819"}},turquoise:{10:{name:"ids-color-palette-turquoise-10",value:"#ECF8F8"},20:{name:"ids-color-palette-turquoise-20",value:"#CFEEEE"},30:{name:"ids-color-palette-turquoise-30",value:"#A8E1E1"},40:{name:"ids-color-palette-turquoise-40",value:"#82D4D4"},50:{name:"ids-color-palette-turquoise-50",value:"#5CC6C7"},60:{name:"ids-color-palette-turquoise-60",value:"#40BDBE"},70:{name:"ids-color-palette-turquoise-70",value:"#39A9AA"},80:{name:"ids-color-palette-turquoise-80",value:"#2F8D8E"},90:{name:"ids-color-palette-turquoise-90",value:"#297B7B"},100:{name:"ids-color-palette-turquoise-100",value:"#1F5E5E"}},white:{name:"ids-color-palette-white",value:"#ffffff",paletteName:"#ffffff"},black:{name:"ids-color-palette-black",value:"#000000",paletteName:"#000000"}},status:{base:{name:"ids-color-status-base",value:"#0066D4",paletteName:"{ids.color.brand.primary.alt.value}"},caution:{name:"ids-color-status-caution",value:"#FFD100",paletteName:"#FFD100"},danger:{name:"ids-color-status-danger",value:"#DA1217",paletteName:"#DA1217"},error:{name:"ids-color-status-error",value:"#DA1217",paletteName:"#DA1217"},success:{name:"ids-color-status-success",value:"#2AC371",paletteName:"#2AC371"},warning:{name:"ids-color-status-warning",value:"#F98300",paletteName:"#F98300"},info:{name:"ids-color-status-info",value:"#0064D4",paletteName:"#0064D4"},status01:{name:"ids-color-status-status-01",value:"#DA1217",paletteName:"#DA1217"},status02:{name:"ids-color-status-status-02",value:"#F98300",paletteName:"#F98300"},status03:{name:"ids-color-status-status-03",value:"#FFD100",paletteName:"#FFD100"},status04:{name:"ids-color-status-status-04",value:"#2AC371",paletteName:"#2AC371"},status05:{name:"ids-color-status-status-05",value:"#0064D4",paletteName:"#0064D4"}},brand:{primary:{lighter:{name:"ids-color-brand-primary-lighter",value:"#55A3F3"},base:{name:"ids-color-brand-primary-base",value:"#0072ED"},alt:{name:"ids-color-brand-primary-alt",value:"#0066D4"},contrast:{name:"ids-color-brand-primary-contrast",value:"#ffffff"}},secondary:{lighter:{name:"ids-color-brand-secondary-lighter",value:"#BBBBBF"},base:{name:"ids-color-brand-secondary-base",value:"#6F6F76"},alt:{name:"ids-color-brand-secondary-alt",value:"#525257"},contrast:{name:"ids-color-brand-secondary-contrast",value:"#28282A"}}},components:{body:{primary:{background:{name:"body-color-primary-background",value:"#ffffff"},font:{name:"body-color-primary-font",value:"#161618"}},secondary:{background:{name:"body-color-secondary-background",value:"#ffffff"}}}}},pt={themeName:"theme-new-dark",palette:{amber:{10:{name:"ids-color-palette-amber-10",value:"#FEF2E5"},20:{name:"ids-color-palette-amber-20",value:"#FDDFBD"},30:{name:"ids-color-palette-amber-30",value:"#FCC888"},40:{name:"ids-color-palette-amber-40",value:"#FBAF50"},50:{name:"ids-color-palette-amber-50",value:"#FA9601"},60:{name:"ids-color-palette-amber-60",value:"#F98300"},70:{name:"ids-color-palette-amber-70",value:"#DF6F00"},80:{name:"ids-color-palette-amber-80",value:"#CD6200"},90:{name:"ids-color-palette-amber-90",value:"#BB5500"},100:{name:"ids-color-palette-amber-100",value:"#A14100"}},amethyst:{10:{name:"ids-color-palette-amethyst-10",value:"#F1EBFC"},20:{name:"ids-color-palette-amethyst-20",value:"#DDCBF7"},30:{name:"ids-color-palette-amethyst-30",value:"#C2A1F1"},40:{name:"ids-color-palette-amethyst-40",value:"#A876EB"},50:{name:"ids-color-palette-amethyst-50",value:"#8D4BE5"},60:{name:"ids-color-palette-amethyst-60",value:"#7928E1"},70:{name:"ids-color-palette-amethyst-70",value:"#6C23C9"},80:{name:"ids-color-palette-amethyst-80",value:"#591DA8"},90:{name:"ids-color-palette-amethyst-90",value:"#4E1A91"},100:{name:"ids-color-palette-amethyst-100",value:"#3B1470"}},azure:{10:{name:"ids-color-palette-azure-10",value:"#E6F1FD"},20:{name:"ids-color-palette-azure-20",value:"#BEDCFA"},30:{name:"ids-color-palette-azure-30",value:"#8ABFF7"},40:{name:"ids-color-palette-azure-40",value:"#55A3F3"},50:{name:"ids-color-palette-azure-50",value:"#1C86EF"},60:{name:"ids-color-palette-azure-60",value:"#0072ED"},70:{name:"ids-color-palette-azure-70",value:"#0066D4"},80:{name:"ids-color-palette-azure-80",value:"#0054B1"},90:{name:"ids-color-palette-azure-90",value:"#004A99"},100:{name:"ids-color-palette-azure-100",value:"#003876"}},emerald:{10:{name:"ids-color-palette-emerald-10",value:"#EBF9F1"},20:{name:"ids-color-palette-emerald-20",value:"#CBEFDC"},30:{name:"ids-color-palette-emerald-30",value:"#A1E4BF"},40:{name:"ids-color-palette-emerald-40",value:"#78D8A3"},50:{name:"ids-color-palette-emerald-50",value:"#4DCC86"},60:{name:"ids-color-palette-emerald-60",value:"#2AC371"},70:{name:"ids-color-palette-emerald-70",value:"#25AF65"},80:{name:"ids-color-palette-emerald-80",value:"#1F9254"},90:{name:"ids-color-palette-emerald-90",value:"#1C7F49"},100:{name:"ids-color-palette-emerald-100",value:"#156138"}},graphite:{10:{name:"ids-color-palette-graphite-10",value:"#EEEEEE"},20:{name:"ids-color-palette-graphite-20",value:"#D3D3D3"},30:{name:"ids-color-palette-graphite-30",value:"#B1B1B1"},40:{name:"ids-color-palette-graphite-40",value:"#8F8F8F"},50:{name:"ids-color-palette-graphite-50",value:"#6C6C6C"},60:{name:"ids-color-palette-graphite-60",value:"#535353"},70:{name:"ids-color-palette-graphite-70",value:"#4A4A4A"},80:{name:"ids-color-palette-graphite-80",value:"#3E3E3E"},90:{name:"ids-color-palette-graphite-90",value:"#363636"},100:{name:"ids-color-palette-graphite-100",value:"#292929"}},ruby:{10:{name:"ids-color-palette-ruby-10",value:"#FBE7E8"},20:{name:"ids-color-palette-ruby-20",value:"#F5C3C4"},30:{name:"ids-color-palette-ruby-30",value:"#EE9496"},40:{name:"ids-color-palette-ruby-40",value:"#E66467"},50:{name:"ids-color-palette-ruby-50",value:"#DF3539"},60:{name:"ids-color-palette-ruby-60",value:"#DA1217"},70:{name:"ids-color-palette-ruby-70",value:"#C31014"},80:{name:"ids-color-palette-ruby-80",value:"#A30D11"},90:{name:"ids-color-palette-ruby-90",value:"#8D0B0E"},100:{name:"ids-color-palette-ruby-100",value:"#6C080B"}},slate:{10:{name:"ids-color-palette-slate-10",value:"#F5F5F5"},20:{name:"ids-color-palette-slate-20",value:"#E0E0E1"},30:{name:"ids-color-palette-slate-30",value:"#C5C5C9"},40:{name:"ids-color-palette-slate-40",value:"#BBBBBF"},50:{name:"ids-color-palette-slate-50",value:"#8E8E95"},60:{name:"ids-color-palette-slate-60",value:"#6F6F76"},70:{name:"ids-color-palette-slate-70",value:"#525257"},80:{name:"ids-color-palette-slate-80",value:"#3B3B3F"},90:{name:"ids-color-palette-slate-90",value:"#28282A"},100:{name:"ids-color-palette-slate-100",value:"#161618"}},"classic-slate":{10:{name:"ids-color-palette-classic-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-classic-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-classic-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-classic-slate-40",value:"#888B94"},50:{name:"ids-color-palette-classic-slate-50",value:"#656871"},60:{name:"ids-color-palette-classic-slate-60",value:"#50535A"},70:{name:"ids-color-palette-classic-slate-70",value:"#414247"},80:{name:"ids-color-palette-classic-slate-80",value:"#313236"},90:{name:"ids-color-palette-classic-slate-90",value:"#212224"},100:{name:"ids-color-palette-classic-slate-100",value:"#1c1819"}},turquoise:{10:{name:"ids-color-palette-turquoise-10",value:"#ECF8F8"},20:{name:"ids-color-palette-turquoise-20",value:"#CFEEEE"},30:{name:"ids-color-palette-turquoise-30",value:"#A8E1E1"},40:{name:"ids-color-palette-turquoise-40",value:"#82D4D4"},50:{name:"ids-color-palette-turquoise-50",value:"#5CC6C7"},60:{name:"ids-color-palette-turquoise-60",value:"#40BDBE"},70:{name:"ids-color-palette-turquoise-70",value:"#39A9AA"},80:{name:"ids-color-palette-turquoise-80",value:"#2F8D8E"},90:{name:"ids-color-palette-turquoise-90",value:"#297B7B"},100:{name:"ids-color-palette-turquoise-100",value:"#1F5E5E"}},white:{name:"ids-color-palette-white",value:"#ffffff",paletteName:"#ffffff"},black:{name:"ids-color-palette-black",value:"#000000",paletteName:"#000000"}},status:{base:{name:"ids-color-status-base",value:"#6F6F76",paletteName:"{ids.color.brand.primary.base.value}"},caution:{name:"ids-color-status-caution",value:"#FFD100",paletteName:"#FFD100"},danger:{name:"ids-color-status-danger",value:"#DA1217",paletteName:"#DA1217"},error:{name:"ids-color-status-error",value:"#DA1217",paletteName:"#DA1217"},success:{name:"ids-color-status-success",value:"#2AC371",paletteName:"#2AC371"},warning:{name:"ids-color-status-warning",value:"#F98300",paletteName:"#F98300"},info:{name:"ids-color-status-info",value:"#0064D4",paletteName:"#0064D4"},status01:{name:"ids-color-status-status-01",value:"#DA1217",paletteName:"#DA1217"},status02:{name:"ids-color-status-status-02",value:"#F98300",paletteName:"#F98300"},status03:{name:"ids-color-status-status-03",value:"#FFD100",paletteName:"#FFD100"},status04:{name:"ids-color-status-status-04",value:"#2AC371",paletteName:"#2AC371"},status05:{name:"ids-color-status-status-05",value:"#0064D4",paletteName:"#0064D4"}},brand:{primary:{lighter:{name:"ids-color-brand-primary-lighter",value:"#BBBBBF"},base:{name:"ids-color-brand-primary-base",value:"#6F6F76"},alt:{name:"ids-color-brand-primary-alt",value:"#0066D4"},contrast:{name:"ids-color-brand-primary-contrast",value:"#ffffff"}},secondary:{lighter:{name:"ids-color-brand-secondary-lighter",value:"#BBBBBF"},base:{name:"ids-color-brand-secondary-base",value:"#8E8E95"},alt:{name:"ids-color-brand-secondary-alt",value:"#525257"},contrast:{name:"ids-color-brand-secondary-contrast",value:"#28282A"}}},components:{body:{primary:{background:{name:"body-color-primary-background",value:"#161618"},font:{name:"body-color-primary-font",value:"#F5F5F5"}},secondary:{background:{name:"body-color-secondary-background",value:"#3B3B3F"}}}}},gt={themeName:"theme-new-contrast",palette:{amber:{10:{name:"ids-color-palette-amber-10",value:"#FEF2E5"},20:{name:"ids-color-palette-amber-20",value:"#FDDFBD"},30:{name:"ids-color-palette-amber-30",value:"#FCC888"},40:{name:"ids-color-palette-amber-40",value:"#FBAF50"},50:{name:"ids-color-palette-amber-50",value:"#FA9601"},60:{name:"ids-color-palette-amber-60",value:"#F98300"},70:{name:"ids-color-palette-amber-70",value:"#DF6F00"},80:{name:"ids-color-palette-amber-80",value:"#CD6200"},90:{name:"ids-color-palette-amber-90",value:"#BB5500"},100:{name:"ids-color-palette-amber-100",value:"#A14100"}},amethyst:{10:{name:"ids-color-palette-amethyst-10",value:"#F1EBFC"},20:{name:"ids-color-palette-amethyst-20",value:"#DDCBF7"},30:{name:"ids-color-palette-amethyst-30",value:"#C2A1F1"},40:{name:"ids-color-palette-amethyst-40",value:"#A876EB"},50:{name:"ids-color-palette-amethyst-50",value:"#8D4BE5"},60:{name:"ids-color-palette-amethyst-60",value:"#7928E1"},70:{name:"ids-color-palette-amethyst-70",value:"#6C23C9"},80:{name:"ids-color-palette-amethyst-80",value:"#591DA8"},90:{name:"ids-color-palette-amethyst-90",value:"#4E1A91"},100:{name:"ids-color-palette-amethyst-100",value:"#3B1470"}},azure:{10:{name:"ids-color-palette-azure-10",value:"#E6F1FD"},20:{name:"ids-color-palette-azure-20",value:"#BEDCFA"},30:{name:"ids-color-palette-azure-30",value:"#8ABFF7"},40:{name:"ids-color-palette-azure-40",value:"#55A3F3"},50:{name:"ids-color-palette-azure-50",value:"#1C86EF"},60:{name:"ids-color-palette-azure-60",value:"#0072ED"},70:{name:"ids-color-palette-azure-70",value:"#0066D4"},80:{name:"ids-color-palette-azure-80",value:"#0054B1"},90:{name:"ids-color-palette-azure-90",value:"#004A99"},100:{name:"ids-color-palette-azure-100",value:"#003876"}},emerald:{10:{name:"ids-color-palette-emerald-10",value:"#EBF9F1"},20:{name:"ids-color-palette-emerald-20",value:"#CBEFDC"},30:{name:"ids-color-palette-emerald-30",value:"#A1E4BF"},40:{name:"ids-color-palette-emerald-40",value:"#78D8A3"},50:{name:"ids-color-palette-emerald-50",value:"#4DCC86"},60:{name:"ids-color-palette-emerald-60",value:"#2AC371"},70:{name:"ids-color-palette-emerald-70",value:"#25AF65"},80:{name:"ids-color-palette-emerald-80",value:"#1F9254"},90:{name:"ids-color-palette-emerald-90",value:"#1C7F49"},100:{name:"ids-color-palette-emerald-100",value:"#156138"}},graphite:{10:{name:"ids-color-palette-graphite-10",value:"#EEEEEE"},20:{name:"ids-color-palette-graphite-20",value:"#D3D3D3"},30:{name:"ids-color-palette-graphite-30",value:"#B1B1B1"},40:{name:"ids-color-palette-graphite-40",value:"#8F8F8F"},50:{name:"ids-color-palette-graphite-50",value:"#6C6C6C"},60:{name:"ids-color-palette-graphite-60",value:"#535353"},70:{name:"ids-color-palette-graphite-70",value:"#4A4A4A"},80:{name:"ids-color-palette-graphite-80",value:"#3E3E3E"},90:{name:"ids-color-palette-graphite-90",value:"#363636"},100:{name:"ids-color-palette-graphite-100",value:"#292929"}},ruby:{10:{name:"ids-color-palette-ruby-10",value:"#FBE7E8"},20:{name:"ids-color-palette-ruby-20",value:"#F5C3C4"},30:{name:"ids-color-palette-ruby-30",value:"#EE9496"},40:{name:"ids-color-palette-ruby-40",value:"#E66467"},50:{name:"ids-color-palette-ruby-50",value:"#DF3539"},60:{name:"ids-color-palette-ruby-60",value:"#DA1217"},70:{name:"ids-color-palette-ruby-70",value:"#C31014"},80:{name:"ids-color-palette-ruby-80",value:"#A30D11"},90:{name:"ids-color-palette-ruby-90",value:"#8D0B0E"},100:{name:"ids-color-palette-ruby-100",value:"#6C080B"}},slate:{10:{name:"ids-color-palette-slate-10",value:"#F5F5F5"},20:{name:"ids-color-palette-slate-20",value:"#E0E0E1"},30:{name:"ids-color-palette-slate-30",value:"#C5C5C9"},40:{name:"ids-color-palette-slate-40",value:"#BBBBBF"},50:{name:"ids-color-palette-slate-50",value:"#8E8E95"},60:{name:"ids-color-palette-slate-60",value:"#6F6F76"},70:{name:"ids-color-palette-slate-70",value:"#525257"},80:{name:"ids-color-palette-slate-80",value:"#3B3B3F"},90:{name:"ids-color-palette-slate-90",value:"#28282A"},100:{name:"ids-color-palette-slate-100",value:"#161618"}},"classic-slate":{10:{name:"ids-color-palette-classic-slate-10",value:"#DEE1E8"},20:{name:"ids-color-palette-classic-slate-20",value:"#C8CBD4"},30:{name:"ids-color-palette-classic-slate-30",value:"#ABAEB7"},40:{name:"ids-color-palette-classic-slate-40",value:"#888B94"},50:{name:"ids-color-palette-classic-slate-50",value:"#656871"},60:{name:"ids-color-palette-classic-slate-60",value:"#50535A"},70:{name:"ids-color-palette-classic-slate-70",value:"#414247"},80:{name:"ids-color-palette-classic-slate-80",value:"#313236"},90:{name:"ids-color-palette-classic-slate-90",value:"#212224"},100:{name:"ids-color-palette-classic-slate-100",value:"#1c1819"}},turquoise:{10:{name:"ids-color-palette-turquoise-10",value:"#ECF8F8"},20:{name:"ids-color-palette-turquoise-20",value:"#CFEEEE"},30:{name:"ids-color-palette-turquoise-30",value:"#A8E1E1"},40:{name:"ids-color-palette-turquoise-40",value:"#82D4D4"},50:{name:"ids-color-palette-turquoise-50",value:"#5CC6C7"},60:{name:"ids-color-palette-turquoise-60",value:"#40BDBE"},70:{name:"ids-color-palette-turquoise-70",value:"#39A9AA"},80:{name:"ids-color-palette-turquoise-80",value:"#2F8D8E"},90:{name:"ids-color-palette-turquoise-90",value:"#297B7B"},100:{name:"ids-color-palette-turquoise-100",value:"#1F5E5E"}},white:{name:"ids-color-palette-white",value:"#ffffff",paletteName:"#ffffff"},black:{name:"ids-color-palette-black",value:"#000000",paletteName:"#000000"}},status:{base:{name:"ids-color-status-base",value:"#003876",paletteName:"{ids.color.brand.primary.alt.value}"},caution:{name:"ids-color-status-caution",value:"#FFD100",paletteName:"#FFD100"},danger:{name:"ids-color-status-danger",value:"#7B0F11",paletteName:"#7B0F11"},error:{name:"ids-color-status-error",value:"#DA1217",paletteName:"#DA1217"},success:{name:"ids-color-status-success",value:"#2AC371",paletteName:"#2AC371"},warning:{name:"ids-color-status-warning",value:"#F98300",paletteName:"#F98300"},info:{name:"ids-color-status-info",value:"#0064D4",paletteName:"#0064D4"},status01:{name:"ids-color-status-status-01",value:"#DA1217",paletteName:"#DA1217"},status02:{name:"ids-color-status-status-02",value:"#F98300",paletteName:"#F98300"},status03:{name:"ids-color-status-status-03",value:"#FFD100",paletteName:"#FFD100"},status04:{name:"ids-color-status-status-04",value:"#2AC371",paletteName:"#2AC371"},status05:{name:"ids-color-status-status-05",value:"#0064D4",paletteName:"#0064D4"}},brand:{primary:{lighter:{name:"ids-color-brand-primary-lighter",value:"#55A3F3"},base:{name:"ids-color-brand-primary-base",value:"#004A99"},alt:{name:"ids-color-brand-primary-alt",value:"#003876"},contrast:{name:"ids-color-brand-primary-contrast",value:"#ffffff"}},secondary:{lighter:{name:"ids-color-brand-secondary-lighter",value:"#3B3B3F"},base:{name:"ids-color-brand-secondary-base",value:"#28282A"},alt:{name:"ids-color-brand-secondary-alt",value:"#161618"},contrast:{name:"ids-color-brand-secondary-contrast",value:"#28282A"}}},components:{body:{primary:{background:{name:"body-color-primary-background",value:"#F5F5F5"},font:{name:"body-color-primary-font",value:"#28282A"}},secondary:{background:{name:"body-color-secondary-background",value:"#ffffff"}}}}},ft={currentTheme:{id:"theme-new-light",name:H.translate("NewLightTheme")},allColors:[{id:"theme-classic-light",colors:ct},{id:"theme-classic-dark",colors:dt},{id:"theme-classic-contrast",colors:ht},{id:"theme-new-light",colors:ut},{id:"theme-new-dark",colors:pt},{id:"theme-new-contrast",colors:gt}],themes:function(){return[{id:"theme-classic-light",name:H.translate("ClassicLightTheme"),themeId:"theme-classic",modeId:"light",modeName:H.translate("Light")},{id:"theme-classic-dark",name:H.translate("ClassicDarkTheme"),themeId:"theme-classic",modeId:"dark",modeName:H.translate("Dark")},{id:"theme-classic-contrast",name:H.translate("ClassicHighContrastTheme"),themeId:"theme-classic",modeId:"contrast",modeName:H.translate("Contrast")},{id:"theme-new-light",name:H.translate("NewLightTheme"),themeId:"theme-new",modeId:"light",modeName:H.translate("Light")},{id:"theme-new-dark",name:H.translate("NewDarkTheme"),themeId:"theme-new",modeId:"dark",modeName:H.translate("Dark")},{id:"theme-new-contrast",name:H.translate("NewHighContrastTheme"),themeId:"theme-new",modeId:"contrast",modeName:H.translate("Contrast")}]},themeColors:function(){var t=this,e=this.allColors.filter((function(e){return e.id===t.currentTheme.id}));return e[0]?e[0].colors:{palette:{},colors:{},brand:{}}},personalizationColors:function(){var t=this.themeColors().palette,e={},i={showBrackets:!1},n="#0066D4";switch(this.currentTheme.id){case"theme-new-light":case"theme-new-contrast":case"theme-new-dark":default:n="#0066D4";break;case"theme-classic-light":n="#2578a9";break;case"theme-classic-contrast":n="#134d71";break;case"theme-classic-dark":n="#50535A"}return e.default={id:"default",name:H.translate("Default",i),backgroundColorClass:"primary-bg-color",value:n},e.alabaster={id:"alabaster",name:H.translate("Alabaster",i),backgroundColorClass:"alabaster",value:t.white.value},e.amber={id:"amber",name:H.translate("Amber",i),backgroundColorClass:"amber09",value:t.amber[90].value},e.amethyst={id:"amethyst",name:H.translate("Amethyst",i),backgroundColorClass:"amethyst06",value:t.amethyst[60].value},e.azure={id:"azure",name:H.translate("Azure",i),backgroundColorClass:"azure07",value:t.azure[70].value},e.emerald={id:"emerald",name:H.translate("Emerald",i),backgroundColorClass:"emerald08",value:t.emerald[80].value},e.graphite={id:"graphite",name:H.translate("Graphite",i),backgroundColorClass:"graphite06",value:t.graphite[60].value},e.ruby={id:"ruby",name:H.translate("Ruby",i),backgroundColorClass:"ruby09",value:t.ruby[90].value},e.slate={id:"slate",name:H.translate("Slate",i),backgroundColorClass:"slate06",value:t.slate[60].value},e.turquoise={id:"turquoise",name:H.translate("Turquoise",i),backgroundColorClass:"turquoise09",value:t.turquoise[90].value},e},setTheme:function(t){t=t.replace("soho","classic").replace("uplift","new");var e=this.themes().filter((function(e){return e.id===t}));return 0===e.length?"":(this.currentTheme=e[0],e)},get uplift(){return this.currentTheme.id.indexOf("-uplift-")>-1||this.currentTheme.id.indexOf("-new-")>-1},get new(){return this.currentTheme.id.indexOf("-uplift-")>-1||this.currentTheme.id.indexOf("-new-")>-1}};var mt="personalize",vt={colors:"",theme:"",font:"",blockUI:!0,noInit:!1};function bt(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,vt),this.init()}bt.prototype={init:function(){return this.handleEvents(),this.settings.noInit||(this.settings.theme?this.setTheme(this.settings.theme):this.setTheme(this.getThemeFromStylesheet()),this.settings.colors&&this.setColors(this.settings.colors),this.settings.font&&this.setFont(this.settings.font)),this},handleEvents:function(){var t=this;return this.element.off("updated.".concat(mt)).on("updated.".concat(mt),(function(){t.updated()})),this},appendStyleSheet:function(t){var e=document.getElementById("soho-personalization");e&&e.parentNode.removeChild(e),(e=document.createElement("style")).setAttribute("id","soho-personalization"),e.appendChild(document.createTextNode(t)),document.head.appendChild(e)},getColorMap:function(t){var e,i=ft.themeColors(),n={amber:i.palette.amber[90].value,amethyst:i.palette.amethyst[60].value,azure:i.palette.azure[70].value,emerald:i.palette.emerald[80].value,graphite:i.palette.graphite[60].value,ruby:i.palette.ruby[90].value,slate:i.palette.slate[60].value,turquoise:i.palette.turquoise[90].value};return c(e={},n.amber,i.palette.amber[t].value),c(e,n.amethyst,i.palette.amethyst[t].value),c(e,n.azure,i.palette.azure[t].value),c(e,n.emerald,i.palette.emerald[t].value),c(e,n.graphite,i.palette.graphite[t].value),c(e,n.ruby,i.palette.ruby[t].value),c(e,n.slate,i.palette.slate[t].value),c(e,n.turquoise,i.palette.turquoise[t].value),e},getColorStyleSheet:function(t){if(t||(t={}),"string"==typeof t&&(t={header:t}),!t||""===t)return this;var e={header:"2578A9"},i=ft.themeColors();t.theme={},t.theme.bg=i.components.body.primary.background.value,t.theme.altbg=i.components.body.secondary.background.value,t.theme.text=i.components.body.primary.font.value,t.theme.altText=i.components.body.secondary.background.value,t.theme.disabledText=i.brand.secondary.alt.value;var n=!1,s=!1,a=!1;i.themeName.indexOf("contrast")>-1&&(s=!0),i.themeName.indexOf("dark")>-1&&(n=!0),(i.themeName.indexOf("uplift")>-1||i.themeName.indexOf("new")>-1)&&(a=!0),t.theme.props={contrast:s,dark:n,newTheme:a};var o={amber:["#db7726","#bb5500"],amethyst:["#9279a6","#7834dd"],azure:["#2578a9","#0563c2","#368AC0","#368ac0"],emerald:["#56932e","#1f9254"],graphite:["#5c5c5c","#808080"],ruby:["#941e1e","#7b0f11"],slate:["#50535a","#98949e"],turquoise:["#206b62","#297b7b"]},r=!1,l="".concat(t.header||e.header).toLowerCase();l=O.validateHex(l),Object.keys(o).forEach((function(t){r=r||o[t].indexOf(l)>-1})),l=r?"white":null;var c=O.validateHex(t.lightest||O.getLuminousColorShade(t.header||e.header,.3));"white"===O.getContrastColor(c,null,l)?(e.text="ffffff",e.subtext="f0f0f0"):(e.text="000000",e.subtext="292929"),t.header=O.validateHex(t.header||e.header),t.text=O.validateHex(t.text||e.text),t.subtext=O.validateHex(t.subtext||e.subtext),t.btnColorHeader=O.validateHex(t.btnColorHeader||O.getLuminousColorShade(t.header,.3)),t.subheader=O.validateHex(t.subheader||O.getLuminousColorShade(t.header,.2)),t.verticalBorder=O.validateHex(t.verticalBorder||O.getLuminousColorShade(t.header,.1)),t.btnColorSubheader=O.validateHex(t.btnColorSubheader||O.getLuminousColorShade(t.header,-.1)),t.inactive=O.validateHex(t.inactive||O.getLuminousColorShade(t.header,-.2)),t.horizontalBorder=O.validateHex(t.horizontalBorder||O.getLuminousColorShade(t.header,-.3)),t.hover=O.validateHex(t.hover||O.getLuminousColorShade(t.header,-.5)),t.button=O.validateHex(t.button||O.getLuminousColorShade(t.text,-.8)),t.lightest=t.btnColorHeader,t.lighter=t.subheader,t.light=t.verticalBorder,t.base=t.header,t.contrast=t.text,t.dark=t.btnColorSubheader,t.darker=t.inactive,t.darkest=t.horizontalBorder,t.btnHoverColor=t.contrast,t.btnBgHoverColor="rgba(0, 0, 0, 0.3) !important",t.focusBoxShadow="0 0 0 2px transparent, 0 0 0 0 ".concat(t.subtext,", 0 0 2px 1px ").concat(t.subtext),t.btnFocusBorderColor=t.contrast,t.btnDisabledColor="rgba(255, 255, 255, 0.3) !important",t.btnOpacity=.8,t.btnPrimaryColor=t.base,t.btnPrimaryColorHover=t.darker,t.btnSecondaryColor=t.darkest,t.btnSecondaryBorderColor=t.base,t.btnTertiaryHoverColor=t.darker,t.btnLinkColor=t.darker,t.tabBottomBorderColor=t.base,t.btnActionsHoverColor=t.base,t.moduleTabsSelectedTextColor=i.palette.white.value,t.headerTabsSelectedTextColor=t.contrast,t.headerTabsTextColor=t.contrast,t.headerTextColor=t.contrast,t.headerIconColor=t.contrast,t.tabHoverColor=t.darkest,t.tabVerticalHoverColor=i.palette.slate[20].value,t.tabHoverTextColor=i.palette.slate[100].value,t.tabTextWeight="400",t.tabTextSelectedWeight="600",t.tabBorderColor=t.darkest,t.tabSelectedColor=t.dark,t.tabHeaderColor=t.dark,t.tabColor=t.base,t.tabVerticalColor=i.palette.slate[10].value,t.headerTabBorder="transparent",t.verticalTabBorder=i.palette.slate[30].value,t.tabTextColor=i.palette.slate[100].value,t.tabSelectedTextColor=t.contrast,t.tabVerticalSelectedTextColor=t.base,t.tabVerticalSelectedColor=i.palette.white.value,t.tabDivider=t.darkest,t.tabFocusIndicator=t.contrast,t.tabFocusIndicatorVert=t.base,t.tabFocusBoxShadow="none",t.tabFocusBoxShadowVert="none",t.tabCloseInactiveColor="rgba(255, 255, 255, 0.85)",t.tabCloseHoverColor=i.palette.white.value,t.btnMenu=t.base,t.btnBoxShadow="0 0 0 2px transparent, 0 0 0 1px ".concat(t.base,", 0 0 4px 3px rgba(54, 138, 192, 0.1)");var d="#ffffff"===t.header,h=this.currentTheme.indexOf("new-dark")>=0,u=this.currentTheme.indexOf("classic-dark")>=0,p=this.currentTheme.indexOf("new-contrast")>=0;t.btnTertiaryBgHoverColor=d?"#E6F1FD":t.lightest,t.btnTertiaryBgHoverColorHeader="rgba(0, 0, 0, 0.3)",t.btnTertiaryHoverColorHeader=i.palette.white.value,t.borderActiveSearchHeader=t.darkest,t.shadowActiveSearchHeader="none",t.borderActiveSearchMobile=t.contrast,(h||u)&&(t.btnLinkColor=t.lighter,t.btnTertiaryBgHoverColor=i.palette.slate[70].value,t.btnTertiaryHoverColor=t.lighter,t.tabVerticalColor=i.palette.slate[90].value,t.tabVerticalSelectedColor=i.palette.slate[100].value,t.tabVerticalHoverColor=i.palette.slate[70].value,t.tabHoverTextColor=i.palette.white.value),d&&(t.tabHoverColor=i.palette.slate[20].value,t.text=i.palette.azure[60].value,t.darker=i.palette.azure[10].value,t.darkest=i.palette.slate[100].value,t.contrast=i.palette.slate[100].value,t.btnHoverColor=i.palette.azure[60].value,t.btnBgHoverColor=i.palette.azure[10].value,t.btnOpacity=1,t.borderActiveSearchHeader=i.palette.azure[60].value,t.shadowActiveSearchHeader="0 0 4px 3px rgba(54, 138, 192, 0.1)",t.focusBoxShadow="0 0 4px 3px rgba(54, 138, 192, 0.1)",t.btnFocusBorderColor=i.palette.azure[60].value,t.borderActiveSearchMobile=i.palette.azure[60].value,t.btnDisabledColor="rgba(0, 0, 0, 0.3) !important",t.btnPrimaryColor=i.palette.azure[60].value,t.btnPrimaryColorHover="#0066D4",t.btnSecondaryColor=i.palette.azure[60].value,t.btnSecondaryBorderColor=i.palette.azure[60].value,t.btnTertiaryBgHoverColor=i.palette.azure[20].value,t.btnTertiaryHoverColor=i.palette.azure[60].value,t.btnTertiaryBgHoverColorHeader="#E6F1FD",t.btnTertiaryHoverColorHeader=i.palette.azure[60].value,t.btnLinkColor=i.palette.azure[60].value,t.tabBottomBorderColor=i.palette.slate[30].value,t.btnActionsHoverColor=i.palette.slate[100].value,t.moduleTabsSelectedTextColor=i.palette.azure[60].value,t.headerTabsSelectedTextColor=i.palette.azure[60].value,t.headerTextColor=i.palette.slate[100].value,t.headerIconColor=i.palette.slate[80].value,t.btnMenu=i.palette.azure[60].value,t.btnBoxShadow="0 0 0 2px transparent, 0 0 0 1px ".concat(t.btnMenu,", 0 0 4px 3px rgba(54, 138, 192, 0.1)"),t.tabFocusIndicator=i.palette.azure[60].value,t.tabFocusIndicatorVert=i.palette.azure[60].value,t.tabFocusBoxShadow="0 0 4px 3px rgba(54, 138, 192, 0.2)",t.tabFocusBoxShadowVert="0 0 4px 3px rgba(54, 138, 192, 0.2)",t.tabTextWeight="400",t.tabTextSelectedWeight="700",t.headerTabBorder=i.palette.slate[30].value,t.verticalTabBorder=i.palette.slate[30].value,t.tabDivider=i.palette.slate[30].value,t.tabHoverColor=i.palette.slate[20].value,t.tabVerticalHoverColor=i.palette.slate[20].value,t.tabSelectedColor=i.palette.white.value,t.tabHeaderColor=i.palette.white.value,t.tabFocusColor=i.palette.slate[100].value,t.tabBorderColor=i.palette.white.value,t.tabColor=i.palette.slate[10].value,t.tabSelectedTextColor=i.palette.azure[60].value,t.tabVerticalSelectedTextColor=i.palette.azure[60].value,t.tabVerticalColor=i.palette.slate[10].value,t.tabVerticalSelectedColor=i.palette.white.value,t.tabCloseInactiveColor="rgb(111, 111, 118)",t.tabCloseHoverColor=t.contrast,a||(t.btnTertiaryBgHoverColor=i.palette.slate[30].value),h&&(t.tabCloseInactiveColor="rgba(255, 255, 255, 0.85)",t.base=i.palette.slate[60].value,t.btnPrimaryColor=t.base,t.dark=t.base,t.contrast=i.palette.white.value,t.focusBoxShadow="0 0 0 2px transparent, 0 0 0 1px ".concat(t.contrast,", 0 0 1px 1px ").concat(t.contrast),t.darker=i.palette.slate[80].value,t.headerIconColor=i.palette.slate[10].value,t.btnHoverColor=i.palette.white.value,t.hyperlinkText=i.palette.white.value,t.lighter=i.palette.slate[50].value,t.btnDisabledColor="rgba(255, 255, 255, 0.4) !important",t.btnPrimaryColorHover=t.darker,t.btnSecondaryColor=t.darkest,t.btnSecondaryBorderColor=t.base,t.btnTertiaryBgHoverColor=i.palette.slate[70].value,t.btnTertiaryHoverColor=t.lightest,t.btnTertiaryBgHoverColorHeader=i.palette.slate[70].value,t.btnTertiaryHoverColorHeader=i.palette.white.value,t.btnLinkColor=t.lighter,t.btnBgHoverColor=i.palette.slate[80].value,t.tabBottomBorderColor=i.palette.slate[60].value,t.btnActionsHoverColor=i.palette.white.value,t.moduleTabsSelectedTextColor=i.palette.slate[10].value,t.headerTextColor=i.palette.slate[10].value,t.headerTabsTextColor=i.palette.slate[10].value,t.headerTabBorder=i.palette.slate[60].value,t.verticalTabBorder=i.palette.slate[60].value,t.tabFocusIndicator=i.palette.white.value,t.tabSelectedColor=i.palette.slate[90].value,t.tabHeaderColor=i.palette.slate[90].value,t.tabBorderColor=i.palette.slate[90].value,t.tabColor=i.palette.slate[80].value,t.tabHoverColor=i.palette.slate[70].value,t.tabVerticalHoverColor=i.palette.slate[70].value,t.tabSelectedTextColor=i.palette.slate[10].value,t.tabVerticalSelectedTextColor=i.palette.slate[10].value,t.tabVerticalColor=i.palette.slate[90].value,t.tabVerticalSelectedColor=i.palette.slate[100].value,t.tabHoverTextColor=i.palette.white.value,t.tabCloseHoverColor=i.palette.white.value,t.tabDivider=i.palette.slate[100].value,t.headerTabsSelectedTextColor=i.palette.slate[10].value,t.tabFocusIndicatorVert=i.palette.white.value,t.tabFocusBoxShadowVert="none"),u&&(t.base="#50535A",t.btnPrimaryColor=t.base,t.dark=t.base,t.contrast=i.palette.white.value,t.focusBoxShadow="0 0 0 2px transparent, 0 0 0 1px ".concat(t.contrast,", 0 0 1px 1px ").concat(t.contrast),t.darker="#313236",t.btnPrimaryColorHover=t.darker,t.btnHoverColor=i.palette.white.value,t.hyperlinkText=i.palette.white.value,t.lighter="#656871",t.btnDisabledColor="rgba(255, 255, 255, 0.4) !important",t.btnSecondaryColor=t.darkest,t.btnSecondaryBorderColor=t.base,t.btnTertiaryBgHoverColor=i.palette.slate[50].value,t.btnTertiaryHoverColor=i.palette.white.value,t.btnLinkColor=t.light,t.btnBgHoverColor="#313236",t.tabBottomBorderColor="#50535A",t.moduleTabsSelectedTextColor=i.palette.white.value,t.headerTextColor=i.palette.graphite[10].value,t.headerIconColor=i.palette.graphite[10].value,t.tabCloseInactiveColor=i.palette.graphite[10].value,t.tabCloseHoverColor=i.palette.white.value,t.tabVerticalHoverColor=i.palette.slate[80].value,t.tabHeaderColor=i.palette.slate[70].value,t.headerTabBorder=i.palette.slate[50].value,t.tabSelectedColor=i.palette.slate[70].value,t.tabColor=i.palette.slate[80].value,t.tabHoverColor=i.palette.slate[80].value,t.tabBorderColor=i.palette.slate[70].value,t.tabSelectedTextColor=i.palette.white.value,t.tabVerticalSelectedTextColor=i.palette.white.value,t.tabHoverTextColor=i.palette.white.value,t.tabVerticalColor=i.palette.slate[70].value,t.tabVerticalSelectedColor=i.palette.slate[80].value),p&&(t.tabBorderColor=i.palette.white.value,t.tabHoverColor=i.palette.slate[20].value,t.tabVerticalSelectedTextColor=i.palette.azure[90].value,t.headerTabsSelectedTextColor=i.palette.azure[90].value,t.moduleTabsSelectedTextColor=i.palette.azure[90].value));var g=this.getColorMap(100),f=s?this.getColorMap(20):this.getColorMap(10);t.darkestPalette=g[t.base]||t.subheader,t.lightestPalette=f[t.base]||t.subheader,d&&(t.lightestPalette=i.palette.azure[10].value,s&&(t.lightestPalette=i.palette.azure[20].value));var m={amber:["#db7726","#bb5500"],amethyst:["#9279a6","#7834dd"],emerald:["#56932e","#1f9254"],slate:["#50535a","#98949e"]},v=!1,b=O.validateHex("".concat(t.header||e.header).toLowerCase());Object.keys(m).forEach((function(t){v=v||m[t].indexOf(b)>-1}));var y=t.light,w=v?.5:.7,x=v?.3:.5;a?(w=v?.6:.8,x=v?.6:.8,n?(y=t.darkest,w=.1):s&&(w=.8,x=.4)):n?(y=t.darkest,w=-.1):s&&(w=.3);var C=O.getLuminousColorShade(y,w);C=O.getDesaturatedColor(C,x),t.baseDisabled=d?"#C5C5C9":C,t.baseDisabledText="#ffffff";var k=d&&(h||u)?"#ffffff":t.text;t.hyperlinkText=k,t.hyperlinkTextHover=e.subtext,t.selection=e.subtext;var T=O.getContrastColor(t.darkest);return e.tooltipText="white"===T?"ffffff":"000000",t.tooltipText=O.validateHex(t.tooltipText||e.tooltipText),function(t){var e=O.hexToRgb(t.base),i=O.hexToRgb(t.hyperlinkText);return"\n.is-personalizable ::selection {\n  background: ".concat(t.selection," !important;\n}\n\n.is-personalizable .header::selection {\n  background-color: ").concat(t.darker," !important;\n}\n\n.is-personalizable:not(.header) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled),\n.btn-primary:not(:disabled):not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane).is-personalizable {\n  background-color: ").concat(t.btnPrimaryColor,";\n  border-color: ").concat(t.btnPrimaryColor,";\n}\n\n.header.is-personalizable .buttonset .searchfield-wrapper.is-open button:not(.close) {\n  background-color: ").concat(t.contrast,";\n  color: ").concat(t.darkest," !important;\n}\n\n.header.is-personalizable .buttonset .searchfield-wrapper.is-open button.go-button:hover {\n  background-color: ").concat(t.secondaryButtonHover," !important;\n  color: ").concat(t.darkest," !important;\n}\n\nhtml.theme-classic-dark .is-personalizable .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):disabled {\n  color: #888B94 !important;\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) {\n  background-color: ").concat(t.lighter,";\n  border-color: ").concat(t.lighter,';\n}\n\nhtml[class*="new-"] .is-personalizable:not(.header) .btn-secondary:not(:disabled):not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane),\nhtml[class*="new-"] .is-personalizable:not(.header) .btn-secondary:not(:disabled):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane) svg.icon,\nhtml[class*="new-"] .is-personalizable:not(.header) .btn-secondary.btn-menu:not(:disabled):not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane),\nhtml[class*="new-"] .is-personalizable:not(.header) .btn-secondary.btn-menu:not(:disabled):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane) svg.icon  {\n  border-color: ').concat(t.btnSecondaryBorderColor,";\n  color: ").concat(t.btnSecondaryBorderColor,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) svg.icon {\n  border-color: ").concat(t.lighter,";\n  color: ").concat(t.lighter,";\n}\n\nhtml.theme-classic-dark .is-personalizable:not(.header) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):disabled {\n  background-color: ").concat(t.baseDisabled," !important;\n  border-color: ").concat(t.baseDisabled,' !important;\n}\n\nhtml[class*="theme-classic-"]:not(.theme-classic-dark) .is-personalizable:not(.header) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):disabled,\nhtml[class*="theme-classic-"]:not(.theme-classic-dark) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane).is-personalizable:disabled {\n  color: ').concat(t.baseDisabledText,";\n  background-color: ").concat(t.baseDisabled,";\n  border-color: ").concat(t.baseDisabled,";\n}\n\n.is-personalizable:not(.header) .btn-tertiary:not(:disabled):hover {\n  background-color: ").concat(t.lightestPalette,";\n}\n\n.is-personalizable:not(.header) .btn-link:not(:disabled):hover {\n  background-color: ").concat(t.lightestPalette,';\n}\n\nhtml[class*="new-"] .is-personalizable:not(.header) .btn-link:not(:disabled):hover {\n  background-color: ').concat(t.btnTertiaryBgHoverColor,';\n}\n\nhtml[class*="new-"] .is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not([disabled]):not(:disabled):hover {\n  background-color: ').concat(t.btnTertiaryBgHoverColor,";\n}\n\n.is-personalizable:not(.header) .btn-link:not(:disabled),\n.btn-link.is-personalizable:not(:disabled),\n.is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not([disabled]):not(:disabled) {\n  color: ").concat(t.btnLinkColor,";\n}\n\n.is-personalizable:not(.header) .btn-link:not(:disabled) .icon,\n.btn-link.is-personalizable:not(:disabled) .icon,\n.is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not([disabled]):not(:disabled) .icon {\n  color: ").concat(t.btnLinkColor," !important;\n}\n\n.is-personalizable:not(.header) .btn-link:not(:disabled):hover .icon,\n.btn-link.is-personalizable:not(:disabled):hover .icon,\n.is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not([disabled]):not(:disabled):hover .icon {\n  color: ").concat(t.btnTertiaryHoverColor," !important;\n}\n\n.is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not(.hide-focus):focus {\n  box-shadow: ").concat(t.btnBoxShadow,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-link:not(:disabled),\nhtml.theme-new-dark .btn-link.is-personalizable:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-link:not(:disabled) .icon,\nhtml.theme-new-dark .btn-link.is-personalizable:not(:disabled) .icon {\n  color: ").concat(t.btnLighter,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-link:not(:disabled):hover,\nhtml.theme-new-dark .btn-link.is-personalizable:not(:disabled):hover,\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-link:not(:disabled):hover .icon,\nhtml.theme-new-dark .btn-link.is-personalizable:not(:disabled):hover .icon,\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-menu.is-open:not(.btn-secondary):not([disabled]):not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not([disabled]):not(:disabled):hover,\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not([disabled]):not(:disabled):hover .icon {\n  color: ").concat(t.btnTertiaryHoverColor,";\n}\n\n.is-personalizable button.is-pressed,\nbutton.is-personalizable.is-pressed {\n  color: ").concat(t.base,";\n}\n\n.is-personalizable button.is-pressed .icon,\nbutton.is-personalizable.is-pressed .icon {\n  color: ").concat(t.base,";\n}\n\n.is-personalizable:not(.header) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\n.btn-primary.is-personalizable:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) {\n  background-color: ").concat(t.btnPrimaryColorHover,";\n  border-color: ").concat(t.btnPrimaryColorHover,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-primary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) {\n  background-color: ").concat(t.lightest,";\n  border-color: ").concat(t.lightest,';\n}\n\nhtml[class*="new-"]:not(.theme-new-dark) .is-personalizable:not(.header) .btn-secondary:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml[class*="new-"]:not(.theme-new-dark) .is-personalizable:not(.header) .btn-secondary.btn-menu.is-open:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) {\n  background-color: ').concat(t.lightestPalette,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary.btn-menu:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary.btn-menu.is-open:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) {\n  color: ").concat(t.lightest," !important;\n  border-color: ").concat(t.lightest," !important;\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary.btn-menu.is-open:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) .icon {\n  color: ").concat(t.lightest," !important;\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary.btn-menu:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary.btn-menu.is-open:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary.btn-menu.is-open:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) {\n  background-color: #28282A;\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-secondary:not(.go-button):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) svg.icon {\n  color: ").concat(t.lightest,';\n}\n\nhtml[class*="new-"] .is-personalizable:not(.header):not(.hero-widget) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml[class*="new-"] .is-personalizable:not(.header):not(.hero-widget) .btn-link:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) {\n  background-color: ').concat(t.lightestPalette,";\n  color: ").concat(t.btnTertiaryHoverColor,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.header):not(.hero-widget) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header):not(.hero-widget) .btn-link:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) {\n  background-color: ").concat(t.btnTertiaryBgHoverColor,";\n  color: ").concat(t.btnTertiaryHoverColor,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.hero-widget) .btn-tertiary:not:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) svg.icon,\nhtml.theme-new-dark .is-personalizable .btn-link:hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable .btn-link:hover:not(:disabled) svg.icon,\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) svg.icon,\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-link:hover:not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.header) .btn-link:hover:not(:disabled) svg.icon {\n  color: ").concat(t.lightest,';\n}\n\nhtml[class*="new-"] .is-personalizable .btn-tertiary:hover:not(:disabled):not(.close):not(.btn-filter):not(.personalize-actionable):not(.destructive) svg.icon {\n  color: ').concat(t.btnTertiaryHoverColor,";\n}\n\nhtml.theme-new-dark .is-personalizable:not(.hero-widget) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled),\nhtml.theme-new-dark .is-personalizable:not(.hero-widget) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) svg.icon {\n  color: ").concat(t.contrast," !important;\n}\n\nhtml.theme-new-dark .is-personalizable:not(.hero-widget) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) svg.icon {\n  color: ").concat(t.contrast," !important;\n}\n\nbutton.is-personalizable button:focus:not(.hide-focus),\n.is-personalizable a.btn:focus:not(.hide-focus),\na.btn.is-personalizable:focus:not(.hide-focus),\n.is-personalizable a.btn-menu:focus:not(.hide-focus),\na.btn-menu.is-personalizable:focus:not(.hide-focus),\n.is-personalizable a.btn-icon:focus:not(.hide-focus),\na.btn-icon.is-personalizable:focus:not(.hide-focus),\n.is-personalizable a.btn-tertiary:focus:not(.hide-focus),\na.btn-tertiary.is-personalizable:focus:not(.hide-focus),\n.is-personalizable a.btn-close:focus:not(.hide-focus),\na.btn-close.is-personalizable:focus:not(.hide-focus) {\n  box-shadow: 0 0 0 2px transparent,\n    0 0 0 1px ").concat(t.base,",\n    0 0 4px 2px rgba(").concat(e.r,", ").concat(e.g,", ").concat(e.b,", 0.3) !important;\n}\n\n.is-personalizable:not(.hero-widget) .btn-menu:not(.btn-primary):not(.btn-secondary).is-open,\n.btn-menu:not(.btn-primary):not(.btn-secondary).is-personalizable.is-open,\n.btn-actions:not(.btn-primary):not(.btn-secondary).is-personalizable.is-open {\n  color: ").concat(t.btnMenu,";\n}\n\n.is-personalizable:not(.hero-widget) .btn-menu:not(.btn-primary):not(.btn-secondary).is-open .icon,\n.btn-menu:not(.btn-primary):not(.btn-secondary).is-personalizable.is-open .icon,\n.btn-actions:not(.btn-primary):not(.btn-secondary).is-personalizable.is-open .icon {\n  color: ").concat(t.btnMenu,";\n}\n\n.is-personalizable:not(.application-menu) .btn-actions:not(.btn-primary):not(.btn-secondary).is-open .icon {\n  color: ").concat(t.btnActionsHoverColor,";\n}\n\n.is-personalizable .hyperlink:not(.today),\n.hyperlink:not(.today).is-personalizable {\n  color: ").concat(t.hyperlinkText," !important;\n  opacity: 0.8;\n}\n\n.is-personalizable .hyperlink:not(.today):hover,\n.hyperlink:not(.today).is-personalizable:hover {\n  color: ").concat(t.hyperlinkTextHover," !important;\n}\n\n.is-personalizable .hyperlink:not(.today):focus:not(.hide-focus),\n.hyperlink:not(.today).is-personalizable:focus:not(.hide-focus) {\n  border-color: ").concat(t.hyperlinkText,";\n  box-shadow: 0 0 4px 3px rgba(").concat(i.r,", ").concat(i.g,", ").concat(i.b,", 0.3);\n}\n\n.is-personalizable button:not(.btn-monthyear-pane):not(.btn-editor) svg.ripple-effect,\nbutton:not(.btn-monthyear-pane):not(.btn-editor).is-personalizable svg.ripple-effect,\n.is-personalizable a svg.ripple-effect,\na.is-personalizable svg.ripple-effect {\n  background-color: ").concat(t.base," !important;\n}\n\n.is-personalizable .btn-primary svg.ripple-effect,\n.btn-primary.is-personalizable svg.ripple-effect,\n.is-personalizable .btn-secondary svg.ripple-effect,\n.btn-secondary.is-personalizable svg.ripple-effect {\n  background-color: ").concat(t.contrast," !important;\n  border-color: ").concat(t.contrast," !important;\n}\n\n.is-personalizable .count-container .instance-count svg.icon {\n  background-color: ").concat(t.base," !important;\n}\n\n.is-personalizable.has-more-button.tab-container.horizontal {\n  &::after {\n    background-image:\n      linear-gradient(to left,\n        rgba($header-bg-color, 1),\n        rgba($header-bg-color, 0));\n    height: 39px;\n  }\n}\n\n.tab-container.module-tabs.is-personalizable {\n  border-top: 1px solid ").concat(t.tabBorderColor,";\n  border-bottom: 1px solid ").concat(t.tabBorderColor,";\n}\n\n.tab-container.is-personalizable.header-tabs:not(.alternate) .tab-focus-indicator.is-visible,\n.tab-container.personalize-header.header-tabs:not(.alternate) .tab-focus-indicator.is-visible,\n.tab-container.is-personalizable.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled).is-selected,\n.tab-container.personalize-header.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled).is-selected,\n.header.is-personalizable .tab-container.header-tabs:not(.alternate) .tab-focus-indicator.is-visible,\n.header.is-personalizable .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab.is-selected,\n.personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab.is-selected:not(.is-disabled),\n.personalize-header.tab-container.header-tabs:not(.alternate) .tab-focus-indicator.is-visible {\n  border-color: ").concat(t.headerTabsSelectedTextColor," !important;\n  box-shadow: ").concat(t.tabFocusBoxShadow," !important;\n}\n\n.header.is-personalizable .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled).is-selected {\n  box-shadow: none !important;\n}\n\n.tab-container.is-personalizable.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled),\n.tab-container.is-personalizable.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled),\n.header.is-personalizable .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled):not(.is-selected),\n.personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled) {\n  color: ").concat(t.contrast,";\n  background-color: ").concat(t.tabHoverColor,";\n  border-bottom: 4px solid ").concat(t.contrast,";\n}\n\n.module-tabs.is-personalizable .tab:not(:first-child) {\n  border-left: 1px solid ").concat(t.tabDivider," !important;\n}\n\n.module-tabs.is-personalizable {\n  background-color: ").concat(t.darker," !important;\n}\n\n.module-tabs.is-personalizable .tab.is-selected {\n  background-color: ").concat(t.tabSelectedColor," !important;\n  color: ").concat(t.moduleTabsSelectedTextColor," !important;\n}\n\n.module-tabs.is-personalizable .tab:not(.is-selected) {\n  background-color: ").concat(t.tabColor," !important;\n}\n\n.module-tabs.is-personalizable .tab {\n  color: ").concat(t.contrast,";\n}\n\n.tab-container.module-tabs.is-personalizable .tab.dismissible svg.close {\n  color: ").concat(t.tabCloseInactiveColor,";\n}\n\n.tab-container.module-tabs.is-personalizable .tab.dismissible svg.close:hover {\n  color: ").concat(t.tabCloseHoverColor,";\n}\n\n.is-personalizable.tab-container.module-tabs .tab-more,\n.is-personalizable.tab-container.module-tabs .add-tab-button {\n  color: ").concat(t.contrast,";\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab.is-selected {\n  background-color: ").concat(t.tabVerticalSelectedColor," !important;\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab a {\n  font-weight: ").concat(t.tabTextWeight," !important;\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab.is-selected a {\n  color: ").concat(t.tabVerticalSelectedTextColor," !important;\n  font-weight: ").concat(t.tabTextSelectedWeight," !important;\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab:not(.is-disabled):hover {\n  background-color: ").concat(t.tabVerticalHoverColor," !important;\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab.is-selected:hover {\n  background-color: ").concat(t.tabVerticalSelectedColor," !important;\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab:not(.is-disabled):hover a {\n  color: ").concat(t.tabHoverTextColor," !important;\n}\n\n.tab-container.vertical.is-personalizable > .tab-list-container > .tab-list > .tab.is-selected:hover a {\n  color: ").concat(t.tabVerticalSelectedTextColor," !important;\n}\n\n.tab-container.module-tabs.is-personalizable .tab-focus-indicator.is-visible:is(.is-selected),\n.tab-container.vertical.is-personalizable .tab-focus-indicator.is-visible:is(.is-selected) {\n  border-color: transparent !important;\n  box-shadow: none !important;\n}\n\n.tab-container.module-tabs.is-personalizable .tab-focus-indicator.is-visible:not(.is-selected) {\n  border-color: ").concat(t.tabFocusIndicator," !important;\n  box-shadow: ").concat(t.tabFocusBoxShadow," !important;\n}\n\n.tab-container.vertical.is-personalizable .tab-focus-indicator.is-visible:not(.is-selected) {\n  border-color: ").concat(t.tabFocusIndicatorVert," !important;\n  box-shadow: ").concat(t.tabFocusBoxShadowVert," !important;\n}\n\n.builder-header.is-personalizable {\n  background-color: ").concat(t.lighter,";\n}\n\n.header.is-personalizable {\n  background-color: ").concat(t.tabHeaderColor,";\n  border-bottom: 1px solid ").concat(t.headerTabBorder,";\n}\n\n.scrollable-flex-header .breadcrumb {\n  background-color: ").concat(t.base,";\n}\n\n.header.is-personalizable.has-tabs .tab-container.header-tabs:not(.alternate) {\n  background-color: ").concat(t.tabHeaderColor,";\n}\n\n.header.is-personalizable .title {\n  color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable h1,\n.header.is-personalizable h2,\n.header.is-personalizable h3 {\n  color: ").concat(t.headerTextColor," !important;\n}\n\n.object-count.personalize-text {\n  color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable button svg.ripple-effect {\n  background-color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable button:not(:disabled),\n.header.is-personalizable button:not(:disabled):not(.close) .icon,\n.header.is-personalizable button:not(:disabled) .app-header.icon > span,\n.subheader.is-personalizable button:not(:disabled):not(.searchfield-category-button),\n.subheader.is-personalizable button:not(:disabled):not(.searchfield-category-button) .icon,\n.subheader.is-personalizable button:not(:disabled):not(.searchfield-category-button) .app-header.icon > span,\n.is-personalizable .personalize-subheader button:not(:disabled),\n.is-personalizable .personalize-subheader button:not(:disabled):not(.close):not(.notification-close) .icon,\n.is-personalizable .personalize-subheader button:not(:disabled) .app-header.icon > span  {\n  color: ").concat(t.headerIconColor,' !important;\n}\n\nhtml[class*="theme-new"] .header.is-personalizable button:not(.close):not(:disabled):hover .icon {\n  color: ').concat(t.btnTertiaryHoverColorHeader,' !important;\n}\n\nhtml[class*="-dark"] .is-personalizable .personalize-subheader button:not(:disabled),\nhtml[class*="-dark"] .is-personalizable .subheader button:not(:disabled),\nhtml[class*="-dark"] .is-personalizable .personalize-subheader button:not(:disabled) .icon,\nhtml[class*="-dark"] .is-personalizable .subheader button:not(:disabled) .icon {\n  color: #ffff !important;\n}\n\n.header.is-personalizable button:not(:disabled) .app-header.icon > span,\n.is-personalizable .personalize-subheader button:not(:disabled) .app-header.icon > span {\n  background-color: ').concat(t.contrast," !important;\n  opacity: .8;\n}\n\n.header.is-personalizable:not(.has-alternate-tabs) .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled):hover {\n  border-bottom: 4px solid ").concat(t.contrast,';\n}\n\nhtml[class*="theme-new"] .header.is-personalizable:not(.has-alternate-tabs) .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled):hover {\n  border-bottom: 2px solid ').concat(t.headerTabsTextColor,';\n}\n\nhtml[class*="theme-new"] .header.is-personalizable:not(.has-alternate-tabs) .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:is(.is-selected):not(.is-disabled):hover {\n  background-color: transparent;\n}\n\n.header.is-personalizable .tab-container.horizontal > .tab-list-container .tab:not(.is-disabled):hover {\n  border-bottom: 4px solid ').concat(t.base,';\n}\n\nhtml[class*="theme-new-"] .header.is-personalizable button:not(:disabled):hover .app-header.icon > span,\nhtml[class*="theme-new-"] .header.is-personalizable .toolbar [class^=\'btn\']:hover:not(.go-button):not(.close):not(.searchfield-category-button):not([disabled]),\nhtml[class*="theme-new-"] .subheader.is-personalizable button:not(.go-button):not(.close):not(.searchfield-category-button):not(:disabled):hover,\nhtml[class*="theme-new-"] .subheader.is-personalizable button:not(:disabled):hover .app-header.icon > span,\nhtml[class*="theme-new-"] .subheader.is-personalizable .toolbar [class^=\'btn\']:hover:not(.go-button):not(.close):not(.searchfield-category-button):not([disabled]),\nhtml[class*="theme-new-"] .personalize-subheader button:not(.go-button):not(.close):not(.searchfield-category-button):not(:disabled):hover,\nhtml[class*="theme-new-"] .personalize-subheader button:not(:disabled):hover .app-header.icon > span,\nhtml[class*="theme-new-"] .personalize-subheader .toolbar [class^=\'btn\']:hover:not(.go-button):not(.close):not(.searchfield-category-button):not([disabled]) {\n  color: ').concat(t.btnTertiaryHoverColor," !important;\n  background-color: ").concat(t.btnTertiaryBgHoverColor,' !important;\n  opacity: 1;\n}\n\nhtml[class*="theme-new-"] .header.is-personalizable button:not(.go-button):not(.close):not(.searchfield-category-button):not(:disabled):hover {\n  background-color: ').concat(t.btnTertiaryBgHoverColorHeader," !important;\n  color: ").concat(t.btnTertiaryHoverColorHeader," !important;\n}\n\n.header .flex-toolbar [class^='btn'][disabled] {\n  color: ").concat(t.btnDisabledColor,";\n}\n\n.header.is-personalizable .buttonset .searchfield-wrapper.non-collapsible.toolbar-searchfield-wrapper.has-categories .btn-icon.close:hover {\n  background-color: ").concat(t.contrast," !important;\n}\n\nhtml.theme-new-dark .header.is-personalizable .buttonset .searchfield-wrapper.non-collapsible.toolbar-searchfield-wrapper.has-categories .btn-icon.close,\nhtml.theme-new-dark .header.is-personalizable .buttonset .searchfield-wrapper.non-collapsible.toolbar-searchfield-wrapper.has-categories .btn-icon.close svg.close,\nhtml.theme-classic-dark .header.is-personalizable .buttonset .searchfield-wrapper.non-collapsible.toolbar-searchfield-wrapper.has-categories .btn-icon.close,\nhtml.theme-classic-dark .header.is-personalizable .buttonset .searchfield-wrapper.non-collapsible.toolbar-searchfield-wrapper.has-categories .btn-icon.close svg.close {\n  background-color: transparent !important;\n}\n\n.header.is-personalizable button:not(:disabled) .app-header.icon > span {\n  background-color: ").concat(t.contrast," !important;\n  opacity: 1;\n}\n\n.header.is-personalizable .go-button.is-personalizable {\n  background-color: ").concat(t.lightest,";\n  border-color:").concat(t.lightest,";\n  color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable.has-tabs .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab.is-selected:not(.is-disabled),\n.personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab.is-selected:not(.is-disabled) {\n  color: ").concat(t.headerTabsSelectedTextColor," !important;\n  opacity: 1;\n  background-color: transparent;\n}\n\n.header.is-personalizable .flex-toolbar .has-collapse-button .collapse-button {\n  background-color: transparent;\n}\n\n.header.is-personalizable .flex-toolbar .has-collapse-button .collapse-button:focus:not(.hide-focus) {\n  box-shadow: none !important;\n  border-color: ").concat(t.borderActiveSearchHeader,";\n}\n\nhtml.theme-new-dark .header.is-personalizable.has-tabs .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab.is-selected:not(.is-disabled) {\n  color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable .tab-container.header-tabs:not(.alternate) .tab-more::before,\n.tab-container.is-personalizable.header-tabs:not(.alternate) .tab-more::before {\n  background-color: ").concat(t.contrast,";\n}\n\n.is-personalizable .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled),\n.header.is-personalizable .tab-container.header-tabs:not(.alternate) .tab-more svg.icon,\n.tab-container.is-personalizable .tab-more svg.icon,\n.is-personalizable.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled),\n.personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled)  {\n  color: ").concat(t.contrast," !important;\n  opacity: .8;\n}\n\nhtml.theme-new-dark .personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled),\nhtml.theme-classic-dark .personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled) {\n  color: #fff !important;\n  opacity: .8;\n}\n\n.header.is-personalizable.has-tabs .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab,\n.is-personalizable.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab,\n.personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab {\n  color: ").concat(t.contrast," !important;\n  opacity: .8;\n}\n\n.header.is-personalizable.has-tabs .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled):not(.is-selected),\n.is-personalizable.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled),\n.personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:hover:not(.is-disabled) {\n  color: ").concat(t.contrast," !important;\n  opacity: 1;\n}\n\nhtml.theme-new-dark .header.is-personalizable.has-tabs .tab-container.header-tabs > .tab-list-container .tab:hover:not(.is-disabled),\n.is-personalizable.tab-container.header-tabs > .tab-list-container .tab:hover:not(.is-disabled)  {\n  color: ").concat(t.contrast,' !important;\n  opacity: 1;\n}\n\nhtml[class*="theme-new-"] .header.is-personalizable.has-tabs .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled),\nhtml[class*="theme-new-"] .is-personalizable.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled),\nhtml[class*="theme-new-"] .personalize-header.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab:not(.is-disabled) {\n  opacity: 1;\n}\n\n.is-personalizable .count-container .instance-count .count {\n  border-color: ').concat(t.contrast," !important;\n  color: ").concat(t.contrast,' !important;\n}\n\nhtml[class*="theme-new-"] .is-personalizable .count-container .instance-count .title {\n  color: ').concat(t.contrast," !important;\n}\n\nhtml.theme-classic-dark .is-personalizable .count-container .instance-count .count {\n  border-color: ").concat(t.subtext," !important;\n  color: ").concat(t.subtext," !important;\n}\n\n.header.is-personalizable .page-title {\n  color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable .flex-toolbar [class^='btn'][disabled] .icon,\n.header.is-personalizable .btn:not(.searchfield-category-button)[disabled] span {\n  color: ").concat(t.btnDisabledColor,";\n}\n\n.header.is-personalizable button:not(:disabled),\n.subheader.is-personalizable button:not(:disabled):not(.searchfield-category-button) {\n  color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable  .flex-toolbar [class^='btn']:focus:not(.hide-focus) .icon {\n  color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable.has-tabs .tab-container.header-tabs > .tab-list-container .tab:hover:not(.is-disabled)::before {\n  background-color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable.has-tabs .animated-bar {\n  background-color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable.has-tabs .tab-list-container .tab.is-selected:not(.is-disabled):hover::before {\n  background-color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable .buttonset .toolbar-searchfield-wrapper:not(.is-open) svg.icon:not(.close),\n.header.is-personalizable .toolbar-searchfield-wrapper:not(.is-open) .icon:not(.close),\n.header .toolbar-searchfield-wrapper .searchfield,\n.masthead .toolbar-searchfield-wrapper .searchfield,\n.header.is-personalizable .toolbar [class^='btn']:focus .icon {\n  color: ").concat(t.contrast,";\n}\n\n.header .toolbar [class^='btn']:hover:not(.go-button):not(.close):not(.searchfield-category-button) {\n  background-color: ").concat(t.base,";\n}\n\n.header.alabaster .toolbar [class^='btn']:hover:not(.go-button):not(.searchfield-category-button) {\n  background-color: #E6F1FD !important;\n}\n\n.personalize-subheader.scrollable-flex-header .breadcrumb:not(.alternate) .hyperlink,\n.personalize-subheader..scrollable-flex-header .breadcrumb:not(.alternate) .breadcrumb-text,\n.personalize-subheader.scrollable-flex-header .breadcrumb:not(.alternate) .hyperlink,\n.personalize-subheader.scrollable-flex-header .breadcrumb:not(.alternate) .breadcrumb-list li::after,\n.personalize-subheader.scrollable-flex-header .breadcrumb:not(.alternate) ol li::after {\n  color: ").concat(t.contrast,";\n}\n\n.personalize-subheader.scrollable-flex-header .breadcrumb .hyperlink:focus:not([disabled]) {\n  border: 1px solid ").concat(t.contrast,";\n}\n\n.personalize-subheader.scrollable-flex-header .breadcrumb:not(.alternate) .hyperlink:hover:not([disabled]) {\n  border-bottom: 1px solid ").concat(t.contrast,";\n}\n\n.header .toolbar-searchfield-wrapper.active.has-focus .searchfield {\n  border-color: ").concat(t.borderActiveSearchHeader,';\n}\n\nhtml[class*="-dark"] .header .toolbar-searchfield-wrapper.active.has-focus .searchfield {\n  border-color: ').concat(t.contrast,";\n}\n\n.header.is-personalizable .btn-actions:not(.btn-primary):not(.btn-secondary).is-open .icon,\n.header.is-personalizable .wizard-header .tick,\n.header.is-personalizable .wizard-header .tick .label,\n.header.is-personalizable .wizard-header a.tick:hover:not(.current):not(.is-disabled):not([disabled]) .label {\n  color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable .wizard-header .tick:focus .label {\n  border-color: ").concat(t.lighter,";\n}\n\n.header.is-personalizable .wizard-header .tick.current::before {\n  border-color: ").concat(t.light,";\n  border-color: ").concat(t.contrast,";\n}\n\n.is-personalizable .personalize-text {\n  color: ").concat(t.contrast,";\n}\n\nhtml[class*='classic-'] .is-personalizable .tab-container.header-tabs.alternate > .tab-list-container .tab.is-selected:not(.is-disabled) {\n  border-color: ").concat(t.contrast,";\n}\n\n.tab-container.header-tabs.alternate > .tab-list-container .tab.is-disabled {\n  color: ").concat(t.contrast,";\n  opacity: 0.6;\n}\n\n.is-personalizable .tab-container.header-tabs > .tab-list-container .tab.is-disabled {\n  color: ").concat(t.contrast,";\n  opacity: 0.6;\n}\n\n.is-personalizable .label + .personalize-text.data {\n  color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable .wizard-header .tick::after {\n  background-color: ").concat(t.base,";\n  border: 2px solid ").concat(t.contrast,";\n}\n\n.header.is-personalizable .wizard-header .bar,\n.header.is-personalizable .wizard-header .tick.complete {\n  background-color: ").concat(t.darkest,";\n}\n\n.header.is-personalizable .wizard-header .tick.complete::after,\n.header.is-personalizable .wizard-header .completed-range {\n  background-color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable .wizard-header .tick.complete .label {\n  color: ").concat(t.contrast,";\n}\n\n.header.is-personalizable .wizard-header .tick.current {\n  box-shadow: none;\n}\n.header.is-personalizable .wizard-header .tick.current::after {\n  background-color: ").concat(t.contrast,";\n  border-color: ").concat(t.contrast,";\n}\n\n.subheader.is-personalizable .go-button.is-personalizable {\n  background-color: ").concat(t.dark,";\n  border-color: ").concat(t.dark,";\n  color: ").concat(t.contrast,";\n}\n\n.subheader.is-personalizable .go-button.is-personalizable:not(.hide-focus) {\n  box-shadow: 0 0 0 1px #FAFAFA, 0 0 0 2px ").concat(t.darkest,", 0 0 4px 2px rgba(41, 41, 41, 0.3);\n}\n\n.is-personalizable .breadcrumb .hyperlink,\n.breadcrumb.is-personalizable .hyperlink {\n  color: ").concat(t.theme.text,";\n}\n.is-personalizable .breadcrumb .hyperlink:hover,\n.breadcrumb.is-personalizable .hyperlink:hover {\n  color: ").concat(t.theme.text,";\n}\n.is-personalizable .breadcrumb .hyperlink:focus:not(.hide-focus),\n.breadcrumb.is-personalizable .hyperlink:focus:not(.hide-focus) {\n  box-shadow: none;\n}\n.is-personalizable .breadcrumb .hyperlink[disabled],\n.breadcrumb.is-personalizable .hyperlink[disabled],\n.is-personalizable .breadcrumb .hyperlink[disabled]:hover,\n.breadcrumb.is-personalizable .hyperlink[disabled]:hover {\n  color: ").concat(t.theme.disabledText,";\n}\n\n.is-personalizable .scrollable-flex-header .breadcrumb:not(.alternate),\n.scrollable-flex-header.is-personalizable .breadcrumb:not(.alternate) {\n  background-color: ").concat(t.base,";\n}\n.is-personalizable .scrollable-flex-header .breadcrumb.truncated:not(.alternate) .breadcrumb-list::before,\n.scrollable-flex-header.is-personalizable .breadcrumb.truncated:not(.alternate) .breadcrumb-list::before {\n  background-image: linear-gradient(to right, ").concat(t.base,", ").concat(O.hexToRgba(t.base,0),");\n}\nhtml[dir='rtl'] .is-personalizable .scrollable-flex-header .breadcrumb.truncated:not(.alternate) .breadcrumb-list::before,\nhtml[dir='rtl'] .scrollable-flex-header.is-personalizable .breadcrumb.truncated:not(.alternate) .breadcrumb-list::before {\n  background-image: linear-gradient(to right, ").concat(O.hexToRgba(t.base,0),", ").concat(t.base,");\n}\n.is-personalizable .scrollable-flex-header .breadcrumb:not(.alternate) .hyperlink,\n.scrollable-flex-header.is-personalizable .breadcrumb:not(.alternate) .hyperlink {\n  color: ").concat(t.contrast,";\n}\n.is-personalizable .scrollable-flex-header .breadcrumb:not(.alternate) .hyperlink:hover,\n.scrollable-flex-header.is-personalizable .breadcrumb:not(.alternate) .hyperlink:hover  {\n  color: ").concat(t.contrast,";\n}\n\n.is-personalizable .scrollable-flex-header .breadcrumb:not(.alternate) .btn-actions.is-open .icon,\n.scrollable-flex-header.is-personalizable .breadcrumb:not(.alternate) .btn-actions.is-open .icon {\n  color: ").concat(t.contrast,";\n}\n.is-personalizable .scrollable-flex-header .breadcrumb:not(.alternate) .btn-actions:focus:not(.hide-focus),\n.scrollable-flex-header.is-personalizable .breadcrumb:not(.alternate) .btn-actions:focus:not(.hide-focus) {\n  box-shadow: 0 0 0 2px transparent, 0 0 0 1px ").concat(t.contrast,", 0 0 1px 2px ").concat(O.hexToRgba(t.contrast,.3)," !important;\n}\n\n.module-tabs.is-personalizable .tab-more {\n  border-left-color: ").concat(t.tabDivider," !important;\n}\n\n.module-tabs.is-personalizable .tab-more.is-open {\n  background-color: ").concat(t.hover," !important;\n}\n\n.module-tabs.is-personalizable .tab-more.is-selected {\n  background-color: ").concat(t.base," !important;\n}\n\n.header .toolbar > .toolbar-searchfield-wrapper.active .searchfield {\n  background-color: ").concat(t.hover," !important;\n  border-bottom-color: ").concat(t.hover," !important;\n}\n\n.header .toolbar > .toolbar-searchfield-wrapper.active .searchfield-category-button {\n  background-color: ").concat(t.hover," !important;\n  border-bottom-color: ").concat(t.hover," !important;\n}\n\n.subheader.is-personalizable {\n  background-color: ").concat(t.lighter," !important;\n}\n\n.builder .sidebar .header {\n  border-right: 1px solid ").concat(t.hover," !important;\n}\n\n.module-tabs.is-personalizable .tab:not(.is-selected):hover,\n.module-tabs.is-personalizable .tab-more:hover {\n  background-color: ").concat(t.tabHoverColor," !important;\n  color: ").concat(t.contrast,";\n}\n\n.is-personalizable.tab-container.module-tabs .tab .icon {\n  color: ").concat(t.contrast,";\n}\n\n.module-tabs.has-toolbar.is-personalizable .tab-list-container + .toolbar {\n  border-left-color: ").concat(t.darkest,' !important;\n}\n\n.module-tabs.is-personalizable [class^="btn"] {\n  background-color: transparent;\n  color: ').concat(t.contrast," !important;\n}\n\n.module-tabs.is-personalizable .tab.is-disabled {\n  background-color: ").concat(t.darker," !important;\n  color: ").concat(t.contrast," !important;\n}\n\n.module-tabs.is-personalizable .tab.is-disabled > svg {\n  fill: ").concat(t.contrast," !important;\n}\n\n.module-tabs.is-personalizable .add-tab-button {\n  border-left-color: ").concat(t.darkest," !important;\n}\n\n.module-tabs.is-personalizable .add-tab-button:hover {\n  background-color: ").concat(t.darker," !important;\n}\n\n.module-tabs.is-personalizable .toolbar-searchfield-wrapper > .searchfield {\n  color: ").concat(t.contrast,";\n}\n\n.module-tabs.is-personalizable .toolbar-searchfield-wrapper > svg {\n  fill: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable .toolbar [class^='btn'][disabled],\n.header.is-personalizable .toolbar [class^='btn'][disabled] .icon,\n.header.is-personalizable .toolbar [class^='btn'][disabled] span {\n  color: ").concat(t.btnDisabledColor,";\n}\n\n.header .toolbar [class^='btn']:focus:not(.hide-focus):not(.collapse-button) {\n  border-color: ").concat(t.contrast,";\n}\n\n.is-personalizable .tab-container.header-tabs:not(.alternate)::before,\n.is-personalizable.tab-container.header-tabs:not(.alternate)::before {\n  background-image: linear-gradient(to right, ").concat(t.tabHeaderColor," , ").concat(O.hexToRgba(t.base,0),") !important;\n}\n\nhtml[dir='rtl'] .is-personalizable .tab-container.header-tabs:not(.alternate)::before,\nhtml[dir='rtl'] .is-personalizable.tab-container.header-tabs:not(.alternate)::before {\n  background-image: linear-gradient(to left, ").concat(t.tabHeaderColor,", ").concat(O.hexToRgba(t.base,0),") !important;\n}\n\n.is-personalizable .tab-container.header-tabs:not(.alternate)::after,\n.is-personalizable.tab-container.header-tabs:not(.alternate)::after {\n  background-image: linear-gradient(to right, ").concat(O.hexToRgba(t.base,0),", ").concat(t.tabHeaderColor,") !important;\n}\n\nhtml[dir='rtl'] .is-personalizable .tab-container.header-tabs:not(.alternate)::after,\nhtml[dir='rtl'] .is-personalizable.tab-container.header-tabs:not(.alternate)::after {\n  background-image: linear-gradient(to left, ").concat(O.hexToRgba(t.base,0),", ").concat(t.tabHeaderColor,") !important;\n}\n\n.is-personalizable.header .section-title {\n  color: ").concat(t.contrast,";\n}\n\n.is-personalizable.header button.application-menu-trigger:hover:not(:disabled) .icon.app-header.go-back > span {\n  background-color: ").concat(t.contrast," !important;\n}\n\n.is-personalizable .tab-container.header-tabs:not(.alternate),\n.is-personalizable.tab-container.header-tabs:not(.alternate) {\n  border-bottom-color: ").concat(t.tabBottomBorderColor,";\n}\n\n.tab-container.is-personalizable.module-tabs .tab a {\n  font-weight: ").concat(t.tabTextWeight," !important;\n}\n\n.tab-container.is-personalizable.module-tabs .tab.is-selected a {\n  font-weight: ").concat(t.tabTextSelectedWeight,' !important;\n}\n\nhtml[class*="theme-new-"]:not(.theme-new-dark) .is-personalizable:not(.header) .btn-icon:not(.destructive):not(:disabled):hover {\n  background-color: ').concat(t.lightestPalette,';\n}\n\nhtml[class*="theme-new-"] .is-personalizable:not(.header) .btn-icon:not(.destructive):not(:disabled):hover .icon {\n  color: ').concat(t.btnTertiaryHoverColor,';\n}\n\nhtml[class*="-dark"] .is-personalizable .btn-tertiary:not(.destructive):not(:disabled):hover,\nhtml[class*="-dark"] .is-personalizable .btn-link:not(:disabled):hover,\nhtml[class*="-dark"] .is-personalizable:not(.header) .btn-menu:not(.btn-secondary):not(:disabled):hover,\nhtml[class*="-dark"] .is-personalizable:not(.header) .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):hover:not(:disabled) {\n  background-color: ').concat(t.btnTertiaryBgHoverColor,';\n}\n\nhtml[class*="theme-new-"] .application-menu.is-personalizable .accordion.panel.inverse .accordion-pane .accordion-header {\n  border: 1px solid ').concat(t.lighter,';\n}\n\nhtml[class*="theme-new-"] .application-menu.is-personalizable button:focus:not(.hide-focus),\nhtml[class*="theme-new-"] .application-menu.is-personalizable .hyperlink:focus:not(.hide-focus)::after {\n  border-color: ').concat(t.contrast," !important;\n  box-shadow: none !important;\n}\n\n.application-menu.is-personalizable .accordion-header.has-filtered-children > a,\n.application-menu.is-personalizable .accordion.panel .accordion-header.has-filtered-children.is-focused {\n  color: ").concat(t.contrast," !important;\n}\n\nhtml[dir='rtl'] .application-menu.is-personalizable {\n  background-color: ").concat(t.lighter,";\n  border-left: ").concat(t.light,';\n}\n\nhtml[class*="theme-new-"] .application-menu.is-personalizable button svg.ripple-effect {\n  background-color: ').concat(t.contrast,' !important;\n}\n\nhtml[class*="theme-new"] .application-menu.is-personalizable .accordion.panel.inverse > .accordion-header.is-focused.is-expanded {\n  border-color: transparent !important;\n}\n\nhtml[class*="theme-new-"] .application-menu.is-personalizable .accordion.panel.inverse > .accordion-header.is-expanded.is-focused::before {\n  border-color: ').concat(t.contrast," !important;\n}\n\n.is-personalizable.tab-container {\n  background-color: ").concat(t.tabColor," !important;\n}\n\n.is-personalizable.tab-container.vertical,\n.is-personalizable.tab-container.vertical > .tab-list-container {\n  background-color: ").concat(t.tabVerticalColor," !important;\n  min-width: 249px;\n  width: 249px;\n}\n\n.is-personalizable.tab-container.vertical {\n  border-right: 1px solid ").concat(t.verticalTabBorder,' !important;\n}\n\n.is-personalizable .tab-container.header-tabs:not(.alternate) > .tab-list-container .tab-list .tab.is-disabled,\n.is-personalizable.tab-container.header-tabs:not(.alternate) > .tab-list-container .tab-list .tab.is-disabled {\n  opacity: .4;\n}\n\nhtml[class*="theme-classic-"] .is-personalizable .tab-container .tab-list-container .tab-list .tab.is-disabled,\nhtml[class*="theme-classic-"] .tab-container.is-personalizable .tab-list-container .tab-list .tab.is-disabled {\n  opacity: .4 !important;\n}\n\n.is-personalizable .personalize-header:not(.header-tabs.alternate) {\n  background-color: ').concat(t.base," !important;\n}\n\n.is-personalizable .personalize-subheader {\n  background-color: ").concat(t.lighter," !important;\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb {\n  background-color: ").concat(t.lighter,";\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb.truncated .breadcrumb-list::before,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb.truncated ol::before {\n  background-image: linear-gradient(to right, ").concat(t.lighter,", ").concat(t.lighter,"80);\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .hyperlink,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .btn-actions .icon,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .breadcrumb-overflow-container::after,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .breadcrumb-list li::after,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb ol li::after  {\n  color: ").concat(t.contrast," !important;\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .hyperlink:hover {\n  border-bottom: 1px solid ").concat(t.contrast,"\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb {\n  background-color: ").concat(t.lighter,";\n}\n\n.is-personalizable .section-wizard {\n  background-color: ").concat(t.lighter," !important;\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb.truncated .breadcrumb-list::before,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb.truncated ol::before {\n  background-image: linear-gradient(to right, ").concat(t.lighter,", ").concat(t.lighter,"80);\n}\n\n.is-personalizable .section-wizard {\n  background-color: ").concat(t.lighter," !important;\n}\n\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .hyperlink,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .btn-actions .icon,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .breadcrumb-overflow-container::after,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .breadcrumb-list li::after,\n.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb ol li::after  {\n  color: ").concat(t.contrast,";\n}\n\n.is-personalizable .section-wizard {\n  background-color: ").concat(t.lighter," !important;\n}\n\n.is-personalizable .personalize-subheader  p:not(.notification-text),\n.personalize-subheader.is-personalizable p:not(.notification-text) {\n  color: ").concat(t.contrast," !important;\n}\n\n.is-personalizable .personalize-subheader  h1,\n.personalize-subheader.is-personalizable h1,\n.is-personalizable .personalize-subheader h2,\n.personalize-subheader.is-personalizable h2 {\n  color: ").concat(t.contrast,' !important;\n}\n\nhtml[class*="-dark"] .is-personalizable .personalize-subheader h1,\nhtml[class*="-dark"] .is-personalizable .personalize-subheader h2 {\n  color: #fff !important;\n}\n\nhtml[class*="-dark"] .is-personalizable .subheader h1,\nhtml[class*="-dark"] .is-personalizable .subheader h2 {\n  color: #fff !important;\n}\n\n.is-personalizable .personalize-subheader button:not(.notification-close) .icon:not(.notification-icon),\n.personalize-subheader.is-personalizable button:not(.notification-close) .icon:not(.notification-icon) {\n  color: ').concat(t.contrast," !important;\n}\n\n.is-personalizable .personalize-subheader .toolbar-searchfield-wrapper.active.is-open *,\n.personalize-subheader.is-personalizable .toolbar-searchfield-wrapper.active.is-open * {\n  color: ").concat(t.theme.text," !important;\n}\n\n.is-personalizable .personalize-actionable,\n.is-personalizable .personalize-actionable svg,\n.is-personalizable .personalize-actionable .icon,\n.is-personalizable .personalize-header .personalize-chart-targeted .label,\n.is-personalizable .personalize-header .info-message .icon,\n.is-personalizable .personalize-header .info-message p,\n.is-personalizable .personalize-header .btn-icon .icon {\n  color: ").concat(t.contrast,' !important;\n}\n\nhtml[class*="theme-new-"] .is-personalizable .personalize-actionable:hover:not([disabled]):not(.personalize-actionable-disabled):not(a) ,\nhtml[class*="theme-new-"] .is-personalizable .personalize-actionable:hover:not([disabled]):not(.personalize-actionable-disabled) svg,\nhtml[class*="theme-new-"] .is-personalizable .personalize-actionable:hover:not([disabled]):not(.personalize-actionable-disabled) .icon  {\n  color: ').concat(t.btnHoverColor,' !important;\n  opacity: 1;\n}\n\nhtml[class*="theme-new-"] .is-personalizable .personalize-actionable:hover:not([disabled]):not(.personalize-actionable-disabled):not(a) {\n  color: ').concat(t.btnHoverColor," !important;\n  background-color: ").concat(t.btnBgHoverColor,';\n  opacity: 1;\n}\n\n.is-personalizable .btn-icon.personalize-actionable {\n  height: 34px;\n  width: 34px;\n}\n\nhtml[class*="theme-new-"] .is-personalizable .btn-icon.personalize-actionable:hover:not([disabled]) {\n  background-color: ').concat(t.btnBgHoverColor," !important;\n}\n\n.is-personalizable .personalize-actionable.is-focused:not(.hide-focus),\n.is-personalizable .personalize-actionable:focus:not(.hide-focus) {\n  border-color: ").concat(t.btnFocusBorderColor," !important;\n  box-shadow: 0 0 4px 3px rgba(255, 255, 255, 0.2);\n}\n\n.is-personalizable .personalize-actionable.hyperlink:focus:not(.hide-focus)::after {\n  border-color: ").concat(t.contrast," !important;\n  opacity: 1;\n  box-shadow: 0 0 4px 3px rgba(255, 255, 255, 0.2);\n}\n\n.is-personalizable .personalize-vertical-border {\n  border-color: ").concat(t.light,";\n}\n\n.is-personalizable .personalize-horizontal-top-border {\n  border-top: 1px solid: ").concat(t.darkest,";\n}\n\n.is-personalizable .personalize-chart-targeted .total.bar {\n  background-color: ").concat(t.btnDisabledColor,";\n}\n\n.is-personalizable .personalize-chart-targeted .chart-percent-text,\n.is-personalizable .personalize-chart-targeted .label {\n  color: ").concat(t.contrast,";\n}\n\n.is-personalizable .info-message,\n.is-personalizable .info-message .icon,\n.is-personalizable .info-message p {\n  color: ").concat(t.contrast," !important;\n}\n\n.is-personalizable .personalize-actionable-disabled,\n.is-personalizable .personalize-actionable-disabled:hover {\n  opacity: .4 !important;\n  cursor: default;\n}\n\n.subheader.is-personalizable .toolbar [class^='btn']:focus:not(.hide-focus),\n.subheader.is-personalizable .flex-toolbar [class^='btn']:focus:not(.hide-focus) {\n  box-shadow: ").concat(t.focusBoxShadow,";\n}\n\n.header.is-personalizable .toolbar [class^='btn']:focus:not(.hide-focus),\n.header.is-personalizable .flex-toolbar [class^='btn']:focus:not(.hide-focus) {\n  box-shadow: ").concat(t.focusBoxShadow,'\n}\n\nhtml.theme-new-dark .header.alabaster.is-personalizable .flex-toolbar [class^=\'btn\']:focus:not(.hide-focus) .icon {\n  color: #fff !important;\n}\n\nhtml.theme-new-dark .header.alabaster.is-personalizable .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled),\nhtml.theme-new-dark .header.alabaster.is-personalizable .btn-tertiary:not(.destructive):not(.is-select):not(.is-select-month-pane):not(.is-cancel):not(.is-cancel-month-pane):not(:disabled) svg.icon {\n  color: #fff !important;\n}\n\nhtml[class*="theme-new-"]:not(.theme-new-dark) .header.alabaster.is-personalizable button:not(:disabled) {\n  color: #000 !important;\n}\n\nhtml[class*="theme-new-"]:not(.theme-new-dark) .header.alabaster.is-personalizable .flex-toolbar [class^=\'btn\'][disabled] span,\nhtml[class*="theme-new-"]:not(.theme-new-dark) .header.alabaster.is-personalizable .flex-toolbar [class^=\'btn\'][disabled] .icon {\n  color: #C5C5C9 !important;\n}\n\nhtml[class*="theme-classic-"]:not(.theme-classic-dark) .header.alabaster.is-personalizable .flex-toolbar [class^=\'btn\'][disabled] span,\nhtml[class*="theme-classic-"]:not(.theme-classic-dark) .header.alabaster.is-personalizable .flex-toolbar [class^=\'btn\'][disabled] .icon {\n  color: #888B94 !important;\n}\n\nhtml[class*="theme-classic-"]:not(.theme-classic-dark) .header.alabaster.is-personalizable button:not(:disabled) {\n  color: #000 !important;\n}\n\nhtml[class*="theme-new-"]:not(.theme-new-dark) .alabaster.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .btn-actions .icon,\nhtml[class*="theme-new-"]:not(.theme-new-dark) .alabaster.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .hyperlink,\nhtml[class*="theme-new-"]:not(.theme-new-dark) .alabaster.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .breadcrumb-overflow-container::after,\nhtml[class*="theme-new-"]:not(.theme-new-dark) .alabaster.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb .breadcrumb-list li::after,\nhtml[class*="theme-new-"]:not(.theme-new-dark) .alabaster.header.is-personalizable + .page-container .scrollable-flex-header.personalize-subheader .breadcrumb ol li::after {\n  color: #000 !important;\n}\n')}(t)},setFont:function(t){$("html").addClass("font-".concat(t))},setColors:function(t){var e;if(""===t||"default"===t||"default"===(null===(e=t)||void 0===e||null==e?void 0:e.header))return this.setColorsToDefault(),this;if(!t)return this;var i=ft.personalizationColors();return void 0!==i[t]&&(t=i[t].value),this.appendStyleSheet(this.getColorStyleSheet(t)),this.settings.colors=t,this.element.triggerHandler("colorschanged",{colors:this.settings.colors.header||this.settings.colors||ft.themeColors().brand.primary.alt.value,isDefault:!1,theme:this.currentTheme||"theme-new-light"}),this},setColorsToDefault:function(){this.settings.colors="";var t=document.getElementById("soho-personalization");t&&t.parentNode.removeChild(t),this.element.triggerHandler("colorschanged",{colors:"default",isDefault:!0,theme:this.currentTheme||"theme-new-light"})},getThemeFromStylesheet:function(){var t=$("#stylesheet, #sohoxi-stylesheet"),e="";if(t.length>0){var i=t.attr("href"),n=(e=i.substring(i.lastIndexOf("/")+1)).lastIndexOf("?");n>-1&&(e=e.slice(0,n)),e=e.replace(".min","").replace(".css","")}return e},setTheme:function(t){var e=$("html");if(t)if(t=t.replace("soho","classic").replace("uplift","new"),this.currentTheme=t,ft.currentTheme.id!==t){["light","dark","high-contrast"].indexOf(t)>-1&&(t+="-theme"),e.removeClass((function(t,e){return e.split(" ").filter((function(t){return t.indexOf("theme")>-1})).join()})).addClass(t),this.blockUi();var i=this,n=$("#stylesheet, #sohoxi-stylesheet"),s=$('<link rel="stylesheet">'),a=n.attr("href");s.on("load",(function(){n.remove(),i.unBlockUi(),i.triggerEvent(t)})).on("error",(function(){i.unBlockUi()}));var o=a?a.substring(0,a.lastIndexOf("/")):"",r=!!a&&a.indexOf(".min")>-1;s.attr({id:n.attr("id"),href:g.stripTags("".concat(o,"/").concat(t).concat(r?".min":"",".css"))}),n.removeAttr("id"),n.before(s),this.settings.theme=t,ft.setTheme(t),this.setColors(this.settings.colors)}else e.hasClass(t)||e.addClass(t)},blockUi:function(){if(this.settings.blockUI){var t="#bdbdbd";switch(ft){case"light":default:t="#f0f0f0";break;case"dark":t="#313236";break;case"high-contrast":t="#d8d8d8"}this.pageOverlay=this.pageOverlay||$('<div class="personalize-overlay"></div>'),this.pageOverlay.css("background",t),$("body").append(this.pageOverlay)}},triggerEvent:function(t){this.element.triggerHandler("themechanged",{colors:this.settings.colors.header||this.settings.colors||ft.themeColors().brand.primary.alt.value,theme:t||"theme-new-light"}),$("body").trigger("resize")},unBlockUi:function(){var t=this;t.settings.blockUI&&t.pageOverlay&&t.pageOverlay.fadeOut(300,(function(){t.pageOverlay.remove(),t.pageOverlay=void 0}))},updated:function(t){if(!t)return this;var e=E.extend({},this.settings);return this.settings=E.mergeSettings(this.element[0],t,this.settings),this.settingsDidChange(e,"theme")&&this.setTheme(this.settings.theme),this.settingsDidChange(e,"colors")&&this.setColors(this.settings.colors),this.settingsDidChange(e,"font")&&this.setFont(this.settings.font),this},settingsDidChange:function(t,e){return this.settings[e]&&this.settings[e]!==t[e]},teardown:function(){return this.element.off("updated.".concat(mt)),this},destroy:function(){this.teardown(),$.removeData(this.element[0],mt)}};var yt={theme:"theme-new-light"};"object"===l(window.SohoConfig)&&"object"===l(window.SohoConfig.personalize)&&(yt=window.SohoConfig.personalize);var wt=new bt(document.documentElement,yt);function xt(t,e){var i,n,s,a,o=0;if(3===t.nodeType)i=t.data.toUpperCase().indexOf(e),(i-=t.data.substr(0,i).toUpperCase().length-t.data.substr(0,i).length)>=0&&((n=document.createElement("mark")).className="highlight",(s=t.splitText(i)).splitText(e.length),a=s.cloneNode(!0),n.appendChild(a),s.parentNode.replaceChild(n,s),o=1);else if(1===t.nodeType&&t.childNodes&&!/(script|style)/i.test(t.tagName))for(var r=0;r<t.childNodes.length;++r)r+=xt(t.childNodes[r],e);return o}$.fn.personalize=function(t){return this.each((function(){var e=$.data(this,mt);e||(e=$.data(this,mt,wt)),e.updated(t)}))},$.fn.disable=function(){return $.each(this.data(),(function(t,e){e instanceof jQuery||e&&e.disable&&e.disable()})),this.prop("disabled",!0),this.attr("disabled",!0).closest(".field").addClass("is-disabled"),this},$.fn.enable=function(){return $.each(this.data(),(function(t,e){e instanceof jQuery||e&&e.enable&&e.enable()})),this.prop({disabled:!1,readonly:!1}),this.removeAttr("disabled readonly").closest(".field").removeClass("is-disabled"),this.closest(".field").find("button.trigger").removeAttr("disabled"),this},$.fn.readonly=function(){return $.each(this.data(),(function(t,e){e instanceof jQuery||e&&e.readonly&&e.readonly()})),this.prop("readonly",!0),this},$.fn.revealText=function(t){if(!this)return this;var e=this;e.addClass("show"===(null==t||null==t?void 0:t.initialState)?"input-show-text":"input-hide-text");var i=$('<span class="input-hideshow-text">'.concat(Soho.Locale.translate("show"===(null==t||null==t?void 0:t.initialState)?"Hide":"Show"),"</span>"));e.after(i),e.hasClass("input-hide-text")&&e.attr("type","password");var n=function(){if(e.hasClass("input-hide-text"))return e.removeClass("input-hide-text").addClass("input-show-text"),e.attr("type","text"),void i.text(Soho.Locale.translate("Hide"));e.removeClass("input-show-text").addClass("input-hide-text"),e.attr("type","password"),i.text(Soho.Locale.translate("Show"))};return i.on("click",(function(){n()})),e.on("keypress",(function(t){18===t.keyCode&&t.ctrlKey&&n()})),this},$((function(){var t,e;$(".label-text").closest("label, .label").each((function(){e=$("input, textarea, select",this),t=e.attr("class"),$(this).addClass((function(){return"inline".concat(t?" inline-".concat(-1===t.indexOf(" ")?t:t.substr(0,t.indexOf(" "))):"")}))}))})),$((function(){$("input:radio").on("click.radios",(function(){this.focus()}))})),$((function(){var t=function(t,e){for(var i=0,n=t.length;i<n;i++)$(t[i]).parent().addClass(e)};t($(".field > input:checkbox, .field > .inline-checkbox"),"field-checkbox"),t($(".field > input:radio, .field > .inline-radio"),"field-radio"),t($(".field > input:checkbox ~ label.checkbox-label.hitbox"),"field-hitbox")})),$((function(){$(".field-checkbox input:checkbox ~ label.checkbox-label.hitbox").each((function(t,e){$(e).innerWidth()>44&&$(e).addClass("hitbox-padding")}))})),$((function(){[].slice.call(document.body.querySelectorAll("label.required .label-text, .label.required .label-text, label:not(.inline).required, .label:not(.inline).required")).forEach((function(t){-1!==t.className.indexOf("accessible")&&(t.querySelector(".required-asterisk")||t.insertAdjacentHTML("beforeend",'<span class="required-asterisk" aria-hidden="true">*</span>'))}));var t=[].slice.call(document.body.querySelectorAll("[data-validate]"));t.forEach((function(t){var e=t.getAttribute("data-validate");e&&e.indexOf("required")>-1&&t.setAttribute("aria-required",!0)})),(t=[].slice.call(document.body.querySelectorAll('input[type="hidden"]'))).forEach((function(t){var e=t.getAttribute("id");if(e){var i=document.body.querySelector('label[for="'.concat(e,'"]'));i&&i.setAttribute("aria-hidden",!0)}}))})),$.fn.highlight=function(t){return this.length&&t&&t.length?this.each((function(){xt(this,t.toUpperCase())})):this},$.fn.unhighlight=function(){return this.find("mark.highlight").each((function(){var t=this.parentNode;t.replaceChild(this.firstChild,this),t.normalize()})).end()};var Ct="arrange",kt={handle:null,itemsSelector:null,connectWith:!1,isVisualItems:!1,placeholder:null,placeholderCssClass:"arrange-placeholder",useItemDimensions:!1};function Tt(t,e){this.settings=E.mergeSettings(t,e,kt),this.element=$(t),this.init()}Tt.prototype={init:function(){this.isTouch=x.features.touch,this.isIe="ie"===x.browser.name,this.handleEvents()},getElementByTouchInList:function(t,e,i){for(var n=!1,s=$(t),a=0,o=s.length;a<o;a++){var r=$(s[a]),l=r.offset();e<=l.left||e>=l.left+r.outerWidth()||i<=l.top||i>=l.top+r.outerHeight()||(n=r)}return n},dragTouchElement:function(t,e){var i=t.originalEvent.changedTouches;e&&e[0]&&i&&i[0]&&this.offset&&(e[0].style.top="".concat(i[0].pageY-this.offset.y,"px"),e[0].style.left="".concat(i[0].pageX-this.offset.x,"px"))},unbind:function(){return this.items.removeClass("draggable").removeAttr("draggable").off("selectstart.arrange ".concat(this.dragStart," ").concat(this.dragEnd," ").concat(this.dragWhileDragging)),$(this.handle,this.items).removeClass("draggable").off("mousedown.arrange mouseup.arrange touchstart.arrange touchend.arrange"),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,kt)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Ct)},getVisualIndex:function(t){var e=this.settings,i=null;if(e.isVisualItems){var n=this.element.children().not('[data-arrange-exclude="true"]');e.itemsSelector&&(n=$(e.itemsSelector,this.element).not('[data-arrange-exclude="true"]')),i=n.index(t)}return i},handleEvents:function(){var t,e,i=this,n=this.settings,s={},a=this.element.children().not('[data-arrange-exclude="true"]'),o=$("<".concat(/^(ul|ol)$/i.test(this.element[0].tagName)?"li":"div",">"));n.itemsSelector&&(a=$(n.itemsSelector,this.element).not('[data-arrange-exclude="true"]'),o=$("<".concat(a.first()[0].tagName," />"))),n.placeholder&&(o=$(n.placeholder)),this.dragStart="dragstart.arrange touchstart.arrange gesturestart.arrange",this.dragEnd="dragend.arrange touchend.arrange touchcancel.arrange gestureend.arrange",this.dragWhileDragging="dragover.arrange dragenter.arrange drop.arrange touchmove.arrange gesturechange.arrange",this.handle=n.handle||this.element.attr("data-arrange-handle"),this.connectWith=this.element.attr("data-arrange-connectWith"),this.placeholders=o,this.isTouch||this.placeholders.addClass("".concat(n.placeholderCssClass," draggable")),$(this.handle,a).addClass("draggable").on("mousedown.arrange touchstart.arrange",(function(){e=!0})).on("mouseup.arrange touchend.arrange",(function(){e=!1})),this.connectWith&&(a=a.add($(this.connectWith).children().not('[data-arrange-exclude="true"]')).data("connectWith",this.connectWith)),this.items=a,this.items.attr("draggable",!0).addClass(this.handle?"":"draggable").add([this,o]).not("a[href], img").on("selectstart.arrange",(function(){return this.dragDrop&&this.dragDrop(),this.isIe})),this.items.add(o).each((function(){$(this).on(i.dragStart,(function(a){if(i.handle&&!e){if(i.isTouch)return;return!1}e=!1,i.dragging=$(this),t=i.dragging.addClass("arrange-dragging").index();var r=n.isVisualItems?i.getVisualIndex(i.dragging):t;$.extend(s,{start:i.dragging,startIndex:r});var l=i.element.triggerHandler("beforearrange",s);if("boolean"==typeof l&&!l||"string"==typeof l&&"false"===l.toLowerCase())i.dragging=null;else{var c,d=i.dragging[0].getBoundingClientRect();if(n.useItemDimensions)o[0].style.width="".concat(d.width,"px"),o[0].style.height="".concat(d.height,"px"),(c=o[0].classList).add.apply(c,u(n.placeholderCssClass.split(" ")));if(i.isTouch){var h=a.originalEvent.changedTouches;h&&h[0]&&(i.offset={x:h[0].pageX-d.left,y:h[0].pageY-d.top})}else{var p=a.originalEvent.dataTransfer,g={x:a.originalEvent.clientX-d.left,y:a.originalEvent.clientY-d.top};p.effectAllowed="move",p.setData("Text","sample"),p.setDragImage(i.dragging[0],g.x,g.y)}}})).on(i.dragEnd,(function(){if(i.dragging){if(i.isTouch){var e,a={opacity:1};n.useItemDimensions&&(a.position=""),i.dragging.css(a),null===(e=i.placeholderTouch)||void 0===e||null==e||e.remove()}if(i.element.removeClass("has-arrange-placeholder"),i.placeholders.filter(":visible").after(i.dragging),i.dragging.removeClass("arrange-dragging").show(),i.placeholders.detach(),t!==i.dragging.index()){var o=n.isVisualItems?i.getVisualIndex(i.dragging):i.dragging.index();$.extend(s,{end:i.dragging,endIndex:o}),i.element.triggerHandler("arrangeupdate",s)}i.dragging=null,i.placeholderTouch=null}})).on(i.dragWhileDragging,(function(t){if(i.dragging){var e,r=this;if(t.preventDefault(),i.isTouch&&!i.placeholderTouch&&(i.placeholderTouch=i.dragging.clone().addClass("is-touch").attr("id","arrange-placeholder-touch").insertBefore(i.dragging),i.dragTouchElement(t,i.placeholderTouch)),"drop"===t.type)return t.stopPropagation(),i.dragging.trigger("dragend.arrange"),!1;if(i.isTouch){var l=t.originalEvent.touches[0];r=i.getElementByTouchInList(a,l.pageX,l.pageY)||r}if(r=$(r),i.isTouch||(t.originalEvent.dataTransfer.dropEffect="move"),a.is(r)&&o.index()!==r.index()){if(i.isTouch){var c={opacity:0};n.useItemDimensions&&(c.position="fixed"),i.dragging.css(c)}else i.dragging.hide();var d;o.index()<r.index()?(o.insertAfter(r),e=r.index(),d=n.isVisualItems?i.getVisualIndex(r):e):(o.insertBefore(r),e=o.index(),d=n.isVisualItems?i.getVisualIndex(o):e),i.element.addClass("has-arrange-placeholder"),$.extend(s,{over:r,overIndex:d}),i.element.triggerHandler("draggingarrange",s),E.fixSVGIcons(r),i.placeholders.not(o).detach()}else i.placeholders.is(this)||(i.placeholders.detach(),i.element.append(o));if(!i.isTouch)return!1;i.dragTouchElement(t,i.placeholderTouch)}}))}))}},$.fn.arrange=function(t){return this.each((function(){var e=$.data(this,Ct);e?e.updated(t):e=$.data(this,Ct,new Tt(this,t))}))};var At="drag",St={axis:null,clone:!1,cloneCssClass:"is-clone",clonePosIsFixed:!1,cloneAppendTo:null,containment:!1,obstacle:!1,underElements:!1,containmentOffset:{left:0,top:0}};function $t(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,St),this.init()}$t.prototype={init:function(){this.handleEvents()},finish:function(t,e){var i={top:e,left:t};this.element.off("mouseup.draggable"),$(document).off("mousemove.draggable mouseup.draggable"),this.settings.underElements&&(i.underElements=this.getElementsFromPoint(i.left,i.top)),i.offset=this.offset,i.clone=this.clone,this.element.trigger("dragend",i),this.element.removeClass("is-dragging"),this.clone&&("x"===this.settings.axis&&delete i.top,"y"===this.settings.axis&&delete i.left,this.clone.remove(),this.clone=null),this.obstacle&&(this.obstacle=null),this.upperYLimit&&(this.upperYLimit=null),this.upperXLimit&&(this.upperXLimit=null),$("body").removeClass("disable-select")},move:function(t,e){var i={left:t,top:e};if("x"===this.settings.axis&&delete i.top,"y"===this.settings.axis&&delete i.left,this.settings.containment&&(this.settings.containment instanceof jQuery?this.container=this.settings.containment:"parent"===this.settings.containment?this.container=this.element.parent():"window"===this.settings.containment?this.container=$(window):"container"===this.settings.containment?this.container=this.element.closest(".page-container"):this.container=$(document),this.upperXLimit||(this.upperXLimit=this.container.width()-this.element.outerWidth()+this.settings.containmentOffset.left),this.upperYLimit||(this.upperYLimit=this.container.height()-this.element.outerHeight()+this.settings.containmentOffset.top),i.top>this.upperYLimit&&(i.top=this.upperYLimit),i.left>this.upperXLimit&&(i.left=this.upperXLimit),i.top<0&&(i.top=0),i.left<0&&(i.left=0),"container"===this.settings.containment&&i.left<=1&&(i.left=1)),this.settings.obstacle){var n=this.clone?this.clone.offset():this.element.offset(),s=this.clone?this.clone.outerWidth():this.element.outerWidth(),a=i.left>n.left;if(!this.obstacle){this.obstacle=$(this.settings.obstacle).not(this.element);var o=$(this.obstacle).offset();this.constraints={top:o.top,left:o.left,bottom:o.top+this.obstacle.outerHeight(),right:o.left+this.obstacle.outerWidth()}}!a&&this.originalPos.left>this.constraints.left&&i.left<=this.constraints.right&&(i.left=this.constraints.right),a&&this.originalPos.left+s<=this.constraints.left&&i.left+s>=this.constraints.left&&(i.left=this.constraints.left-this.obstacle.outerWidth())}var r=function(t,e,i){void 0!==e[i]&&(t[0].style[i]="".concat(e[i],"px"))};r(this.clone||this.element,i,"top"),r(this.clone||this.element,i,"left"),this.settings.underElements&&(i.underElements=this.getElementsFromPoint(i.left,i.top)),i.offset=this.offset,i.clone=this.clone,this.element.trigger("drag",i)},getElementsFromPoint:function(t,e){var i=[];if(document.elementsFromPoint)i=document.elementsFromPoint(t,e);else if(document.msElementsFromPoint)i=document.msElementsFromPoint(t,e);else{for(var n,s,a,o,r=999,l=[];(o=document.elementFromPoint(t,e))&&-1===i.indexOf(o)&&null!==o&&r>-1;)r--,i.push(o),l.push({value:o.style.getPropertyValue("pointer-events")||"",priority:o.style.getPropertyPriority("pointer-events")}),o.style.setProperty("pointer-events","none","important");for(n=0,s=i.length;n<s;n++)a=l[n],i[n].style.setProperty("pointer-events",a.value,a.priority)}return i},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings))},destroy:function(){$.removeData(this.element[0],At),this.element.off("touchstart.draggable MSPointerDown.draggable pointerdown.draggable touchmove.draggable touchend.draggable touchcancel.draggable mousedown.draggable")},handleEvents:function(){var t=this;t.offset=null,t.element.attr("draggable",!x.features.touch),"onpointerdown"in window||"onmspointerdown"in window||t.element.on("touchstart.draggable gesturestart.draggable",(function(e){var i=$(this).position(),n=e.originalEvent;t.offset={x:n.changedTouches[0].pageX-i.left,y:n.changedTouches[0].pageY-i.top},t.originalPos=i,t.element.addClass("is-dragging"),i.offset=t.offset,i.clone=t.clone,t.element.trigger("dragstart",i)})).on("touchmove.draggable gesturechange.draggable",(function(e){e.preventDefault();var i=e.originalEvent;if(!(i.targetTouches.length>1)){var n=i.changedTouches[0].pageX-t.offset.x,s=i.changedTouches[0].pageY-t.offset.y;t.move(n,s)}})).on("touchend.draggable gestureend.draggable touchcancel.draggable",(function(e){e.preventDefault();var i=e.originalEvent.changedTouches[0];t.finish(i.pageX-t.offset.x,i.pageY-t.offset.y)})),t.element.on("mousedown.draggable",(function(e){e.preventDefault();var i=t.settings.clonePosIsFixed?t.element[0].getBoundingClientRect():t.element.position();t.offset={x:e.pageX-i.left,y:e.pageY-i.top},t.originalPos=i,$("body").addClass("disable-select"),$(document).on("mousemove.draggable",(function(e){e.preventDefault(),t.move(e.pageX-t.offset.x,e.pageY-t.offset.y)})),$(document).on("mouseup.draggable",(function(i){i.preventDefault(),t.finish(e.pageX-t.offset.x,i.pageY-t.offset.y)})),t.element.on("mouseup.draggable",(function(e){e.preventDefault(),t.finish(e.pageX-t.offset.x,e.pageY-t.offset.y)})),!t.clone&&t.settings.clone&&(t.clone=t.element.clone(!0),"parent"===t.settings.cloneAppendTo&&(t.settings.cloneAppendTo=t.element.parent()),t.clone.addClass(t.settings.cloneCssClass).appendTo(t.settings.cloneAppendTo||"body")),t.element.addClass("is-dragging"),t.element.trigger("dragstart",[i,t.clone])}))}},$.fn.drag=function(t){return this.each((function(){var e=$.data(this,At);e?e.updated(t):e=$.data(this,At,new $t(this,t))}))};var Et="place",Dt={bleedFromContainer:!1,callback:null,container:null,parent:null,parentXAlignment:"center",parentYAlignment:"center",placement:"bottom",strategies:["nudge"]},It=["nudge","clockwise","flip","shrink","shrink-x","shrink-y"],Lt=["top","left","right","bottom","center"],Pt=["left","center","right"],Mt=["top","center","bottom"],Bt=["x","y","container","containerOffsetX","containerOffsetY","callback","parent","parentXAlignment","parentYAlignment","useParentWidth","useParentHeight","placement","strategies"];function Ot(t){var e=this;return Bt.forEach((function(i){if(null!==t[i])return"x"===i||"y"===i?(e.setCoordinate(i,t[i]),void(e["original".concat(i)]=t[i])):void(e[i]=t[i])})),this.modified=!1,this.sanitize()}function Rt(t,e){this.settings=E.mergeSettings(t,e,Dt),this.element=$(t),this.init()}Ot.prototype={isReasonableDefault:function(t,e){return $.inArray(t,e)>-1},sanitize:function(){var t=this;this.bleedFromContainer=!0===this.bleedFromContainer,this.callback="function"==typeof this.callback?this.callback:Dt.callback,this.container=this.container instanceof $&&this.container.length?this.container:Dt.container,this.containerOffsetX=isNaN(parseInt(this.containerOffsetX,10))?0:this.containerOffsetX,this.containerOffsetY=isNaN(parseInt(this.containerOffsetY,10))?0:this.containerOffsetY,this.parent=this.parent instanceof $&&this.parent.length?this.parent:Dt.parent,this.parentXAlignment=this.isReasonableDefault(this.parentXAlignment,Pt)?this.parentXAlignment:Dt.parentXAlignment,this.parentYAlignment=this.isReasonableDefault(this.parentYAlignment,Mt)?this.parentYAlignment:Dt.parentYAlignment,this.placement=this.isReasonableDefault(this.placement,Lt)?this.placement:Dt.placement,this.useParentHeight=!0===this.useParentHeight,this.useParentWidth=!0===this.useParentWidth,$.isArray(this.strategies)&&this.strategies.length||(this.strategies=["nudge"]),this.strategies.forEach((function(e,i){t.strategies[i]=t.isReasonableDefault(e,It)?e:t.strategies[i]}))},setCoordinate:function(t,e){this.isReasonableDefault(t,["x","y"])&&(isNaN(e)&&(e=0),this[t]=Math.round(e))}},Rt.prototype={init:function(){return this.build().handleEvents()},build:function(){this.element.hasClass("placeable")||this.element.addClass("placeable"),this.originalStyles={};var t=this.element[0].style.height,e=this.element[0].style.width;return t&&(this.originalStyles.height=t),e&&(this.originalStyles.width=e),this},handleEvents:function(){var t=this;return this.element.on("place.".concat(Et),(function(e,i,n){t.place(new Ot({x:i,y:n}))})).on("updated.".concat(Et),(function(){t.updated()})),this},render:function(t){var e=/(px|%)/i;this.element.offset({left:t.x,top:t.y}),t.height&&(this.element[0].style.height=t.height+(e.test("".concat(t.height))?"":"px")),t.width&&(this.element[0].style.width=t.width+(e.test("".concat(t.width))?"":"px"))},place:function(t){var e=[this.element[0].style.left,this.element[0].style.top];return this.element.trigger("beforeplace",[e])?(t instanceof Ot||(t=new Ot(t)),null==t.x&&null==t.y?e:(this.clearOldStyles(),t.parent?this.placeWithParent(t):this.placeWithCoords(t))):e},placeWithParent:function(t){if(!t.parent||!t.parent.length)return[void 0,void 0];var e,i=this,n=this.getContainer(t),s=n.length&&n[0]===document.body;x.rtl&&n.length&&(e=f.getDimensions(n[0]),this.element.css({left:"".concat(e.left,"px"),top:"".concat(e.right,"px")}));var a=f.getDimensions(t.parent[0]),o=s?$(window).scrollLeft():n.scrollLeft(),r=s?$(window).scrollTop():n.scrollTop();function l(t){var e=function(t){var e,n,l=t.placement,c=t.parentXAlignment,d=t.parentYAlignment,h=f.getDimensions(i.element[0]);switch(l){case"top":n=a.top-h.height-t.y+(s?r:0);break;case"left":e=a.left-h.width-t.x+(s?o:0);break;case"right":e=a.right+t.x+(s?o:0);break;default:n=a.bottom+t.y+(s?r:0)}if("top"===l||"bottom"===l){var u=Math.round(s?document.body.offsetWidth:null);switch(c){case"left":e=s&&u<Math.round(h.left)+Math.round(h.width)?a.left+t.x+o:a.left-t.x+(s?o:0);break;case"right":e=s&&Math.round(h.right)-Math.round(h.width)<0?a.left-t.x+o:a.right-h.width+t.x+(s?o:0);break;default:e=a.left+(a.width-h.width)/2+t.x+(s?o:0)}}if("right"===l||"left"===l)switch(d){case"top":n=a.top-t.y+(s?r:0);break;case"bottom":n=a.bottom-h.height+t.y+(s?r:0);break;default:n=a.top+(a.height-h.height)/2+t.y+(s?r:0)}return[e,n]}(t);return t.setCoordinate("x",e[0]),t.setCoordinate("y",e[1]),i.render(t),t=i.handlePlacementCallback(t)}return t.useParentWidth&&(t.width=a.width),t.useParentHeight&&(t.height=a.height),(t=l(t)).strategies.forEach((function(e){(t=i.checkBleeds(t)).bleeds&&(t=function(){switch(e){case"nudge":return i.nudge(t);case"clockwise":return i.clockwise(t);case"flip":return(t=i.flip(t)).setCoordinate("x",t.originalx),t.setCoordinate("y",t.originaly),t=l(t);case"shrink":return i.shrink(t);case"shrink-x":return i.shrink(t,"x");case"shrink-y":return i.shrink(t,"y");default:return t}}(),i.render(t))})),t.element=this.element,this.element.trigger("afterplace",[t]),t},placeWithCoords:function(t){return this.render(t),t=this.handlePlacementCallback(t),this.render(t),(t=this.checkBleeds(t)).bleeds&&(t=this.nudge(t)),this.render(t),(t=this.checkBleeds(t)).bleeds&&(t=this.shrink(t)),this.render(t),this.element.trigger("afterplace",[t]),t},handlePlacementCallback:function(t){var e=t.callback||this.settings.callback;return t.element=this.element,e&&"function"==typeof e&&(t=e(t)),this.render(t),t},accountForScrolling:function(t){var e=t.container,i=window.getComputedStyle(this.element[0]).position;if("fixed"===i)return!1;if(e&&e.length||(e=this.element.parents().filter((function(){return"absolute"===window.getComputedStyle(this).position||"fixed"===i}))),!e||!e.length)return!0;if(e[0]===document.body)return!1;var n=window.getComputedStyle(e[0]);return"fixed"!==(i=n.position)&&("absolute"!==i||"hidden"!==n.overflow)},getContainer:function(t){if(t.container instanceof $&&t.container.length)return t.container;var e=this.element.parents(".modal");return e.length?e:$(document.body)},isParentElementinViewport:function(t){"function"==typeof jQuery&&t instanceof jQuery&&(t=t[0]);var e=t.getBoundingClientRect();return e.top>=0&&e.left>=0&&e.bottom<=(window.innerHeight||document.documentElement.clientHeight)&&e.right<=(window.innerWidth||document.documentElement.clientWidth)},checkBleeds:function(t){var e,i=!1===this.settings.bleedFromContainer?!this.isParentElementinViewport(this.element):this.settings.bleedFromContainer,n=this.getContainer(t),s=n.length&&n[0]===document.body,a=this.element[0].getBoundingClientRect(),o={},r=n?n[0].getBoundingClientRect():{},l=s?$(window).scrollLeft():n.scrollLeft(),c=s?$(window).scrollTop():n.scrollTop(),d=Math.max(document.documentElement.clientHeight,window.innerHeight||0),h=Math.max(document.documentElement.clientWidth,window.innerWidth||0);function u(t){switch(t){case"top":return(i?0:r.top)-(s?-1*c:0);case"left":return(i?0:r.left)-(s?-1*l:0);case"right":return(i?h:r.right)-(s?-1*l:0);default:return(i?d:r.bottom)-(s?-1*c:0)}}o.width=a.width,o.height=a.height,o.top=a.top,o.right=a.right,o.bottom=a.bottom,o.left=a.left;var p=u("right");if(o.width>=p){e=o.width-p+20;var g=o.width-e;t.width=g,this.element[0].style.width="".concat(g,"px"),o.width=g}var f=u("bottom");if(o.height>=f){e=o.height-f+20;var m=o.height-e;t.height=m,this.element[0].style.height="".concat(m,"px"),o.height=m}var v=o.right>u("right"),b=o.left<u("left"),y=o.top<u("top"),w=o.bottom>u("bottom");return v||b||y||w?(t.bleeds={},t.bleeds.right=v?o.right-u("right"):null,t.bleeds.left=b?-(o.left-u("left")):null,t.bleeds.top=y?-(o.top-u("top")):null,t.bleeds.bottom=w?o.bottom-u("bottom"):null,t):(t.bleeds=void 0,t)},nudge:function(t){t.nudges||(t.nudges={x:0,y:0});var e=0;return t.bleeds.right&&(e=Math.abs(t.bleeds.right)+Math.abs(t.containerOffsetX),t.setCoordinate("x",t.x-e),t.nudges.x-=e),t.bleeds.left&&(e=Math.abs(t.bleeds.left)+Math.abs(t.containerOffsetX),t.setCoordinate("x",t.x+e),t.nudges.x+=e),t.bleeds.top&&(e=Math.abs(t.bleeds.top)+Math.abs(t.containerOffsetY),t.setCoordinate("y",t.y+e),t.nudges.y+=e),t.bleeds.bottom&&(e=Math.abs(t.bleeds.bottom)+Math.abs(t.containerOffsetY),t.setCoordinate("y",t.y-e),t.nudges.y-=e),t.wasNudged=!0,t.bleeds=void 0,t},flip:function(t){if(!t.bleeds[t.placement])return t;if(t.attemptedFlips||(t.attemptedFlips=[]),t.attemptedFlips.push(t.placement),t.attemptedFlips.length>3)return t=this.giveup(t);var e=this.accountForScrolling(t),i=["left","right"].indexOf(t.placement)>-1,n=this.settings.bleedFromContainer,s=this.getContainer(t),a=s.length&&s[0]===document.body,o=s?s[0].getBoundingClientRect():{},r=t.parent[0].getBoundingClientRect(),l=a?$(window).scrollLeft():s.scrollLeft(),c=a?$(window).scrollTop():s.scrollTop(),d=Math.max(document.documentElement.clientHeight,window.innerHeight||0),h=Math.max(document.documentElement.clientWidth,window.innerWidth||0);function u(t){switch(t){case"left":return"right";case"right":return"left";case"top":return"bottom";default:return"top"}}function p(i){var s=0;switch(i){case"left":s=(n?0:o.left)-(e?l:0)-r.left+t.containerOffsetX;break;case"right":s=(n?h:o.right)-(e?l:0)-r.right-t.containerOffsetX;break;case"top":s=(n?0:o.top)-(e?c:0)-r.top+t.containerOffsetY;break;default:s=(n?d:o.bottom)-(e?c:0)-r.bottom-t.containerOffsetY}return Math.abs(s)}function g(e){return $.inArray(e,t.attemptedFlips)>-1}return t.placement=function(e){var n=u(e),s=i?"top":"left",a=u(s),o=p(e),r=p(n);if(!g(n))return o>=r?e:(t.wasFlipped=!0,n);var l=t.originalx;t.originalx=t.originaly,t.originaly=l;var c=p(s),d=p(a);if(!g(s)){if(c>=d)return s;if(!g(a))return a}return e}(t.placement),t},clockwise:function(t){return t},shrink:function(t,e){var i=0,n=0,s=null==e||"x"===e,a=null==e||"y"===e,o=this.accountForScrolling(t),r=f.getDimensions(this.element[0]),l=this.settings.bleedFromContainer,c=this.getContainer(t),d=c?c[0].getBoundingClientRect():{},h=c.length&&c[0]===document.body,u=null===t.parent,p=h?$(window).scrollLeft():c.scrollLeft(),g=h?$(window).scrollTop():c.scrollTop(),m=Math.max(document.documentElement.clientHeight,window.innerHeight||0),v=Math.max(document.documentElement.clientWidth,window.innerWidth||0),b=(o?p:0)+(l?0:d.left)+t.containerOffsetX,y=(o?g:0)+(l?0:d.top)+t.containerOffsetY,w=(o?p:0)+(l?v:d.right)-t.containerOffsetX,x=(o?g:0)+(l?m:d.bottom)-t.containerOffsetY,C=w-b,k=x-y;if(!u){var T=f.getDimensions(t.parent[0]),A=T.top-y,S=x-T.bottom,E=T.left-b,D=w-T.right;C=E>D?E:D,k=A>S?A:S}return s&&(r.width>C&&(t.width=C),r.left<b&&(i=b-r.left,t.setCoordinate("x",t.x+i))),a&&(r.height>k&&(t.height=k),r.top<y&&(n=y-r.top,t.setCoordinate("y",t.y+n))),t},giveup:function(t){return t.giveup=!0,t.strategy=this.settings.strategy,t.placement=this.settings.placement,t},clearOldStyles:function(){this.element[0].style.left="",this.element[0].style.top="",this.element[0].style.width="",this.element[0].style.height="";var t=this.originalStyles;return t&&(t.width&&(this.element[0].style.width=t.width),t.height&&(this.element[0].style.height=t.height)),this},setArrowPosition:function(t,e,i){var n=e.parent,s=i.find("div.arrow"),a=e.placement,o={},r=i[0].getBoundingClientRect(),l={},c={},d=!1;if(n&&n.length&&s.length){s[0].removeAttribute("style"),i.removeClass("top right bottom left").addClass(a),n.is(".colorpicker, .datepicker, .timepicker")&&(n=n.next(".trigger")),n.is(".btn-split-menu, .btn-menu, .btn-actions, .btn-filter, .tab, .tab-more")&&(n=n.find(".icon").last()),n.is(".searchfield-category-button")&&(n=n.find(".icon.icon-dropdown")),n.is(".colorpicker-editor-button")&&(n=n.find(".trigger .icon")),n.is(".fontpicker")&&(n=n.find(".icon.icon-dropdown")),n.length||(n=e.parent),o=n.length?n[0].getBoundingClientRect():o,l=s.length?s[0].getBoundingClientRect():l,c={};var h,u,p={};p[(u=a,"right"===u||"left"===u?"margin-top":"margin-left")]=(h=0,"left"!==a&&"right"!==a||(h=o.top+o.height/2-(l.top+l.height/2),c.top=l.top+h,c.bottom=l.bottom+h,(c.top<=r.top||c.bottom>=r.bottom)&&(d=!0)),"top"!==a&&"bottom"!==a||(h=o.left+o.width/2-(l.left+l.width/2),c.left=l.left+h,c.right=l.right+h,(c.left<=r.left||c.right>=r.right)&&(d=!0)),h),d&&(p.display="none"),s.css(p)}},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown().init()},teardown:function(){return this.clearOldStyles(),this.element.removeClass("placeable"),this.element.off("updated.".concat(Et," place.").concat(Et)),this.element.trigger("afterteardown"),this},destroy:function(){this.teardown(),$.removeData(this.element[0],Et)}},$.fn.place=function(t){return this.each((function(){var e=$.data(this,Et);e?e.updated(t):e=$.data(this,Et,new Rt(this,t))}))};var zt="icon",Ft={use:"user-profile",focusable:!1};function Nt(t,e){this.settings=E.mergeSettings(t,e,Ft),this.element=$(t),this.init()}Nt.prototype={init:function(){return this.getExistingUseTag(),this.render().handleEvents()},render:function(){if(this.element.addClass("icon"),!this.element.is("svg"))return this;var t=this.element[0].getElementsByTagName("use")[0];return t?(t.getAttribute("href")!==this.getBasedUseTag()&&t.setAttribute("href",this.getBasedUseTag()),this):this},getBasedUseTag:function(){return G.getBaseURL("#icon-".concat(this.settings.use))},getExistingUseTag:function(){if(!this.element.is("svg"))return this;var t=this.element.children("use");if(!t.length)return this;var e=t.attr("href");!e&&t.attr("xlink:href")&&(e=t.attr("xlink:href"));var i=G.url;return e&&G.element.length&&i.length&&(e=e.replace(i,"")),e=e.replace("#icon-",""),this.settings.use=e,this},handleEvents:function(){var t=this;return this.element.on("updated.".concat(zt),(function(){t.updated()})),this},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown().init()},teardown:function(){return this.element.off("updated.".concat(zt)),this},destroy:function(){this.teardown(),$.removeData(this.element[0],zt)}},$.fn.icon=function(t){return this.each((function(){var e=$.data(this,zt);e?e.updated(t):e=$.data(this,zt,new Nt(this,t))}))},function(){function t(t){var e={icon:"user-profile",classes:["icon"]};return"string"==typeof(t=t||$.extend({},e))&&(t=$.extend({},e,{icon:t.replace("icon-","")})),!t.classes&&t.class&&(t.classes=t.class,delete t.class),t.classes||(t.classes=[].concat(e.classes)),"string"==typeof t.classes&&(t.classes=t.classes.split(" ")),-1===t.classes.indexOf("icon")&&t.classes.push("icon"),t.icon||(t.icon=""),t}$.createIcon=function(e){return e=t(e),M.isValidURL(e.icon)?'<img\n        src="'.concat(e.icon,'"\n        alt="Icon"\n        class="icon custom-icon ').concat(e.classes.join(" "),'"\n        aria-hidden="true"\n        role="presentation"/>'):['<svg class="'.concat(e.classes.join(" "),'" focusable="false" aria-hidden="true" role="presentation">')+'<use href="#icon-'.concat(e.icon,'"></use>')+"</svg>"].join("")},$.createIconElement=function(t){return $($.createIcon(t))},$.createIconPath=function(e){return e=t(e),$.getBaseURL("#icon-".concat(e.icon.replace("icon-","")))},$.fn.changeIcon=function(t){t&&$(this).find("use").attr("href",$.createIconPath({icon:t}))},$.fn.getIconName=function(){var t=$(this),e=t.find("use");return 1===e.length&&e.attr("href")?e.attr("href").substr(e.attr("href").indexOf("#icon-")+6):1===e.length&&e.attr("xlink:href")?e.attr("xlink:href").substr(e.attr("xlink:href").indexOf("#icon-")+6):t.attr("data-icon")}}();var Ht="tooltip",qt={content:null,offset:{top:10,left:10},placement:"top",trigger:"hover",showOnKeyboardFocus:!0,title:null,beforeShow:null,popover:null,closebutton:null,isError:!1,isErrorColor:!1,tooltipElement:null,parentElement:null,keepOpen:!1,isRangeDatepicker:!1,extraClass:null,placementOpts:{},maxWidth:null,initializeContent:!0,headerClass:null,delay:500,onHidden:null,attachToBody:!0,appendTo:'[role="main"]',attributes:null};function Vt(t,e){this.settings=E.mergeSettings(t,e,qt),this.element=$(t),this.componentName,this.init(),this.componentName}Vt.prototype={get componentName(){return this._componentName||(this._componentName=this.isPopover?"popover":Ht),this._componentName},get isPopover(){return null!==this.settings.content&&"object"===l(this.settings.content)||!0===this.settings.popover},get visible(){return this.tooltip.length&&!1===this.tooltip[0].classList.contains("hidden")&&!1===this.tooltip[0].classList.contains("is-hidden")},get canBeShown(){return!this.reopenDelay&&!f.hasClass(this.element[0],"hidden")&&!f.hasClass(this.element[0],"is-hidden")&&this.element.parents(".hidden, .is-hidden").length<1},get popupmenuAPI(){return this.element.data("popupmenu")},get isFocused(){var t=document.activeElement;return!!this.activeElement.is($(t))||!!(this.tooltip&&this.tooltip.length&&this.tooltip[0].contains(t))},init:function(){this.uniqueId=E.uniqueId(this.element,this.componentName),this.isTouch=x.features.touch,this.setup(),this.appendTooltip(),"immediate"!==this.settings.trigger&&this.setContent(this.settings.content,!0),this.handleEvents()},setup:function(){this.activeElement=this.settings.parentElement instanceof $&&this.settings.parentElement.length?this.settings.parentElement:this.element,this.descriptionId=$(".tooltip-description").length+1,this.description=this.element.parent().find(".tooltip-description"),!this.description.length&&this.settings.isError&&(this.description=$('<span id="tooltip-description-'.concat(this.descriptionId,'" class="tooltip-description audible"></span>')).insertAfter(this.element)),this.settings.appendTo||(this.settings.appendTo='[role="main"]'),this.element.is(".dropdown, .multiselect")&&(this.activeElement=this.element.nextAll(".dropdown-wrapper:first").find(">.dropdown"));var t=this.element.attr("title");(!this.settings.popover&&t&&t.length||!this.settings.popover&&this.settings.title)&&(this.settings.content=this.settings.title?this.settings.title:t,this.element.removeAttr("title")),"hover"===this.settings.trigger&&this.isTouch&&this.element.addClass("longpress-target"),this.settings.closebutton=!(!this.settings.closebutton&&!this.element.data("closebutton")),this.element.data("extraClass")&&this.element.data("extraClass").length&&(this.settings.extraClass=this.element.data("extraClass")),Array.isArray(this.settings.attributes)&&E.addAttributes(this.element,this,this.settings.attributes,"trigger"),this.isRTL=H.isRTL(),f.addClass(this.element[0],"has-tooltip")},addAria:function(){if(this.content&&(this.description.text(this.content),this.content=this.addClassToLinks(this.content,"links-clickable"),this.isPopover||this.element.attr("aria-describedby",this.description.attr("id")),this.isPopover)){var t=this.element.prev("input.timepicker").length;if(t||"click"!==this.settings.trigger||this.element.attr("aria-haspopup",!0),t){var e=this.tooltip.attr("id"),i=this.element.prev("input.timepicker");i.attr("role","combobox"),i.attr("aria-haspopup","dialog"),i.attr("aria-owns",e),this.element.attr("role","button"),this.element.attr("aria-controls",e),this.tooltip.attr("role","dialog"),this.tooltip.attr("aria-label",H.translate("TimepickerPopup"))}}},addClassToLinks:function(t,e){if(t instanceof $&&t.length>0)return t;var i=$("<div/>").html(t);return $("a",i).addClass(e),i.html()},appendTooltip:function(){var t=this.settings.tooltipElement,e=this.isPopover?"popover":"tooltip";if(this.tooltip=t?$(t):$("#".concat(this.componentName)),!this.tooltip.length){var i=t?t.substring(1,t.length):this.componentName;this.tooltip=$('<div class="'.concat(e,' bottom is-hidden" id="').concat(i,'">\n        <div class="arrow"></div><div class="tooltip-content"></div></div>')),g.removeWhiteSpaceCharacters(this.tooltip)}this.tooltip.place({container:this.scrollparent,parent:this.activeElement,placement:this.settings.placement,strategy:"flip"}),$.data(this.tooltip[0],this.componentName,this),this.setTargetContainer(),this.addAria()},handleEvents:function(){var t=this,e=t.settings.delay/10;function i(){t.timer&&t.timer.destroy&&t.timer.destroy(!0)}function n(){i(),t.timer=new X({duration:e,timeoutCallback:function(){t.show()}}),J.register(t.timer)}function s(){i(),t.show()}function a(){i(),t.hide()}function o(){t.visible?a():s()}"hover"!==this.settings.trigger||this.settings.isError||((this.element.is(".dropdown, .multiselect, span.longpress-target")?this.activeElement:this.element).on("mouseenter.".concat(Ht),(function(){t.isTouch||n()})).on("mouseleave.".concat(Ht),(function(){t.visible?a():i()})).on("click.".concat(Ht),(function(){t.isTouch||a()})).on("longpress.".concat(Ht),(function(){s()})).on("updated.".concat(Ht),(function(){t.updated()})),this.settings.showOnKeyboardFocus&&(this.element.is(".dropdown, .multiselect, span.longpress-target")?this.activeElement:this.element).on("focus.".concat(Ht),(function(){t.isTouch&&!et.pressedKeys.get("Tab")||n()})).on("blur.".concat(Ht),(function(){t.settings.keepOpen||a()}))),"click"===this.settings.trigger&&this.element.on("click.".concat(Ht),(function(){o()})),"immediate"===this.settings.trigger&&setTimeout((function(){o()}),1),"focus"===this.settings.trigger&&this.element.on("focus.".concat(Ht),(function(){s()})).on("blur.".concat(Ht),(function(){t.settings.keepOpen||a()})),$(window).on("orientationchange.".concat(Ht),(function(){t.visible&&a()}),!1)},setContent:function(t,e){var i=this,n=this.settings.content,s=null==t;function a(){!0!==e&&(i.addAria(),i.render())}if(s&&null==n)return!1;if("function"==typeof n&&(t=n),s&&(t=n),t===this.content)return a(),!0;if(t instanceof $&&t.length)return this.content=t.addClass("hidden"),a(),!0;if("string"==typeof t){if(!t.length)return!1;if(0===(t=H.translate(t,{showAsUndefined:!0})||t).indexOf("#")&&t.indexOf("/")<0){var o=$("".concat(t));return o.length?(this.content=o,a(),!0):(this.content=t,a(),!0)}}else{if("function"!=typeof t)return!1;var r=t.call(this.element);if(!r||"string"!=typeof r||!r.length)return!1;t=r}return this.content=g.sanitizeHTML(t),this.content="<p>".concat(this.content,"</p>"),a(),!0},render:function(){return this.isPopover?this.renderPopover():this.renderTooltip()},renderTooltip:function(){var t=this.tooltip[0].querySelectorAll(".tooltip-title")[0],e=this.tooltip[0].querySelectorAll(".tooltip-content")[0],i=this.settings.extraClass,n=this.content,s=this.tooltip[0],a="tooltip is-hidden";i&&(a+=" ".concat(i)),s.setAttribute("class",a),Array.isArray(this.settings.attributes)&&E.addAttributes(this.tooltip,this,this.settings.attributes),t&&(t.style.display="none"),e&&e.previousElementSibling&&!e.previousElementSibling.classList.contains("arrow")&&e.insertAdjacentHTML("beforebegin",'<div class="arrow"></div>');var o="string"==typeof this.content?n:n[0].innerHTML;f.html(e,o,"<div><p><span><ul><li><a><abbr><b><i><kbd><small><strong><sub><svg><use><br>")},renderPopover:function(){var t=this,e=this.settings.extraClass,i=this.tooltip.find(".tooltip-content"),n=this.tooltip[0].querySelector(".tooltip-title"),s=this.content,a="popover is-hidden";e&&(a+=" ".concat(e)),this.tooltip[0].setAttribute("class",a),Array.isArray(this.settings.attributes)&&E.addAttributes(this.tooltip,this,this.settings.attributes);var o="ie"===x.browser.name&&x.browser.isIE11()&&s instanceof $&&s.length&&"hover"===this.settings.trigger;if("string"==typeof s)s=$(s),f.replaceHtml(i,s),i.find(".hidden").removeClass("hidden");else if(o){var r=s[0].cloneNode(!0),l=r.id;l&&(r.id="".concat(l,"-").concat(this.uniqueId)),f.replaceHtml(i,r.outerHTML)}else f.replaceHtml(i,s);var c=i.width();this.settings.placementOpts||(this.settings.placementOpts={}),this.settings.placementOpts.parent||(this.settings.placementOpts.parent=this.element),o||s[0].classList.remove("hidden"),i[0].firstElementChild.classList.remove("hidden");var d=this.settings.placementOpts.parent.width();if(H.isRTL()?this.settings.placementOpts.parentXAlignment=d>c?"left":"right":this.settings.placementOpts.parentXAlignment=d>c?"right":"left",null!==this.settings.title)if(n)n.style.display="",n.childNodes[0].nodeValue=this.settings.title;else{var h=document.createDocumentFragment();n=document.createElement("div"),f.html(n,this.settings.title,"*"),n.classList.add("tooltip-title"),this.settings.headerClass&&f.addClass(n,this.settings.headerClass,"filled"),h.appendChild(n),this.tooltip[0].insertBefore(h,this.tooltip[0].firstChild)}else n&&(n.style.display="none");if(this.settings.closebutton&&n&&!n.firstElementChild){var u=$('<button type="button" class="btn-icon l-pull-right btn-close">'.concat($.createIcon({classes:["icon-close"],icon:"close"}),"<span>Close</span>")+"</button>").on("click",(function(){t.hide(!0)}));n.appendChild(u[0]),Array.isArray(this.settings.attributes)&&E.addAttributes(u,this,this.settings.attributes,"btn-close",!0)}this.settings.initializeContent&&(this.settings.initializeContent=!1,s.initialize())},open:function(){return this.show()},show:function(t,e){var i=this;if((!this.popupmenuAPI||!this.popupmenuAPI.isOpen)&&this.canBeShown){if(this.isInPopup=!1,t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.settings.beforeShow&&!e){var n=function(t){"boolean"!=typeof t?(i.content=t,i.show({content:t},!0)):t&&i.show(null,!0)};return"string"==typeof this.settings.beforeShow?void window[this.settings.beforeShow](n,this.element):void this.settings.beforeShow(n,this.element)}if(!1!==this.setContent(this.content)&&!1!==this.element.triggerHandler("beforeshow",[this.tooltip])){this.tooltip[0].removeAttribute("style"),this.tooltip[0].classList.add(this.settings.placement),this.tooltip[0].classList.add("is-open"),f.addClass(this.element[0],"has-open-tooltip"),(this.settings.isError||this.settings.isErrorColor)&&this.tooltip[0].classList.add("is-error"),this.position(),E.fixSVGIcons(this.tooltip),this.element.trigger("show",[this.tooltip]);var s=this.isTouch?"touchend":"mouseup";if(this.settings.popover)E.addAttributes(this.tooltip.find(".tooltip-title"),this,this.settings.attributes,"title",!0),E.addAttributes(this.tooltip.find(".btn-close"),this,this.settings.attributes,"btn-close",!0),this.element.attr("aria-expanded","true");else{var a=this.element.closest(".is-personalizable").length>0;this.tooltip[0].classList[a?"add":"remove"]("is-personalizable")}setTimeout((function(){var t,e,n,a;$(document).on("".concat(s,".").concat(Ht,"-").concat(i.uniqueId),(function(t){var e=$(t.target);if(!i.settings.isError&&"focus"!==i.settings.trigger&&!(e.is(i.element)&&e.is("svg.icon")||$("#editor-popup").length&&$("#colorpicker-menu").length||1===$("#editor-popup").length&&1===e.closest(".popupmenu").length)){1===e.closest(".popover").length&&e.closest(".popover").not(".monthview-popup").length&&i.element.prev().is(".datepicker")&&i.hide();var n=!1;0===e.closest(".popover").length&&0===e.closest(".dropdown-list").length&&(e.is("button")&&(e.siblings().hasClass("datepicker")||e.siblings().hasClass("timepicker"))&&i.element.get(0)===e.get(0)||i.element.get(0)===e.get(0)&&e.is("button")&&!e.is("button.trigger")||(i.hide(),n=!0)),0===e.closest(".popover").length&&e.is(".field")&&!n&&i.hide()}})).on("keydown.".concat(Ht,"-").concat(i.uniqueId),(function(t){(27===t.which||i.settings.isError)&&i.hide()})),i.settings.isError&&!i.element.is(":visible, .dropdown")&&i.element.is("[aria-describedby]")&&i.hide(),void 0===window.orientation&&$("body").on("resize.".concat(Ht),(function(){i.hide(),i.settings.keepOpen&&i.show()})),$("body").on("scroll.".concat(Ht),(function(){i.hide()})),null===(t=i.element)||void 0===t||null==t||t.closest(".modal-body-wrapper").on("scroll.tooltip",(function(){i.hide()})),null===(e=i.element)||void 0===e||null==e||e.closest(".scrollable").on("scroll.tooltip",(function(){i.hide()})),null===(n=i.element)||void 0===n||null==n||n.closest(".datagrid-wrapper").on("scroll.tooltip",(function(){i.hide()})),i.settings.isError&&i.tooltip.on("click.".concat(Ht),(function(){i.hide()})),null===(a=i.element)||void 0===a||null==a||a.trigger("aftershow",[i.tooltip])}),i.settings.delay)}}},setTargetContainer:function(){var t=$("body");this.isPopover&&(t=$(this.settings.appendTo)),t.length||(t=$("body"));var e=!1;this.scrollparent=this.element.closest(".page-container.scrollable"),this.scrollparent.length&&(t=this.scrollparent);var i=this.element.closest(".modal");this.settings.attachToBody?t=$("body"):(e=!0,t=i),this.settings.parentElement&&(t=this.settings.parentElement),e?this.tooltip.insertAfter(this.element):t[0].appendChild(this.tooltip[0])},handleAfterPlace:function(t,e){var i=this;this.tooltip.data("place").setArrowPosition(t,e,this.tooltip),setTimeout((function(){i.tooltip.triggerHandler("tooltipafterplace",[e])}))},position:function(){this.setTargetContainer(),this.settings.isRangeDatepicker||this.tooltip[0].classList.remove("is-hidden"),this.settings.maxWidth&&$(this.tooltip).css("max-width",this.settings.maxWidth);var t=this,e=this.isPopover?20:10,i=this.settings.placementOpts||{},n=Math.max(document.documentElement.clientWidth,window.innerWidth||0),s=Math.max(document.documentElement.clientHeight,window.innerHeight||0),a=this.tooltip[0].getBoundingClientRect(),o=$.extend({},{x:0,y:0,container:this.scrollparent,containerOffsetX:i.containerOffsetX||this.settings.offset.left,containerOffsetY:i.containerOffsetY||this.settings.offset.top,parent:i.parent||this.activeElement,placement:i.placement||this.settings.placement,strategies:["flip","nudge"]},i);return"left"!==o.placement&&"right"!==o.placement||(o.x=o.x||e,o.y=0),"top"!==o.placement&&"bottom"!==o.placement||(o.y=o.y||e,o.x=0),a.width>=n&&/left|right/g.test(o.placement)||a.height>=s&&/top|bottom/g.test(o.placement)?this.tooltip[0].classList.add("no-arrow"):this.tooltip[0].classList.remove("no-arrow"),this.tooltip.one("afterplace.tooltip",(function(e,i){t.handleAfterPlace(e,i)})),this.tooltip.data("place")||this.tooltip.place(o),this.tooltip.data("place").place(o),this},close:function(){return this.hide()},hide:function(t){if((!this.settings.keepOpen||t)&&this.visible)if(this.isInPopup)this.settings.content.addClass("hidden");else{if(f.removeClass(this.element[0],"has-open-tooltip"),this.tooltip[0].classList.remove("is-personalizable"),this.tooltip[0].classList.remove("is-open"),this.tooltip[0].classList.add("is-hidden"),this.tooltip[0].style.left="",this.tooltip[0].style.top="",this.tooltip[0].style.maxWidth="",this.tooltip.find(".arrow").removeAttr("style"),this.detachOpenEvents(),0===$(".popover").not(".is-hidden").length&&($(document).off("mouseup.tooltip keydown.tooltip"),$("body").off("resize.tooltip")),this.element.removeAttr("aria-owns"),this.isPopover){var e=this.element.prev("input");e.length&&e.attr("aria-expanded","false").removeAttr("aria-describedby").removeAttr("aria-haspopup").removeAttr("aria-owns")}this.element.triggerHandler("hide",[this.tooltip]),this.settings.onHidden&&this.settings.onHidden({api:this,elem:this.tooltip})}},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings));var e=this;return"immediate"===e.settings.trigger?setTimeout((function(){e.show()}),100):e.setContent(),this},detachOpenEvents:function(){var t=this;this.tooltip.off("click.".concat(Ht)),$(document).off(["keydown.".concat(Ht,"-").concat(t.uniqueId),"mouseup.".concat(Ht,"-").concat(t.uniqueId),"touchend.".concat(Ht,"-").concat(t.uniqueId)].join(" ")),$("body").off(["resize.".concat(Ht),"scroll.".concat(Ht)].join(" ")),this.element.closest(".modal-body-wrapper").off("scroll.".concat(Ht)),this.element.closest(".scrollable").off("scroll.".concat(Ht)),this.element.closest(".datagrid-body").off("scroll.".concat(Ht))},teardown:function(){return this.timer&&this.timer.destroy&&this.timer.destroy(!0),this.description.remove(),this.descriptionId=void 0,this.activeElement=void 0,this.tooltip.hasClass("is-hidden")||this.hide(),this.reopenDelay&&delete this.reopenDelay,this.tooltip&&this.tooltip.length&&(this.tooltip.data("place")&&this.tooltip.data("place").destroy(),this.tooltip.data(this.componentName)&&($.removeData(this.tooltip[0],this.componentName),delete this._componentName)),this.element.off(["mouseenter.".concat(Ht),"mouseleave.".concat(Ht),"longpress.".concat(Ht),"click.".concat(Ht),"updated.".concat(Ht),"focus.".concat(Ht),"blur.".concat(Ht)].join(" ")),f.removeClass(this.element[0],"has-tooltip"),this.detachOpenEvents(),$(window).off("orientationchange.".concat(Ht)),this},destroy:function(){this.teardown(),$.removeData(this.element[0],this.componentName),delete this._componentName}},$.fn.tooltip=function(t){return this.each((function(){var e=$.data(this,Ht);e?e.updated(t):e=$.data(this,Ht,new Vt(this,t))}))};var Wt="notificationbadge",jt={position:"upper-right",color:"alert",attribute:null};function Ut(t,e){this.settings=E.mergeSettings(t,e,jt),this.element=$(t),this.init()}Ut.prototype={init:function(){return this.build().handleEvents()},build:function(){if(this.element.hasClass("btn-tertiary")||this.element.hasClass("btn-icon")&&!this.element.hasClass("application-menu-trigger")){var t=this.element.find("svg");this.notificationBadgeContainerEl=$('<span class="notification-badge-container"></span>'),this.notificationBadgeContainerEl.insertAfter(t)}else this.notificationBadgeContainerEl=this.element.append('<span class="notification-badge-container"></span>');var e='\n      <span class="notification-dot notification-dot-'.concat(this.settings.position," notification-dot-").concat(this.settings.color,'"></span>');return this.element.find(".notification-badge-container").append(e),E.addAttributes(this.element.find(".notification-badge-container"),this,this.settings.attributes,"container"),E.addAttributes(this.element.find(".notification-badge-container .notification-dot"),this,this.settings.attributes,"dot"),this},handleEvents:function(){var t=this;return this.element.on("updated.".concat(Wt),(function(){t.updated()})),this},updated:function(){return this.teardown().init()},show:function(){this.element.find(".notification-dot").removeClass("is-disabled")},hide:function(){this.element.find(".notification-dot").addClass("is-disabled")},teardown:function(){return this.element.off("updated.".concat(Wt)),this},destroy:function(){this.teardown(),this.element&&(this.element.find(".notification-badge-container").remove(),$.removeData(this.element[0],Wt))}},$.fn.notificationbadge=function(t){return this.each((function(){var e=$.data(this,Wt);e?e.updated(t):e=$.data(this,Wt,new Ut(this,t))}))};var Gt="button",_t=["default","btn","btn-primary","btn-secondary","btn-tertiary","btn-destructive"],Yt=["default","btn-icon","btn-menu","btn-actions","btn-toggle","icon-favorite","btn-editor","input"],Kt=["icon-favorite","btn-toggle"],Xt={style:_t[0],type:Yt[0],toggleOnIcon:null,toggleOffIcon:null,hideMenuArrow:null,replaceText:!1,ripple:!0,hitbox:!1,validate:!1,notificationBadge:!1,notificationBadgeOptions:{position:"upper-right",color:"alert"},attributes:null};function Qt(t,e){this.element=$(t),this.settings=E.mergeSettings(t,e,Xt),this.init()}Qt.prototype={get tooltipAPI(){return this.element.data("tooltip")},get icon(){return this.element[0].querySelector("svg:not(.ripple-effect):not(.icon-dropdown)")},get disabled(){return this.element.parent().is(".spinbox-wrapper")?this.isDisabled(this.element.parent().find("input")):this.isDisabled(this.element)},set disabled(t){var e=!0===t;this.element[0].disabled=e},init:function(){void 0!==this.settings.buttonsetAPI&&"function"==typeof this.settings.buttonsetAPI.render&&(this.buttonsetAPI=this.settings.buttonsetAPI,delete this.settings.buttonsetAPI),this.getSettingsFromElement(),this.render()},createNotificationBadge:function(){!this.settings.notificationBadge||this.element.find(".notification-badge-container").length>0||this.element.notificationbadge({position:this.settings.notificationBadgeOptions.position,color:this.settings.notificationBadgeOptions.color})},animateWithJS:function(t){var e=200,i=t[0].style,n="".concat(parseFloat(i.left)-100,"px"),s="".concat(parseFloat(i.top)-100,"px");t[0].style.opacity="0.4",t.animate({opacity:0,left:n,top:s,width:e,height:e},1e3)},createRipple:function(t){if(!(this.disabled||t&&(void 0!==t.which&&1!==t.which||-1===["click","touchstart"].indexOf(t.type)))){$("svg.ripple-effect",this.element).remove();var e,i,n=this.element.offset();t&&(t.originalEvent instanceof MouseEvent?(e=t.pageX-n.left,i=t.pageY-n.top):(t=t.originalEvent)&&t.touches&&1===t.touches.length&&(e=t.touches[0].pageX-n.left,i=t.touches[0].pageY-n.top)),e||(e=this.element.outerWidth()/2),i||(i=this.element.outerHeight()/2);var s=$('<svg class="ripple-effect" focusable="false" aria-hidden="true" role="presentation">\n      <circle r="0" class="ripple-circle"></circle>\n    </svg>');if(s[0].style.left="".concat(e,"px"),s[0].style.top="".concat(i,"px"),this.settings.hitbox?$(this.hitboxArea).prepend(s):this.element.prepend(s),!$.fn.cssPropSupport("animation")||x.browser.isSafari&&!x.features.touch||"firefox"===x.browser.name)s.removeClass("is-animation"),this.animateWithJS(s);else $("svg.ripple-effect",this.element).addClass("is-animation");setTimeout((function(){s.remove()}),1e3)}},render:function(){var t=this;this.renderAttributes(),this.createNotificationBadge();var e=this.element[0].classList;_t.indexOf(this.settings.style)>0&&e.add(this.settings.style),Yt.indexOf(this.settings.type)>0&&e.add(this.settings.type);var i=this.buttonsetAPI,n=!1;if(i){"modal"===i.settings.style&&(n=!0);["default","btn","btn-modal"].indexOf(this.settings.style)>-1&&(e[n?"remove":"add"]("btn"),e[n?"add":"remove"]("btn-modal"));["btn-primary","btn-modal-primary"].indexOf(this.settings.style)>-1&&(e[n?"remove":"add"]("btn-primary"),e[n?"add":"remove"]("btn-modal-primary"));["btn-secondary","btn-modal-secondary"].indexOf(this.settings.style)>-1&&(e[n?"remove":"add"]("btn-secondary"),e[n?"add":"remove"]("btn-modal-secondary"))}this.settings.isDefault&&(this.settings.style="btn-primary",e.remove("btn-primary"),e.add("btn-modal-primary"),delete this.settings.isDefault),"string"==typeof this.settings.cssClass&&(this.element[0].className+=g.stripHTML(" ".concat(this.settings.cssClass))),this.settings.hitbox&&(this.element.addClass("hitbox"),this.hitboxArea=document.createElement("span"),this.hitboxArea.classList.add("hitbox-area"),this.element.prepend(this.hitboxArea)),this.settings.forceDisable&&(this.disabled=this.settings.disabled,delete this.settings.disabled,delete this.settings.forceDisable),this.settings.disabled&&(this.disabled=!0===this.settings.disabled,delete this.settings.disabled);if(this.settings.text){var s=this.element[0].querySelector("span"),a=s instanceof HTMLElement;a||(s=this.element);var o=this.settings.text||$(s).text().trim();o=g.stripHTML(o),a||(this.element[0].innerText="",s=document.createElement("span"),this.element.append($(s)),(this.settings.audible||["btn-icon","btn-actions"].indexOf(this.settings.type)>-1)&&s.classList.add("audible")),s.innerText=o}var r,l=this.icon;this.settings.icon?r=this.settings.icon:"btn-toggle"===this.settings.type?r=this.pressed?this.settings.toggleOnIcon:this.settings.toggleOffIcon:"icon-favorite"===this.settings.type?r=this.pressed?"icon-star-filled":"icon-star-outlined":"btn-actions"===this.settings.type&&(r="icon-more");var c=0!==this.element.closest(".card").length,d=0!==this.element.closest(".widget").length;if(r&&(r=g.stripHTML(r),l instanceof SVGElement||l instanceof HTMLElement?c||d?this.element.prepend($(l)):l.querySelector("use").setAttribute("href","#".concat(r)):(l=$.createIconElement({icon:r.replace("icon-","")}),this.element.prepend($(l)))),this.element.hasClass("btn-menu")&&!this.element.hasClass("btn-icon")&&!this.element.hasClass("btn-actions")){var h=this.element.children("svg.icon"),u=h.find("use"),p=!1;if(h.length>0&&u.length>=1){var f=u.last().attr("href");!f&&u.last().attr("xlink:href")&&(f=u.last().attr("xlink:href")),p=f.indexOf("#icon-dropdown")>-1}p||(h=$.createIconElement({icon:"dropdown",classes:["icon-dropdown"]}),this.element.append(h)),h.hasClass("icon-dropdown")||h.addClass("icon-dropdown"),this.settings.replaceText&&this.element.on("selected.content",(function(t,e){$(this).find("span").text(e.text())})),this.settings.hideMenuArrow&&h.remove()}if(Kt.indexOf(this.settings.type)>-1&&(this.element.on("click.favorite",(function(){t.toggle()})),this.pressed=this.pressed),this.element.hasClass("btn-actions")&&!this.element.parents(".field").length&&!this.element.parents(".toolbar").length&&!this.tooltipAPI){if(!this.element.attr("title")){var m=H.translate("More");return void this.element.attr("title",m).tooltip({content:m})}this.settings.title&&""!==this.element.attr("title")||this.tooltipAPI.destroy()}e[this.settings.ripple?"remove":"add"]("no-ripple"),this.settings.ripple&&this.element.on("touchstart.button click.button",(function(e){t.createRipple(e)})),this.element.data("hidefocus")||this.element.hideFocus()},renderAttributes:function(){Array.isArray(this.settings.attributes)&&E.addAttributes(this.element,this,this.settings.attributes)},getSettingsFromElement:function(){var t,e=this,i=arguments.length>0&&void 0!==arguments[0]&&arguments[0],n={},s=this.element.parents(".buttonset, .modal-buttonset");s.length&&(t=s.data("buttonset"))&&(this.buttonsetAPI=t);var a=this.element[0].classList||"";if(_t.forEach((function(t){"default"!==t&&a.contains(t)&&(n.style=t)})),this.buttonsetAPI&&"modal"===this.buttonsetAPI.settings.style){var o={"btn-modal":"btn","btn-modal-primary":"btn-primary","btn-modal-secondary":"btn-secondary"};Object.keys(o).forEach((function(t){a.contains(t)&&(n.style=o[t])}))}Yt.forEach((function(t){"default"!==t&&a.contains(t)&&(n.type=t)})),this.settings.disabled||(n.disabled=this.disabled);var r=this.element[0].getAttribute("aria-pressed");void 0!==r&&(n.pressed=!0===r,"icon-favorite"===n.type&&(n.toggleOnIcon="icon-star-filled",n.toggleOffIcon="icon-star-outlined"));var l=this.element[0].id;"string"==typeof l&&l.length&&(n.id=l);var c=this.element[0].title;return"string"==typeof c&&c.length&&(n.title=c),a.contains("no-ripple")&&(n.ripple=!1),this.element[0].querySelector("span.audible")&&(n.audible=!0),i||Object.keys(n).forEach((function(t){e.settings[t]=n[t]})),n},toData:function(t){var e=this,i={};return Object.keys(this.settings).forEach((function(t){i[t]=e.settings[t]})),i.pressed=this.pressed,t&&(i.element=this.element[0]),i},get pressed(){return-1!==Kt.indexOf(this.settings.type)&&"true"===this.element[0].getAttribute("aria-pressed")},set pressed(t){var e,i=!0===t,n=this.settings.toggleOffIcon&&this.settings.toggleOnIcon,s=$(this.icon);switch(this.element[0].setAttributeNS(null,"aria-pressed","x"),this.element[0].setAttributeNS(null,"aria-pressed",i),this.element[0].classList[i?"add":"remove"]("is-pressed"),this.settings.type){case"icon-favorite":e=i?"icon-star-filled":"icon-star-outlined";break;case"btn-toggle":n&&(e=i?this.settings.toggleOnIcon:this.settings.toggleOffIcon)}0!==s.length&&"string"==typeof e&&(s.changeIcon(e),E.fixSVGIcons(s))},toggle:function(){this.pressed=!this.pressed},isDisabled:function(t){return(t=t instanceof jQuery?t:$(t)).is("[disabled]")&&!t.is('[disabled="false"]')},teardown:function(){this.element.off(["click.".concat(Gt),"touchstart.".concat(Gt)].join(" "));var t=this.tooltipAPI;this.element.hasClass("btn-actions")&&t&&t.destroy();var e=this.element.data("hidefocus");e&&e.destroy();var i=this.element.data("notificationbadge");return i&&i.destroy(),this},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown(),this.render(),this},destroy:function(){this.teardown(),this.buttonsetAPI&&delete this.buttonsetAPI,$.removeData(this.element[0],Gt)}},$.fn.button=function(t){return this.each((function(){var e=$.data(this,Gt);e?e.updated(t):e=$.data(this,Gt,new Qt(this,t))}))};var Jt="buttonset",Zt={buttons:[],detectHTMLButtons:!1,style:"default"};function te(t,e){if(!(t instanceof HTMLElement))throw new Error("ButtonSet component needs an HTMLElement as a base element.");this.element=t,this.settings=E.mergeSettings(t,e,Zt),this.init()}te.prototype={set disabled(t){t=!0===t,this.element.classList[t?"add":"remove"]("is-disabled"),this.buttons&&this.buttons.forEach((function(e){e.disabled=t}))},init:function(){this.build(),this.render()},build:function(t){var e=this;this.buttons&&t||this.reset(),Array.isArray(this.settings.buttons)&&this.settings.buttons.length?this.settings.buttons.forEach((function(t){e.add(t)})):this.detectHTMLButtons()},render:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]&&arguments[0],i=this.element.classList,n={default:"buttonset",modal:"modal-buttonset"};Object.keys(n).forEach((function(e){var s=n[e];t.settings.style!==e&&i.remove(s)})),i.add(n[this.settings.style]),e||this.buttons.forEach((function(t){t.render()}))},add:function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(!t||"object"!==l(t))throw new Error("Settings object is required to add a new button to the ButtonSet");var i=t.element,n=i instanceof HTMLElement;n||(i=document.createElement("button"));var s=g.stripHTML(t.id);"string"==typeof s&&s.length>0&&(i.id=s);var a=$(i);!e&&n||$(this.element).append(a),t.buttonsetAPI=this,a.button(t);var o=a.data("button");this.buttons.push(o)},remove:function(){var t,e,i=arguments.length>0&&void 0!==arguments[0]?arguments[0]:void 0,n=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(!i)throw new Error("No Button API, HTMLElement, or ID string provided");if(void 0===i||"function"!=typeof i.destroy){if(i instanceof HTMLElement?t=i:"string"==typeof i&&(t=$(this.element).find("#".concat(i.replace("#","")))),e=$(t),void 0===(i=$(t).data("button"))||"function"!=typeof i.destroy)throw new Error("The provided Button API did not match a button existing in this ButtonSet.")}else e=i.element;i.destroy(),n&&e.remove(),this.buttons=this.buttons.filter((function(t){return t!==i}))},removeAll:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.buttons.forEach((function(i){t.remove(i,e)}))},reset:function(){this.buttons=[]},at:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:-1;return this.buttons[t]},detectHTMLButtons:function(){var t=this;this.settings.detectHTMLButtons&&(this.reset(),(E.getArrayFromList(this.element.querySelectorAll("button"))||[]).forEach((function(e){var i=$(e).data("button");i||($(e).button(),i=$(e).data("button"));var n=i.getSettingsFromElement(!0);n.element=e,t.add(n)})))},toData:function(){var t=E.extend({},this.settings);return this.buttons.forEach((function(e){t.buttons.push(e.toData())})),t},teardown:function(t){return this.removeAll(t),this.reset(),this},updated:function(t){var e=this.settings.buttons;return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),Array.isArray(t.buttons)&&e!==t.buttons?(this.settings.buttons=t.buttons,this.teardown(!0),this.init()):(this.render(!0),this.buttons.forEach((function(t){t.updated()}))),this},destroy:function(){this.teardown(!0),$.removeData(this.element,Jt)}},$.fn.buttonset=function(t){return this.each((function(){var e=$.data(this,Jt);e?e.updated(t):e=$.data(this,Jt,new te(this,t))}))};var ee="hyperlink",ie={};function ne(t,e){return this.init(t,e)}ne.prototype={init:function(t,e){if(!this.element&&t instanceof HTMLElement&&(this.element=t),"object"===l(e)){var i=this.settings||ie;this.settings=E.mergeSettings(this.element,e,i)}return this.element.classList.add("hyperlink"),this.focusBehavior||(this.focusBehavior=new C(this.element)),this},handleEvents:function(){return this},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown().init()},teardown:function(){return this}},$.fn.hyperlink=function(t){return this.each((function(){var e=$.data(this,ee);e?e.updated(t):(e=$.data(this,ee,new ne(this,t))).destroy=function(){this.teardown(),$.removeData(this.element,ee),$.removeData(this.element,"hidefocus")}}))};var se={EMPTY_STRING:"",PLACEHOLDER_CHAR:"_",CARET_TRAP:"[]",NON_DIGITS_REGEX:/[^\u0660-\u06690-9\u0966-\u096F\u2E80-\u2FD5\u3190-\u319f\u3400-\u4DBF\u4E00-\u9FCC\uF900-\uFAAD]/g,DIGITS_REGEX:/[\u0660-\u06690-9\u0966-\u096F\u2E80-\u2FD5\u3190-\u319f\u3400-\u4DBF\u4E00-\u9FCC\uF900-\uFAAD]/,ALPHAS_REGEX:/[\u00C0-\u017Fa-zA-Z]/,ANY_REGEX:/[\u00C0-\u017Fa-zA-Z0-9]/,DEFAULT_API_OPTIONS:{locale:"en-US",pattern:void 0,pipe:void 0}};se.LEGACY_DEFS={"#":se.DIGITS_REGEX,0:se.DIGITS_REGEX,x:se.ALPHAS_REGEX,"*":se.ANY_REGEX,"?":/./,"~":/[-0-9]/,a:/[APap]/,m:/[Mm]/},se.DEFAULT_CONFORM_OPTIONS={caretTrapIndexes:[],guide:!0,previousMaskResult:se.EMPTY_STRING,placeholderChar:se.PLACEHOLDER_CHAR,placeholder:se.EMPTY_STRING,selection:{start:0},keepCharacterPositions:!0};var ae={prefix:se.EMPTY_STRING,suffix:se.EMPTY_STRING,allowThousandsSeparator:!0,symbols:{currency:"$",decimal:".",negative:"-",thousands:","},allowDecimal:!0,decimalLimit:2,locale:"",requireDecimal:!1,allowNegative:!1,allowLeadingZeros:!1,integerLimit:null};function oe(t){return t.split(se.EMPTY_STRING).map((function(t){return se.DIGITS_REGEX.test(t)?se.DIGITS_REGEX:t}))}function re(t,e){var i={any:se.ANY_REGEX,digits:se.DIGITS_REGEX,alphas:se.ALPHAS_REGEX};i[e]||(e="any");for(var n=t.toString().length,s=[];n>0;)s.push(i[e]),n-=1;return s}se.numberMask=function(t,e){(e=E.mergeSettings(void 0,e,ae)).locale&&e.locale.length||(e.locale=H.currentLocale.name),e.allowLeadingZeroes&&(V("allowLeadingZeros","allowLeadingZeroes","Number Mask"),e.allowLeadingZeros=e.allowLeadingZeroes,e.allowLeadingZeroes=void 0);var i=e.prefix,n=e.suffix,s=e.symbols.decimal,a=e.symbols.thousands,o=i&&i.length||0,r=n&&n.length||0,l=a&&a.length||0;function c(t){"string"!=typeof t&&(t=se.EMPTY_STRING);var c=t.length;if(t===se.EMPTY_STRING||t[0]===i[0]&&1===c)return i.split(se.EMPTY_STRING).concat([se.DIGITS_REGEX]).concat(n.split(se.EMPTY_STRING));if(t===s&&e.allowDecimal)return i.split(se.EMPTY_STRING).concat(["0",se.CARET_TRAP,s,se.CARET_TRAP,se.DIGITS_REGEX]).concat(n.split(se.EMPTY_STRING));var d,h,u,p=t.lastIndexOf(s),g=-1!==p,f=t[0]===e.symbols.negative&&e.allowNegative;if(t.slice(-1*r)===n&&(t=t.slice(0,-1*r)),g?(d=t.slice(t.slice(0,o)===i?o:0,p),h=oe((h=t.slice(p+1,c)).replace(se.NON_DIGITS_REGEX,se.EMPTY_STRING))):d=t.slice(0,o)===i?t.slice(o):t,e.integerLimit&&"number"==typeof e.integerLimit){var m="."===a?"[.]":"".concat(a),v=(d.match(new RegExp(m,"g"))||[]).length;d=d.slice(0,e.integerLimit+(f?1:0)+v*l)}d=d.replace(se.NON_DIGITS_REGEX,se.EMPTY_STRING),e.allowLeadingZeros||(d=d.replace(/^0+(0$|[^0])/,"$1"));var b={maximumFractionDigits:e.decimalLimit,style:"decimal",useGrouping:!0};return d=e.allowThousandsSeparator?function(t,e,i,n){if(""===t||isNaN(t))return t;var s=H.toLocaleString(Number(t),i.locale,n,e);if(i&&i.allowLeadingZeros&&0===t.indexOf("0")){var a="".concat(t).match(/^0*/)[0].length;for(0===s.indexOf("0")&&(s=s.substring(1));a>0;)a-=1,s="0".concat(s)}return s}(d,a,e,b):d,u=oe(d),(g&&e.allowDecimal||!0===e.requireDecimal)&&(t[p-1]!==s&&u.push(se.CARET_TRAP),u.push(s,se.CARET_TRAP),h&&("number"==typeof e.decimalLimit&&(h=h.slice(0,e.decimalLimit)),u=u.concat(h)),!0===e.requireDecimal&&t[p-1]===s&&u.push(se.DIGITS_REGEX)),o>0&&(u=i.split(se.EMPTY_STRING).concat(u)),f&&(u.length===o&&u.push(se.DIGITS_REGEX),u=[/-/].concat(u)),n.length>0&&(u=u.concat(n.split(se.EMPTY_STRING))),{mask:u}}return c.instanceOf="createNumberMask",c(t)},se.rangeNumberMask=function(t,e){(e=e||{}).delimeter=e.delimeter||"-";var i=H.currentLocale.data.numbers.minusSign,n=function(t){return new RegExp("".concat(t),"g")},s=function(t,e){return(t.match(e)||[]).length},a=t.split(")".concat(e.delimeter,"(")),o=function(t){var a=t.substr(-1),o=s(t,/[\u0028]/g),r=(s(t,/[\u0029]/g),[]);if(1===o){var l=e.allowNegative&&t.substr(1,1)===i;(r=se.numberMask(l?t.substr(1):t,e)).splice(0,0,/[\u0028]/g),")"===a?r.push(/[\u0029]/g):a===e.delimeter&&(r.push(/[\u0029]/g),r.push(n("[".concat(e.delimeter,"]"))))}return r};"number"==typeof e.integerLimit&&(e.integerLimit*=2,e.integerLimit+=e.delimeter.length+4);var r,l=[],c=[];if("("===t.substr(0,1)&&a.length)if(2===s(t,/[\u0028]/g)&&2===s(t,/[\u0029]/g)&&1===s(t,n("\\u0029".concat(e.delimeter,"\\u0028")))&&a[1].indexOf(")")!==a[1].length-1){var d=a[1].substr(0,a[1].indexOf(")")+1);l=o("".concat(a[0],")")),c=[n("[".concat(e.delimeter,"]"))].concat(u(o("(".concat(d))))}else(l=o(2===a.length?"".concat(a[0],")"):a[0])).length&&2===a.length&&(r="".concat(a[0],")"),/^\u0028.+\u0029$/g.test(r))&&(c=[n("[".concat(e.delimeter,"]"))].concat(u(o("(".concat(a[1])))));return{mask:[].concat(u(l),u(c))}},se.adjustRangeNumber=function(t,e,i){if("rangeNumber"===i.settings.process){var n,s,a,o,r,l,c,d=i&&null!==(n=i.settings)&&void 0!==n&&null!==(s=null==n?void 0:n.patternOptions)&&void 0!==s&&(null==s?void 0:s.delimeter)?i.settings.patternOptions.delimeter:"-",h=e&&(null===(a=e.previousMaskResult)||void 0===a||null==a?void 0:a.substr(-1))===d;e&&1===(null===(o=e.selection)||void 0===o||null==o?void 0:o.start)&&1===(null===(r=e.selection)||void 0===r||null==r?void 0:r.end)&&"("!==t?(t="(".concat(t),e.selection.start++,e.selection.end++):e&&1!==(null===(l=e.selection)||void 0===l||null==l?void 0:l.start)&&1!==(null===(c=e.selection)||void 0===c||null==c?void 0:c.end)&&")"===t.substr(-1)&&1===(t.match(/\u0028/g)||[]).length&&!h&&(t="".concat(t).concat(d,"("),e.selection.start+=2,e.selection.end+=2)}return{rawValue:t,opts:e}};var le={format:"M/d/yyyy",symbols:{timeSeparator:":",dayPeriodSeparator:" ",dateSeparator:"/"}},ce={dd:31,d:31,MMM:void 0,MM:12,M:12,y:9999,yy:99,yyyy:9999,h:12,hh:12,H:24,HH:24,mm:60,ss:60,a:void 0};function de(t){return this.configure(t),this}se.dateMask=function(t,e){e=E.mergeSettings(void 0,e,le);var i=[],n=se.DIGITS_REGEX,s=e.format,a=M.removeDuplicates(s.replace(/[dMyHhmsa]+/g,"")),o=function(t){var e=t.split("").map((function(t){return" "===t&&(t="\\s"),"-"===t&&(t="\\-"),t}));return new RegExp("[(".concat(e.join("|"),")]+"))}(a),r=s.match(/(d{1,2}|M{1,4}|y{1,4}|H{1,2}|h{1,2}|m{1,2}|s{1,2}|a{1}|z{1,4}|E{1,4})/g),l=t.split(o),c=ce;return r.forEach((function(t,e){var a,o=c[t];if("a"===t||"ah"===t){var d="AM",h="PM";if(H.calendar()){d=H.calendar().dayPeriods[0],h=H.calendar().dayPeriods[1];for(var u=[],p=0;p<d.length;p++)d[p]&&h[p]&&d[p].toLowerCase()===h[p].toLowerCase()?u.push(d[p].toLowerCase()):u.push(d[p].toLowerCase()+(h[p]?h[p].toLowerCase():""));for(var g=0;g<u.length;g++)i.push(new RegExp("[".concat(u[g],"]"),"i"))}else i.push(/[aApP]/,/[Mm]/);if("ah"===t){var f=l[e].replace(d,"").replace(h,"");i=i.concat(re(f,"digits"))}}else if(o)if(l[e]){var m=l[e].toString(),v=parseInt(m.substr(0,1),10),b=parseInt(c[t].toString().substr(0,1),10);1===t.length&&v>b||"0"!==m&&1===m.length&&v<=b&&void 0!==l[e+1]&&"HH"!==t.toUpperCase()?i.push(n):i=i.concat(re(o,"digits"))}else for(a=o.toString().length;a>0;)i.push(n),a-=1;else i=i.concat(re(t,"alphas"));var y=r[e+1];if(void 0!==y){var w=t.toString().length,x=s.indexOf(t)+w,C=s.indexOf(y),k=s.substring(x,C).split(se.EMPTY_STRING),T=[se.CARET_TRAP].concat(k.concat(se.CARET_TRAP));i=i.concat(T)}})),{mask:i,literals:a.split(""),literalRegex:o}},se.rangeDateMask=function(t,e){var i=t.split(e.delimeter),n=e.delimeter.split(""),s=se.dateMask(i[0],e),a=[];return i[1]&&(a=se.dateMask(i[1],e)),{mask:s.mask.concat(n.concat(a.mask)),literals:n,literalRegex:a.literalRegex}},se.autocorrectedDatePipe=function(t,e){e.dateFormat||(e.dateFormat=H.calendar().dateFormat.short);var i=[],n=e.dateFormat.split(/[^dMy]+/),s={d:31,M:12,yy:99,yyyy:9999},a={d:1,M:1,yy:0,yyyy:1},o=t.conformedValue.split("");n.forEach((function(t){var n=e.dateFormat.indexOf(t),a=parseInt(s[t].toString().substr(0,1),10);parseInt(o[n],10)>a&&(o[n+1]=o[n],o[n]=0,i.push(n))}));var r=new RegExp("[^".concat(t.placeholderChar,"]")),l=t.placeholder.split(r),c=t.conformedValue.split(/\D/g);return!n.some((function(t,e){var i=l[e].length>t.length?l[e].length:t.length,n=c[e]||"",o=parseInt(n,10);return o>s[t]||n.length===i&&o<a[t]}))&&{value:o.join(""),characterIndexes:i}},de.prototype={configure:function(t){if(this.settings||(this.settings={}),!t)return this;if(t.locale&&(this.locale=t.locale),t.pattern)if(Array.isArray(t.pattern)||"function"==typeof t.pattern)this.pattern=t.pattern;else if("string"==typeof t.pattern){var e=E.mergeSettings(void 0,t.definitions||{},se.LEGACY_DEFS);this.pattern=this._convertPatternFromString(t.pattern,e)}return t.patternOptions&&(this.patternOptions=t.patternOptions),this},process:function(t,e){if("string"!=typeof t)throw new Error("No string provided");var i={},n={originalValue:t,caretPos:e.selection.start,maskResult:!1};if("function"==typeof this.pattern){e.patternOptions||(e.patternOptions={});var s=E.extend({},this.patternOptions,e.patternOptions,{caretPos:e.selection.start,previousMaskResult:e.previousMaskResult});i=this.pattern(t,s),Array.isArray(i)&&(i={mask:i});var a=this._processCaretTraps(i.mask);i.mask=a.maskWithoutCaretTraps,i.caretTrapIndexes=a.indexes,e.caretTrapIndexes=a.indexes}else i={mask:this.pattern};if(!1===i.mask)return n;try{n=this._conformToMask(t,i,e)}catch(t){return n}if(e.patternOptions&&e.patternOptions.delimeter&&n.conformedValue&&n.conformedValue!==t&&(n.caretPos=n.conformedValue.length),"function"==typeof e.pipe){var o={};try{o=e.pipe(n,e)}catch(t){o=!1}!1===o?(n.pipeResult=!1,n.pipedValue=e.previousMaskResult):E.isString(o)?(n.pipeResult=!0,n.pipedValue=o,n.pipedCharIndexes=[]):n=E.extend({},n,{pipeResult:o.result,pipedValue:o.value,pipedCharIndexes:o.characterIndexes})}return n},_conformToMask:function(t,e,i){(i=E.mergeSettings(void 0,i,se.DEFAULT_CONFORM_OPTIONS)).placeholder=this._convertMaskToPlaceholder(e.mask,i.placeholderChar);var n=!1,s=!1===i.guide&&void 0!==i.previousMaskResult,a=t.length,o=i.previousMaskResult.length,r=this.pattern.length,l=i.placeholder.length||0,c=i.placeholderChar,d=i.selection.start,h=se.EMPTY_STRING,u=a-o,p=u>0,g=d+(p?-u:0),f=g+Math.abs(u);if(!0===i.keepCharacterPositions&&!p){for(var m=se.EMPTY_STRING,v=g;v<f;v++)i.placeholder[v]===c&&(m+=c);t=t.slice(0,g)+m+t.slice(g,a)}for(var b=t.split(se.EMPTY_STRING).map((function(t,e){return{char:t,isNew:e>=g&&e<f}})),y=a-1;y>=0;y--){var w=b[y];if(w!==i.placeholderChar){var x=y>=g&&o===r;w===i.placeholder[x?y-u:y]&&b.splice(y,1)}}t:for(var C=0;C<l;C++){var k=i.placeholder[C];if(k===c){if(b.length>0)for(;b.length>0;){var T=b.shift(),A=t.slice(C,C+1);if(T.char===c&&!0!==s){h+=c;continue t}if(e.literalRegex&&e.literalRegex.test(T.char)&&A&&(A===T.char||A===c)){if(p&&e.literals.indexOf(t[C-1])>-1){d++;continue t}var S=new RegExp("(".concat(T.char,")"),"g"),$=i.placeholder.match(S).length;t.match(S).length<=$&&(h+=T.char);for(var D=i.placeholder.slice(C).indexOf(T.char);D>0;)C++,D--;continue t}if(e.mask[C].test(T.char)){if(!0===i.keepCharacterPositions&&!1!==T.isNew&&i.previousMaskResult!==se.EMPTY_STRING&&!1!==i.guide&&p){for(var I=b.length,L=null,P=0;P<I;P++){var M=b[P];if(M.char!==c&&!1===M.isNew)break;if(M.char===c){L=P;break}}null!==L?(h+=T.char,b.splice(L,1)):C--}else h+=T.char;continue t}n=!0}!1===s&&(h+=i.placeholder.substr(C,l));break}h+=k}if(s&&!1===p){for(var B=null,O=0;O<h.length;O++)i.placeholder[O]===c&&(B=O);null!==B?h=h.substr(0,B+1):""!==t&&h.indexOf(t)>-1?d+=t.length:h=se.EMPTY_STRING}return{caretPos:d,caretTrapIndexes:i.caretTrapIndexes,conformedValue:h,charactersRejected:n,placeholder:i.placeholder,placeholderChar:c,maskResult:!0}},_processCaretTraps:function(t){for(var e=[],i=t.indexOf(se.CARET_TRAP);-1!==i;)e.push(i),t.splice(i,1),i=t.indexOf(se.CARET_TRAP);return{maskWithoutCaretTraps:t,indexes:e}},_convertMaskToPlaceholder:function(t,e){if(Array.isArray(t)||(t=[]),e||(e=se.PLACEHOLDER_CHAR),-1!==t.indexOf(e))throw new Error("".concat("Placeholder character must not be used as part of the mask. Please specify a character that is not present in your mask as your placeholder character.\n\nThe placeholder character that was received is:").concat(JSON.stringify(e),"\n\n")+"The mask that was received is: ".concat(JSON.stringify(t)));return t.map((function(t){return t instanceof RegExp?e:t})).join(se.EMPTY_STRING)},adjustCaretPosition:function(t){if(0===t.caretPos)return 0;function e(t){return t!==e}var i=t.rawValue.length,n=t.previousMaskResult.length,s=t.placeholder?t.placeholder.length:0,a=t.conformedValue?t.conformedValue.length:0,o=i-n,r=o>0;if(o>1&&!r&&!(0===n))return t.caretPos;var l,c,d=0;if(r&&(t.previousMaskResult===t.conformedValue||t.conformedValue===t.placeholder))d=t.caretPos-o;else{var h=t.conformedValue.toLowerCase(),u=t.rawValue.toLowerCase().substr(0,t.caretPos).split(se.EMPTY_STRING).filter((function(t){return-1!==h.indexOf(t)}));c=u[u.length-1];var p=t.previousPlaceholder.substr(0,u.length).split(se.EMPTY_STRING).filter(e).length,g=t.placeholder.substr(0,u.length).split(se.EMPTY_STRING).filter(e).length!==p,f=void 0!==t.previousPlaceholder[u.length-1]&&void 0!==t.placeholder[u.length-2]&&t.previousPlaceholder[u.length-1]!==t.placeholderChar&&t.previousPlaceholder[u.length-1]!==t.placeholder[u.length-1]&&t.previousPlaceholder[u.length-1]===t.placeholder[u.length-2];!r&&(g||f)&&p>0&&t.placeholder.indexOf(c)>-1&&void 0!==t.rawValue[t.caretPos]&&(l=!0,c=t.rawValue[t.caretPos]);var m=[];t.indexesOfPipedChars&&(m=t.indexesOfPipedChars.map((function(t){return h[t]})));for(var v=m.filter((function(t){return t===c})).length,b=u.filter((function(t){return t===c})).length,y=t.placeholder.substr(0,t.placeholder.indexOf(t.placeholderChar)).split(se.EMPTY_STRING).filter((function(e,i){return e===c&&t.rawValue[i]!==e})).length+b+v+(l?1:0),w=0,x=0;x<a;x++){if(d=x+1,h[x]===c&&w++,w>=y)break}}if(r)for(var C=d,k=d;k<=s;k++){if(!t.placeholder)return C;if(t.placeholder[k]===t.placeholderChar&&(C=k),t.placeholder[k]===t.placeholderChar||-1!==t.caretTrapIndexes.indexOf(k)||k===s)return C}else if(l){for(var T=d-1;T>=0;T--)if(t.conformedValue[T]===c||-1!==t.caretTrapIndexes.indexOf(T)||0===T)return T}else for(var A=d;A>=0;A--)if(t.placeholder[A-1]===t.placeholderChar||-1!==t.caretTrapIndexes.indexOf(A)||0===A)return A},_convertPatternFromString:function(t,e){var i,n,s=[],a=Object.keys(e);for(var o=0;o<t.length;o++)s.push((i=t[o],n=void 0,(n=a.indexOf(i))>-1&&(i=e[a[n]]),i));return s}};var he="mask",ue={definitions:void 0,guide:!1,keepCharacterPositions:!1,maskAPI:de,locale:null,pattern:void 0,patternOptions:{},placeholderChar:"_",pipe:void 0,process:void 0,processOnBlur:!0,processOnInitialize:!0};function pe(t,e){return this.element=t,this.settings=E.mergeSettings(this.element,e,ue),this.init()}pe.prototype={get type(){return this.element.type},init:function(){this.setLocale()},build:function(){this._replaceLegacySettings();if("string"==typeof this.settings.process)switch(this.settings.patternOptions.locale||(this.settings.patternOptions.locale=this.settings.locale),this.settings.process){case"number":this.settings.pattern=se.numberMask,this.element.classList.add("is-number-mask");break;case"rangeNumber":this.settings.pattern=se.rangeNumberMask,this.element.classList.add("is-range-number-mask");break;case"date":var t=$(this.element).data("datepicker");$.fn.datepicker&&$(this.element).data("datepicker")&&(this.settings.patternOptions||this.settings.patternOptions.format||(this.settings.patternOptions.format=t.settings.dateFormat)),this.settings.pattern=se.dateMask;break;case"rangeDate":var e=$(this.element).data("datepicker");$.fn.datepicker&&$(this.element).data("datepicker")&&(this.settings.patternOptions.format=e.settings.dateFormat),this.settings.pattern=se.rangeDateMask;break;default:this.element.classList.remove(["is-number-mask"].join(" "))}return this.mask=new this.settings.maskAPI(this.settings),this.state={previousMaskResult:""},this.handleEvents(),this.settings.processOnInitialize&&this.process(),this},setLocale:function(){var t=this;this.locale=H.currentLocale,this.settings.locale?H.getLocale(this.settings.locale).done((function(e){t.locale=H.cultures[e],t.build()})):this.build()},handleEvents:function(){var t=this;return this.element.addEventListener("keydown",(function(e){"Insert"===e.key&&(t.insertOn=!0)})),this.element.addEventListener("keyup",(function(e){"Insert"===e.key&&(t.insertOn=!1)})),this.changeEventHandler=function(){t.hasTriggeredChangeEvent=!0},this.element.addEventListener("change",this.changeEventHandler),this.focusEventHandler=function(){t.state.previousMaskResult=t.element.value,t.state.initialValue=t.element.value},this.element.addEventListener("focus",this.focusEventHandler),this.inputEventHandler=function(e){return t.hasTriggeredChangeEvent=!1,t.process(e)},this.element.addEventListener("input",this.inputEventHandler),this.blurEventHandler=function(e){if(!0===this.processingJustHappenedOnInput)return e.preventDefault(),!1;if(t.settings.processOnBlur){if(t.element.readOnly)return e.preventDefault(),!1;!t._hasChangedValue()||!t._isEdgeIE()&&t.hasTriggeredChangeEvent||($(t.element).trigger("change"),t.hasTriggeredChangeEvent=!0)}return delete t.state.initialValue,t.process(e)},this.element.addEventListener("blur",this.blurEventHandler),this},process:function(t){if(!this.settings.pattern)return!0;var e=this.mask,i=this.settings.patternOptions||{},n=".";this.locale&&this.locale.data&&this.locale.data.numbers&&this.locale.data.numbers.decimal&&(n=this.locale.data.numbers.decimal),i.symbols&&i.symbols.decimal&&(n=i.symbols.decimal),e.pattern||e.configure({pattern:this.settings.pattern,patternOptions:i});var s=this.element.value,a=!1;if(t&&"input"===t.type&&"insertText"===t.inputType&&(""===s?(s=this.state.previousMaskResult,a=!0):t.data===n&&s.indexOf(n)>-1&&(a=!0)),s===this.state.previousMaskResult)return a&&(this.element.value="",this.element.value="".concat(s)),!1;var o=this.element.selectionStart||0,r=this.element.selectionEnd||0;if(this._isAndroid()&&""===this.state.previousMaskResult&&o!==r&&(E.safeSetSelection(s.length,s.length),o=s.length,r=s.length),(this.insertOn||this.settings.useInsert)&&o<s.length){var l=!1,c=this.settings.patternOptions;if("number"===this.settings.process){var d=c.integerLimit;if(c.allowThousandsSeparator){var h=c.integerLimit-3<=0?0:(c.integerLimit-3)/3;d+=Math.floor(h)}c.allowDecimal&&s.includes(c.symbols.decimal)&&(d+=c.decimalLimit+1),c.allowNegative&&(d+=1),s.length>d&&(l=!0)}else("string"==typeof this.settings.pattern&&s.length>this.settings.pattern.length||this.settings.state&&s.length>this.settings.state.previousPlaceholder.length)&&(l=!0);l&&(/^[!@#$%^&*()_+\-=\\[\]{};':"\\|,.<>\\/?]*$/.test(s.charAt(o))?(s=s.slice(0,o-1)+s.charAt(o)+s.charAt(o-1)+s.slice(o+2),o++,r++):s=s.slice(0,o)+s.slice(o+1))}s=this._getSafeRawValue(s);var u={guide:this.settings.guide,keepCharacterPositions:this.settings.keepCharacterPositions,patternOptions:this.settings.patternOptions,placeholderChar:this.settings.placeholderChar,previousMaskResult:this.state.previousMaskResult,selection:{start:o,end:r}};o!==r&&(u.selection.contents=s.substring(o,r)),"function"==typeof this.settings.pipe&&(u.pipe=this.settings.pipe);var p=se.adjustRangeNumber(s,u,this);s=p.rawValue,u=p.opts;var g=e.process(s,u);if(!g.maskResult)return g.maskResult;var f=g.pipedValue?g.pipedValue:g.conformedValue;""!==f&&i&&i.suffix&&f.indexOf(i.suffix)<0&&(f+=this.settings.patternOptions.suffix),u.patternOptions&&u.patternOptions.delimeter&&f&&f!==s&&(s=f);var m={previousMaskResult:this.state.previousMaskResult||"",previousPlaceholder:this.state.previousPlaceholder||"",conformedValue:f,placeholder:g.placeholder,rawValue:s,caretPos:g.caretPos,placeholderChar:this.settings.placeholderChar};g.pipedCharIndexes&&(m.indexesOfPipedChars=g.pipedCharIndexes),g.caretTrapIndexes&&(m.caretTrapIndexes=g.caretTrapIndexes),g.caretPos=e.adjustCaretPosition(m);var v=this.state.previousMaskResult;return this.state.previousMaskResult=f,this.state.previousPlaceholder=g.placeholder,"firefox"===x.browser.name?this.element.value!==f&&(this.element.value=f):this.element.value=f,E.safeSetSelection(this.element,g.caretPos),v!==f&&($(this.element).trigger("write.mask",[f]),g.maskResult)},_isAndroid:function(){return"android"===(x&&x.os&&x.os.name?x.os.name:"")},_isEdgeIE:function(){var t=x&&x.browser&&x.browser.name?x.browser.name:"";return"ie"===t||"edge"===t},_hasChangedValue:function(){return!(this.state&&this.state.previousMaskResult||""===this.state.previousMaskResult)||this.state.previousMaskResult!==this.state.initialValue},_getSafeRawValue:function(t){if(E.isString(t))return t;if(E.isNumber(t))return String(t);if(null==t)return"";throw new Error("".concat('The "value" provided to the Masked Input needs to be a string or a number. The value received was:\n\n').concat(JSON.stringify(t)))},_replaceLegacySettings:function(){var t=["group","number","date","time"];this.settings.patternOptions=this.settings.patternOptions||{},this.settings.patternOptions.symbols=this.settings.patternOptions.symbols||{};var e=this.element.getAttribute("data-mask")||!1;"string"==typeof e&&e.length&&(this.settings.pattern=e);var i=this.element.getAttribute("data-mask-mode")||!1;if(i&&t.indexOf(i)>-1&&(this.settings.mode=i),this.settings.mode&&(-1===t.indexOf(this.settings.mode)&&delete this.settings.mode,"group"===this.settings.mode?this.settings.process=void 0:"date"===this.settings.mode?this.settings.process="date":this.settings.process=this.settings.mode,delete this.settings.mode),"number"===this.settings.process){this.settings.thousandsSeparator&&(this.settings.patternOptions.allowThousandsSeparator=this.settings.thousandsSeparator,delete this.settings.thousandsSeparator);var n=this.element.getAttribute("data-thousands")||!1;if(n&&(this.settings.patternOptions.allowThousandsSeparator="true"===n),"string"==typeof this.settings.pattern){var s=-1!==this.settings.pattern.indexOf("-");s&&(this.settings.patternOptions.allowNegative=s,this.settings.patternOptions.symbols.negative="-");var a=H.currentLocale.data.numbers?H.currentLocale.data.numbers.group:",",o=this.settings.patternOptions&&this.settings.patternOptions.symbols&&this.settings.patternOptions.symbols.thousands?this.settings.patternOptions.symbols.thousands:a,r=-1!==this.settings.pattern.indexOf(o);this.settings.patternOptions.allowThousandsSeparator=r,r&&(this.settings.patternOptions.symbols.thousands=o);var l="string"==typeof this.settings.patternOptions.symbols.decimal?this.settings.patternOptions.symbols.decimal:".",c=this.settings.pattern.split(l);void 0===this.settings.patternOptions.symbols.decimal&&(this.settings.patternOptions.symbols.decimal=H.currentLocale.data.numbers.decimal),this.settings.patternOptions.integerLimit=c[0].replace(/[^#0]/g,"").length,c[1]&&(this.settings.patternOptions.allowDecimal=!0,this.settings.patternOptions.decimalLimit=c[1].toString().replace(/[^#0]/g,"").length,this.settings.patternOptions.symbols.decimal||(this.settings.patternOptions.symbols.decimal=l))}}var d=this.element.getAttribute("data-must-complete")||!1;d&&(this.settings.mustComplete=d),this.element.getAttribute("data-show-currency")&&(this.settings.showSymbol="currency");var h,u,p=this.settings.showSymbol;if(["currency","percent"].indexOf(p)>-1){h="currency"===(u=this.settings.showSymbol)?{char:H.currentLocale.data.currencySign,format:H.currentLocale.data.currencyFormat}:"percent"===u?{char:H.currentLocale.data.numbers.percentSign,format:H.currentLocale.data.numbers.percentFormat}:{};var g,f,m,v="currency"===p?"¤":h.char,b=new RegExp(v,"g").exec(h.format),y=-1;if(b&&b.length){if(0===(y=h.format.indexOf(b[0])))m="prefix",g=new RegExp("[^".concat(v,"]\\S"),"g"),f=h.format.replace(g,"");else if(y>0){for(m="suffix",g=new RegExp("\\S[^".concat(v,"]"),"g");/\s/.test(h.format.charAt(y-1));)--y;f=h.format.substr(y).replace(g,"")}"currency"===p&&(f=f.replace("¤",h.char)),this.settings.patternOptions[m]=f}}},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown().init()},teardown:function(){return"number"===this.settings.process&&this.element.classList.remove("is-number-mask"),"rangeNumber"===this.settings.process&&this.element.classList.remove("is-range-number-mask"),this.element.removeEventListener("focus",this.focusEventHandler),delete this.focusEventHandler,this.element.removeEventListener("input",this.inputEventHandler),delete this.inputEventHandler,this.blurEventHandler&&(this.element.removeEventListener("blur",this.blurEventHandler),delete this.blurEventHandler),this.changeEventHandler&&(this.element.removeEventListener("change",this.changeEventHandler),delete this.changeEventHandler),this}},$.fn.maskinput=function(t){return this.each((function(){var e=$.data(this,he);e?e.updated(t):(e=$.data(this,he,new pe(this,t))).destroy=function(){this.teardown(),$.removeData(this.element,he)}}))},$.fn.maskedinput=$.fn.maskinput,$.fn.mask=$.fn.maskinput;var ge="popupmenu",fe={menu:null,trigger:"click",autoFocus:!0,mouseFocus:!0,attachToBody:!0,removeOnDestroy:!1,beforeOpen:null,ariaListbox:!1,eventObj:void 0,returnFocus:!0,showArrow:null,triggerSelect:!0,placementOpts:new Ot({containerOffsetX:10,containerOffsetY:"ios"===x.os.name?25:15,strategies:["flip","shrink"]}),offset:{x:0,y:0},predefined:$(),duplicateMenu:null,stretchToWidestMenuItem:!1,attributes:null,cssClass:null};function me(t,e){this.settings=E.mergeSettings(t,e,fe),this.element=$(t),this.init()}me.prototype={init:function(){this.isFirefox="firefox"===x.browser.name,this.setup(),this.addMarkup(),this.handleEvents(),this.iconFilteringSetup(),"immediate"===this.settings.trigger&&this.open(this.settings.eventObj),this.menu.find(".submenu").length&&this.menu.addClass("has-submenu")},isRTL:function(){return x.rtl},get idString(){if(this.id)return this.id.charAt(0).match(/\d/)?"popupmenu-".concat(this.id):"".concat(this.id)},get isOpen(){return f.hasClass(this.element[0],"is-open")||f.hasClass(this.menu[0],"is-open")},get isFocused(){var t=document.activeElement,e=this.element.is(t),i=this.menu&&this.menu.length&&this.menu[0].contains(t);return!(!e&&!i)},setup:function(){this.element[0].getAttribute("data-popupmenu")&&!this.settings.menu&&(this.settings.menu=this.element[0].getAttribute("data-popupmenu").replace(/#/g,"")),this.settings.menuId&&(this.settings.menu=this.settings.menuId,this.id=this.settings.menuId,this.settings.menuId=void 0),this.id||(this.id=(parseInt($(".popupmenu-wrapper").length,10)+1).toString()),this.predefinedItems=$().add(this.settings.predefined)},setMenuOrgContainment:function(t){if(t&&t.length){var e=t.parent();e.is("body")||(this.menuOrgContainment={parent:e,index:e.children().index(t)},this.menu.data("containment",this.menuOrgContainment))}},insertAtContainment:function(t,e){var i=function(t){return t&&t.length};if(e=e||this.menuOrgContainment||{},i(t)&&i(e.parent)){var n=e.index,s=e.parent,a=s.children().length;n<0&&(n=Math.max(0,a+1+n)),n<a?s.children().eq(n).before(t):s.append(t)}},addMarkup:function(){var t,e,i;if(!this.menu||!this.menu.length)switch(l(this.settings.menu)){case"string":t=this.settings.menu||this.idString,this.menu=$("#".concat(t)),this.settings.duplicateMenu&&this.settings.attachToBody&&this.menu.parent().not("body").length>0&&(this.menu.data("trigger",this.element),i=this.menu.data("trigger")[0].id,e=this.menu.clone(),this.setMenuOrgContainment(e),e.detach().appendTo("body"),e.attr("data-trigger",i),this.menu.attr("data-trigger",i));break;case"object":null===this.settings.menu?this.menu=this.element.next(".popupmenu, .popupmenu-wrapper"):this.menu=$(this.settings.menu),(t=this.menu.attr("id"))&&""!==t||(this.menu.attr("id",this.idString),t=this.menu.attr("id")),i=this.element.attr("id")}if(this.menu.is(".popupmenu-wrapper")&&(this.preExistingWrapper=!0,this.wrapper=this.menu,this.menu=this.wrapper.children(".popupmenu").first()),this.menu.parent().is(".popupmenu-wrapper")&&(this.preExistingWrapper=!0,this.wrapper=this.menu.parent()),0!==this.menu.length){this.settings.attributes&&(E.addAttributes(this.element,this,this.settings.attributes,"trigger"),E.addAttributes(this.menu,this,this.settings.attributes,"menu"));var n=this.menu.parent().not(".popupmenu-wrapper, body");this.settings.attachToBody&&n.length>0&&(this.originalParent=this.menu.prev(),this.setMenuOrgContainment(this.menu),this.menu.detach().appendTo("body"),this.settings.duplicateMenu&&this.menu.attr("id","".concat(this.settings.menu,"-original"))),this.menu.is(".popupmenu")||this.menu.addClass("popupmenu").attr("role",this.settings.ariaListbox?"listbox":"menu").attr("aria-labelledby",i),this.menu.data("trigger",this.element),this.wrapper=this.menu.parent(".popupmenu-wrapper"),this.wrapper.length||(this.menu.wrap('<div class="popupmenu-wrapper"></div>'),this.wrapper=this.menu.parent(".popupmenu-wrapper"));var s=this.settings.cssClass;s&&"string"==typeof s&&this.wrapper.addClass(s),this.wrapper.find("svg").each((function(){$(this).data("icon")||$(this).icon()})),0===this.wrapper.parents().filter((function(){var t=this?this.style.overflow:null;return"auto"!==t&&"visible"!==t&&"scroll"!==t})).length&&(this.wrapper[0].style.position="absolute"),this.menu.find(".popupmenu").each((function(t,e){var i=$(e);i.parent().hasClass("wrapper")||i.wrap('<div class="wrapper"></div>')}));var a=!1,o=this.element.parent().attr("class");if(!0===this.settings.showArrow)a=!0;else if(null===this.settings.showArrow){var r=this.element.closest(".toolbar"),c=this.element.closest(".masthead");(this.element.hasClass("btn-menu")||this.element.hasClass("btn-actions")||this.element.hasClass("btn-icon")&&"#icon-more"===this.element.find("use").attr("href")||r.length>0&&!r.is(".formatter-toolbar")||c.length>0||"colorpicker-menu"===this.settings.menu||this.element.is(".searchfield-category-button")||o&&o.indexOf("more")>=0||o&&o.indexOf("btn-group")>=0)&&(a=!0)}if(a){var d=$('<div class="arrow"></div>'),h=this.menu.parent(".popupmenu-wrapper");0===h.find(".arrow").length&&h.addClass("bottom").append(d)}var u=this.element.closest(".field-short").length>0;this.menu[u?"addClass":"removeClass"]("popupmenu-short"),this.element.closest(".masthead").not(".search-results .masthead").length>0&&this.menu.parent(".popupmenu-wrapper").addClass("inverse"),this.element.attr("aria-haspopup",!0),this.element.attr("aria-controls",t),this.markupItems(),this.menu.is(".hidden")&&this.menu.removeClass("hidden"),this.settings.stretchToWidestMenuItem&&this.element.width(parseInt(this.getMaxMenuWidth(),10))}},renderItem:function(t){if(void 0===t)return"";var e=this;function i(t){var i="";return t.forEach((function(t){i+=e.renderItem(t)})),i}if(Array.isArray(t)){var n=i(t),s='<ul class="popupmenu">'.concat(n,"</ul>");return M.stripWhitespace(s)}var a="",o="";if(void 0!==t.separator)return t.heading&&(a+='<li class="heading">'.concat(t.heading,"</li>")),"single"===t.nextSectionSelect?o=" single-selectable-section":"multiple"===t.nextSectionSelect&&(o=" multi-selectable-section"),M.stripWhitespace('\n        <li class="separator'.concat(o,'"></li>\n        ').concat(a,"\n      "));if(t.menu){var r="";t.menuId&&(r=' id="'.concat(t.menuId,'"'));var l="";t.hasIcons&&(l+=" has-icons");var c="";return Array.isArray(t.menu)&&(c=i(t.menu)),t.noMenuWrap?c:M.stripWhitespace("\n        <ul".concat(r,' class="popupmenu').concat(l,'">\n          ').concat(c,"\n        </ul>\n      "))}var d="",h="",u="",p="",g="",f="",m="",v="";return t.disabled&&(d+=" is-disabled"),!1===t.visible&&(h+=" hidden"),t.id&&(p=' id="'.concat(t.id,'"')),"single"===t.selectable&&(g+=" is-selectable"),"multiple"===t.selectable&&(g+=" is-multiselectable"),t.icon&&(u='<svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n        <use href="#icon-'.concat(t.icon,'"></use>\n      </svg>')),Array.isArray(t.submenu)&&(f+=" submenu",m+=this.renderItem(t.submenu),v+='<svg class="arrow icon-dropdown icon" focusable="false" aria-hidden="true" role="presentation">\n        <use href="#icon-dropdown"></use>\n      </svg>'),M.stripWhitespace('<li class="popupmenu-item'.concat(d).concat(h).concat(g).concat(f,'">\n      <a').concat(p,' href="#">').concat(u,"<span>").concat(t.text,"</span>\n        ").concat(v,"\n      </a>\n      ").concat(m,"\n    </li>"))},toData:function(t){var e={},i=[],n=this;(t=t||{}).contextElement||(t.contextElement=this.menu),t.contextElement instanceof HTMLElement&&(t.contextElement=$(t.contextElement)),t.contextElement.is(".popupmenu-wrapper")&&(t.contextElement=t.contextElement.children("ul"));var s=t.contextElement.attr("id");s&&s.length&&(e.menuId="".concat(s));var a,o,r=t.contextElement.hasClass("has-icons");function l(t){var i=$(t),s={};if(!i.hasClass("heading")){if(i.hasClass("separator")){s.separator=!0,i.hasClass("single-selectable-section")&&(a=!0,s.nextSectionSelect="single"),i.hasClass("multi-selectable-section")&&(o=!0,s.nextSectionSelect="multiple");var r=i.next();return r.hasClass("heading")&&(s.heading=r.text().trim()),s}var c=i.children("a"),d=c.children(".icon:not(.close):not(.icon-dropdown):not(.image-user-status .icon)"),h=c.attr("id");s.text=c.text().trim(),s.disabled=i.hasClass("is-disabled"),s.visible=!i.hasClass("hidden"),"string"==typeof h&&h.length&&(s.id=h),d.length&&d[0]instanceof SVGElement&&(s.icon=d[0].querySelector("use").getAttribute("href").replace("#icon-",""));var u=!a&&!o,p=!n.hasSelectableSeparator(i);s.selectable&&u&&p&&("single"===s.selectable?a=!0:"multiple"===s.selectable&&(o=!0),e.selectable=s.selectable);var g=i.find(".popupmenu");return g.length&&(s.submenu=[],g.first().children().each((function(t,e){var i=l(e);i&&s.submenu.push(i)}))),s}}return e.hasIcons=r,t.contextElement.children("li").each((function(t,e){var n=l(e);n&&i.push(n)})),t.noMenuWrap?e=i:e.menu=i,e},markupItems:function(t){var e=this,i=this;t?t instanceof HTMLElement&&(t=$(t)):t=this.menu;var n=t.find("li:not(.heading):not(.separator)");function s(t){var e=$(t),i=e.parents("li"),n=[];return e.add(i).each((function(t,e){n.push($(e).parent().children("li:not(.heading):not(.separator)").index(e))})),n.join("-")}t[0].setAttribute("role","menu"),t[0].setAttribute("aria-labelledby",this.element.attr("id")),n.each((function(t,n){var a=$(n).children("a")[0],o=$(a),r=$(n),l=o.children("span")[0],c=r.children("ul")[0],d=r.children(".wrapper")[0];if(n.setAttribute("role","none"),a){if(a.setAttribute("role",i.settings.ariaListbox?"option":"menuitem"),a.setAttribute("tabindex","-1"),i.settings.attributes){var h=e.element.parents(".flex-toolbar")?"option-".concat(s(n),"-menu-item"):"option-".concat(s(n));E.addAttributes(o,i,i.settings.attributes,h)}if(r.hasClass("is-disabled")?(r.addClass("is-disabled"),a.setAttribute("aria-disabled","true"),a.setAttribute("disabled",!0)):(r.removeClass("is-disabled"),o.removeAttr("aria-disabled"),a.removeAttribute("disabled")),c instanceof HTMLElement&&c.classList.add("popupmenu"),d instanceof HTMLElement&&(n.className+="".concat(f.hasAnyClass(n)?" ":"","submenu"),(c=$(d).children("ul")[0])instanceof HTMLElement&&c.classList.add("popupmenu"),i.element.parent(".datagrid-cell-wrapper").length>0&&$(n).children("div.wrapper").length>0&&($(n).addClass("has-selectable-submenu"),$(n).find(".popupmenu").children().addClass("is-selectable"))),f.hasClass(n,"submenu")&&(l||(a.innerHTML="<span>".concat(a.innerHTML,"</span>"),l=o.children("span")[0]),0===o.find("svg.arrow").length&&o.append($.createIconElement({classes:["arrow","icon-dropdown"],icon:"dropdown"})),a.setAttribute("aria-haspopup","true")),f.hasClass(n,"is-checked")&&(a.setAttribute("role","menuitemcheckbox"),a.setAttribute("aria-checked",!0),!i.hasSelectableSeparator(n))){var u="is-selectable";i.menu.is(".is-multiselectable")&&(u="is-multiselectable"),i.getAdjacentSelectables(n).addClass(u)}f.hasClass(n,"is-not-checked")&&(n.className=n.className.replace("is-not-checked",""),a.setAttribute("role","menuitemcheckbox"),a.removeAttribute("aria-checked"))}})),i.refreshHasIcons(t)},refreshHasIcons:function(t){t||(t=this.menu);var e=t.find("li:not(.heading):not(.separator):not(.hidden)").filter((function(t,e){return $(e).find(".icon:not(.close):not(.icon-dropdown):not(.image-user-status .icon)").length>0})).length>0;t.toggleClass("has-icons",e)},refreshMenuItem:function(t,e,i){if(t&&t instanceof HTMLElement&&e&&$(this.menu)[0].contains(t)){var n,s=t.querySelector("a"),a=t.querySelector(".icon:not(.close):not(.icon-dropdown)");if(e.text&&s.innerText.trim()!==e.text&&(s.innerText="".concat(e.text)),!0===e.disabled?-1===t.className.indexOf("hidden")&&t.classList.add("is-disabled"):t.className.indexOf("is-disabled")>-1&&t.classList.remove("is-disabled"),!0===e.visible?t.className.indexOf("hidden")>-1&&t.classList.remove("hidden"):-1===t.className.indexOf("hidden")&&t.classList.add("visible"),e.icon?a&&((n=a.querySelector("use"))&&n.getAttribute("href").replace("#icon-","")!==e.icon&&a.remove(),n.setAttribute("href","#icon-".concat(e.icon))):a&&a.remove(),e.submenu){t.classList.add("submenu");var o=t.querySelector(".popupmenu");if(o)for(var r=o.children,l=0;l<e.submenu.length;l++)e.submenu[l].isSubmenuItem=!0,this.refreshMenuItem(r.item(l),e.submenu[l],i)}"function"==typeof i&&i.apply(this,[t,e])}},handleEvents:function(){var t=this,e="rightClick"!==this.settings.trigger;function i(t){return t.stopPropagation(),t.preventDefault(),!1}function n(e,i){if(t.keydownThenClick)delete t.keydownThenClick;else{var n=!0===i?0:2;if(!(e.button>n||t.element.is(":disabled"))){["android","ios"].indexOf(x.os.name)>-1&&(t.holdingDownClick=!0),t.close(),function(e){if(t.element.hasClass("is-disabled"))return e.stopPropagation(),void e.preventDefault();var i="rightClick"===t.settings.trigger;if(e.stopPropagation(),e.preventDefault(),i&&t.menu.hasClass("is-open"))return t.close(),void t.open(e);t.menu.add(t.element).hasClass("is-open")?t.close():t.open(e)}(e)}}}"immediate"===this.settings.trigger||(e&&this.element.off("click.popupmenu").on("click.popupmenu",(function(t){n(t,!0)})),e||(this.menu.parent().on("contextmenu.popupmenu",i),x.features.touch?this.element.on("touchstart.popupmenu",(function(t){"ios"===x.os.name&&"INPUT"!==t.target.tagName&&t.preventDefault(),$(t.target).addClass("longpress-target")})).on("touchend.popupmenu",(function(t){$(t.target).removeClass("longpress-target")})).on("longpress.popupmenu",(function(e,i){e.stopPropagation(),t.openedWithTouch=!0,n(i)})):this.element.on("contextmenu.popupmenu",(function(t){i(t),n(t)})))),this.menu.off("dragstart.popupmenu").on("dragstart.popupmenu","a",(function(){return!1})),this.element.not(".autocomplete, ul").on("keydown.popupmenu",(function(e){switch(e.which){case 13:case 32:"click"===t.settings.trigger&&(t.keydownThenClick=!0,t.open(e));break;case 121:e.shiftKey&&t.open(e)}})).on("updated.popupmenu",(function(e,i){e.stopPropagation(),t.updated(i)}))},handleKeys:function(){var t=this;this.menu.on("click.popupmenu","li",(function(e){var i=$(this).find("a");t.handleItemClick(e,i)})),this.menu.on("click.btn-menu.".concat(this.id),(function(t){t.target.closest(".switch")||t.preventDefault()}));var e="li:not(.separator):not(.hidden):not(.heading):not(.group):not(.is-disabled):not(.is-placeholder)";this.settings.mouseFocus&&this.menu.on("mouseenter.popupmenu","li",(function(){t.highlight($(this).children("a"))})),setTimeout((function(){$(document).off("keydown.popupmenu.".concat(this.id)).on("keydown.popupmenu.".concat(this.id),(function(i){var n=i.which;if(27===n)return i.stopPropagation(),i.stopImmediatePropagation(),t.close(!0),!1;if(9===n&&(i.stopPropagation(),t.close(!0)),32===n){i.stopPropagation();var s=$(i.target),a=s.find("input:checkbox");if(a.length)return a.trigger("click"),!0;var o=$();if(s.is(".tab")||s.parent().is(".tab")||s.is(".tab-more"))return i.preventDefault(),!0;if(s.is("li")&&(o=s.children("a")),s.is("a")&&(o=s),o.length)return o.trigger("click"),!0}var r=t.menu.find(":focus");r[0]||(r=$(t.menu[0].querySelector(".is-focused a")));var l,c,d,h,u="colorpicker-menu"===t.settings.menu,p=t.element.is(".autocomplete");37!==n||p||(i.stopPropagation(),i.preventDefault(),r.closest(".popupmenu")[0]!==t.menu[0]&&r.closest(".popupmenu").length>0&&(r.closest(".popupmenu").removeClass("is-open").parent().parent().removeClass("is-submenu-open"),t.highlight(r.closest(".popupmenu").parent().prev("a")))),l=function(t){var i;return t.parent().prevAll(e).each((function(){if(!i){var t=$(this),e=t.children("a");t.is(".is-disabled")||!0===e.prop("disabled")||(i=e)}})),i||h(t)},h=function(t){var i=t.parent().prevAll(e).last(),n=i.children("a");return i.is(".is-disabled")||!0===n.prop("disabled")?c(n):n},c=function(t){var i;return t.parent().nextAll(e).each((function(){if(!i){var t=$(this),e=t.children("a");t.is(".is-disabled")||!0===e.prop("disabled")||(i=e)}})),i||h(t)},d=function(t){var i=t.parent().nextAll(e).last(),n=i.children("a");return i.is(".is-disabled")||!0===n.prop("disabled")?l(n):n};for(var g=0,f=t.menu.children(e),m=f.length,v=0,b=0;b<m;b++){var y=f[b];if(0===v&&(v=y.offsetTop),y.offsetTop!==v)break;g++}if(!u&&38===n||u&&n===(H.isRTL()?39:37)){if(i.stopPropagation(),i.preventDefault(),0===r.parent().prevAll(e).length)return void(0===r.length?t.highlight(t.menu.children(e).last().find("a")):t.highlight(d(r)));t.highlight(l(r))}if(u&&38===n&&(i.stopPropagation(),i.preventDefault(),r.parent().prevAll(e).length>0&&t.highlight($(r.parent().prevAll(e)[g-1]).find("a"))),39!==n||p||(i.stopPropagation(),i.preventDefault(),r.parent().hasClass("submenu")&&(t.openSubmenu(r.parent()),t.highlight(r.parent().find(".popupmenu a:first")))),!u&&40===n||u&&n===(H.isRTL()?37:39)&&!p){if(i.stopPropagation(),i.preventDefault(),0===r.parent().nextAll(e).length)return void(0===r.length?t.highlight(t.menu.children(e).first().find("a")):t.highlight(h(r)));t.highlight(c(r))}u&&40===n&&(i.stopPropagation(),i.preventDefault(),r.parent().nextAll(e).length>0&&t.highlight($(r.parent().nextAll(e)[g-1]).find("a")))}))}),1)},handleItemClick:function(t,e){var i=e.attr("href"),n=[e];return!(!t&&!e||!e.length)&&(!e.hasClass("no-results")&&(e.parent().is(".submenu, .hidden, .is-disabled")||e[0].disabled?(t.preventDefault(),!1):!(e.find("input[checkbox]").length>0)&&(this.element.hasClass("btn-filter")&&(this.iconFilteringUpdate(e),t.preventDefault()),this.isSelectable(e.parent())&&(n=this.select(e)),e.parent().hasClass("is-toggleable")&&e.parent().toggleClass("is-checked"),t&&this.settings.triggerSelect&&(1===n.length&&n.push(void 0),n.push(!0),this.element.triggerHandler("selected",n)),!(!this.menu.hasClass("is-multiselectable")&&!this.isInMultiselectSection(e))||(this.close(),!!this.element.is(".autocomplete")||(i&&"#"!==i.charAt(0)?("_blank"===e.attr("target")?window.open(i,"_blank"):window.location.href=i,!0):(t&&(t.preventDefault(),t.stopPropagation()),!0))))))},iconFilteringSetup:function(t){if(this.element.hasClass("btn-filter")){var e=this.element.find("svg.icon-dropdown"),i=t||$("li.is-checked a",this.menu)||$("li:first a",this.menu),n=i.find("span").text();1===e.length&&this.element.append($.createIconElement({classes:"icon-dropdown",icon:"dropdown"})),e.first().changeIcon(i.find("svg").getIconName()),this.element.find(".audible").text(n)}},iconFilteringUpdate:function(t){if(this.element.hasClass("btn-filter")){var e=t||$("li.is-checked a",this.menu)||$("li:first a",this.menu),i=e.find("span").text();this.element.find(".audible").text(i),this.element.find("svg:not(.ripple-effect):first").changeIcon(e.find("svg").getIconName())}},getPositionFromEvent:function(t){var e=0,i=0;if(t||(t=window.event),!t)return{};if(t.changedTouches){var n=t.changedTouches[0];e=n.pageX,i=n.pageY}else t.pageX||t.pageY?(e=t.pageX,i=t.pageY):(t.clientX||t.clientY)&&(e=t.clientX+document.body.scrollLeft+document.documentElement.scrollLeft,i=t.clientY+document.body.scrollTop+document.documentElement.scrollTop);return{x:e,y:i}},position:function(t){var e=this,i=this.element,n=this.isRTL(),s=this.menu.parent(".popupmenu-wrapper"),a=this.getPositionFromEvent(t),o=this.menu[0].offsetWidth;this.menu[0].offsetHeight;if(s.length){var r=this.element[0].offsetWidth;"click"===this.settings.trigger&&r>o&&this.menu.width(r),i.is("svg, .icon")&&i.closest(".tab").length&&(i=i.closest(".tab")),s.find(".arrow").removeAttr("style");var l=$.extend({},this.settings.placementOpts),c=["flip"];c.push("shrink-y"),l.strategies=c,"immediate"===this.settings.trigger&&this.settings.eventObj||"rightClick"===this.settings.trigger?(l.x=m(0,"x")-(n?o:0)+(n?-1:1)*this.settings.offset.x,l.y=m(0,"y")+this.settings.offset.y,0===l.x&&0===l.y&&(l.x=this.settings.offset.x||0,l.y=this.settings.offset.y||0,l.parent=this.element,l.placement="bottom",l.parentXAlignment=n?"right":"left")):(l.x=this.settings.offset.x||0,l.y=this.settings.offset.y||0,l.parent=this.element,l.placement="bottom",l.strategies.push("nudge"));var d=i.parents(".toolbar"),h=d.length>0,u=i.parents(".title").length>0,p=h&&d.children(".buttonset, .title").length>1,g=i.parents(".pager-pagesize").length>0;d.parents(".header").length>0&&(l.y+=5),i.is(".btn-menu")?g?b():h&&p?u?v():b():v():i.is(".btn-actions")||i.is(".tab-more")?b():i.is(".btn-split-menu, .tab, .searchfield-category-button")&&!i.parent(".pager-pagesize").length&&v(),i.parents(".masthead").length>0&&(l.y+=15);var f=this.element.parents(".page-container:not(.tab-container)");f.length&&(l.container=f.first()),s.one("afterplace.popupmenu",(function(t,i){e.handleAfterPlace(t,i)})),this.wrapperPlace=new Rt(s,l),this.wrapperPlace.place(l)}function m(t,e){return a[e="x"===e||"y"===e?e:"x"]}function v(){l.parentXAlignment=n?"right":"left"}function b(){l.parentXAlignment=n?"left":"right"}},handleAfterPlace:function(t,e){var i=this.menu.parent(".popupmenu-wrapper");return this.wrapperPlace.setArrowPosition(t,e,i),e.height<this.menu[0].offsetHeight&&this.menu[0].classList.contains("colorpicker")&&this.menu[0].classList.add("has-scrollbar"),e.height&&(i[0].style.height="",this.menu[0].style.height=e.height+(/(px|%)/i.test("".concat(e.height))?"":"px")),e.width&&(i[0].style.width="",this.menu[0].style.width=e.width+(/(px|%)/i.test("".concat(e.width))?"":"px")),i.triggerHandler("popupmenuafterplace",[e]),e},callSource:function(t,e,i,n){if("function"==typeof this.settings.beforeOpen){var s=this,a=this.menu;if(!(f.isElement(i)&&$(i).is(".popupmenu, .submenu")&&(a=$(i),i.hasClass("is-open")))){var o=function(i){if(a.children().filter((function(t,e){return-1===s.predefinedItems.index(e)})).off().remove(),!1===i)return!1;var n=$(i);a.append(n);var o=a.parent(".wrapper, .popupmenu-wrapper");return o.length||(o=a.wrap('<div class="wrapper">').parent()),o.removeAttr("style"),s.markupItems(a),e&&(a.is(s.menu)?s.open(t,!0):s.openSubmenu(o.parent("li"),!0)),!0},r={};a.is(this.menu)||(r.contextElement=a),r.element=this.element,r.isSubmenu=n,"string"!=typeof this.settings.beforeOpen?this.settings.beforeOpen(o,r):window[this.settings.beforeOpen](o,r)}}},getMaxMenuWidth:function(){if(!(this.menu instanceof $))return 0;var t=!this.isOpen;t&&(this.wrapper.css({left:"-999999px"}),this.menu.addClass("is-open"));var e=this.menu.width();return t&&(this.wrapper.css({left:""}),this.menu.removeClass("is-open")),e},open:function(t,e,i){var n=this;if(this.menu&&this.menu.length){if((this.element.hasClass("autocomplete")||!1!==this.element.triggerHandler("beforeopen",[this.menu,this]))&&(e||(this.callSource(t,!0),!this.settings.beforeOpen)))if(i||"ios"!==x.os.name){$(".popupmenu.is-open").filter((function(){return 0===$(this).parents(".popupmenu").length})).not(this.menu).each((function(){var t=$(this).data("trigger");if(t&&t.length){var e=$(this).data("trigger").data("popupmenu");e&&"function"==typeof e.close&&e.close()}}));var s=this.element.data("tooltip");s&&s.visible&&s.hide();var a=$(".dropdown.is-open");if(a.length>0){var o=a.parent().prev().data("dropdown");o&&o.closeList("cancel")}var r=document.querySelector(".application-menu.is-open");r instanceof HTMLElement&&!r.contains(this.element[0])&&!r.contains(this.menu[0])&&$(document).triggerHandler("dismiss-applicationmenu"),this.element.addClass("is-open"),this.menu.addClass("is-open").attr("aria-hidden","false"),this.element.hasClass("inverse")&&this.menu.parent(".popupmenu-wrapper").addClass("inverse"),this.position(t),E.fixSVGIcons(this.menu),this.element.closest(".header").length>0&&(this.menu.parent()[0].style.zIndex="9001"),this.menu.find("a").each((function(){var t=$(this),e=t.parent();e.hasClass("is-disabled")?(e.addClass("is-disabled"),t.attr("aria-disabled","true"),t.attr("disabled","disabled")):(e.removeClass("is-disabled"),t.removeAttr("aria-disabled"),t.removeAttr("disabled"))})),setTimeout((function(){$(document).on("touchend.popupmenu.".concat(n.id," click.popupmenu.").concat(n.id),(function(t){var e="colorpicker-menu"===n.settings.menu;2!==t.button&&(n.holdingDownClick?delete n.holdingDownClick:$(t.target).is(n.element)&&!e&&"rightClick"!==n.settings.trigger||(0===$(t.target).closest(".popupmenu").length&&n.close(!0,"rightClick"===n.settings.trigger),$(t.target).hasClass("swatch")&&setTimeout((function(){n.close()}),150)))})),void 0===window.orientation&&$("body").on("resize.popupmenu",(function(){n.handleCloseEvent()})),$(window).on("scroll.popupmenu",(function(){setTimeout((function(){n.close()}),150)})),$(".datagrid-wrapper").on("scroll.popupmenu",(function(){setTimeout((function(){n.close()}),150)})),$(".scrollable, .modal.is-visible .modal-body-wrapper").on("scroll.popupmenu",(function(){setTimeout((function(){n.close()}),150)})),n.element.triggerHandler("open",[n.menu])}),300),$("iframe").each((function(){var t=$(this);t.ready((function(){try{t.contents().find("body").on("click.popupmenu",(function(){n.close()}))}catch(t){}}))})),this.handleKeys();var l,c,d=0;n.menu.find(".popupmenu").removeClass("is-open"),n.menu.on("mouseenter.popupmenu touchend.popupmenu",".submenu:not(.is-disabled)",(function(t){if(!$(t.target).hasClass("popupmenu")){var e=$(this);l=t.pageX,clearTimeout(n.submenuOpenTimeout),n.submenuOpenTimeout=setTimeout((function(){n.openSubmenu(e)}),300),$(document).on("mousemove.popupmenu.".concat(this.id),(function(t){d=t.pageX}))}})).on("mouseleave.popupmenu",".submenu",(function(){$(document).off("mousemove.popupmenu.".concat(this.id));var t=((c=$(this).find("ul")).parent(".wrapper").length>0?parseInt(c.parent(".wrapper")[0].style.left,10):0)<0,e=c.parents(".submenu").length>0,i=d-l<3.5;t&&(i=d-l>=0),(i||e)&&(c.removeClass("is-open").removeAttr("style"),c.parent(".wrapper").removeAttr("style"),c.parent().parent().removeClass("is-submenu-open"),c=null),clearTimeout(n.submenuOpenTimeout)})),n.settings.autoFocus&&setTimeout((function(){var t=":not(.separator):not(.hidden):not(.heading):not(.group):not(.is-disabled)",e=n.menu.children(t).find(".is-selected").children("a");e.length||(e=n.menu.children(t).first().children("a")),n.highlight(e),n.element.triggerHandler("afteropen",[n.menu])}),1)}else setTimeout((function(){n.open(t,e,!0)}),400)}},handleCloseEvent:function(){this.holdingDownClick||this.close()},openSubmenu:function(t,e){if(!f.hasClass(t,"is-disabled")&&!t[0].disabled){var i=t.children(".wrapper, .popupmenu");i.length&&i.is(".wrapper")&&(i=i.children(".popupmenu"));var n=this.element.triggerHandler("beforeopen",[i,this]);!1!==n&&(!e&&(n=this.callSource(null,!0,i,!0),this.settings.beforeOpen)||this.showSubmenu(t))}},showSubmenu:function(t){this.element.triggerHandler("show-submenu",[t]);var e=t.children(".wrapper").filter(":first"),i=this.isRTL();if(0===e.length){var n=t.children("ul").filter(":first");n.wrap('<div class="wrapper"></div>'),e=n.parent()}var s,a=e.children(".popupmenu"),o=t.position().left+t.outerWidth();t.parent().find(".popupmenu").removeClass("is-open").removeAttr("style"),e.children(".popupmenu").addClass("is-open"),s=e.outerWidth(),i&&(o=t.position().left-s),e[0].style.left="".concat(o,"px"),e[0].style.top="".concat(parseInt(t.position().top,10)-5,"px");var r=a.outerWidth();if(e.offset().left+r>$(window).width()+$(document).scrollLeft()||i&&e.offset().left<0){if(e[0].style.left="-9999px",r=a.outerWidth(),o=t.position().left-r,i)o=e.closest(".popupmenu").outerWidth()-4;if(e[0].style.left="".concat(o,"px"),(e.offset().left<0||i&&e.offset().left+r>$(window).width()+$(document).scrollLeft())&&(o=t.position().left-r+Math.abs(e.offset().left)+40,i&&(o=t.position().left-r-30),e[0].style.left="".concat(o,"px"),r=a.outerWidth()),e.offset().left+r>$(window).width()+$(document).scrollLeft()||i&&e.offset().left<0)r-=e.offset().left+r-($(window).width()+$(document).scrollLeft()),a[0].style.width="".concat(r,"px")}var l=a.outerHeight(),c=this.settings.placementOpts.containerOffsetY;if(e.offset().top+l>window.innerHeight-c+$(document).scrollTop()){var d=e.offset().top+l-(window.innerHeight+$(document).scrollTop());if(e[0].style.top="".concat(e.position().top-d-c,"px"),e.offset().top+l>window.innerHeight+$(document).scrollTop()){var h=t.parents(".popupmenu-wrapper:first").offset().top;e[0].style.top="".concat(window.innerHeight+$(document).scrollTop()-l-h,"px")}if(e.offset().top-$(document).scrollTop()<0&&(e[0].style.top=0,e[0].style.top="".concat(-1*e.offset().top,"px"),l=a.outerHeight()),e.offset().top+l>window.innerHeight+$(document).scrollTop())l=l-(e.offset().top+l-(window.innerHeight+$(document).scrollTop()))-32,a[0].style.height="".concat(l,"px")}t.parent().find(".is-submenu-open").removeClass("is-submenu-open"),t.addClass("is-submenu-open")},highlight:function(t){if(t&&t.length){var e=t.parent();e.parent().children("li").removeClass("is-focused"),e.addClass("is-focused"),t&&t.focus(),e.closest(".header").scrollTop(0)}},select:function(t){var e=this.menu.is(".is-selectable"),i=this.menu.is(".is-multiselectable"),n=this.isInSingleSelectSection(t),s=this.isInMultiselectSection(t),a=t.parent(),o=[t];return a.hasClass("is-selectable")&&!t.find(".icon").length||e||n?a.hasClass("is-checked")?(a.removeClass("is-checked"),o.push("deselected"),o):(a.prevUntil(".heading, .separator").add(a.nextUntil(".heading, .separator")).removeClass("is-checked"),a.addClass("is-checked"),o.push("selected"),o):a.hasClass("is-multiselectable")||i||s?a.hasClass("is-checked")?(a.removeClass("is-checked"),o.push("deselected"),o):(a.addClass("is-checked"),o.push("selected"),o):o},getSelected:function(){return this.element.is(".popupmenu")&&this.element.is(".is-selectable, .is-multiselectable")?this.element.children(".is-checked").children("a"):this.menu.is(".is-selectable, .is-multiselectable")?this.menu.children(".is-checked").children("a"):$()},isSelectable:function(t){return $(t).is(".is-selectable, .is-multiselectable")||this.hasSelectableSeparator(t)||this.menu.hasClass("is-selectable")||this.menu.hasClass("is-multiselectable")},getSelectableSeparator:function(t){return $(t).prevAll(".separator.single-selectable-section, .separator.multi-selectable-section").first()},hasSelectableSeparator:function(t){var e=this.getSelectableSeparator(t);return e&&e.length},getAdjacentSelectables:function(t){var e=this.getSelectableSeparator(t),i=":not(.heading):not(.separator):not(.submenu)";return e&&e.length?$(e).nextUntil(".separator").filter(i):$(t).parent().children("li".concat(i))},isInSelectableSection:function(t){V("hasSelectableSeparator()","isInSelectableSection()");var e=t.parent().prevAll().filter(".separator").first();return e.hasClass("multi-selectable-section")||e.hasClass("single-selectable-section")},isInSingleSelectSection:function(t){return t.parent().prevAll().filter(".separator").first().hasClass("single-selectable-section")},isInMultiselectSection:function(t){return t.parent().prevAll().filter(".separator").first().hasClass("multi-selectable-section")},detach:function(){var t=this;$(document).off("touchend.popupmenu.".concat(this.id," click.popupmenu.").concat(this.id," keydown.popupmenu")),$(window).off("scroll.popupmenu orientationchange.popupmenu"),$(".datagrid-wrapper").off("scroll.popupmenu"),$("body").off("resize.popupmenu"),$(".scrollable").off("scroll.popupmenu"),this.menu&&this.menu.length&&this.menu.off("click.popupmenu touchend.popupmenu touchcancel.popupmenu dragstart.popupmenu"),setTimeout((function(){t.menu&&t.menu.length&&t.menu.off("click.btn-menu.".concat(t.id))}),1),$("iframe").each((function(){var t=$(this);try{t.contents().find("body").off("click.popupmenu touchend.popupmenu touchcancel.popupmenu")}catch(t){}}))},close:function(t,e){if(t&&void 0!==t||(t=!1),this.menu&&this.isOpen){var i=this.menu.parent(".popupmenu-wrapper"),n=this.menu.find(".popupmenu");if(this.menu.removeClass("is-open").attr("aria-hidden","true"),this.menu[0]&&(this.menu[0].style.height="",this.menu[0].style.width=""),i[0]&&(i[0].style.left="-999px",i[0].style.height="",i[0].style.width=""),this.menu.find(".submenu").off(["mouseenter.popupmenu","mouseleave.popupmenu"].join(" ")),this.menu.find(".is-submenu-open").removeClass("is-submenu-open"),this.submenuOpenTimeout&&(clearTimeout(this.submenuOpenTimeout),delete this.submenuOpenTimeout),n[0]&&(n[0].style.left="",n[0].style.top="",n[0].style.height="",n[0].style.width=""),this.menu.find(".is-focused").removeClass("is-focused"),$(document).off(["keydown.popupmenu.".concat(this.id),"click.popupmenu.".concat(this.id),"mousemove.popupmenu.".concat(this.id),"touchend.popupmenu.".concat(self.id)].join(" ")),this.menu.off(["click.popupmenu","touchend.popupmenu","touchcancel.popupmenu","mouseenter.popupmenu","mouseleave.popupmenu"].join(" ")),this.menu.find("li").off("click.popupmenu"),delete this.keydownThenClick,delete this.holdingDownClick,this.element.is("ul.popupmenu")&&this.element.closest(".popupmenu-wrapper").prev("button").removeClass("is-open"),this.menu.find(".popupmenu.is-open").removeClass("is-open"),this.element.removeClass("is-open").triggerHandler("close",[t]),this.detach(),"immediate"===this.settings.trigger&&this.destroy(),"INPUT"===this.element[0].tagName&&this.openedWithTouch)return this.element.removeClass("longpress-target"),void delete this.openedWithTouch;delete this.openedWithTouch;var s=!1;if(this.element.is(".colorpicker-editor-button")){var a=this.element.data("toolbaritem")||this.element.data("toolbarflexitem"),o=a?a.toolbarAPI:null;o&&o.overflowedItems.forEach((function(t){"colorpicker"===t.type&&(s=!0)}))}e||!this.settings.returnFocus||x.features.touch||s||("function"!=typeof this.settings.returnFocus?document.activeElement&&"INPUT"===document.activeElement.tagName||this.element&&this.element.focus():this.settings.returnFocus(this,{triggerElement:this.element[0],menuElement:this.menu[0]}))}},teardown:function(){var t,e=this;if(!this.menu||!this.menu.length||!this.menu.parent())return this;var i=this.menu.parent(".popupmenu-wrapper");this.ajaxContent&&this.ajaxContent.off().remove(),this.predefinedItems=$();var n=null===(t=this.menu[0])||void 0===t||null==t?void 0:t.id,s=$('[aria-controls="'.concat(n,'"]')).not(this.element);if(!s.length||"immediate"===this.settings.trigger){var a=this.menu.parent();a.find(".arrow").remove(),a.off("contextmenu.popupmenu"),this.element.hasClass("btn-actions")&&a.removeClass("bottom"),this.preExistingWrapper||!this.menu.parent().is(".popupmenu-wrapper")||s.length||this.menu.unwrap();var o=this.element.parent().children(".searchfield"),r=this.menu.data("containment");o.length?this.menu.insertAfter(o.first()):this.menuOrgContainment?this.insertAtContainment(this.menu):r?this.insertAtContainment(this.menu,r):this.menu.insertAfter(this.element),this.menu.find(".submenu").children("a").each((function(t,e){var i=$(e).find("span").text()||$(e).text();$(e).find("span, svg").remove(),$(e).text(i)})),this.menu.find(".submenu").removeClass("submenu"),this.menu.find(".popupmenu").each((function(){var t;(t=$(this)).parent().is(".popupmenu-wrapper, .wrapper")&&t.unwrap()})),e.wrapperPlace&&(e.wrapperPlace.destroy(),delete e.wrapperPlace),i.off().remove()}return this.menu&&this.menu.length&&this.menu.data("trigger")&&$.removeData(this.menu[0],"trigger"),this.detach(),this.element.removeAttr("aria-controls").removeAttr("aria-haspopup").off("touchend.popupmenu touchcancel.popupmenu click.popupmenu keydown.popupmenu keypress.popupmenu contextmenu.popupmenu updated.popupmenu"),this},updated:function(t){return this.settings=E.mergeSettings(this.element[0],t,this.settings),this.teardown().init()},destroy:function(){var t;this.menu&&(this.close(),this.teardown(),null===(t=this.menu)||void 0===t||null==t||t.triggerHandler("destroy"),this.settings.removeOnDestroy&&this.menu&&this.menu.length&&(this.menu.off().remove(),delete this.menu),$.removeData(this.element[0],ge))}},$.fn.popupmenu=function(t){return this.each((function(){var e=$.data(this,ge);e?e.updated(t):e=$.data(this,ge,new me(this,t))}))};var ve="about",be={appName:"Infor Application Name",content:void 0,copyrightYear:(new Date).getFullYear(),deviceSpecs:!0,productName:void 0,useDefaultCopyright:!0,version:void 0};function ye(t,e){this.settings=E.mergeSettings(t,e,be),this.element=$(t),this.init()}ye.prototype={init:function(){return this.setup().build().handleEvents()},setup:function(){var t=this.element.attr("data-appname");this.isBody=$(this.element).is("body"),this.settings.appName=void 0!==t?t.toString():this.settings.appName;var e=this.element.attr("data-about-content");this.settings.content=void 0!==e?e.toString():this.settings.content;var i=this.element.attr("data-copyright-year");this.settings.copyrightYear=void 0!==i?i.toString():this.settings.copyrightYear;var n=this.element.attr("data-device-specs");n&&(this.settings.deviceSpecs="true"===n);var s=this.element.attr("data-product-name");this.settings.productName=void 0!==s?s.toString():this.settings.productName;var a=this.element.attr("data-use-default");void 0!==a&&(this.settings.useDefaultCopyright="true"===a);var o=this.element.attr("data-version");return this.settings.version=void 0!==o?o.toString():this.settings.version,this.defaultCopyright="".concat(H.translate("AboutText"),' <a class="hyperlink" href="http://www.infor.com" target="_blank">www.infor.com</a>.'),this.defaultCopyright=this.defaultCopyright.replace("{0}",this.settings.copyrightYear),this},build:function(){this.modal=$('<div class="modal about" id="about-modal"></div>'),$('<div class="modal-content"></div>').appendTo(this.modal);var t=$('<div class="modal-header"></div>').appendTo(this.modal.find(".modal-content"));$('<div class="close-container"></div>').append($('<button name="close" type="button" class="btn-icon btn-close hide-focus"></button>').append($.createIconElement({icon:"close",classes:"icon-close"})).append("<span>".concat(H.translate("Close"),"'</span>"))).appendTo(t),$.createIconElement({icon:"logo",classes:["icon","about-logo"]}).attr({viewBox:"0 0 44 44"}).appendTo(t),this.title=$('<h1 class="title"></h1>').text(this.settings.appName).appendTo(this.modal.find(".modal-header"));var e=$('<div class="modal-body"></div>').appendTo(this.modal.find(".modal-content"));if(this.settings.version||this.settings.productName){var i=this.settings.productName?"".concat(this.settings.productName," ").concat(this.settings.version):"".concat(this.settings.version);$("<p></p>").text(i).appendTo(e)}if(this.settings.content&&$('<div class="additional-content"></div>').html(this.settings.content).appendTo(e),!this.settings.useDefaultCopyright&&this.settings.content||$("<p></p>").html(this.defaultCopyright).appendTo(e),this.settings.deviceSpecs){var n="fr-FR"===H.currentLocale.name||"fr-CA"===H.currentLocale.name?" :":":",s=this.getDeviceSpecs(),a='<span class="ds">'.concat(H.translate("OperatingSystem")).concat(n,"  ").concat(x.devicespecs.os.replace(x.devicespecs.currentOSVersion,"")," ").concat(x.devicespecs.currentOSVersion,'</span><br>\n        <span class="platform">').concat(H.translate("Platform")).concat(n," ").concat(s.os,'</span><br>\n        <span class="ds">').concat(H.translate("Mobile")).concat(n," ").concat(x.devicespecs.isMobile,'</span><br>\n        <span class="locale">').concat(H.translate("Locale")).concat(n," ").concat(H.currentLocale.name,'</span><br>\n        <span class="locale">').concat(H.translate("Language")).concat(n," ").concat(H.currentLanguage.name,'</span><br>\n        <span class="browser">').concat(H.translate("Browser")).concat(n," ".concat(x.devicespecs.browserVersionName)," ").concat(x.devicespecs.currentBrowser," (").concat(x.devicespecs.browserVersion,')</span><br>\n        <span class="locale">').concat(H.translate("BrowserLanguage")).concat(n," ").concat(s.locale,'</span><br>\n        <span class="cookiesEnabled">').concat(H.translate("CookiesEnabled")).concat(n," ").concat(s.cookiesEnabled,'</span><br>\n        <span class="version">').concat(H.translate("Version")).concat(n," ").concat($("html").attr("data-sohoxi-version"),"</span><br>");$("<p".concat(H.isRTL()?' dir="rtl"':"","></p>")).html(a).appendTo(e)}return this.buttons=this.modal.find("button"),this.modal.find(".hide-focus").one("blur",(function(){$(this).removeClass("hide-focus")})),this.element.attr("data-modal","about-modal"),$(".modal-body",this.modal)[0].tabIndex=0,this.modal.appendTo("body"),this.modal.modal({trigger:this.isBody?"immediate":"click",attributes:this.settings.attributes}),this.modal.data("modal").aboutAPI=this,this},getDeviceSpecs:function(){var t,e,i,n="Microsoft Internet Explorer"===navigator.appName?navigator.userLanguage:navigator.language;return{browser:(t=navigator.userAgent,e=[],i=t.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i)||[],/trident/i.test(i[1])?(e=/\brv[ :]+(\d+)/g.exec(t)||[],"IE '".concat(e[1])):"Chrome"===i[1]&&null!=(e=t.match(/\b(OPR|Edge)\/(\d+)/))?e.slice(1).join(" ").replace("OPR","Opera"):(i=i[2]?[i[1],i[2]]:[navigator.appName,navigator.appVersion,"-?"],null!==(e=t.match(/version\/(\d+)/i))&&i.splice(1,1,e[1]),i.join(" "))),os:navigator.platform,cookiesEnabled:navigator.cookieEnabled,locale:n}},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this},close:function(){if(this.isBody)this.destroy();else{this.element.trigger("close.about");var t=this.modal.data("modal");t&&t.close(),this.element.trigger("afterclose.about")}},destroy:function(t){if(this.buttons.off(),this.element.off("open.about"),!0!==t){var e=this.modal.data("modal");e&&(e.element.off("beforeopen.about"),e.aboutAPI=null,e.destroy())}this.element.length>0&&$.removeData(this.element[0],ve)},handleEvents:function(){var t=this;return this.element.on("open.about",(function(e){e.stopPropagation(),t.element.trigger("click")})),this.buttons.filter('[name="done"], [name="close"]').on("click.about",(function(){t.close()})),this.element.trigger("beforeopen.about"),this.modal.data("modal").element.on("beforeopen.about",(function(){t.modal.find(".modal-body").scrollTop(0)})),this}},$.fn.about=function(t){return this.each((function(){var e=$.data(this,ve);e?e.updated(t):e=$.data(this,ve,new ye(this,t))}))};var we={direction:"vertical",distance:"auto",timing:300,transition:"cubic-bezier(.17, .04, .03, .94)"};$.fn.animateOpen=function(t){var e=$.fn.transitionEndName();return t=E.mergeSettings(void 0,t,we),this.each((function(){var i,n=this,s=$(this),a="horizontal"===t.direction?"width":"height",o=a.charAt(0).toUpperCase()+a.slice(1),r=isNaN(t.distance)?"auto":"".concat(parseInt(t.distance,10),"px");function l(){i&&clearTimeout(i),s.data("ignore-animation-once")&&$.removeData(s[0],"ignore-animation-once"),s.data("is-animating")&&$.removeData(s[0],"is-animating"),s.off("".concat(e,".animateopen")),n.style.transition="",n.style[a]=r,s.trigger("animateopencomplete")}s.one("animateopenstart.animation",(function(t){t.stopPropagation(),s.off("".concat(e,".animateopen"))})),s.trigger("animateopenstart"),e&&s.one("".concat(e,".animateopen"),l),s.data("is-animating",!0);var c=this.style[a];this.style[a]=r;var d=getComputedStyle(this)[a];this.style[a]=c,this["offset".concat(o)],this.style.transition="".concat(a," ").concat(t.timing,"ms ").concat(t.transition),i=setTimeout(l,t.timing),this.style[a]=d,(s.is(":hidden")||s.is(".no-transition")||s.data("ignore-animation-once"))&&l()}))};var xe={direction:"vertical",timing:300,transition:"cubic-bezier(.17, .04, .03, .94)"};$.fn.animateClosed=function(t){var e=$.fn.transitionEndName();return t=E.mergeSettings(void 0,t,xe),this.each((function(){var i,n=this,s=$(this),a="horizontal"===t.direction?"width":"height",o=a.charAt(0).toUpperCase()+a.slice(1);function r(){i&&clearTimeout(i),s.data("ignore-animation-once")&&$.removeData(s[0],"ignore-animation-once"),s.data("is-animating")&&$.removeData(s[0],"is-animating"),s.off("".concat(e,".animatedclosed")),n.style.transition="",n.style[a]="0px",s.trigger("animateclosedcomplete")}s.one("animateclosedstart",(function(t){t.stopPropagation(),s.off("".concat(e,".animatedclosed"))})),s.trigger("animateclosedstart"),e&&s.one("".concat(e,".animatedclosed"),r),s.data("is-animating",!0),this.style[a]=getComputedStyle(this)[a],this["offset".concat(o)],this.style.transition="".concat(a," ").concat(t.timing,"ms ").concat(t.transition),i=setTimeout(r,t.timing),this.style[a]="0px",(s.is(":hidden")||s.is(".no-transition")||s.data("ignore-animation-once"))&&r()}))},$.fn.isAnimating=function(){return this.each((function(){return!0===$(this).data("is-animating")}))};var Ce="accordion",ke=["classic","plus-minus","chevron"],Te={accordionFocusCallback:function(t,e){return e()},allowOnePane:!0,expanderDisplay:ke[0],enableTooltips:!0,rerouteOnLinkClick:!0,notificationBadge:!1,source:null,tooltipStyle:"light"};function Ae(t){return void 0!==t.displayChevron&&(V("expanderDisplay setting","displayChevron setting"),!1===t.displayChevron?t.expanderDisplay=ke[1]:t.expanderDisplay=ke[0],delete t.displayChevron),t}function Se(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Te),this.settings=Ae(this.settings),this.init()}Se.prototype={init:function(t){this.build(t).handleEvents(t).setupTooltips()},build:function(t,e){var i,n,s,a=this,o=this,r=!0;t&&t instanceof jQuery?(i=t.children("a"),s=(n=t.next(".accordion-pane")).children(".accordion-content"),r=!1,this.headers=this.headers.add(t),this.anchors=this.anchors.add(i),this.panes=this.panes.add(n),this.contentAreas=this.contentAreas.add(s)):(this.headers=this.element.find(".accordion-header"),t=this.element.find(".accordion-header"),this.anchors=t.children("a"),i=this.anchors,this.panes=t.next(".accordion-pane"),n=this.panes,this.contentAreas=n.children(".accordion-content"),s=this.contentAreas);var l=$(".accordion-content").find("textarea");l&&l.parents(".accordion-pane").attr("style","display: block");var c=!1;if(t.each((function(){var t=$(this),e=!1,i=t.parent(),n=i.is(".accordion");function s(){n||(e?i.addClass("has-icons"):t.addClass("no-icon"))}i.is(".accordion-section")&&(i=i.parentsUntil(".accordion"),n=!0),t.children("a").find("svg").detach().insertBefore(t.children("a"));var a=t.children(".icon, svg");if(a.addClass("icon").attr({role:"presentation","aria-hidden":"true",focusable:"false"}),n&&a.length&&(c=!0),(t.is(".list-item")||!n&&t.find("button").length||!n&&t.find("svg").length)&&(e=!0),t.hasClass("is-disabled")&&t.children("a, button").attr("tabindex","-1"),t.next(".accordion-pane").length){e=!0;var r=t.children(".btn");if(!r.length){r=$('<button class="btn" type="button" tabindex="-1"></button>');var l="insertBefore";"plus-minus"!==o.settings.expanderDisplay&&n&&(t.addClass("has-chevron"),l="insertAfter"),r[l](t.children("a")),t.data("addedExpander",r)}a.length&&"plus-minus"===o.settings.expanderDisplay&&a.appendTo(r);var d=r.children(".icon, .svg, .plus-minus");if(!d.length){if("classic"===o.settings.expanderDisplay&&n||"chevron"===o.settings.expanderDisplay)d=$.createIconElement({icon:"caret-down",classes:["chevron"]});else{var h=o.isExpanded(t)?" active":"";d=$('<span class="icon plus-minus'.concat(h,'" aria-hidden="true" role="presentation"></span>'))}d.appendTo(r)}var u={role:"presentation","aria-hidden":"true"};d.is("span")||(u.focusable="false"),d.attr(u),d.is(".chevron")&&n?(t.addClass("has-chevron"),r.insertAfter(t.children("a"))):(t.removeClass("has-chevron"),r.insertBefore(t.children("a"))),"plus-minus"===o.settings.expanderDisplay&&n&&(c=!0),s()}else s()})),c&&this.element.addClass("has-icons"),r&&!this.element.data("updating")){var d=t.filter(".is-selected, .is-expanded");d.next(".accordion-pane").addClass("no-transition"),this.settings.allowOnePane?(d=d.first(),this.expand(d)):d.each((function(t){a.expand($(d[t]))})),this.select(d.last()),d.next(".accordion-pane").removeClass("no-transition")}return n.each((function(){var t=$(this),i=t.prev(".accordion-header");i.children("a").attr({"aria-haspopup":"true",role:"button"});var n=t.children(),s=!0;if(n.each((function(t,e){$(e).is(".accordion-header, .accordion-content")&&!$(e).hasClass("filtered")&&(s=!1)})),t[s?"addClass":"removeClass"]("all-children-filtered"),s&&(t.data("ignore-animation-once",!0),o.collapse(i,!1)),!e){var a="0px";o.isExpanded(i)&&(a="auto"),t.attr("style","height: ".concat(a))}})),e||(this.currentlyFiltered=$()),this.createNotificationBadge(),this},createNotificationBadge:function(){this.settings.notificationBadge&&this.element.find(".accordion-header").each((function(t,e){var i=$(e);if(i.children(".notification-badge-container").length<1){var n=i.data();i.notificationbadge({position:n.options.position,color:n.options.color});var s=i.find(".icon"),a=i.find(".notification-badge-container");a.prepend(s),i.prepend(a)}}))},handleHeaderClick:function(t,e){if(!e||!e.length||this.isDisabled(e)||this.isFiltered(e)||e.data("is-animating"))t.preventDefault();else{if(!$(t.target).is('[class^="btn"]')){var i=e.children("a");return this.handleAnchorClick(t,i)}t.preventDefault()}},handleAnchorClick:function(t,e){var i,n=this,s=e.parent(".accordion-header"),a=s.next(".accordion-pane"),o=e.attr("ng-reflect-href");if(!t||o||s.is(".module-nav-settings-btn")||t.preventDefault(),!s.length||this.isDisabled(s)||this.isFiltered(s))return!1;if(!1!==this.element.triggerHandler("beforeselect",[e]))return this.originalSelection=e,this.select(e),s.is(".module-nav-settings-btn")||this.closePopups(t),!(i=e.attr("href"))||""===i||"#"===i||n.settings.rerouteOnLinkClick&&(window.location.href=i,0)?a.length?(n.toggle(s),n.focusOriginalType(s),!0):(this.isControllingDetails?a.length||n.element.trigger("drilldown",[s[0]]):(n.orignalSelection=e,n.focusOriginalType(s)),this.element.trigger("selected",s),!0):(this.element.trigger("followlink",[e]),!0)},closePopups:function(t){$(".popupmenu.is-open").length&&this.element.find('.accordion-header[aria-haspopup="true"]').each((function(){return $(this).data("popupmenu").close(),void 0!==t&&(t.stopPropagation(),t.stopImmediatePropagation()),!1}))},handleExpanderClick:function(t,e){var i=e.parent(".accordion-header");if(i.length&&!this.isDisabled(i)&&!this.isFiltered(i)&&!i.data("is-animating"))return this.originalSelection=e,t&&t.stopPropagation(),this.closePopups(),i.next(".accordion-pane").length?(this.toggle(i),this.select(i),void this.focusOriginalType(i)):this.handleAnchorClick(t,i.children("a"))},handleKeys:function(t){var e=this,i=t.which,n=$(t.target),s=n.parent(),a=s.children('[class^="btn"]').first(),o=s.children("a"),r=document.querySelectorAll(".accordion-pane");function l(t){t||(t=n),e.originalSelection||(e.originalSelection=t)}if(9===i){this.headers.removeClass("is-selected");var c=u(r).filter((function(t){return!t.classList.contains("is-expanded")}));u(r).filter((function(t){return t.classList.contains("is-expanded")})).forEach((function(t){u(t.children).forEach((function(t){$(t).hasClass("is-disabled")||($(t).find("a").removeAttr("tabindex"),$(t).find("button").attr("tabindex","-1"))}))})),c.forEach((function(t){u(t.children).forEach((function(t){$(t).find("a").attr("tabindex","-1"),$(t).find("button").attr("tabindex","-1")}))})),n.is("a")&&a.length?l(a):l(o)}if(32===i){if(t.preventDefault(),n.is('[class^="btn"]')&&(t.stopPropagation(),t.stopImmediatePropagation(),$("html").hasClass("is-firefox")))return;return a.length?(l(a),this.handleExpanderClick(null,n)):(l(o),this.handleAnchorClick(null,n))}return 37===i||38===i?(t.preventDefault(),l(),t.shiftKey?this.ascend(s):this.prevHeader(s)):39===i||40===i?(t.preventDefault(),l(),t.shiftKey?this.descend(s):this.nextHeader(s)):void 0},toData:function(t,e){var i=[],n=this.element.children(".accordion-header");function s(i,n,a,o){var r=$(i),l=$(i).next(".accordion-pane"),c=i.classList.contains("accordion-content"),d={index:"".concat(void 0!==a?"".concat(a,"."):"").concat(n),type:c?"content":"header"};e&&(d.element=i),i.getAttribute("id")&&(d.id=i.getAttribute("id")),c?(d.content="".concat(r.html()),d.contentText="".concat(r.text().trim().replace(/\n|\s{2,}/g," "))):d.text=r.children("a, span").text().trim();var h=r.children(".icon");if(h.length&&(d.icon="svg"===h[0].tagName.toLowerCase()?h[0].getElementsByTagName("use")[0].getAttribute("href"):""),r.hasClass("is-disabled")&&(d.disabled=!0),l.length){var u=l.children(".accordion-header, .accordion-content"),p=[];if(u.length){var g=p;t&&(g=o),u.each((function(t,e){s(e,t,d.index,g)})),d.children=p}}o.push(d)}n.length||this.element.find(".accordion-section").each((function(n,a){var o=function(i,n,a){var o={index:"".concat(void 0!==a?"".concat(a,"."):"").concat(n),type:"section"};e&&(o.element=i),i.getAttribute("id")&&(o.id=i.getAttribute("id"));var r=$(i).children(".accordion-header"),l=[];return r.length&&r.each((function(t,e){s(e,t,n,l)})),o.children=l,t?l:o}(a,n,void 0);t?i=i.concat(o):i.push(o)}));return n.each((function(t,e){s(e,t,void 0,i)})),i},select:function(t){if(t&&t.length){var e=t,i=e.parent();if(t.is(".accordion-header")&&(e=(i=t).children("a")),e.is('[class^="btn"]')&&(e=t.next("a")),!this.isDisabled(i)&&!this.isFiltered(i))this.deselectAll(),i.addClass("is-selected"),i.parentsUntil(this.element,".accordion-pane").prev(".accordion-header").addClass("child-selected")}},deselect:function(t){if(t&&t.length){var e=t,i=e.parent();t.is(".accordion-header")&&(e=(i=t).children("a")),e.is('[class^="btn"]')&&(e=t.next("a")),i.removeClass("is-selected"),i.parentsUntil(this.element,".accordion-pane").prev(".accordion-header").each((function(t,e){$(e).find(".is-selected").length||$(e).removeClass("child-selected")}))}},deselectAll:function(){var t;null===(t=this.headers)||void 0===t||null==t||t.removeClass("child-selected").removeClass("is-selected")},getSelected:function(){return this.element.find(".is-selected")},isDisabled:function(t){return!!this.element.hasClass("is-disabled")||!!t&&t.hasClass("is-disabled")},isFiltered:function(t){if(!t||!t.length)return!1;var e=t[0].classList;return e.contains("filtered")&&!e.contains("has-filtered-children")},isExpanded:function(t){if(t&&t instanceof Element&&(t=$(t)),t&&t.length)return"true"===t.children("a").attr("aria-expanded")||t.hasClass("is-expanded")},toggle:function(t){var e=this;!t||!t.length||this.isDisabled(t)||this.isFiltered(t)||this.isAnimating||(this.isAnimating=!0,setTimeout((function(){e.isAnimating=!1}),500),this.isExpanded(t)?this.collapse(t):this.expand(t))},expand:function(t,e){if("string"==typeof t&&(t=this.element.find("#".concat(t)).first()),t&&t.length){var i=this,n=t.next(".accordion-pane"),s=t.children("a"),a=$.Deferred();return!1===this.element.triggerHandler("beforeexpand",[s])?a.reject():(this.callSource(s,o)||o.apply(this),a)}function o(){if(!(n=t.next(".accordion-pane"))||!n.length)return a.reject();var o=t.children(".btn");o.length&&(o.children(".plus-minus, .chevron").addClass("active"),o.children(".audible").text(H.translate("Collapse")));var r=t.parentsUntil(i.element).filter(".accordion-pane").prev(".accordion-header").add(t),l=[];i.settings.allowOnePane&&!e&&i.headers&&i.headers.not(r).each((function(){var t=$(this);i.isExpanded(t)&&l.push(i.collapse(t))}));var c=[];function d(t){var e;t&&t.stopPropagation(),(e=$).when.apply(e,c.concat(l)).done((function(){a.resolve()})),n.triggerHandler("afterexpand",[s]),n.css("height","auto"),i.element.trigger("afterexpand",[s])}r&&r.not(t).each((function(){var t=$(this);i.isExpanded(t)||c.push(i.expand(t))})),t.add(n).addClass("is-expanded"),t.children("a").attr("aria-expanded","true"),i.element.trigger("expand",[s]),n.hasClass("no-transition")||this.element.hasClass("no-transition")?d():n.one("animateopencomplete",d).animateOpen()}},expandAll:function(){var t;if(!0!==this.settings.allowOnePane){var e=this,i=$.Deferred(),n=[];return this.headers.each((function(){var t=$(this);e.isExpanded(t)||n.push(e.expand(t))})),(t=$).when.apply(t,n).always((function(){i.resolve()})),i}},collapse:function(t){var e=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];if("string"==typeof t&&(t=this.element.find("#".concat(t)).first()),t&&t.length){var i=this,n=t.next(".accordion-pane"),s=t.children("a"),a=$.Deferred();if(!1===this.element.triggerHandler("beforecollapse",[s]))return a.reject();var o=t.children(".btn");return o.length&&(o.children(".plus-minus, .chevron").removeClass("active"),o.children(".audible").text(H.translate("Expand"))),e&&n.closeChildren&&n.closeChildren(),i.element.trigger("collapse",[s]),n.hasClass("no-transition")||this.element.hasClass("no-transition")?r():n.one("animateclosedcomplete",r).animateClosed(),a}function r(e){e&&e.stopPropagation(),t.add(n).removeClass("is-expanded"),s.attr("aria-expanded","false"),n.triggerHandler("aftercollapse",[s]),i.element.trigger("aftercollapse",[s]),a.resolve()}},collapseAll:function(){var t,e=this,i=$.Deferred(),n=[];return this.headers.each((function(){var t=$(this);e.isExpanded(t)&&n.push(e.collapse(t))})),(t=$).when.apply(t,n).always((function(){i.resolve()})),i},callSource:function(t,e){if(!this.settings.source||"function"!=typeof this.settings.source)return!1;var i=this,n=t.parent(),s=n.next(".accordion-pane"),a={anchor:t,header:n,pane:s};return this.settings.source(a,(function(t,n){!0!==t&&i.updated(n),setTimeout((function(){e.apply(i)}),1)}))},getElements:function(t){var e,i,n,s,a=$(t),o=null;return a.is(".accordion-header")&&(e=a,n=a.children('[class^="btn"]'),i=a.children("a")),a.is(".btn")&&(i=(e=(n=a).parent()).children("a")),a.is("a")&&(n=(e=(i=a).parent()).children(".btn")),s=e.next(".accordion-pane"),e.parent(".accordion-section")&&(o=e.parent(".accordion-section")),{header:e,expander:n,anchor:i,pane:s,section:o}},prevHeader:function(t,e){var i=this.getElements(t),n=i.header.parent().children(),s=n.index(i.header),a=$(n.get(g.ensureAlphaNumeric(s)-1));if(!n.length||0===s){if(i.header.parent(".accordion-pane").length)return this.ascend(i.header);if(i.header.is(":first-child")&&i.section){var o,r=i.section.prev(".accordion-section");(r.length||(r=i.section.parent().children().last(".accordion-section")).length)&&(o=r.children(),a=$(o[o.length-1]))}a.length||(a=n.last())}for(;a.is(".accordion-content")||this.isDisabled(a)||this.isFiltered(a);){if(a.is(":only-child")||a.is(":first-child"))return this.ascend(i.header);a=a.prev()}if(a.is(".accordion-pane")){var l=a.prev(".accordion-header");if(this.isExpanded(l))if(l.next(".accordion-pane").children(":not(.accordion-content)").length&&!e)return this.descend(l,-1);if(!(a=l).length){var c;if(i.header.parent(".accordion-pane").length)return this.ascend(i.header);var d=null===(c=i.header.parent(".accordion-section"))||void 0===c||null==c?void 0:c.prev(".accordion-section");for(d.length&&(a=$(d.children()[0])),a.length||(a=n.last());a.is(".accordion-content")||this.isDisabled(a)||this.isFiltered(a);)a=a.prev()}}this.focusOriginalType(a)},nextHeader:function(t,e){var i=this.getElements(t),n=i.header.parent().children(),s=n.index(i.header),a=$(n.get(g.ensureAlphaNumeric(s)+1));if(!n.length||s===n.length-1){if(i.header.parent(".accordion-pane").length)return this.ascend(i.header,-1);if(i.header.is(":last-child")&&i.section){var o=i.section.next(".accordion-section");(o.length||(o=i.section.parent().children().first(".accordion-section")).length)&&(a=$(o.children()[0]))}a.length||(a=n.first())}for(;a.is(".accordion-content")||this.isDisabled(a)||this.isFiltered(a);){if(a.is(":only-child")||a.is(":last-child"))return this.ascend(i.header);a=a.next()}if(a.is(".accordion-pane")){var r=a.prev(".accordion-header");if(this.isExpanded(r))if(r.next(".accordion-pane").children(":not(.accordion-content)").length&&!e)return this.descend(r);if(!(a=$(n.get(g.ensureAlphaNumeric(s)+2))).length){var l;if(i.header.parent(".accordion-pane").length)return this.ascend(i.header,-1);var c=null===(l=i.header.parent(".accordion-section"))||void 0===l||null==l?void 0:l.next(".accordion-section");for(c.length&&(a=$(c.children()[0])),a.length||(a=n.first());a.is(".accordion-content")||this.isDisabled(a)||this.isFiltered(a);)a=a.next()}}this.focusOriginalType(a)},ascend:function(t,e){e||(e=0);var i=t.parent(".accordion-pane"),n=i.prev();if(-1===e&&!(n=i.next(".accordion-header")).length)return i.parent(".accordion").length?this.nextHeader(i.prev().children("a"),!0):this.ascend(i.prev(),-1);this.focusOriginalType(n)},descend:function(t,e){e||(e=0);var i=t.next(".accordion-pane"),n=i.children(".accordion-header").first();return-1===e&&(n=i.children(".accordion-header").last()),n.length?this.isExpanded(n)?this.descend(n,-1):void this.focusOriginalType(n):this.focusOriginalType(t)},focusOriginalType:function(t){var e=this,i=function(){var i=t.children('[class*="btn"]');e.headers.not(t).removeClass("is-focused"),e.originalSelection.is('[class*="btn"]')&&i.length?i.first()[0].focus():t.is(".searchfield-wrapper")?(t.find("input")[0].focus(),t.addClass("is-focused")):(t.children("a")[0].focus(),t.addClass("is-focused"))};"function"!=typeof this.settings.accordionFocusCallback?i():this.settings.accordionFocusCallback(t,i)},filter:function(t){var e,i=this;if(t){var n=this,s=this.headers.parents().find(".module-nav-accordion");null===(e=this.headers)||void 0===e||null==e||e.removeClass("filtered has-filtered-children hide-focus"),this.panes.removeClass("all-children-filtered no-transition"),s.find(".empty-message").remove(),s.find(".module-nav-separator").show(),this.contentAreas.removeClass("filtered"),this.currentlyFiltered=$();var a=t.not(this.currentlyFiltered),o=[],r=$();this.headers.add(this.contentAreas).not(a).not(".module-nav-settings-btn").addClass("filtered");var l=this.headers.parents().find(".module-nav-footer");if(l)if(1===l.closest(".pinned-optional").length){var c=this.headers.parents().find(".module-nav-footer");c.find(".filtered").removeClass("filtered").addClass("pinned"),c.prev(".module-nav-separator").addClass("pinned")}a.each((function(t,e){var i=$(e).is(".accordion-content"),n=$(e).parents(".accordion-pane");i&&(r.push($(e)),$(n[0]).prev(".accordion-header").each((function(t,e){return o.indexOf(e)<0&&o.push(e),o})));n.length&&n.prev(".accordion-header").each((function(t,e){return o.indexOf(e)<0&&o.push(e),o}))}));var d=$(o);d.addClass("has-filtered-children");var h=this.expand(d,!0);$.when(h).done((function(){i.currentlyFiltered=a,n.build(void 0,!0)})),s.find(".module-nav-separator").not(".pinned").hide(),1===s.length&&this.headers.not(".module-nav-settings-btn").not(".pinned").length===this.headers.not(".module-nav-settings-btn").not(".pinned").filter(".filtered").length&&s.find(".module-nav-search-container").append('<div class="empty-message">\n              <div class="card-empty-icon">\n          <svg class="icon-empty-state is-slate" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-empty-search-data-new"></use>\n          </svg>\n        </div>\n        <div class="card-empty-title">\n          '.concat(H.translate("NoResultsFound"),"\n        </div>\n      </div>"))}},unfilter:function(t,e){var i,n=this;if(this.currentlyFiltered.length||e){t&&t.length||(t=this.currentlyFiltered);var s=[];null===(i=this.headers)||void 0===i||null==i||i.removeClass("filtered has-filtered-children hide-focus"),this.panes.removeClass("all-children-filtered no-transition"),this.contentAreas.removeClass("filtered");var a=this.headers.parents().find(".module-nav-accordion");a.find(".empty-message").remove(),a.find(".module-nav-separator").show(),t.each((function(t,e){var i=$(e).parents(".accordion-pane");i.length&&i.prev(".accordion-header").each((function(t,e){return s.indexOf(e)<0&&s.push(e),s}))}));var o=$(s);o.removeClass("has-filtered-children");var r=[this.collapse(t),this.collapse(o)];$.when(r).done((function(){n.currentlyFiltered=n.currentlyFiltered.not(t)}))}},disable:function(){this.element.addClass("is-disabled"),this.anchors.add(this.headers.children('[class^="btn"]')).attr("tabindex","-1")},enable:function(){this.element.removeClass("is-disabled"),this.anchors.add(this.headers.children('[class^="btn"]')).removeAttr("tabindex")},updated:function(t,e){this.element.data("updating",!0),e&&(this.settings=E.mergeSettings(this.element[0],e,this.settings),this.settings=Ae(this.settings));var i=$(document.activeElement);return $.contains(this.element[0],i[0])||(i=void 0),t&&t instanceof jQuery?this.teardown(t).init(t):this.teardown().init(),i&&i.length&&i.focus(),$.removeData(this.element[0],"updating"),this},teardown:function(t){var e=!1,i=t;(this.currentlyFiltered&&this.unfilter(this.currentlyFiltered),t&&t instanceof jQuery||(i=this.headers,e=!0),i&&i.length)&&(i.off("touchend.accordion click.accordion focusin.accordion focusout.accordion keydown.accordion mousedown.accordion mouseup.accordion").each((function(){var t=$(this),e=t.children(".icon"),i=t.data("hidefocus");if(i&&i.destroy(),e.length){var n=e.data("icon");n&&n.destroy()}var s=t.data("addedExpander");s&&(s.remove(),$.removeData(this,"addedExpander"))})),i.not(".accordion-content").find("a").off("touchend.accordion keydown.accordion click.accordion"),i.children('[class^="btn"]').off("touchend.accordion click.accordion keydown.accordion"));return e&&(this.element.off("updated.accordion selected.accordion"),delete this.anchors,delete this.headers,delete this.panes,delete this.contentAreas),this},destroy:function(){this.teardown(),$.removeData(this.element[0],"accordion")},handleEvents:function(t){var e=this,i=null,n=t,s=!1;function a(t,i){var n=function(t){var e="Header";return t.is("a")&&(e="Anchor"),t.is("button")&&(e="Expander"),e}(i);return i.is(".module-nav-settings-btn")||$("body").children().not(".application-menu, .modal-page-container, .module-nav, .page-container, .resize-app-menu-container").closeChildren(),e["handle".concat(n,"Click")](t,i)}t&&t instanceof jQuery||(n=this.headers,s=!0);var o=!1;return n.addClass("hide-focus").off("click.accordion focusin.accordion focusout.accordion keydown.accordion mousedown.accordion mouseup.accordion").on("click.accordion",(function(t){return a(t,$(this))})).on("focusin.accordion",(function(t){var i=$(t.target);e.originalSelection||(e.originalSelection=i),o?o=!1:(n.not($(this)).removeClass("is-focused"),i.is(":not(.btn)")&&$(this).addClass("is-focused").removeClass("hide-focus"))})).on("focusout.accordion",(function(){$.contains(this,i)&&!$(this).is($(i))||($(this).removeClass("is-focused"),$(this).addClass("hide-focus"))})).on("keydown.accordion",(function(t){e.handleKeys(t)})).on("mousedown.accordion",(function(t){o=!0,$(this).addClass("is-focused"),i=t.target})).on("mouseup.accordion",(function(){i=null})),n.children('[class^="btn"]').off("click.accordion keydown.accordion").on("click.accordion",(function(t){return a(t,$(this))})).on("keydown.accordion",(function(t){e.handleKeys(t)})),s&&this.element.off("selected.accordion updated.accordion").on("selected.accordion",(function(t){t.stopPropagation()})).on("updated.accordion",(function(t,i){t.stopPropagation(),e.updated(i)})),this},hideTooltip:function(){var t=this,e=window;e.tooltip&&(this.removeTooltipData(e.tooltip),e.tooltip.classList.add("is-hidden"),e.tooltip.classList.remove("content-tooltip")),$("body, .scrollable").off("scroll.spantooltip",(function(){t.hideTooltip()}))},removeTooltipData:function(t){(t=t instanceof jQuery?t:$(t)).data("spantooltip")&&$.removeData(t[0],"spantooltip")},setupTooltips:function(){var t=this;if(this.settings.enableTooltips){var e,i=this,n=".accordion-header a span";this.element.off("mouseenter.spantooltip",n).on("mouseenter.spantooltip",n,(function(){var i;(i=this).style.width="auto",i.offsetWidth>i.parentElement.offsetWidth-parseInt($(i).parent().css("padding-left"),10)&&(i.style.width="",e=setTimeout((function(){$(i).tooltip({trigger:"immediate",content:"".concat(i.innerText),extraClass:"light"===t.settings.tooltipStyle?"tooltip-accordion-style":""})}),400))})).off("mouseleave.spantooltip click.spantooltip",n).on("mouseleave.spantooltip click.spantooltip",n,(function(){var t;(t=this).style.width="auto",t.offsetWidth>t.parentElement.offsetWidth-parseInt($(t).parent().css("padding-left"),10)&&(t.style.width="",i.hideTooltip(),clearTimeout(e))}))}}},$.fn.accordion=function(t){return this.each((function(){var e=$.data(this,Ce);e?e.updated(t):e=$.data(this,Ce,new Se(this,t))}))};var $e="actionsheet",Ee="ids-actionsheet-root",De={actions:[],attributes:[],autoFocus:!0,breakpoint:"phone-to-tablet",displayAsActionSheet:"responsive",overlayOpacity:.7,onSelect:null,onCancel:null,tray:!1,trayOpts:{text:null,icon:null,backgroundColor:"slate"},showCancelButton:!0};function Ie(t,e){return this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,De),this.init()}Ie.prototype={init:function(){this.render(),this.handleEvents()},render:function(){var t=this,e=document.querySelector("#".concat(Ee));(e||this.renderRootElems(),e&&void 0===this.rootElem)&&(document.getElementById("ids-actionsheet-root").remove(),this.renderRootElems());var i=this.element[0];if(i.classList.add("ids-actionsheet-trigger"),E.addAttributes(this.element,this,this.settings.attributes,"trigger"),("BUTTON"===i.tagName||i.className.includes("btn"))&&this.element.button(),!this.actionSheetElem){var n="",s=!1;this.settings.actions.forEach((function(e,i){var a="",o=E.stringAttributes(t,t.settings.attributes,"action-".concat(i));e.icon&&(s=!0,a=$.createIcon({icon:e.icon}));var r='<button class="btn-tertiary ids-action" data-index="'.concat(i,'" ').concat(o,">\n          ").concat(a,'\n          <span class="ids-action-text">').concat(e.text,"</span>\n        </button>");n+=r})),this.settings.showCancelButton&&(n+='<div class="separator"></div>\n        <button class="btn-secondary btn-cancel ids-action">\n          <span>Cancel</span>\n        </button>'),this.actionSheetElem=document.createElement("div"),this.actionSheetElem.setAttribute("role","dialog"),E.addAttributes($(this.actionSheetElem),this,this.settings.attributes,"sheet");var a=this.actionSheetElem.classList;a.add("ids-actionsheet"),s&&a.add("has-icons"),this.actionSheetElem.insertAdjacentHTML("afterbegin",n),this.rootElem.append(this.actionSheetElem),$(this.actionSheetElem).find("button").button(),this.settings.tray&&this.renderTrayElement()}},trayBreakpoint:function(){this.trayContainer.style.visibility=m.isAbove("phone-to-tablet")?"hidden":""},trayTransform:function(){this.trayContainer.style.transform="translate(0, -".concat(this.actionSheetElem.offsetHeight-40,"px)")},renderRootElems:function(){var t=document.createDocumentFragment(),e=document.createElement("div"),i=document.createElement("div");e.id=Ee,e.classList.add("ids-actionsheet-container"),e.setAttribute("aria-hidden",!0),t.appendChild(e),i.classList.add("overlay"),i.setAttribute("aria-hidden",!0),e.appendChild(i),document.body.appendChild(e),this.rootElem=e,this.overlayElem=i},renderTrayElement:function(){var t=document.createElement("div"),e=document.createElement("button"),i=document.createElement("span");this.settings.trayOpts.icon&&(e.innerHTML=$.createIcon({icon:this.settings.trayOpts.icon}));var n=e.firstChild;i.innerHTML=this.settings.trayOpts.text,t.classList.add("ids-actionsheet-tray-container",this.settings.trayOpts.backgroundColor),e.classList.add("ids-actionsheet-tray-btn"),i.classList.add("ids-actionsheet-tray-btn-text"),t.appendChild(e),e.appendChild(i),this.rootElem.insertBefore(t,this.rootElem.childNodes[1]),this.trayContainer=t,this.trayBtn=e,this.trayIconElem=n,this.trayTextElem=i,this.trayBreakpoint()},handleEvents:function(){var t=this;this.element.add($(this.trayBtn)).on("click.trigger",(function(){t.visible||(t.open(),t.trayContainer&&t.trayTransform())})),$("body").on("resize",(function(){t.settings.tray&&(t.trayBreakpoint(),t.visible?t.trayTransform():t.trayContainer.style.removeProperty("transform"))}))},get visible(){return this.popupmenuAPI?this.hasOpenPopupMenu:this.rootElem.classList.contains("engaged")},get actionElems(){return u(this.actionSheetElem.querySelectorAll("button")).filter((function(t){return!t.classList.contains("btn-cancel")}))},open:function(){var t=this;this.visible||(this.currentlyNeedsActionSheet?(this.rootElem.classList.add("engaged"),this.overlayElem.removeAttribute("hidden"),window.requestAnimationFrame((function(){t.setOverlayVisibility(),t.addActionSheetOpenEvents(),t.element[0].removeAttribute("aria-hidden"),t.focus()}))):this.openPopupMenu())},openPopupMenu:function(){var t=this,e=E.stringAttributes(this,this.settings.attributes,"menu"),i="<ul ".concat(e,">");this.settings.actions.forEach((function(t,e){var n="";t.icon&&(n=$.createIcon({icon:t.icon}));var s='<li class="ids-action-menu-item" data-index="'.concat(e,'">\n        <a>\n          ').concat(n,'\n          <span class="ids-action-text">').concat(t.text,"</span>\n        </a>\n      </li>");i+=s})),i+="</ul>";var n=$(i).insertAfter(this.element);this.element.popupmenu({autoFocus:this.settings.autoFocus,attributes:this.settings.attributes,menu:n,offset:{y:10},trigger:"immediate",removeOnDestroy:!0}),this.hasPopupmenuEvents=!0,$(this.element).on("selected.popupmenu",(function(e,i){var n=parseInt($(i.parent()).attr("data-index"),10);if(!isNaN(n)){var s=t.actionSheetElem.children[n];s&&t.doSelect(s,null,!0)}})).on("afteropen.popupmenu",(function(){t.focus()})).on("close.popupmenu",(function(e,i){i?t.doCancel(null,!1,i):t.close()}))},get popupmenuAPI(){return $(this.element).data("popupmenu")},get hasOpenPopupMenu(){var t;return null===(t=this.popupmenuAPI)||void 0===t||null==t?void 0:t.isOpen},close:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"",i=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(this.hasPopupmenuEvents)return this.closePopupMenu(e),void(i&&this.focus());this.visible&&(this.removeActionSheetOpenEvents(),this.element[0].setAttribute("aria-hidden","true"),this.rootElem.classList.remove("engaged"),this.trayContainer&&this.trayContainer.style.removeProperty("transform"),this.setOverlayVisibility(),E.waitForTransitionEnd(this.overlayElem,"opacity").then((function(){t.overlayElem.setAttribute("hidden",""),t.triggerCloseEvent(e),i&&t.focus()})))},closePopupMenu:function(t){this.removePopupmenuEvents(),this.triggerCloseEvent(t)},removePopupmenuEvents:function(){this.hasPopupmenuEvents&&($(this.element).off("selected.popupmenu close.popupmenu afteropen.popupmenu"),delete this.hasPopupmenuEvents)},cancel:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.close("cancel",t)},triggerCloseEvent:function(){var t;t="cancel"===(arguments.length>0&&void 0!==arguments[0]?arguments[0]:"")?jQuery.Event("cancelled"):jQuery.Event("close"),this.element.trigger(t,[$(this.actionSheetElem)])},focus:function(){if(this.settings.autoFocus)if(this.hasOpenPopupMenu){var t=this.popupmenuAPI.menu[0].querySelector("a");t&&(t.focus(),this.popupmenuAPI.highlight($(t)))}else if(this.visible){var e=this.actionSheetElem.querySelector("button");e&&(e.focus(),this.resetFocusState(e))}else this.element[0].focus()},get currentlyNeedsActionSheet(){var t=this.settings.breakpoint,e=this.settings.displayAsActionSheet;return"always"===e||"responsive"===e&&m.isBelow(t)},addActionSheetOpenEvents:function(){var t=this;$(document).on("click.disengage",(function(e){e.target.closest(".ids-actionsheet")||t.doCancel(null,!1,!1)})),$(this.actionSheetElem).on("click.action","button",(function(e){var i=e.currentTarget;i.classList.contains("btn-cancel")?t.doCancel(i,!1,!0):t.doSelect(i,!0),t.focus()})).on("keyup.action",(function(e){if(e.target.closest(".ids-actionsheet")&&"Escape"===e.key)t.doCancel(null,!1,!1)})).on("focusin.action","button",(function(e){t.resetFocusState(e.target.closest(".ids-action"))})).on("focusout.action","button",(function(e){t.resetFocusState(e.target.closest(".ids-action"))}))},doSelect:function(t,e){var i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=$(t);"function"==typeof this.settings.onSelect&&this.settings.onSelect(n),this.close(null,i),e&&$(this.element).trigger("selected",[n,null,!0])},doCancel:function(t,e,i){var n=$(t);"function"==typeof this.settings.onCancel&&this.settings.onCancel(n),this.cancel(i),e&&$(this.element).trigger("cancelled",[n])},resetFocusState:function(t){var e=$(t);e.data("hide-focus")||e.hideFocus(),e.addClass("hide-focus")},removeActionSheetOpenEvents:function(){$(document).off("click.disengage"),$(this.actionSheetElem).off("click.action keyup.action")},setOverlayVisibility:function(){var t=0;this.visible&&(t=this.settings.overlayOpacity),this.overlayElem.style.opacity=t?"".concat(t):""},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown(),this.init()},teardown:function(){this.visible&&this.close(),this.element.add($(this.trayBtn)).off("click.trigger"),this.actionSheetElem&&$(this.actionSheetElem).find("button").destroy&&($(this.actionSheetElem).find("button").destroy(),this.actionSheetElem.remove()),this.element[0].classList.remove("ids-actionsheet-trigger")},destroy:function(){this.teardown(),$.removeData(this.element[0],"actionsheet")}},$.fn.actionsheet=function(t){return this.each((function(){var e=$.data(this,$e);return e?e.updated(t):e=$.data(this,$e,new Ie(this,t)),e}))};var Le={};function Pe(t,e){var i=function(t){return t&&!(t instanceof jQuery)}(t);return!!i&&(void 0!==t[e]&&null!==t[e])}function Me(t,e){return!!function(t,e){return Pe(t,e)&&"function"==typeof t[e]}(t,e)&&(t[e](),!0)}function Be(t,e){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[],n=arguments.length>3&&void 0!==arguments[3]&&arguments[3],s=[];return"string"==typeof e&&(e=[e]),$.each(t,(function(t,a){$.each($(a).data(),(function(t,o){i.length&&-1===i.indexOf(t)||e.forEach((function(t){(n&&Me(o,t)||Pe(o,t))&&s.push({elem:$(a),control:o})}))}))})),s}function Oe(t,e){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[];if(t&&e){var n=Be((t=$(t)).find("*").add(t).not(i.join(", ")),e,void 0,!0);return t.trigger("sift-".concat(e,"-complete"),[n]),t}}Le.attachFilter=function(t,e){if(this.element.length){var i=this;"applicationmenu"===t&&(this.searchfield=this.element.find(".searchfield, .searchfield-wrapper"),this.searchfield.is(".searchfield-wrapper")&&(this.searchfield=this.searchfield.find(".searchfield")));var n=$(i.searchEl);n.is(".searchfield-wrapper")&&(n=n.find(".searchfield")),n&&"function"==typeof $.fn.searchfield&&(e instanceof HTMLElement&&e.classList.add("has-searchfield"),$(n).searchfield({clearable:!0,filterMode:i.settings.filterMode,source:function(t,e,n){var s;e(t,null===(s=i.accordionAPI)||void 0===s||null==s?void 0:s.toData(!0,!0),n)},searchableTextCallback:function(t){return t.text||t.contentText||""},resultIteratorCallback:function(t){return t.highlightTarget="text",t},clearResultsCallback:function(){var e,n=$(i.searchEl.querySelector(".searchfield")||i.searchEl).data("searchfield"),s=!0;("applicationmenu"!==t||n.isFocused||(s=!1),n&&s)&&(null===(e=i.accordionAPI)||void 0===e||null==e||e.unfilter())},displayResultsCallback:function(t,e){return Le.filterResultsCallback.apply(i,[n,i.accordionAPI,t,e])}}))}},Le.attachFilterEvents=function(t){var e=this;!0===this.settings.filterable&&this.searchEl&&($(this.searchEl).on("complete.".concat(t),(function(t){e.handleSearchfieldInputEvent(t)})),$(this.searchEl).on("cleared.".concat(t),(function(){var t;null===(t=e.accordionAPI)||void 0===t||null==t||t.unfilter(null,!0)})))},Le.filterResultsCallback=function(t,e,i,n){if(!i)return null==e||null==e||e.unfilter(),void n();var s=$(i.map((function(t){return t.element})));null==e||null==e||e.filter(s,!0),$(t).triggerHandler("filtered",[i]),n()},Le.handleSearchfieldInputEvent=function(){var t;if(this.searchEl){var e=$(this.searchEl).val();e&&""!==e||null===(t=this.accordionAPI)||void 0===t||null==t||t.unfilter(null,!0)}},Le.teardownFilter=function(t){var e;this.searchEl&&($(this.searchEl).off(["cleared.".concat(t),"complete.".concat(t)].join(" ")),null===(e=this.searchAPI)||void 0===e||null==e||e.destroy())};var Re=[".expandable-card",".expandable-area",".accordion","[soho-busyindicator]",".busy-indicator-container"],ze=[];$.fn.destroy=function(){return Oe($(this),"destroy")},$.fn.closeChildren=function(){return Oe($(this),"close",Re)},$.fn.handleResize=function(){return Oe($(this),"handleResize",ze)};var Fe="expandablearea",Ne={trigger:null,bottomBorder:!1,animationSpeed:300,attributes:null};function He(t,e){this.settings=E.mergeSettings(t,e,Ne),this.element=$(t),this.init()}He.prototype={init:function(){this.setup().build().handleEvents()},setup:function(){return this.id=this.element.attr("id"),this.id&&void 0!==this.id||(this.id="expandable-area-".concat($("body").find(".expandable-area").index(this.element))),this.element.prev(".expandable-area-trigger").length>0&&(this.expander=this.element.prev(".expandable-area-trigger"),this.settings.trigger=this.expander.attr("id")||"expandable-area-trigger-".concat($("body").find(".expandable-area").index(this.element))),this.header=this.settings.trigger?this.element:this.element.children(".expandable-header"),this.footer=this.element.children(".expandable-footer"),this.content=this.element.children(".expandable-pane"),this.isCard=this.element.is(".card, .widget"),this},build:function(){var t,e,i=this,n=this.element.hasClass("is-expanded");return(this.expander||this.header).attr({"aria-expanded":"".concat(n),"aria-controls":"".concat(this.id,"-content")}),this.header.attr({id:"".concat(this.id,"-header")}),this.content.attr({id:"".concat(this.id,"-content"),role:"region","aria-labelledby":"".concat(this.id,"-header")}),this.isCard||this.footer.length||this.settings.trigger||(this.footer=$('<div class="expandable-footer"></div>').appendTo(this.element)),this.expander=(t=this,!0===this.isCard?((e=t.header.find("expandable-expander")).length||(e=$("".concat('<a href="#" target="_self" role="button" class="btn-expander"><svg class="chevron icon" focusable="false" aria-hidden="true" role="presenation"><use href="#icon-caret-down"></use></svg><span class="audible">').concat(H.translate("ShowMore"),"</span>")+"</a>").appendTo(t.header)),e):((e=t.footer.find(".expandable-expander")).length||(e=$("".concat('<a href="#" target="_self" role="button" class="expandable-expander hyperlink"><span data-translated="true">').concat(H.translate("ShowMore"),"</span>")+"</a>").prependTo(t.footer)),e)),this.expander.attr("href","#").hideFocus(),this.expander.attr("role","button"),0===this.expander.length&&("string"==typeof this.settings.trigger?this.expander=$("#".concat(this.settings.trigger)):this.settings.trigger instanceof jQuery?this.expander=this.settings.trigger:this.expander=$(this.settings.trigger)),this.settings.bottomBorder&&this.element.addClass("has-bottom-border"),n&&(this.content.addClass("no-transition"),this.element.one("afterexpand.expandable-area",(function(){i.content.removeClass("no-transition")})),this.open()),n||(this.content.addClass("no-transition"),this.element.one("aftercollapse.expandable-area",(function(){i.content.removeClass("no-transition")})),this.close()),this.resize(),E.addAttributes(this.element,this,this.settings.attributes,"",!0),E.addAttributes(this.header,this,this.settings.attributes,"header",!0),E.addAttributes(this.content,this,this.settings.attributes,"content",!0),E.addAttributes(this.expander,this,this.settings.attributes,"expander",!0),E.addAttributes(this.footer,this,this.settings.attributes,"footer",!0),this},isDisabled:function(){return this.element.hasClass("is-disabled")},isExpanded:function(){return this.element.is(".is-expanded")},toggleExpanded:function(){this.isExpanded()?this.close():this.open()},open:function(){var t=this;!1!==this.element.triggerHandler("beforeexpand",[this.element])&&(this.element.addClass("is-expanded"),(this.expander||this.header).attr("aria-expanded","true"),this.expander.addClass("active"),this.element.triggerHandler("expand",[this.element]),this.expander.find('span[data-translated="true"]').text(H.translate("ShowLess")?H.translate("ShowLess"):"Show Less"),this.isCard&&this.expander.find(".icon").addClass("active"),this.content[0]&&(this.content[0].style.display="block"),this.content.one("animateopencomplete",(function(){t.element.triggerHandler("afterexpand",[t.element])})).animateOpen({timing:this.settings.animationSpeed}),this.applyIE11Fix())},close:function(){var t=this;!1!==this.element.triggerHandler("beforecollapse",[this.element])&&(this.expander.removeClass("active"),this.element.triggerHandler("collapse",[this.element]),this.expander.find('span[data-translated="true"]').text(H.translate("ShowMore")?H.translate("ShowMore"):"Show More"),this.isCard&&this.expander.find(".icon").removeClass("active"),this.content.one("animateclosedcomplete",(function(){t.element.removeClass("is-expanded"),(t.expander||t.header).attr("aria-expanded","false"),t.element.triggerHandler("aftercollapse",[t.element]),t.content[0].style.display="none"})).animateClosed({timing:this.settings.animationSpeed}),this.applyIE11Fix())},applyIE11Fix:function(){var t=this,e=$("html").hasClass("ie11");setTimeout((function(){e&&t.element.hasClass("is-expanded")?t.element.css("min-height",t.element.children(".expandable-pane").outerHeight(!0)+t.element.children(".expandable-footer").outerHeight(!0)):e&&!t.element.hasClass("is-expanded")&&t.element.css("min-height","auto")}),300)},resize:function(){var t=this;$("body").on("resize.expandablearea",(function(){t.applyIE11Fix()}))},disable:function(){this.element.addClass("is-disabled")},enable:function(){this.element.removeClass("is-disabled")},destroy:function(){var t;(this.expander||this.header).removeAttr("aria-controls").removeAttr("aria-expanded"),this.header.children("a").off(),null===(t=this.header)||void 0===t||null==t||t.off(),this.header.removeAttr("id"),this.content.removeAttr("id").removeClass("no-transition"),$.removeData(this.element[0],Fe)},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this},handleEvents:function(){var t=this;return this.expander.on("click.expandablearea",(function(e){t.isDisabled()||(e.preventDefault(),t.toggleExpanded())})),this}},$.fn.expandablearea=function(t){return this.each((function(){var e=$.data(this,Fe);e?e.updated(t):e=$.data(this,Fe,new He(this,t))}))};var qe=["startsWith","contains","keyword","wordStartsWith","phraseStartsWith"];var Ve={caseSensitive:!1,filterMode:qe[3],searchableTextCallback:void 0};function We(t){this.settings=E.mergeSettings(void 0,t,Ve),this.init()}We.prototype={init:function(){for(var t,e,i,n=[{setting:this.settings.filterMode,limits:qe,preset:Ve.filterMode}],s=0;s<n.length;s++)t=n[s].setting,e=n[s].limits,i=n[s].preset,-1===$.inArray(t,e)&&(t=i);return this.settings.filterMode===qe[0]&&V("wordStartsWith (filter)","startsWith (filter)"),this},filter:function(t,e,i){if(!t)return!1;if(!($.isArray(t)||t instanceof jQuery))return!1;if(!e||"string"!=typeof e||!e.length)return!1;var n=this,s=[],a=!1;this.settings.caseSensitive||(e=e.toLowerCase()),$.isArray(t)||(t instanceof jQuery||"object"===l(t))&&(t=$.makeArray(t),a=!0);var o=["node","_isFilteredOut"];function r(t){var e,s,a,r=!0;if("function"==typeof n.settings.searchableTextCallback)return n.settings.searchableTextCallback(t);"string"==typeof t?e=t:t instanceof $?e=$(t).text():t instanceof HTMLElement?(r=!1,e=t.innerText):(s=t,a="",Object.keys(s).forEach((function(t){if(!(i&&o.indexOf(t)>0)){var e=a.length?" ":"";a+="".concat(e).concat(s[t])}})),e=a);var l=e;return r&&(l=g.sanitizeHTML(e)),l}return t.forEach((function(t){var i=r(t);n.settings.caseSensitive||(i=i.toLowerCase().trim());var a=!1;if(["startsWith","wordStartsWith"].indexOf(n.settings.filterMode)>-1&&(a=function(t,e){for(var i=t.split(" "),n=0;n<i.length;n++)if(0===i[n].indexOf(e))return!0;return 0===t.indexOf(e)||e.indexOf(" ")>0&&t.indexOf(e)>0}(i,e)),"phraseStartsWith"===n.settings.filterMode&&0===i.indexOf(e)&&(a=!0),"contains"===n.settings.filterMode&&i.indexOf(e)>=0&&(a=!0),"keyword"===n.settings.filterMode)for(var o=e.split(" "),l=0;l<o.length;l++){var c=o[l];if(i.indexOf(c)>=0){a=!0;break}}null===n.settings.filterMode&&(a=!0),a&&s.push(t)})),a&&(s=$(s)),!(!a&&!s.length)&&s},updated:function(t){return this.settings=E.mergeSettings(void 0,t,this.settings),this.teardown().init()},teardown:function(){return this},destroy:function(){return this.teardown()}};var je={compile:function t(e,i,n,s){var a,o=t,r="";function c(t,e){var i=e;return t=t[(e=e.pop?e:e.split(".")).shift()]||t[i],0 in e?c(t,e):t}i=Array.isArray(i)?i:i?[i]:[],i=s?0 in i?[]:[1]:i;var d=function(){var t,s="",d=0,h="object"===l(i[a])?i[a]:{};(h=E.extend({},n,h))[""]={"":i[a]},e.replace(/([\s\S]*?)({{((\/)|(\^)|#)(.*?)}}|$)/g,(function(e,i,n,a,u,p,g){d?s+=d&&!u||d>1?e:i:(r+=i.replace(/{{{(.*?)}}}|{{(!?)(&?)(>?)(.*?)}}/g,(function(t,e,i,n,s,a){return e?c(h,e):n?c(h,a):s?o(c(h,a),h):i?"":new Option(c(h,a)).innerHTML})),t=p),u?--d||(g=c(h,g),/^f/.test(l(g))?r+=g.call(h,s,(function(t){return o(t,h)})):r+=o(s,g,h,t),s=""):++d}))};for(a=0;a<i.length;a++)d();return r}},Ue="autocomplete",Ge={source:[],sourceArguments:{},template:void 0,filterMode:"wordStartsWith",caseSensitive:!1,delay:300,width:null,offset:null,autoSelectFirstItem:!1,highlightMatchedText:!0,highlightCallback:function(t,e){var i=t,n=!1;if("string"==typeof e.alias&&void 0!==t[e.alias]&&(n=!0,i=t[e.alias]),"contains"===e.filterMode)i=i.replace(new RegExp("("+M.escapeRegExp(e.term)+")","ig"),"<i>$1</i>");else if("keyword"===e.filterMode)for(var s=e.term.split(" "),a=0;a<s.length;a++){var o=s[a];o&&(i=i.replace(new RegExp("("+M.escapeRegExp(o)+")","ig"),"<i>$1</i>"))}else{var r=i,l="".concat(r);e.caseSensitive||(l=H.toLowerCase(l));var c=M.escapeRegExp(e.term),d=l.indexOf(c);d>0?i=r.substr(0,d)+"<i>"+r.substr(d,c.length)+"</i>"+r.substr(c.length+d):0===d&&(i="<i>"+r.substr(0,c.length)+"</i>"+r.substr(c.length))}return n?t[e.alias]=i:t=i,t},resultIteratorCallback:function(t,e){var i="string"==typeof t,n={highlightTarget:"label",index:e,listItemId:"ac-list-option".concat(e)};return i?n.label=t:n=E.extend({},n,t),n.hasValue=void 0!==t.value,n.hasValue&&(n.value=t.value),n},clearResultsCallback:void 0,displayResultsCallback:void 0,searchableTextCallback:function(t){return"string"==typeof t?t:t.label}};function _e(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Ge),this.init()}_e.prototype={init:function(){var t=this.element.attr("data-autocomplete");t&&"source"!==t&&(this.settings.source=t,this.element.removeAttr("data-autocomplete"));var e={filterMode:this.settings.filterMode,caseSensitive:this.settings.caseSensitive,highlightMatchedText:this.settings.highlightMatchedText,searchableTextCallback:this.settings.searchableTextCallback};this.listFilter?this.listFilter.updated(e):this.listFilter=new We(e),this.addMarkup(),this.handleEvents()},get isFocused(){var t=document.activeElement,e=this.element[0],i=this.list,n=e.isEqualNode(t),s=i&&i.length&&i[0].contains(t),a=void 0!==$(this.element).data("clearable")&&this.element.parent().find(t);return n||s||a},addMarkup:function(){this.element.addClass("autocomplete").attr({role:"combobox",autocomplete:"off"}),E.addAttributes(this.element,this,this.settings.attributes)},isLoading:function(){return this.element.hasClass("is-loading")&&this.element.hasClass("is-blocked")},openList:function(t,e){var i=this;if(!this.element.is("[disabled], [readonly]")&&!this.isLoading()){if(!this.element.hasClass("searchfield"))if(!1===this.element.triggerHandler("beforeopen.autocomplete",{elem:this.element,value:this.element.val()}))return;var n=this;this.settings.caseSensitive||(t=H.toLowerCase(t)),this.list=$("#autocomplete-list"),0===this.list.length&&(this.list=$('<ul id="autocomplete-list" aria-expanded="true"></ul>').appendTo("body")),E.addAttributes(this.list,this,this.settings.attributes,"list"),n.settings.attributes&&setTimeout((function(){i.list.find("li a").each((function(t){var e=$(this);E.addAttributes(e,n,n.settings.attributes,"list-option".concat(t))}))})),this.list[0].style.height="auto",this.list[0].style.width="".concat(this.element.outerWidth(),"px"),this.list.addClass("autocomplete"),this.list.empty(),this.settings.width&&(this.list[0].style.width=this.settings.width+(/(px|%)/i.test("".concat(this.settings.width))?"":"px"));var s=$(this.element.attr("data-tmpl"));this.tmpl=$(s).length?$(s).text():"string"==typeof s?s:$(this.settings.template).length?$(this.settings.template).text():"string"==typeof this.settings.template?this.settings.template:'<li id="{{listItemId}}" data-index="{{index}}" {{#hasValue}}data-value="{{value}}"{{/hasValue}} role="listitem">\n <a href="#" tabindex="-1">\n   <span>{{{label}}}</span>\n </a>\n</li>';var a=this.listFilter.filter(e,t),o=[];!1!==a&&a.forEach((function(e,i){"string"==typeof e&&(e={label:e,value:e});var s=E.extend(!0,{},e);if(s=n.settings.resultIteratorCallback(s,i),n.settings.highlightMatchedText){var a={filterMode:n.settings.filterMode,caseSensitive:n.settings.caseSensitive,term:t};s.highlightTarget&&(a.alias=s.highlightTarget),n.settings.displayResultsCallback||(s=n.settings.highlightCallback(s,a))}o.push(s)})),this.currentDataSet=o,"function"!=typeof this.settings.displayResultsCallback?this.handleListResults(t,e,o):this.settings.displayResultsCallback(o,(function(){n.element.trigger("listopen",[o])}),t)}},handleListResults:function(t,e,i){var n=this;i.forEach((function(t){if(void 0!==je){var e=je.compile(n.tmpl,t);f.append(n.list,e,"*")}})),this.element.trigger("beforepopulated",[i]);var s={menuId:"autocomplete-list",ariaListbox:!0,mouseFocus:!1,trigger:"immediate",attachToBody:!0,autoFocus:!1,returnFocus:!1,triggerSelect:!1,placementOpts:{parent:this.element,callback:function(t){return!0===t.wasFlipped&&(n.list.add(n.element).addClass("is-ontop"),t.y+=1),t}}};if(this.previouslyOpened){var a=this.element.data("popupmenu");a&&a.position()}else this.element.addClass("is-open").popupmenu(s).one("close.autocomplete",(function(){n.closeList(!0)}));var o=n.list.find("li"),r=$(o[0]).find("span").outerWidth()+20;if(r>parseInt(this.element.outerWidth(),10)){for(var l=0;l<o.length;l++)o.width(r+"px");this.maxWidth=r}if(n.settings.autoSelectFirstItem&&n.list.children().filter(":not(.separator):not(.hidden):not(.heading):not(.group):not(.is-disabled)").first().addClass("is-selected"),this.noSelect=!0,this.element.trigger("populated",[i]).focus(),!this.previouslyOpened){n.list.on("touchend.".concat(Ue," click.").concat(Ue),"a",(function(t){n.select(t)})).on("focusout.".concat(Ue),(function(){n.checkActiveElement()}));var c=n.list.find("a").on("focus.".concat(Ue," touchend.").concat(Ue),(function(){n.highlight($(this),c)}));if(this.settings.offset){var d=this.list.parent()[0];this.settings.offset.left&&(d.style.left="".concat(parseInt(d.style.left,10)+this.settings.offset.left,"px")),this.settings.offset.top&&(d.style.top="".concat(parseInt(d.style.top,10)+this.settings.offset.top,"px"))}}var h=E.getArrayFromList(document.querySelectorAll("#ac-is-arialive"));h&&h.forEach((function(t){t.parentNode.removeChild(t)})),f.append(n.list.parent(".popupmenu-wrapper"),'<span id="ac-is-arialive" aria-live="polite" class="audible">'.concat($.trim(this.list.find(">li:first-child").text()),"</span>"),"<div><span><a><small><img><svg><i><b><use><br><strong><em>"),this.noSelect=!0,this.previouslyOpened=!0,this.element.trigger("listopen",[i])},closeList:function(t){if(this.list)if(this.list.off(["click.".concat(Ue),"keydown.".concat(Ue),"touchend.".concat(Ue),"focusout.".concat(Ue)].join(" ")),this.list.find("a").off("focus.".concat(Ue," touchend.").concat(Ue)),this.element.trigger("listclose"),"function"!=typeof this.settings.clearResultsCallback){var e=this.element.data("popupmenu");e&&(t||e.close(),$("#autocomplete-list").parent(".popupmenu-wrapper").remove(),$("#autocomplete-list").remove(),this.element.add(this.list).removeClass("is-open is-ontop"),delete this.previouslyOpened)}else this.settings.clearResultsCallback()},listIsOpen:function(){return this.list instanceof $&&this.list.length&&this.list.is(":visible")},handleAutocompleteKeydown:function(t){var e=this;if(this.isLoading())return t.preventDefault(),!1;if(!this.listIsOpen())return null;function i(t){t.removeClass("is-selected is-focused")}function n(t){t.addClass("is-selected").find("a").focus()}var s=this.list.find("li:not(.separator):not(.hidden):not(.heading):not(.group):not(.is-disabled)"),a=function(t){var e=t.filter(".is-selected");return 0===e.length&&(e=t.filter(".is-focused")),e}(s);return 40===t.keyCode&&this.listIsOpen()&&a.length&&(e.noSelect=!0,i(a),n(s.eq(s.index(a)+1)),t.preventDefault(),t.stopPropagation()),38===t.keyCode&&this.listIsOpen()&&a.length&&(e.noSelect=!0,i(a),n(s.eq(s.index(a)-1)),t.preventDefault(),t.stopPropagation()),9!==t.keyCode&&13!==t.keyCode||!this.listIsOpen()||(a.length?(t.stopPropagation(),t.preventDefault(),e.noSelect=!0,e.select(a)):e.isFocused||e.closeList()),null},handleAutocompleteInput:function(t){var e=this;return e.isLoading()?(t.preventDefault(),!1):(this.getDataFromSource().done((function(t,i){e.currentDataSet=i,e.openList(t,i)})),null)},checkActiveElement:function(){var t=this;setTimeout((function(){var e=document.activeElement;t.listIsOpen()&&$.contains(t.list[0],e)||t.element.is(e)||t.element.trigger("safe-blur")}),0)},getDataFromSource:function(){var t=this;if(t.isLoading())return!1;var e,i=this.element,n=$.Deferred();function s(e,i){t.element.triggerHandler("complete"),t.element.trigger("requestend",[e,i])}function a(e,i,a){return t.lastTerm!==e?n.reject(e):(s(e,i),!1===a?n.reject(e):n.resolve(g.stripTags(e),i))}return clearTimeout(this.loadingTimeout),this.loadingTimeout=setTimeout((function(){if(!t.isLoading()&&t.isFocused){if(""===(e=i.val()))return t.element.data("popupmenu")&&t.element.data("popupmenu").close(),void s();var o=l(t.settings.source);if(t.element.triggerHandler("start"),t.element.trigger("requeststart",[e]),t.lastTerm=e,"function"===o)t.settings.source(e,a,t.settings.sourceArguments);else if("object"===o){var r=$.isArray(t.settings.source)?t.settings.source:[t.settings.source];a(e,r,!0)}else if(t.settings.source)if("keyword"===t.settings.filterMode){var c=[],d=function(t){if(0===c.length)c=t;else for(var e=0;e<t.length;e++){for(var i=t[e],n=!1,s=0;s<c.length;s++)for(var a=c[s],o=0;o<Object.getOwnPropertyNames(a).length;o++){if(i[Object.getOwnPropertyNames(i)[o]]===a[Object.getOwnPropertyNames(a)[o]]){n=!0;break}}n||c.push(i)}},h=function(t){d(t),a(e,c,!0)},u=e.split(" ");""===u[u.length-1]&&u.splice(-1,1);for(var p=0;p<u.length;p++){var g=u[p];if(g.length>0){var f=t.settings.source.toString(),m=$.getJSON(f+g);p<u.length-1?m.done(d).fail(d):m.done(h).fail(h)}}}else{var v=t.settings.source.toString();$.getJSON(v+e).done((function(t){a(e,t,!0)})).fail((function(){a(e,[],!1)}))}else n.reject(e)}}),t.settings.delay),n},resetFilters:function(){this.openList("",this.currentDataSet)},handleAutocompleteFocus:function(){this.noSelect?this.noSelect=!1:this.element.select()},highlight:function(t,e){var i=this.element.val(),n=t.text().trim();t.find(".display-value").length>0&&(n=t.find(".display-value").text().trim()),e&&e.length&&e.parent("li").removeClass("is-selected"),t.hasClass("no-results")||(t.parent("li").addClass("is-selected"),this.element.val(n).focus()),this.noSelect=!0,i!==n&&this.element.triggerHandler("change")},select:function(t,e){var i,n,s={},a=!1;if(t instanceof $.Event?(a=!0,i=$(t.currentTarget)):i=t,!i.hasClass("no-results")){i.is("li")&&(n=i,i=i.children("a"));var o=(n=i.parent("li")).attr("data-index"),r=n.attr("data-value");if(this.element.attr("aria-activedescendant",n.attr("id")),e&&e.length||(e=this.currentDataSet),o)s=e[parseInt(o,10)];else if(r)for(var c,d=0;d<e.length;d++)(c="object"===l(e[d])&&void 0!==e[d].value?e[d].value.toString():e[d].toString())===r&&("object"===l(e[d])&&(s=e[d]),s.value=c);return s.value&&0!==!s.value.length||(s.value=i.text().trim()),this.highlight(i),this.noSelect=!0,s.label=g.stripHTML(s.label),s.highlightTarget||(s.highlightTarget="label",s.index=parseInt(o,10),s.listItemId="ac-list-option"+s.index,s.hasValue=!0),this.element.trigger("selected",[i,s]),a&&t.preventDefault(),this.closeList(),this.element.focus(),s}},handleAfterListOpen:function(){var t=this;return this.element.offset().left>this.list.offset().left&&this.list.width(this.list.width()+1),this.list.on("keydown.".concat(Ue),(function(e){"Enter"===e.key&&t.handleAutocompleteKeydown(e)})),this},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),t&&t.source&&(this.settings.source=t.source),this.teardown().init(),this},enable:function(){this.element.prop("disabled",!1)},disable:function(){this.element.prop("disabled",!0)},teardown:function(){var t=this.element.data("popupmenu");return t&&t.destroy(),this.element.off(["focus.".concat(Ue),"focusout.".concat(Ue),"input.".concat(Ue),"keydown.".concat(Ue),"listopen.".concat(Ue),"requestend.".concat(Ue),"resetfilter.".concat(Ue),"updated.".concat(Ue)].join(" ")),this},destroy:function(){this.teardown(),$.removeData(this.element[0],Ue)},handleEvents:function(){var t=this;this.element.on("updated.".concat(Ue),(function(){t.updated()})).on("keydown.".concat(Ue),(function(e){t.handleAutocompleteKeydown(e)})).on("input.".concat(Ue),(function(e){t.handleAutocompleteInput(e)})).on("focus.".concat(Ue),(function(){t.handleAutocompleteFocus()})).on("focusout.".concat(Ue),(function(){t.checkActiveElement()})).on("listopen.".concat(Ue),(function(){t.handleAfterListOpen()})).on("resetfilter.".concat(Ue),(function(){t.resetFilters()}))}},$.fn.autocomplete=function(t){return this.each((function(){var e=$.data(this,Ue);if("string"==typeof e){var i="".concat(e),n=E.extend({},t,{source:i||t.source});e=$.data(this,Ue,new _e(this,n))}else e?e.updated(t):e=$.data(this,Ue,new _e(this,t))}))};var Ye="searchfield",Ke=[!1,"mobile",!0],Xe={resultsCallback:void 0,allResultsCallback:void 0,showAllResults:!0,showGoButton:!1,goButtonCopy:void 0,goButtonAction:void 0,categories:void 0,categoryMultiselect:!1,showCategoryText:!1,source:void 0,template:void 0,clearable:!1,collapsible:Ke[0],collapseSize:void 0,tabbable:!0,autocompleteAttribute:"off"},Qe=280;function Je(t,e){this.element=$(t),this.toolbarParent&&!this.isContainedByFlexToolbar&&void 0!==e&&(void 0===e.clearable&&(e.clearable=!0),void 0===e.collapsible&&(e.collapsible=!0,e.tabbable=!1)),this.settings=E.mergeSettings(t,e,Xe),this.settings.collapsible&&(this.settings.tabbable=!1),this.init()}Je.prototype={get toolbarParent(){var t=this.element.parents(".toolbar"),e=this.element.parents(".flex-toolbar");if(!(t.add(e).length<1))return e.length>0?e.first()[0]:t.first()[0]},get buttonsetElem(){if(this.toolbarParent)return this.toolbarParent.querySelector(".buttonset")},get titleElem(){if(this.toolbarParent)return this.toolbarParent.querySelector(".title")},get containmentParent(){var t=this.element.closest(".module-tabs");return t.length?t.first()[0]:this.toolbarParent},get input(){return this.element[0]},get categoryButton(){return this.wrapper.find(".searchfield-category-button")},get isCollapsible(){return!1!==this.settings.collapsible},get isCurrentlyCollapsible(){return!0===this.settings.collapsible||"mobile"===this.settings.collapsible&&m.isBelow("phone-to-tablet")},get isContainedByFlexToolbar(){return!!this.containmentParent&&this.containmentParent.className.indexOf("flex-toolbar")>-1},get toolbarFlexItem(){var t;return this.isContainedByFlexToolbar&&(t=$(this.element).data("toolbarflexitem")),t},get autocompleteAPI(){return $(this.element).data("autocomplete")},get isContextSearch(){return this.wrapper[0].className.indexOf("context")>-1},init:function(){this.coerceBooleanSettings(),this.build(),this.setupEvents()},build:function(){this.id||(this.id=E.uniqueId(this.element,Ye)),this.label=this.element.prev("label, .label"),this.inlineLabel=this.element.closest("label"),this.isInlineLabel=this.element.parent().is(".inline");var t=this.element.attr("data-autocomplete");t&&"source"!==t&&(this.settings.source=t,this.element.removeAttr("data-autocomplete"),$.removeData(this.element,"autocomplete")),this.settings.source&&(this.autocomplete=this.element.data("autocomplete"),this.autocomplete?this.autocomplete.updated(this.settings):(this.element.autocomplete(this.settings),this.autocomplete=this.element.data("autocomplete")));var e=this.settings.autocompleteAttribute;"string"==typeof e&&e.length?this.element.attr("autocomplete","".concat(g.ensureAlphaNumeric(e))):this.element.hasAttr("autocomplete")&&this.element.removeAttr("autocomplete");var i=this.element.attr("placeholder")||this.element.prev("label, .label").text().trim();i&&""!==i||(i=H.translate("Keyword")),this.element.attr({"aria-label":i}),this.wrapper=this.element.parent(".searchfield-wrapper"),this.wrapper&&this.wrapper.length||(this.isInlineLabel?this.wrapper=this.inlineLabel.addClass("searchfield-wrapper"):this.wrapper=this.element.wrap('<span class="searchfield-wrapper"></span>').parent()),this.checkContents();var n="remove";this.settings.collapsible&&this.settings.collapseSize&&(n="add"),this.wrapper[0].classList[n]("has-custom-collapse-size"),this.toolbarParent&&(this.label.prependTo(this.wrapper),this.setInitalWidth());for(var s,a=["context","alternate"],o=0;o<a.length;o++)this.element.hasClass(a[o])&&(s=a[o],this.wrapper.addClass(s),this.element.removeClass(s));!0===this.settings.collapsibleOnMobile&&(this.settings.collapsible=Ke[1]),this.wrapper[0].classList[this.isCurrentlyCollapsible?"remove":"add"]("non-collapsible"),this.wrapper[0].classList[this.toolbarParent?"add":"remove"]("toolbar-searchfield-wrapper"),E.addAttributes(this.element,this,this.settings.attributes,"",!0),E.addAttributes(this.wrapper,this,this.settings.attributes,"wrapper",!0),this.toolbarParent&&!this.isContainedByFlexToolbar&&this.element.add(this.wrapper).addClass("no-transition no-animation");var r=this.wrapper.find(".icon:not(.icon-dropdown)");if(r&&r.length||(r=$.createIconElement("search")),r[this.wrapper.hasClass("context")||this.wrapper.hasClass("has-categories")?"insertBefore":"insertAfter"](this.element).icon(),this.hasCategories()){this.wrapper.addClass("has-categories"),this.categoryButton.length||$('<button type="button" class="btn searchfield-category-button"></button>').insertBefore(this.element),r.appendTo(this.categoryButton),r=this.categoryButton,this.settings.showCategoryText&&this.wrapper.addClass("show-category");var l=r.find(".icon-dropdown");if(l.length||(l=$.createIconElement({classes:"icon-dropdown",icon:"dropdown"}).icon()),l.appendTo(r),this.categoryButton.data("popupmenu"))this.setCategories(this.settings.categories);else{this.list=this.wrapper.find("ul.popupmenu"),this.list&&this.list.length||(this.list=$('<ul class="popupmenu"></ul>'));var c=this.settings.categoryMultiselect?"is-multiselectable":"is-selectable";this.list.addClass(c);var d="is-selectable";this.settings.categoryMultiselect||(d="is-multiselectable"),this.list.removeClass(d),this.setCategories(this.settings.categories),this.list.insertAfter(this.element);var h=this;this.categoryButton.popupmenu({menu:this.list,offset:{y:10},returnFocus:function(){h.isFocused&&h.element.focus()}})}this.setCategoryButtonText()}this.isContainedByFlexToolbar?(this.collapseButton&&this.collapseButton.length||(this.collapseButton=$('\n          <button class="btn-secondary collapse-button" type="button">\n            <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n              <use href="#icon-exit-fullview"></use>\n            </svg>\n            <span class="audible">'.concat(H.translate("Collapse"),"</span>\n          </button>\n        "))),this.wrapper[0].classList.add("has-collapse-button"),this.element.after(this.collapseButton)):this.wrapper[0].classList.remove("has-collapse-button");var u=this.wrapper.next(".go-button");u.length||(u=this.wrapper.find(".go-button")),u.length&&(this.settings.showGoButton=!0,this.goButton=u,this.element.after(this.goButton)),this.settings.showGoButton?(this.goButton&&this.goButton.length||(this.goButton=$('\n          <button class="btn-secondary go-button">\n            <span>'.concat(this.settings.goButtonCopy||H.translate("Go",{showBrackets:!1}),"</span>\n          </button>\n        "))),this.goButton[0].setAttribute("id",E.uniqueId(this.goButton,"searchfield-go-button-")),this.wrapper.addClass("has-go-button"),this.element.after(this.goButton),this.goButton.button()):this.wrapper.removeClass("has-go-button"),this.settings.clearable&&this.makeClearable(),this.settings.button&&(this.wrapper.addClass("has-custom-button"),this.customButton=$('<button class="btn-icon custom-button"></button>'),this.customButton.append($.createIconElement(this.settings.button.icon)),this.element.after(this.customButton));var p=this,f=new X({duration:1,updateCallback:function(){},timeoutCallback:function(){p.calculateSearchfieldWidth()}});J.register(f);var m=0;return $("body").on("resize.".concat(this.id),(function(){p.wrapper.parents(".splitter-container").length>0&&(p.wrapper.width()>=p.wrapper.parent().width()||m>=p.wrapper.parent().width()?p.wrapper.width("".concat(p.wrapper.parent().width()-.1*p.wrapper.parent().width(),"px")):p.wrapper.css("width","")),p.calculateSearchfieldWidth()})),!1===this.settings.collapsible&&this.expand(!0),m=p.wrapper.width(),this},simpleAdjustOnBreakpoint:function(){this.shouldBeFullWidth()?this.isFocused||!this.settings.collapsible&&!this.isContainedByFlexToolbar||this.wrapper[0].classList.remove("is-open"):this.wrapper[0].classList.add("is-open")},adjustOnBreakpoint:function(){if(this.wrapper[0].classList[this.isCurrentlyCollapsible?"remove":"add"]("non-collapsible"),this.shouldBeFullWidth()){if(this.wrapper.removeAttr("style"),this.input.removeAttribute("style"),this.isFocused){if(this.appendToParent(),this.calculateOpenWidth(),this.setOpenWidth(),this.isExpanded)return;return void this.expand(!0)}return this.isCurrentlyCollapsible&&this.isExpanded&&this.collapse(),this.isContainedByFlexToolbar&&this.wrapper[0].classList.remove("is-open"),void this.setInitalWidth()}this.appendToButtonset(),this.isFocused||"mobile"===this.settings.collapsible?this.isExpanded||this.expand(!0):(this.isExpanded&&this.collapse(),this.setInitalWidth())},saveFocus:function(){this.isFocused&&(this.focusElem=document.activeElement)},restoreFocus:function(){if(this.focusElem){var t=this,e=new X({duration:1,updateCallback:function(){},timeoutCallback:function(){t.focusElem&&(t.focusElem.focus(),delete t.focusElem)}});J.register(e)}},appendToParent:function(){this.containmentParent&&!this.wrapper.parent().is($(this.containmentParent))&&(this.isContainedByFlexToolbar||(this.saveFocus(),this.elemBeforeWrapper=this.wrapper.prev(),$(this.containmentParent).find(".buttonset").prepend(this.wrapper),E.fixSVGIcons(this.wrapper),this.restoreFocus()))},appendToButtonset:function(){this.containmentParent&&this.wrapper.parent().is($(this.containmentParent))&&(this.isContainedByFlexToolbar||(this.saveFocus(),this.elemBeforeWrapper instanceof $&&this.elemBeforeWrapper.length?(this.wrapper.insertAfter(this.elemBeforeWrapper),this.elemBeforeWrapper=null):this.wrapper.prependTo($(this.buttonsetElem)),$(this.toolbarParent).triggerHandler("scrollup"),E.fixSVGIcons(this.wrapper),this.restoreFocus()))},shouldBeFullWidth:function(){var t=this.wrapper.closest(".header"),e=!1;return t.length&&(e=t.width()<m.phone),e||m.isBelow("phone-to-tablet")},shouldExpandOnMobile:function(){return!0!==this.settings.collapsible&&("mobile"===this.settings.collapsible||this.shouldBeFullWidth())},coerceBooleanSettings:function(){this.settings=E.coerceSettingsToBoolean(this.settings,["showAllResults","categoryMultiselect","showCategoryText","clearable"])},hasCategories:function(){return this.settings.categories&&$.isArray(this.settings.categories)&&this.settings.categories.length>0},hasGoButton:function(){return this.settings.showGoButton&&this.goButton&&this.goButton.length},hasCustomButton:function(){return this.settings.button&&this.customButton&&this.customButton.length},setupEvents:function(){var t=this,e=this;return e.element.on("updated.".concat(this.id),(function(t,i){e.updated(i)})).on("focus.".concat(this.id),(function(t){e.handleFocus(t)})).on("blur.".concat(this.id),(function(t){e.isContainedByFlexToolbar&&e.handleSafeBlur(t)})).on("click.".concat(this.id),(function(t){e.handleClick(t)})).on("keydown.".concat(this.id),(function(t){e.handleKeydown(t)})).on("beforeopen.".concat(this.id),(function(t,i){e.handlePopupBeforeOpen(t,i)})).on("safe-blur.".concat(this.id),(function(){e.handleSafeBlur()})).on("listclose.".concat(this.id),(function(){e.handleSafeBlur()})).on("input.".concat(this.id),(function(){e.checkContents()})),e.wrapper.on("mouseenter.".concat(this.id),(function(){$(this).addClass("is-hovered")})).on("mouseleave.".concat(this.id),(function(){$(this).removeClass("is-hovered")})),this.hasCategories()&&this.categoryButton.on("selected.".concat(this.id),(function(t,i){e.handleCategorySelected(t,i)})).on("focus.".concat(this.id),(function(t){e.handleCategoryFocus(t)})).on("blur.".concat(this.id),(function(){e.handleSafeBlur()})).on("close.".concat(this.id),(function(){e.handleSafeBlur()})).on("beforeopen.".concat(this.id),(function(t,i){e.handlePopupBeforeOpen(t,i)})),e.hasGoButton()&&e.goButton.on("click.".concat(this.id),(function(t){return e.handleGoButtonClick(t)})).on("click.".concat(this.id),(function(t){return e.handleGoButtonFocus(t)})).on("blur.".concat(this.id),(function(){return e.handleSafeBlur()})),e.hasCustomButton()&&this.customButton.on("click.".concat(this.id),(function(t){return e.settings.button.click(t,e.element.val())})).on("blur.".concat(this.id),(function(){return e.handleSafeBlur()})),this.isCollapsible?(this.wrapper.on("focusin.".concat(this.id),(function(t){e.handleFocus(t)})).on("focusout.".concat(this.id),(function(t){e.handleBlur(t)})).on("keydown.".concat(this.id),(function(t){e.handleKeydown(t)})).on("collapse.".concat(this.id),(function(){e.collapse()})),$("body").on("breakpoint-change.".concat(this.id),(function(){e.adjustOnBreakpoint()})),e.adjustOnBreakpoint()):($("body").on("breakpoint-change.".concat(this.id),(function(){e.simpleAdjustOnBreakpoint()})),e.simpleAdjustOnBreakpoint()),this.collapseButton&&this.collapseButton.length&&this.collapseButton.on("keydown.".concat(this.id),(function(t){e.collapseResponsive(t)})).on("click.".concat(this.id),(function(t){e.collapseResponsive(t)})).on("blur.".concat(this.id),(function(){return e.handleSafeBlur()})),this.toolbarParent&&$(this.toolbarParent).on("navigate.".concat(this.id),(function(){!e.isFocused&&e.isCurrentlyCollapsible&&e.collapse()})).on("rendered.".concat(this.id),(function(){e.element.removeClass("no-transition no-animation"),e.wrapper.removeClass("no-transition no-animation")})),e.element.on("beforepopulated.".concat(this.id),(function(t,i){i.length>0?e.settings.showAllResults&&e.addMoreLink():e.addNoneLink()})),e.settings.clearable&&(e.element.on("cleared.".concat(this.id),(function(){e.autocomplete&&e.autocomplete.closeList(),delete e.settings.value})),e.xButton.on("blur.".concat(this.id),(function(t){e.xButton.hasClass("collapsed")||e.handleSafeBlur(t)}))),e.element.on("listopen.".concat(this.id),(function(){var i=$("#autocomplete-list");e.wrapper.addClass("popup-is-open"),e.element.on("selected.".concat(t.id),(function(t,i,n){if(i.hasClass("more-results")){var s=e.settings.allResultsCallback;s&&"function"==typeof s&&s(n)}})),i.find(".more-results, .no-results").on("focus.".concat(t.id),(function(){var t=$(this);i.find("li").removeClass("is-selected"),t.hasClass("no-results")||(t.parent("li").addClass("is-selected"),e.element.val(""))}))})).on("listclose.".concat(this.id),(function(){var i=$("#autocomplete-list");e.element.off("selected.".concat(t.id)),i.off("focus.".concat(t.id))})),x.browser.isSafari()&&this.xButton&&(this.xButton.on("mouseover.clearable",(function(){t.closeButtonHover=!0})),this.xButton.on("mouseleave.clearable",(function(){t.closeButtonHover=!1}))),this},setAsActive:function(t,e){var i=this;if(t||!this.wrapper.hasClass("active")){this.addDocumentDeactivationEvents();var n=["has-focus","active"];(this.isCurrentlyCollapsible||this.isContainedByFlexToolbar)&&(this.expand(!0),this.isContainedByFlexToolbar&&n.push("is-open")),n.forEach((function(t){i.wrapper[0].classList.add(t)})),this.toolbarParent&&this.toolbarParent.classList.add("searchfield-active"),this.isExpanded||!0===e&&this.element.focus()}},get isFocused(){var t=document.activeElement,e=this.wrapper[0];if(this.settings.tabbable&&e.contains(t)&&$(t).is("button.close"))return!0;if(x.browser.isSafari()&&this.closeButtonHover&&!$(t).is("button.close"))return!0;if(e.contains(t))return!0;if(this.autocomplete){var i=this.autocomplete.list;if(i&&i[0].contains(t))return!0}if(this.categoryButton&&this.categoryButton.length&&this.categoryButton.data("popupmenu").menu.has(t).length)return!0;return!!(this.xButton&&this.xButton.length&&this.xButton.has(t).length)||!!(this.customButton&&this.customButton.length&&this.customButton.has(t).length)},hasFocus:function(){return V("isFocused","hasFocus"),this.isFocused},handleFocus:function(){this.setAsActive(!0)},handleBlur:function(){this.handleSafeBlur()},handleSafeBlur:function(){var t=this;this.blurTimer&&this.blurTimer.destroy(!0),this.blurTimer=new X({duration:1,updateCallback:function(){},timeoutCallback:function(){if(t.isFocused)delete t.settings.value;else if(t.settings.value=$(t.element).val(),t.removeDocumentDeactivationEvents(),t.clearResponsiveState(),t.isCurrentlyCollapsible)t.collapse();else if(t.isContainedByFlexToolbar&&(t.settings.collapsible||m.isBelow("phone-to-tablet")))t.wrapper[0].classList.remove("is-open");else{t.wrapper[0].classList.remove("has-focus","active")}}}),J.register(this.blurTimer)},clearResponsiveState:function(){this.toolbarParent&&this.toolbarParent.classList.remove("searchfield-active")},handleClick:function(){this.setAsActive()},addDocumentDeactivationEvents:function(){if(!0!==this.hasDeactivationEvents){var t=this;$(document).on("click.".concat(this.id),(function(e){t.handleOutsideClick(e)})).on("keydown.".concat(this.id),(function(e){t.handleOutsideKeydown(e)})),this.hasDeactivationEvents=!0}},removeDocumentDeactivationEvents:function(){$(document).off("click.".concat(this.id," keydown.").concat(this.id)),this.hasDeactivationEvents=!1},handleOutsideClick:function(t){var e=t.target;this.isSearchfieldElement(e)||this.handleSafeBlur()},handleKeydown:function(t){var e,i,n,s=t.which,a=t.key;if(27===s&&x.browser.isIE11()&&t.preventDefault(),t.ctrlKey&&8===s&&this.element.val(""),9===s&&this.handleSafeBlur(),this.isContainedByFlexToolbar&&(null===(e=this.toolbarFlexItem)||void 0===e||null===(i=null==e?void 0:e.toolbarAPI)||void 0===i||null===(n=null==i?void 0:i.items)||void 0===n||null==n?void 0:n.length)>1){["ArrowUp","Up","ArrowDown","Down"].indexOf(a)>-1&&this.collapse()}},handleOutsideKeydown:function(t){var e=t.which,i=t.target;9!==e||this.isSearchfieldElement(i)||this.handleSafeBlur()},handlePopupBeforeOpen:function(t,e){if(this.isCollapsible&&(this.isExpanding||!this.isExpanded)||!e)return!1;var i=this.wrapper.hasClass("context")?"addClass":"removeClass",n=this.wrapper.hasClass("alternate")?"addClass":"removeClass";return e[i]("context"),e[n]("alternate"),!!this.isExpanded||(this.categoryButton.focus(),!1)},handleGoButtonClick:function(t){var e=this.settings.goButtonAction;if("function"==typeof e){var i,n=this.element.val();return this.hasCategories()&&(i=this.getCategoryData()),e(t,n,i)}},handleGoButtonFocus:function(){this.setAsActive(!0)},setCategoryButtonText:function(t){if(this.settings.showCategoryText&&this.categoryButton.length){var e="",i=this.wrapper.find(".btn"),n=i.find("span");if(n&&n.length||(n=$('<span class="category"></span>').insertAfter(i.find(".icon").first())),n.empty(),"string"==typeof t&&t.length)n.text(t.trim());else{var s=this.getSelectedCategories();s.length&&(e=s.length>1?"".concat(s.length," ").concat(H.translate("Selected")):s.text().trim(),n.text(e))}}},isSearchfieldElement:function(t){if($.contains(this.wrapper[0],t))return!0;if(this.categoryButton&&this.categoryButton.length&&this.categoryButton.data("popupmenu").menu.has(t).length)return!0;return!1},getFillSize:function(t,e){var i,n,s;function a(t){if(!t)return 0;if(!isNaN(parseInt(t,10)))return parseInt(t,10);if(t instanceof jQuery){if(!t.length)return 0;t.is(".title")&&(t=t.next(".buttonset")),t=t[0]}return t}function o(t,e){if(!isNaN(t))return null==t?0:t;e&&"string"==typeof e||(e="left");var i,n=["left","right"];return-1===$.inArray(e,n)&&(e=n[0]),(t instanceof HTMLElement||t instanceof SVGElement)&&(i=t.getBoundingClientRect()),i[e]}if(t=a(t),e=a(e),i=o(t),n=o(e,(s="left",(t===e||$(e).length&&$(e).is(".buttonset"))&&(s="right"),s)),!i&&!n)return Qe;var r=n-i;return r<=Qe?Qe:r>=450?450:r},setClosedWidth:function(){return this.setInitalWidth()},setOpenWidth:function(){var t=0,e=this.wrapper;if(this.wrapper[0]&&(this.wrapper[0].style.width=this.openWidth),this.hasCategories()){var i=window.getComputedStyle(this.categoryButton[0]);t+=this.categoryButton.width()+(parseInt(i.paddingLeft,10)+parseInt(i.paddingRight,10))+2*parseInt(i.borderLeftWidth,10),1===this.element.parents(".header").length&&m.isBelow("phone-to-tablet")&&(t+=40)}if(this.hasGoButton()){var n=window.getComputedStyle(this.goButton[0]);t+=this.goButton.width()+(parseInt(n.paddingLeft,10)+parseInt(n.paddingRight,10))+2*parseInt(n.borderLeftWidth,10)}if(t>0){var s=e.hasClass("alternate")?2:0;this.input.style.width="calc(100% - ".concat(t+s,"px)")}delete this.openWidth},calculateOpenWidth:function(){var t,e=this.element.parents(".toolbar").children(".buttonset"),i=this.wrapper.next();if(e.length){if(i.is(".title")&&(i=e),this.shouldBeFullWidth()){var n="100%";return m.isBelow("desktop")&&(n="90%"),t=n,$(this.toolbarParent).closest(".tab-container.module-tabs").length&&(t="calc(".concat(n," - 1px)")),void(this.openWidth=t)}var s=e.offset().left+10,a=i;a.length||(a=e.offset().left+10+e.outerWidth(!0)),t=this.getFillSize(s,a),this.openWidth="".concat(t-6,"px")}},calculateSearchfieldWidth:function(){var t,e="100%",i=0;this.element[0].getAttribute("style")&&this.element[0].removeAttribute("style");var n,s=window.getComputedStyle(this.element[0]);(s.width&&!this.isContextSearch&&(e=s.width),this.hasCategories()&&(i+=this.categoryButton.outerWidth(!0),1===this.element.parents(".header").length&&m.isBelow("phone-to-tablet")&&(i+=40)),this.hasGoButton()&&(i+=this.goButton.outerWidth(!0)),i>0)&&(n=null!=this.element[0].parentElement&&(this.element[0].parentElement.classList.contains("alternate")||this.isContainedByFlexToolbar),t="calc(".concat(n?"100%":e," - ").concat(i,"px)"));t&&(this.element[0].style.width=t)},handleCategorySelected:function(t,e){this.element.trigger("selected",[e]),this.settings.showCategoryText&&(this.setCategoryButtonText(t,e.text().trim()),this.calculateSearchfieldWidth(),this.settings.categoryMultiselect||this.setAsActive(!0,!0))},handleCategoryFocus:function(){this.saveFocus(),this.setAsActive(!0)},getCategories:function(){return this.list.children("li:not(.separator)")},getSelectedCategories:function(){return this.getCategories().filter(".is-checked")},getCategoryData:function(t){var e=this.getCategories(),i=[];return e.each((function(){var e=this.classList.contains("is-checked");if(!0!==t||!1!==e){var n={name:this.innerText,checked:e};this.id&&(n.id=this.id);var s=this.getAttribute("data-value");void 0!==s&&(n.value=s),i.push(n)}})),i},updateCategories:function(t){this.settings.categories=t,this.setCategories(this.settings.categories)},setCategories:function(t){this.list.empty();var e=this,i=["string","number"],n=!1;t.forEach((function(t){if("string"==typeof t&&(t={name:t}),t.name){var s="";"string"==typeof t.id&&t.id.length&&(s=' id="'.concat(t.id,'"'));var a="";i.indexOf(l(t.value))>-1&&(a=' data-value="'.concat(t.value,'"'));var o="";!0===t.checked&&!0!==n&&(o=' class="is-checked"',e.settings.categoryMultiselect||(n=!0)),e.list.append("<li".concat(o).concat(s).concat(a,'><a href="#">').concat(t.name,"</a></li>"))}}));var s=this.categoryButton.data("popupmenu");s&&"function"==typeof s.updated&&s.updated()},expand:function(t){var e=this,i=this;return new Promise((function(n){if(i.isExpanded||i.isExpanding||i.isCollapsing)n();else{var s=!i.shouldBeFullWidth();i.isExpanding=!0,s||i.appendToParent(),i.wrapper.removeAttr("style"),i.input.removeAttribute("style");var a=0;i.buttonsetElem&&(a=parseInt(window.getComputedStyle(i.buttonsetElem).width,10));var o={buttonset:a+Qe};e.isContainedByFlexToolbar||e.wrapper[0].classList.add("is-open"),e.calculateOpenWidth(),e.setOpenWidth(),(!t||"ios"===x.os.name||i.isFocused&&document.activeElement!==i.input)&&(i.focusElem&&(i.focusElem=i.input),i.element.parents(".flex-toolbar").length||i.input.focus());var r=[];o&&r.push(o),i.element.trigger("beforeexpand"),$(i.toolbarParent).triggerHandler("recalculate-buttons",r);var l=new X({duration:10,updateCallback:function(){},timeoutCallback:function(){$(i.toolbarParent).triggerHandler("recalculate-buttons",r),i.element.trigger("expanded"),delete i.isExpanding,i.isExpanded=!0,i.settings.clearable&&i.xButton.removeClass("collapsed"),!i.isCurrentlyCollapsible||i.isFocused||i.focusElem||i.handleSafeBlur()}});J.register(l)}}))},collapse:function(){var t=this,e=this;return new Promise((function(i){if(e.isExpanded||!e.isExpanding||e.isCollapsing){e.isCollapsing=!0,e.wrapper.removeAttr("style"),e.appendToButtonset(),e.checkContents(),e.toolbarParent&&e.setInitalWidth(),e.clearResponsiveState(),e.wrapper.removeClass("is-open"),e.wrapper.removeClass("active"),x.browser.isIE11&&e.wrapper[0].classList.remove("is-open"),setTimeout((function(){!t.isContainedByFlexToolbar&&t.isFocused||e.wrapper[0].classList.remove("has-focus")}),300),e.categoryButton&&e.categoryButton.length&&e.categoryButton.data("popupmenu").close(!1,!0),e.element.trigger("beforecollapse"),delete e.isExpanded,delete e.isExpanding;var n=new X({duration:10,updateCallback:function(){},timeoutCallback:function(){delete e.isCollapsing,$(e.toolbarParent).triggerHandler("recalculate-buttons"),e.element.trigger("collapsed"),e.settings.clearable&&e.xButton.addClass("collapsed"),i()}});J.register(n)}else i()}))},collapseResponsive:function(t){if(this.previouslyCollapsedByKey&&"click"===t.type)delete this.previouslyCollapsedByKey;else{var e,i=this,n=1;if(t&&!t.key)n=0,this.toolbarFlexItem&&null!==(e=this.toolbarFlexItem)&&void 0!==e&&(null==e?void 0:e.focused)&&(n=1);"keydown"===t.type&&(this.previouslyCollapsedByKey=!0),this.collapse().then((function(){i.wrapper.trigger("collapsed-responsive",[n])}))}},get hasText(){return""!==(this.input.value?this.input.value.trim():"")},checkContents:function(){return this.wrapper[0].classList[this.hasText?"add":"remove"]("has-text")},setInitalWidth:function(){var t;this.settings.collapsible&&!this.shouldBeFullWidth()&&this.hasText&&(this.settings.collapseSize&&(t="function"==typeof this.settings.collapseSize?this.settings.collapseSize(this):parseInt(this.settings.collapseSize,10)),isNaN(t)?this.wrapper[0]&&(this.wrapper[0].style.width=""):this.wrapper[0]&&(this.wrapper[0].style.width="".concat(t,"px")))},addMoreLink:function(){var t=$("#autocomplete-list"),e=this.element.val();if(!($(".more-results",t).length>0)){var i=$('<li class="separator" role="presentation"></li>').appendTo(t),n=$('<li role="presentation"></li>').appendTo(t);this.moreLink=$('<a href="#" class="more-results" tabindex="-1" role="menuitem"></a>').html("<span>".concat(H.translate("AllResults"),' "').concat(g.ensureAlphaNumericWithSpaces(e),'"</span>')).appendTo(n),this.autocomplete.maxWidth&&i.width("".concat(this.autocomplete.maxWidth,"px"))}},addNoneLink:function(){var t=$("#autocomplete-list");if(!($(".no-results",t).length>0)){var e=$('<li role="presentation" class="is-placeholder"></li>').appendTo(t);this.noneLink=$('<a href="#" class="no-results" disabled="disabled" tabindex="-1" role="menuitem" aria-disabled="true"></a>').html("<span>".concat(H.translate("NoResults"),"</span>")).appendTo(e)}},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings),Object.prototype.hasOwnProperty.call(t,"collapseSize")&&void 0===t.collapseSize&&(this.settings.collapseSize=t.collapseSize)),this.teardown().init()},disable:function(){this.wrapper.addClass("is-disabled"),this.element.prop("disabled",!0)},enable:function(){this.wrapper.removeClass("is-disabled"),this.element.prop("disabled",!1)},teardown:function(){if(this.element.off(["updated.".concat(this.id),"focus.".concat(this.id),"blur.".concat(this.id),"click.".concat(this.id),"keydown.".concat(this.id),"beforeopen.".concat(this.id),"input.".concat(this.id),"listopen.".concat(this.id),"listclose.".concat(this.id),"safe-blur.".concat(this.id),"selected.".concat(this.id),"populated.".concat(this.id),"cleared.".concat(this.id)].join(" ")),this.wrapper.off(["mouseenter.".concat(this.id),"mouseleave.".concat(this.id),"focusin.".concat(this.id),"focusout.".concat(this.id),"keydown.".concat(this.id),"collapse.".concat(this.id)].join(" ")),this.toolbarParent&&$(this.toolbarParent).off("navigate.toolbarsearchfield"),this.goButton&&this.goButton.length&&(this.goButton.off("click.".concat(this.id," blur.").concat(this.id)).remove(),delete this.goButton),this.customButton&&this.customButton.length&&(this.customButton.off("click.".concat(this.id," blur.").concat(this.id)).remove(),delete this.customButton),this.categoryButton&&this.categoryButton.length){var t=this.categoryButton.data("popupmenu");t&&(t.destroy(),t.wrapper.off().remove()),this.categoryButton.off().remove(),delete this.categoryButton}return this.collapseButton&&this.collapseButton.length&&(this.collapseButton.off().remove(),delete this.collapseButton,delete this.previouslyCollapsedByKey),this.removeDocumentDeactivationEvents(),$("body").off("resize.".concat(this.id)),$("body").off("breakpoint-change.".concat(this.id)),this.autocomplete&&this.autocomplete.destroy(),this.wrapper.hasClass("context")&&this.element.addClass("context"),this.element.next(".icon").remove(),this.element.parent().hasClass("searchfield-wrapper")&&(this.element.parent().find("ul").remove(),this.element.parent().find(".icon").remove()),this.xButton&&this.xButton.length&&(this.xButton.off("blur.".concat(this.id)),x.browser.isSafari()&&(this.xButton.off("mouseover.clearable"),this.xButton.off("mouseleave.clearable")),this.xButton.remove()),this},makeClearable:function(){var t=this;this.element.clearable({tabbable:this.settings.tabbable}),this.wrapper.addClass("has-close-icon-button"),this.xButton=this.wrapper.children(".icon.close").length>0?this.wrapper.children(".icon.close"):this.wrapper.children("button.close"),this.settings.tabbable||this.xButton[0].setAttribute("tabindex","-1"),this.element.on("cleared",(function(){return t.checkContents()})),E.addAttributes(this.xButton,this,this.settings.attributes,"btn-close",!0),E.addAttributes(this.element.xButton,this,this.settings.attributes,"btn-close",!0)},clear:function(){this.xButton&&this.xButton.click(),this.element.xButton&&this.element.xButton.click()},destroy:function(){this.teardown(),$.removeData(this.element[0],Ye)}},$.fn.searchfield=function(t){return this.each((function(){var e=$.data(this,Ye);e?e.updated(t):e=$.data(this,Ye,new Je(this,t))}))};var Ze="applicationmenu",ti={breakpoint:"phone-to-tablet",dismissOnClickMobile:!1,filterable:!1,filterMode:"contains",resizable:!1,savePosition:!1,openOnLarge:!1,triggers:[".application-menu-trigger"],onExpandSwitcher:null,onCollapseSwitcher:null};function ei(t,e){return this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,ti),this.init()}ei.prototype={get accordionEl(){return this.accordion[0]},get searchEl(){return this.element[0].querySelector(".searchfield")},get searchAPI(){return this.searchEl?$(this.searchEl).data("searchfield"):void 0},get searchfieldAPI(){return this.searchAPI},init:function(){this.setup().handleEvents()},setup:function(){this.hasTrigger=!1,this.isAnimating=!1,this.element.find(".application-menu-footer").length&&this.element.addClass("has-menu-footer"),this.hasTriggers()||(this.triggers=$()),this.menu=this.element;var t=this.element.attr("data-open-on-large");this.settings.openOnLarge=void 0!==t?"true"===t:this.settings.openOnLarge;var e=this.element.attr("data-breakpoint");this.settings.breakpoint=void 0!==m[e]?e:this.settings.breakpoint,this.modifyTriggers(this.settings.triggers,!1,!0),this.scrollTarget=this.menu.parents(".header");var i=this.menu.prevAll(".masthead"),n=this.menu.prevAll(".module-tabs");i.length>0&&(this.scrollTarget=i,this.menu.addClass("short")),n.length>0&&(this.scrollTarget=n),this.element.find(".application-menu-switcher-panel .accordion").accordion(),this.accordion=this.menu.find(".accordion").last(),this.accordion.addClass("panel").addClass("inverse");var s=this.accordion.data("accordion");s||(this.accordion.accordion(),s=this.accordion.data("accordion")),this.accordionAPI=s,this.attachFilter(),this.renderResizableAppMenu(),this.isOpen()?this.openMenu(!1,!1,!0):this.adjustHeight();var a=this.element.find(".application-menu-switcher-trigger");a.length>0&&(this.switcherTrigger=a,this.switcherPanel=a.next(".expandable-area"),this.switcherPanel.data("expandablearea")||this.switcherPanel.expandablearea({trigger:a}));return this},getAdjacentContainerElement:function(){var t=this.element.next(".page-container");return t.length||(t=$("body")),t},handleTriggerEvents:function(){var t=this;this.triggers.length&&this.triggers.off("click.applicationmenu").on("click.applicationmenu",(function(e){return!$(e.currentTarget).find(".icon.app-header").hasClass("go-back")&&!t.isAnimating&&(t.menu.hasClass("is-open")?t.closeMenu(!0):t.openMenu(void 0,!0),!0)})),this.element.find(".searchfield").on("keydown.applicationmenu",(function(e){t.handleKeyDown(e)}))},handleKeyDown:function(t){return!(121===t.which&&!t.shiftKey)||(t.preventDefault(),this.isOpen()?(this.closeMenu(!0),this.triggers.length&&this.triggers.eq(0).focus()):this.openMenu(),!1)},notify:function(t,e){if(t&&void 0!==t&&(t instanceof HTMLElement&&(t=$(t)),t.is("a"))){var i=t.find(".tag");if(e&&void 0!==e&&0!==parseInt(e,10))return i.length||(i=$('<span class="tag"></span>').appendTo(t)),i.text(e.toString()),i;i.length&&i.remove()}},adjustHeight:function(){var t=this.scrollTarget.is(".is-sticky"),e=this.scrollTarget.outerHeight(!0)-(t?0:$(window).scrollTop());this.scrollTarget.prev().is(".masthead")&&(e+=this.scrollTarget.prev().outerHeight(!0)),this.menu[0].style.height=e>0?"calc(100% - ".concat(e,"px)"):"100%"},toggleScrollClass:function(){if("ie"===x.browser.name&&"11"===x.browser.version){var t=this.element[0];t&&t.classList.contains("has-menu-footer")&&(t.scrollHeight>t.clientHeight?t.classList.add("has-scrollbar"):t.classList.remove("has-scrollbar"))}},isLargerThanBreakpoint:function(){return m.isAbove(this.settings.breakpoint)},isOpen:function(){return this.menu[0].classList.contains("is-open")},testWidth:function(){if(this.isOpen()){if(m.isAbove(this.settings.breakpoint))return void this.element[0].classList.remove("show-shadow");if(this.element[0].classList.add("show-shadow"),this.isAnimating)return;this.userOpened||this.closeMenu()}else m.isBelow(this.settings.breakpoint)||this.userClosed||!this.settings.openOnLarge||this.isAnimating||this.openMenu(!0)},attachFilter:function(){Le.attachFilter.apply(this,[Ze])},attachFilterEvents:function(){Le.attachFilterEvents.apply(this,[Ze])},renderResizableAppMenu:function(){if(this.settings.resizable){this.openMenu(!1,!1,!1),this.element.after('<div class="resizer"></div>'),$("#application-menu, .resizer").wrapAll('<div class="resize-app-menu-container" />'),$(".page-container").wrapAll('<div class="resize-page-container" />'),$(".resize-page-container").insertAfter(".resizer");var t=document.querySelector(".resizer"),e=document.querySelector("#application-menu"),i=$(".tab-panel-container.page-container"),n=$(".page-container");if(t.addEventListener("mousedown",(function(t){t.preventDefault(),window.addEventListener("mousemove",a),window.addEventListener("mouseup",o)})),localStorage&&localStorage.navMenuWidth&&this.settings.savePosition&&(e.style.flexBasis=localStorage.navMenuWidth),this.settings.savePosition&&(n.css("width","calc(100% - ".concat(localStorage.navMenuWidth,")")),i&&i.css("width","calc(100% - ".concat(localStorage.navMenuWidth,")"))),$("html.is-safari")&&this.settings.resizable&&$(".resize-app-menu-container").css("display","flex"),!localStorage.navMenuWidth){var s=$(".resizer").position();n.css("width","calc(100% - ".concat(s.left<300?300:s.left,"px)"))}}function a(t){e.style.flexBasis="".concat((t.pageX<300?300:t.pageX)-e.getBoundingClientRect().left,"px"),$(".page-container").css("width","calc(100% - ".concat(t.pageX<300?300:t.pageX,"px)"))}function o(){window.removeEventListener("mousemove",a,!1),localStorage&&(localStorage.navMenuWidth=e.style.flexBasis)}},openMenu:function(t,e,i){if(!0!==this.isAnimating){var n=this,s=$.fn.transitonEndName;if(i||(this.isAnimating=!0),this.adjustHeight(),this.settings.resizable){var a=parseInt(this.element[0].style.flexBasis,10);(a<300||isNaN(a))&&(a=300),this.settings.savePosition?$(".page-container").css("width","calc(100% - ".concat(localStorage.navMenuWidth,")")):$(".page-container").css("width","calc(100% - ".concat(a,"px)"))}if(this.triggers.each((function(){var t=$(this);t.attr({"aria-expanded":!0}),(t.parents(".header").length>0||t.parents(".masthead").length>0)&&(t.find(".icon.app-header").removeClass("go-back"),t.trigger("icon-change"))})),i||(this.menu.off("".concat(s,".applicationmenu")),this.menu[0].style.display="",this.menu[0].offsetHeight,this.menu.addClass("is-open"),x.features.touch&&$("body").addClass("is-open-touch")),m.isBelow(this.settings.breakpoint)&&this.menu.addClass("show-shadow"),t&&!0===t||this.menu.find(".is-selected > a").focus(),"ios"===x.os.name)this.getAdjacentContainerElement().addClass("ios-click-target");i?o():(this.menu.one("".concat(s,".applicationmenu"),o),this.timeout=setTimeout(o,300)),setTimeout((function(){$(document).on("click.applicationmenu",(function(t){$(t.target).parents(".application-menu").length<1&&!n.isLargerThanBreakpoint()&&n.closeMenu(!0)}))}),0)}function o(){null!==n.timeout&&(clearTimeout(n.timeout),n.timeout=null),e&&(n.userOpened=!0,n.userClosed=void 0),i||(n.isAnimating=!1,n.element.trigger("applicationmenuopen"),$("body").triggerHandler("resize")),n.toggleScrollClass(),n.menu.removeClass("no-transition"),$(".page-container").removeClass("no-transition")}},closeMenu:function(t){if(!0!==this.isAnimating){var e=this,i=$.fn.transitionEndName();if(this.isAnimating=!0,this.triggers.each((function(){var t=$(this);t.attr({"aria-expanded":!1}),(t.parents(".header").length>0||t.parents(".masthead").length>0)&&(t.find(".icon.app-header"),t.trigger("icon-change"))})),"ios"===x.os.name)this.getAdjacentContainerElement().removeClass("ios-click-target");this.menu.one("".concat(i,".applicationmenu"),n),this.timeout=setTimeout(n,300),this.menu.removeClass("is-open show-shadow").find("[tabindex]"),$(document).off("click.applicationmenu"),this.settings.resizable&&$(".page-container").css("width","100%"),x.features.touch&&$("body").removeClass("is-open-touch")}function n(){null!==e.timeout&&(clearTimeout(e.timeout),e.timeout=null),e.menu.off("".concat(i,".applicationmenu")),e.menu[0].style.display="none",e.isAnimating=!1,t&&(e.userOpened=void 0,e.userClosed=!0),e.element.trigger("applicationmenuclose"),$("body").triggerHandler("resize")}},hasTriggers:function(){return void 0!==this.triggers&&this.triggers instanceof $&&this.triggers.length},modifyTriggers:function(t,e,i){var n=this;if(t&&t.length){var s=$(),a=!this.element.is(":visible");$.each(t,(function(t,e){s=s.add($(e))})),this.triggers=this.triggers[e?"not":"add"](s),this.handleTriggerEvents(),this.triggers.each((function(t,e){var i,s=$(e);s.attr("aria-controls","application-menu"),s.attr("aria-expanded","".concat(n.isOpen()));var a=null===(i=e.textContent)||void 0===i||null==i?void 0:i.length,o=s.find(".audible");if(!a){var r=H.translate("AppMenuTriggerTextAlt");o.length?o.text(r):e.insertAdjacentHTML("beforeend",'<span class="audible">'.concat(r,"</span>"))}})),i&&!0===i||(this.updated(),a&&this.element.css("display","none"))}},handleSearchfieldInputEvent:function(){Le.handleSearchfieldInputEvent.apply(this,[Ze])},handleDismissOnClick:function(){this.settings.dismissOnClickMobile&&(this.userOpened=!1,this.isLargerThanBreakpoint()||this.closeMenu())},closeSwitcherPanel:function(){if(this.switcherPanel){var t=this.switcherPanel.data("expandablearea");if(t&&(t.close(),this.settings.dismissOnClickMobile)){if(this.userOpened=!1,this.isLargerThanBreakpoint())return;this.closeMenu()}}},teardown:function(){var t;if(this.menu.off("animateopencomplete animateclosedcomplete").removeClass("short").removeAttr("style"),$(window).off("scroll.applicationmenu"),$("body").off("resize.applicationmenu"),$(document).off(["click.applicationmenu","open-applicationmenu","close-applicationmenu","dismiss-applicationmenu","keydown.applicationmenu","input.applicationmenu"].join(" ")),this.element.find(".expandable-area").off(["beforeexpand.applicationmenu","aftercollapse.applicationmenu","keydown.".concat(Ze,"-switcher")].join(" ")),this.accordion.off(["blur.applicationmenu","selected.applicationmenu","followlink.applicationmenu","afterexpand.applicationmenu","aftercollapse.applicationmenu"].join(" ")),this.element.find(".application-menu-toolbar").off("click.".concat(Ze)),this.accordionAPI&&"function"==typeof(null===(t=this.accordionAPI)||void 0===t||null==t?void 0:t.destroy)){var e,i;if(this.isFiltered)null===(i=this.accordionAPI)||void 0===i||null==i||i.collapse();null===(e=this.accordionAPI)||void 0===e||null==e||e.destroy()}return this.switcherPanel&&(this.switcherPanel.off(["beforeexpand.applicationmenu","aftercollapse.applicationmenu","afterexpand.".concat(Ze)].join(" ")),delete this.switcherTrigger,delete this.switcherPanel),this.teardownFilter(),this.hasTriggers()&&this.triggers.off("click.applicationmenu"),this},teardownFilter:function(){Le.teardownFilter.apply(this,[Ze])},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown().init()},destroy:function(){this.teardown(),$.removeData(this.element[0],Ze)},handleEvents:function(){var t=this,e=this;return this.handleTriggerEvents(),this.menu.on("notify.applicationmenu",(function(t,i,n){e.notify(i,n)})).on("updated.applicationmenu",(function(){e.updated()})),"ie"===x.browser.name&&"11"===x.browser.version&&e.element.find(".expandable-area").on("beforeexpand.applicationmenu",(function(){e.element[0].classList.remove("has-scrollbar")})).on("aftercollapse.applicationmenu",(function(){e.toggleScrollClass()})),this.accordion.on("blur.applicationmenu",(function(){e.closeMenu()})).on("selected.applicationmenu",(function(){e.handleDismissOnClick()})).on("followlink.applicationmenu",(function(){e.handleDismissOnClick()})).on("afterexpand.applicationmenu aftercollapse.applicationmenu",(function(){e.toggleScrollClass()})),this.switcherPanel&&this.switcherPanel.on("beforeexpand.applicationmenu",(function(){var i=t.element.height();e.element.addClass("has-open-switcher"),e.switcherPanel.find(".content").height(i-71),e.settings.onExpandSwitcher&&e.settings.onExpandSwitcher(e,e.element,e.settings),e.element.on("keydown.".concat(Ze,"-switcher"),(function(t){var i=t.key;"Escape"===i&&e.element[0].classList.contains("has-open-switcher")&&(t.preventDefault(),e.closeSwitcherPanel()),"Tab"===i&&(e.keyboardChangedFocus=!0)})),e.switcherPanel.one("afterexpand.".concat(Ze),(function(){var i;t.keyboardChangedFocus||((i=f.focusableElems(e.switcherPanel[0]))[0].classList.remove("hide-focus"),i[0].focus())}))})).on("aftercollapse.applicationmenu",(function(){t.element.removeClass("has-open-switcher"),t.settings.onCollapseSwitcher&&t.settings.onCollapseSwitcher(t,t.element,t.settings),t.element.off("keydown.".concat(Ze,"-switcher")),t.switcherPanel.off("afterexpand.".concat(Ze)),delete t.keyboardChangedFocus})),this.element.find(".application-menu-toolbar").on("click.".concat(Ze),"button",(function(e){e.defaultPrevented||t.handleDismissOnClick()})),$(document).on("open-applicationmenu",(function(t,i,n){e.openMenu(i,n)})).on("close-applicationmenu",(function(t,i){e.closeMenu(i)})).on("dismiss-applicationmenu",(function(){e.handleDismissOnClick()})),$(window).on("scroll.applicationmenu",(function(){e.adjustHeight()})),$("body").on("resize.applicationmenu",(function(){e.testWidth(),e.toggleScrollClass()})),this.attachFilterEvents(),this.settings.openOnLarge&&this.isLargerThanBreakpoint()&&(this.menu.addClass("no-transition"),$(".page-container").addClass("no-transition")),this.testWidth(),setTimeout((function(){e.menu.removeClass("no-transition"),$(".page-container").removeClass("no-transition")}),800),this}},$.fn.applicationmenu=function(t){return this.each((function(){var e=$.data(this,Ze);return e?e.updated(t):e=$.data(this,Ze,new ei(this,t)),e}))};var ii="blockgrid",ni={dataset:[],selectable:!1,paging:!1,withImage:!0,pagerSettings:{pagesize:25,pagesizes:[10,25,50,75],showFirstButton:!1,showLastButton:!1},attributes:null};function si(t,e){return e?(e.dataset&&(t.dataset=e.dataset),e.pagesize&&(V("`pagerSettings.pagesize` setting","`pagesize` setting"),t.pagerSettings.pagesize=e.pagesize,delete t.pagesize),e.pagesizes&&(V("`pagerSettings.pagesizes` setting","`pagesizes` setting"),t.pagerSettings.pagesizes=e.pagesizes,delete t.pagesizes),t):t}function ai(t,e){this.element=$(t),this.settings=E.mergeSettings(t,e,ni),this.settings=si(this.settings,e),this.init()}ai.prototype={get pagerAPI(){return this.element.data("pager")},init:function(){return this.selectedRows=[],this.handlePaging(),this.build().handleEvents()},build:function(){return this.settings.paging&&this.element.empty(),this.render(),this},handleEvents:function(){var t=this,e=this;return this.element.on("click.".concat(ii),".block",(function(t){var i=$(t.currentTarget),n=$(t.target),s=n.is(".checkbox-label")||n.is(".checkbox");setTimeout((function(){e.select(i,s)}),0),t.stopPropagation(),t.preventDefault()})),this.element.on("focus.".concat(ii),".checkbox",(function(t){$(t.currentTarget).parent().addClass("has-focus")})),this.element.on("blur.".concat(ii),".checkbox",(function(t){$(t.currentTarget).parent().removeClass("has-focus")})),this.element.on("keypress.".concat(ii),".block",(function(t){if(32===t.which){var i=$(t.target);e.select(i,!1)}})),this.element.on("updated.".concat(ii),(function(){e.updated()})),this.pagerAPI&&this.element.on("page.".concat(ii),(function(){t.previousFocusedElement=!0,t.build()})).on("pagesizechange.".concat(ii),(function(){t.previousFocusedElement=!0,t.build()})),this},handlePaging:function(){if(this.settings.paging){var t=E.extend({},this.settings.pagerSettings,{componentAPI:this,dataset:this.settings.dataset});this.element.addClass("paginated"),this.element.pager(t)}},selectBlock:function(t,e){return j(this.select,this.selectBlock).apply(this,[t,e])},select:function(t,e){var i=this.element.find(".block"),n=this.element.find(".checkbox"),s=t.find(".checkbox"),a=s.is(":checked"),o="",r=Number(s.data("idx"));if("single"===this.settings.selectable&&(this.selectedRows=[],i.removeClass("is-selected").removeAttr("aria-selected"),n.prop("checked",!1)),"multiple"===this.settings.selectable&&a||"mixed"===this.settings.selectable&&e&&a){t.removeClass("is-selected").removeAttr("aria-selected"),s.prop("checked",!1);for(var l=0;l<this.selectedRows.length;l++)r===this.selectedRows[l].idx&&this.selectedRows.splice(l,1);this.element.triggerHandler("deselected",[{selectedRows:this.selectedRows,action:"deselect"}])}else{if(!1===this.settings.selectable||"mixed"===this.settings.selectable&&!e||(a||(t.addClass("is-selected").attr("aria-selected","true"),s.prop("checked",!0)),this.selectedRows.push({idx:r,data:this.settings.dataset[r],elem:t}),o=a?"deselected":"selected"),"mixed"===this.settings.selectable&&!e){var c=t.hasClass("is-activated");return i.removeClass("is-activated"),void(c?(t.removeClass("is-activated"),this.element.triggerHandler("deactivated",[{row:r,item:this.settings.dataset[r]}])):(t.addClass("is-activated"),this.element.triggerHandler("activated",[{row:r,item:this.settings.dataset[r]}])))}this.element.triggerHandler(a?"deselected":"selected",[{selectedRows:this.selectedRows,action:o}])}},render:function(){var t=this,e=this.settings,i=H?H.translate("Select"):"Select",n="",s=e.dataset;if(this.pagerAPI){var a=this.pagerAPI.state;if(a.pages>1){var o=a.activePage>0?a.activePage-1:0,r=a.pagesize*o,l=a.pagesize*(o+1);if((s=s.slice(r,l)).length!==e.dataset.length)for(var c=0;c<s.length&&!s[c].id;c++)s[c].id=r+c}}var d=[];this.selectedRows.forEach((function(t){d.push(t.idx)}));for(var h=function(){var t,a=s[u],o="mixed"===e.selectable?"0":"-1",r="",l="";if(d.indexOf(a.id)>-1&&(r=" is-selected",l=" checked"),e.withImage){var c=a.imgAlt||a.imageAlt||"".concat(a.maintxt||a.title," ").concat(H.translate("Image"));t='<div class="block is-selectable'.concat(r,'" role="listitem" tabindex="0">\n          <input type="checkbox" aria-hidden="true" role="presentation" class="checkbox" id="checkbox').concat(u,'" tabindex="').concat(o,'" data-idx="').concat(a.id||u,'"').concat(l,'>\n          <label for="checkbox').concat(u,'" class="checkbox-label">\n            <span class="audible">').concat(i,'</span>\n          </label>\n          <img alt="').concat(c,'" src="').concat(a.img||a.image,'" class="image-round">\n          <p> ').concat(a.maintxt||a.title," <br> ").concat(a.subtxt||a.subtitle," </p>\n        </div>")}else{var h="";a.content&&a.content.forEach((function(t){h+="<br>".concat(t)})),t='<div class="block text-block is-selectable'.concat(r,'" role="listitem" tabindex="0">\n        <input type="checkbox" aria-hidden="true" role="presentation" class="checkbox" id="checkbox').concat(u,'" tabindex="').concat(o,'" data-idx="').concat(a.id||u,'"').concat(l,'>\n        <label for="checkbox').concat(u,'" class="checkbox-label text-block">\n          <span class="audible">').concat(i,"</span>\n        </label>\n        <p><b>").concat(a.title,"</b>").concat(h,"</p>\n        </div>")}n+=t},u=0;u<s.length;u++)h();if(this.element.attr("role","list").append(n),e.attributes)for(var p=0;p<s.length;p++){var g=this.element.find("#checkbox".concat(p)),f=this.element.find('.checkbox-label[for="checkbox'.concat(p,'"]'));E.addAttributes(g,this,e.attributes,"blockgrid-checkbox".concat(p)),E.addAttributes(f,this,e.attributes,"blockgrid-checkbox-label".concat(p))}this.previousFocusedElement&&setTimeout((function(){t.element.find(".block").first().focus(),delete t.previousFocusedElement}),0)},renderBlock:function(){return j(this.render,this.renderBlock).apply(this)},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this.settings=si(this.settings,t),this.teardown(),this.init(),this},teardown:function(){return this.element.off("updated.".concat(ii)),this.element.off("click.".concat(ii)),this.element.empty(),this.selectedRows=[],this},destroy:function(){this.teardown(),$.removeData(this.element[0],ii)}},$.fn.blockgrid=function(t){return this.each((function(){var e=$.data(this,ii);e?e.updated(t):e=$.data(this,ii,new ai(this,t))}))};var oi={attributes:null,callback:void 0,content:"",current:!1,disabled:!1,href:void 0,id:void 0};function ri(t){return this.settings=E.mergeSettings(null,t,oi),this.init()}ri.prototype={init:function(){return this.settings.element instanceof HTMLElement&&this.getSettingsFromElement(),this.settings.callback="function"==typeof this.settings.callback?this.settings.callback:null,this.settings.content="string"==typeof this.settings.content?g.stripHTML(this.settings.content):"",this.settings.current=!0===this.settings.current,this.settings.disabled=!0===this.settings.disabled,this.settings.id="string"==typeof this.settings.id?g.stripHTML(this.settings.id):void 0,this.settings.href="string"==typeof this.settings.href?g.stripHTML(this.settings.href):void 0,this},render:function(){var t,e,i=this.element;return i||(i=document.createElement("li"),this.settings.href?(t=document.createElement("a"),i.appendChild(t)):(e=document.createElement("span"),i.appendChild(e)),this.element=i),i.classList.add("breadcrumb-item"),i},refresh:function(){var t=this.element,e=t.querySelector("a"),i=t.querySelector("span:not(.audible)"),n=e?$(e):void 0;if(this.disabled=this.settings.disabled,this.current=this.settings.current,(e||t).id="string"==typeof this.settings.id?this.settings.id:"",(e||i||t).innerHTML="string"==typeof this.settings.content?this.settings.content:"",e){if("string"==typeof this.settings.href){var s="string"==typeof this.settings.href?g.stripHTML(this.settings.href):void 0;e.href=s,e.setAttribute("href",s)}else e.href=void 0,e.removeAttribute("href");n.hyperlink()}return i&&i.parentNode.isEqualNode(t)&&i.classList.add("breadcrumb-text"),Array.isArray(this.settings.attributes)&&E.addAttributes(n||$(t),this,this.settings.attributes),t},checkFocus:function(){var t=this.a;if(t){var e=this.overflowed;(x.browser.isIE11||x.browser.isEdge)&&(e=!1),t.tabIndex=e||this.disabled?-1:0,t.setAttribute("tabindex",t.tabIndex)}},get a(){return this.element.querySelector("a")},set current(t){var e=!0===t;this.settings.current=e,this.element.classList[e?"add":"remove"]("current")},get current(){return this.settings.current},set disabled(t){var e=!0===t;this.settings.disabled=e,this.element.classList[e?"add":"remove"]("is-disabled");var i=this.a;i&&(i.disabled=e,e?(i.setAttribute("disabled",e),i.setAttribute("aria-disabled",e)):(i.removeAttribute("disabled"),i.removeAttribute("aria-disabled")))},get disabled(){var t,e=null===(t=this.a)||void 0===t||null==t?void 0:t.getAttribute("disabled"),i=this.element.getAttribute("is-disabled");return e||i},disable:function(){this.disabled=!0},enable:function(){this.disabled=!1},get overflowed(){var t=this.element,e=t.querySelector("a"),i=t.querySelector("span"),n=(e||i||t).getBoundingClientRect(),s=this.element.parentNode.getBoundingClientRect();return x.rtl?s.right<n.right:s.left>n.left},callback:function(t){if("function"!=typeof this.settings.callback)return!1;for(var e=this.settings.callback,i=arguments.length,n=new Array(i>1?i-1:0),s=1;s<i;s++)n[s-1]=arguments[s];var a=e.apply(this,[t].concat(n));return a||t.preventDefault(),a},get index(){for(var t=this.element,e=0;null!==t.previousSibling;)t=t.previousSibling,e++;return e},getSettingsFromElement:function(){var t=this.settings.element,e=t.querySelector("a");function i(t){return t=t.replace('<span class="audible">Current</span>',""),g.stripHTML(t)}e?(this.settings.content=i(e.innerHTML),this.settings.href=e.href,this.settings.id=e.id):(this.settings.content=i(t.innerHTML),t.id&&(this.settings.id=t.id));var n=!0===t.classList.contains("current"),s=e&&!0===e.classList.contains("current");this.settings.current=n||s,this.settings.disabled=!0===t.classList.contains("is-disabled"),this.element=t,this.fromElement=!0,delete this.settings.element},destroy:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];if(this.element&&this.element.parentNode){if(!this.fromElement){var e=this.a;if(e){var i=$(e);i.off(),i.data("hyperlink").destroy()}}!t&&this.fromElement||(this.element.parentNode.removeChild(this.element),delete this.element)}}};var li="breadcrumb",ci=["default","alternate"],di={breadcrumbs:[],style:ci[0],truncate:!0};function hi(t,e){if(!(t instanceof HTMLElement))throw new Error("`element` argument must be of type `HTMLElement`");return this.element=t,this.settings=E.mergeSettings(t,e,di),this.init(),this}hi.prototype={init:function(){var t=this;Array.isArray(this.breadcrumbs)||(this.breadcrumbs=[]);var e=this.element.querySelector("ol"),i=E.getArrayFromList(this.element.querySelectorAll("li"));e&&(this.list=e),i.length&&i.forEach((function(e){t.add({element:e})})),Array.isArray(this.settings.breadcrumbs)||(this.settings.breadcrumbs=[]),this.settings.breadcrumbs.forEach((function(e){t.add(e)})),this.element.classList.contains(ci[1])&&(this.settings.style=ci[1]),-1===ci.indexOf(this.settings.style)&&(this.settings.style=ci[0]),this.render()},render:function(){var t;this.list?(this.teardownEvents(),this.teardownBreadcrumbs()):(this.list=document.createElement("ol"),this.element.appendChild(this.list));var e=document.createDocumentFragment();if(this.breadcrumbs.forEach((function(t){if(!t.fromElement||!t.element){var i=t.render();e.appendChild(i)}})),this.canDetectResize){if(!this.overflowBtn){var i=document.createElement("div"),n=document.createElement("button"),s=document.createElement("ul"),a=document.createElement("span");i.classList.add("breadcrumb-overflow-container"),n.classList.add("btn-actions"),n.classList.add("overflow-btn"),a.innerText="More Breadcrumbs",a.classList.add("audible"),n.insertAdjacentHTML("afterbegin",$.createIcon({icon:"more"})),n.appendChild(a),i.appendChild(n),i.appendChild(s),this.overflowContainerElem=i,this.overflowBtn=n,this.overflowMenu=s}this.element.insertBefore(this.overflowContainerElem,this.list),$(this.overflowBtn).popupmenu({menu:$(this.overflowMenu),beforeOpen:function(t){var e="";this.overflowed.forEach((function(t,i){var n="",s="";t.settings.disabled&&(n=" is-disabled",s=" disabled");var a='<li class="'.concat(n,'">\n          <a href="#" data-breadcrumb-index="').concat(i,'"').concat(s,">").concat(t.settings.content||"","</a>\n        </li>");e+=a})),t(e)}.bind(this)})}null!==(t=e.childNodes)&&void 0!==t&&(null==t?void 0:t.length)&&this.list.appendChild(e),this.list.setAttribute("aria-label","Breadcrumb"),this.element.classList.add("breadcrumb"),this.list.classList.add("breadcrumb-list"),this.refresh(),this.handleEvents()},add:function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],i=arguments.length>2&&void 0!==arguments[2]&&arguments[2];if(!t)throw new Error("Settings for a new breadcrumb item must be provided.");this.breadcrumbs.forEach((function(e){if(t.id&&e.id===t.id)throw new Error("New breadcrumbs must have a unique ID attribute.")}));var n=new ri(t);this.breadcrumbs.push(n),i&&this.settings.breadcrumbs.push(t),e&&(this.render(),!0===n.settings.current&&this.makeCurrent(n))},remove:function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=this.getBreadcrumbItemAPI(t);n.api.destroy(!0),this.breadcrumbs.splice(n.i,1),i&&this.settings.breadcrumbs.splice(n.i,1),e&&this.render()},removeAll:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0],e=arguments.length>1&&void 0!==arguments[1]&&arguments[1];this.breadcrumbs.forEach((function(t){t.destroy(!0)})),this.breadcrumbs=[],e&&(this.settings.breadcrumbs=[]),t?this.render():this.refresh()},makeCurrent:function(t){var e=this.getBreadcrumbItemAPI(t);this.breadcrumbs.forEach((function(t){var i=t.element;t.current=i.isEqualNode(e.api.element)}))},get current(){var t,e;return this.breadcrumbs.forEach((function(i){!t&&i.current&&(e=(t=i).element)})),e},set disabled(t){var e=!0===t;this.element.classList[e?"add":"remove"]("is-disabled"),this.breadcrumbs.forEach((function(t){t.disabled=e}))},disable:function(){this.disabled=!0},enable:function(){this.disabled=!1},get overflowed(){return this.breadcrumbs.filter((function(t){return t.overflowed}))},get canDetectResize(){return this.settings.truncate&&x.features.resizeObserver},handleEvents:function(){var t=this,e=this;$(this.list).on("click.".concat(li),"li",(function(e){var i=t.getBreadcrumbItemAPI(e.currentTarget);if(i&&i.api){for(var n=arguments.length,s=new Array(n>1?n-1:0),a=1;a<n;a++)s[a-1]=arguments[a];i.api.callback(e,s)}})),this.canDetectResize&&(this.previousSize=this.list.getBoundingClientRect(),this.ro=new ResizeObserver((function(){t.detectCheck&&t.detectCheck.destroy(!0),t.detectCheck=new X({duration:1,timeoutCallback:function(){var t=e.list.getBoundingClientRect();t.width!==e.previousSize.width&&(e.previousSize=t,e.refresh(),delete e.detectCheck)}}),J.register(t.detectCheck)})),this.ro.observe(this.list)),this.overflowBtn&&$(this.overflowBtn).on("selected.".concat(li),(function(e){var i=(arguments.length<=1?void 0:arguments[1])[0].getAttribute("data-breadcrumb-index"),n=t.overflowed[Number(i)];$(n.a).trigger("click")})),this.hasEvents=!0},getBreadcrumbItemAPI:function(t){var e,i,n,s=!1;if(t instanceof ri?(e=t,i=t.element,n=this.breadcrumbs.indexOf(t)):t instanceof HTMLAnchorElement?i=t.parentNode:t instanceof HTMLLIElement?i=t:!isNaN(t)&&t>-1&&(s=!0,n=t,e=this.breadcrumbs[n],i=e.element),!i||e||n||this.breadcrumbs.forEach((function(t,s){t.element.isEqualNode(i)&&(e=t,n=s)})),!e){var a=s?n:i;throw new Error("No matching Breadcrumb was found by ".concat(s?"at index ":" with matching HTML List Item",' "').concat(a,'"'))}return{api:e,li:i,i:n}},refresh:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];t&&this.teardownEvents(),this.breadcrumbs.forEach((function(t){t.refresh()})),this.element.classList["alternate"===this.settings.style?"add":"remove"]("alternate"),this.canDetectResize?this.element.classList[this.overflowed.length||this.element.scrollHeight>55?"add":"remove"]("truncated"):(this.element.classList.remove("truncated"),this.element.classList.add("no-truncate")),this.breadcrumbs.forEach((function(t){t.checkFocus()})),t&&this.handleEvents()},teardown:function(){if(this.teardownEvents(),this.teardownBreadcrumbs(),this.breadcrumbs=[],this.overflowContainerElem){var t=$(this.overflowBtn).data("popupmenu");t&&t.destroy(),this.overflowContainerElem.parentNode&&this.element.removeChild(this.overflowContainerElem)}return this},teardownEvents:function(){this.hasEvents&&(this.ro&&(this.ro.disconnect(),delete this.ro),this.overflowBtn&&$(this.overflowBtn).off(["beforeopen.".concat(li),"selected.".concat(li)].join(" ")),$(this.list).off(),$(this.element).off(),delete this.hasEvents)},teardownBreadcrumbs:function(){this.breadcrumbs.forEach((function(t){t.destroy()}))},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown(),this.init(),this},destroy:function(){this.teardown(),$.removeData(this.element,li)}},$.fn.breadcrumb=function(t){return this.each((function(){var e=$.data(this,li);e?e.updated(t):e=$.data(this,li,new hi(this,t))}))};var ui="busyindicator",pi={blockUI:!0,text:null,displayDelay:1e3,timeToComplete:0,transparentOverlay:!1,overlayOnly:!1,attributes:null};function gi(t,e){this.settings=E.mergeSettings(t,e,pi),this.element=$(t),this.init()}gi.prototype={init:function(){this.inlineLabel=this.element.closest("label"),this.inlineLabelText=this.inlineLabel.find(".label-text"),this.isInlineLabel=this.element.parent().is(".inline"),this.setup().handleEvents()},setup:function(){var t=this.element.attr("data-block-ui"),e=this.element.attr("data-display-delay"),i=this.element.attr("data-completion-time");this.blockUI=void 0!==t?t:this.settings.blockUI,this.settings.overlayOnly||(this.loadingText=this.settings.text?this.settings.text:H.translate("Loading"),this.label&&this.label.text(this.loadingText));var n=void 0!==e&&!isNaN(e)&&parseInt(e,10)>20,s=!isNaN(this.settings.displayDelay)&&this.settings.displayDelay>=20?this.settings.displayDelay:20;return this.delay=n?e:s,this.completionTime=void 0===i||isNaN(i)?this.settings.timeToComplete:parseInt(i,10),this},activate:function(){var t=this;if(!t.isActive()){if(this.container)return t.closeTimeout&&clearTimeout(t.closeTimeout),this.label.remove(),this.settings.overlayOnly||(this.label=$("<span>".concat(this.loadingText,"</span>")).appendTo(this.container),this.element.is("input, .dropdown, .multiselect, .busy-xs, .busy-sm")&&this.label.addClass("audible")),void this.container.removeClass("is-hidden").trigger("afterstart");this.container=$('<div class="busy-indicator-container is-hidden"></div>').attr({"aria-live":"polite",role:"status"}),this.loader=$('<div class="busy-indicator active"></div>').appendTo(this.container),this.settings.overlayOnly||($('<div class="bar one"></div><div class="bar two"></div><div class="bar three"></div><div class="bar four"></div><div class="bar five"></div>').appendTo(this.loader),this.label=$("<span>".concat(this.loadingText,"</span>")).appendTo(this.container));var e="";if(this.blockUI)this.settings.transparentOverlay&&(e=" transparent"),this.originalPositionProp=this.element[0].style.position,"absolute"!==window.getComputedStyle(this.element[0],null).getPropertyValue("position")&&(this.element[0].style.position="relative"),this.overlay=$('<div class="overlay busy is-hidden'.concat(e,'"></div>')).appendTo(this.element),this.container.addClass("blocked-ui");if(this.label&&this.element.is(".busy-xs, .busy-sm")&&this.label.addClass("audible"),this.element.is("input, .dropdown, .multiselect")){var i;if(this.element.addClass("is-loading"),this.blockUI&&this.element.addClass("is-blocked"),this.label&&this.label.addClass("audible"),this.element.is("input"))i=this.element,this.container.insertAfter(this.isInlineLabel?this.inlineLabel:this.element);else i=this.element.data("dropdown").pseudoElem,this.container.appendTo(i.parent());this.overlay&&this.overlay.insertAfter(this.container);for(var n=i.position(),s=i.outerHeight(),a=i.outerWidth(),o=this.container.add(this.overlay),r=function(t,e,i){t.style[e]="".concat(i,"px")},l=0,c=o.length;l<c;l++)r(o[l],"left",n.left),r(o[l],"top",n.top),r(o[l],"bottom",n.bottom),r(o[l],"right",n.right),r(o[l],"height",s),r(o[l],"width",a)}else this.container.appendTo(this.element);setTimeout((function(){t.container&&t.container.removeClass("is-hidden"),t.overlay&&t.overlay.removeClass("is-hidden"),t.blockUI&&t.addScrollParent()}),t.delay),this.element.trigger("afterstart"),$.fn.cssPropSupport("animation")||(t.isAnimating=!0,t.animateWithJS()),this.completionTime>0&&setTimeout((function(){t.element.trigger("complete")}),t.completionTime),E.addAttributes(this.container,this,this.settings.attributes),E.addAttributes(this.container.find(".busy-indicator.active"),this,this.settings.attributes,"busyindicator"),E.addAttributes(this.container.find("span"),this,this.settings.attributes,"text"),E.addAttributes(this.overlay,this,this.settings.attributes,"overlay")}},close:function(t){var e=this;e.isActive()&&(this.removeScrollParent(),t&&(this.element.removeClass("is-loading"),this.element.removeClass("is-blocked")),this.container&&this.container.addClass("is-hidden"),this.overlay&&this.overlay.addClass("is-hidden"),e.closeTimeout=setTimeout((function(){clearTimeout(e.closeTimeout),e.container&&e.container.remove(),e.container=void 0,e.loader=void 0,e.label=void 0,e.overlay&&(e.overlay.remove(),e.element[0].style.position=e.originalPositionProp,e.originalPositionProp=void 0),e.overlay=void 0,e.element.trigger("aftercomplete.busyindicator"),e.element.off("complete.busyindicator"),e.element.trigger("hide.busyindicator")}),600))},animateWithJS:function(){var t=this,e=this.container.find(".bar.one"),i=this.container.find(".bar.two"),n=this.container.find(".bar.three"),s=this.container.find(".bar.four"),a=this.container.find(".bar.five"),o=0;setInterval((function(){t.isAnimating?(1===(o+=1)&&e.addClass("half"),13===o&&(e.removeClass("half").addClass("full"),i.addClass("half")),26===o&&(e.removeClass("full").addClass("half"),i.removeClass("half").addClass("full"),n.addClass("half")),39===o&&(e.removeClass("half"),i.removeClass("full").addClass("half"),n.removeClass("half").addClass("full"),s.addClass("half")),51===o&&(i.removeClass("half"),n.removeClass("full").addClass("half"),s.removeClass("half").addClass("full"),a.addClass("half")),64===o&&(n.removeClass("half"),s.removeClass("full").addClass("half"),a.removeClass("half").addClass("full")),77===o&&(s.removeClass("half"),a.removeClass("full").addClass("half")),90===o&&a.removeClass("half"),103===o&&(o=0)):clearInterval(null)}),10)},addScrollParent:function(){var t=this;if(this.blockUI){var e=this.element[0];this.scrollParent=$(this.getScrollParent(e));var i,n=this.scrollParent[0],s=n?n.offsetHeight:0;this.overlay&&this.overlay[0]&&(i=window.getComputedStyle(this.overlay[0],null).getPropertyValue("position"));var a="absolute"===i;if(s&&this.container&&s<=e.offsetHeight){var o=window.innerHeight||document.documentElement.clientHeight,r=s/2-58,l=function(){if(a&&t.element.is(n)){var i=e.scrollTop;t.overlay.css({top:i}),t.container.css({top:r+i})}else if(E.isInViewport(n)&&((h=n).scrollHeight>=h.clientHeight&&h.clientHeight<o-60))t.container[0].offsetHeight&&t.container.css({top:r+n.scrollTop});else{var s=0,l=0,c=e.getBoundingClientRect();if(c.top<0){var d=c.height+c.top;s=d>o?o:d,l=c.top}else s=o-c.top;t.container.css({top:(s-85)/2-l})}var h};l(),this.scrollParent.off("scroll.parent.busyindicator").on("scroll.parent.busyindicator",(function(){t.container&&l()}))}}},removeScrollParent:function(){this.scrollParent&&(this.scrollParent.off("scroll.parent.busyindicator"),delete this.scrollParent)},getScrollParent:function(t){var e=["overflow","overflow-x","overflow-y"],i=function(t){return e.reduce((function(e,i){return e+function(t,e){return getComputedStyle(t,null).getPropertyValue(e)}(t,i)}),0)},n=/(auto|scroll)/,s=function(t){return n.test(i(t))};return function(t){for(var e=!1,i=t;!e&&i&&i.tagName&&"body"!==i.tagName.toLowerCase();){if(s(i)){e=!0;break}i=i.parentNode}return e?i:null}(t)},updated:function(t){return t.text&&(t.text=g.unescapeHTML(t.text)),this.settings=E.mergeSettings(this.element,t,this.settings),this.setup()},isActive:function(){return!!this.container&&this.container.children(".busy-indicator").is(".active")},destroy:function(){this.removeScrollParent(),this.close(!0),this.element.off("start.busyindicator complete.busyindicator afterstart.busyindicator aftercomplete.busyindicator updated.busyindicator"),$.removeData(this.element[0],ui)},handleEvents:function(){var t=this;return this.element.on("start.busyindicator",(function(e){e.stopPropagation(),t.activate()})).on("afterstart.busyindicator",(function(){t.element.on("complete.busyindicator",(function(e){e.stopPropagation(),t.close(!0)}))})).on("updated.busyindicator",(function(){t.close(!0),t.updated()})),this}},$.fn.busyindicator=function(t){return this.each((function(){var e=$.data(this,ui);e?e.updated(t):e=$.data(this,ui,new gi(this,t))}))},$.fn.selectable=function(){var t=this;return t.hideFocus(),t.on("click",(function(e){t.removeClass("is-selected"),$(e.currentTarget).toggleClass("is-selected")})),this};var fi="cards",mi={bordered:null,noHeader:!1,contentPaddingX:null,contentPaddingY:null,noShadow:!1,dataset:[],template:null,selectable:!1,expandableHeader:!1,verticalButtonAction:!1,detailRefId:void 0,attributes:null};function vi(t,e){this.settings=E.mergeSettings(t,e,mi),this.element=$(t),this.init()}vi.prototype={init:function(){this.selectedRows=[],this.showCardDetailFlag=!1,this.setup().build().handleEvents()},setup:function(){return this.id=this.element.attr("id"),this.id&&void 0!==this.id||(this.id="expandable-card-".concat($("body").find(".card, .widget").index(this.element)),this.element.attr("id",this.id)),this.settings.expandableHeader&&(this.element.addClass("expandable-card"),this.expandableCardHeader=this.element.children(".card-header, .widget-header").addClass("expandable-card-header")),this.cardHeader=this.element.children(".card-header, .widget-header"),this.cardContentPane=this.element.children(".card-pane, .widget-pane"),this.buttonAction=this.cardHeader.children(".btn-actions"),!1!==this.settings.selectable&&(this.cards=this.element),this},build:function(){var t,e,i=this,n=this.element,s=this.element.hasClass("is-expanded"),a=H?H.translate("Select"):"Select",o="single"===this.settings.selectable,r=!0===this.settings.bordered,l=!1===this.settings.bordered,c=this.element.find(".card-header-section.custom-action, .widget-header-section.custom-action").length>0,d=this.settings,h=d.contentPaddingX,u=d.contentPaddingY;(this.renderTemplate(),!1!==this.settings.selectable&&(this.cards.find(".card").addClass("is-selectable"),this.cards.addClass(o?"single":"multiple"),this.element.attr("role","list"),this.element.find(".card, .widget").attr({role:"listitem",tabindex:"0"})),"multiple"===this.settings.selectable)&&this.cards.find(".card, .widget").each((function(t){$(this).find(".card-content, .widget-content").prepend('\n          <input type="checkbox" id="checkbox-'.concat(t,'" aria-hidden="true" tabindex="0" role="presentation"\n            class="checkbox">\n            <label for="checkbox-').concat(t,'" class="checkbox-label">\n              <span class="audible">').concat(a,"</span>\n            </label>\n        "))}));(this.cardContentPane.attr({id:"".concat(this.id,"-content"),role:"region","aria-labelledby":"".concat(this.id,"-header")}),this.settings.expandableHeader&&this.expandableCardHeader)&&(null===(t=this.expandableCardHeader)||void 0===t||null==t||t.attr("role","button"));(this.buttonAction.length>0&&this.settings.verticalButtonAction&&this.buttonAction.addClass("vertical"),s&&(this.cardContentPane.addClass("no-transition"),this.element.one("afterexpand.expandable-card",(function(){i.cardContentPane.removeClass("no-transition")})),this.open()),s||(this.cardContentPane.addClass("no-transition"),this.element.one("aftercollapse.expandable-card",(function(){i.cardContentPane.removeClass("no-transition")})),this.close()),this.settings.expandableHeader||E.addAttributes(this.cardHeader,this,this.settings.attributes,"header",!0),E.addAttributes(this.element,this,this.settings.attributes,"card",!0),E.addAttributes(this.element.find(".card .card-content"),this,this.settings.attributes,"card-content",!0),E.addAttributes(this.element,this,this.settings.attributes,"widget",!0),E.addAttributes(this.element.find(".card .card-content"),this,this.settings.attributes,"widget-content",!0),this.settings.selectable||(this.expandableCardHeader&&E.addAttributes(this.expandableCardHeader,this,this.settings.attributes,"expander",!0),E.addAttributes(this.buttonAction,this,this.settings.attributes,"action",!0),E.addAttributes(this.cardContentPane,this,this.settings.attributes,"content",!0)),r&&n.addClass("bordered"),l&&n.addClass("border-less"),this.settings.noHeader)&&(n.addClass("no-header"),null===(e=this.cardHeader)||void 0===e||null==e||e.remove());if(this.settings.noShadow&&n.addClass("no-shadow"),null!==h||null!==u){var p=n.find(".card-content, .widget-content");null!==h&&p.addClass("padding-x-".concat(h)),null!==u&&p.addClass("padding-y-".concat(u))}if(c&&n.addClass("show-buttons"),"multiple"===this.settings.selectable){var g=this;setTimeout((function(){i.cards.find(".card, .widget").each((function(t){var e=$(this);E.addAttributes(e.find(".checkbox-label"),g,g.settings.attributes,"checkbox-label-".concat(t),!0),E.addAttributes(e.find(".card-content .checkbox"),g,g.settings.attributes,"checkbox-".concat(t),!0)}))}),1)}return H.isRTL()&&this.removeInfoIconTooltip(),this.resizeButtonset(),this},removeInfoIconTooltip:function(){if(this.element.hasClass("card")){var t=this.element.width()<=360;this.cardHeader.find("> .icon").css("display","".concat(t?"none":""))}},select:function(t){var e=this.element.find(".card"),i=t.find(".checkbox"),n=t.hasClass("is-selected"),s="";if("single"===this.settings.selectable&&(this.selectedRows=[],e.removeClass("is-selected").removeAttr("aria-selected")),"multiple"===this.settings.selectable&&n)return t.removeClass("is-selected").removeAttr("aria-selected"),i.prop("checked",!1),void this.element.triggerHandler("deselected",[{selectedRows:this.selectedRows,action:"deselect"}]);!1!==this.settings.selectable&&(t.addClass("is-selected").attr("aria-selected","true"),"multiple"===this.settings.selectable&&i.prop("checked",!0),this.selectedRows.push({data:this.settings.dataset,elem:t}),s=n?"deselected":"selected"),this.element.triggerHandler(n?"deselected":"selected",[{selectedRows:this.selectedRows,action:s}])},renderTemplate:function(){if(this.settings.template){var t=this.settings;"object"===l(je)&&t.dataset&&this.settings.template&&(this.settings.template instanceof $?this.settings.template="".concat(this.settings.template.html()):"string"==typeof this.settings.template&&(M.containsHTML(this.settings.template)||(this.settings.template=$("#".concat(this.settings.template)).html())));var e=je.compile(this.settings.template,{dataset:t.dataset});t.dataset.length>0?this.cards.html(e):0===t.dataset.length&&this.cards.html(e||'<div class="cards auto-height"></div>')}},resizeButtonset:function(){var t=this.element.find(".card-buttonset");if(0!==t.length&&this.element.parent().is("li")){for(var e=this.element.width()<=360,i=function(t,e,i){var n;if(t.hasClass("is-overflowed")){var s=t.find(".card-content-action");s?n=t.find(".card-content-action button"):(s=$('<div class="card-content-action"></div>'),n=$('<button class="btn-actions" type="button"></button>').html("".concat($.createIcon({icon:"more"}),'<span class="audible">').concat(H.translate("MoreActions"),"</span>")).attr("title",H.translate("More")),s.append(n),t.append(s))}else{t.addClass("is-overflowed");var a=$('<div class="card-content-action"></div>');n=$('<button class="btn-actions" type="button"></button>').html("".concat($.createIcon({icon:"more"}),'<span class="audible">').concat(H.translate("MoreActions"),"</span>")).attr("title",H.translate("More")),a.append(n),t.append(a)}var o,r=n.data("popupmenu");r?o=r.settings.menu:(o=$('<ul class="popupmenu"></ul>')).insertAfter(n);for(var l=function(){var t=e[c],i=$(t.innerHTML),n=$('<li><a href="#"></a></li>');n.find("a").append(i),n.data("originalButton",t),n.on("click",(function(){return $(t).triggerHandler("click")})),n.find(".audible").removeClass("audible"),o.prepend(n),$(t).hide()},c=e.length-1,d=0;c>=0&&d<i;c--,d++)l();r?r.updated({menu:o}):n.popupmenu({menu:o})},n=t.children(":visible:not(.card-content-action)"),s=0,a=0,o=0,r=0;r<n.length;r++){var l=$(n.get(r)).hasClass("btn-icon")?1:2;if(a+l>4)break;if(1===l){if(l+o>3)break;o++}a+=l,s=r}if(s++,e)i(t,n,n.length);else if(n.length>s)i(t,n,n.length-s);else if(t.hasClass("is-overflowed")&&a<3){var c=this.element.find(".card-content-action button"),d=c.data("popupmenu"),h=c.parent();if(d&&d.settings.menu){for(var u=d.settings.menu.children(),p=0,g=0;g<u.length;g++){var f=$($($(u).get(g)).data("originalButton")),m=f.hasClass("btn-icon")?1:2;if(a+m>4)break;if(1===m){if(m+o>3)break;o++}f.show(),a+=m,p=g}p++;for(var v=0;v<p;v++)$(u).get(v).remove();0===$(d.settings.menu.children()).length?(d.destroy(),c.remove(),h.remove(),t.removeClass("is-overflowed")):d.updated()}}}},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this},isExpanded:function(){return this.element.is(".is-expanded")},toggleExpanded:function(){this.isExpanded()?this.close():this.open()},isDisabled:function(){return this.element.hasClass("is-disabled")},open:function(){var t,e=this;!1!==this.element.triggerHandler("beforeexpand",[this.element])&&(this.element.addClass("is-expanded"),null===(t=this.expandableCardHeader)||void 0===t||null==t||t.attr("aria-expanded","true"),this.element.triggerHandler("expand",[this.element]),this.cardContentPane[0]&&(this.cardContentPane[0].style.display="block"),this.cardContentPane.one("animateopencomplete",(function(){e.element.triggerHandler("afterexpand",[e.element])})).animateOpen({timing:300}))},close:function(){var t=this;!1!==this.element.triggerHandler("beforecollapse",[this.element])&&(this.element.triggerHandler("collapse",[this.element]),this.cardContentPane.one("animateclosedcomplete",(function(){var e;t.element.removeClass("is-expanded"),null===(e=t.expandableCardHeader)||void 0===e||null==e||e.attr("aria-expanded","false"),t.element.triggerHandler("aftercollapse",[t.element]),t.cardContentPane[0].style.display="none"})).animateClosed({timing:300}))},showCardDetail:function(){this.showCardDetailFlag=!0,this.showCardDetailFlag&&this.element.addClass("show-card-detail"),this.addBackButton()},hideCardDetail:function(){this.showCardDetailFlag=!1,this.element.removeClass("show-card-detail"),this.cardHeader.removeClass("has-back-button"),this.cardHeader.find(".go-back-button").remove()},addBackButton:function(){if(this.cardHeader){this.cardHeader.addClass("has-back-button"),this.cardHeader.prepend('\n      <div class="widget-header-section go-back-button">\n        <button id="go-back" class="btn-icon go-back btn-system" type="button">\n          <span class="audible">Show navigation</span>\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-arrow-left"></use>\n          </svg>\n        </button>\n      </div>\n      ')}},handleEvents:function(){var t,e=this,i=this,n=$(i.element).find(".card-content, .widget-content"),s="#".concat(i.settings.detailRefId);null===(t=this.expandableCardHeader)||void 0===t||null==t||t.on("click.cards",(function(t){i.isDisabled()||(t.preventDefault(),i.toggleExpanded())})),!1!==this.settings.selectable&&this.element.on("click.".concat(fi),".card",(function(t){var e=$(t.currentTarget),n=$(t.target).is(".checkbox");setTimeout((function(){i.select(e,n)}),0),t.stopPropagation(),t.preventDefault()})),n.find(s).on("click",(function(t){t.stopPropagation(),i.showCardDetail();var e=i.element.find(".card-header, .widget-header").find(".go-back");$(e).on("click",(function(t){t.stopPropagation(),i.hideCardDetail()}))})),$("body").on("resize.card",(function(){H.isRTL()&&e.removeInfoIconTooltip(),setTimeout((function(){e.resizeButtonset()}),300)}));var a=this.element.find(".card-header, .widget-header");return this.element.find(".card-content, .widget-content").children().on("scroll.card",(function(t){var e=t.target,i=$(e).siblings(".listview-search, .widget-search"),n=$(e).siblings(".card-search, .widget-search").find(".searchfield-wrapper");e.scrollTop>0?i.length>0?i.addClass("is-scrolling"):n.length>0?n.addClass("is-scrolling"):a.addClass("is-shadow-scrolling"):(i.removeClass("is-scrolling"),n.removeClass("is-scrolling"),a.removeClass("is-shadow-scrolling"))})),this},teardown:function(){var t,e;return $("html").off("themechanged.".concat(fi)),this.element.off("click.".concat(fi)),null===(t=this.expandableCardHeader)||void 0===t||null==t||t.off("click.cards"),this.expandableCardHeader=null,$("body").off("resize.card"),$(this.element.find(".card-content, .widget-content").find("#".concat(this.settings.detailRefId,", .is-selected"))).off(),$(null===(e=this.element.find(".card-header, .widget-header"))||void 0===e||null==e?void 0:e.find(".go-back")).off(),this.selectedRows=[],this.cardContentPane.off(),this.element.find(".card-content").children().off("scroll.card"),this.cardContentPane=null,this},destroy:function(){this.teardown(),$.removeData(this.element[0],fi)}},$.fn.cards=function(t){var e=$(this),i=e.attr("data-dataset"),n=e.attr("data-tmpl"),s=E.parseOptions(this)||{};s.dataset=s.dataset||i,s.template=s.template||n,window[s.dataset]&&(s.dataset=window[s.dataset]),s.template&&s.template.length&&(s.template=$("#".concat(s.template)).html());var a=E.extend({},t,s);return this.each((function(){var t=$.data(this,fi);t?t.updated(a):t=$.data(this,fi,new vi(this,a))}))},$.fn.popover=function(t){return this.each((function(){var e=$.data(this,"popover");e?e.updated(t):e=$.data(this,"popover",new Vt(this,t))}))};var bi={destroy:function(t){var e,i;null===(e=t[0])||void 0===e||(i=null==e?void 0:e.classList).remove.apply(i,["line-chart","bar-chart","bar-chart-stacked","bar-chart-grouped","bubble","column-chart","completion-chart","chart-completion-target-padding","chart-pie","has-right-legend"])},tooltip:{}};bi.isIE="ie"===x.browser.name,bi.isIEEdge="edge"===x.browser.name,bi.tooltipSize=function(t){var e;return null!==(e=this.tooltip)&&void 0!==e&&(null==e?void 0:e.find)||this.appendTooltip(),f.html(this.tooltip.find(".tooltip-content"),t,"*"),{height:this.tooltip.outerHeight(),width:this.tooltip.outerWidth()}},bi.formatToSettings=function(t,e){var i=t.data?t.data:t,n=e.formatter&&".0f"!==e.formatter?d3.format(e.formatter)(i.percent):"".concat(isNaN(i.percentRound)?0:i.percentRound,"%");return"value"===e.show?e.formatter?d3.format(e.formatter)(i.value):i.value:"label"===e.show?i.name:"label (percent)"===e.show?"".concat(i.name," (").concat(n,")"):"label (value)"===e.show?"".concat(i.name," (").concat(e.formatter?d3.format(e.formatter)(i.value):i.value,")"):"percent"===e.show?n:"function"==typeof e.show?e.show(i):i.value},bi.appendTooltip=function(t){var e=this;this.tooltip=$("#svg-tooltip"),0===this.tooltip.length&&(t=t?" ".concat(t):"",this.tooltip=$('<div id="svg-tooltip" class="tooltip right is-hidden'.concat(t,'">\n      <div class="arrow"></div>\n        <div class="tooltip-content">\n          <p><b>32</b> Element</p>\n        </div>\n      </div>')).appendTo("body"),this.isTouch&&(this.tooltip[0].style.pointerEvents="auto",this.tooltip.on("touchend.svgtooltip",(function(){e.hideTooltip()}))))},bi.hideTooltip=function(){var t=this;d3.select("#svg-tooltip").classed("is-personalizable",!1).classed("is-hidden",!0).style("left","-999px"),$("body, .scrollable").off("scroll.chart-tooltip",(function(){t.hideTooltip()}))},bi.removeTooltip=function(){this.tooltip&&this.tooltip.remove&&this.tooltip.remove()},bi.colorRange=function(){var t=ft.themeColors().palette;return ft.new?[t.azure[80].value,t.turquoise[40].value,t.amethyst[60].value,t.graphite[40].value,t.amber[40].value,t.emerald[70].value,t.ruby[60].value,t.azure[30].value,t.amber[70].value,t.graphite[60].value,t.turquoise[60].value,t.emerald[90].value,t.amethyst[30].value,t.azure[50].value,t.ruby[30].value,t.amethyst[80].value,t.emerald[30].value,t.turquoise[80].value,t.graphite[20].value,t.amber[90].value]:[t.azure[70].value,t.turquoise[30].value,t.amethyst[30].value,t.graphite[60].value,t.amber[50].value,t.emerald[60].value,t.ruby[60].value,t.azure[30].value,t.amber[90].value,t.turquoise[80].value,t.ruby[20].value,t.graphite[50].value,t.emerald[50].value,t.azure[50].value,t.amethyst[80].value,t.emerald[30].value,t.turquoise[50].value,t.amber[70].value,t.graphite[20].value,t.azure[20].value]},bi.colorNameRange=function(){return ft.new?["azure08","turquoise04","amethyst06","graphite04","amber04","emerald07","ruby06","azure03","amber07","graphite06","turquoise06","emerald09","amethyst03","azure05","ruby03","amethyst08","emerald03","turquoise08","graphite02","amber09"]:["azure07","turquoise03","amethyst03","graphite06","amber05","emerald06","ruby06","azure03","amber09","turquoise08","ruby02","graphite05","emerald05","amethyst03","azure05","amethyst08","emerald03","turquoise06","amber07","graphite02"]},bi.chartColor=function(t,e,i){var n=bi.colorRange(),s=i&&i.color?i.color:null;if(s){if("error"===s)return ft.themeColors().status.danger.value;if("alert"===s)return ft.themeColors().status.warning.value;if("alertYellow"===s)return ft.themeColors().status.caution.value;if("good"===s)return ft.themeColors().status.success.value;if("neutral"===s)return ft.themeColors().palette.graphite[30].value;if("info"===s)return ft.themeColors().palette.azure[70].value;if(s){if(0===s.indexOf("#"))return i.color;var a=s.slice(-2);if(/\d/.test(a)){var o=s.slice(0,s.length-2);return ft.themeColors().palette[o][10*parseInt(a,10)].value}}}return/^(pie|donut)$/.test(e)?n[t]:/^(bar-single|column-single)$/.test(e)?n[0]:/^(bar|bar-stacked|bar-grouped|bar-normalized|line|scatterplot|column-stacked|column-grouped|column-positive-negative)$/.test(e)?n[t]:""},bi.chartColorName=function(t,e,i){var n=i&&i.color?i.color:null;return n?"error"===n?"alert01":"alert"===n?"alert02":"alertYellow"===n?"alert03":"good"===n?"alert04":"neutral"===n?"graphite03":"info"===n?"azure07":i.color:/^(pie|donut)$/.test(e)?this.colorNameRange()[t]:/^(bar-single|column-single)$/.test(e)?this.colorNameRange()[0]:/^(bar|bar-stacked|bar-grouped|bar-normalized|line|scatterplot|column-stacked|column-grouped|column-positive-negative)$/.test(e)?this.colorNameRange()[t]:""},bi.showTooltip=function(t,e,i,n,s){var a,o,r=this;t<0&&(t=2),this.tooltip[0].style.left="".concat(t,"px"),this.tooltip[0].style.top="".concat(e,"px"),f.html(this.tooltip.find(".tooltip-content"),i,"*"),this.tooltip.removeClass("bottom top left right").addClass(n),this.tooltip.css("max-width",(null==s||null===(a=null==s?void 0:s.tooltip)||void 0===a||null==a?void 0:a.maxWidth)||""),this.tooltip.find(".arrow").css("left",(null==s||null===(o=null==s?void 0:s.arrow)||void 0===o||null==o?void 0:o.left)||""),this.tooltip.removeClass("is-hidden"),$("body").off("scroll.chart-tooltip").on("scroll.chart-tooltip",(function(){r.hideTooltip()})),$(".scrollable").off("scroll.chart-tooltip").on("scroll.chart-tooltip",(function(){r.hideTooltip()}))},bi.addLegend=function(t,e,i,n){var s,a=this;if(0!==t.length){var o,r,l=0,c=0;for(this.hasLegendPopup=!1,s=0;s<t.length;s++)o=t[s].name?6*t[s].name.length:6,l=t[s].name&&o>l?o:l,(c+=o)/$(n).width()*100<=45&&(r=s+1);var d=(l+=55)/$(n).width()*100>45,h=t[0].placement&&"bottom"===t[0].placement||!0===i.forceLegendPopup;h&&d||(r=t.length),h&&$(n).hasClass("has-right-legend")&&$(n).removeClass("has-right-legend");var u=t[0].display&&"twocolumn"===t[0].display,p=u?$('<div class="chart-legend '.concat(t[0].placement&&!h?"is-".concat(t[0].placement):"is-bottom",'"></div>')):$('<div class="chart-legend"></div>');for("pie"!==e&&"donut"!==e||!i.showMobile||(p=$('<div class="chart-legend"><div class="container"></div></div>')),s=0;s<r;s++)if(t[s].name){var f="";(u||t[s].display&&"block"===t[s].display)&&(f+=" lg"),"column-positive-negative"===i.type&&t[s].option&&(f+=" ".concat(t[s].option));var m='<span index-id="chart-legend-'.concat(s,'" class="chart-legend-item').concat(f,'" tabindex="').concat(null!=i&&(null==i?void 0:i.selectable)?0:-1,'" role="button"></span>'),v=bi.chartColor(s,e||(1===t.length?"bar-single":"bar"),t[s]),b=bi.chartColorName(s,e||(1===t.length?"bar-single":"bar"),t[s]),y="";"#"===b.substr(0,1)?(y=$('<span class="chart-legend-color"></span>'),t[s].pattern||y.css("background-color",v)):y=$('<span class="chart-legend-color '.concat(t[s].pattern?"":b,'"></span>')),"scatterplot"===e&&(y=$('<span class="chart-legend-color"></span>'));var w=$('<span class="chart-legend-item-text">'.concat(g.stripTags(t[s].name),"</span>"));if(t[s].pattern&&(y.append('<svg width="12" height="12"><rect height="12" width="12" mask="url(#'.concat(t[s].pattern,')"/></svg>')),y.find("rect").css("fill",v)),t[s].percent){var x=$('<span class="chart-legend-percent"></span>').text(t[s].percent);w.append(x)}t[s].display&&"block"===t[s].display&&(m='<span class="chart-legend-item'.concat(f,'" tabindex="').concat(i.selectable?0:-1,'" role="button"></span>')),u&&(m=d&&h?'<span index-id="chart-legend-'.concat(s,'" class="chart-legend-item').concat(f,' is-one-line" tabindex="').concat(i.selectable?0:-1,'" role="button"></span>'):'<span index-id="chart-legend-'.concat(s,'" class="chart-legend-item').concat(f,' is-two-column" tabindex="').concat(i.selectable?0:-1,'" role="button"></span>')),(m=$(m)).append(y,'<span class="audible">'.concat(H.translate("Highlight"),"</span>"),w);var C,k,T,A,S="legend-".concat(s);if(i.attributes)E.addAttributes(m,t[s],i.attributes,S,!0);else E.addAttributes(m,t[s],null===(C=t[s])||void 0===C||null===(k=null==(T=C)?void 0:T.data)||void 0===k||null==(A=k)?void 0:A.attributes,S,!0);"pie"!==e&&"donut"!==e||!i.showMobile?p.append(m):p.find(".container").append(m),(t[s].display&&"block"===t[s].display||u&&d&&h)&&m.css({float:"none",display:"block",margin:"0 auto",width:"".concat(l,"px")}),"scatterplot"===e&&(self.svg=d3.select(y[0]).append("svg").attr("width","24").attr("height","24").append("path").attr("class","symbol").attr("transform","translate(10, 10)").attr("d",d3.symbol().size("80").type((function(){return d3.symbols[s]}))).style("fill",v)),innerWidth<=480&&t[s].data&&t[s].data.legendAbbrName&&w.replaceWith('<span class="chart-legend-item-text">'.concat(t[s].data.legendAbbrName,"</span>")),innerWidth>=481&&innerWidth<=768&&t[s].data&&t[s].data.legendShortName&&w.replaceWith('<span class="chart-legend-item-text">'.concat(t[s].data.legendShortName,"</span>"))}if(p instanceof $){var D=/^chart-legend-(.+)/;if(p.on("click.chart",".chart-legend-item",(function(){var e=$(this).attr("index-id").match(D)[1];bi.handleElementClick(e,this,t,i,n)})).on("keypress.chart",".chart-legend-item",(function(e){if(13===e.which||32===e.which){var s=$(this).attr("index-id").match(D)[1];bi.handleElementClick(s,this,t,i,n)}})),h&&d&&t.length>r||null!=i&&(null==i?void 0:i.forceLegendPopup)){var I=$('\n      <button class="btn-actions list-button" type="button">\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation" style="min-height: 0">\n          <use href="#icon-bullet-steps"></use>\n        </svg>\n      </button>\n      ');this.hasLegendPopup=!0;for(var L=$('<ul class="popupmenu"></ul>'),P=0;P<t.length;P++){var M=$("<li ".concat(0===P?'class="is-hidden"':"",'><a index-id="chart-legend-').concat(P,'" href="#"><div class="chart-popup-menu"></div></a></li>')),B=$('<span class="chart-popup-menu-text">'.concat(g.stripTags(t[P].name),"</span>")),O=bi.chartColor(P,e||(1===t.length?"bar-single":"bar"),t[P]),R=bi.chartColorName(P,e||(1===t.length?"bar-single":"bar"),t[P]),z="";"#"===R.substr(0,1)?(z=$('<span class="chart-popup-menu-color"></span>'),t[s].pattern||z.css("background-color",O)):z=$('<span class="chart-popup-menu-color '.concat(t[s].pattern?"":R,'"></span>')),M.find("div").append(z,B),L.append(M)}p.css({"min-height":"40px","max-height":"50px",display:"flex"}),$(n).addClass("auto-height"),p.append(I),$(n).append(p),L.insertAfter(I),I.popupmenu({menu:L}),I.on("selected",(function(e,s){var o=$(s[0]).attr("index-id").match(D)[1];bi.handleElementClick(o,a,t,i,n)}))}else $(n).append(p)}}},bi.handleElementClick=function(t,e,i,n,s){var a;if(n.selectable){var o,r=$(null==n||null===(a=null==n?void 0:n.svg)||void 0===a||null==a?void 0:a.node()).closest(".chart-container").data("chart"),l=null==r||null==r?void 0:r.initialSelectCall,c=i[t],d=i[0].display&&"twocolumn"===i[0].display;if("radar"===n.type&&(o=d3.select(n.svg.selectAll(".chart-radar-area").nodes()[t])),"pie"===n.type||"donut"===n.type)o=d3.select(n.svg.selectAll(".slice").nodes()[t]);else if(/positive-negative/.test(n.type))o=!c.option||c.option&&"target"===c.option?n.svg.select(".target-bar"):n.svg.select(".bar.".concat(c.option));else if(-1!==["column","bar","bar-stacked","bar-grouped","bar-normalized","column-grouped","column-stacked","column-positive-negative","positive-negative"].indexOf(n.type))if(n.isGrouped||n.isSingle)o=n.svg.select(".series-".concat(t));else if(n.isStacked&&!n.isSingle){o=d3.select(n.svg.selectAll("bar"===n.type||"bar-stacked"===n.type||"bar-normalized"===n.type?".series-group":".g").nodes()[t]).select(".bar")}if(-1!==["radar","pie","donut","column","bar","bar-stacked","bar-grouped","bar-normalized","column-grouped","column-stacked","column-positive-negative","positive-negative"].indexOf(n.type)&&null!=n&&(null==n?void 0:n.selectable)){var h=$(e),u="true"===h.attr("aria-pressed");bi.clickedLegend=!0,o.dispatch("click"),h.parent().find("[aria-pressed]").removeAttr("aria-pressed"),u||h.attr("aria-pressed","true")}if(c.selectionObj&&null!=n&&(null==n?void 0:n.selectable)&&bi.selectElement(d3.select(c.selectionObj.nodes()[t]),c.selectionInverse,c.data,void 0,n.dataset,l),d&&this.hasLegendPopup){var p="donut"===n.type?"pie":n.type,f=bi.chartColor(t,p||(1===i.length?"bar-single":"bar"),c),m=bi.chartColorName(t,p||(1===i.length?"bar-single":"bar"),c),v="";"#"===m.substr(0,1)?(v=$('<span class="chart-legend-color"></span>'),c.pattern||v.css("background-color",f)):v=$('<span class="chart-legend-color '.concat(c.pattern?"":m,'"></span>'));var b=$('<span class="chart-legend-item-text">'.concat(g.stripTags(c.name),"</span>")),y=s.find(".chart-legend-item");y.empty(),y.append(v,'<span class="audible">'.concat(H.translate("Highlight"),"</span>"),b),s.find(".list-button").data("popupmenu").menu.children().removeClass("is-hidden"),$(s.find(".list-button").data("popupmenu").menu.children().get(t)).addClass("is-hidden")}}},bi.selected=[],bi.clearSelected=function(t){if(t){var e=function(t){Object.prototype.hasOwnProperty.call(t,"selected")&&delete t.selected};!function t(i){Array.isArray(i)&&i.forEach((function(i){e(i),i.data&&t(i.data)})),e(i)}(t)}},bi.selectElement=function(t,e,i,n,s,a){var o=t.node()&&t.classed("is-selected");bi.clearSelected(s),o||(i.selected=!0);var r=[{elem:t.nodes(),data:o?{}:i}];e.classed("is-selected",!1).classed("is-not-selected",!o),t.classed("is-not-selected",!1).classed("is-selected",!o),bi.selected=$.isEmptyObject(r[0].data)?[]:r,a||$(n).triggerHandler("selected",[r])},bi.setSelectedElement=function(t){var e,i=t.dataset,n=/positive-negative/.test(t.type),s=/^(bar|bar-stacked|bar-grouped|bar-normalized)$/.test(t.type),a="pie"===t.type||"donut"===t.type,o=/^(column|column-grouped|column-stacked|column-positive-negative|positive-negative)$/.test(t.type),r=t.svg,l=t.isSingle,c=t.isGrouped,d=t.isStacked,h="selected"===t.task,u=d3.select(t.selector),p=u.classed("positive"),g=t.svg.selectAll(".axis.x .tick"),f=t.svg.selectAll(".axis.y .tick"),m=t.svg.selectAll(".bartext.positive, .target-bartext.positive"),v=t.svg.selectAll(".bartext.negative, .target-bartext.negative"),b=t.svg.selectAll(".target-bartext.positive, .target-bartext.negative"),y=d3.select(t.selector.parentNode),w=parseInt(y.node()?y.attr("data-group-id"):0,10),x=[],C=[];if(i=d||a?i||null:i&&i[w]?i[w].data||i[w]:null,g.style("font-weight","normal"),f.style("font-weight","normal"),m.style("font-weight","normal"),v.style("font-weight","normal"),r.selectAll(".is-selected").classed("is-selected",!1),bi.clearSelected(t.dataset),a&&r.selectAll(".is-not-selected").classed("is-not-selected",!1),n&&Object.prototype.hasOwnProperty.call(t.dataset[0],"targetBarsSelected")&&delete t.dataset[0].targetBarsSelected,h)if(r.selectAll(".bar, .target-bar").style("opacity",.6),bi.clickedLegend&&!a)n?(t.isTargetBar?(t.svg.selectAll(".target-bar").classed("is-selected",!0).style("opacity",1),b.style("font-weight","bolder"),t.dataset[0].targetBarsSelected=!0):(t.svg.selectAll(p?".bar.positive, .target-bar.positive":".bar.negative, .target-bar.negative").classed("is-selected",!0).style("opacity",1),(p?m:v).style("font-weight","bolder")),r.selectAll(".bar").each((function(t,e){var n=d3.select(this);if(n.classed("is-selected")){var s=i?i[e]:t;s.selected=!0,C.push({elem:n.node(),bardata:s})}})),x=C):(o||s)&&(c||l?t.svg.selectAll(".series-"+t.i).classed("is-selected",!0).style("opacity",1):y.classed("is-selected",!0).selectAll(".bar").classed("is-selected",!0).style("opacity",1),r.selectAll(".bar.is-selected").each((function(i,n){var a=d3.select(this);(e=t.dataset)||(e=i),s&&!d?c?e=t.dataset[n].data[t.i]:(e[n][t.i]&&(e=e[n][t.i]),e[t.i]&&e[t.i][n]&&(e=e[t.i][n]),e[n]&&e[n][t.i]&&(e=e[n][t.i])):d&&!l?(e[w]&&e[w].data[n]&&(e=e[w].data[n]),s&&(t.dataset[w].selected=!0)):(e[n].data[t.i]&&(e=e[n].data[t.i]),e[t.i]&&e[t.i].data[n]&&(e=e[t.i].data[n]),e[n]&&e[n].data[t.i]&&(e=e[n].data[t.i])),e.selected=!0,C.push({elem:a.node(),data:e})})),x=C);else if(l&&d&&o)e=i[0]&&i[0].data?i[0].data:t.d,u.classed("is-selected",!0).style("opacity",1),x.push({elem:u.nodes(),data:e[t.i]});else if(!l&&!c||d||!o&&!s){if(o||s){var k="bar"===t.type?"g[role=listitem]:nth-child(".concat(t.i+1,") .bar"):".bar:nth-child(".concat(t.i+1,")");r.selectAll("".concat(o?".axis.x":".axis.y"," .tick:nth-child(").concat(t.i+2,")")).style("font-weight","bolder"),r.selectAll("".concat(k)).classed("is-selected",!0).style("opacity",1),r.selectAll(".bar.is-selected").each((function(e,n){var a=d3.select(this),o=e;i&&(o=s&&d&&void 0!==i[n][t.i]?i[n][t.i]:i[n].data[t.i]),o.selected=!0,C.push({elem:a.nodes(),data:o,index:t.i})})),x=C}else if(a){r.selectAll(".slice").classed("is-selected",!1).classed("is-not-selected",!0).attr("transform","");var T=i&&i[0]&&i[0].data?i[0].data[t.i]:t.d?t.d.data:t.d;(T=T||{}).selected=!0,u.classed("is-selected",!0).classed("is-not-selected",!1).attr("transform","scale(1.025, 1.025)"),x.push({elem:u.nodes(),data:T,index:t.i})}}else{if(r.selectAll("".concat(o?".axis.x":".axis.y"," .tick:nth-child(").concat((c?w:t.i)+2,")")).style("font-weight","bolder"),u.classed("is-selected",!0).style("opacity",1),n){var A=t.isTargetBar?t.i:t.i-t.dataset[0].data.length;r.select(".target-bar.series-".concat(A)).classed("is-selected",!0).style("opacity",1),r.select(".bar.series-".concat(A)).classed("is-selected",!0).style("opacity",1),d3.select(r.selectAll(".bartext").nodes()[A]).style("font-weight","bolder"),d3.select(r.selectAll(".target-bartext").nodes()[A]).style("font-weight","bolder")}(c||n||o)&&((!n&&!o||o&&c)&&y.classed("is-selected",!0).selectAll(".bar").classed("is-selected",!0).style("opacity",1),y.selectAll(".bar").each((function(t,e){var n=d3.select(this);if(n.classed("is-selected")){var s=i?i[e]:t;s.selected=!0,C.push({elem:n.nodes(),data:s,index:e})}})),c?x.push({groupIndex:w,groupElem:y.nodes()[0],groupItems:C}):x=C)}else r.selectAll(".bar, .target-bar").style("opacity",1),m.style("font-weight","bolder"),v.style("font-weight","bolder"),a&&u.classed("is-selected",!1).style("stroke","#fff").style("stroke-width","1px").attr("transform","");bi.clickedLegend&&(bi.clickedLegend=!1),bi.selected=x,t.isTrigger&&$(t.container).triggerHandler(h?"selected":"unselected",[x])},bi.setSelected=function(t,e,i){if(t){var n,s,a,o,r,l=0,c=E.equals,d=i.isPie?i.svg.node().nextSibling:i.svg.node().parentNode.nextSibling,h=d3.select(d),u=h.node()&&h.classed("chart-legend"),p=function(e){var o=!!e;(e=o?d3.select(e):i.svg).selectAll(".bar").each((function(e,r){e&&l<1&&(void 0!==t.fieldName&&void 0!==t.fieldValue&&t.fieldValue===e[t.fieldName]||void 0!==t.index&&t.index===r||t.data&&c(t.data,i.chartData[e.index].data[r])||t.elem&&$(this).is(t.elem))&&(l++,s=d3.select(this),n=r,o&&!i.isStacked&&(a=!0))}))};i.isGrouped||i.isStacked&&!i.isSingle?(i.chartData.forEach((function(e,r){l<1&&(o=$(i.svg.select('[data-group-id="'+r+'"]').node()),(void 0!==t.groupName&&void 0!==t.groupValue&&t.groupValue===e[t.groupName]||void 0!==t.groupIndex&&t.groupIndex===r||t.data&&c(t.data,e)||t.elem&&o.is(t.elem))&&void 0===t.fieldName&&void 0===t.fieldValue&&void 0===t.index&&(l++,s=i.svg.select('[data-group-id="'+r+'"]').select(".bar"),n=r,i.isStacked&&!i.isGrouped&&(a=!0)))})),l<1&&(r=i.svg.selectAll(".series-group")).nodes().length&&r.each((function(){p(this)}))):p(),l>0&&(e||!s.classed("is-selected"))&&(a?u&&$(h.selectAll(".chart-legend-item").nodes()[n]).trigger("click.chart"):s.dispatch("click"))}},bi.labelsColide=function(t){var e=t.selectAll(".x text"),i=!1;return e.each((function(t,n){var s,a=this.getBoundingClientRect();e.each((function(t,e){n!==e&&(s=this.getBoundingClientRect(),!(a.right<s.left||a.left>s.right||a.bottom<s.top||a.top>s.bottom)&&(i=!0))}))})),i},bi.applyAltLabels=function(t,e,i,n,s){(n?t.selectAll(n):t.selectAll(".x text")).each((function(t,n){var a=e[n]?e[n][i]:"";a=a||(s?d3.select(this).text().substring(0,1):"".concat(d3.select(this).text().substring(0,6),"...")),d3.select(this).text(a)}))},bi.triggerContextMenu=function(t,e,i,n){n.preventDefault(),n.stopPropagation(),n.stopImmediatePropagation();var s=$.Event("contextmenu");s.target=e,s.pageX=n.pageX,s.pageY=n.pageY,$(t).trigger(s,[e,i])},bi.calculateTextRenderWidth=function(t,e){var i,n;e=E.mergeSettings(void 0,e,{soho:"700 12px arial",uplift:"600 14px arial",classic:"700 12px arial",new:"600 14px arial"});var s=((null==ft||null===(i=null==ft?void 0:ft.currentTheme)||void 0===i||null==i?void 0:i.id)||"").match(/soho|uplift|new|classic/);if(s=s?s[0]:"new",this.canvas=this.canvas||(this.canvas=document.createElement("canvas")),!this.canvas)return 0;var a=null===(n=this.canvas)||void 0===n||null==n?void 0:n.getContext("2d");return null!=a&&(null==a?void 0:a.font)?(a.font=e[s],a.measureText(t).width):0},bi.calculatePercentage=function(t,e){return 100*t/e},bi.getPercentage=function(t,e){return t*e/100},bi.trimText=function(t,e){return t.length<=e?t:"".concat(t.substr(0,e),"...")},bi.getLabel=function(t,e){var i=t.name;return e.xxsmall||e.xsmall||e.small?i=t.shortName||t.abbrName||t.name:e.medium&&(i=t.abbrName||t.name),i},bi.wrap=function(t,e,i){i||(i=1.27),e||(i=60),t.each((function(){var t=d3.select(this),n=t.text().split(/\s+/).reverse(),s="",a=[],o=0;if(!(n.length<=1))for(var r=i,l=t.attr("y"),c=t.attr("x"),d=parseFloat(t.attr("dy")),h=t.text(null).append("tspan").attr("x",c).attr("y",l).attr("dy","".concat(d,"em"));s=n.pop();)a.push(s),h.text(a.join(" ")),h.node().getComputedTextLength()>e&&(a.pop(),h.text(a.join(" ")),a=[s],h=t.append("tspan").attr("x",c).attr("y",l).attr("dy","".concat(++o*r+d,"em")).text(s))}))};var yi="bullet",wi={dataset:[],animate:!1,redrawOnResize:!0,format:{ranges:null,difference:null}};function xi(t,e){this.settings=E.mergeSettings(t,e,wi),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),this.init()}xi.prototype={init:function(){return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.width=0,this.build().handleEvents(),"initial"===this.settings.animate&&(this.settings.animate=!1),this},build:function(){var t=this,e=this.settings.dataset[0],i=this.element.closest(".is-personalizable").length>0;this.element.addClass("bullet-chart");var n=0,s=[],a=e.data[0]?e.data[0].tooltip:[],o=!1,r=this.element.parent(),c=30,d=35,h=35,u=40,p=r.width()-u-d,g=r.height()-c-h-30;g=g<0?50:g;var f=d3.select(this.element[0]).append("svg").attr("width",p+u+d).attr("height",g+c+h).append("g").attr("transform","translate(".concat(u,",").concat(c,")"));for(var m=function(i,n,s){function a(t,e,i){var n=t(e,i);return void 0!==n?n:e}return e.format&&"function"==typeof e.format[i]?a(e.format[i],n,s):e.format&&"string"==typeof e.format[i]?d3.format(e.format[i])(n):t.settings.format&&"function"==typeof t.settings.format[i]?a(t.settings.format[i],n,s):t.settings.format&&"string"==typeof t.settings.format[i]?d3.format(t.settings.format[i])(n):n},v=function(t,e){return m("ranges",t,e)},b=function(t){return m("difference",t)},y=1===e.data.length,w=function(r){var c=t.settings.animate?400:0,d=20,h=t,u=e.data[r],g=u.ranges.slice().sort(d3.descending),m=u.markers?u.markers.slice().sort(d3.descending):[],w=u.measures?u.measures.slice().sort(d3.descending):[],x=u.ranges.slice().sort(d3.ascending),C=u.markers?u.markers.slice().sort(d3.ascending):[],k=u.measures?u.measures.slice().sort(d3.ascending):[],T=y?"":"-group".concat(r);0===m.length&&(m=w,C=k,o=!0);var A=f.append("g").attr("class","bullet").attr("transform","translate(0, ".concat(70*r,")"));A.append("g").append("text").attr("class","title").attr("dy","-10px").text((function(){return u.title})).call((function(t){var i=t._groups[0][0];E.addAttributes($(i),e,e.attributes,"title".concat(T))})).append("tspan").attr("class","subtitle").attr("dx","15px").text((function(){return u.subtitle})).call((function(t){var i=t._groups[0][0];E.addAttributes($(i),e,e.attributes,"subtitle".concat(T))}));var S=Math.max(g[0],m[0],w[0]),D=Math.min(x[0],C[0],k[0]);D=D<0?D:0;var I,L,P=d3.scaleLinear().domain([D,S]).range([0,p]).nice(),M=(L=(I=P)(0),function(t){return Math.abs(I(t)-L)}),B=A.selectAll("rect.range").data(g),O=!1,R=0;h.element.off("dblclick.".concat(h.namespace)).on("dblclick.".concat(h.namespace),"*",(function(t){t.stopImmediatePropagation()})),B.enter().append("rect").call((function(t){t._groups.forEach((function(t){t.forEach((function(t,i){E.addAttributes($(t),e,e.attributes,"range".concat(i).concat(T))}))}))})).attr("class",(function(t,e){return"range s".concat(e)})).attr("data-idx",r).attr("width",0).attr("x",(function(t){return P(t<0?t:0)})).style("fill",(function(t,i){return e.barColors?e.barColors[i]:""})).attr("height",d).on("click.".concat(h.namespace),(function(){var t=d3.select(this);R=setTimeout((function(){O||h.element.trigger("selected",[t,e.data[t.attr("data-idx")]]),O=!1}),200)})).on("dblclick.".concat(h.namespace),(function(){var t=d3.select(this);clearTimeout(R),O=!0,h.element.trigger("dblclick",[t,e.data[t.attr("data-idx")]])})).on("mouseenter.".concat(h.namespace),(function(t,o,c){var d=d3.select(this),h=e.data[d.attr("data-idx")],u=this.getBoundingClientRect(),p="<p>".concat(v(o,c),"</p>"),g=function(){var t=bi.tooltipSize(p),e=u.left+u.width-t.width/2,n=u.top-t.height+$(window).scrollTop()-5;""!==p&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[i?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(e,n,p,"top"))};if(h.tooltip&&h.tooltip[c]&&(p=h.tooltip[h.tooltip.length-c-1]),a&&"function"==typeof a&&!s[c]){p="";var f=!0;n=setInterval((function(){f&&(f=!1,a((function(t){p=t,s[r]=t}))),""!==p&&(clearInterval(n),g())}),10)}else a="object"===l(a)?"":a,p=s[r]||a||p||"",g()})).on("mouseleave.".concat(h.namespace),(function(){clearInterval(n),bi.hideTooltip()})).on("contextmenu.".concat(h.namespace),(function(t,e){bi.triggerContextMenu(h.element,d3.select(this).nodes()[0],e,t)})).merge(B).transition().duration(c).attr("width",M);var z=A.selectAll("rect.measure").data(w);z.enter().append("rect").call((function(t){t._groups.forEach((function(t){t.forEach((function(t,i){E.addAttributes($(t),e,e.attributes,"measure".concat(i).concat(T))}))}))})).attr("class",(function(t,e){return"measure s".concat(e)})).attr("width",0).attr("height",3).attr("x",(function(t){return P(t<0?t:0)})).style("fill",(function(t,i){return e.lineColors?e.lineColors[i]:""})).attr("y",8.5).merge(z).transition().duration(c).attr("width",M);var F=1===m.length,N=A.selectAll("line.marker").data(m);N.enter().append("line").call((function(t){t._groups.forEach((function(t){t.forEach((function(t,i){var n=F?"marker".concat(T):"marker".concat(i).concat(T);E.addAttributes($(t),e,e.attributes,n)}))}))})).attr("class",o?"hidden":"marker").attr("x1",0).attr("x2",0).style("stroke",(function(t,i){return e.markerColors?e.markerColors[i]:""})).attr("y1",d/6).attr("y2",100/6).merge(N).transition().duration(c).attr("x1",P).attr("x2",P).attr("y1",d/6).attr("y2",100/6);var H=(m[0]>w[0]?"-":"+")+Math.abs(m[0]-w[0]);0!==Math.abs(m[0]-w[0])&&N.enter().append("text").call((function(t){var i=t._groups[0][0];E.addAttributes($(i),e,e.attributes,"difference".concat(T))})).attr("class","inverse").attr("text-anchor","middle").attr("y",14).attr("dx",bi.isRTL?"-20px":"20px").attr("x",0).text(b(H)).merge(N).transition().duration(c).attr("x",(function(){var t=0;return A.selectAll(".measure").each((function(e){var i=M(e),n=P(e);i>t&&(t=i),n>t&&(t=n)})),bi.isRTL?-t:t})).style("opacity",1);var q=A.selectAll("g.tick").data(P.ticks(8)).enter().append("g").attr("class","tick").attr("transform","translate(0,0)").style("opacity",0);q.append("line").attr("y1",d).attr("y2",Math.round(140/4.7)),q.append("text").attr("text-anchor","middle").attr("dy","1.1em").attr("y",Math.round(140/4.7)).attr("class",(function(t){return t<0?"negative-value":"positive-value"})).text((function(t,e){return v(t,e)})),q.transition().duration(c).attr("transform",(function(t){return"translate(".concat(P(t),",0)")})).style("opacity",1),bi.isRTL&&bi.isIE&&f.selectAll("text").attr("transform","scale(-1, 1)")},x=0;x<e.data.length;x++)w(x);return bi.appendTooltip(),this.element.trigger("rendered"),this},handleEvents:function(){var t=this;return this.element.on("updated.".concat(this.namespace),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated()})),this},handleResize:function(){this.width!==this.element.width()&&(this.width=this.element.width(),this.element.is(":visible")&&this.updated())},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset&&(this.settings.dataset=t.dataset),this.element.empty(),this.build()},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".range").off(e(["mouseenter","mouseleave","click","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),delete this.namespace,this},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("bullet-chart"),$.removeData(this.element[0],yi),$.removeData(this.element[0],"chart"))}};var Ci="completion-chart",ki={dataset:[]};function Ti(t,e){this.settings=E.mergeSettings(t,e,ki),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),this.init()}Ti.prototype={init:function(){return this.build().handleEvents(),this},build:function(){var t=this,e=this.settings.dataset[0];0===this.settings.dataset.length&&((e={}).data=[{name:{text:H?H.translate("NoData"):"No data available"},completed:{value:0}}]);var i=e.data[0],n="completion-target"===this.settings.type,s="targeted-achievement"===this.settings.type;this.element.addClass("completion-chart").addClass("".concat("targeted-achievement"===this.settings.type?" chart-targeted-achievement":"")).attr("role","image"),i.total=$.extend({},{value:100},i.total);var a,o=function(t){return void 0===t},r=function(t,e){var i=e?0:"";return o(t)?i:t},l=function(t,e){return e=e||i,Math.round(t/r(e.total.value,!0)*100)},c=function(t){return H.formatNumber(t/100,{style:"percent",maximumFractionDigits:0})},d=function(t,e){var n=t.toString();return-1!==n.indexOf("%")?function(t,e){return t/100*r((e=e||i).total.value,!0)}(n.replace(/%/g,""),e):t},h=function(t,e,i,n){var s=l(e,n),a=(s=s<0?0:s)>100?100:s;if(t[0].style.width="".concat(a,"%"),i>0){var o=l(i,n);t[0].style.left="".concat(o,"%")}0===a&&(t[0].className+="is-empty")},u=function(t,e,i){var n=i?t._value:t.value;return t&&!o(n)&&t.format?function(t,e,i){return".0%"===e?c(l(t,i)):d3.format(e||"m")(t)}(d(n,e),t.format,e):t?d(n,e):0},p=this,g=$.extend({},{show:!1,color1:"",color2:"inverse"},i.percentText);o(g.color)||""!==g.color1||(g.color1=g.color);var m,v={body:$('<div class="total bar" />')},b=function(t){var e={};return(t=t||i).info&&!o(t.info.color)&&0===i.info.color.indexOf("#")&&(e.info=!0),t.completed&&!o(t.completed.color)&&0===t.completed.color.indexOf("#")&&(e.completed=!0),t.remaining&&!o(t.remaining.color)&&0===t.remaining.color.indexOf("#")&&(e.remaining=!0),t.targetline&&!o(t.targetline.color)&&0===t.targetline.color.indexOf("#")&&(e.targetline=!0),e}();if(n||s){var y,w=function(t){var e="",n={};return(t=t||i).total.difference&&(n.value=t.total.value-t.completed.value,n.format=i.total.format),e=(t.total.textOnly?"":u(t.total.difference?n:t.total))+(t.total.text||""),s&&t.remaining?(t.remaining.textOnly?"":u(t.remaining))+(t.remaining.text||""):e}();v.body.addClass("chart-completion-target".concat(s?" chart-targeted-achievement":"")),y=i.completed.color&&"error"===i.completed.color&&i.completed.iconTooltip?i.completed.iconTooltip:"Critical",v.label='<span class="label">\n        <span class="name">'.concat(r(i.name.text)," ").concat(i.completed.color&&"error"===i.completed.color?'<svg class="icon-error icon" focusable="true" role="presentation" title="'.concat(y,'"><use href="#icon-error"></use></svg>'):"",'</span>\n        <span class="l-pull-right total value">').concat(w,"</span>\n      </span>"),$("svg[title]").tooltip()}else{v.body.addClass("chart-completion");var x=r(i.name.text),C=r(i.completed.color),k=r(i.info.color),T=i.info.color&&!b.info?k:"",A=r(i.info.text);b.completed||(T=C);var S="";i.info.color&&b.info&&(S=k),b.completed&&(S=C);var D=i.info&&!o(i.info.value)?r(i.info.value):u(i.completed);v.label='<b class="label name">'.concat(x,'</b>\n      <b class="label info ').concat(T,' colored">\n      <span class="value ').concat(T,'" ').concat(S?'style="color:'.concat(S):"",'">').concat(D,'</span>\n      <span class="text ').concat(T,'" ').concat(S?'style="color:'.concat(S):"",'">').concat(A,"</span>\n      </b>")}return i.remaining?v.remaining="".concat('<div class="target remaining bar').concat(b.remaining?"":" ".concat(r(i.remaining.color)),'"').concat(b.remaining?' style="color:'.concat(i.remaining.color,";background-color:").concat(i.remaining.color,';"'):"",'">').concat(s?"":"<span".concat(n||s?"":' class="audible"',">")+'<span class="value'.concat(b.remaining?"":" ".concat(r(i.remaining.color)),'"').concat(b.remaining?' style="color:'.concat(i.remaining.color,';"'):"",'">').concat(u(i.remaining),"</span><br />")+'<span class="text'.concat(b.remaining?"":" ".concat(r(i.remaining.color)),'"').concat(b.remaining?' style="color:'.concat(i.remaining.color,';"'):"",'">').concat(r(i.remaining.text),"</span>")+"</span>","</div>"):v.remaining='<div class="target remaining bar" style="opacity: 0"></div>',i.completed&&s&&(m=m||i,g._value=m.completed?m.completed.value:0,g.percent=l(r(g._value,!0),m),g.format=".0%",g._text=void 0!==g.text?g.text:void 0!==g.value?c(g.value):u(g,m,!0),g.color=g[g.percent>55?"color2":"color1"],b.percentText=0===g.color.indexOf("#"),v.completed="".concat('<div class="completed bar').concat(b.completed?"":" ".concat(r(i.completed.color)),'"').concat(b.completed?' style="color:'.concat(i.completed.color,";background-color:").concat(i.completed.color,';"'):"",'"></div>').concat(g.show?'<div class="chart-percent-text'.concat(b.percentText||""===g.color?"":" ".concat(g.color),'"').concat(b.percentText?' style="color:'.concat(g.color,';"'):"",">").concat(g._text,"</div>"):"",'<span class="completed-label"').concat(n||s?"":' class="audible"',">")+'<span class="text">'.concat(r(i.completed.text),"</span>")+"</span>"),(i.completed.text||i.completed.format&&!s)&&this.element.addClass("chart-completion-target-padding"),i.completed&&!s&&(v.completed="".concat('<div class="completed bar').concat(b.completed?"":" ".concat(r(i.completed.color)),'"').concat(b.completed?' style="color:'.concat(i.completed.color,";background-color:").concat(i.completed.color,';"'):"",">")+"<span".concat(n||s?"":' class="audible"',">")+'<span class="value'.concat(b.completed?"":" ".concat(r(i.completed.color)),'"').concat(b.completed?' style="color:'.concat(i.completed.color,';"'):"",'">').concat(u(i.completed),"</span><br />")+'<span class="text'.concat(b.completed?"":" ".concat(r(i.completed.color)),'"').concat(b.completed?' style="color:'.concat(i.completed.color,';"'):"",'">').concat(r(i.completed.text),"</span>")+"</span></div>"),i.targetline&&(v.targetline="".concat('<div class="target-line targetline bar').concat(b.targetline?"":" ".concat(r(i.targetline.color)),'"').concat(b.targetline?' style="color:'.concat(i.targetline.color,";background-color:").concat(i.targetline.color,';"'):"",'">')+"<span".concat(n||s?"":' class="audible"',">")+'<span class="value'.concat(b.targetline?"":" ".concat(r(i.targetline.color)),'"').concat(b.targetline?' style="color:'.concat(i.targetline.color,';"'):"",'">').concat(u(i.targetline),"</span><br />")+'<span class="text'.concat(b.targetline?"":" ".concat(r(i.targetline.color)),'"').concat(b.targetline?' style="color:'.concat(i.targetline.color,';"'):"",'">').concat(r(i.targetline.text),"</span>")+"</span></div>"),f.append(v.body,(v.remaining||"")+(v.completed||"")+(v.targetline||""),"<a><use><svg><div><span><br>"),f.append(this.element,v.label+v.body.prop("outerHTML"),"<a><use><svg><div><span><br>"),a={name:$(".name",t.element),info:{value:$(".info .value",t.element),text:$(".info .text",t.element)},plainInfo:{value:$(">.value",t.element),text:$(">.text",t.element)},completed:{bar:$(".completed.bar",t.element),value:$(".completed .value",t.element),text:$(".completed .text, .completed-label .text",t.element)},remaining:{bar:$(".remaining.bar",t.element),value:$(".remaining .value",t.element),text:$(".remaining .text",t.element)},targetline:{bar:$(".targetline",t.element),value:$(".targetline .value",t.element),text:$(".targetline .text",t.element)},total:{bar:$(".total.bar",t.element),value:$(".total.value",t.element)},percentText:$(".chart-percent-text",t.element)},function(){if(e.attributes){var t=function(t,i){return E.addAttributes(t,e,e.attributes,i)};t(a.name,"name"),t(a.info.value,"info-value"),t(a.info.text,"info-text"),t(a.plainInfo.value,"info-value"),t(a.plainInfo.text,"info-text"),t(a.completed.bar,"completed-bar"),t(a.completed.value,"completed-value"),t(a.completed.text,"completed-text"),t(a.remaining.bar,"remaining-bar"),t(a.remaining.value,"remaining-value"),t(a.remaining.text,"remaining-text"),t(a.targetline.bar,"targetline-bar"),t(a.targetline.value,"targetline-value"),t(a.targetline.text,"targetline-text"),t(a.total.bar,"total-bar"),t(a.total.value,"total-value"),t(a.percentText,"percent-text")}}(),function(t){t=t||i,a.name.data("jsonData",{name:t.name}),a.info.value.add(a.info.text).data("jsonData",{info:t.info}),a.completed.bar.add(a.completed.value).add(a.completed.text).data("jsonData",{completed:t.completed}),a.remaining.bar.add(a.remaining.value).add(a.remaining.text).data("jsonData",{remaining:t.remaining}),a.targetline.bar.add(a.targetline.value).add(a.targetline.text).data("jsonData",{targetline:t.targetline}),a.total.bar.add(a.total.value).data("jsonData",{total:t.total}),a.percentText.data("jsonData",{percentText:t.percentText})}(),function(t){var e;if((t=t||i).completed){e=d(t.completed.value,t),h(a.completed.bar,e,0,t);var o,r=a.completed.bar.data("tooltip");o=t.completed.tipText?t.completed.tipText:"string"==typeof t.completed.value?t.completed.value:"".concat(Math.round(t.completed.value/t.total.value*100),"%"),r?r.setContent(o,!1):a.completed.bar.tooltip({content:o})}if(t.remaining){e=d(t.remaining.value,t),h(a.remaining.bar,e,d(t.completed.value,t),t),n&&!s&&setTimeout((function(){var t=$(".remaining",p.element),e=$(".total",p.element),i=$(".completed .value",p.element)[0].getBoundingClientRect(),n=t.find(".value")[0].getBoundingClientRect();t.add(e)[i.right>n.left-20?"addClass":"removeClass"]("overlap")}),500);var l,c=a.remaining.bar.data("tooltip");l=t.remaining.tipText?t.remaining.tipText:"string"==typeof t.remaining.value?t.remaining.value:"".concat(Math.round(t.remaining.value/t.total.value*100),"%"),c?c.setContent(l,!1):a.remaining.bar.tooltip({content:l})}t.targetline&&(e=d(t.targetline.value,t),function(t,e){var i=e>100?100:e<0?0:e;t[0].style.left="".concat(i,"%")}(a.targetline.bar,e))}(),this},handleEvents:function(){var t=this;return this.element.on("updated.".concat(Ci),(function(){t.updated()})),this},updated:function(t){var e=t.type||this.settings.type;return this.settings=t,this.settings.type=e,t&&t.dataset&&(this.settings.dataset=t.dataset),this.element.empty(),this.teardown().init()},teardown:function(){return this.element.off("updated.".concat(Ci)),this},destroy:function(){this.element.empty().removeClass("completion-chart"),bi.removeTooltip(),this.teardown(),$.removeData(this.element[0],Ci),$.removeData(this.element[0],"chart")}};var Ai="sparkline",Si={dataset:[],colors:null,isDots:!1,isPeakDot:!1,isMinMax:!1,isMedianRange:!1};function $i(t,e){this.settings=E.mergeSettings(t,e,Si),this.element=$(t),this.init()}$i.prototype={init:function(){if(!this.settings.colors){var t=ft.themeColors().palette;this.settings.colors=[],this.settings.colors[0]=t.azure[ft.uplift||ft.new?"80":"70"].value,this.settings.colors[1]=t.slate[40].value,this.settings.colors[2]=t.slate[30].value,this.settings.colors[3]=t.slate[20].value}return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.sparklineColors=d3.scaleOrdinal().range(this.settings.colors),this.build().handleEvents()},build:function(){var t,e,i,n=this,s=this,a=this.element.closest(".is-personalizable").length>0,o=[],r=[],c=s.settings.tooltip,d=0,h=0,u=0,p=this.calculateAspectRatioFit({srcWidth:385,srcHeight:65,maxWidth:this.element.width(),maxHeight:600}),g=p.width>300?4:3,f=s.settings.dataset;for(i=0;i<f.length;i++)h=d3.min([d3.min(f[i].data),h]),d=d3.max([d3.max(f[i].data),d]),u=d3.max([f[i].data.length,u]);var m=p.width,v=p.height,b=d3.scaleLinear().domain([0,u]).range([10,m-10]),y=d3.scaleLinear().domain([h,d]).range([v-10,10]),w=d3.line().x((function(t,e){return b(e)})).y((function(t){return y(t)}));bi.appendTooltip();var x=d3.select(this.element[0]).append("svg").attr("height",v).attr("width",m);if(s.settings.isMedianRange){d=d3.max(f[0].data),h=d3.min(f[0].data);var C=m-45,k=d3.median(f[0].data),T=d-h,A=d3.scaleLinear().domain([h,d]).range([0,v]),S=v-A(k>T?k:T),D=v-A(k<T?k:T),I=$("html").is('[class*="dark"]')?"#F0F0F0":"#525257";x.append("g").attr("class","medianrange").attr("transform",(function(){return"translate(".concat(10,",").concat(S,")")})).append("rect").attr("width",C).attr("height",D).style("opacity","0.06").style("fill",I).call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=f[0];E.addAttributes($(t),e,e.attributes,"medianrange")}))}))})).on("mouseenter.".concat(s.namespace),(function(){var e=this.getBoundingClientRect(),n='<p class="sparkline-tooltip">'+H.translate("Median")+": <b>"+k+"</b><br>"+H.translate("Range")+": <b>"+T+"</b>"+(s.settings.isPeakDot?"<br>"+H.translate("Peak")+": <b>"+d+"</b>":"")+"</p>",r=function(){var t=bi.tooltipSize(n),i=e.left+(e.width-t.width)/2,s=e.top-t.height-5;""!==n&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[a?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(i,s,n,"top"))};if(c&&"function"==typeof c&&!o[i]){n="";var h=!0;t=setInterval((function(){h&&(h=!1,c((function(t){n=t,o[i]=t}))),""!==n&&(clearInterval(t),r())}),10)}else c="object"===l(c)?"":c,n=o[i]||c||f[0].tooltip||n||"",r()})).on("mouseleave.".concat(s.namespace),(function(){clearInterval(t),bi.hideTooltip()}))}var L=function(){var t=f[i];x.append("g").append("path").attr("d",w(t.data)).attr("stroke",s.settings.isMinMax?"#999999":n.sparklineColors(i)).attr("class","team connected-line").call((function(e){e._groups.forEach((function(e){e.forEach((function(e){E.addAttributes($(e),t,t.attributes,"connected-line")}))}))}))};for(i=0;i<f.length;i++)L();return h=d3.min(f[0].data),x.selectAll(".point").data(f[0].data).enter().append("circle").attr("r",(function(t){return s.settings.isMinMax&&d===t||s.settings.isMinMax&&h===t?g+1:s.settings.isDots||s.settings.isPeakDot&&d===t?g:0})).attr("class",(function(t){return s.settings.isPeakDot&&d===t&&!s.settings.isMinMax?"point peak":s.settings.isMinMax&&d===t?"point max":s.settings.isMinMax&&h===t?"point min":"point"})).style("fill",(function(t){return s.settings.isPeakDot&&d===t&&!s.settings.isMinMax?"#ffffff":s.settings.isMinMax&&d===t?"#56932E":s.settings.isMinMax&&h===t?"#941E1E":s.sparklineColors(0)})).style("stroke",(function(t){return s.settings.isPeakDot&&d===t&&!s.settings.isMinMax?s.sparklineColors(0):s.settings.isMinMax&&d===t||s.settings.isMinMax&&h===t?"none":"#ffffff"})).style("cursor","pointer").attr("cx",(function(t,e){return b(e)})).attr("cy",(function(t){return y(t)})).call((function(t){t._groups.forEach((function(t){t.forEach((function(t,e){var i=f[0];E.addAttributes($(t),i,i.attributes,"point-".concat(e))}))}))})).on("mouseenter.".concat(s.namespace),(function(t,n){var o=this.getBoundingClientRect(),u="<p>".concat(f[0].name?"".concat(f[0].name,"<br> ").concat(s.settings.isMinMax&&d===n?"".concat(H.translate("Highest"),": "):s.settings.isMinMax&&h===n?"".concat(H.translate("Lowest"),": "):s.settings.isPeakDot&&d===n?"".concat(H.translate("Peak"),": "):""):"","<b>").concat(n,"</b></p>"),p=function(){var t=bi.tooltipSize(u),e=o.left-t.width/2+6,i=o.top-t.height-8;""!==u&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[a?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(e,i,u,"top"))};if(c&&"function"==typeof c&&!r[i]){u="";var m=!0;e=setInterval((function(){m&&(m=!1,c((function(t){u=t,r[i]=t}))),""!==u&&(clearInterval(e),p())}),10)}else c="object"===l(c)?"":c,u=r[i]||c||f[0].tooltip||u||"",p();d3.select(this).attr("r",s.settings.isMinMax&&d===n||s.settings.isMinMax&&h===n?g+2:g+1)})).on("mouseleave.".concat(s.namespace),(function(t,i){clearInterval(e),bi.hideTooltip(),d3.select(this).attr("r",s.settings.isMinMax&&d===i||s.settings.isMinMax&&h===i?g+1:g)})).on("contextmenu.".concat(s.namespace),(function(t,e){var i={value:e,name:f[0].name||""};s.settings.isMinMax&&d===e&&(i.highest=!0),s.settings.isMinMax&&h===e&&(i.lowest=!0),s.settings.isPeakDot&&d===e&&(i.peak=!0),bi.triggerContextMenu(s.element,d3.select(this).nodes()[0],i,t)})),this.element.trigger("rendered",x),this},calculateAspectRatioFit:function(t){var e=Math.min(t.maxWidth/t.srcWidth,t.maxHeight/t.srcHeight);return{width:t.srcWidth*e,height:t.srcHeight*e}},handleEvents:function(){var t=this;return this.element.on("updated.".concat(Ai),(function(e,i){t.updated(i)})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated()})),this},handleResize:function(){this.width!==this.element.width()&&(this.width=this.element.width(),this.element.is(":visible")&&this.updated())},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.element.empty(),this.teardown().init()},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".medianrange").off(e(["mouseenter","mouseleave"])),this.element.find(".point").off(e(["mouseenter","mouseleave","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),delete this.namespace,this},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("sparkline"),$.removeData(this.element[0],Ai),$.removeData(this.element[0],"chart"))}};var Ei="emptymessage",Di={title:null,info:null,icon:null,button:null,height:null,color:"slate",attributes:null};function Ii(t,e){this.settings=E.mergeSettings(t,e,Di),this.element=$(t),this.init()}Ii.prototype={init:function(){this.setup().build()},setup:function(){return this.element.addClass("empty-message"),this},build:function(){var t,e=this.settings,i="small"===e.height;if(null!=e&&null!==(t=null==e?void 0:e.button)&&void 0!==t&&(null==t?void 0:t.isPrimary)&&(this.settings.color="azure"),e.icon&&"null"!==e.icon&&null!==e.icon&&void 0!==e.icon&&!i&&$('<div class="empty-icon">\n          <svg class="icon-empty-state is-'.concat(this.settings.color,'" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#').concat(e.icon,'"></use>\n          </svg></div>')).appendTo(this.element),e.title&&("[NoData]"===e.title&&(e.title=H?H.translate("NoData"):"No data available"),$('<div class="empty-title">'.concat(e.title,"</div>")).appendTo(this.element)),e.info&&!i&&$('<div class="empty-info">'.concat(e.info,"</div>")).appendTo(this.element),e.button&&!i){var n='<div class="empty-actions">\n          <button type="button" class="'.concat(e.button.isPrimary?"btn-primary":"btn-secondary"," ").concat(e.button.cssClass,' hide-focus" ').concat(e.button.id?'id="'.concat(e.button.id,'"'):"",">\n            <span>").concat(e.button.text,"</span>\n          </button>\n        </div>");$(n).appendTo(this.element),e.button.click&&this.element.off("click").on("click","button",e.button.click)}return E.addAttributes(this.element,this,e.attributes,"",!0),E.addAttributes(this.element.find(".empty-actions button"),this,e.attributes,"btn",!0),this},updated:function(t){this.settings=E.mergeSettings(this.element,t,this.settings),this.element.empty(),this.build()},destroy:function(){$.removeData(this.element[0],Ei),this.element.empty(),this.element.removeClass("empty-message")}},$.fn.emptymessage=function(t){return this.each((function(){var e=$.data(this,Ei);e?e.updated(t):e=$.data(this,Ei,new Ii(this,t))}))};var Li="line",Pi={dataset:[],isArea:!1,isBubble:!1,isScatterPlot:!1,showLegend:!0,selectable:!0,hideDots:!1,animate:!0,redrawOnResize:!0,fitHeight:!0,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function Mi(t,e){this.settings=E.mergeSettings(t,e,Pi),this.element=$(t),bi.destroy(t),this.init()}Mi.prototype={init:function(){return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.initialSelectCall=!1,this.settings.localeInfo&&d3.formatDefaultLocale(this.settings.localeInfo),this.build().handleEvents(),"initial"===this.settings.animate&&(this.settings.animate=!1),this.element.trigger("rendered",[this.svg]),this},build:function(){var t,e,i,n,s,a=this,o=this.settings,r=this.element.closest(".is-personalizable").length>0,c="line-chart";c+=o.isBubble?" bubble":"",c+=o.isScatterPlot?" scatterplot":"",this.element.addClass(c);var d=0===o.dataset.length;if(!d){var h=0;o.dataset.forEach((function(t){0===t.data.length&&h++})),d=h===o.dataset.length}if(d)return a.element.emptymessage(o.emptyMessage),this;var u,p={radius:5,radiusOnHover:7,strokeWidth:2,class:"dot"};o.isBubble&&(p.radius=0,p.radiusOnHover=0,p.strokeWidth=0),o.isScatterPlot&&(p.radius=0,p.strokeWidth=50),$.extend(!0,p,o.dots),a.dots=p,this.isRTL=H.isRTL();var g,m,v=[],b=o.tooltip,y={},w={atLeastOne:!1},x=["left","top","right","bottom"];if(o.axisLabels&&$.extend(!0,y,o.axisLabels),!$.isEmptyObject(y))for(g=0,m=x.length;g<m;g++){var C=y[x[g]];C&&"string"==typeof C&&""!==$.trim(C)&&(w[x[g]]=!0,w.atLeastOne=!0)}var k="",T=0,A=o.dataset,S=o.hideDots,D=this.element.parent(),I=!!$(".widget-chart-action",D).length,L=D.width()<450,P=function(t){return void 0!==t&&"function"!=typeof t&&null!==t?"boolean"==typeof t?t?"-45":null:t:null},M={large:P(null===(t=o.xAxis)||void 0===t||null==t?void 0:t.rotate),small:P(null===(e=o.xAxis)||void 0===e||null==e?void 0:e.rotateOnSmallView)},B=!!M.large;B&&(M.use=M.large),L&&M.small&&(B=!0,M.use=M.small),I&&(o.fitHeight=!1);var O=function(t,e){return d3.max(t.data,(function(t){return e?t.value[e]:t.value}))};if(B){A[0].data.forEach((function(t){t.name.length>k.length&&(k=t.name)}));var R=Math.abs(M.use);R=isNaN(R)?0:R,T=k.length*(R>50?5:2)}var z={top:w.top?I?15:40:I?5:30,right:w.right?65:55,bottom:w.bottom?B?60:50:T+35,left:w.left?75:65},F=E.getParentAvailableHeight(a.element[0]),N=o.fitHeight?F:parseInt(D.height(),10),q=D.width()-z.left-z.right,V=N-z.top-z.bottom-30;I&&(V-=42),a.svg=d3.select(this.element[0]).append("svg").attr("width",q+z.left+z.right).attr("height",V+z.top+z.bottom).append("g").attr("transform","translate(".concat(z.left,",").concat(z.top,")"));var W=A[0].data.map((function(t){return t.name})),j={};A[0]&&A[0].valueFormatterString&&$.extend(!0,j,A[0].valueFormatterString);var U,G=function(t,e){return!$.isEmptyObject(j)&&t?d3.format(t)("0.0%"===t?e/100:e):e},_={name:"Name",value:{x:"Value.x",y:"Value.y",z:"Value.z"}};A[0]&&A[0].labels&&$.extend(!0,_,A[0].labels);var Y=(o.xAxis&&o.xAxis.scale?o.xAxis.scale:d3.scaleLinear()).range([0,q]),K=d3.scaleLinear().range([V,0]),X=d3.scaleLinear().range([1,25]);U=o.isBubble?{x:A.map((function(t){return O(t,"x")})),y:A.map((function(t){return O(t,"y")})),z:A.map((function(t){return O(t,"z")}))}:o.isScatterPlot?{x:A.map((function(t){return O(t,"x")})),y:A.map((function(t){return O(t,"y")}))}:A.map((function(t){return O(t)}));var Q,J,Z=d3.max(A.map((function(t){return t.data.length}))),tt=Z<=1?Z:Z-1,et=Y.domain(o.xAxis&&o.xAxis.domain?o.xAxis.domain:[0,o.isBubble||o.isScatterPlot?d3.max(U.x):tt]),it=K.domain([0,d3.max(o.isBubble||o.isScatterPlot?U.y:U)]).nice(),nt=X.domain([0,d3.max(o.isBubble?U.z:o.isScatterPlot?U.y:U)]).nice(),st=tt;(o.xAxis&&o.xAxis.ticks&&(st="auto"===o.xAxis.ticks?Math.max(q/85,2):o.xAxis.ticks,(o.isBubble||o.isScatterPlot&&L)&&(st=Math.round(tt/2))),L)&&(null!==(Q=o.xAxis)&&void 0!==Q&&(null==Q?void 0:Q.numTicksOnSmallView)&&(st=null===(J=o.xAxis)||void 0===J||null==J?void 0:J.numTicksOnSmallView));var at,ot=d3.axisBottom(et).ticks(st).tickPadding(10).tickSize(o.isBubble||o.isScatterPlot?-(V+10):0).tickFormat((function(t,e){var i,n;return"function"==typeof(null===(i=o.xAxis)||void 0===i||null==i?void 0:i.formatter)?o.xAxis.formatter(t,e):"auto"===(null===(n=o.xAxis)||void 0===n||null==n?void 0:n.ticks)?W[t]:o.isBubble||o.isScatterPlot?t:W[t]})),rt=d3.axisLeft(it).ticks(8).tickSize(-(q+20)).tickPadding(a.isRTL?-18:20);if(null!==(i=o.yAxis)&&void 0!==i&&(null==i?void 0:i.formatter)&&rt.tickFormat((function(t,e){return"function"==typeof o.yAxis.formatter?o.yAxis.formatter(t,e):t})),null!==(n=o.yAxis)&&void 0!==n&&(null==n?void 0:n.ticks)&&rt.ticks(o.yAxis.ticks.number,o.yAxis.ticks.format),w.atLeastOne){var lt=a.svg.append("g").attr("class","axis-labels"),ct={top:"translate(".concat(q/2,",").concat(-10,")"),right:"translate(".concat(q+28,",").concat(V/2,")rotate(90)"),bottom:"translate(".concat(q/2,",").concat(V+40,")"),left:"translate(".concat(-40,",").concat(V/2,")rotate(-90)")},dt={top:"rotate(0deg) scaleX(-1) translate(-".concat(q/2,"px, ").concat(-10,"px)"),right:"rotate(90deg) scaleX(-1) translate(-".concat(V/2+5,"px, -").concat(q+28,"px)"),bottom:"rotate(0deg) scaleX(-1) translate(-".concat(q/2,"px, ").concat(V+40,"px)"),left:"rotate(90deg) scaleX(-1) translate(-".concat(V/2-5,"px, ",55,"px)")};for(g=0,m=x.length;g<m;g++)w[at=x[g]]&&lt.append("text").attr("class","axis-label-".concat(at)).attr("text-anchor","middle").attr("transform",a.isRTL?"":ct[at]).style("font-size","1.25em").style("transform",a.isRTL?dt[at]:"").text(y[at])}if(a.svg.append("g").attr("class","x axis").attr("transform","translate(0,".concat(V,")")).call(ot),a.svg.append("g").attr("class","y axis").call(rt),a.svg.selectAll(".tick line").attr("x1","-10"),o.isBubble||o.isScatterPlot){a.svg.selectAll(".x.axis .tick line, .y.axis .tick line").style("opacity",0);var ht=a.svg.select(".x.axis .tick line");ht.attr("x2","-10"),(!o.xAxis||o.xAxis&&!o.xAxis.hideLine)&&ht.style("opacity",1),(!o.yAxis||o.yAxis&&!o.yAxis.hideLine)&&a.svg.select(".y.axis .tick line").style("opacity",1)}if(a.isRTL&&(a.svg.selectAll("text").attr("transform","scale(-1, 1)"),a.svg.selectAll(".y.axis text").style("text-anchor","end")),B){var ut=a.isRTL?"scale(-1, 1) rotate(".concat(M.use,"deg)"):"rotate(".concat(M.use,"deg)");a.svg.selectAll(".x.axis .tick text").attr("y",0).attr("x",(function(){return a.isRTL?-10:-(this.getBBox().width+10)})).attr("dy","1em").attr("style","text-anchor: start; transform: ".concat(ut," !important;"))}o.xAxis&&o.xAxis.formatText&&a.svg.selectAll(".x.axis .tick text").each((function(t){var e=d3.select(this),i=d3.select(this).text(),n=o.xAxis.formatText(i,t);f.html(e.node(),n,"<tspan><text><glyph>")}));var pt=d3.line().x((function(t,e){var i;return et("function"==typeof(null===(i=o.xAxis)||void 0===i||null==i?void 0:i.parser)?o.xAxis.parser(t,e):o.isBubble||o.isScatterPlot?t.value.x:e)})).y((function(t){return it(o.isBubble||o.isScatterPlot?t.value.y:t.value)})),gt={line:{delay:200,prevent:!1,timer:0},dot:{delay:200,prevent:!1,timer:0},symbol:{delay:200,prevent:!1,timer:0}};a.element.off("dblclick.".concat(a.namespace)).on("dblclick.".concat(a.namespace),"*",(function(t){t.stopImmediatePropagation()})),A.forEach((function(t,e){var i=a.svg.append("g").attr("data-group-id",e).attr("class","line-group");if(o.isArea){var n=d3.area().x((function(t,e){return et(e)})).y0(V).y1((function(t){return it(o.isBubble||o.isScatterPlot?t.value.y:t.value)}));i.append("path").datum(t.data).attr("fill",(function(){return bi.chartColor(e,"line",t)})).style("opacity",".2").attr("class","area").attr("d",n).call((function(e){e._groups.forEach((function(e){e.forEach((function(e){E.addAttributes($(e),t,t.attributes,"area")}))}))}))}var s=i.append("path").datum(t.data).attr("d",pt(t.data)).attr("stroke",(function(){return o.isBubble||o.isScatterPlot?"":bi.chartColor(e,"line",t)})).attr("stroke-width",2).attr("fill","none").attr("class","line").call((function(e){e._groups.forEach((function(e){e.forEach((function(e){E.addAttributes($(e),t,t.attributes,"line")}))}))})).on("click.".concat(a.namespace),(function(){var e=this;a.settings.selectable&&(gt.line.timer=setTimeout((function(){gt.line.prevent||bi.selectElement(d3.select(e.parentNode),a.svg.selectAll(".line-group"),t,a.element,A,a.initialSelectCall,a.settings.selectable),gt.line.prevent=!1}),gt.line.delay))})).on("dblclick.".concat(a.namespace),(function(){clearTimeout(gt.line.timer),gt.line.prevent=!0;var e=[{elem:[this.parentNode],data:t}];a.element.triggerHandler("dblclick",[e])})).on("contextmenu.".concat(a.namespace),(function(e){bi.triggerContextMenu(a.element,d3.select(this).nodes()[0],t,e)})),c=s.node().getTotalLength?s.node().getTotalLength():0;s.attr("stroke-dasharray","".concat(c," ").concat(c)).attr("stroke-dashoffset",c).transition().duration(o.animate?600:0).ease(d3.easeCubic).attr("stroke-dashoffset",0);var d=function(e,i,n){var s,a=e.getBoundingClientRect(),c="<p><b>".concat(n.name," </b> ").concat((s=n.value,d3.format(o.formatterString||",")(s)),"</p>"),d=function(){var t=bi.tooltipSize(c),e=a.left-t.width/2+6,i=a.top-t.height-18;e=o.isBubble||o.isScatterPlot?a.left+a.width/2-t.width/2:e,""!==c&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[r?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(e,i,c,"top"))};if(o.isBubble||o.isScatterPlot){for(var h in c='<div class="chart-swatch line"><div class="swatch-caption"><span class="indicator-box"></span>\n            <b>'.concat(n.name,"</b></div>"),n)if(n.hasOwnProperty(h))if("object"!==l(n[h]))_[h]&&(c+="".concat('<div class="swatch-row"><span>').concat(_[h],"</span>")+"<b>".concat(t.name,"</b>")+"</div>");else{var f=n[h];for(var m in f)f.hasOwnProperty(m)&&_[h]&&(c+="".concat('<div class="swatch-row"><span class="text-capitalize">').concat(_[h][m],"</span>")+"<b>".concat(G(j[m],f[m]),"</b>")+"</div>")}c+="</div>"}if(b&&"function"==typeof b&&!v[g]){c="";var y=!0;u=setInterval((function(){y&&(y=!1,b((function(t){c=t,v[g]=t}))),""!==c&&(clearInterval(u),d())}),10)}else{b="object"===l(b)?"":b,c=v[g]||b||n.tooltip||t.tooltip||c||"",d();for(var w=document.querySelectorAll("#svg-tooltip .swatch-caption span"),x=0;x<w.length;x++)w[x].style.backgroundColor=bi.chartColor(o.isBubble||o.isScatterPlot?n.lineIdx:x,"line",n)}d3.select(this).attr("r",(function(t){return o.isBubble?2+nt(t.value.z):p.radiusOnHover}))};S||(o.isScatterPlot||i.selectAll("circle").data(t.data).enter().append("circle").attr("class",p.class).call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,"dot")}))}))})).attr("cx",(function(t,e){return o.xAxis&&o.xAxis.parser?et(o.xAxis.parser(t,e)):et(o.isBubble||o.isScatterPlot?t.value.x:e)})).attr("cy",(function(t){return it(o.isBubble||o.isScatterPlot?0:t.value)})).attr("r",p.radius).style("stroke-width",p.strokeWidth).style("cursor",a.settings.selectable?"pointer":"inherit").style("fill",(function(){return bi.chartColor(e,"line",t)})).style("opacity",o.isBubble||o.isScatterPlot?".7":"1").on("mouseenter.".concat(a.namespace),(function(t,i){i.lineIdx=e,d(this,0,i)})).on("mouseleave.".concat(a.namespace),(function(){clearInterval(u),bi.hideTooltip(),d3.select(this).attr("r",(function(t){return o.isBubble?nt(t.value.z):p.radius}))})).on("click.".concat(a.namespace),(function(t,e){var i=this;a.settings.selectable&&(gt.dot.timer=setTimeout((function(){gt.dot.prevent||bi.selectElement(d3.select(i.parentNode),a.svg.selectAll(".line-group"),e,a.element,A,a.initialSelectCall,a.settings.selectable),gt.dot.prevent=!1}),gt.dot.delay))})).on("dblclick.".concat(a.namespace),(function(t,e){clearTimeout(gt.dot.timer),gt.dot.prevent=!0;var i=[{elem:[this.parentNode],data:e}];a.element.triggerHandler("dblclick",[i])})).on("contextmenu.".concat(a.namespace),(function(t,e){bi.triggerContextMenu(a.element,d3.select(this).nodes()[0],e,t)})),o.isScatterPlot&&i.selectAll(".symbol").data(t.data).enter().append("path").attr("class","symbol").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,"symbol")}))}))})).attr("transform",(function(t){return"translate(".concat(et(t.value.x),",").concat(it(t.value.y),")")})).attr("d",d3.symbol().size(p.strokeWidth).type((function(){return d3.symbols[e]}))).style("opacity",0).style("fill",(function(){return bi.chartColor(e,"line",t)})).on("mouseenter.".concat(a.namespace),(function(t,i){i.lineIdx=e,d(this,0,i)})).on("mouseleave.".concat(a.namespace),(function(){clearInterval(u),bi.hideTooltip(),d3.select(this).attr("r",(function(){return p.radius}))})).on("click.".concat(a.namespace),(function(t,e){var i=this;a.settings.selectable&&(gt.symbol.timer=setTimeout((function(){gt.symbol.prevent||bi.selectElement(d3.select(i.parentNode),a.svg.selectAll(".line-group"),e,a.element,A,a.initialSelectCall,a.settings.selectable),gt.symbol.prevent=!1}),gt.symbol.delay))})).on("dblclick.".concat(a.namespace),(function(t,e){clearTimeout(gt.symbol.timer),gt.symbol.prevent=!0;var i=[{elem:[this.parentNode],data:e}];a.element.triggerHandler("dblclick",[i])})).on("contextmenu.".concat(a.namespace),(function(t,e){bi.triggerContextMenu(a.element,d3.select(this).nodes()[0],e,t)})),o.isBubble&&i.selectAll("circle").attr("cy",(function(t){return it(t.value.y)})).transition().duration(o.animate?750:0).ease(d3.easeCubic).attr("r",(function(t){return nt(t.value.z)})),o.isScatterPlot&&i.selectAll(".symbol").transition().duration(o.animate?750:0).ease(d3.easeCubic).style("opacity",1))})),a.svg.selectAll(".y.axis .tick").attr("class",(function(t){return"tick".concat(0===t?" tick0":"")}));var ft=A.map((function(t){return{color:t.color,name:t.name,selectionObj:a.svg.selectAll(".line-group"),selectionInverse:a.svg.selectAll(".line-group"),data:t}}));if(o.showLegend&&bi.addLegend(ft,o.isScatterPlot?"scatterplot":"line",o,this.element),bi.appendTooltip(),!B&&!o.xAxis&&(null===(s=a.settings.xAxis)||void 0===s||!(null==s?void 0:s.rotate))&&bi.labelsColide(a.svg)){var mt=[];A.forEach((function(t){mt=mt.concat(t.data)})),bi.applyAltLabels(a.svg,mt,"shortName"),bi.labelsColide(a.svg)&&bi.applyAltLabels(a.svg,mt,"abbrName"),bi.labelsColide(a.svg)&&bi.applyAltLabels(a.svg,mt,null,null,!0)}return bi.setSelected=function(t,e){var i,n,s,o=0,r=E.equals,l=function(e,l,c,d){c?(s=a.svg.select('[data-group-id="'.concat(l,'"]')).select(".dot:nth-child(".concat(d+2,")")),("number"==typeof t.groupIndex&&void 0!==t.fieldName&&void 0!==t.fieldValue&&t.groupIndex===l&&t.fieldValue===c[t.fieldName]||void 0!==t.index&&"number"==typeof t.groupIndex&&t.groupIndex===l&&t.index===d||t.elem&&$(s.node()).is(t.elem)||t.data&&r(t.data,c))&&(o++,n=c,i=a.svg.select('[data-group-id="'.concat(l,'"]')))):(s=a.svg.select('[data-group-id="'.concat(l,'"]')),(void 0!==t.groupName&&void 0!==t.groupValue&&t.groupValue===e[t.groupName]||void 0!==t.groupIndex&&t.groupIndex===l||t.elem&&$(s.node()).is(t.elem)||t.data&&r(t.data,e))&&(o++,n=e,i=s))};A.forEach((function(t,e){o<1&&t&&t.data&&(t.data.forEach((function(i,n){o<1&&i&&l(t,e,i,n)})),o<1&&l(t,e))})),o>0&&(e||!i.classed("is-selected"))&&a.settings.selectable&&bi.selectElement(i,a.svg.selectAll(".line-group"),n,a.element,A,a.initialSelectCall)},a.svg._groups.forEach((function(t){t.forEach((function(t){var e=$(t).find("g.y.axis .tick"),i=$(t).find("g.line-group"),n=$(t).find("g.y.axis"),s=e.attr("transform"),o=a.getTransformYAxisValue(s);e.length<2&&n.add(i).attr("transform","translate(0,".concat(o,")"))}))})),this.setInitialSelected(),this.setTextValues(),this.element.trigger("rendered"),this},getTransformYAxisValue:function(t){if(t){var e=t.split(",");return e.splice(0,1).join(""),e.join(",").slice(0,-1)}return"0"},setInitialSelected:function(){var t,e,i=this,n=0,s=function(s,a,o){s.selected&&n<1&&(n++,t=d3.select(i.svg.selectAll(".line-group").nodes()[o]),e=a)};this.settings.dataset.forEach((function(t,e){t&&s(t,t,e)})),this.settings.dataset.forEach((function(t,e){(t||t.data)&&t.data.forEach((function(i){s(i,t,e)}))})),n>0&&i.settings.selectable&&(i.initialSelectCall=!0,bi.selectElement(t,i.svg.selectAll(".line-group"),e,i.element,i.settings.dataset,i.initialSelectCall)),i.initialSelectCall=!1},setTextValues:function(){var t,e,i,n,s,a,o,r,l=this,c=this,d={el:this.element[0].querySelector(".line-chart .axis.y")},h={el:d.el.querySelector(".tick line")},u=[].slice.call(d.el.querySelectorAll(".tick text")),p=null===(t=this.settings)||void 0===t||null===(e=null==t?void 0:t.axisLabels)||void 0===e||null==e?void 0:e.left,g="string"==typeof p&&""!==p.trim(),f={maxWidth:this.element.width(),boxWidth:g?23:43,transMatrix:null===(i=this.svg)||void 0===i||null===(n=null==i?void 0:i.node())||void 0===n||null===(s=null==n?void 0:n.transform)||void 0===s||null===(a=null==s?void 0:s.baseVal)||void 0===a||null===(o=null==a?void 0:a.consolidate())||void 0===o||null==o?void 0:o.matrix};if(f.transY=(null===(r=f.transMatrix)||void 0===r||null==r?void 0:r.f)||30,f.customCss=function(){return{tooltip:{maxWidth:"".concat(f.maxWidth,"px")},arrow:{left:l.isRTL?"calc(100% - 20px)":"20px"}}},d.width=d.el.getBBox?d.el.getBBox().width:0,h.width=h.el&&h.el.getBBox?h.el.getBBox().width:0,f.xDiff=d.width-h.width,this.settings.selectable||this.element.find(".line").css("cursor","inherit"),u.forEach((function(t,e){var i=t.textContent,n=bi.calculateTextRenderWidth(i),s=t.parentNode;if(n>=f.boxWidth){if(f.elem=t,x.browser.isIE11())t.textContent=bi.trimText(i,5);else{t.classList.add("hidden"),d3.select(s).append("foreignObject").attr("overflow","visible").attr("width","".concat(f.boxWidth)).attr("height","16").attr("class","foreign-object tick-y".concat(e)).attr("x","-".concat(f.boxWidth+(l.isRTL?12:16))).attr("y","-1em").attr("requiredFeatures","http://www.w3.org/TR/SVG11/feature#Extensibility").html('<div class="text ellipsis" resizeable="false" xmlns="http://www.w3.org/1999/xhtml"></div>');var a=s.querySelector(".text.ellipsis");if(a){var o=document.createTextNode(i);a.appendChild(o)}f.elem=s.querySelector(".text")}d3.select(f.elem).on("mouseover.".concat(c.namespace),(function(){var t,e,s,a,o=(t=this.getBoundingClientRect(),e=Math.ceil(n/f.maxWidth),s=t.left,a=t.top,s=s<0?30:s,a=(a<0?30:a)-36,e>1&&(a-=.5*(e-1)*33),l.isRTL&&(s=t.left-n+5),{x:s,y:a});bi.showTooltip(o.x,o.y,i,"top",f.customCss())})).on("mouseout.".concat(l.namespace),(function(){return bi.hideTooltip()}))}})),!g){d.width=d.el.getBBox?d.el.getBBox().width:0,h.width=h.el&&h.el.getBBox?h.el.getBBox().width:0,f.xDiff=d.width-h.width;var m=[{min:0,max:23,val:62},{min:23,max:26,val:65},{min:26,max:30,val:67},{min:30,max:32,val:69},{min:32,max:34,val:70},{min:34,max:38,val:71},{min:38,max:42,val:73},{min:42,max:47,val:77},{min:47,max:55,val:78}].filter((function(t){return f.xDiff>=t.min&&f.xDiff<t.max}));m.length&&(f.newX=m[0].val,d3.select(this.svg.node()).attr("transform","translate(".concat(f.newX,",").concat(f.transY,")")),x.browser.isIE11()&&this.isRTL&&d3.selectAll(".line-chart .axis.y .tick text").attr("x",f.boxWidth+3))}},handleEvents:function(){var t=this;return this.element.on("updated.".concat(this.namespace),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated()})),this},width:0,getSelected:function(){return bi.selected},setSelected:function(t,e){var i={svg:this.svg,chartData:this.settings.dataset,isStacked:!1,isGrouped:!1,isSingle:!1};bi.setSelected(t,e,i)},toggleSelected:function(t){this.setSelected(t,!0)},handleResize:function(){var t=this,e=function(){t.width!==t.element.width()&&(t.width=t.element.width(),t.element.is(":visible")&&t.updated())};this.element.closest(".homepage").length?setTimeout((function(){return e()}),300):e()},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this.element.empty(),t&&t.dataset&&(this.settings.dataset=t.dataset),this.build()},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".line-group .line").off(e(["click","dblclick","contextmenu"])),this.element.find(".line-group .symbol").off(e(["mouseenter","mouseleave","click","dblclick","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),this.dots&&this.element&&(this.element.find(".line-group .".concat(this.dots.class)).off(e(["mouseenter","mouseleave","click","dblclick","contextmenu"])),delete this.dots),delete this.namespace,this},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("line-chart"),$.removeData(this.element[0],Li),$.removeData(this.element[0],"chart"))}};var Bi="column",Oi={dataset:[],isStacked:!1,showLegend:!0,animate:!0,selectable:!0,format:null,redrawOnResize:!0,useLine:!1,hideDots:!1,ticks:9,fitHeight:!0,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function Ri(t,e){this.settings=E.mergeSettings(t,e,Oi),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),this.init()}Ri.prototype={init:function(){return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.width=0,this.initialSelectCall=!1,this.settings.localeInfo&&d3.formatDefaultLocale(this.settings.localeInfo),this.build().handleEvents(),"initial"===this.settings.animate&&(this.settings.animate=!1),this},build:function(){var t,e,i,n,s,a=this,o=this.settings,r=this.element.closest(".is-personalizable").length>0,l=function(t){return d3.format(a.settings.formatterString||",")(t)},c=this.settings.dataset;if(this.dataset=c,0===c.length)return a.element.emptymessage(a.settings.emptyMessage),this;var d="",h=0,p=this.element.parent();$(".widget-chart-action",p).length;var g,m,v=p.width()<450,b=function(t){return void 0!==t&&"function"!=typeof t&&null!==t?"boolean"==typeof t?t?"-45":null:t:null},y={large:b(null===(t=this.settings.xAxis)||void 0===t||null==t?void 0:t.rotate),small:b(null===(e=this.settings.xAxis)||void 0===e||null==e?void 0:e.rotateOnSmallView)},w=!!y.large;if(w&&(y.use=y.large),v&&y.small&&(w=!0,y.use=y.small),w){c[0].data.forEach((function(t){t.name.length>d.length&&(d=t.name)}));var x=Math.abs(y.use);x=isNaN(x)?0:x,h=d.length*(x>50?5:2)}var C={},k={atLeastOne:!1},T=["left","top","right","bottom"];if(o.axisLabels&&$.extend(!0,C,o.axisLabels),!$.isEmptyObject(C))for(g=0,m=T.length;g<m;g++){var A=C[T[g]];A&&"string"==typeof A&&""!==$.trim(A)&&(k[T[g]]=!0,k.atLeastOne=!0)}var S=H.isRTL(),D="column-positive-negative"===this.settings.type||"positive-negative"===this.settings.type,I=1===c.length;this.isSingle=I;var L=!(I||!I&&a.settings.isStacked);this.isGrouped=L;var P,M,B,O,R,z,F,N,q,V,W=40,j=40,U=I&&void 0===c[0].name?a.settings.isStacked?20:w?h+35:50:35,G=45,_=E.getParentAvailableHeight(a.element[0]),Y=o.fitHeight?_:parseInt(p.height(),10),K=p.width()-G-j-10,X=Y-W-U-(I&&void 0===c[0].name?a.settings.isStacked||D?30:0:40),Q=d3.min(c,(function(t){return d3.min(t.data,(function(t){return t.value}))})),J=d3.max(c,(function(t){return d3.max(t.data,(function(t){return t.value}))}));D&&(P=d3.min(c,(function(t){return d3.min(t.data,(function(t){return t.target}))})),M=d3.max(c,(function(t){return d3.max(t.data,(function(t){return t.target}))})),Q=d3.min([Q,P]),J=d3.max([J,M]),R={target:null,positive:null,negative:null},N={target:"Target",positive:"Positive",negative:"Negative"},z={target:"neutral",positive:"good",negative:"error"},F={},c[0]&&(c[0].attributes&&$.extend(!0,R,c[0].attributes),c[0].colors&&$.extend(!0,z,c[0].colors),c[0].legends&&$.extend(!0,N,c[0].legends),c[0].patterns&&$.extend(!0,F,c[0].patterns)),q=[],$.each(N,(function(t,e){var i={name:e,color:z[t],pattern:F[t],option:t};R[t]&&(i.data={attributes:R[t]}),q.push(i)}))),this.element.addClass("column-chart");var Z=[],tt=a.settings.tooltip,et=d3.scaleBand().range([0,K]).round(!0).padding("column-grouped"===this.settings.type?.1:0),it=d3.scaleBand(),nt=d3.scaleLinear().range([X,0]),st=null,at=null;if(a.settings.isStacked){var ot={y1:[],y2:[]};s=I?c[0].data.map((function(t,e){var i=0;return 0===e?(ot.y1.push(t.value),ot.y2.push(0)):(i=ot.y1[0]+ot.y2[0],ot.y1[0]=t.value,ot.y2[0]=i),[$.extend({},t,{y0:i,y:t.value,x:t.name,color:t.color,pattern:t.pattern,parentName:t.name,tooltip:t.tooltip})]})):c.map((function(t,e){return t.data.map((function(i,n){var s=0;return 0===e?(ot.y1.push(i.value),ot.y2.push(0)):(s=ot.y1[n]+ot.y2[n],ot.y1[n]=i.value,ot.y2[n]=s),$.extend({},i,{y0:s,y:i.value,x:i.name,color:i.color,pattern:i.pattern,parentName:t.name,tooltip:t.tooltip})}))})),d3.stack()(s),st=d3.scaleBand().domain(d3.range(s[0].length)).rangeRound([0,K],.05),I&&a.settings.isStacked&&st.paddingInner(.095),at=d3.scaleLinear().domain([0,d3.max(s,(function(t){return d3.max(t,(function(t){return t.y0+t.y}))}))]).range([0,X])}var rt=c[0].data.map((function(t){return t.name})),lt=d3.axisBottom(et).tickSize(0).tickPadding(12),ct=d3.axisLeft(nt).tickSize(-K).tickPadding(S?-12:12).ticks(a.settings.ticks||9,d3.format(a.settings.format||"s"));a.settings.yAxis&&(a.settings.yAxis.formatter&&ct.tickFormat((function(t,e){return"function"==typeof a.settings.yAxis.formatter?a.settings.yAxis.formatter(t,e):t})),a.settings.yAxis.ticks&&a.settings.yAxis.ticks.number>1&&a.settings.yAxis.ticks.format&&ct.ticks(a.settings.yAxis.ticks.number,a.settings.yAxis.ticks.format));var dt=d3.select(this.element[0]).append("svg").attr("width",K+G+j).attr("height",X+W+U+(k.atLeastOne?12:0)).append("g").attr("transform","translate(".concat(S?G+5:k.atLeastOne?G+11:G,"\n        ,").concat(W-(k.atLeastOne?5:0),")"));this.svg=dt;var ht,ut,pt=c.map((function(t){return t.name})),gt=c.map((function(t){return d3.max(t.data,(function(t){return D?t.target:t.value}))}));if(a.settings.isStacked&&(ht=s.map((function(t){return d3.max(t,(function(t){return t.y+t.y0}))}))),I&&(pt=c[0].data.map((function(t){return t.name}))),D&&(Q+=Q/nt.ticks().length,gt[0]+=gt[0]/(nt.ticks().length/2)),function(){if(a.settings.isStacked&&I)B=c[0].data;else for(var t,e=c.length-1;e>-1;e--)t=c[e].data.map((function(t){return t})),$.extend(!0,B,t),B=$.map(B,(function(t){return t}))}(),a.settings.isStacked&&!I&&(O=pt.map((function(t,e){return c[e]}))),et.domain(a.settings.isStacked?rt:pt),it.domain(rt).rangeRound([0,I||a.settings.isStacked?K:et.bandwidth()]),nt.domain([Q<0?Q:a.settings.minValue||0,d3.max(a.settings.isStacked?ht:gt)]).nice(),(!I||I&&!a.settings.isStacked)&&dt.append("g").attr("class","x axis").attr("transform","translate(0,".concat(X+(D?10:0),")")).call(lt),dt.append("g").attr("class","y axis").call(ct),S&&Q<0){var mt,vt=0;dt.selectAll(".axis.y text").attr("class",(function(t){return mt=d3.select(this).text().length,vt=mt>vt?mt:vt,t<0?"negative-value":"positive-value"})).attr("x",(function(t){return vt*(t<0?9:5)}))}if(k.atLeastOne){var bt=a.svg.append("g").attr("class","axis-labels"),yt=K-45,wt={top:"translate(".concat(yt/2,",").concat(-10,")"),right:"translate(".concat(yt+53,",").concat(X/2,")rotate(90)"),bottom:"translate(".concat(yt/2,",").concat(X+47,")"),left:"translate(".concat(-35,",").concat(X/2,")rotate(-90)")},xt={top:"rotate(0deg) scaleX(-1) translate(-".concat(yt/2,"px, ").concat(-10,"px)"),right:"rotate(90deg) scaleX(-1) translate(-".concat(X/2+5,"px, -").concat(yt+(S?55:28),"px)"),bottom:"rotate(0deg) scaleX(-1) translate(-".concat(yt/2,"px, ").concat(X+47,"px)"),left:"rotate(90deg) scaleX(-1) translate(-".concat(X/2-5,"px, ").concat(S?35:55,"px)")};for(g=0,m=T.length;g<m;g++)k[ut=T[g]]&&bt.append("text").attr("class","axis-label-".concat(ut)).attr("text-anchor","middle").attr("transform",S?"":wt[ut]).style("font-size","1.25em").style("transform",S?xt[ut]:"").text(C[ut])}var Ct,kt,Tt=[];c.forEach((function(t){Tt.push($.extend({},t,{values:t.data}))})),I&&(Tt=[],pt=c[0].data.forEach((function(t){Tt.push(t)})));var At=35,St=Tt[0]&&Tt[0].values?Tt[0].values.length:0,$t=K/Tt.length>At*(St+1),Et=function(t){return bi.chartColor(0,"",{color:t})},Dt=function(t,e){var i;t.empty()?e():(i=t.size(),t.on("end.".concat(a.namespace),(function(){0===--i&&e()})))},It=function(t){var e;if(t=D&&t,I||D){var i=t?"target-bar":"bar";if(e=a.svg.selectAll("rect.".concat(i)).data(a.settings.isStacked?s:Tt).enter().append("rect").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,i)}))}))})).attr("class",(function(e,i){var n="bar series-".concat(i);return D&&(n=(t?"target-bar series-".concat(i):n)+(e.value>0?" positive":" negative")),n})).attr("width",Math.min.apply(null,[it.bandwidth()-2,At])).attr("x",(function(t){return a.settings.isStacked?st(0):it(t.name)+(it.bandwidth()-At)/2})).attr("y",(function(){return nt(0)>X?X:nt(0)})).attr("height",(function(){return 0})).attr("mask",(function(e){return D?t?F.target?"url(#".concat(F.target,")"):null:e.value<0?F.negative?"url(#".concat(F.negative,")"):null:F.positive?"url(#".concat(F.positive,")"):null:null})).attr("data-index",(function(t,e){return e})).style("cursor",a.settings.selectable?"pointer":"inherit").style("fill",(function(e){return D?Et(t?z.target:e.value<0?z.negative:z.positive):null})),D){dt.selectAll(t?".target-bartext":".bartext").data(Tt).enter().append("text").attr("class",(function(e){return(t?"target-bartext":"bartext")+(e.value>0?" positive":" negative")})).attr("text-anchor","middle").attr("x",(function(t){return(it(t.name)+it.bandwidth()/2)*(S?-1:1)})).attr("y",(function(e){return t?nt(e.target)-6:nt(e.value>0?0:e.value)+12})).style("opacity",0).style("fill",(function(e){return t?"":e.value<0?Et(z.negative):Et(z.positive)})).style("font-weight","bold").text((function(e){return l(t?e.target:e.value)}))}e.transition().duration(a.settings.animate?1e3:0).call(Dt,(function(){dt.selectAll(".target-bartext, .bartext").transition().duration(a.settings.animate?300:0).style("opacity",1)})).attr("y",(function(e){var i=a.settings.isStacked?X-at(e[0].y)-at(e[0].y0):e.value<0?nt(0):nt(e.value);return t?nt(e.target):e.value<0?i:i>X-3?X-2:i})).attr("height",(function(e){var i;return i=a.settings.isStacked?at(e[0].y):e.value<0?X-nt(0)-(X-nt(e.value)):X-nt(e.value)-(X-nt(0)),i=e.value<0?i:i<3?2:i>X?X-nt(e.value):i,t?X-nt(e.target)-(X-nt(0)):i}))}else{if(e=dt.selectAll(".x-value").data(a.settings.isStacked?s:Tt).enter().append("g").attr("class","series-group g").attr("data-group-id",(function(t,e){return e})).attr("transform",(function(t){var e=et(a.settings.isStacked?rt[0]:t.name),i=et.bandwidth();(!a.settings.isStacked&&$t&&i>At*Tt.length*2&&(e+=i/2/Tt.length/2),a.isGrouped&&!a.settings.isStacked)&&(2===t.data.length?i<120?e-=13.5:e=et(t.name):e-=At/(i>150?2:4));if(a.settings.isStacked&&K<290&&i<40){var n,s,o=(null===(n=Tt[0])||void 0===n||null===(s=null==n?void 0:n.data)||void 0===s||null==s?void 0:s.length)||0;e=(K-i*o)/o/2}return"translate(".concat(e,",0)")})).selectAll("rect").data((function(t){return a.settings.isStacked?t:t.values})).enter().append("rect").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,"bar")}))}))})).attr("class",(function(t,e){return"series-".concat(e," bar")})).attr("width",Math.min.apply(null,[it.bandwidth()-2,At])).attr("x",(function(t,e){var i=Math.min.apply(null,[it.bandwidth()-2,At]);return a.settings.isStacked?st(e):it.bandwidth()/2+(i+2)*e-(1===Tt[0].values.length||5===Tt[0].values.length||4===Tt[0].values.length?i/2:0)})).attr("y",(function(){return nt(0)>X?X:nt(0)})).attr("height",(function(){return 0})),a.settings.useLine&&c.length===c.map((function(t){return t.line})).length){var n=d3.scaleBand().rangeRound([0,K]).domain(c.map((function(t){return t.name}))),o=d3.scaleLinear().range([X,0]).domain([0,d3.max(gt)]).nice(),d=d3.line().x((function(t){return n(t.name)+n.bandwidth()/2})).y((function(t){return o(t.line.value)})),h=dt.append("g").attr("class","line-group"),u=c.map((function(t){return t.line})).filter((function(t){return null==t||null==t?void 0:t.attributes})),p="dark"===ft.currentTheme.modeId&&ft.new,g="dark"===ft.currentTheme.modeId&&!ft.new;h.append("path").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e;E.addAttributes($(t),u[0],null===(e=u[0])||void 0===e||null==e?void 0:e.attributes)}))}))})).datum(c).attr("d",d(c)).attr("class","line").style("opacity",0).attr("stroke",g?"#BDBDBD":p?"#C5C5C9":ft.new&&"dark"!==ft.currentTheme.modeId?"#3B3B3F":"#313236").attr("stroke-width",2).attr("fill","none"),a.settings.hideDots||h.selectAll("circle").data(c).enter().append("circle").call((function(t){t._groups.forEach((function(t){t.forEach((function(t,e){E.addAttributes($(t),u[0],u[0].attributes,"dot-".concat(e+1))}))}))})).attr("class","dot").style("opacity",0).attr("cx",(function(t){return n(t.name)+n.bandwidth()/2})).attr("cy",(function(t){return o(t.line.value)})).attr("r",5).style("fill",g?"#BDBDBD":p?"#C5C5C9":ft.new&&"dark"!==ft.currentTheme.modeId?"#3B3B3F":"#313236").style("stroke-width",2).style("cursor","default").on("mouseenter.".concat(a.namespace),(function(t,e){!function(t,e){var i,n,s,a=t.getBoundingClientRect(),o="<p><b>".concat(e.name,"</b> ").concat(e.value,"</p>");i=bi.tooltipSize(o),n=a.left-i.width/2+6,s=a.top-i.height-18,""!==o&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[r?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(n,s,o,"top"))}(this,e.line)})).on("mouseleave.".concat(a.namespace),(function(){bi.hideTooltip()}))}e.transition().duration(a.settings.animate?600:0).call(Dt,(function(){if(a.settings.useLine){var t=dt.select(".line"),e=t.node().getTotalLength();t.style("opacity",1).attr("stroke-dasharray","".concat(e," ").concat(e)).attr("stroke-dashoffset",e).transition().duration(a.settings.animate?300:0).ease(d3.easeCubic).attr("stroke-dashoffset",0).call(Dt,(function(){dt.selectAll(".dot").transition().duration(a.settings.animate?100:0).style("opacity",1)}))}})).attr("y",(function(t){var e=a.settings.isStacked?X-at(t.y)-at(t.y0):t.value<0?nt(0):nt(t.value);return t.value<0?e:e>X-3?X-2:e})).attr("height",(function(t){var e;if(a.settings.isStacked)e=at(t.y);else{if(t.value<=0)return e=X-nt(0)-(X-nt(t.value));e=X-nt(t.value)-(X-nt(0))}return t.value<0?e:e<3?2:e>X?X-nt(t.value):e}))}return e};D&&(Ct=It(!0));var Lt=It();D&&(kt=d3.selectAll([].concat(u(Ct.nodes()),u(Lt.nodes())))),D||(a.settings.isStacked?a.settings.isStacked&&!I?Lt.style("cursor",a.settings.selectable?"pointer":"inherit").style("fill",(function(){var t=d3.select(this.parentNode).attr("data-group-id");return bi.chartColor(t,"bar",c[t])})).attr("mask",(function(){var t=d3.select(this.parentNode).attr("data-group-id");return c[t].pattern?"url(#".concat(c[t].pattern,")"):null})):a.settings.isStacked&&I&&Lt.style("fill",(function(t,e){return bi.chartColor(e,"bar",t[0])})).attr("mask",(function(t){return t[0].pattern?"url(#".concat(t[0].pattern,")"):null})):Lt.style("fill",(function(t,e){return I?bi.chartColor(e,"column-single",c[0].data[e]):bi.chartColor(e,"bar",B[e])})).style("cursor",a.settings.selectable?"pointer":"inherit").attr("mask",(function(t,e){return I?c[0].data[e].pattern?"url(#".concat(c[0].data[e].pattern,")"):null:B[e].pattern?"url(#".concat(B[e].pattern,")"):null}))),$.extend(bi.settings,{svg:dt,chartType:"Column",isSingle:I,isGrouped:L,isStacked:a.settings.isStacked});var Pt=!1,Mt=0;if(a.element.off("dblclick.".concat(a.namespace)).on("dblclick.".concat(a.namespace),"*",(function(t){t.stopImmediatePropagation()})),(D?kt:Lt).on("mouseenter.".concat(a.namespace),(function(t,e){var i,n,o,d,h,u,p,g=$(this).index(),m=this,v=$(this),b="",y=null,w=t.pageY,x=function(t,e){return t&&e?"".concat('<svg width="12" height="12"><rect style="fill: ').concat(e,'" mask="url(#').concat(t,')" height="12" width="12" />')+"</svg>":""},C=g,k=null;a.isGrouped&&((k={elem:d3.select(this.parentNode)}).data=k.elem.datum().values,k.items=k.elem.selectAll(".bar"),k.idx=parseInt(k.elem.attr("data-group-id"),10),C=k.idx*k.data.length+g);var T=function(t){u=bi.tooltipSize(b);var e=bi.tooltip.outerHeight(),s=null;f.isValidElement(y)?(s=y.getBoundingClientRect(),i=s.left-u.width/2+s.width/2,y!==k.elem.node()?n=s.top-u.height-10:(n=w-e-25,t?(n+=e+50)>s.bottom+10&&(n=s.bottom+10):n=s.top-e+25)):(s=v[0].getBoundingClientRect(),i=s.left-u.width/2+v.attr("width")/2,a.settings.isStacked?n=s.top-u.height-10:(s=m.parentNode.getBoundingClientRect(),n=w-e-25,c.length>1&&(i=s.left-u.width/2+s.width/2,t?(n+=e+50)>s.bottom+10&&(n=s.bottom+10):n=s.top-e+25))),""!==b&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[r?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(i,n,b,t?"bottom":"top"))},A=function(){"string"==typeof b?b=function(t,e,i){if(null==i?i=/{{(\w+)}}/g:"string"==typeof i&&(i=new RegExp(i,"g")),"string"==typeof t&&"function"==typeof e&&i instanceof RegExp)for(var n=9999;i.test(t)&&n>0;)t=t.replace(i,(function(t,i){return e(t,i)})),n--;return t}(b,(function(t,i){var n=l(e[i]);return isNaN(n)?e[i]:n})):"number"==typeof b&&(b=b.toString())},S=function(t){b="";var i={elem:m,index:g,data:e};a.isGrouped&&(i.groupElem=k.elem.node(),i.groupItems=k.items.nodes(),i.groupIndex=k.idx,i.groupData=k.data);var n=function(t,e){"string"!=typeof t&&"number"!=typeof t||(b=t,y=e,A())},s=!0;V=setInterval((function(){s&&(s=!1,t(n,i)),""!==b&&(clearInterval(V),T())}),10)};if(a.settings.isStacked){if(I)b="<p><b>".concat(l(e[0].value),"</b> ").concat(e[0].name,"</p>");else{for(b="".concat('<div class="chart-swatch"><div class="swatch-caption"><b>').concat(s[0][g].name,"</b></div>"),o=s.length-1,d=0;o>=d;o--)h=bi.chartColor(o,"bar",c[o]),b+="".concat('<div class="swatch-row"><div style="background-color:').concat(c[o].pattern?"transparent":h,';">').concat(x(c[o].pattern,h),"</div>")+"<span>".concat(s[o][g].parentName,"</span><b>").concat(l(s[o][g].value),"</b>")+"</div>";b+="</div>"}u=bi.tooltipSize(b),i=v[0].getBoundingClientRect().left-u.width/2+v.attr("width")/2,n=v[0].getBoundingClientRect().top-u.height-10}else{if(D)b="".concat('<div class="chart-swatch"><div class="swatch-caption"><b>').concat(e.name,"</b></div>")+'<div class="swatch-row">'+'<div style="background-color:'.concat(F.target?"transparent":Et(z.target),';">').concat(x(F.target,Et(z.target)),"</div>")+"<span>".concat(N.target,"</span><b>").concat(l(e.target),"</b>")+'</div><div class="swatch-row">'+'<div style="background-color:'.concat(e.value<0?F.negative?"transparent":Et(z.negative):F.positive?"transparent":Et(z.positive),';">').concat(e.value<0?x(F.negative,Et(z.negative)):x(F.positive,Et(z.positive)),"</div>")+"<span>".concat(N[e.value<0?"negative":"positive"],"</span><b>").concat(l(e.value),"</b>")+"</div></div>";else if(1===c.length)b="<p><b>".concat(l(e.value),"</b> ").concat(e.name,"</p>");else{var E=d3.select(this.parentNode).datum().values;for(b='<div class="chart-swatch">',o=0,d=E.length;o<d;o++)h=bi.chartColor(o,"bar",B[o]),b+="".concat('<div class="swatch-row"><div style="background-color:').concat(B[o].pattern?"transparent":h,';">').concat(x(B[o].pattern,h),"</div>")+"<span>".concat(E[o].name,"</span><b>").concat(l(E[o].value),"</b>")+"</div>";b+="</div>",p=E.length>6}u=bi.tooltipSize(b),i=v[0].getBoundingClientRect().left-u.width/2+v.attr("width")/2,n=w-bi.tooltip.outerHeight()-25,c.length>1&&(i=this.parentNode.getBoundingClientRect().left-u.width/2+this.parentNode.getBoundingClientRect().width/2,n=this.parentNode.getBoundingClientRect().top-bi.tooltip.outerHeight()+25)}tt&&"function"==typeof tt&&void 0===e.tooltip&&!Z[C]?S(tt):(b=Z[C]||tt||b||"",Z[C]||!1===e.tooltip||void 0===e.tooltip||null===e.tooltip||("function"==typeof e.tooltip?S(e.tooltip):(b=e.tooltip.toString(),A(),Z[C]=b)),"string"==typeof b&&""!==b&&T(p))})).on("mouseleave.".concat(a.namespace),(function(){clearInterval(V),bi.hideTooltip()})).on("contextmenu.".concat(a.namespace),(function(t,e){bi.triggerContextMenu(a.element,d3.select(this).nodes()[0],e,t)})).on("click.".concat(a.namespace),(function(t,e,i,n){var s=this;a.settings.selectable&&(Mt=setTimeout((function(){Pt||(i=o.isSingle||o.isPositiveNegative?$(s).data("index"):$(s).index(),a.doClickAction(t,e,i,s,n)),Pt=!1}),200))})).on("dblclick.".concat(a.namespace),(function(t,e,i){clearTimeout(Mt),Pt=!0,i=$(this).index(),a.doDoubleClickAction(e,i,this)})),a.settings.isGrouped=L,a.settings.isSingle=I,a.settings.svg=this.svg,a.settings.showLegend){var Bt,Ot=c.map((function(t){return t.line}));if(I&&c[0].name)bi.addLegend(c,"column-single",a.settings,a.element);else if(D)bi.addLegend(q,a.settings.type,a.settings,a.element);else if(a.settings.isStacked&&I)bi.addLegend(B,a.settings.type,a.settings,a.element);else if(!I){var Rt=a.settings.isStacked?O:B;a.settings.useLine&&(Bt=Ot.filter((function(t){return t.name})),Rt.push(Bt[0])),Rt=Rt.map((function(t){var e;return!t.attributes||null!==(e=t.data)&&void 0!==e&&(null==e?void 0:e.attributes)||(t.data?t.data.attributes=t.attributes:t.data={attributes:t.attributes}),t})),bi.addLegend(Rt,a.settings.type,a.settings,a.element)}if(a.settings.useLine&&Ot.length){var zt=$(".chart-legend-item-text");Bt[0].name===zt.last().text()&&($(".chart-legend-color").last().addClass("dark"===ft.currentTheme.modeId?"slate03":"slate08"),a.element.find(".chart-legend-item").last().attr("style","pointer-events: none"))}}if(w&&a.settings.xAxis&&null!==(i=a.settings.xAxis)&&void 0!==i&&(null==i?void 0:i.rotate)){var Ft=S?"scale(-1, 1) rotate(".concat(y.use,"deg)"):"rotate(".concat(y.use,"deg)");a.svg.selectAll(".x.axis .tick text").attr("y",0).attr("x",(function(){return S?-10:-(this.getBBox().width+10)})).attr("dy","1em").attr("style","text-anchor: start; transform: ".concat(Ft," !important;"))}if(a.settings.xAxis&&a.settings.xAxis.formatText&&(a.svg.selectAll(".x.axis .tick text").each((function(t){var e=d3.select(this),i=d3.select(this).text(),n=a.settings.xAxis.formatText(i,t);f.html(e.node(),n,"<tspan>")})),a.element.find(".chart-legend").addClass("m-top-0"),a.element.find(".chart-legend .chart-legend-color").each((function(t,e){e.classList.add("m-bottom-0")}))),a.settings.isStacked&&a.element.find(".x.axis").length>0){var Nt=$(".tick"),Ht=$(".x.axis > g"),qt=$(".series-group.g"),Vt=$("g.x.axis > g text").find("tspan").length>1,Wt=Nt.attr("transform").replace("translate(","").replace(")","").split(","),jt=Wt?Wt[0]:0,Ut=Ht[0].getBBox().width/2,Gt=qt.find("rect")[0].getBBox().width/2,_t=Math.round(jt-(Vt?14:10)+Ut-Gt);qt.each((function(){var t=this.getAttribute("transform").replace(/translate\(\d+/,"translate(".concat(_t));this.setAttribute("transform",t)}))}return bi.isRTL&&bi.isIE&&(dt.selectAll("text").attr("transform","scale(-1, 1)"),dt.selectAll(".y.axis text").style("text-anchor","start"),D&&dt.selectAll(".negative-value").style("text-anchor","end")),dt.selectAll(".y.axis .tick").attr("class",(function(t){return"tick".concat(0===t?" tick0":"")})),bi.appendTooltip(),w||a.settings.xAxis||null!==(n=a.settings.xAxis)&&void 0!==n&&(null==n?void 0:n.rotate)||(bi.labelsColide(dt)&&bi.applyAltLabels(dt,Tt,"shortName"),bi.labelsColide(dt)&&bi.applyAltLabels(dt,Tt,"abbrName"),bi.labelsColide(dt)&&(bi.applyAltLabels(dt,Tt,null,null,!0),D&&dt.selectAll(".target-bartext, .bartext").attr("x",(function(){return+d3.select(this).attr("x")-(S?-6:6)})))),bi.setSelected=function(t,e){var i,n,s,o,r,l=0,d=E.equals,h=dt.node().parentNode.nextSibling,u=d3.select(h),p=u.node()&&u.classed("chart-legend"),g=function(e,o){var r=!!e;e=r?d3.select(e):dt,o=void 0!==o?o:0,e.selectAll(".bar").each((function(e,h){e&&l<1&&(void 0!==t.fieldName&&void 0!==t.fieldValue&&t.fieldValue===(I&&a.settings.isStacked?e[0][t.fieldName]:e[t.fieldName])||void 0!==t.index&&t.index===h||t.data&&d(t.data,c[o].data[h])||t.elem&&$(this).is(t.elem))&&(l++,n=d3.select(this),i=h,r&&!a.settings.isStacked&&(s=!0))}))};L||a.settings.isStacked&&!I&&!L?(c.forEach((function(e,r){l<1&&(o=$(dt.select('[data-group-id="'.concat(r,'"]')).node()),(void 0!==t.groupName&&void 0!==t.groupValue&&t.groupValue===e[t.groupName]||void 0!==t.groupIndex&&t.groupIndex===r||t.data&&d(t.data,e)||t.elem&&o.is(t.elem))&&void 0===t.fieldName&&void 0===t.fieldValue&&void 0===t.index&&(l++,n=dt.select('[data-group-id="'.concat(r,'"]')).select(".bar"),i=r,a.settings.isStacked&&!L&&(s=!0)))})),l<1&&(r=dt.selectAll(".series-group")).nodes().length&&r.each((function(t,e){g(this,e)}))):g(),l>0&&(e||!n.classed("is-selected"))&&(s?p&&$(u.selectAll(".chart-legend-item")[0][i]).trigger("click.chart"):d3.select(n.node()).dispatch("click"))},this.setInitialSelected(),this.element.trigger("rendered"),this},setInitialSelected:function(){var t,e,i,n,s=this,a=/positive-negative/.test(s.settings.type),o=s.svg.node().parentNode.nextSibling,r=d3.select(o),l=r.node()&&r.classed("chart-legend"),c=!1,d=0,h=function(i){(i=i?d3.select(i):s.svg).selectAll(".bar").each((function(i,n){i&&((s.isSingle&&s.settings.isStacked?i[0]:i).selected&&d<1&&(d++,e=d3.select(this),t=n))}))};if(a){var u=function(e){c=!0,t=e,d++};if(s.dataset[0].targetBarsSelected)u(0);else{var p={selected:0,i:-1};s.dataset[0].data.forEach((function(t,e){t.selected&&(p.selected++,p.i=e)})),p.selected>1&&(p.elem=s.svg.select(".target-bar.series-".concat(p.i)),u(p.elem.classed("positive")?1:2))}}c||!s.isGrouped&&(!s.settings.isStacked||s.isSingle||s.isGrouped)?h():(s.dataset.forEach((function(i,a){i.selected&&d<1&&(d++,e=s.svg.select('[data-group-id="'.concat(a,'"]')).select(".bar"),t=a,!s.settings.isStacked||s.isSingle||s.isGrouped||(n=!0))})),d<1&&function(){var e=s.svg.selectAll(".series-group");if(e.nodes().length){var n={groups:[],itemsInGroup:0};if(e.each((function(t,e){var s;h(this),((i=Array.isArray(t))||null!=t&&(null==t?void 0:t.data)&&Array.isArray(null==t||null==t?void 0:t.data))&&(n.itemsInGroup=(i?t:t.data).length,d>0&&n.groups.push({i:e,totalSel:(s=i?t:t.data,s.reduce((function(t,e){return e.selected?t+1:t}),0))}))})),n.gLen=n.groups.length,!c&&(i&&n.gLen&&n.groups[0].totalSel>1||n.gLen===e.size()&&!i)){var a=n.groups.filter((function(t){return(null==t||null==t?void 0:t.totalSel)>1})).length;c=i?!!a:!a,i&&(t=n.groups[0].i)}}}()),d>0&&(n||c?l&&(this.initialSelectCall=!0,$(r.node()).find(".chart-legend-item").eq(t).trigger("click.chart")):(this.initialSelectCall=!0,d3.select(e.node()).dispatch("click"))),this.initialSelectCall=!1},handleEvents:function(){var t=this;return this.element.on("updated.".concat(this.namespace),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated()})),this},getSelected:function(){return bi.selected},setSelected:function(t,e){var i={svg:this.svg,chartData:this.settings.dataset,isStacked:!1,isGrouped:!1,isSingle:!1};bi.setSelected(t,e,i)},toggleSelected:function(t){this.setSelected(t,!0)},handleResize:function(){this.width!==this.element.width()&&(this.width=this.element.width(),this.element.is(":visible")&&this.updated())},doClickAction:function(t,e,i,n,s){var a=this,o=n&&d3.select(n).classed("target-bar"),r=n&&d3.select(n).classed("is-selected"),l=parseInt(d3.select(n.parentNode).attr("data-group-id"),10);if(o)for(var c=a.svg.selectAll(".bar").nodes(),d=c.length,h=0;h<d;h++){var u=c[h];if(d3.select(u).classed("is-selected")){r=!1;break}}bi.setSelectedElement({task:r?"unselected":"selected",container:a.element,selector:n,isTrigger:!a.initialSelectCall&&!r,isTargetBar:o,triggerGroup:a.isGrouped,d:e,i:i,type:a.settings.type,dataset:a.dataset,isSingle:a.isSingle,isGrouped:a.isGrouped,isStacked:a.settings.isStacked,svg:a.svg,clickedLegend:!0===s}),r&&!a.initialSelectCall&&a.element.triggerHandler("selected",[d3.select(n).nodes(),{},a.isGrouped?l:i])},doDoubleClickAction:function(t,e,i){var n,s=this;if(this.settings.isGrouped){var a=d3.select(i.parentNode),o=parseInt(null==a||null==a?void 0:a.attr("data-group-id"),10),r=[];a.selectAll(".bar").each((function(t,e){r.push({elem:this,data:t,index:e})})),n=[{groupElem:a.node(),groupIndex:o,groupItems:r}]}else if(s.isSingle&&s.settings.isStacked){var l,c=s.dataset&&null!==(l=s.dataset[0])&&void 0!==l&&(null==l?void 0:l.data)&&s.dataset[0].data[e]?s.dataset[0].data[e]:t;n=[{data:c,index:e,elem:[i]}]}else s.settings.isStacked?(n=[],s.svg.selectAll(".series-group").each((function(){var t=d3.select(this).selectAll(".bar").nodes()[e];n.push({elem:t,data:d3.select(t).datum(),index:e})}))):n=[{data:t,index:e,elem:i}];s.element.triggerHandler("dblclick",[n])},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset&&(this.settings.dataset=t.dataset),this.element.empty(),this.build()},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".target-bartext, .bartext").off("end.".concat(self.namespace)),this.element.find(".bar, .target-bar").off(e(["mouseenter","mouseleave","click","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),delete this.namespace,this},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("column-chart"),$.removeData(this.element[0],Bi),$.removeData(this.element[0],"chart"))}};var zi="bar",Fi={dataset:[],isStacked:!0,isNormalized:!1,isGrouped:!1,showLegend:!0,animate:!0,longText:!1,format:null,redrawOnResize:!0,tooltip:null,useLogScale:!1,ticks:null,selectable:!0,defaultTickCount:5,showLines:!0,labelFactor:1.27,wrapWidth:60,fitHeight:!0,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function Ni(t,e){this.settings=E.mergeSettings(t,e,Fi),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),bi.destroy(t),this.init()}Ni.prototype={init:function(){return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.width=0,this.initialSelectCall=!1,this.tickCountFlag=!1,this.settings.localeInfo&&d3.formatDefaultLocale(this.settings.localeInfo),this.build().handleEvents(),"initial"===this.settings.animate&&(this.settings.animate=!1),this.element.trigger("rendered",[this.svg]),this},build:function(){var t,e,i,n,s,a,o,r,l,c,d,h,u,p=this,g=this,m=this.settings,v=this.element.parent(),b=this.element.closest(".is-personalizable").length>0,y=!!m.formatterString,w=function(t){return d3.format(g.settings.formatterString||",")(t)},x="bar-stacked"===m.type,C="bar-normalized"===m.type,k=[],T=[],A=m.tooltip,S=0,D=30,I=v.width()<450,P=m.dataset;this.isRTL=H.isRTL(),this.dataset=P,this.allZeroValues(this.dataset)&&this.dataset.forEach((function(t){t.data.forEach((function(t){t.value=p.settings.defaultTickCount,p.tickCountFlag=!0}))}));var M=window.innerWidth;this.viewport={xxsmall:M<=380,xsmall:M>=381&&M<=450,small:M>=451&&M<=480,medium:M>=481&&M<=992,large:M>992};var B,O,R={top:20,left:30,right:30,bottom:1===P.length?5:30};if(m.isGrouped&&1===P.length&&(R.bottom=30,S=2),this.element.addClass("bar-chart"),m.isGrouped&&this.element.addClass("bar-chart-grouped"),m.isStacked&&this.element.addClass("bar-chart-stacked"),0===P.length)return g.element.emptymessage(m.emptyMessage),this;var z={},F={atLeastOne:!1},N=["left","top","right","bottom"];if(m.axisLabels&&$.extend(!0,z,m.axisLabels),!$.isEmptyObject(z))for(B=0,O=N.length;B<O;B++){var q=z[N[B]];q&&"string"==typeof q&&""!==$.trim(q)&&(F[N[B]]=!0,F.atLeastOne=!0)}var V=P.map((function(t){return{name:t.name,color:t.color,pattern:t.pattern,attributes:t.attributes}}));this.ellipsis={use:!1,percentageWidth:25,threshold:12},this.viewport.xxsmall?(this.ellipsis.percentageWidth=37,this.ellipsis.threshold=10):this.viewport.xsmall?(this.ellipsis.percentageWidth=28,this.ellipsis.threshold=10):this.viewport.small&&(this.ellipsis.percentageWidth=35);var W={y1:[],y2:[]};P=P.map((function(t,e){return t.data.map((function(t,i){var n=0;return 0===e?(W.y1.push(t.value),W.y2.push(0)):(n=W.y1[i]+W.y2[i],W.y1[i]=t.value,W.y2[i]=n),$.extend({},t,{y0:n,y:t.value,x:bi.getLabel(t,g.viewport),color:t.color,pattern:t.pattern})}))})),n=0,s="",P=P.map((function(t,e){if(!m.isStacked&&V[e]){var i=bi.getLabel(V[e],g.viewport),a=i.length;s=a>n?i:s,n=a>n?a:n}return t.map((function(t){if(m.isStacked){var e=t.x.length;s=e>n?t.x:s,n=e>n?e:n}return $.extend({},t,{x:t.y,y:t.x,x0:t.y0,color:t.color,pattern:t.pattern})}))}));var j=E.getParentAvailableHeight(g.element[0]),U=m.fitHeight?j:parseInt(v.height(),10),G=U-R.top-R.bottom-30,_=1===(null===(t=P)||void 0===t||null==t?void 0:t.length),Y=parseInt(v.width(),10)-R.left,K=U-R.bottom-(m.isStacked?0:15)-(_?30:0),X=bi.calculateTextRenderWidth(s);X<200&&(X+=26),bi.calculatePercentage(X,Y)>55&&(X=bi.getPercentage(Y,this.ellipsis.percentageWidth),this.ellipsis.use=!0),null!=m&&null!==(e=null==m?void 0:m.axisLabels)&&void 0!==e&&(null==e?void 0:e.bottom)&&""===V[0].name&&(K+=30),m.isStacked&&(K-=x||C?10:0);var Q=m.isStacked?V:k,J=m.showLegend&&Q.length>0&&Q[0].attributes&&(null===(i=Q[0].data)||void 0===i||!(null==i?void 0:i.attributes))?0:15;g.svg=d3.select(this.element[0]).append("svg").attr("width",Y+(F.atLeastOne?20:0)).attr("height",K+(F.atLeastOne?5:0)).append("g").attr("class","group").attr("aria-label","".concat(m.dataset[0].name?m.dataset[0].name:"Name Group")).attr("transform","translate(".concat(X,",").concat(R.top-(F.atLeastOne?3:J),")")),"bar"===g.settings.type&&(g.svg.append("title"),g.svg.append("title").text("Bar Chart"));var Z=d3.min(P,(function(t){return d3.min(t,(function(t){return m.isStacked?t.x+t.x0:t.x}))})),tt=d3.max(P,(function(t){return d3.max(t,(function(t){return m.isStacked?t.x+t.x0:t.x}))}));if(m.isStacked&&m.isNormalized){var et=[];P.forEach((function(t){t.forEach((function(t,e){et[e]=(void 0===et[e]?0:et[e])+t.x}))})),P.forEach((function(t){t.forEach((function(t,e){var i=et[e]/100;t.x/=i,t.x0/=i}))})),tt=100}var it,nt=Y-X-R.left;it=m.useLogScale?d3.scaleLog().domain([Z>0?Z:1,tt]).range([1,nt]).nice():d3.scaleLinear().domain([Z<0?Z:0,tt]).range([0,nt]).nice(),m.isStacked?(a=P[0].map((function(t){return t.y})),h=.32):(a=V.map((function(t){return t.name})),function(){for(var t,e=P.length-1;e>-1;e--)t=P[e].map((function(t){return t})),$.extend(!0,k,t),k=$.map(k,(function(t){return t}))}(),o=0,r=k.length,h=(l=G/a.length)/r,d=l-(c=r>1?l-.6*h:D),D=c/r,h=0);var st=d3.scaleBand().domain(a).rangeRound([0,G]).padding(h,h),at=d3.axisBottom().scale(it).tickSize(-G);I&&at.ticks(X<100?5:3),m.isStacked&&m.isNormalized&&at.tickFormat((function(t){return"".concat(t,"%")})),m.useLogScale&&(at.ticks(10,",.1s"),!1===m.showLines&&at.tickSize(0)),m.ticks&&!m.useLogScale&&(g.viewport.small?at.ticks(m.ticks.smallNumber,m.ticks.format):g.viewport.medium?at.ticks(m.ticks.mediumNumber,m.ticks.format):g.viewport.large&&at.ticks(m.ticks.largeNumber,m.ticks.format));var ot=d3.axisLeft().scale(st).tickPadding(15).tickSize(0);g.svg.append("g").attr("class","axis x").attr("transform","translate(0,".concat(G,")")).call(at),g.svg.append("g").attr("class","axis y").call(ot);var rt=g.svg.selectAll("g.group").data(P).enter().append("g").attr("class","series-group").attr("role","list").attr("title","".concat(m.type)).attr("tabindex",0).attr("data-group-id",(function(t,e){return e}));m.isGrouped=g.svg.selectAll(".series-group").nodes().length>1&&!m.isStacked||m.isGrouped&&1===P.length,m.isSingle=1===g.svg.selectAll(".series-group").nodes().length&&m.isStacked;var lt=!1,ct=0,dt=rt.selectAll("rect").data((function(t,e){return t.forEach((function(t){t.index=e,m.isStacked||(t.gindex=o++)})),t})).enter();if(this.settings.isSingle&&(dt=dt.append("g").attr("role","listitem")),dt.append("rect").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,"bar")}))}))})).attr("class",(function(t,e){return"bar series-".concat(e)})).attr("aria-hidden",!0).style("fill",(function(t,e){return m.isStacked?1===V.length?bi.chartColor(e,"bar-single",t):bi.chartColor(t.index,"bar",V[t.index]):bi.chartColor(e,"bar",k[e])})).attr("mask",(function(t,e){return 1===P.length&&P[0][e].pattern?"url(#".concat(P[0][e].pattern,")"):m.isStacked&&V[t.index].pattern?"url(#".concat(V[t.index].pattern,")"):!m.isStacked&&k[e]&&k[e].pattern?"url(#".concat(k[e].pattern,")"):""})).attr("x",(function(t){return m.useLogScale?0:m.isStacked&&!m.isSingle?it(t.x0)+1:it(0)+1})).attr("y",(function(t){return m.isStacked?st(t.y):(l-c)/2+(t.gindex*D+t.gindex*S)+t.index*d})).attr("height",(function(){return m.isStacked?st.bandwidth():D})).attr("width",0).on("mouseenter.".concat(g.namespace),(function(t,e){var i,n,s,a=this,o=dt.select("rect").nodes().indexOf(this),r=0,l=[],c="",d=null,h=d3.select(this),p=this.parentNode,g=d3.select(p).datum(),v=Math.round(g.length/2),x=function(t,e){return t&&e?'<svg width="12" height="12"><rect mask="url(#'.concat(t,')" height="12" width="12" /></svg>'):""},C=null;m.isGrouped&&((C={elem:d3.select(p)}).data=g,C.items=C.elem.selectAll(".bar"),C.idx=parseInt(p.getAttribute("data-group-id"),10));var S=function(){var t=!m.isStacked&&g.length>6,e=$(".series-".concat(v),p)[0],i=(f.isValidElement(d)?d===C.elem.node()?e:d:m.isGrouped?e:h.node()).getBoundingClientRect(),n=$(window),s=i.top+n.scrollTop(),a=i.left+n.scrollLeft(),o=bi.tooltipSize(c),r=a+i.width/2-o.width/2,l=t?s:s-o.height-13;""!==c&&(bi.tooltip&&bi.tooltip.length&&bi.tooltip[b?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(r,l,c,t?"bottom":"top"))},E=function(){"string"==typeof c?c=function(t,e,i){if(null==i?i=/{{(\w+)}}/g:"string"==typeof i&&(i=new RegExp(i,"g")),"string"==typeof t&&"function"==typeof e&&i instanceof RegExp)for(var n=9999;i.test(t)&&n>0;)t=t.replace(i,(function(t,i){return e(t,i)})),n--;return t}(c,(function(t,i){var n=w(e[i]);return isNaN(n)?e[i]:n})):"number"==typeof c&&(c=c.toString())},D=function(t){c="";var i={elem:a,index:o,data:e};m.isGrouped&&(i.groupElem=C.elem.node(),i.groupItems=C.items.nodes(),i.groupIndex=C.idx,i.groupData=C.data);var n=function(t,e){"string"!=typeof t&&"number"!=typeof t||(c=t,d=e,E())},s=!0;u=setInterval((function(){s&&(s=!1,t(n,i)),""!==c&&(clearInterval(u),S())}),10)};if(1===P.length)c="<p><b>".concat(e.y," </b>").concat(e.x,"</p>");else{if(c='<div class="chart-swatch">',m.isStacked){var I=Array.from(this.parentElement.children).indexOf(this);for(i=0,n=P.length;i<n;i++){r=0,s=bi.chartColor(i,"bar",V[i]);for(var L=0,M=P.length;L<M;L++)m.isNormalized?(r+=P[L][I].value,l[L]=P[L][I].value):(r+=P[L][I].x,l[L]=P[L][I].x);c+='<div class="swatch-row">\n                  <div class="swatch-color">'.concat(x(V[i].pattern,s),"</div>\n                  <span>").concat(V[i].name,"</span>\n                  <b> ").concat(y?w(l[i]):"".concat(Math.round(l[i]/r*100),"%")," </b>\n                </div>")}}else for(i=0,n=g.length;i<n;i++)s=bi.chartColor(i,"bar",k[i]),c+='<div class="swatch-row">\n                    <div class="swatch-color">'.concat(x(k[i].pattern,s),"</div>\n                  <span>").concat(g[i].name,"</span><b>").concat(w(g[i].value),"</b>\n                </div>");c+="</div>"}if(r>0&&(c='<span class="chart-tooltip-value"><b>'.concat(m.isStacked?l[e.index]:e.value,'</b> Value</span><span class="chart-tooltip-total"><b>').concat(Math.round(r),"</b> ").concat(H.translate("Total"),"</span>").concat(c)),A&&"function"==typeof A&&!T[o])D(A);else if(c=T[o]||A||c||"",T[o]||!1===e.tooltip||void 0===e.tooltip||null===e.tooltip||("function"==typeof e.tooltip?D(e.tooltip):(c=e.tooltip.toString(),E(),T[o]=c)),"string"==typeof c&&""!==c&&S(),m.isStacked)for(i=0,n=P.length;i<n;i++){s=bi.chartColor(i,"bar",V[i]);var B=$("#svg-tooltip").find(".swatch-row").eq(i);V[i].pattern||B.find("div").css("background-color",s),B.find("rect").css("fill",s)}else for(i=0,n=g.length;i<n;i++){s=bi.chartColor(i,"bar",k[i]);var O=$("#svg-tooltip").find(".swatch-row").eq(i);k[i].pattern||O.find("div").css("background-color",s),O.find("rect").css("fill",s)}})).on("mouseleave.".concat(g.namespace),(function(){clearInterval(u),bi.hideTooltip()})).on("contextmenu.".concat(g.namespace),(function(t,e){bi.triggerContextMenu(g.element,d3.select(this).nodes()[0],e,t)})).on("click.".concat(g.namespace),(function(t,e){var i=dt.select("rect").nodes().indexOf(this),n=this;ct=setTimeout((function(){if(!lt&&g.settings.selectable){var t=Array.from(n.parentElement.children).indexOf(n);g.doClickAction(e,m.isStacked&&!m.isSingle||m.isGrouped?t:i,n)}lt=!1}),200)})).on("dblclick.".concat(g.namespace),(function(t,e){clearTimeout(ct),lt=!0;var i=dt.select("rect").nodes().indexOf(this),n=Array.from(this.parentElement.children).indexOf(this);g.doDoubleClickAction(e,m.isStacked&&!m.isSingle?n:i,this)})),$(dt._parents).each((function(t,e){var i=$(e),n=t;i.attr("aria-label","".concat(m.dataset[t].name?m.dataset[t].name:"Name Label")),i.children(".bar").each((function(t,e){var i=m.dataset[n].name;$(e).attr("aria-label","".concat(m.dataset[n].data[t].value?"".concat(i," ").concat(m.dataset[n].data[t].value):"Name Label"))}))})),g.settings.isSingle&&g.svg.selectAll(".series-group g").append("text").attr("class","audible").text((function(t,e){var i,n;return"".concat(null===(i=m.dataset[0].data[e])||void 0===i||null==i?void 0:i.name," ").concat(null===(n=m.dataset[0].data[e])||void 0===n||null==n?void 0:n.value)})).attr("x",(function(t){return m.useLogScale?0:m.isStacked&&!m.isSingle?it(t.x0)+1:it(0)+1})).attr("y",(function(t){return m.isStacked?st(t.y):(l-c)/2+(t.gindex*D+t.gindex*S)+t.index*d})).lower(),g.element.off("dblclick.".concat(g.namespace)).on("dblclick.".concat(g.namespace),"*",(function(t){t.stopImmediatePropagation()})),g.svg.selectAll(".axis.y text").attr("aria-hidden",!0).attr("x",(function(){return g.isRTL?F.atLeastOne?5:15:F.atLeastOne?-5:-15})),g.svg.selectAll(".axis.x text").attr("aria-hidden",!0).attr("class",(function(t){return t<0?"negative-value":"positive-value"})),g.isRTL&&(bi.isIE||bi.isIEEdge)&&(g.svg.selectAll("text").attr("transform","scale(-1, 1)"),g.svg.selectAll(".y.axis text").style("text-anchor","last")),F.atLeastOne){var ht=g.svg.append("g").attr("class","axis-labels"),ut=Y-95,pt=G-60,gt=$('html[class*="classic-"]').length>0,ft=function(t){if(F[t]){var e;g.svg.selectAll(".axis.y text")._groups[0].forEach((function(t){t.innerHTML.length>=5&&(e=!0)}));var i={top:"translate(".concat(ut/2,",").concat(-4,")"),right:"translate(".concat(e?ut+35:ut+50,",").concat(G/2,")rotate(90)"),bottom:"translate(".concat(ut/2,",").concat(pt+89,")"),left:"translate(".concat(e?-50:-35,",").concat(G/2,")rotate(-90)")},n={top:"rotate(0deg) scaleX(-1) translate(-".concat(ut/2,"px, ").concat(-4,"px)"),right:"rotate(90deg) scaleX(-1) translate(-".concat(G/2+5,"px, -").concat(ut+(p.isRTL&&!e?45:28),"px)"),bottom:"rotate(0deg) scaleX(-1) translate(-".concat(ut/2,"px, ").concat(pt+89,"px)"),left:"rotate(90deg) scaleX(-1) translate(-".concat(G/2-5,"px, ").concat(p.isRTL&&!e?45:!gt&&p.isRTL?60:55,"px)")};ht.append("text").attr("class","axis-label-".concat(t)).attr("text-anchor","middle").attr("transform",g.isRTL?"":i[t]).style("font-size","1.2em").style("transform",g.isRTL?n[t]:"").text(z[t])}};for(B=0,O=N.length;B<O;B++)ft(N[B])}if(I&&m.useLogScale){var mt=d3.selectAll(".x .tick text"),vt=!1;mt.attr("class",(function(t,e){var i=Math.round(mt.size()/2);e>=i&&!vt&&t.toString().startsWith("1")&&(vt=!0,i=t),0!==e&&e!==mt.size()-1&&(m.useLogScale?t!==i:e!==i)&&d3.select(this).remove()}))}if(g.svg.selectAll(".x.axis .tick").attr("class",(function(t){return"tick".concat(0===t?" tick0":"")})),this.setBarWidthToZero(),g.svg.selectAll(".bar").transition().duration(m.animate?600:0).attr("width",(function(t){var e=it(t.x),i=it(0);return!isNaN(e)&&L.isFinite(e)||(e=0),!isNaN(i)&&L.isFinite(i)||(i=0),Math.abs(e-i)})).attr("x",(function(t){return m.useLogScale?0:m.isStacked&&!m.isSingle?it(t.x0)+1:t.x<0?it(t.x)+1:it(0)+1})),m.svg=g.svg,m.showLegend){var bt=m.isStacked?V:k;bt=bt.map((function(t){var e;return!t.attributes||null!==(e=t.data)&&void 0!==e&&(null==e?void 0:e.attributes)||(t.data?t.data.attributes=t.attributes:t.data={attributes:t.attributes}),t})),bi.addLegend(bt,m.type,m,this.element)}return bi.appendTooltip(),this.setInitialSelected(),this.setTextValues(),this.element.trigger("rendered"),this},setTextValues:function(){var t,e=this,i=null===(t=this.settings)||void 0===t||null==t?void 0:t.dataset;if(i&&(!i||i.constructor===Array)){var n,s,a,o,r,l,c=this.element[0].querySelectorAll(".bar-chart .axis.y .tick text"),d={};if(this.ellipsis.use)d.maxWidth=this.element.width(),d.transX=(null===(n=this.svg)||void 0===n||null===(s=null==n?void 0:n.node())||void 0===s||null===(a=null==s?void 0:s.transform)||void 0===a||null===(o=null==a?void 0:a.baseVal)||void 0===o||null===(r=null==o?void 0:o.consolidate())||void 0===r||null===(l=null==r?void 0:r.matrix)||void 0===l||null==l?void 0:l.e)||1,d.customCss=function(){return{tooltip:{maxWidth:"".concat(d.maxWidth,"px")},arrow:{left:e.isRTL?"calc(100% - 20px)":"20px"}}};var h=function(t,i){var n;if(e.ellipsis.use){if("..."===(n=bi.trimText(t.name,e.ellipsis.threshold)).substr(-3)){var s=bi.calculateTextRenderWidth(t.name),a=bi.calculateTextRenderWidth(n),o=c[i].parentNode;if(!x.browser.isIE11()){n=t.name,c[i].classList.add("hidden"),d3.select(o).append("foreignObject").attr("overflow","visible").attr("width","".concat(d.transX)).attr("height","16").attr("class","foreign-object tick-y".concat(i)).attr("x","-".concat(d.transX)).attr("y","-1em").attr("requiredFeatures","http://www.w3.org/TR/SVG11/feature#Extensibility").html('<div class="text ellipsis" resizeable="false" xmlns="http://www.w3.org/1999/xhtml"></div>');var r=o.querySelector(".text.ellipsis");if(r){var l=document.createTextNode(t.name);r.appendChild(l)}}d3.select(o).on("mouseover.".concat(e.namespace),(function(){var i,n,r,l,c=(i=o.getBoundingClientRect(),n=Math.ceil(s/d.maxWidth),r=d.transX-a,l=i.top,r=(r<0?30:r)+20,l=(l<0?30:l)-36,n>1&&(l-=.5*(n-1)*33),e.isRTL&&(r=i.left-s+d.transX),{x:r,y:l});bi.showTooltip(c.x,c.y,t.name,"top",d.customCss())})).on("mouseout.".concat(e.namespace),(function(){return bi.hideTooltip()}))}}else n=bi.getLabel(t,e.viewport);return n};this.settings.selectable||this.element.find(".bar").css("cursor","inherit"),this.settings.isGrouped?i.forEach((function(t,e){c[e].textContent=h(t,e)})):i.forEach((function(t){var e=Object.keys(t).map((function(t){return t}));Object.keys(t).map((function(e){return t[e]})).forEach((function(t,i){t&&t.constructor===Array&&t.forEach((function(t,n){"attributes"!==e[i]&&(c[n].textContent=h(t,n))}))}))}))}},setInitialSelected:function(){var t,e,i,n=this,s=n.svg.node().parentNode.nextSibling,a=d3.select(s),o=a.node()&&a.classed("chart-legend"),r=!1,l=0,c=function(i){(i=i?d3.select(i):n.svg).selectAll(".bar").each((function(i,n){i&&i.selected&&l<1&&(l++,e=d3.select(this),t=n)}))};n.settings.isGrouped||n.settings.isStacked&&!n.settings.isSingle?(n.settings.dataset.forEach((function(s,a){s.selected&&l<1&&(l++,e=n.svg.select('[data-group-id="'.concat(a,'"]')).select(".bar"),t=a,n.settings.isStacked&&!n.settings.isGrouped&&(i=!0))})),l<1&&function(){var e=n.svg.selectAll(".series-group");if(e.nodes().length){var i,s={groups:[],itemsInGroup:0};if(e.each((function(t,e){var i;c(this),Array.isArray(t)&&(s.itemsInGroup=t.length,l>0&&s.groups.push({i:e,d:t,totalSel:(i=t,i.reduce((function(t,e){return e.selected?t+1:t}),0))}))})),s.gLen=s.groups.length,!r&&n.settings.isGrouped&&s.gLen===e.size()&&1===(null===(i=s.groups[0])||void 0===i||null==i?void 0:i.totalSel)){r=!0;for(var a=0,o=s.groups[0].d;a<o;a++)if(s.groups[0].d[a].selected){t=a;break}}}}()):c(),l>0&&(i||r?o&&(this.initialSelectCall=!0,$(a.node()).find(".chart-legend-item").eq(t).trigger("click.chart")):(this.initialSelectCall=!0,e.dispatch("click"))),this.initialSelectCall=!1},setBarWidthToZero:function(){this.tickCountFlag&&($(".bar").css("width","0"),$(".bar").attr("width","0"))},allZeroValues:function(t){return t.every((function(t){return t.data.every((function(t){return 0===t.value}))}))},handleEvents:function(){var t=this;return this.element.on("updated.".concat(this.namespace),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize(),t.setBarWidthToZero()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize(),t.setBarWidthToZero()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated(),t.setBarWidthToZero()})),this},getSelected:function(){return bi.selected},setSelected:function(t,e){var i={svg:this.svg,chartData:this.settings.dataset,isStacked:this.settingsisStacked,isGrouped:this.settings.isGrouped,isSingle:this.settings.isSingle};bi.setSelected(t,e,i)},toggleSelected:function(t){this.setSelected(t,!0)},doClickAction:function(t,e,i){var n=this,s=this.settings,a=i&&d3.select(i).classed("is-selected"),o=parseInt(d3.select(i.parentNode).attr("data-group-id"),10);bi.setSelectedElement({task:a?"unselected":"selected",container:n.element,selector:i,isTrigger:!n.initialSelectCall&&!a,triggerGroup:s.isGrouped,d:t,i:e,type:s.type,dataset:s.dataset,isSingle:s.isSingle,isGrouped:s.isGrouped,isStacked:s.isStacked,svg:n.svg,clickedLegend:s.clickedLegend}),a&&!n.initialSelectCall&&n.element.triggerHandler("selected",[d3.select(i).nodes(),{},s.isGrouped?o:e])},doDoubleClickAction:function(t,e,i){var n,s=this.settings;if(s.isGrouped){var a=d3.select(i.parentNode),o=parseInt(null==a||null==a?void 0:a.attr("data-group-id"),10),r=[];a.selectAll(".bar").each((function(t,e){r.push({elem:this,data:t,index:e})})),n=[{groupElem:a.node(),groupIndex:o,groupItems:r}]}else"bar-stacked"===s.type?(n=[],this.svg.selectAll(".series-group").each((function(){var t=d3.select(this).selectAll(".bar").nodes()[e];n.push({elem:t,data:d3.select(t).datum(),index:e})}))):n=[{data:t,index:e,elem:i}];this.element.triggerHandler("dblclick",[n])},handleResize:function(){var t=this,e=function(){t.width!==t.element.width()&&(t.width=t.element.width(),t.element.is(":visible")&&t.updated())};this.element.closest(".homepage").length?setTimeout((function(){return e()}),300):e()},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset&&(this.settings.dataset=t.dataset),this.element.empty(),this.build().element.trigger("rendered",[this.svg])},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".group .series-group .bar").off(e(["mouseenter","mouseleave","click","dblclick","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),delete this.namespace,this},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("bar-chart"),$.removeData(this.element[0],zi),$.removeData(this.element[0],"chart"))}};var Hi="pie",qi={dataset:[],isDonut:!1,redrawOnResize:!0,hideCenterLabel:!1,showLines:!0,showLinesMobile:!1,selectable:!0,lines:{show:"value",formatter:".0f"},showLegend:!0,legendPlacement:"right",forceLegendPopup:!1,legend:{show:"label (percent)",formatter:".0f"},showTooltips:!0,tooltip:{show:"label (value)",formatter:".0f"},showCenterTooltip:!1,fitHeight:!0,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function Vi(t,e){this.settings=E.mergeSettings(t,e,qi),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),bi.destroy(t),this.init()}Vi.prototype={init:function(){return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.width=0,this.isFirefox="firefox"===x.browser.name,0===this.settings.dataset.length?(this.element.emptymessage(this.settings.emptyMessage),this):(this.initialSelectCall=!1,this.build().handleEvents(),this.element.trigger("rendered",[this.svg]),this)},build:function(){var t=this,e=this.settings;t.svg=d3.select(this.element[0]).append("svg"),t.mainGroup=t.svg.append("g"),t.mainGroup.append("g").attr("class","slices"),t.mainGroup.append("g").attr("class","labels"),t.mainGroup.append("g").attr("class","lines"),this.element.addClass("chart-pie"),(e.showMobile||e.forceLegendPopup)&&(e.legendPlacement="bottom"),e.legendPlacement&&this.element.addClass("has-".concat(e.legendPlacement,"-legend"));var i=parseInt(this.element.width(),10),n=parseInt(this.element.height(),10),s={height:n,width:i};s.isSmallView=s.width<360;var a="bottom"===e.legendPlacement&&s.isSmallView&&!e.showMobile;a&&(this.element.closest(".widget-content").css({height:"","min-height":""}),n=parseInt(this.element.height(),10),s.height=n),!a&&e.fitHeight&&s.isSmallView&&(n=E.getParentAvailableHeight(t.element[0]),s.height=n),("label"===e.lines.show&&"bottom"===e.legendPlacement||"label"===e.lines.show&&"false"===e.showLegend)&&t.mainGroup.attr("transform","translate(".concat(.67777*s.width,", ").concat(s.height/2,")")),"right"===e.legendPlacement&&(s.width=.75*i),e.showMobile&&(s.height=.8*n,this.element.addClass("is-mobile"));var o=0;if(a){o=35,this.element.closest(".widget-content").css({height:"auto","min-height":"auto"});var r=s.height-(23+25*Math.round(e.dataset[0].data.length/2)),l=this.element.closest(".auto-height ");l.length>0&&!l.hasClass("has-bottom-legend")&&(r+=78),e.fitHeight&&(r-=E.getSiblingsHeight(t.element[0])),$(this.svg.node()).css({"min-height":r}),s.height=parseInt(this.element.height(),10)+o}s.radius=Math.min(s.width,s.height)/2,t.dims=s,t.pie=d3.pie().sort(null).value((function(t){return t.value})),t.arc=d3.arc().outerRadius(.75*s.radius).innerRadius(e.isDonut?.5*s.radius:0),t.outerArc=d3.arc().innerRadius(.75*s.radius).outerRadius(.75*s.radius+20),t.svg.attr("width","right"===e.legendPlacement?"75%":"100%").attr("height",e.showMobile?"80%":"100%"),t.mainGroup.attr("transform","translate(".concat(s.width/2,",").concat((s.height-o)/2,")")),s.isSmallView?(a||(t.svg.attr("height","".concat(s.height,"px")),this.element.css({height:"auto"})),e.fitHeight||(this.element.closest(".widget-content").css({height:""}),this.element.css({height:"auto"}))):"auto"===this.element[0].style.height&&this.element.css({height:""}),s.center={x:s.width/2,y:s.height/2},t.key=function(t){return t.data.name};var c=!e.dataset||0===e.dataset.length;this.chartData=c?[]:e.dataset[0].data,this.sum=d3.sum(this.chartData,(function(t){return t.value}));var d=this.chartData.map((function(e){return e.value/t.sum*100})),h=this.roundLargestRemainer(d);this.chartData=this.chartData.map((function(e,i){return e.percent=e.value/t.sum,e.percentRound=h[i],e}));var u=0;if(this.chartData.map((function(t){u+=t.percentRound})),c||0===u||isNaN(u)){var p=ft.themeColors().palette;this.chartData.push({data:{},color:p.slate[30].value,name:"Empty-Pie",value:100,percent:1,percentRound:100})}if(t.updateData(t.chartData),(e.showTooltips||e.showCenterTooltip)&&bi.appendTooltip("is-pie"),e.showLegend){var g=t.chartData.map((function(t){var i=bi.formatToSettings(t,e.legend);return e.legendFormatter&&(i="".concat(t.name," (").concat(d3.format(e.legendFormatter)(t.value),")")),"Empty-Pie"===t.name&&(i=""),{name:i,display:"twocolumn",placement:e.legendPlacement,color:t.color,data:t}}));e.svg=t.svg,bi.addLegend(g,"pie",e,this.element);var f=o>0?"addClass":"removeClass";this.element.find(".chart-legend")[f]("adjusted-height")}return this.setInitialSelected(),this.addCenterLabel(),this.element.trigger("rendered"),this},randomize:function(t){var e=this;this.chartData=this.chartData.map((function(e){return e.value=t?0:Math.random(),e})),this.sum=d3.sum(this.chartData,(function(t){return t.value}));var i=this.chartData.map((function(t){return t.value/e.sum*100})),n=this.roundLargestRemainer(i);return this.chartData=this.chartData.map((function(t,i){return t.percent=t.value/e.sum,t.percentRound=n[i],t})),this.chartData},addCenterLabel:function(){var t=this;if(t.settings.isDonut&&!t.settings.hideCenterLabel){var e=t.settings.dataset[0].centerLabel,i=t.svg.selectAll(".slices");if(i.append("text").attr("dy",".35em").style("text-anchor","middle").attr("class","chart-donut-text").html(e),bi.isIE&&!bi.isIEEdge)if(E.isHTML(e)){var n=i.select(".chart-donut-text"),s=document.createElement("text");s.innerHTML=e,Array.prototype.slice.call(s.childNodes).forEach((function(t){n.append("tspan").attr("style",t.getAttribute&&t.getAttribute("style")).attr("x",t.getAttribute&&t.getAttribute("x")).attr("dy",t.getAttribute&&t.getAttribute("dy")).text(t.textContent)}))}else i.select(".chart-donut-text").text(e)}},updateData:function(t){var e,i,n,s=this,a=this.element.closest(".is-personalizable").length>0,o=!s.settings.dataset||0===s.settings.dataset.length,r=s.svg.select(".slices").selectAll("path.slice").data(s.pie(t),s.key),l={str:null,isTooltip:(null===(e=s.settings.tooltip)||void 0===e||null==e?void 0:e.formatter)&&s.settings.tooltip.formatter!==qi.tooltip.formatter,isLines:(null===(i=s.settings.lines)||void 0===i||null==i?void 0:i.formatter)&&s.settings.lines.formatter!==qi.lines.formatter};l.isTooltip?l.str=s.settings.tooltip.formatter:l.isLines&&(l.str=s.settings.lines.formatter);var c=function(t){var e,i=document.body;if(s.isFirefox&&0===s.settings.dotSize){var n=d3.select(t);n.attr("r",2),e=t.getBoundingClientRect(),n.attr("r",s.settings.dotSize)}else e=t.getBoundingClientRect();return{top:e.top+i.scrollTop,left:e.left+i.scrollLeft}};s.isRTL=H.isRTL();var d=!1,h=0;if(setTimeout((function(){var t;null!==(t=s.settings)&&void 0!==t&&(null==t?void 0:t.showCenterTooltip)&&(s.element.find(".chart-donut-text").off("mouseenter.text").on("mouseenter.text",(function(){var t,e=s.element.find(".slices").get(0),i=c(e),n=null===(t=s.settings.dataset[0])||void 0===t||null==t?void 0:t.centerTooltip,a=i.left,o=i.top+15;$(".is-pie").addClass("is-center"),bi.showTooltip(a,o,n,"top")})),s.element.find(".chart-donut-text").off("mouseleave.text").on("mouseleave.text",(function(){bi.hideTooltip(),$(".is-pie").removeClass("is-center")}))),s.settings.attributes&&(E.addAttributes(s.element.find(".chart-donut-text"),s,s.settings.attributes,"text",!0),E.addAttributes(s.element.find(".slices"),s,s.settings.attributes,"slices",!0),E.addAttributes(s.element.find(".labels"),s,s.settings.attributes,"labels",!0),E.addAttributes(s.element.find(".labels").children(),s,s.settings.attributes,"label",!0),E.addAttributes(s.element.find(".lines"),s,s.settings.attributes,"lines",!0),E.addAttributes(s.element.find(".lines").children("polyline"),s,s.settings.attributes,"polyline",!0),E.addAttributes(s.element.find(".lines").children(".circles"),s,s.settings.attributes,"circle",!0))}),100),s.element.off("dblclick.".concat(s.namespace)).on("dblclick.".concat(s.namespace),"*",(function(t){t.stopImmediatePropagation()})),r.enter().insert("path").style("fill",(function(t,e){return bi.chartColor(e,"pie",t.data)})).attr("class","slice").call((function(t){t._groups.forEach((function(t){t.forEach((function(t,e){var i=t.__data__,n="slice-".concat(e);s.settings.attributes?E.addAttributes($(t),i,s.settings.attributes,n,!0):E.addAttributes($(t),i,i.data.attributes,n,!0)}))}))})).on("contextmenu.".concat(s.namespace),(function(t,e){bi.triggerContextMenu(s.element,d3.select(this).nodes()[0],e,t)})).on("click.".concat(s.namespace),(function(t,e){var i=this;s.settings.selectable&&(h=setTimeout((function(){d||s.doClickAction(e,e.index,i,n),d=!1}),200))})).on("dblclick.".concat(s.namespace),(function(t,e){clearTimeout(h),d=!0,s.doDoubleClickAction(e,e.index,this,n)})).on("mouseenter.".concat(s.namespace),(function(t,e){if(s.settings.showTooltips){var i=s.svg.selectAll("circle").nodes()[e.index],o=c(i),r=s.midAngle(e),d=r<=Math.PI/4&&r>=0||r>7*Math.PI/4,h=r<=3*Math.PI/4&&r>=Math.PI/4,u=r<=5*Math.PI/4&&r>=3*Math.PI/4,p=r<=7*Math.PI/4&&r>=5*Math.PI/4,g="",f=bi.formatToSettings(e,s.settings.tooltip),m={value:l.str&&!l.isTooltip?d3.format(l.str)(e.value):f,percent:l.str?d3.format(l.str)(e.data.percent):"".concat(isNaN(e.data.percentRound)?0:e.data.percentRound,"%")};-1===(g=(g=(g=(g=(g=e.data.tooltip||m.value).replace("{{percent}}","".concat(m.percent))).replace("{{value}}",e.value)).replace("%percent%","".concat(m.percent))).replace("%value%",e.value)).indexOf("<b>")&&(g=(g=g.replace("(","<b>")).replace(")","</b>")),null!=n&&clearTimeout(n),n=setTimeout((function(){!function(){if(""!==g){var t=bi.tooltipSize(g),e=o.left,i=o.top;bi.tooltip&&bi.tooltip.length&&(d||h||u||p)&&bi.tooltip[a?"addClass":"removeClass"]("is-personalizable"),d&&(e-=t.width/2,i-=t.height-5,bi.showTooltip(e,i,g,"top")),h&&(i-=t.height/2,bi.showTooltip(e,i,g,"right")),u&&(e-=t.width/2,bi.showTooltip(e,i,g,"bottom")),p&&(e-=t.width-5,i-=t.height/2,bi.showTooltip(e,i,g,"left"))}}()}),300)}})).on("mouseleave.".concat(s.namespace),(function(){clearTimeout(n),bi.hideTooltip()})).merge(r).transition().duration(s.settings.animationSpeed).attrTween("d",(function(t){this.current=this.current||t;var e=d3.interpolate(this.current,t);return this.current=e(0),function(t){return s.arc(e(t))}})),r.exit().remove(),!o){var u=s.element.parent().width()<520,p=s.settings.showLines;if(!s.settings.showLinesMobile&&p&&(p=!u),s.settings.dotSize=p?2:0,p){var g=s.svg.select(".labels").selectAll("text").data(s.pie(t),s.key);g.enter().append("text").attr("dy",".35em").text((function(t){return u?t.data.shortName:bi.formatToSettings(t,s.settings.lines)})).merge(g).transition().duration(s.settings.animationSpeed).attrTween("transform",(function(t){this.current=this.current||t;var e=d3.interpolate(this.current,t);return this.current=e(0),function(t){var i=e(t),n=s.outerArc.centroid(i);return n[0]=s.dims.radius*(s.midAngle(i)<Math.PI?1:-1),n[0]-=s.midAngle(i)<Math.PI?20:-20,"translate(".concat(n,")")}})).styleTween("text-anchor",(function(t){this.current=this.current||t;var e=d3.interpolate(this.current,t);return this.current=e(0),function(t){var i=e(t);return s.isRTL?s.midAngle(i)>Math.PI?"start":"end":s.midAngle(i)<Math.PI?"start":"end"}})),g.exit().remove();var f=s.svg.select(".lines").selectAll("polyline").data(s.pie(t),s.key);f.enter().append("polyline").merge(f).transition().duration(s.settings.animationSpeed).attrTween("points",(function(t){this.current=this.current||t;var e=d3.interpolate(this.current,t);return this.current=e(0),function(t){var i=e(t),n=s.outerArc.centroid(i);return n[0]=.85*s.dims.radius*(s.midAngle(i)<Math.PI?1:-1),[s.outerArc.centroid(i),s.outerArc.centroid(i),n]}})),f.exit().remove()}var m=s.svg.select(".lines").selectAll("circle").data(s.pie(t),s.key);m.enter().append("circle").attr("class","circles").attr("r",s.settings.dotSize).merge(m).transition().duration(s.settings.animationSpeed).attrTween("transform",(function(t){this.current=this.current||t;var e=d3.interpolate(this.current,t);return this.current=e(0),function(t){var i=e(t);return"translate(".concat(s.outerArc.centroid(i)," )")}})),m.exit().remove()}},setInitialSelected:function(){var t,e=this,i=0;this.svg.selectAll(".slice").each((function(n,s){n&&n.data&&n.data.selected&&i<1&&(i++,e.initialSelectCall=!0,(t=d3.select(this)).on("click.".concat(e.namespace)).call(t.node(),t.datum(),s))})),this.initialSelectCall=!1},handleEvents:function(){var t=this;return this.element.on("updated.".concat(this.namespace),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated()})),this},getSelected:function(){return bi.selected},setSelected:function(t,e){var i,n,s=this,a=0,o=E.equals;this.svg.selectAll(".slice").each((function(e,r){e&&e.data&&a<1&&(void 0!==t.fieldName&&void 0!==t.fieldValue&&t.fieldValue===e.data[t.fieldName]||void 0!==t.index&&t.index===r||t.data&&o(t.data,s.chartData[r].data)||t.elem&&$(this).is(t.elem))&&(a++,i=d3.select(this),n=r)})),a>0&&(e||!i.classed("is-selected"))&&i.on("click.".concat(s.namespace)).call(i.node(),i.datum(),n)},toggleSelected:function(t){this.setSelected(t,!0)},handleResize:function(){this.width!==this.element.width()&&(this.width=this.element.width(),this.element.is(":visible")&&this.updated())},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset&&(this.settings.dataset=t.dataset),this.element.empty(),this.build().element.trigger("rendered",[this.svg])},roundLargestRemainer:function(t){for(var e=0,i=0,n=0,s=[],a=0;a<t.length;a++)e+=parseInt(t[a],10),s[a]=a;var o=100-e;s.sort((function(e,i){return n=t[e]-parseInt(t[e],10),t[i]-parseInt(t[i],10)-n})),t.sort((function(t,e){return n=t-parseInt(t,10),e-parseInt(e,10)-n}));for(var r=0;r<t.length;r++)i=r,t[r]=i<o?parseInt(t[r],10)+1:parseInt(t[r],10);for(var l=[],c=0;c<t.length;c++)l[s[c]]=t[c];return l},doClickAction:function(t,e,i,n){var s=this;clearTimeout(n);var a=i&&d3.select(i).classed("is-selected");bi.setSelectedElement({task:a?"unselected":"selected",container:s.element,selector:a?".chart-container .is-selected":i,isTrigger:!s.initialSelectCall&&!a,d:t.data,i:e,type:s.settings.type,dataset:s.settings.dataset,svg:s.svg}),a&&!s.initialSelectCall&&s.element.triggerHandler("selected",[d3.select(i).nodes(),{},e])},doDoubleClickAction:function(t,e,i,n){var s;clearTimeout(n);var a=this.settings.dataset,o=a&&null!==(s=a[0])&&void 0!==s&&(null==s?void 0:s.data)?a[0].data[e]:t.data||t;o=o||{};var r=[{elem:d3.select(i).nodes(),data:o,index:e}];this.element.triggerHandler("dblclick",[r])},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".slice").off(e(["mouseenter","mouseleave","click","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),delete this.namespace,this},midAngle:function(t){return t.startAngle+(t.endAngle-t.startAngle)/2},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("pie-chart"),$.removeData(this.element[0],Hi),$.removeData(this.element[0],"chart"))}};var Wi="radar",ji={dataset:[],redrawOnResize:!0,margin:{top:0,right:0,bottom:0,left:0},levels:4,maxValue:0,labelFactor:1.27,wrapWidth:60,opacityArea:.2,dotRadius:3,opacityCircles:0,strokeWidth:1,selectable:!0,roundStrokes:!0,showCrosslines:!0,showAxisLabels:!0,colors:null,showTooltips:!0,tooltip:{show:"value",formatter:".0%"},axisFormatter:".0%",showLegend:!0,legendPlacement:"right",emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function Ui(t,e){this.settings=E.mergeSettings(t,e,ji),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),this.init()}Ui.prototype={init:function(){return this.namespace=E.uniqueId({classList:[this.settings.type,"chart"]}),this.width=0,this.setupColors().build().handleEvents(),this.element.trigger("rendered",[this.svg]),this},build:function(){return this.updateData(this.settings.dataset),this.setInitialSelected(),this},setupColors:function(){return this.settings.colors&&!this.useBuiltInColors||(this.settings.colors=bi.colorRange(),this.useBuiltInColors=!0),this},updateData:function(t){var e=this,i=this.settings,n=this.element.closest(".is-personalizable").length>0;if(this.element.addClass("chart-radar"),0!==t.length){var s,a=this.element.parent(),o={w:a.width(),h:a.height(),extra:.957};if("right"===i.legendPlacement&&(o.w*=.75),(ft.uplift||ft.new)&&o.w>=420&&(o.extra-=.12),i.showLegend&&o.w<420&&!i.margin.bottom)s=o.w>405?5:o.w>390?3:o.w>350?.9:.5,o.h-=(420-o.w)*s;o.transform={x:o.w/2+(i.margin.left+i.margin.right)/2,y:o.h/2*o.extra+(i.margin.top+i.margin.bottom)/2};var r,l=d3.scaleOrdinal(i.colors),c=Math.max(i.maxValue,d3.max(t,(function(t){return d3.max(t.data.map((function(t){return t.value})))}))),d=t[0].data.map((function(t){return function(t){var e=328,i=469,n="";return o.w<=e?n=t.shortName||t.abbrName||t.name||"":o.w>=e+1&&o.w<=i?n=t.abbrName||t.name||"":o.w>i&&(n=t.name||""),n}(t)})),h=d.length,u=2*Math.PI/h,p=Math.min(o.w/3,o.h/3)*o.extra,g=d3.scaleLinear().range([0,p]).domain([0,c]),f=this.element[0];d3.select(f).select("svg").remove();var m=d3.select(f).append("svg").attr("width",o.w+(i.margin.left+i.margin.right)).attr("height",o.h+(i.margin.top+i.margin.bottom)).attr("class","chart-radar");this.svg=m;var v=m.append("g").attr("transform","translate(".concat(o.transform.x,",").concat(o.transform.y,")"));if(i.opacityCircles>0){var b=v.append("defs").append("filter").attr("id","glow");b.append("feGaussianBlur").attr("stdDeviation","2.5").attr("result","coloredBlur");var y=b.append("feMerge");y.append("feMergeNode").attr("in","coloredBlur"),y.append("feMergeNode").attr("in","SourceGraphic")}var w=v.append("g").attr("class","chart-radar-axis-wrapper");w.selectAll(".levels").data(d3.range(1,i.levels+1).reverse()).enter().append("circle").attr("class","chart-radar-grid-circle").attr("r",(function(t){return p/i.levels*t})).style("fill-opacity",i.opacityCircles).style("filter",i.opacityCircles>0?"url(#glow)":""),i.showAxisLabels&&w.selectAll(".axis-label").data(d3.range(1,i.levels+1).reverse()).enter().append("text").attr("class","axis-label").attr("x",4).attr("y",(function(t){return-t*p/i.levels})).attr("dy","0.4em").style("font-size",ft.uplift||ft.new?"12px":"10px").attr("fill","#737373").text((function(t){var e=Math.round(10*c)/10;return i.axisFormatter.indexOf("%")>-1?d3.format(i.axisFormatter)(e*t/i.levels):d3.format(i.axisFormatter)(t/i.levels)}));var x=w.selectAll(".axis").data(d).enter().append("g").attr("class","axis");i.showCrosslines&&x.append("line").attr("x1",0).attr("y1",0).attr("x2",(function(t,e){return g(1.05*c)*Math.cos(u*e-Math.PI/2)})).attr("y2",(function(t,e){return g(1.05*c)*Math.sin(u*e-Math.PI/2)})).attr("class","chart-radar-crossline").style("stroke-width","1px"),x.append("text").attr("class","legend").style("font-size",ft.uplift||ft.new?"14px":"12px").attr("text-anchor","middle").attr("dy","0.35em").attr("x",(function(t,e){return g(c*i.labelFactor)*Math.cos(u*e-Math.PI/2)})).attr("y",(function(t,e){return g(c*i.labelFactor)*Math.sin(u*e-Math.PI/2)})).text((function(t){return t})),this.element[o.w<420?"addClass":"removeClass"]("is-small"),o.w>456&&m.selectAll(".chart-radar-axis-wrapper .axis .legend").each((function(){bi.wrap(d3.select(this),i.wrapWidth,i.labelFactor)}));var C=d3.lineRadial().curve(d3.curveBasisClosed).radius((function(t){return g(t.value)})).angle((function(t,e){return e*u}));i.roundStrokes&&C.curve(d3.curveCardinalClosed);var k=v.selectAll(".radarWrapper").data(t.map((function(t){return t.data}))).enter().append("g").attr("class","chart-radar-wrapper"),T=!1,A=0;if(e.element.off("dblclick.".concat(e.namespace)).on("dblclick.".concat(e.namespace),"*",(function(t){t.stopImmediatePropagation()})),k.append("path").attr("class","chart-radar-area").call((function(e){e._groups.forEach((function(e){e.forEach((function(e,i){var n=t[i];E.addAttributes($(e),n,n.attributes,"area")}))}))})).attr("d",(function(t){return C(t)})).style("fill",(function(t,e){return l(e)})).style("fill-opacity",i.opacityArea).style("cursor",e.settings.selectable?"pointer":"inherit").on("contextmenu.".concat(e.namespace),(function(t,i){bi.triggerContextMenu(e.element,d3.select(this).nodes()[0],i,t)})).on("click.".concat(e.namespace),(function(t,i){var n=this;e.settings.selectable&&(A=setTimeout((function(){T||e.doClickAction(i,$(n).index(),n,r,m),T=!1}),200))})).on("dblclick.".concat(e.namespace),(function(t,i){clearTimeout(A),T=!0,e.doDoubleClickAction(i,$(this).index(),this)})),k.append("path").attr("class","chart-radar-stroke").call((function(e){e._groups.forEach((function(e){e.forEach((function(e,i){var n=t[i];E.addAttributes($(e),n,n.attributes,"stroke")}))}))})).attr("d",(function(t){return C(t)})).style("stroke-width","".concat(i.strokeWidth,"px")).style("stroke",(function(t,e){return l(e)})).style("cursor",e.settings.selectable?"pointer":"inherit").style("fill","none").style("filter",i.opacityCircles>0?"url(#glow)":""),k.selectAll(".chart-radar-circle").data((function(t){return t})).enter().append("circle").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,"circle")}))}))})).attr("class","chart-radar-circle").attr("r",i.dotRadius).attr("cx",(function(t,e){return g(t.value)*Math.cos(u*e-Math.PI/2)})).attr("cy",(function(t,e){return g(t.value)*Math.sin(u*e-Math.PI/2)})).style("fill",(function(){return l($(this.parentNode).index()-1)})).style("fill-opacity",.6),v.selectAll(".radar-circle-wrapper").data(t.map((function(t){return t.data}))).enter().append("g").attr("class","radar-circle-wrapper").selectAll(".radar-invisible-circle").data((function(t){return t})).enter().append("circle").call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var e=t.__data__;E.addAttributes($(t),e,e.attributes,"invisible-circle")}))}))})).attr("class","radar-invisible-circle").attr("r",1.5*i.dotRadius).attr("cx",(function(t,e){return g(t.value)*Math.cos(u*e-Math.PI/2)})).attr("cy",(function(t,e){return g(t.value)*Math.sin(u*e-Math.PI/2)})).style("fill","none").style("pointer-events","all").on("mouseenter.".concat(e.namespace),(function(t,e){if(i.showTooltips){var s=$(this).offset(),a=bi.formatToSettings(e,i.tooltip);-1===a.indexOf("<b>")&&(a=(a=a.replace("(","<b>")).replace(")","</b>"));var o=bi.tooltipSize(a),l=s.left,c=s.top;l-=o.width/2-6,c-=o.height+6,null!=r&&clearTimeout(r),r=setTimeout((function(){bi.tooltip&&bi.tooltip.length&&bi.tooltip[n?"addClass":"removeClass"]("is-personalizable"),bi.showTooltip(l,c,a,"top")}),300)}})).on("mouseleave.".concat(e.namespace),(function(){clearTimeout(r),bi.hideTooltip()})).on("contextmenu.".concat(e.namespace),(function(t,i){bi.triggerContextMenu(e.element,d3.select(this).nodes()[0],i,t)})),i.showTooltips&&bi.appendTooltip("is-pie"),i.showLegend){i.legendPlacement&&this.element.addClass("has-".concat(i.legendPlacement,"-legend"));var S=i.dataset.map((function(t,e){var n={name:t.name,display:"twocolumn",placement:i.legendPlacement,color:l(e)};return t.attributes&&(n.data={attributes:t.attributes}),n}));i.svg=e.svg,bi.addLegend(S,"pie",i,this.element)}}else e.element.emptymessage(i.emptyMessage)},setInitialSelected:function(){var t=this.settings;if(Array.isArray(t.dataset))for(var e=0,i=t.dataset.length;e<i;e++)if(t.dataset[e].selected){var n=this.svg.selectAll(".chart-radar-area").nodes(),s=d3.select(n[e]);this.svg.selectAll(".is-selected").classed("is-selected",!1),this.svg.selectAll(".is-not-selected").classed("is-not-selected",!1),this.svg.selectAll(".chart-radar-area").classed("is-not-selected",!0),s.classed("is-selected",!0).classed("is-not-selected",!1),s.style("fill-opacity",t.opacityArea);break}},handleEvents:function(){var t=this;return this.element.on("updated.".concat(this.namespace),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(this.namespace),(function(){t.handleResize()})),this.element.on("resize.".concat(this.namespace),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(this.namespace),(function(){t.updated()})),this},getSelected:function(){return bi.selected},setSelected:function(t,e){var i,n=0,s=this;this.svg.selectAll(".chart-radar-area").each((function(e,a){var o=s.settings.dataset[a];o&&o.data&&n<1&&(void 0!==t.fieldName&&void 0!==t.fieldValue&&t.fieldValue===o[t.fieldName]||void 0!==t.index&&t.index===a||t.elem&&$(this).is(t.elem))&&(n++,i=d3.select(this))})),n>0&&(e||!i.classed("is-selected"))&&d3.select(i.node()).dispatch("click")},toggleSelected:function(t){this.setSelected(t,!0)},handleResize:function(){this.width!==this.element.width()&&(this.width=this.element.width(),this.element.is(":visible")&&this.updated())},doClickAction:function(t,e,i,n,s){clearTimeout(n);var a=this.settings,o=d3.select(i),r=o.classed("is-selected");s.selectAll(".is-selected").classed("is-selected",!1),s.selectAll(".is-not-selected").classed("is-not-selected",!1),bi.clearSelected(a.dataset),r||(s.selectAll(".chart-radar-area").classed("is-not-selected",!0),o.classed("is-selected",!0).classed("is-not-selected",!1),o.style("fill-opacity",a.opacityArea),a.dataset[e].selected=!0);var l={elem:o.nodes(),data:t,index:e};this.element.triggerHandler(r?"deselected":"selected",l),bi.selected=r?[]:l},doDoubleClickAction:function(t,e,i){var n={elem:[i],data:t,index:e};this.element.triggerHandler("dblclick",n)},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset?(this.element.find(".chart-legend").remove(),this.settings.dataset=t.dataset,this.updateData(this.settings.dataset),this):(this.element.empty(),this.setupColors().build())},teardown:function(){var t=this,e=function(e){return"".concat(e.join(".".concat(t.namespace," ")),".").concat(t.namespace)};return this.element&&(this.element.find(".chart-radar-area").off(e(["click","contextmenu"])),this.element.find(".radar-invisible-circle").off(e(["mouseenter","mouseleave","contextmenu"])),this.element.off(e(["updated","resize"]))),$("body").off("resize.".concat(this.namespace)),$("html").off("themechanged.".concat(this.namespace)),delete this.namespace,this},destroy:function(){this.teardown(),bi.removeTooltip(),this.element&&(this.element.empty().removeClass("radar-chart"),$.removeData(this.element[0],Wi),$.removeData(this.element[0],"radar"))}},$.fn.chart=function(t){var e=null;switch(this.empty(),t.type){case"targeted-achievement":case"completion":case"completion-target":if(e=this.data(t.type))return e.updated(t),this;var i=new Ti(this,t);this.data(Ci,i),this.data("chart",i),this.data(t.type,i);break;case"bullet":if(e=this.data(yi))return e.updated(t),this;var n=new xi(this,t);this.data(yi,n),this.data("chart",n);break;case"radar":if(e=this.data(Wi))return e.updated(t),this;var s=new Ui(this,t);this.data(Wi,s),this.data("chart",s);break;case"sparkline":if(e=this.data(Ai))return e.updated(t),this;var a=new $i(this,t);this.data(Ai,a),this.data("chart",a);break;case"sparkline-dots":if(t.isDots=!0,e=this.data(Ai))return e.updated(t),this;var o=new $i(this,t);this.data(Ai,o),this.data("chart",o);break;case"sparkline-peak":if(t.isPeakDot=!0,e=this.data(Ai))return e.updated(t),this;var r=new $i(this,t);this.data(Ai,r),this.data("chart",r);break;case"sparkline-dots-n-peak":if(t.isPeakDot=!0,t.isDots=!0,e=this.data(Ai))return e.updated(t),this;var l=new $i(this,t);this.data(Ai,l),this.data("chart",l);break;case"sparkline-minmax":if(t.isMinMax=!0,e=this.data(Ai))return e.updated(t),this;var c=new $i(this,t);this.data(Ai,c),this.data("chart",c);break;case"sparkline-medianrange":if(t.isMedianRange=!0,e=this.data(Ai))return e.updated(t),this;var d=new $i(this,t);this.data(Ai,d),this.data("chart",d);break;case"sparkline-medianrange-n-peak":if(t.isMedianRange=!0,t.isPeakDot=!0,e=this.data(Ai))return e.updated(t),this;var h=new $i(this,t);this.data(Ai,h),this.data("chart",h);break;case"line":if(e=this.data(Li))return e.updated(t),this;var u=new Mi(this,t);this.data(Li,u),this.data("chart",u);break;case"area":if(t.isArea=!0,e=this.data(Li))return e.updated(t),this;var p=new Mi(this,t);this.data(Li,p),this.data("chart",p);break;case"bubble":if(t.isBubble=!0,e=this.data(Li))return e.updated(t),this;var g=new Mi(this,t);this.data(Li,g),this.data("chart",g);break;case"scatterplot":if(t.isScatterPlot=!0,e=this.data(Li))return e.updated(t),this;var f=new Mi(this,t);this.data(Li,f),this.data("chart",f);break;case"column":case"column-grouped":case"positive-negative":case"column-positive-negative":if(e=this.data(Bi))return e.updated(t),this;var m=new Ri(this,t);this.data(Bi,m),this.data("chart",m);break;case"column-stacked":if(t.isStacked=!0,e=this.data(Bi))return e.updated(t),this;var v=new Ri(this,t);this.data(Bi,v),this.data("chart",v);break;case"bar":if(t.isStacked=!0,e=this.data(zi))return e.updated(t),this;var b=new Ni(this,t);this.data(zi,b),this.data("chart",b);break;case"bar-stacked":if(t.isStacked=!0,e=this.data(zi))return e.updated(t),this;var y=new Ni(this,t);this.data(zi,y),this.data("chart",y);break;case"bar-normalized":if(t.isNormalized=!0,e=this.data(zi))return e.updated(t),this;var w=new Ni(this,t);this.data(zi,w),this.data("chart",w);break;case"bar-grouped":if(t.isStacked=!1,t.isGrouped=!0,e=this.data(zi))return e.updated(t),this;var x=new Ni(this,t);this.data(zi,x),this.data("chart",x);break;case"pie":if(e=this.data(Hi))return e.updated(t),this;var C=new Vi(this,t);this.data(Hi,C),this.data("chart",C);break;case"donut":if(t.isDonut=!0,e=this.data(Hi))return e.updated(t),this;var k=new Vi(this,t);this.data(Hi,k),this.data("chart",k);break;default:e=null}return this};var Gi={isToday:function(t){var e=new Date;return t.getDate()===e.getDate()&&t.getMonth()===e.getMonth()&&t.getFullYear()===e.getFullYear()},firstDayOfWeek:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=t.getDay(),s=new Date(t),a=n>=e||i?n-e:6-n;return s.setDate(t.getDate()-a),s.setHours(0,0,0,0),s},lastDayOfWeek:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,i=this.firstDayOfWeek(t,e);return i.setDate(i.getDate()+6),i.setHours(23,59,59,999),i},dateDiff:function(t,e,i){var n=new Date;return Math.round((e-t)/(36e5*(i?1:Math.abs(n.getTimezoneOffset()))))},monthDiff:function(t,e){var i;return i=12*(e.getFullYear()-t.getFullYear()),i-=t.getMonth(),(i+=e.getMonth())<=0?0:i},add:function(t,e,i){var n=null,s=t instanceof Date?new Date(t.toISOString()):new Date(t);if("days"===i)n=new Date(s.setDate(s.getDate()+e));return n},subtract:function(t,e,i){var n=null,s=t instanceof Date?new Date(t.toISOString()):new Date(t);if("days"===i)n=new Date(s.setDate(s.getDate()-e));return n},isDaylightSavingTime:function(t){var e=new Date(t.getFullYear(),0,1).getTimezoneOffset(),i=new Date(t.getFullYear(),6,1).getTimezoneOffset();return Math.max(e,i)!==t.getTimezoneOffset()},isWithinRange:function(t,e,i){return t<=i&&i<=e},hoursTo24:function(t,e){var i=e>=0;return 12===t&&i?0===e?0:t:t+12*(i?e:0)}},_i={addCalculatedFields:function(t,e,i,n){var s=function(t,i){return H.formatDate(t,E.extend(!0,{locale:e.name},i))},a={pattern:"yyyy-MM-ddTHH:mm:ss.SSS",locale:e.name},o=function(t){var i=H.parseDate(t,a);return H.isIslamic(e.name)&&(i=new Date(i[0],i[1],i[2],i[3],i[4],i[5],i[6])),i},r=function(t){return H.translate(t,{locale:e.name,language:i})};t.color=this.getEventTypeColor(t,n),t.borderColor=this.getEventTypeBorderColor(t,n),t.duration=Math.abs(Gi.dateDiff(new Date(t.ends),new Date(t.starts),!1));var l=new Date,c=Date.UTC(l.getUTCFullYear(),l.getUTCMonth(),l.getUTCDate()),d=new Date(t.starts),h=Date.UTC(d.getUTCFullYear(),d.getUTCMonth(),d.getUTCDate()),u="".concat(d.getDate(),"-").concat(d.getMonth()+1,"-").concat(d.getFullYear()),p="".concat(l.getDate(),"-").concat(l.getMonth()+1,"-").concat(l.getFullYear());t.durationUnits=r(t.duration>1?"Days":"Day"),t.daysUntil=t.starts?Gi.dateDiff(h,c):0;var g,f,m=(new Date(t.ends)-new Date(t.starts))/36e5;if(t.durationHours=m>0&&m<.5?1:Math.round(m),t.isDays=!0,c>h&&u!==p&&(t.daysUntil=1),void 0===t.isAllDay&&(t.isAllDay=!0),t.durationHours<24&&(t.isDays=!1,t.isAllDay=!1,t.durationUnits=r(t.durationHours>1?"Hours":"Hour")),"true"===t.isAllDay.toString()){t.duration=(g=new Date(t.starts),f=new Date(t.ends),Math.abs(Math.round((f-g)/864e5))||1),t.durationUnits=r(t.duration>1?"Days":"Day"),t.isDays=!0,delete t.durationHours}if(t.starts){var v=o(t.starts);t.startsLocale=s(v),H.isIslamic(e.name)&&(t.startsLocale=s(H.gregorianToUmalqura(v))),t.startsHourLocale=s(v,{date:"hour"}),Array.isArray(v)?t.startsHour=parseFloat(v[3]+v[4]/60):t.startsHour=parseFloat(v.getHours()+v.getMinutes()/60)}if(t.ends){var b=o(t.ends);t.endsLocale=s(b),H.isIslamic(e.name)&&(t.endsLocale=s(H.gregorianToUmalqura(b))),t.endsHourLocale=s(b,{date:"hour"}),Array.isArray(b)?t.endsHour=parseFloat(b[3]+b[4]/60):t.endsHour=parseFloat(b.getHours()+b.getMinutes()/60)}if(t.eventTypes=n,t.isAllDay=t.isAllDay.toString(),"false"===t.isAllDay.toString()&&delete t.isAllDay,!t.isAllDay&&t.durationHours>=24&&(t.isAllDay=!1,t.durationUnits=r("Hours"),t.isDays=!1,delete t.duration),t.starts&&t.ends){var y=this.timeDiffInSeconds(o(t.starts),o(t.ends));t.durationInTime=this.timeBySeconds(y)}else t.durationInTime&&delete t.durationInTime;return t},timeDiffInSeconds:function(t,e){var i=Math.abs(e.getTime()-t.getTime());return Math.ceil(i/1e3)},timeBySeconds:function(t){return t=Number(t),{days:Math.floor(t/86400),hours:Math.floor(t%86400/3600),minutes:Math.floor(t%3600/60),seconds:Math.floor(t%60)}},formateTimeString:function(t,e,i){if(t.durationInTime){var n=function(t){return H.translate(t,{locale:e.name,language:i})},s=t.durationInTime,a="";if(s.days){var o=n(s.days>1?"Days":"Day");a+="".concat(s.days," ").concat(o)}if(s.hours){var r=n(s.hours>1?"Hours":"Hour");s.days&&(a+="".concat(s.minutes||s.seconds?", ":" ".concat(n("And")," "))),a+="".concat(s.hours," ").concat(r)}if(s.minutes){var l=n(s.minutes>1?"Minutes":"Minute");(s.days||s.hours)&&(a+="".concat(s.seconds?", ":" ".concat(n("And")," "))),a+="".concat(s.minutes," ").concat(l)}if(s.seconds){var c=n(s.seconds>1?"Seconds":"Second");a+="".concat(s.days||s.hours||s.minutes?" ".concat(n("And")," "):""),a+="".concat(s.seconds," ").concat(c)}""!==a&&(t.duration="",t.durationUnits="",t.durationHours=a)}},getEventTypeColor:function(t,e){var i,n="azure";if(!t.type&&!t.color)return n;var s=e.filter((function(e){return e.id===t.type}));return void 0!==t.color&&1===s.length&&s[0].color!==t.color?s[0].color:"#"===(null===(i=t.color)||void 0===i||null==i?void 0:i.substr(0,1))||t.color?t.color:1===s.length?n=s[0].color||"azure":n},getEventTypeBorderColor:function(t,e){var i,n="",s=e.filter((function(e){return e.id===t.type}));return void 0!==t.borderColor&&1===s.length&&s[0].borderColor!==t.borderColor?s[0].borderColor:"#"===(null===(i=t.borderColor)||void 0===i||null==i?void 0:i.substr(0,1))||t.borderColor?t.borderColor:1===s.length?n=s[0].borderColor||"azure":n},cleanEventData:function(t,e,i,n,s,a,o){var r={pattern:"yyyy-MM-ddTHH:mm:ss.SSS",locale:n.name},l=function(t){return H.formatDate(t,r)},c=function(t,e){return H.parseDate(t,E.extend(!0,{locale:n.name},e))},d="on"===t.isAllDay||"true"===t.isAllDay||t.isAllDay,h=new Date(i),u=h,p=h;if(t.startsLocale&&t.endsLocale&&(u=new Date(c(t.startsLocale)),p=new Date(c(t.endsLocale))),"string"!=typeof t.starts||t.startsLocale||(u=new Date(t.starts)),"string"!=typeof t.ends||t.endsLocale||(p=new Date(t.ends)),H.isValidDate(u)||(u=h),H.isValidDate(p)||(p=h),d)u.setHours(0,0,0,0),t.starts=l(new Date(u)),p.setHours(23,59,59,999),t.ends=l(new Date(p)),t.duration=t.starts===t.ends?1:null,t.isAllDay=!0;else{if(u===p)p.setHours(p.getHours()+parseInt(t.durationHours,10)),t.ends=l("safari"===x.browser.name?p:p.toISOString()),t.duration=null;else if(t.endsHourLocale&&t.startsHourLocale){var g=c(t.startsHourLocale,{date:"hour"}),f=c(t.endsHourLocale,{date:"hour"});u.setHours(g.getHours(),g.getMinutes(),g.getSeconds(),g.getMilliseconds()),p.setHours(f.getHours(),f.getMinutes(),f.getSeconds(),f.getMilliseconds()),"safari"===x.browser.name?(t.starts=l(u),t.ends=l(p)):(t.starts=l(u.toISOString()),t.ends=l(p.toISOString())),t.duration=Gi.dateDiff(new Date(t.starts),new Date(t.ends))}else t.ends=l(new Date(p));t.starts=l(new Date(u)),t.isAllDay=!1}if(void 0===t.comments&&e&&(t.comments=H.translate("NoCommentsEntered",{locale:n.name,language:s}),t.noComments=!0),!t.subject&&e&&(t.subject=H.translate("NoTitle",{locale:n.name,language:s})),t.type||(t.type=o[0].id),void 0===t.id&&e){var m=0===a.length?0:parseInt(a[a.length-1].id,10);t.id=(m+1).toString()}"NewEvent"===t.title&&(t.title=H.translate("NewEvent",{locale:n.name,language:s}))}},Yi="tag",Ki=["default","neutral","secondary","error","alert","good","info"],Xi={audibleContent:void 0,clickable:!1,clickHandler:void 0,content:" ",disabled:!1,dismissible:!1,dismissHandler:void 0,href:void 0,id:void 0,parent:void 0,style:Ki[0],value:void 0};function Qi(t,e){var i;if(this.settings=E.mergeSettings(t,e,Xi),t instanceof HTMLElement?t.querySelector(".tag-content")?i=t:((i=document.createElement("span")).className+="".concat(t.className),t.id&&t.id.length?(i.id=t.id,t.removeAttribute("id")):"string"==typeof this.settings.id&&this.settings.id.length&&(i.id=this.settings.id,delete this.settings.id),t.insertAdjacentElement("beforebegin",i),i.appendChild(t),i.classList.add("tag"),t.className="",t.classList.add("tag-content")):((i=document.createElement("span")).classList.add("tag"),this.settings&&this.settings.parent instanceof HTMLElement&&this.settings.parent.appendChild(i),"string"==typeof this.settings.id&&this.settings.id.length&&(i.id=this.settings.id,delete this.settings.id)),t){var n=t.querySelector(".audible");n&&i.insertAdjacentElement("afterbegin",n)}this.element=i,this.getSettingsFromElement(),this.init()}Qi.prototype={init:function(){this.render(),this.handleEvents()},get contentElement(){return this.element.querySelector(".tag-content")},render:function(){var t=this.element.classList,e=E.getArrayFromList(this.element.classList),i=this.settings.style;e.forEach((function(e){Ki.indexOf(e)>-1&&t.remove(e)})),-1===this.element.className.indexOf(i)&&"default"!==i&&t.add(i),this.settings.disabled&&t.add("is-disabled");var n="";this.settings.audibleContent&&(n='<span class="audible">'.concat(g.stripHTML(this.settings.audibleContent),"</span>"));var s="span",a="",o="",r="",l=this.settings.href&&this.settings.href.length;(l||this.originallyAnchor)&&(s="a",this.settings.disabled&&(r=' tabIndex="-1"')),l&&(a=' href="'.concat(this.settings.href,'"')),(this.settings.clickable||"function"==typeof this.settings.clickHandler)&&(t.add("is-linkable"),o='<button class="btn-linkable" focusable="false" tabIndex="-1">\n        '.concat($.createIcon("caret-right"),'\n        <span class="audible">').concat(H.translate("Link"),"</span>\n      </button>"));var c=0===this.settings.content.length?this.contentElement.textContent:this.settings.content,d="<".concat(s,' class="tag-content"').concat(a).concat(r,">").concat(g.sanitizeHTML(c),"</").concat(s,">"),h="";this.settings.dismissible&&(t.add("is-dismissible"),h='<button class="btn-dismissible" focusable="false" tabindex="-1" type="button">\n        '.concat($.createIcon("close"),'\n        <span class="audible">').concat(H.translate("Close"),"</span>\n      </button>")),this.element.innerHTML="".concat(n).concat(d).concat(o).concat(h),$(this.element).hideFocus()},get dismissibleBtn(){return this.element.querySelector(".btn-dismissible")},getSettingsFromElement:function(){var t=this.element.className,e="";Ki.forEach((function(i){t.indexOf(i)>-1&&(e=i)})),e&&(this.settings.style=e),this.settings.disabled=this.disabled;var i=this.dismissibleBtn,n=this.element.className.indexOf("is-dismissible")>-1;(i||n)&&(this.settings.dismissible=!0);var s=this.element.querySelector("a"),a=this.element.className.indexOf("is-linkable")>-1,o=this.element.querySelector(".btn-linkable");if(s||a||o){this.originallyAnchor=!0;var r=s.getAttribute("href");r&&r.length&&(r=g.stripTags(r),this.settings.href=r)}(a||o)&&(this.settings.clickable=!0);var l=this.element.querySelector(".audible");l instanceof HTMLElement&&(this.settings.audibleContent=g.stripHTML(l.innerText));var c=this.contentElement;c&&(this.settings.content=g.sanitizeHTML(c.innerText))},remove:function(){var t=this.element,e=t.parentNode;this.settings.parentAPI&&(e=this.settings.parentAPI.element),$(e).triggerHandler("beforetagremove",[this]),this.destroy(),t.remove(),$(e).triggerHandler("aftertagremove",[this])},dismiss:function(t){if(!this.disabled){this.remove(t),"function"==typeof this.settings.dismissHandler&&this.settings.dismissHandler(this);var e=this.settings.parentAPI;e&&e.focusPreviousElement(this.element)}},get disabled(){return this.element.className.indexOf("is-disabled")>-1},disable:function(){this.disabled||("A"===this.contentElement.tagName&&(this.contentElement.tabIndex=-1),this.settings.disabled=!0,this.element.classList.add("is-disabled"))},enable:function(){this.disabled&&("A"===this.contentElement.tagName&&(this.contentElement.tabIndex=0),this.settings.disabled=!1,this.element.classList.remove("is-disabled"))},teardown:function(){var t=$(this.element);t.off(["focusin.tag","focusout.tag","keydown.tag","click.tag"].join(" "));var e=this.dismissibleBtn;e&&e.parentNode.removeChild(e);var i=t.data("hidefocus");return i&&"function"==typeof i.destroy&&i.destroy(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Xi)),this.teardown().init()},destroy:function(){this.teardown(),$.removeData(this.element,Yi)},handleEvents:function(){var t=this;$(this.element).on("focusin.tag",(function(){t.element.classList.add("is-focused")})).on("focusout.tag",(function(){t.element.classList.remove("is-focused")}));var e=this;$(this.element).on("click.tag","a, .btn-linkable",(function(t){var i;return"function"==typeof e.settings.clickHandler&&(i=e.settings.clickHandler(e)),!(!1===i||e.settings.href&&"#"===e.settings.href)||(t.preventDefault(),!1)})),this.settings.dismissible&&($(this.element).on("click.tag",".btn-dismissible",(function(e){t.dismiss(e)})),$(this.element).on("keydown.tag","a",(function(e){8===e.keyCode&&t.dismiss(e)})))}};var Ji="taglist",Zi={tags:[]};function tn(t,e){var i=this;if(!(t instanceof HTMLElement))throw new Error("TagList base element must be defined");this.settings=E.mergeSettings(t,e,Zi),this.element=t;var n=E.getArrayFromList(this.element.querySelectorAll(".tag"));n.length&&(this.settings.tags=[],n.forEach((function(t){i.settings.tags.push(new Qi(t))}))),this.init()}tn.prototype={get length(){return this.tags.length},get elements(){return this.tags.map((function(t){return t.element}))},init:function(){var t=this;this.tags=[],Array.isArray(this.settings.tags)&&this.settings.tags.forEach((function(e){t.add(e)})),this.render()},getTargetProp:function(){return"content"},add:function(t){var e,i=this;if(!t)throw new Error("No object was provided");if(Array.isArray(t)){var n=[];return t.forEach((function(t){n.push(i.add(t))})),n}if(P.isEmpty(t))throw new Error("Provided object has no unique keys to use for creation of a Tag.");return t instanceof Qi?(e=t,t.settings.parentAPI=this):(t.parent=this.element,t.parentAPI=this,e=new Qi(void 0,t)),-1===this.tags.indexOf(e)&&this.tags.push(e),e},remove:function(t,e){t instanceof Qi&&(t=t.settings);var i,n=this.getTargetProp(t),s=this.tags.filter((function(e){return e.settings[n]===t[n]||(i=e,!1)}));return s.length<this.tags.length&&(this.tags=s),e&&i.remove(),i},removeAll:function(){this.tags.forEach((function(t){t.remove()})),this.tags=[]},render:function(){var t=this;this.tags.forEach((function(e){0===$(t.element).has(e.element).length?($(t.element).append(e.element),e.init()):e.render()})),this.element.classList[this.tags.length?"remove":"add"]("empty")},focusByIndex:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0,e=this.elements,i=e.length;t<0&&(t=0),t>i&&(t=i);var n=e[t],s=n.querySelector("a");return s&&s.focus(),n},focusPreviousElement:function(t){if(!(t instanceof HTMLElement))throw new Error("".concat(t," is not an HTMLElement"));var e=this.elements.indexOf(t);if(!(e>-1))throw new Error("".concat(t," does not exist within this TagList"));return this.focusByIndex(e-1)},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings),Array.isArray(t.tags)&&(this.settings.tags=t.tags)),this.removeAll(),this.init()},destroy:function(){this.tags.forEach((function(t){t.remove()})),delete this.tags,$(this.element).data(Ji)&&$.removeData(this.element,Ji)}};var en="virtual-scroll",nn={height:300,itemHeight:50,bufferSize:20,data:[],scrollTop:0,itemCount:0,itemTemplate:null,startIndex:0};function sn(t,e){this.settings=E.mergeSettings(t,e,nn),this.element=$(t),this.init()}sn.prototype={init:function(){return this.container=this.element.find(".ids-virtual-scroll")[0],this.stringTemplate='<div class="ids-virtual-scroll-item">${productName}</div>',this.itemCount=this.settings.data.length,this.itemTemplate=this.settings.itemTemplate||this.itemTemplate,this.handleEvents(),this.applyHeight(),this.renderItems(!1),this},handleEvents:function(){var t=this;return this.timeout=null,this.scrollHandler=function(e){t.handleScroll(e)},this.container.addEventListener("scroll",this.scrollHandler,{passive:!0}),this},handleScroll:function(t){var e=this;this.timeout&&cancelAnimationFrame(this.timeout);var i=t.target;this.timeout=requestAnimationFrame((function(){e.scrollTop=i.scrollTop}))},renderItems:function(t){var e=this;if(this.settings.data&&(t||0!==this.data.length)){var i=this.startIndex,n=this.startIndex+this.visibleItemCount();if(this.lastStart!==i||this.lastEnd!==n){this.lastStart=i,this.lastEnd=n;var s=this.settings.data.slice(i,n),a="";s.map((function(t,i){var n=e.itemTemplate(t,i);return a+=n,n})),this.itemContainer.style.transform="translateY(".concat(this.offsetY,"px)"),this.itemContainer.innerHTML=a,this.element.trigger("afterrendered",this,{elem:this,startIndex:i,endIndex:n})}}},applyHeight:function(){this.container.style.height="".concat(this.settings.height,"px"),this.container.querySelector(".ids-virtual-scroll-viewport").style.height="".concat(this.viewPortHeight,"px"),this.itemContainer=this.element.find(".contents")[0],this.itemContainer.style.transform="translateY(".concat(this.offsetY,"px)"),this.isTable=this.element.find(".ids-data-grid-container").length>0,this.isTable&&(this.element.find(".ids-virtual-scroll")[0].style.overflow="inherit")},visibleItemCount:function(){var t=Math.ceil(this.height/this.itemHeight)+2*this.bufferSize;return t=Math.min(Number(this.itemCount)-this.startIndex,t)},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,nn)),this.unbind().init()},unbind:function(){this.container.removeEventListener("scroll",this.scrollHandler),$.removeData(this.element[0],en)},destroy:function(){this.unbind(),$.removeData(this.element[0],en)},set height(t){this.settings.height=t},get height(){return 0===this.data.length?0:this.settings.height},set itemHeight(t){this.settings.itemHeight=t},get itemHeight(){return this.settings.itemHeight},set bufferSize(t){this.settings.bufferSize=t},get bufferSize(){return this.settings.bufferSize||20},set scrollTop(t){this.container.scrollTop===t?(this.settings.scrollTop=t,this.renderItems(!1)):this.container.scrollTop=t},get scrollTop(){return this.settings.scrollTop||0},scrollTo:function(t){this.scrollTop=Number(t)*this.itemHeight},get viewPortHeight(){return Number(this.itemCount)*Number(this.itemHeight)},set itemCount(t){this.settings.itemCount=t},get itemCount(){return this.settings.itemCount},get offsetY(){return Number(this.startIndex)*Number(this.itemHeight)},get startIndex(){var t=Math.floor(Number(this.scrollTop)/Number(this.itemHeight))-Number(this.bufferSize);return t=Math.max(0,t)},itemTemplate:function(t){return this.injectTemplate(this.stringTemplate,t)},injectTemplate:function(t,e){return t.replace(/\${(.*?)}/g,(function(t,i){return e[i]}))},set data(t){if(t)return this.settings.data=t,this.itemCount=t.length,this.lastStart=null,this.lastEnd=null,this.scrollTop=0,this.applyHeight(),void this.renderItems(!0);this.settings.data=null},get data(){var t;return null==this||null===(t=null==this?void 0:this.settings)||void 0===t||null==t?void 0:t.data}};var an="dropdown",on=["none","all","group"],rn={closeOnSelect:!0,cssClass:null,dropdownIcon:"dropdown",extraListWrapper:!1,filterMode:"contains",virtualScroll:!1,maxSelected:void 0,moveSelected:"none",moveSelectedToTop:void 0,multiple:!1,noSearch:!1,showEmptyGroupHeaders:!1,showSelectAll:!1,showTags:!1,showSearchUnderSelected:!1,source:void 0,sourceArguments:{},reload:"none",empty:!1,delay:300,maxWidth:null,placementOpts:null,onKeyDown:null,tagSettings:{},tagListMaxHeight:120,allTextString:null,selectedTextString:null,selectAllFilterOnly:!0,appendTo:'[role="main"]',attributes:null,width:void 0,widthTarget:void 0,noResultsTextString:"NoResultsFound"};function ln(t,e){this.settings=E.mergeSettings(t,e,rn),this.element=$(t),this.init()}ln.prototype={get value(){var t=this.settings.reload,e=this.settings.multiple;if("typeahead"===t)return e?this.selectedValues:this.element.val();for(var i,n=[],s=this.element[0].options,a=0;a<s.length;a++)(i=s[a]).selected&&n.push(i.value||i.text);return e||1!==n.length?n:n[0]},get overflowed(){if(!this.isMobile()||this.isMobile()&&!this.isOpen()){var t=this.pseudoElem.find("span").css("max-width","");if(Math.round(t.width())>Math.round(this.pseudoElem.width()))return t.css("max-width","100%"),!0}return!1},get isFocused(){var t=document.activeElement,e=this.pseudoElem.length&&this.pseudoElem.is($(t)),i=this.list&&this.list.length&&this.list[0].contains(t),n=this.tagListAPI&&this.tagListAPI.element.contains(t);return!!(e||i||n)},get isShortField(){return this.element.closest(".field-short").length>0||this.element.closest(".form-layout-compact").length>0},get selectedValues(){return this.selectedOptions.map((function(t){return t.value}))},get selectedOptions(){return E.getArrayFromList(this.element[0].querySelectorAll("option")).filter((function(t){return t.selected}))},init:function(){var t=this,e=this.element.attr("id");e=e?g.stripTags(e):"",this.inlineLabel=this.element.closest("label"),this.inlineLabelText=this.inlineLabel.find(".label-text"),this.isInlineLabel=this.element.parent().is(".inline"),this.timer=null,this.filterTerm="",this.keydownFlag=!1,e||(e=E.uniqueId(this.element,"dropdown"),f.setAttribute(this.element[0],"id",e),f.setAttribute(this.element.parent().find("label").first()[0],"for",e)),"ios"!==x.os.name&&"android"!==x.os.name||(this.settings.noSearch=!0),this.element.is(":disabled")&&this.element.closest(".field").addClass("is-disabled");var i=this.element[0].classList;0===i.length&&(this.element[0].classList="dropdown");var n,s=i.contains("dropdown-xs")?"dropdown input-xs":i.contains("dropdown-sm")?"dropdown input-sm":i.contains("dropdown-lg")?"dropdown input-lg":"dropdown",a=this.element.attr("style");this.isHidden=a&&a.indexOf("display: none")>=0,this.isInlineLabel?n=this.inlineLabel.next(".dropdown-wrapper"):(n=this.element.parent(".dropdown-wrapper")).length||(n=this.element.next(".dropdown-wrapper")),this.wrapper=n,this.isWrapped=this.wrapper.length>0,this.isWrapped||(this.wrapper=$('<div class="dropdown-wrapper"></div>').insertAfter(this.element)),this.isWrapped?(this.pseudoElem=this.wrapper.find(".".concat(s)),this.trigger=this.wrapper.find(".trigger")):this.pseudoElem=$("div#".concat(e,"-shdo")),i.contains("text-align-reverse")?s+=" text-align-reverse":i.contains("text-align-center")&&(s+=" text-align-center"),this.label=$('label[for="'.concat(g.stripTags(e),'"]')),this.pseudoElem.length?this.pseudoElem[0].setAttribute("class",s):this.pseudoElem=$('<div class="'.concat(s,'">')),this.isWrapped||this.pseudoElem.append($("<span></span>"));var o=f.getAttributes(this.element[0]),r=this.getDataAttributes(o,["data-validate"]);this.pseudoElem.attr(r.obj).attr({role:"combobox","aria-haspopup":"listbox","aria-autocomplete":"list","aria-expanded":"false"}),this.renderPseudoElemLabel(),this.settings.attributes&&E.addAttributes(this.pseudoElem,this,this.settings.attributes,"dropdown",!0),function(t){var e=t.element.prop("disabled"),i=t.element.prop("readonly");e?t.disable():i?t.readonly():t.enable()}(this),this.isWrapped||this.wrapper.append(this.pseudoElem,this.trigger),this.icon=this.wrapper.find(".trigger").find(".icon"),this.icon.length||(this.icon=$.createIconElement({icon:this.settings.dropdownIcon||"dropdown"}),this.wrapper.append(this.icon)),this.element.prop("multiple")&&!this.settings.multiple&&(this.settings.multiple=!0),this.settings.multiple&&!this.element.prop("multiple")&&this.element.prop("multiple",!0),this.settings.showTags&&(this.settings.showSearchUnderSelected=!0,this.pseudoElem[0].classList.add("has-tags"),this.renderTagList());var l=this.element.attr("data-source");l&&"source"!==l&&(this.settings.source=l);var c=this.element.attr("data-maxselected");function d(t,e){switch(t){case!e||"true":return"all";case!!e&&"false":return"none";default:return on.indexOf(t)>-1?t:"none"}}c&&!isNaN(c)&&(this.settings.maxSelected=parseInt(c,10)),void 0!==this.settings.moveSelectedToTop&&(this.settings.moveSelected=this.settings.moveSelectedToTop);var h=this.element.attr("data-move-selected");this.settings.moveSelected=h?d(h,!0):d(this.settings.moveSelected),this.settings.reloadSourceOnOpen&&(this.settings.reload="open",delete this.settings.reloadSourceOnOpen);var u=this.element.attr("data-close-on-select");u&&!this.settings.closeOnSelect&&(this.settings.closeOnSelect="true"===u);var p=this.element.attr("data-no-search");p&&!this.settings.noSearch&&(this.settings.noSearch="true"===p);for(var m=["-xs","-sm","-mm","-md","-lg"],v=this.element.attr("class"),b=null,y=0;y<m.length;y++)b=m[y],v.match(b)&&this.pseudoElem.addClass("dropdown".concat(b));return this.dataset=[],this.listfilter=new We({filterMode:this.settings.filterMode}),this.setListIcon(),this.setDisplayedValues(),this.setInitial(),this.setWidth(),setTimeout((function(){t.toggleTooltip()}),0),this.element.triggerHandler("rendered"),this.handleEvents()},renderTagList:function(){var t=this;function e(e){t.settings.dismissHandler&&t.settings.dismissHandler(e);var i=t.selectedOptions.filter((function(t){return g.stripHTML(t.value)===e.settings.value}));i.length&&t.deselect(i[0]),t.tagListAPI.element.classList[t.selectedOptions.length?"remove":"add"]("empty"),t.isOpen()&&t.updateList()}var i=this.toTagData();i.forEach((function(i){i.dismissHandler=e,t.settings.clickHandler&&(i.clickHandler=t.settings.clickHandler),t.isDisabled()&&(i.disabled=!0)}));var n=this.pseudoElem.children("span")[0];this.tagListAPI?this.tagListAPI.updated({tags:i}):(this.tagListAPI=new tn(n,{tags:i}),n.classList.add("tag-list")),this.tagListAPI.element.classList[this.selectedOptions.length?"remove":"add"]("empty"),isNaN(this.settings.tagListMaxHeight)||(n.classList.add("scrollable"),n.style.maxHeight=i.length?"".concat(this.settings.tagListMaxHeight,"px"):"");var s=n.scrollHeight>n.clientHeight;this.pseudoElem[0].classList[s?"add":"remove"]("has-scrollbar"),this.isOpen()&&this.position()},renderPseudoElemLabel:function(){var t,e=this.element.prev(),i=this.element.parent().find(".error-message .message-text"),n=null===(t=this.pseudoElem)||void 0===t||null==t?void 0:t.hasClass("error"),s=i.text(),a="";this.selectedOptions.forEach((function(t){a.length&&(a+=", "),a+=$(t).text().trim()})),this.pseudoElem.attr({"aria-label":"".concat(e.text(),", ").concat(n?s:a)})},inputTimer:function(){if(this.inputTimeout)return!1;var t=this;return this.inputTimeout=setTimeout((function(){clearTimeout(t.inputTimeout),t.inputTimeout=null}),200),!0},setWidth:function(){var t=this.element[0].style;t.width&&(this.pseudoElem[0].style.width=t.width),"absolute"===t.position&&(this.pseudoElem[0].style.position="absolute",this.pseudoElem[0].style.left=t.left,this.pseudoElem[0].style.top=t.top,this.pseudoElem[0].style.bottom=t.bottom,this.pseudoElem[0].style.right=t.right)},setItemIcon:function(t){if(!t.icon)return t.isIcon=!1,t.html="",void this.listIcon.items.push(t);"object"===l(t.icon)&&(t.obj=t.icon,t.icon=t.icon.icon,t.obj.color?0===t.obj.color.indexOf("#")?t.specColor=t.obj.color:t.classList=" ".concat(t.obj.color):t.obj.class&&(0===t.obj.class.indexOf("#")?t.specColor=t.obj.class:t.classList=" ".concat(t.obj.class)),t.obj.colorOver?0===t.obj.colorOver.indexOf("#")?t.specColorOver=t.obj.colorOver:t.classListOver=" ".concat(t.obj.colorOver):t.obj.classOver&&(0===t.obj.classOver.indexOf("#")?t.specColorOver=t.obj.classOver:t.classListOver=" ".concat(t.obj.classOver))),t.isIcon=t.icon&&t.icon.length,t.classList&&t.classList.length&&(t.isClassList=!0),t.classListOver&&t.classListOver.length&&(t.isClassListOver=!0),"swatch"===t.icon?(t.isSwatch=!0,t.html='<span class="swatch '.concat(t.isClassList?t.classList:"",'"></span>')):t.html=$.createIcon({icon:t.isIcon?t.icon:"",class:"listoption-icon ".concat(t.isClassList?" ".concat(t.classList):"")}),this.listIcon.items.push(t)},setListIcon:function(){var t=this,e=!t.settings.multiple&&t.element.find("[data-icon]").length>0,i=e?this.element.find("option"):[];if(t.listIcon={hasIcons:e,items:[]},e){var n=0;i.each((function(e,i){var s=$(this).attr("data-icon"),a=null;"string"==typeof s&&s.length&&"{}"!==s&&((a=0!==s.indexOf("{")?s:$.fn.parseOptions(this,"data-icon"))instanceof Object&&""===a.icon||(t.setItemIcon({html:"",icon:a,value:i.value||null}),t.listIcon.items[e]&&t.listIcon.items[e].isIcon&&n++))})),e=n>0}if(e){var s=$.createIconElement({icon:"",class:"listoption-icon"});t.pseudoElem.prepend(s),t.listIcon.pseudoElemIcon=s,t.listIcon.idx=-1}t.listIcon.hasIcons=e},toggleTooltip:function(){this.overflowed?this.setTooltip():this.tooltipApi&&this.removeTooltip()},setTooltip:function(){var t=this,e=$(this.element[0].selectedOptions),i=this.getOptionText(e);this.tooltipApi=this.pseudoElem.find("span").tooltip({content:g.escapeHTML(i),trigger:this.isMobile()?"immediate":"hover"}).on("blur.dropdowntooltip",(function(){t.removeTooltip()})).data("tooltip")},removeTooltip:function(){this.tooltipApi&&(this.tooltipApi.element.off("blur.dropdowntooltip"),this.tooltipApi.destroy(),this.tooltipApi=null)},setItemIconOverColor:function(t){var e=this;if(e.listIcon.hasIcons){var i=t?t.find(".listoption-icon"):null;e.list.find("li").each((function(n){var s=$(this),a=s.find(".listoption-icon"),o=e.listIcon.items[n];o&&(s.is(".is-focused")&&o.isClassListOver&&a.removeClass(o.classListOver).addClass(o.classList),i&&s.is(t)&&o.isClassListOver&&(i.removeClass(o.classList),i.addClass(o.classListOver)))}))}},updateItemIcon:function(t){var e=this;if(e.listIcon.hasIcons){var i=e.pseudoElem.find(".listoption-icon"),n=t.index(),s=e.listIcon.items[n];if(i.each((function(t,e){return e.remove()})),!s)return e.listIcon.pseudoElemIcon="",void(e.listIcon.idx=n);var a=$.createIconElement({icon:s.icon,class:["listoption-icon"]});e.pseudoElem.prepend(a),e.listIcon.pseudoElemIcon=a,s.isClassList&&a.addClass(s.classList)}},updateList:function(t){var e=this,i=this.settings,n=e.isMobile(),s=void 0!==e.list&&null!==e.list&&e.list.length>0,a="",o="",r=0,l=this.element.find("optgroup").length,c="",d=!0===this.settings.multiple,h="".concat(this.settings.moveSelected),u=!0===this.settings.showSelectAll,p={all:H.translate("All"),selected:H.translate("Selection"),labelText:e.isInlineLabel?e.inlineLabelText.text():this.label.text()};p.all="string"==typeof i.allTextString&&""!==i.allTextString?e.settings.allTextString:"".concat(p.all),p.selected="string"==typeof i.selectedTextString&&""!==i.selectedTextString?e.settings.selectedTextString:"".concat(p.selected);var g=this.settings.extraListWrapper?'<div class="dropdown-list-wrapper">':"",f=this.settings.extraListWrapper?"</div>":"",m=E.getAttribute(this,"id",this.settings.attributes),v="".concat(m?"".concat(m,"-"):"","dropdown-list"),b="".concat(m?"".concat(m,"-"):"","listbox"),y=' role="listbox"';this.element[0].classList.contains("text-align-reverse")?c=" text-align-reverse":this.element[0].classList.contains("text-align-center")&&(c=" text-align-center"),s?(this.list.attr("id",v),this.listUl.attr("id","".concat(b))):(a='<div class="dropdown-list'.concat(c).concat(n?" mobile":"").concat(this.settings.multiple?" multiple":"",'" id="').concat(v,'" ').concat(this.settings.multiple?'aria-multiselectable="true"':"",'>\n        <label for="dropdown-search" class="audible">').concat(this.settings.noSearch?H.translate("PressDown"):H.translate("TypeToFilter"),'</label>\n        <input type="text" class="dropdown-search').concat(c,'" ').concat(this.settings.noSearch?'aria-readonly="true"':"",' id="dropdown-search" autocomplete="off" />\n        <span class="trigger">').concat(n?$.createIcon({icon:"close",classes:["close"]}):$.createIcon({icon:this.settings.dropdownIcon||"dropdown"}),"</span>"),this.settings.virtualScroll?a+='<div class="virtual-scroll-container">\n            <div class="ids-virtual-scroll">\n              <div class="ids-virtual-scroll-viewport">\n                <ul id="'.concat(b,'"').concat(y,' class="contents" aria-label="').concat(H.translate("Dropdown"),'"></ul>\n              </div>\n            </div>\n          </div>'):a+="".concat(g,'<ul id="').concat(b,'"').concat(y,' aria-label="').concat(H.translate("Dropdown"),'">'));var w=this.element.find("option"),C=this.element.find("optgroup"),k=function(t,e){return e.selected},T=[];"typeahead"===this.settings.reload&&(k=function(t,i){return!!e.selectedValues&&e.selectedValues.indexOf(i.value)>-1});var A=w.filter(k);if("typeahead"===this.settings.reload&&A.prop("selected",!0),d&&u&&w.length){var S=w.not("[disabled], .hidden").length===A.not("[disabled], .hidden").length;o+='<li role="none" class="dropdown-select-all-list-item'.concat(S?" is-selected":"",'">')+'<a role="option" href="#" id="dropdown-select-all-anchor" class="dropdown-select-all-anchor">'.concat(H.translate("SelectAll"),"</a>")+"</li>"}("group"===h&&(C&&C.length?C.each((function(t,e){var i=A.filter((function(){return $.contains(e,this)}));T.push(i)})):h="all"),"all"===h&&(w=w.not(A),A.length>0&&(o+=e.buildLiHeader(p.selected)),A.each((function(i){o+=e.buildLiOption(this,i,t),r++})),!l&&w.length>0&&(o+=e.buildLiHeader(p.all))),this.settings.virtualScroll?o="":w.each((function(i){var n,s=i+r,a=$(this),l=a.parent();l.is("optgroup")&&C.length&&(n=l.index(),C.index(l)>-1&&(C=C.not(l),o+=e.buildLiHeader("".concat(l.attr("label"))),"group"===h&&T[n].each((function(i){o+=e.buildLiOption(this,i,t),r++})))),"none"!==h&&a.is(":selected")||(o+=e.buildLiOption(this,s,t))})),s)?(this.listUl.html(o),e.settings.attributes&&this.listUl.find(".dropdown-option a").each((function(t){var i=$(this);E.addAttributes(i,e,e.settings.attributes,"option-".concat(t),!0)}))):(a+=this.settings.virtualScroll?"".concat(o,"</div>"):"".concat(o,"</ul>").concat(f,"</div>"),this.list=$(a),this.listUl=this.list.find("ul"),this.searchInput=this.list.find("#dropdown-search"));if("ios"!==x.os.name&&"android"!==x.os.name||this.searchInput.attr("readonly","readonly"),this.virtualScrollElem=this.listUl.closest(".virtual-scroll-container"),this.opts=w,this.listIcon.hasIcons&&""!==this.listIcon.pseudoElemIcon){this.list.addClass("has-icons");var D=this.listIcon.pseudoElemIcon.clone();D.addClass("listoption-icon"),D.appendTo(this.list)}l&&this.listUl.addClass("has-groups"),this.hasTooltips=this.listUl.find("[title]").length>0||this.element.find("[title]").length>0,this.isOpen()&&(this.position(),this.highlightOption(this.listUl.find("li:visible:not(.separator):not(.group-label):not(.is-disabled)").first()))},buildLiHeader:function(t){return'<li role="none" class="group-label" focusable="false">\n      '.concat(t,"\n    </li>")},buildLiOption:function(t,e,i){var n=this,s="",a=t.innerHTML,o=f.getAttributes(t),r=o.getNamedItem("value"),l=o.getNamedItem("title"),c=l?' title="'.concat(l.value,'"'):"",d=o.getNamedItem("data-badge"),h=o.getNamedItem("data-badge-color"),u="",p=t.selected?" is-selected":"",g=t.disabled?" is-disabled":"",m=t.className?" ".concat(t.className.value):"",v=m.indexOf("clear")>-1?' class="clear-selection"':"",b=' tabIndex="'.concat(e&&0===e?0:-1,'"'),y=" ".concat(n.getDataAttributes(o,["data-badge","data-badge-color","data-val","data-icon"]).str),w=(r&&"value"in r?r.value:a).replace(/"/g,"/quot/"),x="";if(n.listIcon.hasIcons&&n.listIcon.items[e]&&(x=n.listIcon.items[e].html)," "===y&&(y=""),d&&(u='<span class="badge '.concat(h?h.value:"azure07",'">').concat(d.value,"</span>")),m.indexOf("clear")>-1&&""===a&&(a=H.translate("ClearSelection")),i&&i.length>0){var C=n.getSearchRegex(i);a=a.replace(C,'<span class="dropdown-highlight">$1</span>').trim()}return n.listIcon.hasIcons&&n.listIcon.items[e]&&n.listIcon.items[e].isSwatch&&(m+=" is-swatch"),s+='<li class="dropdown-option'.concat(p).concat(g).concat(m,'" data-val="').concat(w,'" ').concat(y).concat(c,' role="none">\n      <a id="list-option-').concat(e,'" href="#" ').concat(v,' role="option" ').concat(p?'aria-selected="true"':'aria-selected="false"').concat(b,">").concat(x).concat(a).concat(u,"</a></li>")},setDisplayedValues:function(){var t=$(this.element[0].selectedOptions),e=this.getOptionText(t);if(t.hasClass("clear")&&(e=""),this.settings.showTags&&this.tagListAPI)this.renderTagList();else{if(this.settings.empty&&0===t.length){var i=this.pseudoElem.find("span").first();return f.html(i,'<span class="audible">'.concat(this.label.text()," </span>"),"<div><p><span><ul><li><a><abbr><b><i><kbd><small><strong><sub><svg><use><br>"),i=$("#".concat(this.element.attr("id"))).next().find("span").first(),f.html(i,'<span class="audible">'.concat(this.label.text()," </span>"),"<div><p><span><ul><li><a><abbr><b><i><kbd><small><strong><sub><svg><use><br>"),this.setPlaceholder(e),void this.scrollRelease(this.listUl)}var n=this.element.attr("maxlength");n&&(e=e.substr(0,n)),e=e.trim();var s=this.pseudoElem.find("span");s.length>0&&(s[0].innerHTML='<span class="audible">'.concat(this.label.text()," </span>").concat(g.escapeHTML(e))),this.settings.showSearchUnderSelected||this.setPlaceholder(e)}this.previousActiveDescendant=t.first().val(),this.renderPseudoElemLabel(),this.updateItemIcon(t),this.setBadge(t)},copyClass:function(t,e,i){t.hasClass(i)&&e.addClass(i)},setInitial:function(){this.element.is(":disabled")&&this.disable(),this.element.is("[readonly]")&&this.readonly(),this.isHidden&&(this.pseudoElem.hide().prev("label").hide(),this.pseudoElem.next(".icon").hide())},setPlaceholder:function(t){if(this.placeholder=this.placeholder||{text:this.element.attr("placeholder")},this.placeholder.text){var e="string"!=typeof t||"string"==typeof t&&""===t;this.placeholder.elem=this.placeholder.elem||this.pseudoElem.find("span:not(.audible)"),this.placeholder.elem.attr("data-placeholder-text",e?this.placeholder.text:"")}else delete this.placeholder},ignoreKeys:function(t,e){var i=e.which;if(e.altKey&&38===i||i>111&&i<124&&!this.settings.onKeyDown)return e.stopPropagation(),e.preventDefault(),!1;if(!t.is(":disabled")&&!t.hasClass("is-readonly")){if(e.ctrlKey)if(this.settings.onKeyDown)if(!1===this.settings.onKeyDown(e))return e.stopPropagation(),e.preventDefault(),!1;return!0}},handleSearchEvents:function(){var t=this;this.filterTerm||(this.searchKeyMode=!1);var e,i=this.searchInput[0].value;"ie"===x.browser.name&&"11"===x.browser.version||(e=String.fromCodePoint(8)),"."!==i&&i!==e||(this.searchInput[0].value=""),this.searchInput.on("keydown.".concat(an),(function(e){var i=$(t);return t.handleKeyDown(i,e)})).on("input.".concat(an),(function(e){t.isFiltering=!0,t.handleAutoComplete(e)}))},filterList:function(t){var e,i=this,n=!1;this.settings.source&&"typeahead"===this.settings.reload&&(n=!0,this.callSource());var s=this,a=!1;null===(e=s.listUl)||void 0===e||null==e||e.find(".no-results").remove();var o,r=$(".dropdown-option",this.listUl),l=$(".group-label",this.listUl);if(r.length&&this.list&&(!this.list||this.list.length)){t||(t="");var c=function(){if(o=!n&&t&&t.length?i.listfilter.filter(r,t):r,i.list.addClass("search-mode"),i.list.find(".trigger").find(".icon").attr("class","icon search").changeIcon("search"),i.searchInput.removeAttr("aria-activedescendant"),i.unhighlightOptions(),o&&(o.length||t)){if(o.removeClass("hidden"),r.not(o).add(l).addClass("hidden"),i.filteredItems=r.filter(o),i.filteredItems.each((function(e){var i=$(this);i.children("a").attr("tabindex",0===e?"0":"-1"),a||(s.highlightOption(i),a=!0),s.highlightSearchTerm(i,t)})),l.each((function(){var t=$(this).nextUntil(".group-label, .selector").not(".hidden");(s.settings.showEmptyGroupHeaders||t.length)&&$(this).removeClass("hidden")})),0===o.length){var e='<li class="dropdown-option is-disabled is-placeholder no-results" role="presentation"><svg class="listoption-icon icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-search"></use></svg><span>'.concat(H.translate(i.settings.noResultsTextString),"</span></li>");s.listUl.append(e)}t="";var c=document.body.scrollHeight;window.scrollY+window.innerHeight>=c?s.position():s.shrinkTop(s)}else i.resetList()};if(this.settings.virtualScroll)return $(this.virtualScroller.element).one("afterrendered",(function(){r=$(".dropdown-option",i.listUl),c()})),this.virtualScroller.data=this.listfilter.filter(this.opts,t),void(r=$(".dropdown-option",this.listUl));c()}},shrinkTop:function(t){if(t.list&&t.list[0].classList.contains("is-ontop")){var e=parseInt(t.list[0].offsetHeight,10),i=parseInt(t.searchInput[0].offsetHeight,10);t.list[0].style.top="".concat(t.wrapper[0].offsetParent.offsetTop+(e-i)+4,"px")}},resetList:function(){if(this.settings.source&&"typeahead"===this.settings.reload)return this.filterTerm="",this.searchKeyMode=!1,void this.callSource(null,!0);if(this.list&&(!this.list||this.list.length)){this.list.removeClass("search-mode");var t=this.listUl.find("li");t.removeAttr("style").each((function(){var t=$(this).children("a"),e=$(this),i=g.escapeHTML(t.text()),n=0!==e.children("a").find("svg").length?(new XMLSerializer).serializeToString(e.children("a").find("svg")[0]):"",s=e.children("a").find(".swatch"),a=0!==s.length?s[0].outerHTML:"";t[0]&&(t[0].innerHTML=a+n+i)})),this.list.hasClass("is-ontop")?this.list[0].style.top="".concat(this.pseudoElem.offset().top-this.list.height()+this.pseudoElem.outerHeight()-2,"px"):this.list[0].style.top="",delete this.filteredItems,this.settings.multiple&&this.updateList(),this.settings.virtualScroll&&(this.virtualScroller.data=this.opts),t.removeClass("hidden"),this.position(),2===this.list.find("svg").length&&this.list.find("svg").last().remove()}},selectBlank:function(){var t=this.element.find("option").filter((function(){return!this.value||0===$.trim(this.value).length}));t.length&&(t[0].selected=!0,t[0].setAttribute("selected",!0),this.element.triggerHandler("updated"),this.element.trigger("change").triggerHandler("selected"))},handleKeyDown:function(t,e){var i,n=this,s=this.element[0].selectedIndex||-1,a=this.element[0].options,o=e.which,r=this;if(!this.ignoreKeys(t,e))return!1;if(this.isLoading())return!0;if(r.settings.onKeyDown&&!1===r.settings.onKeyDown(e))return e.stopPropagation(),e.preventDefault(),!1;if(r.isOpen()){a=r.settings.virtualScroll?this.opts:this.listUl.find("li:visible:not(.separator):not(.group-label):not(.is-disabled)"),s=-1,$(a).each((function(t){($(this).is(".is-focused")||r.settings.virtualScroll&&$(this).is(":selected"))&&(s=t)}));var l="mac"===x.os.name;if((!l&&"Delete"===e.key||l&&"Backspace"===e.key||"Backspace"===e.key)&&this.settings.noSearch){var c=$(a[0]);this.highlightOption(c),"Backspace"===e.key&&(e.stopPropagation(),e.preventDefault())}}switch(o){case 37:case 8:case 46:if(!r.isOpen())return r.selectBlank(),e.stopPropagation(),e.preventDefault(),!1;break;case 9:return r.isOpen()?(1===r.filterTerm.length?r.selectStartsWith(r.filterTerm):!this.settings.multiple&&a.length&&s>-1&&r.selectOption(this.correctValue($(a[s]))),r.closeList("tab"),this.activate()):1===r.filterTerm.length&&r.selectStartsWith(r.filterTerm),!0;case 27:if(r.isOpen()){r.element.closest(".modal.is-visible").data("listclosed",!0);var d=r.pseudoElem?r.pseudoElem.parents("td"):null;return r.closeList("cancel"),r.activate(),d&&d.focus(),e.stopPropagation(),!1}break;case 32:case 13:if(r.isOpen()){if(32===o&&!0===r.searchKeyMode)break;if(e.preventDefault(),a.length&&s>-1){var h=$(a[s]);r.selectListItem(h)}}return e.stopPropagation(),!1;case 38:return!!e.shiftKey||(this.searchKeyMode=!1,this.keydownFlag=!0,s>0&&(i=$(a[s-1]),this.highlightOption(i),r.setItemIconOverColor(i),i.parent().find(".is-focused").removeClass("is-focused"),i.addClass("is-focused")),this.settings.virtualScroll&&$(this.virtualScroller.element).one("afterrendered",(function(){n.highlightOption(i)})),e.stopPropagation(),e.preventDefault(),!1);case 40:return!!e.shiftKey||(this.searchKeyMode=!1,this.keydownFlag=!0,s<a.length-1&&(i=$(a[s+1]),this.highlightOption(i),r.setItemIconOverColor(i),i.parent().find(".is-focused").removeClass("is-focused"),i.addClass("is-focused")),this.settings.virtualScroll&&$(this.virtualScroller.element).one("afterrendered",(function(){n.highlightOption(i)})),e.stopPropagation(),e.preventDefault(),!1);case 35:this.searchKeyMode=!1;var u=$(a[a.length-1]);return this.highlightOption(u),e.stopPropagation(),!1;case 36:this.searchKeyMode=!1;var p=$(a[0]);return this.highlightOption(p),e.stopPropagation(),!1}return r.isOpen()?!0!==this.settings.noSearch||r.isControl(o)||this.handleAutoComplete(e):r.isControl(o)||this.settings.source||this.settings.noSearch||(r.isFiltering=!0,r.filterTerm=$.actualChar(e),r.searchInput&&r.searchInput.length&&r.searchInput.val($.actualChar(e)),r.toggle()),this.searchKeyMode=!0,r.searchInput&&r.searchInput.attr("aria-activedescendant",""),!0},handlePseudoElemKeydown:function(t){var e=$(t.target),i=t.key;if(e.is(".is-readonly"))return!0;var n="Esc"===i||"Escape"===i;if(["Alt","Shift","Control","Meta"].indexOf(i)>-1)return!1;if(!this.ignoreKeys(e,t))return!1;if(this.settings.onKeyDown&&!1===this.settings.onKeyDown(t))return t.stopPropagation(),t.preventDefault(),!1;if(["ArrowDown","ArrowUp","Down","Up","Enter","Spacebar"," "].indexOf(i)>-1)return this.isOpen()||this.open(),this.settings.noSearch&&!t.ctrlKey&&this.handleKeyDown(e,t),!1;var s="mac"===x.os.name;return!s&&"Delete"===i||s&&"Backspace"===i||"Backspace"===i&&this.settings.noSearch?(this.selectBlank(),"Backspace"===i&&(t.stopPropagation(),t.preventDefault()),!0):n||"Tab"===i?!(1===this.filterTerm.length||this.settings.noSearch&&this.isOpen())||this.handleKeyDown(e,t):(this.handleAutoComplete(t),!0)},handleAutoComplete:function(t){var e=this;if(!this.isLoading()){var i=this;if(this.timer&&clearTimeout(this.timer),"input"===t.type)this.filterTerm=i.settings.noSearch?this.searchInput.val().toLowerCase():this.searchInput.val();else{if(this.filterTerm+=$.actualChar(t).toLowerCase(),""===this.filterTerm)return;t.key===this.filterTerm||t.key.toLowerCase()!==this.filterTerm||i.settings.noSearch||(this.filterTerm=t.key)}this.timer=setTimeout((function(){var t;if(i.settings.noSearch&&""!==i.filterTerm)return i.isOpen()?i.highlightStartsWith(i.filterTerm):i.selectStartsWith(i.filterTerm),void(i.filterTerm="");if(i.searchKeyMode=!0,i.isOpen()||""===i.filterTerm){var s;if(0===(null===(t=e.list)||void 0===t||null==t?void 0:t.find("ul li.hidden").length))null===(s=e.list)||void 0===s||null==s||s.find(" > svg.listoption-icon:not(.swatch)").changeIcon("icon-empty-circle");n()}else i.open(n)}),i.settings.delay)}function n(){""===i.filterTerm?i.resetList():i.filterList(i.filterTerm.toLowerCase())}},isControl:function(t){return t>7&&t<48||t>90&&t<94||t>111&&t<146},activate:function(t){var e=this,i=this.pseudoElem;t&&(i=this.searchInput),this.currentlyScrolledPos&&(this.listUl.scrollTop(this.currentlyScrolledPos),delete this.currentlyScrolledPos),t&&i&&(i.hasClass("is-readonly")||!0===i.prop("readonly"))||(e.isMobile()||e.filterTerm||!i||(i[0].setSelectionRange?i[0].setSelectionRange(0,i[0].value.length):"INPUT"===i[0].tagName&&i[0].select()),i&&"INPUT"!==document.activeElement.tagName&&i[0].focus())},getSearchRegex:function(t){var e;try{e=new RegExp("(".concat(M.escapeRegExp(t),")"),"i")}catch(t){e=/[\s\S]*/i}return e},getOptionText:function(t){var e="";return t||(t=this.element.find("option:selected")),t.each((function(){e.length>0&&(e+="".concat(H.currentLocale.data.punctuation.comma," ")),e+=$(this).text().trim()})),e},open:function(t){var e=this;function i(){e.isMobile()&&$(".tooltip:not(.is-hidden)").hide(),e.updateList(),e.openList(),t&&"function"==typeof t&&t.call(this)}this.inputTimer()&&(this.element.is(":disabled")||this.pseudoElem.hasClass("is-disabled")||this.pseudoElem.hasClass("is-readonly")||e.callSource(i)||i())},openList:function(){var t,e=this,i=this.previousActiveDescendant?this.list.find('.dropdown-option[data-val="'.concat(this.previousActiveDescendant.replace(/"/g,"/quot/"),'"]')):this.list.find(".is-selected"),n=this;if(this.touchPrevented=!1,this.scrollLock(this.listUl),$("select").each((function(){var t=$(this).data();t.dropdown&&t.dropdown.closeList("cancel")})),$(".popupmenu.is-open").filter((function(){return 0===$(this).parents(".popupmenu").length})).each((function(){var t=$(this).data("trigger");if(t&&t.length){var e=$(this).data("trigger").data("popupmenu");e&&"function"==typeof e.close&&e.close()}})),this.isOpen()||this.list.appendTo("body"),this.list.show(),this.list.attr("data-element-id",this.element.attr("id")),this.isShortField&&this.list[0].classList.add("dropdown-short"),n.settings.attributes){E.addAttributes(this.list.find("input"),this,this.settings.attributes,"search",!0),this.list.find("label").attr("for",this.list.find("input").attr("id")),E.addAttributes(this.list.find("label"),this,this.settings.attributes,"search-label"),E.addAttributes(this.list.find(".trigger svg"),this,this.settings.attributes,"trigger",!0),E.addAttributes(this.listUl,this,this.settings.attributes,"listbox",!0),E.addAttributes(this.list,this,this.settings.attributes,"list"),this.list.find(".dropdown-option a").each((function(t){var e=$(this);E.addAttributes(e,n,n.settings.attributes,"option-".concat(t),!0)}));var s=E.getAttribute(this,"id",this.settings.attributes);this.pseudoElem.attr("aria-controls",s?"".concat(s,"-listbox"):"dropdown-list"),this.pseudoElem.attr("aria-expanded")}if(this.pseudoElem.attr("aria-expanded","true").addClass("is-open"),this.searchInput.attr("aria-activedescendant",i.children("a").attr("id")),this.searchInput.prop("readonly",this.settings.noSearch),this.settings.showSearchUnderSelected&&this.list.find(".trigger").find(".icon").attr("class","icon search").changeIcon("search"),this.isInGrid=1===this.pseudoElem.closest(".datagrid-row").length,this.pseudoElem.parent().hasClass("is-inline")&&(this.isInGrid=!1),this.isInGrid){var a=this.pseudoElem.closest(".datagrid").attr("class").replace("datagrid","");this.list.addClass("datagrid-dropdown-list ".concat(a))}1===this.pseudoElem.closest(".datagrid-filter-wrapper").length&&this.list.addClass("datagrid-filter-dropdown");var o=this.settings.cssClass;if(o&&"string"==typeof o&&this.list.addClass(o),this.position(),this.list.hasClass("datagrid-filter-dropdown")&&null!==document.querySelector(".datagrid-container")){var r=this.dropdownParent.closest(".datagrid-container").getBoundingClientRect(),l=document.querySelector(".datagrid-filter-dropdown").getBoundingClientRect(),c=document.querySelector(this.settings.appendTo).getBoundingClientRect().right-r.right,d=document.querySelector(".datagrid-filter-dropdown").getBoundingClientRect(),h=!1;H.isRTL()&&d.left+20<0&&(h=!0),d.right+20>(window.innerWidth||document.documentElement.clientWidth)&&(h=!0),r.right<l.right&&h&&(this.list[0].style.right="".concat(c,"px"),this.list[0].style.left=""),H.isRTL()&&l.left<0&&(this.list[0].style.left="".concat(r.left,"px"))}if(this.settings.virtualScroll){var u=-1,p=null;this.opts.toArray().filter((function(t,e){return t.selected?(u=e,p=t,t):null})),this.virtualScroller=new sn(this.virtualScrollElem,{height:304,itemHeight:32,itemTemplate:function(t,e){var s=n.buildLiOption(e,t);return e.selected&&(p=e,i=$(p)),s=n.highlightSearchTerm($(s),n.filterTerm)},data:this.opts}),setTimeout((function(){$(e.virtualScroller.element).on("afterrendered",(function(){e.highlightOption($(p))})),e.virtualScroller.scrollTo(u)}))}if(this.setListWidth(),(this.settings.maxWidth||this.settings.width)&&this.position(),"string"==typeof this.filterTerm&&this.filterTerm.length>0)this.searchInput.val(this.filterTerm);else if(!this.settings.showSearchUnderSelected){var g=$(this.selectedOptions),f=this.getOptionText(g);this.searchInput.val(f)}var m=this.settings.multiple;if(this.highlightOption(i,m),this.settings.multiple&&this.listUl.find(".is-selected").length>0&&(this.highlightOption(this.listUl.find(".dropdown-option").eq(0)),setTimeout((function(){n.listUl.scrollTop(0)}),0)),this.handleSearchEvents(),this.activate(!0),this.element.trigger("listopened"),this.isMobile()&&n.searchInput.on("keypress.dropdown",(function(t){13===t.which&&n.closeList("select")})),n.list.removeClass("dropdown-tall").on("touchend.list click.list","li",(function(t){if(!n.touchPrevented){var e=n.selectListItem($(this));t.preventDefault(),e&&t.stopPropagation()}})).on("mouseenter.list","li",(function(){n.keydownFlag?n.keydownFlag=!1:n.highlightOption($(this),!0)})),n.list.on("contextmenu.dropdown",(function(t){n.element.trigger("listcontextmenu",t)})),this.hasTooltips){var v=function(){n.timer&&n.timer.destroy&&n.timer.destroy(!0)},b=function(t,e){$(t).tooltip({trigger:"immediate",content:e}),$(t).focus()},y=function(t){var e;v(),$("#tooltip").hide(),null===(e=$("#tooltip").data("tooltip"))||void 0===e||null==e||e.destroy()};n.list.on("mouseover.tooltip","li[title], li[data-title]",(function(t){var e;e=t.currentTarget,v(),n.timer=new X({duration:50,timeoutCallback:function(){var t=e.getAttribute("title");if(!t)return t=e.getAttribute("data-title"),void b(e,t);e.setAttribute("data-title",t),e.removeAttribute("title"),b(e,t)}}),J.register(n.timer)})).on("mouseleave.tooltip, mousewheel.tooltip, click.tooltip","li[title], li[data-title]",(function(t){y(t.currentTarget)}))}function w(t){var e=$(t.target);if(n.touchPrevented||n.isDropdownElement(e)&&!e.is(".icon"))return t.preventDefault(),void(n.touchPrevented=!1);n.closeList("cancel")}this.scrollRelease(this.listUl),setTimeout((function(){n.list.addClass("is-closable")}),100),$(document).on("touchstart.dropdown",(function(e){n.touchPrevented=!1,t={x:e.originalEvent.touches[0].pageX,y:e.originalEvent.touches[0].pageY},$(document).on("touchmove.dropdown",(function(e){var i=e.originalEvent.touches[0].pageX,s=e.originalEvent.touches[0].pageY;(i>=t.x+10||i<=t.x-10||s>=t.y+10||s<=t.y-10)&&(n.touchPrevented=!0)}))})).on("touchend.dropdown touchcancel.dropdown",(function(t){if($(document).off("touchmove.dropdown"),t.preventDefault(),n.touchPrevented)return t.stopPropagation(),!1;w(t)})).on("click.dropdown",w);var x=$(".modal.is-visible .modal-body-wrapper"),C=n.element.parents(".scrollable, .scrollable-x, .scrollable-y, .card-content");C.length&&(x=C),x.length&&(this.parentScrollableArea=x,this.parentScrollableArea.on("scroll.dropdown",(function(t){e.scrollDocument(t)}))),$("body").on("resize.dropdown",(function(){n.position(),void 0===window.orientation&&n.closeList("cancel")})).on("keydown.dropdown",(function(t){var i=$(t.target);!e.isOpen()||i.hasClass("dropdown-search")||i.hasClass("dropdown")||e.handleKeyDown(i,t)})),void 0!==window.orientation&&$(window).on("orientationchange.dropdown",(function(){n.closeList("cancel")}))},isDropdownElement:function(t){return t.closest(".dropdown, .multiselect").length>0||t.closest(".dropdown-list").length>0||!0===self.touchmove},scrollDocument:function(t){var e=this,i=$("*:focus");e.touchPrevented||e.isDropdownElement($(t.target))||i.is(".timepicker")?e.touchPrevented=!1:e.closeList("cancel")},position:function(){var t=this,e={parentXAlignment:"left",placement:"bottom",strategies:["flip","shrink-y","nudge"]};this.list[0].removeAttribute("style"),this.listUl[0].removeAttribute("style");var i=this.pseudoElem;this.isInGrid&&(i=this.element.closest(".datagrid-cell-wrapper")),this.widthTargetEnabled&&(i=$(this.settings.widthTarget));var n=window.getComputedStyle(i[0]),s=2*parseInt(n.borderLeftWidth,10),a=Math.round(parseInt(i[0].clientWidth,10)+s);this.searchInput[0].style.cssText="width: ".concat(a,"px !important");var o=Math.round(this.list.width()),r=this.widthTargetEnabled||o<=a;if(this.searchInput[0].style.width="",e.parent=i,e.useParentWidth=r,!this.settings.showSearchUnderSelected){var l=window.devicePixelRatio>1,c="chrome"===x.browser.name;e.y=-(parseInt(i[0].clientHeight,10)+parseInt(n.borderTopWidth,10)+parseInt(n.borderBottomWidth,10)-(!c&&l?1:0)),e.x=0}t.settings.placementOpts&&t.settings.placementOpts.x&&(e.x=t.settings.placementOpts.x),this.setListWidth(),this.list.one("afterplace.dropdown",(function(e,i){t.list[!0===i.wasFlipped?"addClass":"removeClass"]("is-ontop"),t.settings.virtualScroll||t.listUl[!0===i.wasFlipped?"prependTo":"appendTo"](t.settings.extraListWrapper?t.list.find(".dropdown-list-wrapper"):t.list);var n=window.getComputedStyle(t.list[0]),s=n.top?parseInt(n.top,10):0,a=$("html").hasClass("is-chrome"),o=0;"firefox"===x.browser.name&&(o=4);var r,l=parseInt(t.listUl[0].offsetHeight,10),c=parseInt(t.list[0].offsetHeight,10)+5,d=parseInt(t.searchInput[0].offsetHeight,10),h=parseInt(t.list[0].offsetTop,10)+parseInt(t.list[0].offsetHeight,10)>=window.innerHeight;return d<c-2*d&&l+d>=c&&(r="".concat(c-d-5,"px"),h&&(t.list[0].style.maxHeight="".concat(parseInt(c,10)-5,"px"))),i.wasFlipped&&(r="".concat(c-d-o-5,"px"),t.isShortField||(t.list[0].style.top="".concat(s,"px"))),r&&!t.settings.virtualScroll&&(t.listUl[0].style.maxHeight=r),void 0===r&&t.list[0].classList.contains("is-ontop")&&(r="".concat(c-d-o-7),t.list[0].style.top="".concat(s-r,"px")),a||"bottom"!==i.placement||(t.list[0].style.top="".concat(s-1,"px")),i})).place(e),this.list.data("place").place(e)},selectListItem:function(t){var e=t.closest("li");if(e.length&&(t=e),t.is(".separator, .group-label"))return!1;t.is(".dropdown-select-all-anchor")&&(t=t.parent());var i=t[0].querySelector("a");if(this.settings.multiple&&t[0].classList.contains("dropdown-select-all-list-item")){var n=!t[0].classList.contains("is-selected");return t[0].classList[n?"add":"remove"]("is-selected"),n?i.setAttribute("aria-selected","true"):i.setAttribute("aria-selected","false"),this.selectAll(n),!0}var s=this.correctValue(t);if(s.is(":disabled"))return!1;if(this.selectOption(s),this.settings.closeOnSelect&&this.closeList("select"),this.isMobile())return!0;if(this.activate(!this.settings.closeOnSelect),this.settings.showSelectAll&&this.list){var a=this.element.find("option"),o=a.filter(":selected");a.length>o.length?this.list.find(".dropdown-select-all-list-item").removeClass("is-selected").attr("aria-selected","false"):this.list.find(".dropdown-select-all-list-item").addClass("is-selected").attr("aria-selected","true")}return!0},correctValue:function(t){var e=t.attr("data-val")||t.attr("value");if(!e)return t;e=e.replace(/"/g,"/quot/");var i=this.element.find('option[value="'.concat(e,'"]'));return(0===i.length||i.length>1)&&(i=this.element.find("option").filter((function(){var t=$(this),i=t.attr("value");return t.text()===e||i&&i.replace(/"/g,"/quot/")===e}))),i},close:function(){return this.closeList("cancel")},closeList:function(t){this.list&&this.list.is(":visible")&&this.isListClosable()&&this.inputTimer()&&(this.list.off("contextmenu.dropdown"),this.touchmove&&(this.touchmove=!1),this.filterTerm="",this.searchKeyMode=!1,this.setDisplayedValues(),delete this.filteredItems,this.tagListAPI&&this.scrollTagList(),this.searchInput.off(["input.".concat(an),"keydown.".concat(an)].join(" ")),this.list.off(["click.".concat(an),"touchmove.".concat(an),"touchend.".concat(an),"touchcancel.".concat(an),"mousewheel.".concat(an),"mouseenter.".concat(an)].join(" ")).remove(),this.pseudoElem.removeClass("is-open").attr("aria-expanded","false"),this.searchInput.removeAttr("aria-activedescendant"),$(document).off(["click.".concat(an),"scroll.".concat(an),"touchstart.".concat(an),"touchmove.".concat(an),"touchend.".concat(an),"touchcancel.".concat(an)].join(" ")),this.parentScrollableArea&&(this.parentScrollableArea.off("scroll.dropdown"),delete this.parentScrollableArea),$("body").off("resize.dropdown"),$(window).off("orientationchange.dropdown"),this.element.trigger("listclosed",t),"click"!==t&&this.activate(),this.toggleTooltip(),this.list=null,this.searchInput=null,this.listUl=null)},get widthTargetEnabled(){return"parent"===this.settings.width&&"string"==typeof this.settings.widthTarget},setListWidth:function(){if(this.settings.maxWidth){var t="number"==typeof this.settings.maxWidth?"".concat(this.settings.maxWidth,"px"):"auto";this.list.css("max-width",t)}if(this.settings.width){var e="number"==typeof this.settings.width?"".concat(this.settings.width,"px"):"auto";if(this.widthTargetEnabled){var i=document.querySelector(this.settings.widthTarget);i&&(e="".concat(i.clientWidth,"px"))}this.list.css("width",e)}},scrollToOption:function(t){t&&0!==t.length&&(this.scrollLock(this.listUl),this.settings.virtualScroll?(t[0].closest(".ids-virtual-scroll").scrollTop=t[0].offsetTop-t[0].closest(".ids-virtual-scroll").offsetTop,t[0].scrollIntoView({block:"center"})):t[0].parentNode.scrollTop=t[0].offsetTop-t[0].parentNode.offsetTop,t.focus(),this.searchInput.focus(),this.scrollRelease(this.listUl))},scrollLock:function(){var t;null===(t=this.parentScrollableArea)||void 0===t||null==t||t.off("scroll.dropdown")},scrollRelease:function(){var t=this;setTimeout((function(){var e;null===(e=t.parentScrollableArea)||void 0===e||null==e||e.off("scroll.dropdown").on("scroll.dropdown",(function(e){t.scrollDocument(e)}))}),"ios"===x.os.name?800:400)},scrollTagList:function(){var t=this;this.tagListAPI&&setTimeout((function(){t.isFocused||(t.tagListAPI.element.scrollTop=0)}),5)},handleBlur:function(){return this.closeList("cancel"),!0},isLoading:function(){return this.element.is(".is-loading")&&this.element.is(".is-blocked")},isOpen:function(){return!!this.pseudoElem.hasClass("is-open")},toggle:function(){this.isOpen()||this.isLoading()?this.closeList("cancel"):this.open()},toggleList:function(){return j(this.toggle,this.toggleList).apply(this)},highlightOption:function(t,e){var i;this.isOpen()&&(t&&(0===t.length&&(t=this.list.find(".dropdown-option").eq(0)),t.is(".separator, .group-label")||(t.is("option")?(i=t,t=this.listUl.find('li[data-val="'.concat(i.val(),'"]'))):i=this.element.find("option").filter((function(e,i){return i.value===t.attr("data-val")})),this.setItemIconOverColor(),this.list.find(".is-focused").removeClass("is-focused").children("a").attr({tabindex:"-1"}),t.length&&i.length&&!i.hasClass(".is-disabled")&&!i.is(":disabled")&&(this.searchInput.attr("aria-activedescendant",t.children("a").attr("id")),this.setItemIconOverColor(t),t.addClass("is-focused").children("a").attr({tabindex:"0"}),e&&!1!==e&&void 0!==e||this.scrollToOption(t)))))},highlightSearchTerm:function(t,e){if(!t)return"";if(!e)return t[0].outerHTML;var i=t.children("a"),n=this.getSearchRegex(e),s=t.text();s=(s=(s=(s=(s=(s=(s=(s=(s=(s=(s=g.escapeHTML(s)).replace(/&lt;/g,"&#16;")).replace(/&gt;/g,"&#17;")).replace(/&apos;/g,"&#18;")).replace(/&quot;/g,"&#19;")).replace(/&amp;/g,"&")).replace(n,'<span class="dropdown-highlight">$1</span>').trim()).replace(/&#16;/g,"&lt;")).replace(/&#17;/g,"&gt;")).replace(/&#18;/g,"&apos;")).replace(/&#19;/g,"&quot;");var a=0!==t.children("a").find("svg").length?(new XMLSerializer).serializeToString(t.children("a").find("svg")[0]):"",o=t.children("a").find(".swatch"),r=0!==o.length?o[0].outerHTML:"";return i[0]&&(i[0].innerHTML=r+a+s),t[0].outerHTML},unhighlightOptions:function(t,e){t&&t.length||(t=this.list.find(".is-selected")),this.setItemIconOverColor(),t.removeClass("is-focused").children("a").attr({tabindex:"-1"}),this.searchInput.removeAttr("aria-activedescendant"),e&&!1!==e&&void 0!==e||this.scrollToOption(t.first())},renderListItem:function(t){var e,i=t.value;i=null===(e=i)||void 0===e||null==e?void 0:e.replace(/"/g,"/quot/");var n=t.selected,s=this.listUl.find('li[data-val="'.concat(i,'"]')),a=s.children("a");if(s){if(n)return s[0].classList.add("is-selected"),void a[0].setAttribute("aria-selected","true");s[0].classList.remove("is-selected"),a[0].setAttribute("aria-selected","false")}},selectAll:function(t){var e=this,i="option:not(.is-disabled):not(:disabled)",n="li.dropdown-option:not(.separator):not(.group-label):not(.is-disabled)",s=E.getArrayFromList(this.element[0].querySelectorAll(i)),a=E.getArrayFromList(this.listUl[0].querySelectorAll(n)),o=s[s.length-1];this.settings.selectAllFilterOnly&&this.filteredItems&&(s=[],(a=$.makeArray(this.filteredItems)).forEach((function(t){var i=t.getAttribute("data-val");if(i){var n=e.element[0].querySelector('option[value="'.concat(i,'"]'));n&&s.push(n)}}))),t?(a.forEach((function(t){t.classList.add("is-selected"),t.querySelector("a").setAttribute("aria-selected","true")})),s.forEach((function(t){t.selected=!0,t.setAttribute("selected",!0)}))):(a.forEach((function(t){t.classList.remove("is-selected"),t.querySelector("a").setAttribute("aria-selected","false")})),s.forEach((function(t){t.selected=!1,t.setAttribute("selected",!1),t.removeAttribute("selected")}))),this.previousActiveDescendant=o.value||"",this.setDisplayedValues(),this.updateItemIcon(o),!this.settings.selectAllFilterOnly&&this.list[0].classList.contains("search-mode")&&this.resetList(),this.activate(!0),this.setBadge(o),this.toggleTooltip(),this.element.trigger("change").triggerHandler("selected")},selectOptions:function(t,e){Array.isArray(t)&&(t=$(t));var i=this;t.each((function(){i.selectOption($(this),e)})),i.element.trigger("change").triggerHandler("selected")},selectOption:function(t,e){var i;if(t&&t.length)if(this.scrollLock(this.listUl),t.is("li")&&(i=t,0===(t=this.element.find('option[value="'.concat(t.attr("data-val"),'"]'))).length&&(t=this.element.find("option").filter((function(){return $(this).text()===i.attr("data-val")}))),t.prop("disabled")))this.scrollRelease(this.listUl);else{var n=t.val();"string"==typeof t?(i=this.listUl.find('li[data-val="'.concat(t,'"]')),n=t,t=this.element.find('option[value="'.concat(n,'"]'))):"string"==typeof n&&(n=n.replace(/"/g,"/quot/"),i=this.listUl.find('li[data-val="'.concat(n,'"]')));var s=this.listUl.find("li").first().hasClass("is-selected"),a=!0,o=!0,r=this.selectedValues,l=!1;!t.hasClass("clear")&&i||(l=!0),l&&this.deselectAll(),Array.isArray(r)||(r=[r]);var c=r.indexOf(n)>-1;this.settings.multiple&&c&&(a=!1),a?c&&!s?o=!1:(this.select(t[0]),this.previousActiveDescendant=n,this.settings.noSearch&&this.searchMatches&&delete this.searchMatches):(this.deselect(t[0]),this.previousActiveDescendant=void 0);var d=this.listUl[0].scrollTop;d>0&&(this.currentlyScrolledPos=d),this.renderListItem(t[0]),this.setDisplayedValues(),this.updateItemIcon(t),!e&&o&&(this.element.trigger("change").triggerHandler("selected",[t,a]),this.toggleTooltip()),this.settings.multiple&&this.activate(!0),this.setBadge(t);var h=this.element.attr("id");if(x.browser.isIE11()&&h){var u=$("#".concat(h)).html(),p=$("#".concat(h)).val();u&&this.element.html(u),p&&this.element.val(p)}$(".tooltip:not(.is-hidden)").hide(),this.scrollRelease(this.listUl)}},select:function(t){t&&t instanceof HTMLOptionElement&&(this.settings.multiple&&this.selectedValues.length>=this.settings.maxSelected||(t.selected=!0))},selectValue:function(t){if("string"==typeof t){var e=this.element.find('option[value="'.concat(t,'"]'));e&&e.length&&(this.select(e[0]),this.updated())}},deselect:function(t){t&&t instanceof HTMLOptionElement&&(t.selected=!1,this.element.trigger("change"))},deselectValue:function(t){if("string"==typeof t){var e=this.element.find('option[value="'.concat(t,'"]'));e&&e.length&&(this.deselect(e[0]),this.updated())}},deselectAll:function(){var t=this;this.element.find("option").each((function(e,i){$(i).prop("selected",!1),t.deselectValue(i)}))},firstOptionStartingWith:function(t){if("string"==typeof t){var e=this.element[0],i=-1,n=0;this.searchMatches&&this.searchMatches[t]||(this.searchMatches={},this.searchMatches[t]=[]);for(var s=0;s<e.options.length;s++){if(0===e.options[s].innerText.toLowerCase().indexOf(t)){if(-1===this.searchMatches[t].indexOf(s)){this.searchMatches[t].push(s),i=s;break}n++}else;}if(-1===i){if(!this.searchMatches[t].length)return;n===this.searchMatches[t].length&&(i=this.searchMatches[t][0],this.searchMatches[t].length=1)}return e.options[i]}},selectStartsWith:function(t){this.filterTerm="";var e=this.firstOptionStartingWith(t);this.select(e),this.updated(),this.element.trigger("change")},highlightStartsWith:function(t){if(this.isOpen()){var e=this.firstOptionStartingWith(t);this.highlightOption($(e))}},setBadge:function(t){if(this.badges){var e=this.element.parent().find(".badge");0===e.length&&(this.element.parent().find(".dropdown-wrapper").append('<span class="badge">1</span>'),e=this.element.parent().find(".badge")),e.attr("class","badge ".concat(t.attr("data-badge-color")?t.attr("data-badge-color"):"azure07")).text(t.attr("data-badge"))}},callSource:function(t,e){var i=this,n="";if(e||!this.isOpen()||this.element.hasClass("search-mode")||(n=this.searchInput.val()),!this.settings.source)return!1;this.isFiltering=!1;var s=l(this.settings.source),a=function(e,s){var a="",o=i.element.val();function r(t){return t.replace(/"/g,"'")}function l(t){if(null!=t){var e="",n="",s="",l="";"string"==typeof t&&(t={value:t}),void 0!==t.value&&(t.value=r(t.value)),void 0!==t.id&&(isNaN(t.id)||(t.id="".concat(t.id)),t.id=r(t.id),e=' id="'.concat(t.id,'"')),void 0!==t.label&&t.label.length&&(t.label=r(t.label),s=t.label);var c=i.selectedValues.indexOf(o)>-1;i.settings.multiple?o.forEach((function(e){e===t.value&&(t.selected=!0)})):(t.value===o||c)&&(t.selected=!0),t.selected&&(n=" selected"),t.disabled&&(l=" disabled"),s.length||s===t.value||(s+=t.value),a+="<option".concat(e,' value="').concat(t.value,'"').concat(n).concat(l,">\n          ").concat(s,"\n        </option>")}}if(!i.isFiltering&&!E.equals(e,i.dataset)&&(i.dataset=e,!s)){i.element.empty();for(var c=0;c<e.length;c++){var d=void 0;if(e[c].group){d=e[c].options,a+='<optgroup label="'.concat(e[c].group,'">');for(var h=0;h<d.length;h++)l(d[h]);a+="</optgroup>"}else l(e[c])}i.element.append(a)}i.element.triggerHandler("complete"),i.element.trigger("requestend",[n,e]),"function"==typeof t?t():i.updateList(n)};if(i.element.triggerHandler("start"),i.element.trigger("requeststart"),"function"===s)this.settings.source(a,n,this.settings.sourceArguments);else if("object"===s){var o=$.isArray(this.settings.source)?this.settings.source:[this.settings.source];a(o)}else{var r=this.settings.source.toString();$.getJSON(r).done((function(t){a(t)})).fail((function(){a([])}))}return!0},getDataAttributes:function(t,e){if(t){"string"==typeof t&&(t=[t]);var i=e||[];"string"==typeof i&&(i=[i]);var n={obj:{},str:"",isExclude:function(t){return $.inArray(t,i)>-1}};for(var s in t)t.hasOwnProperty(s)&&(n.name="".concat(t[s].name),n.isData="data-"===n.name.substr(0,5),n.isData&&!n.isExclude(n.name)&&(n.obj[n.name]=t[s].value,n.str+=" ".concat(n.name,'="').concat(t[s].value,'"')));return{str:n.str,obj:n.obj}}},setCode:function(t){var e=this,i=function(){e.element.val(t),e.updated()};e.callSource(i)||i()},isMobile:function(){return["ios","android"].indexOf(x.os.name)>-1},isListClosable:function(){return this.list.hasClass("is-closable")},toTagData:function(){var t=[],e=this.element[0].id;return e||(e=E.uniqueId(this.element[0],this.element[0].className)),this.selectedOptions.forEach((function(i){t.push({content:i.innerText.trim(),dismissible:!0,href:"#",id:"".concat(e,"-tag-").concat(i.value),style:"secondary",value:i.value})})),t},disable:function(){this.element.prop("disabled",!0).prop("readonly",!1),this.pseudoElem.addClass("is-disabled"),this.pseudoElem.is($(document.activeElement))&&this.pseudoElem.blur(),this.pseudoElem.addClass("is-disabled").removeClass("is-readonly").removeAttr("aria-readonly").attr("tabindex","-1").prop("readonly",!1).prop("disabled",!0),this.closeList("cancel"),this.settings.showTags&&this.pseudoElem.find(".tag").addClass("is-disabled")},isDisabled:function(){return this.element.prop("disabled")},enable:function(){this.element.prop("disabled",!1).prop("readonly",!1),this.pseudoElem.prop("disabled",!1).prop("readonly",!1).attr("tabindex","0").removeClass("is-disabled").removeClass("is-readonly").removeAttr("aria-readonly"),this.settings.showTags&&this.pseudoElem.find(".tag").removeClass("is-disabled")},readonly:function(){this.element.prop("disabled",!1).prop("readonly",!0),this.pseudoElem.removeClass("is-disabled").addClass("is-readonly").attr("aria-readonly","true").attr("tabindex",this.element.attr("tabindex")||"0").prop("disabled",!1).prop("readonly",!0),this.closeList("cancel")},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.closeList("cancel"),this.pseudoElem&&this.pseudoElem.hasClass("is-open")&&this.pseudoElem.removeClass("is-open").removeAttr("aria-expanded"),this.settings.multiple&&!0===this.settings.multiple?this.element.prop("multiple",!0):this.element.prop("multiple",!1),!0===this.element.prop("readonly")?this.readonly():(this.pseudoElem.removeClass("is-readonly"),this.pseudoElem.removeAttr("aria-readonly")),this.pseudoElem[this.element.prop("disabled")?"addClass":"removeClass"]("is-disabled"),this.updateList(),this.setListIcon(),this.setDisplayedValues(),this.toggleTooltip(),this.element.trigger("has-updated"),this},destroy:function(){this.placeholder&&delete this.placeholder,this.currentlyScrolledPos&&delete this.currentlyScrolledPos,$.removeData(this.element[0],an),this.closeList("cancel"),this.pseudoElem.off().remove(),this.icon.remove(),this.wrapper.remove(),this.listfilter.destroy(),this.element.removeAttr("style"),this.element.closest("form").off("reset.dropdown"),this.element.off(),this.label.off();var t=document.body.querySelector("#dropdown-list");t&&this.isOpen()&&t.parentNode.removeChild(t)},handleEvents:function(){var t=this,e=this;function i(t){var e=$(t.target).parents(".tag").length>0,i=!1;return t.target instanceof HTMLElement&&"string"==typeof t.target.className&&(i=t.target.classList.contains("tag")),!(!i&&!e)}var n=!1;this.pseudoElem.on("keydown.dropdown",(function(e){return t.handlePseudoElemKeydown(e)})).on("click.dropdown touchstart.dropdown",(function(t){t.originalEvent.ctrlKey&&t.originalEvent.shiftKey||t.stopPropagation(),n||(e.dropdownParent=t.currentTarget,e.toggle())})).on("mouseup.dropdown touchend.dropdown",(function(t){2===t.button&&t.stopPropagation()})).on("mousedown.dropdown touchstart.dropdown",(function(t){n=i(t)})).on("touchcancel.dropdown",(function(t){i(t)||(t.stopPropagation(),e.toggle(),t.preventDefault())})),this.tagListAPI&&this.pseudoElem.on("focusout.dropdown",(function(){t.scrollTagList()})),e.element.on("activated.dropdown",(function(){e.label.trigger("click")})).on("updated.dropdown",(function(t){t.stopPropagation(),e.updated()})).on("openlist.dropdown",(function(){e.toggle()})),e.element.closest("form").on("reset.dropdown",(function(){setTimeout((function(){e.element.triggerHandler("updated")}),1)})),this.label.on("click",(function(){e.isDisabled()||e.pseudoElem.focus()}))}},$.fn.dropdown=function(t){return this.each((function(){var e=$.data(this,an);e?e.updated(t):e=$.data(this,an,new ln(this,t))}))};var cn="colorpicker",dn={themes:{light:{border:"matched-only",checkmark:{one:[1,2],two:[3,10]}},dark:{border:"matched-only",checkmark:{one:[1,2],two:[3,10]}},contrast:{border:"all",checkmark:{one:[1,3],two:[4,10]}}},customColors:!1,colors:[{label:"Slate",number:"10",value:"161618"},{label:"Slate",number:"09",value:"28282A"},{label:"Slate",number:"08",value:"3B3B3F",border:"dark"},{label:"Slate",number:"07",value:"525257",border:"dark"},{label:"Slate",number:"06",value:"6F6F76"},{label:"Slate",number:"05",value:"8E8E95"},{label:"Slate",number:"04",value:"BBBBBF"},{label:"Slate",number:"03",value:"C5C5C9"},{label:"Slate",number:"02",value:"E0E0E1"},{label:"Slate",number:"01",value:"FAFAFA",border:"light, contrast"},{label:"Amber",number:"10",value:"A14100"},{label:"Amber",number:"09",value:"BB5500"},{label:"Amber",number:"08",value:"CD6200"},{label:"Amber",number:"07",value:"DF6F00"},{label:"Amber",number:"06",value:"F98300"},{label:"Amber",number:"05",value:"FA9601"},{label:"Amber",number:"04",value:"FBAF50"},{label:"Amber",number:"03",value:"FCC888"},{label:"Amber",number:"02",value:"FDDFBD"},{label:"Amber",number:"01",value:"FEF2E5"},{label:"Ruby",number:"10",value:"6C080B"},{label:"Ruby",number:"09",value:"8D0B0E"},{label:"Ruby",number:"08",value:"A30D11"},{label:"Ruby",number:"07",value:"C31014"},{label:"Ruby",number:"06",value:"DA1217"},{label:"Ruby",number:"05",value:"DF3539"},{label:"Ruby",number:"04",value:"E66467"},{label:"Ruby",number:"03",value:"EE9496"},{label:"Ruby",number:"02",value:"F5C3C4"},{label:"Ruby",number:"01",value:"FBE7E8"},{label:"Turquoise",number:"10",value:"1F5E5E"},{label:"Turquoise",number:"09",value:"297B7B"},{label:"Turquoise",number:"08",value:"2F8D8E"},{label:"Turquoise",number:"07",value:"39A9AA"},{label:"Turquoise",number:"06",value:"40BDBE"},{label:"Turquoise",number:"05",value:"5CC6C7"},{label:"Turquoise",number:"04",value:"82D4D4"},{label:"Turquoise",number:"03",value:"A8E1E1"},{label:"Turquoise",number:"02",value:"CFEEEE"},{label:"Turquoise",number:"01",value:"ECF8F8"},{label:"Emerald",number:"10",value:"156138"},{label:"Emerald",number:"09",value:"1C7F49"},{label:"Emerald",number:"08",value:"1F9254"},{label:"Emerald",number:"07",value:"25AF65"},{label:"Emerald",number:"06",value:"2AC371"},{label:"Emerald",number:"05",value:"4DCC86"},{label:"Emerald",number:"04",value:"78D8A3"},{label:"Emerald",number:"03",value:"A1E4BF"},{label:"Emerald",number:"02",value:"CBEFDC"},{label:"Emerald",number:"01",value:"EBF9F1"},{label:"Amethyst",number:"10",value:"3B1470"},{label:"Amethyst",number:"09",value:"4E1A91"},{label:"Amethyst",number:"08",value:"591DA8"},{label:"Amethyst",number:"07",value:"6C23C9"},{label:"Amethyst",number:"06",value:"7928E1"},{label:"Amethyst",number:"05",value:"8D4BE5"},{label:"Amethyst",number:"04",value:"A876EB"},{label:"Amethyst",number:"03",value:"C2A1F1"},{label:"Amethyst",number:"02",value:"DDCBF7"},{label:"Amethyst",number:"01",value:"F1EBFC"},{label:"Azure",number:"10",value:"003876"},{label:"Azure",number:"09",value:"004A99"},{label:"Azure",number:"08",value:"0054B1"},{label:"Azure",number:"07",value:"0066D4"},{label:"Azure",number:"06",value:"0072ED"},{label:"Azure",number:"05",value:"1C86EF"},{label:"Azure",number:"04",value:"55A3F3"},{label:"Azure",number:"03",value:"8ABFF7"},{label:"Azure",number:"02",value:"BEDCFA"},{label:"Azure",number:"01",value:"E6F1FD"}],placeIn:null,showLabel:!1,editable:!0,disabled:!1,uppercase:!0,colorOnly:!1,clearable:!0,clearableText:null,popupmenuSettings:{}};function hn(t,e){this.settings=E.mergeSettings(t,e,dn),e&&e.colors&&(this.settings.colors=e.colors),this.element=$(t),this.init()}hn.prototype={init:function(){this.isIe="ie"===x.browser.name,this.isIeEdge="edge"===x.browser.name,this.isIe11=this.isIe&&"11"===x.browser.version,this.inlineLabel=this.element.closest("label"),this.inlineLabelText=this.inlineLabel.find(".label-text"),this.isInlineLabel=this.element.parent().is(".inline"),this.settings.clearableText||(this.settings.clearableText=H?H.translate("None"):"None"),this.build(),this.handleEvents(),this.setCustomWidth()},build:function(){this.isEditor="editor"===this.settings.placeIn;var t=this.element,e=this.isEditor?this.element.attr("data-value"):this.element.val(),i="swatch".concat(e&&""!==$.trim(e)?"":" is-empty");if(!this.isEditor)if(this.isInlineLabel?this.inlineLabel.addClass("colorpicker-container"):(this.container=$('<span class="colorpicker-container"></span>'),t.wrap(this.container)),this.container=t.parent(),this.swatch=$('<span class="'.concat(i,'"></span>')).prependTo(this.container),this.settings.showLabel){var n=t.data("mask");n&&"function"==typeof n.destroy&&n.destroy()}else{t.mask({pattern:["#",/[0-9a-fA-F]/,/[0-9a-fA-F]/,/[0-9a-fA-F]/,/[0-9a-fA-F]/,/[0-9a-fA-F]/,/[0-9a-fA-F]/]})}var s=this.element.children(".trigger");this.container&&this.container.length&&(s=this.container.children(".trigger")),s&&s.length&&s.children(".icon").length||(this.icon=$.createIconElement("dropdown").appendTo(this.isEditor?this.element:this.container),this.icon.wrap('<span class="trigger"></span>')),e&&this.setColor(e),(this.element.is(":disabled")||this.settings.disabled)&&this.disable(),this.element.is(":disabled")&&this.container&&this.container.closest(".field").addClass("is-disabled"),this.element.prop("readonly")&&this.readonly(),this.settings.editable||this.settings.disabled||this.readonly(),this.settings.colorOnly&&this.element.parent().addClass("color-only"),this.element.attr("autocomplete","off"),this.addAria(),E.addAttributes(this.element,this,this.settings.attributes,"colorpicker"),E.addAttributes(this.element.parent().find(".trigger"),this,this.settings.attributes,"trigger")},getHexFromLabel:function(t){for(var e=0;e<this.settings.colors.length;e++){var i=this.settings.colors[e],n=H.translate(i.label,!0);if(t===i.label+i.number||t===n+i.number){var s=i.value;return"#"!==s.substr(0,1)&&(s="#".concat(s)),s}}return""},getLabelFromHex:function(t){if(!t)return"";for(var e=0;e<this.settings.colors.length;e++){var i=this.settings.colors[e];if(t.replace("#","")===i.value.replace("#",""))return this.translateColorLabel(i.label)+i.number}return""},setCustomWidth:function(){if(this.element[0].style&&this.element[0].style.width){var t,e=parseInt(this.element[0].style.width,10);this.container.css({width:e}),this.element.css({width:e-4-Number(null===(t=this.swatch)||void 0===t||null==t?void 0:t.width())})}},getHexValue:function(){return this.element.attr("value")},getLabelValue:function(){return this.settings.showLabel?this.element.val():this.getLabelFromHex(this.element.val())},addAria:function(){this.element.attr({role:"combobox","aria-autocomplete":"list"}),$('label[for="'.concat(this.element.attr("id"),'"]')).append('<span class="audible">'.concat(H.translate("UseArrow"),"</span>"))},toggleList:function(){var t=this,e=$("#colorpicker-menu");if(!(this.element.is(":disabled")||this.element.prop("readonly")&&this.settings.editable||e.length&&this.isPickerOpen)){e=this.updateColorMenu();var i=E.extend({},{ariaListbox:!0,menuId:"colorpicker-menu",trigger:"immediate",attachToBody:!0,placementOpts:{containerOffsetX:10,containerOffsetY:10,parentXAlignment:H.isRTL()?"right":"left",strategies:["flip","nudge","shrink"]},offset:{x:0,y:10}},this.settings.popupmenuSettings);this.element.popupmenu(i).on("open.colorpicker",(function(){t.element.parent().addClass("is-open"),t.isPickerOpen=!0})).on("close.colorpicker",(function(){[].slice.call(t.menu[0].querySelectorAll("a")).forEach((function(t){var e=$(t).data("tooltip");e&&e.hide()})),e.on("destroy.colorpicker",(function(){t.element.off("open.colorpicker selected.colorpicker close.colorpicker"),t.menu.off("destroy.colorpicker").remove()})),t.element.parent().removeClass("is-open"),t.isPickerOpen=!1,t.element.trigger("listclosed","select")})).on("selected.colorpicker",(function(e,i){t.element.is(".colorpicker-editor-button")||(t.itemLabel=i.data("label")),t.isEditor||t.setColor(i.data("value"),t.itemLabel);var n=!1;if(t.element.is(".colorpicker-editor-button")){var s=t.element.data("toolbaritem")||t.element.data("toolbarflexitem"),a=s?s.toolbarAPI:null;a&&a.overflowedItems.forEach((function(t){"colorpicker"===t.type&&(n=!0)}))}n||t.element.focus(),t.element.trigger("change")})),this.menu=$("#colorpicker-menu"),setTimeout((function(){t.menu.find(".is-selected").focus()}),1)}},setColor:function(t,e){t=t||"";var i=this.settings,n=t,s=e;"#"!==t.toString().substr(0,1)&&""!==t&&(n="#".concat(n));var a=/(^#[0-9a-fA-F]{6}$)|(^#[0-9a-fA-F]{3}$)/i.test(n);if(i.showLabel&&e===i.clearableText)this.setValueOnField({hex:n,label:i.clearableText,isEmpty:!0});else{if(!a){if(!i.showLabel)return n="#"!==n?n:"",void this.setValueOnField({hex:n,invalid:!0});s=t.replace("#",""),n=this.getHexFromLabel(s)}s||(s=this.getLabelFromHex(n)),this.setValueOnField({hex:n,label:s})}},setValueOnField:function(t){var e=this.settings,i=this.isEditor?"data-value":"value",n="";t.isEmpty||"string"!=typeof t.hex||(n=e.uppercase?t.hex.toUpperCase():t.hex.toLowerCase()),this.swatch&&(t.isEmpty||t.invalid?(this.swatch.addClass(t.isEmpty?"is-empty":"is-invalid"),this.swatch[0].style.backgroundColor=""):(this.swatch.removeClass("is-empty is-invalid"),this.swatch[0].style.backgroundColor=n)),this.element[0].value=e.showLabel?t.label:n,this.element[0].setAttribute(i,n),this.element[0].setAttribute("aria-describedby",t.label||"")},translateColorLabel:function(t){return t?"string"==typeof H.translate(t,!0)?H.translate(t,!0):t:""},getThemeMode:function(t){var e=Object.keys(dn.themes).filter((function(e){return t.indexOf(e)>-1})),i="light";return e.length>0&&(i=e[0]),i},getThemeVariant:function(t){return this.getThemeMode(t)},updateColorMenu:function(){var t=this,e=this.settings,i=!!$("#colorpicker-menu").length,n=$('<ul id="colorpicker-menu" class="popupmenu colorpicker"></ul>'),s=wt.currentTheme,a=this.getThemeMode(s),o="all"===e.themes[a].border,r=e.themes[a].checkmark,l="";$("#colorpicker-menu").remove();for(var c=function(){var a=$("<li></li>"),c=$('<a href="#"><span class="swatch"></span></a>').appendTo(a),h=(t.translateColorLabel(e.colors[d].label)||e.colors[d].label)+(e.colors[d].number||""),u=parseInt(e.colors[d].number,10),p=new RegExp("\\b".concat(s,"\\b")),g=e.colors[d].value,f=!1,m=t.isEditor?t.element.attr("data-value"):t.element.val();e.showLabel&&!t.isEditor&&(m=t.getHexFromLabel(m)),(o||p.test(e.colors[d].border))&&(f=!0),m&&"".concat(m).toLowerCase().replace("#","")==="".concat(g).toLowerCase()&&(r&&$.each(r,(function(t,e){u>=e[0]&&u<=e[1]&&(l=" checkmark-".concat(t))})),h!==t.itemLabel&&void 0!==t.itemLabel||c.addClass("is-selected".concat(l))),g=e.uppercase?g.toUpperCase():g.toLowerCase();var v=c.find(".swatch");v[0]&&(v[0].style.backgroundColor="#".concat(g)),null==v||null==v||v.addClass(f?"is-border":""),c[0].setAttribute("data-label",h),c[0].setAttribute("data-value",g),c[0].setAttribute("title","".concat(h," #").concat(g)),c.tooltip(),E.addAttributes(c,t,t.settings.attributes,h),i||n.append(a)},d=0,h=e.colors.length;d<h;d++)c();if(!i){if(e.clearable){var u=$("<li></li>"),p="foreColor"===this.element.attr("data-action")?"000000":"",g=$('<a href="#" title="'.concat(e.clearableText,'"><span class="swatch is-empty').concat(o?" is-border":"",'"></span></a>')).appendTo(u);g.data("label",e.clearableText).data("value",p).tooltip(),E.addAttributes(g,this,this.settings.attributes,"clear"),n.append(u)}$("body").append(n)}return n},enable:function(){this.element.prop("disabled",!1),this.element.prop("readonly",!1),this.element.parent().removeClass("is-disabled is-readonly")},disable:function(){this.element.prop("disabled",!0),this.settings.placeIn||this.element.parent().addClass("is-disabled")},readonly:function(){this.enable(),this.element.prop("readonly",!0),this.element.parent().addClass("is-readonly"),this.settings.editable||this.element.parent().addClass("is-not-editable")},isDisabled:function(){return this.element.prop("disabled")},decimal2rgb:function(t){return"number"!=typeof t?t:"rgb(".concat(255&t,", ").concat((65280&t)>>8,", ").concat((16711680&t)>>16,")")},rgb2hex:function(t){if(!t||-1===t.search("rgb"))return t;if("rgba(0, 0, 0, 0)"===t)return"transparent";var e=function(t){return"0".concat(parseInt(t,10).toString(16)).slice(-2)},i=t.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+))?\)$/);return"#".concat(e(i[1])).concat(e(i[2])).concat(e(i[3]))},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown(),this.init()},teardown:function(){this.element.off(["keypress.".concat(cn),"keyup.".concat(cn),"blur.".concat(cn),"openlist.".concat(cn),"change.".concat(cn),"paste.".concat(cn)].join(" ")),this.swatch&&this.swatch.length&&(this.swatch.off("click.".concat(cn)),this.swatch.remove(),delete this.swatch);var t=this.element.data("mask");if(t&&t.destroy(),this.icon&&this.icon.length){var e=this.icon.parent(".trigger");this.icon.off().remove(),e.off().remove(),delete this.icon}this.container&&this.container.length&&(this.container.find(".trigger").remove(),this.element.unwrap(),delete this.container)},destroy:function(){return this.teardown(),$.removeData(this.element[0],cn),this},handleEvents:function(){var t,e,i,n,s,a,o=this,r=this.element,l=r.parent();null===(t=(e=this.icon).parent)||void 0===t||null===(i=null==(n=t.call)?void 0:n.call(t,e))||void 0===i||null==i||i.on("click.colorpicker",(function(){return o.toggleList()})),null===(s=this.swatch)||void 0===s||null==s||s.on("click.colorpicker",(function(){return o.toggleList()})),r.on("focus.colorpicker",(function(){a=r.val(),l.addClass("is-focused")})).on("blur.colorpicker",(function(){l.removeClass("is-focused"),o.isIeEdge&&!r.is(".is-open")&&a!==r.val()&&r.triggerHandler("change")})).on("openlist.colorpicker",(function(){o.toggleList()}));var c="blur.colorpicker paste.colorpicker change.colorpicker";c+=this.isIe11?"keypress.colorpicker":"keyup.colorpicker",r.on(c,(function(){var t=o.isEditor?r.attr("data-value"):r.val();o.settings.showLabel?o.setColor(r.attr("value"),t):o.setColor(t)})),r.on("keydown.colorpicker",(function(t){38!==t.keyCode&&40!==t.keyCode||o.toggleList(),13===t.keyCode&&o.setColor(r.val())}))}},$.fn.colorpicker=function(t){return this.each((function(){var e=$.data(this,cn);e?e.updated(t):e=$.data(this,cn,new hn(this,t))}))};var un="toolbarflexitem";var pn=[{type:"button",selector:'button:not(.btn-menu):not(.btn-actions):not(.colorpicker-editor-button), input[type="button"]:not(.btn-menu):not(.btn-actions):not(.colorpicker-editor-button)',filter:function(t){return!$(t).parents(".searchfield-wrapper, .toolbar-searchfield-wrapper").length}},{type:"menubutton",selector:".btn-menu"},{type:"actionbutton",selector:".btn-actions"},{type:"colorpicker",selector:".colorpicker-editor-button"},{type:"hyperlink",selector:"a[href]",filter:function(t){if("A"!==t.nodeName)throw new Error("Unspecified error occured");return $(t).parents(".popupmenu").length<1}},{type:"checkbox",selector:'input[type="checkbox"]'},{type:"radio",selector:'input[type="radio"]'},{type:"searchfield",selector:".searchfield"},{type:"toolbarsearchfield",selector:".toolbarsearchfield"}],gn={button:"button",actionbutton:"popupmenu",colorpicker:"colorpicker",menubutton:"popupmenu",hyperlink:"hyperlink",searchfield:"searchfield",toolbarsearchfield:"searchfield"},fn={disabled:!1,readOnly:!1,hidden:!1,componentSettings:void 0,allowTabs:!1,attributes:null};function mn(t,e){this.element=t,this.settings=E.mergeSettings(this.element,e,fn),this.init()}mn.prototype={type:void 0,isToolbarFlexItem:!0,init:function(){this.type=function(t){var e=!1;if(pn.forEach((function(i){$(t).is(i.selector)&&("function"!=typeof i.filter||i.filter(t))&&(e=i.type)})),!e)throw new Error("Element ".concat(t," is not a valid Toolbar Item Type."));return e}(this.element),this.section=this.element.parentElement,this.toolbar=this.section.parentElement,this.trueSelected=!1},get focusable(){return!0!==this.disabled&&("searchfield"===this.type||("actionbutton"!==this.type||!0!==this.hasNoOverflowedItems)&&(!0!==this.overflowed&&this.visible))},get focused(){return 0===this.element.tabIndex},set focused(t){t?this.element.tabIndex=0:this.element&&(this.element.tabIndex=-1)},get selected(){return this.trueSelected},set selected(t){if(t)return this.trueSelected=!0,this.element.classList.add("is-selected"),this.triggerSelectedEvent(),void(this.selectedAnchor&&delete this.selectedAnchor);this.trueSelected=!1,this.element.classList.remove("is-selected")},get componentAPI(){var t=$(this.element),e=gn[this.type];if(e)return t.data(e)},get toolbarAPI(){return this.settings.toolbarAPI?this.settings.toolbarAPI:$(this.toolbar).data("toolbar-flex")},triggerSelectedEvent:function(){if(!(["searchfield","toolbarsearchfield"].indexOf(this.type)>-1)){var t=[this];["menubutton","actionbutton"].indexOf(this.type)>-1&&this.selectedAnchor&&t.push(this.selectedAnchor),$(this.element).trigger("selected",t,t[1])}},show:function(){this.visible=!0},hide:function(){this.visible=!1},set visible(t){var e="searchfield"===this.type||"toolbarsearchfield"===this.type;if(t)return e&&this.element.parentNode.classList.remove("hidden"),void this.element.classList.remove("hidden");e&&this.element.parentNode.classList.add("hidden"),this.element.classList.add("hidden")},get visible(){return-1===this.element.className.indexOf("hidden")},enable:function(){this.disabled=!1,this.hasReadOnly&&(this.readOnly=!1)},get disabled(){return this.element.disabled},set disabled(t){if(t)return this.element.disabled=!0,void(this.element.readOnly=!1);this.element.disabled=!1},get hasReadOnly(){return"readOnly"in this.element},get readOnly(){return!!this.hasReadOnly&&this.element.readOnly},set readOnly(t){if(this.hasReadOnly)return t?(this.disabled=!1,this.element.disabled=!1,void(this.element.readOnly=!0)):void(this.element.readOnly=!1)},get overflowed(){var t=x.rtl,e=this.element.getBoundingClientRect(),i=this.section.getBoundingClientRect();return t?e.left<i.left:e.right>i.right},set hasNoOverflowedItems(t){if("actionbutton"===this.type&&this.componentAPI){var e=this.componentAPI.toData({noMenuWrap:!0}).length-this.predefinedItems.length<1;if(t&&e)return this.element.classList.add("no-overflowed-items"),this.trueHasNoOverflowedItems=!0,void(this.focused&&(this.toolbarAPI.focusedItem=this,this.toolbarAPI.navigate(-1,void 0)));this.trueHasNoOverflowedItems=!1,this.element.classList.remove("no-overflowed-items")}},get hasNoOverflowedItems(){return!this.componentAPI||this.trueHasNoOverflowedItems},get index(){return this.toolbarAPI.items.indexOf(this)},handleEvents:function(){var t=this,e=this,i=$(this.element);["menubutton","actionbutton","colorpicker"].indexOf(this.type)>-1&&i.on("selected.".concat(un),(function(i,n){if(!t.selectedAnchor){if(i.stopPropagation(),"actionbutton"===t.type){var s,a=$(n).parent(),o=a.data("originalButton"),r=$(o).data("toolbarflexitem");if(a.parents("ul").length>1&&(s=a.data("original-menu-element"),o=a.parents("li").last().data("originalButton"),r=$(o).data("toolbarflexitem")),r)return s?(i.preventDefault(),r.selectedAnchor=$(s).children("a")):r.selectedAnchor=n,void(r.selected=!0)}e.selectedAnchor=n,e.selected=!0}})),"actionbutton"===this.type&&(i.on("beforeopen.".concat(un),this.handleActionButtonBeforeOpen.bind(this)),$("body").off("resize.".concat(un)).on("resize.".concat(un),this.handleActionButtonResize.bind(this))),this.settings.allowTabs||i.on("focus.".concat(un),this.handleFocus.bind(this))},handleActionButtonBeforeOpen:function(){this.refreshMoreActionsMenu()},handleActionButtonResize:function(){this.refreshMoreActionsMenu()},handleFocus:function(t){t.target&&t.target===this.element&&(this.toolbarAPI.focusedItem=this)},render:function(){var t,e=this;(this.disabled=this.disabled,this.hasReadOnly&&(this.readonly=this.readonly),Array.isArray(this.settings.attributes))&&(this.settings.componentSettings||(this.settings.componentSettings={}),this.settings.componentSettings.attributes?(t=[],this.settings.attributes.forEach((function(i){var n=e.settings.componentSettings.attributes.find((function(t){return t.name===i.name}));n?t.push(n):t.push({name:i.name,value:"".concat(i.value,"-").concat(e.type,"-").concat(e.index)})}))):t=this.settings.attributes.map((function(t){return{name:t.name,value:"".concat(t.value,"-").concat(e.type,"-").concat(e.index)}})),this.settings.componentSettings.attributes=t);var i,n=$(this.element),s=gn[this.type];s&&((i=n.data(s))?i.updated(this.settings.componentSettings):n[s](this.settings.componentSettings)),"actionbutton"===this.type&&(this.renderMoreActionsMenu(),this.refreshMoreActionsMenu()),this.handleEvents()},renderMoreActionsMenu:function(){var t=this.componentAPI;if(t&&this.toolbarAPI){var e=t.menu;e&&e.length||(e=$('<ul class="popupmenu"></ul>').insertAfter(this.element)),this.teardownPredefinedItems();var i=this.toolbarAPI.toPopupmenuData();!function t(e,i){e.menuId&&delete e.menuId;var n=i?"submenu":"menu";Array.isArray(e[n])&&e[n].forEach((function(e){t(e,!0)}))}(i);var n=$(t.renderItem(i));this.predefinedItems=n,this.linkToolbarItems(i),e.prepend(this.predefinedItems),t.updated({menu:e,predefined:n}),this.menuRendered=!0}},refreshMoreActionsMenu:function(){if("actionbutton"===this.type){var t=this.componentAPI;if(t&&this.toolbarAPI&&!t.isOpen){t.settings.beforeOpen||this.predefinedItems&&this.predefinedItems.length||!this.toolbarAPI.items.length||this.renderMoreActionsMenu();var e=!0;this.toolbarAPI.items.forEach((function(e){if(e.actionButtonLink){var n=e.toPopupmenuData();n.overflowed=e.overflowed,n.id&&delete n.id,t.refreshMenuItem(e.actionButtonLink,n,i)}})),t.refreshHasIcons(),this.hasNoOverflowedItems=e}}function i(t,i){if(!i.isSubmenuItem){if(!0===i.overflowed)return t.classList.add("is-overflowed"),i.visible&&t.classList.remove("hidden"),void(e=!1);t.classList.remove("is-overflowed"),t.classList.add("hidden")}}},unlinkToolbarItems:function(){function t(e){var i=$($(e).data("original-menu-element"));if(i.removeData("action-button-link"),$(e).removeData("original-menu-element"),i.hasClass("submenu"))for(var n=e.querySelector(".popupmenu").children,s=0;s<n.length;s++)t(n[s])}"actionbutton"===this.type&&this.menuRendered&&this.predefinedItems&&this.predefinedItems.length&&this.predefinedItems.each((function(e,i){var n=$(i).data("originalButton"),s=$(n).data("toolbarflexitem");if(s&&(s.actionButtonLink=null,$(i).removeData("original-button"),"menubutton"===s.type))for(var a=i.querySelector(".popupmenu").children,o=0;o<a.length;o++)t(a[o])}))},linkToolbarItems:function(t){function e(t,i){var n=t.elementLink;$(n).data("action-button-link",i),$(i).data("original-menu-element",n);var s=t.submenu;if(s&&s.length)for(var a=i.querySelector(".popupmenu").children,o=0;o<a.length;o++)e(s[o],a[o])}"actionbutton"===this.type&&t&&(Array.isArray(t)||(t=t.menu),this.predefinedItems.each((function(i,n){var s=t[i].itemLink;s.actionButtonLink=n,$(n).data("original-button",s.element);var a=t[i].submenu;if(a&&a.length)for(var o=n.querySelector(".popupmenu").children,r=0;r<o.length;r++)e(a[r],o[r])})))},toPopupmenuData:function(){if("searchfield"!==this.type&&"toolbarsearchfield"!==this.type&&"actionbutton"!==this.type){var t={itemLink:this,disabled:this.disabled,visible:this.visible},e=this.element.querySelector(".icon:not(.close):not(.icon-dropdown) > use");if(e&&e.getAttribute("href")&&(t.icon=e.getAttribute("href").replace("#icon-","")),e&&e.getAttribute("xlink:href")&&(t.icon=e.getAttribute("xlink:href").replace("#icon-","")),/\b(button|menubutton|colorpicker)\b/g.test(this.type)&&(t.text=this.element.textContent.trim()),"menubutton"===this.type){var i=this.componentAPI.menu;if(i.length){var n=this.componentAPI.toData({noMenuWrap:!0}),s=this.componentAPI.element[0].id;s&&(t.id="".concat(this.toolbarAPI.uniqueId,"-").concat(s)),t.submenu=function t(e,i){var n=e.querySelectorAll(":scope > li:not(.heading)");return i.forEach((function(e,i){if(e.elementLink=n[i],e.submenu){var s=n[i].querySelector(".popupmenu");e.submenu=t(s,e.submenu)}})),i}(i[0],n)}else t.submenu=[]}return t}},toData:function(){var t={type:this.type,disabled:this.disabled,focused:this.focused,selected:this.selected,overflowed:this.overflowed,visible:this.visible};this.hasReadOnly&&(t.readOnly=this.readOnly),this.actionButtonLink&&(t.actionButtonLink=this.actionButtonLink),this.componentAPI&&(t.componentAPI=this.componentAPI);var e=this.element.querySelector(".icon:not(.close):not(.icon-dropdown) > use");return e&&e.getAttribute("href")&&(t.icon=e.getAttribute("href").replace("#icon-","")),e&&e.getAttribute("xlink:href")&&(t.icon=e.getAttribute("xlink:href").replace("#icon-","")),/\b(button|menubutton|colorpicker)\b/g.test(this.type)&&(t.text=this.element.textContent.trim()),"actionbutton"===this.type&&(t.predefinedItems=this.predefinedItems),"menubutton"!==this.type&&"actionbutton"!==this.type||(t.submenu=this.componentAPI.toData({noMenuWrap:!0})),t},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown(),this.init()},teardownPredefinedItems:function(){"actionbutton"===this.type&&(this.unlinkToolbarItems(),this.predefinedItems&&this.predefinedItems.length&&this.predefinedItems.remove())},teardown:function(){$(this.element).off("selected.".concat(un)).off("beforeopen.".concat(un)).off("focus.".concat(un)),$("body").off("resize.".concat(un)),this.teardownPredefinedItems(),delete this.type,delete this.selected,delete this.focusable,delete this.visible,delete this.disabled,delete this.readOnly,delete this.section,delete this.toolbar,delete this.trueSelected,delete this.menuRendered},destroy:function(){var t=$(this.element).data("popupmenu");t&&t.destroy&&t.destroy();var e=$(this.element).data("searchfield");e&&e.destroy&&e.destroy(),this.teardown()}},$.fn.toolbarflexitem=function(t){return this.each((function(){var e=$.data(this,un);if(e)e.updated(t);else{var i=(e=$.data(this,un,new mn(this,t))).destroy;e.destroy=function(){"function"==typeof i&&i.call(this),this.element&&$.removeData(this.element,un),this.element=void 0}}}))};var vn="toolbar-flex",bn={allowTabs:!1,attributes:null,beforeMoreMenuOpen:null,moreMenuSettings:{}};function yn(t,e){this.element=t,this.settings=E.mergeSettings(this.element,e,bn),this.init()}yn.prototype={trueFocusedItem:void 0,sections:[],items:[],init:function(){var t=this,e=this;if(this.uniqueId=E.uniqueId(this.element),this.items=this.getElements().map((function(e){var i={};return $(e).hasClass("btn-actions")&&(i=t.settings.moreMenuSettings||i,t.settings.beforeMoreMenuOpen&&(i.beforeOpen=t.settings.beforeMoreMenuOpen)),$(e).toolbarflexitem({toolbarAPI:t,componentSettings:i,allowTabs:t.settings.allowTabs,attributes:t.settings.attributes}),$(e).data("toolbarflexitem")})),this.items){if(this.sections=E.getArrayFromList(this.element.querySelectorAll(".toolbar-section")),this.sections.forEach((function(t){t.classList.contains("buttonset")&&$(t).buttonset({detectHTMLButtons:!0})})),this.jQueryEl=$(this.element),this.jQueryEl.hasClass("contextual-toolbar")){var i;if(this.more=this.jQueryEl.find(".btn-actions"),0===this.more.length&&!this.jQueryEl.hasClass("no-actions-button")){var n=this.jQueryEl.find(".more");n.length||(n=$('<div class="toolbar-section more"></div>').appendTo(this.jQueryEl)),this.more=$('<button class="btn-actions" type="button"></button>').html("".concat($.createIcon({icon:"more"}),'<span class="audible">').concat(H.translate("MoreActions"),"</span>")).attr("title",H.translate("More")).appendTo(n)}this.buttonset=$(".toolbar-section.buttonset"),this.buttonsetItems=this.buttonset.children("button").add(this.buttonset.find("input")),this.toolBarItems=this.buttonsetItems.add(this.more);var s=this.more.data("popupmenu"),a=this.more.attr("aria-controls");s?this.moreMenu=s.menu:(this.moreMenu=$("#".concat(a)),this.moreMenu.length&&void 0!==a||(this.moreMenu=this.more.next(".popupmenu, .popupmenu-wrapper")),this.moreMenu.length||(this.moreMenu=$('<ul id="popupmenu-toolbar-'.concat(this.uniqueId,'" class="popupmenu"></ul>')).insertAfter(this.more)),this.moreMenu.is(".popupmenu-wrapper")&&(this.moreMenu=this.moreMenu.children(".popupmenu")));var o,r=[];this.toolBarItems.not(this.more).not(".ignore-in-menu").filter((function(){return $(this).parent(".buttonset, .inline").length})).each((function(){r.push(e.buildMoreActionsMenuItem($(this)))})),r.reverse(),$.each(r,(function(t,i){""!==i.text()&&i.prependTo(e.moreMenu)})),this.defaultMenuItems=this.moreMenu.children("li:not(.separator)"),this.hasDefaultMenuItems=this.defaultMenuItems.length>0,this.settings.moreMenuSettings&&Array.isArray(this.settings.moreMenuSettings.attributes)&&(o=this.settings.moreMenuSettings.attributes),o&&o.length||!Array.isArray(this.settings.attributes)||(o=this.settings.attributes.map((function(t){var e="".concat(t.value,"-actionbutton");return{name:t.name,value:e}}))),null!==(i=o)&&void 0!==i&&(null==i?void 0:i.length)||(o=null);var l=E.extend({},this.settings.moreMenuSettings,{trigger:"click",menu:this.moreMenu,attributes:o},this.hasDefaultMenuItems?{predefined:this.defaultMenuItems}:{});s?this.more.on("beforeopen.toolbar-flex",(function(){e.refreshMoreActionsMenu(e.moreMenu)})).triggerHandler("updated",[l]):this.more.popupmenu(l).on("beforeopen.toolbar-flex",(function(){e.refreshMoreActionsMenu(e.moreMenu)}))}this.settings.allowTabs||(this.items.forEach((function(e){e.focused&&(void 0===t.focusedItem?t.focusedItem=e:e.focused=!1)})),this.focusedItem||(this.focusedItem=this.items[0]),this.focusedItem&&(this.focusedItem.focused=!0)),this.render(),this.handleEvents()}},buildMoreActionsMenuItem:function(t){var e,i;if(!0===t.data("skipit"))return t.data("skipit",void 0),i;var n=t.data("action-button-link");n&&n.length?i=n.parent():(i=$("<li></li>"),n=$('<a href="#"></a>').appendTo(i),t.data("action-button-link",n),n.data("original-button",t)),t.hasClass("hidden")&&i.addClass("hidden"),t.is(":disabled")?(i.addClass("is-disabled"),n.prop("disabled",!0)):(i.removeClass("is-disabled"),n.prop("disabled",!1)),n.text(this.getItemText(t));var s=$.getBaseURL("#icon-dropdown"),a=t.children(".icon").filter((function(){var t=$(this).getIconName();return t&&t!==s&&-1===t.indexOf("dropdown")}));a&&a.length&&(n.html("<span>".concat(n.text(),"</span>")),a.clone().detach().prependTo(n));var o=i.find("b");o.length&&(this.moreMenu.addClass("has-icons"),o.detach().prependTo(i));var r=null===(e=this.toolbarItems)||void 0===e||null==e?void 0:e.not(this.more).not(".searchfield").index(t);if(t.is(".btn-menu")){var l,c;this.settings.attributes&&this.settings.attributes.length&&(c=this.settings.attributes.map((function(t){return{name:t.name,value:"".concat(t.value,"-menubutton-").concat(r)}}))),null!==(l=c)&&void 0!==l&&(null==l?void 0:l.length)||(c=null),t.data("popupmenu")?n.children(".icon.arrow").length||n.append($.createIcon({classes:"icon arrow icon-dropdown",icon:"dropdown"})):t.popupmenu({attributes:c});var d=t.data("popupmenu").menu,h=d.clone();!function t(e,n,s){var a,o=e.children("li"),r=n.attr("id");n.children("li").each((function(e,i){var n=$(i),s=o.eq(e),a=n.children("a"),r=s.children("a"),l=n.attr("id"),c=a.attr("id");l&&n.removeAttr("id").attr("data-original-menu-item",l),c&&a.removeAttr("id").attr("data-original-menu-anchor",c),r.data("action-button-link",a),a.data("original-button",r);var d=s.children(".wrapper").children(".popupmenu"),h=n.children(".wrapper").children(".popupmenu");d.length&&h.length&&t(d,h,n)})),n.removeAttr("id").attr("data-original-menu",r),s.addClass("submenu"),s.is(i)&&((a=s.children(".wrapper"))&&a.length||(a=$('<div class="wrapper"></div>')),a.html(n),s.append(a))}(d,h,i)}return t.is("[data-popdown]")&&t.popdown(),i},getItemText:function(t){if(!t)return"";var e=t.find("span").first(),i=t.attr("title"),n=t.data("tooltip"),s=n&&"string"==typeof n.content?n.content:void 0;return""!==i&&void 0!==i?i:s||(e.length?e.text():t.text())},refreshMoreActionsMenu:function(t){var e=this;$("li > a",t).each((function(){var t,i=$(this),n=i.parent(),s=i.data("originalButton"),a=e.getItemText(s);s&&(i.find("span").length?i.find("span").text(a.trim()):i.text(a.trim()),s.parent().is(".is-disabled")||s.is(":disabled")?(n.addClass("is-disabled"),i.prop("disabled",!0),i.attr("tabindex","-1")):(n.removeClass("is-disabled"),i.prop("disabled",!1)),s.is("a")&&(s.parent("li").is(".is-checked")?n.addClass("is-checked"):n.removeClass("is-checked")),s.is(".btn-menu")&&(t=i.parent().find(".popupmenu").first(),e.refreshMoreActionsMenu(t)))}))},renderButtonSet:function(){var t,e,i,n;if(this.jQueryEl.hasClass("contextual-toolbar")){var s=(null===(t=this.toolBarItems)||void 0===t||null==t?void 0:t.not(".btn-actions").length)>5,a=s?null===(e=this.toolBarItems)||void 0===e||null==e?void 0:e.not(".btn-actions").slice(0,5):null===(i=this.toolBarItems)||void 0===i||null==i?void 0:i.not(".btn-actions");a.removeClass("is-overflowed");for(var o,r,l,c,d,h=s?null===(n=this.toolBarItems)||void 0===n||null==n?void 0:n.not(".btn-actions").slice(5):$(),u=$(),p=0;p<a.length;p++)o=a[p],r=void 0,l=void 0,c=void 0,d=void 0,r=H.isRTL(),l=o.getBoundingClientRect(),c=o.parentElement.getBoundingClientRect(),d=r?l.left<=c.left:l.right>=c.right||l.right,!0==l.bottom>=c.bottom||!0===d?h.push(a[p]):u.push(a[p]);h.addClass("is-overflowed");for(var g=0;g<u.length;g++){var f=$(u[g]).data("action-button-link");f&&f.parent()[0].classList.add("hidden")}for(var m=0;m<h.length;m++){var v=$(h[m]).data("action-button-link");v&&v.parent()[0].classList.remove("hidden")}h.length>0?(this.more.parent().css("display","inline-block"),this.more.parent().siblings(".buttonset").css("width","calc(55% - 41px")):(this.more.parent().css("display","none"),this.more.parent().siblings(".buttonset").css("width","calc(55% - 1px"))}},render:function(){var t=this,e=this;this.element.setAttribute("role","toolbar"),Array.isArray(this.settings.attributes)&&E.addAttributes($(this.element),this,this.settings.attributes),this.items.forEach((function(i){var n=$(i.element);if(n.is(".toolbar-searchfield-wrapper, .searchfield-wrapper")&&(n=n.children(".searchfield")),!n.data("searchfield")&&n.is(".searchfield")){var s=$.extend({},E.parseSettings(n[0]));t.settings.collapsibleFilter&&(s.collapsible=t.settings.collapsibleFilter),n.searchfield(s),E.addAttributes(n,e,e.settings.attributes,"searchfield")}i.render()}))},handleEvents:function(){var t=this;this.settings.allowTabs||(this.keydownListener=this.handleKeydown.bind(this),this.element.addEventListener("keydown",this.keydownListener),this.keyupListener=this.handleKeyup.bind(this),this.element.addEventListener("keyup",this.keyupListener),this.clickListener=this.handleClick.bind(this),this.element.addEventListener("click",this.clickListener)),$(this.element).on("selected.".concat(vn),(function(t){for(var e=arguments.length,i=new Array(e>1?e-1:0),n=1;n<e;n++)i[n-1]=arguments[n]})),$(this.element).on("collapsed-responsive.".concat(vn),(function(e,i){e.stopPropagation(),t.navigate(i,null,!0)})),$(this.element).hasClass("contextual-toolbar")&&($(this.element).on("recalculate-buttonset.toolbar-flex-".concat(this.uniqueId),(function(){t.renderButtonSet()})),this.more.on("beforeopen.buttonset.toolbar-flex-".concat(this.uniqueId),(function(){t.renderButtonSet()})),$("body").on("resize.toolbar-flex-".concat(this.uniqueId),(function(){t.renderButtonSet()})))},handleKeydown:function(t){var e=t.target;$(e).data("toolbarflexitem")&&this.handleItemKeydown(t)},handleItemKeydown:function(t){var e=H.isRTL(),i=t.key,n=this.getItemFromElement(t.target);function s(){t.preventDefault()}if("Enter"!==i)if(" "!==i){if(["ArrowLeft","Left","ArrowUp","Up"].indexOf(i)>-1){if("searchfield"===n.type&&("ArrowLeft"===i||"Left"===i))return;return this.navigate(e?1:-1,void 0,!0),void s()}if(["ArrowRight","Right","ArrowDown","Down"].indexOf(i)>-1){if("searchfield"===n.type&&("ArrowRight"===i||"Right"===i))return;this.navigate(e?-1:1,void 0,!0),s()}}else"hyperlink"===n.type&&this.select(t.target);else this.clickByEnterKey=!0},handleKeyup:function(t){this.clearClickByEnter(t)},handleClick:function(t){var e=t.target;$(e).data("toolbarflexitem")&&this.handleItemClick(t),this.clearClickByEnter()},handleItemClick:function(t){var e=this.getItemFromElement(t.target);this.select(e),this.focusedItem=e},clearClickByEnter:function(t){this.clickByEnterKey&&("menubutton"!==this.type&&"actionbutton"!==this.type||t.preventDefault(),delete this.clickByEnterKey)},getElements:function(){var t=[],e=[];return pn.forEach((function(t){e.push(t.selector)})),e=e.join(", "),E.getArrayFromList(this.element.querySelectorAll(e)).forEach((function(e){var i=!1;pn.forEach((function(n){!i&&$(e).is(n.selector)&&("function"!=typeof n.filter||n.filter(e))&&(i=!0,t.push(e))}))})),t},getItemFromElement:function(t){if(t instanceof mn||t.isToolbarFlexItem)return t;for(var e,i=0;i<this.items.length;i++)this.items[i].element.innerHTML===t.innerHTML&&(e=this.items[i]);if(!e)throw new Error("No Toolbar Item instance available for element ".concat(t,"."));return e},get buttonsets(){return this.sections.filter((function(t){return t.classList.contains("buttonset")}))},get buttonsetAPIs(){return this.buttonsets.map((function(t){return $(t).data("buttonset")}))},get searchfieldAPI(){for(var t=0;t<this.items.length;t++)if("searchfield"===this.items[t].type)return this.items[t].componentAPI},get focusedItem(){if(this.trueFocusedItem)return this.trueFocusedItem;for(var t=0;t<this.items.length;t++)if(!0===this.items[t].focused)return this.items[t]},set focusedItem(t){if(0!==this.items.length){for(var e=0;e<this.items.length;e++)this.items[e].settings.allowTabs?this.items[e].focused=!0:this.items[e].focused=!1;t.focused=!0,this.trueFocusedItem=t}},get hasFocusableItems(){for(var t=0;t<this.items.length;t++)if(!0===this.items[t].focusable)return!0;return!1},get canManageFocus(){var t=document.activeElement;if(this.element.contains(t))return!0;for(var e=0;e<this.items.length;e++)if("searchfield"===this.items[e].type&&this.items[e].componentAPI.isFocused)return!0;return"BODY"===t.tagName},get overflowedItems(){for(var t=[],e=0;e<this.items.length;e++)!0===this.items[e].overflowed&&t.push(this.items[e]);return t},navigate:function(t,e,i){if(!1!==this.hasFocusableItems){var n=0+t;for(null==e&&(e=this.items.indexOf(this.focusedItem));0!==t;)t>0&&(e===this.items.length-1?e=0:e++,--t),t<0&&(0===e?e=this.items.length-1:--e,t++);var s=this.items[e];!1!==s.focusable?this.focusedItem===s&&1===this.items.length||(this.focusedItem=s,i&&this.canManageFocus&&this.focusedItem.element.focus()):this.navigate(n>0?1:-1,e,i)}},select:function(t){var e=this.getItemFromElement(t);switch(e.type){case"searchfield":case"actionbutton":case"menubutton":if(this.clickByEnterKey)return;e.selected=!0;break;default:e.selected=!0}},toPopupmenuData:function(){var t={noMenuWrap:!0};return t.menu=this.items.filter((function(t){return"actionbutton"!==t.type&&"searchfield"!==t.type})).map((function(e){return function(e){var i=e.toPopupmenuData();return i&&(i.icon&&(t.hasIcons=!0),i.selectable&&(t.selectable=i.selectable)),i}(e)})),t},toData:function(){var t={};return t.items=this.items.map((function(t){return t.toData()})),t},get disabled(){return this.trueDisabled},set disabled(t){this.trueDisabled=t,!0!==t?this.element.classList.remove("is-disabled"):this.element.classList.add("is-disabled")},isItemOverflowed:function(t){if(!t)return!1;var e;if(t instanceof HTMLElement||t instanceof $){if(!(e=$(t).data("toolbarflexitem")))return!1;t=e}return!(this.items.indexOf(t)<0)&&t.overflowed},updated:function(t){"object"===l(t)&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown(),this.init()},teardown:function(){var t,e,i;this.settings.allowTabs||(null===(t=this.element)||void 0===t||null==t||t.removeEventListener("keydown",this.keydownListener),null===(e=this.element)||void 0===e||null==e||e.removeEventListener("keyup",this.keyupListener),null===(i=this.element)||void 0===i||null==i||i.removeEventListener("click",this.clickListener));$(this.element).off("selected.".concat(vn)),$(this.element).off("collapsed-responsive.".concat(vn)),$(this.element).hasClass("contextual-toolbar")&&($(this.element).off("recalculate-buttonset.toolbar-flex-".concat(this.uniqueId)),this.more.off("beforeopen.buttonset.toolbar-flex-".concat(this.uniqueId)),$("body").off("resize.toolbar-flex-".concat(this.uniqueId))),this.items.forEach((function(t){t.teardown()})),delete this.items,delete this.sections,delete this.trueFocusedItem},destroy:function(){this.items.forEach((function(t){t.destroy()})),this.teardown()}},$.fn.toolbarflex=function(t){return this.each((function(){var e=$.data(this,vn);if(e)e.updated(t);else{var i=(e=$.data(this,vn,new yn(this,t))).destroy;e.destroy=function(){"function"==typeof i&&i.call(this),this.element&&$.removeData(this.element,vn),this.element=void 0}}}))};var wn="calendartoolbar",xn={month:(new Date).getMonth(),year:(new Date).getFullYear(),locale:null,showToday:!0,showNextPrevious:!0,onOpenCalendar:null,onChangeView:null,isAlternate:!1,isMenuButton:!0,showViewChanger:!1,viewChangerValue:"month",isMonthPicker:!1,inPage:!1,inPageTitleAsButton:!0,hitbox:!1};function Cn(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,xn),this.init()}Cn.prototype={init:function(){this.setLocale().build().handleEvents().handleResize()},build:function(){var t=this,e=this.settings,i=function(e){return H.translate(e,{locale:t.locale.name,language:t.language})};this.element[0].classList.add("flex-toolbar"),this.element[0].setAttribute("data-init","false");var n={text:i("Today"),class:"today"},s=e.inPage?" is-ripple":"";e.inPage=M.toBoolean(e.inPage),e.inPageTitleAsButton=M.toBoolean(e.inPageTitleAsButton),e.inPage&&(this.element.addClass("is-inpage"),e.showToday||(s="",n.text=i("Apply"),n.class="apply is-apply hidden")),n.class+=" hyperlink".concat(s).concat(e.disableToday?" is-disabled":"");var a=e.showToday||e.inPage?'<a class="'.concat(n.class,'" href="#">').concat(n.text,"</a>"):"",o=e.hitbox?'data-options="{hitbox: true}"':"",r=e.showNextPrevious?"":" no-next-previous",l=e.showNextPrevious?'<button type="button" class="btn-icon prev" '.concat(o,'>\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n          <use href="#icon-caret-left"></use>\n        </svg>\n        <span>').concat(i("PreviousMonth"),'</span>\n      </button>\n      <button type="button" class="btn-icon next" ').concat(o,'>\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-caret-right"></use>\n          </svg>\n          <span>').concat(i("NextMonth"),"</span>\n      </button>"):"",c=e.showNextPrevious?'<button type="button" class="btn-icon prev">\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n          <use href="#icon-caret-left"></use>\n        </svg>\n        <span>'.concat(i("PreviousMonth"),'</span>\n      </button>\n      <button type="button" class="btn-icon next">\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-caret-right"></use>\n          </svg>\n          <span>').concat(i("NextMonth"),"</span>\n      </button>"):"";if(e.isAlternate||e.inPage){e.isAlternate&&this.element[0].classList.add("is-alternate");var d='\n        <button type="button" class="btn btn-monthyear-pane expandable-area-trigger" id="btn-monthyear-pane">\n          <span class="month">november</span>\n          <span class="year">2019</span>\n          <svg class="icon icon-closed" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-dropdown"></use>\n          </svg>\n          <svg class="icon icon-opened" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-dropdown"></use>\n          </svg>\n        </button>',h=e.isMenuButton?d:'<span class="month">november</span><span class="year">2015</span>',u='\n        <div class="toolbar-section buttonset l-align-'.concat(this.isRTL?"left":"right").concat(r,'">\n          ').concat(a,"\n          ").concat(c,"\n        </div>"),p="";if(e.inPage){var g='\n          <button type="button" class="btn btn-cal-month-year" id="btn-cal-month-year">\n            '.concat($.createIcon("calendar"),'\n            <span class="month">november</span>\n            <span class="year">2019</span>\n          </button>'),f='\n          <button class="btn-icon btn-inpage-cal" type="button" id="btn-inpage-cal">\n            <span class="audible"></span>\n            '.concat($.createIcon("calendar"),"\n          </button>");p=e.inPageTitleAsButton?e.isMenuButton?"".concat(f," ").concat(d):g:"".concat(f,' <span class="month">november</span><span class="year">2015</span>')}this.element[0].innerHTML='\n        <div class="toolbar-section">\n          '.concat(e.inPage?p:h,"\n        </div>\n        ").concat(u)}else this.element[0].innerHTML='\n        <div class="toolbar-section'.concat(r,'">\n          ').concat(l,'\n          <span class="monthview-datepicker">\n            <span class="hidden month" data-month="9">9</span>\n            <span class="hidden year">2019</span>\n            <span class="audible">').concat(i("SelectDay"),'</span>\n            <span tabindex="0" aria-label="').concat(i("Today"),'" id="monthview-datepicker-field" class="datepicker input-auto" data-validation="">October 2019</span>\n          </span>\n          ').concat(a,'\n        </div>\n        <div class="toolbar-section buttonset l-align-right">\n          ').concat(e.showViewChanger?'<label for="'.concat(e.viewChangerValue,'-calendar-view-changer" class="label audible">').concat(i("ChangeView"),'</label>\n            <select id="').concat(e.viewChangerValue,'-calendar-view-changer" name="').concat(e.viewChangerValue,'-calendar-view-changer" class="dropdown">\n              <option value="month"').concat("month"===e.viewChangerValue?" selected":"",">").concat(i("Month"),'</option>\n              <option value="week"').concat("week"===e.viewChangerValue?" selected":"",">").concat(i("Week"),'</option>\n              <option value="day" ').concat("day"===e.viewChangerValue?" selected":"",">").concat(i("Day"),"</option>\n            </select>\n          </div>"):"","\n        </div>\n      ");return this.element.toolbarflex({allowTabs:!0}),this.monthPicker=this.element.find("#monthview-datepicker-field").datepicker({dateFormat:H.calendar(this.locale.name,e.language,e.calendarName).dateFormat.year,locale:e.locale,language:e.language,onOpenCalendar:e.onOpenCalendar,isMonthPicker:e.isMonthPicker,showToday:e.showToday,hitbox:e.hitbox}),e.showViewChanger&&(this.viewChanger=this.element.find("#".concat(e.viewChangerValue,"-calendar-view-changer")).dropdown()),this.todayLink=this.element.find(".hyperlink.today, .hyperlink.is-apply"),this.monthPickerApi=this.monthPicker.data("datepicker"),this.element.find("button, a").hideFocus(),this.setInternalDate(this.isIslamic?[e.year,e.month,1]:new Date(e.year,e.month,1)),E.addAttributes(this.element.find(".next"),this,e.attributes,"".concat(e.viewChangerValue,"-view-btn-next")),E.addAttributes(this.element.find(".prev"),this,e.attributes,"".concat(e.viewChangerValue,"-view-btn-prev")),E.addAttributes(this.element.find(".today"),this,e.attributes,"".concat(e.viewChangerValue,"-view-today")),E.addAttributes(this.element.find("#monthview-datepicker-field + .trigger"),this,e.attributes,"".concat(e.viewChangerValue,"-view-datepicker-trigger"),!0),E.addAttributes(this.element.find("#monthview-datepicker-field"),this,e.attributes,"".concat(e.viewChangerValue,"-view-datepicker"),!0),E.addAttributes($('.monthview-header #month-calendar-view-changer [value="month"]'),this,e.attributes,"month-view-changer-month"),E.addAttributes($('.monthview-header #month-calendar-view-changer [value="week"]'),this,e.attributes,"month-view-changer-week"),E.addAttributes($('.monthview-header #month-calendar-view-changer [value="day"]'),this,e.attributes,"month-view-changer-day"),E.addAttributes($(".monthview-header #month-calendar-view-changer"),this,e.attributes,"month-view-changer"),E.addAttributes($('.week-view-header #week-calendar-view-changer [value="month"]'),this,e.attributes,"week-view-changer-month"),E.addAttributes($('.week-view-header #week-calendar-view-changer [value="week"]'),this,e.attributes,"week-view-changer-week"),E.addAttributes($('.week-view-header #week-calendar-view-changer [value="day"]'),this,e.attributes,"week-view-changer-day"),E.addAttributes($(".week-view-header #week-calendar-view-changer"),this,e.attributes,"week-view-changer"),this},setInternalDate:function(t){H.isIslamic(this.locale.name)?(this.currentYear=t[0],this.currentMonth=t[1],this.currentDay=t[2],this.currentDateIslamic=t):(this.currentYear=t.getFullYear(),this.currentMonth=t.getMonth(),this.currentDay=t.getDate(),this.currentDate=t),this.currentCalendar&&this.currentCalendar.months||(this.currentCalendar=H.calendar(this.locale.name,this.settings.language,this.settings.calendarName));var e=m.isBelow("slim")&&this.currentCalendar.months?"".concat(this.currentCalendar.months.abbreviated[this.currentMonth]," ").concat(this.currentYear):H.formatDate(new Date(this.currentYear,this.currentMonth,this.currentDay),{date:"year",locale:this.locale.name,language:this.settings.language});this.monthPicker.text(e);var i=this.currentCalendar.months?this.currentCalendar.months.wide[this.currentMonth]:"";if(this.element.find("span.month").attr("data-month",this.currentMonth).text(i),this.element.find("span.year").text(" ".concat(this.currentYear)),this.currentCalendar.dateFormat.year&&"y"===this.currentCalendar.dateFormat.year.substr(1,1)){var n=H.formatDate(this.currentDate,{date:"year",locale:this.locale.name}).split(" "),s=n[0];3===n.length&&this.element.find("span.month").attr("data-month",this.currentMonth).text("".concat(n[1]," ").concat(n[2])),this.element.find("span.year").text("".concat(s," ")),this.element.find("span.year").insertBefore(this.element.find("span.month"))}return this},setCurrentCalendar:function(){return this.currentCalendar=H.calendar(this.locale.name,this.settings.language,this.settings.calendarName),this.isIslamic="islamic-umalqura"===this.currentCalendar.name,this.isRTL="right-to-left"===(this.locale.direction||this.locale.data.direction),this},setLocale:function(){var t=this;return this.settings.language?(H.getLocale(this.settings.language),this.language=this.settings.language):this.language=H.currentLanguage.name,!this.settings.locale||this.locale&&this.locale.name===this.settings.locale?this.settings.locale||(this.locale=H.currentLocale,this.setCurrentCalendar()):H.getLocale(this.settings.locale).done((function(e){t.locale=H.cultures[e],t.language=t.settings.language||t.locale.language,t.setCurrentCalendar()})),this},setViewChangerValue:function(t){var e,i;this.settings.viewChangerValue=t,null===(e=this.viewChanger)||void 0===e||null===(i=null==e?void 0:e.val(t))||void 0===i||null==i||i.trigger("updated")},handleEvents:function(){var t=this,e=this;return this.monthPicker.off("change.calendar-toolbar-p").on("change.calendar-toolbar-p",(function(){var t=$(this).data("datepicker");e.setInternalDate(t.isIslamic?t.currentDateIslamic:t.currentDate),e.element.trigger("change-date",{selectedDate:t.currentDate,isToday:!1})})),this.todayLink.off("click.calendar-toolbar-t").on("click.calendar-toolbar-t",(function(e){var i={selectedDate:t.currentDate,isToday:!0};!e.target.classList.contains("today")&&e.target.classList.contains("apply")&&(i.isToday=!1,i.isApply=!0),t.element.trigger("change-date",i),e.preventDefault()})),this.element.find(".prev").off("click.calendar-toolbar-b").on("click.calendar-toolbar-b",(function(){t.element.trigger("change-prev",{selectedDate:t.currentDate,isToday:!1})})),this.element.find(".next").off("click.calendar-toolbar-b").on("click.calendar-toolbar-b",(function(){t.element.trigger("change-next",{selectedDate:t.currentDate,isToday:!1})})),this.settings.onChangeView&&this.viewChanger&&this.viewChanger.off("change.calendar-toolbar-v").on("change.calendar-toolbar-v",(function(e){t.settings.onChangeView({viewName:e.currentTarget.value,elem:e.currentTarget,api:t})})),this},resize:function(){if(this.viewChanger&&(m.isBelow("phablet")?(this.viewChanger.next().addClass("dropdown-wrapper-small"),this.viewChanger.next().find(".dropdown").css({width:"".concat(m.isBelow("phone")?"60":"90","px")}),$(".today").css(m.isBelow("slim")?{display:"none"}:{})):(this.viewChanger.next().removeClass("dropdown-wrapper-small"),this.viewChanger.next().find(".dropdown").removeAttr("style"),$(".today").removeAttr("style"))),this.monthPicker){var t=m.isBelow("slim")&&this.currentCalendar.months?"".concat(this.currentCalendar.months.abbreviated[this.currentMonth]," ").concat(this.currentYear):H.formatDate(new Date(this.currentYear,this.currentMonth,this.currentDay),{date:"year",locale:this.locale.name,language:this.settings.language});this.monthPicker.text(t)}},handleResize:function(){return this.resize(),$(window).on("resize",this.resize),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,xn)),this.teardown().init()},teardown:function(){return $(window).off("resize",this.resize),this.element.off(),this.monthPicker.off(),this.todayLink.off(),this.element.find(".prev .next").off(),this},destroy:function(){this.teardown(),$.removeData(this.element[0],wn)}};var kn="monthview",Tn=[{name:"Public Holiday",color:"azure06",dates:[]},{name:"Weekends",color:"turquoise06",dayOfWeek:[]}],An={locale:null,language:null,month:(new Date).getMonth(),year:(new Date).getFullYear(),day:(new Date).getDate(),activeDate:null,activeDateIslamic:null,isPopup:!1,inPage:!1,inPageTitleAsButton:!0,inPageToggleable:!0,inPageExpanded:!0,headerStyle:"full",firstDayOfWeek:null,disable:{callback:null,dates:[],years:[],minDate:"",maxDate:"",dayOfWeek:[],isEnable:!1,restrictMonths:!1},legend:[],hideDays:!1,showMonthYearPicker:!0,showWeekNumber:!1,yearsAhead:3,yearsBack:2,range:{useRange:!1,start:"",end:"",separator:" - ",minDays:0,maxDays:0,selectForward:!1,selectBackward:!1,includeDisabled:!1},displayRange:{start:"",end:"",months:{showAbbr:!1}},selectable:!0,onSelected:null,onKeyDown:null,showToday:!0,showNextPrevious:!0,onChangeView:null,isMonthPicker:!1,hitbox:!1};function Sn(t,e){this.settings=E.mergeSettings(t,e,An),this.settings.legend&&0===this.settings.legend.length&&(this.settings.legend=Tn),this.element=$(t),this.init()}Sn.prototype={init:function(){return this.setLocaleThenBuild()},setLocaleThenBuild:function(){var t=this,e=H.getLocale(this.settings.language),i=H.getLocale(this.settings.locale);$.when(i,e).done((function(e,i){t.locale=H.cultures[e]||H.currentLocale,t.language=i||t.settings.language||t.locale.language,t.settings.language=t.language,t.setCurrentCalendar(),t.settings.displayRange.start&&t.settings.displayRange.end?t.buildRange():t.build()}))},build:function(){var t=this;if("false"===this.settings.showMonthYearPicker&&(this.settings.showMonthYearPicker=!1),this.settings.inPage=M.toBoolean(this.settings.inPage),this.settings.inPage){var e="is-inpage".concat(this.settings.inPageToggleable?"":" not-toggleable");this.element.addClass(e),this.settings.yearsAhead=4,this.settings.yearsBack=1}this.setCurrentCalendar(),this.prevButton=""+'<button type="button" class="btn-icon prev">\n        '.concat($.createIcon("caret-left"),"\n        <span>").concat(H.translate("PreviousMonth",{locale:this.locale.name,language:this.language}),"</span>\n      </button>"),this.nextButton=""+'<button type="button" class="btn-icon next">\n        '.concat($.createIcon("caret-right"),"\n        <span>").concat(H.translate("NextMonth",{locale:this.locale.name,language:this.language}),"</span>\n      </button>"),this.table=$('<table class="monthview-table" aria-label="'.concat(H.translate("Calendar",{locale:this.locale.name}),'" role="application"></table>')),this.dayNames=$("<thead>\n        <tr>\n          <th>SU</th>\n          <th>MO</th>\n          <th>TU</th>\n          <th>WE</th>\n          <th>TH</th>\n          <th>FR</th>\n          <th>SA</th>\n        </tr>\n      </thead>").appendTo(this.table),this.days=$('<tbody>\n        <tr>\n          <td class="alternate">26</td>\n          <td class="alternate">27</td>\n          <td class="alternate">28</td>\n          <td class="alternate">29</td>\n          <td class="alternate" >30</td>\n          <td class="alternate">31</td>\n          <td>1</td>\n        </tr><tr>\n          <td>2</td>\n          <td>3</td>\n          <td>4</td>\n          <td>5</td>\n          <td>6</td>\n          <td>7</td>\n          <td>8</td>\n        </tr><tr>\n          <td>9</td>\n          <td>10</td>\n          <td>11</td>\n          <td>12</td>\n          <td>13</td>\n          <td>14</td>\n          <td>15</td>\n        </tr><tr>\n          <td>16</td>\n          <td>17</td>\n          <td>18</td>\n          <td>19</td>\n          <td class="is-today">20</td>\n          <td>21</td>\n          <td>22</td>\n        </tr><tr>\n          <td>23</td>\n          <td>24</td>\n          <td>25</td>\n          <td>26</td>\n          <td>27</td>\n          <td>28</td>\n          <td class="alternate">1</td>\n        </tr><tr>\n          <td class="alternate">2</td>\n          <td class="alternate">3</td>\n          <td class="alternate">4</td>\n          <td class="alternate">5</td>\n          <td class="alternate">6</td>\n          <td class="alternate">7</td>\n          <td class="alternate">8</td>\n        </tr>\n      </tbody>').appendTo(this.table),this.monthYearPane=$(""),this.settings.showMonthYearPicker&&(this.settings.isPopup||this.settings.inPage)&&(this.monthYearPane=$('<div class="monthview-monthyear-pane expandable-area '.concat(this.settings.hideDays?" is-expanded":"",'">\n        <div class="expandable-pane">\n          <div class="content"><div class="picklist-section is-month"></div><div class="picklist-section is-year"></div></div>\n        </div>\n      </div>')));var i="";this.settings.inPage&&this.settings.inPageToggleable&&(i=$('<div class="monthview-inpage-calendar expandable-area'.concat(this.settings.inPageExpanded?" is-expanded":"",'"><div class="expandable-pane"></div></div>'))).find(".expandable-pane").append(this.table),this.settings.hideDays&&(this.table="");var n=this.settings.inPage&&this.settings.inPageToggleable&&""!==this.table?i:this.table;this.settings.showWeekNumber&&(this.weekNumberTable=$('<table class="monthview-week-table" role="application"></table>'),this.weekHeader=$(""+"<thead>\n          <tr>\n            <th><span>".concat(H.translate("WeekNumber"),"</span></th>\n          </tr>\n        </thead>")).appendTo(this.weekNumberTable),this.weekNumber=$("<tbody></tbody>").appendTo(this.weekNumberTable),this.getWeekNumbers(this.settings.month,this.settings.year),this.monthAndWeekContainer=$('<div class="monthview-week-container"></div>'),this.monthAndWeekContainer.append(this.weekNumberTable),this.monthAndWeekContainer.append(n)),this.header=$('<div class="monthview-header"><div class="calendar-toolbar"></div></div>'),"full"===this.settings.headerStyle?this.monthPicker=this.header.find("#monthview-datepicker-field"):this.settings.showToday&&this.header.find(".btn-icon.prev").before('<a class="hyperlink today" href="#">'.concat(H.translate("Today",{locale:this.locale.name,language:this.language}),"</a>")),this.showMonth(this.settings.month,this.settings.year),this.calendar=this.element.addClass("monthview").append(this.header,this.monthYearPane,this.settings.showWeekNumber?this.monthAndWeekContainer:n),this.settings.showWeekNumber&&this.calendar.addClass("has-monthview-week-table"),this.settings.isPopup||this.settings.inPage||this.element.addClass("is-fullsize"),this.addLegend();var s=new Date,a={onOpenCalendar:function(){return t.currentDate},locale:this.settings.locale,language:this.settings.language,year:this.currentYear,month:this.currentMonth,showToday:this.settings.showToday,showNextPrevious:this.settings.showNextPrevious,isMonthPicker:"full"===this.settings.headerStyle,isAlternate:"full"!==this.settings.headerStyle,isMenuButton:!("full"===this.settings.headerStyle&&!this.settings.inPage)&&this.settings.showMonthYearPicker,showViewChanger:this.settings.showViewChanger,onChangeView:this.settings.onChangeView,attributes:this.settings.attributes,inPage:this.settings.inPage,inPageTitleAsButton:this.settings.inPageTitleAsButton,hitbox:this.settings.hitbox};return"function"==typeof this.settings.disable.callback?$.when(this.isDateDisabled(s.getFullYear(),s.getMonth(),s.getDate())).then((function(t){a.disableToday=t})):a.disableToday=this.isDateDisabled(s.getFullYear(),s.getMonth(),s.getDate()),this.calendarToolbarEl=this.header.find(".calendar-toolbar"),this.calendarToolbarAPI=new Cn(this.calendarToolbarEl[0],a),this.handleEvents(),E.addAttributes(this.element,this,this.settings.attributes),this},buildRange:function(){var t=this,e=864e5,i=new Date(this.settings.displayRange.start),n=new Date(this.settings.displayRange.end),s=Gi.isDaylightSavingTime(i)&&!Gi.isDaylightSavingTime(n)?n:new Date(n.getTime()+e),a=Gi.firstDayOfWeek(i,this.settings.firstDayOfWeek,!0),o=Math.ceil((s-a)/6048e5);if(this.settings.showMonthYearPicker=!1,this.settings.inPage=M.toBoolean(this.settings.inPage),this.settings.inPage){var r="is-inpage".concat(this.settings.inPageToggleable?"":" not-toggleable");this.element.addClass(r),this.settings.yearsAhead=4,this.settings.yearsBack=1}this.setCurrentCalendar(),this.table=$('<table class="monthview-table" aria-label="'.concat(H.translate("Calendar",{locale:this.locale.name}),'" role="application"></table>')),this.dayNames=$("<thead>\n        <tr>\n          <th>SU</th>\n          <th>MO</th>\n          <th>TU</th>\n          <th>WE</th>\n          <th>TH</th>\n          <th>FR</th>\n          <th>SA</th>\n        </tr>\n      </thead>").appendTo(this.table),this.day="\n    <tr>\n      <td></td>\n      <td></td>\n      <td></td>\n      <td></td>\n      <td></td>\n      <td></td>\n      <td></td>\n    </tr>\n    ",this.days=$(""+"<tbody>\n       ".concat(Array(o).fill(this.day).join(""),"\n      </tbody>")).appendTo(this.table),this.monthYearPane=$(""),this.settings.showMonthYearPicker&&(this.settings.isPopup||this.settings.inPage)&&(this.monthYearPane=$('<div class="monthview-monthyear-pane expandable-area '.concat(this.settings.hideDays?" is-expanded":"",'">\n        <div class="expandable-pane">\n          <div class="content"><div class="picklist-section is-month"></div><div class="picklist-section is-year"></div></div>\n        </div>\n      </div>')));var l="";this.settings.inPage&&this.settings.inPageToggleable&&(l=$('<div class="monthview-inpage-calendar expandable-area'.concat(this.settings.inPageExpanded?" is-expanded":"",'"><div class="expandable-pane"></div></div>'))).find(".expandable-pane").append(this.table),this.settings.hideDays&&(this.table="");var c=this.settings.inPage&&this.settings.inPageToggleable&&""!==this.table?l:this.table;return this.header=$('<div class="monthview-header"><div class="calendar-toolbar"></div></div>'),this.header.addClass("hidden"),this.showRange(this.settings.displayRange.start,this.settings.displayRange.end),this.calendar=this.element.addClass("monthview").append(this.header,this.monthYearPane,c),this.settings.isPopup||this.settings.inPage||this.element.addClass("is-fullsize"),this.addLegend(),this.calendarToolbarEl=this.header.find(".calendar-toolbar"),this.calendarToolbarAPI=new Cn(this.calendarToolbarEl[0],{onOpenCalendar:function(){return t.currentDate},locale:this.settings.locale,language:this.settings.language,year:this.currentYear,month:this.currentMonth,showToday:this.settings.showToday,showNextPrevious:this.settings.showNextPrevious,isMonthPicker:"full"===this.settings.headerStyle,isAlternate:"full"!==this.settings.headerStyle,isMenuButton:!("full"===this.settings.headerStyle&&!this.settings.inPage)&&this.settings.showMonthYearPicker,showViewChanger:this.settings.showViewChanger,onChangeView:this.settings.onChangeView,attributes:this.settings.attributes,inPage:this.settings.inPage,inPageTitleAsButton:this.settings.inPageTitleAsButton}),this.handleEvents(),E.addAttributes(this.element,this,this.settings.attributes),this},setCurrentCalendar:function(){this.currentCalendar=H.calendar(this.locale.name,this.language,this.settings.calendarName),this.isIslamic="islamic-umalqura"===this.currentCalendar.name,this.isRTL="right-to-left"===(this.locale.direction||this.locale.data.direction),this.conversions=this.currentCalendar.conversions},setMonthYearFromPane:function(){var t=parseInt(this.monthYearPane.find(".is-year .is-selected a").attr("data-year"),10),e=parseInt(this.monthYearPane.find(".is-month .is-selected a").attr("data-month"),10);isNaN(e)||isNaN(t)||this.showMonth(e,t),this.monthYearPane.data("expandablearea").close()},loadLegend:function(t){if(!(void 0===t||t.length<1)){this.settings.legend=t,this.addLegend();var e=this,i=this.currentMonth,n=this.currentYear;this.days.find("td").removeClass("is-colored"),t.forEach((function(t){if(t.dates&&t.dates.forEach((function(t){var i=new Date(t),n=i.getMonth(),s=i.getDate(),a=i.getFullYear(),o=e.days.find("td[data-key=".concat(M.padDate(a,n,s),"]"));o.length&&e.setLegendColor(o,a,n,s)})),t.dayOfWeek){var s=new Date(n,i+1,0),a=s.getDate(),o=s.getDay();t.dayOfWeek.forEach((function(t){var s;o===t?(s=e.days.find("td[data-key=".concat(M.padDate(n,i,a),"]")),e.setLegendColor(s,n,i,a)):o>t&&(s=e.days.find("td[data-key=".concat(M.padDate(n,i,a-(o-t)),"]")),e.setLegendColor(s,n,i,a-(o-t)));for(var r=a-(o+1);r-(6-t)>0;r-=7)s=e.days.find("td[data-key=".concat(M.padDate(n,i,r-(6-t)),"]")),e.setLegendColor(s,n,i,r-(6-t))}))}}))}},getWeekCount:function(t,e){var i=new Date(e,t-1,1),n=new Date(e,t,0),s=i.getDay()+n.getDate();return Math.ceil(s/7)},getWeekNumbers:function(t,e){this.weekNumber.empty();for(var i="",n=this.getWeekCount(t+1,e),s=new Date(e,t,1),a=new Date(s.getFullYear(),0,1),o=Math.ceil(((s.getTime()-a.getTime())/864e5+a.getDay()+1)/7),r=0;r<n;r++)i+="<tr><td><span>".concat(o+r,"</span></td></tr>");this.weekNumber.append(i)},showMonth:function(t,e){var i=this;this.element.trigger("beforemonthrendered");var n,s=this,a=this.settings,o=new Date;if(o.setHours(0),o.setMinutes(0),o.setSeconds(0),n=this.isIslamic?a.activeDate||H.gregorianToUmalqura(o):a.activeDate&&a.activeDate.getDate()?a.activeDate:o,this.setCurrentCalendar(),this.isIslamic)if(a.activeDateIslamic)n=a.activeDateIslamic;else{var r=new Date(e,t,this.currentDay||this.settings.day),l=H.gregorianToUmalqura(r);this.todayDateIslamic=H.gregorianToUmalqura(o),a.activeDateIslamic=[],a.activeDateIslamic[0]=l[0],a.activeDateIslamic[1]=l[1],a.activeDateIslamic[2]=l[2],e=l[0],t=l[1],n=l,this.currentDay=l[2]}e.toString().length<4&&(e=(new Date).getFullYear()),12===t&&(e++,t=0,this.currentMonth=t,this.currentYear=e,this.currentDate.setFullYear(e),this.currentDate.setMonth(t)),t<0&&(e--,t=11,this.currentMonth=t,this.currentYear=e,this.currentDate.setFullYear(e),this.currentDate.setMonth(t)),this.currentDay=this.currentDay||this.settings.day,this.currentCalendar&&this.currentCalendar.days||(this.currentCalendar=H.calendar(this.locale.name,this.language,this.settings.calendarName));var c=this.currentCalendar.days.narrow;c=c||this.currentCalendar.days.abbreviated,a.isPopup||a.inPage||(c=this.currentCalendar.days.abbreviated);var d=this.currentCalendar.months.wide[t];this.currentMonth=t,this.currentYear=e,this.settings.showWeekNumber&&this.getWeekNumbers(this.currentMonth,this.currentYear);var h=this.currentCalendar.firstDayofWeek||0;if(a.firstDayOfWeek&&(h=a.firstDayOfWeek),this.dayNames.find("th").each((function(t){$(this).text(c[(t+h)%7])})),this.yearFirst=this.currentCalendar.dateFormat.year&&"y"===this.currentCalendar.dateFormat.year.substr(1,1),this.header.find(".month").attr("data-month",t).text("".concat(g.stripTags(d)," ")),this.header.find(".year").text(" ".concat(e)),this.yearFirst&&!this.isIslamic&&!this.isRTL){n.setFullYear(e);var u=H.formatDate(n,{date:"year",locale:this.locale.name}).split(" ")[0];this.header.find(".year").text("".concat(u," ")),this.header.find(".year").insertBefore(this.header.find(".month"))}"full"===a.headerStyle&&this.calendarToolbarAPI&&this.calendarToolbarAPI.setInternalDate(this.isIslamic?[e,t,1]:new Date(e,t,1)),this.appendMonthYearPicker(t,e);var p,f,m,v=(this.firstDayOfMonth(e,t)-h+7)%7,b=this.daysInMonth(e,t+(this.isIslamic?1:0)),y=this.daysInMonth(e,t+(this.isIslamic?0:1)),w=1,x=1,C=!1,k=function(t,e){C=e,t.addClass("is-selected".concat(a.range.useRange?" range":"")).attr("aria-selected","true").attr("tabindex","0")};if(this.dayMap=[],this.days.find("td").each((function(i){var o=$(this).removeClass("alternate prev-month next-month is-selected range is-today"),r=a.inPage?" is-ripple":"";if(o.removeAttr("aria-selected"),o.removeAttr("tabindex"),i<v&&(m=b-v+1+i,f=0===t?11:t-1,p=0===t?e-1:e,s.setDisabled(o,p,f,m),s.setLegendColor(o,p,f,m),s.dayMap.push({key:M.padDate(p,f,m),elem:o}),o.addClass("alternate prev-month").html('<span class="day-container'.concat(r,'"><span aria-hidden="true" class="day-text">').concat(g.stripTags(m),"</span></span>")),o.attr("data-key",M.padDate(p,f,m))),i>=v&&x<=y){if(s.dayMap.push({key:M.padDate(e,t,x),elem:o}),o.html('<span class="day-container'.concat(r,'"><span aria-hidden="true" class="day-text">').concat(g.stripTags(x),"</span></span>")),o.attr("data-key",M.padDate(e,t,x)),s.isIslamic)x===n[2]&&k(o,!0);else{var l=n.getHours(),c=n.getMinutes(),d=s.isSeconds?n.getSeconds():0,h=function(t){return t?t.setHours(l,c,d,0):0},u=h(new Date(e,t,x)),C=s.currentDate||n;a.day&&!a.activeDate||u!==h(C)||k(o,!0)}x===s.todayDay&&s.currentMonth===s.todayMonth&&s.currentYear===s.todayYear&&o.addClass("is-today"),o.attr("aria-label",H.formatDate(new Date(s.currentYear,s.currentMonth,x),{date:"full",locale:s.locale.name}));var T=M.padDate(s.currentYear,s.currentMonth,x);return o.attr("data-key",T),s.setDisabled(o,e,t,x),s.setLegendColor(o,e,t,x),o.attr("role","link"),void x++}x>=y+1&&(m=w,f=11===t?0:t+1,p=11===t?e+1:e,s.dayMap.push({key:M.padDate(p,f,m),elem:o}),s.setDisabled(o,p,f,m),s.setLegendColor(o,p,f,m),o.addClass("alternate next-month").html('<span class="day-container'.concat(r,'"><span aria-hidden="true" class="day-text">').concat(w,"</span></span>")),o.attr("data-key",M.padDate(p,f,m)),w++)})),!C&&!a.range.useRange){var T=s.dayMap.filter((function(n){return n.key===M.padDate(e,t,i.settings.day)}));T.length&&k(T[0].elem,!1)}var A=this.days.find("tr").eq(5);7===A.find("td.alternate").length?A.hide():A.show(),this.currentDate||(this.isIslamic?(this.currentDateIslamic=[this.currentYear,this.currentMonth,this.currentDay],this.currentDate=H.umalquraToGregorian(this.currentYear,this.currentMonth,this.currentDay)):this.currentDate=new Date(this.currentYear,this.currentMonth,this.currentDay)),this.setRangeSelection(),this.validatePrevNext(),a.range.useRange||0!==this.element.find("td.is-selected").length||(this.element.find("td[tabindex]").removeAttr("tabindex"),this.element.find("td:not(.alternate) .day-text").first().closest("td").attr("tabindex","0")),this.element.trigger("monthrendered",{year:e,month:t,elem:this.element,api:this}),this.element.trigger("aftermonthrendered")},showRange:function(t,e){var i,n=this,s=new Date,a=new Date(t),o=new Date(e),r=parseInt(a.getMonth(),10),l=parseInt(a.getFullYear(),10),c=Gi.monthDiff(a,o),d=this.settings;if(d.disable.minDate||d.disable.MaxDate||(d.disable.minDate=Gi.subtract(a,1,"days").toISOString(),d.disable.maxDate=Gi.add(o,1,"days").toISOString()),s.setHours(0),s.setMinutes(0),s.setSeconds(0),i=this.isIslamic?d.activeDate||H.gregorianToUmalqura(s):d.activeDate&&d.activeDate.getDate()?d.activeDate:s,this.setCurrentCalendar(),this.isIslamic)if(d.activeDateIslamic)i=d.activeDateIslamic;else{var h=new Date(l,r,this.currentDay||this.settings.day),u=H.gregorianToUmalqura(h);this.todayDateIslamic=H.gregorianToUmalqura(s),d.activeDateIslamic=[],d.activeDateIslamic[0]=u[0],d.activeDateIslamic[1]=u[1],d.activeDateIslamic[2]=u[2],l=u[0],r=u[1],i=u,this.currentDay=u[2]}l.toString().length<4&&(l=(new Date).getFullYear()),this.currentCalendar&&this.currentCalendar.days||(this.currentCalendar=H.calendar(this.locale.name,this.language,this.settings.calendarName));var p=this.currentCalendar.days.narrow;p=p||this.currentCalendar.days.abbreviated,d.isPopup||d.inPage||(p=this.currentCalendar.days.abbreviated);var f=this.currentCalendar.firstDayofWeek||0;d.firstDayOfWeek&&(f=d.firstDayOfWeek),this.dayNames.find("th").each((function(t){$(this).text(p[(t+f)%7])}));var m=Gi.firstDayOfWeek(a,this.settings.firstDayOfWeek,!0),v=m.getDate(),b=!1,y=m.getMonth(),w=m.getFullYear(),x=y,C=w;c=Gi.monthDiff(m,o);for(var k=new Map,T=0;T<=c;T++)(x=y+T)>11&&(C+=1,x%=12),k.set(x,this.daysInMonth(C,x+(this.isIslamic?0:1)));var A=function(t,e){b=e,t.addClass("is-selected".concat(d.range.useRange?" range":"")).attr("aria-selected","true").attr("tabindex","0")},S=k.get(y),E=new Set(k.keys());if(this.dayMap=[],this.days.find("td").each((function(){var t,e=$(this).removeClass("alternate prev-month next-month is-selected range is-today"),s=d.inPage?" is-ripple":"";if(e.removeAttr("aria-selected"),e.removeAttr("tabindex"),n.isIslamic)v===i[2]&&A(e,!0);else{var a=i.getHours(),o=i.getMinutes(),r=n.isSeconds?i.getSeconds():0,l=function(t){return t?t.setHours(a,o,r,0):0};l(new Date(w,y,v))===l(n.currentDate||i)&&A(e,!0)}if(v>S&&(++y>11&&(y=0,w++),S=k.get(y),v=1),v===n.todayDay&&y===n.todayMonth&&w===n.todayYear&&e.addClass("is-today"),n.dayMap.push({key:M.padDate(w,y,v),elem:e}),e.html('<span class="day-container'.concat(s,'"><span aria-hidden="true" class="day-text">').concat(g.stripTags(v),"</span></span>")),e.attr("data-key",M.padDate(w,y,v)),n.isIslamic)v===i[2]&&A(e,!0);else{var c=i.getHours(),h=i.getMinutes(),u=n.isSeconds?i.getSeconds():0,p=function(t){return t?t.setHours(c,h,u,0):0};p(new Date(C,x,v))===p(n.currentDate||i)&&A(e,!0)}v===n.todayDay&&y===n.todayMonth&&w===n.todayYear&&e.addClass("is-today"),e.attr("aria-label",H.formatDate(new Date(w,y,v),{date:"full",locale:n.locale.name}));var f=M.padDate(w,y,v);e.attr("data-key",f),n.setDisabled(e,w,y,v),n.setLegendColor(e,w,y,v),n.setMonthLabel(e,w,y,v,E,null===(t=d.displayRange.months)||void 0===t||null==t?void 0:t.showAbbr),e.attr("role","link"),v++})),!b){var D=n.dayMap.filter((function(t){return t.key===M.padDate(s.getFullYear(),s.getMonth(),s.getDate())}));D.length&&A(D[0].elem,!1)}this.element.trigger("rangerendered",{year:l,month:r,elem:this.element,api:this})},getTime:function(t,e){if(this.isIslamic)return t[0]+"0".concat(t[1]).slice(-2)+"0".concat(t[2]).slice(-2);if(e){var i=new Date(t);return i.setHours(0,0,0),i.getTime()}return t.getTime()},setRangeSelection:function(){if(this.settings.range.useRange){var t={};t.date=new Date(this.currentYear,this.currentMonth,1),t.date.setDate(t.date.getDate()-(this.days.find(".prev-month:visible").length+1));var e=M.padDate(t.date.year||t.date[0],t.date.month||t.date[1],t.date.day||t.date[2]);t.cell=this.days.find('[data-key="'.concat(e,'"]')),this.setRangeOnCell(!this.settings.range.second&&t.cell)}},appendMonthYearPicker:function(t,e){var i=this;if(this.settings.showMonthYearPicker){var n='<ul class="picklist is-month">',s=this.settings.inPage?' class="is-ripple"':"",a=this.currentCalendar.months.wide;n+='<li class="picklist-item up"><a href="#" tabindex="0"'.concat(s,'><span class="audible">').concat(H.translate("PreviousMonth"),'</span><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-caret-up"></use></svg></a></li>');for(var o=t<6?0:6,r=0;r<6;r++){var l=r+o,c=a[l];n+='<li class="picklist-item'.concat(l===t?" is-selected ":"",'"><a href="#" tabindex="').concat(l===t?"0":"-1",'" data-month="').concat(l,'"').concat(s,">").concat(c,"</a></li>")}n+='<li class="picklist-item down">\n        <a tabindex="0"'.concat(s,'>\n          <span class="audible">').concat(H.translate("NextMonth"),'</span>\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-caret-down"></use>\n          </svg>\n        </a>\n      </li></ul>'),this.monthYearPane.find(".picklist-section.is-month").empty().append(n);var d=[],h='<ul class="picklist is-year">';h+='<li class="picklist-item up"><a href="#" tabindex="0"'.concat(s,'><span class="audible">').concat(H.translate("PreviousYear"),'</span><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-caret-up"></use></svg></a></li>');for(var u=this.settings.yearsBack;u>=1;u--)d.push(parseInt(e,10)-u);d.push(e);for(var p=1;p<=this.settings.yearsAhead;p++)d.push(parseInt(e,10)+p);d.map((function(t){h+='<li class="picklist-item'.concat(e===t?" is-selected ":"",'"><a href="#" tabindex="').concat(e===t?"0":"-1",'" data-year="').concat(t,'"').concat(s,">").concat(t,"</a></li>")})),h+='<li class="picklist-item down">\n        <a tabindex="0"'.concat(s,'>\n          <span class="audible">').concat(H.translate("NextYear"),'</span>\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-caret-down"></use>\n          </svg>\n        </a>\n      </li></ul>'),this.monthYearPane.find(".picklist-section.is-year").empty().append(h),this.yearFirst&&this.monthYearPane.addClass("is-yearfirst"),E.addAttributes(this.monthYearPane.find(".picklist-section.is-year .picklist-item.up a"),this,this.settings.attributes,"btn-picklist-year-up"),E.addAttributes(this.monthYearPane.find(".picklist-section.is-year .picklist-item.down a"),this,this.settings.attributes,"btn-picklist-year-down"),this.settings.inPage&&(E.addAttributes(this.monthYearPane.find(".btn-inpage-cal"),this,this.settings.attributes,"btn-inpage-cal"),E.addAttributes(this.monthYearPane.find(".btn-cal-month-year"),this,this.settings.attributes,"btn-cal-month-year"),E.addAttributes(this.monthYearPane.find(".btn-monthyear-pane"),this,this.settings.attributes,"btn-monthyear"),E.addAttributes(this.monthYearPane.find(".apply"),this,this.settings.attributes,"btn-apply"),E.addAttributes(this.monthYearPane.find(".picklist-section.is-month .picklist-item.up a"),this,this.settings.attributes,"btn-picklist-month-up"),E.addAttributes(this.monthYearPane.find(".picklist-section.is-month .picklist-item.down a"),this,this.settings.attributes,"btn-picklist-month-down")),this.monthYearPane.find(".is-year .picklist-item").not(".up, .down").each((function(){var t=$(this).find("a");E.addAttributes(t,i,i.settings.attributes,"btn-picklist-".concat(t.text().toLowerCase()))})),this.monthYearPane.find(".is-month .picklist-item").not(".up, .down").each((function(t){E.addAttributes($(this).find("a"),i,i.settings.attributes,"btn-picklist-".concat(t))}))}},firstDayOfMonth:function(t,e){if(this.isIslamic){var i=H.umalquraToGregorian(t,e,1);return null===i?1:i.getDay()}return new Date(t,e,1).getDay()},daysInMonth:function(t,e){if(this.isIslamic){for(var i=this.conversions.yearInfo[this.islamicYearIndex(t)][0],n=0;n<=e;n++){if(n===e)return 29+(1&i);i>>=1}return 0}return new Date(t,e,0).getDate()},islamicYearIndex:function(t){var e=t-1318;return e<0||e>=this.conversions.yearInfo.length?0:e},setDisabled:function(t,e,i,n){var s=this.settings;function a(){t.addClass("is-disabled").attr("aria-disabled","true").removeClass("is-selected range").removeAttr("aria-selected")}if(t.removeClass("is-disabled").removeAttr("aria-disabled"),"function"==typeof s.disable.callback)$.when(this.isDateDisabled(e,i,n)).then((function(t){t&&a()}));else{var o=this.isDateDisabled(e,i,n);(o&&!s.disable.isEnable||!o&&s.disable.isEnable)&&a()}},setMonthLabel:function(t,e,i,n,s,a){var o=this.settings.inPage?' class="is-ripple"':"";if(s.has(i)){var r=t.text(),l=a?this.currentCalendar.months.abbreviated[i]:this.currentCalendar.months.wide[i];s.delete(i),t.html('<span class="day-container'.concat(o,'"><span aria-hidden="true" class="day-text month-label">').concat(l," ").concat(r,"</span></span>"))}},isDateDisabled:function(t,e,i){var n=this.settings;if("function"==typeof n.disable.callback){var s=$.Deferred();return $.when(n.disable.callback(t,e,i)).then((function(t){s.resolve(t)})),s.promise()}var a="string"==typeof n.disable.minDate?n.disable.minDate.replace(/-/g,"/"):n.disable.minDate,o="string"==typeof n.disable.maxDate?n.disable.maxDate.replace(/-/g,"/"):n.disable.maxDate,r=new Date(a).setHours(0,0,0,0),c=new Date(o).setHours(0,0,0,0),d=this.isIslamic?H.umalquraToGregorian(t,e,i):new Date(t,e,i);if(!d)return!1;if(-1!==n.disable.dayOfWeek.indexOf(d.getDay()))return!0;if(n.legend)for(var h=0;h<n.legend.length;h++)if(n.legend[h].disableWeek&&-1!==n.legend[h].disableWeek.indexOf(d.getDay()))return!0;var u=d.getFullYear();if((d=d.setHours(0,0,0,0))<=r||d>=c)return!0;/string|number/.test(l(n.disable.years))&&(n.disable.years=[n.disable.years]);for(var p=0,g=n.disable.years.length;p<g;p++)if(u===Number(n.disable.years[p]))return!0;n.disable.dates.length&&"string"==typeof n.disable.dates&&(n.disable.dates=[n.disable.dates]);for(var f=0,m=n.disable.dates.length;f<m;f++){if(d===new Date(n.disable.dates[f]).setHours(0,0,0,0))return!0}return!1},getDateRange:function(t,e,i){var n=[],s=this.isIslamic?u(t):new Date(t);for(i=void 0!==i?i:this.settings.range.includeDisabled;this.getTime(e)>=this.getTime(s);)(i||!i&&!this.isDateDisabled(this.isIslamic?s[0]:s.getFullYear(),this.isIslamic?s[1]:s.getMonth(),this.isIslamic?s[2]:s.getDate()))&&(this.isIslamic?n.push(s):n.push(new Date(s))),this.isIslamic?s[2]+=1:s.setDate(s.getDate()+1);return n},setLegendColor:function(t,e,i,n){if(this.settings.showLegend&&t[0]){var s=this,a=this.settings.inPage?t.find(".day-text"):t;if(a[0]){var o=this.getLegendColor(e,i,n);if(a[0].style.backgroundColor="",a.off("mouseenter.legend mouseleave.legend"),o){var r;if(-1===o.indexOf("#")){var l=o.replace(/[0-9]/g,""),c=10*o.substr(o.length-2,2);o=ft.themeColors().palette[l][c].value}var d=O.hexToRgba(o,.3),h=O.hexToRgba(o,.7),u=null===(r=this.table[0])||void 0===r||null==r?void 0:r.querySelector("thead th");t.addClass("is-colored"),a[0].setAttribute("data-hex",o),s.settings.inPage?a[0].style.setProperty("--legendcolor",d):a[0].style.backgroundColor=d,a.on("mouseenter.legend",(function(){var t=$(this),e=t.find("span")[0],i=t.find(".day-text")[0],n=window.getComputedStyle(u).getPropertyValue("color");this.style.backgroundColor=h,s.settings.inPage&&(this.style.color=n),e&&(e.style.backgroundColor="transparent"),i&&(i.style.backgroundColor="transparent")})).on("mouseleave.legend",(function(){var t=$(this),e=t.find("span")[0],i=t.find(".day-text")[0];s.settings.inPage?(this.style.color="",this.style.backgroundColor=""):this.style.backgroundColor=d,e&&(e.style.backgroundColor=""),i&&(i.style.backgroundColor="")}))}}}},getLegendColor:function(t,e,i){var n=this.settings;if(n.showLegend&&n.legend){for(var s=new Date(t,e,i),a=s.setHours(0,0,0,0),o=0;o<n.legend.length;o++){var r=n.legend[o];if(r.dayOfWeek&&-1!==r.dayOfWeek.indexOf(s.getDay()))return r.color;if(r.dates)for(var l=0;l<r.dates.length;l++){if(a===new Date(r.dates[l]).setHours(0,0,0,0))return r.color}}return""}},handleEvents:function(){var t=this,e=this,i=this.settings;if(this.element.off("updated.".concat(kn)).on("updated.".concat(kn),(function(){t.updated()})),this.header.off("click.monthview").on("click.monthview",".btn-icon.prev, .btn-icon.next",(function(){var t=$(this).is(".next"),n={},s={month:e.currentMonth,year:e.currentYear};i.range.useRange?(t?n.date=new Date(s.year,s.month+1,e.element.find(".next-month:visible").length+1):(n.date=new Date(s.year,s.month,1),n.date.setDate(n.date.getDate()-(e.days.find(".prev-month:visible").length+1))),s.month+=t?1:-1):(e.currentMonth+=t?1:-1,e.currentDate.setMonth(e.currentMonth),e.currentYear=parseInt(e.element[0].querySelector("span.year").innerText,10),e.currentDate.setFullYear(e.currentYear),s.month=e.currentMonth,s.year=e.currentYear),e.showMonth(s.month,s.year),i.range.useRange&&(n.formatedDate=H.formatDate(n.date,{date:"full",locale:e.locale.name}),n.cell=e.days.find('[aria-label="'.concat(n.formatedDate,'"]')),e.setRangeOnCell(!i.range.second&&n.cell))})),i.range.useRange&&(this.header.off("mouseover.datepicker").on("mouseover.datepicker","button",(function(){i.range.extra&&e.setRangeOnCell($(this).is(".next")?i.range.extra.maxCell:i.range.extra.minCell)})).off("focus.datepicker").on("focus.datepicker","button:not(.hide-focus)",(function(){i.range.extra&&e.setRangeOnCell($(this).is(".next")?i.range.extra.maxCell:i.range.extra.minCell)})),this.days.off("mouseover.datepicker").on("mouseover.datepicker","td",(function(){e.setRangeOnCell(this)}))),this.calendarToolbarEl&&this.calendarToolbarEl.off("change-date.monthview").on("change-date.monthview",(function(e,i){i.isToday&&t.settings.isPopup||(i.isApply?t.setMonthYearFromPane():i.isToday?t.setToday():t.selectDay(i.selectedDate,!1,!0))})),i.selectable&&this.element.addClass("is-selectable").off("click.monthview-day").on("click.monthview-day","td",(function(e){var i=e.currentTarget.getAttribute("data-key");t.lastClickedKey=i,e.currentTarget.classList.contains("is-disabled")||t.selectDay(i,!1,!0)})),i.inPage){var n=this.element.find(".monthview-inpage-calendar");n.expandablearea({animationSpeed:150,trigger:this.element.find(i.showMonthYearPicker?"#btn-inpage-cal":"#btn-cal-month-year")}),n.on("afterexpand",(function(){t.element.trigger("monthviewexpanded")})),n.on("aftercollapse",(function(){t.element.trigger("monthviewcollapsed")}));var s=n.data("expandablearea");if(i.showMonthYearPicker){var a=this.element.find(".monthview-monthyear-pane");a.expandablearea({animationSpeed:150,trigger:this.element.find("#btn-monthyear-pane")}).on("beforeexpand",(function(){var t=s&&!s.isExpanded();a.css("position",t?"static":"")}))}this.element.off("click.monthview-ripple").on("click.monthview-ripple",".is-ripple",(function(t){e.setRipple(this,t)}))}return this.handleMonthYearPane().handleKeys(),this},setRipple:function(t,e){var i;if(t&&null!==(i=t.classList)&&void 0!==i&&(null==i?void 0:i.contains("is-ripple"))){var n=e.touches?e.touches[0]:e,s=t.getBoundingClientRect(),a=2*Math.sqrt(Math.pow(s.width,2)+Math.pow(s.height,2)),o=n.clientX?n.clientX-s.left:t.offsetWidth/2,r=n.clientY?n.clientY-s.top:t.offsetHeight/2,l=t.style.cssText,c=t.getAttribute("data-hex"),d=O.getContrastColor(c),h={ripple:O.getLuminousColorShade(c,"white"===d?"0.3":"-0.3"),elem:O.hexToRgba(c,.7)},u=null!==c?"--ripple-background: ".concat(h.ripple,"; background-color: ").concat(h.elem,"; "):"";t.style.cssText="".concat(u,"--s: 0; --o: 1"),t.offsetTop,t.style.cssText="".concat(u,"--t: 1; --o: 0; --d: ").concat(a,"; --x:").concat(o,"; --y:").concat(r,";"),$(t).off("transitionend.monthview-ripple").on("transitionend.monthview-ripple",(function(e){var i;"transform"===(e.propertyName||(null===(i=e.originalEvent)||void 0===i||null==i?void 0:i.propertyName))&&(t.style.cssText=l)}))}},handleMonthYearPane:function(){var t=this,e=this.settings,i=e.inPage?' class="is-ripple"':"",n=function(n){var s=t.monthYearPane[0].querySelector(".picklist.is-month"),a=t.currentCalendar.months.wide,o=s.children,r=o.length-2,l=parseInt(o["up"===n?1:r].querySelector("a").getAttribute("data-month"),10),c=$(s),d=c.find("li.is-selected"),h=d.index();d.removeClass("is-selected");var u=o[1],p=o[r],g=$('<li class="picklist-item"></li>');if("up"===n){var f=0===l?11:l-1,m='<a href="#" tabindex="-1" data-month="'.concat(f,'"').concat(i,">").concat(a[f],"</a>");g.append(m),p.remove(),g.insertBefore(u)}else{var v=11===l?0:l+1,b='<a href="#" tabindex="-1" data-month="'.concat(v,'"').concat(i,">").concat(a[v],"</a>");g.append(b),u.remove(),g.insertAfter(p)}e.inPage||$(c.find("li.picklist-item").get(h)).addClass("is-selected").attr("tabindex","0")},s=function(n){var s=t.monthYearPane[0].querySelector(".picklist.is-year"),a=s.children,o=a.length-2,r=parseInt(a["up"===n?1:o].querySelector("a").getAttribute("data-year"),10),l=$(s),c=l.find("li.is-selected"),d=c.index();c.removeClass("is-selected");var h=a[1],u=a[o],p=$('<li class="picklist-item"></li>');if("up"===n){var g=r-1,f=$('<a href="#" tabindex="-1" data-year="'.concat(g,'"').concat(i,">").concat(g,"</a>"));E.addAttributes(f,t,e.attributes,"btn-picklist-".concat(g)),p.append(f),u.remove(),p.insertBefore(h)}else{var m=r+1,v=$('<a href="#" tabindex="-1" data-year="'.concat(m,'"').concat(i,">").concat(m,"</a>"));E.addAttributes(v,t,e.attributes,"btn-picklist-".concat(m)),p.append(v),h.remove(),p.insertAfter(u)}e.inPage||$(l.find("li.picklist-item").get(d)).addClass("is-selected").attr("tabindex","0")},a=null;this.monthYearPane.off("touchstart.monthviewpane mousedown.monthviewpane").on("touchstart.monthviewpane mousedown.monthviewpane",".picklist.is-year li",(function(t){return e.inPage&&"li"===t.target.tagName.toLowerCase()||(a=setInterval((function(){return t.currentTarget.classList.contains("up")&&s("up"),t.currentTarget.classList.contains("down")&&s("down"),t.stopImmediatePropagation(),t.stopPropagation(),!1}),200)),!0})).off("touchend.monthviewpane touchcancel.monthviewpane mouseup.monthviewpane mouseout.monthviewpane").on("touchend.monthviewpane touchcancel.monthviewpane mouseup.monthviewpane mouseout.monthviewpane",".picklist.is-year li",(function(){return clearInterval(a),!0}));var o=function(i,n){var s,a=function(i){var n=t.monthYearPane[0].querySelector(".is-".concat(i," .is-selected a"));return!n&&e.inPage?t.monthYearPane[0].querySelector(".is-".concat(i," .picklist-item:last-child a")):n};s=e.inPage?{month:parseInt(a("month").getAttribute("data-month"),10),year:parseInt(a("year").getAttribute("data-year"),10)}:"is-month"===n?{month:parseInt(i.getAttribute("data-month"),10),year:parseInt(a("year").getAttribute("data-year"),10)}:{month:parseInt(a("month").getAttribute("data-month"),10),year:parseInt(i.getAttribute("data-year"),10)},e.range.useRange||(t.currentMonth=s.month,t.currentDate.setMonth(t.currentMonth),t.currentYear=s.year,t.currentDate.setFullYear(t.currentYear),s.month=t.currentMonth,s.year=t.currentYear),function(e,i){var n=t.monthYearPane[0].querySelector(".picklist.".concat(i," .is-selected"));f.removeClass(n,"is-selected"),n&&n.querySelector("a").setAttribute("tabindex","-1"),f.addClass(e.parentNode,"is-selected"),e.setAttribute("tabindex","0")}(i,n),t.element.hasClass("".concat(n,"only"))&&t.monthYearPane.parent().find("button.is-select-month").click()};this.monthYearPane.off("click.picklist-month").on("click.picklist-month",".picklist.is-month li",(function(t){e.inPage&&"li"===t.target.tagName.toLowerCase()||(t.currentTarget.classList.contains("up")?n("up"):t.currentTarget.classList.contains("down")?n("down"):(o(t.target,"is-month"),t.preventDefault()))})),this.monthYearPane.off("click.picklist-month-a").on("click.picklist-month-a",".picklist.is-month li a",(function(t){t.preventDefault()})),this.monthYearPane.off("click.picklist-year").on("click.picklist-year",".picklist.is-year li",(function(t){e.inPage&&"li"===t.target.tagName.toLowerCase()||(t.currentTarget.classList.contains("up")?s("up"):t.currentTarget.classList.contains("down")?s("down"):(o(t.target,"is-year"),t.preventDefault()))})),this.monthYearPane.off("click.picklist-year-a").on("click.picklist-year-a",".picklist.is-year li a",(function(t){t.preventDefault()})),this.monthYearPane.on("expand.monthviewpane",(function(){if(!e.hideDays){if(t.element.find(".btn-icon, td.is-selected").attr("disabled","true"),t.element.find("td.is-selected").removeAttr("tabindex"),e.inPage){t.element.find(".btn-icon.prev, .btn-icon.next").hide();var i=t.element.find(".hyperlink.today, .hyperlink.apply");i.hasClass("is-apply")?i.removeClass("hidden"):i.removeClass("today is-ripple").addClass("apply").text(H.translate("Apply",{locale:t.locale.name,language:t.language}))}else t.monthYearPane.find(".content").css("height",t.header.parent().height()-t.header.height()-55),t.element.find(".hyperlink.today").hide();t.element.find(".is-select").removeClass("is-select").addClass("is-select-month-pane"),t.element.find(".is-cancel").removeClass("is-cancel").addClass("is-cancel-month-pane").text(H.translate("Cancel",{locale:t.locale.name,language:t.language}))}setTimeout((function(){t.monthYearPane.find(".is-month .is-selected a").focus(),t.monthYearPane.parent().hasClass("is-yearonly")&&t.monthYearPane.find(".is-year .is-selected a").focus()}))})).on("collapse.monthviewpane",(function(){if(!e.hideDays){if(e.inPage){t.element.find(".btn-icon.prev, .btn-icon.next").show();var i=t.element.find(".hyperlink.apply");i.hasClass("is-apply")?i.addClass("hidden"):i.removeClass("apply").addClass("today").text(H.translate("Today",{locale:t.locale.name,language:t.language}))}else t.element.find(".hyperlink.today").show();t.element.find(".btn-icon").removeAttr("disabled"),t.element.find("td.is-selected").attr("tabindex","0"),t.element.find(".is-select-month-pane").addClass("is-select").removeClass("is-select-month-pane"),t.element.find(".is-cancel-month-pane").addClass("is-cancel").removeClass("is-cancel-month-pane").text(H.translate("Clear",{locale:t.locale.name,language:t.language}))}})).on("aftercollapse.monthviewpane",(function(){t.element.find(".hyperlink.today").addClass("is-ripple")}));var r=function(t,e){var i,n=t.currentTarget,s=t.currentTarget.parentNode,a="prev"===e?s.previousSibling:s.nextSibling;a||(a=s.parentNode.children["prev"===e?s.parentNode.children.length-1:0]);var o=a.querySelector("a");n.setAttribute("tabindex","-1"),null===(i=s.parentNode.querySelector(".is-selected"))||void 0===i||null==i||i.classList.remove("is-selected"),f.addClass(a,"is-selected"),o.setAttribute("tabindex","0"),o.focus()};return this.monthYearPane.off("keydown.monthviewpane").on("keydown.monthviewpane",".picklist-item a",(function(e){var i=e.currentTarget.parentNode.classList.contains("up"),a=e.currentTarget.parentNode.classList.contains("down"),l=e.currentTarget.parentNode.parentNode.classList.contains("is-year"),c=e.currentTarget.parentNode.parentNode.classList.contains("is-month"),d=!1;"ArrowUp"===e.key&&(l||c)?(r(e,"prev"),d=!0):"ArrowDown"===e.key&&(l||c)?(r(e,"next"),d=!0):"Enter"===e.key&&(i||a?(l?s(i?"up":"down"):c&&n(i?"up":"down"),d=!0):(l||c)&&(o(e.target,l?"is-year":"is-month"),d=!0)),d&&(e.preventDefault(),t.setRipple(e.currentTarget,e))})),this},selectDay:function(t,e,i){this.isIslamic&&"string"!=typeof t&&(this.currentDateIslamic=H.gregorianToUmalqura(t),t=M.padDate(this.currentDateIslamic[0],this.currentDateIslamic[1],this.currentDateIslamic[2])),this.isIslamic||"string"==typeof t||(t=M.padDate(t.getFullYear(),t.getMonth(),t.getDate()));var n=this.dayMap.filter((function(e){return e.key===t})),s=parseInt(t.substr(0,4),10),a=parseInt(t.substr(4,2),10)-1,o=parseInt(t.substr(6,2),10);if(this.isIslamic?(this.currentDateIslamic=t,this.currentDate=H.umalquraToGregorian(s,a,o)):this.currentDate=new Date(s,a,o),this.currentYear=s,this.currentMonth=a,this.currentDay=o,(0===n.length||n[0].elem.hasClass("alternate"))&&(this.showMonth(a,s),n=this.dayMap.filter((function(e){return e.key===t}))),0!==!n.length){var r=n[0].elem[0],l={node:r,key:t,day:o,month:a,year:s,close:e};delete this.isKeyClick,this.element.find("td.is-selected").removeClass("is-selected").removeAttr("tabindex"),$(r).addClass("is-selected").attr("tabindex","0").focus(),(i="full"===this.settings.headerStyle||i)&&this.settings.onSelected&&this.settings.onSelected(r,l),i&&this.element.trigger("selected",l)}},setToday:function(){this.selectDay(new Date,!1,!0)},handleKeys:function(){var t=this,e=this.settings;this.element.off("keydown.monthview").on("keydown.monthview",".monthview-table",(function(i){var n=i.keyCode||i.charCode||0,s=$(i.target),a=t.days.find("td:visible"),o=a.length,r=null,l=null,c=!1,d=new Date(e.disable.minDate),h=new Date(e.disable.maxDate),u=function(){t.datepickerApi&&e.range.useRange&&e.range.first&&e.range.first.date&&e.range.second&&e.range.second.date&&t.datepickerApi.resetRange({isData:!0})};if(t.settings.onKeyDown&&!1===t.settings.onKeyDown({e:i,key:n,cell:s,node:t.element}))return i.stopPropagation(),i.preventDefault(),!1;if(40===n){if(c=!0,!t.validateDisplayRange(Gi.add(t.currentDate,7,"days")))return!1;e.range.useRange?(r=a.index(i.target)+7,l=a.eq(r),r<o&&(u(),t.setRangeOnCell(l.is(".is-selected")?null:l),t.setRangeSelBeforeFirstSel(l),t.activeTabindex(l,!0))):e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?((t.currentDate.getMonth()<h.getMonth()||h.getDate()-1>=t.currentDate.getDate()+7)&&t.currentDate.setDate(t.currentDate.getDate()+7),t.selectDay(t.currentDate,!1,!1)):(t.currentDate.setDate(t.currentDate.getDate()+7),t.selectDay(t.currentDate,!1,!1))}if(38===n){if(c=!0,!t.validateDisplayRange(Gi.subtract(t.currentDate,7,"days")))return!1;e.range.useRange?(r=a.index(i.target)-7,l=a.eq(r),r>-1&&(u(),t.setRangeOnCell(l.is(".is-selected")?null:l),t.setRangeSelBeforeFirstSel(l),t.activeTabindex(l,!0))):e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?((t.currentDate.getMonth()>d.getMonth()||d.getDate()+1<=t.currentDate.getDate()-7)&&t.currentDate.setDate(t.currentDate.getDate()-7),t.selectDay(t.currentDate,!1,!1)):(t.currentDate.setDate(t.currentDate.getDate()-7),t.selectDay(t.currentDate,!1,!1))}if(37===n||189===n&&!i.shiftKey){if(c=!0,!t.validateDisplayRange(Gi.subtract(t.currentDate,1,"days")))return!1;e.range.useRange?(r=a.index(i.target)-1,l=a.eq(r),r>-1&&(u(),t.setRangeOnCell(l.is(".is-selected")?null:l),t.setRangeSelBeforeFirstSel(l),t.activeTabindex(l,!0))):e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?((t.currentDate.getMonth()>d.getMonth()||d.getDate()+1!==t.currentDate.getDate())&&t.currentDate.setDate(t.currentDate.getDate()-1),t.selectDay(t.currentDate,!1,!1)):(t.currentDate.setDate(t.currentDate.getDate()-1),t.selectDay(t.currentDate,!1,!1))}if(39===n||187===n&&i.shiftKey){if(c=!0,!t.validateDisplayRange(Gi.add(t.currentDate,1,"days")))return!1;e.range.useRange?(r=a.index(i.target)+1,l=a.eq(r),r<o&&(u(),t.setRangeOnCell(l.is(".is-selected")?null:l),t.setRangeSelBeforeFirstSel(l),t.activeTabindex(l,!0))):e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?((t.currentDate.getMonth()<h.getMonth()||h.getDate()-1!==t.currentDate.getDate())&&t.currentDate.setDate(t.currentDate.getDate()+1),t.selectDay(t.currentDate,!1,!1)):(t.currentDate.setDate(t.currentDate.getDate()+1),t.selectDay(t.currentDate,!1,!1))}if(33!==n||i.altKey||(c=!0,u(),e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?d.getMonth()!==t.currentDate.getMonth()&&(t.currentDate.setMonth(t.currentDate.getMonth()-1),t.selectDay(t.currentDate,!1,!1)):(t.currentDate.setMonth(t.currentDate.getMonth()-1),t.selectDay(t.currentDate,!1,!1))),34!==n||i.altKey||(c=!0,u(),e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?t.currentDate.getMonth()!==h.getMonth()&&(t.currentDate.setMonth(t.currentDate.getMonth()+1),t.selectDay(t.currentDate,!1,!1)):(t.currentDate.setMonth(t.currentDate.getMonth()+1),t.selectDay(t.currentDate,!1,!1))),33===n&&i.ctrlKey&&(c=!0,u(),t.currentDate.setFullYear(t.currentDate.getFullYear()-1),t.selectDay(t.currentDate,!1,!1)),34===n&&i.ctrlKey&&(c=!0,u(),t.currentDate.setFullYear(t.currentDate.getFullYear()+1),t.selectDay(t.currentDate,!1,!1)),36===n){c=!0;var p,g=t.currentDate;u(),e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?d.getMonth()!==t.currentDate.getMonth()?p=new Date(g.getFullYear(),g.getMonth(),1):(p=new Date(d.getFullYear(),d.getMonth(),d.getDate())).setDate(p.getDate()+1):p=new Date(g.getFullYear(),g.getMonth(),1),t.currentDate=p,t.isIslamic&&(t.currentDateIslamic=H.gregorianToUmalqura(t.currentDate)),t.selectDay(t.currentDate,!1,!1)}if(35===n){c=!0;var f,m=t.currentDate;u(),e.disable.restrictMonths&&e.disable.minDate&&e.disable.maxDate?t.currentDate.getMonth()!==h.getMonth()?f=new Date(m.getFullYear(),m.getMonth()+1,0):(f=new Date(h.getFullYear(),h.getMonth(),h.getDate())).setDate(f.getDate()-1):f=new Date(m.getFullYear(),m.getMonth()+1,0),t.currentDate=f,t.isIslamic&&(t.currentDateIslamic=H.gregorianToUmalqura(t.currentDate)),t.selectDay(t.currentDate,!1,!1)}if(84===n){if(e.range.useRange&&t.datepickerApi){u();var v=!(e.range.first&&e.range.first.date&&(!e.range.second||e.range.second&&!e.range.second.date));t.datepickerApi.setToday(v),!v&&t.datepickerApi&&"function"==typeof t.datepickerApi.closeCalendar&&t.datepickerApi.closeCalendar()}else t.setToday();c=!0}if(32===n||13===n){var b;if(c=!0,e.range.useRange)return(!e.range.first||e.range.first&&!e.range.first.date)&&a.removeClass("is-selected"),s.focus().trigger("click"),!1;var y=t.getCellDate(s);t.isIslamic?(t.currentDateIslamic=[y.year,y.month,y.day],t.currentDate=H.umalquraToGregorian(t.currentDateIslamic[0],t.currentDateIslamic[1],t.currentDateIslamic[2])):t.currentDate=new Date(y.year,y.month,y.day),t.selectDay(t.currentDate,!0,!0),t.setRipple(null===(b=i.target)||void 0===b||null==b?void 0:b.querySelector(".is-ripple"),i)}return!c||(i.stopPropagation(),i.preventDefault(),!1)}))},validateDisplayRange:function(t){var e,i,n=M.padDate(t.getFullYear(),t.getMonth(),t.getDate());if(this.settings.disable.minDate&&this.settings.disable.maxDate?(e=new Date(this.settings.disable.minDate),i=new Date(this.settings.disable.maxDate)):this.settings.displayRange.start&&this.settings.displayRange.end&&(e=new Date(this.settings.displayRange.start),i=new Date(this.settings.displayRange.end)),this.settings.displayRange.start&&this.settings.displayRange.end){if(!this.dayMap.map((function(t){return t.key})).includes(n))return!1;if(e&&i){if(e.getDate()>=t.getDate()&&e.getMonth()===t.getMonth()||e.getMonth()>t.getMonth())return!1;if(i.getDate()<=t.getDate()&&i.getMonth()===t.getMonth()||i.getMonth()<t.getMonth())return!1}}return!0},validatePrevNext:function(){if(this.settings.disable.restrictMonths&&this.settings.disable.minDate&&this.settings.disable.maxDate){var t=new Date(this.settings.disable.minDate),e=new Date(this.settings.disable.maxDate);if(this.element.find(".prev").prop("disabled",!1),this.element.find(".next").prop("disabled",!1),t.getFullYear()>this.currentYear||this.currentYear>e.getFullYear())return this.element.find(".prev").prop("disabled",!0),void this.element.find(".next").prop("disabled",!0);this.currentMonth-1<t.getMonth()&&this.element.find(".prev").prop("disabled",!0),this.currentMonth+1>e.getMonth()&&this.element.find(".next").prop("disabled",!0)}},addLegend:function(){var t=this.settings;if(t.showLegend&&t.legend){this.legend&&this.legend.length?this.legend.empty():this.legend=$('<div class="monthview-legend"></div>');for(var e=0;e<t.legend.length;e++){var i=t.legend[e],n=i.color;if(n&&-1===n.indexOf("#")){var s=n.replace(/[0-9]/g,""),a=10*n.substr(n.length-2,2);n=ft.themeColors().palette[s][a].value}var o=""+'<div class="monthview-legend-item">\n          <span class="monthview-legend-swatch '.concat(n?'" style="background-color: '.concat(O.hexToRgba(n,.3)):' default"','"></span>\n          <span class="monthview-legend-text">').concat(i.name,"</span>\n        </div>");this.legend.append(o)}if(this.table.after(this.legend),t.inPage){var r=this.legend[0].querySelector(".monthview-legend-item:first-child"),l=this.legend[0].querySelector(".monthview-legend-item:last-child");if(r&&l){var c=function(t){return null==t||null==t?void 0:t.getBoundingClientRect().top};this.legend[c(r)!==c(l)?"addClass":"removeClass"]("is-wrapped")}}}},setRangeOnCell:function(t){var e=this,i=this.settings;if(i.range.useRange&&i.range.first&&!i.range.second){var n=i.range.first,s=i.range.extra,a=s.cellLength-1,o=n.rowIdx+n.cellIdx+a*n.rowIdx;if((t=$(t)).length&&!t.is(".is-disabled, .is-selected")){var r=t.closest("tr"),l=t.index(),c=r.index(),d=c+l+a*c,h=e.getCellDate(t),u=this.getTime(this.isIslamic?[h.year,h.month,h.day]:new Date(h.year,h.month,h.day)),p=this.getDifferenceToDate(i.range.first.date,i.range.maxDays);e.days.find("td:visible").each((function(t){var n=$(this);u>e.getTime(i.range.first.date)&&!i.range.selectBackward&&(!i.range.maxDays||i.range.maxDays>0&&u<=p.aftertime)&&(t>o&&t<=d||u>e.getTime(s.max)&&t<=d)?n.addClass("range-next"):u<e.getTime(i.range.first.date)&&!i.range.selectForward&&(!i.range.maxDays||i.range.maxDays>0&&u>=p.beforetime)&&(t<o&&t>=d||u<e.getTime(s.min)&&t>=d)?n.addClass("range-prev"):n.removeClass("range-next range-prev")}))}else t.length||e.days.find("td").removeClass("range-next range-prev")}!t&&i.range.second&&e.setRangeSelected()},getDifferenceToDate:function(t,e,i){var n=this,s={},a=function(t,e,a){for(t=n.isIslamic?H.umalquraToGregorian(t):new Date(t);e>0;)t.setDate(t.getDate()+(a?1:-1)),(i||!i&&!n.isDateDisabled(t.getFullYear(),t.getMonth(),t.getDate()))&&(e--,s[a?"after":"before"]=n.isIslamic?H.gregorianToUmalqura(new Date(t)):new Date(t));a&&s.after?s.aftertime=n.getTime(s.after):s.before&&(s.beforetime=n.getTime(s.before))};return i=void 0!==i?i:this.settings.range.includeDisabled,a(t,e),a(t,e,!0),s},setRangeSelected:function(){var t=this,e=this,i=this.settings;if(i.range.useRange&&i.range.second&&i.range.second.date&&this.days&&this.days.length){var n=i.range.data?new Date(i.range.data.start):i.range.first.date,s=i.range.data?new Date(i.range.data.end):i.range.second.date;this.days.find("td").removeClass("range range-next range-prev range-selection end-date is-selected"),this.days.find("td:visible").each((function(){var a,o=$(this),r=o.is(".is-disabled")&&!i.range.includeDisabled,l=o.is(".is-disabled")&&i.range.includeDisabled?" include-disabled":"",c=e.getTime((a=e.getCellDate(o),t.isIslamic?[a.year,a.month,a.day]:new Date(a.year,a.month,a.day))),d=e.getTime(n,!0),h=e.getTime(s,!0);c!==d&&c!==h||r?(!e.isIslamic&&c>d&&c<h&&!r||e.isIslamic&&(d>c&&h<c||d<c&&h>c)&&!r)&&o.addClass("range-selection".concat(l)):o.addClass("is-selected".concat(l).concat(d!==h?" range-selection".concat(c!==h||e.isIslamic?"":" end-date"):""))}))}},setRangeSelByClick:function(){var t=0,e=this.settings;if(e.range.useRange)if(e.range.first&&e.range.first.date&&e.range.second&&e.range.second.date){if(e.showTime&&this.datepickerApi){var i={time1:this.getTime(e.range.first.date),time2:this.getTime(e.range.second.date),first:this.datepickerApi.setTime(e.range.first.date),second:this.datepickerApi.setTime(e.range.second.date)};i.time1!==this.getTime(i.first)&&(this.datepickerApi.setRangeToElem(i.first,!0),this.datepickerApi.setRangeToElem(i.second,!1),t=2)}t=0===t?1:t}else{var n;if(!e.range.first||e.range.first&&!e.range.first.date)n=this.dayMap.filter((function(t){return t.elem.is(".is-selected")})),n&&n.length&&(this.days.find("td:visible").removeClass("is-selected").removeAttr("aria-selected"),n[0].elem.focus().trigger("click"),t=3);else if(!e.range.second||e.range.second&&!e.range.second.date){if((n=this.days.find("td.range-prev:visible").first()).length||(n=this.days.find("td.range-next:visible").last()),n.length||(n=this.dayMap.filter((function(t){return t.elem.is(".is-selected")}))),n&&n.length)(n[0].elem||n).focus().trigger("click"),t=3}}return t},getCellDate:function(t){var e=parseInt(t.text(),10),i=parseInt(this.header.find(".month").attr("data-month"),10),n=parseInt(this.header.find(".year").text(),10);return t.hasClass("prev-month")?0===i?(i=11,n--):i--:t.hasClass("next-month")&&(11===i?(i=0,n++):i++),{year:n,month:i,day:e}},setRangeSelBeforeFirstSel:function(t){var e=this.settings;e.range.useRange&&$("#monthview-popup:visible")&&(e.range.first||($("td",this.element).removeClass("is-selected"),t.addClass("is-selected")))},activeTabindex:function(t,e){return $("td",this.element).removeAttr("tabindex"),t.attr("tabindex",0),e&&t.focus(),t},updated:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return t||(t={}),t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.destroy().init()},teardown:function(){var t,e,i;return null===(t=this.header)||void 0===t||null==t||t.off(),null===(e=this.days)||void 0===e||null==e||e.off(),null===(i=this.element)||void 0===i||null==i||i.off(),this.monthYearPane&&(this.monthYearPane.off(),this.monthYearPane=null),this},destroy:function(){var t;if(null===(t=this.calendarToolbarAPI)||void 0===t||null==t||t.destroy(),this.teardown(),this.element&&this.element.empty(),this.element&&this.element[0])try{$.removeData(this.element[0],kn)}catch(t){}return this}};var $n="weekview",En={eventTypes:[{id:"example",label:"Example",color:"emerald07",checked:!0,click:function(){}}],filteredTypes:[],events:[],locale:null,language:null,firstDayOfWeek:0,startDate:null,endDate:null,showAllDay:!0,showTimeLine:!0,startHour:7,endHour:19,disable:{dayOfWeek:[]},newEventDefaults:{title:"NewEvent",subject:"",comments:"",durationHours:1},showToday:!0,showViewChanger:!0,hitbox:!1,onChangeView:null,onChangeWeek:null,onRenderWeek:null,eventTooltip:"overflow",iconTooltip:"overflow"};function Dn(t,e){this.settings=E.mergeSettings(t,e,En),this.element=$(t),this.init()}Dn.prototype={init:function(){return this.settings.startDate?this.settings.firstDayOfWeek=this.settings.startDate.getDay():this.settings.startDate=Gi.firstDayOfWeek(new Date,this.settings.firstDayOfWeek),this.settings.endDate?this.settings.endDate.setHours(23,59,59,999):this.settings.endDate=Gi.lastDayOfWeek(new Date,this.settings.firstDayOfWeek),this.setLocaleThenBuild()},setLocaleThenBuild:function(){var t=this,e=H.getLocale(this.settings.language),i=H.getLocale(this.settings.locale);return $.when(i,e).done((function(e,i){t.locale=H.cultures[e]||H.currentLocale,t.language=i||t.settings.language||t.locale.language,t.settings.language=t.language,t.setCurrentCalendar(),t.build()})),this},setCurrentCalendar:function(){return this.currentCalendar=H.calendar(this.locale.name,this.settings.language,this.settings.calendarName),this.isRTL="right-to-left"===(this.locale.direction||this.locale.data.direction),this.conversions=this.currentCalendar.conversions,this},build:function(){return this.id=this.id||E.uniqueId(this.element,$n),this.isMobileWidth=m.isBelow("phone-to-tablet"),this.addToolbar(),this.showWeek(this.settings.startDate,this.settings.endDate),this.handleEvents(),E.addAttributes(this.element,this,this.settings.attributes),this},renderAllEvents:function(t){if(!this.settings.onRenderWeek||t){var e=this.settings.events.slice(0);e.sort((function(t,e){return t.starts<e.starts?-1:t.starts>e.starts?1:0})),this.removeAllEvents();for(var i=0;i<e.length;i++){var n=e[i];this.settings.filteredTypes.indexOf(n.type)>-1||this.renderEvent(n)}}else this.callOnRenderWeek()},callOnRenderWeek:function(){var t=this;this.settings.onRenderWeek(this.element,(function(e,i){i&&i.length>0&&(t.settings.eventTypes=i),e&&e.length>0&&(t.settings.events=e,t.renderAllEvents(!0))}),{api:t,settings:this.settings})},removeAllEvents:function(){for(var t=this.element[0].querySelectorAll(".calendar-event"),e=0;e<t.length;e++)t[e].parentNode.removeChild(t[e]);for(var i=0;i<this.dayMap.length;i++)this.dayMap[i].events=[]},renderEvent:function(t){var e=new Date(t.starts),i=M.padDate(e.getFullYear(),e.getMonth(),e.getDate());if(H.isIslamic(this.locale.name)){var n=H.gregorianToUmalqura(e);i=M.padDate(n[0],n[1],n[2])}var s=new Date(t.ends),a=M.padDate(s.getFullYear(),s.getMonth(),s.getDate());if(H.isIslamic(this.locale.name)){var o=H.gregorianToUmalqura(s);a=M.padDate(o[0],o[1],o[2])}var l=this.dayMap.filter((function(t){return t.key>=i&&t.key<=a}));if(t.endKey=a,t.startKey=i,t=_i.addCalculatedFields(t,this.locale,this.language,this.settings.eventTypes),l.forEach((function(e){return e.events.push(t)})),1!==l.length||t.isAllDay||(this.isStackedView()?this.appendToDayContainer(t):this.appendEventToHours(l[0].elem,t)),1===l.length&&t.isAllDay&&this.appendEventToAllDay(l[0].elem,t),l.length>1)if("true"===t.isAllDay||!0===t.isAllDay)for(var c=0;c<l.length;c++){var d=0===c?"calendar-event-start":"calendar-event-continue";c===l.length-1&&(d="calendar-event-ends"),this.appendEventToAllDay(l[c].elem,t,d)}else{t.overnightStartsHour=t.startsHour,t.overnightEndsHour=t.endsHour;for(var h=0;h<l.length;h++){var u=r({},t);u.endsHour=this.settings.endHour+.6,h>0&&(u.startsHour=this.settings.startHour,u.startKey=u.endKey),h===l.length-1&&(u.endsHour=u.overnightEndsHour),this.isStackedView()?this.appendToDayContainer(u,!0):this.appendEventToHours(l[h].elem,u,!0)}}},createEventElement:function(t,e){var i=document.createElement("a");return f.addClass(i,"calendar-event",t.color,e),i.setAttribute("data-id",t.id),i.setAttribute("data-key",t.startKey),i.setAttribute("href","#"),i},appendEventToAllDay:function(t,e,i){var n=t.querySelector(".week-view-all-day-wrapper");if(n){var s=this.createEventElement(e,i);"calendar-event-continue"!==i&&"calendar-event-ends"!==i||s.setAttribute("tabindex","-1"),s.innerHTML='<div class="calendar-event-content">\n      '.concat(e.icon?'<span class="calendar-event-icon"><svg class="icon '.concat(e.icon,'" focusable="false" aria-hidden="true" role="presentation" data-status="').concat(e.status,'"><use href="#').concat(e.icon,'"></use></svg></span>'):"",'\n      <span class="calendar-event-title">').concat(e.shortSubject||e.subject,"</span>\n    </div>");var a=n.querySelectorAll(".calendar-event").length;if(a>=1&&(s.style.top="".concat(22*a,"px")),a>2)for(var o=this.element[0].querySelectorAll(".week-view-all-day-wrapper"),r=0;r<o.length;r++)o[r].style.height="".concat(44+23*(a-1),"px");n.appendChild(s),E.addAttributes($(s),this,this.settings.attributes,"week-view-event-".concat(e.id)),this.attachTooltip(s,e)}},appendEventToHours:function(t,e){for(var i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=this.element[0].querySelectorAll("td:nth-child(".concat(t.cellIndex+1,")")),s=0;s<n.length;s++){var a=n[s],o=parseFloat(a.parentNode.getAttribute("data-hour"),10),r=Math.round(e.startsHour);if(r>e.startsHour?o===r-.5:o===r){var l=e.endsHour-e.startsHour,c="",d=this.createEventElement(e);if(i?c=" ".concat(H.formatHourRange(e.overnightStartsHour,e.overnightEndsHour,{locale:this.locale})):(l<.5&&f.addClass(d,"reduced-padding",e.color),l<1.5&&f.addClass(d,"is-ellipsis"),l>2&&(c=" ".concat(H.formatHourRange(e.startsHour,e.endsHour,{locale:this.locale})))),e.startsHour+l>this.settings.endHour&&(f.addClass(d,"is-cutoff",e.color),l=this.settings.endHour+1-e.startsHour),l<.25&&(l=.25),e.startsHour>o){var h=e.startsHour-o,u=a.parentNode.offsetHeight;d.style.top="".concat(h/.016666666666666784*(u/30),"px")}d.style.height="".concat(2*l*25+1.5*l,"px"),d.innerHTML='<div class="calendar-event-content">\n          '.concat(e.icon?'<span class="calendar-event-icon"><svg class="icon '.concat(e.icon,'" focusable="false" aria-hidden="true" role="presentation" data-status="').concat(e.status,'"><use href="#').concat(e.icon,'"></use></svg></span>'):"","\n          ").concat(i?'<span style="font-weight: bold">Overnight</span><br/>':"",'\n          <span class="calendar-event-title">').concat(e.shortSubject||e.subject).concat(c,"</span>\n        </div>");var p=a.querySelector(".week-view-cell-wrapper"),g=a.querySelectorAll(".calendar-event"),m=g.length;if(m>0){var v=100/(m+1),b=0;for(b=0;b<m;b++)g[b].style.width="".concat(v,"%"),b>0&&this.isRTL&&(g[b].style.right="".concat(v*b,"%")),b>0&&!this.isRTL&&(g[b].style.left="".concat(v*b,"%"));d.style.width="".concat(v,"%"),this.isRTL?d.style.right="".concat(v*b,"%"):d.style.left="".concat(v*b,"%")}p.appendChild(d),E.addAttributes($(d),this,this.settings.attributes,"week-view-event-".concat(e.id)),this.attachTooltip(d,e)}}},appendToDayContainer:function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],i=this.element[0].querySelector('.week-view-body-cell[data-key="'.concat(t.startKey,'"]'));if(i){var n;n=" ".concat(e?H.formatHourRange(t.overnightStartsHour,t.overnightEndsHour,{locale:this.locale,keepPeriod:!0,pattern:this.settings.timePattern}):H.formatHourRange(t.startsHour,t.endsHour,{locale:this.locale,keepPeriod:!0,pattern:this.settings.timePattern}));var s=this.createEventElement(t),a='<span class="calendar-event-title">\n          '.concat(this.settings.timeFirst?n:t.shortSubject||t.subject,"</br>").concat(this.settings.timeFirst?t.shortSubject||t.subject:n,"\n        </span>");s.innerHTML='<div class="calendar-event-content">\n        '.concat(t.icon?'<span class="calendar-event-icon"><svg class="icon '.concat(t.icon,'" focusable="false" aria-hidden="true" role="presentation" data-status="').concat(t.status,'"><use href="#').concat(t.icon,'"></use></svg></span>'):"","\n        ").concat(e?'<span style="font-weight: bold">Overnight</span><br/>':"","\n        ").concat(a,"\n      </div>"),null==i||null==i||i.appendChild(s)}},attachTooltip:function(t,e){var i=this;if("overflow"!==this.settings.iconTooltip){var n=t.querySelector(".calendar-event-icon");n&&("function"==typeof this.settings.iconTooltip?this.settings.iconTooltip({settings:this.settings,event:e}):e[this.settings.iconTooltip]&&(n.setAttribute("title",e[this.settings.iconTooltip]),$(n).tooltip({content:n.innerText})))}"overflow"!==this.settings.eventTooltip&&("function"==typeof this.settings.eventTooltip?this.settings.eventTooltip({settings:this.settings,event:e}):e[this.settings.eventTooltip]&&(t.setAttribute("title",e[this.settings.eventTooltip]),$(t).tooltip({content:t.innerText}))),e.shortSubject||"overflow"!==this.settings.eventTooltip&&"overflow"!==this.settings.iconToolTip||(t.setAttribute("title",e.subject),$(t).tooltip({beforeShow:function(t,e){var n=e[0].querySelector(".calendar-event-title"),s=e[0].querySelector(".calendar-event-icon"),a=s?s.querySelector(".icon").getAttribute("data-status"):"";n.offsetWidth>e[0].scrollWidth-(s?s.offsetWidth:0)?t("".concat(n.innerText).concat(a?" (".concat(H.translate(a,{locale:i.locale.name},!1),")"):"")):t(!1)}}))},showWeek:function(t,e){var i=this;this.numberOfDays=Math.round((e-t)/864e5);var n=new Date(t.getTime()),s=new Date(e.getTime());if(H.isIslamic(this.locale.name)&&(t=H.gregorianToUmalqura(t),t=new Date(t[0],t[1],t[2],t[3],t[4],t[5],t[6]),e=H.gregorianToUmalqura(e),e=new Date(e[0],e[1],e[2],e[3],e[4],e[5],e[6])),this.dayMap=[],this.isDayView=!1,this.element.removeClass("is-day-view stacked-view is-borderless toolbar-hidden"),this.settings.borderless&&this.element.addClass("is-borderless"),this.settings.hideToolbar&&this.element.addClass("toolbar-hidden"),0!==this.numberOfDays&&1!==this.numberOfDays||(this.element.addClass("is-day-view"),this.isDayView=!0,this.element.find("#calendar-view-changer").val("day").trigger("updated")),this.hasIrregularDays=7!==this.numberOfDays,this.settings.responsive&&!this.isDayView&&this.isStackedView()&&this.isMobileWidth)this.showWeek(t,t);else{var a=this.isStackedView();a&&this.element.addClass("stacked-view"),this.weekContainer=a?this.createStackedTemplate(t,e):this.createTableTemplate(t,e),this.element.find(".week-view-container").remove(),this.element.append(this.weekContainer),this.element.find(".week-view-header-cell").each((function(t,e){var n=e.getAttribute("data-key"),s=i.element.find('.week-view-footer-cell[data-key="'.concat(n,'"]'))[0];n&&i.dayMap.push({key:n,elem:e,footer:s})})),this.addTimeLine(),this.showToolbarMonth(t,e),this.renderDisable(),this.renderLegend(),this.renderAllEvents();var o={isDayView:this.isDayView,isStackedView:a,startDate:H.isIslamic(this.locale.name)?n:t,endDate:H.isIslamic(this.locale.name)?s:e,elem:this.element,api:this};this.element.trigger("weekrendered",o),this.settings.onChangeWeek&&this.settings.onChangeWeek(o),this.settings.startDate=H.isIslamic(this.locale.name)?n:t,this.settings.endDate=H.isIslamic(this.locale.name)?s:e}},createTableTemplate:function(t,e){this.weekHeader='<thead class="week-view-table-header"><tr><th class="week-view-header-cell"><div class="week-view-header-wrapper"><span class="audible">'.concat(H.translate("Hour"),"</span></div>"),this.settings.showAllDay&&(this.weekHeader+='<div class="week-view-all-day-wrapper">'.concat(H.translate("AllDay",this.locale.name),"</div>")),this.weekHeader+="</th>";for(var i=new Date(t.getTime());i<=e;i.setDate(i.getDate()+1)){var n=H.formatDate(i,{pattern:"d",locale:this.locale.name}),s=H.formatDate(i,{pattern:"EEE",locale:this.locale.name}),a=this.currentCalendar.dateFormat.dayOfWeek,o=a?"EEE"===a.split(" ")[0]:"d EEE";this.weekHeader+='<th class="week-view-header-cell" data-key="'.concat(M.padDate(i.getFullYear(),i.getMonth(),i.getDate()),'"><div class="week-view-header-wrapper').concat(Gi.isToday(i)?" is-today":"",'"><span class="week-view-header-day-of-week').concat(o?"":" is-emphasis",'">').concat(o?s:n,'</span><span class="week-view-header-day-of-week').concat(o?" is-emphasis":"",'">').concat(o?n:s,"</span></div>"),this.settings.showAllDay&&(this.weekHeader+='<div class="week-view-all-day-wrapper"></div>'),this.weekHeader+="</th>"}this.weekHeader+="</tr></thead>",this.weekBody="<tbody>";for(var r=this.settings.startHour;r<=this.settings.endHour;r++){for(var l='<tr class="week-view-hour-row" data-hour="'.concat(r,'"><td><div class="week-view-cell-wrapper">').concat(H.formatHour(r,{locale:this.locale}),"</div></td>"),c='<tr class="week-view-half-hour-row" data-hour="'.concat(r,'.5"><td><div class="week-view-cell-wrapper"></div></td>'),d=new Date(t.getTime());d<=e;d.setDate(d.getDate()+1)){var h=M.padDate(d.getFullYear(),d.getMonth(),d.getDate());l+='<td data-key="'.concat(h,'"><div class="week-view-cell-wrapper"></div></td>'),c+='<td data-key="'.concat(h,'"><div class="week-view-cell-wrapper"></div></td>')}l+="</tr>",c+="</tr>",this.weekBody+=l+c}return this.weekBody+="</tbody>",'<div class="week-view-container"><table class="week-view-table">'.concat(this.weekHeader).concat(this.weekBody,"</table></div>")},createStackedTemplate:function(t,e){for(var i="",n="",s="",a=new Date(t.getTime());a<=e;a.setDate(a.getDate()+1)){var o="".concat(M.padDate(a.getFullYear(),a.getMonth(),a.getDate())),r=H.formatDate(a,{pattern:"d",locale:this.locale.name}),l=H.formatDate(a,{pattern:"EEE",locale:this.locale.name}),c=Gi.isToday(a);i+='<div data-key="'.concat(o,'" class="week-view-header-cell">\n        <div class="week-view-header-wrapper ').concat(c?"is-today":"",'">\n          <span class="week-view-header-day-of-week is-emphasis">').concat(r,'</span>\n          <span class="week-view-header-day-of-week">').concat(l,"</span>\n        </div>\n        ").concat(this.settings.showAllDay?'<div class="week-view-all-day-wrapper"></div>':"","\n      </div>"),n+='<div data-key="'.concat(o,'" class="week-view-body-cell"></div>'),s+='<div data-key="'.concat(o,'" class="week-view-footer-cell"></div>')}return'<div class="week-view-container">\n      <div class="week-view-stacked-header">'.concat(i,'</div>\n      <div class="week-view-stacked-body">').concat(n,"</div>\n      ").concat(this.settings.showFooter?'<div class="week-view-stacked-footer">'.concat(s,"</div>"):"","\n    </div>")},isStackedView:function(){return!!this.settings.stacked},renderDisable:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],e=this.settings.disable.dayOfWeek.length>0?this.settings.disable.dayOfWeek:t;if(!(!e.length>0)){var i=e;if(1===this.dayMap.length){var n=this.dayMap[0],s=n.key,a=s.substring(0,4),o=s.substring(4,6),r=s.substring(6,8),l=new Date(a,o-1,r);if(i.includes(l.getDay())){var c=n.elem;this.renderDisableToContainer(c)}}else for(var d=0;d<i.length;d++){var h=this.dayMap[i[d]].elem;this.renderDisableToContainer(h)}}},renderDisableToContainer:function(t){for(var e=this.element[0].querySelectorAll("td:nth-child(".concat(t.cellIndex+1,")")),i=0;i<e.length;i++){var n=e[i];$(n.querySelector(".week-view-cell-wrapper")).addClass("is-disabled")}var s=t.querySelector(".week-view-all-day-wrapper");s&&s.classList.add("is-disabled")},renderLegend:function(){var t=this;if(this.settings.showLegend&&!this.isStackedView())for(var e=function(e,i){var n="string"==typeof e?new Date(e):e,s=M.padDate(n.getFullYear(),n.getMonth(),n.getDate());if(H.isIslamic(t.locale.name)){var a=H.gregorianToUmalqura(n);s=M.padDate(a[0],a[1],a[2])}var o=new Date(e),r=M.padDate(o.getFullYear(),o.getMonth(),o.getDate());if(H.isIslamic(t.locale.name)){var l=H.gregorianToUmalqura(o);r=M.padDate(l[0],l[1],l[2])}var c=t.dayMap.filter((function(t){return t.key>=s&&t.key<=r}));if(c.length>0){var d=i.color;if(-1===d.indexOf("#")){var h=d.replace(/[0-9]/g,""),u=10*d.substr(d.length-2,2);d=ft.themeColors().palette[h][u].value}for(var p=O.hexToRgba(d,.3),g=c[0].elem,f=t.element[0].querySelectorAll("td:nth-child(".concat(g.cellIndex+1,")")),m=0;m<f.length;m++){var v=f[m],b=$(v.querySelector(".week-view-cell-wrapper"));b.addClass("is-colored"),b[0].setAttribute("data-hex",d),b[0].style.backgroundColor=p}var y=$(g.querySelector(".week-view-all-day-wrapper"));y&&(y.addClass("is-colored"),y[0].setAttribute("data-hex",d),y[0].style.backgroundColor=p)}},i=this.settings.legend,n=function(n){var s=i[n].dates,a=i[n].dayOfWeek,o=i[n].disableWeek;if(s)for(var r=0;r<s.length;r++)e(s[r],i[n]);if(a)for(var l=function(){var s=parseInt(t.dayMap[c].key.substring(0,4),10),o=parseInt(t.dayMap[c].key.substring(4,6),10)-1,r=parseInt(t.dayMap[c].key.substring(6,8),10),l=new Date(s,o,r);a.forEach((function(t){l.getDay()===t&&e(l,i[n])}))},c=0;c<t.dayMap.length;c++)l();o&&t.renderDisable(o)},s=0;s<i.length;s++)n(s)},showToolbarMonth:function(t,e){if(!this.settings.hideToolbar){var i=H.formatDate(t,{pattern:"MMMM",locale:this.locale.name}),n=H.formatDate(e,{pattern:"MMMM",locale:this.locale.name}),s=H.formatDate(t,{pattern:"yyyy",locale:this.locale.name}),a=H.formatDate(e,{pattern:"yyyy",locale:this.locale.name}),o=this.isMobileWidth&&this.monthField?H.formatDate(e,{pattern:"MMM yyyy",locale:this.locale.name}):H.formatDate(e,{date:"year",locale:this.locale.name});if(n!==i){var r=H.formatDate(t,{pattern:"MMM",locale:this.locale.name}),l=this.isMobileWidth&&this.monthField?H.formatDate(e,{pattern:"MMM yyyy",locale:this.locale.name}):H.formatDate(e,{pattern:"MMMM yyyy",locale:this.locale.name});o="".concat(r," - ").concat(l)}if(a!==s){var c=H.formatDate(t,{pattern:"MMM yyyy",locale:this.locale.name}),d=H.formatDate(e,{pattern:"MMM yyyy",locale:this.locale.name});o="".concat(c," - ").concat(d)}this.monthField.text(o)}},addTimeLine:function(){var t=this;if(this.settings.showTimeLine&&!this.isStackedView()){var e=function(){var e=new Date,i=e.getHours(),n=e.getMinutes(),s=i-t.settings.startHour+n/60;t.markers.css("top",52*s)};this.timeMarker||(this.element.find(".week-view-hour-row:nth-child(1) td").prepend('<div class="week-view-time-marker"></div>'),this.markers=$(".week-view-time-marker"),e(),this.timer=setInterval((function(){e()}),3e4))}},addToolbar:function(){var t=this;if(!this.settings.hideToolbar){var e=this.isDayView?"day":"week";this.header=$('<div class="week-view-header"><div class="calendar-toolbar"></div></div>').appendTo(this.element),this.calendarToolbarEl=this.header.find(".calendar-toolbar"),this.calendarToolbarAPI=new Cn(this.calendarToolbarEl[0],{onOpenCalendar:function(){return t.settings.startDate},locale:this.settings.locale,language:this.settings.language,year:this.currentYear,month:this.currentMonth,showToday:this.settings.showToday,isAlternate:!1,isMenuButton:!0,isMonthPicker:!0,showViewChanger:this.settings.showViewChanger,hitbox:this.settings.hitbox,onChangeView:this.settings.onChangeView,viewChangerValue:e,attributes:this.settings.attributes}),this.monthField=this.header.find(this.settings.attributes?"#".concat(this.settings.attributes[0].value,"-").concat(e,"-view-datepicker"):"#monthview-datepicker-field")}},setToday:function(){var t=new Date;this.isIslamic&&"string"!=typeof t&&(this.currentDateIslamic=H.gregorianToUmalqura(t),t=M.padDate(this.currentDateIslamic[0],this.currentDateIslamic[1],this.currentDateIslamic[2])),this.isIslamic||"string"==typeof t||(t=M.padDate(t.getFullYear(),t.getMonth(),t.getDate()));var e=parseInt(t.substr(0,4),10),i=parseInt(t.substr(4,2),10)-1;this.calendarToolbarAPI.setInternalDate(this.isIslamic?[e,i,1]:new Date(e,i,1))},setWeekFromDate:function(t){this.numberOfDays=7,this.settings.startDate=Gi.firstDayOfWeek(t,this.settings.firstDayOfWeek),this.settings.startDate.setHours(0,0,0,0),this.settings.endDate=new Date(this.settings.startDate),this.settings.endDate.setDate(this.settings.endDate.getDate()+this.numberOfDays-1),this.settings.endDate.setHours(23,59,59,59),this.currentDay?this.showWeek(this.currentDay,this.currentDay):this.showWeek(this.settings.startDate,this.settings.endDate)},handleEvents:function(){var t=this;this.element.off("updated.".concat($n)).on("updated.".concat($n),(function(){t.updated()})),this.element.off("change-date.".concat($n)).on("change-date.".concat($n),(function(e,i){var n=i.isToday?new Date:i.selectedDate;t.isDayView?(t.settings.startDate=n,t.settings.endDate=n):(t.hasIrregularDays&&n.getDay()===t.settings.firstDayOfWeek?t.settings.startDate=n:t.settings.startDate=Gi.firstDayOfWeek(n,t.settings.firstDayOfWeek),t.settings.startDate.setHours(0,0,0,0),t.settings.endDate=new Date(t.settings.startDate),t.settings.endDate.setDate(t.settings.endDate.getDate()+t.numberOfDays-1),t.settings.endDate.setHours(23,59,59,59)),t.showWeek(t.settings.startDate,t.settings.endDate),i.isToday&&t.setToday()})),this.element.off("change-next.".concat($n)).on("change-next.".concat($n),(function(){t.advanceDays(!0)})),this.element.off("change-prev.".concat($n)).on("change-prev.".concat($n),(function(){t.advanceDays(!1)}));var e=function(e,i){var n=e.getAttribute("data-id"),s=t.settings.events.filter((function(t){return t.id===n}));s&&0!==s.length&&t.element.trigger(i,{settings:t.settings,event:s[0]})};return this.element.off("click.".concat($n)).on("click.".concat($n),".calendar-event",(function(t){e(t.currentTarget,"eventclick"),t.preventDefault()})),this.element.off("dblclick.".concat($n)).on("dblclick.".concat($n),".calendar-event, td",(function(i){if($(i.currentTarget).is("td")){var n=i.currentTarget.getAttribute("data-key"),s=$(i.currentTarget).parent().attr("data-hour"),a=Math.floor(s),o=60*(s-a);if(!n)return;var r=new Date(n.substr(0,4),n.substr(4,2)-1,n.substr(6,2),a,o),l=E.extend({},t.settings.newEventDefaults);l.startKey=n,l.endKey=n,l.starts=r,l.ends=r,i.stopPropagation(),_i.cleanEventData(l,!1,r,t.locale,t.language,t.settings.events,t.settings.eventTypes),t.showModalWithCallback(r,l,!0,$(i.currentTarget)),t.element.trigger("updated")}e(i.currentTarget,"eventdblclick")})),$("body").off("breakpoint-change.".concat(this.id)).on("breakpoint-change.".concat(this.id),(function(){return t.onBreakPointChange()})),$(window).on("resize.".concat(this.id),(function(){t.element.trigger("breakpoint-change.".concat(t.id))})),this},onBreakPointChange:function(){if(this.isMobileWidth=m.isBelow("phone-to-tablet"),this.isStackedView()&&this.settings.responsive)if(!this.isDayView&&this.isMobileWidth){var t=new Date,e=Gi.isWithinRange(this.settings.startDate,this.settings.endDate,t)?t:this.settings.startDate;this.showWeek(e,e)}else this.isDayView&&this.setWeekFromDate(this.settings.startDate)},advanceDays:function(t){var e=this.isDayView?1:this.numberOfDays;t||(e=-e),this.settings.startDate.setDate(this.settings.startDate.getDate()+e),this.isDayView?(this.settings.endDate=new Date(this.settings.startDate),this.settings.startDate.setHours(0,0,0,0),this.settings.endDate.setHours(23,59,59,999)):this.settings.endDate.setDate(this.settings.endDate.getDate()+e),this.isDayView&&(this.currentDay=this.settings.startDate),this.showWeek(this.settings.startDate,this.settings.endDate)},addEvent:function(t){t.startsLocale||(t=_i.addCalculatedFields(t,this.locale,this.language,this.settings.eventTypes)),_i.cleanEventData(t,!0,this.settings.startDate,this.locale,this.language,this.settings.events,this.settings.eventTypes),this.settings.events.push(t),this.renderEvent(t)},showModalWithCallback:function(t,e,i,n){var s=this;this.showEventModal(t,e,(function(t,e){for(var n=t.querySelectorAll("input, textarea, select"),a=0;a<n.length;a++)e[n[a].id]="checkbox"===n[a].getAttribute("type")?n[a].checked:n[a].value;i?s.addEvent(e):s.updateEvent(e)}),n)},showEventModal:function(t,e,i,n){var s=this;if(this.settings.modalTemplate){this.modalVisible()&&this.removeModal(),this.modalContents=document.createElement("div"),f.addClass(this.modalContents,"calendar-event-modal","hidden"),document.getElementsByTagName("body")[0].appendChild(this.modalContents),e=_i.addCalculatedFields(e,this.locale,this.language,this.settings.eventTypes),this.renderTmpl(e||{},this.settings.modalTemplate,this.modalContents);var a,o=this.getDayEvents(t),r=!0;o.elem=$(o.elem),a=6===o.elem.index()?this.isRTL?"right":"left":this.isRTL?"left":"right",n||"day"!==this.activeView||(n=$(".week-view-header-wrapper"),a=this.isRTL?"left":"right"),n||(n=o.elem);var l=this.settings.modalOptions||{content:$(this.modalContents),closebutton:!0,popover:!0,placementOpts:{parent:n,strategies:["flip","nudge","shrink-y"],parentXAlignment:"center",parentYAlignment:"center",placement:a},title:e.title||e.subject,trigger:"immediate",keepOpen:!0,extraClass:"calendar-popup calendar-popup-mobile",tooltipElement:"#calendar-popup",headerClass:e.color,initializeContent:!1};n.off("hide.calendar").on("hide.calendar",(function(){r?s.removeModal():(i(s.modalContents,e),s.element.trigger("hidemodal",{elem:s.modalContents,event:e}),s.removeModal(),r=!0)})).popover(l).off("show.calendar").on("show.calendar",(function(i,a){s.element.trigger("showmodal",{elem:s.modalContents,event:e}),a.find("#isAllDay").off().on("click.calendar",(function(t){$(t.currentTarget).prop("checked")?(a.find("#durationHours").prop("disabled",!0),a.find("#endsHourLocale").prop("disabled",!0),a.find("#startsHourLocale").prop("disabled",!0)):(a.find("#durationHours").prop("disabled",!1),a.find("#endsHourLocale").prop("disabled",!1),a.find("#startsHourLocale").prop("disabled",!1))})),a.find("#type").val(e.type).dropdown(),a.find("#comments").val(e.comments),a.find("#subject").focus(),a.find("button").on("click",(function(t){var e=n.data("popover"),i=t.currentTarget.getAttribute("data-action");r="submit"!==i,e&&e.hide(!0)})),a.find(".datepicker").datepicker({locale:s.settings.locale,language:s.settings.language});var o=a.find(".timepicker");if(o){o.val(H.formatHour(t.getHours()+t.getMinutes()/60));var l=e.durationHours?e.durationHours:1;if(o.length>1)for(var c=1;c<o.length;c++)$(o[c]).val(H.formatHour(t.getHours()+l+t.getMinutes()/60));o.timepicker({locale:s.settings.locale,language:s.settings.language})}s.translate(a)}))}},modalVisible:function(){return null!==document.querySelector(".calendar-event-modal")},removeModal:function(){this.modalContents=null,this.activeElem&&(this.activeElem.off(),this.activeElem.data("popover")&&this.activeElem.data("popover").destroy()),f.remove(document.getElementById("calendar-popup")),f.remove(document.querySelector(".calendar-event-modal")),$("#timepicker-popup").hide()},renderTmpl:function(t,e,i,n){if("object"===l(je)&&e){if(e instanceof $?e="".concat(e.html()):"string"==typeof e&&(M.containsHTML(e)||(e=$("#".concat(e)).html())),t.color=_i.getEventTypeColor(t,this.settings.eventTypes),t.startsLong=H.formatDate(t.starts,{date:"long",locale:this.locale.name}),t.endsLong=H.formatDate(t.ends,{date:"long",locale:this.locale.name}),t.startsHoursLong="".concat(H.formatDate(t.starts,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(t.starts,{date:"hour",locale:this.locale.name})),t.endsHoursLong="".concat(H.formatDate(t.ends,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(t.ends,{date:"hour",locale:this.locale.name})),H.isIslamic(this.locale.name)){var s=H.gregorianToUmalqura(new Date(t.starts)),a=H.gregorianToUmalqura(new Date(t.ends));t.startsLong=H.formatDate(s,{date:"long",locale:this.locale.name}),t.endsLong=H.formatDate(a,{date:"long",locale:this.locale.name}),t.startsHoursLong="".concat(H.formatDate(s,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(s,{date:"hour",locale:this.locale.name})),t.endsHoursLong="".concat(H.formatDate(a,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(a,{date:"hour",locale:this.locale.name}))}t.typeLabel=this.getEventTypeLabel(t.type);var o=je.compile(e,{event:t});i.classList.remove("has-only-one"),n?f.append(i,o,"*"):(i.innerHTML=o,i.classList.add("has-only-one"))}},getEventTypeLabel:function(t){var e="";if(!t)return e;var i=this.settings.eventTypes.filter((function(e){return e.id===t}));return 1===i.length&&(e=i[0].label),e},getDayEvents:function(t){if("string"==typeof t||this.isRTL||(t=M.padDate(t.getFullYear(),t.getMonth(),t.getDate())),H.isIslamic(this.locale.name)){var e=H.gregorianToUmalqura(t);t=M.padDate(e[0],e[1],e[2])}var i=this.dayMap.filter((function(e){return e.key===t})),n={date:t,events:[]};return 0===i.length?[]:(n.events=i[0].events,n.elem=i[0].elem,n)},translate:function(t){var e=this;$(t).find('[data-translate="text"]').each((function(t,i){var n=$(i);n.text(H.translate(n.attr("data-translate-key")||n.text(),{showAsUndefined:!1,showBrackets:!1,language:e.settings.language,locale:e.settings.locale}))}))},selectHeader:function(t){var e=t instanceof Date?M.padDate(t.getFullYear(),t.getMonth(),t.getDate()):t,i={all:".week-view-table-header th"};i.current="".concat(i.all,'[data-key="').concat(e,'"]'),[].slice.call(this.element[0].querySelectorAll(i.all)).forEach((function(t){t.classList.remove("is-selected")}));var n=this.element[0].querySelector(i.current);n&&n.classList.add("is-selected")},clearEvents:function(){this.settings.events=[],this.renderAllEvents()},updateEvent:function(t){for(var e=t.id,i=this.settings.events.length-1;i>=0;i--)this.settings.events[i].id===e&&(this.settings.events[i]=E.extend(!0,this.settings.events[i],t),_i.cleanEventData(this.settings.events[i],!0,this.settings.startDate,this.locale,this.language,this.settings.events,this.settings.eventTypes));this.renderAllEvents()},deleteEvent:function(t){for(var e=t.id,i=this.settings.events.length-1;i>=0;i--)this.settings.events[i].id===e&&this.settings.events.splice(i,1);this.renderAllEvents()},updated:function(t){return t||(t={}),t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),t.locale?(this.destroy().init(),this):(this.renderAllEvents(),this)},teardown:function(){return this.element.off(),$("body").off("breakpoint-change.".concat(this.id)),clearInterval(this.timer),this.timer=null,this},destroy:function(){return this.teardown(),this.element.empty(),$.removeData(this.element[0],$n),$.removeData(this.element[0],"init"),$.removeData(this.element[0],"automationId"),this}};var In="calendar",Ln={eventTypes:[],events:[],locale:null,language:null,month:(new Date).getMonth(),year:(new Date).getFullYear(),day:(new Date).getDate(),showViewChanger:!0,onRenderMonth:null,template:null,mobileTemplate:null,upcomingEventDays:14,modalTemplate:null,menuId:null,menuSelected:null,eventTooltip:"overflow",iconTooltip:"overflow",newEventDefaults:{title:"NewEvent",subject:"",isAllDay:!0,comments:""},onChangeView:null,showToday:!0,hitbox:!1,weekViewSettings:{firstDayOfWeek:0,startHour:7,endHour:19,showAllDay:!0,showTimeLine:!0},disable:{callback:null,dates:[],years:[],minDate:"",maxDate:"",dayOfWeek:[],isEnable:!1,restrictMonths:!1},showEventLegend:!0,dayLegend:null,displayRange:{start:"",end:"",months:{showAbbr:!1}}};function Pn(t,e){this.settings=E.mergeSettings(t,e,Ln),this.element=$(t),this.init()}Pn.prototype={init:function(){return this.setLocaleThenBuild()},build:function(){return this.setCurrentCalendar().renderEventTypes().renderMonthView().renderWeekView().handleEvents().renderEventLegend(),this},setLocaleThenBuild:function(){var t=this,e=H.getLocale(this.settings.language),i=H.getLocale(this.settings.locale);return $.when(i,e).done((function(e,i){t.locale=H.cultures[e]||H.currentLocale,t.language=i||t.settings.language||t.locale.language,t.settings.language=t.language,t.setCurrentCalendar(),t.build().handleEvents()})),this},setCurrentCalendar:function(){return this.isRTL="right-to-left"===(this.locale.direction||this.locale.data.direction),this},renderEventLegend:function(){var t=this.settings;if($(".calendar-event-legend").remove(),t.showEventLegend){this.eventLegend=$('<div class="calendar-event-legend"></div>'),this.monthviewTable=$(".monthview-table");for(var e=0;e<t.eventTypes.length;e++){var i=t.eventTypes[e],n=i.color,s=$('<div class="calendar-event-legend-item"></div>');s.append('<span class="calendar-event-legend-swatch '.concat(n,'"></span>\n          <span class="calendar-event-legend-text">').concat(i.label,"</span>")),this.eventLegend.append(s),E.addAttributes(s,this,this.settings.attributes,"legend-mobile-".concat(i.id),!0)}this.monthviewTable.after(this.eventLegend)}},renderEventTypes:function(){if(this.eventTypeContainer=document.querySelector(".calendar-event-types"),!this.eventTypeContainer)return this;for(var t=0;t<this.settings.eventTypes.length;t++){var e=this.settings.eventTypes[t],i=e.custom?e.color:"".concat(e.color,"07"),n='<input type="checkbox" class="checkbox '.concat(i,'" name="').concat(e.id,'" id="').concat(e.id,'" ').concat(e.checked?'checked="true"':""," ").concat(e.disabled?'disabled="true"':"",' />\n          <label for="').concat(e.id,'" class="checkbox-label">').concat(e.translationKey?H.translate(e.translationKey,{locale:this.locale.name,language:this.language}):e.label,"</label><br/>");this.eventTypeContainer.insertAdjacentHTML("beforeend",n);var s=$(this.eventTypeContainer).find("#".concat(e.id));E.addAttributes($(this.eventTypeContainer).find("#".concat(e.id)),this,this.settings.attributes,"legend-".concat(e.id),!0),s.next("label").attr("for",s[0].id)}return this},renderMonthView:function(){var t=this;return this.monthViewContainer=this.element[0].querySelector(".calendar-monthview"),this.activeView="month",this.onChangeToMonth=function(e){t.settings.onChangeView?t.settings.onChangeView(e):t.changeView(e.viewName)},this.monthView=new Sn(this.monthViewContainer,{onRenderMonth:this.settings.onRenderMonth,onSelected:this.settings.onSelected,selectable:!0,locale:this.settings.locale,language:this.settings.language,month:this.settings.month,year:this.settings.year,day:this.settings.day,eventTooltip:this.eventTooltip,iconTooltip:this.iconTooltip,showToday:this.settings.showToday,showViewChanger:this.settings.showViewChanger,onChangeView:this.onChangeToMonth,disable:this.settings.disable,showLegend:null!==this.settings.dayLegend,legend:this.settings.dayLegend,hitbox:this.settings.hitbox,attributes:this.settings.attributes,firstDayOfWeek:this.settings.firstDayOfWeek,displayRange:{start:this.settings.displayRange.start,end:this.settings.displayRange.end,months:{showAbbr:this.settings.displayRange.months.showAbbr}}}),this.monthViewHeader=document.querySelector(".calendar .monthview-header"),this.renderAllEvents(),E.addAttributes($(".calendar"),this,this.settings.attributes),E.addAttributes($(".calendar-monthview"),this,this.settings.attributes,"monthview"),this},renderWeekView:function(){var t=this;if(this.weekViewContainer=document.querySelector(".calendar .calendar-weekview"),!this.weekViewContainer)return this;this.weekViewContainer.classList.add("week-view"),this.weekViewContainer.classList.add("hidden"),this.onChangeToWeekDay=function(e){t.settings.onChangeView?t.settings.onChangeView(e):t.changeView(e.viewName)};var e=Gi.firstDayOfWeek(new Date(this.currentDate()),this.settings.weekViewSettings.firstDayOfWeek),i=Gi.lastDayOfWeek(new Date(this.currentDate()),this.settings.weekViewSettings.firstDayOfWeek);return e.setHours(0,0,0,0),i.setHours(23,59,59,999),this.weekView=new Dn(this.weekViewContainer,{locale:this.settings.locale,language:this.settings.language,startDate:e,endDate:i,eventTypes:this.settings.eventTypes,events:this.settings.events,firstDayOfWeek:this.settings.weekViewSettings.firstDayOfWeek,showAllDay:this.settings.weekViewSettings.showAllDay,showTimeLine:this.settings.weekViewSettings.showTimeLine,startHour:this.settings.weekViewSettings.startHour,endHour:this.settings.weekViewSettings.endHour,disable:this.settings.disable,showLegend:null!==this.settings.dayLegend,legend:this.settings.dayLegend,showToday:this.settings.showToday,showViewChanger:this.settings.showViewChanger,hitbox:this.settings.hitbox,onChangeView:this.onChangeToWeekDay,eventTooltip:this.settings.eventTooltip,iconTooltip:this.settings.iconTooltip,attributes:this.settings.attributes,modalTemplate:this.settings.modalTemplate}),this.weekViewHeader=document.querySelector(".calendar .calendar-weekview .monthview-header"),this.weekView.settings.filteredTypes=this.filterEventTypes(),this.weekView.settings.onChangeWeek=function(e){t.monthView.selectDay(e.startDate,!1,!0)},this.weekView.renderAllEvents(),E.addAttributes($(".calendar-weekview"),this,this.settings.attributes,"weekview"),this},changeView:function(t){if(t!==this.activeView&&this.weekViewContainer){var e=this.currentDate(),i=new Date(e),n=new Date(e);switch(i.setHours(0,0,0,0),n.setHours(23,59,59,999),t){case"day":this.monthViewContainer.classList.add("hidden"),this.weekViewContainer.classList.remove("hidden"),this.activeView="day",this.weekView.settings.filteredTypes=this.filterEventTypes(),this.weekView.showWeek(i,n),this.clearEventDetails(),this.weekView.calendarToolbarAPI.setViewChangerValue(this.activeView);break;case"week":this.monthViewContainer.classList.add("hidden"),this.weekViewContainer.classList.remove("hidden"),this.activeView="week",i=Gi.firstDayOfWeek(i,this.settings.firstDayOfWeek),n=Gi.lastDayOfWeek(i,this.settings.firstDayOfWeek),this.weekView.settings.filteredTypes=this.filterEventTypes(),this.weekView.showWeek(i,n),this.weekView.calendarToolbarAPI.setViewChangerValue(this.activeView),this.clearEventDetails(),this.monthView.selectDay(e,!1,!0),this.weekView.selectHeader(e);break;case"month":this.monthViewContainer.classList.remove("hidden"),this.weekViewContainer.classList.add("hidden"),this.activeView="month",this.monthView.showMonth(this.settings.month,this.settings.year),this.monthView.calendarToolbarAPI.setViewChangerValue(this.activeView),this.monthView.selectDay(e,!1,!0)}}},appendUpcomingEvent:function(t){if(this.upcomingEventsContainer=document.querySelector(".calendar-upcoming-events"),this.upcomingEventsContainer&&!(t.daysUntil>0)){var e=Math.abs(t.daysUntil);if(!(e<0||e>this.settings.upcomingEventDays)){var i=document.createElement("a");i.setAttribute("href","#"),i.setAttribute("data-key",t.startKey),f.addClass(i,"calendar-upcoming-event");var n="",s=H.formatDate(t.starts,{pattern:"d",locale:this.locale.name}),a=H.formatDate(t.ends,{pattern:"d",locale:this.locale.name}),o="".concat(H.formatDate(t.starts,{pattern:"MMMM",locale:this.locale.name})," ").concat(s===a?s:"".concat(s,"-").concat(a),", ").concat(H.formatDate(t.starts,{pattern:"yyyy",locale:this.locale.name}));if(parseInt(a,10)<parseInt(s,10)){var r=new Date(t.starts);r.setDate(1),r.setMonth(r.getMonth()+1);var l=r.getFullYear();o="".concat(H.formatDate(t.starts,{pattern:"MMMM",locale:this.locale.name})," ").concat(s," - ").concat(H.formatDate(r,{pattern:"MMMM",locale:this.locale.name})," ").concat(a,", ").concat(l)}n+='\n      <span class="calendar-upcoming-date">'.concat(o,'</span>\n      <span class="calendar-upcoming-event-color ').concat(t.color||"",'">').concat(t.color||"",'</span>\n      <span class="calendar-upcoming-description">').concat(t.subject||"",'</span>\n      <span class="calendar-upcoming-status-text">').concat(t.status||"",'</span>\n      <span class="calendar-upcoming-duration">').concat(t.isDays?t.duration:t.durationHours," ").concat(t.durationUnits||"","</span>"),i.innerHTML=n,this.upcomingEventsContainer.appendChild(i),E.addAttributes($(i),this,this.settings.attributes,"upcoming-event-".concat(t.id))}}},renderEventDetails:function(t,e){if(this.settings.events&&"month"===this.activeView){var i=this.settings.events.filter((function(e){return e.id===t}));if(i&&0!==i.length&&(this.eventDetailsContainer=document.querySelector(".calendar-event-details"),this.eventDetailsContainer)){var n=$.extend(!0,{},i[0]);n.durationHours&&!n.isDays&&_i.formateTimeString(n,this.locale,this.language),this.renderTmpl(n,this.settings.template,this.eventDetailsContainer,e>1),this.eventDetailsMobileContainer=document.querySelector(".calendar-event-details-mobile.listview"),this.eventDetailsMobileContainer&&this.renderTmpl(n,this.settings.mobileTemplate,this.eventDetailsMobileContainer,e>1);var s=$(this.eventDetailsContainer).data("accordion");s&&s.destroy(),this.eventDetailsMobileContainer&&this.translate(this.eventDetailsMobileContainer),this.translate(this.eventDetailsContainer)}}},translate:function(t){var e=this;$(t).find('[data-translate="text"]').each((function(t,i){var n=$(i);n.text(H.translate(n.attr("data-translate-key")||n.text(),{showAsUndefined:!1,showBrackets:!1,language:e.settings.language,locale:e.settings.locale}))}))},renderSelectedEventDetails:function(){var t=this.getDayEvents();if(this.clearEventDetails(),t.events&&0!==t.events.length){for(var e=0;e<t.events.length;e++)this.renderEventDetails(t.events[e].id,t.events.length);$(".calendar .list-detail",this.element).css("display","block"),$(this.eventDetailsContainer).accordion(),this.eventDetailsMobileContainer&&$(this.eventDetailsMobileContainer).addClass("listview").listview({selectable:!1,hoverable:!1}),f.hasClass(this.eventDetailsContainer,"has-only-one")&&$(this.eventDetailsContainer).find(".accordion-header, .accordion-header a").off("click"),$(".accordion-pane.no-transition",this.element).removeClass("no-transition")}},renderDay:function(t){this.monthView.selectDay(t);var e=new Date(this.currentDate()),i=new Date(this.currentDate());e.setHours(0,0,0,0),i.setHours(23,59,59,999),"day"===this.activeView&&this.weekView.showWeek(e,i),"week"===this.activeView&&(e=Gi.firstDayOfWeek(e,this.settings.firstDayOfWeek),i=Gi.lastDayOfWeek(e,this.settings.firstDayOfWeek),this.weekView.showWeek(e,i))},clearEventDetails:function(){this.eventDetailsContainer=document.querySelector(".calendar-event-details"),this.eventDetailsMobileContainer=document.querySelector(".calendar-event-details-mobile.listview"),this.eventDetailsContainer&&(this.eventDetailsContainer.innerHTML=""),this.eventDetailsMobileContainer&&(this.eventDetailsMobileContainer.innerHTML="")},clearUpcomingEvents:function(){this.upcomingEventsContainer&&(this.upcomingEventsContainer.innerHTML="")},filterEventTypes:function(){var t=[];if(!this.eventTypeContainer)return t;for(var e=this.eventTypeContainer.querySelectorAll(".checkbox"),i=0;i<e.length;i++){var n=e[i];n.checked||t.push(n.getAttribute("name"))}return t},renderAllEvents:function(t){if(this.settings.onRenderMonth&&!t)return this.callOnRenderMonth(),this;var e=this.filterEventTypes();this.removeAllEvents(),this.clearUpcomingEvents(),this.clearEventDetails();var i=this.settings.events.slice(0);i.sort((function(t,e){return t.starts<e.starts?-1:t.starts>e.starts?1:0}));for(var n=0;n<i.length;n++){var s=i[n];e.indexOf(s.type)>-1||this.renderEvent(s)}return this.renderSelectedEventDetails(),this.weekView&&(this.weekView.settings.filteredTypes=e,this.weekView.settings.events=this.settings.events,this.weekView.renderAllEvents()),this},renderEvent:function(t){var e=this,i=H.newDateObj(t.starts),n=H.isIslamic(this.locale.name),s=M.padDate(i.getFullYear(),i.getMonth(),i.getDate());if(n){var a=H.gregorianToUmalqura(i);s=M.padDate(a[0],a[1],a[2])}var o=H.newDateObj(t.ends),r=M.padDate(o.getFullYear(),o.getMonth(),o.getDate());if(n){var l=H.gregorianToUmalqura(o);r=M.padDate(l[0],l[1],l[2])}var c=e.monthView.dayMap.filter((function(t){return t.key>=s&&t.key<=r}));t.endKey=r,t.startKey=s,t=_i.addCalculatedFields(t,this.locale,this.language,this.settings.eventTypes);for(var d=-1,h=0;h<e.monthView.dayMap.length;++h)if(e.monthView.dayMap[h].key>=s&&e.monthView.dayMap[h].key<=r){d=h;break}if(1===c.length&&e.appendEvent(c[0].elem[0],t,"event-day-start-end",d),c.length>1)for(var u=0;u<c.length;u++){var p=0===u?"event-day-start":"event-day-span";c.length-1===u&&(p="event-day-end"),e.appendEvent(c[u].elem[0],t,p,d+u)}this.appendUpcomingEvent(t,c,d)},removeAllEvents:function(){for(var t=this.monthViewContainer.querySelectorAll(".calendar-event-more"),e=0;e<t.length;e++)t[e].parentNode.removeChild(t[e]);for(var i=this.monthViewContainer.querySelectorAll(".calendar-event"),n=0;n<i.length;n++)i[n].parentNode.removeChild(i[n]);for(var s=this.monthViewContainer.querySelectorAll(".calendar-event-spacer"),a=0;a<s.length;a++)s[a].parentNode.removeChild(s[a]);for(var o=0;o<this.monthView.dayMap.length;o++)this.monthView.dayMap[o].events=[]},appendEvent:function(t,e,i,n){var s,a,o,r,l=this,c=t.querySelectorAll(".calendar-event").length;if(n>-1&&(this.monthView.dayMap[n].events||(this.monthView.dayMap[n].events=[]),this.monthView.dayMap[n].events.push(e)),c>=3){var d=t.querySelector(".calendar-event-more"),h=function(t,e){t.setAttribute("data-count",e),t.innerHTML="<span>+ ".concat(e," ").concat(H.translate("More",{locale:l.locale.name,language:l.language}).replace("...",""),"</span>")};return d?h(d,parseInt(d.getAttribute("data-count"),10)+1):(r=document.createElement("span"),f.addClass(r,"calendar-event-more"),h(r,1),t.querySelector(".day-container").appendChild(r),$(t).off("click.".concat(In)).on("click.".concat(In),".calendar-event-more span",(function(){var t=l.monthView.dayMap[n].key;l.monthView.selectDay(t,!1,!0),l.changeView("day")}))),this}r=document.createElement("a"),f.addClass(r,"calendar-event",null!==(s=e.cssClass)&&void 0!==s?s:e.color,i),r.setAttribute("data-id",e.id),r.setAttribute("data-key",e.startKey),("#"===(null===(a=e.color)||void 0===a||null==a?void 0:a.substr(0,1))||e.color)&&["ruby","amber","emerald","azure","turqoise","amethyst","graphite","slate"].indexOf(e.color)<0&&(r.style.backgroundColor=e.color,r.classList.remove(e.color)),("#"===(null===(o=e.borderColor)||void 0===o||null==o?void 0:o.substr(0,1))||e.borderColor)&&(r.style.borderLeftColor=e.borderColor),r.innerHTML='<div class="calendar-event-content">\n      '.concat(e.icon?'<span class="calendar-event-icon"><svg class="icon '.concat(e.icon,'" focusable="false" aria-hidden="true" role="presentation" data-status="').concat(e.status,'"><use href="#').concat(e.icon,'"></use></svg></span>'):"",'\n      <span class="calendar-event-title">').concat(e.shortSubject||e.subject,"</span>\n    </div>");var u=this.monthViewContainer.querySelectorAll('.calendar-event.event-day-start[data-id="'.concat(e.id,'"]'));if(u[0])for(var p=u[0].parentNode.children,g=t.querySelector(".day-container").children.length;g<p.length;g++){var m=p[g].getAttribute("data-id");if(this.getString(m)===this.getString(e.id))break;if(!p[g].classList.contains("day-text")){var v=document.createElement("span");v.classList.add("calendar-event-spacer"),t.querySelector(".day-container").appendChild(v)}}if(t.querySelector(".day-container").appendChild(r),E.addAttributes($(r),this,this.settings.attributes,"event-".concat(e.id)),"overflow"!==this.settings.iconTooltip){var b=r.querySelector(".calendar-event-icon");b&&("function"==typeof this.settings.iconTooltip?this.settings.iconTooltip({month:this.settings.month,year:this.settings.year,event:e}):e[this.settings.iconTooltip]&&(b.setAttribute("title",e[this.settings.iconTooltip]),$(b).tooltip({content:b.innerText})))}return"overflow"!==this.settings.eventTooltip&&("function"==typeof this.settings.eventTooltip?this.settings.eventTooltip({month:this.settings.month,year:this.settings.year,event:e}):e[this.settings.eventTooltip]&&(r.setAttribute("title",e[this.settings.eventTooltip]),$(r).tooltip({content:r.innerText}))),e.shortSubject||"overflow"!==this.settings.eventTooltip&&"overflow"!==this.settings.iconToolTip||(r.setAttribute("title",e.subject),$(r).tooltip({beforeShow:function(t,e){var i=e[0].querySelector(".calendar-event-title"),n=e[0].querySelector(".calendar-event-icon"),s=n?n.querySelector(".icon").getAttribute("data-status"):"";i.offsetWidth>e[0].scrollWidth-(n?n.offsetWidth:0)?t("".concat(i.innerText).concat(s?" (".concat(H.translate(s,{locale:l.locale.name,language:l.language},!1),")"):"")):t(!1)}})),this},getString:function(t){return"string"==typeof t?t:t?t.toString():null},getEventTypeLabel:function(t){var e="";if(!t)return e;var i=this.settings.eventTypes.filter((function(e){return e.id===t}));return 1===i.length&&(e=i[0].label),e},handleEvents:function(){var t=this,e=this;this.element.off("updated.".concat(In)).on("updated.".concat(In),(function(){t.updated()})),this.isSwitchingMonth=!1,this.element.off("monthrendered.".concat(In)).on("monthrendered.".concat(In),(function(e,i){t.isSwitchingMonth=!0,t.modalVisible()&&t.removeModal(),t.settings.year=i.year,t.settings.month=i.month,t.renderAllEvents(),setTimeout((function(){t.isSwitchingMonth=!1}),500)})),this.element.off("change.".concat(In)).on("change.".concat(In),".checkbox",(function(){t.renderAllEvents(!0)})),$(this.monthViewContainer).off("selected.".concat(In)).on("selected.".concat(In),(function(){t.renderSelectedEventDetails()})),this.element.off("click.".concat(In,"-upcoming")).on("click.".concat(In,"-upcoming"),".calendar-upcoming-event",(function(e){var i=e.currentTarget.getAttribute("data-key");t.renderDay(i)})),this.element.off("contextmenu.".concat(In)).on("contextmenu.".concat(In),".calendar-event",(function(i){i.stopPropagation();var n=function(){return e.settings.menuId&&$("#".concat(e.settings.menuId)).length>0},s=i.currentTarget.getAttribute("data-id"),a=t.settings.events.filter((function(t){return t.id===s}));if(t.element.triggerHandler("contextmenu",{originalEvent:i,month:t.settings.month,year:t.settings.year,event:a[0]}),!E.isSubscribedTo(e.element[0],i,"contextmenu","calendar")&&!n())return!0;if(i.preventDefault(),e.closePrevPopupmenu(),!n())return!0;var o=$(i.currentTarget);return o.popupmenu({attachToBody:!0,menuId:t.settings.menuId,eventObj:i,trigger:"immediate",offset:{y:5}}),o.off("selected.calendar").on("selected.calendar",(function(t,i){if(e.settings.menuSelected&&e.settings.menuSelected(t,i,s),"delete-event"===i.attr("data-action")&&e.deleteEvent({id:s}),"show-event"===i.attr("data-action")){var n=this.getAttribute("data-key");e.monthView.selectDay(n)}})),!1}));var i=0,n=!1;return this.element.off("click.".concat(In,"-event")).on("click.".concat(In,"-event"),".calendar-event",(function(s){i=setTimeout((function(){if(!n){var i=s.currentTarget.getAttribute("data-id"),a=t.settings.events.filter((function(t){return t.id===i}));if(!a||0===a.length)return;var o=$(s.currentTarget),r=o.find(".calendar-event-title");(s.currentTarget.classList.contains("event-day-span")||s.currentTarget.classList.contains("event-day-end"))&&(r=e.element.find('.event-day-start[data-id="'.concat(o.attr("data-id"),'"] .calendar-event-title'))),t.showModalWithCallback(a[0],!1,r),e.weekView&&"month"!==e.activeView||t.element.triggerHandler("eventclick",{month:t.settings.month,year:t.settings.year,event:a[0]})}n=!1}),100)})),this.element.off("dblclick.".concat(In,"-event")).on("dblclick.".concat(In,"-event"),".calendar-event",(function(e){clearTimeout(i),n=!0;var s=e.currentTarget.getAttribute("data-id"),a=t.settings.events.filter((function(t){return t.id===s}));a&&0!==a.length&&t.element.trigger("eventdblclick",{month:t.settings.month,year:t.settings.year,event:a[0]})})),this.element.off("dblclick.".concat(In)).on("dblclick.".concat(In),"td",(function(e){var i=e.currentTarget.getAttribute("data-key");if(i&&!t.isSwitchingMonth){var n=new Date(i.substr(0,4),i.substr(4,2)-1,i.substr(6,2)),s=E.extend({},t.settings.newEventDefaults);s.startKey=i,s.endKey=i,s.starts=n,s.ends=n,e.stopPropagation(),_i.cleanEventData(s,!1,t.currentDate(),t.locale,t.language,t.settings.events,t.settings.eventTypes),t.showModalWithCallback(s,!0),t.element.triggerHandler("dblclick",{eventData:s,api:t})}})),this.element.find(".listview").off("click.".concat(In,"-mobile")).on("click.".concat(In,"-mobile"),(function(e){var i=$(e.target).closest("li"),n=i.attr("data-id"),s=t.settings.events.filter((function(t){return t.id===n}));s&&0!==s.length&&(t.showModalWithCallback(s[0],!1,i),t.element.triggerHandler("eventclick",{month:t.settings.month,year:t.settings.year,event:s[0]}))})),this},closePrevPopupmenu:function(){[].slice.call(this.element[0].querySelectorAll(".is-open:not(.popupmenu)")).forEach((function(t){var e=$(t);e.data("popupmenu")&&e.trigger("close")}))},callOnRenderMonth:function(){var t=this;this.settings.onRenderMonth(this.element,(function(e,i){i&&i.length>0&&(t.settings.eventTypes=i,t.weekView&&(t.weekView.settings.eventTypes=i),t.renderEventLegend(),t.renderEventTypes()),e&&e.length>0&&(t.settings.events=e,t.renderAllEvents(!0),t.weekView&&(t.weekView.settings.events=e,t.weekView.renderAllEvents(!0)))}),{api:t,month:this.settings.month,year:this.settings.year})},currentDate:function(){var t=this.isIslamic?this.monthView.currentDateIslamic:this.monthView.currentDate;return H.isValidDate(t)?t:new Date},getDayEvents:function(t){if(t||(t=this.currentDate()),"string"==typeof t||this.isRTL||(t=M.padDate(t.getFullYear(),t.getMonth(),t.getDate())),H.isIslamic(this.locale.name)){var e=H.gregorianToUmalqura(t);t=M.padDate(e[0],e[1],e[2])}var i=this.monthView.dayMap.filter((function(e){return e.key===t}));"month"!==this.activeView&&(i=this.weekView.dayMap.filter((function(e){return e.key===t})));var n={date:this.monthView.currentDate,events:[]};return 0===i.length?[]:(n.events=i[0].events,n.elem=i[0].elem,n)},renderTmpl:function(t,e,i,n){if("object"===l(je)&&e){if(e instanceof $?e="".concat(e.html()):"string"==typeof e&&(M.containsHTML(e)||(e=$("#".concat(e)).html())),t.color=_i.getEventTypeColor(t,this.settings.eventTypes),t.startsLong=H.formatDate(t.starts,{date:"long",locale:this.locale.name}),t.endsLong=H.formatDate(t.ends,{date:"long",locale:this.locale.name}),t.startsHoursLong="".concat(H.formatDate(t.starts,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(t.starts,{date:"hour",locale:this.locale.name})),t.endsHoursLong="".concat(H.formatDate(t.ends,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(t.ends,{date:"hour",locale:this.locale.name})),H.isIslamic(this.locale.name)){var s=H.gregorianToUmalqura(new Date(t.starts)),a=H.gregorianToUmalqura(new Date(t.ends));t.startsLong=H.formatDate(s,{date:"long",locale:this.locale.name}),t.endsLong=H.formatDate(a,{date:"long",locale:this.locale.name}),t.startsHoursLong="".concat(H.formatDate(s,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(s,{date:"hour",locale:this.locale.name})),t.endsHoursLong="".concat(H.formatDate(a,{date:"long",locale:this.locale.name})," ").concat(H.formatDate(a,{date:"hour",locale:this.locale.name}))}t.typeLabel=this.getEventTypeLabel(t.type);var o=je.compile(e,{event:t});i.classList.remove("has-only-one"),n?f.append(i,o,"*"):(i.innerHTML=o,i.classList.add("has-only-one"))}},addEvent:function(t){_i.cleanEventData(t,!0,this.currentDate(),this.locale,this.language,this.settings.events,this.settings.eventTypes),this.settings.events.push(t),this.renderEvent(t),this.renderSelectedEventDetails(),this.weekView&&(this.weekView.settings.events=this.settings.events,this.weekView.renderAllEvents())},updateEvent:function(t){for(var e=t.id,i=this.settings.events.length-1;i>=0;i--)this.settings.events[i].id===e&&(this.settings.events[i]=E.extend(!0,this.settings.events[i],t),_i.cleanEventData(this.settings.events[i],!0,this.currentDate(),this.locale,this.language,this.settings.events,this.settings.eventTypes));this.renderAllEvents(),this.weekView&&this.weekView.updateEvent(t)},deleteEvent:function(t){for(var e=t.id,i=this.settings.events.length-1;i>=0;i--)this.settings.events[i].id===e&&this.settings.events.splice(i,1);this.renderAllEvents(),this.weekView&&this.weekView.deleteEvent(t)},clearEvents:function(){this.settings.events=[],this.renderAllEvents(),this.weekView&&this.weekView.clearEvents()},showEventModal:function(t,e,i){var n=this;if(this.settings.modalTemplate){this.modalVisible()&&this.removeModal(),this.modalContents=document.createElement("div"),f.addClass(this.modalContents,"calendar-event-modal","hidden"),document.getElementsByTagName("body")[0].appendChild(this.modalContents),t=_i.addCalculatedFields(t,this.locale,this.language,this.settings.eventTypes),this.renderTmpl(t||{},this.settings.modalTemplate,this.modalContents);var s=this.getDayEvents(),a=!0;s.elem=$(s.elem);var o=6===s.elem.index()?this.isRTL?"right":"left":this.isRTL?"left":"right";i||"day"!==this.activeView||(i=$(".week-view-header-wrapper"),o=this.isRTL?"left":"right"),i||(i=s.elem);var r=this.settings.modalOptions||{content:$(this.modalContents),closebutton:!0,popover:!0,placementOpts:{parent:i,strategies:["flip","nudge","shrink-y"],parentXAlignment:"center",parentYAlignment:"center",placement:o},title:t.title||t.subject,trigger:"immediate",keepOpen:!0,extraClass:"calendar-popup calendar-popup-mobile",tooltipElement:"#calendar-popup",headerClass:t.color,initializeContent:!1};i.off("hide.calendar").on("hide.calendar",(function(){a?n.removeModal():(e(n.modalContents,t),n.element.trigger("hidemodal",{elem:n.modalContents,event:t}),n.removeModal(),a=!0)})).popover(r).off("show.calendar").on("show.calendar",(function(e,s){n.element.trigger("showmodal",{elem:n.modalContents,event:t}),s.find("#isAllDay").off().on("click.calendar",(function(t){$(t.currentTarget).prop("checked")?(s.find("#durationHours").prop("disabled",!0),s.find("#endsHourLocale").prop("disabled",!0),s.find("#startsHourLocale").prop("disabled",!0)):(s.find("#durationHours").prop("disabled",!1),s.find("#endsHourLocale").prop("disabled",!1),s.find("#startsHourLocale").prop("disabled",!1))})),s.find("#type").val(t.type).dropdown(),s.find("#comments").val(t.comments),s.find("#subject").focus(),s.find("button").on("click",(function(t){var e=i.data("popover"),n=t.currentTarget.getAttribute("data-action");a="submit"!==n,e&&e.hide(!0)})),s.find(".datepicker").datepicker({locale:n.settings.locale,language:n.settings.language}),s.find(".timepicker").timepicker({locale:n.settings.locale,language:n.settings.language}),n.translate(s)})),$("#calendar-popup").one("tooltipafterplace.calendar",(function(t,e){var i=e.element.find(".arrow"),n=parseInt(i.css("margin-top"),10);if(s.elem.parent().index()>=3){var a=parseInt(e.element.offset().top,10),o=a+e.element.height(),r=$("html").height()+10;if(o>r){var l=a-(o-r)-25;e.element.css("top","".concat(l,"px")),i.css("margin-top","".concat(n+(a-l)-18,"px"))}}else e.element.height()>580&&i.css("margin-top","".concat(n-18,"px"))})),this.activeElem=i}},showModalWithCallback:function(t,e,i){var n=this;this.showEventModal(t,(function(t,i){for(var s=t.querySelectorAll("input, textarea, select"),a=0;a<s.length;a++)i[s[a].id]="checkbox"===s[a].getAttribute("type")?s[a].checked:s[a].value;e?n.addEvent(i):n.updateEvent(i)}),i)},modalVisible:function(){return null!==document.querySelector(".calendar-event-modal")},removeModal:function(){this.modalContents=null,this.activeElem&&(this.activeElem.off(),this.activeElem.data("popover")&&this.activeElem.data("popover").destroy()),f.remove(document.getElementById("calendar-popup")),f.remove(document.querySelector(".calendar-event-modal")),$("#timepicker-popup").hide()},updated:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return t||(t={}),t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),t.locale||t.template||t.upcomingEventDays||t.mobileTemplate?(this.destroy().init(),this):(this.weekView&&t.events&&(this.weekView.settings.events=t.events),this.weekView&&t.eventTypes&&(this.weekView.settings.events=t.events),this.weekView&&t.weekViewSettings&&(this.weekView.settings=E.mergeSettings(this.element[0],t.weekViewSettings,this.weekViews.settings)),this.settings.displayRange.start&&this.settings.displayRange.end?this.monthView.showRange(this.settings.displayRange.start,this.settings.displayRange.end):this.monthView.showMonth(this.settings.month,this.settings.year),this.renderAllEvents(),this.weekView&&t.weekViewSettings&&this.weekView.renderAllEvents(),this)},teardown:function(){return this.element&&this.element.off(),$(this.monthViewContainer).off(),this.monthView&&this.monthView.destroy(),this.weekView&&this.weekView.destroy(),this},destroy:function(){return this.eventTypeContainer&&(this.eventTypeContainer.innerHTML=""),this.monthViewContainer&&(this.monthViewContainer.innerHTML=""),this.upcomingEventsContainer&&(this.upcomingEventsContainer.innerHTML=""),this.eventDetailsContainer&&(this.eventDetailsContainer.innerHTML=""),this.eventDetailsMobileContainer&&(this.eventDetailsMobileContainer.innerHTML=""),this.removeModal(),this.teardown(),$.removeData(this.element[0],In),this}},$.fn.calendar=function(t){return this.each((function(){var e=$.data(this,In);e?e.updated(t):e=$.data(this,In,new Pn(this,t))}))};var Mn="circlepager",Bn={slidesToShow:1,startingSlide:null,loop:!1,attributes:null,showArrows:!1,footerContainment:!1,hideBulletsOnOverflow:!1,pageSelectorInputText:null};function On(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Bn),this.init()}On.prototype={init:function(){this.setElements(),this.slides.length&&(this.createControls(),this.handleEvents(),this.initActiveSlide(),this.showCollapsedView(),this.addAttributes())},setElements:function(){var t=this.settings;this.setResponsiveSlidesToShow(),this.container=$(".slides",this.element),this.slidesJQ=$(".slide",this.element),this.slidesToShow=t.slidesToShow,this.slides=[],this.isVisible=!0;for(var e=0,i=this.slidesJQ.length;e<i;e++)this.slides.push({node:$(this.slidesJQ[e])});this.activeIndex=null!==t.startingSlide&&t.startingSlide>-1&&t.startingSlide<this.slides.length?t.startingSlide:0,E.addAttributes(this.element,this,this.settings.attributes)},createControls:function(){for(var t,e,i,n,s,a,o,r,l,c=this.slides.length,d={bullets:""},h=[],u=0,p=0,g=c;p<g;p+=this.slidesToShow){e="",u++,r=1===this.slidesToShow||c-p==1,n=H.translate(r?"SlideOf":"SlidesOf"),i=r?"#slide {0} of {1}":"#slides {0} and {1} of {2}";for(var f=0;f<this.slidesToShow&&p+f<c;f++)e+="".concat(p+f+1,", ");n=n.replace(r?"{1}":"{2}",c),i=i.replace(r?"{1}":"{2}",c),o=(e=e.slice(0,-2)).lastIndexOf(","),a=e.substr(o+2),r?(l="",(t=this.slides[p].node).is(".is-disabled, [disabled]")&&!t.is('[disabled="false"]')&&(l=' disabled tabindex="-1"',this.slides[p].isDisabled=!0),t.is(".active")&&null===this.settings.startingSlide&&""===l&&(this.activeIndex=p),n=(s=t.attr("data-button-text"))&&s.length?s:n.replace("{0}",e),i=i.replace("{0}",e)):(e=e.substr(0,o),n=n.replace("{1}",a).replace("{0}",e),i=i.replace("{1}",a).replace("{0}",e)),i=i.toLowerCase().replace(/[\s,--]+/g,"-"),d.bullets+='<a href="'.concat(i,'" class="control-button hyperlink hide-focus is-ripple"').concat(l,'><span class="audible">').concat(n,"</span></a>"),h.push({href:i,text:n,isDisabled:l})}var m=$(".btn-previous",this.element),v=$(".btn-next",this.element),b=this.element.width();this.isBulletsNav=b>44*u;var y=!this.isBulletsNav;if(!this.isBulletsNav&&this.settings.hideBulletsOnOverflow)m.length||(d.previous=""+'<button class="btn-previous" type="button">\n            '.concat($.createIcon("left-arrow"),'\n            <span class="audible"> ').concat(H.translate("Previous"),"</span>\n          </button>")),v.length||(d.next=""+'<button class="btn-next" type="button">\n            '.concat($.createIcon("right-arrow"),'\n            <span class="audible">').concat(H.translate("Next"),"</span>\n          </button>"));else{var w;null===(w=this.pagerApi)||void 0===w||null==w||w.destroy();var x=$(".controls .btn-controls-previous, .btn-controls-next, .pager-container",this.element);m.add(v).add(x).remove(),this.element.closest(".card, .widget").find(".card-footer .circlepager-controls, .widget-footer .circlepager-controls").remove(),this.settings.showArrows&&(y=b<44*u+100,d.controlsPrevious=""+'<button class="btn-icon btn-controls-previous" type="button">\n            '.concat($.createIcon("previous-page"),'\n            <span class="audible"> ').concat(H.translate("Previous"),"</span>\n          </button>"),d.controlsNext=""+'<button class="btn-icon btn-controls-next" type="button">\n            '.concat($.createIcon("next-page"),'\n            <span class="audible">').concat(H.translate("Next"),"</span>\n          </button>"))}if(this.activeIndex>0&&this.activeIndex>u-1&&(this.activeIndex=u-1),u>1){var C=!0;d.all='<div class="controls circlepager-controls">',d.previous||d.next?(C=!1,d.all+="".concat(d.bullets,"</div>").concat(d.previous||"").concat(d.next||"")):y?(C=!1,d.all+='<div class="pager-container" data-init="false"></div></div>'):d.all+="".concat(d.controlsPrevious||"").concat(d.bullets).concat(d.controlsNext||"","</div>"),this.element.append(d.all).find("button").button(),C&&this.SetFooterContainment(),this.pagerApi=this.element.find(".pager-container").pager({type:"standalone",dataset:h,pagesize:1,showPageSizeSelector:!1,showPageSelectorInput:!0,footerContainmentClass:"footer-circlepager",footerContainment:this.settings.footerContainment,pageSelectorInputText:this.settings.pageSelectorInputText}).data("pager"),this.setPagerButtonsState()}},SetFooterContainment:function(){var t=this;if(this.element.closest(".card, .widget").length&&this.settings.footerContainment){["widget","card"].forEach((function(e){var i=t.element.closest(".".concat(e,"-content"));if(i.length){var n=i.next(".".concat(e,"-footer"));n.find(".circlepager-controls").remove(),n.length||(n=$('<div class="'.concat(e,'-footer footer-circlepager"></div>')).insertAfter(i)),t.element.find(".circlepager-controls").appendTo(n)}}))}},setPagerButtonsState:function(){if(this.pagerApi){var t={prevAndFirst:this.pagerApi.state.activePage>1,nextAndLast:this.pagerApi.state.activePage<this.pagerApi.state.pages};this.pagerApi.updated({enableFirstButton:t.prevAndFirst,enablePreviousButton:t.prevAndFirst,enableNextButton:t.nextAndLast,enableLastButton:t.nextAndLast,activePage:this.pagerApi.state.activePage})}},isVisibleInContainer:function(t){if(t&&t[0]){var e=t[0].getBoundingClientRect(),i=this.element[0].getBoundingClientRect();return e.left>i.left&&e.left<i.left+i.width&&e.top>i.top&&e.top<i.top+i.height}return-1},updateSlidesToShow:function(t){if(this.isActive)return this.settings.slidesToShow=t||1,this.updated(),this},setResponsiveSlidesToShow:function(){var t=this.element.width();this.resSlidesToShow=this.resSlidesToShow||this.settings.slidesToShow,t<300&&this.settings.slidesToShow>1?this.settings.slidesToShow=1:this.settings.slidesToShow=this.resSlidesToShow},responsiveSlidesToShow:function(t){var e=this;this.isActive&&(this.setResponsiveSlidesToShow(),this.slidesToShow=t||this.settings.slidesToShow,this.hideSlides(),this.element.parents(".card-content").addClass("unset-overflow"),this.unbind().slidesJQ.css("width",""),this.slides.length&&setTimeout((function(){e.createControls(),e.handleEvents(),e.initActiveSlide(),e.showCollapsedView(),e.removeOverflowedControls()}),0))},show:function(t){if(this.isActive){t=void 0!==t?t:this.activeIndex,this.activeIndex=t;var e=this.slides.length,i=t>-1?"".concat(H.isRTL()?100*(e-t-1):0-100*t,"%"):0;this.controlButtons.removeClass("is-active").eq(t).addClass("is-active"),this.container[0].style.left=i,!this.isBulletsNav&&this.settings.hideBulletsOnOverflow?(this.element.addClass("is-bullets-nav-hidden"),this.controlButtons.find("span").addClass("audible").end().eq(t).find("span").removeClass("audible")):(this.element.removeClass("is-bullets-nav-hidden"),this.controlButtons.find("span").addClass("audible")),this.isFocus&&this.isBulletsNav&&(this.isFocus=!1,this.controlButtons.eq(t).focus())}},first:function(){this.show(0)},last:function(){this.show(Math.round(this.slides.length/this.slidesToShow)-1)},prev:function(){var t,e=this;if(t=this.activeIndex>0?this.activeIndex-1:this.settings.loop?Math.round(this.slides.length/this.slidesToShow)-1:0,this.slides[t].isDisabled)return setTimeout((function(){e.prev()}),0),this.activeIndex=t,!1;this.show(t)},next:function(){var t,e=this;if(t=this.activeIndex>=Math.round(this.slides.length/this.slidesToShow)-1?this.settings.loop?0:this.activeIndex:this.activeIndex+1,this.slides[t].isDisabled)return setTimeout((function(){e.next()}),0),this.activeIndex=t,!1;this.show(t)},showCollapsedView:function(){if(this.isActive=!0,this.element.addClass("is-active"),this.container[0].style.width="".concat(100*this.slides.length,"%"),this.settings.slidesToShow>1&&this.slidesJQ.eq(0).width()*this.slidesToShow>this.element.width())this.responsiveSlidesToShow(this.slidesToShow-1);else{for(var t=0,e=this.slidesJQ.length;t<e;t++)this.slidesJQ[t].style.width="".concat(100/this.slidesToShow/this.slides.length,"%"),this.element.parents(".card-content").removeClass("unset-overflow"),this.showSlides(),this.slidesJQ.addClass("transition");this.show()}},removeOverflowedControls:function(){var t,e=this.controlButtons.parent(),i=null===(t=e[0])||void 0===t||null==t?void 0:t.nextElementSibling;e.length>1&&i&&i.remove()},addAttributes:function(){for(var t=0,e=this.controlButtons.length;t<e;t++){var i=$(this.controlButtons[t]);E.addAttributes(i,this,this.settings.attributes,"control-".concat(t+1))}},showSlides:function(){return this.slidesJQ.addClass("is-visible"),this},hideSlides:function(){return this.slidesJQ.removeClass("is-visible"),this},showExpandedView:function(){this.isActive=!1,this.element.removeClass("is-active"),this.element&&this.element[0]&&(this.element[0].style.width=""),this.container&&this.container[0]&&(this.container[0].style.width="",this.container[0].style.left="")},initActiveSlide:function(){if(this.slides[this.activeIndex].isDisabled)return this.next(),!1;this.show();var t=this.controlButtons.hasClass("is-active"),e=this.controlButtons.find(".pager-toolbar").length>0;(t||e)&&(this.showSlides(),this.slidesJQ.addClass("transition"))},setRipple:function(t,e){var i;if(t&&null!==(i=t.classList)&&void 0!==i&&(null==i?void 0:i.contains("is-ripple"))){var n=e&&e.touches?e.touches[0]:e||{},s=t.getBoundingClientRect(),a=2*Math.sqrt(Math.pow(s.width,2)+Math.pow(s.height,2)),o=n.clientX?n.clientX-s.left:t.offsetWidth/2,r=n.clientY?n.clientY-s.top:t.offsetHeight/2,l=t.style.cssText,c=t.getAttribute("data-hex"),d=O.getContrastColor(c),h={ripple:O.getLuminousColorShade(c,"white"===d?"0.3":"-0.3"),elem:O.hexToRgba(c,.7)},u=null!==c?"--ripple-background: ".concat(h.ripple,"; background-color: ").concat(h.elem,"; "):"";t.style.cssText="".concat(u,"--s: 0; --o: 1"),t.offsetTop,t.style.cssText="".concat(u,"--t: 1; --o: 0; --d: ").concat(a,"; --x:").concat(o,"; --y:").concat(r,";"),$(t).off("transitionend.monthview-ripple").on("transitionend.monthview-ripple",(function(e){var i;"transform"===(e.propertyName||(null===(i=e.originalEvent)||void 0===i||null==i?void 0:i.propertyName))&&(t.style.cssText=l)}))}},unbind:function(){var t,e=this.element.closest(".card, .widget").find(".card-footer .circlepager-controls, .widget-footer .circlepager-controls");if(this.controlButtons&&!this.controlButtons.length&&(this.controlButtons=e.find(".control-button")),null===(t=this.pagerApi)||void 0===t||null==t||t.destroy(),$("body").off("resize.circlepager"),this.element.find(".pager-container").off("page.circlepager"),this.element.off("focus.circlepager keydown.circlepager","*"),this.controlButtons){for(var i=0,n=this.controlButtons.length;i<n;i++)$(this.controlButtons[i]).off("click.circlepager");this.controlButtons.off("click.circlepager keydown.circlepager")}return $(".btn-previous, .btn-controls-previous",this.element).add(e.find(".btn-controls-previous")).off("click.circlepager"),$(".btn-next, .btn-controls-next",this.element).add(e.find(".btn-controls-next")).off("click.circlepager"),$(".controls",this.element).remove(),this.element.closest(".card, .widget").find(".footer-circlepager").remove(),this.showExpandedView(),this.element.closest(".tab-panel-container").prev(".tab-container").off("activated.circlepager"),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Bn)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Mn)},handleEvents:function(){var t=this,e=this,i=this.element.closest(".card, .widget").find(".card-footer .circlepager-controls, .widget-footer .circlepager-controls");this.controlButtons=$(".control-button",this.element),this.controlButtons.length||(this.controlButtons=i.find(".control-button")),$(".btn-previous, .btn-controls-previous",this.element).add(i.find(".btn-controls-previous")).off("click.circlepager").on("click.circlepager",(function(e){t.prev(),e.stopImmediatePropagation()})),$(".btn-next, .btn-controls-next",this.element).add(i.find(".btn-controls-next")).off("click.circlepager").on("click.circlepager",(function(e){t.next(),e.stopImmediatePropagation()}));for(var n=function(e){var i=$(t.controlButtons[e]);i.hideFocus(),i.off("click.circlepager").on("click.circlepager",(function(n){n.preventDefault(),t.slides[e].isDisabled||(t.setRipple(i[0],n),t.show(e))}))},s=0,a=this.controlButtons.length;s<a;s++)n(s);this.element.off("focus.circlepager","*").on("focus.circlepager","*",(function(t){var i=!1;if(!e.isVisibleInContainer($(this)))for(var n=e.element.find(":focusable"),s=0,a=n.length;s<a;s++)if(e.isVisibleInContainer(n.eq(s))){n.eq(s).focus(),i=!0;break}if(t.stopPropagation(),i)return!1})),this.element.off("keydown.circlepager","*").on("keydown.circlepager","*",(function(t){var i=!1,n=t.which||t.keyCode||t.charCode||0,s=$(":focusable"),a=s.index(this);if(9===n)if(t.shiftKey){for(var o=a-1;o>=0;o--)if(e.element.has(s.eq(o)).length<1||e.isVisibleInContainer(s.eq(o))){s.eq(o).focus(),i=!0;break}}else e.isVisibleInContainer(s.eq(a+1))||(e.controlButtons.first().focus(),i=!0);if(t.stopPropagation(),i)return!1})),this.controlButtons.off("keydown.circlepager").on("keydown.circlepager",(function(i){var n=!1,s=i.which||i.keyCode||i.charCode||0,a=H.isRTL();if(-1!==[37,39].indexOf(s)&&(e.isFocus=!0,i.altKey?37===s&&!a||39===s&&a?e.first():e.last():(!a&&37===s||a&&39===s?e.prev():e.next(),n=!0)),n)return i.preventDefault(),i.stopPropagation(),t.setRipple(i.currentTarget,i),!1})),$("body").off("resize.circlepager").on("resize.circlepager",b((function(){e.responsiveSlidesToShow()}),0)),e.element.closest(".tab-panel-container").prev(".tab-container").off("activated.circlepager").on("activated.circlepager",(function(){e.responsiveSlidesToShow()})),this.element.find(".pager-container").off("page.circlepager").on("page.circlepager",(function(e,i){t.setPagerButtonsState(),t.show(i.activePage-1)}))}},$.fn.circlepager=function(t){return this.each((function(){var e=$.data(this,Mn);e?e.updated(t):e=$.data(this,Mn,new On(this,t))}))},$.fn.calendartoolbar=function(t){return this.each((function(){var e=$.data(this,wn);e?e.updated(t):e=$.data(this,wn,new Cn(this,t))}))};var Rn="compositeform",zn={breakpoint:"phone-to-tablet",trigger:null,expandedText:H.translate("ShowLess"),collapsedText:H.translate("ShowMore")};function Fn(t,e){this.settings=E.mergeSettings(t,e,zn),this.element=$(t),this.init(),W("CompositeForm")}Fn.prototype={init:function(){return this.build().handleEvents()},build:function(){var t=this.element[0].classList;t.contains("composite-form")||t.add("composite-form");var e=this.element.find(".expandable-area");return e.length?(this.hasSummary=!0,this.expandableArea=e,this.expandableAreaAPI=this.expandableArea.data("expandablearea"),this.expandableAreaAPI||(this.expandableArea.expandablearea({trigger:this.settings.trigger}),this.expandableAreaAPI=this.expandableArea.data("expandablearea")),this.expander=this.expandableAreaAPI.expander,this.setExpanderText(this.settings.expandedText)):this.hasSummary=!1,this.checkResponsive(),this},handleEvents:function(){var t=this;function e(){var e=t.expandableAreaAPI.isExpanded();t.setExpanderText(t.settings[e?"expandedText":"collapsedText"])}return $("body").off("resize.".concat(Rn)).on("resize.".concat(Rn),(function(e){t.checkResponsive(e)})),this.element.on("updated.".concat(Rn),(function(){t.updated()})),this.hasSummary&&this.expandableArea.on("expand.".concat(Rn),e).on("collapse.".concat(Rn),e),this},checkResponsive:function(){var t=this.element[0].classList;m.isBelow(this.settings.breakpoint)?t.add("is-in-responsive-mode"):(t.remove("is-in-responsive-mode"),this.isSideOriented()&&!this.expandableAreaAPI.isExpanded()&&this.expandableAreaAPI.open())},setExpanderText:function(t){if(this.hasSummary&&t instanceof String&&t.length){var e=this.expander.find("span");e||(e=this.expander),e.text(t)}},isSideOriented:function(){return this.element[0].classList.contains("on-side")},updated:function(){return this.teardown().init()},teardown:function(){return $("body").off("resize.".concat(Rn)),this.element.off("updated.".concat(Rn)),this.hasSummary&&this.expandableArea.off("expand.".concat(Rn," collapse.").concat(Rn)),this},destroy:function(){this.teardown(),$.removeData(this.element[0],Rn)}},$.fn.compositeform=function(t){return this.each((function(){var e=$.data(this,Rn);e?e.updated(t):e=$.data(this,Rn,new Fn(this,t))}))};var Nn="contextualactionpanel",Hn={content:null,initializeContent:!0,title:"Contextual Action Panel",cssClass:null,modalSettings:{buttons:null,centerTitle:!1,id:null,showCloseBtn:!1,title:void 0,trigger:"click",useFlexToolbar:!1}},qn=["buttons","centerTitle","id","showCloseButton","trigger","useFlexToolbar"];function Vn(t){var e={showCloseButton:"showCloseBtn"};return qn.forEach((function(i){if(-1===[null,void 0].indexOf(t[i])){var n=i;t.modalSettings||(t.modalSettings={}),e[i]?(n=e[i],t.modalSettings[n]=t[i]):t.modalSettings[i]=t[i],delete t[i],V("settings.modalSettings.".concat(n),"settings.".concat(i))}})),t}function Wn(t,e){this.settings=E.mergeSettings(t,e,Hn),this.settings=Vn(this.settings),this.element=$(t),this.init()}Wn.prototype={get modalAPI(){var t;return this.panel&&this.panel.length&&(t=this.panel.data("modal")),t},get toolbarAPI(){var t;return this.toolbar&&this.toolbar.length&&(t=this.toolbar[0].classList.contains("flex-toolbar")?this.toolbar.data("toolbar-flex"):this.toolbar.data("toolbar")),t},get buttonsetAPI(){var t=E.getArrayFromList(this.panel[0].querySelectorAll(".buttonset"));return $(t[0]).data("buttonset")},init:function(){this.setup().build().handleEvents()},setup:function(){var t,e=this.element.next(".contextual-action-panel"),i=this.id;return"string"==typeof dataModal&&(t=$("#".concat(i))).length&&(e=t),1===this.element.closest(".popupmenu").length&&(e=this.element.closest(".popupmenu").next(".contextual-action-panel")),e[0]&&(e[0].style.display="none",e.addClass("is-animating"),this.panel=e),this},build:function(){var t,e,i,n,s=this,a=this.settings.content;if(this.id=(null===(t=this.settings)||void 0===t||null===(e=null==t?void 0:t.modalSettings)||void 0===e||null==e?void 0:e.id)||(null==a||null==a?void 0:a.attr("id"))||E.uniqueId(this.element,"contextual-action-modal"),null!==(i=this.settings)&&void 0!==i&&null!==(n=null==i?void 0:i.modalSettings)&&void 0!==n&&(null==n?void 0:n.id)||(this.settings.modalSettings.id=this.id),!this.panel||!this.panel.length)if(a instanceof jQuery){if(a.is(".contextual-action-panel")?this.panel=a:(a.wrap('<div class="contextual-action-panel"></div>'),this.panel=a.parent()),this.panel.addClass("modal").appendTo("body"),a.is("iframe"))return a.ready((function(){s.completeBuild(),a.show()})),s;a.show()}else this.panel=$('<div class="contextual-action-panel">'.concat(a,"</div>")).appendTo("body"),this.panel.addClass("modal").attr("id",this.settings.modalSettings.id);return this.settings.modalSettings.cssClass&&this.panel.addClass("".concat(this.settings.modalSettings.cssClass)),this.completeBuild(),this},completeBuild:function(){var t,e,i=!1,n=!1,s=!0;this.title=this.settings.modalSettings.title?this.settings.modalSettings.title:this.settings.title,this.panel.find("svg").icon(),0===this.panel.find(".modal-content").length&&((t=this.panel.children()).is("iframe")&&(e=t.contents(),this.toolbar=e.find(".toolbar, .flex-toolbar"),i=!0),i||(t.wrapAll('<div class="modal-content"></div>').wrapAll('<div class="modal-body"></div>'),this.panel.addClass("modal")));var a=this.panel.find(".modal-header");0===a.length&&(a=$('<div class="modal-header"></div>')).insertBefore(this.panel.find(".modal-body")),this.header=a;var o=this.panel.find(".toolbar .buttonset, .flex-toolbar .buttonset").first();if(this.toolbar||(this.toolbar=this.panel.find(".toolbar, .flex-toolbar").first()),!this.toolbar.length)if(s=!1,this.settings.modalSettings.buttons&&this.settings.modalSettings.buttons.forEach((function(t){"input"===t.type&&(n=!0)})),this.settings.title&&this.settings.modalSettings.centerTitle||this.settings.modalSettings.title&&this.settings.modalSettings.centerTitle){var r=n?'<div class="toolbar-section search"></div>':"",l='<div class="flex-toolbar">\n          <div class="toolbar-section static"></div>\n          <div class="toolbar-section title center-text">\n            <h2>'.concat(this.title,"</h2>\n          </div>\n          ").concat(r,'\n          <div class="toolbar-section buttonset static"></div>\n        </div>');this.toolbar=$(l)}else if(!o.length){var c=this.settings.modalSettings.useFlexToolbar?"flex-toolbar":"toolbar",d=this.settings.modalSettings.useFlexToolbar?'<div class="toolbar-section title"><h2>'.concat(this.title,"</h2></div>"):"",h=this.settings.modalSettings.useFlexToolbar?"toolbar-section buttonset":"buttonset",u='<div class="'.concat(h,'"></div>'),p=this.settings.modalSettings.useFlexToolbar&&n?'<div class="toolbar-section search"></div>':"",g='<div class="'.concat(c,'">\n          ').concat(d,"\n          ").concat(p,"\n          ").concat(u,"\n        </div>"),f=$(g);f.appendTo(this.panel.find(".modal-header")),this.toolbar=f,o=f.children(".buttonset")}if(this.toolbar.appendTo(this.header),!s){o&&(o.length||this.settings.modalSettings.centerTitle)||(o=$('<div class="toolbar-section buttonset"></div>')).appendTo(this.toolbar);var m=this.toolbar.find(".title");if(!m.length){var v=this.settings.modalSettings.centerTitle?" center-text":"";m=$('\n          <div class="toolbar-section title'.concat(v,'">\n            <h2>').concat(this.title,"</h2>\n          </div>\n        ")),o?m.insertBefore(o):this.toolbar.prepend(m)}m.length||(m=$('\n          <div class="title">\n            '.concat(this.title,"\n          </div>\n        ")),this.toolbar.prepend(m))}if(i||this.panel.detach().appendTo("body"),this.element.attr("data-modal")||this.element.attr("data-modal",this.settings.modalSettings.id),this.panel.attr("id")||this.panel.attr("id",this.settings.modalSettings.id),this.panel.modal(this.settings.modalSettings),this.buttons=this.panel.find(".buttonset").children("button"),this.closeButton=this.panel.find(".modal-header").find('.btn-close, [name="close"], button.close-button'),this.settings.modalSettings.showCloseBtn&&!this.closeButton.length){var b=H.translate("Close");if(this.closeButton=$('\n        <button class="btn-close" type="button" title="'.concat(b,'">\n          ').concat($.createIcon("close"),'\n          <span class="audible">').concat(b,"</span>\n        </button>\n      ")),this.settings.modalSettings.useFlexToolbar){var y=$('<div class="toolbar-section static"></div>').append(this.closeButton),w=this.toolbar.find(".toolbar-section.more");y.insertAfter(w.length?w:o)}else o.append(this.closeButton)}return this.closeButton.length&&this.toolbar.addClass("has-close-button"),this.toolbar.is(".toolbar")&&this.toolbar.toolbar(),this.toolbar.is(".flex-toolbar")&&this.toolbar.toolbarflex(),E.fixSVGIcons(this.element),this},handleEvents:function(){var t=this,e=this;function i(t){e.element.triggerHandler(t.type)}return this.panel.addClass("is-animating").off("open.contextualactionpanel").on("open.contextualactionpanel",(function(t){i(t),e.panel.removeClass("is-animating")})).off("close.contextualactionpanel").on("close.contextualactionpanel",(function(t){i(t)})).off("beforeclose.contextualactionpanel").on("beforeclose.contextualactionpanel",(function(t){i(t)})).off("beforeopen.contextualactionpanel").on("beforeopen.contextualactionpanel",(function(n){e.settings.initializeContent&&$(t).initialize(),i(n)})).off("afteropen.contextualactionpanel").on("afteropen.contextualactionpanel",(function(){if(e.toolbar&&e.toolbar.trigger("recalculate-buttons"),e.toolbar.length){var t=e.toolbar.find(".buttonset > .is-selected");if(t.length||(t=e.toolbar.find(".buttonset > *:first-child")).is(".searchfield-wrapper")&&(t=t.children(".searchfield")),!t.length&&e.toolbar.is(".flex-toolbar"))return void(t=e.toolbar.find("button").first()).focus();var i=e.toolbar.data("toolbar");i&&i.setActiveButton(t,!0)}e.panel.find(".modal-body-wrapper").find(":focusable").first().focus(),E.fixSVGIcons(e.panel)})),e.closeButton&&e.closeButton.length&&e.closeButton.on("click.contextualactionpanel",(function(){e.handleToolbarSelected()})),this},handleToolbarSelected:function(){this.close(!0)},teardown:function(){var t=this,e=t.toolbar.children(".buttonset");this.panel.off("open.contextualactionpanel close.contextualactionpanel beforeclose.contextualactionpanel beforeopen.contextualactionpanel afterclose.contextualactionpanel"),e.children("*:not(.searchfield)").off("click.contextualactionpanel"),e.children(".btn-menu").each((function(){var t=$(this).data("popupmenu");t&&t.destroy()})),t.header&&t.header.remove(),t.panel.find(".modal-body").children().first().unwrap().unwrap(),t.element.removeAttr("data-modal"),t.closeButton&&t.closeButton.length&&(t.closeButton.off("click.contextualactionpanel"),delete t.closeButton),t.element.trigger("afterteardown")},close:function(){var t,e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];"immediate"===this.settings.modalSettings.trigger&&(t=!0),this.modalAPI&&this.modalAPI.close(t,!1,e)},disable:function(){this.element.prop("disabled",!0),this.panel.hasClass("is-visible")&&this.close()},enable:function(){this.element.prop("disabled",!1)},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this.settings=Vn(this.settings),this.setup(),this.modalAPI&&this.modalAPI.updated(this.settings.modalSettings),this},destroy:function(){this.modalAPI&&this.modalAPI.isOpen&&this.modalAPI.close(!0),$.removeData(this.element[0],Nn),this.toolbar&&this.toolbar.data("toolbar")&&this.toolbar.data("toolbar").destroy(),this.toolbar&&this.toolbar.data("toolbarFlex")&&this.toolbar.data("toolbarFlex").destroy()},destroyAll:function(){at.destroyAll(!0)}},$.fn.contextualactionpanel=function(t){return this.each((function(){var e,i,n=null==t||null===(e=null==t?void 0:t.modalSettings)||void 0===e||null==e?void 0:e.id;!n&&null!=t&&(null==t?void 0:t.content)&&(null==t||null==t?void 0:t.content)instanceof jQuery&&(n=null==t||null===(i=null==t?void 0:t.content)||void 0===i||null==i?void 0:i.attr("id"));var s=at.findById(n);s?s.updated(t):$.data(this,Nn,new Wn(this,t))}))};var jn="timepicker",Un=["standard","range"],Gn=function(){return{locale:null,language:null,timeFormat:void 0,minuteInterval:5,secondInterval:5,mode:Un[0],roundToInterval:!0,parentElement:null,returnFocus:!0,attributes:null,tabbable:!0,hourRange:[0,24]}};function _n(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Gn),this.init()}_n.prototype={init:function(){this.setLocaleThenBuild()},setLocaleThenBuild:function(){var t=this,e=H.getLocale(this.settings.language),i=H.getLocale(this.settings.locale);return $.when(i,e).done((function(e,i){t.locale=H.cultures[e]||H.currentLocale,t.language=i||t.settings.language||t.locale.language||null,t.settings.language=t.language,t.setCurrentCalendar(),t.build().handleEvents().addAttributes()})),this},setCurrentCalendar:function(){return this.currentCalendar=H.calendar(this.locale.name,this.settings.language,this.settings.calendarName),void 0===this.settings.timeFormat&&(this.settings.timeFormat=this.currentCalendar.timeFormat||"h:mm a"),this.isRTL="right-to-left"===(this.locale.direction||this.locale.data.direction),this.build(),this},setup:function(){function t(t,e){(!e||["minute","second"].indexOf(e)<0)&&(e="minute");var i=Gn()["".concat(e,"Interval")];if(void 0===t||isNaN(t))return i;var n=parseInt(t,10);return n>0&&n<60?n:i}function e(t){return!0===t}var i;return this.id="".concat(E.uniqueId(this.element,"timepicker"),"-id"),this.hoursId="timepicker-hours-".concat(this.id.toString()),this.minutesId="timepicker-minutes-".concat(this.id.toString()),this.secondsId="timepicker-seconds-".concat(this.id.toString()),this.periodId="timepicker-period-".concat(this.id.toString()),this.element.is("[data-round-to-interval]")&&(this.settings.roundToInterval=e(this.element.attr("data-round-to-interval"))),this.element.is("[data-minute-interval]")&&(this.settings.minuteInterval=t(this.element.attr("data-minute-interval"),"minute")),this.settings.timeFormat=(i=24===parseInt(this.element.attr("data-force-hour-mode"),10)?"HH:mm":this.settings.timeFormat)&&i.toUpperCase().match("H")&&i.match("mm")?i:Gn().timeFormat,this.settings.minuteInterval=t(this.settings.minuteInterval,"minute"),this.settings.secondInterval=t(this.settings.secondInterval,"second"),this.settings.mode=function(t){return $.inArray(t,["standard","range"])>-1?t:Gn().mode}(this.settings.mode),this.settings.roundToInterval=e(this.settings.roundToInterval),this.dayPeriods=this.currentCalendar.dayPeriods,this},build:function(){if(this.setup(),this.element.attr("autocomplete","off"),this.settings.parentElement)return this.settings.parentElement.empty(),this.trigger=$(),this.buildStandardPopup(),this.setupStandardEvents(),this;var t=this.element.next();return t.is("button.trigger")?this.trigger=t:this.trigger=$('<button class="btn-icon trigger" type="button">\n        <span class="audible"></span>\n        '.concat($.createIcon("clock"),"\n      </button>")).insertAfter(this.element),this.element.is(":disabled")&&this.disable(),this.makeTabbable(!this.element.is(":disabled")&&"-1"!==this.element.attr("tabindex")&&this.settings.tabbable),this.addAria().addMask().handleEvents().roundMinutes(),this},addAria:function(){this.element.attr({role:"combobox","aria-expanded":"false"}),this.trigger.attr({role:"button","aria-haspopup":"dialog"}),this.label=$('label[for="'.concat(this.element.attr("id"),'"]')),this.label.find(".audible").remove(),this.label.append('<span class="audible">'.concat(H.translate("UseArrow",{locale:this.locale.name,language:this.language}),"</span>"));var t=H.translate("TimepickerTriggerButton");return this.trigger.children("span.audible").text(t),this},addAttributes:function(){E.addAttributes(this.element,this,this.settings.attributes),E.addAttributes(this.trigger,this,this.settings.attributes,"trigger")},handleKeys:function(){var t=this;this.element.on("keydown.timepicker",(function(e){var i=!1;return 27===e.which&&t.isOpen()&&(i=!0,t.closeTimePopup()),40!==e.which||t.isOpen()||(i=!0,t.openTimePopup()),!i||(e.stopPropagation(),e.preventDefault(),!1)}))},handleBlur:function(){var t=this;this.element.on("blur.timepicker",(function(){t.roundMinutes(),setTimeout((function(){t.isOpen()&&0===t.popup.find(":focus").length&&t.closeTimePopup()}),20)}))},is24HourFormat:function(t){return t||(t=this.settings.timeFormat),!!t&&(t.match("H")||[]).length>0},hourText:function(t){return((this.settings.timeFormat.toUpperCase().match("HH")||[]).length>0&&t<10?"0":"")+t},hasSeconds:function(t){return t||(t=this.settings.timeFormat),(t.match("ss")||[]).length>0},hasDayPeriods:function(t){return t||(t=this.settings.timeFormat),(t.match("a")||[]).length>0},getTimeSeparator:function(){return this.currentCalendar.dateFormat.timeSeparator},roundMinutes:function(){if(M.toBoolean(this.settings.roundToInterval)){var t=this.element.val(),e=this.getTimeSeparator(),i=t?t.split(e):[],n=this.settings.minuteInterval;if(i[1]){if(!this.is24HourFormat(this.settings.timeFormat)){var s=i[1].split(" ");i[1]=s[0],s[1]&&i.push(s[1])}if(i[1]=parseInt(i[1],10),i[1]%n!=0){i[1]=Math.round(i[1]/n)*n,i[1]=i[1].toString(),i[1]=(i[1].length<2?"0":"")+i[1],"60"===i[1]&&(i[1]="00",i[0]=(parseInt(i[0],10)+1).toString());var a="".concat(i[0]+e+i[1]," ").concat(i[2]?i[2]:"");this.element.val(a)}}}},addMask:function(){this.element.data("mask")&&"object"===l(this.element.data("mask"))&&this.element.data("mask").destroy(),this.element.data("mask",void 0);var t={keepCharacterPositions:!0,process:"date",patternOptions:{format:this.settings.timeFormat}};return this.addedValidation=!1,this.element[0].getAttribute("data-validate")||(this.addedValidation=!0,this.element.attr("data-validate","time").attr("data-validation-events",JSON.stringify({time:"blur change enter"})).validate()),t&&this.element.mask(t),this},getMaxHourRange:function(t,e,i){var n=this,s=i||t.period;return e?"AM"===s?n.settings.hourRange[1]>12?12:n.settings.hourRange[1]:n.settings.hourRange[1]>12?n.settings.hourRange[1]-12:12:n.settings.hourRange[1]},buildStandardPopup:function(){var t,e=this,i=$('<div class="timepicker-popup-content"></div>'),n=this.getTimeSeparator(),s="";this.initValues=e.getTimeFromField();var a=$('<div class="time-parts"></div>').appendTo(i),o=this.is24HourFormat(),r=this.hasSeconds(),l=this.hasDayPeriods(),c=o?0:1,d=o?24:13;for(this.hourSelect=$('<select id="'.concat(this.hoursId,'" data-options="{\'noSearch\': \'true\'}" class="hours dropdown"></select>'));c<d;){if(t="",c>e.getMaxHourRange(this.initValues,l))break;parseInt(e.initValues.hours,10)===c&&(t=" selected"),e.hourSelect.append($("<option".concat(t,">").concat(e.hourText(c),"</option>"))),c++}var h=$('<div class="time-part"></div>');h.append($('<label for="'.concat(this.hoursId,'">').concat(H.translate("Hours",{locale:this.locale.name,language:this.language}),"</label>"))),h.append(this.hourSelect),h.append($('<span class="label colons">'.concat(n,"</span>"))),a.append(h);var u=0;for(this.minuteSelect=$('<select id="'.concat(this.minutesId,'" data-options="{\'noSearch\': \'true\'}" class="minutes dropdown"></select>'));u<=59;)s=u<10?"0".concat(u):u,t="",parseInt(e.initValues.minutes,10)===u&&(t=" selected"),e.minuteSelect.append($("<option".concat(t,">").concat(s,"</option>"))),u+=e.settings.minuteInterval;this.minuteSelect.find("option[selected]").length||this.minuteSelect.prepend($("<option selected>".concat(e.initValues.minutes,"</option>")));var p=$('<div class="time-part"></div>');if(p.append($('<label for="'.concat(this.minutesId,'">').concat(H.translate("Minutes",{locale:this.locale.name,language:this.language}),"</label>"))),p.append(this.minuteSelect),r&&p.append($('<span class="label colons">'.concat(n,"</span>"))),a.append(p),r){var g=0;for(this.secondSelect=$('<select id="'.concat(this.secondsId,'" data-options="{\'noSearch\': \'true\'}" class="seconds dropdown"></select>'));g<=59;)s=g<10?"0".concat(g):g,t="",(parseInt(e.initValues.seconds,10)===g||!e.initValues.seconds&&"00"===s)&&(t=" selected"),this.secondSelect.append($("<option".concat(t,">").concat(s,"</option>"))),g+=e.settings.secondInterval;this.secondSelect.find("option[selected]").length||this.secondSelect.prepend($("<option selected>".concat(e.initValues.seconds,"</option>")));var f=$('<div class="time-part"></div>');f.append($('<label for="'.concat(this.secondsId,'">').concat(H.translate("Seconds",{locale:this.locale.name,language:this.language}),"</label>"))),f.append(this.secondSelect),a.append(f)}if(!o&&l){this.periodSelect=$('<select id="'.concat(this.periodId,'" data-options="{\'noSearch\': \'true\'}" class="period dropdown"></select>')),a.append($('<span class="label colons"></span>'));for(var m=this.currentCalendar.dayPeriods,v=0,b=new RegExp(e.initValues.period,"i");v<2;)t="",b.test(m[v])&&(t=" selected"),this.periodSelect.append($('<option value="'.concat(m[v],'"').concat(t,">").concat(m[v],"</option>"))),v++;var y=$('<div class="time-part"></div>');y.append($('<label for="'.concat(this.periodId,'">').concat(H.translate("Period",{locale:this.locale.name,language:this.language}),"</label>"))),y.append(this.periodSelect),a.append(y)}if(this.settings.parentElement)this.settings.parentElement.append(i),e.popup=this.settings.parentElement.find(".timepicker-popup-content").addClass("timepicker-popup").attr("id","timepicker-popup");else{i.append('<div class="modal-buttonset"><button type="button" class="btn-modal-primary set-time">'.concat(H.translate("SetTime",{locale:this.locale.name,language:this.language}),"</button></div>"));var w=this.element,x=this.isRTL?"right":"left";this.element.parent().is(".datagrid-cell-wrapper")&&(x="center",w=this.element.next(".icon, .trigger")),this.trigger.popover({content:i,trigger:"immediate",placement:"bottom",placementOpts:{parent:w,parentXAlignment:x,strategies:["flip","nudge","shrink"]},tooltipElement:"#timepicker-popup"}).on("show.timepicker",(function(t,i){e.afterShow(i)})).on("hide.timepicker",(function(){e.settings.returnFocus&&e.element.focus()}))}this.trigger.data("popover")&&(e.popup=this.trigger.data("popover").tooltip,e.popup.addClass("timepicker-popup"))},afterShow:function(t){var e=this;t.find("button").button();var i=this.hourSelect.data("dropdown");this.hourSelect.val(this.initValues.hours),i.pseudoElem.find("span").text(this.initValues.hours),i.renderPseudoElemLabel(),E.addAttributes(i.wrapper.find(".dropdown"),this,this.settings.attributes,"hours");var n=this.minuteSelect.data("dropdown");if(this.minuteSelect.val(this.initValues.minutes),n.pseudoElem.find("span").text(this.initValues.minutes),n.renderPseudoElemLabel(),E.addAttributes(n.wrapper.find(".dropdown"),this,this.settings.attributes,"minutes"),this.secondSelect){var s=this.secondSelect.data("dropdown");this.secondSelect.val(this.initValues.seconds),s.pseudoElem.find("span").text(this.initValues.seconds),s.renderPseudoElemLabel(),E.addAttributes(s.wrapper.find(".dropdown"),this,this.settings.attributes,"seconds")}if(e.hasDayPeriods()){var a=this.periodSelect.data("dropdown");this.periodSelect.val(this.initValues.period),a.pseudoElem.find("span").text(this.initValues.period),a.renderPseudoElemLabel(),E.addAttributes(a.wrapper.find(".dropdown"),this,this.settings.attributes,"period")}t.find("div.dropdown").first().focus(),t.find(".set-time").off("click.timepicker").on("click.timepicker",(function(t){t.preventDefault(),e.setTimeOnField(),e.closeTimePopup()})),E.addAttributes(t.find(".set-time"),this,this.settings.attributes,"btn"),this.settings.parentElement||t.on("keydown.timepicker","button, div.dropdown",(function(t){return 9!==(t.keyCode||t.charCode||0)||(e.containFocus(t),t.stopPropagation(),t.preventDefault(),!1)}))},containFocus:function(t){var e=t.shiftKey,i=this.popup.find(":focusable"),n=i.index($(":focus")),s=n+1>=i.length?0:n+1,a=n-1<0?i.length:n-1;n=e?a:s,i.eq(n).focus()},setupStandardEvents:function(){var t=this;t.popup.on("touchend.timepicker touchcancel.timepicker",".set-time",(function(t){t.preventDefault(),t.target.click()})).on("keydown.timepicker","input.dropdown",(function(e){var i=!1;if(27===e.which&&(i=!0,t.closeTimePopup(),t.element.focus()),32===e.which&&(i=!0,t.popup.find(".set-time").click()),37===e.which||39===e.which){i=!0;var n=t.popup.find('input[id$="-shdo"]');if(37===e.which){var s=n.eq(n.index(this)-1);s&&0!==s.length||(s=n.eq(n.length)),s.focus()}if(39===e.which){var a=n.eq(n.index(this)+1);a&&0!==a.length||(a=n.eq(0)),a.focus()}}return!i||(e.preventDefault(),e.stopPropagation(),!1)})),t.trigger.on("hide.timepicker",(function(){t.onPopupHide()}))},buildRangePopup:function(){},setupRangeEvents:function(){},getTimeFromField:function(t){var e,i,n=this,s=t||this.element.val(),a=this.getTimeSeparator(),o={},r=this.settings.timeFormat,l=!!this.dayPeriods.filter((function(t){return t.indexOf(".")>-1})).length&&s.indexOf(".")>-1&&r.indexOf("a")>-1&&r.indexOf("ah")<0&&!this.is24HourFormat();(s=s.replace(l?/[T\s:-]/g:/[T\s:.-]/g,a).replace(/z/i,"")).indexOf(a)<0&&4===s.length&&"HHmm"===n.settings.timeFormat&&(s="".concat(s.substr(0,2)).concat(a).concat(s.substr(2))),e=(s=s.replace("午","午".concat(a))).split(a);var c="ah"!==r.replace(" ","").slice(0,2)&&!$(".datepicker").length>0,d=this.currentCalendar.timeFormat.toLowerCase().indexOf("a"),h=-1!==d&&d<this.currentCalendar.timeFormat.toLowerCase().indexOf("h");this.is24HourFormat()||!h||c||(e=[e[1],e[2],e[0]]);var u,p=this.currentCalendar.dayPeriods;function g(t){return t.replace(/^\s+|\s+$/g,"")}function f(t){return!t||isNaN(t)?"00":t=(t=parseInt(t,10))<10?"0".concat(t):t}return e[2]&&(p[0].indexOf(".")>-1||p[1].indexOf(".")>-1)&&(p[0].replace(".","")===e[2]&&(e[2]=p[0]),p[1].replace(".","")===e[2]&&(e[2]=p[1])),this.is24HourFormat()||h||(i=e[e.length-1].split(" "),e.pop(),e=e.concat(i)),e[0]&&e[0].length&&!isNaN(e[0])||(e[0]="1"),e[0]=parseInt(e[0],10),isNaN(e[0])||(e[0]="".concat(parseInt(e[0],10))),o.hours=n.hourText(e[0]),e[1]?(e[1]=g(e[1]),e[1]=f(e[1]),o.minutes=e[1]):o.minutes="00",(u=e[2])?(u=g(u),n.hasSeconds()?(u=f(u),o.seconds=u):(function(t){return n.dayPeriods.indexOf(t)>-1}(u)||(u=n.translateDayPeriod("AM")),o.period=u)):(n.hasSeconds()&&(u="00",o.seconds=u),n.is24HourFormat()||(u=n.translateDayPeriod("AM"),o.period=u)),e[3]?(e[3]=g(e[3]),o.period=e[3]):!this.is24HourFormat()&&this.hasSeconds()&&(o.period=this.translateDayPeriod("AM")),o},getTimeFromField2:function(t){function e(t){return!t||isNaN(t)?"00":t=(t=parseInt(t,10))<10?"0".concat(t):t}var i=this.hasSeconds()?this.currentCalendar.dateFormat.timestamp:this.currentCalendar.dateFormat.hour,n=this.hasSeconds()?"timestamp":"hour",s=H.parseDate(t||this.element.val(),{date:n}),a=(s?s.getHours():"1").toString(),o=(a>=12?this.translateDayPeriod("PM"):this.translateDayPeriod("AM")).toString();!this.is24HourFormat()&&a>12&&(a=(parseInt(a,10)-12).toString()),this.is24HourFormat()||"0"!==a||(a="12");var r=i.indexOf("a")>-1?o:void 0,l=s?s.getMinutes():"00",c=s?s.getSeconds():"00";return{hours:e(a),minutes:e(l),seconds:e(c),period:r}},translateDayPeriod:function(t){return/am|pm|AM|PM/i.test(t)?(this.currentCalendar.dayPeriods||["AM","PM"])[/AM|am/i.test(t)?0:1]:t},setTimeOnField:function(){var t,e,i,n,s,a=this.settings.timeFormat,o=this.hourSelect?null===(t=this.hourSelect[0])||void 0===t||null==t?void 0:t.value:"",r=this.minuteSelect?null===(e=this.minuteSelect[0])||void 0===e||null==e?void 0:e.value:"",l=this.secondSelect?null===(i=this.secondSelect[0])||void 0===i||null==i?void 0:i.value:"",c=this.periodSelect?null===(n=this.periodSelect[0])||void 0===n||null==n?void 0:n.value:"",d=null==H||null===(s=null==H?void 0:H.calendar().dayPeriods)||void 0===s||null==s?void 0:s.indexOf(c),h=new Date;h.setHours(Gi.hoursTo24(parseInt(o,10),d),r,l);var u=H.formatDate(h,{date:"hour",pattern:a});this.element.val(u).trigger("change"),this.element.focus()},isOpen:function(){return!(!this.popup||!this.popup.is(":visible"))},openTimePopup:function(){var t=this,e=this;e.setup(),this.element.is(":disabled")||this.element.attr("readonly")||(this.popup&&!this.popup.hasClass("is-hidden")&&e.closeTimePopup(),this.element.addClass("is-active is-open"),"range"===e.settings.mode?(e.buildRangePopup(),e.setupRangeEvents()):(e.buildStandardPopup(),e.setupStandardEvents()),this.trigger.data("popover")&&this.element.attr("aria-expanded","true"),this.popup.find("div.dropdown").first().focus(),setTimeout((function(){$("select.period.dropdown").on("change",(function(i){var n,s=$(i.target).find(":checked").val();t.initValues=e.getTimeFromField();var a=t.is24HourFormat(),o=a?0:1,r=a?24:13,l=$("select.hours.dropdown"),c=l.siblings(".dropdown-wrapper").find(".dropdown").children("span").text();c.indexOf(H.translate("Hours"))>-1&&(c=c.split(" ")[1]),l.empty();for(var d=e.getMaxHourRange(t.initValues,t.hasDayPeriods(),s);o<r&&!(o>d);)n="",parseInt(c,10)===o&&(n=" selected"),l.append($("<option".concat(n,">").concat(e.hourText(o),"</option>"))),o++;c>d&&(c=1),$("select.hours.dropdown").siblings(".dropdown-wrapper").find(".dropdown").children("span").html(c)}))}),10))},closeTimePopup:function(){this.trigger.data("popover")&&(this.trigger.data("popover").hide(),this.element.attr("aria-expanded","false"))},onPopupHide:function(){if("standard"===this.settings.mode){var t=this.hourSelect,e=this.minuteSelect,i=this.secondSelect,n=this.periodSelect;t&&t.data("dropdown")&&"function"==typeof t.data("dropdown").destroy&&t.data("dropdown").destroy(),e&&e.data("dropdown")&&"function"==typeof e.data("dropdown").destroy&&e.data("dropdown").destroy(),this.hasSeconds()&&i&&i.data("dropdown")&&"function"==typeof i.data("dropdown").destroy&&i.data("dropdown").destroy(),this.hasDayPeriods()&&n&&n.data("dropdown")&&"function"==typeof n.data("dropdown").destroy&&n.data("dropdown").destroy(),this.popup.off("click.timepicker touchend.timepicker touchcancel.timepicker keydown.timepicker")}this.trigger.off("hide.timepicker show.timepicker"),this.trigger.data("popover").destroy(),this.trigger.data("popover",void 0),$("#timepicker-popup").remove(),this.element.removeClass("is-active is-open")},toggleTimePopup:function(){this.isOpen()?this.closeTimePopup():this.openTimePopup()},value:function(t){var e=this.element.val();if(!t||!1===t)return e;var i=this.currentCalendar.dateFormat.timeSeparator,n=new RegExp(i,"g"),s=(e=e.replace(n,"")).split(" ");return s[0].length<4&&(e="0".concat(s[0]).concat(s[1]?s[1]:"")),e},enable:function(){this.element.removeAttr("disabled readonly").closest(".field").removeClass("is-disabled"),this.trigger.removeAttr("disabled")},readonly:function(){this.enable(),this.element.attr("readonly","readonly")},disable:function(){this.enable(),this.element.attr("disabled","disabled").closest(".field").addClass("is-disabled"),this.trigger.prop("disabled",!0)},isDisabled:function(){return this.element.prop("disabled")},makeTabbable:function(t){this.trigger.attr("tabIndex",t?0:-1)},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown(),this.init(),this.element.data("validate")&&this.element.data("validate")instanceof Object&&this.element.data("validate").updated()},teardown:function(){this.trigger.off("keydown.timepicker"),this.element.off("focus.timepicker blur.timepicker keydown.timepicker click.timepicker"),this.popup&&this.closeTimePopup(),this.trigger.remove();var t=this.element.data("mask");return t&&"function"==typeof t.destroy&&t.destroy(),this.addedValidation&&($.removeData(this.element[0],"validate"),$.removeData(this.element[0],"validationEvents"),this.element.removeAttr("data-validate").removeData("validate validationEvents"),delete this.addedValidation),this.label.find(".audible").remove(),$("#timepicker-popup").remove(),this},destroy:function(){this.teardown(),$.removeData(this.element[0],jn)},handleEvents:function(){var t=this;return this.trigger.off("click.timepicker").on("click.timepicker",(function(){t.toggleTimePopup()})),this.label&&this.label.off("click.timepicker").on("click.timepicker",(function(){t.element[0].focus()})),this.handleKeys(),this.handleBlur(),this}},$.fn.timepicker=function(t){return this.each((function(){var e=$.data(this,jn);e?e.updated(t):e=$.data(this,jn,new _n(this,t))}))};var Yn=new function(){var t=this;this.ValidationTypes=[],this.ValidationTypes.error={type:"error",titleMessageID:"Error",pagingMessageID:"ErrorOnPage",errorsForm:!0},this.ValidationTypes.alert={type:"alert",titleMessageID:"Alert",pagingMessageID:"AlertOnPage",errorsForm:!1},this.ValidationTypes.success={type:"success",titleMessageID:"Success",pagingMessageID:"SuccessOnPage",errorsForm:!1},this.ValidationTypes.confirm=this.ValidationTypes.success,this.ValidationTypes.info={type:"info",titleMessageID:"Info",pagingMessageID:"InfoOnPage",errorsForm:!1},this.ValidationTypes.icon={type:"icon",titleMessageID:"Icon",pagingMessageID:"IconOnPage",errorsForm:!1,icon:"user-profile"},this.rules={required:{isNotEmpty:function(t,e){return!(!("placeholder"in document.createElement("input"))&&e&&(t===e.attr("placeholder")||t===H.translate("Required")))&&("string"==typeof t?(t=$.trim(t.replace(/<\/?[^>]*>/g,"")),0!==$.trim(t).length):"number"==typeof t?!isNaN(t):$.isArray(t)?t.length>0:!!t)},isRadioChecked:function(t){var e=t.attr("name");return e&&e.length&&$('input[name="'.concat(e,'"]:radio:checked')).length},check:function(t,e){this.message=H.translate("Required");return!(!$(e).is(".datepicker")||!$("#monthview-popup").is(":visible"))||(e.is(":radio")?this.isRadioChecked(e):this.isNotEmpty(t,e))},message:"Required",type:"error",id:"required"},date:{check:function(t,e,i){if(this.message=H.translate("InvalidDate"),i&&i.column){var n=i.column.formatter(i.row,i.cell,t,i.column,!0);if(n instanceof Date)return n&&n.getTime&&!isNaN(n.getTime())}if(t instanceof Date)return t&&t.getTime&&!isNaN(t.getTime());var s=t.indexOf(":")>-1?H.calendar().dateFormat.datetime:H.calendar().dateFormat.short,a=null;if(e&&e.data("datepicker")&&((a=e.data("datepicker")).settings.showTime&&(this.message=H.translate("InvalidDateTime")),s=a.pattern),i&&i.column){var o=i.column;"string"==typeof o.sourceFormat?s=i.column.sourceFormat:"string"==typeof o.dateFormat&&(s=i.column.dateFormat)}var r=!("MMMM"===s||"MMM"===s||"MM"===s||"MMMM d"===s||"d MMMM"===s||"yyyy"===s);return a&&(s={locale:a.locale.name,pattern:s,calendarName:a.currentCalendar.name}),!(void 0===H.parseDate(t,s,r)&&""!==t)},message:"Invalid Date",type:"error",id:"date"},availableDate:{check:function(e,i,n){this.message=H.translate("UnavailableDate");var s=!0;if(""===e)return s;if(!t.rules.date.check(e,i))return s=!1,this.message="",s;var a=i.data("datepicker"),o=a?a.settings:{};if(n&&n.column){var r=n.column.formatter(n.row,n.cell,e,n.column,!0);r instanceof Date&&(e=r),n.column.editorOptions&&(o=n.column.editorOptions)}if(!o||!o.disable)return!0;var c,d,h,u,p=Object.keys(o.disable).length>0,g=e;if("string"==typeof g){var f="locale"!==o.dateFormat?o.dateFormat:H.calendar().dateFormat.short;if(o.showTime){var m=o.timeFormat||H.calendar().timeFormat;f.indexOf(m)<0&&(f+=" ".concat(m))}if(a&&a.isIslamic&&(f={pattern:a.pattern,locale:a.locale.name}),n&&n.column){var v=n.column;v.sourceFormat?f="string"==typeof v.sourceFormat?{pattern:v.sourceFormat}:v.sourceFormat:v.dateFormat&&(f="string"==typeof v.dateFormat?{pattern:v.dateFormat}:v.dateFormat)}g=H.parseDate(g,f)}a&&a.isIslamic&&g instanceof Date&&(g=H.umalquraToGregorian(g.getFullYear(),g.getMonth(),g.getDate()));var b=o.useUTC?H.dateToUTC(g):g;if(b&&p&&"function"==typeof o.disable.callback)return!o.disable.callback(b.getFullYear(),b.getMonth(),b.getDate());if(b instanceof Array)return s;if(b&&p){h=o.useUTC?H.dateToUTC(new Date(o.disable.minDate)).setHours(0,0,0,0):new Date(o.disable.minDate).setHours(0,0,0,0),u=o.useUTC?H.dateToUTC(new Date(o.disable.maxDate)).setHours(0,0,0,0):new Date(o.disable.maxDate).setHours(0,0,0,0),o.disable.dayOfWeek&&-1!==o.disable.dayOfWeek.indexOf(b.getDay())&&(s=!1);var y=b.getFullYear();if(((b=b.setHours(0,0,0,0))<=h||b>=u)&&(s=!1),/string|number/.test(l(o.disable.years))&&(o.disable.years=[o.disable.years]),o.disable.years)for(var w=0,x=o.disable.years.length;w<x;w++)if(y===Number(o.disable.years[w])){s=!1;break}if(o.disable.dates)for(o.disable.dates.length&&"string"==typeof o.disable.dates&&(o.disable.dates=[o.disable.dates]),c=0,d=o.disable.dates.length;c<d;c++)if(b===(o.useUTC?H.dateToUTC(o.disable.dates[c]):new Date(o.disable.dates[c])).setHours(0,0,0,0)){s=!1;break}}return p&&(s=!!(s&&!o.disable.isEnable||!s&&o.disable.isEnable)),s},message:"Unavailable Date",type:"error",id:"availableDate"},rangeDate:{check:function(e,i){var n=this;this.message=H.translate("rangeDate");var s=!0,a=i.data("datepicker");if(a){var o=a.settings,r=e.split(o.range.separator),l=function(e,a){i.val(a),n.message="".concat(t.rules[e].message," (").concat(a,")"),t.rules[e].check(a,i)||(s=!1)};if(e.indexOf(a.pattern)>-1){if(1===r.length)l("date",r[0]);else if(2===r.length){var c=r[0]===a.pattern?1:0;l("date",r[c]),s&&l("availableDate",r[c])}}else 1===r.length?(l("date",r[0]),s&&l("availableDate",r[0])):2===r.length&&(l("date",r[0]),s&&l("date",r[1]),s&&l("availableDate",r[0]),s&&l("availableDate",r[1]))}return i.val(e),s},message:"Range Dates",type:"error",id:"rangeDate"},email:{check:function(t){this.message=H.translate("EmailValidation");return!t.length||/^([\w-]+(?:\.[\w-]+)*)@((?:[\w-]+\.)*\w[\w-]{0,66})\.([a-z]{2,16}(?:\.[a-z]{2})?)$/i.test(t)},message:"EmailValidation",type:"error",id:"email"},enableSubmit:{check:function(e,i){var n=i.closest(".signin").find('button[type="submit"]');return e.length&&(t.rules.email.check(e)||t.rules.passwordConfirm.check(e,i))?n.enable():n.disable(),!0},message:"",type:"error",id:"enableSubmit"},emailPositive:{check:function(e,i){if($.trim(e).length&&!i.is("[readonly]")){t.rules.emailPositive.positive=!0,this.message=H.translate("EmailValidation");var n=t.rules.email.check(e,i);return n&&(this.message=""),n}return t.rules.emailPositive.positive=!1,!0},message:"EmailValidation",type:"error",id:"emailPositive"},passwordReq:{check:function(t){this.message=H.translate("PasswordValidation");return!t.length||t.match(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[^a-zA-Z0-9])(?!.*\s).{10,}$/)},message:"PasswordValidation",type:"error",id:"passwordReq"},passwordConfirm:{check:function(e,i){this.message=H.translate("PasswordConfirmValidation");var n=$('input[type="password"]:not('.concat(i.attr("id"),")"),i.closest(".signin")).eq(0).val(),s=e===n&&t.rules.passwordReq.check(n);return!e.length||s},message:"PasswordConfirmValidation",type:"error",id:"passwordConfirm"},time:{check:function(t,e){if(!t)return!0;this.message=H.translate("InvalidTime");var i=e&&e.data("timepicker")?e.data("timepicker").settings:{},n=i&&i.timeFormat?i.timeFormat:H.calendar().timeFormat;if(e.attr("data-options")&&i&&!i.timeFormat){var s=JSON.parse(e.attr("data-options"));s.patternOptions&&s.patternOptions.format&&(n=s.patternOptions.format)}var a=H.parseDate(t,{dateFormat:n,strictTime:!0});return a instanceof Date&&!Number.isNaN(a)&&"Invalid Date"!==a.toString()},message:"Invalid Time",type:"error",id:"time"},test:{check:function(t){return"1"===t},message:"Value is not valid (test).",type:"error",id:"test"}}},Kn="toast",Xn={title:"(Title)",message:"(Content)",position:"top right",audibleOnly:!1,progressBar:!0,timeout:6e3,allowLink:!1,draggable:!1,savePosition:!1,uniqueId:null};function Qn(t,e){this.element=$(t),this.settings=E.mergeSettings(t,e,Xn),this.init()}Qn.prototype={init:function(){this.show()},show:function(){var t=this,e=this.settings,i=parseFloat(L.convertDelayToFPS(e.timeout)),n=e.allowLink?g.stripTags(e.message,"<a><br><p>"):g.stripHTML(e.message),s=!1,a=100;this.uniqueId=e.uniqueid?this.generateUniqueId("usersettings-position"):"";var o=$("#toast-container".concat(this.uniqueId)),r=$('\n      <div class="toast">\n        <span aria-relevant="additions text" aria-live="polite">\n          <span class="toast-title">'.concat(g.stripHTML(e.title),'</span>\n          <span class="toast-message">').concat(n,"</span>\n        </span>\n      </div>")),l=$('\n      <button type="button" class="btn-icon btn-close" title="'.concat(H.translate("Close"),'" aria-hidden="true">\n        ').concat($.createIcon("close"),'\n        <span class="audible">').concat(H.translate("Close"),"</span>\n      </button>\n    ")),c=$('<div class="toast-progress"></div>');o.length||(o=$('<div id="toast-container'.concat(this.uniqueId,'" class="toast-container"></div>')).appendTo("body")),o.removeClass("toast-top-left toast-top-right toast-bottom-right toast-bottom-left").addClass("toast-".concat(e.position.replace(" ","-"))),e.timeout=e.audibleOnly?100:e.timeout,e.progressBar&&r.append(c),o.append(r),r.addClass(e.audibleOnly?"audible":"effect-scale"),r.append(l),t.createDraggable(r,o);var d=o.children().length;this.toastIndex=d;for(var h=0,u=this.element.find(".toast-container").children();h<u.length;h++)d=u.length;E.addAttributes(r,this,this.settings.attributes),E.addAttributes(r.find("button"),this,this.settings.attributes,"btn-close");var p=new X({duration:L.convertDelayToFPS(e.timeout),timeoutCallback:function(){t.remove(r,d)},updateCallback:function(t){a=(t.duration-t.elapsedTime)/i*100,H.isRTL()&&(a=100-a),e.progressBar&&(c[0].style.width="".concat(a,"%"))}});function f(e){p.destroy(!0),t.remove(e,d)}J.register(p),$(document).on("keydown.toast-".concat(d," keyup.toast-").concat(d),(function(t){var e=(t=t||window.event).which||t.keyCode;t.ctrlKey&&80===e&&(s="keydown"===t.type,p[s?"pause":"resume"]()),"keydown"===t.type&&27===e&&(t.stopImmediatePropagation(),t.preventDefault(),f(r))})),r.on("mousedown.toast touchstart.toast mouseup.toast touchend.toast",(function(t){s=!!/mousedown|touchstart/i.test(t.type),p[s?"pause":"resume"]()})),l.on("click.toast",(function(){f(r)}))},createDraggable:function(t,e){var i=this;if(this.settings.draggable&&t[0]&&e[0]){var n=x.features.touch,s=e.parent();e.css({top:"",left:"",right:""});var a=e[0].getBoundingClientRect(),o=this.restorePosition(),r=a;o&&((r={top:o.top,left:o.left,width:t.outerWidth(),height:t.outerHeight()}).right=r.left+r.width,r.bottom=r.top+r.height,this.isPosInViewport(r)||(r=a));var l={top:"".concat(r.top,"px"),left:"".concat(r.left,"px")};e.is(".toast-bottom-right, .toast-top-right")&&(l.right="auto"),e.css(l),e.addClass("is-draggable");t.off("mousedown.toast touchstart.toast").on("mousedown.toast touchstart.toast",(function(t){n||t.preventDefault(),$(t.target).is("a, .btn-close")||e.drag({containment:"document"}).off("dragstart.toast").on("dragstart.toast",(function(){t.stopImmediatePropagation(),e.attr("aria-grabbed","true"),s.attr("aria-dropeffect","move")})).off("dragend.toast").on("dragend.toast",(function(){e.removeAttr("aria-grabbed"),s.removeAttr("aria-dropeffect"),i.savePosition({left:parseFloat(e.css("left")),top:parseFloat(e.css("top"))});var t=e.data("drag");t&&"function"==typeof t.destroy&&t.destroy()}))}));var c=function(t){var e=t.dragApi,i=t.x,n=t.y,s=e.getElementsFromPoint(i,n)[0];return!$(s).closest(".toast").length},d=function(t){var e=$.Event("keyup");$.extend(e,{ctrlKey:!0,altKey:!0,keyCode:80}),t.trigger(e)},h=$(document);h.off("mouseup.toast").on("mouseup.toast",(function(t){if(1===$("#toast-container".concat(i.uniqueId," .toast")).length){var n=e.data("drag");if(n&&"function"==typeof n.getElementsFromPoint){var s={dragApi:n,x:t.pageX,y:t.pageY};c(s)&&d(h)}}})).off("touchend.toast").on("touchend.toast",(function(t){if(1===$("#toast-container".concat(i.uniqueId," .toast")).length){var n=e.data("drag");if(n&&"function"==typeof n.getElementsFromPoint){var s=t.originalEvent;if(s.targetTouches.length>1)return;var a=s.changedTouches[0],o={dragApi:n,x:a.pageX,y:a.pageY};c(o)&&d(h)}}}))}},savePosition:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.settings.savePosition&&this.canUseLocalStorage()&&!$.isEmptyObject(t)&&(localStorage[this.uniqueId]=JSON.stringify(t),this.element.triggerHandler("settingschanged",[t]))},restorePosition:function(){if(!this.settings.savePosition||!this.canUseLocalStorage())return null;var t=localStorage[this.uniqueId];return t?JSON.parse(t):null},canUseLocalStorage:function(){try{if(localStorage.getItem)return!0}catch(t){return!1}return!1},generateUniqueId:function(t){return t=null==t?"":t,"toast-".concat(this.settings.uniqueid||"","-").concat(t).replace(/--/g,"-").replace(/-$/g,"")},isPosInViewport:function(t){return t.top>=0&&t.left>=0&&t.bottom<=(window.innerHeight||document.documentElement.clientHeight)&&t.right<=(window.innerWidth||document.documentElement.clientWidth)},unbind:function(t,e){return t.closest(".toast-container").off("dragstart.toast dragend.toast"),t.off("mousedown.toast mouseup.toast touchstart.toast touchend.toast"),t.find(".btn-close").off("click.toast"),void 0!==e&&$(document).off(["keydown.toast-".concat(e),"keyup.toast-".concat(e)].join(" ")),this},remove:function(t,e){var i=this,n=function(){t.remove(),!$("#toast-container".concat(i.uniqueId," .toast")).length&&i.destroy()};if(this.unbind(t,e),this.settings.audibleOnly)n();else{t.attr("aria-live","").attr("aria-relevent","").hide(),t.addClass("effect-scale-hide");var s=new X({duration:20,updateCallback:function(){},timeoutCallback:function(){n()}});J.register(s)}},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,Xn)),this.show()},destroy:function(){var t=this,e=$("#toast-container".concat(this.uniqueId));e[0]&&[].slice.call(e[0].querySelectorAll(".toast")).forEach((function(e){t.settings.audibleOnly=!0,t.remove($(e))}));$(document).off(["mouseup.toast","touchend.toast"].join(" ")),e.remove(),delete this.toastIndex,delete this.uniqueId,$.removeData(this.element[0],Kn)}},$.fn.toast=function(t){return this.each((function(){var e=$.data(this,Kn);e?e.updated(t):e=$.data(this,Kn,new Qn(this,t))}))};var Jn="Validator",Zn={inline:!0,message:"",type:"error",showTooltip:!1,isHelpMessage:!1};function ts(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Zn),this.init()}ts.prototype={init:function(){this.fields="input[data-validate], input[data-validation], textarea[data-validate], textarea[data-validation], select[data-validate], select[data-validation], div[data-validate], div[data-validation]",this.isPlaceholderSupport=!!("placeholder"in document.createElement("input")),this.inputs=this.element.find(this.fields),this.element.is(this.fields)&&(this.inputs=$().add(this.element)),this.timeout=null,this.attachEvents()},extractEvents:function(t){if(t.indexOf("{")>-1&&(t=JSON.parse(t.replace(/'/g,'"'))),"object"===l(t)){var e="";for(var i in t)-1===e.indexOf(t[i])&&(e+="".concat(t[i]," "));t=e=e.split(" ").join(".validate ")}return t},filterValidations:function(t,e){var i=[];if(!t)return[];if(t.indexOf("{")>-1&&(t=JSON.parse(t.replace(/'/g,'"'))),"object"===l(t))for(var n in t)e&&t[n].indexOf(e)>-1&&i.push(n);return i},setErrorIconOpacity:function(t){if(t.is(":text")&&t.is('[data-error-type="tooltip"]')){var e=this.calculateTextWidth(t.val()),i=t.outerWidth()-35;t.closest(".field, .field-short").find(".icon-error")[e>i?"addClass":"removeClass"]("lower-opacity")}},calculateTextWidth:function(t,e){this.canvas=this.canvas||(this.canvas=document.createElement("canvas"));var i=this.canvas.getContext("2d");return i.font=e||"14px arial",i.measureText(t).width},attachEvents:function(){var t=this,e="[data-validate],[data-validation]";this.inputs.each((function(){var t=$(this),e=t.attr("data-validate")||t.attr("data-validation");e&&e.indexOf("required")>-1&&t.addClass("required")})),this.inputs.filter("input, textarea, div").filter(e).not("input[type=file]").each((function(){var e=$(this),i=e.attr("data-validation-events"),n=t.extractEvents(i||"blur.validate change.validate keyup.validate");n.indexOf("enter.validate")>-1&&e.off("keypress.enter.validate").on("keypress.enter.validate",(function(e){var i=$(this);13===e.which&&t.validate(i,!0,e)})),e.off(n).on(n,(function(i){if("keyup"===i.type){if(9===i.keyCode)return;t.setErrorIconOpacity(e)}var n=$(this),s=n.data("handleEvent".concat([i.type||""]));(!n.is("[readonly]")||n.parent().is(".field-fileupload")||n.is(".lookup.is-not-editable"))&&(!s||s.type!==i.type||"validate"!==i.handleObj.namespace||n.is(".lookup")||n.closest(".modal:visible").length)&&(n.data("handleEvent".concat([i.type||""]),i.handleObj),setTimeout((function(){n.closest(".modal-engaged").length&&!n.closest(".modal-body").length||t.validate(e,!0,i)}),300))}))})),this.inputs.filter("input[type=checkbox]").filter(e).off("click.validate").on("click.validate",(function(e){t.validate($(this),!0,e)})),this.inputs.filter(":radio").off("click.validate").on("click.validate",(function(e){var i=$(this).parent().find(".radio").first();t.validate(i,!0,e)}));var i=this.inputs.filter("select").filter(e);if(i.length&&(i.off("change.validate listopened.validate listclosed.validate").on("change.validate",(function(e){t.validate($(this),!0,e)})).on("listopened.validate",(function(){var t=$(this),e=t.data("tooltip");e&&document.activeElement===t.data("dropdown").searchInput[0]&&e.hide(),$("#validation-tooltip").hide()})).on("listclosed.validate",(function(){var t=$(this),e=t.data("tooltip"),i=t.data("dropdown");x.features.touch&&(i.pseudoElem.focus(),setTimeout((function(){i.pseudoElem.blur()}),100)),i&&i.wrapper&&(e=i.wrapper.find(".icon-error").data("tooltip")),e&&document.activeElement!==t.data("dropdown").searchInput[0]&&e.show()})),i.filter((function(){return void 0!==$(this).data("dropdown")})).each((function(){$(this).data("dropdown").pseudoElem.off("blur.validate").on("blur.validate",(function(e){var i=$(this).closest(".field, .field-short").find("select");t.validate(i,!0,e)}))}))),this.element.is("form")&&this.element.attr("data-validate-on")){this.element.off("submit.validate").on("submit.validate",(function e(i){i.stopPropagation(),i.preventDefault(),t.validateForm((function(i){t.element.off("submit.validate"),t.element.triggerHandler("validated",i),t.element.data("isValid",i),t.element.on("submit.validate",e)}))}))}},validateForm:function(t){var e,i=this,n=[];i.inputs=this.element.find(i.fields),i.inputs.each((function(t){var e=$(this);if(e.attr("data-validate")){if("true"===e.attr("data-disable-validation")||e.hasClass("disable-validation"))return;for(var s=i.validate(e,!1,t),a=0;a<s.length;a++)n.push(s[a])}})),(e=$).when.apply(e,n).then((function(){t(!0)}),(function(){t(!1)}))},setPrimaryBtn:function(t,e){var i=t.find("[data-validate]:visible, select[data-validate], :checkbox[data-validate]"),n=!0;i.length>0&&i.each((function(){var t=$(this);if(!(t.closest(".datagrid-filter-wrapper").length>0)){var e=null!==t[0].offsetParent;t.is(".required")&&(e&&t.is(".editor")&&!t.html()&&(n=!1),!e&&!t.is("select, :checkbox")||t.val()||t.is(".editor")||(n=!1)),!e&&!t.is("select, :checkbox")||t.isValid()||(n=!1)}})),n?e.removeAttr("disabled"):e.attr("disabled","disabled")},value:function(t){return t.is("input[type=checkbox]")?t.prop("checked"):t.is("div")?t[0].innerHTML:t.val()},getTypes:function(t,e){var i,n=this.filterValidations(t.attr("data-validation-events"),e.type);return i=t.is("input.dropdown")&&t.prev().prev("select").attr("data-validate")?t.prev().prev("select").attr("data-validate").split(" "):t.is("input.dropdown")&&t.prev().prev("select").attr("data-validation")?t.prev().prev("select").attr("data-validation").split(" "):t.attr("data-validation")?t.attr("data-validation").split(" "):t.attr("data-validate")?t.attr("data-validate").split(" "):[],n.length>0&&(i=i.filter((function(t){return-1!==n.indexOf(t)}))),"keyup"===e.type&&(i=i.filter((function(t){return"date"!==t&&"time"!==t}))),i},setIconOnParent:function(t,e){var i,n,s,a=$.createIcon({classes:["icon-".concat(e)],icon:"".concat(e)}),o=t.closest(".tab-panel, .expandable-pane"),r=t.closest(".row.flex-align-bottom"),l=o.attr("id"),c=t.closest(".tab-container, .tab-panel-container, .expandable-area");if(r&&r.length&&($(".".concat(e),r).length?r.addClass("has-messages"):r.removeClass("has-messages")),c.is(".tab-panel-container")&&(c=c.prev(".tab-container")),c.is(".tab-container"))(i=$('.tab-list a[href="#'.concat(l,'"]'),c).closest(".tab")).length?$(".count",i).length&&(i=$(".count",i)):(l=$('a[href="#'.concat(l,'"]'),".popupmenu").closest(".popupmenu").attr("id"),i=$('.tab-list .tab[aria-controls="'.concat(l,'"]'),c),n=l);else{if(!c.is(".expandable-area"))return;i=$('.expandable-header[aria-controls="'.concat(l,'"] .title'),c)}n&&n.length&&(s=o.add($($(n).attr("href"))),$('a[role="menuitem"]',"#".concat(n)).each((function(){s=s.add($($(this).attr("href")))})));var d=$('a[href="#'.concat(o.attr("id"),'"]'),"#".concat(l));o&&$(".".concat(e),o).length||s&&$(".".concat(e),s).length?($(".".concat(e),o).length||(d.removeClass("is-".concat(e)),$(".icon-".concat(e),d).remove()),$(".".concat(e),o).length&&$(".".concat(e),s).length&&!$(".icon-".concat(e),d).length&&d.addClass("is-".concat(e)).append(a),$(".icon-".concat(e),i).length||c.hasClass("module-tabs")||i.addClass("is-".concat(e)).append(a)):((i=i.add(d)).removeClass("is-".concat(e)),$(".icon-".concat(e),i).remove())},validate:function(t,e,i){if(t.data("handleEvent".concat([i.type||""]),null),"true"===t.attr("data-disable-validation")||t.hasClass("disable-validation")||t.is(":disabled"))return[];var n,s,a,o,r,l=this,c=l.getTypes(t,i)||[],d=[],h=[],u=l.value(t),p=t.attr("placeholder");function g(e,i,s,a){!1===(n=Yn.rules[s]).positive&&l.removePositive(t),r=Yn.ValidationTypes[n.type]||Yn.ValidationTypes.error;var o="tooltip"!==t.attr("data-".concat(r.type,"-type"));if(e)0===$.grep(h,(function(t){return t===r.type})).length?(a.resolve(),n.positive&&(l.addMessage(t,n,o,i),l.removeMessage(t,n,!0),a.resolve(),l.addPositive(t))):r.errorsForm||a.resolve();else{if(!l.isPlaceholderSupport&&u===p&&n.message!==H.translate("Required"))return;l.addMessage(t,n,o,i),h.push(n.type),r.errorsForm?a.reject():a.resolve()}l.setIconOnParent(t,n.type);var c=l.getField(t).data("".concat(r.type,"message"));e&&c&&c.filter((function(t){return t.id===n.id||n.message})).length>0&&l.removeMessage(t,n,!0);var d=t.closest(".modal").find(".btn-modal-primary").not(".no-validation");d.length&&l.setPrimaryBtn(t.closest(".modal"),d);var g=t.closest("#calendar-popup").find(".btn-modal-primary").not(".no-validation");g.length&&l.setPrimaryBtn(t.closest("#calendar-popup"),g),"error"===n.type&&t.closest("form").triggerHandler("aftervalidate",{field:t,rule:n,isValid:e})}for(a=0,o=c.length;a<o;a++)n=Yn.rules[c[a]],s=$.Deferred(),n&&(n.async?n.check(u,t,g,s):g(n.check(u,t),e,c[a],s),d.push(s));return d},getField:function(t){return t.is("select")&&void 0!==t.data("dropdown")&&(t=t.data("dropdown").pseudoElem),t},hasError:function(t){return this.getField(t).hasClass("error")},addMessage:function(t,e,i,n,s){if(!(""===e.message||t.is(".dropdown, .multiselect")&&$("#dropdown-list").is(":visible"))){s=s||!1;var a=this.getField(t),o=a.data("".concat(e.type,"message")),r=Yn.ValidationTypes[e.type]||Yn.ValidationTypes.error;if(s||a.addClass("icon"===e.type?"custom-icon":e.type),!(o&&o===e.message||o&&o.filter((function(t){return(t.id||t.message)===(e.id||e.message)&&t.message===e.message})).length>0)){var l=e.message;if(o){for(var c=0;c<o.length;c++)l="• ".concat(o[c].message);l+="<br>• ".concat(e.message)}o||(o=[]),t.closest(".field, .field-short").find('[data-rule-id="'.concat(e.id||e.message,'"]')).remove(),(t.hasClass("dropdown")||t.hasClass("multiselect"))&&t.parent().find('.dropdown-wrapper > [data-rule-id="'.concat(e.id||e.message,'"]')).off("click.validate").remove(),(o=o.filter((function(t){return t.id!==e.id}))).push({id:e.id,message:e.message,type:e.type}),a.data("".concat(r.type,"message"),o),void 0!==$.fn.toast&&$("body").toast({title:H.translate(r.titleMessageID),audibleOnly:!0,message:l}),i?(t.data("isValid",!1),i&&this.showInlineMessage(t,e,s)):this.showTooltipMessage(t,l,r.type,n)}}},showIcon:function(t,e){var i=this.getField(t).addClass("icon"===e?"custom-icon":e),n=$.createIconElement({classes:["icon-".concat(e)],icon:e}),s=i.closest(".field, .field-short"),a=t.parent();return 0===s.find("svg.icon-".concat(e)).length?(a.is(".editor-container")&&t.parent().addClass("is-error"),1===t.parent(":not(.editor-container)").find(".btn-actions").length?a.find(".btn-actions").before(n):1===a.find(".data-description").length?a.find(".data-description").before(n):1===a.find(".field-info").length?a.find(".field-info").before(n):t.is("textarea")?t.after(n):t.is(".dropdown, .multiselect")?a.find(".dropdown-wrapper").append(n):(t.is(".spinbox")||t.is(".lookup"),a.append(n)),$(".icon-success",s).remove()):n=s.find("svg.icon-error"),n},showTooltipMessage:function(t,e,i,n){if(!t.is(":radio")){var s=t.is("button")?t:this.showIcon(t,i),a=s.data("tooltip"),o=t;if(t.is(".dropdown, .multiselect")&&void 0!==t.data("dropdown")){var r=t.data("dropdown").pseudoElem;o=r,r.addClass("icon"===i?"custom-icon":i)}t.closest(".field, .field-short").find(".formatter-toolbar").addClass("icon"===i?"custom-icon":i),a?a.content=e:(s.tooltip({content:e,placement:"bottom",placementOpts:{callback:function(t){var e=o[0].getBoundingClientRect(),i=a.tooltip[0].getBoundingClientRect(),n=$("html").is('[dir="rtl"]'),s=t.x,r=0;return n?i.left<e.left&&(r+=e.left-i.left):i.right>e.right&&(r+=-1*(i.right-e.right)),t.setCoordinate("x",s+r),t.nudges||(t.nudges={}),t.nudges.x=r,t}},trigger:"focus",isError:!0,tooltipElement:"#validation-tooltip"}),a=s.data("tooltip")),t.on("focus.validate",(function(){a&&a.show()})).on("blur.validate",(function(){a&&a.hide()})),a&&t.attr("data-error-type","tooltip"),this.setErrorIconOpacity(t),n&&a&&a.show()}},toggleRadioMessage:function(t,e,i,n,s){var a,o,r=t.attr("name");r&&r.length&&(a=$(':radio[name="'.concat(r,'"], :radio[name="').concat(r,'"] + label')),o=t.parent().is(".inline")?$(':radio[name="'.concat(r,'"]:last')).parent():$(':radio[name="'.concat(r,'"]:last + label')),s?(a.addClass("icon"===i?"custom-icon":i),$(n).addClass("radio-group-".concat(i)).insertAfter(o)):(a.removeClass(i),o.next(".radio-group-".concat(i)).remove()))},showInlineMessage:function(t,e,i){i=i||!1;var n,s,a=this.getField(t),o=Yn.ValidationTypes[e.type]||Yn.ValidationTypes.error;e.icon=e.icon||o.icon;var r="".concat(t.attr("id")||"id","-").concat(e.type);"error"===e.type||ft.currentTheme.id&&ft.new,s="".concat(o.type),n="icon"===e.type?""+'<div id="'.concat(r,'" class="custom-icon-message" data-rule-id="').concat(e.id||e.message,'">\n          ').concat($.createIcon({classes:["icon-custom"],icon:e.icon}),'\n          <pre class="audible">\n            ').concat(H.translate(o.titleMessageID),'\n          </pre>\n          <p class="message-text">').concat(e.message,"</p>\n        </div>"):""+'<div id="'.concat(r,'" class="').concat(o.type,'-message" data-rule-id="').concat(e.id||e.message,'">\n          ').concat($.createIcon({classes:["icon-".concat(o.type)],icon:s}),'\n          <pre class="audible">\n            ').concat(H.translate(o.titleMessageID),'\n          </pre>\n          <p class="message-text">').concat(e.message,"</p>\n        </div>"),t.is(":radio")?this.toggleRadioMessage(t,e.message,o.type,n,!0):(a.closest(".field, .field-short").find(".formatter-toolbar").addClass("icon"===o.type?"custom-icon":o.type),a.closest(".field, .field-short").append(n),a.closest(".field, .field-short").find(".colorpicker-container").addClass("icon"===o.type?"custom-icon":o.type)),i||a.addClass("icon"===e.type?"custom-icon":e.type),t.is(".spinbox")&&a.closest(".spinbox-wrapper").addClass("icon"===o.type?"custom-icon":o.type),"error"===o.type&&t.parent().find(".icon-success").remove(),t.attr("aria-describedby",r),t.attr("aria-invalid","true");var l=t.parent().find(".dropdown.error"),c=l.attr("aria-label"),d="".concat(c).concat(e.message);l&&l.attr("aria-label",d),t.triggerHandler(o.type,{field:t,message:e.message}),t.closest("form").triggerHandler(o.type,{field:t,message:e.message})},addPositive:function(t){var e=$.createIcon({icon:"success",classes:"icon-success"});$(".icon-success",t.parent(".field, .field-short")).length||t.parent(".field, .field-short").append(e)},removeMessage:function(t,e){var i=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],n=this.getField(t),s=n.data("".concat(e.type,"message")),a=e.id||e.message;if(e&&("error"!==e.type||s)&&0!==s.filter((function(t){return t.id===a})).length){var o=s.filter((function(t){return t.id!==a})),r=0===o.length;if(r?n.removeData("".concat(e.type,"message")):n.data("".concat(e.type,"message"),o),t.closest(".field, .field-short").find('[data-rule-id="'.concat(e.id||e.message,'"]')).remove(),(t.hasClass("dropdown")||t.hasClass("multiselect"))&&t.parent().find('.dropdown-wrapper > [data-rule-id="'.concat(e.id||e.message,'"]')).off("click.validate").remove(),r&&"error"===e.type&&i&&(t.triggerHandler("valid",{field:t,message:""}),t.closest("form").triggerHandler("valid",{field:t,message:""})),r&&(t.removeClass("".concat(e.type," custom-icon")).data("isValid",!0),(t.hasClass("dropdown")||t.hasClass("multiselect"))&&(t.next().next().removeClass("".concat(e.type," custom-icon")),t.next().find("div.dropdown").removeClass("".concat(e.type," custom-icon"))),t.is(":radio")&&this.toggleRadioMessage(t,"",e.type,"",!1),t.hasClass("spinbox")&&t.closest(".spinbox-wrapper").removeClass("".concat(e.type," custom-icon")),t.hasClass("colorpicker")&&t.parent(".colorpicker-container").removeClass(e.type),t.closest(".field-fileupload").length>0&&t.closest(".field-fileupload").find("input.".concat(e.type)).removeClass(e.type),t.removeAttr("aria-describedby").removeAttr("aria-invalid"),"tooltip"===t.attr("data-".concat(e.type,"-type")))){var l=t.closest(".field, .field-short").find(".icon-error").data("tooltip");l&&l.destroy(),void 0!==t&&this.hideTooltipMessage(t),this.inputs&&this.inputs.filter("input, textarea").off("focus.validate"),t.parent().find(".dropdown-wrapper > .icon-".concat(e.type)).off("click.validate").remove(),t.parent().find(".icon-".concat(e.type)).remove(),t.next(".icon-".concat(e.type)).off("click.validate").remove(),t.parent(".field, .field-short").find(".icon-".concat(e.type)).remove(),t.next(".inforCheckboxLabel").next(".icon-".concat(e.type)).remove()}}},hideTooltipMessage:function(t){var e=t.data("tooltip");e&&e.destroy(),t.off("focus.validate"),t.off("blur.validate")},removePositive:function(t){$(".icon-success",t.parent(".field, .field-short")).remove()},resetForm:function(t){var e=t.find("input, select, textarea, div.dropdown, div.editor");e.removeClass("error"),t.find(".error").removeClass("error"),t.find(".icon-error").remove(),t.find(".icon-success").remove(),t.find(".error-message").remove(),e.removeClass("alert"),t.find(".alert").removeClass("alert"),t.find(".icon-alert").remove(),t.find(".alert-message").remove(),e.removeClass("info"),t.find(".info").removeClass("info"),t.find(".icon-info").remove(),t.find(".info-message").remove(),setTimeout((function(){$("#validation-errors").addClass("is-hidden")}),300),e.data("isDirty",!1).removeClass("isDirty"),t.find(".isDirty").removeClass("isDirty"),t.is("form")&&t[0].reset();var i=$.fn.validation.ValidationTypes;Object.keys(i).forEach((function(t){e.removeData("".concat(t,"message"))})),e.removeData("isValid")},isFormValid:function(t){if($(t).find(".error-message").length>0)return!1;for(var e=$(t).find('[data-validate*="required"]'),i=0;i<e.length;i++){var n=$(e[i]),s=this.value(n);if((n.is(":visible")||n.is("select"))&&!s)return!1}return!0},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.init()},destroy:function(){this.inputs=null}};var es="validate",is={type:"error",inline:!0};$.fn.validation=Yn,$.fn.getMessage=function(t){t||(t={type:"error"});var e="".concat(t.type,"message"),i=$.fn.getField($(this)).data(e),n="";if(i){if(1===i.length)return i[0].message;for(var s=0;s<i.length;s++)n+="• ".concat(i[s].message)}return n},$.fn.getMessages=function(t){t||(t={type:"error"});var e="".concat(t.type,"message"),i=$.fn.getField($(this)).data(e);return i||(i=[]),i},$.fn.getField=function(t){return t.is("select")&&void 0!==t.data("dropdown")&&(t=t.data("dropdown").pseudoElem),t},$.fn.getErrorMessage=function(t){return V("$.fn.getMessage","$.fn.getErrorMessage"),t=E.extend({},t,is),$(this).getMessage(t)},$.fn.scrollIntoView=function(t,e){"boolean"!=typeof t&&(t=void 0);var i=new ts(this,e).getField($(this));i[0].scrollIntoView(t),i.focus(),i.is("input.checkbox")&&(i=i.next(".checkbox-label"))[0]&&i[0].scrollIntoView(t)},$.fn.addMessage=function(t){return this.each((function(){var e=$.data(this,Jn);e||(e=$.data(this,Jn,new ts(this,t)));var i={message:t.message,type:t.type,triggerEvents:t.triggerEvents,icon:t.icon,id:t.id||t.message},n=$(this);e.addMessage(n,i,t.inline,t.showTooltip,t.isAlert),e.setIconOnParent(n,t.type)}))},$.fn.addError=function(t){V("$.fn.addMessage","$.fn.addError");var e=!0;return"boolean"==typeof t.inline&&!1===t.inline&&(e=!1),(t=E.extend({},t,is)).inline=e,this.each((function(){$(this).addMessage(t)}))},$.fn.removeMessage=function(t){return t||(t={type:"error"}),this.each((function(){var e=$.data(this,Jn);e||(e=$.data(this,Jn,new ts(this,t)));var i=$(this),n="".concat(t.type,"message"),s=$.fn.getField(i).data(n);if(i.hasClass("error")&&"error"===t.type&&i.removeClass("error"),s){for(var a=0;a<s.length;a++)e.removeMessage(i,s[a],t.triggerEvents);e.setIconOnParent(i,t.type),$.removeData(this,Jn)}}))},$.fn.removeError=function(t){return V("$.fn.removeMessage","$.fn.removeError"),t=E.extend({},t,is),this.each((function(){return $(this).removeMessage(t)}))},$.fn.validate=function(t,e){var i="string"==typeof t;return this.each((function(){var n=$.data(this,es);if(n&&"string"!=typeof n){if(i)return void("function"==typeof n[t]&&n[t](e));n.updated(t)}else n=$.data(this,es,new ts(this,t))}))},$.fn.isValid=function(){var t=$(this).data("isValid");return null==t||t},$.fn.validateField=function(){var t=$(this),e=Soho.components.Validator.prototype;e&&e.validate&&e.validate(t,!1,0)},$.fn.resetForm=function(){Soho.components.Validator.prototype.resetForm(this)};var ns="datepicker",ss=[{name:"Public Holiday",color:"azure06",dates:[]},{name:"Weekends",color:"turquoise06",dayOfWeek:[]}],as={showTime:!1,useCurrentTime:!1,timeFormat:void 0,minuteInterval:void 0,secondInterval:void 0,mode:void 0,roundToInterval:void 0,dateFormat:"locale",placeholder:!1,firstDayOfWeek:0,disable:{callback:null,dates:[],years:[],minDate:"",maxDate:"",dayOfWeek:[],isEnable:!1,restrictMonths:!1},showLegend:!1,showMonthYearPicker:!0,showWeekNumber:!1,hideDays:!1,hitbox:!1,yearsAhead:3,yearsBack:2,legend:[],range:{useRange:!1,start:"",end:"",separator:" - ",minDays:0,maxDays:0,selectForward:!1,selectBackward:!1,includeDisabled:!1,selectWeek:!1},calendarName:null,locale:null,language:null,useUTC:!1,hideButtons:!1,showToday:!0,onOpenCalendar:null,isMonthPicker:!1,attributes:null,autocompleteAttribute:"off",useMask:!0,tabbable:!0,incrementWithKeyboard:!1,todayWithKeyboard:!1};function os(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,as),this.settings.legend&&0===this.settings.legend.length&&(this.settings.legend=ss),this.init()}os.prototype={init:function(){this.build(),this.handleEvents()},build:function(){var t,e=this.settings.autocompleteAttribute;"string"==typeof e&&e.length?this.element.attr("autocomplete","".concat(g.ensureAlphaNumeric(e))):this.element.hasAttr("autocomplete")&&this.element.removeAttr("autocomplete");var i=this.element.next();i.is("button.trigger")?this.trigger=i:this.trigger=$('<button class="btn-icon trigger" type="button" '.concat(this.settings.hitbox?'data-options="{hitbox: true}"':"",'>\n        <span class="audible"></span>\n        ').concat($.createIcon("calendar"),"\n      </button>")).insertAfter(this.element),this.element.hasClass("hidden")&&this.trigger.addClass("hidden"),this.element.is(":disabled")&&this.disable(),this.element.is("[readonly]")&&this.readonly(),this.makeTabbable(!this.element.is(":disabled")&&"-1"!==this.element.attr("tabindex")&&this.settings.tabbable),this.setExtraAttributes(),this.settings.range.selectWeek&&(this.settings.selectForward=!0,this.settings.minDays=6,this.settings.maxDays=7),this.setLocale(),this.addAria(),this.settings.locale||this.settings.language||this.setCurrentCalendar(),null!==(t=this.trigger)&&void 0!==t&&(null==t?void 0:t.data("button"))||this.trigger.button()},setLocale:function(){var t=this,e=this.settings;this.locale=H.currentLocale,this.settings.language?(H.getLocale(this.settings.language),this.language=this.settings.language):this.language=H.currentLanguage.name,e.locale&&H.getLocale(e.locale).done((function(e){var i=t.getSimilarApi("locale",e);i.forEach((function(i){i.locale=H.cultures[e],i.language=t.settings.language||i.locale.language,i.setCurrentCalendar()})),0===i.length&&(t.locale=H.cultures[e],t.language=t.settings.language||t.locale.language,t.setCurrentCalendar())})),e.language&&H.getLocale(e.language).done((function(){t.getSimilarApi("language",e.language).forEach((function(t){t.language=e.language}))}))},getSimilarApi:function(t,e){var i=[].slice.call(document.querySelectorAll(".datepicker")),n=[];return i.forEach((function(i){var s=$(i).data("datepicker");s&&s.settings[t]===e&&n.push(s)})),n},setCurrentCalendar:function(){this.currentCalendar=H.calendar(this.settings.locale||this.locale.name,this.settings.language,this.settings.calendarName),this.isIslamic="islamic-umalqura"===this.currentCalendar.name,this.isRTL="right-to-left"===(this.locale.direction||this.locale.data.direction),this.isFullMonth=this.settings.dateFormat.indexOf("MMMM")>-1,this.setFormat(),this.mask()},addAria:function(){this.element.attr({role:"combobox","aria-expanded":"false"}),this.trigger.attr({role:"button","aria-haspopup":"dialog"}),this.label=$('label[for="'.concat(this.element.attr("id"),'"]'));var t=". ".concat(H.translate("PressDown",{locale:this.locale.name,language:this.language})),e=this.label.children("span.audible");e.length?e.text(t):this.label.append('<span class="audible">'.concat(t,"</span>")),this.trigger.children(".audible").text(H.translate("DatePickerTriggerButton"))},handleKeys:function(t){var e=this,i=this.settings;if(t.is("#monthview-popup"))return t.off("keydown.datepicker").on("keydown.datepicker",".monthview-table, .monthview-monthyear-pane",(function(t){var i=!1,n=t.keyCode||t.charCode||0;return 9===n&&(e.containFocus(t),i=!0),27===n&&(e.closeCalendar(),e.element.focus(),t.stopImmediatePropagation(),i=!0),84===n&&(e.closeCalendar(),e.element.focus(),i=!0),!i||(t.stopPropagation(),t.preventDefault(),!1)})),void t.off("keydown.datepicker-tab").on("keydown.datepicker-tab","td, input, div.dropdown, button",(function(t){return 9!==(t.keyCode||t.charCode||0)||(i.range.useRange&&$(t.target).is(".next")&&!i.range.selectWeek?e.calendarAPI.days.find("td:visible:last").attr("tabindex",0).focus():e.containFocus(t),t.stopPropagation(),t.preventDefault(),!1)}));t.off("keydown.datepicker").on("keydown.datepicker",(function(t){var i,n,s=!1,a=t.keyCode||t.charCode||0,o=(null===(i=e.settings)||void 0===i||null===(n=null==i?void 0:i.dateFormat)||void 0===n||null==n?void 0:n.indexOf("-"))>-1;if(40!==a||e.isOpen()||(s=!0,e.openCalendar(),setTimeout((function(){e.setFocusAfterOpen()}),200)),84===a&&e.settings.todayWithKeyboard&&(s=!0,e.setToday()),!o&&e.settings.incrementWithKeyboard){var r=$(t.currentTarget).val();(189===a||109===a||x.browser.isFirefox()&&173===a)&&(s=!0,e.adjustDay(!1,r)),(187===a||107===a||x.browser.isFirefox()&&61===a)&&(s=!0,e.adjustDay(!0,r))}return!s||(t.stopPropagation(),t.preventDefault(),!1)}))},containFocus:function(t){var e=t.shiftKey,i=this.popup.find(":focusable"),n=$(t.currentTarget);(n.is(".monthview-monthyear-pane, #btn-monthyear-pane")||n.closest(".is-monthyear.is-monthonly").length>0)&&(i=this.popup.find(":focusable").not("td").not(".picklist-item a").add(".picklist-item.is-selected a:visible, .picklist-item.up a:visible, .picklist-item.down a:visible"));var s=i.index($(":focus"));s=e?s-1<0?i.length-1:s-1:s+1>=i.length?0:s+1,i.eq(s).focus()},setFormat:function(){var t=this.settings,e="object"===l(H)&&this.currentCalendar.dateFormat?this.currentCalendar.dateFormat:null,i="object"===l(H)&&this.currentCalendar.timeFormat?this.currentCalendar.timeFormat:null;"object"===l(H)&&this.settings.calendarName&&(e=H.calendar(this.settings.locale,this.settings.language,this.settings.calendarName).dateFormat,i=H.calendar(this.settings.locale,this.settings.language,this.settings.calendarName).timeFormat),"object"===l(e)&&void 0!==e.short&&(e=e.short),"locale"===t.dateFormat?(this.pattern=e+(t.showTime?" ".concat(t.timeFormat||i):""),t.dateFormat=this.pattern):this.pattern=t.dateFormat+(t.showTime&&t.timeFormat?" ".concat(t.timeFormat):""),this.show24Hours=(this.pattern.match("H")||[]).length>0,this.isSeconds=(this.pattern.match("ss")||[]).length>0},mask:function(){if(this.settings.useMask){this.setFormat();var t=this.settings,e={process:"date",keepCharacterPositions:!0,patternOptions:{format:this.pattern}},i="date availableDate",n={date:"change blur enter",availableDate:"change blur"};t.range.useRange&&(e.process="rangeDate",e.patternOptions.delimeter=t.range.separator,i="rangeDate",n={rangeDate:"change blur"}),e.processOnInitialize=!1,this.isFullMonth?this.pattern=this.settings.dateFormat:void 0===this.element.data("mask")&&this.element.mask(e),this.addedValidation=!1,this.element[0]&&this.element[0].getAttribute&&!this.element[0].getAttribute("data-validate")&&(this.addedValidation=!0,this.element.attr({"data-validate":i,"data-validation-events":JSON.stringify(n)}).validate()),this.setPlaceholder()}},setPlaceholder:function(){var t,e=this,i=this.settings,n=this.pattern;!i.placeholder||this.element.attr("placeholder")&&"M / D / YYYY"!==this.element.attr("placeholder")||(i.range.useRange&&(n=i.range.first&&i.range.first.date?(t=i.range.first.date,H.formatDate(t,{pattern:e.pattern,locale:e.locale.name})+i.range.separator+this.pattern):this.pattern+i.range.separator+this.pattern),this.element[0].setAttribute("placeholder",n))},setExtraAttributes:function(){this.settings.attributes&&(E.addAttributes(this.element,this,this.settings.attributes),E.addAttributes(this.trigger,this,this.settings.attributes,"trigger"))},isOpen:function(){return this.popup&&this.popup.is(":visible")&&!this.popup.hasClass("is-hidden")},loadLegend:function(t){void 0===t||t.length<1||this.calendarAPI.loadLegend(t)},openCalendar:function(){var t=this,e=this,i=this.settings,n={};if(!this.element.is(":disabled")&&!this.element.attr("readonly")||0!==this.element.closest(".monthview").length){$("#validation-tooltip").addClass("is-hidden"),this.element.addClass("is-active is-open").trigger("listopened"),this.timepickerContainer=$('<div class="datepicker-time-container"></div>');var s='<button type="button" class="is-cancel btn-tertiary">\n      '.concat(H.translate(this.settings.isMonthPicker?"Cancel":"Clear",{locale:this.locale.name,language:this.language}),"\n    </button>"),a=' <button type="button" class="is-select btn-primary">\n      '.concat(H.translate("Apply",{locale:this.locale.name,language:this.language}),"\n    </button>");if(this.footer=$(""+'<div class="popup-footer">\n        '.concat(this.isRTL?a+s:s+a,"\n      </div>")),i.hideDays&&(this.footer=$(""+'<div class="popup-footer">\n          <button type="button" class="is-cancel btn-tertiary">\n            '.concat(H.translate("Clear",{locale:this.locale.name,language:this.language}),'\n          </button>\n          <button type="button" class="is-select-month btn-primary">\n            ').concat(H.translate("Apply",{locale:this.locale.name,language:this.language}),"\n          </button>\n        </div>"))),i.hideButtons&&(this.footer=$("")),i.showTime&&(void 0===i.timeFormat?n.timeFormat=this.pattern.slice(this.pattern.indexOf(" ")).trim():n.timeFormat=i.timeFormat,void 0!==i.minuteInterval&&(n.minuteInterval=i.minuteInterval),void 0!==i.secondInterval&&(n.secondInterval=i.minuteInterval),void 0!==i.mode&&(n.mode=i.mode),void 0!==i.roundToInterval&&(n.roundToInterval=i.roundToInterval)),this.calendarContainer=$('<div class="monthview-container"></div>'),this.setValueFromField(),this.settings.showTime){""===this.element.val()&&this.currentDate&&this.currentDate.getDate()&&!this.settings.useCurrentTime&&(this.currentDate.setHours(0),this.currentDate.setMinutes(0),this.currentDate.setSeconds(0));var o=this.settings.timeFormat?this.settings.timeFormat:null;o||(o=this.isSeconds?this.currentCalendar.dateFormat.timestamp:this.currentCalendar.dateFormat.hour),n.parentElement=this.timepickerContainer,n.locale=this.settings.locale,n.language=this.settings.language,this.time=H.formatDate(this.currentDate,{pattern:o,locale:this.locale.name,language:this.language}),this.timepicker=this.timepickerContainer.timepicker(n).data("timepicker"),this.setUseCurrentTime(),this.timepickerContainer.find(".dropdown").dropdown(),this.timepickerContainer.on("change.datepicker",(function(){var i=t.setTime(e.isIslamic?e.currentDateIslamic:e.currentDate);t.setValue(i,!0,!0)})),setTimeout((function(){t.timepicker.initValues=t.timepicker.getTimeFromField(t.time),t.timepicker.afterShow(t.timepickerContainer)}),1)}this.todayDate=new Date,this.todayMonth=this.todayDate.getMonth(),this.todayYear=this.todayDate.getFullYear(),this.todayDay=this.todayDate.getDate(),this.isIslamic&&(this.todayDateIslamic=H.gregorianToUmalqura(this.todayDate),this.todayYear=this.todayDateIslamic[0],this.todayMonth=this.todayDateIslamic[1],this.todayDay=this.todayDateIslamic[2]),this.settings.year=this.currentYear,this.settings.month=this.currentMonth,this.isIslamic&&(this.settings.activeDateIslamic=this.activeDate instanceof Date?H.gregorianToUmalqura(this.activeDate):this.activeDate),this.settings.onOpenCalendar?(this.settings.activeDate=this.settings.onOpenCalendar(),this.currentDate=this.settings.activeDate,this.isIslamic?(this.settings.activeDateIslamic=H.gregorianToUmalqura(this.settings.activeDate),this.settings.year=this.settings.activeDateIslamic[0],this.settings.month=this.settings.activeDateIslamic[1]):(this.settings.year=this.settings.activeDate.getFullYear(),this.settings.month=this.settings.activeDate.getMonth())):(this.settings.activeDate=this.currentDate||this.todayDate,this.settings.activeDateIslamic=this.currentDateIslamic||this.todayDateIslamic),this.settings.isPopup=!0,this.settings.headerStyle="simple",this.settings.onSelected=function(i,n){if(t.currentDate=new Date(n.year,n.month,n.day),e.settings.range.useRange&&e.settings.range.first&&e.settings.range.selectWeek){var s=Gi.firstDayOfWeek(new Date,t.settings.firstDayOfWeek),a=Gi.lastDayOfWeek(new Date,t.settings.firstDayOfWeek);return e.settings.range.first={},e.settings.range.second=void 0,e.setWeekRange({day:s.getDate(),month:s.getMonth(),year:s.getFullYear()},{day:a.getDate(),month:a.getMonth(),year:a.getFullYear()}),e.closeCalendar(),void e.element.focus()}e.settings.range.useRange&&e.settings.range.first&&!e.settings.range.selectWeek||(e.insertDate(t.currentDate),n.close&&(e.closeCalendar(),e.element.focus()))},this.settings.range.useRange&&this.settings.range.selectWeek&&(this.settings.onKeyDown=function(t){return 37!==t.key&&39!==t.key&&(38!==t.key&&40!==t.key&&13!==t.key)}),this.calendarAPI=new Sn(this.calendarContainer,this.settings),this.calendar=this.calendarAPI.element,this.calendar.on("beforemonthrendered",(function(){t.element.trigger("beforemonthrendered")})),this.calendar.on("monthrendered",(function(e,i){t.element.trigger("monthrendered",i)})),this.element.trigger("monthrendered",{year:this.calendarAPI.currentYear,month:this.calendarAPI.currentMonth,elem:this.calendar,api:this.calendarAPI}),i.showTime&&this.calendar.addClass("is-timepicker"),i.hideDays&&(this.calendar.addClass("is-monthyear"),"MMMM"!==i.dateFormat&&"MMM"!==i.dateFormat&&"MM"!==i.dateFormat||this.calendar.addClass("is-monthonly"),"yyyy"===i.dateFormat&&this.calendar.addClass("is-yearonly")),this.calendar.append(i.showTime?this.timepickerContainer:"",this.footer);var r=this.element,l=this.isRTL?"right":"left",c=this.element.parent();c.is(".datagrid-cell-wrapper")&&(l="center",r=this.element.next(".icon, .trigger"));var d="bottom";c.offset().top<400&&c.offset().top>185&&window.innerHeight<800&&(d=H.isRTL()?"left":"right");var h={content:this.calendar,placementOpts:{parent:r,parentXAlignment:l,strategies:["flip","nudge","shrink"]},placement:d,popover:!0,appendTo:this.settings.appendTo,trigger:"immediate",isRangeDatepicker:i.range.useRange,extraClass:this.settings.range.selectWeek?"monthview-popup is-range-week":"monthview-popup",tooltipElement:"#monthview-popup",initializeContent:!1};this.trigger.popover(h).off("show.datepicker").on("show.datepicker",(function(){if(window.innerHeight<400&&t.popupClosestScrollable&&1===t.popupClosestScrollable.length&&(t.popup.find(".arrow").hide(),t.popup.css({"min-height":$("html").hasClass("theme-new-light")?"":"".concat(t.popupClosestScrollable[0].scrollHeight-521,"px"),height:""}),(window.innerHeight-t.popup.height())/window.innerHeight<.15&&t.popupClosestScrollable.children().not(".popover").css("padding-top","80px"),t.popupClosestScrollable.css("min-height","375px")),(window.innerHeight-t.popup.height())/window.innerHeight<.15&&t.popup.css({top:"".concat(t.popup.position().top<0?0:t.popup.position().top+50,"px")}),t.settings.hideButtons&&t.popup.addClass("hide-buttons"),t.settings.showMonthYearPicker&&(t.popup.find(".expandable-area").expandablearea({animationSpeed:150,trigger:"btn-monthyear-pane"}),t.popup.find(".btn-monthyear-pane").button()),t.settings.range.selectWeek){var e=t.popup.find("tbody");t.popup.find(".monthview-table tr").hover((function(t){var i=$(t.currentTarget);e.find("td").removeClass("is-selected range-selection end-date"),i.find("td").addClass("range-selection")}))}})).off("hide.datepicker").on("hide.datepicker",(function(){setTimeout((function(){t.popupClosestScrollable.children().not(".popover").css("padding-top",""),t.popupClosestScrollable.add(t.popup).css("min-height",""),t.closeCalendar()}),1)})),this.handleKeys($("#monthview-popup")),$(".monthview-footer a",this.calendar).button(),this.popup=$("#monthview-popup"),this.popupClosestScrollable=this.popup.closest(".scrollable"),this.popup.attr("role","dialog"),this.originalDate=this.element.val(),this.calendarAPI.currentDate=this.currentDate,this.calendarAPI.currentDateIslamic=this.currentDateIslamic,this.calendarAPI.validatePrevNext(),this.calendarAPI.days.off("click.datepicker").on("click.datepicker","td",(function(){var t=$(this);if(t.hasClass("is-disabled"))e.calendarAPI.activeTabindex(t,!0);else{if(!i.range.useRange||i.range.first&&!i.range.second||i.range.selectWeek||e.calendarAPI.days.find(".is-selected").removeClass("is-selected range").removeAttr("aria-selected"),i.range.useRange&&i.range.selectWeek){var n=e.calendarAPI.getCellDate(e.calendar.find("td.range-selection").first()),s=e.calendarAPI.getCellDate(e.calendar.find("td.range-selection").last());return void e.setWeekRange(n,s)}i.range.useRange||e.calendarAPI.days.find(".is-selected").removeClass("is-selected").removeAttr("aria-selected").removeAttr("tabindex");var a=$(this);a.addClass("is-selected".concat(i.range.useRange?" range":"")).attr("aria-selected","true"),e.lastValue=null,e.insertSelectedDate(a),i.range.useRange?e.isFocusAfterClose=!0:i.showTime||(e.closeCalendar(),e.element.focus())}})),this.footer.off("click.datepicker").on("click.datepicker","button",(function(t){var n=$(this);if(n.hasClass("is-cancel")&&(e.settings.isMonthPicker||(e.element.val("").trigger("change").trigger("input"),e.currentDate=null,e.clearRangeDates()),e.closeCalendar(),e.element.focus()),n.hasClass("is-cancel-month-pane")&&e.calendarAPI.monthYearPane.data("expandablearea").close(),n.hasClass("is-select-month")||n.hasClass("is-select-month-pane")){var s=parseInt(e.calendarAPI.monthYearPane.find(".is-year .is-selected a").attr("data-year"),10),a=parseInt(e.calendarAPI.monthYearPane.find(".is-month .is-selected a").attr("data-month"),10);e.currentDate=new Date(s,a,1),e.isIslamic&&(e.currentDateIslamic[0]=s,e.currentDateIslamic[1]=a,e.currentDateIslamic[2]=1,e.currentYear=s,e.currentMonth=a,e.currentDay=1,e.currentDate=H.umalquraToGregorian(s,a,1)),i.range.useRange?e.isFocusAfterClose=!1:n.hasClass("is-select-month")&&(e.insertDate(e.isIslamic?e.currentDateIslamic:e.currentDate),e.closeCalendar()),n.hasClass("is-select-month-pane")&&e.calendarAPI.showMonth(a,s)}if(n.hasClass("is-select")){var o=e.calendarAPI.setRangeSelByClick();if(1===o)e.closeCalendar();else if(2===o)e.element.trigger("change",[i.range.data]).trigger("input",[i.range.data]),e.closeCalendar();else{if(3===o)return void t.preventDefault();e.insertSelectedDate(),e.closeCalendar()}n.hasClass("is-select-month-pane")||e.element.focus()}t.preventDefault(),n.hasClass("is-select-month-pane")&&(e.calendarAPI.monthYearPane.data("expandablearea").close(),e.popup.find(".btn-primary.is-select").focus())})),this.popup.off("click.datepicker-today").on("click.datepicker-today",".hyperlink.today:not(.is-disabled)",(function(t){t.preventDefault(),i.range.useRange?(e.setToday(!0),(!i.range.second||i.range.second&&!i.range.second.date)&&t.preventDefault()):(e.setToday(),e.closeCalendar())})),setTimeout((function(){E.addAttributes(t.popup.find(".btn-monthyear-pane"),t,t.settings.attributes,"btn-monthyear"),E.addAttributes(t.popup.find(".is-cancel"),t,t.settings.attributes,"btn-cancel"),E.addAttributes(t.popup.find(".is-select"),t,t.settings.attributes,"btn-select"),e.calendarAPI.validatePrevNext(),e.setFocusAfterOpen()}),50)}},setUseCurrentTime:function(){if(this.settings.useCurrentTime&&this.timepicker.minuteSelect.length){var t=this.isSeconds&&this.timepicker.secondSelect.length,e=function(t){return(t<10?"0":"")+t},i=function(t,e){t.find("option:contains(".concat(e,")")).length||(t.find("option:selected").prop("selected",!1),t.prepend($("<option selected >".concat(e,"</option>"))))},n=new Date;i(this.timepicker.minuteSelect,e(n.getMinutes())),t&&i(this.timepicker.secondSelect,e(n.getSeconds())),"string"==typeof this.time&&""!==this.time&&this.currentDate&&this.currentDate.getDate()&&(i(this.timepicker.minuteSelect,e(this.currentDate.getMinutes())),t&&i(this.timepicker.secondSelect,e(this.currentDate.getSeconds())))}},clearRangeDates:function(){var t=this.settings;t.range.useRange&&(t.range.start=as.range.start,t.range.end=as.range.end,t.range.data&&delete t.range.data)},insertSelectedDate:function(t){var e=this,i=e.calendarAPI.getCellDate(t||e.calendar.find("td.is-selected").last()),n=i.day,s=i.month,a=i.year;e.currentDate=new Date(a,s,n),e.isIslamic&&(e.currentDateIslamic[0]=a,e.currentDateIslamic[1]=s,e.currentDateIslamic[2]=n,e.currentYear=a,e.currentMonth=s,e.currentDay=n,e.currentDate=H.umalquraToGregorian(a,s,n)),e.insertDate(e.isIslamic?e.currentDateIslamic:e.currentDate)},setWeekRange:function(t,e){var i=this.settings;i.range.first.date=new Date(t.year,t.month,t.day),i.range.second=void 0,this.setValue(new Date(e.year,e.month,e.day)),this.calendarAPI.days.find(".is-selected").removeClass("is-selected range").removeAttr("aria-selected")},close:function(){this.closeCalendar()},closeCalendar:function(){var t=this.calendarAPI&&this.calendarAPI.days.length?this.calendarAPI.days.find("td.is-selected"):null;this.resetRange({cell:t}),this.settings.showTime&&this.timepickerControl&&this.timepickerControl.isOpen()&&this.timepickerControl.closeTimePopup(),this.popup&&this.popup.length&&this.popup.hide().remove();var e=this.trigger.data("popover");e&&e.destroy(),this.calendarAPI&&delete this.calendarAPI.datepickerApi,this.element.hasClass("is-active")&&(this.element.trigger("listclosed"),this.element.removeClass("is-active is-open"))},setFocusAfterOpen:function(){var t=this.settings;if(this.calendar){if(t.range.useRange&&(t.range.first&&t.range.first.label&&(!t.range.second||t.range.second&&!t.range.second.date)&&this.setRangeFirstPart(t.range.first.date),this.calendarAPI.setRangeSelected(),t.range.second&&t.range.first.date&&t.range.second.date&&this.element.val(this.getRangeValue()),this.popup.removeClass("is-hidden")),this.calendarAPI.datepickerApi=this,t.range.useRange&&t.range.selectWeek){var e=this.calendar.find("td.is-selected").first().parent();return this.calendar.find("td[tabindex]").removeAttr("tabindex"),void e.attr("tabindex","0").focus()}this.calendarAPI.activeTabindex(this.calendar.find("td.is-selected"),!0)}},setRangeFirstPart:function(t){var e=this.settings,i=function(t){return new Date(t.year,t.month,t.day)},n=this.calendarAPI.days.find("td:visible:first"),s=this.calendarAPI.days.find("td:visible:last"),a=this.isIslamic?M.padDate(t[0],t[1],t[2]):M.padDate(t.getFullYear(),t.getMonth(),t.getDate()),o=this.calendarAPI.days.find('[data-key="'.concat(a,'"]')),r=o.closest("tr");this.isIslamic?(this.currentDate=H.umalquraToGregorian(t[0],t[1],t[2],t[3],t[4],t[5],t[6]),this.currentDateIslamic=t):this.currentDate=t;var l=this.calendarAPI.getCellDate(n),c=this.calendarAPI.getCellDate(s);e.range.first={date:this.isIslamic?u(t):t,cell:o,row:r,rowIdx:r.index(),cellIdx:o.index()},e.range.extra={minCell:n,maxCell:s,min:this.isIslamic?[l.year,l.month,l.day]:i(l),max:this.isIslamic?[c.year,c.month,c.day]:i(c),cellLength:r.children("td").length},this.calendarAPI.settings.range.first=e.range.first,this.calendarAPI.settings.range.extra=e.range.extra},insertDate:function(t,e){var i=this.settings,n="",s="",a="";if(t instanceof Array)n=t[0],s=t[1],a=t[2].toString();else{if(!(t instanceof Date)||isNaN(this.getTime(t)))return;n=t.getFullYear(),s=t.getMonth(),a=t.getDate().toString()}if(this.calendarAPI.currentMonth===s&&this.calendarAPI.currentYear===n||this.calendarAPI.showMonth(s,n),this.isOpen()){var o=this.calendarAPI.days.find("td:not(.alternate)").filter((function(){return $(this).text().toLowerCase()===a}));o.hasClass("is-disabled")||(this.settings.showTime&&(e?(this.time=H.formatDate(t,{pattern:this.currentCalendar.dateFormat.hour,locale:this.locale.name,language:this.language}),this.settings.roundToInterval&&($("#timepicker-minutes").val(""),t=this.setTime(t))):t=this.setTime(t)),this.setValue(t,!0),i.range.useRange?this.calendarAPI.days.find(".is-selected").removeAttr("aria-selected").removeAttr("tabindex"):this.calendarAPI.days.find(".is-selected").removeClass("is-selected range").removeAttr("aria-selected").removeAttr("tabindex"),o.addClass("is-selected".concat(i.range.useRange?" range":"")).attr({"aria-selected":!0})),this.calendarAPI.activeTabindex(o,!0)}},setValue:function(t,e,i){var n=this.settings;if(this.currentDate=t,""===this.currentDate||void 0===this.currentDate)return this.element.val(""),void this.clearRangeDates();t instanceof Array&&(this.currentDateIslamic=t,this.currentDate=H.umalquraToGregorian(t[0],t[1],t[2],t[3],t[4],t[5])),n.range.useRange?i||this.setRangeToElem(t,!1):this.element.val(H.formatDate(t,{pattern:this.pattern,locale:this.locale.name})),e&&(n.range.useRange?i||this.element.trigger("change",[n.range.data]).trigger("input",[n.range.data]):this.element.trigger("change").trigger("input"))},getTime:function(t){return Array.isArray(t)?t.join(""):t.getTime()},setRangeToElem:function(t,e){var i=this,n=this.settings,s=function(t){return H.formatDate(t,{pattern:i.pattern,locale:i.locale.name})},a=s(t),o=!1;if(this.isOpen()){var r=this.isIslamic?M.padDate(t[0],t[1],t[2]):M.padDate(t.getFullYear(),t.getMonth(),t.getDate()),l=this.calendarAPI.days.find('[data-key="'.concat(r,'"]')),c=l.closest("tr");n.range.second&&(n.range.second.date?this.resetRange({cell:l}):(delete n.range.second.date,$.isEmptyObject(n.range.second)&&delete n.range.second));var d={};n.range.first&&(d.date=this.getTime(t),d.firstdate=this.getTime(n.range.first.date),d.min=this.calendarAPI.getDifferenceToDate(n.range.first.date,n.range.minDays),d.max=this.calendarAPI.getDifferenceToDate(n.range.first.date,n.range.maxDays)),!n.range.first||e?(this.setRangeFirstPart(t),a=this.getRangeValue(),this.setPlaceholder()):!n.range.second&&n.range.selectBackward&&d.date>d.firstdate||n.range.selectForward&&d.date<d.firstdate||n.range.maxDays>0&&d.date>d.max.aftertime||d.date<d.max.beforetime?(this.resetRange({cell:l}),this.setRangeFirstPart(t),a=this.getRangeValue(),this.setPlaceholder()):(o=!0,this.isIslamic?(this.currentDate=H.umalquraToGregorian(t[0],t[1],t[2],t[3],t[4],t[5],t[6]),this.currentDateIslamic=t):this.currentDate=t,n.range.minDays>0&&(d.date>=d.firstdate&&d.date<d.min.aftertime?(t=d.min.after,d.date===d.firstdate&&(d.date=this.getTime(t))):d.date<d.firstdate&&d.date>d.min.beforetime&&(t=d.min.before),r=this.isIslamic?M.padDate(t[0],t[1],t[2]):M.padDate(t.getFullYear(),t.getMonth(),t.getDate()),c=(l=this.calendarAPI.days.find('[data-key="'.concat(r,'"]'))).closest("tr")),d.date>d.firstdate?n.range.second={date:t,cell:l,row:c,rowIdx:c.index(),cellIdx:l.index()}:(n.range.second=n.range.first,n.range.first={date:t,cell:l,row:c,rowIdx:c.index(),cellIdx:l.index()}),a=this.getRangeValue())}else if(!e){o=!0;var h=t||new Date;this.currentMonth=h.getMonth(),this.currentYear=h.getFullYear(),this.currentDay=h.getDate(),this.currentDate=h,n.range.first=n.range.first||{},n.range.second=n.range.second||{},n.range.first.date=h,n.range.second.date=h,n.range.end&&t&&("string"==typeof n.range.end?n.range.second.date=H.parseDate(n.range.end,{pattern:this.pattern,locale:this.locale.name},!1):n.range.second.date=n.range.end),a=this.getRangeValue()}this.element.val(a),o?(n.range.data={value:a,dates:this.calendarAPI?this.calendarAPI.getDateRange(n.range.first.date,n.range.second.date):[n.range.first.date],startDate:n.range.first.date,start:s(n.range.first.date),endDate:n.range.second.date,end:s(n.range.second.date)},this.closeCalendar(),this.isFocusAfterClose&&(delete this.isFocusAfterClose,this.element.focus())):n.range.data={value:a,dates:[n.range.first.date],startDate:n.range.first.date,start:s(n.range.first.date)}},resetRange:function(t){t=t||{},this.settings.range.useRange&&(delete this.settings.range.first,delete this.settings.range.second,delete this.settings.range.extra,t.isData&&delete this.settings.range.data,this.calendarAPI&&(delete this.calendarAPI.settings.range.first,delete this.calendarAPI.settings.range.second,delete this.calendarAPI.settings.range.extra,t.isData&&delete this.calendarAPI.settings.range.data),this.calendarAPI&&this.calendarAPI.days.length&&this.calendarAPI.days.find("td").removeClass("range range-next range-prev range-selection end-date is-selected"),t.cell&&t.cell.addClass("is-selected"))},setRangeValueFromField:function(){var t,e=this,i=function(t){return H.formatDate(t,{pattern:e.pattern,locale:e.locale.name})},n=function(t){return H.parseDate(t,{pattern:e.pattern,locale:e.locale.name},!1)},s=function(t){return t&&"function"==typeof t.getTime?t:new Date},a=function(t){var i=n(t[0]),a=n(t[1]),o=e.isIslamic&&Array.isArray(i);return o||(i=H.gregorianToUmalqura(new Date(t[0])),a=H.gregorianToUmalqura(new Date(t[1]))),i&&a?(i=e.getTime(s(i)),a=e.getTime(s(a)),e.isIslamic&&!o&&(t[0]=H.gregorianToUmalqura(new Date(t[0])),t[1]=H.gregorianToUmalqura(new Date(t[1]))),i>a?[t[1],t[0]]:[t[0],t[1]]):t},o=this.settings,r={};(r.value=o.range.value||this.element.val().trim(),r.isEmpty=""===r.value,!r.isEmpty&&r.value.indexOf(o.range.separator)>-1?r.dates=a(r.value.split(o.range.separator)):!r.isEmpty&&r.value.indexOf(o.range.separator.slice(0,-1))>-1?r.dates=r.value.split(o.range.separator.slice(0,-1)):r.isEmpty||-1!==r.value.indexOf(o.range.separator)||(r.dates=[i(r.value)]),!o.range.data&&o.range.start&&o.range.end&&r.isEmpty)&&(t="string"==typeof o.range.start&&"string"==typeof o.range.end?a([o.range.start,o.range.end]):"string"!=typeof o.range.start&&"string"==typeof o.range.end?a([i(o.range.start),o.range.end]):"string"==typeof o.range.start&&"string"!=typeof o.range.end?a([o.range.start,i(o.range.end)]):a([i(o.range.start),i(o.range.end)]),o.range.start=i(t[0]),o.range.end=i(t[1]));if(o.range.first=o.range.first||{},o.range.second=o.range.second||{},o.range.data&&o.range.data.startDate?o.range.first.date=o.range.data.startDate:o.range.start&&"string"==typeof o.range.start?o.range.first.date=n(o.range.start):r.dates&&(o.range.first.date=this.isIslamic&&Array.isArray(r.dates[0])?r.dates[0]:n(r.dates[0])),o.range.data&&o.range.data.endDate?o.range.second.date=o.range.data.endDate:o.range.end&&"string"==typeof o.range.end?o.range.second.date=n(o.range.end):r.dates&&(o.range.second.date=this.isIslamic&&Array.isArray(r.dates[1])?r.dates[1]:n(r.dates[1])),this.calendarAPI&&this.calendarAPI.setRangeSelected(),!r.isEmpty&&(r.isEmpty||o.range.data))return!1;var l=i(o.range.first.date);return l&&this.element.val(l),!0},setValueFromField:function(){var t=this,e=this.settings;if(this.setCurrentCalendar(),e.range.useRange&&""===this.element.val().trim()&&this.resetRange({isData:!0}),e.range.useRange&&(""!==this.element.val().trim()||e.range.start&&e.range.end||e.range.data&&e.range.data.startDate&&e.range.data.endDate)&&!this.setRangeValueFromField())return this.currentDate&&"function"==typeof this.currentDate.getMonth&&(this.currentMonth=this.currentDate.getMonth(),this.currentYear=this.currentDate.getFullYear(),this.currentDay=this.currentDate.getDate()),void(this.currentDate&&this.isIslamic&&(this.currentYear=this.currentDateIslamic[0],this.currentMonth=this.currentDateIslamic[1],this.currentDay=this.currentDateIslamic[2]));var i=this,n=this.element.val(),s=n;if(this.isIslamic&&n){var a=H.parseDate(this.element.val(),{pattern:this.pattern,locale:this.locale.name});a instanceof Date?s=H.umalquraToGregorian(a.getFullYear(),a.getMonth(),a.getDate()):a instanceof Array&&(s=H.umalquraToGregorian(a[0],a[1],a[2],a[3],a[4],a[5]))}var o=function(){var e=(new Date).getDate();if(t.calendarAPI){var i=t.calendarAPI.dayMap.filter((function(t){return t.elem.is(".is-selected")}));i.length&&(e=parseInt(i[0].key.substr(6),10))}return e}();this.currentDate=s||new Date,"string"==typeof this.currentDate&&(this.currentDate=H.parseDate(this.currentDate,{pattern:this.pattern,locale:this.locale.name,calendarName:this.settings.calendarName},!1),this.pattern&&-1===this.pattern.indexOf("d")&&this.currentDate.setDate(o)),void 0===this.currentDate&&(this.currentDate=H.parseDate(s,{pattern:this.pattern,locale:this.locale.name,calendarName:this.settings.calendarName},!1)),this.isIslamic?(this.currentDateIslamic=H.gregorianToUmalqura(this.currentDate),this.currentYear=this.currentDateIslamic[0],this.currentMonth=this.currentDateIslamic[1],this.currentDay=this.currentDateIslamic[2]):(this.currentDate=this.currentDate||new Date,this.currentMonth=this.currentDate.getMonth(),this.currentYear=this.currentDate.getFullYear(),this.currentDay=this.currentDate.getDate());var r=i.pattern,l=!(r.indexOf("MMMM")>-1||r.indexOf("MMM")>-1||"yyyy"===r||"MMMM"===r||"MMM"===r||"MM"===r),c=i.element.val().trim();if(""!==c&&!e.range.useRange){var d=H.parseDate(c,{pattern:i.pattern,locale:this.locale.name,calendarName:this.settings.calendarName},l),h=0;h=this.isIslamic?d[3]:null==d||null==d?void 0:d.getHours(),!this.isIslamic&&d&&h<12&&i.element.val().trim().indexOf(this.currentCalendar.dayPeriods[1])>-1&&""!==this.currentCalendar.dayPeriods[1]&&d.setHours(h+12),this.isIslamic&&d&&h<12&&i.element.val().trim().indexOf(this.currentCalendar.dayPeriods[1])>-1&&(h+=12),!this.isIslamic&&i.pattern&&-1===i.pattern.indexOf("d")&&d.setDate(o),this.isIslamic&&i.pattern&&-1===i.pattern.indexOf("d")&&(d[2]=o),void 0===d||""===i.element.val().trim()||e.range.useRange||(i.setValue(d),c!==i.element.val().trim()&&this.element.trigger("change").trigger("input"))}e.range.useRange&&e.range.first&&e.range.first.date&&e.range.second&&(e.range.second.date?this.element.val(this.getRangeValue()):this.setRangeToElem(this.currentDate,!0))},enable:function(){this.element.removeAttr("disabled readonly").closest(".field").removeClass("is-disabled"),this.trigger.prop("disabled",!1)},disable:function(){this.element.removeAttr("readonly").attr("disabled","disabled").closest(".field").addClass("is-disabled"),this.trigger.prop("disabled",!0)},isDisabled:function(){return this.element.prop("disabled")},readonly:function(){this.enable(),this.element.attr("readonly","readonly"),this.trigger.prop("disabled",!1)},isReadonly:function(){return this.element.prop("readonly")},makeTabbable:function(t){this.trigger.attr("tabIndex",t?0:-1)},setToday:function(t){var e=this.settings;if(this.lastValue=null,this.currentDate=new Date,!this.isReadonly()&&!this.isDisabled()){if(this.settings.useCurrentTime||(this.currentDate=this.setTime(this.currentDate)),this.isIslamic){var i=H.gregorianToUmalqura(this.currentDate);this.currentDateIslamic=i}var n=this.isIslamic?this.currentDateIslamic:this.currentDate,s=!1;if(this.isOpen())if(e.range.useRange)if(!e.range.first||e.range.first&&!e.range.first.date){if(this.calendarAPI.days.find("td:visible").removeClass("is-selected").removeAttr("aria-selected"),this.insertDate(n,!0),t&&this.calendarAPI){var a=this.calendarAPI.dayMap.filter((function(t){return t.elem.is(".is-selected")}));a&&a.length&&setTimeout((function(){a[0].elem.focus()}),0)}}else e.range.first&&e.range.first.date&&(!e.range.second||e.range.second&&!e.range.second.date)?(this.setRangeToElem(n,!1),s=!0):e.range.first&&e.range.first.date&&e.range.second&&e.range.second.date&&(this.resetRange({isData:!0}),this.insertDate(n,!0));else this.insertDate(n,!0);else{if(e.range.useRange)this.setRangeToElem(this.currentDate);else{var o={pattern:this.pattern,locale:this.locale.name},r=H.formatDate(n,o);this.element.val(r)}s=!0}s&&(e.range.useRange?this.element.trigger("change",[e.range.data]).trigger("input",[e.range.data]):this.element.trigger("change").trigger("input"))}},adjustDay:function(t,e){if(!this.isReadonly()&&!this.isDisabled()){this.currentDate||(e.length>0&&/(0?[1-9]|1[012])\/(0?[1-9]|[12][0-9]|3[01])\/\d{4}/.test(e)?this.setValue(new Date(e),!0,!0):this.setToday());var i={pattern:this.pattern,locale:this.locale.name},n=this.isIslamic?this.currentDateIslamic:this.currentDate;this.isIslamic?(n[2]+=t?1:-1,(n=H.umalquraToGregorian(this.currentDateIslamic[0],this.currentDateIslamic[1],this.currentDateIslamic[2],this.currentDateIslamic[3],this.currentDateIslamic[4],this.currentDateIslamic[5],this.currentDateIslamic[6])).setDate(n.getDate()+(t?1:-1)),n=H.gregorianToUmalqura(n)):n.setDate(n.getDate()+(t?1:-1)),this.element.val(H.formatDate(n,i)),this.element.trigger("change").trigger("input")}},setTime:function(t){var e=void 0!==this.popup;if(!this.timepicker||!e)return this.settings.useCurrentTime||t.setHours(0,0,0,0),t;var i=this.popup.find(".dropdown.hours").val(),n=this.popup.find(".dropdown.minutes").val(),s=this.isSeconds?this.popup.find(".dropdown.seconds").val():0,a=this.popup.find(".dropdown.period"),o=a.val();return i=a.length&&o===this.currentCalendar.dayPeriods[1]&&i<12?parseInt(i,10)+12:i,i=!a.length||o!==this.currentCalendar.dayPeriods[0]&&o||12!==parseInt(i,10)?i:0,t instanceof Array?(t[3]=i?parseInt(i,10):t[3],t[4]=n?parseInt(n,10):t[4],t[5]=s?parseInt(s,10):t[5]):(t=new Date(t)).setHours(i,n,s),t},getCurrentDate:function(){return this.currentDate},updated:function(t){void 0!==t&&(this.settings=E.mergeSettings(this.element,t,as)),this.teardown(),this.init(),this.element.data("validate")&&this.element.data("validate")instanceof Object&&this.element.data("validate").updated()},getRangeValue:function(){var t=this,e=this.settings,i=function(e){return H.formatDate(e,{pattern:t.pattern,locale:t.locale.name})};return e.range.useRange&&e.range.first&&e.range.first.date&&e.range.second&&e.range.second.date?"".concat(i(e.range.first.date)+e.range.separator+i(e.range.second.date)):e.range.useRange&&e.range.first&&e.range.first.date?e.placeholder?"".concat(i(e.range.first.date)+e.range.separator+this.pattern):i(e.range.first.date):""},teardown:function(){this.isOpen()&&this.closeCalendar(),this.element.off("blur.datepicker change.datepicker-rangeclear keyup.datepicker-rangeclear"),this.trigger.remove(),this.element.removeAttr("placeholder"),this.calendarAPI&&this.calendarAPI.destroy(),this.calendar&&this.calendar.length&&this.calendar.remove(),this.popup&&this.popup.length&&this.popup.remove();var t=this.element.data("mask");return t&&t.destroy(),this.element.removeAttr("data-mask"),this.element.removeData("mask"),this.element.off("keydown.datepicker blur.validate change.validate keyup.validate focus.validate"),this.addedValidation&&(this.element.removeAttr("data-validate").removeData("validate validationEvents"),delete this.addedValidation),this},destroy:function(){this.closeCalendar(),this.teardown(),this.element[0]&&$.removeData(this.element[0],ns)},handleEvents:function(){var t=this,e=this,i=this.settings;if(this.trigger.on("click.datepicker",(function(){e.isOpen()?e.closeCalendar():e.openCalendar()})),this.handleKeys(this.element),e.element.on("blur.datepicker",(function(){""!==t.element.val().trim()&&t.setValueFromField()})),this.element.on("change.datepicker-rangeclear keyup.datepicker-rangeclear",(function(){t.isOpen()||""!==t.element.val().trim()||e.clearRangeDates()})),!this.isOpen()&&i.range.useRange&&!i.range.first){this.setRangeValueFromField();var n=this.getRangeValue();n&&this.element.val(n)}}},$.fn.datepicker=function(t){return this.each((function(){var e=$.data(this,ns);e?e.updated(t):e=$.data(this,ns,new os(this,t))}))};var rs=["p","h1","h2","h3","h4","h5","h6","blockquote","pre","code","div"];function ls(t,e){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"p",n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"",s=arguments.length>4?arguments[4]:void 0;if(!t||"string"!=typeof t||!e.length)throw new Error('"id" property must be defined and unique.');if(this.id=t,!e||"string"!=typeof e||!e.length)throw new Error('"displayName" property must be defined.');return this.displayName=e,i=i.toLowerCase(),-1===rs.indexOf(i)&&(i="p"),this.tagName=i,"string"==typeof n&&n.length&&(this.className=n),s instanceof CSSStyleDeclaration&&(this.styleProps=s),this}ls.prototype={render:function(t){return t=g.stripHTML(t),"<".concat(this.tagName,">").concat(t,"</").concat(this.tagName,">")}};var cs="fontpicker";function ds(){return{popupmenuSettings:{offset:{y:10}},styles:[new ls("default",H.translate("FontPickerNormal")),new ls("header1",H.translate("FontPickerHeader").replace("{0}","1"),"h3"),new ls("header2",H.translate("FontPickerHeader").replace("{0}","2"),"h4")]}}function hs(t,e){if(!(t instanceof HTMLElement))throw new Error('Property "element" is not an HTMLElement type');this.settings=E.mergeSettings(t,e,ds()),e&&Array.isArray(e.styles)&&(this.settings.styles=e.styles),this.element=t,this.init()}hs.prototype={get menuAPI(){var t=$(this.element).data("popupmenu");if(t)return t},get selected(){var t;return this.settings.styles.forEach((function(e){e.selected&&(t=e)})),t||(this.settings.styles[0].selected=!0,this.settings.styles[0])},getStyleById:function(t){var e;if(this.settings.styles.forEach((function(i){i.id===t&&(e=i)})),!e)throw new Error('No FontPickerStyle available with id "'.concat(t,'"'));return e},getStyleByTagName:function(t){var e;if(this.settings.styles.forEach((function(i){i.tagName===t&&(e=i)})),!e)throw new Error('No FontPickerStyle available with tagName "'.concat(t,'"'));return e},get disabled(){return this.trueDisabled},set disabled(t){this.trueDisabled=t,this.element.disabled=!0===t},get supportedTagNames(){var t=[];return this.settings.styles.forEach((function(e){t.push(e.tagName)})),t},init:function(){return Array.isArray(this.settings.styles)&&this.settings.styles.length||(this.settings.styles=ds().styles),this.build().handleEvents()},build:function(){var t=$(this.element),e=this.element.querySelector("svg.icon.icon-dropdown");e||(e=$.createIcon({icon:"dropdown",classes:["icon-dropdown"]}),this.element.insertAdjacentHTML("beforeend",e)),this.ddIcon=this.element.querySelector("svg.icon.icon-dropdown"),t.button();var i=t.next(".popupmenu");i&&i.length||(i=$('<ul class="popupmenu fontpicker-menu"></ul>').insertAfter(this.element)),i.html(this.buildMenuHTML());var n=E.extend({},this.settings.popupmenuSettings,{menu:i,stretchToWidestMenuItem:!0});return t.popupmenu(n),this.render(),this.audible=$('<span class="audible">'.concat(H.translate("FontPickerStyle"),"</span>")),this.element.prepend(this.audible[0]),this},render:function(){var t=this.selected,e=this.element.querySelector("span:not(.audible)");$(e).html(t.displayName)},buildMenuHTML:function(){var t="";return this.settings.styles.forEach((function(e){var i=e.render(e.displayName);t+='<li class="fontpicker-style">\n        <a href="#" data-val="'.concat(e.id,'">').concat(i,"</a>\n      </li>")})),"".concat(t)},handleEvents:function(){var t=this,e=this;return $(this.element).on("selected.".concat(cs),(function(e,i){var n=i.attr("data-val");t.select(n)})).on("updated.".concat(cs),(function(t,i){e.updated(i)})),this},select:function(t,e){if(!t||"string"!=typeof t&&!(t instanceof ls))throw new Error('"id" property must be defined in order to select.');var i;i=t instanceof ls?t:this.getStyleById(t),this.settings.styles.forEach((function(t){t.selected=!1})),i.selected=!0,this.render(i),e||$(this.element).triggerHandler("font-selected",[i])},updated:function(t){var e;void 0!==t&&(Array.isArray(t.styles)&&(e=t.styles),this.settings=E.mergeSettings(this.element,t,this.settings),e&&(this.settings.styles=e));return this.teardown().init()},teardown:function(){var t=this.element.querySelector("svg.icon.icon-dropdown");t.parentNode.removeChild(t);var e=this.menuAPI;e&&"function"==typeof e.destroy&&e.destroy();var i=$(this.element).data("button");return i&&"function"==typeof i.destroy&&i.destroy(),$(this.element).off(["selected.".concat(cs),"updated.".concat(cs)].join(" ")),this},destroy:function(){this.teardown(),$.removeData(this.element,cs)}},$.fn.fontpicker=function(t){return this.each((function(){var e=$.data(this,cs);e?e.updated(t):e=$.data(this,cs,new hs(this,t))}))};var us="toolbar",ps={rightAligned:!1,maxVisibleButtons:3,resizeContainers:!0,favorButtonset:!0,moreMenuSettings:void 0,noSearchfieldReinvoke:!1,attributes:null};function gs(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,ps),this.init()}gs.prototype={init:function(){return this.setup().build().handleEvents()},setup:function(){return this.settings.maxVisibleButtons<=0&&(this.settings.maxVisibleButtons=ps.maxVisibleButtons),this},build:function(){var t,e=this;if(this.element.attr("role","toolbar"),this.settings.resizeContainers&&this.element.is(":not(:hidden)")&&this.element[0].classList.add("do-resize"),Array.isArray(this.settings.attributes)&&E.addAttributes(this.element,this,this.settings.attributes),this.buildAriaLabel(),this.id||(this.id=parseInt($(".toolbar, .formatter-toolbar").index(this.element),10)),this.isModalToolbar=this.element.parent().is(".modal-body"),this.title=this.element.children(".title"),this.title.length&&!this.isModalToolbar?(this.element[0].classList.add("has-title"),this.cutoffTitle=!1,this.title.on("beforeshow.toolbar",(function(){return e.cutoffTitle})).tooltip({content:"".concat(this.title.text().trim())})):this.element[0].classList.remove("has-title"),this.buttonset=this.element.children(".buttonset"),this.buttonset.length||(this.buttonset=$('<div class="buttonset"></div>'),this.title.length?this.buttonset.insertAfter(this.title):this.buttonset.prependTo(this.element)),this.element[this.settings.rightAligned?"addClass":"removeClass"]("right-aligned"),this.more=this.element.find(".btn-actions"),0===this.more.length&&!this.element.hasClass("no-actions-button")){var i=this.element.find(".more");i.length||(i=$('<div class="more"></div>').appendTo(this.element)),this.more=$('<button class="btn-actions" type="button"></button>').html("".concat($.createIcon({icon:"more"}),'<span class="audible">').concat(H.translate("MoreActions"),"</span>")).attr("title",H.translate("More")).appendTo(i)}(this.buttonsetItems=this.buttonset.children("button").add(this.buttonset.find("input")),this.items=this.buttonsetItems.add(this.title.children("button")).add(this.more),this.items.filter('button, input[type="button"], [class^="btn"]').each((function(t){var i=$(this);i.data("button")||i.button(),!$(this).data("tooltip")&&i.attr("title")&&i.tooltip(),!i.is(".btn-menu")&&Array.isArray(e.settings.attributes)&&E.addAttributes(i,e,e.settings.attributes,"button-".concat(t))})),this.settings.noSearchfieldReinvoke)||this.items.filter(".searchfield, .toolbar-searchfield-wrapper, .searchfield-wrapper").each((function(t,i){var n=$(i);if(n.is(".toolbar-searchfield-wrapper, .searchfield-wrapper")&&(n=n.children(".searchfield")),!n.data("searchfield")){var s=$.extend({},E.parseSettings(n[0]));n.searchfield(s)}E.addAttributes(n,e,e.settings.attributes,"searchfield")}));var n=this.more.data("popupmenu"),s=this.more.attr("aria-controls");n?this.moreMenu=n.menu:(this.moreMenu=$("#".concat(s)),this.moreMenu.length&&void 0!==s||(this.moreMenu=this.more.next(".popupmenu, .popupmenu-wrapper")),this.moreMenu.length||(this.moreMenu=$('<ul id="popupmenu-toolbar-'.concat(this.id,'" class="popupmenu"></ul>')).insertAfter(this.more)),this.moreMenu.is(".popupmenu-wrapper")&&(this.moreMenu=this.moreMenu.children(".popupmenu")));var a,o=[];this.items.not(this.more).not(".ignore-in-menu").filter((function(){return $(this).parent(".buttonset, .inline").length})).each((function(){o.push(e.buildMoreActionsMenuItem($(this)))})),o.reverse(),$.each(o,(function(t,i){""!==i.text()&&i.prependTo(e.moreMenu)})),this.defaultMenuItems=this.moreMenu.children("li:not(.separator)"),this.hasDefaultMenuItems=this.defaultMenuItems.length>0,this.settings.moreMenuSettings&&Array.isArray(this.settings.moreMenuSettings.attributes)&&(a=this.settings.moreMenuSettings.attributes),a&&a.length||!Array.isArray(this.settings.attributes)||(a=this.settings.attributes.map((function(t){var e="".concat(t.value,"-actionbutton");return{name:t.name,value:e}}))),null!==(t=a)&&void 0!==t&&(null==t?void 0:t.length)||(a=null);var r=E.extend({},this.settings.moreMenuSettings,{trigger:"click",menu:this.moreMenu,attributes:a},this.hasDefaultMenuItems?{predefined:this.defaultMenuItems}:{});n?this.more.on("beforeopen.toolbar",(function(){e.refreshMoreActionsMenu(e.moreMenu)})).triggerHandler("updated",[r]):this.more.popupmenu(r).on("beforeopen.toolbar",(function(){e.refreshMoreActionsMenu(e.moreMenu)})),function(){e.items.attr("tabindex","0");var t=e.buttonset.children("button");t&&t.attr("tabindex","0");var i=e.items.filter(".is-selected");if(i.length)return e.activeButton=i.first().attr("tabindex","0"),void e.items.not(e.activeButton).removeClass("is-selected");i=e.items.filter(":visible:not(:disabled)").first().attr("tabindex","0"),e.activeButton=i,e.element.is(":hidden, .is-hidden")?e.activeButton=e.items.filter(":not(:disabled)").first().attr("tabindex","0"):e.isItemOverflowed(i)&&(i.attr("tabindex","-1"),e.activeButton=e.more.addClass("is-selected").attr("tabindex","0"))}(),this.adjustMenuItemVisibility(),this.handleResize(),this.element.triggerHandler("rendered");var l=this.buttonset.find(".searchfield-wrapper, .toolbar-searchfield-wrapper");l.length&&l.trigger("reanimate");var c=this.buttonset.find(".toolbar-searchfield-wrapper .searchfield");return c.length&&c[0].setAttribute("tabindex","0"),this},buildMoreActionsMenuItem:function(t){var e;if(!0===t.data("skipit"))return t.data("skipit",void 0),e;var i=t.data("action-button-link");i&&i.length?e=i.parent():(e=$("<li></li>"),i=$('<a href="#"></a>').appendTo(e),t.data("action-button-link",i),i.data("original-button",t)),t.hasClass("hidden")&&e.addClass("hidden"),t.is(":disabled")?(e.addClass("is-disabled"),i.prop("disabled",!0)):(e.removeClass("is-disabled"),i.prop("disabled",!1)),i.text(this.getItemText(t));var n=$.getBaseURL("#icon-dropdown"),s=t.children(".icon").filter((function(){var t=$(this).getIconName();return t&&t!==n&&-1===t.indexOf("dropdown")}));s&&s.length&&(i.html("<span>".concat(i.text(),"</span>")),s.clone().detach().prependTo(i));var a=e.find("b");a.length&&(this.moreMenu.addClass("has-icons"),a.detach().prependTo(e));var o=this.items.not(this.more).not(".searchfield").index(t);if(t.is(".btn-menu")){var r,l;this.settings.attributes&&this.settings.attributes.length&&(l=this.settings.attributes.map((function(t){return{name:t.name,value:"".concat(t.value,"-menubutton-").concat(o)}}))),null!==(r=l)&&void 0!==r&&(null==r?void 0:r.length)||(l=null),t.data("popupmenu")?i.children(".icon.arrow").length||i.append($.createIcon({classes:"icon arrow icon-dropdown",icon:"dropdown"})):t.popupmenu({attributes:l});var c=t.data("popupmenu").menu,d=c.clone();!function t(i,n,s){var a,o=i.children("li"),r=n.attr("id");n.children("li").each((function(e,i){var n=$(i),s=o.eq(e),a=n.children("a"),r=s.children("a"),l=n.attr("id"),c=a.attr("id");l&&n.removeAttr("id").attr("data-original-menu-item",l),c&&a.removeAttr("id").attr("data-original-menu-anchor",c),r.data("action-button-link",a),a.data("original-button",r);var d=s.children(".wrapper").children(".popupmenu"),h=n.children(".wrapper").children(".popupmenu");d.length&&h.length&&t(d,h,n)})),n.removeAttr("id").attr("data-original-menu",r),s.addClass("submenu"),s.is(e)&&((a=s.children(".wrapper"))&&a.length||(a=$('<div class="wrapper"></div>')),a.html(n),s.append(a))}(c,d,e)}return t.is("[data-popdown]")&&t.popdown(),e},refreshMoreActionsMenu:function(t){var e=this;$("li > a",t).each((function(){var t,i=$(this),n=i.parent(),s=i.data("originalButton"),a=e.getItemText(s);s&&(i.find("span").length?i.find("span").text(a.trim()):i.text(a.trim()),s.isHiddenAtBreakpoint()||s.parent().isHiddenAtBreakpoint()?n.addClass("hidden"):n.removeClass("hidden"),s.parent().is(".is-disabled")||s.is(":disabled")?(n.addClass("is-disabled"),i.prop("disabled",!0),i.attr("tabindex","-1")):(n.removeClass("is-disabled"),i.prop("disabled",!1)),s.is("a")&&(s.parent("li").is(".is-checked")?n.addClass("is-checked"):n.removeClass("is-checked")),s.is(".btn-menu")&&(t=i.parent().find(".popupmenu").first(),e.refreshMoreActionsMenu(t)))}))},getItemText:function(t){if(!t)return"";var e,i=t.find("span").first(),n=t.attr("title"),s=t.data("tooltip"),a=s&&"string"==typeof s.content?s.content:void 0;return e=""!==n&&void 0!==n?n:a||(i.length?i.text():t.text()),g.stripHTML(e)},handleEvents:function(){var t=this;return this.items.on("keydown.toolbar",(function(e){t.handleKeys(e)})).on("click.toolbar",(function(e){t.handleClick(e)})).on("collapsed.toolbar.searchfield",(function(){setTimeout((function(){t.handleResize()}),150)})),this.items.filter(".btn-menu, .btn-actions").on("close.toolbar",(function(){var e,i=$(this);i.is(".is-overflowed")?(e=t.getLastVisibleButton())&&e.length&&e[0].focus():document.activeElement&&"INPUT"===document.activeElement.tagName||(i&&i.focus(),t.buttonset.scrollTop(0))})),this.items.not(this.more).on("selected.toolbar",(function(e,i){e.stopPropagation(),t.handleSelected(e,i)})),this.more.on("keydown.toolbar",(function(e){t.handleKeys(e)})).on("beforeopen.toolbar",(function(){t.adjustMenuItemVisibility()})).on("selected.toolbar",(function(e,i){e.stopPropagation(),t.handleSelected(e,i)})),this.more.on("show-submenu.toolbar",(function(e,i){t.handleTransferToMenuButtonItem(e,i)})),this.element.on("updated.toolbar",(function(e,i){e.stopPropagation(),t.updated(i)})).on("recalculate-buttons.toolbar",(function(e,i){t.handleResize(i)})).on("scrollup.toolbar",(function(){var e=t.element.parents(".tab-container.module-tabs");e.length&&e.scrollTop(0)})),$("body").on("resize.toolbar-".concat(this.id),(function(){t.handleResize()})),this.handleResize(),this},handleTransferToMenuButtonItem:function(t,e){var i=e.children("a").data("original-button");if(i){var n=i.data("popupmenu");n&&"function"==typeof n.settings.beforeOpen&&(n.callSource(t),this.buildMoreActionsMenuItem(i))}},handleSelected:function(t,e){var i,n,s,a,o=e.data("original-button");if(e.parent().is(".hidden, .is-disabled")||e.is("[readonly], [disabled]"))t.preventDefault();else{if(o&&o.length>0){if(i=o.listEvents(),n=this.element.listEvents(),this.setActiveButton(o),o.is("[data-popdown]")&&(a=o.data("popdown"),this.isItemOverflowed(o)&&(a.settings.trigger=this.more,a.updated())),o.attr("ng-click")||o.attr("data-ng-click"))return void o.trigger("click");for(var r=["click","touchend","touchcancel"],l=0;l<r.length;l++){var c=r[l];if(n&&n[c]&&n[c].delegateCount>0){var d=this.element,h=$.Event(c);return h.target=d.find(o)[0],void d.trigger(h)}if(i&&i[c]||o[0]["on".concat(c)])return void o.trigger(c)}return a=(s=o.parents(".popupmenu")).data("trigger"),s.length&&a instanceof $&&a.length?void a.triggerHandler("selected",[o]):void this.triggerSelect(o)}this.triggerSelect(e)}},handleClick:function(t){return this.setActiveButton($(t.currentTarget)),this.triggerSelect($(t.currentTarget)),!1},handleKeys:function(t){var e=this,i=t.which,n=$(t.target),s=n.is(".btn-actions"),a=H.isRTL();(37===i&&n.is(":not(input)")||38===i&&n.is(":not(input.is-open)"))&&(t.preventDefault(),s?e.setActiveButton(a?e.getFirstVisibleButton():e.getLastVisibleButton()):e.navigate(a?1:-1)),(39===i&&n.is(":not(input)")||40===i&&n.is(":not(input.is-open)"))&&(t.preventDefault(),s?e.setActiveButton(a?e.getLastVisibleButton():e.getFirstVisibleButton()):e.navigate(a?-1:1))},handleResize:function(t){for(var e=this.getButtonsetButtons(),i=0;i<e.length;i++)e[i].removeClass("is-overflowed");if(this.settings.resizeContainers){var n=t?t.title:void 0,s=t?t.buttonset:void 0;this.sizeContainers(n,s)}this.element.is(":not(:hidden)")&&(this.adjustMenuItemVisibility(),this.toggleMoreMenu())},sizeContainers:function(t,e){var i=this.element[0],n=this.title[0],s=this.buttonset[0],a=this.more[0];if(this.title&&this.title.length){if(this.element.is(":hidden"))return s.style.width="",n.style.width="",void i.classList.remove("do-resize");s.style.width="",n.style.width="",i.classList.contains("do-resize")||i.classList.add("do-resize");var o=$(i).getHiddenSize(),r=$(s).getHiddenSize(),l=$(n).getHiddenSize(),c=$(a).getHiddenSize(),d=parseInt(o.padding.left,10)+parseInt(o.padding.right,10);isNaN(c.width)&&(c.width=50),(isNaN(r.width)||r.width<0)&&(r.width=0);var h,u=this,p=void 0!==t&&!isNaN(t),g=void 0!==e&&!isNaN(e);this.cutoffTitle=!1;var f=function(){if(p)return parseInt(t,10);if(!0===u.settings.favorButtonset){var e=b();return e=e>0?e:$(s).width(),$(window).width()-(d+e+c.width-10)}return l.scrollWidth}(),m=b();if(this.settings.favorButtonset)return f<74&&(this.cutoffTitle=!0,h=Math.abs(f-74),f=74,m-=h),s.style.width=v(m+2),n.style.width=v(f-2),this.element.is(":not(:hidden)")&&(this.adjustMenuItemVisibility(),this.toggleMoreMenu(s.offsetWidth<s.scrollWidth)),n.textContent&&f<M.textWidth(n.textContent.trim())&&(this.cutoffTitle=!0),void(f>M.textWidth($(n).find("h1").text().trim())&&(this.cutoffTitle=!1));m<0&&(this.cutoffTitle=!0,h=Math.abs(m-0),m=0,f-=h),n.style.width=v(f+2),s.style.width=v(m-2)}function v(t){return"".concat(t,"px")}function b(){var i;i=g?parseInt(e,10):!0===u.settings.favorButtonset?r.width:o.width,!g&&o.width-i<l.width&&(i-=d+(p?parseInt(t,10):l.scrollWidth)+c.width);var n=u.buttonsetItems.filter(".searchfield").data("searchfield");if(n){!0===n.settings.showGoButton&&(i=i+n.goButton.width()>i?i+n.goButton.width():i);var s=n.wrapper.width()-i;s>0&&(i+=s)}return i}},navigate:function(t){var e,i=this.items.filter(":visible:not(:disabled)"),n=i.index(this.activeButton)+t;n>=0&&n<i.length&&(e=i.eq(n)),n>=i.length&&(e=i.first()),-1===n&&(e=i.last()),this.isItemOverflowed(e)&&(e=this.more),this.setActiveButton(e)},getLastVisibleButton:function(){for(var t,e,i=$(this.items.get().reverse()).not(this.more),n=0;!t&&n<i.length;){if(e=$(i[n]),!this.isItemOverflowed(e)){t=e;break}n++}for(t&&0!==t.length||(t=i.first());t.length&&t.is(".separator, *:disabled, *:hidden");)t=t.prev();return t},getFirstVisibleButton:function(){for(var t=0,e=this.items,i=e.eq(t);i.is(".separator, *:disabled, *:hidden");)t++,i=e.eq(t);return i},setActiveButton:function(t,e){if(t[0]!==this.activeButton[0]){var i=this;this.items.add(this.more).attr("tabindex","-1").removeClass("is-selected"),this.activeButton=function(){if(t.is("a"))return n();var e=i.more.data("tooltip");return t[0]===i.more[0]?(e&&e.tooltip.is(":not(.hidden)")&&e.hide(),n()):i.isItemOverflowed(t)&&!t.is(".searchfield")?n():t}(),this.activeButton.addClass("is-selected").attr("tabindex","0"),!e&&this.activeButton[0]&&(this.activeButton[0].focus(),this.element.triggerHandler("navigate",[this.activeButton]))}function n(){return i.moreButtonIsDisabled()||!i.element.hasClass("has-more-button")?i.getLastVisibleButton():i.more}},triggerSelect:function(t){var e=$(t);e.is(this.more)||e.is(".btn-menu, li.submenu")||this.element.triggerHandler("selected",[e])},getButtonsetButtons:function(){for(var t,e=[],i=this.buttonsetItems,n=0;n<i.length;n++)void 0!==(t=i.eq(n)).data("action-button-link")&&t.is(":not(.searchfield)")&&e.push(t);return e},getVisibleButtons:function(t){var e,i=this,n=[],s=[];t&&Array.isArray(t)||(t=this.getButtonsetButtons());for(var a=0;a<t.length;a++)t[a][0].classList.remove("is-overflowed");for(var o=0;o<t.length;o++)e=t[o],i.isItemOverflowed(e)?n.push(e):s.push(e);return{visible:s,hidden:n}},adjustMenuItemVisibility:function(t){var e="removeClass";function i(t,i){var n=t[0],s=t.data("action-button-link").parent()[0],a=t.isHiddenAtBreakpoint();if(i)return s.classList.add("hidden"),void n.classList.remove("is-overflowed");a||s.classList.remove("hidden"),n.classList.add("is-overflowed"),t.find(".icon").length&&(e="addClass")}t||(t=this.getVisibleButtons());for(var n=0;n<t.visible.length;n++)i(t.visible[n],!0);for(n=0;n<t.hidden.length;n++)i(t.hidden[n],!1);var s=0;this.moreMenu.find(".icon").each((function(){$(this).parent().parent().hasClass("hidden")||$(this).hasClass("icon-dropdown")||s++})),s>0&&(e="addClass"),this.moreMenu[e]("has-icons")},isItemOverflowed:function(t){if(this.moreButtonIsDisabled())return!1;if(!t||0===t.length)return!0;var e=this.buttonsetItems.filter(":not(.hidden)").index(t),i=this.settings.maxVisibleButtons;if(this.moreButtonIsVisible()&&(i-=1),this.title.length&&e>=i)return!0;this.buttonset.scrollTop()>0&&this.buttonset.scrollTop(0),t instanceof $&&t.length&&(t=t[0]);var n=t.classList,s=window.getComputedStyle(t);if(n.contains("btn-actions"))return!0;if(n.contains("searchfield")||"INPUT"===t.nodeName)return!1;if("none"===s.display)return!0;var a=H.isRTL(),o=t.getBoundingClientRect(),r=this.buttonset[0].getBoundingClientRect(),l=a?o.left<=r.left:o.right>=r.right;return!0===o.bottom>=r.bottom||!0===l},moreButtonIsDisabled:function(){return this.element[0].classList.contains("no-actions-button")},hasNoMoreButton:function(){return j(this.moreButtonIsDisabled,this.hasNoMoreButton).apply(this)},moreButtonIsVisible:function(){return this.element[0].classList.contains("has-more-button")},toggleMoreMenu:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];if(!this.moreButtonIsDisabled()){var e="removeClass";(this.moreMenu.children("li:not(.separator)").not(".hidden").length>0||t)&&(e="addClass"),this.element[e]("has-more-button");var i=this.more.data("popupmenu");if("removeClass"===e){if(!i)return;i.close();var n=i.menu.find("li:not(.separator)").children("a"),s=!1;if(n.add(this.more).each((function(){document.activeElement===this&&(s=!0)})),s){var a=this.getLastVisibleButton()[0];a&&a.focus()}}}},buildAriaLabel:function(){if(!this.element.attr("aria-label")){var t=1===this.element.closest(".header").length,e=this.element.attr("id")||"",i=this.element.children(".title"),n=this.element.prev("label"),s=this.element.prev(".label"),a=t?$("header.header").find("h1").text():i.length?i.filter("div").text():n.length?n.text():s.length?s.text():"".concat(e," ").concat(H.translate("Toolbar"));this.element.attr("aria-label",a.replace(/\s+/g," ").trim())}},updated:function(t){this.settings.noSearchfieldReinvoke&&(this.keepSearchfield=!0),t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown().init()},enable:function(){this.element.prop("disabled",!1),this.items.prop("disabled",!1),this.more.prop("disabled",!1)},disable:function(){this.element.prop("disabled",!0),this.items.prop("disabled",!0),this.more.prop("disabled",!0).data("popupmenu").close()},teardown:function(){var t=this;$("body").off("resize.toolbar-".concat(this.id));var e=void 0===this.moreMenu?[]:this.moreMenu.children("li");if(e.each((function(){t.teardownMoreActionsMenuItem($(this),!0)})),e.not(this.defaultMenuItems).remove(),delete this.defaultMenuItems,delete this.hasDefaultMenuItems,this.items.each((function(t,e){var i=$(e).data("tooltip");i&&"function"==typeof i.destroy&&i.destroy();var n=$(e).data("button");n&&"function"==typeof n.destroy&&n.destroy(),e.classList.remove("is-overflowed"),e.removeAttribute("tabindex")})),this.items.off(["keydown.".concat(us),"click.".concat(us),"focus.".concat(us),"blur.".concat(us),"close.".concat(us),"selected.".concat(us)].join(" ")),delete this.items,this.title&&this.title.length){var i=this.title.off("beforeshow.toolbar").data("tooltip");i&&"function"==typeof i.destroy&&i.destroy(),this.title[0].style.width="",delete this.cutoffTitle,delete this.title}if(this.buttonsetItems&&delete this.buttonsetItems,this.buttonset.children(".searchfield-wrapper").length)if(this.settings.noSearchfieldReinvoke||this.keepSearchfield)this.keepSearchfield&&delete this.keepSearchfield;else{var n=this.buttonset.children(".searchfield-wrapper").children(".searchfield"),s=n.data("searchfield");s&&"function"==typeof s.destroy&&n.data("searchfield").destroy()}return this.buttonset&&this.buttonset.length&&(this.buttonset[0].style.width="",delete this.buttonset),this.moreMenu&&delete this.moreMenu,this.more.length&&void 0!==this.more.data("popupmenu")&&(this.more.off(["keydown.".concat(us),"beforeopen.".concat(us),"selected.".concat(us),"show-submenu.".concat(us)].join(" ")),this.more.data("popupmenu").destroy(),delete this.more),this.activeButton&&delete this.activeButton,this.element.off(["updated.".concat(us),"recalculate-buttons.".concat(us),"scrollup.".concat(us)].join(" ")),this.element[0].classList.remove("do-resize"),this.element.removeAttr("role").removeAttr("aria-label"),this},teardownMoreActionsMenuItem:function(t,e){var i=this,n=$(t),s=n.children("a"),a=s.data("original-button");s.off("mousedown.toolbar click.toolbar touchend.toolbar touchcancel.toolbar");var o,r=n.find(".icon");r.length&&r.remove(),n.is(".submenu")&&(o=n.children(".wrapper").children(".popupmenu")).children("li").each((function(){i.teardownMoreActionsMenuItem($(this),!0)})),a&&a.length&&($.removeData(s[0],"original-button"),$.removeData(a[0],"action-button-link"),s.remove(),o&&o.off().parent(".wrapper").off().remove(),e&&n.remove())},destroy:function(){this.teardown(),$.removeData(this.element[0],us)}},$.fn.toolbar=function(t){return this.each((function(){var e=$.data(this,us);e?e.updated(t):e=$.data(this,us,new gs(this,t))}))};var fs="editor",ms=["p","h1","h2","h3","h4","h5","h6","blockquote","pre","code"],vs={buttons:{editor:["fontPicker","separator","bold","italic","underline","strikethrough","separator","foreColor","backColor","separator","justifyLeft","justifyCenter","justifyRight","separator","quote","orderedlist","unorderedlist","separator","anchor","separator","image","separator","clearFormatting","separator","source"],source:["fontPicker","separator","bold","italic","underline","strikethrough","separator","foreColor","backColor","separator","justifyLeft","justifyCenter","justifyRight","separator","quote","orderedlist","unorderedlist","separator","anchor","separator","image","separator","clearFormatting","separator","visual"]},excludeButtons:{editor:["backColor"],source:["backColor"]},rows:{editor:null,source:null},delay:200,placeholder:null,pasteAsPlainText:!1,anchor:{url:"http://www.example.com",class:"hyperlink",target:"NewWindow",isClickable:!1,showIsClickable:!1},image:{url:"/images/placeholder-80x80.png"},onLinkClick:null,showHtmlView:!1,preview:!1,paragraphSeparator:"p",useFlexToolbar:!0,useSourceFormatter:!1,formatterTabsize:4,fontpickerSettings:{popupmenuSettings:{showArrow:!1,offset:{y:0}}},attributes:null};function bs(t,e){this.settings=E.mergeSettings(t,e,vs),null!=e&&(null==e?void 0:e.buttons)&&(this.settings.buttons=e.buttons),this.element=$(t),this.init()}bs.prototype={get toolbarAPI(){var t;return this.toolbar&&this.toolbar.length&&(t=this.toolbar.data(this.settings.useFlexToolbar?"toolbar-flex":"toolbar")),t},init:function(){var t=this;if(this.id="".concat(E.uniqueId(this.element,"editor"),"-id"),this.container=this.element.parent(".field, .field-short").addClass("editor-container"),this.label=this.element.prevAll(".label").first(),!this.previewRendered&&(this.element.hasClass("is-preview")||this.settings.preview))return this.container[0].classList.add("is-preview"),this.element[0].classList.remove("is-disabled","is-readonly","is-preview"),this.element[0].setAttribute("contenteditable",!1),this.element[0].removeAttribute("aria-multiline"),void this.element[0].removeAttribute("role");var e=this.settings;if(e.anchor=$.extend({},vs.anchor,e.anchor),e.image=$.extend({},vs.image,e.image),e.anchor.defaultUrl=e.anchor.url,e.anchor.defaultClass=e.anchor.class,e.anchor.defaultTargetText=e.anchor.target,e.anchor.defaultIsClickable=e.anchor.isClickable,e.anchor.targets=e.anchor.targets||{SameWindow:"",NewWindow:"_blank"},$.each(this.settings.anchor.targets,(function(e,i){t.settings.anchor.defaultTargetText.toLowerCase()===e.toLowerCase()&&(t.settings.anchor.target=i,t.settings.anchor.defaultTarget=i)})),(this.settings.firstHeader||this.settings.secondHeader)&&(Array.isArray(this.settings.fontpickerSettings.styles)||(this.settings.fontpickerSettings.styles=[]),this.settings.fontpickerSettings.styles.length||this.settings.fontpickerSettings.styles.push(new ls("legacyDefault","Default")),this.settings.firstHeader&&(V("`fontpickerSettings.styles` setting","`firstHeader` setting","Editor Component"),this.settings.fontpickerSettings.styles.push(new ls("legacyHeader1","Header 1",this.settings.firstHeader)),delete this.settings.firstHeader),this.settings.secondHeader&&(V("`fontpickerSettings.styles` setting","`secondHeader` setting","Editor Component"),this.settings.fontpickerSettings.styles.push(new ls("legacyHeader2","Header 2",this.settings.secondHeader)),delete this.settings.secondHeader)),e.buttons&&e.buttons.editor){for(var i=!1,n=[new ls("default","Default","p")],s=e.buttons.editor.filter((function(t){return"header"===t.substr(0,6)})),a=0;a<s.length;a++){var o=s[a].substr(6,1);i=!0,n.push(new ls("header".concat(o),"Header ".concat(o),"h".concat(o)))}"separator"===e.buttons.editor[0]&&e.buttons.editor.splice(0,1),i&&(e.buttons.editor=e.buttons.editor.filter((function(t){return"header"!==t.substr(0,6)})),e.fontpickerSettings={styles:n}),i&&(e.buttons.editor=["fontPicker"].concat(e.buttons.editor))}return e.anchor.defaultTarget||(e.anchor.target&&$.trim(e.anchor.target).length?e.anchor.defaultTarget=e.anchor.target:(e.anchor.defaultTargetText=H.translate("SameWindow"),e.anchor.defaultTarget=e.anchor.targets[e.anchor.defaultTargetText])),this.setup(),"string"==typeof e.paragraphSeparator&&/^(p|br|div)$/.test(e.paragraphSeparator)&&document.execCommand("defaultParagraphSeparator",!1,e.paragraphSeparator),this.element.hasClass("is-readonly")&&this.readonly(),this.settings.showHtmlView&&this.toggleSource(),this.setRowsHeight(),this},setup:function(){this.isActive=!0,this.modals={},this.initElements().bindSelect().bindPaste().setPlaceholders().bindWindowActions().setupKeyboardEvents().onPasteTriggered()},initElements:function(){return this.element.attr({contentEditable:!0,"aria-multiline":!0,role:"textbox","aria-labelledby":this.container.find(".label").length?this.container.find(".label")[0].id:"no-id"}),this.element.attr("data-editor",!0),this.initTextarea(),this.bindParagraphCreation().bindTab(),this.createToolbar().bindButtons().bindModals().bindAnchorPreview(),this},sourceViewActive:function(){return this.element.hasClass("source-view-active")},setPlaceholders:function(){return this.settings.placeholder?(this.element.attr("placeholder",this.settings.placeholder),this):this},getCurrentElement:function(){return this.sourceViewActive()?this.textarea:this.element},bindSelection:function(t){var e=this;clearTimeout(t),t=setTimeout((function(){e.checkSelection()}),this.settings.delay)},bindSelect:function(){var t=this;return this.getCurrentElement().on("mouseup.editor",(function(){t.bindSelection("")})),this},bindParagraphCreation:function(){var t=this;return this.getCurrentElement().on("keyup.editor",(function(e){var i=t.getSelectionStart(),n=i.tagName.toLowerCase();if(i&&i.getAttribute("data-editor")&&0===i.children.length&&document.execCommand("formatBlock",!1,"p"),13===e.which){if(i=t.getSelectionStart(),"blockquote"===n)return;"li"===n||t.isListItemChild(i)||(e.shiftKey||document.execCommand("formatBlock",!1,"p"),"a"===n&&document.execCommand("unlink",!1,null))}t.bindSelection("")})),this},bindTab:function(){var t=this;return this.getCurrentElement().on("keydown.editor",(function(e){if(9===e.which){var i=t.getSelectionStart();if(!i||!i.tagName)return;var n=i.tagName.toLowerCase();"pre"===n&&(e.preventDefault(),document.execCommand("insertHtml",null,"    ")),"li"===n&&(e.preventDefault(),document.execCommand(e.shiftKey?"outdent":"indent",e))}if(8===e.which){var s=t.element.text().toString().trim().replace(/\s/g,"");window.getSelection().toString().trim().replace(/\s/g,"")===s&&t.element.html("")}})),this},setExcludedButtons:function(){var t,e,i,n=this,s=function(t,e){var i=-1,n=0;return t.filter((function(t,s){var a=!0;return e.indexOf(t)>-1&&"separator"!==t?(n++,a=!1):"separator"===t&&s-n-1===i&&(n=0,a=!1),"separator"===t&&(i=s),a}))};return t=n.settings,e=t.buttons,i=t.excludeButtons,n.sourceViewActive()?i&&i.source&&i.source.length?s(e.source,i.source):e.source:i&&i.editor&&i.editor.length?s(e.editor,i.editor):e.editor},createToolbar:function(){var t=this;if(this.toolbar)return this;var e=[this.settings.useFlexToolbar?"flex-toolbar":"toolbar","editor-toolbar","formatter-toolbar"].join(" "),i="",n="";this.settings.useFlexToolbar&&(i="toolbar-section ",n='<div class="toolbar-section more">\n        <button class="btn-actions btn-editor">\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-more"></use>\n          </svg>\n        </button>\n      </div>');for(var s=this.setExcludedButtons(),a="",o=0,r=s.length;o<r;o+=1){var c=this.buttonTemplate(s[o]);c&&this.element.hasClass("source-view-active")&&"visual"!==s[o]&&(c=c.replace('type="button"','type="button" disabled')),c&&(a+=c)}var d="";this.settings.attributes&&(d=this.getAutomationAttributes("-editor-toolbar"),d=JSON.stringify(d),d=' data-options=\'{"attributes": '.concat(d,"}'"));var h='<div class="'.concat(e,'" data-init="false" id="editor-toolbar-').concat(this.id,'"').concat(d,'>\n      <div class="').concat(i,'buttonset">\n        ').concat(a,"\n      </div>\n      ").concat(n,"\n    </div>");this.element.parent().find(".icon-dirty").length?this.toolbar=$(h).insertBefore(this.element.parent().find(".icon-dirty")):this.toolbar=$(h).insertBefore(this.sourceViewActive()?this.element.prev():this.element);var u=this.toolbar.find('[data-action="fontStyle"]').first();if(u&&u.length){if(this.settings.attributes){var p=this.getAutomationAttributes("-editor-fontpicker");"object"===l(this.settings.fontpickerSettings.popupmenuSettings)?this.settings.fontpickerSettings.popupmenuSettings.attributes=p:this.settings.fontpickerSettings.popupmenuSettings||(this.settings.fontpickerSettings.popupmenuSettings={attributes:p})}u.fontpicker(this.settings.fontpickerSettings),this.fontPickerElem=u,this.fontPickerElem.tooltip({placementOpts:{x:40}})}var g=this.toolbar.find('[data-action="foreColor"], [data-action="backColor"]');return g.colorpicker({placeIn:"editor",popupmenuSettings:{offset:{y:0},showArrow:!1}}),$(".trigger",g).off("click.colorpicker"),this.toolbar[this.settings.useFlexToolbar?"toolbarflex":"toolbar"]({moreMenuSettings:{offset:{y:0},showArrow:!1}}),this.toolbar.find("button[title]").tooltip(),g.on("beforeopen.editor",(function(e,i){var n=t.toolbar.data("toolbar")||t.toolbar.data("toolbar-flex");n&&n.overflowedItems.forEach((function(t){"colorpicker"===t.type&&i&&i.parent(".popupmenu-wrapper").off("afterplace.editor").one("afterplace.editor",(function(t,e){if(void 0===e.width){var i,n,s=(((null===(i=e.container)||void 0===i||null==i?void 0:i.outerWidth())||-1)-((null===(n=e.element)||void 0===n||null==n?void 0:n.outerWidth())||-1))/2;s>-1&&e.element.css("left","".concat(s,"px"))}}))}))})),this},getAutomationAttributes:function(t){var e=this,i=this.settings,n=i.attributes;if(i.attributes&&"string"==typeof t&&""!==t)if(Array.isArray(i.attributes))n=[],i.attributes.forEach((function(i){var s="function"==typeof i.value?i.value(e):i.value;n.push({name:i.name,value:s+t})}));else{var s="function"==typeof i.attributes.value?i.attributes.value(this):i.attributes.value;n={name:i.attributes.name,value:s+t}}return n},switchToolbars:function(){this.destroyToolbar(),this.setupTextareaEvents(),this.createToolbar(),this.bindButtons().bindModals().bindAnchorPreview(),this.bindSelect().bindPaste().setupKeyboardEvents(),this.toolbar.find("button").button()},initTextarea:function(){var t=this;return this.textarea&&!this.settings.showHtmlView?this:(this.textarea=this.createTextarea(),this.textarea.text(g.sanitizeHTML(this.element.html().toString())),this.container.on("input.editor keyup.editor",".editor",b((function(e){72!==e.keyCode&&!e.ctrlKey&&$(".modal.is-visible").length<1&&(t.textarea.text(g.sanitizeHTML(t.element.html().toString())),t.resetEmptyEditor(e),t.element.trigger("change"))}),400)),$("html").on("themechanged.".concat(fs),(function(){t.setRowsHeight(),t.sourceView.hasClass("hidden")||t.adjustSourceLineNumbers()})),this.setupTextareaEvents(),this.textarea)},setRowsHeight:function(){var t,e,i,n,s,a,o,r,l=ft.currentTheme.id&&(ft.currentTheme.id.indexOf("uplift")>-1||ft.currentTheme.id.indexOf("new")>-1);null!==(t=this.settings)&&void 0!==t&&null!==(e=null==t?void 0:t.rows)&&void 0!==e&&(null==e?void 0:e.editor)&&this.element.height(Number(null===(s=this.settings)||void 0===s||null===(a=null==s?void 0:s.rows)||void 0===a||null==a?void 0:a.editor)*(l?26:22.2));null!==(i=this.settings)&&void 0!==i&&null!==(n=null==i?void 0:i.rows)&&void 0!==n&&(null==n?void 0:n.source)&&this.element.parent().find(".editor-source").height(26*Number(null===(o=this.settings)||void 0===o||null===(r=null==o?void 0:o.rows)||void 0===r||null==r?void 0:r.source)+15)},resetEmptyEditor:function(){this.savedSelection=this.saveSelection();var t="p"===this.settings.paragraphSeparator?"p":"div",e=this.element.html().toString().trim();"<br>"!==(e=this.element.html().toString().trim())&&e!=="<".concat(t,"><br></").concat(t,">")||this.element.html(""),this.element.contents().filter((function(){return 3===this.nodeType&&""!==this.textContent.trim()})).wrap("<".concat(t,"></").concat(t,">")),e=this.element.html().toString().trim(),this.textarea.html(g.sanitizeHTML(e));var i=this.saveSelection(),n=function(t){return"number"==typeof t},s=function(t){return function(t){return t&&t[0]&&n(t[0].startOffset)&&n(t[0].endOffset)}(t)?{start:t[0].startOffset,end:t[0].endOffset}:null},a=s(this.savedSelection),o=s(i);null!==a&&null!==o&&a.start===o.start&&a.end===o.end||this.restoreSelection(this.savedSelection)},removeWhiteSpace:function(){var t=this.element.html().toString().trim();((t||"").match(/>\s+</g)||[]).length>0&&this.element.html(t.replace(/>\s+</g,"><"))},orderedListTypeStyling:function(){var t=this.getCurrentElement(),e=this.element,i="a",n="A",s="i",a="I";switch(t.find("ol").attr("type")){case i:e.addClass("type-l-alpha"),e.removeClass("type-u-alpha");break;case n:e.addClass("type-u-alpha"),e.removeClass("type-l-alpha");break;case s:e.addClass("type-l-roman"),e.removeClass("type-u-roman");break;case a:e.addClass("type-u-roman"),e.removeClass("type-l-roman")}},createTextarea:function(){this.sourceView=$("<div></div>").attr({class:"editor-source editable hidden",id:"editor-source-".concat(this.id)}).insertBefore(this.element),$("<ul></ul>").addClass("line-numbers").appendTo(this.sourceView);var t=$('<div class="text-container"></div>').appendTo(this.sourceView),e="source-textarea-".concat($('[id^="source-textarea-"]').length+1),i="".concat(this.element.prev(".label").addClass("audible").text()," - HTML Source View");return $('<label class="audible" for="'.concat(e,'">').concat(i,"</label>")).appendTo(t),$('<textarea id="'.concat(e,'" data-options="{ characterCounter: false }" class="editable"></textarea>')).appendTo(t)},triggerClick:function(t,e){$('button[data-action="'.concat(e,'"]'),this.toolbar).trigger("click.editor")},setupKeyboardEvents:function(){var t=this,e=this.getCurrentElement(),i={b:66,e:69,h:72,i:73,l:76,bl:55,n:56,q:81,r:82,u:85,h3:51,h4:52,space:32,sv:192};return e.on("keydown.editor",(function(n){switch((n=n||window.event).which?i.charCode=n.which:n.keyCode?i.charCode=n.keyCode:i.charCode=!1,n.ctrlKey&&i.charCode){case i.h3:t.triggerClick(n,"append-".concat(t.settings.firstHeader));break;case i.h4:t.triggerClick(n,"append-".concat(t.settings.secondHeader));break;case i.b:t.triggerClick(n,"bold"),n.preventDefault();break;case i.e:t.triggerClick(n,"justifyCenter");break;case i.h:t.triggerClick(n,"anchor"),n.preventDefault();break;case i.i:t.triggerClick(n,n.shiftKey?"image":"italic"),n.shiftKey||n.preventDefault();break;case i.bl:n.shiftKey&&t.triggerClick(n,"insertunorderedlist"),n.preventDefault();break;case i.l:n.shiftKey||t.triggerClick(n,"justifyLeft"),n.preventDefault();break;case i.n:n.shiftKey&&t.triggerClick(n,"insertorderedlist");break;case i.q:t.triggerClick(n,"append-blockquote");break;case i.r:t.triggerClick(n,"justifyRight");break;case i.u:t.triggerClick(n,"underline"),n.preventDefault();break;case i.space:t.triggerClick(n,"clearFormatting");break;case i.sv:t.triggerClick(n,e===t.element?"source":"visual")}})),this.element.on("mousedown.editor","a",(function(t){var e=$(this).attr("href");"firefox"!==x.browser.name&&"mac"===x.os.name&&(t.metaKey||t.ctrlKey)&&(window.open(e,"_blank"),t.preventDefault())})),this.element.on("updated.editor",(function(){t.updated()})),this},setupTextareaEvents:function(){var t=this;return this.textarea.off(".editor").on("input.editor keyup.editor",(function(){t.sourceView.hasClass("hidden")||t.adjustSourceLineNumbers()})).on("focus.editor",(function(){t.sourceView.addClass("is-focused")})).on("blur.editor",(function(e){t.sourceView.removeClass("is-focused"),t.element.empty().html(g.sanitizeHTML(t.textarea.val())),t.element.data("validate")&&t.element.data("validate").validate(t.element,!0,e)})),this},adjustSourceLineNumbers:function(){var t=this.textarea.parent(),e=parseInt(getComputedStyle(this.textarea[0]).lineHeight,10),i=this.textarea.innerHeight()-this.textarea.height();this.textarea[0].style.height="";var n=this.textarea[0].scrollHeight,s=Math.floor((n-i)/e),a=this.sourceView.find(".line-numbers"),o=a.find("li").length,r="",l=0;if(!this.lineNumbers||s!==this.lineNumbers){if(this.lineNumbers)if(this.lineNumbers<s){for(;l<s-this.lineNumbers;)r+='<li role="presentation"><span>'.concat(o+l+1,"</span></li>"),l++;a.append(r)}else this.lineNumbers>s&&(l=this.lineNumbers-s,a.find("li").slice(-l).remove());else{for(this.lineNumbers=s;l<this.lineNumbers;)r+='<li role="presentation"><span>'.concat(l+1,"</span></li>"),l++;a.append(r)}this.lineNumbers=s}t[0].style.width="calc(100% - ".concat(a.outerWidth()+2,"px)"),n===this.textarea[0].scrollHeight?this.textarea[0].style.height="".concat(a[0].scrollHeight-13,"px"):this.adjustSourceLineNumbers()},wrapTextInTags:function(t,e,i){var n,s;switch(i){case"bold":n=["<b>","</b>"];break;case"italic":n=["<i>","</i>"];break;case"underline":n=["<u>","</u>"];break;case"strikethrough":n=["<strike>","</strike>"];break;case"append-blockquote":n=["<blockquote>","</blockquote>"];break;default:n=["",""]}if("anchor"===i){var a=$('<a href="'.concat(t,'">').concat(e,"</a>"));this.settings.anchor.class&&$.trim(this.settings.anchor.class).length&&a.addClass(this.settings.anchor.class),this.settings.anchor.target&&$.trim(this.settings.anchor.target).length&&a.attr("target",this.settings.anchor.target),this.settings.anchor.isClickable?a.attr("contenteditable",!1):a.removeAttr("contenteditable"),s=a[0].outerHTML}else s=n[0]+t+e+n[1];return s},insertTextAreaContent:function(t,e){var i,n,s,a,o=this.textarea[0],r=o.value;t=t||"",document.selection&&"TEXTAREA"===o.tagName?($(o).focus(),(i=document.selection.createRange()).text=this.wrapTextInTags(t,i.text,e),$(o).focus()):o.selectionStart||"0"===o.selectionStart?(n=o.selectionStart,s=o.selectionEnd,a=o.scrollTop,i=this.wrapTextInTags(t,r.substring(n,s),e),o.value=r.substring(0,n)+i+r.substring(s,r.length),$(o).focus(),o.selectionStart=n+i.length,o.selectionEnd=n+i.length,o.scrollTop=a):(o.value+=this.wrapTextInTags(t,o.value,e),$(o).focus(),o.value=o.value)},buttonTemplate:function(t){var e=this.getButtonLabels(this.settings.buttonLabels);return{bold:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ToggleBold"),'" data-action="bold" data-element="b">').concat(e.bold,"</button>"),italic:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ToggleItalic"),'" data-action="italic" data-element="i">').concat(e.italic,"</button>"),underline:'<button type="button" class="btn btn-editor underline" title="'.concat(H.translate("ToggleUnderline"),'" data-action="underline" data-element="u">').concat(e.underline,"</button>"),strikethrough:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("StrikeThrough"),'" data-action="strikethrough" data-element="strike">').concat(e.strikethrough,"</button>"),foreColor:'<button type="button" class="btn btn-editor colorpicker-editor-button" title="'.concat(H.translate("TextColor"),'" data-action="foreColor" data-element="foreColor" data-init="false"><span class="swatch"></span>').concat(e.foreColor,"</button>"),backColor:'<button type="button" class="btn btn-editor colorpicker-editor-button" title="'.concat(H.translate("BackgroundColor"),'" data-action="backColor" data-element="backColor">').concat(e.backColor,"</button>"),superscript:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("Superscript"),'" data-action="superscript" data-element="sup">').concat(e.superscript,"</button>"),subscript:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("Subscript"),'" data-action="subscript" data-element="sub">').concat(e.subscript,"</button>"),separator:'<div class="separator"></div>',anchor:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("InsertLink"),'" data-action="anchor" data-modal="modal-url-').concat(this.id,'" data-element="a">').concat(e.anchor,"</button>"),image:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("InsertImage"),'" data-action="image" data-modal="modal-image-').concat(this.id,'" data-element="img">').concat(e.image,"</button>"),header1:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ToggleH3"),'" data-action="append-').concat(this.settings.firstHeader,'" data-element="').concat(this.settings.firstHeader,'">').concat(e.header1,"</button>"),header2:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ToggleH4"),'" data-action="append-').concat(this.settings.secondHeader,'" data-element="').concat(this.settings.secondHeader,'">').concat(e.header2,"</button>"),quote:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("Blockquote"),'" data-action="append-blockquote" data-element="blockquote">').concat(e.quote,"</button>"),orderedlist:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("OrderedList"),'" data-action="insertorderedlist" data-element="ol">').concat(e.orderedlist,"</button>"),unorderedlist:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("UnorderedList"),'" data-action="insertunorderedlist" data-element="ul">').concat(e.unorderedlist,"</button>"),fontPicker:'<button type="button" class="btn btn-editor fontpicker" title="'.concat(H.translate("ChooseFont"),'" data-action="fontStyle" data-init="false"><span>',"FontPicker","</span></button>"),justifyLeft:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("JustifyLeft"),'" data-action="justifyLeft" >').concat(e.justifyLeft,"</button>"),justifyCenter:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("JustifyCenter"),'" data-action="justifyCenter">').concat(e.justifyCenter,"</button>"),justifyRight:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("JustifyRight"),'" data-action="justifyRight" >').concat(e.justifyRight,"</button>"),clearFormatting:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ClearFormatting"),'" data-action="clearFormatting" >').concat(e.clearFormatting,"</button>"),source:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ViewSource"),'" data-action="source" >').concat(e.source,"</button>"),visual:'<button type="button" class="btn btn-editor" title="'.concat(H.translate("ViewVisual"),'" data-action="visual" >').concat(e.visual,"</button>")}[t]||!1},getIcon:function(t,e,i){return'<span class="audible">'.concat(H.translate(t),"</span>").concat($.createIcon({classes:i||"",icon:e}))},getButtonLabels:function(t){var e,i={bold:this.getIcon("Bold","bold"),italic:this.getIcon("Italic","italic"),underline:this.getIcon("Underline","underline"),superscript:'<span aria-hidden="true"><b>x<sup>1</sup></b></span>',subscript:'<span aria-hidden="true"><b>x<sub>1</sub></b></span>',strikethrough:this.getIcon("StrikeThrough","strike-through"),foreColor:this.getIcon("TextColor","fore-color"),backColor:this.getIcon("BackgroundColor","back-color"),anchor:this.getIcon("InsertLink","link"),image:this.getIcon("InsertImage","insert-image"),header1:this.getIcon("ToggleH3","h3"),header2:this.getIcon("ToggleH4","h4"),quote:this.getIcon("Blockquote","quote"),orderedlist:this.getIcon("OrderedList","number-list"),unorderedlist:this.getIcon("UnorderedList","bullet-list"),pre:'<span aria-hidden="true"><b>0101</b></span>',indent:'<span aria-hidden="true"><b>&rarr;</b></span>',outdent:'<span aria-hidden="true"><b>&larr;</b></span>',justifyLeft:this.getIcon("JustifyLeft","left-text-align"),justifyCenter:this.getIcon("JustifyCenter","center-text"),justifyRight:this.getIcon("JustifyRight","right-text-align"),clearFormatting:this.getIcon("ClearFormatting","clear-formatting"),source:this.getIcon("ViewSource","html","html-icon"),visual:this.getIcon("ViewVisual","visual","visual-icon")};if("object"===l(t)&&(e=t),"object"===l(e))for(var n in e)e.hasOwnProperty(n)&&(i[n]=e[n]);return i},activateButton:function(t){this.toolbar.find('[data-element="'.concat(t,'"]')).addClass("is-active")},bindButtons:function(){var t=this,e=this;function i(t,i){var n=i instanceof mn?$(i.element):$(t.target);if(!n.is(".btn-actions")){var s=n.attr("data-action"),a=e.getCurrentElement();return t.preventDefault(),a.focus(),e.checkSelection(),e.sourceViewActive()||n.toggleClass("is-active"),s&&e.execAction(s,t),("ie"===x.browser.name||x.browser.isEdge())&&a.trigger("change"),!n[0].classList.contains("longpress-target")&&void 0}}return this.settings.useFlexToolbar?(this.toolbar.on("selected.editor",i),this.toolbar.on("click.editor",".colorpicker-editor-button",i)):this.toolbar.on("click.editor","button",i),this.fontPickerElem&&this.fontPickerElem.on("font-selected",(function(e,i){t.execFormatBlock(i.tagName)})),this},bindModals:function(){var t=this;return this.modals={url:this.createURLModal(),image:this.createImageModal()},$('[name="em-target-'.concat(this.id,'"]')).dropdown(),$("#modal-url-".concat(this.id,", #modal-image-").concat(this.id)).modal({noRefocus:!1}).on("beforeopen",(function(){t.savedSelection=t.saveSelection(),$(this).attr("id")!=="modal-url-".concat(t.id)||t.selectionRange})).off("open").on("open",(function(){var e=x.features.touch,i=$(this).attr("id"),n=$("input:first",this),s=$(".modal-buttonset .btn-modal-primary",this);$('[name="em-url-'.concat(t.id,'"]')).val(t.settings.anchor.url),$('[name="em-class-'.concat(t.id,'"]')).val(t.settings.anchor.class),$('[name="em-target-'.concat(t.id,'"]')).val(t.settings.anchor.target).trigger("updated"),$('[name="em-isclickable-'.concat(this.id,'"]')).prop("checked",t.settings.anchor.isClickable),setTimeout((function(){e&&i==="modal-image-".concat(t.id)?s.focus():n.focus().select()}),10)})).off("beforeclose").on("beforeclose",(function(){var e=$(this).is(".editor-modal-image")?"image":"anchor";if(t.toolbar){var i=t.toolbar.find('[data-action="'.concat(e,'"]')),n=$(this).data("modal");i&&n&&(n.settings.noRefocus=t.isBtnOverflowedItem(i))}})).off("close").on("close",(function(e,i){if(t.restoreSelection(t.savedSelection),!i)if($(this).attr("id")==="modal-url-".concat(t.id)){var n=$(t.findElementInSelection("a",t.element[0]));n.length?t.updateCurrentLink(n):t.createLink($('[name="em-url-'.concat(t.id,'"]'),this))}else t.settings.attributes?t.settings.attributes.length>1?t.insertImage($('[data-automation-id="'.concat(t.settings.attributes[t.settings.attributes.length-1].value,'-editor-modal-input0"')).val()):t.insertImage($("input[name=image-".concat(t.id,"]")).val()):t.insertImage($('input[id*="image-editor"]').val())})),this},createURLModal:function(){var t=this,e=this.settings,i=$("#modal-url-".concat(this.id));if(i.length>0)return i;var n,s="",a=!0;$.each(e.anchor.targets,(function(e,i){s+='<option value="'.concat(i,'">').concat(H.translate(e),"</option>"),t.settings.anchor.defaultTargetText.toLowerCase()===e.toLowerCase()&&(a=!1)})),a&&(s+='<option value="'.concat(e.anchor.target,'">').concat(e.anchor.target,"</option>"));var o="";e.attributes&&(n=this.getAutomationAttributes("-editor-modal"),o=JSON.stringify(n),o=' data-options=\'{"attributes": '.concat(o,"}'"));var r=$('<div class="modal editor-modal-url" id="modal-url-'.concat(this.id,'"></div>')).html('<div class="modal-content">\n        <div class="modal-header">\n          <h1 class="modal-title">'.concat(H.translate("InsertLink"),'</h1>\n        </div>\n        <div class="modal-body">\n          <div class="field">\n            <label for="em-url-').concat(this.id,'" class="required">').concat(H.translate("Url"),'</label>\n            <input id="em-url-').concat(this.id,'" name="em-url-').concat(this.id,'" data-validate="required" type="text" value="').concat(e.anchor.url,'">\n          </div>\n          ').concat(e.anchor.showIsClickable?'<div class="field">\n            <input type="checkbox" class="checkbox" id="em-isclickable-'.concat(this.id,'" name="em-isclickable-').concat(this.id,'" checked="').concat(e.anchor.isClickable,'">\n            <label for="em-isclickable-').concat(this.id,'" class="checkbox-label"> ').concat(H.translate("Clickable"),"</label>\n          </div>"):"",'\n          <div class="field">\n            <label for="em-class-').concat(this.id,'">').concat(H.translate("CssClass"),'</label>\n            <input id="em-class-').concat(this.id,'" name="em-class-').concat(this.id,'" type="text" value="').concat(e.anchor.class,'">\n          </div>\n          <div class="field">\n            <label for="em-target-').concat(this.id,'" class="label"> ').concat(H.translate("Target"),'</label>\n            <select id="em-target-').concat(this.id,'" name="em-target-').concat(this.id,'" class="dropdown"').concat(o,">\n              ").concat(s,'\n            </select>\n          </div>\n          <div class="modal-buttonset">\n            <button type="button" class="btn-modal btn-cancel"> ').concat(H.translate("Cancel"),'</button>\n            <button type="button" class="btn-modal-primary"> ').concat(H.translate("Insert"),"</button>\n          </div>\n        </div>\n      </div>"));if(e.attributes){var l=[].slice.call(r[0].querySelectorAll('[type="text"]')),c=[].slice.call(r[0].querySelectorAll("button"));l.forEach((function(e,i){return E.addAttributes($(e),t,n,"input".concat(i),!0)})),c.forEach((function(e,i){return E.addAttributes($(e),t,n,"button".concat(i),!0)}))}return r.appendTo("body")},createImageModal:function(){var t=this,e=$("#modal-image-".concat(this.id));if(e.length>0)return e;var i=$('<div class="modal editor-modal-image" id="modal-image-'.concat(this.id,"\"></div>'")).html('<div class="modal-content">\n        <div class="modal-header">\n          <h1 class="modal-title">'.concat(H.translate("InsertImage"),'</h1>\n        </div>\n        <div class="modal-body no-scroll">\n          <div class="field">\n            <label for="image-').concat(this.id,'" class="required">').concat(H.translate("Url"),'</label>\n            <input id="image-').concat(this.id,'" name="image-').concat(this.id,'" type="text" data-validate="required" value="').concat(this.settings.image.url,'">\n          </div>\n          <div class="modal-buttonset">\n            <button type="button" class="btn-modal btn-cancel">\n              ').concat(H.translate("Cancel"),'</button>\n            <button type="button" class="btn-modal-primary">\n              ').concat(H.translate("Insert"),"</button>\n          </div>\n        </div>\n      </div>"));if(this.settings.attributes){var n=this.getAutomationAttributes("-editor-modal"),s=[].slice.call(i[0].querySelectorAll('[type="text"]')),a=[].slice.call(i[0].querySelectorAll("button"));s.forEach((function(e,i){return E.addAttributes($(e),t,n,"input".concat(i),!0)})),a.forEach((function(e,i){return E.addAttributes($(e),t,n,"button".concat(i),!0)}))}return i.appendTo("body")},bindAnchorPreview:function(){this.element.find("a").tooltip({content:function(){return $(this).attr("href")}})},updateCurrentLink:function(t){var e=g.stripTags(document.querySelector('[name="em-url-'.concat(this.id,'"]')).value),i=g.stripTags(document.querySelector('[name="em-class-'.concat(this.id,'"]')).value),n=g.stripTags(document.querySelector('[name="em-target-'.concat(this.id,'"]')).value),s=this.settings.anchor.showIsClickable?document.querySelector('[name="em-isclickable-'.concat(this.id,'"]')).checked:this.settings.anchor.isClickable;t&&(t[0].setAttribute("href",this.fixLinkFormat(e&&$.trim(e).length?e:this.settings.anchor.defaultUrl)),t[0].setAttribute("class",i&&$.trim(i).length?i:this.settings.anchor.defaultClass),t[0].setAttribute("data-url",(e&&$.trim(e).length?e:this.settings.anchor.defaultUrl).replace("http://",""))),s?t.attr("contenteditable",!1):t.removeAttr("contenteditable"),n&&$.trim(n).length?t.attr("target",n):t.removeAttr("target")},createLink:function(t){var e=this;this.restoreSelection(this.savedSelection);var i=g.stripTags(this.fixLinkFormat(t[0].value));t.val(i),this.settings.anchor.url=t.val(),this.settings.anchor.class=document.querySelector('[name="em-class-'.concat(this.id,'"]')).value,this.settings.anchor.target=document.querySelector('[name="em-target-'.concat(this.id,'"]')).value,this.settings.anchor.isClickable=this.settings.anchor.showIsClickable?document.querySelector('[name="em-isclickable-'.concat(this.id,'"]')).checked:this.settings.anchor.isClickable;var n=$('<a data-url="'.concat(i,'" href="').concat(i,'">').concat(i,"</a>"));if(this.settings.anchor.class&&$.trim(this.settings.anchor.class).length&&n.addClass(this.settings.anchor.class),this.settings.anchor.target&&$.trim(this.settings.anchor.target).length&&n.attr("target",this.settings.anchor.target),this.settings.anchor.isClickable?n.attr("contenteditable",!1):n.removeAttr("contenteditable"),this.sourceViewActive())this.insertTextAreaContent(t.val(),"anchor");else{var s,a,o,r,l;if(!this.selection.isCollapsed||x.browser.isIE11()){if(window.getSelection&&(s=window.getSelection()).getRangeAt&&s.rangeCount){a=s.getRangeAt(0),""!==(r="".concat(a)).trim()&&n.html(r);for(var c=(o=a.commonAncestorContainer.children)?o.length:0,d=0;d<c;d++){var h=o[d];h instanceof HTMLImageElement&&(l=h)}l&&n.html(l.outerHTML),a.deleteContents();var u=document.createElement("div");u.innerHTML=n[0].outerHTML;for(var p,f,m=document.createDocumentFragment();p=u.firstChild;)f=m.appendChild(p);a.insertNode(m),f&&((a=a.cloneRange()).setStartAfter(f),a.collapse(!0),s.removeAllRanges(),s.addRange(a))}}else document.execCommand("insertHtml",null,n[0].outerHTML),setTimeout((function(){e.getCurrentElement().focus()}),1);this.element.trigger("change"),this.bindAnchorPreview()}},fixLinkFormat:function(t){return t.match(/^https?:\/\//)?t:"http://".concat(t)},checkSelection:function(){var t;if(void 0===this.selection&&this.sourceViewActive())return t=this.textarea.val().substring(this.textarea[0].selectionStart,this.textarea[0].selectionEnd).toString().trim(),void this.hideToolbarActions();t=window.getSelection();var e=this.getSelectionElement();return e?this.checkSelectionElement(t,e):this.hideToolbarActions(),this},getSelectionElement:function(){var t,e,i,n=window.getSelection(),s=function(t){var e=t;try{for(;!e.getAttribute("data-editor");)e=e.parentNode}catch(t){return!1}return e};try{e=(t=n.getRangeAt(0).commonAncestorContainer).parentNode,i=t.getAttribute("data-editor")?t:s(e)}catch(t){i=s(e)}return i},checkSelectionElement:function(t,e){var i=this.sourceViewActive()?this.sourceView[0]:this.element[0];this.selection=t,this.selectionRange=this.selection.getRangeAt(0),i!==e?this.hideToolbarActions():this.setToolbarButtonStates()},setToolbarButtonStates:function(){return this.toolbar.find("button").removeClass("is-active"),this.checkActiveButtons(),this},checkActiveButtons:function(){this.checkButtonState("bold"),this.checkButtonState("italic"),this.checkButtonState("underline"),this.checkButtonState("strikethrough"),this.checkColorButtonState("foreColor"),this.toolbar.find('.buttonset [data-action="backColor"]').length&&this.checkColorButtonState("backColor"),this.fontPickerElem&&this.checkButtonState("fontStyle");for(var t=this.getSelectedParentElement();void 0!==t.tagName&&-1===ms.indexOf(t.tagName.toLowerCase)&&(this.activateButton(t.tagName.toLowerCase()),!this.element.is(t));)t=t.parentNode},checkButtonState:function(t){if(document.queryCommandState)if(this.fontPickerElem&&"fontStyle"===t){for(var e,i,n=this.fontPickerElem.data("fontpicker"),s=n.supportedTagNames,a=this.getSelectionParentElement(),o=$(a).add($(a).parentsUntil(this.element)),r=0;r<o.length&&void 0===i;r++)if(e=o[r].tagName.toLowerCase(),s.indexOf(e)>-1){i=n.getStyleByTagName(e),n.select(i,!0);break}}else document.queryCommandState(t)?this.toolbar.find('[data-action="'.concat(t,'"]')).addClass("is-active"):this.toolbar.find('[data-action="'.concat(t,'"]')).removeClass("is-active")},rangeSelectsSingleNode:function(t){var e=t.startContainer;return e===t.endContainer&&e.hasChildNodes()&&t.endOffset===t.startOffset+1},getSelectedParentElement:function(){var t=this.selectionRange;return this.rangeSelectsSingleNode(t)?t.startContainer.childNodes[t.startOffset]:3===t.startContainer.nodeType?t.startContainer.parentNode:t.startContainer},hideToolbarActions:function(){void 0!==this.toolbar&&this.toolbar.removeClass("is-active")},bindPaste:function(){var t=this,e=t.getCurrentElement();return this.pasteWrapper=function(e){var i,n,s;i=e.originalEvent.clipboardData&&e.originalEvent.clipboardData.getData?e.originalEvent.clipboardData.getData("text/plain"):!(!window.clipboardData||!window.clipboardData.getData)&&window.clipboardData.getData("Text");var a="";if(t.sourceViewActive())return this;if(i&&!e.defaultPrevented){for(e.preventDefault(),s=i.split(/[\r\n]/g),n=0;n<s.length;n+=1)""!==s[n]&&(navigator.userAgent.match(/firefox/i)&&0===n?a+="<p>".concat(t.htmlEntities(s[n]),"</p>"):/\.(gif|jpg|jpeg|tiff|png)$/i.test(s[n])?a+='<img src="'.concat(t.htmlEntities(s[n]),'" />'):a+="<p>".concat(t.htmlEntities(s[n]),"</p>"));if(document.queryCommandSupported("insertText"))return document.execCommand("insertHTML",!1,a),!1;t.pasteHtmlAtCaret(a)}},this.pasteWrapperHtml=function(e){if(t.sourceViewActive())return this;var i,n,s,a,o;if((e.clipboardData||e.originalEvent)&&(e.clipboardData&&e.clipboardData.types?n=e.clipboardData:e.originalEvent&&e.originalEvent.clipboardData&&e.originalEvent.clipboardData.getData&&(n=e.originalEvent.clipboardData)),n&&n.types)((i=n.types)instanceof DOMStringList&&i.contains("text/html")||i.indexOf&&-1!==i.indexOf("text/html")||x.browser.isEdge())&&(s=e.originalEvent.clipboardData.getData("text/html")),i instanceof DOMStringList&&i.contains("text/plain")&&(s=e.originalEvent.clipboardData.getData("text/plain")),"object"===l(i)&&i[0]&&"text/plain"===i[0]&&!i[1]&&(s=g.escapeHTML(e.originalEvent.clipboardData.getData("text/plain"))),i instanceof Array&&i.indexOf("text/plain")>-1&&i.indexOf("text/html")<0&&(s=g.escapeHTML(e.originalEvent.clipboardData.getData("text/plain")));else for(o=(window.clipboardData?window.clipboardData.getData("Text"):"").split(/[\r\n]/g),s="",a=0;a<o.length;a+=1)""!==o[a]&&(navigator.userAgent.match(/firefox/i)&&0===a?s+="<p>".concat(t.htmlEntities(o[a]),"</p>"):/\.(gif|jpg|jpeg|tiff|png)$/i.test(o[a])?s+='<img src="'.concat(t.htmlEntities(o[a]),'" />'):s+="<p>".concat(t.htmlEntities(o[a]),"</p>"));return t.pastedData=x.browser.isIE11()?s:t.getCleanedHtml(s),$.when(t.element.triggerHandler("beforepaste",[{pastedData:t.pastedData}])).done((function(){if(t.pastedData&&!e.defaultPrevented){if(x.browser.isIE11()||x.browser.isEdge()||e.preventDefault(),document.queryCommandSupported("insertText"))return document.execCommand("insertHTML",!1,t.pastedData),!1;t.pasteHtmlAtCaret(t.pastedData)}t.element.triggerHandler("afterpaste",[{pastedData:t.pastedData}]),t.pastedData=null})),!!x.browser.isIE11()&&void 0},e.on("paste.editor",t.settings.pasteAsPlainText?t.pasteWrapper:t.pasteWrapperHtml),this},pasteHtmlAtCaret:function(t){var e,i,n=this,s="x-text-content-templ-x";if(window.getSelection){if((e=window.getSelection()).getRangeAt&&e.rangeCount){(i=e.getRangeAt(0)).deleteContents(),x.browser.isIE11()&&(t=s);var a=document.createElement("div");a.innerHTML=t;for(var o,r,l=document.createDocumentFragment();o=a.firstChild;)r=l.appendChild(o);if(i.insertNode(l),r&&((i=i.cloneRange()).setStartAfter(r),i.collapse(!0),e.removeAllRanges(),e.addRange(i)),x.browser.isIE11()){for(var c=50,d=$.Deferred(),h=document.createDocumentFragment();n.element[0].childNodes.length>0;)h.appendChild(n.element[0].childNodes[0]);!function e(i,n){--c<0?d.reject():i.childNodes&&i.childNodes.length>0?(t=i.innerHTML,i.innerHTML="",i.appendChild(n),d.resolve()):setTimeout((function(){e(i,n)}),5)}(n.element[0],h),$.when(d).done((function(){var e="",i=n.element.find(":contains(+ ".concat(s,")")).filter((function(){return this.textContent===s}));i.length||(i=n.element.find(":contains(+ ".concat(s,")")).filter((function(){return this.textContent.indexOf(s)>-1&&"UL"!==this.tagName}))),t=n.getCleanedHtml(t);var a="";/(^(\s+?)?<li)/gi.test(t)?/((\s+?)?<\/li>(\s+?)?$)/gi.test(t)?(i.is("li")||(t="<ul>".concat(t,"</ul>")),a=t):a=i.is("li")?"".concat(t,"</li>"):(e=t.match(/<\/ul|<\/ol/gi)||[]).length?t:"".concat(t,"</li></ul>"):/((\s+?)?<\/li>(\s+?)?$)/gi.test(t)&&(i.is("li")?a="<li>".concat(t):((e=t.match(/<ul|<ol/gi)||[]).length?t+=e[e.length-1].replace(/<(ul|ol)/gi,"<$1>"):t="<ul>".concat(t,"</ul>"),a=t)),a&&f.html(i,a,"*"),(e=n.element[0].innerHTML).indexOf(s)>-1&&(e=e.replace(s,t)),n.element[0].innerHTML=n.getCleanedHtml(e)}))}}}else document.selection&&"Control"!==document.selection.type&&document.selection.createRange().pasteHTML(t)},getCleanedHtml:function(t){var e,i=t||"",n=["start","xmlns","xmlns:o","xmlns:w","xmlns:x","xmlns:m","onmouseover","onmouseout","onmouseenter","onmouseleave","onmousemove","onload","onfocus","onblur","onclick","style"];this.isWordFormat(i)&&(i=this.cleanWordHtml(i));for(var s=0,a=n.length;s<a;s++)e=new RegExp(" ".concat(n[s],'="(.*?)"'),"gi"),i=this.stripAttribute(i,n[s],e),e=new RegExp(" ".concat(n[s],"='(.*?)'"),"gi"),i=this.stripAttribute(i,n[s],e);return(i=(i=(i=(i=(i=(i=(i=(i=(i=(i=i.replace(/\sng-[a-z-]+/,"")).replace(/<!--(.*?)-->/gm,"")).replace(/\s\s+/g," ").replace(/\s>+/g,">")).replace(/<(ul)(.*?)>/gi,"<$1>")).replace(/<li><\/li>/gi,"")).replace(/<(ul|ol)><\/(ul|ol)>/gi,"")).replace(/<\/?(html|body)(.*?)>/gi,"")).replace(/<head\b[^>]*>(.*?)<\/head>/gi,"")).replace(/<(div|span|p)> <\/(div|span|p)>/gi," ")).replace(/<[^(br|/>)]+>[\s]*<\/[^>]+>/gi,"")).indexOf("·")>-1&&(i=[(i=[(i=(i=(i=i.replace(/<\/p>/gi,"</li>")).replace(/<p><span><span>·<\/span><\/span>/gi,"<li>")).replace(/<\/li>\s<li>/gi,"</li><li>")).slice(0,i.indexOf("<li>")),"<ul>",i.slice(i.indexOf("<li>"))].join("")).slice(0,i.lastIndexOf("</li>")),"</ul>",i.slice(i.lastIndexOf("</li>"))].join("")),i},htmlEntities:function(t){return String(t).replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;")},bindWindowActions:function(){var t=this,e=this.container,i=this.getCurrentElement(),n=this;return this.element.on("DOMNodeInserted",(function(t){var e=$(t.target),i=$("<b>helper</b>");"IMG"===t.target.tagName?e.removeAttr("id style srcset"):"SPAN"===t.target.tagName&&(e.before(i),i.after(e.contents()),i.add(e).remove())})),this.container.on("focusin.".concat(fs),".editor, .editor-source",(function(){var t=$(this);e.addClass("is-active"),setTimeout((function(){t.hasClass("error")&&(e.parent().find(".editor-toolbar").addClass("error"),e.parent().find(".editor-source").addClass("error"))}),100)})).on("focusout.".concat(fs),".editor, .editor-source",(function(){e.removeClass("is-active"),e.parent().find(".editor-toolbar").removeClass("error"),e.parent().find(".editor-source").removeClass("error")})),this.container.on("mouseenter.".concat(fs),(function(){t.element.hasClass("error")||t.container.addClass("is-hover")})).on("mouseleave.".concat(fs),(function(){t.container.removeClass("is-hover")})),n.settings.onLinkClick&&e.on("click.editorlinks","a",(function(e){n.settings.onLinkClick(e,{elem:t,url:e.currentTarget.getAttribute("data-url")}),e.preventDefault(),e.stopImmediatePropagation(),e.stopPropagation()})),this.label.on("click.editor",(function(){i.focus()})),i.attr("aria-label",this.label.text()),this},restoreSelection:function(t){var e=window.getSelection();if(t||(t=this.savedSelection),t){e.removeAllRanges();for(var i=0,n=t.length;i<n;i+=1)e.addRange(t[i])}},saveSelection:function(){var t,e=window.getSelection();if(e.getRangeAt&&e.rangeCount){t=[];for(var i=0,n=e.rangeCount;i<n;i+=1)t.push(e.getRangeAt(i));return t}return null},getSelectionStart:function(){var t=document.getSelection().anchorNode;return t&&3===t.nodeType?t.parentNode:t},getrange:function(){return window.getSelection().getRangeAt(0)},findElementInSelection:function(t,e){var i,n,s,a=this.getrange();if(a){for(i=s=a.commonAncestorContainer||a.parentElement();i!==e;i=i.parentNode)if(i&&i.tagName&&i.tagName.toLowerCase()===t)return i;if(!a.collapsed&&(void 0===a.text||a.text)&&s.getElementsByTagName){i=s.getElementsByTagName(t),n=document.createRange?document.createRange():document.body.createTextRange();for(var o=0,r=i.length;o<r;o++)if(document.createRange){if(n.selectNodeContents(i[o]),a.compareBoundaryPoints(Range.END_TO_START,n)<0&&a.compareBoundaryPoints(Range.START_TO_END,n)>0)return i[o]}else if(n.moveToElementText(i[o]),a.compareEndPoints("StartToEnd",n)<0&&a.compareEndPoints("EndToStart",n)>0)return i[o]}}},restoreLinkSelection:function(){var t=$(this.findElementInSelection("a",this.element[0]));if(this.settings.anchor.url=this.settings.anchor.defaultUrl,this.settings.anchor.class=this.settings.anchor.defaultClass,this.settings.anchor.target=this.settings.anchor.defaultTarget,this.settings.anchor.isClickable=this.settings.anchor.defaultIsClickable,t.length){this.settings.anchor.url=t.attr("href"),this.settings.anchor.class=t.attr("class"),this.settings.anchor.target=t.attr("target");var e=t.attr("contenteditable");!1!==e&&"false"!==e||(this.settings.anchor.isClickable=!0)}},execAction:function(t){if(this.getCurrentElement()===this.element)t.indexOf("append-")>-1?this.execFormatBlock(t.replace("append-","")):"anchor"===t?(this.restoreLinkSelection(),this.modals.url.data("modal").open()):"image"===t?this.modals.image.data("modal").open():"foreColor"===t||"backColor"===t?this.execColorActions(t):"clearFormatting"===t?this.clearFormatting():"source"===t||"visual"===t?this.toggleSource():document.execCommand(t,!1,null);else switch(t){case"visual":this.toggleSource();break;case"anchor":this.modals.url.data("modal").open();break;default:this.insertTextAreaContent(null,t)}this.checkSelection()},insertImage:function(t){document.execCommand("insertImage",!1,t)},toggleSource:function(t){var e=this,i=function(t){return"string"==typeof t&&"false"===t.toLowerCase()||"boolean"==typeof t&&!1===t||"number"==typeof t&&0===t};if(this.sourceViewActive()&&!t){var n=this.textarea.val();$.when(this.element.triggerHandler("beforepreviewmode",n)).done((function(t){i(t)||function(t){var i=t||e.textarea.val();i=g.sanitizeHTML(i),i=e.getCleanedHtml(i),e.element.empty().removeClass("source-view-active hidden"),e.sourceView.addClass("hidden").removeClass("is-focused"),e.element.trigger("focus.editor"),e.switchToolbars(),e.textarea.off("input.editor-firechange"),setTimeout((function(){var t;e.orderedListTypeStyling(),e.element[0].innerHTML=i,i=e.element[0].innerHTML,e.element.triggerHandler("afterpreviewmode",i);var n=null===(t=e.toolbar)||void 0===t||null==t?void 0:t.find('[data-action="source"]');n&&!e.isBtnOverflowedItem(n)&&n.focus()}),0)}(t)}))}else{var s=this.element.html();$.when(this.element.triggerHandler("beforesourcemode",s)).done((function(t){i(t)||function(t){var i,n=t||e.element.html().trim().replace(/\s+/g," ").replace(/<br( \/)?>/g,"<br>\n").replace(/<\/p> /g,"</p>\n\n").replace(/<\/blockquote>( )?/g,"</blockquote>\n\n");e.settings.useSourceFormatter&&(n=e.formatHtml(n)),e.element.addClass("source-view-active"),e.switchToolbars(),e.textarea.val(n).focus(),e.sourceView.removeClass("hidden"),e.element.addClass("hidden"),e.adjustSourceLineNumbers(),e.textarea.focus(),n=e.textarea.val(),e.textarea.off("input.editor-firechange").on("input.editor-firechange",(function(){e.element.trigger("change")})),e.element.triggerHandler("aftersourcemode",n);var s=null===(i=e.toolbar)||void 0===i||null==i?void 0:i.find('[data-action="visual"]');s&&!e.isBtnOverflowedItem(s)&&s.focus()}(t)}))}},isBtnOverflowedItem:function(t){var e=this.toolbar.data("toolbar")||this.toolbar.data("toolbar-flex"),i=!1;return e&&e.overflowedItems.forEach((function(e){t.is(e.element)&&(i=!0)})),i},clearFormatting:function(){var t,e=this,i=this.getSelectionParentElement(),n=i.tagName,s={},a=function(t){var e=window.getSelection(),i=!1;if(x.browser.isIE11()){var n=e.getRangeAt(0),s=document.createRange();s.selectNode(t);var a=n.compareBoundaryPoints(Range.START_TO_END,s),o=n.compareBoundaryPoints(Range.START_TO_START,s),r=n.compareBoundaryPoints(Range.END_TO_START,s),l=n.compareBoundaryPoints(Range.END_TO_END,s);i=a!==o||r!==l||a!==l}else i=e.containsNode(t,!0);return i};i&&n&&(n=n.toLowerCase(),s=function(){for(var t=function(t){return t&&t.style&&""!==t.style.textAlign},n=i,s=t(n),a=9999;!s&&a>0;)a--,s=(n=n?n.parentNode:null)&&n===e.element[0]||t(n);var o={found:n&&n!==i&&n!==e.element[0]};return o.found&&(o.elem=n,o.textAlign=n.style.textAlign),o}(),function(){var t=function(t){var e=[].slice.call(t.querySelectorAll("li"));if(e.length>0){var i=document.createDocumentFragment();e.forEach((function(t){var e=document.createTextNode(t.textContent);i.appendChild(e),i.appendChild(document.createElement("br"))}));var n=e[0].parentNode;n.parentNode.insertBefore(i,n.nextSibling),n.parentNode.removeChild(n)}};if("li"===n)t(i.parentNode);else if(/ul|ol/.test(n))t(i);else{var e=i.classList.contains("editor")?i:$(i).closest(".editor").length?i.parentNode:null;e&&[].slice.call(e.querySelectorAll("ul, ol")).forEach((function(e){a(e)&&t(e)}))}}(),function(){var t=function(t){var e=t.parentNode,i=document.createElement("p");i.innerHTML=t.innerHTML,e.replaceChild(i,t)};ms.indexOf(n)>-1?"p"!==n&&(document.execCommand("removeFormat",!1,null),t(i)):ms.forEach((function(e){"p"!==e&&[].slice.call(i.querySelectorAll(e)).forEach((function(e){return t(e)}))}));for(var s,a=[].slice.call(e.element[0].querySelectorAll("blockquote, pre")),o=0,r=a.length;o<r;o++)s=void 0,s=!1,function t(e){for(var n=0,a=e.length;n<a;n++){var o=e[n],r=[].slice.call(o.children);o===i&&(s=!0),r.length&&!s&&t(r)}}([].slice.call(a[o].children)),s&&(document.execCommand("removeFormat",!1,null),t(a[o]))}(),t=function(t){var e=t.parentNode,i=t.firstChild;e.insertBefore(i,t),e.removeChild(t),e.normalize()},"a"===n?t(i):[].slice.call(i.querySelectorAll("a")).forEach((function(e){a(e)&&t(e)})));var o=i.parentNode;o&&o!==this.element[0]&&[].slice.call(o.querySelectorAll("[align]")).forEach((function(t){return t.removeAttribute("align")}));document.execCommand("removeFormat",!1,null),s.found&&(s.elem.style.textAlign=s.textAlign)},getSelectionParentElement:function(){var t,e=null;return window.getSelection?(t=window.getSelection()).rangeCount&&1!==(e=t.getRangeAt(0).commonAncestorContainer).nodeType&&(e=e.parentNode):(t=document.selection)&&"Control"!==t.type&&(e=t.createRange().parentElement()),e},checkColorButtonState:function(t){var e=$('[data-action="'.concat(t,'"]'),this.toolbar),i=e.data("colorpicker"),n=document.queryCommandValue(t);return i&&"rgb(0, 0, 0)"!==n&&("firefox"===x.browser.name&&"backColor"===t&&(n=$(window.getSelection().focusNode.parentNode).css("background-color")),"number"==typeof n&&(n=i.decimal2rgb(n)),n=n?i.rgb2hex(n):"",e.attr("data-value",n),e.find(".swatch").css("background-color",["transparent","#1a1a1a","#f0f0f0","#ffffff","#313236"].indexOf(n.toLowerCase())>-1?"":n)),{cpBtn:e,cpApi:i,color:n}},execColorActions:function(t){var e=this,i=this.checkColorButtonState(t),n=i.cpBtn,s=i.cpApi;n.one("selected.editor",(function(i,s){var a=s;if(void 0===a||a instanceof $||!a.element||(a=$(s.element)),a.is("a")){var o="".concat(a.data("value")).toLowerCase();if(-1===(o="#"!==o?o:"").indexOf("#")&&(o="#".concat(o)),"#"===o&&(o=""),n.attr("data-value",o),n.find(".swatch").css("background-color",o),"ie"===x.browser.name||"foreColor"===t)o?document.execCommand(t,!1,o):document.execCommand("removeFormat",!1,t);else{document.execCommand("fontSize",!1,"2");for(var r=e.getSelectionParentElement().parentNode.getElementsByTagName("font"),l=0,c=r.length;l<c;l++)r[l].hasAttribute("style")&&(r[l].style.backgroundColor="");setTimeout((function(){for(var t=0,e=r.length;t<e;t++)r[t].hasAttribute("size")&&(r[t].style.backgroundColor=o,r[t].removeAttribute("size"))}),0)}}})),s.toggleList()},execFormatBlock:function(t){if(void 0===this.selection&&this.checkSelection(),this.selection&&this.selection instanceof Selection){var e=$(this.selection.anchorNode).is("div.editor")?$(this.selection.anchorNode).get(0):$(this.selection.anchorNode).parents("div.editor").get(0);if(void 0!==e&&this.element.get(0)===e){var i=this.getSelectionData(this.selection.anchorNode);if("blockquote"===t&&i.el&&"blockquote"===i.el.parentNode.tagName.toLowerCase())return document.execCommand("outdent",!1,null);if(i.tagName===t&&(t="p"),"ie"===x.browser.name){if("blockquote"===t)return document.execCommand("indent",!1,t);t="<".concat(t,">")}document.queryCommandState("bold")&&document.execCommand("bold",!1,t),document.queryCommandState("italic")&&document.execCommand("italic",!1,t),document.queryCommandState("underline")&&document.execCommand("underline",!1,t),document.queryCommandState("strikethrough")&&document.execCommand("strikethrough",!1,t),i.el&&i.el.innerHTML&&"<ul>"===i.el.innerHTML.substr(0,4)&&(document.execCommand("insertunorderedlist",!1,t),document.execCommand("removeFormat",!1,t)),this.selection.anchorNode&&this.selection.anchorNode.parentNode&&"UL"===this.selection.anchorNode.parentNode.parentElement.nodeName&&(document.execCommand("insertunorderedlist",!1,t),document.execCommand("removeFormat",!1,t)),i.el&&i.el.innerHTML&&"<ol>"===i.el.innerHTML.substr(0,4)&&(document.execCommand("insertorderedlist",!1,t),document.execCommand("removeFormat",!1,t)),this.selection.anchorNode&&this.selection.anchorNode.parentNode&&"OL"===this.selection.anchorNode.parentNode.parentElement.nodeName&&(document.execCommand("insertorderedlist",!1,t),document.execCommand("removeFormat",!1,t)),document.execCommand("formatBlock",!1,t),this.checkActiveButtons()}}},getSelectionData:function(t){var e;for(t&&t.tagName&&(e=t.tagName.toLowerCase());t&&-1===ms.indexOf(e);)(t=t.parentNode)&&t.tagName&&(e=t.tagName.toLowerCase());return{el:t,tagName:e}},isListItemChild:function(t){for(var e=t.parentNode,i=e.tagName.toLowerCase();-1===ms.indexOf(i)&&"div"!==i;){if("li"===i)return!0;if(!(e=e.parentNode)||!e.tagName)return!1;i=e.tagName.toLowerCase()}return!1},destroyToolbar:function(){var t=function(t){return t||$()};this.toolbar=t(this.toolbar),this.element=t(this.element),this.textarea=t(this.textarea);var e=this.toolbar.data("toolbar")||this.toolbar.data("toolbar-flex");e&&e.destroy();for(var i=this.toolbar.find("button"),n=0,s=i.length;n<s;n++){var a=$(i[n]).data("tooltip");a&&"function"==typeof a.destroy&&a.destroy();var o=$(i[n]).data("button");o&&"function"==typeof o.destroy&&o.destroy()}for(var r=$('[data-action="foreColor"], [data-action="backColor"]',this.element),l=0,c=r.length;l<c;l++){var d=$(r[l]).data("colorpicker");d&&"function"==typeof d.destroy&&d.destroy()}if(this.fontPickerElem){this.fontPickerElem.off("font-selected.".concat(fs));var h=this.fontPickerElem.data("fontpicker");h&&h.destroy(),delete this.fontPickerElem}this.toolbar.off(["click.".concat(fs),"selected.".concat(fs)].join(" ")),this.toolbar.remove(),delete this.toolbar;var u=["blur","DOMNodeInserted","focus","input.".concat(fs),"keydown.".concat(fs),"keypress.".concat(fs),"keyup.".concat(fs),"mouseup.".concat(fs),"mousedown.".concat(fs),"paste.".concat(fs)].join(" ");this.element.off(u),this.textarea.off(u),this.element.prev(".label").off("click.editor"),this.container.closest(".editor-container").off("focus.editor blur.editor click.editorlinks");var p=this.checkColorButtonState("foreColor"),g=p.cpBtn;if(g.off("selected.editor"),(g=(p=this.checkColorButtonState("backColor")).cpBtn).off("selected.editor"),delete this.pasteWrapper,delete this.pasteWrapperHtml,delete this.selectionHandler,$(window).off("resize.editor"),this.modals)for(var f=Object.keys(this.modals),m=0,v=f.length;m<v;m++){var b=$("#modal-".concat(f[m],"-").concat(this.id)),y=b.data("modal");b.off("beforeclose.editor close.editor open.editor beforeopen.editor"),y&&"function"==typeof y.destroy&&y.destroy(),b.remove()}delete this.modals,this.element.trigger("destroy.toolbar.editor")},setPreviewMode:function(){var t=this.container[0].classList,e=this.element[0].classList;if(!t.contains("is-preview")){if(!this.isEditable()){var i=["is-disabled","is-readonly","is-preview"];t.remove.apply(t,i),e.remove.apply(e,i)}this.sourceViewActive()&&this.toggleSource(),t.add("is-preview"),e.remove("is-preview"),this.element[0].setAttribute("contenteditable",!1),this.element[0].removeAttribute("aria-multiline"),this.element[0].removeAttribute("role"),[].slice.call(this.element[0].querySelectorAll("a")).forEach((function(t){var e=$(t).data("tooltip");e&&"function"==typeof e.destroy&&e.destroy()}))}},destroyPreviewMode:function(){if(this.container[0]){var t=this.container[0].classList;t.contains("is-preview")&&(t.remove("is-preview"),this.previewRendered?(this.element[0].setAttribute("aria-multiline",!0),this.element[0].setAttribute("role","textbox"),this.bindAnchorPreview()):(this.previewRendered=!0,this.init()))}},isPreview:function(){return!!this.container[0]&&this.container[0].classList.contains("is-preview")},isEditable:function(){var t=this,e=!0,i=function(t,e){return t.classList.contains(e)};return["is-disabled","is-readonly","is-preview"].forEach((function(n){(i(t.container[0],n)||i(t.element[0],n))&&(e=!1)})),e},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,vs)),this.teardown().init()},teardown:function(){this.destroyToolbar();var t=this.getCurrentElement();return this.sourceView&&(this.sourceView.off(".editor"),this.sourceView.remove(),delete this.sourceView),delete this.textarea,this.lineNumbers&&delete this.lineNumbers,this.selection&&delete this.selection,this.selectionRange&&delete this.selectionRange,t.off("keyup.editor"),t.off("mouseup.editor"),this.container.off(["focusin.".concat(fs),"focusout.".concat(fs),"mouseneter.".concat(fs),"mouseleave.".concat(fs),"input.".concat(fs),"keyup.".concat(fs)].join(" ")),this.container.removeClass("editor-container"),delete this.container,this.label.off("click.".concat(fs)),delete this.label,this.element.attr("contenteditable","false"),this.element.off(["mousedown.".concat(fs),"updated.".concat(fs)].join(" ")),$("html").off("themechanged.".concat(fs)),delete this.id,delete this.isActive,this},destroy:function(){this.teardown(),$.removeData(this.element[0],fs)},disable:function(){this.destroyPreviewMode(),this.element.addClass("is-disabled").attr("contenteditable","false"),this.container.addClass("is-disabled")},enable:function(){this.destroyPreviewMode(),this.element.removeClass("is-disabled is-readonly").attr("contenteditable","true"),this.container.removeClass("is-disabled is-readonly")},readonly:function(){this.destroyPreviewMode(),this.element.removeClass("is-readonly").attr("contenteditable","false"),this.container.addClass("is-readonly")},editable:function(){this.enable()},preview:function(){this.container[0].classList.contains("is-preview")||this.setPreviewMode()},onPasteTriggered:function(){"firefox"!==x.browser.name&&document.addEventListener&&document.addEventListener("paste",(function(t){if(void 0!==t.clipboardData){var e=t.clipboardData.items[0];if(0===e.type.indexOf("image")){var i=e.getAsFile(),n=new FileReader;n.onload=function(t){var e=t.target.result;document.execCommand("insertImage",!1,e)},n.readAsDataURL(i)}}}),!1)},isWordFormat:function(t){return/<font face="Times New Roman"|class="?Mso|style="[^"]*\bmso-|style='[^'']*\bmso-|w:WordDocument/i.test(t)||/class="OutlineElement/.test(t)||/id="?docs\-internal\-guid\-/.test(t)},cleanWordHtml:function(t){var e=t;e=(e=(e=(e=(e=(e=e.replace(/<!--[\s\S]+?-->/gi,"")).replace(/<(!|script[^>]*>.*?<\/script(?=[>\s])|\/?(\?xml(:\w+)?|img|meta|link|style|\w:\w+)(?=[\s\/>]))[^>]*>/gi,"")).replace(/<(\/?)s>/gi,"<$1strike>")).replace(/&nbsp;/gi," ")).replace(/<span\s+style\s*=\s*"\s*mso-spacerun\s*:\s*yes\s*;?\s*"\s*>([\s\u00a0]*)<\/span>/gi,(function(t,e){return e.length>0?e.replace(/./," ").slice(Math.floor(e.length/2)).split("").join(" "):""}))).replace(/(\n|\r| class=(\'|")?Mso[a-zA-Z]+(\'|")?)/g," ");for(var i=["style","script","applet","embed","noframes","noscript"],n=0,s=i.length;n<s;n++){var a=new RegExp("<".concat(i[n],".*?").concat(i[n],"(.*?)>"),"gi");e=e.replace(a,"")}return e},stripAttribute:function(t,e,i){return"style"===e?this.stripStyles(t,i):t.replace(i,"")},stripStyles:function(t,e){var i=["color","font-size","background","font-weight","font-style","text-decoration","text-align"];return t.replace(e,(function(t){for(var e=(t=t.replace(/( style=|("|\'))/gi,"")).split(";"),n="",s=0;s<e.length;s++){var a=e[s].split(":");n+=i.indexOf((a[0]||"").trim())>-1?"".concat(a[0],":").concat(a[1],";"):""}return""!==n?' style="'.concat(n,'"'):""}))},formatHtml:function(t){t=t.trim();var e=this.settings,i=t.split(/</),n=0,s="";function a(t){var i="",n=t*("number"==typeof e.formatterTabsize?e.formatterTabsize:4);if(t>-1)for(;n--;)i+=" ";return i}for(var o=0,r=i.length;o<r;o++){var l=i[o].split(/>/);2===l.length?("/"===i[o][0]&&n--,s+=a(n),"/"!==i[o][0]&&n++,o>0&&(s+="<"),s+="".concat(l[0].trim(),">\n"),""!==l[1].trim()&&(s+="".concat(a(n)+l[1].trim().replace(/\s+/g," "),"\n")),l[0].match(/^(area|base|br|col|command|embed|hr|img|input|link|meta|param|source)/)&&n--):s+="".concat(a(n)+l[0],"\n")}return s.trim()},clear:function(){this.element.empty(),this.textarea.empty(),this.sourceView.find(".line-numbers").empty(),this.sourceView.find(".textarea-print").empty()}},$.fn.editor=function(t){return this.each((function(){var e=$.data(this,fs);e?e.updated(t):e=$.data(this,fs,new bs(this,t))}))};var ys="hierarchy",ws={legend:[],legendKey:"",dataset:[],newData:[],templateId:"",leafHeight:null,leafWidth:null,beforeExpand:null,renderSubLevel:!1,layout:"horizontal",rootId:null,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function xs(t,e){this.settings=E.mergeSettings(t,e,ws),this.element=$(t),this.init()}xs.prototype={init:function(){var t=this.settings;if(null===this.rootId&&void 0===this.element.attr("id")?this.rootId="hierarchyChart":this.element.attr("id")?this.settings.rootId=this.element.attr("id"):this.rootId="hierarchyChart",t.colorClass=["azure08","turquoise02","amethyst06","slate06","amber06","emerald07","ruby06"],this.handleEvents(),void 0!==t.dataset&&0!==t.dataset.length&&Array.isArray(t.dataset)){if(t.dataset[0]&&t.dataset[0].children&&t.dataset[0].children.length>0||this.isStackedLayout()?this.render(t.dataset[0]):t.dataset&&t.dataset.children.length>0&&this.render(t.dataset),null!==t.leafHeight&&null!==t.leafWidth){var e="'body .hierarchy .leaf,body .hierarchy .sublevel .leaf,body .hierarchy .container .root.leaf { width: ".concat(t.leafWidth,"px;  height: ").concat(t.leafHeight,"px;  }'");$('<style type="text/css" id="hierarchyLeafStyles">'.concat(e,"</style>")).appendTo("body")}t.layout&&this.setLayout(t.layout),this.setAutomationAttributes()}else this.element.emptymessage(t.emptyMessage)},setAutomationAttributes:function(){var t=this;[].slice.call(this.element[0].querySelectorAll(".leaf")).forEach((function(e){var i=$(e),n=i.data();if(n&&n.attributes){var s=i.find(".btn-expand, .btn-collapse");E.addAttributes(i,t,n.attributes,"hierarchy-leaf"),E.addAttributes(s,t,n.attributes,"hierarchy-btn-toggle")}}))},setLayout:function(t){switch(this.isPagingLayout()&&(t="paging"),this.isMobileOnly()&&(t="mobile-only"),t){case"horizontal":default:this.element.addClass("layout-is-horizontal");break;case"stacked":this.element.addClass("layout-is-stacked");break;case"paging":this.element.addClass("layout-is-paging");break;case"mobile-only":this.element.addClass("layout-is-mobile-only")}},handleEvents:function(){var t=this,e=this,i=this.settings;e.element.off("click.hierarchy").on("click.hierarchy",".btn",(function(t){if(!e.isStackedLayout()){i.newData.length>0&&(i.newData=[]);var n=$(this).closest(".leaf").attr("id"),s=$("#".concat(n)).data(),a={branch:$(this).closest("li"),leaf:$(this).closest(".leaf"),button:$(this)};s.isExpanded?e.collapse(t,s,a):e.expand(t,s,a)}})),this.element.on("keypress",".leaf",(function(t){var i=$(this).attr("id"),n=$("#".concat(i));13===t.which&&(n.isExpanded?e.collapse(t,n):e.expand(t,n))})),e.element.off("dblclick.hierarchy").on("dblclick.hierarchy",".leaf",(function(e){var i=e.currentTarget.id,n={event:e,data:$("#".concat(i)).data()};e.stopImmediatePropagation(),t.element.trigger("dblclick",n)})),e.element.on("mouseup",".leaf, .back button",(function(t){var e=$(this),i=$(t.target),n=e.closest(".hierarchy").data("hierarchy"),s=e.data(),a=$(this).attr("id"),o={target:t.target,pageX:t.pageX,pageY:t.pageY},r=i.is("button"),l=!i.hasClass("btn-back"),c=i.is(".btn-back"),d=i.find("use").prop("href"),h=!!d&&"#icon-caret-up"===d.baseVal,u=!!d&&"#icon-caret-down"===d.baseVal,p=!!d&&"#icon-caret-right"===d.baseVal,g=i.hasClass("btn-actions"),f=i.is("a")&&i.parent().parent().is("ul.popupmenu"),m=e.hasClass("ancestor"),v="selected";if(t.stopImmediatePropagation(),(!f||!$(i).parent().data("disabled"))&&($(".leaf.is-selected").removeClass("is-selected"),$("#".concat(a)).addClass("is-selected"),r&&h&&l&&(v=m?"back":"collapse"),r&&u&&l&&(v="expand"),c&&(v="back"),g&&(v="actions",n.buildActionsMenu(s,e)),f&&(v="action"),r&&p&&l&&(v="forward"),3===t.which&&(v="rightClick"),n)){var b={id:a,data:s,actionReference:f?i.data("actionReference"):null,targetInfo:o,eventType:v,isForwardEvent:n.isForwardEvent(v),isBackEvent:n.isBackEvent(v),isAddEvent:n.isAddEvent(v),isExpandEvent:n.isExpandEvent(v),isCollapseEvent:n.isCollapseEvent(v),isSelectedEvent:n.isSelectedEvent(v),isActionsEvent:n.isActionsEvent(v),isActionEvent:n.isActionEvent(v),allowLazyLoad:n.allowLazyLoad(s,v)};e.trigger("selected",b)}}))},selectLeaf:function(t){var e=$("#".concat(t));$(".leaf.is-selected").removeClass("is-selected"),e.addClass("is-selected");var i={data:e.data(),actionReference:null,isForwardEvent:!1,isBackEvent:!1,isAddEvent:!1,isExpandEvent:!1,isCollapseEvent:!1,isSelectedEvent:!0,isActionsEvent:!1,isActionEvent:!1,allowLazyLoad:!1};e.trigger("selected",i)},updateActions:function(t,e){var i=$(t.targetInfo.target).closest(".leaf"),n=t.data,s=$(i).find(".popupmenu"),a=s.data("trigger").data().popupmenu,o=s.find("li").not(":eq(0)");$(o).each((function(t,e){$(e).remove()})),n.menu.actions=e,s.append(this.getActionMenuItems(n)),a.keydownThenClick=!0,a.open(),a.handleAfterPlace(null,{element:s.parent(),parent:$(i).find(".btn-actions"),placement:"bottom"})},buildActionsMenu:function(t,e){var i=$(e).find(".popupmenu"),n=[];void 0!==t.menu&&(i.empty(),t.menu.details&&(i.addClass("has-detail-fields"),n.push('<li><div class="detail-fields">'.concat(t.menu.details.map((function(t){return'<div class="dt-fields-row"><div class="dt-fields-cell">'.concat(t.key,'</div><div class="dt-fields-cell">').concat(t.value,"</div></div>")})).join(""),"</div></li>"))),t.menu.actions&&n.push(this.getActionMenuItems(t)),n.forEach((function(t){i.append(t)})))},getActionMenuItems:function(t){var e,i=this,n=t.menu.actions.map((function(t){return void 0===t.disabled&&(t.disabled=!1),t})),s=function(t){return E.stringAttributes(i,e,"option-".concat(t))};return t.attributes&&(e=this.getAutomationAttributes(t,"-hierarchy-popupmenu")),n.map((function(t,e){return t.hasOwnProperty("data")&&"separator"===t.data.type?'<li class="separator"></li>':'\n        <li data-disabled="'.concat(t.disabled,'" class="').concat(t.menu?"submenu":"",'">\n          <a href="').concat(t.url,'"').concat(s(e)," data-action-reference='")+JSON.stringify(t.data)+"'>\n            ".concat(t.value,"\n            ").concat(t.menu?'<svg class="arrow icon-dropdown icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-dropdown"></use></svg>':"","\n          </a>\n          ").concat(t.menu?'<div class="wrapper" role="application" aria-hidden="true">\n            <ul class="popupmenu">\n              '.concat(t.menu.map((function(i,n){return'\n              <li data-disabled="'.concat(t.disabled,'">\n                <a href="').concat(t.url,'"').concat(s("".concat(e,"-").concat(n)),' data-action-reference="').concat(JSON.stringify(t.data),'">').concat(i.value,"</a>\n              </li>")})).join(""),"\n            </ul>\n          </div>"):"","\n        </li>")})).join("")},isBackEvent:function(t){return"back"===t},isForwardEvent:function(t){return"forward"===t},isAddEvent:function(t){return"add"===t},isExpandEvent:function(t){return"expand"===t},isCollapseEvent:function(t){return"collapse"===t},isSelectedEvent:function(t){return"selected"===t},isActionsEvent:function(t){return"actions"===t},isActionEvent:function(t){return"action"===t},allowLazyLoad:function(t,e){return void 0!==t&&void 0!==e&&(!t.isLoaded&&!t.isLeaf&&"expand"===e)},data:function(t,e,i,n){void 0===n&&(n={});var s=this.settings,a=e.isRootNode?e:e[0],o=[];function r(e,s,a){Object.keys(s).forEach((function(e){"id"===e&&t===s.id&&(s.isLoaded||s.isRootNode||function(t,e){e.insert&&(delete t.isLeaf,t.isExpanded=!0),0!==i.length&&e.insert?t.children=[i]:t.children=i}(s,n),o.push(s))})),s.children&&l(e,s.children)}function l(t,e,i){if(void 0===e.length)r(t,e);else for(var n=0,s=e.length;n<s;n++)r(t,e[n])}return s.newData.length>0&&(s.newData=[]),void 0!==i&&l(this,a),0!==o.length&&$("#".concat(o[0].id)).data(o[0]),o[0]},add:function(t,e,i){var n=this.settings,s=void 0!==e.id?e.id:t,a=$("#".concat(s)),o=a.parent().hasClass("leaf-container")?a.parent().parent():a.parent(),r={},l="sub-level"!==o.parent().attr("class"),c=1===o.children(".sublist").length;if(l?c?r.element=o.children(".sublist"):(r.el=o.append("<ul class='sublist'></ul>"),r.element=$(r.el).find(".sublist")):(r.el=o.children("ul"),r.element=$(r.el)),0===r.element.length&&(r.el=o.append("<ul></ul>"),r.element=$(r.el).find("ul")),!e.isRootNode){for(var d=0,h=i.length;d<h;d++)n.newData.push(i[d]);this.createLeaf(i,r.element)}this.updateState(a,!1,null,"add")},closePopupMenu:function(t){var e=t.find(".btn-actions");0!==e.length&&e.data("popupmenu").close()},expand:function(t,e,i){var n=this.settings,s=i.leaf,a=s.next();this.closePopupMenu(s),a.animateOpen(),this.element.trigger("expanded",[e,n.dataset]),s.hasClass("root")&&(a=a.next("ul")).animateOpen(),s.parent().removeClass("branch-collapsed").addClass("branch-expanded"),this.updateState(s,!1,null,"expand")},collapse:function(t,e,i){var n=this,s=this.settings,a=i.leaf,o=a.next();this.closePopupMenu(a),o.animateClosed().on("animateclosedcomplete",(function(){n.element.trigger("collapsed",[e,s.dataset])})),a.hasClass("root")&&(o=o.next("ul")).animateClosed(),a.parent().removeClass("branch-expanded").addClass("branch-collapsed"),this.updateState(a,!1,null,"collapse")},render:function(t){var e=this,i=this.settings,n=i.legend,s=t.children,a=[],o="".concat(E.uniqueId(this.element,"hierarchy")),r={legend:"<legend><ul></ul></legend>",chart:'<ul class="container"><li class="chart"></li></ul>',toplevel:this.isPagingLayout()?'<ul class="child-nodes"></ul>':'<ul class="top-level"></ul>',sublevel:this.isPagingLayout()?"":'<ul class="sub-level"></ul>'};$("#".concat(this.settings.rootId)).append(r.chart);var l=$("#".concat(this.settings.rootId," .chart"));if(0!==n.length){$("#".concat(this.settings.rootId)).prepend(r.legend);var c=$("#".concat(this.settings.rootId," legend"));this.createLegend(c)}if(this.isSingleChildWithChildren()&&$(l).addClass("has-single-child"),this.setColor(t),this.isPagingLayout()&&t.parentDataSet){var d=' id="'.concat(o,'-back-button" data-automation-id="automation-id-').concat(o,'-back-button"'),h='<div class="back">\n        <button type="button" class="btn-icon hide-focus btn-back"'.concat(d,'>\n          <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n            <use href="#icon-caret-left"></use>\n          </svg>\n          <span>Back</span>\n        </button></div>'),u=$(h).appendTo(l);setTimeout((function(){return u.next($(".leaf")).addBack($(".back")).wrapAll('<div class="back-container"></div>')})),u.children("button").data(t),$(l).addClass("has-back")}if(t.isMultiRoot){var p='<div class="leaf multiRoot"><div><h2>'.concat(t.multiRootText,"</h2></div></div>");p=g.sanitizeHTML(p),a.push(p),$(a[0]).addClass("root").appendTo(l)}else if(null!==t.ancestorPath&&void 0!==t.ancestorPath){t.ancestorPath.push(t.centeredNode);var f="".concat(t.ancestorPath.map((function(t){return" ".concat(e.getTemplate(t)," ")})).join(""));f=g.sanitizeHTML(f),a.push(f),$(a[0]).addClass("root ancestor").appendTo(l);var m=this.element.find(".leaf.root");m.each((function(i,n){e.updateState(n,!1,t.ancestorPath[i],"add"),i===m.length-1&&$(n).addClass("is-selected")}))}else{var v,b=t.centeredNode;this.isStackedLayout()&&null!==b?v=this.getTemplate(b):this.isStackedLayout()||(v=this.getTemplate(t)),v&&(v=g.sanitizeHTML(v),a.push(v),$(a[0]).addClass("root is-selected").appendTo(l)),b&&null!==b?this.updateState(this.element.find(".leaf.root"),!0,b,void 0):this.updateState(this.element.find(".leaf.root"),!0,t,void 0)}function y(t,e,i){if(null!=e)for(var n=0,s=e.length;n<s;n++){k(t,e[n])}}if(s&&s.length>0)for(var w=0,x=s.length;w<x;w++){var C=t.children[w].children;if(this.isLeaf(s[w])&&!this.isPagingLayout()&&i.renderSubLevel||this.isPagingLayout()?this.createLeaf(t.children[w],$(r.toplevel)):this.createLeaf(t.children[w],$(r.sublevel)),null!=C){y(this,t.children[w].children)}}function k(t,e,i){Object.keys(e).forEach((function(i){if("children"===i){var n=e.id,s=e,a=e.children;return null!=a&&a.length>0&&t.add(n,s,a),y(t,a)}return!0}))}var T=(this.element.find(".container").outerWidth()-$(window).width())/2;this.element.scrollLeft(T);var A=this.element.find(".top-level");0===this.element.find(".sub-level").length&&1===A.length&&A.addClass("no-sublevel")},isPagingLayout:function(){return this.settings.layout&&"paging"===this.settings.layout},isMobileOnly:function(){return this.settings.layout&&"mobile-only"===this.settings.layout},isStackedLayout:function(){return this.settings.layout&&"stacked"===this.settings.layout},isSingleChildWithChildren:function(){if(this.isStackedLayout())return!1;var t=this.settings;if(t.dataset&&t.dataset[0]&&t.dataset[0].children){for(var e=t.dataset[0].children.length,i=0;e--;)t.dataset[0].children[e].isLeaf||i++;return 1===i}return!1},getAutomationAttributes:function(t,e){var i=this,n=t.attributes;if(t.attributes&&"string"==typeof e&&""!==e)if(Array.isArray(t.attributes))n=[],t.attributes.forEach((function(t){var s="function"==typeof t.value?t.value(i):t.value;n.push({name:t.name,value:s+e})}));else{var s="function"==typeof t.attributes.value?t.attributes.value(this):t.attributes.value;n={name:t.attributes.name,value:s+e}}return n},getTemplate:function(t){var e=this,i=je.compile("{{#dataset}}".concat($("#".concat(g.stripTags(this.settings.templateId))).html(),"{{/dataset}}"),{dataset:t});return setTimeout((function(){var i=$("#btn-".concat(g.stripTags(t.id)));if(0!==i.length){var n=e.getAutomationAttributes(t,"-hierarchy-popupmenu");i.hideFocus().popupmenu({attachToBody:!1,attributes:n})}}),1),$(i).prop("outerHTML")},createLegend:function(t){for(var e=this.settings,i=0,n=0,s=e.legend.length;n<s;n++){var a=e.legend[n].label,o=e.colorClass[n];n-1+1==4&&(t.append("<ul></ul>"),i++),t.children("ul").eq(i).append(""+'<li>\n          <span class="key '.concat(o,'"></span>\n          <span>').concat(a,"</span>\n        </li>"))}},createLeaf:function(t,e){var i=this,n=$("#".concat(i.settings.rootId," .chart"),i.container),s=e.attr("class"),a=void 0!==s?$("#".concat(i.settings.rootId," .").concat(s)):e;function o(t){i.setColor(t);var n=i.getTemplate(t),s=i.settings.rootId,o=1===a.length?a:e,r=t.isExpanded||void 0===t.isExpanded?"branch-expanded":"branch-collapsed";if(t.isLeaf&&(r=""),1!==$("#".concat(s," #").concat(t.id)).length&&(o.append("<li class=".concat(r,">").concat($(n)[0].outerHTML,"</li>")),t.children)){for(var l="",c=0,d=t.children.length;c<d;c++){i.setColor(t.children[c]);var h=i.getTemplate(t.children[c]);t.children.length,l+="<li>".concat($(h)[0].outerHTML,"</li>")}(o=$("#".concat(s," #").concat(g.stripTags(t.id))).parent()).append("<ul>".concat(l,"</ul>"));for(var u=t.children.length;u--;){var p=$("#".concat(s," #").concat(g.stripTags(t.children[u].id)));i.updateState(p,!1,t.children[u],void 0)}}}if(a.length<1&&("top-level"===s?e.insertAfter(".root"):e.appendTo(n)),t.length)for(var r=0,l=t.length;r<l;r++)t.length,o(t[r]),i.updateState($("#".concat(i.settings.rootId," #").concat(g.stripTags(t[r].id))),!1,t[r],void 0);else o(t),i.updateState($("#".concat(i.settings.rootId," #").concat(g.stripTags(t.id))),!1,t,void 0)},setColor:function(t){var e=this,i=this.settings;if(this.setRootColor(t),this.isStackedLayout()&&(t.ancestorPath&&null!==t.ancestorPath&&t.ancestorPath.forEach((function(t){e.setRootColor(t)})),t.centeredNode&&null!==t.centeredNode&&this.setRootColor(t.centeredNode)),t.children&&!t.isRootNode)for(var n=0,s=t.children.length;n<s;n++)for(var a=0,o=i.legend.length;a<o;a++)t.children[n][i.legendKey]===i.legend[a].value&&(t.children[n].colorClass=i.colorClass[a])},setRootColor:function(t){for(var e=this.settings,i=0,n=e.legend.length;i<n;i++){if(t[e.legendKey]===e.legend[i].value){t.colorClass=e.colorClass[i];break}""===t[e.legendKey]&&(t.colorClass="default-color")}},isLeaf:function(t){var e=this.settings;return t.isLeaf||e.beforeExpand?t.isLeaf:!(t.children&&t.children.length>0)},updateState:function(t,e,i,n){if($.isEmptyObject($(t).data())&&i){var s=void 0===i?{}:i;$(t).data(s)}var a=this.settings,o=$(t).find(".btn"),r=this.isPagingLayout()?"caret-right":"caret-up",l=$(t).data();void 0===l&&void 0!==i&&(l=i),(l.children&&0!==l.children.length||"add"===n)&&(l.isExpanded=!0,l.isLoaded=!0),e&&(l.isRootNode=!0,l.isLoaded=!0),(void 0===l.isExpanded&&l.children||"expand"===n)&&(l.isExpanded=!0),void 0!==l.isExpanded&&"collapse"!==n||(l.isExpanded=!1),l.isExpanded?(o.find("svg.icon").changeIcon(r),o.addClass("btn-expand").removeClass("btn-collapse")):(o.find("svg.icon").changeIcon("caret-down"),o.addClass("btn-collapse").removeClass("btn-expand")),(l.isLeaf||l.isRootNode)&&o.addClass("btn-hidden"),l.isLeaf&&(l.isLoaded=!1,l.isExpanded=!1),this.isPagingLayout()&&(l.parentDataSet=a.dataset),$(t).data(l)},reload:function(t){this.destroy(),this.element.hierarchy(t)},unbind:function(){this.element.empty()},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,ws)),this.unbind().init()},destroy:function(){this.unbind(),this.element.removeData(ys)}},$.fn.hierarchy=function(t){return this.each((function(){var e=$.data(this,ys);e?e.updated(t):e=$.data(this,ys,new xs(this,t))}))};var Cs="fieldfilter",ks={dataset:[],dropdownOpts:{},template:""+"<label>".concat(H.translate("FieldFilter"),'</label>\n    <select class="dropdown no-init field-filter-dropdown">\n      {{#dataset}}\n        <option\n          {{#value}} value="{{value}}"{{/value}}\n          {{#selected}} selected{{/selected}}\n          {{#disabled}} class="is-disabled" disabled{{/disabled}}\n          {{#icon}} data-icon="{{icon}}"{{/icon}}\n        >{{text}}</option>\n      {{/dataset}}\n    </select>')};function Ts(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,ks),this.init()}Ts.prototype={init:function(){this.render(),this.handleEvents(),this.setFiltered()},render:function(t){var e=this.settings;if(t=t||e.dataset,"object"===l(je)&&t&&e.template){e.template instanceof $?e.template="".concat(e.template.html()):"string"==typeof e.template&&(M.containsHTML(e.template)||(e.template=$("#".concat(e.template)).html()));var i=je.compile(e.template,{dataset:e.dropdownOpts.source?[]:t}),n=""+'<label for="ffdropdown-empty" class="audible">\n          '.concat(H.translate("FieldFilter"),'\n        </label>\n        <select id="ffdropdown-empty" name="ffdropdown-empty" class="dropdown no-init field-filter-dropdown"></select>');t.length>0?this.element.before(i):0===t.length&&this.element.before(i||n);var s=this.element.attr("id")||this.element.attr("name");void 0===s&&(s=E.uniqueId(this.element,"fieldfilter-"),this.element[0].setAttribute("id",s));var a="".concat(s,"-ff");this.field=this.element.closest(".field, .field-short");var o=H.isRTL();e.dropdownOpts=e.dropdownOpts||{},o&&"object"===l(e.dropdownOpts)&&(e.dropdownOpts.placementOpts?e.dropdownOpts.placementOpts.x=this.element.outerWidth():e.dropdownOpts.placementOpts={x:this.element.outerWidth()}),e.dropdownOpts.cssClass=e.dropdownOpts.cssClass?"".concat(e.dropdownOpts.cssClass," ffdropdown"):"ffdropdown",e.dropdownOpts.noSearch=!0,this.ffdropdown=this.field.find("select.dropdown.field-filter-dropdown"),this.ffdropdown.dropdown(e.dropdownOpts).prev("label").addClass("audible"),this.ffdropdown[0].setAttribute("id",a),this.ffdropdown[0].setAttribute("name",a),this.ffdropdown.prev("label")[0].setAttribute("for",a);var r=this.ffdropdown.prev("label").prev("label").text();this.field.addClass("fieldfilter-wrapper"),this.field.find("div.dropdown span.audible").text(r),this.ddApi=this.ffdropdown.data("dropdown"),this.ddApi&&this.ddApi.icon&&this.ddApi.icon.addClass("ffdropdown-icon"),E.addAttributes(this.field,this,this.settings.attributes,"",!0),E.addAttributes(this.field.find("label:not(.audible)"),this,this.settings.attributes,"label",!0),E.addAttributes(this.field.find("select"),this,this.settings.attributes,"select",!0)}},setFiltered:function(){if(this.ddApi){var t=this.ddApi.element.find("option:selected");this.filtered=this.getTriggerData(t)}return this},getTriggerData:function(t){var e=this.settings,i=e.dropdownOpts.source&&this.ddApi?this.ddApi.dataset:e.dataset;return{idx:t.index(),item:t,data:i[t.index()]}},getFilterType:function(){return this.setFiltered(),this.filtered},setFilterType:function(t){if(this.ddApi){var e=-1,i=this.settings,n=i.dropdownOpts.source&&this.ddApi?this.ddApi.dataset:i.dataset;if("number"==typeof t&&t>-1&&t<n.length)e=t;else if("string"==typeof t){var s=this.ffdropdown.find('option[value="'.concat(t,'"]'));s.length||(s=this.ffdropdown.find("option").filter((function(){return $(this).text()===t}))),s.length&&(e=s.index())}-1!==e&&e!==this.ffdropdown[0].selectedIndex&&(this.ffdropdown[0].selectedIndex=e,this.ddApi.updated(),this.ffdropdown.triggerHandler("change"),this.setFiltered(),this.element.triggerHandler("filtered",[this.filtered]))}},handleEvents:function(){var t=this;return this.ffdropdown.on("listopened.".concat(Cs),(function(){var e=t.field.is(".field-short")?42:52;$("#dropdown-list ul").width(t.element.outerWidth()+e)})).on("selected.".concat(Cs),(function(e,i){var n=t.getTriggerData(i);t.element.triggerHandler("filtered",[n])})),this},readonly:function(){return this.ffdropdown.readonly(),this},enable:function(){return this.ffdropdown.enable(),this},disable:function(){return this.ffdropdown.disable(),this},unbind:function(){return this.ffdropdown.off(".".concat(Cs)),this.ddApi&&"function"==typeof this.ddApi.destroy&&this.ddApi.destroy(),this.ffdropdown.add(this.ffdropdown.prev("label")).remove(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element[0],t,ks)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Cs)}},$.fn.fieldfilter=function(t){return this.each((function(){var e=$.data(this,Cs);e?e.updated(t):e=$.data(this,Cs,new Ts(this,t))}))};var As="fieldoptions",Ss={};function $s(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Ss),this.init(),W("FieldOptions")}$s.prototype={init:function(){this.setElements(),this.handleEvents()},setElements:function(){var t=this;this.isFirefox="firefox"===x.browser.name,this.isSafari=x.browser.isSafari(),this.field=this.element.closest(".field, .radio-group"),this.targetElem=this.element;var e=this.field.find("label");return e&&(this.label=e),this.element[0].className.match(/(dropdown|multiselect)/)&&(this.targetElem=this.element.data("dropdown").pseudoElem),this.element[0].className.match(/(fileupload)/)&&(this.targetElem=this.field.find('.fileupload[type="text"]')),this.field.addClass("is-fieldoptions"),this.fieldParent=this.element.closest(".field").parent(),this.trigger=this.field.find(".btn-actions"),this.isFirefox&&this.trigger.length&&(this.trigger[0].contentEditable=!0,this.trigger.on("keydown.".concat(As),(function(t){9!==(t.which||t.keyCode||t.charCode||0)&&(t.preventDefault(),t.stopPropagation())}))),setTimeout((function(){t.popupmenuApi=t.trigger.data("popupmenu"),t.popupmenuApi&&(t.popupmenuApi.settings.returnFocus=!1,t.popupmenuApi.settings.offset.y=10)}),100),this},handleEvents:function(){var t=this,e=this,i=this.element.data("datepicker"),n=this.element.data("timepicker"),s=this.element.data("dropdown"),a=this.element.data("lookup")||this.element.hasClass("lookup"),o=this.element.is(".checkbox"),r=this.element.is(".fileupload"),l=this.element.is(".searchfield"),c=this.element.is(".colorpicker"),d=this.element.closest(".radio-group").length>0,h=this.element.is(".data")&&this.element.closest(".summary-form").length>0,u=function(t){return $(":focus").is(t)},p=function(e){(e||t.element).addClass("is-focused")},g=function(e){(e||t.element).removeClass("is-focused")},f=function(){e.element.add(e.trigger).add(e.field).add(e.fieldParent).addClass("is-active")},m=function(){e.element.add(e.trigger).add(e.field).add(e.fieldParent).removeClass("is-active")},v=function(e){var o=!u(t.element);return o=!t.trigger.is(e.relatedTarget)&&o,o=!t.trigger.is(".is-open")&&o,o=(!i||!i.isOpen())&&o,o=(!n||!n.isOpen())&&o,o=!$(e.relatedTarget).prev().is(t.element)&&o,o=(!s||!s.isOpen())&&o,o=!(a&&a.modal&&a.modal.isOpen())&&o,o=(!c||!t.element.is(".is-open"))&&o},b=function(e){e.trigger&&e.trigger.off("show.".concat(As)).on("show.".concat(As),(function(){f()})).off("hide.".concat(As)).on("hide.".concat(As),(function(e){v(e)&&(m(),t.element.removeClass("is-open"))}))},y=function(){var e,i=t.element.height();if(h){var n=parseInt(t.element.css("line-height"),10);if(i>n){t.element.removeClass("is-singleline"),e=-1*((t.element.outerHeight()-t.trigger.height())/2+0);var s=n-16;e+=s?s/2:0}else t.element.addClass("is-singleline"),e=2}else d&&(e=(i-10-t.trigger.height())/2*-1);return e},w=function(){t.trigger.css({top:"".concat(y()-1,"px")})};return x.features.touch?(this.field.addClass("visible"),this.trigger.on("beforeopen.".concat(As),(function(e){var a;(a=u(t.element),a=(!i||!i.isOpen())&&a,a=(!n||!n.isOpen())&&a,s&&s.isOpen()||!a)||f()})).on("close.".concat(As),(function(t){v(t)&&m()}))):(this.field.removeClass("visible"),this.field.on("mouseover.".concat(As),(function(){e.element.prop("disabled")||e.element.closest("is-disabled").length||e.field[0].className.indexOf("visible")<0&&e.field[0].classList.add("visible")})).on("mouseout.".concat(As),(function(){e.field[0].className.indexOf("visible")>-1&&e.field[0].classList.remove("visible")}))),s&&setTimeout((function(){var e=t.trigger.data("popupmenu");e&&e.menu.closest(".popupmenu-wrapper").css({"z-index":"4502"})}),0),(i||n)&&b(i||n),n&&(n.settings.returnFocus=!1),(a||c)&&(this.field.on("click.".concat(As),".lookup-wrapper .trigger, .colorpicker-container .trigger",(function(){f()})),c&&this.element.on("beforeopen.".concat(As),(function(){f()}))),o&&(this.trigger.addClass("is-checkbox"),!x.features.touch&&this.isSafari&&this.field.on("click.".concat(As),".checkbox-label",(function(){f()})).on("mouseout.".concat(As),".checkbox-label",(function(){m()}))),r&&(this.element.on("change.".concat(As),(function(){t.targetElem.focus()})),this.field.on("click.".concat(As),".trigger, .trigger-close",(function(){f()}))),l&&setTimeout((function(){t.trigger.add(t.trigger.next(".popupmenu")).appendTo(t.element.closest(".searchfield-wrapper"))}),0),h&&(w(),this.targetElem.add(this.trigger).on("keydown.".concat(As),(function(t){13===(t.which||t.keyCode||t.charCode||0)&&setTimeout((function(){f()}),0)})),this.targetElem.attr("tabindex",0).on("click.".concat(As),(function(){f()})),$(document).on("click.".concat(As),(function(e){$(e.target).is(t.element)||m()})),$("body").on("resize.".concat(As),(function(){w()}))),d&&(w(),this.element.on("focusin.".concat(As),".radio",(function(){var e=t.isSafari?200:0;p(),setTimeout((function(){f()}),e)})).on("focusout.".concat(As),".radio",(function(){g()})),$("body").on("resize.".concat(As),(function(){w()}))),this.targetElem.on("focusin.".concat(As),(function(){f(),d&&t.isSafari&&p()})).on("focusout.".concat(As),(function(e){var i=t.isSafari?200:0;d&&t.isSafari&&g(),setTimeout((function(){v(e)&&m()}),i)})),this.trigger.on("focusin.".concat(As," click.").concat(As),(function(){f()})).on("focusout.".concat(As),(function(t){v(t)&&m()})).on("selected.".concat(As),(function(){t.popupmenuApi.settings.returnFocus=!0})).on("close.".concat(As),(function(t){v(t)&&m()})),(this.isSafari||r)&&(d&&this.element.attr("tabindex",0),this.targetElem.on("keydown.".concat(As),(function(e){9!==(e.which||e.keyCode||e.charCode||0)||e.shiftKey||(d?(t.targetElem.find(":checked, .radio:first").not(":disabled").focus(),t.targetElem.find(".radio").off("keydown.".concat(As)).on("keydown.".concat(As),(function(i){9!==(i.which||i.keyCode||i.charCode||0)||e.shiftKey||setTimeout((function(){t.trigger.focus()}),0)}))):t.trigger.focus(),f(),e.preventDefault(),e.stopPropagation())}))),this.element.on("listopened.".concat(As),(function(){f()})).on("listclosed.".concat(As),(function(){m()})),this},enable:function(){return this.trigger.prop("disabled",!1),this},disable:function(){return this.trigger.prop("disabled",!0),this},unbind:function(){return this.field.off(["click.".concat(As),"mouseover.".concat(As),"mouseout.".concat(As)].join(" ")),this.element.off(["beforeopen.".concat(As),"change.".concat(As),"focusin.".concat(As),"focusout.".concat(As),"listclosed.".concat(As),"listopened.".concat(As)].join(" ")),this.trigger.off(["beforeopen.".concat(As),"click.".concat(As),"focusin.".concat(As),"focusout.".concat(As),"selected.".concat(As),"close.".concat(As)].join(" ")),this.targetElem.off(["click.".concat(As),"keydown.".concat(As)].join(" ")),$("body").off(["resize.".concat(As)].join(" ")),$(document).off(["click.".concat(As)].join(" ")),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Ss)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],As)}},$.fn.fieldoptions=function(t){return this.each((function(){var e=$.data(this,As);e?e.updated(t):e=$.data(this,As,new $s(this,t))}))};var Es="fileupload",Ds={};function Is(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Ds),this.init()}Is.prototype={init:function(){this.build()},build:function(){var t=this,e=this,i=this.element,n=!i.is("input.fileupload");this.fileInput=n?i.find("input"):i,i.closest(".field, .field-short").addClass("field-fileupload");var s=i.find("input").attr("name");n||(s=i.attr("id")||i.attr("name"));var a=n?i.find("input").attr("class"):i.attr("class");a=a?" ".concat(a):"";var o=H.translate("FileUpload");this.shadowLabel=$('<label for="'.concat(s,'-filename">').concat(i.text(),' <span class="audible">').concat(o,"</span></label>")),this.shadowField=$('<input readonly id="'.concat(s,'-filename" class="fileupload-background-transparent').concat(a,'" type="text">'));var r='<span class="trigger">'.concat($.createIcon("folder"),"</span>"),l='<span class="trigger-close" tabindex="-1">'.concat($.createIcon("close"),"</span>");if(!n){var c=i.prev("label");0===c.length&&(c=i.parent().prev("label")),c.hasClass("required")&&this.shadowLabel.addClass("required"),this.shadowLabel.html("".concat(c.text(),' <span class="audible">').concat(o,"</span>")),c.addClass("audible").add(this.fileInput).attr("tabindex","-1").attr("aria-hidden","true")}0===i.parent().find('input[type="text"]').length&&(i.before(this.shadowLabel,this.shadowField),this.fileInput.after(r,l));var d=this.fileInput.attr("value");d&&d.length>0&&(this.shadowField.val(d),this.fileInput.attr("value","")),this.textInput=this.shadowField,this.svg=i.parent().find(".trigger"),this.svgClose=i.parent().find(".trigger-close"),this.textInput.on("keydown.fileupload",(function(e){var n=!1;13===e.which||32===e.which?(i.parent().find('[type="file"]').trigger("click"),n=!0):8!==e.which||i.prop("readonly")||i.prop("disabled")||(x.browser.isIE11()&&e.preventDefault(),t.clearUploadFile(),n=!0),n&&e.stopPropagation()})),this.svg.on("click.fileupload",(function(e){t.fileInput.trigger("click"),n&&t.fileInput.data("handleEvent".concat([e.type||""]),e.handleObj)})),this.svgClose.on("click.fileupload",(function(i){t.clearUploadFile(),t.svgClose.removeClass("is-visible"),n&&t.fileInput.data("handleEvent +".concat([i.type||""]),i.handleObj),e.textInput.trigger("change")})),this.fileInput.is(":disabled")&&this.textInput.prop("disabled",!0),i.hasClass("required")&&(this.shadowLabel.addClass("required"),i.removeClass("required")),this.fileInput.attr("data-validate")&&(this.textInput.attr("data-validate",this.fileInput.attr("data-validate")),this.textInput.validate(),this.shadowLabel.addClass(this.fileInput.attr("data-validate"))),this.fileInput.attr("readonly")&&(this.textInput.prop("disabled",!1),this.textInput[0].classList.remove("fileupload-background-transparent"),this.fileInput.attr("disabled","disabled")),this.fileInput.on("change.fileupload",(function(){this.files.length>0?(e.textInput.val(this.files[0].name).trigger("change"),e.svgClose.addClass("is-visible")):e.clearing||e.clearUploadFile()})),this.fileInput.on("click.fileupload",(function(e){var i=t.fileInput.data("handleEvent".concat([e.type||""]));i&&i.type===e.type&&"fileupload"===e.handleObj.namespace&&(t.fileInput.data("handleEvent".concat([e.type||""]),null),e.preventDefault())})),this.textInput.on("dragenter.fileupload",(function(){t.fileInput.css("z-index","1")})),this.textInput.on("dragleave.fileupload, dragend.fileupload, drop.fileupload",(function(){setTimeout((function(){t.fileInput.css("z-index","-1")}),1)})),E.addAttributes(i,this,this.settings.attributes),E.addAttributes(this.svg,this,this.settings.attributes,"btn-trigger"),E.addAttributes(this.svgClose,this,this.settings.attributes,"btn-trigger-close")},clearUploadFile:function(){this.clearing=!0,this.fileInput.add(this.textInput).val(""),this.svgClose.removeClass("is-visible"),this.fileInput.triggerHandler("change"),this.clearing=!1},unbind:function(){return this.svg.add(this.svgClose).off("click.fileupload"),this.fileInput.off("change.fileupload"),this.fileInput.prev("label"),this.textInput.off(),this.element.closest(".field-fileupload").removeClass("field-fileupload").find('>label:first, >[type="text"]:first, .trigger, .trigger-close, .icon-dirty, .msg-dirty').remove(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Ds)),this},destroy:function(){this.unbind(),this.shadowField.remove(),this.shadowLabel.remove(),$.removeData(this.element[0],Es)},disable:function(){this.textInput.prop("disabled",!0),this.fileInput.prop("disabled",!0)},enable:function(){this.textInput.prop("disabled",!1).prop("readonly",!1),this.fileInput.removeAttr("disabled")},readonly:function(){this.textInput.prop("readonly",!0),this.fileInput.prop("disabled",!0),this.textInput.prop("disabled",!1),this.textInput.removeClass("fileupload-background-transparent")}},$.fn.fileupload=function(t){return this.each((function(){var e=$.data(this,Es);e?e.updated(t):e=$.data(this,Es,new Is(this,t))}))};var Ls="fileuploadadvanced",Ps={isStandalone:!0,standaloneClass:"standalone",allowedTypes:"*",maxFiles:99999,maxFilesInProcess:99999,maxFileSize:-1,fileName:"myfile",isDisabled:!1,showBrowseButton:!0,send:null,textDropArea:null,textDropAreaWithBrowse:null,textBtnCancel:null,textBtnCloseError:null,textBtnRemove:null,errorAllowedTypes:null,errorMaxFileSize:null,errorMaxFiles:null,errorMaxFilesInProcess:null};function Ms(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Ps),this.init()}Ms.prototype={init:function(){return this.build(),this.handleEvents(),this},build:function(){var t,e=this.settings,i=e.isStandalone?e.standaloneClass:"";if(e.textDropArea=e.textDropArea||H.translate("TextDropArea"),e.textDropAreaWithBrowse=e.textDropAreaWithBrowse||H.translate("TextDropAreaWithBrowse"),e.textBtnCancel=e.textBtnCancel||H.translate("TextBtnCancel"),e.textBtnCloseError=e.textBtnCloseError||H.translate("TextBtnCloseError"),e.textBtnRemove=e.textBtnRemove||H.translate("TextBtnRemove"),e.errorAllowedTypes=e.errorAllowedTypes||"<em>".concat(H.translate("Error"),"</em>: ").concat(H.translate("ErrorAllowedTypes")),e.errorMaxFileSize=e.errorMaxFileSize||"<em>".concat(H.translate("Error"),"</em>: ").concat(H.translate("ErrorMaxFileSize")),e.errorMaxFiles=e.errorMaxFiles||"<em>".concat(H.translate("Error"),"</em>: ").concat(H.translate("ErrorMaxFiles")),e.errorMaxFiles=e.errorMaxFiles.replace("{n}",e.maxFiles),e.errorMaxFilesInProcess=e.errorMaxFilesInProcess||"<em>".concat(H.translate("Error"),"</em>: ").concat(H.translate("ErrorMaxFilesInProcess")),this.element.is(".is-disabled")&&(e.isDisabled=!0),e.isDisabled&&(i+=" is-disabled"),e.showBrowseButton){var n="",s=E.uniqueId(this.element,"fileupload-adv-"),a=e.allowedTypes.split(/[\s|]+/g),o=e.maxFilesInProcess>1?" multiple":"";if(o+=e.isDisabled?" disabled":"",1===a.length)"*"!==a[0]&&(n=".".concat(a[0]));else for(var r=0,l=a.length;r<l;r++)n+=".".concat(a[r]+(r!==l-1?",":""));t=""+'<div class="fileupload-wrapper '.concat(i,'">\n          <div class="container drop-area">\n            ').concat($.createIcon("upload"),'\n            <label class="fileupload-adv-browse-lbl">\n              <span>').concat(e.textDropAreaWithBrowse,'</span>\n              <input type="file" name="').concat(s,'" accept="').concat(n,'"').concat(o," />\n            </label>\n          </div>\n        </div>")}else t=""+'<div class="fileupload-wrapper '.concat(i,'">\n          <div class="container drop-area">\n            ').concat($.createIcon("upload"),"\n            <p>").concat(e.textDropArea,"</p>\n          </div>\n        </div>");f.append(this.element,t,"<div><svg><use><label><span><input>"),this.dropArea=$(".drop-area",this.element),E.addAttributes(this.dropArea,this,this.settings.attributes),E.addAttributes(this.dropArea.find("svg"),this,this.settings.attributes,"icon"),E.addAttributes(this.dropArea.find("label"),this,this.settings.attributes,"label")},handleEvents:function(){var t=this,e=this,i=this.settings;if(this.dropArea.on("dragenter.fileuploadadvanced",(function(t){e.element.triggerHandler("filesdragenter"),t.stopPropagation(),t.preventDefault(),i.isDisabled||$(this).addClass("hover")})).on("dragover.fileuploadadvanced",(function(t){t.stopPropagation(),t.preventDefault()})).on("drop.fileuploadadvanced",(function(t){var n=t.originalEvent.dataTransfer.files;t.preventDefault(),i.isDisabled||(e.element.triggerHandler("filesdroped",[n]),$(this).removeClass("hover is-focus"),e.handleFileUpload(n))})),i.showBrowseButton&&!i.isDisabled){var n=this.dropArea.find(".fileupload-adv-browse-lbl"),s=n.find('input[type="file"]');n.click((function(t){$(t.target).is('.hyperlink, input[type="file"]')||t.preventDefault()})),s.hideFocus(),s.on("hidefocusremove.fileuploadadvanced",(function(e){e.stopPropagation(),t.dropArea.addClass("is-focus")})).on("hidefocusadd.fileuploadadvanced",(function(e){e.stopPropagation(),t.dropArea.removeClass("is-focus")})).on("change.fileuploadadvanced",(function(t){t.stopPropagation(),e.handleFileUpload(this.files),this.value=""}))}$(document).on("dragenter.fileuploadadvanced dragover.fileuploadadvanced drop.fileuploadadvanced",(function(t){t.stopPropagation(),t.preventDefault(),"dragover"===t.type&&e.dropArea.removeClass("hover")}))},handleFileUpload:function(t){if(null!=t&&(null==t?void 0:t.length)){var e=this.settings;$("span.msg",this.element).closest(".error").remove(),this.totalCompleted=this.totalCompleted||0,this.totalFileList=this.totalFileList||[],this.counter=this.counter||0;var i=$(".progress",this.element).length;if(this.totalCompleted+t.length+i>e.maxFiles)this.showError(e.errorMaxFiles);else if(i>=e.maxFilesInProcess)this.showError(e.errorMaxFilesInProcess);else{for(var n=e.fileName.replace("[]",""),s=0,a=t.length;s<a;s++)if(this.isFileTypeAllowed(t[s].name))if(-1!==e.maxFileSize&&t[s].size>e.maxFileSize)this.showError(e.errorMaxFileSize,t[s]);else{this.element.triggerHandler("beforecreatestatus",[t[s]]);var o=new FormData;o.append("".concat(n,"[]"),t[s]);var r=this.createStatus(t[s]);r.container.find(".status-icon .action").focus(),this.totalFileList.push(r),this.element.triggerHandler("aftercreatestatus",[t[s]]),"function"==typeof e.send?e.send(o,r):this.sendFileToServer(o,r)}else this.showError(e.errorAllowedTypes,t[s]);e.showBrowseButton&&this.dropArea.find('.fileupload-adv-browse-lbl input[type="file"]').val("")}}},createStatus:function(t){var e=this,i=this,n=this.settings,s=$(""+'<div class="container">\n        <div class="file-row">\n          <span class="status-icon">\n            <button type="button" class="btn-icon action">\n              '.concat($.createIcon({icon:"close",classes:["icon-close"]}),"\n              <span>").concat(n.textBtnCancel,'</span>\n            </button>\n          </span>\n          <span class="description">').concat(t.name,'</span>\n          <div class="l-pull-right">\n            <span class="size">').concat(this.formatFileSize(t.size),'<span id="bar"> | </span><span class="percent">0%</span></span>\n          </div>\n        </div>\n        <div class="progress-row">\n          <span class="progress">\n            <span class="progress-bar" data-value="0"></span>\n          </span>\n        </div>\n      </div>')),a=$(".action",s).button(),o=$(".l-pull-right",s),r=$(".progress-bar",s).progress({animationLength:10}),l=$(".percent",s),c=$("#bar",s),d=this.counter;this.dropArea.after(s);var h=function(){if(e.totalFileList){var t=e.totalFileList.findIndex((function(t){return t.index===d}));t>=0&&e.totalFileList.splice(t,1)}};return this.counter++,{file:t,container:s,index:d,setProgress:function(i){e.element.triggerHandler("fileprogress",[{file:t,progress:i}]),l.text(i<=100?"".concat(i,"%"):""),i>100&&c.text(""),r.attr("data-value",i).triggerHandler("updated")},setAbort:function(n){a.on("click.fileuploadadvanced",(function(){e.element.triggerHandler("fileaborted",[t]),n&&"function"==typeof n.abort&&n.abort(),a.off("click.fileuploadadvanced"),s.remove(),i.totalCompleted--,h()}))},setCompleted:function(e){e=e&&"function"==typeof e.remove?e:{remove:function(){}},s.addClass("completed"),a.after($.createIcon("check")),o.append(""+'<button type="button" class="btn-icon action">\n          '.concat($.createIcon({classes:["icon-close"],icon:"close"}),"\n          <span>").concat(n.textBtnRemove,"</span>\n        </button>")),$(".action",o).button().on("click.fileuploadadvanced",(function(){$(this).off("click.fileuploadadvanced"),i.element.triggerHandler("beforefileremove",[t]),s.hasClass("completed")&&i.totalCompleted--,s.remove(),e.remove(),i.element.triggerHandler("fileremoved",[t])})),r.destroy(),a.off("click.fileuploadadvanced"),a.add(r.closest(".progress-row")).remove(),i.totalCompleted++,h(),i.element.triggerHandler("filecompleteuploading",[t])},setFailed:function(e){i.element.triggerHandler("filefailed",[{error:e,file:t}]),h(),s.remove(),i.showError(e,t)}}},sendFileToServer:function(t,e){var i={remove:function(){}},n=0,s=parseFloat(e.file.size),a=new $.fn.timer((function(){e.setCompleted(i)}),s);$(a.event).on("update",(function(t,i){n=Math.ceil(i.counter/s*100),e.setProgress(n)})),e.setAbort({abort:function(){}})},setFailed:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;this.totalFileList&&this.totalFileList.length>0&&this.totalFileList[e].setFailed(t)},showError:function(t,e){var i,n=this,s=this.settings;i=t===s.errorMaxFiles||t===s.errorMaxFilesInProcess?$(""+'<div class="container error">\n          <div class="file-row">\n            <span class="status-icon file-error">\n              '.concat($.createIcon({classes:["icon-error"],icon:"error"}),'\n            </span>\n            <span class="msg">').concat(t,'</span>\n            <div class="l-pull-right">\n              <button type="button" class="btn-icon action">\n                ').concat($.createIcon({classes:["icon-close"],icon:"close"}),"\n              </button>\n            </div>\n          </div>\n        </div>")):$(""+'<div class="container error">\n          <div class="file-row">\n            <span class="status-icon file-error">\n              '.concat($.createIcon({classes:["icon-error"],icon:"error"}),'\n            </span>\n            <span class="description">').concat(e.name,'</span>\n            <div class="l-pull-right file-error">\n              <span class="size">').concat(this.formatFileSize(e.size),'</span>\n              <button type="button" class="btn-icon action">\n                ').concat($.createIcon({classes:["icon-close"],icon:"close"}),'\n              </button>\n            </div>\n          </div>\n          <div class="msg">\n            <p>').concat(t,"</p>\n          </div>\n        </div>")),$(".action",i).button().on("click.fileuploadadvanced",(function(){n.element.triggerHandler("beforeerrormessageremove",[{error:t,file:e}]),i.remove()})),this.dropArea.after(i)},isFileTypeAllowed:function(t){var e=this.settings.allowedTypes.toLowerCase().split(/[\s|]+/g),i=t.split(".").pop().toLowerCase();return!("*"!==this.settings.allowedTypes&&$.inArray(i,e)<0)},formatFileSize:function(t){var e=1e9,i=1e6,n=1e3;return"number"!=typeof t?"":t>=e?"".concat((t/e).toFixed(2)," GB"):t>=i?"".concat((t/i).toFixed(2)," MB"):"".concat((t/n).toFixed(2)," KB")},enable:function(){this.settings.isDisabled=!1,this.element.find(".fileupload-wrapper").removeClass("is-disabled").find('.fileupload-adv-browse-lbl input[type="file"]').removeAttr("disabled")},disable:function(){this.settings.isDisabled=!0,this.element.find(".fileupload-wrapper").addClass("is-disabled").find('.fileupload-adv-browse-lbl input[type="file"]').attr("disabled","disabled")},unbind:function(){this.dropArea.find('.fileupload-adv-browse-lbl input[type="file"]').off("hidefocusremove.fileuploadadvanced hidefocusadd.fileuploadadvanced change.fileuploadadvanced"),this.dropArea.off("dragenter.fileuploadadvanced dragover.fileuploadadvanced drop.fileuploadadvanced"),$(document).off("dragenter.fileuploadadvanced dragover.fileuploadadvanced drop.fileuploadadvanced"),$(".action",this.element).off("click.fileuploadadvanced");var t=$(".fileupload-wrapper",this.element);return t.off().find("*").off(),t.remove(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Ps)),this.unbind().init()},destroy:function(){this.unbind(),$(".fileupload-wrapper",this.element).remove(),$.removeData(this.element[0],Ls)}},$.fn.fileuploadadvanced=function(t){return this.each((function(){var e=$.data(this,Ls);e?e.updated(t):e=$.data(this,Ls,new Ms(this,t))}))};var Bs="homepage",Os={animate:!0,columns:3,editing:!1,easing:"blockslide",gutterSize:16,widgetWidth:360,widgetHeight:368,useSmall:!1,timeout:100};function Rs(t,e){this.settings=E.mergeSettings(t,e,Os),this.element=$(t),this.init()}Rs.prototype={get state(){var t=this.rowsAndCols.length,e=t?this.rowsAndCols[0].length:0,i=this.settings;return-1===this.rowsAndCols[t-1].indexOf(!1)&&(t-=1),void 0===this.editing&&(this.editing=i.editing),{rows:t,cols:e,containerHeight:i.gutterSize+(i.gutterSize+i.widgetHeight)*t,matrix:this.rowsAndCols,blocks:this.blocks,editing:this.editing}},init:function(){var t=this;this.setSmall(),this.setColumns(),this.initHeroWidget(),this.handleEvents(),this.initEdit(),setTimeout((function(){t.resize(t,!1)}),100),this.element.parent().addClass("homepage-background")},setColumns:function(){var t=this.settings.columns,e=parseInt(this.element.attr("data-columns"),10);isNaN(e)||t===e||t!==Os.columns||(t=e),this.columns=t},setSmall:function(){var t=this.settings,e=this.element.attr("data-use-small");if(t.useSmall||e&&"true"===e.toString().toLowerCase()){var i=Os,n={gutterSize:24,widgetWidth:260,widgetHeight:260};t.useSmall=!0,t.gutterSize===i.gutterSize&&(t.gutterSize=n.gutterSize),t.widgetWidth===i.widgetWidth&&(t.widgetWidth=n.widgetWidth),t.widgetHeight===i.widgetHeight&&(t.widgetHeight=n.widgetHeight)}},initColumns:function(t){t=t||0,this.rowsAndCols[t]=[];for(var e=0,i=this.columns;e<i;e++)this.rowsAndCols[t][e]=!0},initHeroWidget:function(){var t=this.element.parent().find(".hero-widget");t.length>1&&(t=t.not(":first").remove()),this.heroWidget=t},initRowsAndCols:function(){this.rowsAndCols=[],this.initColumns()},initEdit:function(){var t=this,e=this,i=e.element.find(".card, .widget, .small-widget");void 0===e.editing&&(e.editing=e.settings.editing),e.editing?(i.attr("draggable",!0),i.css("cursor","move"),e.guide=$("<div>").addClass("drop-indicator").append("\n      <div class='edge'></div>\n      <div class='line'></div>\n      <div class='edge'></div>\n      "),i.each((function(i,n){var s=$(n),a=$("<button>").addClass("card-remove").append('\n        <span class="audible">Remove Widget</span>\n          <svg icon="close" soho-icon="" class="icon" aria-hidden="true" focusable="false" role="presentation">\n            <use xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="#icon-close"></use>\n          </svg>\n        ');if(!s.hasClass("no-remove")){var o=s.children(".widget-header");a.insertBefore(o).on("click.card-remove",(function(){var i=function(){var t=s.clone(!0);s.remove(),e.refresh(!1),setTimeout((function(){e.element.triggerHandler("removecard",[s,e.state]),t.triggerHandler("removecard",[s,e.state]),t.remove()}),0)};if(t.settings&&"function"==typeof t.settings.onBeforeRemoveCard){var n=t.settings.onBeforeRemoveCard(s);n&&n.then&&"function"==typeof n.then?n.then((function(){return i()})):n&&i()}else i()}))}})),i.on("mouseenter.card",(function(){var t=$(this),i=$("<div>").addClass("ui-resizable-handle ui-resizable-e").drag({axis:"x"}).on("dragstart.handle",(function(n){n.stopPropagation(),n.preventDefault(),t.addClass("ui-resize-passive"),t.css({opacity:.9,zIndex:90}),$(window).on("mousemove.handle",(function(n){var s=n.clientX-t.offset().left;s<e.settings.widgetWidth/2?i.css({left:e.settings.widgetWidth/2}):t.width(s)})).on("mouseup.handle",(function(){t.removeClass("ui-resize-passive"),t.css({zIndex:"auto"}),$(window).off("mousemove.handle").off("mouseup.handle"),t.removeClass("double-width triple-width quad-width quintuple-width");var i=t.width()/e.settings.widgetWidth;i>4.5?t.addClass("quintuple-width"):i>3.5?t.addClass("quad-width"):i>2.5?t.addClass("triple-width"):i>1.5&&t.addClass("double-width"),$(".ui-resizable-handle").remove(),t.css({opacity:1,width:""}),e.refresh(!1),setTimeout((function(){t.triggerHandler("resizecard",[t,e.state]),e.element.triggerHandler("resizecard",[t,e.state])}),0)}))})),n=$("<div>").addClass("ui-resizable-handle ui-resizable-s").drag({axis:"y"}).on("dragstart.handle",(function(i){i.stopPropagation(),i.preventDefault(),t.addClass("ui-resize-passive"),t.css({opacity:.9,zIndex:90}),$(window).on("mousemove.handle",(function(i){var s=i.clientY-t.offset().top;s<e.settings.widgetHeight?n.css({top:e.settings.widgetHeight}):t.height(s)})).on("mouseup.handle",(function(){t.removeClass("ui-resize-passive"),t.css({zIndex:"auto"}),$(window).off("mousemove.handle").off("mouseup.handle"),t.removeClass("double-height"),t.height()/e.settings.widgetHeight>1.5&&t.addClass("double-height"),$(".ui-resizable-handle").remove(),t.css({opacity:1,height:""}),e.refresh(!1),setTimeout((function(){t.triggerHandler("resizecard",[t,e.state]),e.element.triggerHandler("resizecard",[t,e.state])}),0)}))}));0===t.has(".ui-resizable-handle").length&&t.append(i,n),t.addClass("editable-hover-border")})).on("mouseleave.card",(function(){var t=$(this);t.hasClass("ui-resize-passive")||$(".ui-resizable-handle").remove(),t.removeClass("editable-hover-border")})).on("dragstart.card",(function(){var t=$(this);t.addClass("is-dragging"),t.css("opacity","0.99")})).on("dragover.card",(function(t){t.preventDefault()})).on("dragenter.card",(function(t){t.preventDefault();var i=$(this),n=$(".is-dragging");i.is(n)&&0===$(".drop-indicator").length||(n.index()<i.index()?(e.guide.css("right","-14px"),e.guide.css("left","")):(e.guide.css("left","-14px"),e.guide.css("right","")),i.append(e.guide),e.refresh(!1))})).on("dragend.card",(function(){var t=$(this);if(t.hasClass("is-dragging")){var n=$(i).has(".drop-indicator");t.css("opacity",""),t.index()<n.index()?t.insertAfter(n):t.insertBefore(n),t.removeClass("is-dragging"),e.guide.remove(),e.refresh(!1),setTimeout((function(){t.triggerHandler("reordercard",[t,e.state]),e.element.triggerHandler("reordercard",[t,e.state])}),0)}}))):(i.removeAttr("draggable"),i.children(".card-remove").remove(),i.off("mouseenter.card mouseleave.card dragstart.card dragenter.card dragend.card"),i.not(".card-list .card").css("cursor","auto"))},setEdit:function(t){void 0!==t&&(this.editing=t,this.initEdit(),this.refresh(!1))},getAvailability:function(t){for(var e=!1,i={},n=this.rowsAndCols.length,s=0,a=n;s<a&&!e;s++)for(var o=0,r=!0,l=this.rowsAndCols[s].length;o<l&&!e;o++)if(this.rowsAndCols[s][o]&&t.w+o<=l){if(t.w>1&&l>o+1)for(var c=0;c<t.w;c++)if(!this.rowsAndCols[s][o+c]){r=!1;break}if(t.h>1&&n>s+1)for(var d=0;d<t.h;d++)if(!this.rowsAndCols[s+d][o]){r=!1;break}r&&(i.row=s,i.col=o,e=!0)}return Object.getOwnPropertyNames(i).length||(this.initColumns(n),i.row=n,i.col=0),i},fitBlock:function(t,e,i){var n=!0;if(i.x=e,i.y=t,1===i.w&&1===i.h)this.rowsAndCols[t][e]=!1;else if(1!==i.w)for(var s=t,a=i.h+t;s<a;s++)for(var o=e,r=i.w+e;o<r;o++)this.rowsAndCols[s]||this.initColumns(s),this.rowsAndCols[s][o]=!1;else for(var l=t,c=i.h+t;l<c;l++)for(var d=e,h=i.h+e;d<h;d++)this.rowsAndCols[l]||this.initColumns(l),this.rowsAndCols[l][e]=!1;for(var u=0,p=this.rowsAndCols[t].length;u<p;u++)this.rowsAndCols[t][u]&&(n=!1);n&&(this.rowsAndCols[t+1]||this.initColumns(t+1))},setBlocks:function(){var t=this.element.find(".card, .widget, .small-widget").not(".card-list .card");this.blocks=[];for(var e=0,i=t.length;e<i;e++){var n=$(t[e]),s=n.hasClass("double-height")?2:1,a=void 0;if(n.hasClass("auto-height")){var o=this.settings.widgetHeight+this.settings.gutterSize;n.height()>o*s&&(s=Math.ceil(n.height()/o))}a=n.hasClass("sextuple-width")?6:n.hasClass("quintuple-width")?5:n.hasClass("quad-width")?4:n.hasClass("triple-width")?3:n.hasClass("double-width")?2:1,this.blocks.push({w:a,h:s,elem:n,text:n.text()})}if(this.columns>1)for(var r=0,l=0,c=0,d=this.blocks.length;r<d;r++)this.blocks[r].w>=this.columns&&r&&c&&c<=this.columns/2&&this.arrayIndexMove(this.blocks,r,l),(c+=this.blocks[r].w)>=this.columns&&(c=0,l=this.blocks[l].w>=this.columns?l+1:r)},arrayIndexMove:function(t,e,i){t.splice(i,0,t.splice(e,1)[0])},resize:function(t,e){var i,n=6*t.settings.widgetWidth+5*t.settings.gutterSize,s=5*t.settings.widgetWidth+4*t.settings.gutterSize,a=4*t.settings.widgetWidth+3*t.settings.gutterSize,o=3*t.settings.widgetWidth+2*t.settings.gutterSize,r=2*t.settings.widgetWidth+t.settings.gutterSize,l=t.settings.widgetWidth,c=n,d=t.element.outerWidth(),h=d>=n,u=d>=s&&d<=n,p=d>=a&&d<=s,g=d>=o&&d<=a,f=d>=r&&d<=o,m=d<=r,v=t.element.find("> .content");this.setColumns(),(h||6===t.columns)&&(i=6,c=n),(u||5===t.columns)&&(i=5,c=s),(p||4===t.columns)&&(i=4,c=a),(g||3===t.columns)&&(i=3,c=o),(f||2===t.columns)&&(i=2,c=r),(m||1===t.columns)&&(i=1,c=l),t.columns=i||this.settings.columns,v.length&&(v[0].style.marginLeft="-".concat(c/2,"px")),this.setBlocks(),this.initRowsAndCols();for(var b=0,y=t.blocks.length;b<y;b++){var w=t.blocks[b];w.elem.removeClass("to-single to-double to-triple to-quad to-quintuple"),w.w>t.columns&&(w.w=t.columns,1===t.columns?w.elem.addClass("to-single"):2===t.columns?w.elem.addClass("to-double"):3===t.columns?w.elem.addClass("to-triple"):4===t.columns?w.elem.addClass("to-quad"):5===t.columns&&w.elem.addClass("to-quintuple"));var x=t.getAvailability(w),C=t.settings.widgetWidth+t.settings.gutterSize,k=C*t.columns,T={left:H.isRTL()?k-(C*w.w+C*x.col):C*x.col,top:(t.settings.widgetHeight+t.settings.gutterSize)*x.row};if(e&&!this.editing){var A=t.settings.easing;"blockslide"===A?(t.applyCubicBezier(w.elem,[.09,.11,.24,.91]),w.elem[0].style.left="".concat(T.left,"px"),w.elem[0].style.top="".concat(T.top,"px")):w.elem.animate(T,t.settings.timeout,A)}else t.applyCubicBezier(w.elem,null),w.elem[0].style.left="".concat(T.left,"px"),w.elem[0].style.top="".concat(T.top,"px");t.fitBlock(x.row,x.col,w)}t.element.triggerHandler("resize",[t.columns,t.state])},applyCubicBezier:function(t,e){var i=e?"all .3s cubic-bezier(".concat(e,")"):"none";t[0].style.transition=i},refresh:function(t){var e=this;setTimeout((function(){t=void 0!==t?t:e.settings.animate,e.resize(e,t)}),0)},detachEvents:function(){$("body").off("resize.homepage"),$(".application-menu").off("applicationmenuopen.homepage applicationmenuclose.homepage")},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Os)),this.detachEvents().init()},destroy:function(){this.detachEvents(),$.removeData(this.element[0],Bs)},handleEvents:function(){var t=this;$("body").on("resize.homepage",(function(){t.resize(t,t.settings.animate)})),$(".application-menu").on("applicationmenuopen.homepage applicationmenuclose.homepage",(function(){t.resize(t,t.settings.animate)}))}},$.fn.homepage=function(t){return this.each((function(){var e=$.data(this,Bs);e?e.updated(t):e=$.data(this,Bs,new Rs(this,t))}))};var zs="link";function Fs(t){this.element=$(t),this.init()}Fs.prototype={init:function(){this.build()},build:function(){var t=this.element.find(".text .title");if(t[0].offsetHeight<t[0].scrollHeight||t[0].offsetWidth<t[0].scrollWidth){var e={title:t.text(),maxWidth:this.element.hasClass("dual")?350:171};t.tooltip(e)}return this}},$.fn.link=function(){return this.each((function(){var t=$.data(this,zs);t||(t=$.data(this,zs,new Fs(this)))}))};var Ns="pager",Hs=[".pager-first > .btn-icon",".pager-prev > .btn-icon",".pager-next > .btn-icon",".pager-last > .btn-icon",".pager-no > .btn-icon",".pager-count input",".pager-pagesize button"].join(", "),qs={componentAPI:void 0,type:"list",position:"bottom",activePage:1,hideOnOnePage:!1,source:null,pagesize:15,pagesizes:[15,25,50,75],footerContainment:!0,footerContainmentClass:null,pageSelectorInputText:null,showPageSelectorInput:!1,showPageSizeSelector:!0,smallPageSizeSelector:!1,pageSizeSelectorText:"RecordsPerPage",pageSizeSelectorTextNoToken:"RecordsPerPageNoToken",onPageSizeChange:null,showFirstButton:!0,enableFirstButton:!0,showPreviousButton:!0,enablePreviousButton:!0,showNextButton:!0,enableNextButton:!0,showLastButton:!0,enableLastButton:!0,indeterminate:!1,onFirstPage:null,onPreviousPage:null,onNextPage:null,onLastPage:null,firstPageTooltip:"FirstPage",previousPageTooltip:"PreviousPage",nextPageTooltip:"NextPage",lastPageTooltip:"LastPage",pageSizeMenuSettings:{attachToBody:!1},attributes:null,tabbable:!0};function Vs(t,e){var i;this.settings=E.mergeSettings(t,e,qs),this.settings.dataset=e.dataset,this.settings.pagesizes=(null===(i=e.pagesizes)||void 0===i||null==i?void 0:i.length)>0?e.pagesizes:this.settings.pagesizes,this.element=$(t),this.init()}Vs.prototype={get state(){var t,e,i,n=0,s=1,a=this.settings.indeterminate,o=999999999,r=this.settings.pagesize,l=this.previousOperation,c=this.previousTrigger,d=this.settings.dataset;if(isNaN(this.serverDatasetTotal)){if(d&&d.length){if(n=d.length,this.isFilteredClientside)n=d.filter((function(t){return!t._isFilteredOut})).length}}else n=this.serverDatasetTotal;return this.grandTotal&&(i=this.grandTotal),s=a?o:n<1?1:Math.ceil(n/r),isNaN(this.filteredTotal)||(e=this.filteredTotal,a?s=o:t=e<1?1:Math.ceil(e/r)),{activePage:this.activePage,indeterminate:this.settings.indeterminate,grandTotal:i,pagesize:r,pages:s,trigger:c,total:n,type:l,filteredActivePage:this.filteredActivePage,filteredTotal:e,filteredPages:t}},get pagingInfo(){return V("state","pagingInfo"),this.state},get focusableElements(){return E.getArrayFromList(this.pagerBar[0].querySelectorAll(Hs))},get numberButtons(){if(!this.isTable){var t=this.pagerBar[0].querySelectorAll("li:not(.pager-prev):not(.pager-next):not(.pager-first):not(.pager-last):not(.pager-pagesize)");return E.getArrayFromList(t)}},get pageSelectorInput(){if(!this.isListView)return this.pagerBar[0].querySelector(".pager-count input")},get pageSizeSelectorButton(){if(this.settings.showPageSizeSelector)return this.pagerBar[0].querySelector(".pager-pagesize button")},get showSmallPageSizeSelector(){return!!this.settings.showPageSizeSelector&&(!0===this.settings.smallPageSizeSelector||this.isListView&&this.element.parents(".list-detail").length)},init:function(){var t=this;this.setup(),this.createPagerBar(),this.setActivePage(this.settings.activePage,!0),setTimeout((function(){t.triggerPagingEvents(t.settings.activePage)})),this.handleEvents()},setup:function(){if(this.element.attr("data-pagesize")){var t=Number(this.element.attr("data-pagesize"));isNaN(t)||(this.settings.pagesize=t),this.element.removeAttr("data-pagesize")}if(-1===this.settings.pagesizes.indexOf(this.settings.pagesize)){this.settings.pagesizes.push(this.settings.pagesize),this.settings.pagesizes=this.settings.pagesizes.sort((function(t,e){return t-e}))}this.handleDeprecatedSettings();var e=this.element.parents(".card, .widget");return this.element.is("tbody")&&(this.isTable=!0,this.settings.type="table",this.mainContainer=this.element.closest(".datagrid-container"),this.settings.componentAPI||(this.settings.componentAPI=this.mainContainer.data("datagrid")),e.length&&e[0].classList.add("has-datagrid")),this.element.is(".listview")&&(this.isTable=!1,this.isListView=!0,this.mainContainer=this.element,this.settings.componentAPI||(this.settings.componentAPI=this.element.data("listview"))),this.element.attr({role:"region","aria-label":H.translate("Pagination")}),this},createPagerBar:function(){var t=this;if(!this.pagerBar&&(this.pagerBar=this.element.prev(".pager-toolbar"),0===this.pagerBar.length&&(this.pagerBar=$('<ul class="pager-toolbar"></ul>')),"standalone"===this.settings.type?(this.pagerBar.addClass("is-standalone"),this.isListView?this.mainContainer.after(this.pagerBar):this.element.append(this.pagerBar)):this.isTable?this.mainContainer.after(this.pagerBar):"bottom"===this.settings.position?this.element.after(this.pagerBar):this.element.before(this.pagerBar),this.isListView&&this.pagerBar.addClass("is-listview"),this.element.closest(".card, .widget").length&&this.settings.footerContainment)){var e=this;["widget","card"].forEach((function(i){var n=e.element.closest(".".concat(i,"-content"));if(n.length){var s=n.next(".".concat(i,"-footer"));s.length||(s=$('<div class="'.concat(i,'-footer"></div>')).insertAfter(n)),t.settings.footerContainmentClass&&s.addClass(t.settings.footerContainmentClass.toString()),e.pagerBar.appendTo(s)}}))}},handleDeprecatedSettings:function(){void 0!==this.settings.attachPageSizeMenuToBody&&(V("pageSizeMenuSettings.attachToBody (setting)","attachPageSizeMenuToBody (setting)"),this.settings.pageSizeMenuSettings.attachToBody=this.settings.attachPageSizeMenuToBody,delete this.settings.attachPageSizeMenuToBody)},handleEvents:function(){var t=this;this.pagerBar.on("click.pager",".btn-icon",(function(e){var i=e.currentTarget,n=i.parentNode,s=document.querySelector(".datagrid-cell-wrapper .dropdown"),a=document.querySelector(".datagrid-dropdown-list");if(e.preventDefault(),$(this).attr("disabled"))return!1;if(a){var o=s.parentNode.parentNode;a.remove(),o.classList.remove("is-editing")}var r=t.state,l=r.activePage;if(r.filteredTotal&&(l=r.filteredActivePage),f.hasClass(n,"pager-prev"))return t.setActivePage(l-1,!1,"prev"),t.triggerPagingEvents(l),!1;if(f.hasClass(n,"pager-next"))return t.setActivePage((-1===l?1:l)+1,!1,"next"),t.triggerPagingEvents(l),!1;if(f.hasClass(n,"pager-first"))return t.setActivePage(1,!1,"first"),t.triggerPagingEvents(l),!1;if(f.hasClass(n,"pager-last"))return t.setActivePage(t.pageCount(),!1,"last"),t.triggerPagingEvents(l),!1;var c=Number(i.getAttribute("data-page"));return(isNaN(c)||c<1)&&(c=1),t.setActivePage(c,!1,"page"),t.triggerPagingEvents(l),!1})),t.pagerBar.on("keydown.pager",$(t.focusableElements),(function(e){if($(".popupmenu.is-open").length>0||t.settings.tabbable)return!0;var i=(e=e||window.event).which||e.keyCode||e.charCode||!1,n=37===i||40===i,s=39===i||38===i,a=e.target;if("INPUT"===a.nodeName&&(n=40===i,s=38===i),!n&&!s)return!0;var o="INPUT"===a.nodeName?a.parentNode.parentNode:a.parentNode,r=o.nextSibling,l=o.previousSibling,c=null===l,d=null===r;if(e.preventDefault(),n&&!c){var h=l.querySelector("button, input");return h.removeAttribute("tabindex"),a.setAttribute("tabindex","-1"),h.focus(),!1}if(n&&c){var u=o.parentNode.lastChild.querySelector("button, input");return u.removeAttribute("tabindex"),a.setAttribute("tabindex","-1"),u.focus(),!1}if(s&&!d){var p=r.querySelector("button, input");return p.removeAttribute("tabindex"),a.setAttribute("tabindex","-1"),p.focus(),!1}if(s&&d){var g=o.parentNode.firstChild.querySelector("button, input");g.removeAttribute("tabindex"),a.setAttribute("tabindex","-1"),g.focus()}return!1}))},reset:function(t,e){this.activePage=this.settings.activePage,this.previousOperation=t,this.previousTrigger=e,this.teardown(),this.init()},showPageSizeSelector:function(t){t="true"==="".concat(t).toLowerCase(),this.settings.showPageSizeSelector=t,t&&(this.isShowPageSizeSelectorCall=t)},showButton:function(t,e){if(-1!==["first","last","next","previous"].indexOf(t)){var i=".pager-".concat(t," .btn-icon");"previous"===t&&(i=".pager-prev .btn-icon");var n=this.pagerBar[0].querySelector(i);e="true"==="".concat(e).toLowerCase(),this.settings["show".concat(M.capitalize(t),"Button")]=e,n&&(e?n.parentNode.classList.remove("hidden"):n.parentNode.classList.add("hidden"))}},enableButton:function(t,e){if(-1!==["first","last","next","previous"].indexOf(t)){var i=".pager-".concat(t," .btn-icon");"previous"===t&&(i=".pager-prev .btn-icon");var n=this.pagerBar[0].querySelector(i);if(e="true"==="".concat(e).toLowerCase(),this.settings["enable".concat(M.capitalize(t),"Button")]=e,n){var s=$(n),a=this.settings["".concat(t,"PageTooltip")];if(e){n.disabled=!1,n.parentNode.classList.remove("is-disabled"),s.removeAttr("disabled aria-disabled");var o=s.data("tooltip");o&&!o.content&&(o.content=a)}else{if(n.disabled=!0,n.parentNode.classList.add("is-disabled"),s.attr({disabled:"disabled","aria-disabled":"true"}),!s.find(".disabled-tooltip").length)s.data("tooltip").destroy(),s.append('<div class="disabled-tooltip" title="'.concat(a,'"></div>')),s.find(".disabled-tooltip").tooltip()}}}},setActivePage:function(t,e,i){var n="number"==typeof t.searchActivePage||"number"==typeof t.filteredActivePage||"number"==typeof this.filteredActivePage,s=this.filteredActivePage||this.activePage||this.settings.activePage;return"object"===l(t)?s=n?t.filteredActivePage||t.searchActivePage:t.activePage?t.activePage:1:isNaN(t)||(s=t),n?this.filteredActivePage=s:this.activePage=s,this.previousOperation=i,this.render(),void 0===s||0===s||isNaN(s)||s===this.activePage&&!e?this.activePage:(this.settings.componentAPI&&this.settings.componentAPI.saveUserSettings&&this.settings.componentAPI.saveUserSettings(),s)},adjustPageCount:function(t){var e=this.state,i="number"==typeof e.filteredActivePage||"number"==typeof this.filteredActivePage,n=e.pages;return i&&e.filteredPages&&(n=e.filteredPages),(t<1||t>n)&&(t=this.filteredActivePage||this.activePage),t},pageCount:function(t){var e=this.isShowPageSizeSelectorCall;return delete this.isShowPageSizeSelectorCall,void 0===t&&this.settings.indeterminate&&(this._pageCount=this.settings.pagesize),void 0!==t||this.settings.source||e?(void 0!==t&&(this._pageCount=t),this._pageCount=this._pageCount||1,this.settings.indeterminate?999999999:this._pageCount):this._pageCount},renderButtons:function(){var t,e=this.activePage,i=this.state.pages,n="",s=!1,a=!1,o=!1,r=!1,l=!this.settings.enableFirstButton,c=!this.settings.enablePreviousButton,d=!this.settings.enableNextButton,h=!this.settings.enableLastButton,u=this.settings.indeterminate&&!0===this.firstPage,p=this.settings.indeterminate&&!0===this.lastPage,g=this.settings.dataset&&this.settings.dataset.length;(this.state.filteredPages&&(e=this.state.filteredActivePage,i=this.state.filteredPages),this.pagerBar)&&(null===(t=this.pagerBar)||void 0===t||null==t||t.children("li").find("> .btn-icon, > .btn-icon[disabled] .disabled-tooltip").each((function(t,e){var i=$(e);i.data("tooltip")&&i.data("tooltip").destroy()})));this.settings.showFirstButton&&((u||!this.settings.indeterminate&&"standalone"!==this.settings.type&&1===e)&&(l=!0),s=!0),this.settings.showPreviousButton&&((u||!this.settings.indeterminate&&"standalone"!==this.settings.type&&1===e)&&(c=!0),a=!0),this.settings.showNextButton&&((p||!this.settings.indeterminate&&"standalone"!==this.settings.type&&e===i)&&(d=!0),o=!0),this.settings.showLastButton&&((p||!this.settings.indeterminate&&"standalone"!==this.settings.type&&e===i)&&(h=!0),r=!0);var f=[],m=this.settings.showPageSizeSelector?190:0,v=(this.pagerBar.width()-m)/40,b=Math.floor(v);if((a||o)&&(b-=1),(r||s)&&(b-=1),!this.isTable&&"standalone"!==this.settings.type&&g&&b>=i&&(l=!0,h=!0),!this.settings.indeterminate){for(var y=Math.floor(b/2),w=e-y,x=e+y;w<1;)++w,x<i&&++x;for(;x>i;)w>1&&--w,--x;for(var C=w;C<x+1;C++)f.push(C)}function k(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"",e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",i=arguments.length>2?arguments[2]:void 0,n=arguments.length>3?arguments[3]:void 0,s=arguments.length>4&&void 0!==arguments[4]?arguments[4]:"",a=arguments.length>5&&void 0!==arguments[5]&&arguments[5],o=arguments.length>6&&void 0!==arguments[6]&&arguments[6],r=arguments.length>7&&void 0!==arguments[7]&&arguments[7],l="",c="",d="",h="",u="";return n&&(u=' data-page="'.concat(n,'"')),i&&(h=' title="'.concat(i,'"')),r&&(s+=" hidden"),a&&(s+=" selected",l=' aria-disabled="true"'),o&&(c=" disabled",l=' aria-disabled="true"',d='<div class="disabled-tooltip"'.concat(h,"></div>"),n||(h="")),'<li class="'.concat(s,'">\n        <button type="button" class="btn-icon"').concat(h).concat(u).concat(l).concat(c,">\n          ").concat(d,"<span>").concat(e,"</span>\n          ").concat(t,"\n        </button>\n      </li>")}if($(this.numberButtons).remove(),s&&(n+=k($.createIcon({icon:"first-page"}),H.translate("FirstPage"),this.settings.firstPageTooltip,null,"pager-first",!1,l,!1)),a&&(n+=k($.createIcon({icon:"previous-page"}),H.translate("PreviousPage"),this.settings.previousPageTooltip,null,"pager-prev",!1,c,!1)),!this.isTable&&g&&!this.settings.showPageSelectorInput){var T="";f.forEach((function(t){T+=t===(e||1)?k(t,H.translate("PageOn"),null,t,"pager-no",!0,!1,!1):k(t,H.translate("Page"),null,t,"pager-no",!1,!1,!1)})),n+=T}o&&(n+=k($.createIcon({icon:"next-page"}),H.translate("NextPage"),this.settings.nextPageTooltip,null,"pager-next",!1,d,!1)),r&&(n+=k($.createIcon({icon:"last-page"}),H.translate("LastPage"),this.settings.lastPageTooltip,null,"pager-last",!1,h,!1)),this.pagerBar[0].innerHTML=n,r||s||this.settings.showPageSizeSelector?this.pagerBar[0].classList.remove("two-button"):this.pagerBar[0].classList.add("two-button"),this.pagerBar.children("li").find("> .btn-icon").button().tooltip(),this.pagerBar.children("li").find("> .btn-icon[disabled] .disabled-tooltip").tooltip()},renderPageSelectorInput:function(){if(this.isTable&&!this.settings.indeterminate||this.settings.showPageSelectorInput){var t=this.activePage,e=this.state.pages||1;if(this.state.filteredPages&&(t=this.state.filteredActivePage,e=this.state.filteredPages),this.pageSelectorInput)e>1&&this.pagerBar.find(".pager-total-pages").text(e),this.pagerBar.find(".pager-count input").val(t);else{var i=this.settings.pageSelectorInputText||H.translate("PageOf");i=(i=i.replace("{0}",'<input class="new-mask" name="pager-pageno" value="'.concat(t,'" autocomplete="off">'))).replace("{1}",'<span class="pager-total-pages">'.concat(e,"</span>")),$('<li class="pager-count"><label>'.concat(i," </label>")).insertAfter(this.pagerBar.find(".pager-prev"))}var n=null,s="".concat(e).replace(/\d/g,"#"),a=this,o={pattern:s,mode:"number",processOnInitialize:!1};$(this.pageSelectorInput).mask(o).on("focus",(function(){n=parseInt($(this).val(),10)})).on("blur",(function(){r($(this)),n=null})).on("keydown",(function(t){return 13!==t.which||(r($(this)),t.stopPropagation(),t.preventDefault(),!1)}))}function r(t){var e=a.adjustPageCount(parseInt(t.val(),10));if(n!==e){var i=a.activePage;a.state.filteredPages&&(i=a.state.filteredActivePage),t.val(a.setActivePage(e,!1,"page")),a.triggerPagingEvents(i)}else t.val(n)}},renderPageSizeSelectorButton:function(){var t=this;if(this.settings.showPageSizeSelector&&!(this.settings.pagesizes.length<2)){if(!this.pageSizeSelectorButton){var e=$('<li class="pager-pagesize"></li>'),i=$.createIcon({icon:"dropdown"}),n=H.translate(this.settings.pageSizeSelectorText).replace("{0}",this.settings.pagesize),s="",a="<span>".concat(n,"</span>");this.showSmallPageSizeSelector&&(s=' class="audible"',n=H.translate(this.settings.pageSizeSelectorTextNoToken),a='<span class="record-count">'.concat(this.settings.pagesize,"</span>\n        <span").concat(s,">").concat(n,"</span>"));var o=$('<button type="button" class="btn-menu">\n        '.concat(a,"\n        ").concat(i,"\n      </button>")).appendTo(e);e.appendTo(this.pagerBar);var r="";this.showSmallPageSizeSelector&&(r='<li class="heading">'.concat(n,"</li>"));for(var l=0;l<this.settings.pagesizes.length;l++){var c=this.settings.pagesizes[l];r+='<li class="'.concat(c===this.settings.pagesize?" is-checked":"",'"><a href="#">').concat(c,"</a></li>")}var d=$('<ul class="popupmenu is-selectable">'.concat(r,"</ul>"));o.after(d)}var h=$(this.pageSizeSelectorButton),u=E.extend({},{placementOpts:{parent:h,parentXAlignment:H.isRTL()?"left":"right",strategies:["flip"]}},this.settings.pageSizeMenuSettings);if(h.popupmenu(u),h.on("selected.pager",(function(e,i){t.changePageSize(i)})),this.settings.tabbable){var p=h.data("popupmenu");this.pagerBar.off("keydown.".concat(Ns),".pager-pagesize button").on("keydown.".concat(Ns),".pager-pagesize button",(function(t){40===(t.which||t.keyCode||t.charCode)&&p&&!p.isOpen&&(p.open(),t.preventDefault(),t.stopPropagation())}))}}},render:function(){var t=this.state,e=t.pages;this.preserveFocus=this.preserveFocus||!1;var i=$(":focus").closest(".pager-pagesize").length>0;!this.preserveFocus&&i&&(this.preserveFocus=!0),t.filteredPages&&(e=t.filteredPages),this.pageCount(e),this.pageSizeSelectorButton&&this.teardownPageSizeSelector(),this.renderButtons(),this.renderPageSelectorInput(),this.renderPageSizeSelectorButton(),this.renderBar(),this.preserveFocus&&this.element.find(".pager-pagesize button").focus()},renderBar:function(t){var e,i,n=this,s=(t=t?E.extend({},this.state,t):this.state).activePage;t.filteredActivePage&&(s=t.filteredActivePage),this.settings.showFirstButton||this.showButton("first",!1),this.settings.enableFirstButton||this.enableButton("first",!1),this.settings.showPreviousButton||this.showButton("previous",!1),this.settings.enablePreviousButton||this.enableButton("previous",!1),this.settings.showNextButton||this.showButton("next",!1),this.settings.enableNextButton||this.enableButton("next",!1),this.settings.showLastButton||this.showButton("last",!1),this.settings.enableLastButton||this.enableButton("last",!1),this.settings.showPageSizeSelector&&this.showPageSizeSelector(!0),void 0!==t.firstPage&&(!1===t.firstPage&&(this.enableButton("first",!0),this.enableButton("previous",!0)),!0!==t.firstPage&&1!==s||(this.enableButton("first",!1),this.enableButton("previous",!1))),void 0!==t.lastPage&&(!1===t.lastPage&&(this.enableButton("next",!0),this.enableButton("last",!0)),!0!==t.lastPage&&s!==this.pageCount()||(this.enableButton("next",!1),this.enableButton("last",!1)));var a=this.pagerBar[0]?this.pagerBar[0].classList:null,o=[".pager-first",".pager-prev",".pager-next",".pager-last",".pager-count"],r=function(){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];(arguments.length>0&&void 0!==arguments[0]?arguments[0]:[]).forEach((function(e){t?n.pagerBar.find(e).addClass("hidden"):n.pagerBar.find(e).removeClass("hidden")}))};this.settings.hideOnOnePage&&t.total<=t.pagesize?r(o):this.hidePagerBar(t)&&a?a.add("hidden"):this.settings.hideOnOnePage&&a&&a.contains("hidden")&&(a.remove("hidden"),r(o,!1)),this.initTabIndexes(),E.addAttributes(this.pagerBar,this,this.settings.attributes),E.addAttributes(this.pagerBar.find(".pager-first button"),this,this.settings.attributes,"btn-first"),E.addAttributes(this.pagerBar.find(".pager-prev button"),this,this.settings.attributes,"btn-prev"),E.addAttributes(this.pagerBar.find(".pager-next button"),this,this.settings.attributes,"btn-next"),E.addAttributes(this.pagerBar.find(".pager-last button"),this,this.settings.attributes,"btn-last"),E.addAttributes(this.pagerBar.find(".pager-pagesize button"),this,this.settings.attributes,"btn-pagesize"),E.addAttributes(this.pagerBar.find("input"),this,this.settings.attributes,"pagesize-input"),this.pagerBar.find(".pager-pagesize .popupmenu li").each((function(){var t=$(this).find("a"),e=t.text();E.addAttributes(t,n,n.settings.attributes,"pagesize-opt-".concat(e))}));var l=null===(e=this.pagerBar.find(".btn-menu"))||void 0===e||null===(i=null==e?void 0:e.data("popupmenu"))||void 0===i||null==i?void 0:i.menu;l&&l.find("li").each((function(){var t=$(this).find("a"),e=t.text();E.addAttributes(t,n,n.settings.attributes,"pagesize-opt-".concat(e))}))},initTabIndexes:function(){if(!this.settings.tabbable){var t=$(this.focusableElements);t.attr("tabindex","-1"),t.filter(":not([disabled])").first().removeAttr("tabindex")}},triggerPagingEvents:function(t){var e=this.state;t||(t=this.state.activePage),"first"===e.type&&(this.settings.onFirstPage&&this.settings.onFirstPage(this,e),this.element.trigger("firstpage",e)),"prev"===e.type&&(this.settings.onPreviousPage&&this.settings.onPreviousPage(this,e),this.element.trigger("previouspage",e)),"next"===e.type&&(this.settings.onNextPage&&this.settings.onNextPage(this,e),this.element.trigger("nextpage",e)),"last"===e.type&&(this.settings.onLastPage&&this.settings.onLastPage(this,e),this.element.trigger("lastpage",e)),"pageinfo"!==e.type&&this.element.trigger("page",e)},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),Array.isArray(t.pagesizes)&&t.pagesizes.length&&(this.settings.pagesizes=t.pagesizes);var e={activePage:this.settings.activePage,indeterminate:this.settings.indeterminate,pagesize:this.settings.pagesize};return this.handleDeprecatedSettings(),this.updatePagingInfo(e),this},changePageSize:function(t){var e=t;e.closest(".popupmenu").find(".is-checked").removeClass("is-checked"),e.parent("li").addClass("is-checked"),this.settings.pagesize=parseInt(e.text(),10),this.settings.componentAPI&&(this.settings.componentAPI.settings.pagesize=this.settings.pagesize),this.setActivePage(1,!0,"first"),this.settings.onPageSizeChange&&this.settings.onPageSizeChange(this,{tag:t,pagesize:this.settings.pagesize,settings:this.settings}),this.element.trigger("pagesizechange",{tag:t,pagesize:this.settings.pagesize,settings:this.settings})},updatePagingInfo:function(t,e){if(t){if(t.pagesize&&(this.settings.pagesize=t.pagesize,this.isTable&&this.settings.componentAPI&&(this.settings.componentAPI.settings.pagesize=t.pagesize)),t.isFilteredClientside?(this.isFilteredClientside=!0,delete this.serverDatasetTotal):this.isFilteredClientside&&delete this.isFilteredClientside,delete this.firstPage,delete this.lastPage,void 0!==t.firstPage&&(this.firstPage=t.firstPage,this.settings.enableFirstButton=!t.firstPage,this.settings.enablePreviousButton=!t.firstPage),void 0!==t.lastPage&&(this.lastPage=t.lastPage,this.settings.enableNextButton=!t.lastPage,this.settings.enableLastButton=!t.lastPage),isNaN(t.grandTotal)||(this.grandTotal=t.grandTotal),this.isFilteredClientside||(this.serverDatasetTotal=t.total),isNaN(t.filteredTotal)?(this.filteredTotal||this.filteredActivePage)&&(delete this.filteredTotal,delete this.filteredActivePage):(this.filteredTotal=t.filteredTotal,this.filteredActivePage=t.searchActivePage||t.filteredActivePage||1),t.type||(t.type="pageinfo"),this.settings.source||this.settings.dataset)return this.setActivePage(t,!1,t.type),void("initial"===t.type||e||this.triggerPagingEvents());this.teardown(),this.render(),this.handleEvents()}},hidePagerBar:function(t){return!(!t||!0!==t.firstPage||!0!==t.lastPage||!t.hideDisabledPagers)},teardown:function(){var t;(this.numberButtons&&this.numberButtons.forEach((function(t){var e=t.querySelector(".btn-icon"),i=$(e).data("button"),n=$(e).data("tooltip");i&&i.destroy(),n&&n.destroy()})),this.pagerBar.off(["click.".concat(Ns),"keydown.".concat(Ns)].join(" ")),this.pageSelectorInput)&&($(this.pageSelectorInput).off(["focus.".concat(Ns),"blur.".concat(Ns),"keydown.".concat(Ns)].join(" ")),null===(t=$(this.pageSelectorInput).data("mask"))||void 0===t||null==t||t.destroy());this.pageSizeSelectorButton&&($(this.pageSizeSelectorButton).off("selected.".concat(Ns)),this.teardownPageSizeSelector()),this.pagerBar[0].innerHTML="",delete this.firstPage,delete this.lastPage},teardownPageSizeSelector:function(){var t=$(this.pageSizeSelectorButton).data("popupmenu");if(t&&t.menu){var e=t.menu;t.destroy(),e.remove()}},destroy:function(){this.teardown(),this.pagerBar&&this.pagerBar.remove(),$.removeData(this.element[0],Ns)}},$.fn.pager=function(t){return this.each((function(){var e=$.data(this,Ns);e?e.updated(t):e=$.data(this,Ns,new Vs(this,t))}))};var Ws="listview",js={dataset:[],template:null,description:null,paging:!1,pagesize:10,searchable:!1,highlight:!0,selectable:"single",selectOnFocus:!0,showCheckboxes:!0,hoverable:!0,emptyMessage:null,source:null,forceToRenderOnEmptyDs:!1,disableItemDeactivation:!1,allowDeselect:!0,showPageSizeSelector:!1,listFilterSettings:null,pagerSettings:{showFirstButton:!1,showLastButton:!1},searchTermMinSize:1,initializeContents:!1,attributes:null,attributesOverride:null};function Us(t,e){return this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,js),this.init(),this}Us.prototype={get pagerAPI(){return this.element.data("pager")},get pagerSettings(){var t={};this.settings.pagerSettings&&(t=this.settings.pagerSettings),t.dataset=this.settings.dataset,t.source=this.settings.source,t.type="list";for(var e=["pagesize","showPageSizeSelector"],i=0;i<e.length;i++)void 0===this.settings[e[i]]||t[e[i]]||(t[e[i]]=this.settings[e[i]]);return t},init:function(){this.setup(),this.handleEvents(),this.refresh(),this.selectedItems=[],this.lastSelectedItem=0,this.isSelectedAll=!1,this.sortInit("listview","click.listview","data-sortlist"),this.handleResize()},setup:function(){var t=this,e=this,i=this.element.closest(".card, .widget"),n=this.element.attr("data-selectable"),s=this.element.attr("data-select-onfocus");if($(".listview-filter-wrapper").length>0){var a=$(".listview-filter-wrapper"),o=a.siblings(".searchfield-wrapper");o.addClass("no-animate");var r=a.css("width");o.addClass("has-listview-filters"),o.css("width","calc(100% - ".concat(r,")"))}if(this.element.attr("data-pagesize")){var c=Number(this.element.attr("data-pagesize"));isNaN(c)||(this.settings.pagesize=c),this.element.removeAttr("data-pagesize")}if(this.element.hasClass("card-list")&&this.element.find("li .card").selectable(),!this.settings.dataset.length&&this.element.is("ul")&&this.element.children("li").length){var d=this.element.children("li");this.settings.template||(this.settings.template="{{#dataset}}<li>{{text}}</li>{{/dataset}}"),d.each((function(e,i){t.settings.dataset.push({text:$(i).text()})})),d.remove()}if(this.settings.dataset&&"string"==typeof this.settings.dataset){var h=window[this.settings.dataset];h&&h.length&&(this.settings.dataset=h)}n&&n.length&&(this.settings.selectable=n),s&&s.length&&(this.settings.selectOnFocus=JSON.parse(s)),e.actionButton=i.find(".btn-actions"),e.actionButton.length>0&&(e.actionButton.data("popupmenu")||e.actionButton.popupmenu()),this.element.attr({tabindex:"-1","x-ms-format-detection":"none"}),this.settings.attributes&&E.addAttributes(this.element,this,this.settings.attributes,"listview",this.settings.attributesOverride),(this.element.is(".paginated")||!0===this.settings.paging)&&this.element.pager(this.pagerSettings);var u=this.element.parents(".card-content, .widget-content");if(this.element.prev().is(".listview-search")&&u[0]&&u.css({overflow:"visible"}),this.settings.searchable){this.searchfield=this.element.parent().find(".searchfield, .autocomplete"),this.searchfield.length;var p={filterMode:"contains"};"object"===l(this.settings.listFilterSettings)&&(p=E.extend({},p,this.settings.listFilterSettings)),this.listfilter=new We(p)}this.settings.emptyMessage&&(e.emptyMessageContainer=$("<div>").emptymessage(this.settings.emptyMessage))},getTotals:function(t){var e,i={count:t.length};if(t[0]){for(e in t[0])i[e]=0;for(var n=0;n<t.length;n++)for(e in t[n])i[e]+=parseFloat(t[n][e]);return i}},render:function(t,e){var i=this,n="function"==typeof this.settings.source,s={},a=t,o=0,r=a?a.length:0,c=this.settings.pagesize,d=t.length;if((e&&(c=e.pagesize||c,d=e.filteredTotal||d),!n&&this.pagerAPI&&(this.renderPager(e,!0),e=this.pagerAPI.state),d>c)&&(this.filteredDataset?e.filteredPages:e.pages)>1){var h=e.activePage>0?e.activePage-1:0;this.filteredDataset&&(h=e.filteredActivePage-1),o=e.pagesize*h,r=e.pagesize*(h+1),a=t.slice(o,r)}if("object"===l(je)&&a&&this.settings.template){this.settings.template instanceof $?this.settings.template="".concat(this.settings.template.html()):"string"==typeof this.settings.template&&(M.containsHTML(this.settings.template)||(this.settings.template=$("#".concat(this.settings.template)).html())),this.settings.template.indexOf("{{#totals}}")>-1&&(s=this.getTotals(t));var u=je.compile(this.settings.template,{dataset:a,totals:s});this.element.parent().is(".scrollable-flex-content")&&this.element.parent().find(".empty-message").remove(),a.length>0||this.settings.forceToRenderOnEmptyDs?this.element.html(u):i.emptyMessageContainer&&this.element.parent().is(".scrollable-flex-content")?(this.element.empty(),f.append(this.element.parent(),this.emptyMessageContainer[0].outerHTML,"<div><svg><use><span><b>")):i.emptyMessageContainer?(this.element.empty(),f.append(this.element,this.emptyMessageContainer[0].outerHTML,"<div><svg><use><span><b>")):0===a.length&&this.element.html(u||"<ul></ul>")}var p=this.element.closest(".card, .widget");$("ul",this.element).attr({role:"listbox","aria-label":this.settings.description||p.find(".card-title, .widget-title").text()||"List"});var g=this.element.find("li, tbody > tr").first(),m=this.element.find("li, tr"),v="multiple"===this.settings.selectable||"mixed"===this.settings.selectable;this.focusItem=g.attr("tabindex",0),this.settings.selectable||1!==g.find("a").length||g.removeAttr("tabindex");var b=m.length>a.length;m.each((function(t){var e=$(this);if(e.attr("role","option"),x.features.touch&&e.addClass("is-touch"),i.settings.attributes&&E.addAttributes(e,i,i.settings.attributes,"listview-item-".concat(t),i.settings.attributesOverride),v){i.element.addClass("is-multiselect");var n=i.element.prevAll(".toolbar, .contextual-toolbar");n.length&&n.data("toolbar")&&n.data("toolbar").toggleMoreMenu(),i.settings.showCheckboxes&&0===e.children(".listview-selection-checkbox").length&&e.prepend('<label class="listview-selection-checkbox l-vertical-center inline inline-checkbox">\n              <input tabindex="-1" type="checkbox" class="checkbox">\n              <span class="label-text" role="presentation">\n                <span class="audible">'.concat(H.translate("Checkbox")," ").concat(H.translate("NotSelected"),".</span>\n              </span>\n            </label>"))}else i.element.removeClass("is-multiselect"),e.find(".listview-selection-checkbox").remove();if(b){var s=o+t;if(s<i.settings.dataset.length){var a=i.settings.dataset[s];e.css("display",void 0===a._isFilteredOut||a._isFilteredOut?"":"none")}}else e.css("display","");e.attr({"aria-posinset":o+t+1,"aria-setsize":d}),e.hasClass("is-disabled")&&e.attr("aria-disabled","true"),i.settings.highlight&&i.searchTerm&&e.highlight(i.searchTerm);var r=e.find(".link-item .hyperlink");r.length>0&&(r[0].offsetHeight<r[0].scrollHeight||r[0].offsetWidth<r[0].scrollWidth)&&r.tooltip({title:r.text()})})),i.settings.initializeContents&&this.element.find("ul").initialize(),this.element.trigger("rendered",[a]),this.element.off("updated.listview").on("updated.listview",(function(t,e){i.updated(e)}))},renderPager:function(t,e){this.pagerAPI&&this.pagerAPI.updatePagingInfo(t,e)},getPageableElements:function(){var t=this.element.children();return t.is("ul")&&(t=t.children()),t},refresh:function(t,e){this.loadData(t,e),this.list&&this.render(this.list.data)},loadData:function(t,e,i){var n=!1,s=this;if(t=t||this.settings.dataset,this.filteredDataset&&(t=this.filteredDataset),e||(this.pagerAPI?(t.length!==this.pagerAPI.settings.dataset.length&&this.pagerAPI.updated({dataset:t}),e=this.pagerAPI.state):e={activePage:1}),this.filteredDataset?e.filteredTotal=t.length:e.total=t.length,Array.isArray(t)){var a=this.settings.source;if("string"!=typeof t||0!==t.indexOf("http")&&0!==t.indexOf("/")||(a=t,n=!0),!a&&!n||this.filteredDataset||i)this.render(t,e);else switch(l(a)){case"function":a(e,o);break;case"string":return void(0!==a.indexOf("http")&&0!==a.indexOf("/")||$.getJSON(a,o));default:this.settings.dataset=a,t=a,this.render(a,e)}}function o(e,i){s.settings.dataset=e,t=e;var n=s.pagerAPI?s.pagerAPI.activePage:1;"string"==typeof i&&(i={activePage:n,pagesize:s.settings.pagesize,total:t.length,type:i}),s.pagerAPI&&(s.renderPager(i,!0),i=s.pagerAPI.state),s.render(t,i)}},toggleAll:function(){this[this.isSelectedAll?"deselectItemsBetweenIndexes":"selectItemsBetweenIndexes"]([0,$("li, tbody tr",this.element).length-1]),this.isSelectedAll=!this.isSelectedAll},selectItemsBetweenIndexes:function(t){E.clearSelection(),t.sort((function(t,e){return t-e}));for(var e=t[0];e<=t[1];e++){var i=$("li, tbody tr",this.element).eq(e);i.is(".is-disabled, .is-selected")||this.select(i)}},deselectItemsBetweenIndexes:function(t){t.sort((function(t,e){return t-e}));for(var e=t[0];e<=t[1];e++){var i=$("li, tbody tr",this.element).eq(e);!i.is(".is-disabled")&&i.is(".is-selected")&&this.select(i)}},handleResize:function(){var t=$("li .listview-heading, tr .listview-heading",this.element),e=t.eq(1),i=e.width();e.length&&i&&(t[0].style.width="".concat(i,"px")),this.pagerAPI&&this.pagerAPI.render()},handleSearch:function(t,e){this.filter(e)},filter:function(t){if(t){var e=(t=$(t)).val().trim(),i={searchActivePage:1};if(e){if(e.length<this.settings.searchTermMinSize)this.resetSearch();else if(this.searchTerm!==e){this.searchTerm=e,this.element.unhighlight(),this.settings.dataset.forEach((function(t){t._isFilteredOut=!1}));var n=this.listfilter.filter(this.settings.dataset,this.searchTerm,!0);n.length||(n=[]),i.filteredTotal=n.length,i.searchActivePage=1,n.forEach((function(t){t._isFilteredOut=!0})),this.filteredDataset=n,this.loadData(null,i),this.element.trigger("filtered",{elem:this.element,filteredResults:n,term:this.searchTerm})}}else{if(!this.searchTerm)return;this.resetSearch()}}},resetSearch:function(){this.element.unhighlight(),this.settings.dataset.forEach((function(t){delete t._isFilteredOut})),this.filteredDataset&&delete this.filteredDataset,this.searchTerm&&delete this.searchTerm;var t={activePage:1,filteredTotal:void 0,searchActivePage:void 0};this.refresh(null,t)},focus:function(t){t.is(":hidden")||t.is(".is-disabled")||(this.focusItem&&this.focusItem.removeAttr("tabindex"),this.focusItem=t.attr("tabindex",0).focus(),this.settings.selectable||1!==t.find("a").length||(t.find("a").focus(),t.removeAttr("tabindex")),this.settings.selectOnFocus&&"multiple"!==this.settings.selectable&&"mixed"!==this.settings.selectable&&this.select(t))},remove:function(t){"number"==typeof t&&(t=$(this.element.children()[0]).children().eq(t)),t.is(".is-selected")&&this.select(t,!0),t.remove()},clear:function(){$(this.element.children()[0]).empty()},removeAllSelected:function(){var t=this;$.each(this.selectedItems,(function(e,i){t.remove(i)}))},clearAllSelected:function(){var t=this;$.each(this.selectedItems,(function(e,i){t.select(i)}))},sortInit:function(t,e,i){i&&""!==$.trim(i)&&$("[".concat(i,"]")).each((function(){var n=$(this),s=$.fn.parseOptions(n,i);n.on(e,(function(e){$(s.list).data(t).setSortColumn(s),e.preventDefault()}))}))},setSortColumn:function(t){var e=t.orderBy||this.list.sort.field,i=t.order;if(this.list.data||e){i=i?"desc"===i:!(this.list.sort&&this.list.sort[e]&&this.list.sort[e].reverse),(t.reloadApi||t.reloadApiNoSort)&&this.loadData(),t.reloadApiNoSort&&(e=this.list.sort.field,i=this.list.sort[e].reverse);var n=this.sortFunction(e,i);this.list.data.sort(n),this.render(this.list.data),this.list.sort={field:e},this.list.sort[e]={reverse:i},this.element.trigger("sorted",[this.element,this.list.sort])}},sortFunction:function(t,e,i){i||(i=function(t){return"string"==typeof(t=null==t?"":t)&&(t=t.toUpperCase(),isNaN(parseFloat(t))||(t=parseFloat(t))),t});var n=i?function(e){return i(e[t])}:function(e){return e[t]};return e=e?-1:1,function(t,i){return t=n(t),i=n(i),e*((t>i)-(i>t))}},deselect:function(t){this.settings.allowDeselect&&("number"==typeof t&&(t=$(this.element.children()[0]).children().eq(t)),t.is(".is-selected")&&this.select(t))},unselect:function(t){return j(this.deselect,this.unselect).apply(this,[t])},select:function(t,e){var i,n=this,s="mixed"===n.settings.selectable;if(n.selectedItems=[],"number"==typeof t&&(t=$(this.element.children()[0]).children().eq(t)),(!(i=t.hasClass("is-selected"))||this.settings.allowDeselect)&&(t.is('[tabindex="0"]')||(this.focusItem&&this.focusItem.removeAttr("tabindex"),t.attr("tabindex",0)),!1!==this.settings.selectable&&"false"!==this.settings.selectable)){if("multiple"!==this.settings.selectable&&"mixed"!==this.settings.selectable){var a=t.parent().children(),o=t.parent().find(".is-selected");this.element.hasClass("card-list")&&(a=a.not(".card"),o=o.not(".card")),a.removeAttr("aria-selected"),o.removeClass("is-selected"),n.selectedItems[0]=$(this)}if(i?(n.selectedItems=[],t.removeClass("is-selected hide-selected-color")):this.settings.selectable&&(t.addClass("is-selected".concat(s?" hide-selected-color":"")),n.lastSelectedItem=t.index()),t.parent().find(".is-selected").each((function(t){n.selectedItems[t]=$(this)})),t.find(".listview-selection-checkbox input").prop("checked",!i),"multiple"===this.settings.selectable||"mixed"===this.settings.selectable){var r=i?"NotSelected":"Selected";t.attr("aria-selected",!i),t.find(".label-text .audible").text("".concat(H.translate("Checkbox")," ").concat(r,"."))}if(!e){for(var l=i?"unselected":"selected",c=[],d=0;d<this.selectedItems.length;d++){var h=this.selectedItems[d][0].getAttribute("aria-posinset");c.push(this.settings.dataset[h-1])}this.element.triggerHandler(l,{selectedItems:this.selectedItems,elem:t,selectedData:c}),"unselected"===l&&this.element.triggerHandler("deselected",{selectedItems:this.selectedItems,elem:t,selectedData:c})}var u=this.element.closest(".card, .widget");u.length||(u=this.element.parent());var p=u.find(".listview-toolbar, .contextual-toolbar"),g=p.data("toolbar");if(n.selectedItems.length>0){if(g&&g.toggleMoreMenu(),p.one("animateopencomplete",(function(){n.element.addClass("is-toolbar-open"),p.trigger("recalculate-buttons").removeClass("is-hidden")})),p[0]){var f=p[0].classList.contains("contextual-toolbar");p[0].style.display=f?"block":""}p.animateOpen({distance:40});var m=p.find(".title, .selection-count");m&&m.length||(m=$(p.hasClass("contextual-toolbar")?'<div class="toolbar-section title selection-count"></div>':'<div class="title selection-count"></div>'),p.prepend(m));var v="(".concat(H.translate(1===n.selectedItems.length?"RecordSelected":"RecordsSelected"),")");v=v.replace("{0}",n.selectedItems.length),m.text(v)}else p.addClass("is-hidden").one("animateclosedcomplete",(function(t){t.stopPropagation(),this.style.display="none",n.element.removeClass("is-toolbar-open")})).animateClosed()}},toggleItemActivation:function(t){t.hasClass("is-activated")?this.settings.disableItemDeactivation||this.deactivateItem(t):this.activateItem(t)},activateItem:function(t){var e="number"==typeof t?t:t.index(),i=this.element.find("li.is-activated");"number"==typeof t&&(t=this.element.find("ul").children().eq(t)),this.deactivateItem(i),!1!==this.element.triggerHandler("beforeactivate",[{index:e,elem:t,data:this.settings.dataset[e]}])&&(t.addClass("is-activated"),this.element.triggerHandler("itemactivated",[{index:e,elem:t,data:this.settings.dataset[e]}]))},activatedItem:function(){var t=this.element.find("li.is-activated"),e=t.index();return{index:e,elem:t,data:this.settings.dataset[e]}},deactivateItem:function(t){"number"==typeof t&&(t=this.element.find("ul").children().eq(t)),void 0===t&&(t=this.element.find("li.is-activated")),t.removeClass("is-activated");var e=t.index();e<0||this.element.triggerHandler("itemdeactivated",[{index:e,elem:t,data:this.settings.dataset[e]}])},getSelected:function(){return this.element.find(".is-selected")},updateSearch:function(){if(this.settings.searchable&&this.filteredDataset){delete this.filteredDataset,this.searchTerm&&delete this.searchTerm;var t=this.searchfield.data("searchfield");t&&t.xButton&&t.xButton.trigger("click")}},updated:function(t){(null==t||null==t?void 0:t.selectable)!==this.settings.selectable&&$(this.element.children()[0]).find("li").removeClass("is-selected is-activated");return t&&(this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset&&(this.settings.dataset=t.dataset)),this.updateSearch(),this.refresh(t&&t.dataset?t.dataset:null),this},disable:function(){this.element.addClass("is-disabled")},enable:function(){this.element.removeClass("is-disabled")},teardown:function(){return $("body").off("resize.listview"),this.element.prev(".listview-header").off("click.listview"),this.searchfield&&this.searchfield.off(["contents-checked.searchable-listview","cleared.searchable-listview"].join(" ")),this.element.off("change.selectable-listview",".listview-checkbox input"),this.element.off("contextmenu.listview longpress.listview dblclick.listview","li, tr"),this.element.off("click.listview","li, tr, input[checkbox]"),this.element.off("keydown.listview","li, tr, a"),this.element.off("focus.listview","li, tbody tr"),this.element.off("focus.listview click.listview touchend.listview keydown.listview change.selectable-listview updated.listview").empty(),this.filteredDataset&&delete this.filteredDataset,this.searchTerm&&delete this.searchTerm,this},destroy:function(){this.teardown(),this.element.removeData(Ws)},handleEvents:function(){var t=this,e=!1,i=!1,n="multiple"===t.settings.selectable||"mixed"===t.settings.selectable;if(this.element.off("focus.listview","li, tbody tr").on("focus.listview","li, tbody tr",(function(n){var s=$(this);if(!(n.originalEvent&&n.originalEvent.target&&$(n.originalEvent.target).is(".icon-favorite"))){if(s.is(":first-child")&&s.hasClass("is-disabled")){var a=$.Event("keydown.listview");a.keyCode=40,e=!0,s.trigger(a)}e||s.hasClass("is-disabled")||!t.settings.selectOnFocus||"multiple"===t.settings.selectable||"mixed"===t.settings.selectable||(t.select(s),e=!0,i=!0)}})),this.element.off("keydown.listview","li, tr, a").on("keydown.listview","li, tr, a",(function(e){var i=$(this),s=i.is("a")?i.closest("li"):$(this),a=s.is("a")?s.closest("ul"):s.parent(),o=e.keyCode||e.charCode||0,r=e.metaKey;if(0===s.index()&&38===e.keyCode)return!1;if((40===o||38===o)&&!r){var l=40===e.keyCode?s.nextAll(":not(.is-disabled):visible:first"):s.prevAll(":not(.is-disabled):visible:first");l.length&&($(e.target).is(s)||e.shiftKey||i.is("a"))&&t.focus(l),e.preventDefault(),e.stopPropagation()}if(35===o||40===o&&r){var c=a.children().last();return t.focus(c),e.stopPropagation(),!1}if(36===o||38===o&&r){var d=a.children().first();return t.focus(d),e.stopPropagation(),!1}return 32===o&&$(e.target).is(s)&&(n&&e.shiftKey?t.selectItemsBetweenIndexes([t.lastSelectedItem,s.index()]):t.select(s),e.preventDefault()),n&&(e.ctrlKey||e.metaKey)&&65===o&&(t.toggleAll(),t.focus(s),e.preventDefault()),!0})),this.settings.selectable){this.element.addClass("is-selectable");var s=$(this.element).closest(".list-detail, .builder");this.element.off("click.listview","li, tr, input[checkbox]").on("click.listview","li, tr, input[checkbox]",(function(a){var o=$(this),r=$(a.target).closest(".listview-selection-checkbox").length>0,l="mixed"===t.settings.selectable,c=$(a.target);c.hasClass("icon-favorite")||c.hasClass("hyperlink")||(!o.hasClass("is-selected")&&i&&(i=!1),i||o.hasClass("is-disabled")||l&&!r||(e=!0,n&&a.shiftKey?(t.selectItemsBetweenIndexes([t.lastSelectedItem,o.index()]),a.preventDefault()):t.select(o),o.focus()),o.hasClass("is-disabled")||!l||r||(o.focus(),t.toggleItemActivation(o)),s.length>0&&$(window).outerWidth()<767&&!o.hasClass("is-disabled")&&!r&&t.element.trigger("drilldown",[o]),i=!1,a.preventDefault(),a.stopPropagation(),t.element.trigger("click",[{elem:o,data:t.settings.dataset[o.attr("aria-posinset")-1],index:o.index(),originalEvent:a}]))})),this.element.off("dblclick.listview","li, tr").on("dblclick.listview","li, tr",(function(e){var i=$(this);return e.preventDefault(),e.stopPropagation(),t.element.trigger("dblclick",[{elem:$(this),data:t.settings.dataset[i.attr("aria-posinset")-1],index:i.index(),originalEvent:e}]),!1})),this.element.off("contextmenu.listview longpress.listview","li, tr").on("contextmenu.listview longpress.listview","li, tr",(function(e){var i=$(this);return e.preventDefault(),e.stopPropagation(),t.element.trigger("contextmenu",[{elem:$(this),data:t.settings.dataset[i.attr("aria-posinset")-1],index:i.index(),originalEvent:e}]),!1}))}this.settings.hoverable&&"false"!==this.settings.hoverable||(this.element.removeClass("is-selectable"),this.element.addClass("disable-hover")),this.settings.selectable&&"false"!==this.settings.selectable||(this.element.removeClass("is-selectable"),this.element.addClass("disable-hover")),"multiple"!==this.settings.selectable&&"mixed"!==this.settings.selectable||this.element.off("change.selectable-listview",".listview-checkbox input").on("change.selectable-listview",".listview-checkbox input",(function(t){$(this).parent().trigger("click"),t.stopPropagation()})),this.settings.searchable&&this.searchfield.off("contents-checked.searchable-listview").on("contents-checked.searchable-listview",(function(e){t.handleSearch(e,$(this))})).off("cleared.searchable-listview").on("cleared.searchable-listview",(function(){t.resetSearch(),t.element.trigger("filtered",{elem:t.element,filteredResults:[],term:""})})),this.pagerAPI&&this.element.off("page.listview").on("page.listview",(function(e,i){t.handlePageChange(i)})).off("pagesizechange.listview").on("pagesizechange.listview",(function(e,i){t.handlePageSizeChange(i)})),$("body").off("resize.listview").on("resize.listview",(function(e){t.handleResize(e)})),t.element.prev(".listview-header").off("click.listview").on("click.listview",(function(){var e=$(this).find(".plus-minus");e.hasClass("active")?(e.removeClass("active"),t.element.animateClosed()):(e.addClass("active"),t.element.animateOpen())}))},handlePageSizeChange:function(t){t.activePage=1,this.loadData(void 0,t)},handlePageChange:function(t){this.loadData(void 0,t)}},$.fn.listview=function(t){var e=$(this),i=e.attr("data-dataset"),n=e.attr("data-tmpl"),s=E.parseOptions(this)||{};s.dataset=s.dataset||i,s.template=s.template||n,window[s.dataset]&&(s.dataset=window[s.dataset]),s.template&&s.template.length&&(s.template=$("#".concat(s.template)).html());var a=E.extend({},t,s);return this.each((function(){var t=$.data(this,Ws);t?t.updated(a):t=$.data(this,Ws,new Us(this,a))}))};var Gs="listbuilder",_s={dataset:[],handle:".handle",btnAdd:"add",btnEdit:"edit",btnDelete:"delete",btnGoUp:"goup",btnGoDown:"godown",attributes:null,template:'<ul data-handle=".handle">{{#dataset}}{{#text}}<li{{#value}} data-value="{{value}}"{{/value}}{{#selected}} selected="selected"{{/selected}}{{#disabled}} class="is-disabled"{{/disabled}}><span class="handle" focusable="false" aria-hidden="true" role="presentation">&#8286;</span><div class="item-content"><p>{{text}}</p></div></li>{{/text}}{{/dataset}}</ul>',templateNewItem:'<li data-value="{{text}}" role="option"><span class="handle" focusable="false" aria-hidden="true" role="presentation">&#8286;</span><div class="item-content"><p>{{text}}</p></div></li>',templateItemContent:"<p>{{text}}</p>"};function Ys(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,_s),e&&e.dataset&&(this.settings.dataset=e.dataset),this.init()}Ys.prototype={init:function(){var t=this;this.loadListview().initDataset().setElements().handleEvents(),setTimeout((function(){t.setSelected()}),0)},loadListview:function(){var t,e=this.settings,i=$(".listview",this.element);return e.attributes&&(t=this.getLvAutomationAttributes("-listbuilder")),!e.dataset.length&&i.length&&$("li",i).length?this.listApi=i.listview({selectable:"single",attributes:t}).data("listview"):i.length&&(this.listApi=i.listview({dataset:e.dataset,template:e.template,selectable:"single",attributes:t}).data("listview")),this},getLvAutomationAttributes:function(t){var e,i=this,n=this.settings;if(n.attributes&&"string"==typeof t&&""!==t)if(Array.isArray(n.attributes))e=[],n.attributes.forEach((function(n){var s="function"==typeof n.value?n.value(i):n.value;e.push({name:n.name,value:s+t})}));else{var s="function"==typeof n.attributes.value?n.attributes.value(this):n.attributes.value;e={name:n.attributes.name,value:s+t}}return e},updateLvAutomationAttributes:function(t){if(this.settings.attributes&&t){for(var e,i=this.dataset.length-1,n=!0,s=1e3;n&&s>0;){var a="-listbuilder-listview-item-".concat(i);e=this.getLvAutomationAttributes(a),n=this.findAutomationAttributesItem(e),i++,s--}E.addAttributes(t,this,e)}},findAutomationAttributesItem:function(t){var e=!1;if(this.settings.attributes&&t&&Array.isArray(t))for(var i=0,n=t.length;i<n;i++){var s=t[i];if(this.listApi.element.find("[".concat(s.name,'="').concat(s.value,'"]')).length){e=!0;break}}return e},initDataset:function(){var t=this.settings,e=$(".listview li",this.element);this.dataset=[];for(var i=0,n=e.length;i<n;i++){var s=void 0,a=$(e[i]);t.dataset?(s=JSON.parse(JSON.stringify(t.dataset[i]))).node=a:s=this.extractNodeData(a),this.dataset.push(s)}return this},extractNodeData:function(t){var e={node:t,text:$.trim($(".item-content",t).text())},i=t.attr("data-value");return void 0!==i&&(e.value=i),e},setElements:function(){var t=this,e=this.settings,i=function(e){return t.isjQuery(e)?e:"string"==typeof e?$('[data-action="'.concat(e,'"]'),t.element):null};e.btnAdd=i(e.btnAdd),e.btnGoUp=i(e.btnGoUp),e.btnGoDown=i(e.btnGoDown),e.btnEdit=i(e.btnEdit),e.btnDelete=i(e.btnDelete);var n="listbuilder-btn";return E.addAttributes(e.btnAdd,this,e.attributes,"".concat(n,"-add")),E.addAttributes(e.btnGoUp,this,e.attributes,"".concat(n,"-goup")),E.addAttributes(e.btnGoDown,this,e.attributes,"".concat(n,"-godown")),E.addAttributes(e.btnEdit,this,e.attributes,"".concat(n,"-edit")),E.addAttributes(e.btnDelete,this,e.attributes,"".concat(n,"-delete")),this.topButtons=e.btnAdd.add(e.btnGoUp).add(e.btnGoDown).add(e.btnEdit).add(e.btnDelete),this.topButtons.tooltip(),this.ul=$(".listview ul",this.element),this.arrangeApi=this.ul.arrange({handle:e.handle,placeholder:e.templateNewItem}).data("arrange"),this},handleEvents:function(){var t,e=this,i=this,n=i.settings,s=function(t,e){t.off("click.listbuilder").on("click.listbuilder",(function(){i[e]()}))};return s(n.btnAdd,"addItem"),s(n.btnGoUp,"moveItemUp"),s(n.btnGoDown,"moveItemDown"),s(n.btnEdit,"editItem"),s(n.btnDelete,"deleteItem"),this.arrangeApi.element.on("beforearrange.listbuilder",(function(t,i){var s=e.getDataByNode(i.start),a=n.templateItemContent.replace(/{{text}}/g,s.data.text);e.arrangeApi.placeholders.attr("data-value",s.data.text).find(".item-content").html(a)})).on("arrangeupdate.listbuilder",(function(i,n){e.updateAttributes(),e.arrayIndexMove(e.dataset,n.startIndex,n.endIndex),(t=e.getDataByNode(n.end)).indexBeforeMove=n.startIndex,e.element.triggerHandler("arrangeupdate",[t])})),$("li:not(.is-disabled) ".concat(this.arrangeApi.handle),this.ul).on("mousedown.listbuilder touchstart.listbuilder",(function(){var t=$(this);t.is(".is-selected")||t.trigger("click")})),$(".listview",this.element).off("selected.listbuilder").on("selected.listbuilder",(function(i,n){t=e.getDataByNode(n.elem[0]),e.element.triggerHandler("selected",[t])})),this.updatedEventsStr="arrangeupdate.listbuilder aftergoup.listbuilder aftergodown.listbuilder exiteditmode.listbuilder",this.element.off(this.updatedEventsStr).on(this.updatedEventsStr,(function(t,i){e.element.triggerHandler("updated",[i])})),this},addItem:function(){var t=this,e=this.settings;$.when(this.element.triggerHandler("beforeadd")).done((function(){var i,n,s=0,a=t.listApi.selectedItems[0],o=e.templateNewItem.replace(/{{text}}/g,H.translate("NewItem"));a&&a.length>0?(s=(n=t.getDataByNode(a)).index+1,$(o).insertAfter(a),i=$("li",t.ul).eq(s)):(t.ul.prepend(o),i=$("li:first-child",t.ul)),t.dataset.push(t.extractNodeData(i)),t.arrayIndexMove(t.dataset,t.dataset.length-1,s),t.updateAttributes(),t.updateLvAutomationAttributes(i),i.trigger("click"),t.arrangeApi.updated(),t.editItem(!0),n={index:s,data:t.dataset[s]},t.element.triggerHandler("afteradd",[n])}))},moveItemUp:function(){var t=this,e=t.listApi.selectedItems[0];if(e&&e.length>0){var i=t.getDataByNode(e);void 0!==i.index&&i.index>0&&$.when(t.element.triggerHandler("beforegoup",[i])).done((function(){var n=e.prev();e.insertBefore(n),t.updateAttributes(),t.arrayIndexMove(t.dataset,i.index,i.index-1),i.indexBeforeMove=i.index,i.index--,t.element.triggerHandler("aftergoup",[i])}))}},moveItemDown:function(){var t=this,e=t.listApi.selectedItems[0];if(e&&e.length>0){var i=t.getDataByNode(e);void 0!==i.index&&i.index<t.dataset.length-1&&$.when(t.element.triggerHandler("beforegodown",[i])).done((function(){var n=e.next();e.insertAfter(n),t.updateAttributes(),t.arrayIndexMove(t.dataset,i.index,i.index+1),i.indexBeforeMove=i.index,i.index++,t.element.triggerHandler("aftergodown",[i])}))}},editItem:function(t){var e=this.listApi.selectedItems[0];e&&e.length>0&&(e.is(".is-editing")?this.commitEdit(e,t):this.makeEditable(e,t))},makeEditable:function(t,e){var i=this,n=i.getDataByNode(t),s=$(".item-content",t);void 0!==n.index&&n.index<i.dataset.length&&$.when(i.element.triggerHandler("beforeedit",[n])).done((function(){var a=g.escapeHTML((s.text().trim()||"").toString()),o=$('<input name="edit-input" class="edit-input" type="text" value="'.concat(a,'" />'));t.addClass("is-editing"),s.html(o),setTimeout((function(){o.focus().select()}),0),o.on("click.listbuilder",(function(){return!1})).on("blur.listbuilder",(function(){return i.commitEdit(t,e)})).on("keypress.listbuilder",(function(n){13===(n.keyCode||n.charCode||0)&&(i.commitEdit(t,e),t.focus())})),i.element.triggerHandler("entereditmode",[n])}))},commitEdit:function(t,e){var i=this.settings,n=this.getDataByNode(t),s=$(".item-content",t),a=$(".edit-input",s);a.val(g.escapeHTML((a.val()||"").toString())),e&&(n.data.value=a.val()),n.data.text=a.val(),a.off("click.listbuilder blur.listbuilder keypress.listbuilder"),s.html(i.templateItemContent.replace(/{{text}}/g,a.val())),t.removeClass("is-editing"),this.element.triggerHandler("exiteditmode",[n])},deleteItem:function(){var t=this,e=t.listApi.selectedItems[0];if(e&&e.length>0){var i=t.getDataByNode(e);void 0!==i.index&&$.when(t.element.triggerHandler("beforedelete",[i])).done((function(){t.listApi.removeAllSelected(),t.updateAttributes(),t.dataset.splice(i.index,1),t.element.triggerHandler("afterdelete",[i])}))}},getDataByNode:function(t){for(var e={},i=0,n=this.dataset.length;i<n;i++){var s=this.dataset[i];if($(s.node).is(t)){e={index:i,data:s};break}}return e},arrayIndexMove:function(t,e,i){t.splice(i,0,t.splice(e,1)[0])},isjQuery:function(t){return t&&(t instanceof jQuery||t.constructor.prototype.jquery)},moveCursorToEnd:function(t){setTimeout((function(){if("number"==typeof t.selectionStart)t.selectionEnd=t.value.length,t.selectionStart=t.value.length;else if(void 0!==t.createTextRange){var e=t.createTextRange();e.collapse(!1),e.select()}}),100)},updateAttributes:function(){for(var t=$("li",this.ul),e=0,i=t.length;e<i;e++)$(t[e]).attr({"aria-posinset":e+1,"aria-setsize":i})},updateDataset:function(t){var e=this,i=$("li",this.ul),n=$(".listview",this.element).data("listview");n.deselectItemsBetweenIndexes([0,i.length-1]),this.settings.dataset=t,n.loadData(this.settings.dataset),this.initDataset().setElements().handleEvents(),setTimeout((function(){e.setSelected()}),0)},setSelected:function(){for(var t=$("li[selected]",this.ul),e=0,i=t.length;e<i;e++){var n=$(t[e]);n.removeAttr("selected"),n.is(".is-selected")||n.trigger("click")}return this},select:function(t){var e=this.getListItem(t);e&&!e.is(".is-selected")&&e.trigger("click")},unselect:function(t){var e=this.getListItem(t);e&&e.is(".is-selected")&&e.trigger("click")},getListItem:function(t){var e=$();if(this.isElement(t)&&$.contains(this.ul,t))e=this.isjQuery(t)?t:$(t);else{var i=parseInt(t,10),n=$("li",this.ul);!isNaN(i)&&i>-1&&i<n.length?e=n.eq(i):"first"==="".concat(t).toLowerCase()?e=n.first():"last"==="".concat(t).toLowerCase()&&(e=n.last())}return e.length<1?null:e.length>1?e.eq(0):e},isElement:function(t){return this.isjQuery(t)&&t.get(0)instanceof Element||t instanceof Element},enable:function(){this.element.removeClass("is-disabled").find(".toolbar .buttonset button").removeAttr("disabled").end().find(".toolbar .buttonset button[data-original-disabled]").attr("disabled","disabled").removeAttr("data-original-disabled"),this.ul.find("li").removeClass("is-disabled").end().find("li[data-original-disabled]").removeAttr("data-original-disabled")},disable:function(){this.element.addClass("is-disabled").find(".toolbar .buttonset button[disabled]").attr("data-original-disabled","disabled").end().find(".toolbar .buttonset button").attr("disabled","disabled"),this.ul.find("li.is-disabled").attr("data-original-disabled","is-disabled").end().find("li").addClass("is-disabled")},unbind:function(){if(this.element.off(this.updatedEventsStr),$(".listview",this.element).off("selected.listbuilder"),$("li ".concat(this.arrangeApi.handle),this.ul).off("mousedown.listbuilder touchstart.listbuilder"),this.arrangeApi.element.off("beforearrange.listbuilder arrangeupdate.listbuilder").destroy(),this.topButtons.off("click.listbuilder"),this.topButtons)for(var t=0,e=this.topButtons.length;t<e;t++){var i=$(this.topButtons[t]).data("tooltip");i&&"function"==typeof i.destroy&&i.destroy()}return this.listApi&&"function"==typeof this.listApi.destroy&&this.listApi.destroy(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,_s)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Gs)}},$.fn.listbuilder=function(t){return this.each((function(){var e=$.data(this,Gs);e?e.updated(t):e=$.data(this,Gs,new Ys(this,t))}))};var Ks="modal",Xs=["click","immediate"],Qs=[!1,"responsive","always"],Js={trigger:"click",buttons:null,isAlert:!1,content:null,cssClass:null,autoFocus:!0,id:null,frameHeight:180,frameWidth:0,beforeShow:null,useFlexToolbar:!1,showCloseBtn:!1,closeBtnOptions:{attributes:[],closeBtnTooltip:"Close"},maxWidth:null,fullsize:Qs[0],breakpoint:"phone-to-tablet",overlayOpacity:.7,noRefocus:!1,triggerButton:null,hideUnderneath:!1,suppressEnterKey:!1,icon:null,iconClass:null};function Zs(t){return t.trigger&&-1===Xs.indexOf(t.trigger)&&(t.trigger=Js.trigger),t.fullsize&&-1===Qs.indexOf(t.fullsize)&&(t.fullsize=Js.fullsize),t.breakpoint&&-1===m.available.indexOf(t.breakpoint)&&(t.breakpoint=Js.breakpoint),t}function ta(t,e){this.settings=E.mergeSettings(t,e,Js),this.settings=Zs(this.settings),this.originalElement=$(t),this.element=$(t),this.init(),this.reStructure()}ta.prototype={get isCAP(){return this.element.is(".contextual-action-panel")},get capAPI(){var t;return this.trigger&&this.trigger.length?t=this.trigger.data("contextualactionpanel"):this.mainContent&&this.mainContent.length&&this.mainContent.is("body")&&(t=this.mainContent.data("contextualactionpanel")),t},get isAttachedToBody(){return this.trigger.length&&this.trigger.is("body")},get visible(){return this.element.is(".is-visible")},set visible(t){this.element[0].classList[!0===t?"add":"remove"]("is-visible")},set active(t){this.element[0].classList[!0===t?"add":"remove"]("is-active")},get active(){return this.element[0].classList.contains("is-active")},get isOnTop(){var t=0,e=this.element;return $(".modal.is-visible").each((function(){t<this.style.zIndex&&(t=this.style.zIndex)})),t===e[0].style.zIndex},get currentlyNeedsFullsize(){return"always"===this.settings.fullsize||"responsive"===this.settings.fullsize&&m.isBelow(this.settings.breakpoint)||"responsive"===this.settings.fullsize&&this.isAndroid()&&"phone-to-tablet"===this.settings.breakpoint},get closeBtn(){var t=this.capAPI;return t&&t.element instanceof $?t.closeButton[0]:this.element[0].querySelector(".modal-content > button.btn-close")},init:function(){var t=this;if(this.id=this.element.attr("id")||parseInt($(".modal").length,10)+1,this.namespace="".concat(Ks,"-").concat(this.id),this.trigger=$('[data-modal="'.concat(this.element.attr("id"),'"]')),this.element.is("body")&&(this.trigger=this.element),this.oldActive=this.settings.triggerButton?this.useJqEl(this.settings.triggerButton):this.trigger,"click"!==this.settings.trigger||this.isAttachedToBody||this.trigger.on("click.".concat(t.namespace),(function(e){$(e.currentTarget).is(t.trigger)&&t.open()})),"immediate"===this.settings.trigger){var e=new X({duration:1,timeoutCallback:function(){t.open()}});J.register(e)}if(t.isCancelled=!1,this.settings.content){this.settings.trigger=this.settings.content instanceof jQuery?this.settings.trigger:"immediate",this.appendContent();var i=new X({duration:1,timeoutCallback:function(){t.open()}});J.register(i)}else{if(this.settings.beforeShow)return this.settings.trigger=this.settings.content instanceof jQuery?this.settings.trigger:"immediate",this.appendContent(),void this.callSource();this.isCAP?this.addButtons(this.settings.buttons):this.renderButtonset(),this.element[0].classList[this.settings.hideUnderneath?"add":"remove"]("hide-underneath"),this.registerModal(),this.element.appendTo("body"),this.element[0].style.display="none"}},appendContent:function(){var t=this,e=!1,i=this.settings.maxWidth?' style="max-width: '.concat(this.settings.maxWidth,'px;"'):"",n=$(this.settings.content).hasClass("splitter-container");if(this.element=$('\n      <div class="modal">\n        <div class="modal-content"'.concat(i,'>\n          <div class="modal-header ').concat(n?"splitter-header":"",'"><h1 class="modal-title"></h1></div>\n          <div class="modal-body-wrapper ').concat(n?"splitter-wrapper":"",'">\n            <div class="modal-body').concat(this.settings.forceOverflow?" overflow":"",'"></div>\n          </div>\n        </div>\n      </div>\n    ')),this.settings.showCloseBtn&&!this.isCAP){var s=$('\n        <button type="button" class="btn-icon btn-close" title="'.concat(H.translate(this.settings.closeBtnOptions.closeBtnTooltip,{showAsUndefined:!1,showBrackets:!1}),'" aria-hidden="true">\n          ').concat($.createIcon("close"),'\n          <span class="audible">').concat(H.translate("Close"),"</span>\n        </button>\n      "));$(window).width()>=400&&this.element.addClass("has-close-btn"),this.element.find(".modal-content").append(s),s.on("click.".concat(this.namespace),(function(){return t.close()})).tooltip(),this.settings.closeBtnOptions.attributes&&E.addAttributes(s,this,this.settings.closeBtnOptions.attributes,"",!0)}if(this.settings.id&&(this.element.attr("id",this.settings.id),E.addAttributes(this.element,this,this.settings.attributes,"",!0)),$(this.settings.content).is(".modal")?(this.element=$(this.settings.content),e=this.element.parent().hasClass("modal-wrapper")):this.settings.content&&this.settings.content.length>0&&(this.settings.content instanceof jQuery&&this.settings.content.parent().is(".modal-body")?(e=!0,this.element=this.settings.content.closest(".modal")):this.element.find(".modal-body").append(this.settings.content),this.settings.content instanceof jQuery&&!this.settings.beforeShow&&(this.settings.content.removeClass("hidden is-hidden"),this.settings.content.show())),this.settings.beforeShow&&(this.busyIndicator=$('<div class="overlay busy"></div><div class="busy-indicator-container blocked-ui" aria-live="polite" role="status"><div class="busy-indicator active"><div class="bar one"></div><div class="bar two"></div><div class="bar three"></div><div class="bar four"></div><div class="bar five"></div></div><span>Loading...</span></div>'),$("body").append(this.busyIndicator)),e||this.element.appendTo("body"),this.settings.cssClass&&this.element.addClass(this.settings.cssClass),this.settings.title&&(this.element.find(".modal-title")[0].innerHTML=g.stripTags(this.settings.title,"<div><span><a><small><img><svg><i><b><use><br><strong><em>")),e||(this.isCAP?this.addButtons(this.settings.buttons):this.renderButtonset()),this.registerModal(),this.settings.icon){var a=this.settings.iconClass?" icon-".concat(this.settings.iconClass):"",o=$('<svg class="icon'.concat(a,'"\n        focusable="false" aria-hidden="true" role="presentation">\n        <use href="#icon-').concat(this.settings.icon,'"></use>\n      </svg>'));this.element.find(".modal-title svg.icon").length>0||this.element.find(".modal-title").prepend(o)}E.fixSVGIcons(this.element)},registerModal:function(){this.originalElement.is(this.element)||(this.originalElement.data("modalElementLink",this.element[0]),this.element.data("modal",this)),at.register(this)},reStructure:function(){var t=$(".modal-body",this.element),e=$("hr:first-child",t),i=$(".modal-buttonset",this.element);t&&t.length&&!t.parent().hasClass("modal-body-wrapper")&&t.wrap('<div class="modal-body-wrapper"></div>'),e&&e.length&&!e.parent().hasClass("modal-content")&&e.insertAfter(this.element.find(".modal-header")),i&&i.length&&!i.parent().hasClass("modal-content")&&i.insertAfter(this.element.find(".modal-body-wrapper"))},disableSubmit:function(){var t=this.element,e=t.find(".modal-buttonset button").filter(".btn-modal-primary").not(".no-validation"),i=t.find("select.dropdown[data-validate]"),n=t.find("[data-validate]:visible");if(i.each((function(){$(this).next(".dropdown-wrapper").is(":visible")&&(n=n.add(this))})),n.length>0){e.removeAttr("disabled");var s=!0;n.each((function(){var t=$(this);if(!(t.closest(".datagrid-filter-wrapper").length>0)){var i=t.is(".dropdown")&&t.next(".dropdown-wrapper").is(":visible")||null!==t[0].offsetParent;t.is(".required")?i&&!t.val()&&(s=!1):(t.validateField(),i&&!t.isValid()&&(s=!1)),i&&t.is(".error")&&(s=!1),s&&e.removeAttr("disabled")}})),s||e.is(":disabled")||e.attr("disabled","true")}},renderButtonset:function(){var t=this,e=this.settings.buttons,i={style:"modal"},n=this;if(!this.buttonsetElem){var s=this.element.find(".modal-buttonset");if(s&&s.length)i.detectHTMLButtons=!0;else{if(!Array.isArray(e)||!e.length)return;var a=this.element.find(".modal-body").parent(),o=this.settings.useFlexToolbar?"toolbar-section ":"",r='<div class="'.concat(o,'modal-buttonset"></div>');(s=$(r)).insertAfter(a),i.buttons=e}this.buttonsetElem=s,s.buttonset(i),this.buttonsetAPI=s.data("buttonset"),e=this.buttonsetAPI.toData().buttons}var l=this.buttonsetAPI.buttons,c=$(),d=100/l.length;l.forEach((function(i,s){var a=Array.isArray(e)&&e.length?e[s]:void 0,o=!1;if(a)if(i.element[0].setAttribute("id",e[s].id||E.uniqueId(i.element,"button","modal")),e[s].tabindex&&i.element[0].setAttribute("tabindex",e[s].tabindex),$(i.element).on("click.".concat(n.namespace),(function(t){var e=a.click;e&&(e.apply(n.element[0],[t,n]),o=!0)})),$(i.element).not(["[data-ng-click]","[ng-click]","[onclick]",":submit",".btn-menu",".btn-actions",".colorpicker",".fontpicker"].join(", ")).on("click.".concat(n.namespace),(function(e){o||($(e.target).is(".btn-cancel")&&(n.isCancelled=!0),0===t.openSubComponents.length&&n.close())})),a.validate&&i.element[0].classList.add("no-validation"),a.attributes)E.addAttributes(i.element,t,a.attributes,"",t);else if(i.element.is("[class*=primary]"))E.addAttributes(i.element,t,n.settings.attributes,"primary",!0);else if(i.element.is("[class*=cancel]")||i.element.text()===H.translate("Cancel"))E.addAttributes(i.element,t,n.settings.attributes,"cancel",!0);else{var r;E.addAttributes(i.element,t,n.settings.attributes,null===(r=i.element)||void 0===r||null==r?void 0:r.prop("id"),!0)}i.element[0].style.width="".concat(d,"%"),c.add(i)}))},addButtons:function(t){var e,i=this,n=this,s=this.element.find(".modal-body").parent(),a=this.element.find(".flex-toolbar"),o=100,r=!1;if(this.modalButtons=t,!t){var l=this.element.find(".modal-buttonset button");o=100/l.length;for(var c=0,d=l.length;c<d;c++)l[c].style.width="".concat(o,"%");return l.button(),void l.not("[data-ng-click], [ng-click], [onclick], :submit").on("click.".concat(n.namespace),(function(t){$(t.target).is(".btn-cancel")&&(n.isCancelled=!0),0===i.openSubComponents.length&&n.close()}))}this.isCAP?(r=!0,e=this.element.find(".buttonset")):(e=this.element.find(".modal-buttonset")).length||(e=$('<div class="modal-buttonset"></div>').insertAfter(s)),o=100/t.length,t&&e.empty();for(var h=function(i,s){var l=this,c=$('<button type="button">\n        <span></span>\n      </button>'),d=c.find("span");d.text(i.text),c.attr("type",i.type||"button"),"separator"===i.cssClass&&(c=$('<div class="separator"></div>')),i.cssClass?c.attr("class",i.cssClass):i.isDefault?c.addClass("btn-modal-primary"):c.addClass("btn-modal"),i.disabled&&(c[0].disabled=!0===i.disabled),i.audible&&d.addClass("audible"),void 0===i.validate||i.validate||c.addClass("no-validation"),E.addAttributes(c,n,i.attributes,"",!0);for(var h={},u=["id","name","text","tabindex"],p=0;p<u.length;p++)i[u[p]]&&(h[u[p]]=i[u[p]]);if("input"===i.type){var g=$('<label class="audible" for="filter">'.concat(i.text,"</label>")),f=$('<input class="searchfield">').attr(h);return a.length?a.find(".toolbar-section.search").append(g,f):e.append(g,f),void f.searchfield(i.searchfieldSettings)}i.icon&&"#"===i.icon.charAt(0)&&$.createIconElement({classes:["#icon-close"===i.icon?"icon-close":""],icon:i.icon.substr(6)}).prependTo(c),c[0].setAttribute("id",i.id||E.uniqueId(n.element,"button","modal"));var m=t[s].click;c.on("click.".concat(n.namespace),(function(t){m?m.apply(n.element[0],[t,n]):0===l.openSubComponents.length&&n.close()})),r||(c[0].style.width="".concat(o,"%")),c.button(),(n.settings.useFlexToolbar||n.settings.centerTitle)&&i.align?("left"===i.align&&a.find(".toolbar-section").eq(0).append(c),"center"===i.align&&a.find(".toolbar-section").eq(1).find("h2").append(c),"right"===i.align&&a.find(".toolbar-section").eq(2).append(c)):e.append(c)},u=0;u<t.length;u++)h(t[u],u)},isAndroid:function(){return"android"===(x&&x.os&&x.os.name?x.os.name:"")},sizeInner:function(){var t=this.element.find(".detailed-message"),e=$(window).height()-t.offset().top-150;t[0].style.maxHeight="".concat(e,"px"),t[0].style.overflow="auto",t[0].style.width="".concat(t.width(),"px")},callSource:function(){if("function"==typeof this.settings.beforeShow){var t=this;this.settings.beforeShow((function(e){return!1!==e&&($("#modal-busyindicator").trigger("complete.busyindicator"),!0===e?(t.busyIndicator&&(t.busyIndicator.remove(),delete t.busyIndicator),!0):(e instanceof jQuery||(e=$(e)),t.open(!0),t.element.find(".modal-body").empty(),t.element.find(".modal-body").append(e),e.show(),!0))}),{})}},open:function(t){var e,i=this;$("#validation-errors, #tooltip, #validation-tooltip").addClass("is-hidden"),this.busyIndicator&&(this.busyIndicator.remove(),delete this.busyIndicator),this.settings.triggerButton?this.oldActive=this.useJqEl(this.settings.triggerButton):this.trigger&&0!==this.trigger.length&&!this.trigger.is("body")||(this.oldActive=$(":focus")),this.overlay||(this.overlay=$(at.overlayElem)),this.root||(this.root=$(at.rootElem));var n=this.element.parent();if(n.is(".modal-wrapper")||(this.element.wrap('<div class="modal-wrapper"></div>'),n=this.element.parent()),n.parent().is(this.root)||n.insertAfter(this.overlay),1===this.element.find(".detailed-message").length&&($("body").on("resize.".concat(this.namespace),(function(){i.sizeInner()})),this.sizeInner()),e=this.element.triggerHandler("beforeopen",[this]),this.settings.isAlert||$("body").triggerHandler("beforeopen",[this]),this.isCancelled=!1,!1!==e&&(t||(this.callSource(),!this.settings.beforeShow))){var s=$(".resize-app-menu-container");if(s.length>0&&s.append(this.root),at.currentlyActive=this,this.settings.isAlert){var a=this.element.find("#message-title");E.addAttributes(a,this,this.settings.attributes,"title",!0);var o=this.element.find("#message-text");E.addAttributes(o,this,this.settings.attributes,"message",!0),E.addAttributes(this.element,this,this.settings.attributes,"modal",!0),this.element.attr("aria-labelledby",a.attr("id")),this.element.attr("aria-describedby",o.attr("id"))}else{var r=this.element.find("h1:first");E.addAttributes(r,this,this.settings.attributes,"title",!0),E.addAttributes(this.element.find(".btn-close"),this,this.settings.attributes,"btn-close",!0);var l=r.attr("id");l||(l="".concat(this.element.attr("id")?this.element.attr("id"):"h1","-title"),r.attr("id",l));var c=this.element.find(".modal-body"),d="".concat(this.element.attr("id")?this.element.attr("id"):"message","-text");this.element.attr("aria-labelledby",l),E.addAttributes(this.element,this,this.settings.attributes,"",!0),this.element.find(".toolbar .title").length?(this.element.find(".toolbar .title").attr("id",d),this.element.attr("aria-describedby",d)):(c.attr("id",d),this.element.attr("aria-describedby",d))}this.mainContent=$("body").children(".scrollable-container"),this.mainContent.length||(this.mainContent=$("body")),this.removeNoScroll=!this.mainContent.hasClass("no-scroll"),this.mainContent.addClass("no-scroll"),$("body").on("resize.".concat(this.namespace),(function(){i.resize()})),this.element[0].style.display="";var h=this,u=new X({duration:1,timeoutCallback:function(){h.resize(),h.element.attr("role",h.settings.isAlert?"alertdialog":"dialog"),h.element.attr("aria-modal","true")}}),p=new X({duration:at.modalFadeDuration,timeoutCallback:function(){h.visible=!0}});J.register(u),J.register(p),$(this.element).off("keypress.".concat(this.namespace)).on("keypress.".concat(this.namespace),(function(t){var e=$(t.target);e.is(".editor, .searchfield, textarea, :button")||e.closest(".tab-list").length||$("#dropdown-list").length||13===t.which&&i.isOnTop&&!e.closest("form").find(":submit").length&&i.element.find(".btn-modal-primary:enabled").length&&(t.stopPropagation(),t.preventDefault(),h.settings.suppressEnterKey||e.hasClass("fileupload")&&!e.hasClass("colorpicker")||i.element.find(".btn-modal-primary:enabled").trigger("click"))})),$(".skip-link").on("focus.".concat(this.namespace),(function(t){t.preventDefault();var e=i.element.find(":focusable").first();e.focus(),"function"==typeof h.settings.onFocusChange&&h.settings.onFocusChange(h,e[0])})),this.element.find(".paginated").on("afterpaging.".concat(this.namespace),(function(){i.resize()}));var g=new X({duration:5,timeoutCallback:function(){h.disableSubmit()}});J.register(g),this.element.find("[data-validate]").removeClass("disable-validation");var f=new X({duration:20,timeoutCallback:function(){var t;t=h,h.element.trigger("open",[t]),function(t){if(h.settings.autoFocus){h.changeObserver=new MutationObserver((function(){h.setFocusableElems()})),h.changeObserver.observe(h.element[0],{childList:!0,subtree:!0}),h.setFocusableElems();var e,i,n=$(h.focusableElems).not(":hidden").first();if(0===n.length&&(n=t.element.find(".btn-modal-primary")),1===n.length&&n.is(".btn-modal")&&(n=t.element.find(".btn-modal-primary")),h.buttonsetAPI&&0===t.buttonsetElem.find(".btn-modal-primary").length){var s=$(".modal-buttonset button")[0];n=t.buttonsetElem.find(s)}if(n.is(".modal-wrapper a")&&(n=t.element.find(".modal-content .message a").removeClass("hide-focus")),n.is(".tab:not(.is-selected) a")&&(e=(i=n.closest(".tab-container")).find(".is-selected")).length)return n=e,void i.data("tabs").select(e.children("a").attr("href"));n.is("input")&&n.closest(".modal-header").length>0||n.focus(),"function"==typeof h.settings.onFocusChange&&h.settings.onFocusChange(h,n[0])}}(h)}});J.register(f);var m=new X({duration:30,timeoutCallback:function(){h.element.trigger("afteropen")}});J.register(m)}},resize:function(){var t,e,i=this.currentlyNeedsFullsize;i?this.element[0].classList.add("display-fullsize"):(this.element[0].classList.remove("display-fullsize"),t=.9*$(window).height()-this.settings.frameHeight,e=1*$(window).width()-this.settings.frameWidth);var n=this.element.find(".modal-body-wrapper");if(n.length&&(i?(n[0].style.maxHeight="",n[0].style.maxWidth=""):(n[0].style.maxHeight="".concat(t,"px"),n[0].style.maxWidth="".concat(e,"px"))),this.element.hasClass("lookup-modal")){var s=this.element.find(".datagrid-wrapper"),a=this.element.find(".pager-toolbar"),o=s.closest(".datagrid-container");t=t-(o.prev().is(".toolbar")?130:67)-(o.next().is(".pager-toolbar")?35:0)+(a.length?-15:0),void 0!==s[0]&&(i?(s[0].style.maxHeight="",s[0].style.maxWidth=""):void 0!==t&&void 0!==e&&(s[0].style.maxHeight="".concat(t,"px"),s[0].style.maxWidth="".concat(e,"px")))}this.element.hasClass("datagrid-columns-dialog")&&(n[0].style.overflow="hidden",t>220?(this.element.find(".modal-body")[0].style.height="",this.element.find(".listview.alternate-bg")[0].style.maxHeight="",this.element.find(".listview.alternate-bg")[0].style.height="",this.element.find(".listview.alternate-bg")[0].style.minHeight=""):(this.element.find(".modal-body")[0].style.height="".concat(t,"px"),this.element.find(".listview.alternate-bg")[0].style.maxHeight="".concat(t-41,"px"),this.element.find(".listview.alternate-bg")[0].style.height="".concat(t-41,"px"),this.element.find(".listview.alternate-bg")[0].style.minHeight=0)),this.settings.showCloseBtn&&!this.isCAP&&($(window).width()<400&&this.element.hasClass("has-close-btn")?this.element.removeClass("has-close-btn"):$(window).width()>=400&&!this.element.hasClass("has-close-btn")&&this.element.addClass("has-close-btn"));var r=this.element.find(".toolbar");r.length&&r.triggerHandler("recalculate-buttons")},isOpen:function(){return V("visible","isOpen"),this.visible},get openSubComponents(){var t=Be(this.element.find("*"),["isOpen","visible"],["datepicker","dropdown","popupmenu","timepicker","tooltip"]),e=[];return t.forEach((function(t){var i=t.control,n="boolean"==typeof i.isPopover&&!0===i.isPopover,s="function"==typeof i.isOpen&&i.isOpen(),a="boolean"==typeof i.isOpen&&!0===i.isOpen,o=!!n&&("boolean"==typeof i.visible&&!0===i.visible);(s||a||o)&&e.push(i)})),e},get isFocused(){var t=!1,e=document.activeElement;this.focusableElems||this.setFocusableElems(),this.focusableElems.forEach((function(i){if(!t){var n,s,a,o,r=$(i);if((r.is($(e))||"function"==typeof i.contains&&i.contains(e))&&(t=!0),r.is("div.dropdown, div.multiselect"))t=null===(n=r.parent().prev("select").data("dropdown"))||void 0===n||null==n?void 0:n.isFocused;if(r.is(".lookup"))t=null===(s=r.data("lookup"))||void 0===s||null==s?void 0:s.isFocused;if(r.is(".btn-menu, .btn-actions"))t=null===(a=r.data("popupmenu"))||void 0===a||null==a?void 0:a.isFocused;if(r.is(".searchfield"))t=null===(o=r.data("searchfield"))||void 0===o||null==o?void 0:o.isFocused}}));var i=$(e).parents(".tooltip, .popover");return i.length&&i.each((function(e,i){var n=i.className.contains("popover")?"popover":"tooltip",s=$(i).data(n);s&&s.isFocused&&(t=!0)})),t},setFocusableElems:function(){var t=f.focusableElems(this.element[0],[],["option"]);this.focusableElems=t,this.focusableElems.first=t[0],this.focusableElems.last=t[t.length-1]},setFocus:function(t){if(-1!==["last","first"].indexOf(t)){var e;switch(this.focusableElems||this.setFocusableElems(),t){case"last":e=this.focusableElems.last;break;case"first":e=this.focusableElems.first}e&&(e.focus(),e.classList.remove("hide-focus"),"function"==typeof this.settings.onFocusChange&&this.settings.onFocusChange(this,e))}},close:function(t,e){var i=arguments.length>3?arguments[3]:void 0;if(!(arguments.length>2&&void 0!==arguments[2]&&arguments[2])&&!this.visible)return!0;if(!1===this.element.triggerHandler("beforeclose".concat(i?".".concat(i):"")))return!1;var n=this;this.element.find("[data-validate]").addClass("disable-validation"),this.changeObserver&&(this.changeObserver.disconnect(),delete this.changeObserver),this.isCAP&&this.element.addClass("is-animating"),this.mainContent&&this.removeNoScroll&&this.mainContent.removeClass("no-scroll"),$("body").off("resize.".concat(this.namespace," focusin.").concat(n.namespace)),this.element.off("keypress.".concat(this.namespace," keydown.").concat(this.namespace)),this.visible=!1,this.active=!1,delete this.dontCheckFocus,i?n.element.trigger("close.".concat(i),n.isCancelled):n.element.trigger("close",n.isCancelled),!this.settings.noRefocus&&this.isFocused&&document.activeElement.blur(),$("#validation-errors, #tooltip, #validation-tooltip").addClass("is-hidden");var s=$(this.closeBtn).data("tooltip");s&&(s.hide(),s.reopenDelay=!0),$(".skip-link").off("focus.".concat(this.namespace));var a=new X({duration:at.modalFadeDuration,timeoutCallback:function(){i?n.element.trigger("afterclose.".concat(i),n.isCancelled):n.element.trigger("afterclose"),s&&delete s.reopenDelay,("immediate"===n.settings.trigger||t)&&(!n.isCAP||n.isCAP&&!n.capAPI)&&n.destroy(i),e||at.refresh(),n.settings.noRefocus||(!n.oldActive&&n.settings.triggerButton&&(n.oldActive=n.useJqEl(n.settings.triggerButton)),n.oldActive&&$(n.oldActive).is("a:visible, button:visible, input:visible, textarea:visible")?n.oldActive.focus():n.trigger.parents(".toolbar, .formatter-toolbar").length<1&&n.trigger.focus())}});return J.register(a),!1},useJqEl:function(t){return t instanceof jQuery?t:$(t)},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element,t,this.settings),this.settings=Zs(this.settings)),"immediate"===this.settings.trigger&&this.open(),this},destroy:function(t){var e=this;function i(){e.trigger.off("click.".concat(e.namespace)),e.element.off("keypress.".concat(e.namespace," keydown.").concat(e.namespace," beforeclose.").concat(e.namespace," afterclose.").concat(e.namespace)),e.element.find("button, .btn-close").off("click.".concat(e.namespace)),e.element.find(".paginated").off("afterpaging.".concat(e.namespace)),$(".skip-link").off("focus.".concat(e.namespace)),$("body").off("resize.".concat(e.namespace)),$(e.element).off("keydown.".concat(e.namespace)),1===e.element.find(".detailed-message").length&&$("body").off("resize.".concat(e.namespace)),e.isCAP&&e.capAPI&&e.capAPI.destroy(),e.aboutAPI&&e.aboutAPI.destroy(!0),e.buttonsetAPI&&(e.buttonsetAPI.buttons.forEach((function(t){$(t.element).off("click.".concat(e.namespace))})),e.buttonsetAPI.destroy(),e.buttonsetElem.remove(),delete e.buttonsetAPI,delete e.buttonsetElem),delete e.focusableElems,e.trigger.off("click.".concat(e.namespace)),e.element[0].removeAttribute("data-modal");var t=e.element.parent(".modal-wrapper");t.length&&t.remove();var i=new X({duration:21,timeoutCallback:function(){var t=null,i=e.element?e.element.data(Ks):null;i?t=e.element[0]:(i=e.trigger?e.trigger.data(Ks):null)&&(t=e.trigger[0]),t&&i&&i.overlay&&$.removeData(t,Ks)}});J.register(i),at.unregister(e),e.originalElement instanceof $&&$.removeData(e.originalElement[0],"modalElementLink"),$.removeData(e.element[0],Ks)}this.element.trigger("beforedestroy".concat(t?".".concat(t):""))&&(this.visible?(this.element.on("afterclose.".concat(e.namespace),(function(){i()})),this.close(!0)):i())}};var ea={title:"Message Title",status:"",message:"Message Summary",width:"auto",maxWidth:null,buttons:null,showCloseBtn:!1,cssClass:null,returnFocus:null,allowedTags:"<a><b><br><br/><del><em><i><ins><mark><small><strong><sub><sup>",audibleLabel:"",overlayOpacity:.7,hideUnderneath:!1,attributes:null,noRefocus:!1};function ia(t,e){this.element=$(t),this.settings=E.mergeSettings(t,e,ea),this.init()}ia.prototype={init:function(){var t,e=this,i=this,n=this.settings.allowedTags,s=!0;if(this.settings.allowedTags.length>0||(s=!1),this.message=$('<div class="modal message" '.concat(this.settings.id?'id="'.concat(this.settings.id,'"'):"","></div>")),this.messageContent=$('<div class="modal-content"></div>'),this.title=$('<h1 class="modal-title" id="message-title">'.concat(s?g.stripTags(this.settings.title,n):g.stripHTML(this.settings.title),"</h1>")).appendTo(this.messageContent).wrap('<div class="modal-header"></div>'),this.content=$('<div class="modal-body"><div class="message" id="message-text">'.concat(s?g.stripTags(this.settings.message,n):g.stripHTML(this.settings.message),"</div></div>")).appendTo(this.messageContent),""!==this.settings.audibleLabel&&this.title.prepend('<span class="audible">'.concat(H.translate(this.settings.audibleLabel),"</span>")),this.element.is("body")||(t=this.element,this.content.empty().append(t.show())),this.message.append(this.messageContent).appendTo("body"),this.message.modal({trigger:"immediate",buttons:this.settings.buttons,resizable:this.settings.resizable,close:this.settings.close,showCloseBtn:this.settings.showCloseBtn,isAlert:!0,overlayOpacity:this.settings.overlayOpacity,hideUnderneath:this.settings.hideUnderneath,attributes:this.settings.attributes,noRefocus:this.settings.noRefocus}),"auto"!==this.settings.width&&(this.content.closest(".modal")[0].style.maxWidth="none",this.content.closest(".modal")[0].style.width=this.settings.width+(/(px|%)/i.test("".concat(this.settings.width))?"":"px")),this.settings.maxWidth&&(this.content.find(".message")[0].style.maxWidth=this.settings.maxWidth),this.settings.cssClass&&this.message.addClass(this.settings.cssClass),this.settings.showCloseBtn){var a=$('\n        <button type="button" class="btn-icon btn-close" title="'.concat(H.translate("Close"),'" aria-hidden="true">\n          ').concat($.createIcon("close"),'\n          <span class="audible">').concat(H.translate("Close"),"</span>\n        </button>\n      "));this.message.find(".modal-content").append(a),a.on("click",(function(){return e.destroy()}))}this.message.on("beforeclose.message",(function(){return i.element.triggerHandler("beforeclose")})).on("beforeopen.message",(function(){return i.element.triggerHandler("beforeopen")})).on("open.message",(function(){i.element.trigger("open")})).on("afterclose.message",(function(){i.destroy(),i.settings.returnFocus&&i.settings.returnFocus.focus(),$(document).off("keypress.message keydown.message")})),$(document).on("keypress.message keydown.message",(function(t){27===(t.which||t.keyCode)&&setTimeout((function(){var t=i.message.data("modal");void 0!==t&&t.close()}),0)})),"error"===this.settings.status?this.title.addClass("has-status is-error").prepend($.createIconElement("error")):"alert"===this.settings.status?this.title.addClass("has-status is-alert").prepend($.createIconElement("alert")):"success"===this.settings.status?this.title.addClass("has-status is-success").prepend($.createIconElement("success")):"info"===this.settings.status?this.title.addClass("has-status is-info").prepend($.createIconElement("info")):this.title.removeClass("has-status is-error is-alert is-success is-info").find("svg").remove()},destroy:function(){var t=this.message.data("modal");void 0!==t&&t.destroy(),this.message.off("beforeclose.message beforeopen.message open.message afterclose.message").remove()}},$.fn.message=function(t){return this.each((function(){return new ia(this,t)}))},$.fn.modal=function(t){return this.each((function(){var e=$.data(this,Ks),i=$(this);if(i.is(".modal")||(e=i.closest(".modal").data(Ks)),e&&t)return e.updated(t),void("function"==typeof e[t]&&e[t]());e&&!t||(e=$.data(this,Ks,new ta(this,t)))}))},$.fn.monthview=function(t){return this.each((function(){var e=$.data(this,kn);e?e.updated(t):e=$.data(this,kn,new Sn(this,t))}))};var na="multiselect",sa={closeOnSelect:!1,empty:!0,filterMode:"contains",maxSelected:void 0,moveSelected:"all",multiple:!0,showEmptyGroupHeaders:!1,showSelectAll:!1,showTags:!1,source:void 0,allTextString:null,selectedTextString:null};function aa(t,e){this.settings=E.mergeSettings(t,e,sa),this.element=$(t),this.init()}aa.prototype={init:function(){this.build()},build:function(){return this.element.dropdown(this.settings),this.dropdown=this.element.data("dropdown"),this},enable:function(){this.dropdown.enable()},disable:function(){this.dropdown.disable()},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.build()},destroy:function(){this.dropdown.destroy(),this.element.off(),$.removeData(this.element[0],na)}},$.fn.multiselect=function(t){return this.each((function(){var e=$.data(this,na);e?e.updated(t):e=$.data(this,na,new aa(this,t))}))};var oa=[!1,"collapsed","expanded"],ra='<svg\n  class="icon-custom"\n  focusable="false"\n  aria-hidden="true"\n  width="32"\n  height="32"\n  viewBox="0 0 32 32"\n  fill="none"\n  xmlns="http://www.w3.org/2000/svg">\n  <rect width="32" height="32" rx="8" fill="#1C86EF"/>\n  <path d="M7.8315 20V11.2727H13.5076V12.598H9.41246V14.9673H13.2136V16.2926H9.41246V18.6747H13.5417V20H7.8315ZM15.0524 20V13.4545H16.5268V14.5668H16.6035C16.7399 14.1918 16.9657 13.8991 17.2811 13.6889C17.5964 13.4759 17.9728 13.3693 18.4103 13.3693C18.8535 13.3693 19.2271 13.4773 19.5311 13.6932C19.8379 13.9062 20.0538 14.1974 20.1788 14.5668H20.247C20.3919 14.2031 20.6362 13.9134 20.9799 13.6974C21.3265 13.4787 21.737 13.3693 22.2115 13.3693C22.8137 13.3693 23.3052 13.5597 23.6859 13.9403C24.0666 14.321 24.2569 14.8764 24.2569 15.6065V20H22.7101V15.8452C22.7101 15.4389 22.6021 15.142 22.3862 14.9545C22.1703 14.7642 21.9061 14.669 21.5936 14.669C21.2214 14.669 20.9302 14.7855 20.72 15.0185C20.5126 15.2486 20.4089 15.5483 20.4089 15.9176V20H18.8961V15.7812C18.8961 15.4432 18.7939 15.1733 18.5893 14.9716C18.3876 14.7699 18.1234 14.669 17.7967 14.669C17.5751 14.669 17.3734 14.7259 17.1916 14.8395C17.0098 14.9503 16.8649 15.108 16.7569 15.3125C16.649 15.5142 16.595 15.75 16.595 16.0199V20H15.0524Z" fill="white"/>\n</svg>',la=function(){return ra},ca=function(t){return oa.includes(t)},da=function(t,e){e.classList.remove("mode-collapsed","mode-expanded"),ca(t)&&t&&e.classList.add("mode-".concat(t))},ha=function(t,e,i){var n;"collapsed"===e?$(t).tooltip({placementOpts:{x:16},placement:"right",title:(i||t).textContent.trim()}):null===(n=$(t).data("tooltip"))||void 0===n||null==n||n.destroy()},ua="modulenavswitcher",pa={displayMode:oa[0],generate:!0,icon:la,noSearch:!1,changeIconOnSelect:!0,moduleButtonText:"Standard Module",roleDropdownLabel:"Roles",roles:[]};function ga(t,e){this.settings=E.mergeSettings(t,e,pa),this.element=$(t),this.init()}ga.prototype={get accordionEl(){var t;return null===(t=this.element.parents(".accordion").first())||void 0===t||null==t?void 0:t[0]},get accordionAPI(){return this.accordionEl?$(this.accordionEl).data("accordion"):void 0},get containerEl(){return this.element.parents(".module-nav-container")[0]},get moduleButtonContainerEl(){return this.element[0].querySelector(".module-nav-section.module-btn")},get moduleButtonEl(){return this.moduleButtonContainerEl.querySelector("button")},get moduleButtonIconEl(){var t;return null===(t=this.moduleButtonEl)||void 0===t||null==t?void 0:t.querySelector("svg, img, .icon, .custom-icon")},get moduleButtonAPI(){return this.moduleButtonEl?$(this.moduleButtonEl).data("button"):void 0},get roleDropdownContainerEl(){return this.element[0].querySelector(".module-nav-section.role-dropdown")},get roleDropdownEl(){return this.element[0].querySelector(".dropdown")},get roleDropdownAPI(){return this.roleDropdownEl?$(this.roleDropdownEl).data("dropdown"):void 0},init:function(){return this.build().handleEvents()},build:function(){return this.renderChildComponents(),this.setDisplayMode(this.settings.displayMode),this.moduleButtonIconEl&&this.settings.icon===la||this.setModuleButtonIcon(),$(this.roleDropdownEl).find("option").length||this.renderDropdownOptions(!0),this},handleEvents:function(){var t=this,e=this;return this.element.on("updated.".concat(ua),(function(){e.updated()})),this.element.on("keydown.".concat(ua),(function(e){var i,n,s=e.key;"ArrowUp"!==s?"ArrowDown"===s&&(null===(i=t.accordionAPI)||void 0===i||null==i||i.nextHeader(t.element)):null===(n=t.accordionAPI)||void 0===n||null==n||n.prevHeader(t.element)})),this},renderChildComponents:function(){this.renderModuleButton(),this.renderRoleDropdown()},renderModuleButton:function(){var t;!this.settings.generate||this.moduleButtonContainerEl&&this.moduleButtonEl||this.element[0].insertAdjacentHTML("afterbegin",(t=this.settings.moduleButtonText,'<div class="module-nav-section module-btn">\n  <button id="module-nav-homepage-btn" class="btn-icon btn-tertiary">\n    '.concat(ra,"\n    <span>").concat(t||(H?H.translate("ModuleSwitch"):"Switch Modules"),"</span>\n  </button>\n</div>"))),this.moduleButtonEl&&($(this.moduleButtonEl).button({ripple:!1}),ha(this.element,this.settings.displayMode,this.moduleButtonEl))},renderRoleDropdown:function(){var t,e=this;!this.settings.generate||this.roleDropdownContainerEl&&this.roleDropdownEl||this.element[0].insertAdjacentHTML("beforeend",(t=this.settings.roleDropdownLabel,'<div class="module-nav-section role-dropdown">\n  <label for="module-nav-role-switcher" class="label audible">'.concat(t||"Roles",'</label>\n  <select id="module-nav-role-switcher" name="module-nav-role-switcher" class="dropdown"></select>\n</div>'))),$(this.roleDropdownEl).find("option").length||this.renderDropdownOptions(),$(this.roleDropdownEl).dropdown({cssClass:"role-dropdown",dropdownIcon:"expand-all",extraListWrapper:!0,width:"parent",widthTarget:".module-nav-switcher",noSearch:this.settings.noSearch}).off("change.module-nav").on("change.module-nav",(function(t){e.settings.changeIconOnSelect&&e.selectRole(t.currentTarget.value)}))},renderDropdownOptions:function(){var t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];Array.isArray(this.settings.roles)&&this.settings.roles.length&&this.setRoles(this.settings.roles,t)},setDisplayMode:function(t){ca(t)&&(this.settings.displayMode!==t&&(this.settings.displayMode=t),da(t,this.element[0]))},setModuleButtonIcon:function(t){var e,i,n="";switch(l(this.settings.icon)){case"function":n=this.settings.icon(this);break;case"string":M.isValidURL(this.settings.icon)?(e=this.settings.icon,i=this.settings.moduleButtonText,n='<img src="'.concat(e,'" alt="').concat(i||"Module Button Icon",'" class="custom-icon" aria-hidden="true" role="presentation" />')):n="<"!==this.settings.icon.charAt(0)&&document.querySelector("symbol#icon-".concat(this.settings.icon))?function(t){return'<svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n  <use href="#icon-'.concat(t,'"></use>\n</svg>')}(this.settings.icon):this.settings.icon}if(t&&(n=t),n.length){var s=this.moduleButtonIconEl;null==s||null==s||s.remove(),this.moduleButtonEl.insertAdjacentHTML("afterbegin",n)}},setRoles:function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(Array.isArray(t)&&this.roleDropdownEl){var i,n="";if(this.roleDropdownEl&&(this.roleDropdownEl.innerHTML=""),t.length&&t.forEach((function(t){var e,i,s,a;n+=(e=t.value,i=t.text,s=t.icon,a=t.iconColor,'<option\n  value="'.concat(e,'"\n  ').concat(s?" data-icon=\"{icon: '".concat(s,"'").concat(a?", colorOver: '".concat(a,"'"):""):"",'}">').concat(i,"</option>"))})),n.length)if(null===(i=this.roleDropdownEl)||void 0===i||null==i||i.insertAdjacentHTML("afterbegin",n),e){this.roleDropdownAPI.updated();var s=this.roleDropdownAPI.selectedValues;s.length&&this.selectRole(s[0])}}},selectRole:function(t){var e,i=this.roleDropdownAPI.listIcon.items;null!=i&&(null==i?void 0:i.length)&&(i.forEach((function(i){e||i.value!==t||(e=i.html)})),e&&this.setModuleButtonIcon(e))},toggleModuleButtonFocus:function(t){var e,i;if(this.moduleButtonEl){var n=(null===(e=document.activeElement)||void 0===e||null==e?void 0:e.isEqualNode(this.moduleButtonEl))||(null===(i=this.moduleButtonEl)||void 0===i||null==i?void 0:i.classList.contains("is-focused")),s="boolean"==typeof t?t:!n;this.moduleButtonEl.classList[s?"add":"remove"]("is-focused")}},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown().init()},teardown:function(){var t,e;(this.element.off("updated.".concat(ua," keydown.").concat(ua)),this.moduleButtonEl)&&(null===(t=$(this.moduleButtonEl).data("button"))||void 0===t||null==t||t.destroy());this.roleDropdownEl&&(null===(e=$(this.roleDropdownEl).data("dropdown"))||void 0===e||null==e||e.destroy());return this},destroy:function(){this.teardown(),$.removeData(this.element[0],ua)}},$.fn.modulenavswitcher=function(t){return this.each((function(){var e=$.data(this,ua);e?e.updated(t):e=$.data(this,ua,new ga(this,t))}))};var fa="modulenav",ma={accordionSettings:{expanderDisplay:"classic",tooltipStyle:"dark"},displayMode:oa[0],filterable:!1,initChildren:!0,pinSections:!1,showDetailView:!1,mobileBehavior:!0,breakpoint:"phone-to-tablet",showOverlay:!0,enableOutsideClick:!1,autoCollapseOnMobile:!0},va=function(t,e){var i=!1;return t instanceof HTMLElement&&(void 0===e&&t.scrollHeight>t.clientHeight||!0===e?(t.classList.add("has-scrollbar"),i=!0):t.classList.remove("has-scrollbar")),i};function ba(t,e){this.settings=E.mergeSettings(t,e,ma),this.element=$(t),this.init()}ba.prototype={previousDisplayMode:!1,get accordionAPI(){return this.accordionEl?$(this.accordionEl).data("accordion"):void 0},get containerEl(){var t;return null===(t=this.element[0])||void 0===t||null==t?void 0:t.parentElement},get pageContainerEl(){return this.element.next(".page-container")},get switcherAPI(){return this.switcherEl?$(this.switcherEl).data("modulenavswitcher"):void 0},get searchAPI(){return this.searchEl?$(this.searchEl).data("searchfield"):void 0},get settingsAPI(){return this.settingsEl?$(this.settingsEl).data("modulenavsettings"):void 0},init:function(){return this.build().handleEvents()},build:function(){return this.renderChildComponents(),this.setDisplayMode(this.settings.displayMode),this.setPinSections(this.settings.pinSections),this.setScrollable(),this.setShowDetailView(this.settings.showDetailView),this.configureResize(),this},renderChildComponents:function(){this.detailViewEl=this.element[0].querySelector(".module-nav-detail"),this.switcherEl=this.element[0].querySelector(".module-nav-switcher"),this.itemMenuEl=this.element[0].querySelector(".module-nav-main"),this.settingsEl=this.element[0].querySelector(".module-nav-settings"),this.footerEl=this.element[0].querySelector(".module-nav-footer"),this.accordionEl=this.element[0].querySelector(".accordion"),this.searchEl=this.element[0].querySelector(".searchfield"),this.renderSeparators(),this.settings.initChildren&&(this.switcherAPI||$(this.switcherEl).modulenavswitcher({displayMode:this.settings.displayMode}),this.settingsAPI||$(this.settingsEl).modulenavsettings({displayMode:this.settings.displayMode}),this.accordionAPI||$(this.accordionEl).accordion(this.settings.accordionSettings)),this.accordionEl&&this.configureAccordion(),this.searchEl&&this.configureSearch()},renderSeparators:function(){var t,e;(this.switcherEl||this.searchEl)&&(null===(t=this.itemMenuEl)||void 0===t||null==t||t.insertAdjacentHTML("beforebegin",'<div class="module-nav-separator">&nbsp;</div>')),this.footerEl&&(null===(e=this.footerEl)||void 0===e||null==e||e.insertAdjacentHTML("beforebegin",'<div class="module-nav-separator">&nbsp;</div>'))},handleEvents:function(){var t=this;return this.element.on("updated.".concat(fa),(function(){t.updated()})),this.accordionEl&&($(this.accordionEl).on("rendered.".concat(fa),(function(){t.configureAccordion()})),$(this.accordionEl).on("beforeexpand.".concat(fa),(function(e){return e.preventDefault(),"expanded"===t.settings.displayMode})),$(this.accordionEl).on("afterexpand.".concat(fa),(function(){t.setScrollable()})),$(this.accordionEl).on("aftercollapse.".concat(fa),(function(){t.setScrollable()})),$(this.accordionEl).on("selected.".concat(fa),(function(e,i){$(i).is("#module-nav-settings-btn")||t.handleAutoCollapseOnMobile()})),$(this.accordionEl).on("followlink.".concat(fa),(function(e,i){$(i).is("#module-nav-settings-btn")||t.handleAutoCollapseOnMobile()}))),$("body").on("resize.".concat(fa),b((function(){t.checkMobileBehavior()}),0)),this.settings.mobileBehavior||this.enableOutsideClickEvent(),this},enableOutsideClickEvent:function(){var t=this;this.settings.enableOutsideClick&&"expanded"===this.settings.displayMode&&$(this.containerEl).on("click.".concat(fa),(function(e){t.checkOutsideClick(e.target)}))},checkMobileBehavior:function(){if(this.settings.mobileBehavior){if("expanded"===this.settings.displayMode){var t=this.pageContainerEl.find(".page-overlay");return m.isAbove(this.settings.breakpoint)?(this.element[0].classList.remove("show-shadow"),this.pageContainerEl[0].classList.add("has-module-nav-offset"),this.settings.enableOutsideClick=!1,this.removeOutsideClickEvent(),void t.remove()):(this.element[0].classList.add("show-shadow"),this.pageContainerEl[0].classList.remove("has-module-nav-offset"),0===t.length&&this.settings.showOverlay&&$('<div class="page-overlay"></div>').appendTo(this.pageContainerEl),this.settings.enableOutsideClick=!0,void this.enableOutsideClickEvent())}"collapsed"!==this.settings.displayMode||this.isLargerThanBreakpoint()?"collapsed"===this.settings.displayMode&&this.isLargerThanBreakpoint()&&da("collapsed",this.containerEl):da(!1,this.containerEl)}},removeOutsideClickEvent:function(){$(this.containerEl).off("click.".concat(fa))},checkOutsideClick:function(t){if("expanded"===this.settings.displayMode){var e=t;!e||$(e).is(".application-menu-trigger")||this.element[0].contains(e)||(this.setDisplayMode(!!this.isLargerThanBreakpoint()&&"collapsed"),this.removeOutsideClickEvent())}},handleHamburgerClick:function(){var t=this.settings.displayMode;"expanded"===t&&this.isLargerThanBreakpoint()?this.updated({displayMode:"collapsed"}):"collapsed"===t&&this.isLargerThanBreakpoint()?this.updated({displayMode:"expanded"}):"expanded"!==t||this.isLargerThanBreakpoint()?"collapsed"!==t||this.isLargerThanBreakpoint()?!1===t&&this.updated({displayMode:"expanded"}):this.updated({displayMode:"expanded"}):this.updated({displayMode:!1})},handleSearchfieldInputEvent:function(){Le.handleSearchfieldInputEvent.apply(this,[fa])},collapseAccordionHeaders:function(){var t=this;this.accordonEl&&this.accordionAPI&&u(this.containerEl.querySelectorAll(".accordion-header")).forEach((function(e){$(e).next(".accordion-pane").length&&t.accordionAPI.collapse($(e),!0)}))},configureAccordion:function(){var t=this;if(this.accordionEl&&this.accordionAPI){var e=E.mergeSettings(this.accordionEl,{accordionFocusCallback:function(e,i){e.is(".module-nav-switcher")?"expanded"===t.settings.displayMode?e.find(".dropdown").focus():e.find(".module-btn button").focus():i()}},this.settings.accordionSettings);this.accordionAPI.updated(e);var i=this.accordionEl.querySelectorAll(".accordion-section > .accordion-header, .accordion-section > soho-module-nav-settings > .accordion-header");i.length&&u(i).forEach((function(e){ha(e,t.settings.displayMode)}))}},configureResize:function(){var t=this;"undefined"!=typeof ResizeObserver&&(this.ro||(this.ro=new ResizeObserver((function(){t.setPinSections(t.settings.pinSections),t.setScrollable(),t.checkMobileBehavior()}))),this.accordionEl&&this.ro.observe(this.accordionEl))},configureSearch:function(){var t;this.settings.filterable?(Le.attachFilter.apply(this,[fa]),Le.attachFilterEvents.apply(this,[fa])):$(this.searchEl).searchfield(),this.searchEl.classList.add("module-nav-search"),null===(t=$(this.searchEl).parents(".accordion-section"))||void 0===t||null==t||t[0].classList.add("module-nav-search-container")},setDisplayMode:function(t){var e,i;ca(t)&&("expanded"!==this.settings.displayMode&&(this.previousDisplayMode=this.settings.displayMode),this.settings.displayMode!==t&&(this.settings.displayMode=t),da(t,this.containerEl),$("#validation-errors, #tooltip, #validation-tooltip").addClass("is-hidden"),null===(e=this.switcherAPI)||void 0===e||null==e||e.setDisplayMode(t),null===(i=this.settingsAPI)||void 0===i||null==i||i.setDisplayMode(t),"expanded"!==this.settings.displayMode?this.collapseAccordionHeaders():this.settings.mobileBehavior||this.enableOutsideClickEvent(),this.element.trigger("displaymodechange",[t,this.previousDisplayMode]),this.checkMobileBehavior())},setPinSections:function(t){this.containerEl.classList[t?"add":"remove"]("pinned-optional")},setScrollable:function(){this.settings.pinSections?(this.setMainAccordionScrollable(!1),this.setAccordionSectionsScrollable()):(this.setAccordionSectionsScrollable(!1),this.setMainAccordionScrollable())},setMainAccordionScrollable:function(t){var e=this.accordionEl;va(e,t)},setAccordionSectionsScrollable:function(t){var e,i=!1,n=null===(e=this.accordionEl)||void 0===e||null==e?void 0:e.querySelectorAll(".accordion-section");n&&n.length&&u(n).forEach((function(e){va(e,t)&&(i=!0);var n=e.classList.contains("module-nav-search-container");(e.classList.contains("module-nav-header")||n)&&(t?e.classList.add("next-scrollbar"):e.classList.remove("next-scrollbar"))})),this.containerEl.classList[i?"add":"remove"]("has-section-scrollbars")},setShowDetailView:function(t){var e;this.containerEl.classList[t?"add":"remove"]("show-detail"),null===(e=this.detailViewEl)||void 0===e||null==e||e.classList[t?"add":"remove"]("visible")},isLargerThanBreakpoint:function(){return m.isAbove(this.settings.breakpoint)},handleAutoCollapseOnMobile:function(){this.settings.autoCollapseOnMobile&&(this.userOpened=!1,this.isLargerThanBreakpoint()||this.setDisplayMode(this.previousDisplayMode))},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings),this.switcherAPI&&(this.switcherAPI.settings.displayMode=this.settings.displayMode),this.settingsAPI&&(this.settingsAPI.settings.displayMode=this.settings.displayMode)),this.teardown().init()},teardown:function(){var t,e,i,n;return this.teardownEvents(),Le.teardownFilter.apply(this,[fa]),$(this.element).find(".module-nav-separator").remove(),this.detailViewEl=null,null===(t=this.switcherAPI)||void 0===t||null==t||t.destroy(),this.switcherEl=null,this.itemMenuEl=null,null===(e=this.settingsAPI)||void 0===e||null==e||e.destroy(),this.settingsEl=null,this.footerEl=null,null===(i=this.accordionAPI)||void 0===i||null==i||i.destroy(),this.accordionEl=null,null===(n=this.searchAPI)||void 0===n||null==n||n.destroy(),this.searchEl=null,this},teardownEvents:function(){this.removeOutsideClickEvent(),this.element.off("updated.".concat(fa)),$(this.accordionEl).off("rendered.".concat(fa)),$(this.accordionEl).off("beforeexpand.".concat(fa)),$(this.accordionEl).off("afterexpand.".concat(fa)),$(this.accordionEl).off("aftercollapse.".concat(fa)),$("body").off("resize.".concat(fa))},teardownResize:function(){this.ro&&(this.ro.disconnect(),this.ro=null)},destroy:function(){this.teardown(),this.teardownResize(),$.removeData(this.element[0],fa)}},$.fn.modulenav=function(t){return this.each((function(){var e=$.data(this,fa);e?e.updated(t):e=$.data(this,fa,new ba(this,t))}))};var ya="modulenavsettings",wa={displayMode:!1};function xa(t,e){this.settings=E.mergeSettings(t,e,wa),this.element=$(t),this.init()}xa.prototype={get accordionEl(){var t;return null===(t=this.element.parents(".accordion").first())||void 0===t||null==t?void 0:t[0]},get menuAPI(){return $(this.element).data("popupmenu")||void 0},init:function(){return this.build().handleEvents()},build:function(){return this.renderChildComponents(),this.setDisplayMode(this.settings.displayMode),this.configurePopupMenu(),this},handleEvents:function(){var t=this;return this.element.on("updated.".concat(ya),(function(){t.updated()})),this},renderChildComponents:function(){var t;this.containerEl=this.element.parents(".module-nav-container")[0],this.detectPopupMenu(),null!==(t=this.menuEl)&&void 0!==t&&(null==t?void 0:t.length)||(this.element[0].insertAdjacentHTML("afterend",'<div class="popupmenu-wrapper><ul class="popupmenu"></ul></div>'),this.menuEl=this.element.next(".popupenu-wrapper").find(".popupmenu"))},detectPopupMenu:function(){if(this.menuEl=this.element.next(".popupmenu"),this.element.next(".popupmenu-wrapper").length&&(this.menuEl=this.element.next(".popupenu-wrapper").find(".popupmenu")),!this.menuEl.length){var t=$(".popupmenu-wrapper.module-nav-settings-menu").find(".popupmenu");t.length&&(this.menuEl=t)}},configurePopupMenu:function(){this.menuEl.length||this.detectPopupMenu(),this.element.popupmenu(this.getPopupmenuConfig())},getPopupmenuConfig:function(){var t=this,e={placement:"bottom",containerOffsetX:8,containerOffsetY:8,callback:function(e){return"collapsed"===t.element.closest(".module-nav").data("modulenav").settings.displayMode&&(e.x+=20,e.containerOffsetY=20,e.placement="right"),e}};return{cssClass:"module-nav-settings-menu",offset:{x:0,y:10},placementOpts:e}},setDisplayMode:function(t){ca(t)&&(this.settings.displayMode!==t&&(this.settings.displayMode=t),da(t,this.element[0]),this.configurePopupMenu())},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown().init()},teardown:function(){var t;return this.element.off("updated.".concat(ya)),null===(t=this.popupmenuAPI)||void 0===t||null==t||t.destroy(),this.menuEl=null,this},destroy:function(){this.teardown(),$.removeData(this.element[0],ya)}},$.fn.modulenavsettings=function(t){return this.each((function(){var e=$.data(this,ya);e?e.updated(t):e=$.data(this,ya,new xa(this,t))}))};var Ca="notification",ka={message:"Hi! Im a notification message.",type:"alert",parent:".header",link:"#",linkText:"Click here to view.",attributes:null};function Ta(t,e){this.settings=E.mergeSettings(t,e,ka),this.element=$(t),this.init()}Ta.prototype={init:function(){return this.build().handleEvents()},build:function(){this.notificationEl=document.createElement("div"),this.notificationEl.classList.add("notification"),this.notificationEl.classList.add(this.settings.type);var t='\n      <svg class="icon notification-icon icon-'.concat(this.settings.type,'" focusable="false" aria-hidden="true" role="presentation">\n        <use href="#icon-').concat(this.settings.type,'"></use>\n      </svg>'),e=this.settings.linkText?'<a class="notification-link" href="'.concat(this.settings.link,'">').concat(this.settings.linkText,"</a>"):"",i='<p class="notification-text">'.concat(x.rtl?"".concat(e).concat(this.settings.message,"&lrm;"):"".concat(this.settings.message).concat(e),"</p>"),n='\n      <button type="text" class="notification-close">\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n          <use href="#icon-close"></use>\n        </svg>\n        <span class="audible">'.concat(H.translate("Close"),"</span>\n      </button>");this.notificationEl.innerHTML=t.concat(i,n);var s=document.querySelector(this.settings.parent);return $(this.settings.parent).closest(".contextual-action-panel").length>0?($(this.notificationEl).css({"max-width":"".concat($(s).width(),"px")}),$(s).closest(".modal-body-wrapper").prepend(this.notificationEl)):s.append(this.notificationEl),$(this.notificationEl).animateOpen(),E.addAttributes($(this.notificationEl),this,this.settings.attributes),E.addAttributes($(this.notificationEl).find(".notification-icon"),this,this.settings.attributes,"icon"),E.addAttributes($(this.notificationEl).find(".notification-text"),this,this.settings.attributes,"text"),E.addAttributes($(this.notificationEl).find(".notification-text a"),this,this.settings.attributes,"link"),E.addAttributes($(this.notificationEl).find("button.notification-close"),this,this.settings.attributes,"btn-close"),E.addAttributes($(this.notificationEl).find("button.notification-close .icon"),this,this.settings.attributes,"icon-close"),this.setTooltip(),this},handleEvents:function(){var t=this,e=this;return $(this.notificationEl).off("click.".concat(Ca)).on("click.".concat(Ca),".notification-close",(function(){e.destroy()})),$(window).on("resize.".concat(Ca),(function(){t.setTooltip()})),this},setTooltip:function(){var t=$(this.notificationEl).find(".notification-text")[0],e=$(this.notificationEl).data("tooltip");t.offsetHeight<t.scrollHeight||t.offsetWidth<t.scrollWidth?e?e.setContent(this.settings.message):$(this.notificationEl).tooltip({content:this.settings.message}):e&&e.destroy()},registerNotification:function(){lt.register(this)},close:function(t){t?lt.closeById(t):this.destroy()},closeLatest:function(){lt.closeLatest()},closeAll:function(){lt.closeAll()},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,ka)),this.teardown().init()},teardown:function(){return this.element.off("click.".concat(Ca),".notification-close"),this},destroy:function(){this.notificationEl&&this.notificationEl.parentNode&&this.notificationEl.parentNode.removeChild(this.notificationEl),lt.unregister(this),this.teardown(),$.removeData(this.element[0],Ca)}},$.fn.notification=function(t){return this.each((function(){$.data(this,Ca,new Ta(this,t))}))};var Aa="progress",Sa={value:0};function $a(t,e){this.settings=E.mergeSettings(t,e,Sa),this.element=$(t),this.init()}$a.prototype={init:function(){var t=this;return this.update(this.settings.value||this.element.attr("data-value")),this.element.off("updated.progress").on("updated.progress",(function(e){e.stopPropagation(),t.update()})),this},updateAria:function(t){var e=this;this.element.attr({role:"progressbar","aria-valuenow":t,"aria-valuemax":"100"});var i=this.element.parent();this.tooltipApi=this.tooltipApi||i.data("tooltip"),this.tooltipApi?(this.tooltipApi.content="".concat(t,"%"),this.tooltipApi.visible&&t<=100&&this.tooltipApi.tooltip.hasClass("process".concat(this.tooltipId))&&this.tooltipApi.tooltip.find(".tooltip-content").text(this.tooltipApi.content)):(i[0].setAttribute("title","".concat(t,"%")),i.tooltip(),this.tooltipApi=i.data("tooltip"),this.tooltipId=this.tooltipApi.uniqueId,i.on("aftershow.progress",(function(){e.tooltipApi.tooltip.addClass("process".concat(e.tooltipId))})).on("hide.progress mouseout.progress",(function(){e.tooltipApi.tooltip.removeClass("process".concat(e.tooltipId))})))},unbind:function(){var t=this.element.parent();return t.off("aftershow.progress hide.progress mouseout.progress"),this.tooltipApi=this.tooltipApi||t.data("tooltip"),this.tooltipApi&&(this.tooltipApi.destroy(),delete this.tooltipApi),this.tooltipId&&delete this.tooltipId,this.element.off("updated.progress"),this},update:function(t){var e=this.element[0].getAttribute("data-value")||0;/number|string/.test(l(t))&&(this.element[0].setAttribute("data-value",t),e=t),this.element[0].style.width="".concat(e,"%"),this.updateAria(e)},updated:function(t){return void 0!==t&&t.value&&(this.settings.value=t.value),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Aa)}},$.fn.progress=function(t){return this.each((function(){var e=$.data(this,Aa);e?e.updated(t):e=$.data(this,Aa,new $a(this,t))}))};var Ea="popdown",Da={keepOpen:!1,trigger:void 0,autoFocus:!1,toggleOnFocus:!1,firstLastTab:null};function Ia(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Da),this.init()}Ia.prototype={init:function(){return this.setup().build().handleEvents()},setup:function(){var t=this;function e(e){return!!e&&("string"==typeof e&&(e.match("#")&&0===e.indexOf("#")||(e="#".concat(e)),e=$(e)),!!e.length&&(t.popdown=e,!0))}return this.popdown=$(),this.trigger=this.element,(this.settings.trigger instanceof $||f.isElement(this.settings.trigger))&&(this.trigger=$(this.settings.trigger)),this.trigger.is(".dropdown, .multiselect")&&(this.trigger=$("#".concat(this.element.attr("id"),"-shdo"))),e(this.trigger.attr("data-popdown"))||e(this.trigger.next(".popdown")),this.id=this.popdown.attr("id"),this.id||(this.id="popdown-".concat($("body").find(".popdown").index(this.popdown)),this.popdown.attr("id",this.id)),this},build:function(){this.popdown.addClass("popdown").removeClass("hidden"),this.popdown.children(".popdown-contents").length||this.popdown.children().wrap('<div class="popdown-contents"></div>'),this.arrow=$('<div class="arrow"></div>').prependTo(this.popdown),this.place();var t=this.trigger.attr("aria-expanded");return t&&void 0!==t||this.trigger.removeAttr("aria-expanded"),"true"===t&&this.open(),this.trigger&&this.trigger[0].setAttribute("aria-controls",this.id),this},handleEvents:function(){var t=this,e=this;return this.trigger.on("click.popdown",(function(){e.toggle()})).on("updated.popdown",(function(){e.updated()})).on("clickoutside.popdown",(function(t){e.settings.keepOpen||e.handleFocusOut(t)})),this.setFirstLastTab(),this.changeObserver=new MutationObserver((function(){t.setFocusableElems()})),this.changeObserver.observe(this.element[0],{childList:!0,subtree:!0}),this.setFocusableElems(),this.settings.toggleOnFocus&&this.trigger.on("focus.popdown",(function(){t.toggle()})),this},hasValidTriggerSetting:function(){return this.settings.trigger instanceof $||f.isElement(this.settings.trigger)},get isFocused(){return this.hasFocus()},hasFocus:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:void 0,e=!1,i=document.activeElement;if(t instanceof HTMLElement||(t=void 0),this.trigger.is($(i)))return!0;if(this.popdown[0].contains(i))return!0;if(t){if(t.classList.contains("overlay"))return!0;if(this.popdown[0].contains(t))return!0}this.focusableElems||this.setFocusableElems(),this.focusableElems.forEach((function(n){if(!e){var s,a,o,r=$(n);if((r.is($(i))||"function"==typeof n.contains&&n.contains(i))&&(e=!0),r.is("div.dropdown, div.multiselect"))e=null===(s=r.parent().prev("select").data("dropdown"))||void 0===s||null==s?void 0:s.isFocused;if(r.is(".lookup")){var l,c=r.data("lookup");if(!(e=null==c||null==c?void 0:c.isFocused)&&t)e=null==c||null===(l=null==c?void 0:c.modal)||void 0===l||null==l?void 0:l.element[0].contains(t)}if(r.is(".btn-menu, .btn-actions"))e=null===(a=r.data("popupmenu"))||void 0===a||null==a?void 0:a.isFocused;if(r.is(".searchfield"))e=null===(o=r.data("searchfield"))||void 0===o||null==o?void 0:o.isFocused}}));var n=$(i).parents(".tooltip, .popover");return n.length&&n.each((function(t,i){var n=i.className.contains("popover")?"popover":"tooltip",s=$(i).data(n);s&&s.isFocused&&(e=!0)})),e},setFirstLastTab:function(){var t=this,e=this.settings;if(e.firstLastTab&&/function|boolean|object/.test(l(e.firstLastTab))){var i=null,n=null,s=null;if("object"===l(e.firstLastTab)?(e.firstLastTab.first&&((i=e.firstLastTab.first instanceof jQuery?e.firstLastTab.first:$(e.firstLastTab.first)).first(),this.popdown[0].contains(i[0])||(i=null)),e.firstLastTab.last&&((n=e.firstLastTab.last instanceof jQuery?e.firstLastTab.last:$(e.firstLastTab.last)).first(),this.popdown[0].contains(n[0])||(n=null)),"function"==typeof e.firstLastTab.callback&&(s=e.firstLastTab.callback)):"function"==typeof e.firstLastTab?s=e.firstLastTab:("boolean"==typeof e.firstLastTab||e.firstLastTab)&&(s=this.closeAndContinue),s){if(!i||!n||i&&!i.length||n&&!n.length){var a=this.popdown.find('input:not(:disabled):not([tabindex^="-"]),\n            select:not(:disabled):not([tabindex^="-"]),\n            textarea:not(:disabled):not([tabindex^="-"]),\n            [tabindex]:not(:disabled):not([tabindex^="-"])');(!i||i&&!i.length)&&(i=a.first()),(!n||n&&!n.length)&&(n=a.last())}this.focusableElem={first:i,last:n},i.on("keydown.popdown",(function(e){9===e.keyCode&&e.shiftKey&&(e.preventDefault(),s({e:e,self:t,first:i}))})),n.on("keydown.popdown",(function(e){9!==e.keyCode||e.shiftKey||(e.preventDefault(),s({e:e,self:t,last:n}))}))}}},setFocusableElems:function(){var t=f.focusableElems(this.popdown[0],["div.dropdown","div.multiselect",".lookup-wrapper > span.trigger"],["select","option"]);this.focusableElems=t,this.focusableElems.first=t[0],this.focusableElems.last=t[t.length-1]},closeAndContinue:function(t){var e=$(document).find(":focusable"),i=e.index(t.self.trigger);-1!==(i=t.e.shiftKey?i-1<0?-1:i-1:i+1>=e.length?-1:i+1)&&e.eq(i).focus(),t.self.close()},isOpen:function(){return"true"===this.trigger.attr("aria-expanded")},handleFocusOut:function(t){var e=this;e.focusableElems.includes(t.target)||e.hasFocus(t.target)?e.keyTarget=t.target:"BODY"===t.target.tagName&&e.keyTarget?delete e.keyTarget:e.close()},open:function(){if(!this.isAnimating){var t=this;if(this.isAnimating=!0,this.trigger.attr("aria-expanded","true"),this.position(),this.popdown.addClass("visible"),this.settings.autoFocus)(this.focusableElems?this.focusableElems.first:this.popdown.find(":focusable").first()).focus();this.addEventsTimer&&(this.addEventsTimer.destroy(!0),delete this.addEventsTimer),this.addEventsTimer=new X({duration:30,timeoutCallback:function(){$("body").on("resize.popdown",(function(e){t.handleFocusOut(e)})),$(document).on("click.popdown",(function(){t.element.trigger("clickoutside.popdown")})),$(document).on("keydown.popdown",(function(e){var i=$(e.target).parents(".popdown");switch(e.which||e.keyCode){case 27:i.length&&t.close();break;case 9:t.handleFocusOut(e)}})),t.isAnimating=!1}}),J.register(this.addEventsTimer)}},close:function(){if(!this.isAnimating){this.addEventsTimer&&(this.addEventsTimer.destroy(!0),delete this.addEventsTimer),this.keyTarget&&delete this.keyTarget;var t=this;this.isAnimating=!0,this.trigger.attr("aria-expanded","false"),this.popdown.removeClass("visible"),$("body").off("resize.popdown"),$(document).off("click.popdown focusout.popdown keydown.popdown"),this.closeTimer&&(this.closeTimer.destroy(!0),delete this.closeTimer),this.closeTimer=new X({duration:30,timeoutCallback:function(){t.popdown[0].style.display="none",t.isAnimating=!1}}),J.register(this.closeTimer)}},toggle:function(){this.isOpen()?this.close():this.open()},place:function(){this.scrollparent=$("body"),this.popdown.detach().appendTo(this.scrollparent)},position:function(){var t={offset:{left:0,top:0},scrollDistance:{left:0,top:0}},e=window.innerHeight+$(document).scrollTop(),i=window.innerWidth-2+$(document).scrollLeft();this.popdown.removeAttr("style"),this.arrow.removeAttr("style"),this.scrollparent.length&&(t.offset=this.scrollparent.offset(),t.scrollDistance.top=this.scrollparent.scrollTop(),t.scrollDistance.left=this.scrollparent.scrollLeft(),e-=t.offset.top+t.scrollDistance.top,i-=t.offset.left+t.scrollDistance.left);var n,s=!1,a=!1,o=this.trigger,r=o.offset(),l=0,c=0;this.popdown.addClass("bottom"),this.popdown[0].style.left="".concat(r.left,"px"),this.popdown[0].style.top="".concat(r.top+o.outerHeight(!0)+11,"px"),this.arrow[0].style.left="".concat(o.outerWidth(!0)/2,"px"),this.arrow[0].style.top="".concat(-11,"px"),(n=this.popdown.offset()).left<0&&(s=!0,l=0-n.left);var d=n.left+this.popdown.outerWidth(!0);if(d>i&&(s=!0,l=d-i+(H.isRTL()?20:0)),s){this.popdown[0].style.left="".concat(n.left+-1*l,"px");var h=this.popdown[0].getBoundingClientRect(),u=o[0].getBoundingClientRect(),p=h.right-u.right+10;this.arrow[0].style.left="auto",this.arrow[0].style.right="".concat(p,"px"),n=this.popdown.offset()}n.top<0&&(a=!0,c=0-n.top);var g=n.top+this.popdown.outerHeight(!0);g>e&&(a=!0,c=g-e),a&&(this.arrow[0].style.display="none",this.popdown[0].style.top="".concat(n.top+-1*c,"px"),this.arrow[0].style.top="".concat(parseInt(this.arrow[0].style.top,10)- -1*c,"px"),n=this.popdown.offset()),(n.top<0||n.top+this.popdown.outerHeight(!0)>e)&&(this.popdown[0].style.top=0,g=(n=this.popdown.offset()).top+this.popdown.outerHeight(!0),this.popdown[0].style.height="".concat(parseInt(this.popdown[0].style.height,10)-(g-e),"px"))},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.hasValidTriggerSetting()&&(this.trigger=$(this.settings.trigger)),this},teardown:function(){return this.isOpen()&&this.close(),this.trigger&&this.trigger.off("updated.popdown click.popdown focus.popdown clickoutside.popdown").removeAttr("aria-controls").removeAttr("aria-expanded"),this.changeObserver&&(this.changeObserver.disconnect(),delete this.changeObserver),this.focusableElem&&this.focusableElem.first instanceof jQuery&&(this.focusableElem.first.off("keydown.popdown"),this.focusableElem.last.off("keydown.popdown"),delete this.focusableElem),this.focusableElems&&this.focusableElems.first instanceof jQuery&&(this.focusableElems.first.off("keydown.popdown"),this.focusableElems.last.off("keydown.popdown"),delete this.focusableElems),this.originalParent&&this.originalParent.length&&this.popdown.detach().appendTo(this.originalParent),this.arrow.remove(),this},destroy:function(){this.teardown(),$.removeData(this.element[0],Ea)}},$.fn.popdown=function(t){return this.each((function(){var e=$.data(this,Ea);e?e.updated(t):e=$.data(this,Ea,new Ia(this,t))}))};var La="rating",Pa={};function Ma(t,e){this.settings=E.mergeSettings(t,e,Pa),this.element=$(t),this.init()}Ma.prototype={init:function(){this.handleEvents(),this.allInputs=this.element.find("input"),this.enable()},handleEvents:function(){for(var t=this,e=$("input",this.element),i=function(i){var n=$(e[i]),s=$(e[i+1]),a=function(t){return t.is(".is-filled")||t.is(".is-half")};n.on("click.".concat(La),(function(){t.element.is(".is-readonly")||t.val(a(n)&&!a(s)?i:i+1)}))},n=0,s=e.length;n<s;n++)i(n)},fillStar:function(t,e){$(t).addClass("is-filled").removeClass("is-half").next(e).find("svg").changeIcon("star-filled")},emptyStar:function(t,e){$(t).removeClass("is-filled").removeClass("is-half").next(e).find("svg").changeIcon("star-outlined")},halfStar:function(t,e){$(t).addClass("is-half").removeClass("is-filled").next(e).find("svg").changeIcon("star-half")},val:function(t){if(0===arguments.length||""===t||isNaN(t)||-1===L.sign(t))return this.currentValue;this.currentValue=parseFloat(t,10);for(var e=Math.floor(this.currentValue),i=0,n=this.allInputs.length;i<n;i++){var s=$(this.allInputs[i]),a=s.parent().is(".inline")?"svg":"label";t%1!=0&&e===i?this.halfStar(s,a):i<t?this.fillStar(s,a):this.emptyStar(s,a),i+1===e&&s.prop("checked",!0)}return e<=0&&t>0&&$(this.allInputs[0]).prop("checked",!0),this.currentValue},readonly:function(){var t=$(this.element);t.addClass("is-readonly"),t.find("input").attr("disabled","")},enable:function(){$(this.element).removeClass("is-readonly").find("input").removeAttr("disabled")},unbind:function(){return this.element.find("input").off("click.".concat(La)),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Pa)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],La)}},$.fn.rating=function(t){return this.each((function(){var e=$.data(this,La);e?e.updated(t):e=$.data(this,La,new Ma(this,t))}))};var Ba="signin",Oa={};function Ra(t,e){this.settings=E.mergeSettings(t,e,Oa),this.element=$(t),this.init()}Ra.prototype={init:function(){this.handleKeys()},isCapslock:function(t){var e=!1,i=!1;return(t=t||window.event).which?e=t.which:t.keyCode&&(e=t.keyCode),t.shiftKey?i=t.shiftKey:t.modifiers&&(i=!!(4&t.modifiers)),!!(e>=97&&e<=122&&i)||e>=65&&e<=90&&!i},unbind:function(){return $("body").off("keypress.signin blur.signin change.signin"),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Oa)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Ba)},handleKeys:function(){var t=this,e=$.createIconElement({classes:"icon-capslock",icon:"capslock"});document.msCapsLockWarningOff=!0,this.element.on("keypress.signin",'[type="password"]',(function(i){var n=$(this),s=n.parent(".field"),a=$(".icon-capslock",s);t.isCapslock(i)&&!n.hasClass("error")?a.length||(s.append(e),$("body").toast({audibleOnly:!0,message:H.translate("CapsLockOn")})):a.remove()})).on("blur.signin change.signin",'[type="password"]',(function(){var t=$(this),i=t.closest(".field"),n=$(".icon-capslock",i);setTimeout((function(){n&&n.length&&(t.hasClass("error")?n.remove():i.append(e))}),150)}))}},$.fn.signin=function(t){return this.each((function(){var e=$.data(this,Ba);e?e.updated(t):e=$.data(this,Ba,new Ra(this,t))}))};var za="slider",Fa={value:[50],min:0,max:100,range:!1,step:void 0,ticks:[],tooltipContent:void 0,tooltipPosition:"top",persistTooltip:!1,showTooltipOnLoad:!1,sliderBox:!1,attributes:null};function Na(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Fa),this.init()}function Ha(t){return t%1==0}function qa(t,e){return e&&!isNaN(e)&&0!==e||(e=1),Math.round(t/e)*e}function Va(t,e){var i=t[0],n=t[1],s=e[0],a=e[1];return Math.sqrt(Math.pow(s-i,2)+Math.pow(a-n,2))}Na.prototype={init:function(){return this.buildSettings().addMarkup().bindEvents()},buildSettings:function(){var t,e=this;if(this.element.is(":disabled")&&this.element.closest(".field").addClass("is-disabled"),this.settings||(this.settings={}),this.settings.value=void 0!==this.element.attr("value")?this.element.attr("value"):this.settings.value,this.settings.min=void 0!==this.element.attr("min")?parseInt(this.element.attr("min"),10):this.settings.min,this.settings.max=void 0!==this.element.attr("max")?parseInt(this.element.attr("max"),10):this.settings.max,this.settings.range=void 0!==this.element.attr("data-range")?"true"===this.element.attr("data-range"):this.settings.range,this.settings.step=isNaN(this.element.attr("step"))?this.settings.step:Number(this.element.attr("step")),""===this.settings.value&&(this.settings.value=this.settings.min),1===this.settings.step&&(this.settings.step=void 0),void 0!==this.element.attr("data-ticks")){try{t=JSON.parse(e.element.attr("data-ticks"))}catch(e){t=[]}$.isArray(t)&&(this.settings.ticks=t)}var i="true"===this.element.attr("data-tooltip-persist")||!0===this.element.attr("data-tooltip-persist");if(this.settings.persistTooltip=void 0!==this.element.attr("data-tooltip-persist")?i:this.settings.persistTooltip,this.settings.tooltip=this.settings.tooltipContent,void 0!==this.element.attr("data-tooltip-content"))try{e.settings.tooltip=JSON.parse(e.element.attr("data-tooltip-content"))}catch(t){e.settings.tooltip=["",""]}if("string"==typeof this.settings.tooltip)if(-1===this.settings.tooltip.indexOf(","))this.settings.tooltip=[this.settings.tooltip,""];else{var n=this.settings.tooltip.split(",");this.settings.tooltip=[n[0]],this.settings.tooltip.push(n[1]?n[1]:"")}this.settings.tooltip&&1===this.settings.tooltip.length&&this.settings.tooltip.push(""),this.ticks=[];var s={value:this.settings.min,description:e.getModifiedTextValue(this.settings.min)},a={value:this.settings.max,description:e.getModifiedTextValue(this.settings.max)};if(this.settings.ticks){var o=e.settings.ticks||[];if($.isArray(o)&&o.length>0){var r=o.filter((function(t){return t.value===e.settings.min})),l=o.filter((function(t){return t.value===e.settings.max}));r.length>0&&(s.description=r[0].description,s.color=r[0].color,o=$.grep(o,(function(t){return t!==r[0]}))),l.length>0&&(a.description=l[0].description,a.color=l[0].color,o=$.grep(o,(function(t){return t!==l[0]})))}e.ticks.push(s);for(var c=0;c<o.length;c++){var d={};void 0!==o[c].value&&(d.value=o[c].value,d.description=void 0!==o[c].description?o[c].description:"",d.color=o[c].color,e.ticks.push(d))}e.ticks.push(a)}else this.ticks.push(s,a);if($.isArray(this.settings.value))this.settings.value[0]=isNaN(this.settings.value[0])?(this.settings.min+this.settings.max)/2:parseInt(this.settings.value[0],10);else if("number"==typeof this.settings.value)this.settings.value=[this.settings.value];else if(-1===this.settings.value.indexOf(","))this.settings.value=[isNaN(this.settings.value)?(this.settings.min+this.settings.max)/2:parseInt(this.settings.value,10)];else{var h=this.settings.value.split(",");h[0]=isNaN(h[0])?this.settings.min:parseInt(h[0],10),h[1]=isNaN(h[1])?this.settings.max:parseInt(h[1],10),this.settings.value=h}return this.settings.range&&!this.settings.value[1]&&this.settings.value.push(this.settings.max),this},addMarkup:function(){var t=this,e=!1;if("INPUT"!==t.element[0].tagName)throw new Error('Element with ID "'.concat(t.element.id,"\" cannot invoke a slider;  it's not an Input element."));t.originalElement={type:t.element.attr("type")},t.element.attr("type","hidden"),t.wrapper=$('<div class="slider-wrapper"></div>').attr("id","".concat(t.element.attr("id"),"-slider")).insertAfter(t.element),t.hitarea=$('<div class="slider-hit-area"></div>').appendTo(t.wrapper),t.range=$('<div class="slider-range"></div>').appendTo(t.wrapper),this.element.hasClass("vertical")&&(this.wrapper.addClass("vertical"),e=!0),this.isRtlHorizontal=H.isRTL()&&!e,this.isRtlVertical=H.isRTL()&&e;var i=this.element.attr("style");i&&(i.match(/min-height/)&&(this.wrapper[0].style.minHeight=this.element[0].style.minHeight,i=i.replace("min-height","")),i.match(/height/)&&(this.wrapper[0].style.height=this.element[0].style.height),i.match(/min-width/)&&(this.wrapper[0].style.minWidth=this.element[0].style.minWidth,i=i.replace("min-width","")),i.match(/width/)&&(this.wrapper[0].style.width=this.element[0].style.width)),t.handles=[];var n=t.element.prev("label").text(),s=$('<div class="slider-handle'.concat(t.settings.range?" lower":"",'" tabindex="0"></div>')).attr("aria-label","".concat(t.settings.range?H.translate("SliderMinimumHandle"):H.translate("SliderHandle")," ").concat(n));if(t.handles.push(s),t.settings.range){var a=$('<div class="slider-handle higher" tabindex="0"></div>').attr("aria-label","".concat(H.translate("SliderMaximumHandle")," ").concat(n));t.handles.push(a)}($.each(t.handles,(function(i,n){n.attr({role:"slider","aria-orientation":e?"vertical":"horizontal","aria-valuemin":t.settings.min,"aria-valuemax":t.settings.max}).hideFocus(),n.appendTo(t.wrapper)})),t.ticks.forEach((function(i){!function(i){var n=t.isRtlHorizontal?100-t.convertValueToPercentage(i.value):t.convertValueToPercentage(i.value),s="calc(".concat(n,"% - 4px)");i.element=$('<div class="tick" data-value="'.concat(i.value,'"></div>')),i.label=$('<span class="label">'.concat(i.description,"</span>")),i.element[0].style[e?"bottom":"left"]=s,i.element.append(i.label),t.wrapper.append(i.element),e||(i.label[0].style.left="".concat(-(i.label.outerWidth()/2-i.element.width()/2),"px"))}(i)})),this.settings.sliderBox)&&(t.wrapper.addClass("has-slider-box"),$('<input type="text" class="slider-box" aria-required="true" data-mask data-options="{ \'pattern\' : \'###\' }" ></input>').insertAfter(t.wrapper));t.value(t.settings.value),t.updateRange(),$.each(t.handles,(function(i,n){t.settings.tooltip&&(n.tooltip({content:function(){return"".concat(t.getModifiedTextValue(Math.floor(t.value()[i])))},placement:e?"right":t.settings.tooltipPosition,trigger:"focus",keepOpen:t.settings.persistTooltip}),n.removeAttr("aria-describedby"))})),!0===this.element.prop("readonly")?this.readonly():!0===this.element.prop("disabled")&&this.disable(),E.addAttributes(this.element,this,this.settings.attributes,"",!0),E.addAttributes(this.wrapper,this,this.settings.attributes,"wrapper",!0),E.addAttributes(this.hitarea,this,this.settings.attributes,"hitarea",!0),E.addAttributes(this.range,this,this.settings.attributes,"range",!0),E.addAttributes.apply(E,u(this.handles).concat([this,this.settings.attributes,"handle",!0]));for(var o=0,r=this.ticks.length;o<r;o++){var l=$(this.ticks[o].element);E.addAttributes(l,this,this.settings.attributes,"tick-".concat(o+1))}return this.settings.showTooltipOnLoad&&setTimeout((function(){$.each(t.handles,(function(t,e){e.data("tooltip").show()}))})),t},handleRangeClick:function(t){if(t.preventDefault(),!this.isDisabled()&&!this.isDragging){var e=this,i=this.wrapper.hasClass("vertical"),n="click"!==t.originalEvent.type?t.originalEvent.changedTouches[0].pageX:t.pageX,s="click"!==t.originalEvent.type?t.originalEvent.changedTouches[0].pageY:t.pageY,a=n-e.wrapper.offset().left-$(document).scrollLeft(),o=s-e.wrapper.offset().top-$(document).scrollTop(),r=[a,o],l=[e.handles[0].offset().left+e.handles[0].width()/2-e.wrapper.offset().left-$(document).scrollLeft(),e.handles[0].offset().top+e.handles[0].height()/2-e.wrapper.offset().top-$(document).scrollTop()],c=e.value(),d=Va(r,l),h=c[0],u=e.handles[0];u.addClass("hide-focus");var p=function(){if(i){var t=e.wrapper.height();return(t-o)/t*100}return a/e.wrapper.width()*100}(),g=e.convertPercentageToValue(p);e.handles[1]&&(d>Va(r,[e.handles[1].offset().left+e.handles[1].width()/2-e.wrapper.offset().left-$(document).scrollLeft(),e.handles[1].offset().top+e.handles[1].height()/2-e.wrapper.offset().top-$(document).scrollTop()])?(e.value([void 0,g]),u=e.handles[1],h=c[1]):e.value([g])),e.checkHandleDifference(u,h,g);var f=e.settings.step?e.settings.step:0;g<h?e.decreaseValue(t,u,g,f):e.increaseValue(t,u,g,f),e.settings.tooltip||u.focus()}},activateHandle:function(t){t.addClass("is-active")},deactivateHandle:function(t){t.removeClass("is-active")},enableHandleDrag:function(t){if(!this.isDisabled()){var e=this,i={containment:"parent",axis:this.isVertical()?"y":"x",clone:!1};t.drag(i).on("drag.slider touchmove.slider",(function(t,i){!function(t,i,n){if(!e.isDisabled()){var s=function(){var s,a;if("touchmove"===t.type){var o=t.originalEvent.changedTouches[0].pageX,r=t.originalEvent.changedTouches[0].pageY;s=o-e.wrapper.offset().left-$(document).scrollLeft(),a=r-e.wrapper.offset().top-$(document).scrollTop()}else s=n.left,a=n.top;if(e.isVertical()){var l=e.wrapper.height()-i.outerHeight();return(l-(a=Math.min(Math.max(a,0),l)))/l*100}var c=e.wrapper.width()-i.outerWidth();return(s=Math.min(Math.max(s,0),c))/c*100}(),a=e.convertPercentageToValue(s);if(e.settings.range){var o=e.value();i.hasClass("higher")&&a<=o[0]&&(a=o[0]),i.hasClass("lower")&&a>=o[1]&&(a=o[1])}e.settings.step&&(a=qa(a,e.settings.step)),t.defaultPrevented||(e.value(i.hasClass("higher")?[void 0,a]:[a]),e.updateRange(),e.updateTooltip(i),e.element.trigger("sliding",i,a))}}(t,$(t.currentTarget),i),e.isDragging=!0})).on("dragstart touchstart",(function(){$(this).addClass("is-dragging"),e.range.addClass("is-dragging"),e.element.trigger("slidestart",t)})).on("dragend touchend",(function(i){e.settings.step||(e.value($(i.currentTarget).hasClass("higher")?[void 0,qa(e._value[1],e.settings.step)]:[qa(e._value[0],e.settings.step)]),e.updateRange(),e.updateTooltip($(i.currentTarget))),$(this).removeClass("is-dragging"),e.range.removeClass("is-dragging"),e.element.trigger("slidestop",t),e.element.trigger("change",{element:e.element,value:e._value}),setTimeout((function(){e.isDragging=!1}),0)}))}},disableHandleDrag:function(t){t.off("drag.slider dragstart dragend touchmove.slider touchstart touchend"),this.range.removeClass("is-dragging"),t.removeClass("is-dragging");var e=t.data("drag");e&&e.destroy()},bindInputSliderEvent:function(t,e){var i=this;if(this.settings.sliderBox){var n=t.parent().siblings(".slider-box");n.on("keyup.slider".concat(e),(function(e){if(13===e.keyCode){var n=$(e.currentTarget).val();i.value(n),i.updateRange(),i.updateTooltip(t)}})),n.on("blur.slider".concat(e),(function(e){var n=$(e.currentTarget).val();i.value(n),i.updateRange(),i.updateTooltip(t)}))}},convertValueToPercentage:function(t){return(t-this.settings.min)/(this.settings.max-this.settings.min)*100},convertPercentageToValue:function(t){var e=t/100*(this.settings.max-this.settings.min)+this.settings.min;return this.isRtlHorizontal?this.settings.max-e+this.settings.min:e},getIncrement:function(){var t=.1*(this.settings.max-this.settings.min);return void 0!==this.settings.step&&t<=this.settings.step&&(t=this.settings.step),t},handleKeys:function(t,e){if(!e.isDisabled()){var i=t.which,n=$(t.currentTarget);switch(n.removeClass("hide-focus"),i){case 33:e.increaseValue(t,n,void 0,this.getIncrement());break;case 34:e.decreaseValue(t,n,void 0,this.getIncrement());break;case 35:e.increaseValue(t,n,this.settings.max);break;case 36:e.decreaseValue(t,n,this.settings.min);break;case 38:case 39:e.isRtlHorizontal&&39===i?e.decreaseValue(t,n):e.increaseValue(t,n);break;case 37:case 40:e.isRtlHorizontal&&37===i?e.increaseValue(t,n):e.decreaseValue(t,n)}}},increaseValue:function(t,e,i,n){t.preventDefault(),clearTimeout(e.data("animationTimeout"));var s,a,o,r=this.value().slice(0),l=void 0!==n?n:void 0!==this.settings.step?this.settings.step:1;if(e.hasClass("higher"))o=(a=(s=void 0!==i?i:r[1])+(l=Ha(s)?l:isNaN(s%l)?0:s%l)<this.settings.max?s+l:this.settings.max)%l?a:qa(a,l),this.value([void 0,o]);else{s=void 0!==i?i:r[0];var c=void 0===r[1]?this.settings.max:r[1];o=(a=s+(l=Ha(s)?l:isNaN(s%l)?0:l-s%l)<c?s+l:c)%l?a:qa(a,l),this.value([o])}this.checkHandleDifference(e,s,o),this.updateRange(),this.updateTooltip(e)},decreaseValue:function(t,e,i,n){t.preventDefault(),clearTimeout(e.data("animationTimeout"));var s,a,o,r=this.value(),l=void 0!==n?n:void 0!==this.settings.step?this.settings.step:1;if(e.hasClass("higher")){s=void 0!==i?i:r[1];var c=void 0===r[0]?this.settings.min:r[0];o=(a=s-(l=Ha(s)?l:isNaN(s%l)?0:l-s%l)>c?s-l:c)%l?a:qa(a,l),this.value([void 0,o])}else o=(a=(s=void 0!==i?i:r[0])-(l=Ha(s)?l:isNaN(s%l)?0:s%l)>this.settings.min?s-l:this.settings.min)%l?a:qa(a,l),this.value([o]);this.checkHandleDifference(e,s,o),this.updateRange(),this.updateTooltip(e)},updateRange:function(){for(var t=this,e=this.value(),i=[],n=this.getColorClosestToValue(),s=t.wrapper.hasClass("vertical"),a=0;a<this.ticks.length;a++){(this.settings.range?e[0]<this.ticks[a].value&&this.ticks[a].value<=e[1]:this.ticks[a].value<=e[0])?(this.ticks[a].element.addClass("complete"),n&&(this.ticks[a].element[0].style.backgroundColor=n,this.ticks[a].element.addClass("inherit"))):(this.ticks[a].element.removeClass("complete"),n&&(this.ticks[a].element[0].style.backgroundColor="",this.ticks[a].element.removeClass("inherit")))}n&&(this.range[0].style.backgroundColor=n,$.each(this.handles,(function(t,e){e[0].style.backgroundColor=n,e[0].style.borderColor=n}))),$.each(t.ticks,(function(e){t.ticks[e].label[0].style.color=""})),i[0]=this.convertValueToPercentage(e[0]),void 0!==e[1]&&(i[1]=this.convertValueToPercentage(e[1]));var o=s?["bottom","top"]:t.isRtlHorizontal?["right","left"]:["left","right"],r={};function l(t,e){var i=s?o[1]:o[0],n=s?100-e:e;t.css(i,"calc(".concat(n,"% - ").concat(t.outerWidth()/2,"px)"))}(this.handles[1]?(r[o[0]]="".concat(i[0],"%"),r[o[1]]="".concat(100-i[1],"%")):(r[o[0]]="0%",r[o[1]]="".concat(100-i[0],"%")),this.range.css(r),this.handles[0].hasClass("is-animated")&&this.handles[0].data("animationTimeout",setTimeout((function(){t.handles[0].removeClass("is-animated").trigger("slide-animation-end"),t.range.removeClass("is-animated")}),201)),l(this.handles[0],i[0]),this.handles[1]&&(this.handles[1].hasClass("is-animated")&&this.handles[1].data("animationTimeout",setTimeout((function(){t.handles[1].removeClass("is-animated").trigger("slide-animation-end"),t.range.removeClass("is-animated")}),201)),l(this.handles[1],i[1]),this.handles[0].attr("aria-valuemax",e[1]),this.handles[1].attr("aria-valuemin",e[0])),this.settings.sliderBox)&&t.wrapper.siblings(".slider-box").val(Math.trunc(i[0]))},checkHandleDifference:function(t,e,i){var n=this.convertValueToPercentage(e),s=this.convertValueToPercentage(i);Math.abs(n-s)>3&&(t.addClass("is-animated"),this.range.addClass("is-animated"))},updateTooltip:function(t){if(this.settings.tooltip){if(!t){var e,i=this.handles[0].data("tooltip");return this.handles[1]&&(e=this.handles[1].data("tooltip")),i.hide(),void(e&&e.hide())}var n=t.data("tooltip");t.hasClass("is-animated")?(n.hide(),t.one("slide-animation-end",(function(){s()}))):s()}function s(){n.position(),t.focus()}},getColorClosestToValue:function(){for(var t,e,i={light:{default:"#000000","very-poor":"#a13030",poor:"#d66221",adequate:"#f2bc41",good:"#9cce7c","very-good":"#76b051",superior:"#488421"},dark:{default:"#ffffff","very-poor":"#a13030",poor:"#d66221",adequate:"#f2bc41",good:"#9cce7c","very-good":"#76b051",superior:"#488421"},contrast:{default:"#000000","very-poor":"#a13030",poor:"#d66221",adequate:"#e4882b",good:"#76b051","very-good":"#56932e",superior:"#397514"}}[ft.currentTheme.id.split("-")[2]],n=this.value()[0],s=0;s<this.ticks.length;s++)(e=this.ticks[s].color)&&n>=this.ticks[s].value&&(t=e,t=e.indexOf("#")>-1?e:i[e]||i.default);return t},value:function(t,e){var i=this;return void 0===t&&void 0===e||(t&&$.isArray(t)&&(void 0===t[1]||isNaN(t[1])||(e=t[1]),isNaN(t[0])||(t=t[0])),(void 0===t||isNaN(t))&&$.isArray(i._value)&&void 0!==i._value[0]&&(t=i._value[0]),(void 0===e||isNaN(e))&&$.isArray(i._value)&&void 0!==i._value[1]&&(e=i._value[1]),i._value=[t,e],i.element.val(void 0!==e?i._value:i._value[0]),$.each(i.handles,(function(t,e){var n=i._value[t],s=i.getModifiedTextValue(n);$.each(i.ticks,(function(t,e){e.value===n&&(s=e.description)})),e.attr({"aria-valuenow":i._value[t],"aria-valuetext":s})})),0===this.element.next().find(".slider-handle.is-dragging").length&&i.element.trigger("change",{element:i.element,value:i._value})),i._value},getModifiedTextValue:function(t){return this.settings.tooltip?this.settings.tooltip[0]+t+this.settings.tooltip[1]:t},enable:function(){this.element.prop("disabled",!1),this.element.prop("readonly",!1),this.wrapper.removeClass("is-readonly"),this.wrapper.removeClass("is-disabled");var t=this;return $.each(this.handles,(function(e,i){t.enableHandleDrag(i)})),this.wrapper.find("[tabindex]").attr("tabindex","0"),this},disable:function(){var t=this;return this.element.prop("disabled",!0),this.element.prop("readonly",!1),this.wrapper.removeClass("is-readonly"),setTimeout((function(){t.wrapper.addClass("is-disabled");var e=t;$.each(t.handles,(function(t,i){e.disableHandleDrag(i)}))})),this.wrapper.find("[tabindex]").attr("tabindex","-1"),this},readonly:function(){var t=this;return this.element.prop("disabled",!0),this.element.prop("readonly",!0),this.wrapper.removeClass("is-disabled"),this.wrapper.addClass("is-readonly"),$.each(this.handles,(function(e,i){t.disableHandleDrag(i)})),this},isDisabled:function(){return this.element.prop("disabled")},isVertical:function(){return this.wrapper.hasClass("vertical")},setValue:function(t,e){var i=this.value();this.checkHandleDifference(this.handles[0],i[0],t),this.handles[1]&&this.checkHandleDifference(this.handles[1],i[1],e);var n=this.value(t,e);return this.updateRange(),this.updateTooltip(),n},refresh:function(t,e){return j(this.setValue,this.refresh).apply(this,[t,e])},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.element.removeAttr("value"),this.teardown().init()},teardown:function(){var t=this;return $.each(t.handles,(function(e,i){t.disableHandleDrag(i),i.off("mousedown.slider click.slider blur.slider keydown.slider keyup.slider")})),this.wrapper.off("click.slider touchend.slider touchcancel.slider").remove(),this.element.attr("type",g.ensureAlphaNumeric(this.originalElement.type)),this},destroy:function(){this.teardown(),$.removeData(this.element[0],za)},bindEvents:function(){var t=this;return $.each(t.handles,(function(e,i){i.on("mousedown.slider",(function(){t.isDisabled()||$(this).focus()})).on("click.slider",(function(t){t.preventDefault()})).on("keydown.slider",(function(e){t.activateHandle(i),t.handleKeys(e,t)})).on("keyup.slider blur.slider",(function(){t.deactivateHandle(i)})),t.enableHandleDrag(i),t.bindInputSliderEvent(i,e)})),t.wrapper.on("click.slider touchend.slider touchcancel.slider",(function(e){t.handleRangeClick(e)})),t.element.on("updated.slider",(function(){t.updated()})),t}},$.fn.slider=function(t){return this.each((function(){var e=$.data(this,za);e?e.updated(t):e=$.data(this,za,new Na(this,t))}))};var Wa="spinbox",ja={autocorrectOnBlur:!0,min:-2147483647,max:2147483647,step:null,maskOptions:null,attributes:null};function Ua(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,ja),this.init()}Ua.prototype={get isTouch(){return x.features.touch},get isWrapped(){return this.element.parent().is(".spinbox-wrapper")},get isInlineLabel(){return this.element.parent().is(".inline")},init:function(){this.inlineLabel=this.element.closest("label"),this.inlineLabelText=this.inlineLabel.find(".label-text"),this.element.is(":disabled")&&this.element.closest(".field").addClass("is-disabled"),this.setInitialValue().addMarkup().bindEvents().setWidth()},setWidth:function(){var t=this.element[0].style;return t.width&&(this.element.parent()[0].style.width="".concat(parseInt(t.width,10)+2*this.element.parent().find(".down").outerWidth(),"px")),this},setInitialValue:function(){var t=this.checkForNumeric(this.element.val());return this.element.val(t),this.element.attr("data-trackdirty")&&this.element.data("original",t),/(iPad|iPhone|iPod)/g.test(navigator.userAgent)&&this.element.attr("pattern","\\d*"),this.element.attr("max")?this.settings.max=this.element.attr("max"):this.settings.max&&this.element.attr("max",this.settings.max),this.element.attr("step")?this.settings.step=this.element.attr("step"):this.settings.step&&this.element.attr("step",this.settings.step),this.element.attr("min")?this.settings.min=this.element.attr("min"):this.settings.min&&this.element.attr("min",this.settings.min),this},addMarkup:function(){var t=this;if(this.isInlineLabel)this.inlineLabel.addClass("spinbox-wrapper");else if(!this.isWrapped){var e='<span class="spinbox-wrapper"></span>';if(this.element.is(".field-options")){var i=this.element.closest(".field").find(".btn-actions");this.element.add(i).add(i.next(".popupmenu")).wrapAll(e)}else this.element.wrap(e)}this.isWrapped&&(this.buttons={down:this.element.parent().find(".down"),up:this.element.parent().find(".up")}),this.buttons.up.length||(this.buttons.up=$("<span ".concat(this.isTouch?"":'aria-hidden="true"',' class="spinbox-control up">+</span>')).insertAfter(this.element),this.buttons.up.button()),this.buttons.down.length||(this.buttons.down=$("<span ".concat(this.isTouch?"":'aria-hidden="true"',' class="spinbox-control down">-</span>')).insertBefore(this.element),this.buttons.down.button());var n=this.element[0].className;["input-xs","input-sm","input-mm","input-md","input-lg"].forEach((function(e){if(n.indexOf(e)>-1){var i=e.replace("input","spinbox");t.isWrapped?t.element.parent(".spinbox-wrapper").addClass(i):t.isInlineLabel&&t.inlineLabel.addClass(i)}}));var s=this.element.attr("min"),a=this.element.attr("max"),o={role:"spinbutton"};s||a||(a="9999999"),o.min=s||0,o.max=a,this.element.data("mask")&&"object"===l(this.element.data("mask"))&&this.element.data("mask").destroy(),o["aria-valuemin"]=s||0,o["aria-valuemax"]=a||0,this.element.attr(o),this.updateAria(this.element.val());var r={process:"number",patternOptions:{allowDecimal:!1,allowThousandsSeparator:!1,allowNegative:Math.min(this.settings.min,this.settings.max)<0,decimalLimit:0,integerLimit:String(this.settings.max).length}};return this.element.mask(this.settings.maskOptions||r),this.isDisabled()&&this.disable(),this.element.attr("readonly")&&this.readonly(),E.addAttributes(this.element.parent(),this,this.settings.attributes,"wrapper",!0),E.addAttributes(this.element,this,this.settings.attributes,"spinbox",!0),E.addAttributes(this.buttons.down,this,this.settings.attributes,"btn-down",!0),E.addAttributes(this.buttons.up,this,this.settings.attributes,"btn-up",!0),this},enableLongPress:function(t,e){e.addButtonStyle(t),e.longPressInterval=setInterval((function(){$(t.currentTarget).is(":hover")&&e.handleClick(t)}),250)},disableLongPress:function(t,e){e.removeButtonStyle(t),clearInterval(e.longPressInterval),e.longPressInterval=null},handleClick:function(t){(t.preventDefault(),this.isDisabled()||1!==t.which||this.isReadonly())||($(t.currentTarget).hasClass("up")?this.increaseValue():this.decreaseValue(),this.element.focus())},handleKeyDown:function(t,e){var i=t.which;if(13!==i){if(-1!==$.inArray(i,[37,38,39,40])&&!this.isReadonly())switch(i){case 38:e.addButtonStyle(e.buttons.up),e.increaseValue();break;case 40:e.addButtonStyle(e.buttons.down),e.decreaseValue()}}else e.correctValue()},handleKeyup:function(t,e){if(!e.isDisabled()&&!this.isReadonly()){var i=t.which;switch(i){case 38:case 39:H.isRTL()&&39===i?e.removeButtonStyle(e.buttons.down):e.removeButtonStyle(e.buttons.up);break;case 37:case 40:H.isRTL()&&39===i?e.removeButtonStyle(e.buttons.up):e.removeButtonStyle(e.buttons.down)}e.updateAria(e.element.val())}},handleAfterPaste:function(t){var e=Number(t.element.attr("min")),i=Number(t.element.attr("max")),n=Number(t.element.val());n<e&&(n=e),n>i&&(n=i),t.updateVal(n)},correctValue:function(t){var e=Number(this.element.val()),i=parseInt(this.element.attr("min"),10),n=parseInt(this.element.attr("max"),10);return e<i?(t&&t.preventDefault(),this.updateVal(i)):e>n?(t&&t.preventDefault(),this.updateVal(n)):void 0},increaseValue:function(){var t=this.element.attr("max"),e=this.checkForNumeric(this.element.val())+Number(this.element.attr("step")||1);return t&&e>t?this.updateVal(t):this.updateVal(e)},decreaseValue:function(){var t=this.element.attr("min"),e=this.checkForNumeric(this.element.val())-Number(this.element.attr("step")||1);return t&&e<t?this.updateVal(t):this.updateVal(e)},updateVal:function(t){this.element.val(t).trigger("change"),this.updateAria(t)},checkForNumeric:function(t){return""===t?t:$.isNumeric(t)?Number(t):(t=parseInt(t,10),$.isNumeric(t)?Number(t):0)},updateAria:function(t){var e=this.element.attr("min"),i=this.element.attr("max");t=this.checkForNumeric(t),this.element[0].setAttribute("aria-valuenow",t||"0"),this.element[0].setAttribute("autocomplete","off"),this.setIsDisabled(this.buttons.up,""!==t&&i&&t>=i?"disable":"enable"),this.setIsDisabled(this.buttons.down,""!==t&&e&&t<=e?"disable":"enable")},addButtonStyle:function(t){if(!this.isDisabled()&&!this.isReadonly()){var e=t;t.currentTarget&&(e=$(t.currentTarget)),e.addClass("is-active")}},removeButtonStyle:function(t){if(!this.isDisabled()&&!this.isReadonly()){var e=t;t.currentTarget&&(e=$(t.currentTarget)),e.removeClass("is-active")}},enable:function(){this.element.prop("disabled",!1),this.element.parent(".spinbox-wrapper").removeClass("is-disabled is-readonly"),this.element.prop("readonly",!1)},disable:function(){var t=this;this.element.prop("disabled",!0),this.element.attr("disabled","disabled"),this.element.parent(".spinbox-wrapper").addClass("is-disabled"),setTimeout((function(){t.element.parent(".spinbox-wrapper").addClass("is-disabled")}))},readonly:function(){this.element.prop("readonly",!0),this.element.parent(".spinbox-wrapper").addClass("is-readonly")},isReadonly:function(){return this.element.prop("readonly")},isDisabled:function(){return this.element.prop("disabled")},setIsDisabled:function(t,e){t[(e=void 0===e||!(!e||"enable"===e))?"addClass":"removeClass"]("is-disabled"),t[0].disabled=e},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this},destroy:function(){var t=this.element.data("mask");for(var e in t&&"function"==typeof t.destroy&&t.destroy(),this.buttons){var i=$(e).data("button");i&&i.destroy()}this.buttons.up.off("click.spinbox mousedown.spinbox"),this.buttons.up.remove(),this.buttons.down.off("click.spinbox mousedown.spinbox"),this.buttons.up.add(this.buttons.down).off("click.spinbox-control"),this.buttons.down.remove(),this.element.off("focus.spinbox blur.spinbox keydown.spinbox keyup.spinbox"),this.element.unwrap(),$.removeData(this.element[0],Wa)},bindEvents:function(){var t=this,e=!1;this.element.on("focus.spinbox",(function(){t.element.addClass("no-hover"),t.element.parent(".spinbox-wrapper").addClass("is-focused"),t.element.removeClass("is-hovered")})).on("blur.spinbox",(function(){t.element.removeClass("no-hover"),t.element.parent(".spinbox-wrapper").removeClass("is-focused"),t.element.removeClass("is-hovered"),t.settings.autocorrectOnBlur&&t.correctValue()})).on("keydown.spinbox",(function(e){t.handleKeyDown(e,t)})).on("keyup.spinbox",(function(e){t.handleKeyup(e,t)})).on("mouseover.spinbox",(function(){t.element.parent(".spinbox-wrapper").hasClass("is-focused")||t.element.addClass("is-hovered")})).on("mouseleave.spinbox",(function(){t.element.removeClass("is-hovered")})).on("afterpaste.mask",(function(){t.handleAfterPaste(t)}));var i=this.buttons.up.add(this.buttons.down[0]);return i.on("touchstart.spinbox mousedown.spinbox",(function(i){if(1===i.which){if(e||t.handleClick(i),t.isTouch)return;e=!0,t.enableLongPress(i,t),$(document).one("mouseup",(function(){t.disableLongPress(i,t),e=!1,t.element.focus()}))}})),i.on("click.spinbox-control",(function(){i.hasClass("is-disabled")&&$("svg.ripple-effect").remove()})),this}},$.fn.spinbox=function(t){return this.each((function(){var e=$.data(this,Wa);e?e.updated(t):e=$.data(this,Wa,new Ua(this,t))}))};var Ga="splitter",_a={axis:"x",side:"left",resize:"immediate",containment:null,save:!0,maxWidth:{left:"auto",right:"auto"},attributes:null};function Ya(t,e){this.settings=E.mergeSettings(t,e,_a),this.element=$(t),this.init()}Ya.prototype={init:function(){return this.build().handleEvents()},build:function(){var t,e=this,i=this,n=this.settings,s=this.element,a=s.parent(),o="x"===n.axis?"left":"top",r=a.is(".content")?a.parent():a,l=s.find(".splitter-drag-handle"),c=299,d=a.width();this.isRTL=H.isRTL(),setTimeout((function(){t=a.height()}),0),this.docBody=$("body"),this.isRightSide=s.is(".splitter-right")||"x"===n.axis&&"right"===n.side,this.isHorizontal=s.is(".splitter-horizontal")||"y"===n.axis,n.uniqueId=E.uniqueId(this.element,"splitter"),l.length||(l=$('<div class="splitter-drag-handle"><span class="audible">'.concat(H.translate("SplitterDragHandle"),"</span></div>"))).appendTo(s);var h=function(t){if($(t.target).is(".splitter-btn"))l[0].style.opacity="0";else{l[0].style.opacity="";var i,n,a,o=s[0].getBoundingClientRect();if(e.isHorizontal)return(i=t.clientX-o.left-16)<16&&(i=0),a=i>(n=o.width-32)?n:i,void(l[0].style.left="".concat(a,"px"));(i=t.clientY-o.top-16)<16&&(i=0),a=i>(n=o.height-32)?n:i,l[0].style.top="".concat(a,"px")}};s.off("mouseenter.splitter").on("mouseenter.splitter",(function(t){h(t)})).off("mousemove.splitter").on("mousemove.splitter",(function(t){h(t)}));var u=function(){var a=0,o=!1,r=function(e,n,s){i.splitTo(e,t),$(n)[s?"addClass":"removeClass"]("rotate")};e.splitterCollapseButton=$('<button type="button" class="splitter-btn" id="splitter-collapse-btn"><span class="audible">'.concat(H.translate("SplitterCollapseButton"),'</span><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-double-chevron"></use></svg></button>')),e.splitterCollapseButton.appendTo(s),s[0].offsetLeft>10&&e.splitterCollapseButton.addClass("rotate"),e.splitterCollapseButton.click((function(){if(!s.is("is-dragging")){if(i.isRTL&&!i.isHorizontal||!i.isRightSide&&"left"===n.side||i.isRightSide&&"right"===n.side){var t=i.getContainerWidth()-s.outerWidth(),e=t;o||(a=t-a,c=t-c);var l=s[0].offsetLeft;i.isRightSide&&"right"===n.side&&(l=s[0].offsetLeft+1),a>=e?l>=t?r(c,this,!0):(a=l,r(e,this,!1)):l<t?(a=l,r(e,this,!1)):(r(a,this,!0),a=e)}else{var d=s[0].offsetLeft;a<=0?d<=10?r(c,this,!0):(a=d,r(0,this,!1)):d>10?(a=d,r(0,this,!1)):(r(a,this,!0),a=0)}o=!0}}))};if(this.isRightSide){var p=r.prev();p.is(".main")?(this.leftSide=p,d=r.parent().outerWidth()-d):(this.leftSide=r,s.addClass("splitter-right")),r.addClass("is-right-side").next().addClass("flex-grow-shrink is-right-side").parent().addClass("splitter-container"),n.collapseButton&&u(),this.setSplitterContainer(r.parent())}else this.isHorizontal?(this.topPanel=s.prev(),d=this.topPanel.height(),a.addClass("splitter-container is-horizontal"),s.next().addClass("flex-grow-shrink"),s.addClass("splitter-horizontal")):(this.rightSide=r,this.leftSide=r.prev().parent(),r.prev().addClass("flex-grow-shrink").parent().addClass("splitter-container"),n.collapseButton&&u(),this.setSplitterContainer(r.parent()));if(this.isRTL&&!this.isHorizontal){var g=this.getContainerWidth();d=g>=d?g-d:d}return localStorage&&n.save&&!isNaN(parseInt(localStorage[n.uniqueId],10))&&(d=localStorage[n.uniqueId]),d=parseInt(d,10),this.isHorizontal?s[0].style.top="".concat(d,"px"):s[0].style.top=0,this.splitTo(d,t),d<=10&&this.splitterCollapseButton&&this.splitterCollapseButton.removeClass("rotate"),this.documentWidth=0,this.element.drag({axis:n.axis,containment:n.containment||"x"===n.axis?"document":"parent",containmentOffset:{left:0,top:0}}).on("dragstart.splitter",(function(){var t=r.parent().find("iframe");if(i.documentWidth=$(document).width(),t.length>0)for(var e=0,n=t.length;e<n;e++){var s=$(t[e]),a="".concat(parseInt(getComputedStyle(s.parent()[0]).width,10),"px"),o=$('<div class="overlay splitter-overlay"></div>');o.css("width",a),s.before(o)}})).on("dragend.splitter",(function(a,l){r.parent().find(".splitter-overlay").remove();var c=s[0].getBoundingClientRect(),d=window.innerWidth-c.left,h=e.isRightSide&&"right"===e.settings.side||!e.isRightSide&&"left"===e.settings.side;"y"!==n.axis&&0===c.left&&(r[0].style.width="0px",l[o]=10),n.collapseButton&&(l[o]<=10||h&&d<=21?$("#splitter-collapse-btn").removeClass("rotate"):$("#splitter-collapse-btn").addClass("rotate")),"end"===n.resize&&i.splitTo(l[o],t),"immediate"===n.resize&&e.isRTL&&!e.isHorizontal&&setTimeout((function(){var n=parseInt(e.element.css("left"),10);i.splitTo(n,t)}),0)})).on("drag.splitter",(function(t,e){return!(e.left<=0)&&("immediate"===n.resize&&i.splitTo(e[o],a.height()),!0)})),"y"===n.axis&&this.element.addClass("splitter-horizontal"),this.element.attr({"aria-dropeffect":"move",tabindex:"0","aria-grabbed":"false"}),E.addAttributes(this.element,this,this.settings.attributes,"",!0),E.addAttributes(this.element.find(".splitter-drag-handle"),this,this.settings.attributes,"handle",!0),E.addAttributes(this.element.find(".splitter-drag-handle svg.icon"),this,this.settings.attributes,"icon",!0),this},setSplitterContainer:function(t){this.container=this.element.closest(".splitter-container"),!this.container.length&&t&&t.length&&(t.addClass("splitter-container"),this.container=this.element.closest(".splitter-container"))},getContainerWidth:function(){var t;return(null===(t=this.container)||void 0===t||null==t?void 0:t.outerWidth())||0},toggleSelection:function(){this.element.toggleClass("is-dragging")},resizeTop:function(t,e,i){(e>i||e<0)&&(e=parseInt(i,10)/2),this.topPanel[0].style.height="".concat(e,"px")},resizeLeft:function(t,e){var i=this.isRTL?e:this.leftSide.outerWidth()-e;this.rightSide[0].style.width="".concat(i,"px"),t[0].style.left=""},resizeRight:function(t,e){var i=e,n=e-1;if(this.isRTL&&!this.isHorizontal){var s=this.getContainerWidth();i=s>=e?s-e:e,n=e}this.leftSide[0].style.width="".concat(i,"px"),t[0].style.left="".concat(n,"px")},splitTo:function(t,e){var i=this.settings,n=this.element;this.isRightSide?((!this.isRTL&&t>i.maxWidth.right||this.isRTL&&t<i.maxWidth.right)&&(t=i.maxWidth.right),this.resizeRight(n,t)):this.isHorizontal?this.resizeTop(n,t,e):((!this.isRTL&&t>i.maxWidth.left||this.isRTL&&t<i.maxWidth.left)&&(t=i.maxWidth.left),this.resizeLeft(n,t)),this.element.trigger("split",[t]),this.docBody.triggerHandler("resize",[this]),this.settings.save&&localStorage&&(localStorage[this.settings.uniqueId]=t),this.split=t,this.parentHeight=e},unbind:function(){return this.element.off("updated.".concat(Ga)),this.splitterCollapseButton&&this.splitterCollapseButton.remove(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,_a)),this.unbind().init()},destroy:function(){return this.unbind(),$.removeData(this.element[0],Ga),this},handleEvents:function(){var t=this;return this.element.on("updated.".concat(Ga),(function(){t.updated()})).on("keydown.".concat(Ga),(function(e){32===e.which&&(t.toggleSelection(),e.preventDefault()),37===e.which&&t.splitTo(t.split-15,t.parentHeight),39===e.which&&t.splitTo(t.split+15,t.parentHeight)})),this}},$.fn.splitter=function(t){return this.each((function(){var e=$.data(this,Ga);e?e.updated(t):e=$.data(this,Ga,new Ya(this,t))}))};var Ka="swaplist",Xa={searchable:!1,available:null,selected:null,additional:null,availableClass:".available",selectedClass:".selected",additionalClass:".full-access",availableBtn:".btn-moveto-selected",selectedBtnLeft:".btn-moveto-left",selectedBtnRight:".btn-moveto-right",additionalBtn:".btn-moveto-selected",draggable:{available:!0,selected:!0,additional:!0},keepInList:{available:!1,selected:!1,additional:!1},attributes:null,attributesOverride:!0,template:'<ul data-swap-handle=".handle">{{#dataset}}{{#text}}<li data-id="{{id}}"{{#value}} data-value="{{value}}"{{/value}}{{#selected}} selected="selected"{{/selected}}{{#disabled}} class="is-disabled"{{/disabled}}><span class="handle" focusable="false" aria-hidden="true" role="presentation">&#8286;</span><div class="swaplist-item-content"><p>{{text}}</p></div></li>{{/text}}{{/dataset}}</ul>'};function Qa(t,e){this.settings=E.mergeSettings(t,e,Xa),this.element=$(t),this.init()}Qa.prototype={init:function(){var t=this,e=this.settings;e.draggable=$.extend(!0,Xa.draggable,e.draggable),this.isTouch=x.features.touch,this.isAdditional=$("".concat(e.additionalClass," .listview"),this.element).length>0,this.isTouch&&this.element.addClass("is-touch"),this.loadListview(),this.initDataset(),this.setElements(),this.isMultiSelectClass(),setTimeout((function(){t.makeDraggable(),t.handleEvents(),t.initSelected(e.availableClass),t.initSelected(e.additionalClass)}),0)},loadListview:function(){for(var t=this.settings,e=[{dataset:t.available,class:t.availableClass,draggable:t.draggable.available},{dataset:t.selected,class:t.selectedClass,draggable:t.draggable.selected},{dataset:t.additional,class:t.additionalClass,draggable:t.draggable.additional}],i=0,n=e.length;i<n;i++){var s=e[i],a=$("".concat(s.class," .listview"),this.element),o=a.data("listview"),r={dataset:s.dataset||[],selectable:"multiple",showCheckboxes:!1};(!0===t.searchable||"true"===t.searchable)&&$("".concat(s.class," .searchfield"),this.element).length>0&&(r.searchable=!0),!s.dataset&&a.length&&$("li",a).length?a.listview(r):a.length&&(o&&o.destroy(),-1===t.template.indexOf('data-id="{{id}}"')&&(t.template=t.template.replace("<li",'<li data-id="{{id}}"')),r.template=t.template,r.dataset=s.dataset||[],r.dataset.forEach((function(t){/string|number/.test(l(t.id))&&""!==t.id||(t.id="sw".concat((Date.now().toString(36)+Math.round(Math.random()*Math.pow(36,12)).toString(36).substr(2,5)).toLowerCase()))})),0===r.dataset.length&&(r.forceToRenderOnEmptyDs=!0),r.attributesOverride=t.attributesOverride,r.attributes=this.getLvAutomationAttributes(s.class),a.listview(r)),s.draggable||$("ul",a).addClass("is-not-droppable")}},getLvAutomationAttributes:function(t){var e,i=this,n=this.settings;if(n.attributes&&"string"==typeof t&&t.length>1){var s="-swaplist-".concat(t.substring(1));if(Array.isArray(n.attributes))e=[],n.attributes.forEach((function(t){var n="function"==typeof t.value?t.value(i):t.value;e.push({name:t.name,value:n+s})}));else{var a="function"==typeof n.attributes.value?n.attributes.value(this):n.attributes.value;e={name:n.attributes.name,value:a+s}}}return e},setElements:function(){var t=this,e=this.settings,i="";if(this.offset=null,this.containers=$("".concat(e.availableClass,",").concat(e.selectedClass,",").concat(e.additionalClass),this.element),this.actionButtons=$("".concat(e.availableBtn,",").concat(e.additionalBtn,",").concat(e.selectedBtnLeft,",").concat(e.selectedBtnRight),this.element),this.selectedButtons=$("".concat(e.selectedBtnLeft,",").concat(e.selectedBtnRight),this.element),this.tabButtonsStr="".concat(e.availableBtn,", ").concat(e.additionalBtn,", ").concat(this.selectedButtons.length>1?e.selectedBtnRight:e.selectedBtnLeft),this.dragElements="ul:not(.is-not-droppable), ul:not(.is-not-droppable) li:not(.is-disabled)",this.dragStart="dragstart.swaplist touchstart.swaplist gesturestart.swaplist",this.dragEnterWhileDragging="dragenter.swaplist",this.dragOverWhileDragging="dragover.swaplist touchmove.swaplist gesturechange.swaplist",this.dragEnd="dragend.swaplist touchend.swaplist touchcancel.swaplist gestureend.swaplist",this.selections={items:[],owner:null,related:null,droptarget:null,isInSelection:null,isHandle:null,placeholder:null,placeholderTouch:null,dragged:null,draggedIndex:null},e.numOfSelectionsClass="num-of-selections",e.itemContentClass="swaplist-item-content",e.itemContentTempl=$('<div><p><span class="'.concat(e.numOfSelectionsClass,'">###</span>\n      <span class="').concat(e.numOfSelectionsClass,'-text">&nbsp;</span></p><div/>')),e.attributes){var n=[{btn:$("".concat(e.availableClass," ").concat(e.availableBtn),this.element),suffix:(e.availableClass||"").substring(1)},{btn:$("".concat(e.selectedClass," ").concat(e.selectedBtnLeft),this.element),suffix:"".concat((e.selectedClass||"").substring(1),"-left")},{btn:$("".concat(e.selectedClass," ").concat(e.selectedBtnRight),this.element),suffix:"".concat((e.selectedClass||"").substring(1),"-right")},{btn:$("".concat(e.additionalClass," ").concat(e.additionalBtn),this.element),suffix:(e.additionalClass||"").substring(1)}];n.forEach((function(t){t.suffix="swaplist-btn-".concat(t.suffix)})),n.forEach((function(i){return E.addAttributes(i.btn,t,e.attributes,i.suffix,e.attributesOverride)})),E.addAttributes(this.element,this,e.attributes,"swaplist",e.attributesOverride)}e.draggable.available||(i="".concat(e.availableClass," ").concat(e.availableBtn,",").concat(e.selectedBtnLeft),$(i,this.element).prop("disabled",!0)),e.draggable.selected||(i="".concat(e.selectedBtnLeft,",").concat(e.selectedBtnRight),e.draggable.available||(i+=",".concat(e.additionalClass," ").concat(e.additionalBtn)),this.isAdditional&&e.draggable.additional||(i+=",".concat(e.availableClass," ").concat(e.availableBtn)),$(i,this.element).prop("disabled",!0),$("".concat(e.additionalClass," ").concat(e.additionalBtn),this.element).addClass("is-rotate")),e.draggable.additional||$("".concat(e.additionalClass," ").concat(e.additionalBtn,",").concat(e.selectedBtnRight),this.element).prop("disabled",!0)},isMultiSelectClass:function(){for(var t=this.settings,e=[t.availableClass,t.selectedClass,t.additionalClass],i=0,n=e.length;i<n;i++){var s=$("".concat(e[i]," .listview"),this.element);$("li",s).length||s.addClass("is-multiselect")}},initSelected:function(t){if((t=this.isjQuery(t)?t:$(t,this.element)).length){for(var e=$(".listview",t).data("listview"),i=$("li[selected]",t),n=0,s=i.length;n<s;n++){var a=$(i[n]);a.removeAttr("selected"),e.select(a)}this.moveElements(t,this.settings.selectedClass)}},moveElements:function(t,e){var i=this;if(null!==e){t="string"!=typeof t?t:$(t,this.element),e="string"!=typeof e?e:$(e,this.element);var n=$(".listview",t).data("listview");if(this.clearSelections(),this.selections.owner=t,this.selections.droptarget=e,this.isTouch){if(n.selectedItems)for(var s=0,a=n.selectedItems.length;s<a;s++)this.selections.items[s]=n.selectedItems[s].closest("li")}else this.selections.items=n.selectedItems;if(this.setSelectionsItems(this.selections.owner),this.unselectElements(n),this.selections.items.length){this.selections.move={items:this.selections.itemsData,from:this.getContainer(this.selections.itemsData)};var o=this.element.triggerHandler("beforeswap",[this.selections.move]);if("boolean"==typeof o&&!o||"string"==typeof o&&"false"===o.toLowerCase())return;var r=$("ul",e),l=$("li",r).length,c=this.selections.items.length+l;if(this.selections.items)for(var d=this.getDataList(e),h=function(){var n=$(i.selections.items[u]);if(i.isKeepInList(e)&&d.some((function(t){return t.id===n.data().id})))n.remove();else{var s=i.isKeepInList(t)?n.clone():n;s.attr({"aria-posinset":l+u+1,"aria-setsize":c}).find("mark.highlight").contents().unwrap(),r.append(s)}},u=0,p=this.selections.items.length;u<p;u++)h();this.afterUpdate($(".listview",e).data("listview"))}}},getContainer:function(t){var e=this.settings;if("object"===l(t[0])&&t[0].node){var i=t[0].node.closest(".card"),n="";return i.is(e.availableClass)?n=e.availableClass:i.is(e.selectedClass)?n=e.selectedClass:i.is(e.additionalClass)&&(n=e.additionalClass),{container:i,class:n}}return null},unselectElements:function(t){if(t.selectedItems)for(var e=0,i=t.selectedItems.length;e<i;e++)t.select($(t.selectedItems[e]));t&&t.element&&t.element.find("li").attr("aria-selected",!1).removeClass("is-selected")},isDragAndDropSupports:function(){var t=document.createElement("div");return"draggable"in t||"ondragstart"in t&&"ondrop"in t},isMatchMediaSupports:function(){return void 0!==window.matchMedia||void 0!==window.msMatchMedia},viewport:function(){var t=window,e="inner";return"innerWidth"in window||(e="client",t=document.documentElement||document.body),{width:t["".concat(e,"Width")],height:t["".concat(e,"Height")]}},isMaxWidth:function(t){return this.isMatchMediaSupports()&&window.matchMedia("(max-width: ".concat(t,"px)")).matches||this.viewport().width<=t},makeDraggable:function(){var t=this,e=$("ul",this.element);this.isDragAndDropSupports&&(this.handle=e.first().attr("data-swap-handle"),this.handle=!this.isTouch&&$(this.handle,e).length>0?this.handle:null,$(this.handle,e).addClass("draggable").off("mousedown.swaplist touchstart.swaplist").on("mousedown.swaplist touchstart.swaplist",(function(){t.selections.isHandle=!0})).off("mouseup.swaplist touchend.swaplist").on("mouseup.swaplist touchend.swaplist",(function(){t.selections.isHandle=!1})),this.targets=e.attr({"aria-dropeffect":"none"}),this.items=$("li:not(.is-disabled)",this.element).not("a[href], img").off("selectstart.swaplist").on("selectstart.swaplist",(function(){return this.dragDrop&&this.dragDrop(),!1})).end().attr({draggable:!0}).addClass(this.handle?"":"draggable"))},getElementByTouchInList:function(t,e,i){for(var n=!1,s=$(t),a=0,o=s.length;a<o;a++){var r=$(s[a]),l=r.offset();e<=l.left||e>=l.left+r.outerWidth()||i<=l.top||i>=l.top+r.outerHeight()||(n=r)}return n},draggTouchElement:function(t,e){var i=t.originalEvent.changedTouches[0];e[0].style.top="".concat(i.pageY-this.offset.y,"px"),e[0].style.left="".concat(i.pageX-this.offset.x,"px")},hasModifier:function(t){return t.ctrlKey||t.metaKey||t.shiftKey},addDropeffects:function(){if(this.targets)for(var t=0,e=this.targets.length;t<e;t++)$(this.targets[t]).attr({"aria-dropeffect":"move",tabindex:0});if(this.selections&&this.selections.items)for(var i=0,n=this.selections.items.length;i<n;i++)$(this.selections.items[i]).attr({"aria-grabbed":!0,tabindex:0})},clearDropeffects:function(){if(this.targets.attr({"aria-dropeffect":"none"}).removeAttr("tabindex"),this.selections&&this.selections.items)for(var t=0,e=this.selections.items.length;t<e;t++){var i=$(this.selections.items[t]);i.removeAttr("aria-grabbed".concat(i.is(":focus")?"":" tabindex"))}},clearSelections:function(){this.selections.items=[],this.selections.itemsData=[],this.selections.owner=null,this.selections.related=null,this.selections.droptarget=null,this.selections.isInSelection=null,this.selections.dragged=null,this.selections.placeholder=null,this.selections.placeholderTouch=null,$("ul, li",this.element).removeClass("over"),$("#sl-placeholder-container, #sl-placeholder-touch, #sl-placeholder-touch2, #sl-placeholder").remove()},setSelectionsItems:function(t){t=this.isjQuery(t)?t:$(t,this.element);var e=$(".listview li",t),i=t.find(".listview").data("listview"),n=this.getDataList(t),s=!1;i&&i.filteredDataset&&(n=u(i.filteredDataset),s=!0);for(var a=0,o=e.length;a<o;a++){var r=$(e[a]),l=n[a];s&&(l.node=r,delete l._isFilteredOut),r.is(".is-selected")&&this.selections.itemsData.push(l)}},initDataset:function(){var t=this.settings,e=[{type:"available",dataset:t.available,class:t.availableClass},{type:"selected",dataset:t.selected,class:t.selectedClass},{type:"additional",dataset:t.additional,class:t.additionalClass}];this.dataset={available:[],selected:[]},this.isAdditional&&(this.dataset.additional=[]);for(var i=0,n=e.length;i<n;i++)for(var s=e[i],a=$("".concat(s.class," .listview li"),this.element),o=0,r=a.length;o<r;o++){var l=void 0,c=void 0,d=$(a[o]);s.dataset?delete(l=JSON.parse(JSON.stringify(s.dataset[o]))).selected:(l={text:$.trim($(".swaplist-item-content",d).text())},(c=d.attr("data-value"))&&(l.value=c)),this.dataset[s.type]&&(l.node=d,this.dataset[s.type].push(l))}},getDataList:function(t){var e=this.settings,i=this.dataset;return(t=this.isjQuery(t)?t:$(t,this.element)).is(e.additionalClass)?i.additional:t.is(e.selectedClass)?i.selected:t.is(e.availableClass)?i.available:[]},arrayIndexMove:function(t,e,i){t.splice(i,0,t.splice(e,1)[0])},syncDataset:function(t,e){var i=this,n=$(".listview li",e),s=t.find(".listview").data("listview"),a=e.find(".listview").data("listview"),o=this.getDataList(t),r=this.getDataList(e),l=function(t,e){if(t&&e){var i={owner:t.getAttribute("data-id"),item:e.getAttribute("data-id")};return void 0!==i.owner&&void 0!==i.item&&i.owner===i.item}return!1};if(t.is(e)){for(var c=[],d=0,h=n.length;d<h;d++)for(var u=n[d],p=0,g=o.length;p<g;p++){var f=o[p];if(l(f.node[0],u)){c.push(f);break}}o.splice(0,o.length);for(var m=0,v=c.length;m<v;m++)o.push(c[m])}else for(var b=0,y=this.selections.items.length;b<y;b++)for(var w=this.selections.items[b],x=!0,C=0,k=n.length;C<k&&x;C++){if($(n[C]).data().id===w.data().id)for(var T=function(){var n=o[A];if(l(n.node[0],w[0]))return i.isKeepInList(e)&&r.some((function(t){return t.id===n.id}))||(r.push(n),i.arrayIndexMove(r,r.length-1,C)),i.isKeepInList(t)||o.splice(A,1),x=!1,"break"},A=0,S=o.length;A<S;A++){if("break"===T())break}}s.updated({dataset:o}),a.updated({dataset:r}),this.makeDraggable()},isjQuery:function(t){return t&&(t instanceof jQuery||t.constructor.prototype.jquery)},isKeepInList:function(t){var e=this.settings;return t.is(e.availableClass)&&e.keepInList.available||t.is(e.selectedClass)&&e.keepInList.selected||t.is(e.additionalClass)&&e.keepInList.additional},updateAttributes:function(t){for(var e=$("li",t),i=0,n=e.length;i<n;i++)$(e[i]).attr({"aria-posinset":i+1,"aria-setsize":n})},afterUpdate:function(t){var e=this.selections.droptarget.find("li:focus").index(),i=".card.".concat(this.selections.droptarget[0].classList[1]," li");t&&(this.selections.placeholder&&(t.select(this.selections.placeholder),this.selections.placeholder.focus()),this.unselectElements(t),this.syncDataset(this.selections.owner,this.selections.droptarget),this.updateAttributes($(".listview",this.selections.owner)),this.updateAttributes($(".listview",this.selections.droptarget)),this.selections.items.length&&(this.selections.move=$.extend(!0,this.selections.move,{to:this.getContainer([{node:this.selections.droptarget.find("li:first")}])}),this.element.triggerHandler("swapupdate",[this.selections.move]))),this.selections.items.forEach((function(t){t.show()})),this.clearDropeffects(),this.clearSelections(),this.items.removeClass("is-dragging is-dragging-touch"),e>=0&&this.element.find(i).eq(e).focus()},getItems:function(t){return t=this.isjQuery(t)?t:$(t,this.element),this.getDataList(t)},getAvailable:function(){return this.getDataList(this.settings.availableClass)},getSelected:function(){return this.getDataList(this.settings.selectedClass)},getAdditional:function(){return this.getDataList(this.settings.additionalClass)},draggedMakeSelected:function(t,e){var i=!1;if(!this.selections.isInSelection){if(t.selectedItems)for(var n=0,s=t.selectedItems.length;n<s;n++)if(e.is(t.selectedItems[n]))return i=!0,!1;i||(t.select(e),this.selections.isInSelection=!0)}return!0},updateDataset:function(t){for(var e=this.settings,i=[{type:"available",dataset:t.available,class:e.availableClass},{type:"selected",dataset:t.selected,class:e.selectedClass},{type:"additional",dataset:t.additional,class:e.additionalClass}],n=0,s=i.length;n<s;n++){var a=i[n],o=$("".concat(a.class," .listview"),this.element),r=o.data("listview");r&&(r.deselectItemsBetweenIndexes([0,$("li",o).length-1]),e[a.type]=a.dataset||[],r.loadData(e[a.type]))}this.initDataset(),this.makeDraggable(),this.initSelected(e.availableClass),this.initSelected(e.additionalClass)},unbind:function(){return this.actionButtons.off("click.swaplist"),this.containers.off("keydown.swaplist",".listview"),this.selectedButtons.off("keydown.swaplist"),this.element.off("keydown.swaplist",this.tabButtonsStr),this.element.off("".concat(this.dragStart," ").concat(this.dragEnterWhileDragging," ").concat(this.dragOverWhileDragging," ").concat(this.dragEnd),this.dragElements),$("#sl-placeholder-container, #sl-placeholder-touch, #sl-placeholder-touch2, #sl-placeholder").remove(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,Xa)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],Ka)},handleEvents:function(){var t=this,e=t.settings,i=t.selections;t.actionButtons.off("click.swaplist").on("click.swaplist",(function(){var i=$(this),n=i.closest(".card"),s=null;n.is(e.availableClass)?(s=e.draggable.selected?e.selectedClass:t.isAdditional&&e.draggable.additional?e.additionalClass:null,t.moveElements(e.availableClass,s)):n.is(e.additionalClass)?(s=e.draggable.selected?e.selectedClass:e.draggable.available?e.availableClass:null,t.moveElements(e.additionalClass,s)):n.is(e.selectedClass)&&(i.is(e.selectedBtnLeft)?t.moveElements(e.selectedClass,e.availableClass):i.is(e.selectedBtnRight)&&t.moveElements(e.selectedClass,e.additionalClass))})),t.containers.on("keydown.swaplist",".listview",(function(i){var n=$(this).closest(t.containers);77===(i=i||window.event).keyCode&&t.hasModifier(i)&&(!n.is(e.selectedClass)||n.is(e.selectedClass)&&1===t.selectedButtons.length?n.find(t.actionButtons).trigger("click.swaplist"):t.selectedButtons.first().focus(),i.preventDefault())})),t.selectedButtons.on("keydown.swaplist",(function(e){var i,n,s=$(this);13!==(e=e||window.event).keyCode&&32!==e.keyCode||(s.trigger("click.swaplist"),e.preventDefault()),(37===e.keyCode||39===e.keyCode)&&t.selectedButtons.length>1&&(i=t.selectedButtons.index(this),n=37===e.keyCode?i>0?i-1:t.selectedButtons.length-1:i<t.selectedButtons.length-1?i+1:0,t.selectedButtons[n].focus())})),t.element.on("keydown.swaplist",t.tabButtonsStr,(function(t){var e=$(this);if(9===(t.keyCode||t.which)&&!t.shiftKey){var i=e.closest(".card")[0],n=[].slice.call(i.querySelectorAll("li[tabindex]")),s=n.length,a=!1;if(s&&n.forEach((function(t){-1!==parseInt(t.getAttribute("tabindex"),10)&&(a=!0)})),!a){var o=i.querySelector("li");o&&o.setAttribute("tabindex",0)}}})),t.containers.on("filtered.swaplist",".listview",(function(){t.makeDraggable()})),t.element.on("mousedown.swaplist",t.dragElements,(function(e){t.handle&&$(e.target).closest("li").attr({draggable:$(e.target).is(".draggable")});e.stopPropagation()})).on(t.dragStart,t.dragElements,(function(n){if(n.stopImmediatePropagation(),!t.handle||i.isHandle){var s,a,o=$(n.target).closest("li"),r=$(".listview",o.closest(".card")).data("listview");if(r){t.isTouch||t.draggedMakeSelected(r,o),t.clearSelections(),i.owner=o.closest(".card"),i.dragged=o,i.draggedIndex=o.index(),i.placeholder=o.clone(!0),i.placeholder.attr("id","sl-placeholder"),t.setSelectionsItems(i.owner),i.items=r.selectedItems,t.selections.move={items:t.selections.itemsData,from:t.getContainer(t.selections.itemsData)};var l=t.element.triggerHandler("beforeswap",[t.selections.move]);if("boolean"==typeof l&&!l||"string"==typeof l&&"false"===l.toLowerCase())i.dragged=null;else{if($(".".concat(e.numOfSelectionsClass),e.itemContentTempl).html(i.items.length),$(".".concat(e.numOfSelectionsClass,"-text"),e.itemContentTempl).text(H.translate("ItemsSelected")),t.addDropeffects(),t.isTouch){s=o[0].getBoundingClientRect(),a=n.originalEvent.changedTouches[0],t.offset={x:a.pageX-s.left,y:a.pageY-s.top};for(var c=0,d=t.containers.length;c<d;c++)t.containers[c].style.zIndex="1";i.placeholderTouch=i.dragged.clone(!0),i.items.length>1&&!$("#sl-placeholder-touch2").length&&i.dragged.clone().addClass("is-dragging-touch").attr("id","sl-placeholder-touch2").insertBefore(i.dragged).hide(),i.placeholderTouch.attr("id","sl-placeholder-touch").removeClass("is-selected").hide(),(t.element.is(".one-third")&&t.isMaxWidth(766)?t.element.parent():t.element).prepend('<ul id="sl-placeholder-container"></ul>'),$("#sl-placeholder-container").append(i.placeholderTouch),$("#sl-placeholder-container")[0].style.width="".concat(i.owner.width(),"px"),$("#sl-placeholder-touch")[0].style.width="".concat(i.owner.width(),"px"),t.draggTouchElement(n,i.placeholderTouch)}else i.dragged.addClass("is-dragging"),n.originalEvent.dataTransfer.setData("text",""),i.items.length>1&&$(".".concat(e.itemContentClass),i.dragged).html(e.itemContentTempl.html());n.stopPropagation()}}}else n.stopPropagation()})).on(t.dragEnterWhileDragging,t.dragElements,(function(e){i.dragged&&(t.element.triggerHandler("draggingswap",[i.move]),i.related=e.target,$("ul, li",t.element).removeClass("over"),$(e.target).closest("ul, li").addClass("over"),i.droptarget=$(i.related).closest(".card"),$('[aria-grabbed="true"]',t.element).not(i.dragged).slideUp(),e.stopPropagation())})).on(t.dragOverWhileDragging,t.dragElements,(function(n){if(i.dragged){var s,a=$(this),o=$(".listview",i.dragged.closest(".card")).data("listview");if(t.isTouch){if(t.handle&&!i.isHandle)return;i.isInSelection||(t.draggedMakeSelected(o,i.dragged),i.items=o.selectedItems,$(".".concat(e.numOfSelectionsClass),e.itemContentTempl).html(i.items.length)),s=n.originalEvent.touches[0],a=t.getElementByTouchInList($("ul, li",t.element),s.pageX,s.pageY)||a,i.dragged.addClass("is-dragging"),i.placeholderTouch.addClass("is-dragging is-dragging-touch"),i.placeholderTouch.show(),$('[aria-grabbed="true"]',t.element).not(i.dragged).not(i.placeholderTouch).not("#sl-placeholder-touch2").slideUp(),i.items.length>1&&($(".".concat(e.itemContentClass),i.placeholderTouch.add("#sl-placeholder-touch2")).html(e.itemContentTempl.html()),$("#sl-placeholder-touch2").show(),i.dragged.hide()),t.draggTouchElement(n,i.placeholderTouch),t.element.triggerHandler("draggingswap",[i.move]),i.related=a,$("ul, li",this.element).removeClass("over"),a.closest("ul, li").addClass("over"),i.droptarget=i.related.closest(".card")}n.preventDefault(),n.stopPropagation()}})).on(t.dragEnd,t.dragElements,(function(n){if(i.dragged&&i.droptarget){var s=$(i.related).closest("li"),a=$("ul",i.droptarget),o=$("li",a).length,r=i.items.length+o;if(t.unselectElements($(".listview",i.owner).data("listview")),$.each(i.items,(function(t,e){if((e=$(e)).find("mark.highlight").contents().unwrap(),o&&!$(i.related).is("ul")){var n=s.index()<i.draggedIndex&&i.owner.is(i.droptarget),l=n?e:$(i.items[i.items.length-1-t]),c=s.index()+(n?t+1:t+2);e.attr({"aria-posinset":c,"aria-setsize":r}),s[n?"before":"after"](l)}else e.attr({"aria-posinset":o+t+1,"aria-setsize":r}),a.append(e);e.focus()})),i.items.length>1&&($(".".concat(e.itemContentClass),i.dragged).html($(".".concat(e.itemContentClass),i.placeholder).html()),t.isTouch&&i.dragged.show()),t.isTouch)for(var l=0,c=t.containers.length;l<c;l++)t.containers[l].style.zIndex="";t.makeDraggable(),i.isHandle=null,$('[aria-grabbed="true"]',t.element).show(),t.afterUpdate($(".listview",i.droptarget).data("listview")),n.preventDefault(),n.stopPropagation()}}))}},$.fn.swaplist=function(t){return this.each((function(){var e=$.data(this,Ka);e?e.updated(t):e=$.data(this,Ka,new Qa(this,t))}))};var Ja="swipe-container",Za={swipeType:"reveal"};function to(t,e){return this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Za),this.init()}to.prototype={init:function(){this.render(),this.handleEvents()},render:function(){this.container=this.element[0].querySelector(".swipe-container"),this.swipeType=this.settings.swipeType;var t=this.element[0].querySelector(".btn-swipe-action-left"),e=this.element[0].querySelector(".btn-swipe-action-right");t&&"reveal"===this.swipeType&&(t.setAttribute("tabindex","-1"),t.setAttribute("aria-hidden","true")),e&&"reveal"===this.swipeType&&(e.setAttribute("tabindex","-1"),e.setAttribute("aria-hidden","true")),t&&"reveal"===this.swipeType&&(this.container.scrollLeft=85)},handleEvents:function(){var t=this,e=0,i=0,n=0;if("reveal"===this.swipeType)return this.element.find(".btn-swipe-action-right, .btn-swipe-action-left").on("click.swipe-action",(function(){t.container.scrollLeft=85,t.element.focus()})),void $(this.container).on("scroll.swipe-action",(function(){var e,i;null===(e=t.element.find(".btn-actions"))||void 0===e||null===(i=null==e?void 0:e.data("popupmenu"))||void 0===i||null==i||i.close()}));this.element.on("touchstart.swipe-action",(function(t){e=t.changedTouches[0].screenX}),{passive:!0}),this.element.on("touchend.swipe-action",(function(n){var s,a,o="";(i=n.changedTouches[0].screenX)<e&&(o="left"),i>e&&(o="right"),o&&(t.element.find(".swipe-action-".concat("left"===o?"right":"left"," button")).click(),null===(s=t.element.find(".btn-actions"))||void 0===s||null===(a=null==s?void 0:s.data("popupmenu"))||void 0===a||null==a||a.close())}),{passive:!0}),$(this.container).on("scroll.swipe-action",(function(e){var i,s,a=e.target,o=a.scrollLeft/(a.scrollWidth-a.clientWidth)*100;if(null===(i=t.element.find(".btn-actions"))||void 0===i||null===(s=null==i?void 0:i.data("popupmenu"))||void 0===s||null==s||s.close(),!(Math.abs(n-o)<1)){n=o;var r="";0===o&&(r="right"),o>98&&(r="left"),r&&t.element.find(".swipe-action-".concat("left"===r?"right":"left"," button")).click()}}))},set swipeType(t){if("continuous"===t)return this.settings.swipeType=t,void this.container.classList.add("continuous");this.settings.swipeType="reveal",this.container.classList.remove("continuous")},get swipeType(){return this.settings.swipeType},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown(),this.init()},teardown:function(){this.element.find(".btn-swipe-action-right, .btn-swipe-action-left").off("click.swipe-action"),this.element.off("scroll.swipe-action").off("click.swipe-action"),this.element.off("touchstart.swipe-action"),this.element.off("touchend.swipe-action"),$(this.container).off("scroll.swipe-action")},destroy:function(){this.teardown(),$.removeData(this.element[0],"swipeaction")}},$.fn.swipeaction=function(t){return this.each((function(){var e=$.data(this,Ja);return e?e.updated(t):e=$.data(this,Ja,new to(this,t)),e}))};var eo="scrollaction",io={scrollActionTarget:".js-scroll-target",classToAdd:"scrolled-down"};function no(t,e){this.settings=E.mergeSettings(t,e,io),this.element=$(t),this.init()}no.prototype={init:function(){this.handleEvents()},unbind:function(){var t=this.settings;return this.element.off("scroll.scrollaction"),t&&$(t.scrollActionTarget).removeClass(t.classToAdd),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,io)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],eo)},handleEvents:function(){var t=this,e=this.settings;this.lastScrollTop=0,this.element.on("scroll.scrollaction",(function(){var i=t.element.scrollTop();i>t.lastScrollTop?$(e.scrollActionTarget).addClass(e.classToAdd):$(e.scrollActionTarget).removeClass(e.classToAdd),t.lastScrollTop=i}))}},$.fn.scrollaction=function(t){return this.each((function(){var e=$.data(this,eo);e?e.updated(t):e=$.data(this,eo,new no(this,t))}))};var so="stepchart",ao={steps:null,completed:null,inProgress:null,iconType:null,completedText:null,extraText:"",completedColor:null,allCompletedColor:null,inProgressColor:null,attributes:null};function oo(t,e){return this.init(t,e)}oo.prototype={init:function(t,e){if(!this.element&&t instanceof HTMLElement&&(this.element=$(t)),"object"===l(e)||void 0===this.settings){var i=this.settings||ao;this.settings=E.mergeSettings(this.element[0],e,i)}return this.render()},render:function(){var t=$('<div class="step-chart-steps"></div>'),e='\n      <svg class="icon {icon-name}" focusable="false" aria-hidden="true" role="presentation">\n        <use href="#icon-success"></use>\n      </svg>\n    ',i=null===this.settings.completed&&null===this.settings.steps;if(i&&(this.settings.completed=0,this.settings.steps=1),this.element.attr("data-options")&&(this.settings=E.parseSettings(this.element)),this.element.children().length>0)return this;for(var n=0;n<this.settings.steps;n++){var s=$('<div class="step-chart-step"></div>');E.addAttributes(s,this.settings,this.settings.attributes,"step".concat(n)),n<this.settings.completed&&(s.addClass("is-complete"),this.settings.completedColor&&s.css("background-color",this.settings.completedColor)),n===this.settings.inProgress-1&&(s.addClass("is-inprogress"),this.settings.inProgressColor&&s.css("background-color",this.settings.inProgressColor)),t.append(s)}var a=this.settings.completedText||H.translate("StepsCompleted");a=(a=a.replace("{0}",this.settings.completed)).replace("{1}",this.settings.steps),i&&(a=H?H.translate("NoData"):"No data available");var o=$('<span class="step-chart-label">'.concat(a,"</span>"));if(this.settings.steps===this.settings.completed&&(t.addClass("is-complete"),o.append(e.replace("{icon-name}","icon-success"))),this.settings.iconType&&o.append(e.replace("{icon-name}",this.settings.iconType)),this.settings.extraText){var r=this.settings.extraText;r=(r=(r="{1} Days Overdue"===(r="{0} Days Remaining"===r?H.translate("DaysRemaining"):r)?H.translate("DaysOverdue"):r).replace("{0}",this.settings.steps-this.settings.completed)).replace("{1}",this.settings.completed),o.append('<span class="step-chart-label-small">'.concat(r,"</span>"))}return this.element.append(o,t),this.settings.steps===this.settings.completed&&this.settings.allCompletedColor&&(t.find(".step-chart-step").css("background-color",this.settings.allCompletedColor),o.find(".icon").attr("style","fill: ".concat(this.settings.allCompletedColor,"!important"))),E.addAttributes(o,this.settings,this.settings.attributes,"label"),E.addAttributes(o.find(".icon"),this.settings,this.settings.attributes,"icon"),E.addAttributes(o.find(".step-chart-label-small"),this.settings,this.settings.attributes,"label-small"),this},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),this.element.empty(),this.render()},teardown:function(){return this.element.empty(),this.settings=null,$.removeData(this.element[0],so),this},destroy:function(){return this.teardown(),this}},$.fn.stepchart=function(t){return this.each((function(){var e=$.data(this,so);e?e.updated(t):(e=$.data(this,so,new oo(this,t))).destroy=function(){this.teardown(),$.removeData(this.element,so),$.removeData(this.element,"options")}}))};var ro="tabs",lo=["horizontal","vertical","module-tabs","header-tabs"],co={addTabButton:!1,addTabButtonCallback:null,addTabButtonTooltip:!1,appMenuTrigger:!1,appMenuTriggerText:void 0,appMenuTriggerTextAudible:!1,ajaxOptions:null,beforeActivate:void 0,containerElement:null,changeTabOnHashChange:!1,hashChangeCallback:null,lazyLoad:!0,moduleTabsTooltips:!1,multiTabsTooltips:!1,countsPosition:void 0,source:null,sourceArguments:{},tabCounts:!1,verticalResponsive:!1,attributes:null,maxWidth:null,sortable:!1};function ho(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,co),this.init()}ho.prototype={init:function(){return this.setup().build().setupEvents()},setup:function(){return this.tabsIndex=$(".tab-container").index(this.element),this.settings.multiTabsTooltips=this.element.closest(".multitabs-container").length>0,this},build:function(){var t,e=this,i=this,n=!1,s=["en-US","de-DE","ja-JP","zh-CN","zh-Hans","zh-TW","zh-Hant"];(t=this.element.next(".tab-panel-container")).length||(t=this.element.children(".tab-panel-container"),this.isVerticalTabs()||(n=!0));var a=$("body > .page-container, .application-menu + .page-container");this.element.closest(".header").length>0&&a.length&&(t=a),this.element.next(".application-menu").length&&(t=this.element.next().next(".page-container"),n=!1),this.settings.containerElement&&$(this.settings.containerElement).length&&(t=$(this.settings.containerElement),n=!1),t&&t.length||(t=$('<div class="tab-panel-container"></div>'),n=!0),t[0].classList.contains("tab-panel-container")||t[0].classList.add("tab-panel-container"),n&&t.insertAfter(this.element),this.container=t;var o=this.element.children(".tab-list-container");!o.length&&this.isScrollableTabs()&&(o=$('<div class="tab-list-container" tabindex="-1"></div>').prependTo(this.element)),o.length&&(this.tablistContainer=o);var r=!0,l=this.element.closest(".header");lo.forEach((function(t,i){e.element.hasClass(lo[i])&&(r=!1)})),r&&(l.length?i.element.addClass("header-tabs"):i.element.addClass("horizontal")),i.settings.tabCounts&&i.element.addClass("has-counts"),this.tablist=this.element.children(".tab-list"),this.tablist.length||(this.tablistContainer&&(this.tablist=this.tablistContainer.children(".tab-list")),this.tablist.length||(this.tablist=$('<ul class="tab-list"></ul>'),this.tablistContainer?this.tablist.appendTo(this.tablistContainer):this.tablist.appendTo(this.element))),this.tablistContainer&&(this.tablist.parent().is(this.tablistContainer)||this.tablistContainer.append(this.tablist),this.tablistContainer.on("mousewheel.tabs",(function(t){t.deltaY&&(this.scrollLeft+=t.deltaY)}))),i.tablist.attr({class:"tab-list",role:"tablist"}),this.checkResponsive(!1),this.renderHelperMarkup(),i.anchors=i.tablist.children("li:not(.separator)").children("a"),i.anchors.each((function(){var t,e=$(this),n=e[0].textContent.toLowerCase().trim().split(" ").join("-");e.attr({role:"tab","aria-selected":"false",tabindex:"-1"}).parent().attr("role","tab").addClass("tab"),e.parent().hasClass("dismissible")&&!e.parent().children(".icon").length&&(t=$.createIconElement({icon:"close",classes:"icon close"})).insertAfter(e),i.settings.attributes&&(E.addAttributes(e,i,i.settings.attributes,"".concat(n,"-a")),t&&E.addAttributes(t,i,i.settings.attributes,"".concat(n,"-close-btn")));var a=e.nextAll("ul").first();a.length>0&&(a.addClass("dropdown-tab"),e.parent().addClass("has-popupmenu").popupmenu({menu:a,trigger:"click",attachToBody:!0,attributes:i.settings.attributes}),e.removeAttr("role").removeAttr("aria-expanded").removeAttr("aria-selected"),e.parent().children(".icon.icon-more").length||$.createIconElement({classes:"icon-more",icon:"dropdown"}).insertAfter(e));i.settings.tabCounts&&0===$(this).find(".count").length&&(s.includes(H.currentLocale.name)&&"bottom"!==i.settings.countsPosition||"top"===i.settings.countsPosition?$(this).prepend('<span class="count">0 </span>'):(i.settings.countsPosition,$(this).append('<span class="count">0 </span>'))),(i.settings.moduleTabsTooltips||i.settings.multiTabsTooltips)&&e.on("beforeshow.toolbar",(function(){return"yes"===e.data("cutoffTitle")})).tooltip({content:"".concat(e.text().trim())})})),i.panels=$(),i.anchors.each((function(){var t,e=$(this),n=e.parent(),s=n.data("popupmenu"),a=e.attr("href");if("#"===a.substr(0,1)){if("#/"===a.substr(0,2)&&(a=a.replace("#/","#")),void 0!==a&&"#"!==a){if(a=a.replace(/\//g,"\\/"),t=$(a),n.is(":not(.has-popupmenu)")&&!t.length)return;if(i.settings.attributes){var o=e[0].textContent.toLowerCase().trim().split(" ").join("-");E.addAttributes(t,i,i.settings.attributes,"".concat(o,"-panel"))}e.data("panel-link",t),t.data("tab-link",e),i.panels=i.panels.add(t)}s&&s.menu.children("li").each((function(){var t=$(this),e=t.children("a"),n=e.attr("href"),s=$(n);if(e.data("panel-link",s),s.data("tab-link",e),i.panels=i.panels.add(s),i.anchors=i.anchors.add(e),t.hasClass("dismissible")){var a=t.children(".icon");a.length||(a=$.createIconElement({icon:"close",classes:"icon close"})),a.detach().appendTo(e)}})).on("click.popupmenu",".icon",(function(t){var e=$(this),n=e.closest("li");n.is(".dismissible")&&e.is(".icon")&&(t.preventDefault(),t.stopPropagation(),i.closeDismissibleTab(n.children("a").attr("href")))}))}})),i.panels.addClass("tab-panel").attr({role:"tabpanel"}).find("h3:first").attr("tabindex","0"),i.panels.each((function(){var t=$(this);t.parent().is(i.container)||i.container.append(t)}));var c,d,h=":not(.separator):not(.is-disabled):not(.is-hidden)",u=this.tablist.children("li".concat(h)),p=this.tablist.children("li.is-selected".concat(h)),g=p.children("a");return i.nestedTabControls=i.panels.find(".tab-container"),u.length&&(this.settings.changeTabOnHashChange&&(c=window.location.hash)&&c.length&&(d=u.find('a[href="'.concat(c,'"]'))).length&&(g=(p=d.first().parent()).children("a")),p.length||(g=(p=u.not(".add-tab-button, .application-menu-trigger").first()).children("a")),p.length&&this.activate(g.attr("href"),g)),this.isModuleTabs()&&this.element.children(".toolbar").length&&this.element.addClass("has-toolbar"),this.isModuleTabs()&&u.filter(".application-menu-trigger").length>0&&(u.find('a[tabindex="0"]').attr("tabindex","-1"),u.filter(".application-menu-trigger").find("a").attr("tabindex","0")),this.setOverflow(),this.positionFocusState(g),this.tablistContainer&&(this.tablistContainer.on("scroll.tabs",(function(){e.renderEdgeFading()})),this.renderEdgeFading()),this.createSortable(),"undefined"!=typeof ResizeObserver&&(this.ro=new ResizeObserver((function(){$("body").triggerHandler("resize")})),this.ro.observe(this.element[0]),this.containerElement&&this.ro.observe(this.containerElement[0])),setTimeout((function(){e.positionFocusState()}),100),this},createSortable:function(){var t=this;if(this.settings.sortable&&(!x.features.touch||this.isModuleTabs()||this.isVerticalTabs())){var e=this.tablist?this.tablist[0]:null,i=$(".tab-panel-container");if(e){[].slice.call(e.querySelectorAll(".is-disabled, .separator, .application-menu-trigger")).forEach((function(t){return t.setAttribute("data-arrange-exclude","true")})),e.setAttribute("data-options",'{"placeholderCssClass": "tab arrange-placeholder", "useItemDimensions": "true"}'),e.classList.add("arrange");var n=this.tablist.data("arrange");n||(n=this.tablist.arrange().data("arrange"));var s="has-placeholder",a=null;this.isModuleTabs()||this.isVerticalTabs()||this.isHeaderTabs()||this.tablist.on("beforearrange.tabs",(function(){a=$(this).closest(".tab-container")[0]})).on("draggingarrange.tabs",(function(){var t;null===(t=a)||void 0===t||null==t||t.classList.add(s)})),this.tablist.on("dragover.tabs",(function(t){t.preventDefault()})),i.on("dragover.tabpanel",(function(t){t.preventDefault()})),this.tablist.on("drag.tabs",(function(t){if($(".multitabs-section.is-hidden").length){var e=$(t.target).parents(".multitabs-section");e.hasClass("alternate")?e.find(".overlay-left").addClass("has-overlay"):e.find(".overlay-right").addClass("has-overlay")}})),this.tablist.on("dragend.tabs",(function(e){var i=$(e.target),n=$(document.elementFromPoint(e.pageX,e.pageY)),s=i.parents(".multitabs-section");s.find(".overlay-left").removeClass("has-overlay"),s.find(".overlay-right").removeClass("has-overlay");var a=n.parents(".module-tabs");i.get(0)!==n.get(0)&&s[0]===n.parents(".multitabs-section")[0]&&$(".multitabs-section").each((function(t,e){s[0]!==e&&$(e).hasClass("is-hidden")&&(a=$(e).children(".module-tabs"))}));var o=i.attr("href").replace("#",""),r=a.data("tabs");if(void 0!==r){var l=r.getTab(null,i);if(t.element.parent()[0]!==r.element.parent()[0]){var c=l.clone(),d=t.getPanel(o).children();t.remove(o),r.add(o,{name:c.children("a").text().trim(),content:d,doActivate:!0}),t.element.children(".tab-list").children().length<1&&t.element.parent().addClass("is-hidden"),r.element.children(".tab-list").children().length>0&&r.element.parent().removeClass("is-hidden")}}})),this.tablist.on("arrangeupdate.tabs",(function(){var t;null===(t=a)||void 0===t||null==t||t.classList.remove(s)})),this.element.on("aftertabadded.tabs",(function(){var t;null===(t=n)||void 0===t||null==t||t.updated()}))}}},renderHelperMarkup:function(){var t,e,i=this.tablistContainer||this.tablist;this.isModuleTabs()&&(i=this.tablist),this.focusState=this.element.find(".tab-focus-indicator"),this.focusState.length||(this.focusState=$('<div class="tab-focus-indicator" role="presentation"></div>').insertBefore(this.tablist)),this.moreButton||(this.moreButton=$()),this.isVerticalTabs()?this.moreButton.length&&(this.moreButton.off().removeData().remove(),this.moreButton=$()):(this.moreButton.length||(this.moreButton=i.next(".tab-more")),this.moreButton.length||(this.moreButton=$("<div>").attr({class:"tab-more"}),this.moreButton.append($('<span class="more-text">').text(H.translate("More"))),this.moreButton.append($.createIconElement({classes:"icon-more",icon:"dropdown"}))),i.after(this.moreButton)),null!==(t=this.moreButton)&&void 0!==t&&(null==t?void 0:t.length)&&this.settings.attributes&&E.addAttributes(this.moreButton,this,this.settings.attributes,"btn-more");var n,s=this.tablist.find(".application-menu-trigger");if(!0===this.settings.appMenuTrigger)if(this.isModuleTabs()){if(!s.length){var a=this.settings.appMenuTriggerTextAudible?' class="audible"':"";s=$('\n            <li class="tab application-menu-trigger">\n              <a href="#">\n                <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n                    <use href="#icon-menu"></use>\n                </svg>\n                <span'.concat(a,">").concat(this.settings.appMenuTriggerText||H.translate("AppMenuTriggerText"),"</span>\n              </a>\n            </li>\n          ")),this.tablist.prepend(s)}}else this.isVerticalTabs()&&s.length&&s.off().removeData().remove();else s.length&&(this.isVerticalTabs()?s.off().removeData().remove():this.tablist.prepend(s));s.length&&(this.settings.attributes&&E.addAttributes(s,this,this.settings.attributes,"appmenu-trigger-btn"),null===(n=$("#application-menu").data("applicationmenu"))||void 0===n||null==n||n.modifyTriggers([s.children("a")],null,!0));this.settings.addTabButton?this.addTabButton&&this.addTabButton.length||(this.addTabButton=$('\n          <div class="add-tab-button" tabindex="-1" role="button">\n            <span aria-hidden="true" role="presentation">+</span>\n            <span class="audible">'.concat(H.translate("AddNewTab"),"</span>\n          </div>\n        ")),this.addTabButton.insertAfter(this.moreButton),this.element.addClass("has-add-button")):this.addTabButton&&this.addTabButton.length&&(this.addTabButton.off().removeData().remove(),this.addTabButton=void 0,this.element.removeClass("has-add-button")),null!==(e=this.addTabButton)&&void 0!==e&&(null==e?void 0:e.length)&&this.settings.attributes&&E.addAttributes(this.addTabButton,this,this.settings.attributes,"btn-add");var o=this.element.find(".more-actions-button .btn-actions");return o.length&&(this.moreActionsBtn=o),this.settings.addTabButtonTooltip&&this.addTabButton.tooltip({content:H.translate("AddNewTab")}),this},setupEvents:function(){var t=this,e=this;return this.tablist.on("mousedown.tabs","> li",(function(t){if(e.handleAddFocusData(t,$(this)),$(t.target).hasClass("close")&&$(t.target).parent().hasClass("has-popupmenu")){var i=$(this).data("popupmenu").menu,n=[];$.each(i[0].children,(function(t,e){n.push(e.children[0].href)})),e.closeDismissibleTabs(n)}return 3!==t.which&&!$(t.target).hasClass("close")&&e.handleTabClick(t,$(this))})).on("click.tabs","a",(function(t){var e=$(t.currentTarget);!0!==this.wasTapped?("touchend"===t.type&&(this.wasTapped=!0),"#"===e.attr("href").substr(0,1)&&t.preventDefault()):this.wasTapped=!1})).on("click.tabs",".icon",(function(t){var i=$(this);if(!i.is("[disabled]")&&!i.parent().hasClass("is-disabled")){var n=$(i).parent();n.hasClass("dismissible")&&(t.preventDefault(),t.stopPropagation(),e.closeDismissibleTab(n.children("a").attr("href")))}})).on("focus.tabs","a",(function(t){return e.handleTabFocus(t,$(this))})).on("blur.tabs","a",(function(){$(this).parent().removeClass("is-focused")})).on("keydown.tabs","a",(function(e){return t.handleTabKeyDown(e)})),e.tablist.find("li").filter(".has-popupmenu").each((function(t,i){var n=$(i),s=n.children("a");function a(t){t.altKey&&n.is(".dismissible")&&e.closeDismissibleTab(s.attr("href"))}n.data("popupmenu").menu.on("keydown.popupmenu","a",(function(t){switch(t.which){case 27:n.addClass("is-selected"),s.focus();break;case 8:case 46:a(t)}})),n.on("selected.tabs",(function(t,i){var n=$(this),a=$(i).attr("href");return!!e.activate(a,$(i))&&(e.positionFocusState(s),s.focus(),e.scrollTabList(n),n.addClass("is-selected"),!1)}))})),e.tablist.find("li").filter(".dismissible").each((function(t,i){$(i).children("a").on("keydown.tabs",(function(t){e.handleDismissibleTabKeydown(t)}))})),this.setupHelperMarkupEvents(),this.panels.on("keydown.tabs",(function(t){e.handlePanelKeydown(t)})),this.element.on("focusout.tabs",(function(){var t=!$.contains(e.element[0],document.activeElement),i=0===e.tablist.children(".has-popupmenu.is-open").length;t&&i&&!e.moreButton.is(".is-selected, .popup-is-open")&&e.hideFocusState()})).on("updated.tabs",(function(t,i){e.updated(i)})).on("activated.tabs",(function(t){t.stopPropagation()})).on("add.tabs",(function(t,i,n,s){e.add(i,n,s)})).on("remove.tabs",(function(t,i){e.remove(i)})),$("body").on("resize.tabs".concat(this.tabsIndex),(function(){e.handleResize()})),e.handleResize(!0),this},setupHelperMarkupEvents:function(){var t,e=this;this.moreButton.on("click.tabs",(function(t){e.handleMoreButtonClick(t)})).on("keydown.tabs",(function(t){e.handleMoreButtonKeydown(t)})).on("focus.tabs",(function(t){e.handleMoreButtonFocus(t)})).on("mousedown.tabs",(function(t){e.handleAddFocusData(t,$(this))})),this.settings.addTabButton&&this.addTabButton.on("click.tabs",(function(){e.handleAddButton()})).on("keydown.tabs",(function(t){e.handleAddButtonKeydown(t)})).on("focus.tabs",(function(t){e.handleAddButtonFocus(t)})),null!==(t=this.moreActionsBtn)&&void 0!==t&&(null==t?void 0:t.length)&&this.moreActionsBtn.on("keydown.tabs",(function(t){e.handleMoreActionsButtonKeydown(t)}))},removeHelperMarkupEvents:function(){return this.moreButton&&this.moreButton.length&&this.moreButton.off("click.tabs keydown.tabs focus.tabs mousedown.tabs"),this.addTabButton&&this.addTabButton.length&&this.addTabButton.off("click.tabs keydown.tabs focus.tabs"),this},handleAddFocusData:function(t,e){var i=e;return i.is(".is-disabled")?(t.preventDefault(),!1):(i.is(this.moreButton)||(i=i.children("a")),this.hideFocusState(),i.data("focused-by-click",!0),!0)},handleTabClick:function(t,e){var i;if(this.element.is(".is-disabled")||e&&(e.is(".is-disabled")||e.is(".separator")))return t.stopPropagation(),t.preventDefault(),!1;if(!this.handleAppMenuTabKeydown(t))return!0;var n=e.children("a");if(n.data("focused-by-click",!0),this.popupmenu&&this.popupmenu.element.hasClass("is-open")&&this.popupmenu.close(),e.is(".has-popupmenu"))return this.positionFocusState(n),!0;var s=n.attr("href");return e.is(".add-tab-button")&&(e=(n=this.handleAddButton()).parent(),s=n.attr("href"),this.element.trigger("tab-added",[n])),2===t.which?(e.is(".dismissible")&&$(t.target).is(".close")&&this.closeDismissibleTab(s),t.preventDefault(),!0):!this.activate(s,n)||(this.changeHash(s),null===(i=this.focusState)||void 0===i||null==i||i.removeClass("is-visible"),n.focus(),this.isScrollableTabs()&&this.scrollTabList(e),this.positionFocusState(n),!0!==this.settings.lazyLoad||!this.isURL(s))},handleMoreButtonClick:function(t){return this.element.is(".is-disabled")||this.moreButton.is(".is-disabled")?(t.preventDefault(),t.stopPropagation(),!1):(this.moreButton.data("focused-by-click",!0),this.hasMoreButton()||t.stopPropagation(),this.moreButton.hasClass("popup-is-open")?(this.popupmenu.close(),this.moreButton.removeClass("popup-is-open")):this.buildPopupMenu(),this.hideFocusState(),!0)},hasMoreActions:function(){var t=this.element.find(".more-actions-button > .btn-actions");return t&&t.length>0},handleMoreActionsButtonKeydown:function(t){if(this.element.is(".is-disabled")||this.moreActionsBtn.is(".is-disabled"))return t.stopPropagation(),t.preventDefault(),!1;var e=this;var i,n,s=this.addTabButton&&this.addTabButton.length,a=this.tablist.children("li".concat(":not(.separator):not(.is-disabled):not(:hidden)"));switch(t.which){case 37:case 38:if(s)this.addTabButton.focus();else{var o=this.findLastVisibleTab();this.hasMoreButton()?(n=o.attr("href"),t.preventDefault(),setTimeout((function(){e.buildPopupMenu(n),e.positionFocusState(e.moreButton,!0)}),0)):i=o}break;case 39:case 40:i=a.first()}return i&&(t.preventDefault(),i.children("a").focus(),this.isScrollableTabs()&&this.scrollTabList(i)),!0},handleTabFocus:function(t,e){if(this.element.is(".is-disabled"))return t.preventDefault(),!1;var i=e.parent(),n=e.data("focused-by-click"),s=void 0===n||n&&!1===n;return $.removeData(e[0],"focused-by-click"),this.isTabOverflowed(i)?(this.buildPopupMenu(e.attr("href")),s&&this.moreButton.addClass("is-focused"),this.positionFocusState(this.moreButton)):(s&&i.addClass("is-focused"),this.positionFocusState(e,s)),!0},handleMoreButtonFocus:function(t){var e;if(this.element.is(".is-disabled"))t.preventDefault();else{var i=this.moreButton.data("focused-by-click"),n=i&&!1===i;$.removeData(this.moreButton[0],"focused-by-click"),null===(e=this.focusState)||void 0===e||null==e||e.removeClass("is-visible"),this.positionFocusState(this.moreButton,n)}},handleTabKeyDown:function(t){if(this.element.is(".is-disabled"))return t.preventDefault(),!1;if(t.shiftKey||t.ctrlKey||t.metaKey||t.altKey&&8!==t.which)return!0;var e=this,i=[8,13,32];function n(){return $.inArray(t.which,i)>-1}if(t.which<32&&!n()||t.which>46)return!0;if(n()&&!this.handleAppMenuTabKeydown(t))return!0;function s(i){t.preventDefault(),setTimeout((function(){e.buildPopupMenu(i)}),0)}var a,o=":not(.separator):not(.is-disabled):not(:hidden)",r=$(t.currentTarget).parent(),l=r.children("a"),c=e.tablist.children("li".concat(o)),d=H.isRTL();function h(){for(var t=c.index(r)-1;t>-1&&!a;){if(c.eq(t).is(o))return c.eq(t);t-=1}if(e.hasMoreActions())return e.moreActionsBtn;if(e.settings.addTabButton)return e.addTabButton;var i=e.tablist.children("li".concat(o)).last();return e.hasMoreButton()&&e.isScrollableTabs()&&s(i.find("a").attr("href")),i}function u(){for(var t=c.index(r)+1;t<c.length&&!a;){if(c.eq(t).is(o))return c.eq(t);t++}var i=e.tablist.children("li".concat(o)).first();return e.hasMoreButton()&&e.isScrollableTabs()?(s(i.find("a").attr("href")),i):e.settings.addTabButton?e.addTabButton:e.hasMoreActions()?e.moreActionsBtn:i}function p(){if(r.hasClass("has-popupmenu"))r.data("popupmenu").open();else{var t=l.attr("href");r.is(".add-tab-button")&&(l=e.handleAddButton(),r=l.parent(),t=l.attr("href"),e.element.trigger("tab-added",[l])),e.activate(t)&&(e.changeHash(t),(l.attr("ng-click")||l.attr("data-ng-click"))&&l.click(),l[0].focus(),e.hideFocusState(),e.handleOutboundLink(t))}}switch(t.which){case 8:return t.altKey&&r.is(".dismissible")&&(t.preventDefault(),e.closeDismissibleTab(l.attr("href"))),!0;case 13:case 32:return p(),this.positionFocusState(),!1;case 38:a=h(),t.preventDefault();break;case 37:a=d?u():h(),t.preventDefault();break;case 40:a=u(),t.preventDefault();break;case 39:a=d?h():u(),t.preventDefault()}if(a){var g=a.is(".add-tab-button"),f=a.is(".btn-actions"),m=g||f?a:a.children("a");if(this.isTabOverflowed(a))return s(a.children("a").attr("href"));g?e.addTabButton.focus():f?e.moreActionsBtn.focus():(m.focus(),this.isScrollableTabs()&&(this.scrollTabList(m),e.positionFocusState(m,!0)))}return!0},handleDismissibleTabKeydown:function(t){var e=t.which,i=$(t.target);if(i.is("a")&&(i=i.parent()),t.altKey&&46===e){if(i.children("a").is("[disabled]")||i.hasClass("is-disabled"))return;t.preventDefault(),this.closeDismissibleTab(i.children("a").attr("href"))}},handleAppMenuTabKeydown:function(t){var e=$(t.target);if(!e.parent().is(".application-menu-trigger")&&!e.is(".application-menu-trigger"))return!0;var i=$("#application-menu");return!!i.length&&(t.preventDefault(),i.hasClass("is-open")?(i.trigger("close-applicationmenu",[!0]),!1):(i.trigger("open-applicationmenu",[!0,!0]),!1))},handleMoreButtonKeydown:function(t){if(this.element.is(".is-disabled"))return t.preventDefault(),!1;var e=this,i=H.isRTL();function n(){t.preventDefault(),e.buildPopupMenu(e.tablist.find(".is-selected").children("a").attr("href")),e.positionFocusState(e.moreButton,!0)}function s(){t.preventDefault(),e.findLastVisibleTab()}switch(t.which){case 37:if(i){n();break}s();break;case 38:s();break;case 13:case 32:t.preventDefault();break;case 39:if(i){s();break}n();break;case 40:n()}return!0},handlePanelKeydown:function(t){var e=t.which,i=$(t.target),n=this.anchors.filter("#".concat(i.attr("id")));return this.anchors.filter("#".concat(i.attr("id"))).parent().is(".dismissible")&&t.altKey&&46===e?(t.preventDefault(),this.closeDismissibleTab(n.attr("href"))):t.ctrlKey&&38===e&&$.contains(document.activeElement,i[0])?(t.preventDefault(),this.activate(n.attr("href"),n)):void 0},triggerEventAfterTabAdded:function(t){var e=this.anchors.filter('[href="#'.concat(t,'"]'));this.element.triggerHandler("aftertabadded",[e])},handleAddButton:function(){var t=this,e=this.settings.addTabButtonCallback;if(e&&"function"==typeof e){var i=e();return this.triggerEventAfterTabAdded("newTabId"),this.anchors.filter('[href="#'.concat(i,'"]'))}var n,s=this.tablist.children().index(this.addTabButton),a=(t.idCounter="number"==typeof t.idCounter?t.idCounter:-1,t.idCounter++,"new-tab-".concat(t.idCounter)),o={name:((n=a.toString().split("-")).forEach((function(t,e){n[e]=t.charAt(0).toUpperCase()+t.slice(1)})),n.join(" ")),content:"&nbsp;",isDismissible:!0};s<0&&(s=this.tablist.find("li:not(.separator)").length);var r=this.element.triggerHandler("before-tab-added",[a,o,s]);return r?(r.newId&&r.newId.length&&"string"==typeof r.newId&&(a=r.newId),r.settings&&"object"===l(r.settings)&&(o=r.settings),isNaN(r.newIndex)||(s=r.newIndex),this.add(a,o,s),this.triggerEventAfterTabAdded("newId"),this.anchors.filter('[href="#'.concat(a,'"]'))):(this.add(a,o,s),this.triggerEventAfterTabAdded("newId"),this.anchors.filter('[href="#'.concat(a,'"]')))},handleAddButtonKeydown:function(t){if(this.element.is(".is-disabled"))return t.preventDefault(),!1;var e=t.which;if([13,32,37,38,39,40].indexOf(e)<0)return!1;var i,n=this,s=H.isRTL(),a="li:not(.separator):not(.is-disabled):not(:hidden)";function o(){t.preventDefault(),i=n.tablist.find(a).last(),setTimeout((function(){n.buildPopupMenu(i.children("a").attr("href")),n.positionFocusState(n.moreButton,!0)}),0)}function r(){n.hasMoreActions()?n.moreActionsBtn.focus():i=n.tablist.find(a).first()}switch(e){case 37:if(s){r();break}o();break;case 38:o();break;case 13:case 32:return t.preventDefault(),this.handleAddButton();case 39:if(s){o();break}r();break;case 40:r()}return i&&(i.children("a").focus(),this.isScrollableTabs&&(t.preventDefault(),this.scrollTabList(i))),!0},handleAddButtonFocus:function(){this.tablist.find("li:not(.separator)").add(this.moreButton).removeClass("is-focused"),this.addTabButton.addClass("is-focused"),this.positionFocusState(this.addTabButton,!0)},handleResize:function(t){t||this.checkResponsive(!0),this.setOverflow();var e=this.tablist.find(".is-selected");(!e.length||this.moreButton.is(".is-selected")||this.isTabOverflowed(e))&&(e=this.moreButton);var i=this.element.find(".application-menu-trigger");i.length>0&&i.hasClass("is-focused")&&(e=i.find("a")),e.length?this.positionFocusState(e):this.hideFocusState(),this.handleVerticalTabResize(),this.renderVisiblePanel(),this.renderEdgeFading()},checkResponsive:function(t){var e=this,i=e.element[0].classList;function n(){e.removeHelperMarkupEvents(),e.renderHelperMarkup(),e.setupHelperMarkupEvents()}function s(){i.contains("is-in-responsive-mode")&&(i.add("vertical"),i.remove("is-in-responsive-mode"),i.remove("header-tabs"),i.remove("alternate"),t&&n())}this.isResponsiveVerticalTabs()&&m.isBelow("phone-to-tablet")?i.contains("is-in-responsive-mode")||(i.add("is-in-responsive-mode"),i.add("header-tabs"),i.contains("is-personalizable")||i.add("alternate"),i.remove("vertical"),t&&n()):s()},handleVerticalTabResize:function(){if(this.isVerticalTabs()&&this.isNested()&&!this.isNestedInLayoutTabs()&&!this.isHidden()){var t=window.getComputedStyle(this.element[0]),e=t.getPropertyValue("height")+t.getPropertyValue("margin-top")+t.getPropertyValue("margin-bottom");this.tablist[0].style.height=e}},handleOutboundLink:function(t,e){"#"!==t.charAt(0)&&("/"!==t.charAt(0)||e&&!1!==e||(t=window.location.origin+t),window.location=t)},hasMoreButton:function(){return this.element[0].classList.contains("has-more-button")},isInResponsiveMode:function(){return this.element[0].classList.contains("is-in-responsive-mode")},isModuleTabs:function(){return this.element.hasClass("module-tabs")},isVerticalTabs:function(){return this.element.hasClass("vertical")},isResponsiveVerticalTabs:function(){return!0===this.settings.verticalResponsive},isHeaderTabs:function(){return this.element.hasClass("header-tabs")},isScrollableTabs:function(){return!this.isModuleTabs()&&!this.isVerticalTabs()},isHidden:function(){return this.element.is(":hidden")},isNested:function(){return this.element.closest(".tab-panel").length},isActive:function(t){return!(!t||!t.length||1===t.length&&t.indexOf("#")<1)&&this.getPanel(t)[0].classList.contains("can-show")},isNestedInLayoutTabs:function(){var t=this.element.closest(".module-tabs").length,e=this.element.closest(".header-tabs").length,i=this.element.closest(".tab-panel-container").length;return t>0||e>0||i>0},isTab:function(t){return t instanceof jQuery&&t.length&&t.is("li.tab")},isAnchor:function(t){return t instanceof jQuery&&t.length&&t.is("a")},getAnchor:function(t){return this.isAnchor(t)?t:(-1===t.indexOf("#")&&"/"!==t.charAt(0)&&(t="#".concat(t)),this.anchors.filter('[href="'.concat(t,'"]')))},getPanel:function(t){return this.isTab(t)&&(t=t.children("a")),this.isAnchor(t)&&(t=t.attr("href")),t&&""!==t&&"#"!==t?("#/"===t.substr(0,2)&&(t=t.replace("#/","#")),this.panels.filter('[id="'.concat(t.replace(/#/g,""),'"]'))):$()},getMenuItem:function(t){return this.isAnchor(t)&&(t=t.attr("href")),-1===t.indexOf("#")&&(t="#".concat(t)),this.moreMenu.children().children().filter('[data-href="'.concat(t,'"]')).parent()},getPreviousTab:function(t){for(var e=this.getTab(null,t),i="li:not(.separator):not(:hidden):not(.is-disabled)",n=this.tablist.find(i),s=n.index(e),a=n.eq(0===s?1:s-1);a.length&&!a.is(i);)a=n.eq(n.index(a)-1);if(a.is(".has-popupmenu")){var o=a.data("popupmenu");o&&(a=o.menu.children("li").first())}return a},activateAdjacentTab:function(t,e){var i=this.doGetTab(t,e);"string"==typeof t&&(e=t),i.is(".is-selected")?i.prevAll("li.tab").not(".hidden").not(".is-disabled").length>0?this.select($(i.prevAll("li.tab").not(".hidden").not(".is-disabled")[0]).find("a")[0].hash,!0):i.nextAll("li.tab").not(".hidden").length>0&&this.select($(i.nextAll("li.tab").not(".hidden").not(".is-disabled")[0]).find("a")[0].hash,!0):this.select($(this.element.find("li.tab.is-selected")[0]).find("a")[0].hash,!0)},activatePreviousTab:function(t,e){var i=this.getTab(null,t);if(e&&e instanceof jQuery||(e=this.getPreviousTab(t)),!e.length)return this.hideFocusState(),e;var n=e.children("a");if(i.is(".is-selected")){if(!this.activate(n.attr("href"),n))return e;n.focus()}return this.positionFocusState(n),e},isURL:function(t){return!(!t||0===t.indexOf("#"))},activate:function(t,e){var i=this;if(i.isURL(t))return this.callSource(t,e,!0);for(var n,s,a,o=i.getAnchor(t),r=o.parent(),l=i.getPanel(t),c=l[0],d=i.anchors.parents().filter(".is-selected"),h=0;h<i.panels.length;h++)i.panels[h].classList.contains("is-visible")&&(n=$(i.panels[h]));if(n||(n=i.panels),!i.element.trigger("beforeactivated",[o]))return!1;function u(e){if(l.length<1){if(i.settings.source)return i.callSource(t,o),!0}else{n[0].classList.remove("can-show"),n[0].classList.remove("is-visible"),n.closeChildren(),i.element.trigger("activated",[o]),c.classList.add("can-show"),i.renderVisiblePanel(),c.offsetHeight;var h=new X({duration:L.convertDelayToFPS(150),timeoutCallback:function(){i.element.trigger("afteractivated",[o])}});J.register(h),c.classList.add("is-visible")}i.updateAria(o),d.add(i.moreButton).removeClass("is-selected"),r[0]&&r[0].classList.contains("tab")&&(s=r,a=r);var u,p=r.parents(".popupmenu");p.length&&(u=p.data("trigger")).length&&(s=u,a=u),i.isTabOverflowed(a)&&(a=i.moreButton,s=i.moreButton),s&&s.addClass("is-selected");var g=l.find(".toolbar");return g.length&&g.each((function(){var t=$(this).data("toolbar");t&&"function"==typeof t.handleResize&&t.handleResize()})),E.fixSVGIcons(l[0]),e||!1}return this.settings.beforeActivate&&"function"==typeof this.settings.beforeActivate?this.settings.beforeActivate(d,r,u):u(!0)},renderEdgeFading:function(){if(this.isScrollableTabs()&&this.tablistContainer){var t=this.tablistContainer[0],e=Math.abs(t.scrollLeft),i=t.scrollWidth,n=parseInt(window.getComputedStyle(t).getPropertyValue("width"),10);this.element[0].classList[e>1?"add":"remove"]("scrolled-right"),this.element[0].classList[i-e-1<=n?"remove":"add"]("scrolled-left")}},callSource:function(t,e,i){var n=this;if((null==i||!1===i)&&!this.settings.source)return!1;if(!0!==this.settings.lazyLoad)return!1;var s=l(this.settings.source),a=function(s){if(null!=s){if(s=g.sanitizeHTML(s),i){var a=n.element[0].id||"",o=E.uniqueId(e,"tab",a);t="#".concat(o),e[0].setAttribute("href",t)}n.createTabPanel(t,s,!0),n.activate(t),n.element.triggerHandler("complete"),n.element.trigger("requestend",[t,s])}};function o(t,e){var i=$.extend({dataType:"html"},e,{url:t}),n=$.ajax(i);return n.done(a),n}if(this.container.triggerHandler("start"),this.container.trigger("requeststart"),i)return o(t,this.ajaxOptions);if("function"===s)return this.settings.source(a,t,this.settings.sourceArguments);if("string"===s){var r=t.replace(/#/g,""),c=this.settings.source.toString();if(!(c.indexOf(r)>-1)){var d="tab=".concat(r),h=c.indexOf("?"),u=c.indexOf("#"),p=c.length;h<0?(d="?".concat(d),u>-1&&(p=u+1)):(d+="&",p=h+1),c=M.splice(c,p,0,d)}return o(c,this.ajaxOptions)}return!1},renderVisiblePanel:function(){this.resizeNestedTabs()},changeHash:function(t){if(this.settings.changeTabOnHashChange){t||(t=""),t.replace(/#/g,"");var e=this.settings.hashChangeCallback;e&&"function"==typeof e?e(t):window.location.hash=t,this.element.triggerHandler("hash-change",[t])}},updateAria:function(t){t&&(this.anchors.attr({"aria-selected":"false",tabindex:"-1"}),this.moreButton.attr({tabindex:"-1"}),t.length&&this.element.is(":not(.is-disabled)")&&(t.parent().removeClass("is-selected"),this.isTabOverflowed(t.parent())?this.moreButton.attr({tabindex:"0"}):t.attr({"aria-selected":"true",tabindex:"0"}).parent().addClass("is-selected")))},resizeNestedTabs:function(){this.nestedTabControls.each((function(t,e){var i=$(e).data("tabs");i&&i.handleResize&&"function"==typeof i.handleResize&&i.handleResize(!0)}))},add:function(t,e,i){if(!t)return this;e||(e={});var n=0===this.tablist.find("li").not(".application-menu-trigger, .add-tab-button").length;function s(t){if("string"===l(t))if(null!==t.match(/^#/g)){var e=$(t);t=e.length?e:t}return t}t="".concat(t.replace(/#/g,"")),e.name=e.name?e.name.toString():"&nbsp;",e.isDismissible=!!e.isDismissible&&!0===e.isDismissible,e.isDropdown=!!e.isDropdown&&!0===e.isDropdown,e.content&&(e.content=s(e.content)),e.dropdown&&(e.dropdown=s(e.dropdown));var a,o=$('<li class="tab" role="presentation"></li>'),r=$('<a id="#'.concat(t,'" href="#').concat(t,'" role="tab" aria-expanded="false" aria-selected="false" tabindex="-1">').concat(g.escapeHTML(e.name),"</a>")),c=this.createTabPanel(t,e.content);o.html(r),e.isDismissible&&(a=$.createIconElement({icon:"close",classes:"close icon"}),E.addAttributes(a,this,e.attributes,"close-btn"),o.addClass("dismissible"),o.append(a)),this.settings.tabCounts&&r.prepend('<span class="count">0 </span>'),e.dropdown;var d=this.settings.attributes||[];if(Array.isArray(e.attributes)&&(d=d.concat(e.attributes)),d.length&&(E.addAttributes(r,this,d,"".concat(t,"-a")),E.addAttributes(c,this,d,"".concat(t,"-panel"))),function(t,e){var i,n=t.tablist.children("li"),s=n.not(".application-menu-trigger, .add-tab-button"),a=n.length-1;if(!n.length)return o.appendTo(t.tablist),void c.appendTo(t.container);var r=n.filter(".add-tab-button"),l=n.filter(".application-menu-trigger");(null==e||isNaN(e))&&(e=n.length),n.length>s.length&&(e>n.length-1&&r&&r.length&&(e-=1),function(t){return t<=0}(e)&&l&&l.length&&(e+=1));var d=n.eq(e).length>0;a=d?e:a,i="insertAfter",d&&(i="insertBefore"),o[i](n.eq(a)),c.appendTo(t.container)}(this,i),this.panels=$(this.panels.add(c)),this.anchors=$(this.anchors.add(r)),r.data("panel-link",c),c.data("tab-link",r),(this.settings.moduleTabsTooltips||this.settings.multiTabsTooltips)&&r.on("beforeshow.toolbar",(function(){return"yes"===r.data("cutoffTitle")})).tooltip({content:"".concat(r.text().trim())}),this.isModuleTabs()&&this.adjustModuleTabs(),this.setOverflow(),n){if(this.positionFocusState(r),!this.activate(r.attr("href")))return this.triggerEventAfterTabAdded(t),this;r.focus()}return e.doActivate&&this.activate(r.attr("href")),this.triggerEventAfterTabAdded(t),this},remove:function(t,e){var i,n,s=this,a=this.doGetTab(null,t);if(!a||!a.length)return!1;var o=a.children("a"),r=this.getPanel(t),l=r&&r.length,c=this.tablist.children("li").index(a),d=".application-menu-trigger, .separator, .is-disabled",h=a.prev();if(!e&&!1===this.element.triggerHandler("beforeclose",[o]))return!1;var u=!1;null!==(i=o[0])&&void 0!==i&&(null==i?void 0:i.href)&&null!==(n=s.getActiveTab()[0])&&void 0!==n&&(null==n?void 0:n.href)&&o[0].href===s.getActiveTab()[0].href?u=!0:h=$(this.tablist.children("li").not(d).filter(".is-selected")),l&&(this.panels=$(this.panels.not(r))),this.anchors=$(this.anchors.not(o)),(this.settings.moduleTabsTooltips||this.settings.multiTabsTooltips)&&o.off("beforeshow.toolbar").data("tooltip").destroy();var p=a.data("popupmenu");a.hasClass("has-popupmenu")&&p&&(p.menu.children("li").each((function(){s.remove($(this).children("a").attr("href"))})),p.destroy());var g=a.closest(".dropdown-tab"),f=g.data("trigger");a.find(".icon").off().removeData().remove(),a.off(),o.off(),a.removeData().empty(),a.remove(),l&&(r.removeData().empty(),r.removeData(),r.remove());var m,v=o.data("moremenu-link");v&&(v.parent().off().removeData().remove(),o.removeData()),(m=g).length&&0===m.children("li:not(.separator)").length&&(h=this.getPreviousTab(f),setTimeout((function(){s.remove(f)}),1)),f&&f.length&&f.data("popupmenu").close(),this.isModuleTabs()&&this.adjustModuleTabs(),this.setOverflow(),this.element.trigger("close",[a]);for(var b=c-1;b>-1;)b=-1,h.is(d)&&(h=this.tablist.children("li").not(d)[0],b-=1);if(h&&h.length||(h=this.tablist.children("li").not(d).first()),!h.length)return this.hideFocusState(),this.element.trigger("afterclose",[a]),this;var y=h.children("a"),w=y;return this.positionFocusState(y),u&&(h.is(".has-popupmenu")&&h.data("popupmenu")&&(w=h.data("popupmenu").menu.children().first().children("a")),!this.activate(w.attr("href")))||(y.focus(),this.element.trigger("afterclose",[a])),this},createTabPanel:function(t,e,i){t=t.replace(/#/g,"");var n=$('<div id="'.concat(g.stripTags(t),'" class="tab-panel" role="tabpanel"></div>'));return e instanceof $&&(e=e[0].outerHTML),f.html(n[0],e||"","*"),!0===i&&this.container.append(n),this.panels=$(this.panels.add(n)),n},checkPopupMenuItems:function(t){function e(t){if(!t||!t.menu)return $();var e=t.menu.children("li");return e.length?e:(t.destroy(),$())}if(t.is(".has-popupmenu"))return e(t.data("popupmenu"));var i=t.closest(".dropdown-tab");return i.length?e(i.data("popupmenu")):$()},getTab:function(t,e){var i,n=this,s=$();return e instanceof $&&e.length>0?e.is("a")?e.parent():e:t?(i=$(t.currentTarget)).is(".tab")?i:i.closest(".tab").length?i.closest(".tab").first():null:e?function(t){if(!t||""===t||"#"===t)return null;-1===t.indexOf("#")&&(t="#".concat(t));var e=n.anchors.filter('[href="'.concat(t,'"]'));return e.length?e.parent():null}(e):s},doGetTab:function(t,e){return(t||e)&&(!t||t instanceof $.Event||"string"==typeof t)?t?"string"!=typeof t?this.getTab(t):this.getTab(null,t):this.getTab(null,e):$()},hide:function(t,e){var i=this.doGetTab(t,e);this.activateAdjacentTab(t,e),i.addClass("hidden");var n=$(this.element.find("li.tab.is-selected")[0]).find("a");return n[0]&&this.select(n[0].hash),this.positionFocusState(),this},show:function(t,e){this.doGetTab(t,e).removeClass("hidden");var i=$(this.element.find("li.tab.is-selected")[0]).find("a");return i[0]&&this.select(i[0].hash),this.positionFocusState(),this},disableTab:function(t,e){var i=this.doGetTab(t,e);return this.activateAdjacentTab(t,e),i.addClass("is-disabled"),this.positionFocusState(),this},enableTab:function(t,e){return this.doGetTab(t,e).removeClass("is-disabled"),this.positionFocusState(),this},rename:function(t,e,i){if(t&&"string"==typeof t&&(i=e,e=t),i){i=i.toString();var n,s=this.doGetTab(t,e),a=this.settings.tabCounts,o=this.settings.moduleTabsTooltips||this.settings.multiTabsTooltips,r=s.hasClass("dismissible"),l=s.children("a");if(a&&(n=l.find(".count").clone()),l.text(i),this.popupmenu){var c="#"!==e.charAt(0)?"#".concat(e):e,d=this.popupmenu.menu.find('a[href="'.concat(c,'"]'));d.text(i),r&&d.append($.createIconElement({icon:"close",classes:"icon close"}))}a&&l.prepend(n),o&&l.data("tooltip").setContent(i.trim());var h=this.tablist.children("li").length<2?s:void 0;this.positionFocusState(h)}},updateCount:function(t,e,i){if(t&&"string"==typeof t&&(i=e,e=t),this.settings.tabCounts&&i){var n=this.doGetTab(t,e);n.children("a").find(".count").text("".concat(i.toString()," "));var s=this.tablist.children("li").length<2?n:void 0;this.positionFocusState(s)}},getActiveTab:function(){var t=this.panels.filter(":visible");return this.anchors.filter('[href="#'.concat(t.first().attr("id"),'"]'))},getVisibleTabs:function(){var t=this,e=$();return this.tablist.find("li:not(.separator):not(.hidden):not(.is-disabled):not(.application-menu-trigger)").each((function(){var i=$(this);t.isTabOverflowed(i)||(e=e.add(i))})),e},getOverflowTabs:function(){var t=this,e=$();return this.tablist.find("li:not(.separator):not(.hidden):not(.is-disabled):not(.application-menu-trigger)").each((function(){var i=$(this);t.isTabOverflowed(i)&&(e=e.add(i))})),e},setOverflow:function(){var t,e,i,n=this,s=this.element[0],a=this.tablist[0],o="has-more-button",r=this.hasMoreButton(),l=this.isScrollableTabs();function c(){n.isModuleTabs()&&n.adjustModuleTabs()}c(),l?i=this.tablistContainer[0].scrollWidth>this.tablistContainer[0].offsetWidth:(t=window.getComputedStyle(a,null),e=parseInt(t.getPropertyValue("height"),10)+1,i=a.scrollHeight>e),i?r||(s.classList.add(o),c()):r&&(s.classList.remove(o),c()),this.adjustSpilloverNumber()},adjustModuleTabs:function(){var t=this,e=this.tablist.find("li:not(.separator):not(.application-menu-trigger):not(:hidden)"),i=this.tablist.find(".application-menu-trigger"),n=i.length>0,s=this.tablist.outerWidth(),a=120,o=n?i.outerWidth():0;if(e.children("a").removeAttr("style"),e.removeAttr("style"),this.setMaxWidth(),e.each((function(){var i=$(this);t.isTabOverflowed(i)&&(e=e.not(i))})),!e.length)return a=s-o+101,void(this.moreButton[0].style.width="".concat(a,"px"));var r,l,c=window.getComputedStyle(e.eq(0).children()[0]),d=parseInt(c.paddingLeft,10)+parseInt(c.paddingRight,10);this.moreButton[0].hasAttribute("style")&&this.moreButton[0].removeAttribute("style"),(a=(s-o)/e.length)<120&&(a=120);for(var h="no",u=1===this.element.closest(".side-by-side").length,p=0;p<e.length;p++){(r=e.eq(p).children("a"))[0].style.width="",(!0===this.settings.moduleTabsTooltips||this.settings.multiTabsTooltips)&&(h="no",(l=parseInt(window.getComputedStyle(e[p]).width,10))>a-d&&(h="yes"),null!==this.settings.maxWidth&&r.width()!==l&&(h="yes"),r.data("cutoffTitle",h));var g=0;"ios"===x.os.name&&x.devicespecs.isMobile&&u&&(g=25),e[p].style.width="".concat(a-g,"px"),r[0].style.width="".concat(a-g,"px")}this.adjustSpilloverNumber()},setMaxWidth:function(){if(null!==this.settings.maxWidth&&void 0!==this.settings.maxWidth)for(var t=this.tablist.find("li:not(.separator):not(.application-menu-trigger):not(:hidden)"),e=0;e<t.length;e++){var i=t.eq(e).children("a");t[e].style.width="".concat(this.settings.maxWidth,"px"),i[0].style.width="".concat(this.settings.maxWidth,"px")}},adjustSpilloverNumber:function(){var t=this.moreButton.find(".more-text"),e=this.tablist.find("li:not(.separator):not(.hidden):not(.is-disabled):not(.application-menu-trigger)"),i=this.getOverflowTabs();e.length<=i.length?t.text("".concat(H.translate("Tabs"))):t.text("".concat(H.translate("More")));var n=this.moreButton.find(".count");n.length||(n=$('<span class="count"></span>'),this.moreButton.children("span").first().prepend(n)),n.text("".concat(i.length," "))},select:function(t,e){var i=t.replace(/#/g,""),n=this.getAnchor(i);this.positionFocusState(void 0,!1),this.activate(n.attr("href"))&&(this.changeHash(i),e||n.focus())},buildPopupMenu:function(t){var e=this;e.popupmenu&&(e.popupmenu.destroy(),$("#tab-container-popupmenu").off("focus.popupmenu").removeData().remove(),$(document).off("keydown.popupmenu"));var i=$("#tab-container-popupmenu"),n="";this.isScrollableTabs()&&(n=" is-selectable"),0===i.length?i=$('<ul id="tab-container-popupmenu" class="tab-list-spillover'.concat(n,'">')).appendTo("body"):i.html("");var s=e.tablist.children("li"),a=H.isRTL();function o(t){var n=$(t),s=n.children("a");if(!n.is(":hidden")&&(e.isScrollableTabs()||e.isTabOverflowed(n)))if(n.is(".separator"))n.clone().appendTo(i);else{var a=n.clone(),o=a.children("a");if(a[0].classList.remove("tab"),a[0].classList.contains("is-selected")&&(a[0].classList.remove("is-selected"),e.isScrollableTabs()&&a[0].classList.add("is-checked")),a[0].removeAttribute("style"),a.children(".icon").off().appendTo(o),a.appendTo(i),n.data("moremenu-link",o),o.find(".icon-more").removeData().remove(),o.data("original-tab",s),o.onclick=void 0,n.is(".has-popupmenu")){var r=$("#".concat(t.getAttribute("aria-controls"))),l=r.clone(),c=l.children("li");l[0].classList.remove("has-popupmenu"),c.each((function(t){var e=$(this).children("a"),i=r.children("li").eq(t).children("a");e.data("original-tab",i),i.data("moremenu-link",e)})),l.insertAfter(o)}}}for(var r=0;r<s.length;r++)o(s[r]);e.tablist.children("li:not(.separator)").removeClass("is-focused");var l=1;this.isScrollableTabs()||(l=3);var c=e.settings.attributes;e.moreButton.popupmenu({autoFocus:!1,attachToBody:!0,menu:"tab-container-popupmenu",trigger:"immediate",offset:{x:l},attributes:c}),e.moreButton.addClass("popup-is-open"),e.popupmenu=e.moreButton.data("popupmenu"),e.positionFocusState(e.moreButton),e.moreButton.on("close.tabs",(function(){$(this).off("close.tabs selected.tabs"),e.moreButton.removeClass("popup-is-open"),e.positionFocusState(void 0)})).on("selected.tabs",(function(t,i){var n=i.attr("href"),s=n.substr(1,n.length),a=e.doGetTab(s)||$(),o=a?a.children("a"):$(),r=i.data("original-tab").parent();r.is(".add-tab-button")&&(r=(o=e.handleAddButton()).parent(),n=o.attr("href"),e.element.trigger("tab-added",[o])),e.activate(n),a.length&&o.length&&"function"==typeof o[0].onclick&&o[0].onclick.apply(o[0]),e.moreButton.focus(),e.scrollTabList(a)}));var d=e.popupmenu.menu;function h(t){var i=$(this),n=i.closest("li");if(n.is(".dismissible")&&i.is(".close")){if(t.preventDefault(),t.stopPropagation(),n.is(".dismissible")&&n.is(".has-popupmenu")&&n.is(".submenu")){var s=n.find(".wrapper").children().children(),a=[];$.each(s,(function(t,e){a.push(e.children[0].href)})),e.closeDismissibleTabs(a)}else e.closeDismissibleTab(n.children("a").attr("href"));e.popupmenu.close()}}d.on("destroy.popupmenu",(function(){d.off(),e.hideFocusState(),$("#tab-container-popupmenu").removeData().remove()})).on("touchend.popupmenu touchcancel.popupmenu",".icon",h).on("click.popupmenu",".icon",h),t?e.popupmenu.highlight(d.find('a[href="'.concat(t,'"]'))):e.tablist.children(".is-selected").length>0?e.popupmenu.highlight(d.find('a[href="'.concat(e.tablist.children(".is-selected").children("a").attr("href"),'"]'))):e.popupmenu.highlight(d.find("li:first-child > a")),$(document).bindFirst("keydown.popupmenu",(function(t){var i,n=t.which,s=$(t.target);function o(){return this===document.activeElement}switch(n){case 8:case 46:t.altKey&&s.parent().is(".dismissible")&&e.closeDismissibleTab(s.attr("href"));break;case 37:i=a?40:38,s.is("a")&&(s.parent().is(":not(:first-child)")&&t.preventDefault(),$(document).trigger({type:"keydown.popupmenu",which:i}));break;case 38:d.find("li:first-child > a").filter(o).length>0&&(t.preventDefault(),$(document).off(t),e.popupmenu.close(),e.findLastVisibleTab());break;case 39:i=a?38:40,s.is("a")&&!s.parent(".submenu").length&&$(document).trigger({type:"keydown.popupmenu",which:i});break;case 40:!function(){var i=d.find("li:last-child > a");if(i.filter(o).length>0&&i.parent().is(":not(.submenu)")){if(t.preventDefault(),$(document).off(t),e.popupmenu.close(),e.settings.addTabButton)return void e.addTabButton.focus();if(e.hasMoreActions())return void e.moreActionsBtn.focus();e.focusFirstVisibleTab()}}()}}))},isTabOverflowed:function(t){if(this.isVerticalTabs()||this.isScrollableTabs()||this.tablist.find(".arrange-dragging").length)return!1;if(!t)return!1;this.tablist.scrollTop()>0&&this.tablist.scrollTop(0);var e=Math.round(t[0].getBoundingClientRect().top),i=Math.round(this.tablist[0].getBoundingClientRect().top+1);return this.isModuleTabs()&&(i+=1),e>i},findLastVisibleTab:function(){var t=this.tablist.children("li:not(.separator):not(.hidden):not(.is-disabled)"),e=t.first();if(this.isScrollableTabs())return t.last().find("a").focus();for(;!this.isTabOverflowed(e);)e=t.eq(t.index(e)+1);return t.eq(t.index(e)-1).find("a").focus()},focusFirstVisibleTab:function(){this.tablist.children("li:not(.separator):not(.hidden):not(.is-disabled)").eq(0).find("a").focus()},scrollTabList:function(t){if(this.tablistContainer&&t&&t instanceof $&&t.length){var e,i=f.getDimensions(t[0]),n=f.getDimensions(this.tablistContainer[0]),s=i.left,a=i.right;s<n.left+40&&(e=-1*Math.round(Math.abs(n.left-s))-40),a>n.right-40&&(e=Math.round(Math.abs(a-n.right))+40),0===e&&(e=void 0),this.tablistContainer.smoothScroll(e,250)}},hideFocusState:function(){var t;null===(t=this.focusState)||void 0===t||null==t||t.removeClass("is-visible")},positionFocusState:function(t,e){var i=this;if(void 0!==t?t=$(t):i.moreButton.hasClass("is-selected")?t=i.moreButton:i.tablist.children(".is-selected").length>0&&(t=i.tablist.children(".is-selected").children("a")),!t||void 0===t||!t.length||t.is(this.moreButton)&&this.isScrollableTabs()){var n;null===(n=this.focusState)||void 0===n||null==n||n.removeClass("is-visible")}else{t.is("a")&&(t=t.parent()),t.is(".add-tab-button, .tab-more")?this.focusState.parent().is(this.element)||this.focusState.prependTo(this.element):this.focusState.parent().is(this.tablistContainer)||this.focusState.prependTo(this.tablistContainer);var s,a,o,r,l,c,d,h=this.focusState[0],u=f.getDimensions(t[0]),p=t[0].classList,g=!this.isHeaderTabs()||this.isHeaderTabs()&&this.element[0].classList.contains("alternate"),m=H.isRTL(),v=this.isVerticalTabs()?0:this.moreButton.outerWidth(!0)-8,b=this.element,y=this.tablistContainer,w=y&&this.focusState.parent().is(y),x=t[0].getBoundingClientRect().width/t[0].offsetWidth*100,C=$('html[class*="classic-"]').length>0,k=b.hasClass("alternate"),T=b.hasClass("header-tabs"),A=t.is(".add-tab-button");s=u,o=y,r=x,d=(a=b)[0].getBoundingClientRect(),s.top-=d.top-2,i.isVerticalTabs()&&(s.top+=a[0].scrollTop),m?s.right=d.right-s.right:s.left-=d.left,o&&(c=o?Math.abs(o[0].scrollLeft):0,s.left+=c,s.right+=c,m&&(s.right-=1,g||(s.left-=v,s.right-=v)),r<99&&(s.width=100*s.width/r,s.left=100*s.left/r,s.right=100*s.right/r),w&&(l=parseInt(window.getComputedStyle(a[0])["padding".concat(m?"Right":"Left")],10),s.left+=m?l:-1*l,s.right+=m?l:-1*l),s.right<0&&(s.right=0)),!g||i.isVerticalTabs()||i.isModuleTabs()||(s.height-=1,s.top+=2),i.isModuleTabs()&&!i.isVerticalTabs()?(s.top-=3,s.width-=4,s.left+=2,C||(s.height-=2)):s.height-=4,C||k||!T||A||(s.top-=9,s.height+=2),C&&k&&T&&(s.height-=4),u=s;var S="";Object.keys(u).forEach((function(t){S.length&&(S+=" "),S+="".concat(t,": ").concat(u[t],"px;")})),h.style.cssText=S;var E=p.contains("is-selected")?"add":"remove";h.classList[E]("is-selected");var D=!0===e?"add":"remove";h.classList[D]("is-visible");var I=this.element.find(".application-menu-trigger, .tab");I.length>0&&I.hasClass("is-focused")&&h.classList.add("is-visible")}},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown(),this.init();var e=this.tablist.data("arrange");return e&&(null==e||null==e||e.updated()),this},disableOtherTabs:function(){return this.disable(!0)},disable:function(t){t||this.element.prop("disabled",!0).addClass("is-disabled"),this.disabledElems||(this.disabledElems=[]);var e=this,i=this.tablist.children("li:not(.separator)");t&&(i=i.filter(":not(.application-menu-trigger)")),i.each((function(){var i=$(this),n=i.children("a");if(!(t&&e.isActive(n.attr("href"))||((i.is(".is-disabled")||!0===n.prop("disabled"))&&e.disabledElems.push({elem:i,originalTabindex:i.attr("tabindex"),originalDisabled:n.prop("disabled")}),i.addClass("is-disabled"),n.prop("disabled",!0),i.is(".application-menu-trigger")||i.is(".add-tab-button")))){var s=$(n.attr("href"));s.addClass("is-disabled"),s.find("*").each((function(){var t=$(this);t.is("input.dropdown, input.multiselect")||(("-1"===t.attr("tabindex")||t.attr("disabled"))&&e.disabledElems.push({elem:t,originalTabindex:t.attr("tabindex"),originalDisabled:t.prop("disabled")}),null==t||null==t||t.disable())}))}})),this.moreButton.addClass("is-disabled"),this.isModuleTabs()&&!t&&this.element.children(".toolbar").disable(),this.updateAria($())},enable:function(){this.element.prop("disabled",!1).removeClass("is-disabled");var t=this;this.tablist.children("li:not(.separator)").each((function(){var e=$(this),i=e.children("a");if(e.removeClass("is-disabled"),i.prop("disabled",!1),!e.is(".application-menu-trigger")&&!e.is(".add-tab-button")){var n=$(i.attr("href"));n.removeClass("is-disabled"),n.find("*").each((function(){var t=$(this);t.enable&&"function"==typeof t.enable&&t.enable()})),$.each(t.disabledElems,(function(t,e){var i=e.elem.is(".tab")?e.elem.children("a"):e.elem;e.elem.disable&&"function"==typeof e.elem.disable&&e.elem.disable(),e.elem.is("li")?e.elem.addClass("is-disabled"):e.elem.is("input.dropdown, input.multiselect")||(e.elem.attr("tabindex",e.originalTabindex),i.prop("disabled",e.originalDisabled))}))}})),this.moreButton.removeClass("is-disabled"),this.isModuleTabs()&&this.element.children(".toolbar").enable(),this.disabledElems=[],this.updateAria(this.tablist.find(".is-selected > a"))},closeDismissibleTab:function(t){return this.remove(t)},closeDismissibleTabs:function(t){var e=this;t.forEach((function(t){var i=t.match(/#.*/);return e.remove(i[0])}))},teardown:function(){this.panels.removeAttr("style"),this.tablist.off().removeAttr("role").removeAttr("aria-multiselectable");var t=this.tablist.children("li");if(t.off().removeAttr("role").removeClass("is-selected"),t.filter(".has-popupmenu").each((function(){var t=$(this).data("popupmenu");t&&(t.menu.children("li:not(.separator)").each((function(){var t=$(this).children("a"),e=t.data("panel-link");$.removeData(t[0],"panel-link"),e&&e.length&&$.removeData(e[0],"tab-link")})),t.destroy())})),this.panels.off(),this.anchors.off().removeAttr("role").removeAttr("aria-expanded").removeAttr("aria-selected").removeAttr("tabindex"),this.ro&&(this.ro.disconnect(),delete this.ro),(this.settings.moduleTabsTooltips||this.settings.multiTabsTooltips)&&this.anchors.each((function(){var t=$(this).data("tooltip");t&&"function"==typeof t.destroy&&t.destroy()})),this.element.off("focusout.tabs updated.tabs activated.tabs aftertabadded.tabs"),$("body").off("resize.tabs".concat(this.tabsIndex)),this.tabsIndex=void 0,this.moreButton.data("popupmenu")){var e=this.moreButton.data("popupmenu");e.menu.find("li:not(.separator)").each((function(){var t=$(this).children("a");t.data("original-tab")&&$.removeData(t[0],"original-tab")})),e.destroy()}return this.removeHelperMarkupEvents(),this.tablistContainer&&this.tablistContainer.off("mousewheel.tabs"),this.focusState.removeData().remove(),this.focusState=void 0,$(".tab-panel input").off("error.tabs valid.tabs"),this.addTabButton&&(this.settings.addTabButtonTooltip&&this.addTabButton.data("tooltip").destroy(),this.addTabButton.remove(),this.addTabButton=void 0),this.element.find(".close.icon").remove(),this},destroy:function(){this.teardown(),$.removeData(this.element[0],ro)}},$.fn.tabs=function(t){return this.each((function(){var e=$.data(this,ro);e?e.updated(t):(e=$.data(this,ro,new ho(this,t)),this.element=void 0)}))},$.fn.verticaltabs=$.fn.tabs,$.fn.tag=function(t){return this.each((function(){var e=$.data(this,Yi);e?e.updated(t):e=$.data(this,Yi,new Qi(this,t))}))},$.fn.taglist=function(t){return this.each((function(){var e=$.data(this,Ji);e?e.updated(t):e=$.data(this,Ji,new tn(this,t))}))};var uo="textarea",po={autoGrow:!1,autoGrowMaxHeight:null,characterCounter:!0,maxLength:null,printable:!0,charRemainingText:null,charMaxText:null,attributes:null};function go(t,e){this.settings=E.mergeSettings(t,e,po),this.element=$(t),this.init()}go.prototype={init:function(){this.element.is(":disabled")&&this.element.closest(".field").addClass("is-disabled"),this.isChrome=/Chrome/.test(navigator.userAgent)&&/Google Inc/.test(navigator.vendor),this.isSafari=-1!==navigator.userAgent.indexOf("Safari")&&-1===navigator.userAgent.indexOf("Chrome")&&-1===navigator.userAgent.indexOf("Android"),this.element.addClass(this.element.is(".textarea-xs")?"input-xs":this.element.is(".textarea-sm")?"input-sm":this.element.is(".textarea-def")?"input-md":this.element.is(".textarea-lg")?"input-lg":""),this.settings.characterCounter&&this.getMaxLength()&&(this.counter=this.element.siblings(".textarea-wordcount"),0===this.counter.length&&(this.counter=$('<span class="textarea-wordcount">Chars Left..</span>').insertAfter(this.element))),this.settings.printable&&(this.printarea=$('<span class="textarea-print"></span>').insertBefore(this.element)),this.element.hasClass("autogrow")&&(this.settings.autoGrow=!0),this.setupAutoGrow(),this.handleEvents(),this.updateCounter()},isSelected:function(t){return"number"==typeof t.selectionStart?0===t.selectionStart&&t.selectionEnd===t.value.length:void 0!==document.selection&&document.selection.createRange().text===t.value},isPrintable:function(t,e){return e?t>64&&t<91||t>=33&&t<=38||t>=40&&t<=43||126===t||58===t||60===t||t>=123&&t<=125||94===t||95===t||t>=62&&t<=64:13===t||t>=48&&t<=57||t>=97&&t<=122||59===t||61===t||t>=44&&t<=47||96===t||t>=91&&t<=93||39===t},setupAutoGrow:function(){if(this.settings.autoGrow&&this.element.length){var t=this.element[0];this.settings.autoGrowMaxHeight&&(t.style.maxHeight="".concat(this.settings.autoGrowMaxHeight,"px")),t.style.overflow="hidden",this.autoGrow()}},autoGrow:function(){if(this.settings.autoGrow){var t=this.element[0],e=t.offSetHeight,i=this.settings.autoGrowMaxHeight||0,n=t.scrollHeight;if(i>0&&i<n){if(n=i,t.style.overflow="",e===n)return}else t.style.overflow="hidden";t.style.height="5px",t.style.height="".concat(t.scrollHeight+2,"px")}},countLinebreaks:function(t){return(t.match(/\n/g)||[]).length},updateCounter:function(){var t=this,e=t.element.val(),i=this.isChrome||this.isSafari,n=e.length+(i?this.countLinebreaks(e):0),s=t.getMaxLength(),a=parseInt(s,10)-n,o=(t.settings.charRemainingText?t.settings.charRemainingText:"CharactersLeft"===H.translate("CharactersLeft")?"Characters Left":H.translate("CharactersLeft")).replace("{0}",a.toString());t.counter&&(n>=s?(o=t.settings.charMaxText?t.settings.charMaxText.replace("{0}",s):H.translate("CharactersMax")+s,t.counter.text(o),t.counter.removeClass("almost-empty")):(t.counter.text(o),a<10?t.counter.addClass("almost-empty"):t.counter.removeClass("almost-empty"))),t.printarea.text(t.element.val())},enable:function(){this.element.prop("disabled",!1).prop("readonly",!1),this.element.removeAttr("disabled readonly").closest(".field").removeClass("is-disabled")},disable:function(){this.enable(),this.element.prop("disabled",!0)},isDisabled:function(){return this.element.prop("disabled")},readonly:function(){this.element.prop("readonly",!0)},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.destroy(),this.init(),this.element.data("trackdirty")&&this.element.data("trackdirty")instanceof Object&&this.element.data("trackdirty").updated(),this.element.data("validate")&&this.element.data("validate")instanceof Object&&this.element.data("validate").updated()},getMaxLength:function(){return this.settings.maxLength?this.settings.maxLength:this.element.attr("maxlength")?parseInt(this.element.attr("maxlength"),10):void 0},destroy:function(){this.printarea&&this.printarea.length&&this.printarea.remove(),this.counter&&this.counter.length&&this.counter.remove(),this.element.off()},handleEvents:function(){var t=this,e=this;this.element.on("paste.textarea",(function(i){var n=t.element[0].selectionStart,s=t.element[0].selectionEnd,a=t.element.val(),o=i.originalEvent.clipboardData.getData("text/plain"),r=e.getMaxLength(),l=(a.slice(0,n)+o+a.slice(s,a.length-1)).substring(0,r),c=e.isChrome||e.isSafari,d=c?e.countLinebreaks(l):0;if(l.length+(c?e.countLinebreaks(l):0)>r){var h=l.substring(0,r-d);e.element.val(h),e.updateCounter(),i.preventDefault()}})).on("keyup.textarea",(function(t){var i=e.element.val(),n=e.isChrome||e.isSafari,s=i.length+(n?e.countLinebreaks(i):0),a=e.getMaxLength();return e.updateCounter(),s>=a?(t.preventDefault(),!1):(e.settings.autoGrow&&e.autoGrow(),!0)})).on("focus.textarea",(function(){e.counter&&e.counter.addClass("focus")})).on("updated.textarea",(function(){e.updated()})).on("keypress.textarea",(function(t){var i=e.element.val(),n=e.isChrome||e.isSafari,s=i.length+(n?e.countLinebreaks(i):0),a=e.getMaxLength();[97,99,118,120].indexOf(t.which)>-1&&(t.metaKey||t.ctrlKey)?e.updateCounter():e.isPrintable(t.which,t.shiftKey)&&s>=a&&!e.isSelected(this)&&t.preventDefault()})).on("blur.textarea",(function(){e.updateCounter(),e.counter&&e.counter.removeClass("focus")}))}},$.fn.textarea=function(t){return this.each((function(){var e=$.data(this,uo);e?e.updated(t):e=$.data(this,uo,new go(this,t))}))},$.fn.toolbarsearchfield=$.fn.searchfield;var fo="trackdirty",mo={};function vo(t,e){this.settings=E.mergeSettings(t,e,mo),this.element=$(t),this.init()}vo.prototype={init:function(){this.isIe="ie"===x.browser.name,this.isIeEdge="edge"===x.browser.name,this.handleEvents()},valMethod:function(t,e){switch(t.attr("type")){case"checkbox":case"radio":return t.prop("checked");case"file":if("boolean"==typeof e&&e){var i=t.attr("original");return void 0!==i?i:""}return t.parent().find('[type="text"]').val();default:if(t.is("textarea")&&1===t.closest(".editor-source").length){var n=t.is(":visible")?t.val():t.text();return this.trimEditorText(n)}return t.val()}},trimEditorText:function(t){return t.trim().replace(/>\s+</g,"><").replace(/\s+/g," ").replace(" has-tooltip","").replace(/<br(\s+)?\/?>/g,"<br>\n").replace(/<\/p>(\s+)/g,"</p>\n\n").replace(/<\/blockquote>(\s+)?/g,"</blockquote>\n\n").replaceAll("&nbsp;"," ")},getAbsolutePosition:function(t){var e=t.position();return t.parents().each((function(){var t=this;if("relative"===window.getComputedStyle(t,null).position)return!1;e.left+=t.scrollLeft,e.top+=t.scrollTop})),{left:e.left,top:e.top}},unbind:function(){if(this.element.removeClass("dirty").off("resetdirty.dirty change.dirty doresetdirty.dirty"),this.settings&&"object"===l(this.settings.d)){var t=this.settings.d;$(".icon-dirty, .msg-dirty",t.field).add(t.icon).add(t.msg).remove()}return $.removeData(this.element[0],"original"),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,mo)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],fo)},handleEvents:function(){var t=this,e=this.element;if(e.is(".editor")){var i=e.parent().find("textarea");i.data("original",this.valMethod(i))}e.data("original",this.valMethod(e,!0)).on("resetdirty.dirty",(function(){if(e.is(".editor")){var i=e.parent().find("textarea");i.data("original",t.valMethod(i))}e.data("original",t.valMethod(e)).triggerHandler("doresetdirty.dirty")})).on("change.dirty doresetdirty.dirty",(function(i){var n,s=e,a=e.closest(".field, .radio-group"),o=$("label:visible",a),r={class:"",style:""};if(a.is(".field-fileupload")&&(s=a.find("input.fileupload-background-transparent")),a.is(".editor-container")&&(s=a.find(".editor-source")),o[0]||(o=e.next("label")),"true"===e.attr("data-trackdirty")){if(e.addClass("dirty"),e.is('[type="checkbox"], [type="radio"]')&&(r.class+=" dirty-".concat(e.attr("type")),r.class+=e.is(":checked")?" is-checked":""),e.is("select")&&(r.class+=" is-select",s=e.next(".dropdown-wrapper").find(".dropdown")),r.icon=s.prev(),null==r||null===(n=null==r?void 0:r.icon)||void 0===n||!(null==n?void 0:n.is(".icon-dirty"))){e.is(".colorpicker")&&!H.isRTL()&&(r.rect=t.getAbsolutePosition(e),r.style="left:".concat(r.rect.left,"px; top:").concat(r.rect.top,"px;")),r.icon='<span class="icon-dirty'.concat(r.class,'"></span>'),r.msg=H.translate("MsgDirty")||"",r.msg='<span class="audible msg-dirty">'.concat(r.msg,"</span>");var l=$($(s[0].parentElement).find("input")[0]);s=e.is('[type="radio"]')?l:s,0===$(s[0].parentElement).find(".icon-dirty").length&&(s.before(r.icon),o.append(r.msg),r.style&&s.prev(".icon-dirty")[0]&&(s.prev(".icon-dirty")[0].style.cssText=r.style)),r.icon=s.prev(),r.msg=o.find(".msg-dirty")}var c=e.data("original"),d=t.valMethod(e);if(r.field=a,t.settings.d=r,a.is(".editor-container")){var h=a.find("textarea");c=h.data("original"),d=a.find(".editor-source").is(":visible")?h.val():h.text(),d=t.trimEditorText(d)}if(d===c||e.attr("multiple")&&E.equals(d,c))return e.removeClass("dirty"),$(".icon-dirty, .msg-dirty",a).add(r.icon).add(r.msg).remove(),void e.trigger("doresetdirty"===i.type?"afterresetdirty":"pristine");e.trigger("dirty")}}))}},$.fn.trackdirty=function(t){return this.each((function(){var e=$.data(this,fo);e&&e instanceof vo?e.updated(t):e=$.data(this,fo,new vo(this,t))}))};var bo="tree",yo={dataset:null,source:null,selectable:"single",hideCheckboxes:!1,menuId:null,useStepUI:!1,folderIconOpen:"open-folder",folderIconClosed:"closed-folder",sortable:!1,onBeforeSelect:null,onExpand:null,onCollapse:null,expandTarget:"node",useExpandTarget:!1,expandIconOpen:"plusminus-folder-open",expandIconClosed:"plusminus-folder-closed",expandPlusminusRotate:!0,showChildrenCount:!1,childrenAutoCount:!0};function wo(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,yo),this.init()}function xo(t,e){var i=$(t),n=i.data("".concat(bo,"Api"));if(n){var s={collapse:n.collapseNode,expand:n.expandNode};"function"==typeof s[e]&&s[e].apply(n,i)}}wo.prototype={init:function(){this.isIe11="ie"===x.browser.name&&"11"===x.browser.version,this.initTree(),this.handleKeys(),this.setupEvents(),-1===this.loadData(this.settings.dataset)&&(this.syncDataset(),this.initSelected(),this.focusFirst(),this.attachMenu(this.settings.menuId),this.createSortable(),this.childrenCountInit())},initTree:function(){var t=this,e=this.settings,i=this.element.find("a"),n=this.element.attr("data-selectable");e.useExpandTarget&&(e.expandTarget="icon"),e.selectable=void 0===n||"single"!==n.toLowerCase()&&"multiple"!==n.toLowerCase()?e.selectable:n,this.isMultiselect="multiple"===e.selectable,e.hideCheckboxes=e.hideCheckboxes||!this.isMultiselect,this.element.addClass(this.isMultiselect?" is-multiselect":""),i.each((function(){var e=$(this);t.decorateNode(e)}))},initSelected:function(){var t=this;[].slice.call(this.element[0].querySelectorAll("li")).forEach((function(e){e.setAttribute("role","none"),t.setNodeStatus($(e.querySelector("a")))})),this.syncDataset()},focusFirst:function(){var t=this.element[0].querySelector("a");t&&t.setAttribute("tabindex","0")},setFocus:function(t){(t=this.isjQuery(t)?t[0]:t)&&(t.focus(),f.removeClass(t,"hide-focus"))},decorateNode:function(t,e,i,n){var s,a=(t=this.isjQuery(t)?t:$(t))[0].getAttribute("data-badge"),o=t[0].getAttribute("data-alert-icon"),r=t.parentsUntil(this.element,"ul[role=group].is-disabled").length>0,l=f.hasClass(t[0],"is-disabled")||r;if(void 0!==a&&(a=E.parseSettings(t,"data-badge")),t[0].setAttribute("role","treeitem"),t[0].setAttribute("tabindex","-1"),t[0].setAttribute("aria-selected","false"),l){f.addClass(t[0],"is-disabled"),t[0].setAttribute("aria-disabled","true");var c=t.next();if(c[0]&&"ul"===c[0].tagName.toLowerCase()&&f.hasClass(c[0],"is-open"))[].slice.call(c[0].querySelectorAll("a")).forEach((function(t){f.addClass(t,"is-disabled"),t.setAttribute("aria-disabled","true")})),[].slice.call(t[0].parentNode.querySelectorAll("ul")).forEach((function(t){return f.addClass(t,"is-disabled")}))}s=t.parentsUntil(this.element,"ul").length-1,t[0].setAttribute("aria-level",s+1);var d=t.parent().index();t[0].setAttribute("aria-posinset",d+1);var h=t.closest("li").siblings().length+1;t[0].setAttribute("aria-setsize",h);var u=t.next("ul"),p=!1;u[0]&&u.children().length>0&&(p=f.hasClass(u[0],"is-open"),t[0].setAttribute("aria-expanded",p?"true":"false"));var m=t.contents().filter((function(){return!$(this).is(".tree-badge, .tree-children-count")})).text();t[0].textContent="",0===t.children("svg.icon-tree").length&&(e||""===e?t[0].insertAdjacentHTML("afterbegin",$.createIcon({icon:e,classes:["icon-tree"]})):t[0].insertAdjacentHTML("afterbegin",$.createIcon({icon:"tree-node",classes:["icon-tree"]})),this.settings.useStepUI&&t[0].insertAdjacentHTML("afterbegin",$.createIcon({icon:o,classes:["step-alert","icon-".concat(o)]}))),u[0]&&t[0].insertAdjacentHTML("beforeend",this.getExpandTargetHtml(p)),!this.isMultiselect||this.settings.hideCheckboxes&&!1!==i||t[0].insertAdjacentHTML("beforeend",'<span class="tree-checkbox"></span>');var v=this.getBadgeHtml(a);""!==v&&t[0].insertAdjacentHTML("beforeend",v);var b=document.createElement("span");if(f.addClass(b,"tree-text"),b.textContent=m,b.innerHTML.indexOf("&lt;svg")>0&&(b.innerHTML=g.unescapeHTML(b.innerHTML)),t[0].appendChild(b),u[0]&&t[0].insertAdjacentHTML("beforeend",this.getChildrenCountHtml(n)),this.hasIconClass(t)&&this.setTreeIcon(t.find("svg.icon-tree"),t[0].getAttribute("class")),u[0]&&"ul"===u[0].tagName.toLowerCase()){var y=t[0].getAttribute("class");if(this.settings.useExpandTarget&&y&&!n){var w=y.match(/(icon-)[^\s]+/g);w&&(n={icon:w[0]})}u[0].setAttribute("role","group"),f.addClass(u[0].parentNode,"folder"),this.setFolderIcon(t,f.hasClass(u[0],"is-open"),n),y&&-1===y.indexOf("open")&&-1===y.indexOf("closed")&&(l&&f.addClass(t[0],"is-disabled"),this.setFolderIcon(t,f.hasClass(u[0],"is-open"),n)),this.hasIconClass(t)&&(y=t[0].getAttribute("class"),(/open|close/g.test(y)||!u[0]||this.settings.useExpandTarget)&&this.setTreeIcon(t.find("svg.icon-tree"),f.hasClass(u[0],"is-open")?y:y.replace("open","closed")))}t.hideFocus()},setTreeIcon:function(t,e){if(t&&"string"==typeof e){t=this.isjQuery(t)?t:$(t);var i=e.replace(/#?icon-|hide-focus|is-selected|is-active|\s?/gi,"");t.changeIcon(i)}},expandAll:function(t){var e=this,i=t;(i=void 0!==i?this.isjQuery(i)?$.makeArray(i):i:[].slice.call(this.element[0].querySelectorAll("ul[role=group]"))).forEach((function(t){var i=t.previousElementSibling;if(f.addClass(t.parentNode,"is-open"),f.addClass(t,"is-open"),t.style.height="",i&&"a"===i.tagName.toLowerCase()){var n=i.querySelector("svg.icon-tree");e.setTreeIcon(n,e.settings.folderIconOpen),i.setAttribute("aria-expanded",!0),e.hasIconClass(i)&&e.setTreeIcon(n,i.getAttribute("class"))}}))},collapseAll:function(t){var e=this,i=t;(i=void 0!==i?this.isjQuery(i)?$.makeArray(i):i:[].slice.call(this.element[0].querySelectorAll("ul[role=group]"))).forEach((function(t){var i=t.previousElementSibling;if(f.removeClass(t.parentNode,"is-open"),f.removeClass(t,"is-open"),t.style.height=0,i&&"a"===i.tagName.toLowerCase()){var n=i.querySelector("svg.icon-tree");e.setTreeIcon(n,e.settings.folderIconClosed),i.setAttribute("aria-expanded",!1),f.removeClass(i,"is-selected"),e.hasIconClass(i)&&e.setTreeIcon(n,i.getAttribute("class").replace("open","closed"))}}))},isjQuery:function(t){return t&&(t instanceof jQuery||t.constructor.prototype.jquery)},selectNodeById:function(t){this.selectNodeByJquerySelector("#".concat(t))},selectNodeByJquerySelector:function(t){var e=this.isjQuery(t)?t:$(t);if(e.length&&!e.is(".is-disabled")){var i=e.parentsUntil(this.element,"ul[role=group]");this.expandAll(i),this.selectNode(e,!0)}},unSelectedNode:function(t,e){if(0!==t.length){var i=this;$("a",this.element).attr("tabindex","-1"),t.attr("tabindex","0"),$("a:not(.is-disabled)",t.parent()).attr("aria-selected","false").parent().removeClass("is-selected"),this.syncNode(t),this.setNodeStatus(t),e&&t.focus(),[].slice.call(this.element[0].querySelectorAll("li")).forEach((function(t){return f.removeClass(t,"is-active")})),f.addClass(t[0].parentNode,"is-active"),setTimeout((function(){var e=t.data("jsonData")||{};i.element.triggerHandler("unselected",{node:t,data:e})}),0)}},selectNode:function(t,e){var i,n=this,s=this.settings;0!==t.length&&("function"==typeof s.onBeforeSelect?(i=s.onBeforeSelect(t))&&i.done&&"function"==typeof i.done?i.done((function(i){i&&n.selectNodeFinish(t,e)})):i&&n.selectNodeFinish(t,e):n.selectNodeFinish(t,e))},selectNodeFinish:function(t,e,i){if((this.isMultiselect||"icon"===this.settings.expandTarget)&&i)if("click"===i.type||"touch"===i.type){if(f.hasClass(i.target,"icon")&&f.hasClass(t[0].parentNode,"folder"))return}else if("keydown"===i.type){var n=i.charCode||i.keyCode;if(37===n||39===n)return}var s=this,a=[].slice.call(this.element[0].querySelectorAll("a"));if(a.forEach((function(t){return t.setAttribute("tabindex","-1")})),t[0].setAttribute("tabindex","0"),this.isMultiselect){[].slice.call(t[0].parentNode.querySelectorAll("a:not(.is-disabled)")).forEach((function(t){t.setAttribute("aria-selected","true"),f.addClass(t,"is-selected"),f.addClass(t.parentNode,"is-selected")}))}else{if(f.hasClass(t[0],"is-selected"))return;a.forEach((function(t){var e=$(t),i=e.data("jsonData");i&&(delete i.selected,e.data("jsonData",i)),t.setAttribute("aria-selected","false"),f.removeClass(t,"is-selected"),f.removeClass(t.parentNode,"is-selected")})),t[0].setAttribute("aria-selected","true"),f.addClass(t[0],"is-selected"),f.addClass(t[0].parentNode,"is-selected")}this.syncNode(t),this.loading||this.setNodeStatus(t),e&&t.focus(),[].slice.call(this.element[0].querySelectorAll("li")).forEach((function(t){return f.removeClass(t,"is-active")})),f.addClass(t[0].parentNode,"is-active"),setTimeout((function(){var e=t.data("jsonData")||{};s.element.triggerHandler("selected",{node:t,data:e})}),0)},setNodeStatus:function(t){var e,i=this,n=t.data("jsonData");if(this.isMultiselect){var s=function(t,e){t.forEach((function(t){var n=$(t.querySelector("a")),s=i.getSelectedStatus(n,e);"mixed"===s?(f.removeClass(t,"is-selected","is-partial"),f.addClass(t,"is-partial")):s?(f.removeClass(t,"is-selected","is-partial"),f.addClass(t,"is-selected")):f.removeClass(t,"is-selected","is-partial"),i.syncNode(n)}))},a=!1;s(e=[].slice.call(t[0].parentNode.querySelectorAll("li.folder")),a),a=!(!e.length&&n&&!n.selected),e=t.parentsUntil(this.element,"li.folder"),s(e=[].slice.call(e.toArray()),a)}else{var o=t[0],r=o.parentNode;n&&n.selected?(f.addClass(r,"is-selected"),f.addClass(o,"is-selected"),o.setAttribute("aria-selected",!0)):(f.removeClass(r,"is-selected","is-partial"),f.removeClass(o,"is-selected","is-partial"),o.setAttribute("aria-selected",!1))}},getSelectedStatus:function(t,e){var i,n=0,s=0,a=0;return t.parent().find("a").each((function(t){e&&0===t||(n++,(i=$(this).data("jsonData"))&&i.selected?s++:a++)})),n===s||n!==a&&"mixed"},getNodeAsJQ:function(t){return"string"==typeof t?$("#".concat(t)):this.isjQuery(t)?t:$(t)},expandNode:function(t,e){var i=this.getNodeAsJQ(t),n=!this.isTrue(e);this.toggleNode(i,null,{isExpandNode:!0,noSelect:n})},collapseNode:function(t,e){var i=this.getNodeAsJQ(t),n=!this.isTrue(e);this.toggleNode(i,null,{isExpandNode:!1,noSelect:n})},toggleNode:function(t,e,i){var n,s=t.next(),a=this,o=this.settings;if(s[0]&&"ul"===s[0].tagName.toLowerCase()&&"group"===s[0].getAttribute("role")){var r=t.find("svg.icon-tree").getIconName(),c=function(i){if("function"==typeof o.onCollapse)(n=o.onCollapse(t))&&n.done&&"function"==typeof n.done?n.done((function(n){n&&!i&&a.selectNodeFinish(t,focus,e)})):n&&!i&&a.selectNodeFinish(t,focus,e);else if("icon"!==o.expandTarget||i)i||a.selectNodeFinish(t,focus,e);else{var l=t[0].parentNode;e&&f.hasClass(e.target,"icon")&&f.hasClass(l,"folder")&&a.selectNodeFinish(t,focus,e)}if(t.closest(".folder").removeClass("is-open"),r===o.folderIconOpen||o.useExpandTarget)a.setFolderIcon(t,!1,!1,o.expandPlusminusRotate);else if(/open|close/g.test(r)&&!o.useExpandTarget){var c=r.replace("open","closed");i||(c=c.replace(/\s?is-selected/,"")),a.setTreeIcon(t.find("svg.icon-tree"),c)}var d=t.closest(".folder a");if(a.hasIconClass(d)){var h=d.attr("class");if(/open|close/g.test(h)||o.useExpandTarget){var u=h.replace("open","closed");i||(u=u.replace(/\s?is-selected/,"")),a.setTreeIcon(d.find("svg.icon-tree"),u)}}a.isAnimating=!0,a.isMultiselect||"icon"!==!o.expandTarget||i||(a.unSelectedNode(t.parent().find("li.is-selected"),!1),f.removeClass(t[0],"is-selected")),s.one("animateclosedcomplete",(function(){f.removeClass(s[0],"is-open"),a.isAnimating=!1})).animateClosed(),t[0].setAttribute("aria-expanded","true"!==t[0].getAttribute("aria-expanded")),a.element.triggerHandler("collapsed",{node:t,data:t.data("jsonData")})},d=function(i){if("function"==typeof o.onExpand)(n=o.onExpand(t))&&n.done&&"function"==typeof n.done?n.done((function(n){n&&!i&&a.selectNodeFinish(t,focus,e)})):n&&!i&&a.selectNodeFinish(t,focus,e);else if("icon"!==o.expandTarget||i)i||a.selectNodeFinish(t,focus,e);else{var l=t[0].parentNode;e&&f.hasClass(e.target,"icon")&&f.hasClass(l,"folder")&&a.selectNodeFinish(t,focus,e)}var c=t.data("jsonData");if(o.source&&c.children&&0===c.children.length){var d={node:t,data:t.data("jsonData")};return f.addClass(t[0],"is-loading"),a.loading=!0,a.settings.source(d,(function(e){var n=c.id,r=a.findById(n);r.children=e,a.addChildNodes(r,t.parent()),f.removeClass(t[0],"is-loading"),a.loading=!1,a.accessNode(s,t),c.children=e,t.data("jsonData",c),t.data("".concat(bo,"Api"),a),f.hasClass(t[0].parentNode,"folder")&&a.childrenCountInit(t[0].parentNode),a.isMultiselect||"icon"!==!o.expandTarget||i||a.selectNode(t,!0),i||a.initSelected()})),void a.element.triggerHandler("expanded",{node:t,data:t.data("jsonData")})}if(r===o.folderIconClosed||o.useExpandTarget)a.setFolderIcon(t,!0,!1,o.expandPlusminusRotate);else if(/open|close/g.test(r)&&!o.useExpandTarget){var h=r.replace("closed","open");i||(h=h.replace(/\s?is-selected/,"")),a.setTreeIcon(t.find("svg.icon-tree"),h)}a.accessNode(s,t),a.element.triggerHandler("expanded",{node:t,data:t.data("jsonData")})};if("object"===l(i)&&function(t){var e=!1;if("boolean"==typeof t)e=!0;else if("string"==typeof t){var i=t.toLowerCase();"true"!==i&&"false"!==i||(e=!0)}return e}(i.isExpandNode))if(i.isExpandNode){d(i.noSelect);var h=t.parentsUntil(this.element,"ul[role=group]");this.expandAll(h)}else c(i.noSelect);else f.hasClass(s[0],"is-open")?c():d()}},accessNode:function(t,e){var i=this,n=e.attr("class");e.closest(".folder").addClass("is-open");var s=e.find("svg.icon-tree").getIconName();if(s===this.settings.folderIconClosed||this.settings.useExpandTarget?this.setFolderIcon(e,!0,!1,this.settings.expandPlusminusRotate):/open|close/g.test(s)&&!this.settings.useExpandTarget&&this.setTreeIcon(e.find("svg.icon-tree"),s.replace("closed","open").replace(/\s?is-selected/,"")),this.hasIconClass(n)){var a=t[0]&&"ul"===t[0].tagName.toLowerCase();(/open|close/g.test(n)||!a||this.settings.useExpandTarget)&&this.setTreeIcon(e.find("svg.icon-tree"),n.replace("is-selected",""))}this.isAnimating=!0,t.one("animateopencomplete",(function(){i.isAnimating=!1})).addClass("is-open").css("height",0).animateOpen(),e[0].setAttribute("aria-expanded","true"!==e[0].getAttribute("aria-expanded"))},openNode:function(t,e){var i=this,n=e.data("jsonData");if(i.settings.source&&n.children&&0===n.children.length){var s={node:e,data:e.data("jsonData")};return e.addClass("is-loading"),i.loading=!0,void i.settings.source(s,(function(s){var a=n.id,o=i.findById(a);o.children=s,i.addChildNodes(o,e.parent()),e.removeClass("is-loading"),i.loading=!1,i.accessNode(t,e),n.children=s,e.data("jsonData",n),e.data("".concat(bo,"Api"),i),i.selectNode(e,!0),i.initSelected()}))}i.accessNode(t,e)},hasIconClass:function(t){return"string"!=typeof t&&(this.isjQuery(t)&&(t=t.length>1?t.first()[0]:t[0]),t=t.getAttribute("class")),t&&t.indexOf("icon")>-1},closeNode:function(t,e){var i=this;i.setTreeIcon(e.closest(".folder").removeClass("is-open").end().find("svg.icon-tree"),i.settings.folderIconClosed),i.hasIconClass(e.closest(".folder a"))&&i.setTreeIcon(e.closest(".folder a").find("svg.icon-tree"),e.closest(".folder a").attr("class").replace("open","closed").replace(/\s?is-selected/,"")),i.isAnimating=!0,i.isMultiselect||(i.unSelectedNode(e.parent().find("li.is-selected"),!1),e.removeClass("is-selected")),t.one("animateclosedcomplete",(function(){t.removeClass("is-open"),i.isAnimating=!1})).animateClosed(),e.attr("aria-expanded","true"!==e.attr("aria-expanded"))},setupEvents:function(){var t=this;t.element.on("updated.tree",(function(e,i){t.updated(i),t.initTree()}))},handleKeys:function(){var t=this;this.element.on("click.tree","a:not(.is-clone)",(function(e){var i=$(this),n=this.parentNode;return E.clearSelection(),f.hasClass(i[0],"is-disabled")||f.hasClass(i[0],"is-loading")||(t.isMultiselect||"icon"===t.settings.expandTarget?f.hasClass(e.target,"icon")&&f.hasClass(n,"folder")?t.toggleNode(i,e):f.hasClass(n,"is-selected")||f.hasClass(n,"is-partial")?t.unSelectedNode(i,!0):t.selectNode(i,!0):(t.selectNode(i,!0),t.toggleNode(i,e)),e.stopPropagation()),t.popupEl&&t.popupEl.data("popupmenu")&&(t.popupEl.data("popupmenu").close(),t.popupEl=null),t.syncDataset(),!1})),this.element.on("focus.tree","a",(function(){if(0===parseInt(this.getAttribute("aria-level"),10)&&1===parseInt(this.getAttribute("aria-posinset"),10)&&f.hasClass(this,"is-disabled")){var t=$.Event("keydown.tree");return t.keyCode=40,void $(this).trigger(t)}})),this.element.on("keydown.tree","a",(function(e){var i,n,s=e.charCode||e.keyCode,a=$(this);if(!t.isAnimating){if(40===s){var o=t.getNextNode(a);t.setFocus(o)}if(38===s){var r=t.getPreviousNode(a);t.setFocus(r)}if(32===e.keyCode&&a.trigger("click.tree"),37===s)return H.isRTL()?a.next().hasClass("is-open")?(n=a.next().find("a:first"),t.setFocus(n)):t.toggleNode(a,e):a.next().hasClass("is-open")?t.toggleNode(a,e):(n=a.closest(".folder").find("a:first"),t.setFocus(n)),e.stopPropagation(),!1;if(39===s)return H.isRTL()?a.next().hasClass("is-open")?t.toggleNode(a,e):(i=a.closest(".folder").find("a:first"),t.setFocus(i)):a.next().hasClass("is-open")?(i=a.next().find("a:first"),t.setFocus(i)):(t.toggleNode(a,e),t.setFocus(a)),e.stopPropagation(),!1;36===s&&(i=t.element.find("a:first:visible"),t.setFocus(i)),35===s&&(i=t.element.find("a:last:visible"),t.setFocus(i))}})),this.element.on("keypress.tree","a",(function(e){var i=e.charCode||e.keyCode,n=$(this);if(i>=37&&i<=40||32===i)return e.stopPropagation(),!1;0!==e.which&&n.closest("li").nextAll().find("a:visible").each((function(){var i=$(this);if(i.text().substr(0,1).toLowerCase()===String.fromCharCode(e.which).toLowerCase())return t.setFocus(i),!1}))}))},loadData:function(t){if(!t)return-1;var e=this;e.element.empty(),e.loading=!0,t=this.arrangeDataset(t);var i="";e.jsonData=[];for(var n=0,s=t.length;n<s;n++)i+=e.getNodeHtml(t[n],n);e.element[0].insertAdjacentHTML("beforeend",i),[].slice.call(e.element[0].querySelectorAll('a[role="treeitem"]')).forEach((function(t,i){var n=$(t),s=e.jsonData[i];n.data("jsonData",s),n.data("".concat(bo,"Api"),e),s.selected&&e.selectNode(n,s.focus),e.addAutomationAttributes(n,s),n.is(".hide-focus")&&n.hideFocus()}));for(var a=[].slice.call(e.element[0].querySelectorAll("select.dropdown")),o=0;o<a.length;o++){var r=a[o];e.jsonData[o].disabled?$(r).dropdown().disable():$(r).dropdown().on("selected.tree",(function(){var t=e.findById(this.parentElement.previousElementSibling.id);t.text=this.value,e.updateNode(t)}))}e.jsonData=void 0,e.loading=!1,e.syncDataset(),e.initSelected(),e.focusFirst(),e.attachMenu(e.settings.menuId),e.createSortable(),this.childrenCountInit()},arrangeDataset:function(t){if(!this.hasKeyInData("parent",t))return t;t=t||this.settings.dataset;var e=JSON.parse(JSON.stringify(t));!function t(i){for(var n=0;n<i.length;n++)void 0!==i[n].parent&&(s=i[n],a=void 0,a=!1,function t(e){for(var i=0;i<e.length&&!a;i++)e[i].id===s.parent&&((n=e[i]).children=n.children||[],n.children.push(s),a=!0),void 0!==e[i].children&&t(e[i].children);var n}(e)),void 0!==i[n].children&&t(i[n].children);var s,a}(t);return function t(e,i){for(var n=0;n<e.length;n++)void 0!==e[n].children&&t(e[n].children,e[n].id),void 0===i&&void 0!==e[n].parent||void 0!==i&&void 0!==e[n].parent&&i!==e[n].parent?(e.splice(n,1),n--):delete e[n].parent}(e),this.settings.dataset=e,e},hasKeyInData:function(t,e){for(var i=!1,n=function e(n){for(var s in n)if(n.hasOwnProperty(s)){var a=n[s];if("object"!==l(a)||i?t===s&&(i=!0):e(a),i)break}},s=0,a=(e=e||this.settings.dataset).length;s<a&&!i;s++)n(e[s]);return i},getNodeHtml:function(t,e,i,n){var s="",a="";i=i||0,e+=1;var o=this.settings,r=n||t.disabled||!1,c=t.children&&Array.isArray(t.children),d={id:void 0!==t.id?' id="'.concat(t.id,'"'):"",href:' href="'.concat(void 0!==t.href?t.href:"#",'"'),expanded:' aria-expanded="'.concat(t.open?"true":"false",'"'),icon:t.icon||""===t.icon?t.icon:"tree-node",alertIcon:"",alertIconAttr:void 0!==t.alertIcon?' data-alert-icon="'.concat(t.alertIcon,'"'):"",text:'<span class="tree-text">'.concat(t.text,"</span>"),class:["hide-focus"],ariaDisabled:r?'aria-disabled="true"':"",checkbox:!this.isMultiselect||this.settings.hideCheckboxes&&!1!==t.hideCheckbox?"":'<span class="tree-checkbox"></span>',badge:"object"===l(t.badge)?this.getBadgeHtml(t.badge):"",expandTarget:c?this.getExpandTargetHtml(t.open):"",childrenCount:c?this.getChildrenCountHtml(t):""};this.jsonData.push(t),o.useStepUI&&(d.alertIcon='<svg class="icon step-alert icon-'.concat(t.alertIcon,'" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-').concat(t.alertIcon,'"></use>'));var h=c?"folder":"";if(""!==(h+=t.selected?" is-selected":"")&&(h+=t.open?" is-open":"",h=' class="'.concat(h,'"')),r&&d.class.push("is-disabled"),t.icon&&(d.icon=t.icon,(!c||c&&/open|closed/i.test(t.icon))&&d.class.push(t.icon)),c&&(!o.useExpandTarget||o.useExpandTarget&&!t.icon)&&(t.open?(d.icon=t.icon&&/open|closed/i.test(t.icon)?t.icon:o.folderIconOpen,n=r):d.icon=t.icon&&/open|closed/i.test(t.icon)?t.icon.replace("open","closed"):o.folderIconClosed),d.icon="#icon-".concat(d.icon.replace(/^#?icon-?/,"")),d.class=' class="'.concat(d.class.join(" "),'"'),d.style="","dropdown"===t.type&&(d.style='style="display: none"',t.data)){s='<select class="dropdown" close-on-select="true">';for(var u=0;u<t.data.length;u++){var p=t.data[u];p.value===t.text?(a=p.text,s+='<option value="'.concat(p.value,'" selected>').concat(p.text,"</option>")):s+='<option value="'.concat(p.value,'">').concat(p.text,"</option>")}s+='</select><div class="dropdown-wrapper"><div class="dropdown"><span>'.concat(a),s='<div class="treeDropdown" style="width: 80px; margin-left: 35px; margin-bottom: -15px">'.concat(s+='</span></div><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-dropdown"></use></svg></div>',"</div>")}var g="\n      <li".concat(h,'>\n        <a role="treeitem" aria-selected="false" tabindex="-1" ').concat(d.style,'\n          aria-level="').concat(i,'"\n          aria-position="').concat(e,'"\n          aria-setsize="').concat(e,'"\n          ').concat(d.id+d.href+d.class+d.expanded+d.ariaDisabled+d.alertIconAttr,'>\n            <svg class="icon-tree icon" focusable="false" aria-hidden="true" role="presentation"><use href="').concat(d.icon,'"></use>\n            </svg>').concat(d.expandTarget+d.checkbox+d.alertIcon+d.badge+d.text+d.childrenCount,"\n        </a>\n        ").concat(s);if(c){g+='<ul class="folder'.concat(t.open?" is-open":"",'" role="group">');for(var f=0,m=t.children.length;f<m;f++)g+=this.getNodeHtml(t.children[f],f,i+1,n);g+="</ul>"}return g+="</li>"},getExpandTargetHtml:function(t){var e="";if(this.settings.useExpandTarget){var i="icon-expand-target",n=t?{icon:this.settings.expandIconOpen,class:"open"}:{icon:this.settings.expandIconClosed,class:"close"};e=$.createIcon({icon:n.icon,classes:[i,"".concat(i,"-").concat(n.class)]})}return e},getChildrenCountHtml:function(t){var e="";return this.settings.showChildrenCount&&(e='<span class="tree-children-count">\n          (<span class="tree-children-count-text">'.concat(this.getDataChildrenCount(t),"</span>)\n        </span>")),e},getDataChildrenCount:function(t){return t&&/string|number/.test(l(t.childrenCount))?parseInt(t.childrenCount,10):0},getActualChildrenCount:function(t){var e=0;if(t){var i=t.nextElementSibling;i&&"ul"===i.tagName.toLowerCase()&&(e=i.childElementCount)}return e},updateChildrenCount:function(t,e){if(t){var i=t.querySelector(".tree-children-count-text");i&&(0===(e=/string|number/.test(l(e))?parseInt(e,10):this.getActualChildrenCount(t))&&parseInt(i.textContent,10)>0&&"function"==typeof this.settings.source&&!f.hasClass(t.nextElementSibling,"is-open")||(i.textContent=e))}},updateAutoChildrenCount:function(t){var e=this;this.settings.showChildrenCount&&t&&[].slice.call(t.querySelectorAll("a")).forEach((function(t){return e.updateChildrenCount(t)}))},childrenCountInit:function(t){var e=this;(t=t||this.element,t=this.isjQuery(t)?t[0]:t,this.settings.showChildrenCount&&t)&&(this.settings.childrenAutoCount?this.updateAutoChildrenCount(t):[].slice.call(t.querySelectorAll("a[data-children-count]")).forEach((function(t){var i=t.getAttribute("data-children-count");void 0!==i&&e.updateChildrenCount(t,i)})))},setFolderIcon:function(t,e,i,n){var s=this.settings,a=this.isjQuery(t)?t:$(t);if(t=a[0]){var o,r={el:{tree:t.querySelector("svg.icon-tree"),expand:t.querySelector("svg.icon-expand-target")},icon:e?{tree:s.folderIconOpen,expand:s.expandIconOpen,rotate:"forward"}:{tree:s.folderIconClosed,expand:s.expandIconClosed,rotate:"backward"}};s.useExpandTarget?(o={el:r.el.expand,icon:r.icon.expand,rotateClass:"rotate-".concat(r.icon.rotate)},this.setTreeIcon(o.el,o.icon),n&&this.rotatePlusminus(o),(!(i=i||a.data("jsonData"))||i&&!i.icon)&&(o={el:r.el.tree,icon:r.icon.tree},this.setTreeIcon(o.el,o.icon))):(o={el:r.el.tree,icon:r.icon.tree},this.setTreeIcon(o.el,o.icon))}},rotatePlusminus:function(t){this.settings.useExpandTarget&&this.settings.expandPlusminusRotate&&(f.addClass(t.el,t.rotateClass),$(t.el).one("webkitAnimationEnd.tree oAnimationEnd.tree msAnimationEnd.tree animationend.tree",(function(){f.removeClass(t.el,t.rotateClass)})))},getBadgeHtml:function(t){var e={html:"",style:"",class:["badge","tree-badge"]};return t&&!t.remove&&(e.text="",void 0!==t.text&&(e.text=t.text.toString(),1===e.text.length&&e.class.push("round")),/info|good|error|alert|pending/i.test(t.type)?e.class.push(t.type):t.type&&"#"===t.type.charAt(0)&&7===t.type.length&&(e.style="background-color: ".concat(t.type," !important;")),t.backColor&&(e.style="background-color: ".concat(t.backColor," !important;")),t.foreColor&&(e.style+="color: ".concat(t.foreColor," !important;")),""!==e.style&&(e.style=' style="'.concat(e.style,'"')),""!==e.text&&(t.type&&-1!==t.type.indexOf("pending")&&(e.text=""),e.html='<span class="'.concat(e.class.join(" "),'"').concat(e.style,">").concat(e.text,"</span>"))),e.html},addToDataset:function(t,e){var i,n,s;if(t.parent&&(i=this.findById(t.parent)),e instanceof jQuery&&e.is("li")){var a=(n=this.findById($(e[0].parentNode.parentNode).find("a")[0].id)).children||n.node.data("jsonData").children;s=a.findIndex((function(t){return t.text.trim()===$(e).text().trim()})),a.splice(s,0,t)}return"bottom"!==e||t.parent||i||this.settings.dataset.push(t),"top"!==e||t.parent||i||this.settings.dataset.unshift(t),t.parent&&i&&(i.children||(i.children=[]),"bottom"===e?i.children.push(t):i.children.unshift(t)),!(t.parent&&!i)},findById:function(t,e){for(var i in e||(e=this.settings.dataset),e){var n,s,a,o,r=e[i];if(r.id===t)return r;if(r.children||null!==(n=r.node)&&void 0!==n&&null!==(s=null==(a=n)?void 0:a.data("jsonData"))&&void 0!==s&&(null==(o=s)?void 0:o.children)){var l,c,d,h,u=this.findById(t,r.children||(null===(l=r.node)||void 0===l||null===(c=null==(d=l)?void 0:d.data("jsonData"))||void 0===c||null==(h=c)?void 0:h.children));if(u)return u}}return null},getNodeByIdIfSelected:function(t,e){var i=this.findById(t,e);return i&&i.selected?i:null},getSelectedNodes:function(){var t,e,i=[];return $("li.is-selected",this.element).each((function(){t=$("a:first",this),e=t.data("jsonData"),i.push({node:t,data:e})})),i},getNextNode:function(t){var e=t.parent().next().find("a:first"),i=t.next();if(i.is("ul.is-open")&&(e=i.find("a:first")),e.hasClass("is-disabled")&&(e=e.parent().next().find("a:first")),0===e.length)for(var n=0,s=t;n<1e3&&!(e=(s=s.parent().closest(".folder")).next().find("a:first")).length;n++);return e.hasClass("is-disabled")&&(e=this.getNextNode(e)),e},getPreviousNode:function(t){var e=t.parent().prev().find("a:first"),i=e.parent();return i.is(".folder.is-open")&&i.find("ul.is-open a").length&&!i.find("ul.is-disabled").length&&(e=i.find("ul.is-open a:last")),e.hasClass("is-disabled")&&(i=(e=e.parent().prev().find("a:first")).parent()).is(".folder.is-open")&&i.find("ul.is-open a").length&&!i.find("ul.is-disabled").length&&(e=i.find("ul.is-open a:last")),0===e.length&&(e=t.closest("ul").prev("a")),e.hasClass("is-disabled")&&(e=this.getPreviousNode(e)),e},syncDataset:function(t){var e=[],i=this;t=t||this.element,[].slice.call(t.children("li").toArray()).forEach((function(t){var n=t.querySelector("a");n&&e.push(i.syncNode(n))})),this.settings.dataset=e,this.element.triggerHandler("rendered",{data:this.settings.dataset})},syncNode:function(t){var e,i=this,n=this.isjQuery(t)?t:$(t);if(t=n[0]){var s=t.parentNode,a=function(t,e){return f.hasClass(t,e)};e={node:n,id:t.getAttribute("id"),text:t.querySelector(".tree-text").textContent},(a(t,"is-open")||s&&"li"===s.tagName.toLowerCase()&&a(s,"is-open"))&&(e.open=!0);var o=t.getAttribute("href");o&&(e.href=o),a(s,"is-selected")&&(e.selected=!0),a(t,"is-disabled")&&(e.disabled=!0);var r=t.getAttribute("class");r&&r.indexOf("icon")>-1&&(e.icon=r.replace(/\s?hide-focus\s?/g,""));var l=n.next();if(l[0]&&"ul"===l[0].tagName.toLowerCase())e.children=[],[].slice.call(l.children("li").toArray()).forEach((function(t){var n=t.querySelector("a");n&&e.children.push(i.syncNode(n))}));var c=n.data("jsonData");c&&(delete c.selected,delete c.children,e=$.extend({},c,e));var d=E.parseSettings(n,"data-automation-attributes");d.attributes&&c&&(E.extend(c,d),this.addAutomationAttributes(n,c)),n.data("jsonData",e),n.data("".concat(bo,"Api"),i)}return e},addAutomationAttributes:function(t,e){if((t=this.isjQuery(t)?t:$(t)).length&&e.attributes){var i=t.find(".tree-text"),n=t.find(".icon-tree"),s=t.find(".icon-expand-target"),a=t.find(".tree-checkbox");E.addAttributes(t,this,e.attributes,"tree-link"),E.addAttributes(i,this,e.attributes,"tree-link-text"),E.addAttributes(n,this,e.attributes,"tree-icon"),E.addAttributes(s,this,e.attributes,"tree-icon-expand-target"),E.addAttributes(a,this,e.attributes,"tree-checkbox")}},addNode:function(t,e,i){var n,s,a,o,r=this,c="object"===l(t.badge)?JSON.stringify(t.badge):t.badge;t.href=void 0!==t.href?t.href:"#",e=e||"bottom";var d=document.createElement("a");void 0!==t.id&&d.setAttribute("id",t.id),d.setAttribute("href",t.href),void 0!==c&&d.setAttribute("data-badge",c),void 0!==t.alertIcon&&d.setAttribute("data-alert-icon",t.alertIcon),t.text&&(d.textContent=/&#?[^\s].{1,9};/g.test(t.text)?g.unescapeHTML(t.text):t.text),t.disabled&&f.addClass(d,"is-disabled"),t.icon&&f.addClass(d,t.icon);var h=document.createElement("li");if(h.setAttribute("role","none"),o=h,t.open&&f.addClass(h,"is-open"),h.appendChild(d),"dropdown"===t.type&&(d.setAttribute("style","display: none"),t.data)){s='<select class="dropdown" close-on-select="true">';for(var u=0;u<t.data.length;u++)(a=t.data[u]).value===t.text?(n=a.text,s+='<option value="'.concat(a.value,'" selected>').concat(a.text,"</option>")):s+='<option value="'.concat(a.value,'">').concat(a.text,"</option>");s+='</select><div class="dropdown-wrapper"><div class="dropdown"><span>'.concat(n),s+='</span></div><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-dropdown"></use></svg></div>',$('<div class="treeDropdown" style="width: 80px; margin-left: 35px; margin-bottom: -15px">'.concat(s,"</div>")).appendTo(h),t.disabled?$(h).find("select.dropdown").dropdown().disable():$(h).find("select.dropdown").dropdown().on("selected.tree",(function(){var t=r.findById(this.parentElement.previousElementSibling.id);t.text=this.value,r.updateNode(t)}))}var p,m=!!this.loading||this.addToDataset(t,e);(t.parent instanceof jQuery&&(m=!0),e)&&((p="string"!=typeof e||/^(top|bottom|before|after)$/g.test(e)?e instanceof jQuery?e:$(e):$(/\.|#/.test(e[0])?e:"#".concat(e)).first()).length&&("before"===i&&($(h).insertBefore(p.is("li")?p:p.closest("li")),m=!0),"after"===i&&($(h).insertAfter(p.is("li")?p:p.closest("li")),m=!0),"top"===i&&p.is("ul")&&(p.prepend(h),m=!0)));if(!(e instanceof jQuery)||t.parent&&m||t.parent instanceof jQuery||/^(top|bottom|before|after)$/g.test(i)||(e[0].appendChild(h),m=!0),"bottom"!==e||t.parent&&m||this.element[0].appendChild(h),"top"!==e||t.parent&&m||this.element.prepend(h),t.parent){var v="top"===e?e:null;m&&"string"==typeof t.parent&&(h=this.element.find("#".concat(t.parent)).parent(),!t.disabled&&h.is(".is-selected")&&void 0===t.selected&&(t.selected=!0),o=this.addAsChild(t,h,v)),t.parent&&t.parent instanceof jQuery&&(h=t.parent,t.parent.is("a")&&(h=t.parent.parent()),o=this.addAsChild(t,h,v)),this.isjQuery(h)&&(t.node=h.find("ul li a#".concat(t.id)))}else void 0!==t.tempidsparent&&(t.parent=t.tempidsparent,delete t.tempidsparent),h=$(h),this.addChildNodes(t,h),t.node=h.children("a").first();return d=$(d),this.decorateNode(d,t.icon,t.hideCheckbox,t),this.childrenCountInit(d.parentsUntil(this.element,".folder:last")),t.selected&&this.selectNode(d,t.focus),d.data("jsonData",t),d.data("".concat(bo,"Api"),r),this.addAutomationAttributes(d,t),this.createSortable(),$(o)},addAsChild:function(t,e,i){var n=(e=this.isjQuery(e)?e[0]:e).querySelector("ul"),s=!!n;return s||(e.insertAdjacentHTML("beforeend",'<ul class="folder" role="group"></ul>'),n=e.querySelector("ul")),t.open&&f.addClass(n,"is-open"),s||this.decorateNode(e.querySelector("a")),void 0!==t.parent&&(t.tempidsparent=t.parent),t.parent="",this.addNode(t,$(n),i)},addChildNodes:function(t,e){var i=this,n=(e=this.isjQuery(e)?e[0]:e).querySelector("ul");t.children?(n||(e.insertAdjacentHTML("beforeend",'<ul class="folder'.concat(t.open?" is-open":"",'" role="group"></ul>')),n=e.querySelector("ul")),n.innerHTML="",t.children&&t.children.forEach((function(t){return i.addNode(t,$(n))}))):n&&n.parentNode.removeChild(n)},isTrue:function(t){return null!=t&&("boolean"==typeof t&&!0===t||"string"==typeof t&&"true"===t.toLowerCase())},isFalse:function(t){return null!=t&&("boolean"==typeof t&&!1===t||"string"==typeof t&&"false"===t.toLowerCase())},updateNode:function(t){var e=t.node?{node:t.node}:this.findById(t.id);if(e&&e.node[0]){var i,n=e.node[0].parentNode,s=e.node[0].querySelector(".tree-text"),a=this.isTrue(t.disabled)||this.isFalse(t.enabled),o=this.isTrue(t.enabled)||this.isFalse(t.disabled),r=null===t.children;if(t.badge){var l=e.node[0].querySelector(".tree-badge");if(!l&&!t.badge.remove&&void 0!==t.badge.text&&""!==$.trim(t.badge.text)){var c=document.createElement("span");f.addClass(c,"tree-badge","badge"),s.parentNode.insertBefore(c,s),l=e.node[0].querySelector(".tree-badge")}l&&(void 0!==t.badge.text&&(t.badge.text=t.badge.text.toString(),l.textContent=t.badge.text,f.removeClass(l,"round"),1===t.badge.text.length&&f.addClass(l,"round")),void 0!==t.badge.type&&(f.removeClass(l,"info","good","error","alert","pending"),/info|good|error|alert|pending/i.test(t.badge.type)?f.addClass(l,t.badge.type):"#"===t.badge.type.charAt(0)&&7===t.badge.type.length&&(l.style.backgroundColor=t.badge.type),-1!==t.badge.type.indexOf("pending")&&(l.textContent="")),e.badge=t.badge,this.parseBool(t.badge.remove)&&(l.parentNode.removeChild(l),void 0!==e.badge&&delete e.badge))}if(t.text&&(s.textContent=t.text,e.text=t.text),null===t.icon){var d=e.node[0].nextElementSibling;d&&"ul"===d.tagName.toLowerCase()?this.setFolderIcon(e.node[0],f.hasClass(d,"is-open"),!1,this.settings.expandPlusminusRotate):this.setTreeIcon(e.node[0].querySelector("svg.icon-tree"),"icon-tree-node")}else if(t.icon){this.setTreeIcon(e.node[0].querySelector("svg.icon-tree"),t.icon),e.icon=t.icon;var h=e.node.data("jsonData");h&&(h.icon=t.icon,e.node.data("jsonData",h),e.node.data("".concat(bo,"Api"),this))}else t.children&&t.children.length&&!f.hasClass(n,"folder")&&this.convertFileToFolder(e.node);if(a)if(f.addClass(e.node[0],"is-disabled"),e.node[0].setAttribute("aria-disabled","true"),f.hasClass(n,"folder")&&f.hasClass(n,"is-open"))[].slice.call(n.querySelectorAll("a, ul[role=group]")).forEach((function(t){f.addClass(t,"is-disabled"),t.setAttribute("aria-disabled","true")}));if(o)if(!(e.node.parentsUntil(this.element,"ul[role=group].is-disabled").length>0))if(f.removeClass(e.node[0],"is-disabled"),e.node[0].removeAttribute("aria-disabled"),f.hasClass(n,"folder")&&f.hasClass(n,"is-open"))[].slice.call(n.querySelectorAll("a, ul[role=group]")).forEach((function(t){f.removeClass(t,"is-disabled"),t.removeAttribute("aria-disabled")}));if(t.childrenCount&&this.updateChildrenCount(e.node[0],t.childrenCount),t.node&&this.syncDataset(),t.children||r)null!==(i=t.children)&&void 0!==i&&(null==i?void 0:i.length)?this.addChildNodes(t,n):this.removeChildren(t,n);this.syncDataset(),this.createSortable()}},parseBool:function(t){return!/^(false|0)$/i.test(t)&&!!t},removeChildren:function(t,e){var i=(e=this.isjQuery(e)?e[0]:e).querySelector("ul");this.setTreeIcon(e.querySelector("svg.icon-tree"),t.icon||"icon-tree-node"),f.removeClass(e,"folder","is-open"),i&&i.parentNode.removeChild(i)},removeNode:function(t){var e=this.findById(t.id),i=null;t instanceof jQuery?(i=(e=t).parentsUntil(this.element,".folder:last"),e.parent().remove()):e&&(i=e.node.parentsUntil(this.element,".folder:last"),e.node.parent().remove()),this.childrenCountInit(i),e&&this.syncDataset()},attachMenu:function(t){var e=this;function i(t,e){void 0===$(":focus")[0]&&null!=e&&(null==e?void 0:e.triggerElement)&&e.triggerElement.focus()}t&&this.element.off("contextmenu.tree").on("contextmenu.tree","a",(function(n){var s=$(this);return n.preventDefault(),e.popupEl=$(n.currentTarget).popupmenu({menuId:t,eventObj:n,trigger:"immediate",attachToBody:!0,returnFocus:i}).off("selected").on("selected",(function(t,i){e.element.triggerHandler("menuselect",{node:s,item:i})})),e.element.triggerHandler("menuopen",{menu:$("#".concat(t)),node:s}),!1}))},createSortable:function(){if(this.settings.sortable){var t,e,i,n=this;if(n.targetArrow=n.element[0].previousElementSibling,n.linkSelector="a:not(.is-dragging-clone):not(.is-disabled)",!n.targetArrow||n.targetArrow&&!f.hasClass(n.targetArrow,"tree-drag-target-arrow")){var s=document.createElement("div");f.addClass(s,"tree-drag-target-arrow"),n.element[0].parentNode.insertBefore(s,n.element[0]),n.targetArrow=n.element[0].previousElementSibling}e=setInterval((function(){n.loading||(clearInterval(e),[].slice.call(n.element[0].querySelectorAll(n.linkSelector)).forEach((function(e){var s=$(e);s.data("drag")||s.on("mousedown.tree",(function(t){t.preventDefault(),i=3!==t.which&&(!f.hasClass(t.target,"icon")||!f.hasClass(e.parentNode,"folder"))})).drag({clone:!0,cloneAppendTo:s.closest("li"),clonePosIsFixed:!0}).on("dragstart.tree",(function(a,o,r){if(!r||!i)return f.removeClass(e,"is-dragging"),void(r&&r[0].parentNode.removeChild(r[0]));(t=r)[0].removeAttribute("id"),f.addClass(t[0],"is-dragging-clone"),[].slice.call(t[0].querySelectorAll(".tree-checkbox, .tree-badge")).forEach((function(t){return t.parentNode.removeChild(t)}));var l=s.closest("ul");n.sortable={startIndex:$(n.linkSelector,n.element).index(s),startNode:s,startIcon:$("svg.icon-tree",s).getIconName(),startUl:l,startLi:s.closest("li"),startFolderNode:l.prev("a"),startWidth:s.outerWidth()},n.element.triggerHandler("sortstart",n.sortable),a.preventDefault(),a.stopImmediatePropagation()})).on("drag.tree",(function(e,i){t&&(t[0].style.left="".concat(i.left,"px"),t[0].style.top="".concat(i.top,"px"),t[0].style.opacity="1",n.setDragOver(t,i))})).on("dragend.tree",(function(e,i){var s;if(n.targetArrow.style.display="none",[].slice.call(n.element[0].querySelectorAll(n.linkSelector)).forEach((function(t){return f.removeClass(t,"is-over")})),t&&null!==(s=n.sortable)&&void 0!==s&&(null==s?void 0:s.overDirection)){t[0].style.left="".concat(i.left,"px"),t[0].style.top="".concat(i.top,"px");var a=n.sortable.startNode.parent(),o=n.sortable.overNode.parent();"over"===n.sortable.overDirection?(f.hasClass(o[0],"folder")||n.convertFileToFolder(n.sortable.overNode),o[0].querySelector("ul").appendChild(a[0]),f.hasClass(o[0],"is-open")||n.toggleNode(n.sortable.overNode,e)):"up"===n.sortable.overDirection?a.insertBefore(o):"down"===n.sortable.overDirection&&(f.hasClass(o[0],"is-open")&&f.hasClass(o[0],"folder")?$("ul:first",o).prepend(a):a.insertAfter(o)),!n.sortable.startUl[0].querySelector("li")&&n.sortable.startFolderNode.data("oldData")&&"file"===n.sortable.startFolderNode.data("oldData").type&&n.convertFolderToFile(n.sortable.startFolderNode),E.fixSVGIcons(a),n.element.triggerHandler("sortend",n.sortable),n.syncDataset(),n.isMultiselect&&n.initSelected(),n.updateAutoChildrenCount(n.element[0]),delete n.sortable}}))})))}),10)}},setDragOver:function(t,e){var i,n,s,a,o,r,l,c,d,h,u,p,g=this,m=t[0].querySelector("svg.icon-tree"),v=g.element[0].getBoundingClientRect(),b=20;if(g.sortable){var y=function(){g.sortable.overNode=null,g.sortable.overIndex=null,g.sortable.overDirection=null,g.targetArrow.style.display="none",g.setTreeIcon(m,"icon-cancel")};e.top>v.top-b&&e.top<v.bottom+b&&e.left>v.left-b-g.sortable.startWidth&&e.left<v.left+v.height+b?(b=2,(l=[].slice.call(g.element[0].querySelectorAll(g.linkSelector))).forEach((function(t,v){if(u=null,c=t.getBoundingClientRect(),e.top>c.top-b&&e.top<c.bottom+b){if(o=$(t),!!o.parentsUntil(g.element,".folder").filter((function(){return $("a:first",this).is(g.sortable.startNode)&&g.sortable.startLi.is(".folder")})).length)return void y();a=t.parentNode,d=c.left,r=o.closest("ul"),i=parseInt(a.style.marginTop,10)>0?2:0,n=v-1===g.sortable.startIndex&&r.is(g.sortable.startUl),s=v+1===g.sortable.startIndex&&r.is(g.sortable.startUl),l.forEach((function(t){return f.removeClass(t,"is-over")})),p=function(){u?(g.setTreeIcon(m,g.sortable.startIcon),"over"===u?(g.targetArrow.style.display="none",f.hasClass(t,"is-disabled")||f.addClass(t,"is-over")):(l.forEach((function(t){return f.removeClass(t,"is-over")})),h="up"===u?c.top-1.5-(f.hasClass(a,"is-active")?3:0):c.bottom+(a.nextElementSibling&&f.hasClass(a.nextElementSibling,"is-active")?-1:1.5)+i,g.targetArrow.style.left="".concat(d,"px"),g.targetArrow.style.top="".concat(h,"px"),g.targetArrow.style.display="block"),g.sortable.overNode=o,g.sortable.overIndex=v,g.sortable.overDirection=u):y()},v!==g.sortable.startIndex&&(e.left>c.left-b-g.sortable.startWidth&&e.left<c.right+b?u=!n&&e.top<c.top?"up":!s&&e.top>c.top+2*b?"down":"over":!n&&e.top<c.top?u="up":s||(u="down")),p()}}))):y()}},convertFileToFolder:function(t){var e=$("svg.icon-tree",t),i=document.createElement("ul");i.setAttribute("role","group"),f.addClass(i,"folder");var n={icon:e.getIconName(),type:"file"};if(this.hasIconClass(t)){var s=t.attr("class").replace(/\s?is-selected/,"");n.iconClass=s,this.settings.useExpandTarget||t.removeClass(s)}t.data("oldData",n);var a=t[0].parentNode;a&&"li"===a.tagName.toLowerCase()&&(f.addClass(a,"folder"),a.appendChild(i)),e[0].insertAdjacentHTML("afterend",this.getExpandTargetHtml(!1)),t[0].insertAdjacentHTML("beforeend",this.getChildrenCountHtml()),this.setTreeIcon(e,this.settings.folderIconClosed)},convertFolderToFile:function(t){var e=t.parent(".folder");e.removeClass("folder is-open"),$(".icon-expand-target, .tree-children-count",t).remove(),$("ul:first",e).remove(),e.length&&(this.setTreeIcon($("svg.icon-tree",t),t.data("oldData")?t.data("oldData").icon:"tree-node"),t.data("oldData")&&t.data("oldData").iconClass&&t.addClass(t.data("oldData").iconClass),t.data("oldData",null))},unbind:function(){return this.settings.sortable&&(this.element.find("a").each((function(){var t=$(this),e=t.data("drag");t.off("mousedown.tree"),e&&e.destroy&&e.destroy()})),this.element.prev(".tree-drag-target-arrow").remove()),this.element.off("contextmenu.tree updated.tree click.tree focus.tree keydown.tree keypress.tree"),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,yo)),this.unbind().init()},destroy:function(){this.unbind(),this.element.empty(),$.removeData(this.element[0],bo)},disable:function(){[].slice.call(this.element[0].querySelectorAll("a")).forEach((function(t){f.addClass(t,"is-disabled"),t.setAttribute("aria-disabled","true")}))},enable:function(){[].slice.call(this.element[0].querySelectorAll("a")).forEach((function(t){f.removeClass(t,"is-disabled"),t.removeAttribute("aria-disabled")}))},preserveEnablementState:function(){var t=[].slice.call(this.element[0].querySelectorAll("a")),e=[];return t.forEach((function(t){f.hasClass(t,"is-disabled")||!0===t.getAttribute("aria-disabled")?e.push({nodeId:t.id,state:"disabled"}):e.push({nodeId:t.id,state:"enabled"})})),this.settings.originalEnablementState=e,e},restoreEnablementState:function(){var t=this.settings,e=[].slice.call(this.element[0].querySelectorAll("a"));void 0!==t.originalEnablementState&&null!==t.originalEnablementState&&e.forEach((function(e){t.originalEnablementState.forEach((function(t){t.nodeId===e.id&&("disabled"===t.state?(f.addClass(e,"is-disabled"),e.setAttribute("aria-disabled","true")):(f.removeClass(e,"is-disabled"),e.removeAttribute("aria-disabled")))}))}))}},$.fn.tree=function(t){var e=this.find("a");return e.find("[onerror]").length&&(this.find("[onerror]").removeAttr("onerror"),e.find("*").each((function(){var t=$(this);this.outerHTML=g.sanitizeHTML(t.prop("outerHTML"))}))),this.each((function(){var e=$.data(this,bo);e?e.updated(t):e=$.data(this,bo,new wo(this,t))}))},$.fn.collapse=function(){return this.each((function(){xo(this,"collapse")}))},$.fn.expand=function(){return this.each((function(){xo(this,"expand")}))};var Co="treemap",ko={dataset:[],redrawOnResize:!0,margin:{top:20,right:20,bottom:20,left:20},colors:null,showLabel:!0,labelFormatter:".0%",showTitle:!0,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new"}};function To(t,e){this.settings=E.mergeSettings(t,e,ko),e&&e.dataset&&(this.settings.dataset=e.dataset),this.element=$(t),this.init()}To.prototype={init:function(){return this.build().handleEvents(),this.element.trigger("rendered",[this.svg]),this},build:function(){return this.setupColors(),this.updateData(this.settings.dataset),this},setupColors:function(){if(!this.settings.colors||this.useBuiltInColors){var t=ft.themeColors().palette;this.settings.colors=[t.azure[100].value,t.azure[90].value,t.azure[80].value,t.azure[70].value,t.azure[60].value,t.azure[40].value,t.azure[30].value,t.azure[20].value,t.amethyst[100].value,t.amethyst[90].value,t.amethyst[80].value,t.amethyst[70].value,t.amethyst[60].value,t.amethyst[50].value,t.amethyst[40].value,t.amethyst[40].value,t.amethyst[30].value,t.amethyst[20].value,t.turquoise[100].value,t.turquoise[90].value,t.turquoise[80].value,t.turquoise[70].value,t.turquoise[60].value,t.turquoise[50].value,t.turquoise[40].value,t.turquoise[40].value,t.turquoise[30].value,t.turquoise[20].value],this.useBuiltInColors=!0}},updateData:function(t){var e=this,i=Object.create(this.settings.margin),n=this.element.parent().width()-i.left-i.right,s=this.element.parent().height()-i.top-i.bottom,a=d3.scaleOrdinal().range(this.settings.colors);if(0!==t.length){E.addAttributes(this.element,t,this.settings.attributes,"",!0),this.settings.showTitle&&t.name&&(d3.select(this.element[0]).append("div").attr("class","chart-treemap-title").text(t.name),s-=35,i.top-=19);var o=d3.treemap().size([n,s]);this.root=d3.select(this.element[0]).classed("chart-treemap",!0).append("div").style("position","relative").style("width","".concat(n+i.left+i.right,"px")).style("height","".concat(s+i.top+i.bottom,"px")).style("left","".concat(i.left,"px")).style("top","".concat(i.top,"px"));var r=d3.hierarchy(t,(function(t){return t.children})).sum((function(t){return t.value})),l=o(r);this.total=d3.sum(l.leaves(),(function(t){return t.value})),bi.appendTooltip(),this.root.datum(r).selectAll(".chart-treemap-node").data(l.leaves()).enter().append("div").attr("class","chart-treemap-node").style("left",(function(t){return"".concat(t.x0,"px")})).style("top",(function(t){return"".concat(t.y0,"px")})).style("width",(function(t){return"".concat(Math.max(0,t.x1-t.x0-1),"px")})).style("height",(function(t){return"".concat(Math.max(0,t.y1-t.y0-1),"px")})).style("background",(function(t){return t&&t.parent&&t.parent.data?a(t.parent.data.name):""})).call((function(t){t._groups.forEach((function(t){t.forEach((function(t){var i,n=t.__data__;E.addAttributes($(t),n,e.settings.attributes,null===(i=n.data.name)||void 0===i||null==i?void 0:i.toLowerCase(),!0)}))}))})).on("mouseenter.treemap",(function(t,e){var i=this.getBoundingClientRect(),n=this.querySelector(".chart-treemap-percent").innerText,s=e.parent.data.tooltip;if(!(!s&&i.width>60&&i.height>50)){s=s?(s=s.replace("%percent%",n)).replace("{{percent}}",n):"<p><b>".concat(e.data.name," </b> ").concat(n,"</p>");var a=bi.tooltipSize(s),o=i.left-a.width/2+12,r=i.top-a.height-6;bi.showTooltip(o,r,s,"top")}})).on("mouseleave.treemap",(function(){clearInterval(undefined),bi.hideTooltip()})),this.root.selectAll(".chart-treemap-node").append("span").attr("class","chart-treemap-text").text((function(t){return t.data.name})),this.settings.showLabel&&this.root.selectAll(".chart-treemap-node").append("span").attr("class","chart-treemap-percent").text((function(t){return d3.format(e.settings.labelFormatter)(t.value/e.total)}))}else this.element.emptymessage(this.settings.emptyMessage)},handleEvents:function(){var t=this;return this.element.on("updated.".concat(Co),(function(){t.updated()})),this.settings.redrawOnResize&&($("body").on("resize.".concat(Co),(function(){t.handleResize()})),this.element.on("resize.".concat(Co),(function(){t.handleResize()}))),$("html").on("themechanged.".concat(Co),(function(){t.updated()})),this},handleResize:function(){this.width!==this.element.width()&&(this.width=this.element.width(),this.element.is(":visible")&&this.updated())},updated:function(t){return this.settings=E.mergeSettings(this.element,t,this.settings),t&&t.dataset&&(this.settings.dataset=t.dataset),this.setupColors(),this.element.empty(),this.build()},teardown:function(){return this.element.off("updated.".concat(Co)),$("body").off("resize.".concat(Co)),$("html").off("themechanged.".concat(Co)),this},destroy:function(){this.element.empty().removeClass("chart-treemap"),bi.removeTooltip(),this.teardown(),$.removeData(this.element[0],Co)}},$.fn.treemap=function(t){return this.each((function(){var e=$.data(this,Co);e?e.updated(t):e=$.data(this,Co,new To(this,t))}))},$.fn.virtualscroll=function(t){return this.each((function(){var e=$.data(this,en);e?e.updated(t):e=$.data(this,en,new sn(this,t))}))},$.fn.weekview=function(t){return this.each((function(){var e=$.data(this,$n);e?e.updated(t):e=$.data(this,$n,new Dn(this,t))}))};var Ao="wizard",So={ticks:null,ticksTemplate:'<a class="tick {{state}}" href="{{href}}"><span class="label" data-shortlabel="{{shortLabel}}">{{label}}</span></a>'};function $o(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,So),this.init()}function Eo(t,e,i,n,s,a){var o=s.placeholder;if(o&&""===t){return"[object Function]"==={}.toString.call(o)?o=o(e,i,n,s,a):a&&a instanceof Object&&o in a&&(o=a[o]),o}return""}function Do(t,e,i,n,s){var a,o=n?n.disabled:null;return"function"==typeof o?o(t,e,i,n,s):null!=(a=o)&&("boolean"==typeof a&&!0===a||"string"==typeof a&&"true"===a.toLowerCase())}$o.prototype={init:function(){this.namespace=E.uniqueId({classList:[Ao]}),this.build().handleEvents()},build:function(){return this.header=this.element.find(".wizard-header"),this.header.length||(this.header=$('<div class="wizard-header"></div>').appendTo(this.element)),this.bar=this.element.find(".bar"),this.bar.length||(this.bar=$('<div class="bar"></div>').appendTo(this.header)),this.completedRange=this.element.find(".completed-range"),this.completedRange.length||(this.completedRange=$('<div class="completed-range"></div>').appendTo(this.bar)),this.buildTicks().updateRange(),this},buildTicks:function(){var t=this.settings;if(this.ticks=this.bar.children(".tick"),!this.ticks.length&&t.ticks){for(var e=function(t,e){return t.replace(/{{(\w+)}}/g,(function(t,i){return e[i]}))},i={state:"",href:"#",label:""},n=0;n<t.ticks.length;n++){var s=e(t.ticksTemplate,$.extend({},i,t.ticks[n]));s=s.replace(/(\s(\w|-)+=("|')?)?undefined("|')?/gim,"");var a=$(s);t.ticks[n].ngClick&&a.attr("ng-click",t.ticks[n].ngClick),this.bar.append(a)}this.ticks=this.bar.children(".tick")}return this.positionTicks(),this.setLabelsData(),this.adjustLabels(),this.ticks.toArray().forEach((function(t){t.classList.contains("is-disabled")&&["onclick","ng-click"].forEach((function(e){return t.removeAttribute(e)}))})),this.header[0].style.opacity="1",this},setLabelsData:function(){var t=this,e=[].slice.call(this.bar[0].querySelectorAll(".label"));return this.labels=[],e.forEach((function(e){var i=e.textContent,n=e.getAttribute("data-shortlabel");n||(n=e.getAttribute("shortlabel"));var s={label:i,shortLabel:n,node:e,jqEl:$(e)};n&&"undefined"!==n||(e.removeAttribute("data-shortlabel"),e.removeAttribute("shortlabel"),delete s.shortLabel),t.labels.push(s)})),this},adjustLabels:function(){var t=this,e=this.header[0].getBoundingClientRect(),i=parseFloat(window.getComputedStyle(this.bar[0]).width),n=100/(this.labels.length-1)/100*i-15,s=[],a=[],o=function(e){delete e.isEllipsis,e.node.classList.remove("is-ellipsis"),e.node.style.width="",t.removeTooltip(e)},r=function(e){var i=t.labels[e].node;if(-1===s.indexOf(e)){var a=t.labels[e].shortLabel;a?(i.textContent=a,s.push(e),o(t.labels[e])):(i.classList.add("is-ellipsis"),i.style.width="".concat(n,"px"),t.setTooltip(t.labels[e]),t.labels[e].isEllipsis=!0)}i.style.left="-".concat((i.offsetWidth-i.parentNode.offsetWidth)/2,"px")},l=function(){var i=function(t,e,i){if(t.isEllipsis){var n=parseFloat(t.node.style.width)-e;t.node.style.width="".concat(n,"px"),t.node.style.left="-".concat((n-t.node.parentNode.offsetWidth)/2,"px")}else{var s=parseFloat(t.node.style.left)+e*(i?-1:1);t.node.style.left="".concat(s,"px")}},n=t.labels[0],s=n.node.rect||n.node.getBoundingClientRect();e.left>s.left&&i(n,e.left-s.left),s=(n=t.labels[t.labels.length-1]).node.rect||n.node.getBoundingClientRect(),e.right<s.right&&i(n,s.right-e.right,!0)};this.labels.forEach((function(t){var e=t.node;e.textContent=t.label,e.style.left="-".concat((e.offsetWidth-e.parentNode.offsetWidth)/2,"px"),o(t)})),l();var c=function(t,e){return!(t.right<e.left||t.left>e.right||t.bottom<e.top||t.top>e.bottom)},d=function(){t.labels.forEach((function(t){t.rect=t.node.getBoundingClientRect()})),a=[];for(var e=0,i=t.labels.length;e<i;e++)if(e<i-1){var n=e+1;c(t.labels[e].rect,t.labels[n].rect)&&a.push([e,n])}},h=50;do{d(),a.forEach((function(e){var i=t.labels[e[0]].node,n=t.labels[e[1]].node,s=i.offsetWidth>n.offsetWidth?0:1,a=e[s],o=s?e[0]:e[1];r(a),c(t.labels[a].rect,t.labels[o].rect)&&r(o)})),l(),d(),h--}while(a.length&&h>0);return l(),this.labels.forEach((function(t){delete t.isEllipsis,delete t.rect})),this},setTooltip:function(t){var e=this;t.jqEl.tooltip({content:t.label,placement:"bottom"}).on("blur.".concat(Ao),(function(){return e.removeTooltip(t)}))},removeTooltip:function(t){var e=t.jqEl.data("tooltip");e&&(e.element.off("blur.".concat(Ao)),e.destroy())},positionTicks:function(){var t=this.ticks.length,e=100/(t-1);this.ticks.toArray().forEach((function(i,n){var s=function(i){return i===t-1?100:e*i}(n),a=H&&H.isRTL()?100-s:s;i.style.left="".concat(a,"%"),i.classList.contains("is-disabled")&&i.setAttribute("tabindex","-1")}))},updateRange:function(){var t=function(t,e){return parseFloat(window.getComputedStyle(t)[e])},e=this.ticks.filter(".current").last(),i=0;return e.length&&(i=100*t(e[0],"left")/t(e.parent()[0],"width"),i=H&&H.isRTL()?100-i:i),this.completedRange[0].style.width="".concat(i,"%"),this},updated:function(t){return t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.buildTicks().updateRange(),this},teardown:function(){var t=this;return this.labels.forEach((function(e){t.removeTooltip(e),e.jqEl.off().removeData()})),delete this.labels,this.ticks.off("click.".concat(Ao)),this.element.off("updated.".concat(Ao)),$("body").off("resize.".concat(this.namespace)),this.ticks.remove(),this},select:function(t,e){return j(this.activate,this.select).apply(this,[t,e])},activate:function(t,e){if(void 0===t&&!e)return this;var i,n=this;if(void 0===t||null!=t&&t.type&&t.target||e||(e=t),e=void 0===e?(i=$(t.target)).is(".label")?i.parent():i:"number"==typeof e?n.ticks.eq(e):e,t&&(e.is("[disabled], .is-disabled")||!e.is("a")))return t.preventDefault(),t.stopPropagation(),this;if(!1===this.element.triggerHandler("beforeactivate",[e]))return this;var s=this.ticks.index(e);return this.ticks.removeClass("complete current").eq(s).addClass("current").prevAll(".tick").addClass("complete"),this.updateRange(),this.element.trigger("activated",[e]),setTimeout((function(){n.element.trigger("afteractivated",[e])}),300),this},destroy:function(){this.teardown(),$.removeData(this.element[0],Ao)},handleEvents:function(){var t=this,e=this;return this.element.on("updated.".concat(Ao),(function(){e.updated()})),this.ticks.on("click.".concat(Ao),(function(t){e.activate(t,$(this))})),$("body").on("resize.".concat(this.namespace),(function(){t.adjustLabels()})),this}},$.fn.wizard=function(t){return this.each((function(){var e=$.data(this,Ao);e?e.updated(t):e=$.data(this,Ao,new $o(this,t))}))};var Io={Text:function(t,e,i){return null==i||""===i?"":i.toString()},Input:function(t,e,i,n){return n.inlineEditor?'<label for="datagrid-inline-input-'.concat(t,"-").concat(e,'" class="audible">').concat(n.name,'</label><input id="datagrid-inline-input-').concat(t,"-").concat(e,'" class="').concat("right"===n.align?"is-number-mask":"",'" value="').concat(i,'">'):null==i||""===i?"":i.toString()},Placeholder:function(t,e,i,n,s){var a=Eo(i,t,e,i,n,s);return""!==a?'<span class="is-placeholder">'.concat(a,"</span>"):null==i||""===i?"":i.toString()},Ellipsis:function(t,e,i,n){var s=null==i||""===i?"":i.toString();return n.textOverflow="ellipsis",s},Password:function(t,e,i){return(null==i||""===i?"":i.toString()).replace(/./g,"*")},Readonly:function(t,e,i){return'<span class="is-readonly">'.concat(null==i?"":i,"</span>")},RowNumber:function(t,e,i,n,s,a){var o,r=a.runningCount||1;return(null==a||null===(o=null==a?void 0:a.pagerAPI)||void 0===o||null==o?void 0:o.activePage)>1&&(r+=(a.pagerAPI.activePage-1)*a.pagerAPI.settings.pagesize),'<span class="is-readonly">'.concat(r,"</span>")},Date:function(t,e,i,n,s,a){var o,r=null==i?"":i,l=!1,c=arguments.length>6&&void 0!==arguments[6]&&arguments[6]?null:n.dateFormat,d=s?Eo(r,t,e,i,n,s):"";return""!==d&&(l=!0),"string"==typeof i&&i?(o=n.sourceFormat?H.parseDate(i,"string"==typeof n.sourceFormat?{pattern:n.sourceFormat}:n.sourceFormat):H.parseDate(i,"string"==typeof c?{pattern:c}:c))?r=H.formatDate(o,"string"==typeof c?{pattern:c}:c):"NaN/NaN/NaN"!==(r=H.formatDate(i,"string"==typeof c?{pattern:c}:c))&&r||""===r||(r=i):i&&(r=H.formatDate(i,"string"==typeof c?{pattern:c}:c)),n.editor&&!0!==s?(null!=r&&""!==r||(r="",d&&(l=!0,r=d)),'<span class="trigger '.concat(l?"is-placeholder":"",'">').concat(r,"</span>").concat($.createIcon({icon:"calendar",classes:["icon-calendar"]}))):l?"":r},Time:function(t,e,i,n,s){var a,o=null==i?"":i,r="object"===l(H)&&H.calendar().dateFormat?H.calendar().dateFormat.short:null,c="object"===l(H)&&H.calendar().timeFormat?H.calendar().timeFormat:null;return"string"==typeof i&&i?(a=H.formatDate(function(t){if(""===t)return null;var e=t.match(/(\d+)(?::(\d\d))(?::(\d\d))?\s*([pP]?)/i);if(null===e)return null;var i=function(t){return isNaN(parseInt(t,10))?0:parseInt(t,10)},n=H.currentLocale.data.calendars[0].dayPeriods,s=new Date,a=i(e[1]);return(t.indexOf(n[0])>-1||t.indexOf(n[1])>-1)&&(a-=12===a?12:0,a+=t.indexOf(n[1])>-1?12:0),s.setHours(a),s.setMinutes(i(e[2])),s.setSeconds(i(e[3])),s}(i),{pattern:"".concat(r," ").concat(n.sourceFormat||n.timeFormat||c)}))&&(o=a.slice(a.indexOf(" "))):i&&(a=H.formatDate(i,{pattern:"".concat(r," ").concat(n.sourceFormat||n.timeFormat||c)}))&&(o=a.slice(a.indexOf(" "))),o=o.replace(/^\s/,""),n.editor&&!0!==s?'<span class="trigger">'.concat(o,"</span>").concat($.createIcon({icon:"clock",classes:["icon-clock"]})):o},Autocomplete:function(t,e,i){return null==i?"":i},Lookup:function(t,e,i,n,s){var a=null==i?"":i,o=!1,r=Eo(a,t,e,i,n,s);return""!==r&&(o=!0),n.editor?(n.editorOptions&&"function"==typeof n.editorOptions.field&&(a=n.editorOptions.field(s,null,null),o=!1),null!=a&&""!==a||(a="",r&&(o=!0,a=r)),'<span class="trigger '.concat(o?"is-placeholder":"").concat("right"===n.align?"align-text-right":"",'">').concat(a,"</span>").concat($.createIcon({icon:"search-list",classes:["icon-search-list"]}))):o?'<span class="is-placeholder">'.concat(r,"</span>"):a},Decimal:function(t,e,i,n,s){var a=i,o=/[\s]*($)?[#,0-9\s\.]+%?[\s]*/;void 0!==H&&null!=a&&""!==a&&o.test(a)&&(a=H.formatNumber(i,n.numberFormat));var r=Eo(a=null!=a&&"NaN"!==a&&o.test(a)?a:"",t,e,i,n,s);return""!==r?'<span class="is-placeholder">'.concat(r,"</span>"):a},Integer:function(t,e,i,n){var s=i;return void 0!==H&&null!=s&&""!==s&&(s=H.formatNumber(i,n.numberFormat||{style:"integer"})),null==s||"NaN"===s?"":s},Hyperlink:function(t,e,i,n,s,a){var o=Do(t,e,i,n,s)?" disabled":"",r=n.href||"#";if(n.href&&"function"==typeof n.href){if(null==(r=n.href(t,e,s,n)))return n.text||i}else r=r.replace("{{value}}",i);var l=n.text||i;return l||n.icon?n.icon?'<a href="'.concat(r,'"').concat(o,' class="btn-icon row-btn ').concat(n.cssClass||"",'" ').concat(a.settings.rowNavigation?'tabindex="-1"':"").concat(n.hyperlinkTooltip?' title="'.concat(n.hyperlinkTooltip,'"'):"",">\n          ").concat($.createIcon({icon:n.icon,file:n.iconFile}),'\n          <span class="audible">').concat(l,"</span>\n        </a>"):'<a href="'.concat(r,'"').concat(o," ").concat(a.settings.rowNavigation?'tabindex="-1"':"",' class="hyperlink ').concat(n.cssClass||"",'"').concat(n.target?' target="'.concat(n.target,'"'):"").concat(n.hyperlinkTooltip?' title="'.concat(n.hyperlinkTooltip,'"'):"",">").concat(l,"</a>"):""},Template:function(t,e,i,n,s){var a=n.template,o="";return je&&s&&a&&(o=je.compile("{{#dataset}}".concat(a,"{{/dataset}}"),{dataset:s})),o},Drilldown:function(){var t=H.translate("Drilldown");return void 0===t&&(t=""),'<button type="button" tabindex="-1" class="btn-icon small datagrid-drilldown">\n         '.concat($.createIcon({icon:"drilldown"}),"\n        <span>").concat(t,"</span>\n      </button>")},RowReorder:function(){var t=H.translate("ReorderRows");return void 0===t&&(t="Reorder Rows"),'<div class="datagrid-reorder-icon draggable">\n         '.concat($.createIcon({icon:"drag"}),'\n        <span class="audible">').concat(t,"</span>\n      </div>")},Checkbox:function(t,e,i,n,s,a){var o;o=n.isChecked?n.isChecked(i):void 0!==i&&(!0===i||1===parseInt(i,10));var r='<span class="hidden" aria-hidden="true" role="presentation"> '.concat(H.translate(o?"Yes":"No"),"</span>"),l=a.wasJustUpdated;return a.wasJustUpdated=!1,'<div class="datagrid-checkbox-wrapper"><span role="checkbox" class="datagrid-checkbox'.concat(o?" is-checked ".concat(l?" ":" no-animation"):"",'"></span>').concat(r,"</div>")},SelectionCheckbox:function(t,e,i,n,s,a){var o=void 0!==i&&!0===i;i||(o=a.isRowSelected(s));var r=" ";if(a.settings.columnIds.length>0)for(var l=0;l<a.settings.columnIds.length;l++)r+=s[a.settings.columnIds[l]];return r=g.ensureAlphaNumericWithSpaces(r),'<div class="datagrid-checkbox-wrapper"><span role="checkbox" aria-label="'.concat(n.name?n.name:H.translate("Select")+r,'" class="datagrid-checkbox datagrid-selection-checkbox').concat(o?" is-checked no-animate":"",'"></span></div>')},SelectionRadio:function(t,e,i,n,s,a){var o=void 0!==i&&!0===i;i||(o=a.isRowSelected(s));var r=" ";if(a.settings.columnIds.length>0)for(var l=0;l<a.settings.columnIds.length;l++)r+=s[a.settings.columnIds[l]];return r=g.ensureAlphaNumericWithSpaces(r),'<div class="datagrid-checkbox-wrapper"><span role="radio" aria-label="'.concat(n.name?n.name:H.translate("Select")+r,'" class="datagrid-radio datagrid-selection-radio ').concat(o?" is-checked":"",'"></span></div>')},Actions:function(t,e,i,n){return'<button type="button" class="btn-actions" aria-haspopup="true" aria-expanded="false" aria-owns="'.concat(n.menuId,'">\n        <span class="audible">').concat(n.title||H.translate("More"),"</span>\n        ").concat($.createIcon({icon:"more"}),"\n      </button>")},Textarea:function(t,e,i){return'<span class="datagrid-multiline-text">'.concat(null==i?"":i,"</span>")},Editor:function(t,e,i,n){var s=null==i?"":i,a="is-editor";return a+=n.singleline?" is-singleline":" datagrid-multiline-text",a+=n.contentTooltip?" content-tooltip":"",'<div class="'.concat(a,'">').concat(g.unescapeHTML(s),"</div>")},Expander:function(t,e,i,n,s,a){var o=E.stringAttributes(a,a.settings.attributes,"btn-expand-row-".concat(t));return"<button ".concat(o,' type="button" aria-label="').concat(H.translate("ExpandCollapse"),'" ').concat(s.isSummaryRow?"disabled=true":"",' class="btn-icon datagrid-expand-btn ').concat(s.expanded?"is-expanded":"",'" tabindex="-1">\n      <span class="icon plus-minus ').concat(s.expanded?"active":"",'"></span>\n      </button>').concat(i?"<span> ".concat(i,"</span>"):"")},GroupRow:function(t,e,i,n,s,a){var o,r=a.settings.groupable,l={left:"",center:"",right:""},c=null!=a&&null!==(o=null==a?void 0:a.settings)&&void 0!==o&&(null==o?void 0:o.rowReorder)?this.RowReorder():"",d="",h=void 0===r.expanded||r.expanded;r.expanded&&"function"==typeof r.expanded&&(h=r.expanded(t,e,i,n,s,a));for(var u=0;u<r.fields.length;u++)d+=s[r.fields[u]]+(0===u?"":",");r.groupRowFormatter&&(d=r.groupRowFormatter(t,e,i,n,s,a));var p='<button type="button" class="btn-icon datagrid-expand-btn'.concat(h?" is-expanded":"",'" tabindex="-1">\n    <span class="icon plus-minus').concat(h?" active":"",'"></span>\n    <span class="audible">').concat(H.translate("ExpandCollapse"),"</span>\n    </button>").concat(c,"<span> ").concat(d,"</span>");return l[a.getContainer(r.fields?r.fields[0]:"")]=p,l},GroupFooterRow:function(t,e,i,n,s,a){var o=a.settings.groupable,r={left:"",center:"",right:""},l=a.settings.frozenColumns.left.length,c=a.settings.frozenColumns.right.length,d=a.columnIdxById(o.aggregate),h=a.getContainer(o.aggregate);return r.left='<td role="gridcell" colspan="'.concat(l,'"><div class="datagrid-cell-wrapper"></div></td><td role="gridcell"><div class="datagrid-cell-wrapper">').concat("left"===h?s.sum:"<span>&nbsp;</span>","</div></td>"),r.center='<td role="gridcell" colspan="'.concat(d-l-c,'"><div class="datagrid-cell-wrapper"></div></td><td role="gridcell"><div class="datagrid-cell-wrapper">').concat("center"===h?s.sum:"<span>&nbsp;</span>","</div></td>"),r.right='<td role="gridcell" colspan="'.concat(c,'"><div class="datagrid-cell-wrapper"></div></td><td role="gridcell"><div class="datagrid-cell-wrapper">').concat("right"===h?s.sum:"<span>&nbsp;</span>","</div></td>"),o.groupFooterRowFormatter&&(r[h]=o.groupFooterRowFormatter(d,t,e,i,n,s,a)),r},SummaryRow:function(t,e,i,n){var s="",a=n.summaryText||"";return"after"===n.summaryTextPlacement&&(s=a,a=""),void 0!==H&&n.numberFormat&&null!=i&&""!==i&&(i=H.formatNumber(+i,n.numberFormat)),a+(null==i||""===i?"":i.toString())+s},Tree:function(t,e,i,n,s,a){var o=!s||s.expanded,r=a&&a.settings.treeDepth&&a.settings.treeDepth[t]?a.settings.treeDepth[t].depth:0,l=s&&s.isAllChildrenFiltered,c=l?" disabled":"";o&&l&&(o=!1);var d="extra-small"===a.settings.rowHeight||"small"===a.settings.rowHeight?24:27,h='<button type="button" class="btn-icon datagrid-expand-btn'.concat(o?" is-expanded":"",'" tabindex="-1"').concat(r?' style="margin-left: '.concat(r?"".concat(d*(r-1),"px"):"",'"'):"").concat(c,'>\n      <span class="icon plus-minus ').concat(o?" active":"",'"></span>\n      <span class="audible">').concat(H.translate("ExpandCollapse"),"</span>\n      </button>").concat(i?" <span>".concat(i,"</span>"):""),u=' <span class="datagrid-tree-node"'.concat(r?' style="margin-left: '.concat(r?"".concat(d*r,"px"):"",'"'):"",">").concat(i,"</span>");return s&&s[n.children?n.children:"children"]?h:u},ClassRange:function(t,e,i,n){var s=n.ranges,a="",o="";if(!s)return{};for(var r=0;r<s.length;r++){i>=s[r].min&&i<=s[r].max&&(a=s[r].classes,o=s[r].text?s[r].text:a.split(" ")[0]),i===g.escapeHTML(s[r].value)&&(a=s[r].classes,o=s[r].text?s[r].text:i)}return{classes:a,text:o}},Badge:function(t,e,i,n){var s=n.color,a=n.name;if(n.ranges){var o=Io.ClassRange(t,e,i,n);s=o.classes,a=o.text}return'<span class="badge '.concat(s,'">').concat(i,' <span class="audible">').concat(a,"</span></span>")},Tag:function(t,e,i,n){var s,a=Io.ClassRange(t,e,i,n);return null!=n&&null!==(s=null==n?void 0:n.editorOptions)&&void 0!==s&&(null==s?void 0:s.clickable)?'<span class="tag is-linkable hide-focus '.concat(a.classes,'"><a class="tag-content" href="#">#').concat(i,'</a><button class="btn-linkable" focusable="false" tabindex="-1">\n        <svg class="icon caret-right" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-caret-right"></use></svg>\n      </button></span>'):'<span class="tag '.concat(a.classes,' hide-focus"><span class="tag-content">').concat(i,"</span></span>")},Alert:function(t,e,i,n){var s=Io.ClassRange(t,e,i,n),a=$.createIcon({icon:"".concat(s.classes),classes:["icon","datagrid-alert-icon","icon-".concat(s.classes)]});return"".concat(a,'<span class="datagrid-alert-text">').concat("value"===s.text?i:s.text,"</span>")},Image:function(t,e,i,n){return'<img class="'.concat(n.click?"datagrid-img actionable":"datagrid-img",'" src="').concat(i,'" alt="').concat(n.alt?n.alt:H.translate("Image"),'"').concat(n.dimensions?' style="height:'.concat(n.dimensions.height,";width:").concat(n.dimensions.height,'"'):"","/>")},Color:function(t,e,i,n){var s=Io.ClassRange(t,e,i,n),a=null==i||""===i?"":i.toString();return'<span class="'.concat(s.classes,'">').concat(a,"</span>")},Colorpicker:function(t,e,i,n){var s=null==i||""===i?"":i.toString();if(n.inlineEditor)return s;var a="swatch".concat(""===i?" is-empty":"");return s='<span class="colorpicker-container trigger dropdown-trigger"><span class="'.concat(a,'" style="background-color: ').concat(i,'"></span><input class="colorpicker" id="colorpicker-').concat(e,'" name="colorpicker-').concat(e,'" type="text" role="combobox" aria-autocomplete="list" value="').concat(i,'">'),s+='<span class="trigger">'.concat($.createIcon({icon:"dropdown"}),"</span></span>")},Button:function(t,e,i,n,s,a){var o,r=Do(t,e,i,n,s)?" disabled":"";o=n.text?n.text:null==i||""===i?"":i.toString();var l='<button type="button"'.concat(r,' class="').concat(n.icon?"btn-icon":"btn-secondary"," row-btn ").concat(n.cssClass?n.cssClass:"",'"').concat(a.settings.rowNavigation?' tabindex="-1"':""," >");return n.icon&&(l+=$.createIcon({icon:n.icon,file:n.iconFile})),l+="<span>".concat(o,"</span></button>")},MultiSelect:function(t,e,i,n,s){var a,o,r,c=i,d=!1;if(n.options&&void 0!==i){("object"===l(a=n.caseInsensitive&&"string"==typeof i?i.toLowerCase():i)||Array.isArray(a))&&(c="");for(var h=0,u=n.options.length;h<u;h++){if(o=n.options[h],r=n.caseInsensitive&&"string"==typeof o.value?o.value.toLowerCase():o.value,("object"===l(a)||Array.isArray(a))&&n.editorOptions.multiple)for(var p=0;p<a.length;p++)n.options[h].value===a[p]&&(c+="".concat(o.label,", "));if(r===a){c=o.label;break}}c=c.replace(/,\s*$/,"")}var g=Eo(c,t,e,i,n,s);""!==g&&(d=!0,c=g);var f='<span class="trigger dropdown-trigger '.concat(d?"is-placeholder":"",'">').concat(c,"</span>").concat($.createIcon({icon:"dropdown"}));if(n.inlineEditor){f='<label for="full-dropdown" class="audible">'.concat(n.name,'</label><select id="datagrid-dropdown').concat(t,'" class="dropdown">');for(var m=0,v=n.options.length;m<v;m++){var b=n.options[m],y=void 0;y=void 0!==b.label?b.label:void 0!==b.value?b.value:"",f+="<option".concat(void 0===b.id?"":' id="'.concat(b.id,'"'),'  value="').concat(b.value,'"').concat(b.selected||b.value===a?" selected ":"",">").concat(y,"</option>")}f+='</select>\n      <div class="dropdown-wrapper is-inline">\n        <div class="dropdown"><span>'.concat(c,'</span></div>\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n          <use href="#icon-dropdown"></use>\n        </svg>\n      </div>')}return f},Dropdown:function(t,e,i,n,s){var a,o,r=i,l=!1;if(n.options&&void 0!==i){a=n.caseInsensitive&&"string"==typeof i?i.toLowerCase():i;for(var c=0,d=n.options.length;c<d;c++)if(o=n.options[c],(n.caseInsensitive&&"string"==typeof o.value?o.value.toLowerCase():o.value)===a){r=o.label;break}}var h=Eo(r,t,e,i,n,s);""!==h&&(l=!0,r=h);var u='<span class="trigger dropdown-trigger '.concat(l?"is-placeholder":"",'">').concat(r,"</span>").concat($.createIcon({icon:"dropdown"}));if(n.inlineEditor){u='<label for="full-dropdown" class="audible">'.concat(n.name,'</label><select id="datagrid-dropdown').concat(t,'" class="dropdown">');for(var p=0,g=n.options.length;p<g;p++){var f=n.options[p],m=void 0;m=void 0!==f.label?f.label:void 0!==f.value?f.value:"",u+="<option".concat(void 0===f.id?"":' id="'.concat(f.id,'"'),'  value="').concat(f.value,'"').concat(f.selected||f.value===a?" selected ":"",">").concat(m,"</option>")}u+='</select>\n      <div class="dropdown-wrapper is-inline">\n        <div class="dropdown"><span>'.concat(r,'</span></div>\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n          <use href="#icon-dropdown"></use>\n        </svg>\n      </div>')}return u},Fileupload:function(t,e,i,n){var s=null==i||""===i?"":i.toString();return n.inlineEditor||(s=""===$.trim(s)?'<span class="trigger">'.concat(s,"</span>").concat($.createIcon({icon:"folder",classes:["icon-fileupload"]})):'<span class="trigger is-clearable">'.concat(s,"</span>").concat($.createIcon({icon:"close",classes:["icon-close"]})).concat($.createIcon({icon:"folder",classes:["icon-fileupload"]}))),s},Spinbox:function(t,e,i,n){var s=null==i||""===i?"":i.toString();return n.inlineEditor&&(s='<label for="spinbox-'.concat(e,'" class="audible">Quantity</label>\n        <span class="spinbox-wrapper"><span class="spinbox-control down">-</span>\n        <input id="spinbox-').concat(e,'" name="spinbox-').concat(e,'" type="text" class="spinbox" value="').concat(i,'">\n        <span class="spinbox-control up">+</span></span>')),s},Favorite:function(t,e,i,n,s,a){var o;o=n.isChecked?n.isChecked(i):void 0!==i&&!0===i;var r=n.editor&&a.settings.editable;return o?'<span aria-label="'.concat(H.translate("Favorite"),'" class="icon-favorite').concat(r?" is-editable":"",'">').concat($.createIcon({icon:"star-filled"}),"</span>"):n.showEmpty?'<span aria-label="'.concat(H.translate("Favorite"),'" class="icon-favorite').concat(r?" is-editable":"",'">').concat($.createIcon({icon:"star-outlined"}),"</span>"):""},Status:function(t,e,i,n,s){return s&&s.rowStatus?"".concat($.createIcon({icon:s.rowStatus.icon,classes:["icon","icon-".concat(s.rowStatus.icon)]}),'<span class="audible">').concat(s.rowStatus.text,"</span>"):"<span></span>"},TargetedAchievement:function(t,e,i,n){var s=H.formatNumber(100*i,n.numberFormat||{minimumFractionDigits:2,maximumFractionDigits:2}),a=s.replace(",","."),o="".concat(s,"%"),r=Io.ClassRange(t,e,a,n),l=n.target;n.text&&(o=(o=(o=(o=(o=(o=(o=n.text).replace("{{value}}",i)).replace("<% value %>",i)).replace("<%value%>",i)).replace("{{percent}}",s)).replace("<% percent %>",s)).replace("<%percent%>",s),n.showPercentText=!0);var c=n.ranges&&r.classes?r.classes:"primary";return'<div class="total bar chart-completion-target chart-targeted-achievement">\n              <div class="target remaining bar" style="width: '.concat(l||0,'%;"></div>\n              <div class="completed bar ').concat(c,'" style="width: ').concat(a,'%;"></div>\n              ').concat(n.showPercentText?'<div class="chart-targeted-text l-center">'.concat(o,"</div>\n            </div>"):'<div class="audible">'.concat(s,"%</div>"))},ProcessIndicator:function(t,e,i){if(!i.steps)return"";for(var n='<div class="process-indicator compact responsive"><div class="display">',s=0;s<i.steps;s++){var a=s===(null==i||null==i?void 0:i.current)-1;n+='<div class="step">\n        <div class="lines">\n          <span class="indicator '.concat(a?" current processing":""," ").concat(s<(null==i||null==i?void 0:i.current)?" darker":"",'"></span>\n          <span class="separator ').concat(s<(null==i||null==i?void 0:i.current)?" darker":"",'"></span>\n        </div>\n      </div>')}return n+="</div></div>"}},Lo={cleanExtra:function(t,e){var i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=[];if(!e&&t)n=Lo.datasetToHtml(t,i);else{var s=e.settings.groupable?e.originalDataset:e.settings.dataset,a=e.table.clone(!0);(e.settings.frozenColumns.left.length||e.settings.frozenColumns.right.length)&&a.find(".datagrid-header tr:first()").html(e.headerNodes().clone(!0)),n=Lo.appendRows(s,a[0],e,i)}if(!t&&!n[0].querySelector("thead")){for(var o=n[0].querySelector("tbody"),r=n[0].createTHead(),l=r.insertRow(0),c=e.headerNodes(),d=0;d<c.length;d++){var h=c[d],u=l.insertCell(d);u.innerHTML=h.querySelector(".datagrid-header-text").textContent.trim(),u.setAttribute("class",h.classList),u.setAttribute("id",h.getAttribute("id")),h.getAttribute("data-exportable")&&u.setAttribute("data-exportable",h.getAttribute("data-exportable"))}o.parentNode.insertBefore(r,o)}(n=function(t){var e=function(t){t.parentNode&&t.parentNode.removeChild(t)},i=[];return[].slice.call(t[0].querySelectorAll("tr")).forEach((function(t){t.classList.contains("is-hidden")||t.classList.contains("datagrid-expandable-row")?e(t):[].slice.call(t.querySelectorAll("th, td, div, span")).forEach((function(t){if(t.classList.contains("is-hidden"))e(t);else{var n=t.getAttribute("data-exportable");if(n&&"no"===n){var s=t.id.match(/-header-\d+/g);return s=(s=s?s[0].match(/\d+/g):null)?s[0]:t.id.slice(-1),s=parseInt(s,10),i.push(s+1),void e(t)}var a=t.getAttribute("aria-colindex");if("td"===t.tagName.toLowerCase()&&a&&-1!==i.indexOf(parseInt(a,10)))e(t);else{for([].slice.call(t.querySelectorAll(".is-hidden, .datagrid-expand-btn, .is-draggable-target, .handle, .sort-indicator, .datagrid-filter-wrapper")).forEach((function(t){return e(t)}));t.attributes.length>0;)t.removeAttribute(t.attributes[0].name);var o=t.textContent;"="===o.substr(0,1)&&""!==o.substr(1,1)&&(t.textContent="'".concat(o,"'"))}}}))})),t}(n),e&&e.settings.exportConvertNegative)&&[].slice.call(n[0].querySelectorAll("td")).forEach((function(t){t.textContent=t.textContent.replace(/^(.+)(-$)/,"$2$1")}));return n},save:function(t,e,i){var n=(e.match(/\.([^.]*?)(?=\?|#|$)/)||[])[1],s="string"==typeof n&&/\b(xlsx|xls)\b/g.test(n),a=new Blob([t],{type:"application/csv;charset=utf-8;"});if(i&&!s){for(var o=[255,254],r=0;r<t.length;++r){var l=t.charCodeAt(r);o.push(255&l),o.push(l/256>>>0)}a=new Blob([new Uint8Array(o)],{type:"text/plain;charset=UTF-16LE;"})}if(window.URL.createObjectURL){s&&(a=new Blob([t],{type:"application/vnd.ms-excel;charset=utf-8;"}));var c=URL.createObjectURL(a),d=document.createElement("a");d.href=c,d.download=e,document.body.appendChild(d),d.click(),document.body.removeChild(d),URL.revokeObjectURL(c)}else{var h=document.createElement("a");h.href=this.base64(t),h.download=e,document.body.appendChild(h),h.click(),document.body.removeChild(h)}},datasetToHtml:function(t){for(var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],i="<tbody>",n=function(n){i+="<tr>",Object.keys(t[n]).forEach((function(s,a){var o=t[n];if(e&&"id"!==s&&!s.includes("Id"))if(isNaN(o[s])){var r=Date.parse(o[s]);isNaN(r)||(o[s]=H.formatDate(r))}else{var l=parseFloat(o[s]);o[s]=H.formatNumber(l,{style:Number.isInteger(l)?"integer":"decimal"})}o&&Object.prototype.hasOwnProperty.call(o,s)&&(i+="<td>".concat(o[s],"</td>"))})),i+="</tr>"},s=0,a=t.length;s<a;s++)n(s);return i+="</tbody>",i=g.sanitizeHTML(i),$("<table></table>").append(i)}};Lo.appendRows=function(t,e,i){var n,s=arguments.length>3&&void 0!==arguments[3]&&arguments[3],a=(n=e)&&(n instanceof jQuery||n.constructor.prototype.jquery)?e:$(e);e=a[0];var o="",r=e.querySelector("tbody");r.innerHTML="";var l=function t(e,n){if(!e._isFilteredOut){var a=i.rowHtml(e,n,n,!1,!1,n,!0,s),r=document.createElement("tr");if(r.innerHTML=a.left+a.center+a.right,o+=r.outerHTML,e.children)for(var l=0,c=e.children.length;l<c;l++)t(e.children[l],l)}};return t.forEach((function(t,e){l(t,e)})),o=g.sanitizeHTML(o),r.insertAdjacentHTML("beforeend",o),a},Lo.base64=function(t){return window.btoa?"data:application/vnd.ms-excel;base64,".concat(window.btoa(unescape(encodeURIComponent(t)))):"data:application/vnd.ms-excel;,".concat(unescape(encodeURIComponent(t)))},Lo.copyToDataSet=function(t,e,i,n,s){for(var a=function(t,e,i,n){for(var s=0,a=t.length;s<a;s++){var o=e.columns[n];o.formatter!==Io.Readonly&&("input"===o.editor.name?("integer"===o.filterType||"decimal"===o.filterType||"number"===o.filterType)&&isNaN(t[s].trim())||(i[o.field]=t[s]):"input"===o.editor.name&&(isNaN(Date.parse(t[s]))||(i[o.field]=new Date(t[s])))),n++}},o=t.length,r=s.settings.columns.length,l=0;l<o;l++){var c=t[l].split("\t"),d=i;if(e<n.length){var h=n[e];a(c,s.settings,h,d)}else{for(var u={},p=0;p<r;p++)u[s.settings.columns[p].field]="";a(c,s.settings,u,d),n.push(u)}e++}s.renderRows(),s.syncSelectedUI(),s.pagerRefresh("bottom")},Lo.exportToExcel=function(t,e,i){var n,s=arguments.length>3&&void 0!==arguments[3]&&arguments[3],a=arguments.length>4?arguments[4]:void 0,o={worksheet:e||"Worksheet",table:Lo.cleanExtra(i,a,s)[0].innerHTML};t="".concat(t||a.element[0].id||"Export",".xls"),Lo.save((n=o,'<html xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel" xmlns="http://www.w3.org/TR/REC-html40"><head>\x3c!--[if gte mso 9]><xml><x:ExcelWorkbook><x:ExcelWorksheets><x:ExcelWorksheet><x:Name>{worksheet}</x:Name><x:WorksheetOptions><x:Panes></x:Panes><x:DisplayGridlines></x:DisplayGridlines></x:WorksheetOptions></x:ExcelWorksheet></x:ExcelWorksheets></x:ExcelWorkbook></xml><![endif]--\x3e<meta http-equiv="content-type" content="text/plain; charset=UTF-8"/></head><body><table border="1px solid #999999">{table}</table></body></html>'.replace(/{(\w+)}/g,(function(t,e){return n[e]}))),t)},Lo.exportToCsv=function(t,e){var i,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"sep=,",s=arguments.length>3&&void 0!==arguments[3]&&arguments[3],a=arguments.length>4?arguments[4]:void 0,o=function(t){return/^(false|0+|null)$/gi.test(t)},r={firstLine:!0,char:",",flexChar:!0,flexUtf8:!0};if("sep=,"!==n&&!o(n)){var c=function(t){return'"'!==t?t:r.char};if((i=n)&&"object"===l(i)&&i.constructor===Object)r.firstLine=!o(n.firstLine),r.flexChar=!o(n.flexChar),r.flexUtf8=!o(n.flexUtf8),"string"!=typeof n.char||1!==n.char.length||o(n.char)||(r.char=c(n.char));else if("string"==typeof n)if(1!==n.length||o(n)){if(/^sep=.$/.test(n)){var d=n.replace("sep=","");r.char=o(d)?r.char:c(d)}}else r.char=c(n)}var h=Lo.cleanExtra(e,a,s),u=!/^[\u0000-\u007f]*$/.test(h[0].textContent)&&r.flexUtf8,p=function(t,e){var i=e&&r.flexChar?"\t":r.char,n=[];return[].slice.call(t[0].querySelectorAll("tr")).forEach((function(t){var e=[];[].slice.call(t.querySelectorAll("td, th")).forEach((function(t){return e.push(t.textContent.replace(/\r?\n|\r/g,"").replace(/"/g,'""').trim())})),n.push(e.join('"'.concat(i,'"')))})),r.firstLine&&(r.flexUtf8&&!e||!r.flexUtf8&&","!==r.char)&&n.unshift(["sep=".concat(r.char)]),"".concat(r.flexUtf8?"":"\ufeff",'"').concat(n.join('"\n"'),'"')}(h,u);t="".concat(t||a.element[0].id||"Export",".csv"),Lo.save(p,t,u)};var Po,Mo,Bo,Oo,Ro=(Po=E.equals,Mo=function(t,e){return t.reduce((function(t,i){var n,s=function(t,e){for(var i={},n=0,s=e.length;n<s;n++)i[e[n]]=t[e[n]];return i}(i,e);return n=s,t.some((function(t){return Po(t,n)}))||t.push(s),t}),[])},Bo=function(t,e){var i=[];return t.map((function(t,n){for(var s in e)if(e[s]!==t[s])return;t.idx=n,i.push(t)})),i},(Oo=function(t,e){return Mo(t,e).map((function(e){return{key:e,values:Bo(t,e).map((function(t){return t}))}}))}).register=function(t,e){return Oo[t]=function(t,i,n){return this.extra=n,Oo(t,i).map(e,this)}},Oo);Ro.register("none",(function(t){return $.extend({},t.key,{values:t.values})})),Ro.register("sum",(function(t){var e=this.extra,i=t.values.map((function(t){return t[e]})).filter((function(t){return null!=t}));return $.extend({},t.key,{values:t.values},{sum:i.reduce((function(t,e){return Number(t)+Number(e)}),0)})})),Ro.register("max",(function(t){var e=this.extra;return $.extend({},t.key,{values:t.values},{max:Math.max.apply(Math,u(t.values.map((function(t){return t[e]})).filter((function(t){return null!=t}))))})})),Ro.register("min",(function(t){var e=this.extra;return $.extend({},t.key,{values:t.values},{min:Math.min.apply(Math,u(t.values.map((function(t){return t[e]})).filter((function(t){return null!=t}))))})})),Ro.register("avg",(function(t){var e=this.extra,i=t.values.map((function(t){return t[e]})).filter((function(t){return null!=t}));return $.extend({},t.key,{values:t.values},{avg:i.reduce((function(t,e){return Number(t)+Number(e)}),0)/i.length})})),Ro.register("count",(function(t){var e=this.extra,i=t.values.map((function(t){return t[e]})).filter((function(t){return null!=t}));return $.extend({},t.key,{values:t.values},{count:i.length})})),Ro.register("list",(function(t){var e=this.extra;return $.extend({},t.key,{values:t.values},{list:t.values.map((function(t){for(var i=[],n=0,s=e.list.length;n<s;n++){var a=!!e.exclude&&t[e.exclude];t[e.list[n]]&&!a&&i.push({value:t[e.list[n]],key:e.list[n]})}return i}))})}));var zo={aggregate:function(t,e){for(var i={},n=this,s=function(){if(e[a].aggregator){var s=e[a].field;n.sum=function(t,e){if(e._isFilteredOut)return t;var i;i=s.indexOf(".")>-1?s.split(".").reduce((function(t,e){return t?t[e]:""}),e):e[s],i=Number(i);var n=B.decimalPlaces(i),a=B.decimalPlaces(t);return Number((t+i).toFixed(Math.max(n,a)))};var o=t.reduce(n[e[a].aggregator],0);if(s.indexOf(".")>-1)for(var r=i,l=0,c=s.split(".").length;l<c;l++)l===s.split(".").length-1?r[s.split(".")[l]]=o:(s.split(".")[l]in r||(r[s.split(".")[l]]={}),r=r[s.split(".")[l]]);else i[s]=o}},a=0,o=e.length;a<o;a++)s();return i}},Fo="lookup",No="lookup-datagrid";function Ho(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[],i=arguments.length>2?arguments[2]:void 0,n=[];return null!=t&&(null==t?void 0:t.length)||null!=e&&(null==e?void 0:e.length)?null!=e&&(null==e?void 0:e.length)?t.length?(t.forEach((function(t){var s=e.find((function(e){return e.name===t.name}));n.push(s||{name:t.name,value:"".concat(t.value,"-").concat(i)})})),n):e:n=t.map((function(t){return{name:t.name,value:"".concat(t.value,"-").concat(i)}})):n}var qo={click:null,field:"id",title:null,icon:"icon-search-list",buttons:[],options:null,beforeShow:null,modalContent:null,editable:!0,autoApply:!0,validator:null,autoWidth:!1,clickArguments:{},delimiter:",",minWidth:null,clearable:!1,attributes:null,tabbable:!0,dblClickApply:!1,allowDuplicates:!1};function Vo(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,qo),this.init()}function Wo(t,e,i,n,s,a,o,r,l){if(a.align&&t.addClass("l-".concat(a.align,"-text")),a.maxLength&&t.attr("maxlength",a.maxLength),a.uppercase&&t.addClass("uppercase-text"),a.mask&&"function"==typeof a.mask){var c=a.mask(e,i,n,a,l);t.mask({pattern:c,mode:a.maskMode})}else if(a.maskOptions&&"function"==typeof a.maskOptions){var d=a.maskOptions(e,i,n,a,l);t.mask(d)}else a.mask&&t.mask({pattern:a.mask,mode:a.maskMode});var h={patternOptions:{allowNegative:!0,allowDecimal:!0,integerLimit:4,decimalLimit:2,symbols:{thousands:H.currentLocale.data.numbers?H.currentLocale.data.numbers.group:",",decimal:H.currentLocale.data.numbers?H.currentLocale.data.numbers.decimal:".",negative:H.currentLocale.data.numbers?H.currentLocale.data.numbers.minusSign:"-"}},process:"number"},u=!1;a.maskOptions&&(u=!0),a.numberFormat&&(u=!0,h={patternOptions:{decimalLimit:a.numberFormat.maximumFractionDigits}}),a.maskOptions&&"function"==typeof a.maskOptions&&(u=!1),u&&(a.maskOptions=E.extend(!0,{},h,a.maskOptions),t.mask(a.maskOptions)),a.align&&"right"===a.align||t.removeClass("is-number-mask")}Vo.prototype={get isFocused(){var t,e,i=document.activeElement,n=this.element.is(i),s=null===(t=this.element.parent(".lookup-wrapper")[0])||void 0===t||null==t?void 0:t.contains(i),a=null===(e=this.modal)||void 0===e||null==e?void 0:e.element[0].contains(i);return n||s||a},init:function(){this.inlineLabel=this.element.closest("label"),this.inlineLabelText=this.inlineLabel.find(".label-text"),this.isInlineLabel=!!this.inlineLabelText.length,this.build(),this.handleEvents(),this.grid=null,this.selectedRows=null},build:function(){var t=this,e=this.element;var i,n=(i="lookup-wrapper",e.is(".input-xs")&&(i+=" xs"),e.is(".input-sm")&&(i+=" sm"),e.is(".input-lg")&&(i+=" lg"),i);if(this.element.is(".has-actions")&&(n+=" has-actions-wrapper"),this.isInlineLabel)this.inlineLabel.addClass(n);else if(this.container=$('<span class="'.concat(n,'"></span>')),this.element.is(".field-options")){var s=this.element.closest(".field").find(".btn-actions");e.add(s).add(s.next(".popupmenu")).wrapAll(this.container)}else e.wrap(this.container);var a=this.element.next();if(a.is("button.trigger"))this.icon=a;else{this.icon=$('<button class="btn-icon trigger" type="button">\n        <span class="audible"></span>\n        '.concat($.createIcon(this.settings.icon),"\n      </button>"));var o=this.icon.children(".icon");"icon-search-list"!==this.settings.icon?o.addClass("has-custom-icon"):x.rtl&&o.addClass("icon-rtl-rotate")}e.after(this.icon),e.hasClass("hidden")&&this.icon.addClass("hidden"),this.settings.autoWidth&&this.applyAutoWidth(),this.minWidth||(this.settings.minWidth=this.settings.options&&this.settings.options.paging?482:400),e.attr("data-mask")&&e.mask(),this.element.is(":disabled")&&this.disable(),this.settings.editable||this.element.attr("readonly","true").addClass("is-not-editable"),this.makeTabbable(!this.element.is(":disabled")&&"-1"!==this.element.attr("tabindex")&&this.settings.tabbable),this.settings.clearable&&e.searchfield({clearable:!0,attributes:this.settings.attributes,tabbable:!1}),this.settings.clearable&&this.settings.clear&&e.on("cleared",(function(e){t.settings.clear(e,t,t.settings.clearArguments)})),this.element.parent(".lookup-wrapper").next().is(".btn-actions")&&(this.element.data("fieldoptions")&&this.element.data("fieldoptions").destroy(),this.element.fieldoptions()),this.addAria(),void 0===e.attr("autocomplete")&&e.attr("autocomplete","off"),Array.isArray(this.settings.attributes)&&(E.addAttributes(e,this,this.settings.attributes,"input"),E.addAttributes(this.icon,this,this.settings.attributes,"trigger"))},addAria:function(){this.label=this.isInlineLabel?this.inlineLabelText:$('label[for="'.concat(this.element.attr("id"),'"]'));var t="".concat(H.translate("Lookup"),". ").concat(H.translate("PressDown")),e=this.label.children("span.audible");e.length?e.text(t):this.label.append('<span class="audible">'.concat(t,"</span>"));var i=H.translate("LookupTriggerButton");this.icon.children("span.audible").text(i)},handleEvents:function(){var t=this;this.icon.on("click.lookup",(function(e){t.openDialog(e)})),this.element.on("keyup.lookup",(function(e){$("#autocomplete-list").length>0||40===e.which&&t.openDialog(e)}))},setTooltip:function(){var t=this;setTimeout((function(){var e=t.element[0].scrollWidth>t.element[0].offsetWidth,i=t.element.data("tooltip");e?t.element.tooltip({content:t.element.val()}):i&&!e&&i.destroy()}),100)},openDialog:function(t){var e=this;if((this.isChanged=!1,!this.isOpen)&&(!1!==e.element.triggerHandler("beforeopen")&&!(e.isDisabled()||e.isReadonly()&&!e.element.hasClass("is-not-editable"))))if(e.settings.click)e.settings.click(t,this,e.settings.clickArguments);else{if(this.isOpen=!0,this.settings.beforeShow){return this.element.triggerHandler("start"),void this.settings.beforeShow(this,(function(t){var i=e.element&&e.element.is(":visible");return t&&e.createGrid(t),!i||"boolean"==typeof t&&!1===t?(e.isOpen=!1,!1):(e.createModal(),e.element.triggerHandler("complete"),e.element.trigger("open",[e.modal,e.grid]),e.settings.validator&&e.settings.validator(e.element,e.modal,e.grid),!0)}))}if(this.settings.options){e.createModal(),e.element.trigger("open",[e.modal,e.grid]),e.modal.element.find(".btn-actions").removeClass("is-selected"),e.grid&&e.grid.cellNode(0,0,!0).attr("tabindex","0");var i=e.modal.element.find(".toolbar .more > .btn-actions");i.length&&setTimeout((function(){E.fixSVGIcons(i)}),600),e.element.trigger("afteropen",[e.modal,e.grid]),e.settings.validator&&e.settings.validator(e.element,e.modal,e.grid)}}},createModal:function(){var t=this,e='<div id="'.concat(No,'"></div>'),i=$('label[for="'.concat(t.element.attr("id"),'"]'));var n,s,a,o,l,c,d=t.isInlineLabel?t.inlineLabelText:i.length?i.clone().find("span").remove().end().text():t.settings.title?t.settings.title:null!==(n=t.settings)&&void 0!==n&&null!==(s=null==n?void 0:n.options)&&void 0!==s&&null!==(a=null==s?void 0:s.toolbar)&&void 0!==a&&(null==a?void 0:a.title)?null===(o=t.settings)||void 0===o||null===(l=null==o?void 0:o.options)||void 0===l||null===(c=null==l?void 0:l.toolbar)||void 0===c||null==c?void 0:c.title:"",h=this.settings.modalContent;h&&h instanceof jQuery&&(e=h,h.show()),!h||h instanceof jQuery||(e=h);var u=this.settings.buttons;(this.settings.options&&"multiple"===this.settings.options.selectable&&0===u.length||!t.settings.autoApply&&0===u.length)&&(u=[{text:H.translate("Cancel"),click:function(e,i){i.oldActive=t.icon,i.close()}},{text:H.translate("Apply"),click:function(e,i){t.insertRows(),i.oldActive=t.icon,i.close()},isDefault:!0}]),this.settings.options&&"single"===this.settings.options.selectable&&0===u.length&&t.settings.autoApply&&(u=[{text:H.translate("Cancel"),click:function(e,i){i.oldActive=t.icon,i.close()}},{text:H.translate("Apply"),click:function(e,i){i.oldActive=t.icon,i.close(),t.insertRows()},isDefault:!0}]);var p=this.settings.options&&this.settings.options.toolbar&&this.settings.options.toolbar.keywordFilter,g={triggerButton:this.element,title:d,content:e,buttons:u,cssClass:"lookup-modal".concat(p?"":" lookup-no-search"),attributes:Ho(this.settings.attributes,[],"modal")};t.settings.modalSettings&&(g=r(r({},g),t.settings.modalSettings)),$("body").modal(g).off("open.lookup").on("open.lookup",(function(){t.createGrid()})).off("close.lookup").on("close.lookup",(function(){delete t.isOpen,t.element.triggerHandler("close",[t.modal,t.grid]),t.setTooltip()})),t.modal=$("body").data("modal"),this.settings.title||t.modal.element.find(".modal-title").append(' <span class="datagrid-result-count"></span>'),t.modal.element.off("afterclose.lookup").on("afterclose.lookup",(function(){t.closeTearDown()})),setTimeout((function(){$(".modal.is-visible .searchfield").on("keypress.lookup",(function(t){return 13!==t.keyCode}))}),300)},closeTearDown:function(){var t=$(".modal.is-visible .searchfield").off("keypress.lookup");t.data()&&t.data("searchfield")&&t.data("searchfield").destroy(),t.data()&&t.data("searchfield")&&(t.data("searchfield").destroy(),t.removeData()),t=null,this.grid&&this.grid.destroy&&($(this.grid.element).off("selected.lookup"),this.grid.deSelectAllRows(),this.grid.clearFilter(),this.grid.destroy(),this.grid=null)},doMerge:function(){for(var t={},e=[],i=arguments.length,n=new Array(i),s=0;s<i;s++)n[s]=arguments[s];for(var a=0;a<n.length;a++)for(var o=0;o<n[a].length;o++)!0!==t[n[a][o]]&&(e[e.length]=n[a][o],t[n[a][o]]=!0);return e},createGrid:function(t){var e,i,n,s,a=this,o=this;t?(s=t,No=t.attr("id"),o.settings.options=t.data("datagrid").settings):s=o.modal.element.find("#".concat(No)),this.settings.minWidth&&(s=this.applyMinWidth(s)),o.settings.dblClickApply&&(o.settings.options.dblClickApply=o.settings.dblClickApply),o.settings.options&&("single"===o.settings.options.selectable&&o.settings.autoApply&&(o.settings.options.cellNavigation=!1,s.find("tr").addClass("is-clickable")),o.settings.options.isList=!0,o.settings.attributes&&(o.settings.options.attributes=Ho(o.settings.attributes,[],"datagrid")),s.data("datagrid")||s.datagrid(o.settings.options)),o.grid=s.data("datagrid"),!this.settings.title&&o.modal&&o.modal.element.find(".title").not(".selection-count").remove();var r=null===(e=this.settings)||void 0===e||null===(i=null==e?void 0:e.options)||void 0===i||null===(n=null==i?void 0:i.toolbar)||void 0===n||null==n?void 0:n.keywordFilter;if(!r&&o.modal&&o.modal.element.find(".toolbar").appendTo(o.modal.element.find(".modal-header")),r&&o.grid&&(!o.grid.filterExpr||o.grid.filterExpr&&o.grid.filterExpr[0]&&""!==o.grid.filterExpr[0].value)&&o.grid.keywordSearch(""),s.off("selected.lookup"),o.initValues)o.initValues.forEach((function(t){t&&o.selectGridRows(t.value.toString())})),delete o.initValues;else{var l=o.element.val();if(l&&!o.settings.options.source)("multiple"===o.settings.options.selectable&&l.indexOf(this.settings.delimiter)>1?l.split(this.settings.delimiter):[l]).forEach((function(t){return o.selectGridRows(t.toString())}))}this.settings.options.source&&s.off("afterpaging.lookup").on("afterpaging.lookup",(function(t,e){if(o.settings.options.source){var i=o.element.val();a.selectGridRows(i)}a.element.trigger("afterpaging",[e,a])})),this.settings.options&&(this.settings.dblClickApply&&"single"===this.settings.options.selectable&&s.on("ischanged.lookup",(function(){a.isChanged=!0})),s.on("selected.lookup",(function(t,e,i,n){o.settings.options.source&&"multiple"===o.settings.options.selectable?o.element.trigger("selected",[e,i,n,o]):(a.isChanged=!0,"deselect"===i&&(o.grid.recentlyRemoved||(o.grid.recentlyRemoved=[]),o.grid.recentlyRemoved.push(n)),e&&0!==e.length?(o.settings.validator&&o.settings.validator(o.element,o.modal,o.grid),"single"===o.settings.options.selectable&&o.settings.autoApply&&"deselectall"!==i&&(o.modal.close(),o.insertRows()),o.element.trigger("selected",[e,i,n,o])):"deselect"===i&&o.element.trigger("selected",[e,i,n,o]))}))),this.settings.options.source&&this.grid.element.one("afterrender.lookup",(function(){var t,e=a.settings.options.source&&(null===(t=a.selectedRows)||void 0===t||null==t?void 0:t.length);if(!a.initValues||a.initValues&&!a.initValues.length||e)if(a.initValues=[],e)setTimeout((function(){var t,e,i=null===(t=a.grid)||void 0===t||null===(e=null==t?void 0:t.pagerAPI)||void 0===e||null==e?void 0:e.state;if(i){var n={min:i.activePage*i.pagesize-i.pagesize,max:i.activePage*i.pagesize-1};a.selectedRows.forEach((function(t){var e;a.initValues.push({value:t.data[a.settings.field],visited:(e=t,e.pagingIdx>=n.min&&e.pagingIdx<=n.max)})})),a.grid._selectedRows=a.selectedRows.slice(),a.grid.syncSelectedRows(),a.grid.syncSelectedUI()}}),310);else{var i=a.element.val();if(i)(i.indexOf(a.settings.delimiter)>1?i.split(a.settings.delimiter):[i]).forEach((function(t){a.initValues.push({value:t,visited:!!a.grid.settings.dataset.filter((function(e){return function(t,e){return(t[a.settings.field]||"").toString()===e.toString()}(e,t)})).length})}))}}))},updateSelectedRows:function(t){var e=this;if(this.settings.options.source&&(t=Array.isArray(t)?t:[],this.selectedRows.length!==t.length)){this.selectedRows=t.slice();var i=this.selectedRows.map((function(t){return t.data?t.data[e.settings.field]:""})).join(this.settings.delimiter);this.grid&&(this.grid._selectedRows=this.selectedRows.slice(),this.grid.syncSelectedRows(),this.grid.syncSelectedUI()),this.element.val(i).trigger("change",[this.selectedRows]),this.element.trigger("input",[this.selectedRows])}},selectGridRows:function(t){var e=t,i=!1;if(t){if(this.grid&&this.settings.options.source)for(var n=this.grid._selectedRows.length-1;n>-1;n--)isNaN(this.grid._selectedRows[n].idx)&&this.grid._selectedRows.splice(n,1);if(e.indexOf(this.settings.delimiter)>1){for(var s=e.split(this.settings.delimiter),a=!1,o=!1,r=0;r<s.length;r++){if(o=!1,this.grid.recentlyRemoved)for(var l=0;l<this.grid.recentlyRemoved.length;l++)this.grid.recentlyRemoved[l][this.settings.field].toString()===s[r].toString()&&(o=!0);if(!o&&(a=this.selectRowByValue(this.settings.field,s[r]),this.grid&&this.settings.options.source&&!a)){for(var c=!1,d=0;d<this.grid._selectedRows.length;d++)this.grid._selectedRows[d].data[this.settings.field].toString()===s[r].toString()&&(c=!0);if(this.grid.recentlyRemoved)for(var h=0;h<this.grid.recentlyRemoved.length;h++)this.grid.recentlyRemoved[h][this.settings.field].toString()===s[r].toString()&&(c=!0);if(!c){var u={};u[this.settings.field]=s[r],this.grid._selectedRows.push({data:u})}i=!0}}if(i){this.modal.element.find(".contextual-toolbar .selection-count").text("".concat(s.length," ").concat(H.translate("Selected"))),this.grid.syncSelectedUI()}this.selectRowByValue(this.settings.field,e)}else{var p=[];p.push(e);for(var g=!1,f=!1,m=0;m<p.length;m++){if(this.grid.recentlyRemoved)for(var v=0;v<this.grid.recentlyRemoved.length;v++)this.grid.recentlyRemoved[v][this.settings.field].toString()===p[m].toString()&&(f=!0);if(!f){if(g=this.selectRowByValue(this.settings.field,p[m]),this.grid&&this.settings.options.source&&!g){for(var b=!1,y=0;y<this.grid._selectedRows.length;y++)this.grid._selectedRows[y].data[this.settings.field].toString()===p[m].toString()&&(b=!0);if(this.grid.recentlyRemoved)for(var w=0;w<this.grid.recentlyRemoved.length;w++)this.grid.recentlyRemoved[w][this.settings.field].toString()===p[m].toString()&&(b=!0);if(!b){({})[this.settings.field]=p[m]}i=!0}if(g&&this.grid.recentlyRemoved)for(var x=0;x<this.grid.recentlyRemoved.length;x++)this.grid.recentlyRemoved[x][this.settings.field].toString()===p[m].toString()&&(i=!1)}}if(i){this.modal.element.find(".contextual-toolbar .selection-count").text("".concat(p.length," ").concat(H.translate("Selected"))),this.grid.syncSelectedUI()}}}},selectRowByValue:function(t,e){if(!this.settings.options)return!1;var i=this.settings.options.source?this.grid.settings.dataset:this.settings.options.dataset,n=[];if(!i)return!1;for(var s=0;s<i.length;s++){var a,o,r=!1;if("function"==typeof this.settings.match&&this.settings.match(e,i[s],this.element,this.grid)&&(r=!0),"function"!=typeof this.settings.match&&(null===(a=i[s][t])||void 0===a||null==(o=a)?void 0:o.toString())===(null==e||null==e?void 0:e.toString())&&(r=!0),r){var l=this.grid.settings.source?this.grid.actualRowIndex(this.grid.tableBody.find("tr").eq(s)):s;n.push(l)}}return!!(this.grid&&n.length>0)&&(this.grid.selectRows(n,!1,!0),!0)},insertRows:function(){if(this.isChanged||this.settings.options.source){var t=[];this.selectedRows=this.grid.selectedRows().slice();for(var e=0;e<this.selectedRows.length;e++){var i="";i="function"==typeof this.settings.field?this.settings.field(this.selectedRows[e].data,this.element,this.grid):this.selectedRows[e].data[this.settings.field],t.push(i);var n=this.selectedRows[e].idx;this.settings.options.dataset&&this.settings.options.dataset[n]&&delete this.settings.options.dataset[n]._selected}this.settings.allowDuplicates||(t=t.filter((function(e,i){return t.indexOf(e)===i}))),this.element.val(t.join(this.settings.delimiter)).trigger("change",[this.selectedRows]),this.element.trigger("input",[this.selectedRows]),this.applyAutoWidth(),this.element.focus()}},enable:function(){this.element.prop("disabled",!1).prop("readonly",!1).removeClass("is-not-editable"),this.element.closest(".field").removeClass("is-disabled"),this.icon.prop("disabled",!1)},disable:function(){this.element.prop("disabled",!0),this.element.closest(".field").addClass("is-disabled"),this.icon.prop("disabled",!0)},readonly:function(){this.element.prop("readonly",!0),this.icon.prop("disabled",!1)},applyAutoWidth:function(){var t=this.element.val(),e=t.length,i=t===t.toUpperCase(),n=!isNaN(t);this.element.attr("size",e+(i&&!n?2:1))},applyMinWidth:function(t){if(null==this.settings.minWidth)return t;if($(window).width()>this.settings.minWidth){this.modal.element.addClass("has-minwidth");var e="".concat(this.settings.minWidth,"px");t.css({"min-width":e})}return t},isDisabled:function(){return this.element.prop("disabled")},isReadonly:function(){return this.element.prop("readonly")},makeTabbable:function(t){this.icon.attr("tabIndex",t?0:-1)},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings))},updateDataset:function(t,e){this.settings.options.dataset=t,e&&e.activePage&&(this.settings.options.activePage=e.activePage),e&&e.pagesize&&(this.settings.options.pagesize=e.pagesize),this.grid&&this.grid.updateDataset(t,e)},destroy:function(){$(".modal .searchfield").off("keypress.lookup"),$("body").off("open.lookup close.lookup"),this.icon.off("click.lookup"),this.icon.remove(),this.element.off("keyup.lookup"),this.element.unwrap(),this.modal&&this.modal.element&&(this.modal.element.off("afterclose.lookup"),"function"==typeof this.modal.destroy&&this.modal.destroy()),$.removeData(this.element[0],Fo)}},$.fn.lookup=function(t){return this.each((function(){var e=$.data(this,Fo);e?e.updated(t):e=$.data(this,Fo,new Vo(this,t))}))};var jo={Input:function(t,e,i,n,s,a,o,r){this.name="input",this.originalValue=i,this.useValue=!!s.inlineEditor,this.init=function(){s.inlineEditor?this.input=n.find("input"):this.input=$('<input type="'.concat(s.inputType||"text",'" autocomplete="off"/>')).appendTo(n),Wo(this.input,t,e,i,0,s,0,0,r)},this.val=function(t){var e;return t&&this.input.val(t),s&&s.numberFormat&&"percent"===s.numberFormat.style?(e=this.input.val().trim().replace(/(\s%?|%)$/g,""),H.parseNumber(e)/100):s.numberFormat?H.parseNumber(this.input.val(),s.numberFormat):this.input.val()},s.inlineEditor||(this.focus=function(){this.input.focus(),o.settings.selectOnEdit&&this.input.select()}),this.destroy=function(){var t=this;s.inlineEditor||setTimeout((function(){t.input.remove()}),0)},this.init()},Textarea:function(t,e,i,n,s,a,o){this.name="textarea",this.originalValue=i,this.init=function(){n.addClass("datagrid-textarea-cell-wrapper");var t=n.get(0).scrollHeight,e=s.editorOptions&&s.editorOptions.minHeight?'style="min-height: '.concat(s.editorOptions.minHeight,'px"'):"";this.input=$("<textarea ".concat(e,' class="textarea">').concat(this.originalValue,"</textarea>")).appendTo(n);var i=s.editorOptions?s.editorOptions:{};"characterCounter"in i||(i.characterCounter=!1),this.api=this.input.data("autogrow-start-height",t).textarea(s.editorOptions).data("textarea"),this.input.on("click.textareaeditor",(function(t){t.stopPropagation()})),s.maxLength&&this.input.attr("maxlength",s.maxLength),s.uppercase&&this.input.addClass("uppercase-text")},this.val=function(t){return t&&this.input.focus().val(t),this.input.val()},this.focus=function(){this.input.focus(),o.settings.selectOnEdit&&this.input.select()},this.destroy=function(){var t=this;n.removeClass("datagrid-textarea-cell-wrapper"),setTimeout((function(){t.input.off("click.textareaeditor"),t.input.remove()}),0)},this.init()},Editor:function(t,e,i,n,s,a,o){this.name="editor",this.originalValue=i,this.init=function(){var t=this,e=$.extend({},{buttons:{editor:["header1","header2","separator","bold","italic","underline","separator","foreColor","separator","justifyLeft","justifyCenter","justifyRight"],source:[]},excludeButtons:{editor:[]}},s.editorOptions);this.editorWidth=o.setUnit(e.width||n.outerWidth()),delete e.width,n[0].innerHTML='<div class="editor-wrapper" style="width: '.concat(this.editorWidth,';">\n          <div class="editor" data-init="false">').concat(g.sanitizeHTML(g.unescapeHTML(i)),"</div>\n        </div>"),this.td=n.closest("td"),this.input=$(".editor",n),this.input.popover({content:$(".editor-wrapper",n),placementOpts:{x:0,y:"-".concat(parseInt(n[0].style.height,10)+35),parent:this.td,parentXAlignment:H.isRTL()?"right":"left",strategies:["flip","nudge","shrink"]},placement:"bottom",popover:!0,trigger:"immediate",tooltipElement:"#editor-popup",extraClass:"editor-popup",onHidden:function(){o.commitCellEdit(t.input)}}).editor(e).on("keydown.editor",(function(e){var i=e.which||e.keyCode||e.charCode||0;if((e.ctrlKey||e.metaKey)&&(13===i||10===i)){var n=t.input.data("tooltip");n&&(n.hide(),o.setNextActiveCell(e))}})),E.fixSVGIcons($("#editor-popup"))},this.val=function(){return g.escapeHTML(this.input.html())},this.focus=function(){var t=this;setTimeout((function(){t.input.focus()}),0)},this.destroy=function(){var t=this;n.removeAttr("style"),o.quickEditMode=!1,t.input.data("editor")&&(t.input.destroy(),t.input.remove())},this.init()},Checkbox:function(t,e,i,n,s,a,o){this.name="checkbox",this.originalValue=i,this.useValue=!0,this.container=n,this.init=function(){this.input=$('<input type="checkbox" class="checkbox"/>').appendTo(n),this.input.after('<label class="checkbox-label"></label>'),s.align&&this.input.addClass("l-".concat(s.align,"-text"))},this.val=function(t){var e;if(void 0===t)return this.input.prop("checked");e=s.isChecked?s.isChecked(t):t,"click"!==a.type&&("keydown"!==a.type||32!==a.keyCode)||$(a.target).is(".datagrid-checkbox-wrapper, .datagrid-cell-wrapper")||(e=!e,o.setNextActiveCell(a)),this.input.prop("checked",e)},this.focus=function(){this.input.trigger("focusout"),this.container.parent().focus()},this.destroy=function(){var t=this;setTimeout((function(){t.input.next(".checkbox-label").remove(),t.input.remove()}),0)},this.init()},Colorpicker:function(t,e,i,n,s,a,o){this.name="colorpicker",this.originalValue=i,this.useValue=!0,i=g.stripTags(i),this.init=function(){this.input=$('<input id="colorpicker-'.concat(e,'" name="colorpicker-').concat(e,'" class="colorpicker" value="').concat(i,'" type="text" autocomplete="off"/>')).appendTo(n),this.input.colorpicker(s.editorOptions)},this.val=function(t){return t?this.input.val(t):this.input.val()},this.focus=function(){var e=this;this.input.trigger("openlist"),o.rowNodes(t).removeClass("is-hover-row"),this.input.focus(),o.settings.selectOnEdit&&this.input.select(),this.input.off("listclosed").on("listclosed",(function(){o.commitCellEdit(e.input),n.parent("td").focus()}))},this.destroy=function(){},this.init()},MultiSelect:function(t,e,i,n,s,a,o,r){this.name="multiselect",this.originalValue=i,this.useValue=!0,this.cell=o.activeCell,this.init=function(){if(s.inlineEditor)this.input=n.find("select");else{if(this.input=$('<select class="multiselect"></select>').appendTo(n),s.options){var c,d,h;i=o.fieldValue(r,s.field);for(var u=s.caseInsensitive&&"string"==typeof i?i.toLowerCase():i,p=0;p<s.options.length;p++)c=$("<option></option>"),d=s.options[p],h=s.caseInsensitive&&"string"==typeof d.value?d.value.toLowerCase():d.value,(d.selected||u===h)&&(c.attr("selected","true"),this.originalValue=h),c.attr("value",d.value).attr("id",d.id).attr("data-type",l(d.value)),c.text(d.label),this.input.append(c)}var g=s.editorOptions||{};g.cssClass&&/is-editing/g.test(g.cssClass)||(g.cssClass=g.cssClass||"",g.cssClass+=" is-editing"),this.input.multiselect(g),this.input.data("multiselect").settings.sourceArguments={column:s,container:n,grid:o,cell:e,event:a,row:t,rowData:r,value:i}}},this.val=function(t){if(this.datasetValue=t,void 0!==t){var e=s.caseInsensitive&&"string"==typeof t?t.toLowerCase():t;this.input.val(t),this.input.find("option").each((function(){var t=$(this),i=t.attr("value"),n=t.attr("data-type"),a=i;"number"===n?a=parseFloat(i):"boolean"===n?a="true"===i:"string"===n&&s.caseInsensitive&&(a=i.toLowerCase()),a===e&&t.attr("selected","true")}))}var i=this.input.find(":selected"),n=i.attr("value"),a=i.attr("data-type");return i.length>1&&(n=[],i.map((function(t,e){return n.push(e.value)}))),"number"===a?n=parseFloat(n):"boolean"===a&&(n="true"===n),void 0===n&&(n=i.text()),n},this.focus=function(){var e=this;"click"===a.type?(this.input.trigger("openlist"),o.rowNodes(t).removeClass("is-hover-row"),$("#dropdown-list input").focus()):this.input[0].parentNode.querySelector("div.dropdown").focus();if(!o.settings.selectOnEdit){var i=$("#dropdown-search")[0];if(i)if("number"==typeof i.selectionStart)i.selectionEnd=i.value.length,i.selectionStart=i.selectionEnd;else if(void 0!==i.createTextRange){var s=i.createTextRange();s.collapse(!1),s.select()}}this.input.off("listclosed").on("listclosed",(function(t,i){o.commitCellEdit(e.input),"select"!==i?"tab"===i&&setTimeout((function(){n.parent("td").focus()}),100):n.parent("td").focus()}))},this.destroy=function(){},this.init()},Dropdown:function(t,e,i,n,s,a,o,r){this.name="dropdown",this.originalValue=i,this.useValue=!0,this.cell=o.activeCell,this.init=function(){var c=this;if(s.inlineEditor)this.input=n.find("select");else{if(this.input=$('<select class="dropdown"></select>').appendTo(n),s.options){var d,h,u;i=o.fieldValue(r,s.field);for(var p=s.caseInsensitive&&"string"==typeof i?i.toLowerCase():i,g=0;g<s.options.length;g++)d=$("<option></option>"),h=s.options[g],u=s.caseInsensitive&&"string"==typeof h.value?h.value.toLowerCase():h.value,(h.selected||p===u)&&(d.attr("selected","true"),this.originalValue=u),d.attr("value",h.value).attr("id",h.id).attr("data-type",l(h.value)),d.text(h.label),this.input.append(d)}var f=s.editorOptions||{};f.cssClass&&/is-editing/g.test(f.cssClass)||(f.cssClass=f.cssClass||"",f.cssClass+=" is-editing"),this.input.dropdown(f),this.input.on("requestend",(function(){c.val(c.datasetValue)})),this.input.data("dropdown").settings.sourceArguments={column:s,container:n,grid:o,cell:e,event:a,row:t,rowData:r,value:i}}},this.val=function(t){if(this.datasetValue=t,void 0!==t){var e=s.caseInsensitive&&"string"==typeof t?t.toLowerCase():t;this.input.val(t),this.input.find("option").each((function(){var t=$(this),i=t.attr("value"),n=t.attr("data-type"),a=i;"number"===n?a=parseFloat(i):"boolean"===n?a="true"===i:"string"===n&&s.caseInsensitive&&(a=i.toLowerCase()),a===e&&t.attr("selected","true")}))}var i=this.input.find(":selected"),n=i.attr("value"),a=i.attr("data-type");return"number"===a?n=parseFloat(n):"boolean"===a&&(n="true"===n),void 0===n&&(n=i.text()),n},this.focus=function(){var e=this;"click"===a.type?(this.input.trigger("openlist"),o.rowNodes(t).removeClass("is-hover-row"),$("#dropdown-list input").focus()):this.input[0].parentNode.querySelector("div.dropdown").focus();if(!o.settings.selectOnEdit){var i=$("#dropdown-search")[0];if(i)if("number"==typeof i.selectionStart)i.selectionEnd=i.value.length,i.selectionStart=i.selectionEnd;else if(void 0!==i.createTextRange){var s=i.createTextRange();s.collapse(!1),s.select()}}this.input.off("listclosed").on("listclosed",(function(t,i){o.commitCellEdit(e.input),"select"!==i?"tab"===i&&setTimeout((function(){n.parent("td").focus()}),100):n.parent("td").focus()}))},this.destroy=function(){},this.init()},Date:function(t,e,i,n,s,a,o){this.name="date",this.originalValue=i,this.init=function(){this.input=$('<input class="datepicker" autocomplete="off"/>').appendTo(n),s.editorOptions||(s.editorOptions={}),s.editorOptions.dateFormat||(s.editorOptions.dateFormat=s.dateFormat),"boolean"!=typeof s.editorOptions.tabbable&&(s.editorOptions.tabbable=!1),this.input.datepicker(s.editorOptions)},this.val=function(i){return"0000"!==i&&"000000"!==i&&"00000000"!==i||(i=""),i&&this.input.val(i),Io.Date(t,e,this.input.val(),s,!0)},this.focus=function(){var t=this;this.input.focus(),o.settings.selectOnEdit&&this.input.select(),"click"===a.type&&$(a.target).is(".icon")&&this.input.parent().find(".icon").trigger("click"),this.input.on("listclosed",(function(){setTimeout((function(){t.input.trigger("focusout"),n.parent().focus(),o.setNextActiveCell(a)}),1)}))},this.destroy=function(){var t=this;setTimeout((function(){o.quickEditMode=!1,t.input.remove()}),0)},this.init()},Fileupload:function(t,e,i,n,s,a,o){var r=this,l=E.mergeSettings(void 0,s.editorOptions,{allowedTypes:"*"}),c=l.allowedTypes.split(/[\s|]+/g),d=E.uniqueId(this,"fileupload-".concat(t,"-").concat(e,"-")),h=l.useMultiple?" multiple":"",u=l.isDisabled?" disabled":"",p="";if(1===c.length)"*"!==c[0]&&(p=".".concat(g.ensureAlphaNumeric(c[0])));else for(var f=0,m=c.length;f<m;f++)p+=".".concat(g.ensureAlphaNumeric(c[f])+(f!==m-1?",":""));""!==p&&(p=' accept="'.concat(p,'"')),this.name="fileupload",this.originalValue=i,this.status="init",this.useValue=!0,this.init=function(){d=g.ensureAlphaNumeric(d),h=g.ensureAlphaNumeric(h),u=g.ensureAlphaNumeric(u),this.input=$('<input id="'.concat(d,'" name="').concat(d,'" class="fileupload" type="file" ').concat(p).concat(h).concat(u,' autocomplete="off"/>')),n.append("<label>".concat(this.input[0].outerHTML,"</label>")),this.api=this.input.fileupload(s.editorOptions).data("fileupload"),this.input.closest("td").addClass("is-fileupload").find("label:eq(1)").addClass("audible")},this.val=function(t){var e=this;return new Promise((function(i){setTimeout((function(){var n;t?(t=g.stripTags(t),e.input.attr("value",t),n=t):n=e.input.val(),i(n)}),300)}))},this.focus=function(){var t=function(){""!==r.originalValue&&(r.status="clear"),o.commitCellEdit(r.input)},e=function(){r.api&&($("body").one("focusin.fileuploadeditor",(function(){setTimeout((function(){"change"!==r.status&&(r.status="cancel",o.commitCellEdit(r.input))}),100)})),r.api.fileInput.trigger("click"))};if(r.input.on("change.fileuploadeditor",(function(){r.status="change",o.commitCellEdit(r.input)})),"keydown"===a.type){var i=a.which||a.keyCode||a.charCode||0;8===i||46===i?t():13===i||10===i?e():o.commitCellEdit(r.input)}if("click"===a.type){var n=$(a.target);n.is(".icon-close")?t():n.is(".icon-fileupload")?e():o.commitCellEdit(r.input)}},this.destroy=function(){r.input.off("change.fileuploadeditor"),o.quickEditMode=!1,r.api&&r.api.destroy()},this.init()},Time:function(t,e,i,n,s,a,o){this.name="time",this.originalValue=i,this.init=function(){this.input=$('<input class="timepicker" autocomplete="off" />').appendTo(n),s.editorOptions||(s.editorOptions={}),"boolean"!=typeof s.editorOptions.tabbable&&(s.editorOptions.tabbable=!1),this.api=this.input.timepicker(s.editorOptions).data("timepicker")},this.val=function(t){return t&&this.input.val(t),this.input.val()},this.focus=function(){var t=this;this.input.focus(),o.settings.selectOnEdit&&this.input.select(),"click"===a.type&&$(a.target).is(".icon")&&this.input.parent().find(".icon").trigger("click"),this.api.trigger.on("hide.editortime",(function(){setTimeout((function(){t.input.trigger("focusout"),n.parent().focus(),o.setNextActiveCell(a)}),1)}))},this.destroy=function(){var t=this;t.api&&t.api.trigger&&t.api.trigger.off("hide.editortime"),setTimeout((function(){o.quickEditMode=!1,t.input.remove()}),0)},this.init()},Lookup:function(t,e,i,n,s,a,o,r){this.name="lookup",this.originalValue=i,this.init=function(){this.input=$('<input class="lookup '.concat("right"===s.align?"align-text-right":"",'" data-init="false" autocomplete="off" />')).appendTo(n),Wo(this.input,t,e,i,0,s,0,0,r),s.editorOptions||(s.editorOptions={}),s.editorOptions.options||(s.editorOptions.options={}),s.editorOptions.options.toolbar||(s.editorOptions.options.toolbar={}),s.editorOptions.options.toolbar.title||(s.editorOptions.options.toolbar.title=s.name),void 0===s.editorOptions.tabbable&&(s.editorOptions.tabbable=!1),this.input.lookup(s.editorOptions),n.find("span.trigger").attr("tabindex","-1"),this.input.data("lookup").settings.clickArguments={column:s,container:n,grid:o,cell:e,event:a,row:t,rowData:r,value:i}},this.val=function(t){return t?this.input.val(t):this.input.val()},this.focus=function(){var t=this,e=t.input.data("lookup"),i=t.input.closest("td");"keydown"===a.type&&(t.input.focus(),o.settings.selectOnEdit&&t.input.select(),i.on("keydown.editorlookup",(function(t){40===t.keyCode&&(t.preventDefault(),t.stopPropagation())}))),"click"===a.type&&($(a.target).is("svg")?e.openDialog(a):(t.input.focus(),o.settings.selectOnEdit&&t.input.select(),i.on("touchcancel.editorlookup touchend.editorlookup",".trigger",(function(){e.openDialog()})))),t.input.on("change.editorlookup",(function(){o.quickEditMode=!1}))},this.destroy=function(){var t=this,e=this.input.closest("td");setTimeout((function(){o.quickEditMode=!1,e.off("keydown.editorlookup").find(".trigger").off("touchcancel.editorlookup touchend.editorlookup"),t.input.remove()}),0)},this.init()},Autocomplete:function(t,e,i,n,s,a,o){this.name="autocomplete",this.originalValue=i,this.init=function(){this.input=$('<input class="autocomplete datagrid-autocomplete" autocomplete="off" />').appendTo(n),s.editorOptions||(s.editorOptions={}),s.editorOptions.width=n.parent().width(),s.editorOptions.offset={left:-1,top:"medium"===o.settings.rowHeight?1:5},s.maxLength&&this.input.attr("maxlength",s.maxLength),s.uppercase&&this.input.addClass("uppercase-text"),this.input.autocomplete(s.editorOptions)},this.val=function(t){return t?this.input.val(t):this.input.val()},this.focus=function(){o.quickEditMode=!0,this.input.focus(),o.settings.selectOnEdit&&this.input.select()},this.destroy=function(){var t=this;setTimeout((function(){o.quickEditMode=!1,t.input.remove()}),0)},this.init()},Spinbox:function(t,e,i,n,s,a,o){var r=this;this.name="spinbox",this.originalValue=i,this.useValue=!0,this.init=function(){if(s.inlineEditor)r.input=n.find("input");else{var t='<label for="spinbox-'.concat(e,'" class="audible">Quantity</label>\n        <span class="spinbox-wrapper"><span class="spinbox-control down">-</span>\n        <input id="spinbox-').concat(e,'" name="spinbox-').concat(e,'" type="text" class="spinbox" value="\'+ ').concat(i,' +\'" autocomplete="off" />\n        <span class="spinbox-control up">+</span></span>');f.append(n,t,"<label><span><input>"),r.input=n.find("input"),s.editorOptions||(s.editorOptions={}),r.input.spinbox(s.editorOptions)}},this.val=function(t){return t&&r.input.val(t),parseInt(r.input.val(),10)},this.focus=function(){o.quickEditMode=!0,r.input.focus(),o.settings.selectOnEdit&&r.input.select()},this.destroy=function(){s.inlineEditor||setTimeout((function(){o.quickEditMode=!1;var t=r.val();r.input&&r.input.data("spinbox")&&r.input.data("spinbox").destroy(),r.input&&r.input.remove(),n.text(t)}),0)},this.init()},Favorite:function(t,e,i,n,s,a,o){this.name="favorite",this.useValue=!0,this.originalValue=i,this.init=function(){this.input=$('<span class="icon-favorite">'.concat($.createIcon({icon:i?"star-filled":"star-outlined"}),'<input type="checkbox"></span>')).appendTo(n),this.input=this.input.find("input")},this.val=function(t){var e;if(void 0===t)return this.input.prop("checked");e=s.isChecked?s.isChecked(t):t,"click"!==a.type&&("keydown"!==a.type||32!==a.keyCode)||$(a.target).is(".datagrid-cell-wrapper")||(e=!e,o.setNextActiveCell(a)),this.input.prop("checked",e),this.input.find("use").attr("href",e?"#icon-star-filled":"#icon-star-outlined")},this.focus=function(){this.input.trigger("focusout").focus()},this.destroy=function(){var t=this;setTimeout((function(){t.input.parent().remove()}),0)},this.init()}},Uo="datagrid",Go={actionableMode:!1,cellNavigation:!0,rowNavigation:!0,showHoverState:!0,alternateRowShading:!1,columns:[],frozenColumns:{left:[],right:[],expandRowAcrossAllCells:!0},dataset:[],columnReorder:!1,saveColumns:!1,saveUserSettings:{},focusAfterSort:!1,editable:!1,selectOnEdit:!0,isRowDisabled:null,isList:!1,menuId:null,headerMenuId:null,menuSelected:null,menuBeforeOpen:null,headerMenuSelected:null,headerMenuBeforeOpen:null,uniqueId:null,rowHeight:"large",fixedRowHeight:null,selectable:!1,selectChildren:!0,onBeforeSelect:null,allowSelectAcrossPages:null,selectAllCurrentPage:!1,groupable:null,showNewRowIndicator:!0,stretchColumn:null,stretchColumnOnChange:!1,spacerColumn:!1,stickyHeader:!1,columnSizing:"all",twoLineHeader:!1,clickToSelect:!0,keyRowSelect:!1,toolbar:!1,initializeToolbar:!0,columnIds:[],paging:!1,pagesize:25,pagesizes:[],showPageSizeSelector:!0,indeterminate:!1,source:null,hidePagerOnOnePage:!1,filterable:!1,filterWhenTyping:!0,disableClientFilter:!1,disableClientSort:!1,resultsText:null,showFilterTotal:!0,virtualized:!1,virtualRowBuffer:10,rowReorder:!1,showDirty:!1,resizeMode:"flex",showSelectAllCheckBox:!0,allowOneExpandedRow:!0,enableTooltips:!1,disableRowDeactivation:!1,disableRowDeselection:!1,sizeColumnsEqually:!1,expandableRow:!1,exportConvertNegative:!1,columnGroups:null,treeGrid:!1,onPostRenderCell:null,onDestroyCell:null,onEditCell:null,onExpandRow:null,onExpandChildren:null,onCollapseChildren:null,onKeyDown:null,emptyMessage:{title:H?H.translate("NoData"):"No data available",info:"",icon:"icon-empty-no-data-new",height:null},searchExpandableRow:!0,allowChildExpandOnMatchOnly:!1,allowChildExpandOnMatch:!1,activeCheckboxSelection:!1,dblClickApply:!1,attributes:null,allowPasteFromExcel:!1,fallbackImage:"insert-image",fallbackSize:{height:40,width:40},fallbackTooltip:{content:"Image could not load",delay:200}};function _o(t,e){this.settings=E.mergeSettings(t,e,Go),this.element=$(t),e.dataset&&(this.settings.dataset=e.dataset),void 0===this.settings.frozenColumns.expandRowAcrossAllCells&&(this.settings.frozenColumns.expandRowAcrossAllCells=Go.frozenColumns.expandRowAcrossAllCells),this.init()}_o.prototype={get pagerAPI(){var t;return this.tableBody&&this.tableBody.length&&(t=this.tableBody.data("pager")),t},init:function(){var t=$("html");this.isTouch=x.features.touch,this.isSafari=t.is(".is-safari"),this.isWindows=-1!==navigator.userAgent.indexOf("Windows"),this.appendTooltip(),this.initSettings(),this.setOriginalColumns(),this.removeToolbarOnDestroy=!1,this.nonVisibleCellErrors=[],this.recordCount=0,this.canvas=null,this.totalWidths={left:0,center:0,right:0},this.stretchColumnWidth=0,this.stretchColumnDiff=0,this.stretchColumnIdx=-1,this.editor=null,this.activeCell={node:null,cell:null,row:null},this.dontSyncUi=!1,this.widthPercent=!1,this.rowSpans=[],this.filterRowRendered=!1,this.scrollLeft=0,this.scrollTop=0,this._selectedRows=[],this.restoreColumns(),this.restoreUserSettings(),this.appendToolbar(),this.setTreeDepth(),this.setRowGrouping(),this.setTreeRootNodes(),this.firstRender(),this.hasScrollbarY(),this.handleEvents(),this.handleKeys(),this.element.trigger("rendered",[this.element,this.element.find("thead"),this.pagerAPI])},initSettings:function(){this.ignoredColumnById("rowStatus"),this.setInlineActionableMode(),this.sortColumn={sortField:null,sortAsc:!0},this.gridCount=$(".datagrid").length+1,this.lastSelectedRow=0,this.contextualToolbar=this.element.prev(".contextual-toolbar"),this.contextualToolbar.addClass("datagrid-contextual-toolbar")},ignoredColumnById:function(t){var e=this.settings;if(t&&e.columns&&(!e.columns||e.columns.length)){for(var i={index:-1},n=0,s=e.columns.length;n<s;n++)e.columns[n].id===t&&(i.index=n);i.index>-1&&e.columns.splice(i.index,1)}},setInlineActionableMode:function(){var t=this.settings;if(t.columns&&(!t.columns||t.columns.length)){for(var e={index:-1},i=0,n=t.columns.length;i<n;i++)t.columns[i].inlineEditor&&(e.index=i);this.inlineMode=e.index>-1,this.inlineMode?(this.element[0].classList.add("has-inline-editor"),this.settings.actionableMode=!0):this.element[0].classList.remove("has-inline-editor")}},render:function(t,e,i){if(e||(e={}),t&&(e.type="filterrow"),this.settings.source)return e.preserveSelected=this.settings.allowSelectAcrossPages,void this.triggerSource(i?"initial":e);this.loadData(this.settings.dataset,e)},firstRender:function(){var t=this,e=t.settings.columns.some((function(t){return void 0!==t.width}));if(this.hasLeftPane=this.settings.frozenColumns.left.length>0,this.hasRightPane=this.settings.frozenColumns.right.length>0,this.hasLeftPane&&(t.bodyWrapperLeft=$('<div class="datagrid-wrapper left"></div>'),t.tableLeft=$("<table></table>").addClass("datagrid").attr("role",this.settings.treeGrid?"treegrid":"grid").appendTo(t.bodyWrapperLeft),t.element.append(t.bodyWrapperLeft)),t.bodyWrapperCenter=$('<div class="datagrid-wrapper center scrollable-x'.concat(this.hasRightPane?"":" scrollable-y",'"></div>')),t.table=$("<table></table>").addClass("datagrid").attr("role",this.settings.treeGrid?"treegrid":"grid").appendTo(t.bodyWrapperCenter),t.element.append(t.bodyWrapperCenter),e&&t.table.css("width","inherit"),this.hasRightPane&&(t.bodyWrapperRight=$('<div class="datagrid-wrapper right scrollable-y"></div>'),t.tableRight=$("<table></table>").addClass("datagrid").attr("role",this.settings.treeGrid?"treegrid":"grid").appendTo(t.bodyWrapperRight),t.element.append(t.bodyWrapperRight)),this.element.removeClass("datagrid").addClass("datagrid-container").attr("x-ms-format-detection","none"),this.settings.stickyHeader&&this.element.removeClass("datagrid").addClass("is-sticky"),"normal"!==this.settings.rowHeight&&"large"!==this.settings.rowHeight){var i="short"===this.settings.rowHeight?"small":this.settings.rowHeight;i="normal"===this.settings.rowHeight?"large":i,this.hasLeftPane&&t.tableLeft.addClass("".concat(i,"-rowheight")),t.table.addClass("".concat(i,"-rowheight")),this.hasRightPane&&t.tableRight.addClass("".concat(i,"-rowheight")),this.element.addClass("".concat(i,"-rowheight"))}(this.settings.isList?$(this.element).addClass("is-gridlist"):$(this.element).removeClass("is-gridlist"),t.table.empty(),t.clearCache(),t.container=t.element.closest(".datagrid-container"),t.renderRows(),t.renderHeader(),this.stretchColumnDiff<0)&&(this.bodyColGroup.find("col").eq(t.getStretchColumnIdx())[0].style.width="".concat(this.stretchColumnWidth,"px"));this.settings.emptyMessage&&(t.setEmptyMessage(this.settings.emptyMessage),t.checkEmptyMessage()),t.buttonSelector=".btn, .btn-secondary, .btn-primary, .btn-modal-primary, .btn-tertiary, .btn-icon, .btn-actions, .btn-menu, .btn-split",$(t.buttonSelector,t.table).button(),this.handlePaging(),this.triggerSource("initial")},hasScrollbarY:function(){var t=this,e=parseInt(t.bodyWrapperCenter[0].offsetHeight,10),i=parseInt(t.bodyWrapperCenter[0].scrollHeight,10)>e+2||t.element[0].scrollHeight>t.element[0].clientHeight;t.hasVerticalScrollbar=!1!==i},editCell:function(t,e,i){var n=this;setTimeout((function(){n.setActiveCell(t,e),n.makeCellEditable(t,e,i)}),100)},renderRow:function(t,e){var i=this,n="bottom"!==e,s="",a="",o="";function r(){n?(i.hasLeftPane&&f.prepend(i.tableBodyLeft,a,"*"),f.prepend(i.tableBody,s,"*"),i.hasRightPane&&f.prepend(i.tableBodyRight,o,"*")):(i.hasLeftPane&&f.append(i.tableBodyLeft,a,"*"),f.append(i.tableBody,s,"*"),i.hasRightPane&&f.append(i.tableBodyRight,o,"*"))}i.emptyMessageContainer&&i.emptyMessageContainer.hide();var l="bottom"===e?"last":"first",c="last"===l?i.settings.dataset.length-1:0,d=i.settings.dataset.length-1,h=i.rowHtml(t,c,d);if(i.hasLeftPane&&h.left&&(a+=h.left),h.center&&(s+=h.center),i.hasRightPane&&h.right&&(o+=h.right),i.settings.groupable)for(var u=$(".datagrid-rowgroup-header").find("span:not([class])"),p=0;p<u.length;p++){var g=$(u[p]);if(t.type===g.text().trim()){f.after(g.parents("tr"),h.center,"*");break}}if(i.settings.paging&&!i.settings.groupable){var m=i.settings.dataset.length-i.pagerAPI.pageCount()*i.pagerAPI.settings.pagesize==1?1:0,v=("bottom"===e?i.pagerAPI.pageCount():1)+m;"bottom"!==e&&1===i.pagerAPI.activePage?r():(i.pagerAPI.setActivePage(v,!1,l),i.pagerAPI.triggerPagingEvents(i.pagerAPI.currentPage))}i.settings.paging||i.settings.groupable||r(),i.settings.paging&&i.tableBody.children().length>i.pagerAPI.settings.pagesize&&i.tableBody.children().last().remove(),i.refreshIndexes(),i.afterRender()},refreshIndexes:function(){this.element.find("tr").removeAttr("aria-rowindex"),this.element.find("tr").removeAttr("data-index"),this.hasLeftPane&&this.element.find(".datagrid-wrapper.left tbody > tr").each((function(t,e){var i=$(e);i.attr("aria-rowindex",t+1),i.attr("data-index",t)}));var t=this.settings.paging&&this.pagerAPI.activePage>1?(this.pagerAPI.activePage-1)*this.settings.pagesize:0;this.element.find(".datagrid-wrapper.center tbody > tr").each((function(e,i){var n=$(i);n.attr("aria-rowindex",e+t+1),n.attr("data-index",e+t)})),this.hasRightPane&&this.element.find(".datagrid-wrapper.right tbody > tr").each((function(t,e){var i=$(e);i.attr("aria-rowindex",t+1),i.attr("data-index",t)}))},addRow:function(t,e){var i,n,s=this,a=this;this.saveDirtyRows();var o=this.addToDataset([t],e)[0];this.restoreDirtyRows(),this.setRowGrouping(),this.settings.groupable||this.pagerRefresh(e),this._selectedRows.forEach((function(t){void 0!==t.pagingIdx&&t.pagingIdx>=o&&(t.idx++,t.pagingIdx++)})),this.clearCache(),this.renderRow(t,e),this.settings.groupable&&(n=this.dataRowNode(o),o=this.visualRowIndex(n)),this.syncSelectedUI(),setTimeout((function(){a.setActiveCell(o,0),s.settings.groupable||(n=s.visualRowNode(o)),i={row:o,cell:0,target:n,value:t,oldValue:{}},a.element.triggerHandler("addrow",i)}),100)},addRows:function(t,e){var i,n=this,s=this;this.saveDirtyRows();var a=this.addToDataset(t,e);if(-1!==a){this.restoreDirtyRows(),this.setRowGrouping(),this.settings.groupable||this.pagerRefresh(e),this._selectedRows.forEach((function(t){a.forEach((function(e){void 0!==t.pagingIdx&&t.pagingIdx>=e&&(t.idx++,t.pagingIdx++)}))})),this.clearCache(),this.renderRows();var o=[];this.settings.groupable&&a.forEach((function(t,e){var i=n.dataRowNode(t);o.push(i),a[e]=n.visualRowIndex(i)})),this.syncSelectedUI(),setTimeout((function(){s.setActiveCell(a[0],0),n.settings.groupable||a.forEach((function(t){o.push(n.visualRowNode(t))})),i={rows:a,cell:0,target:o,value:t,oldValue:{}},s.element.triggerHandler("addrow",i)}),100)}},addToDataset:function(t,e){if(t&&0===t.length)return-1;var i=this.settings,n=[],s={icon:"new",text:H.translate("NewRow"),tooltip:H.translate("NewRow")},a=!1,o=this.settings.dataset;function r(t,e){t.rowStatus=t.rowStatus||s,o.splice(e,0,t),n.push(e)}if(e&&"top"!==e||(e="top",a=!0),i.groupable){o=this.originalDataset||o;if("string"==typeof e){var l=this.settings.groupable.fields[0];t.forEach((function(t){var e=function(t,e,i,n){for(var s=0,a=t.length;s<a;s++){var o=n?a-1-s:s;if(t[o][e]===i)return o}return-1}(o,l,t[l],!a);r(t,e>-1?a?e:e+1:0)}))}else t.forEach((function(t,i){r(t,e+i)}))}else"string"==typeof e?t.forEach((function(t,e){r(t,a?e:o.length)})):t.forEach((function(t,i){r(t,e+i)}));return n},pagerRefresh:function(t,e){if(this.pagerAPI){var i={};if("string"==typeof t?i.activePage="top"===t?1:this.pagerAPI.state.pages:"number"==typeof t&&(i.activePage=Math.floor(t/(this.pagerAPI.settings.pagesize+1))),this.settings.source||(i.total=this.settings.dataset.length,i.pagesize=this.settings.pagesize),e){var n=Math.ceil(this.settings.dataset.length/this.settings.pagesize);i.activePage=this.pagerAPI.activePage>n?n:this.pagerAPI.activePage}this.renderPager(i,!0)}},removeRow:function(t,e,i){var n=this.tableBody.find('tr[aria-rowindex="'.concat(t+1,'"]')),s=this.settings.groupable&&this.originalDataset?this.originalDataset:this.settings.dataset,a=s[t];this.saveDirtyRows(),this.unselectRow(t,e),s.splice(t,1),this.restoreDirtyRows(),this.settings.selectable&&!this.settings.groupable&&this.settings.groupable&&this.originalDataset&&this.syncDatasetWithSelectedRows(),this.preventSelection=!0,e||(this.setRowGrouping(),this.pagerRefresh("top",!0),this.clearCache(),this.renderRows()),0!==this.nonVisibleCellErrors.length&&(this.nonVisibleCellErrors=$.grep(this.nonVisibleCellErrors,(function(e){return e.row!==t})),this.showNonVisibleCellErrors()),delete this.preventSelection,this.syncSelectedUI();var o={row:t,cell:null,target:n,item:a,oldValue:a};return i?o:(this.element.trigger("rowremove",o),!0)},removeSelected:function(t){this._selectedRows.sort((function(t,e){return t.idx<e.idx?-1:t.idx>e.idx?1:0}));for(var e=[],i=this._selectedRows.length-1;i>=0;i--)e.push({row:this._selectedRows[i].idx,item:this._selectedRows[i],cell:null,target:null}),this.removeRow(this._selectedRows[i].idx,i>0,t);t&&this.element.trigger("rowremove",e)},updateDataset:function(t,e){if(this.settings.toolbar&&this.settings.toolbar.keywordFilter){var i=this.element.parent().find(".toolbar").find(".searchfield");i.val(""),i.parent().removeClass("has-text"),this.clearFilter()}this.loadData(t,e),this.syncSelectedRowsIdx()},triggerSource:function(t,e,i){if(this.settings.source){var n=this,s={};this.pagerAPI&&(s=this.pagerAPI.state),"string"==typeof t?(s.type=t,s.trigger=i):t&&(s=E.extend({},s,t)),e&&"function"!=typeof e&&"string"==typeof e.type&&(s.type=e.type),!1!==this.element.triggerHandler("beforepaging",s)&&(this.sortColumn&&this.sortColumn.sortId&&(s.sortAsc=this.sortColumn.sortAsc,s.sortField=this.sortColumn.sortField,s.sortId=this.sortColumn.sortId),this.filterExpr&&this.filterExpr.length&&(s.filterExpr=this.filterExpr),this.element.trigger("paging",s),this.settings.source(s,(function(t,i){i.activePage>-1&&(n.activePage=i.activePage),i.grandTotal&&(n.grandTotal=i.grandTotal),n.loadData(t,i),e&&"function"==typeof e&&e(!0),setTimeout((function(){n.afterPaging(s),n.element.trigger("afterpaging",s)}))})))}},afterPaging:function(t){this.settings.paging&&(this.settings.source&&(this.pagerAPI&&this.pagerAPI.hidePagerBar(t)?this.element.removeClass("paginated"):this.element.addClass("paginated"),t.total&&(this.recordCount=t.total,this.displayCounts(t.total))),!this.settings.source&&this.filterExpr&&this.filterExpr[0]&&"all"===this.filterExpr[0].column&&this.highlightSearchRows(this.filterExpr[0].value))},loadData:function(t,e){var i,n;this.settings.dataset=t,e||(e={}),"filterrow"===e.type&&(e.activePage=this.pagerAPI&&this.pagerAPI.activePage||1,e.pagesize=this.settings.pagesize,e.total=e.total||-1,e.type="filterrow"),e.activePage||(e.activePage=1,e.pagesize=this.settings.pagesize,e.total=-1,this.settings.treeGrid&&(e.preserveSelected=!0)),this.settings.source&&e.grandTotal?this.grandTotal=e.grandTotal:this.grandTotal=null,this.pagerAPI&&this.settings.showDirty&&this.settings.source&&/first|last|next|prev|sorted/.test(e.type)&&(this.dirtyArray=void 0),this.settings.groupable&&this.settings.dataset[0]&&!this.settings.dataset[0].values&&(this._selectedRows=[],this.originalDataset=null,this.clearDirty()),this.setTreeDepth(),this.settings.source&&(this.originalDataset=null,this.setRowGrouping()),this.setTreeRootNodes(),e.preserveSelected||!this.settings.source||this.settings.allowSelectAcrossPages||(this.settings.selectAllCurrentPage?this.unSelectAllRowsCurrentPage():this.unSelectAllRows()),e.preserveSelected||"initial"!==e.type&&e.type||(this.settings.selectAllCurrentPage?this.unSelectAllRowsCurrentPage():this.unSelectAllRows()),this.settings.disableClientFilter&&(this.restoreFilter=!0,this.restoreSortOrder=!0,this.savedFilter=this.filterConditions()),"initial"===e.type?(this.clearCache(),this.restoreUserSettings(),this.renderRows(),this.renderHeader()):0===this.element.find(".datagrid-filter-wrapper .is-open").length?(this.clearCache(),this.setTreeDepth(),this.setRowGrouping(),this.setTreeRootNodes(),this.renderRows()):(this.clearCache(),this.renderRows()),this.cellNode(0,0,!0).attr("tabindex","0"),this.renderPager(e,!0),this.displayCounts(),this.settings.paging&&this.settings.source&&null!==(i=e)&&void 0!==i&&(null==i?void 0:i.filterExpr)&&"all"===(null===(n=e.filterExpr[0])||void 0===n||null==n?void 0:n.column)&&this.highlightSearchRows(e.filterExpr[0].value)},uniqueId:function(t){return t=null==t?"":t,this.gridCount=this.gridCount?this.gridCount:$(".datagrid").length+1,(this.settings.uniqueId?"".concat(this.settings.uniqueId,"-").concat(t):"".concat(window.location.pathname.split("/").pop().replace(/\.xhtml|\.shtml|\.html|\.htm|\.aspx|\.asp|\.jspx|\.jsp|\.php/g,"").replace(/[^-\w]+/g,"").replace(/\./g,"-").replace(/ /g,"-").replace(/%20/g,"-"),"-").concat(this.element.attr("id")||"datagrid","-").concat(this.gridCount||0).concat(t)).replace(/--/g,"-")},visibleColumns:function(t){for(var e=[],i=0;i<this.settings.columns.length;i++){var n=this.settings.columns[i];n.hidden||(t&&"selectionCheckbox"===n.id||e.push(n))}return e},getColumnGroup:function(t){for(var e=0,i=this.settings.columnGroups,n=0;n<i.length;n++)if(!i[n].hidden&&(e+=i[n].colspan)>=t)return this.uniqueId("-header-group-".concat(n));return""},updateColumnGroup:function(t,e){var i=this.settings.columnGroups;if(this.originalColGroups||(this.originalColGroups=E.deepCopy(i)),this.settings.groupable)for(var n=this.tableBody.find(".datagrid-rowgroup-header"),s=this.visibleColumns().length,a=0;a<n.length;a++)n[a].children[0].setAttribute("colspan",s);else if(i&&this.colGroups){var o=0,r=-1;this.originalColGroups.forEach((function(e,i){var n=parseInt(e.colspan,10);t>=o&&t<=o+n&&(r=i),o+=n}));var l=parseInt(this.colGroups[r].getAttribute("colspan"),10),c=this.colGroups[r].classList.contains("is-hidden");if(!e||!c)return e&&!c?(this.colGroups[r].classList.remove("is-hidden"),void this.colGroups[r].setAttribute("colspan",l+1)):void(l-1!=0?this.colGroups[r].setAttribute("colspan",l-1):this.colGroups[r].classList.add("is-hidden"));this.colGroups[r].classList.remove("is-hidden")}},updateGroupHeadersAfterColumnReorder:function(t,e){var i=this.settings.columnGroups;if(i){this.originalColGroups||(this.originalColGroups=E.deepCopy(i));var n=i.map((function(t){return parseInt(t.colspan,10)})),s={from:null,to:null,total:0};n.forEach((function(i,n){s.total+=i,s.total>t&&null===s.from&&(s.from=n),s.total>e&&null===s.to&&(s.to=n)})),s.from!==s.to&&(i[s.from].colspan-=1,i[s.to].colspan+=1)}},headerText:function(t){var e=t.name?g.stripTags(t.name,"<div><span><a><small><svg><i><b><use><br><strong><em>"):"";return e||"drilldown"!==t.id?e:(e=H.translate("Drilldown"),'<span class="audible">'.concat(e,"</span>"))},getContainer:function(t){return this.settings.frozenColumns.left&&this.settings.frozenColumns.left.indexOf(t)>-1?"left":this.settings.frozenColumns.right&&this.settings.frozenColumns.right.indexOf(t)>-1?"right":"center"},renderHeader:function(){var t,e=this,i={left:"",center:"",right:""},n=this.settings.columnGroups;if(n){this.element.addClass("has-group-headers");var s,a=this.settings.columns,o=a.length,r=this.visibleColumns().length,l=n.map((function(t){return parseInt(t.colspan,10)})),c=function(){return l.reduce((function(t,e){return t+e}),0)},d=function(){var t=c();return t>o?t-o:o-t};if(i.left+='<tr role="row" class="datagrid-header-groups">',i.center+='<tr role="row" class="datagrid-header-groups">',i.right+='<tr role="row" class="datagrid-header-groups">',c()>o)for(var h=!0,u=l.length-1;u>=0&&h;u--)s=d(),l[u]>=s?(l[u]-=s,h=!1):l[u]=0;var p=0,g=0;if(l.forEach((function(s,r){for(var l=s,c=p+s;p<c;p++)p<o&&a[p].hidden&&l--;var d=n[r].hidden||l<1?' class="is-hidden"':"",h=' colspan="'.concat(l>0?l:1,'"'),u=n[r].align?"l-".concat(n[r].align,"-text"):"";t=e.uniqueId("-header-group-".concat(r)),l>0&&(g+=l);var f=e.getContainer(e.settings.columns[r].id);i[f]+="<th".concat(d).concat(h,' id="').concat(t,'" class="').concat(u,'"><div class="datagrid-column-wrapper"><span class="datagrid-header-text">').concat(n[r].name,"</span></div></th>")})),g<r){var m=' colspan="'.concat((s=r-g)>0?s:1,'"');e.hasRightPane?i.right+="<th".concat(m,"></th>"):i.center+="<th".concat(m,"></th>")}this.settings.spacerColumn&&(i.center+='<th class="datagrid-header-groups-spacer-column"></th>'),i.left+="</tr><tr>",i.center+="</tr><tr>",i.right+="</tr><tr>"}else i.left+='<tr role="row">',i.center+='<tr role="row">',i.right+='<tr role="row">';for(var v=0;v<this.settings.columns.length;v++){var b,y,w,x,C,k,T=e.settings.columns[v],A=e.getContainer(T.id),S=e.uniqueId("-header-".concat(v)),D=void 0===T.sortable||T.sortable,I=void 0===T.resizable||T.resizable,L=void 0===T.exportable||T.exportable,P="selectionCheckbox"===T.id,M=this.getHeaderAlignmentClass(T);(e.hasLeftPane||e.hasRightPane)&&(e.settings.frozenColumns.left&&e.settings.frozenColumns.left.indexOf(T.id)>-1||e.settings.frozenColumns.right&&e.settings.frozenColumns.right.indexOf(T.id)>-1)&&(T.hideable=!1),void 0===T.hideable&&(T.hideable=!0);var B="";B+=D?" is-sortable":"",B+=I?" is-resizable":"",B+=T.hidden?" is-hidden":"",B+=T.filterType?" is-filterable":"",B+="ellipsis"===T.textOverflow?" text-ellipsis":"",B+=null!=T&&(null==T?void 0:T.headerIcon)?" header-icon":"",B+=null!=T&&(null==T?void 0:T.headerCssClass)?" ".concat(T.headerCssClass):"",B=""!==(B+=""!==M?M:"")?' class="'.concat(B.substr(1),'"'):"";var O=E.stringAttributes(this,this.settings.attributes,"col-".concat(null===(b=T.id)||void 0===b||null==(y=b)?void 0:y.toLowerCase()));O||(O='id="'.concat(S,'"')),i[A]+='<th scope="col" role="columnheader" '.concat(O," ").concat(P?' aria-checked= "false"':"",' data-column-id="').concat(T.id,'"').concat(T.field?' data-field="'.concat(T.field,'"'):"").concat(T.headerTooltip?' title="'.concat(T.headerTooltip,'"'):"").concat(T.headerTooltipCssClass?' tooltipClass="'.concat(T.headerTooltipCssClass,'"'):"").concat(!1===T.reorderable?' data-reorder="false"':"").concat(n?' headers="'.concat(e.getColumnGroup(v),'"'):"",' data-exportable="').concat(L?"yes":"no",'"').concat(B,">");var R="";D&&(R="".concat('<div class="sort-indicator"><span class="sort-asc">').concat($.createIcon({icon:"dropdown"}),"</span>")+'<span class="sort-desc">'.concat($.createIcon({icon:"dropdown"}),"</div>"));var z=void 0!==(null==T||null==T?void 0:T.headerIconTooltip)||(null==T||null===(w=null==T?void 0:T.headerIconTooltip)||void 0===w||null==(x=w)?void 0:x.length)>0?null==T||null==T?void 0:T.headerIconTooltip:"",F='\n        <svg class="icon datagrid-header-icon" focusable="false" aria-hidden="true" role="presentation" title="'.concat(z,'">\n          <use href="#icon-').concat(T.headerIcon,'"></use>\n        </svg>\n      ');i[A]+='<div class="'.concat(P?"datagrid-checkbox-wrapper ":"datagrid-column-wrapper").concat(M,'">\n      <span class="datagrid-header-text').concat(T.required?" required":"",'">').concat(e.headerText(this.settings.columns[v])).concat(" l-center-text"===M?R:"","</span>\n      ").concat(null!==(C=this.settings.columns[v])&&void 0!==C&&(null==(k=C)?void 0:k.headerIcon)?F:""),P&&(e.settings.showSelectAllCheckBox?i[A]+='<span class="datagrid-checkbox" aria-label="Selection" role="checkbox" tabindex="0"></span>':i[A]+='<span class="datagrid-checkbox" aria-label="Selection" role="checkbox" style="display:none" tabindex="0"></span>'),D&&" l-center-text"!==M&&(i[A]+=R),i[A]+="</div>".concat(e.filterRowHtml(T,v),"</th>")}this.settings.spacerColumn&&(i.center+='<th class="datagrid-header-spacer-column"></th>'),i.left+="</tr>",i.center+="</tr>",i.right+="</tr>",void 0===e.headerRow?(e.hasLeftPane&&(e.headerRowLeft=$('<thead class="datagrid-header left" role="rowgroup">'.concat(i.left,"</thead>")),e.tableLeft.find("colgroup").after(e.headerRowLeft)),e.headerRow=$('<thead class="datagrid-header center"> role="rowgroup"'.concat(i.center,"</thead>")),e.table.find("colgroup").after(e.headerRow),e.hasRightPane&&(e.headerRowRight=$('<thead class="datagrid-header right" role="rowgroup">'.concat(i.right,"</thead>")),e.tableRight.find("colgroup").after(e.headerRowRight))):(e.hasLeftPane&&f.html(e.headerRowLeft,i.left,"*"),f.html(e.headerRow,i.center,"*"),e.hasRightPane&&f.html(e.headerRowRight,i.right,"*")),n&&e.headerRow&&(e.colGroups=$.makeArray(this.container.find(".datagrid-header-groups th"))),e.syncHeaderCheckbox(this.settings.dataset),e.setScrollClass(),e.attachFilterRowEvents(),e.settings.columnReorder&&e.createDraggableColumns(),this.restoreSortOrder=!1,this.setSortIndicator(this.sortColumn.sortId,this.sortColumn.sortAsc),this.restoreFilter?(this.restoreFilter=!1,this.applyFilter(this.savedFilter,"restore"),this.savedFilter=null):this.filterExpr&&this.filterExpr.length>0&&this.setFilterConditions(this.filterExpr),this.activeEllipsisHeaderAll()},getHeaderAlignmentClass:function(t){return void 0===t.headerAlign?t.align?" l-".concat(t.align,"-text"):"":" l-".concat(t.headerAlign,"-text")},filterSetDatepicker:function(t,e,i){var n=t.data("datepicker"),s=t.data("is-range");i=i||{};var a=function(){n&&"function"==typeof n.destroy&&n.destroy(),t.datepicker(i)};if(n&&s||"in-range"!==e)n&&!s||"in-range"===e||(i.range={useRange:!1},t.removeData("is-range"),t.val(t.val().split(" - ")[0]),a());else{if(t.data("is-range",!0),0!==t.val().length&&-1===t.val().indexOf("-")){var o=t.parents("th").attr("data-column-id"),r=this.columnById(o)[0].dateFormat,l="MM/dd/yyyy"===r?H.formatDate(new Date(t.val()),{pattern:r}):H.formatDate(t.val(),{pattern:r});t.val("".concat(l," - ").concat(l)),i.range={useRange:!0,start:l,end:l}}else i.range={useRange:!0};a()}},filterRowHtml:function(t,e){var i=this,n="",s=this.getHeaderAlignmentClass(t);if(t.filterType){var a,o,r=t,l=i.uniqueId("-header-filter-".concat(e)),c=Array.isArray(r.filterRowEditorOptions)?r.filterRowEditorOptions:r.options,d="";if("select"===r.filterType&&c){for(var h=!1,u=0,p=c.length;u<p;u++)if(!c[u].label){h=!0;break}h||(d="<option></option>")}n='<div class="datagrid-filter-wrapper'.concat(s,'" ').concat(i.settings.filterable?"":' style="display:none"',">").concat(i.filterButtonHtml(r),'<label class="audible" for="').concat(l,'">').concat(r.name,"</label>");var g=E.stringAttributes(this,this.settings.attributes,"filter-".concat(null===(a=r.id)||void 0===a||null==a?void 0:a.toLowerCase()));g||(g='id="'.concat(l,'"'));var f=r.enterKeyHint?' enterkeyhint="'.concat(r.enterKeyHint,'"'):' enterkeyhint="enter"';switch(r.filterType){case"checkbox":break;case"date":n+="<input ".concat(r.filterDisabled?" disabled":"",' type="text" class="datepicker" ').concat(g," ").concat(f,"/>");break;case"integer":o={patternOptions:{allowNegative:!0,allowThousandsSeparator:!1,allowDecimal:!1,symbols:{thousands:H.currentLocale.data.numbers?H.currentLocale.data.numbers.group:",",decimal:H.currentLocale.data.numbers?H.currentLocale.data.numbers.decimal:".",negative:H.currentLocale.data.numbers?H.currentLocale.data.numbers.minusSign:"-"}},process:"number"},r.filterMaskOptions&&(r.filterMaskOptions=E.extend(!0,{},o,r.filterMaskOptions)),r.maskOptions&&(r.maskOptions=E.extend(!0,{},o,r.maskOptions)),n+="<input".concat(r.filterDisabled?" disabled":"",' type="text" ').concat(g," ").concat(f," />");break;case"percent":case"decimal":var m={patternOptions:{allowNegative:!0,allowDecimal:!0,symbols:{thousands:H.currentLocale.data.numbers?H.currentLocale.data.numbers.group:",",decimal:H.currentLocale.data.numbers?H.currentLocale.data.numbers.decimal:".",negative:H.currentLocale.data.numbers?H.currentLocale.data.numbers.minusSign:"-"}},process:"number"};r.numberFormat?(o={patternOptions:{decimalLimit:r.numberFormat.maximumFractionDigits}},r.filterMaskOptions&&(r.filterMaskOptions=E.extend(!0,{},o,m,r.filterMaskOptions)),r.maskOptions&&(r.maskOptions=E.extend(!0,{},o,m,r.maskOptions))):(r.filterMaskOptions&&"function"!=typeof r.filterMaskOptions&&(r.filterMaskOptions=E.extend(!0,{},m,r.filterMaskOptions)),r.maskOptions&&"function"!=typeof r.maskOptions&&(r.maskOptions=E.extend(!0,{},m,r.maskOptions))),n+="<input".concat(r.filterDisabled?" disabled":"",' type="text" ').concat(g," ").concat(f," />");break;case"contents":case"select":if(n+="<select ".concat(g," ").concat("select"===r.filterType?'class="dropdown"':'multiple class="multiselect"').concat(r.filterDisabled?" disabled":"",">").concat(d),c)for(var v=0,b=c.length;v<b;v++){var y=c[v],w=r.caseInsensitive&&"string"==typeof y.value?y.value.toLowerCase():y.value;y&&""!==w&&(n+='<option value = "'.concat(w,'">').concat(y.label,"</option>"))}n+='</select><div class="dropdown-wrapper"><div class="dropdown"><span></span></div><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-dropdown"></use></svg></div>';break;case"multiselect":if(n+="<select ".concat(g,' class="multiselect" multiple').concat(r.filterDisabled?" disabled":"",">"),c)for(var x=0,C=c.length;x<C;x++){var k=c[x],T=r.caseInsensitive&&"string"==typeof k.value?k.value.toLowerCase():k.value;k&&"string"==typeof k.label&&(n+='<option value = "'.concat(T,'">').concat(k.label,"</option>"))}n+='</select><div class="dropdown-wrapper"><div class="dropdown"><span></span></div><svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-dropdown"></use></svg></div>';break;case"time":n+="<input ".concat(r.filterDisabled?" disabled":"",' type="text" class="timepicker" ').concat(g," ").concat(f,"/>");break;case"lookup":n+="<input ".concat(r.filterDisabled?" disabled":"",' type="text" class="lookup" ').concat(g," ").concat(f," >");break;default:n+="<input".concat(r.filterDisabled?" disabled":"",' tabindex="0" type="text" ').concat(g," ").concat(f,"/>")}n+="</div>"}return t.filterType||(n='<div class="datagrid-filter-wrapper is-empty'.concat(" ".concat(s),'"></div>')),n},attachFilterRowEvents:function(){var t=this,e=this;if(this.settings.filterable){this.element.addClass("has-filterable-columns"),this.settings.twoLineHeader&&this.element.addClass("has-two-line-header");var i,n=null;this.element.off("click.datagrid-filter").on("click.datagrid-filter",".btn-filter",(function(){var t=this,i=$(this),s=i.data("popupmenu"),a=i.closest('th[role="columnheader"]'),o="";return s?s.close(!0,!0):i.off("beforeopen.datagrid-filter").on("beforeopen.datagrid-filter",(function(t,i,s){e.hideTooltip(),n=s,o=a.find(".btn-filter .icon-dropdown:first").getIconName().replace("filter-","")})).popupmenu({trigger:"immediate",offset:{y:15},placementOpts:{strategies:["flip","nudge"]}}).off("selected.datagrid-filter").on("selected.datagrid-filter",(function(){var i=$(t).data("popupmenu");i&&i.destroy(),n=null;var s=e.columnById(a.attr("data-column-id"))[0],r=a.find("input"),l=a.find(".btn-filter .icon-dropdown:first").getIconName().replace("filter-","");if(s&&/date|integer|decimal/.test(s.filterType))if("date"===s.filterType){var c=s.editorOptions||{};s.dateFormat&&(c.dateFormat=s.dateFormat),e.filterSetDatepicker(r,l,c)}else{var d=s.filterMaskOptions?s.filterMaskOptions:s.maskOptions,h=(null==d||null==d?void 0:d.rangeNumberDelimeter)||"-",u="in-range"===l,p=r.data("mask"),g=u?{process:"rangeNumber",patternOptions:{delimeter:h}}:{process:"number",patternOptions:{delimeter:""}};p&&p.settings.process!==g.process&&(s.filterMaskOptions&&(s.filterMaskOptions=E.extend(!0,{},s.filterMaskOptions,g)),s.maskOptions&&(s.maskOptions=E.extend(!0,{},s.maskOptions,g)),p.updated(g))}var f=r.val(),m=a.attr("data-column-id"),v={operator:l,defaultOperator:o,value:f,columnId:m};e.element.triggerHandler("filteroperatorchanged",[v]),e.applyFilter(null,"selected")})),!1})),this.element.off("keydown.datagrid-filter-input").on("keydown.datagrid-filter-input",".datagrid-filter-wrapper input",(function(t){return clearTimeout(i),t.stopPropagation(),13!==t.which||(e.applyFilter(null,"enter"),t.preventDefault(),t.stopPropagation(),!1)})),this.element.off("focus.datagrid-filter-focus").on("focus.datagrid-filter-focus",".datagrid-filter-wrapper input",(function(){var t;null===(t=n)||void 0===t||null==t||t.close();var e=$("#dropdown-list").attr("data-element-id"),i=$("#".concat(e)).data("dropdown");i&&i.isOpen()&&i.closeList("click")})),this.element.off("click.datagrid-filter-click").on("click.datagrid-filter-click","th",(function(){var t;null===(t=n)||void 0===t||null==t||t.close();var e=$("#dropdown-list").attr("data-element-id"),i=$("#".concat(e)).data("dropdown");i&&i.isOpen()&&i.closeList("click")})),this.settings.filterWhenTyping&&this.element.off("keyup.datagrid-filter-input").on("keyup.datagrid-filter-input",".datagrid-filter-wrapper input",(function(n){13!==n.which&&(t.activeCell&&t.activeCell.isFocused&&(t.activeCell.isFocused=!1),clearTimeout(i),i=setTimeout((function(){e.applyFilter(null,"keyup")}),400))})),this.element.find(".datagrid-header tr:not(.datagrid-header-groups) th").each((function(){var t=e.columnById($(this).attr("data-column-id"))[0],i=$(this);if(!t)return!0;i.find("select.dropdown").each((function(){var n=$(this);n.dropdown(t.editorOptions).on("selected.datagrid",(function(){e.applyFilter(null,"selected")})).on("listopened.datagrid",(function(){var t=n.data("dropdown");t&&(E.isInViewport(t.list[0])||e.adjustPosLeft(t.list[0]))})),n.data("dropdown").settings.sourceArguments={column:t,container:i,grid:e,cell:t,event:void 0,row:-1,rowData:{},value:void 0}})),i.find("select.multiselect").each((function(){var n=this,s=$(this);s.multiselect(t.editorOptions).on("selected.datagrid",(function(){var t=$(n);$("#".concat(t.attr("id"))).val(t.val()),e.applyFilter(null,"selected")})),s.data("dropdown").settings.sourceArguments={column:t,container:i,grid:e,cell:t,event:void 0,row:-1,rowData:{},value:void 0}})),t.filterMaskOptions?i.find("input").mask(t.filterMaskOptions):t.maskOptions&&i.find("input").mask(t.maskOptions),t.filterMask?i.find("input").mask(t.filterMask):t.mask&&i.find("input").mask(t.mask);var n=i.find(".datepicker");n.length&&"function"==typeof $().datepicker&&n.datepicker(t.editorOptions||{dateFormat:t.dateFormat}).on("listclosed.datepicker",(function(){e.applyFilter(null,"selected")}));var s=i.find(".lookup");s.length&&"function"==typeof $().lookup&&(t.editorOptions&&(t.editorOptions.clickArguments?t.editorOptions.clickArguments.grid=e:t.editorOptions.clickArguments={grid:e},t.editorOptions.clearArguments?t.editorOptions.clearArguments.grid=e:t.editorOptions.clearArguments={grid:e}),s.lookup(t.editorOptions||{}),e.settings.filterWhenTyping&&s.on("change",(function(){e.applyFilter(null,"selected")})));var a=i.find(".timepicker");return a.length&&"function"==typeof $().timepicker&&(a.timepicker(t.editorOptions||{timeFormat:t.timeFormat}),a.on("change",(function(){e.applyFilter(null,"enter")}))),t.filterMask?i.find("input").mask({pattern:t.filterMask,mode:t.maskMode}):t.mask&&i.find("input").mask({pattern:t.mask,mode:t.maskMode}),null})),e.filterRowRendered=!0}},filterItemHtml:function(t,e,i){var n=$.createIcon({classes:"icon icon-filter",icon:"filter-".concat(t)});return'<li class="'.concat(t," ").concat(i?" is-checked":"",'"><a href="#">').concat(n,"<span>").concat(H.translate(e),"</span></a></li>")},filterButtonHtml:function(t){var e;if(!t.filterType)return"";var i=this,n=t.filterDisabled,s=$.isArray(t.filterConditions)?t.filterConditions:[],a=!s.some((function(t){return"string"!=typeof t&&t.selected})),o=function(t,e,n){var o,r;a?(o=!(!s.length||s[0]!==t)||n,r=s):(r=[],o="string"!=typeof t&&t.selected,s.forEach((function(t){"string"==typeof t?r.push(t):r.push(t.value)})));var l,c,d="string"==typeof t?t:t.value;return!s.length||(l=d,c=(c=r)||s,$.inArray(l,c)>-1)?i.filterItemHtml(d,e,o):""},r=E.stringAttributes(this,this.settings.attributes,"btn-filter-".concat(null===(e=t.id)||void 0===e||null==e?void 0:e.toLowerCase())),l=function(t,e){return'<button type="button" '.concat(r,' class="btn-menu btn-filter').concat(e?" ".concat(e):"",'" data-init="false" ').concat(n?" disabled":"").concat(t?' data-default="'.concat(t,'"'):"",' tabindex="0" type="button"><span class="audible">Filter</span>')+'<svg class="icon-dropdown icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-filter-{{icon}}"></use></svg>'.concat($.createIcon({icon:"dropdown",classes:"icon-dropdown"}),'</button><ul class="popupmenu has-icons is-translatable is-selectable">')},c=function(t){var e=t;switch("string"!=typeof e&&(e=t.value),t=e.split("-").map((function(t){return t=t.charAt(0).toUpperCase()+t.slice(1)})).join("")){case"StartWith":t=t.replace("StartWith","StartsWith");break;case"EndWith":t=t.replace("EndWith","EndsWith");break;case"LessEquals":t=t.replace("LessEquals","LessOrEquals");break;case"GreaterEquals":t=t.replace("GreaterEquals","GreaterOrEquals")}return t},d=function(t,e){var i=e;return t.forEach((function(t){"string"!=typeof t&&t.selected&&(i=t.value)})),i},h="",u="";return"contents"===t.filterType||"select"===t.filterType||"multiselect"===t.filterType?"":("text"===t.filterType&&(u=d(s,s.length?s[0]:"contains"),h=0===s.length?(h=l(u)+o("contains","Contains",!0)+o("does-not-contain","DoesNotContain")+o("equals","Equals")+o("does-not-equal","DoesNotEqual")+o("is-empty","IsEmpty")+o("is-not-empty","IsNotEmpty")+o("end-with","EndsWith")+o("does-not-end-with","DoesNotEndWith")+o("start-with","StartsWith")+o("does-not-start-with","DoesNotStartWith")).replace("{{icon}}",u):(h=l(u)+s.map((function(t){return o(t,c(t))})).join("")).replace("{{icon}}",u)),"checkbox"===t.filterType&&(h=(h+=l(u=s.length?s[0]:"selected-notselected","btn-filter-checkbox")+o("selected-notselected","All",!0)+o("selected","Selected")+o("not-selected","NotSelected")).replace("{{icon}}",u)),"checkbox"!==t.filterType&&"text"!==t.filterType&&(h=(h+=l(u=s.length?s[0]:"equals")+o("equals","Equals","lookup"===t.filterType||"integer"===t.filterType||"decimal"===t.filterType||"date"===t.filterType||"time"===t.filterType)+o("does-not-equal","DoesNotEqual")+o("is-empty","IsEmpty")+o("is-not-empty","IsNotEmpty")).replace("{{icon}}",u)),/\b(date|integer|decimal|percent)\b/g.test(t.filterType)&&(h+=o("in-range","InRange")),/\b(date|time)\b/g.test(t.filterType)&&(u=d(s,s.length?s[0]:"equals"),h=0===s.length?(h=l(u)+o("less-than","EarlyThan")+o("less-equals","EarlyOrEquals")+o("greater-than","LaterThan")+o("greater-equals","LaterOrEquals")).replace("{{icon}}",u):(h=l(u)+s.map((function(t){return o(t,c(t))})).join("")).replace("{{icon}}",u)),/\b(integer|decimal|percent)\b/g.test(t.filterType)&&(h=(h+="".concat(o("less-than","LessThan")).concat(o("less-equals","LessOrEquals")).concat(o("greater-than","GreaterThan")).concat(o("greater-equals","GreaterOrEquals"))).replace("{{icon}}","less-than")),"lookup"===t.filterType&&(u=d(s,s.length?s[0]:"contains"),h=0===s.length?(h=l(u)+o("contains","Contains",!0)+o("does-not-contain","DoesNotContain")+o("equals","Equals")+o("does-not-equal","DoesNotEqual")+o("is-empty","IsEmpty")+o("is-not-empty","IsNotEmpty")+o("end-with","EndsWith")+o("does-not-end-with","DoesNotEndWith")+o("start-with","StartsWith")+o("does-not-start-with","DoesNotStartWith")).replace("{{icon}}",u):(h=l(u)+s.map((function(t){return o(t,c(t))})).join("")).replace("{{icon}}",u)),h+="</ul>")},toggleFilterRow:function(){this.settings.filterable?(this.element.find(".datagrid-filter-wrapper").hide(),this.settings.filterable=!1,this.filterRowRendered=!1,this.element.removeClass("has-filterable-columns"),this.element.removeClass("has-two-line-header"),this.element.triggerHandler("closefilterrow")):(this.settings.filterable=!0,this.filterRowRendered=!0,this.element.addClass("has-filterable-columns"),this.settings.twoLineHeader&&this.element.addClass("has-two-line-header"),this.element.find(".datagrid-filter-wrapper").show(),this.element.triggerHandler("openfilterrow"),this.attachFilterRowEvents()),this.setupTooltips()},applyFilter:function(t,e){var i=this,n=this,s=!1,a=function t(e){for(var i=0,n=e.length;i<n;i++)void 0!==e[i]._isFilteredOut&&delete e[i]._isFilteredOut,e[i].children&&t(e[i].children)};t?this.setFilterConditions(t):t=this.filterConditions(),void 0===this.filterExpr&&(this.filterExpr=[]),this.pagerAPI&&JSON.stringify(t)!==JSON.stringify(this.filterExpr)&&(this.filterExpr=t,s=!0,this.settings.treeGrid&&a(this.settings.dataset));var o=function(e){for(var i=!0,s=function(s){var a=n.columnById(t[s].columnId)[0];if(void 0===a)return{v:!1};var o,r=e&&void 0!==e[a.field]?e[a.field]:n.fieldValue(e,a.field),l=null==r?"":r.toString().toLowerCase(),c=t[s].value.toString().toLowerCase(),d=null,h=null,u=null,p=!1;if("percent"===a.filterType&&(c=(c/100).toString(),"decimal"==="".concat(a.name).toLowerCase()?(r=Io.Decimal(!1,!1,r,a),c=Io.Decimal(!1,!1,c,a)):"integer"==="".concat(a.name).toLowerCase()&&(r=Io.Integer(!1,!1,r,a),c=Io.Integer(!1,!1,c,a))),"text"===a.filterType){var g=a.formatter,f=t[s].columnId;l=null==(r=(r=n.formatValue(g,s,f,r,a,e,n)).replace(/(<([^>]+)>)|(amp;)|(&lt;([^>]+)&gt;)/gi,"").trim().toLowerCase())?"":r.toString().toLowerCase()}if("contents"!==a.filterType&&"select"!==a.filterType&&"multiselect"!==a.filterType||(r=null==r?"":r.toString().toLowerCase()),!("number"!=typeof r&&(isNaN(r)||""===r||t[s].value instanceof Array)||/^(date|time|text)$/.test(a.filterType)))if(r=null===r?r:parseFloat(r),a&&"rangeNumber"===(null===(o=a.maskOptions)||void 0===o||null==o?void 0:o.process)){var m,v=null===(m=a.maskOptions.patternOptions)||void 0===m||null==m?void 0:m.delimeter;v=")".concat(v,"("),"("===c.substr(0,1)&&(c=c.substr(1)),")"===c.substr(-1)&&(c=c.substr(0,c.length-1)),c=c.split(v).map((function(t){return H.parseNumber(t)})).sort((function(t,e){return t-e})),p=!0}else c=H.parseNumber(c),p=!1;if("date"===a.filterType||"time"===a.filterType){"string"==typeof r&&(r=a.formatter(!1,!1,r,a,!0));var b=function(e,i){return(i=H.parseDate(i,t[s].format))&&("time"===a.filterType&&(i.setDate(1),i.setMonth(0),i.setYear(0)),i=i.getTime()),e instanceof Date?(e=new Date(e.getTime()),"time"===a.filterType?(e.setDate(1),e.setMonth(0),e.setYear(0)):a.editorOptions&&a.editorOptions.showTime||(e.setHours(0),e.setMinutes(0),e.setSeconds(0),e.setMilliseconds(0)),e=e.getTime()):"string"==typeof e&&e&&(e=a.sourceFormat?H.parseDate(e,"string"==typeof a.sourceFormat?{pattern:a.sourceFormat}:a.sourceFormat):H.parseDate(e,{pattern:t[s].format}))&&("time"===a.filterType?(e.setDate(1),e.setMonth(0),e.setYear(0)):a.editorOptions&&a.editorOptions.showTime||(e.setHours(0),e.setMinutes(0),e.setSeconds(0),e.setMilliseconds(0)),e=e.getTime()),{rValue:e,cValue:i}},y=null;if("in-range"===t[s].operator){var w=n.settings.columns.indexOf(a),x=n.element.find(".datagrid-header th:eq(".concat(w,") .datagrid-filter-wrapper input")).data("datepicker");x&&(h=x.settings.range.separator,(d=x.settings.range.data)&&d.start?y=b(r,d.start):h&&c.indexOf(h)>-1&&(y=b(r,(u=c.split(h))[0])))}else y=b(r,t[s].value);r=y?y.rValue:r,c=y?y.cValue:c}switch(t[s].operator){case"equals":if(t[s].value instanceof Array){i=!1;for(var C=0;C<t[s].value.length;C++){t[s].value[C].toLowerCase()===r&&(""!==r.toString()||""===t[s].value[C])&&(i=!0)}}else i=r===c&&""!==r;break;case"does-not-equal":i=r!==c;break;case"contains":i=l.indexOf(c)>-1&&""!==r.toString();break;case"does-not-contain":i=-1===l.indexOf(c);break;case"end-with":i=l.lastIndexOf(c)===l.length-c.toString().length&&""!==l&&l.length>=c.toString().length;break;case"start-with":i=0===l.indexOf(c)&&""!==l;break;case"does-not-end-with":i=!(i=l.lastIndexOf(c)===l.length-c.toString().length&&""!==l&&l.length>=c.toString().length);break;case"does-not-start-with":i=!(0===l.indexOf(c)&&""!==l);break;case"is-empty":i=""===l;break;case"is-not-empty":if(""===r){i=""!==r;break}i=!(null===r);break;case"in-range":if(i=!1,d&&d.startDate&&d.endDate){var k=d.startDate.getTime(),T=d.endDate.getTime();i=r>=k&&r<=T&&null!==r}else if(u){var A=H.parseDate(u[0],t[s].format),S=H.parseDate(u[1],t[s].format);A&&S&&(A=A.getTime(),S=S.getTime(),i=r>=A&&r<=S&&null!==r)}if(p){var $=""!==r&&null!==r;i=r>=c[0]&&r<=c[1]&&$}break;case"less-than":i=r<c&&""!==r&&null!==r;break;case"less-equals":i=r<=c&&""!==r&&null!==r;break;case"greater-than":i=r>c&&""!==r&&null!==r;break;case"greater-equals":i=r>=c&&""!==r&&null!==r;break;case"selected":if(a&&a.isChecked){i=a.isChecked(r);break}i=("1"===l||"true"===l||!0===r||1===r)&&""!==l;break;case"not-selected":if(a&&a.isChecked){i=!a.isChecked(r);break}i="0"===l||"false"===l||!1===r||0===r||""===l;break;case"selected-notselected":i=!0}return i?void 0:{v:!1}},a=0;a<t.length;a++){var o=s(a);if("object"===l(o))return o.v}return i};if(!this.settings.disableClientFilter)if(this.settings.treeGrid){a(this.settings.dataset),function(){for(var e=!0,i=0,n=t.length;i<n;i++)"checkbox"!==t[i].filterType&&""===t[i].value.toString().trim()||(e=!1),"text"===t[i].filterType&&"is-empty"===t[i].operator&&(e=!1);return e}()||function t(e){for(var n=!0,s=0,a=e.length;s<a;s++){var r=e[s],l=!0;if(r.children)if(i.settings.allowChildExpandOnMatch&&i.settings.allowChildExpandOnMatchOnly){var c=o(r);c||(l=t(r.children)),c?(r._isFilteredOut=!1,n=!1):l?(r._isFilteredOut=!o(r),r._isFilteredOut||(n=!1)):n=!1}else(l=t(r.children))?(r._isFilteredOut=!o(r),r._isFilteredOut||(n=!1)):n=!1;else r._isFilteredOut=!o(r),r._isFilteredOut||(n=!1)}return n}(this.settings.dataset)}else if(this.settings.groupable)for(var r,c=0,d=this.settings.dataset.length;c<d;c++){for(var h=!0,u=0,p=this.settings.dataset[c].values.length;u<p;u++)r=!o(this.settings.dataset[c].values[u]),this.settings.dataset[c].values[u]._isFilteredOut=r,r||(h=!1);this.settings.dataset[c]._isFilteredOut=h}else for(var g,f=0,m=this.settings.dataset.length;f<m;f++)g=!o(this.settings.dataset[f]),this.settings.dataset[f]._isFilteredOut=g;if(this.setChildExpandOnMatch(),this.settings.source||this.settings.disableClientFilter||(this.clearCache(),this.renderRows()),s){var v={trigger:e,type:"filtered"};this.settings.source&&(v.preserveSelected=this.settings.allowSelectAcrossPages),this.setSearchActivePage(v)}this.settings.disableClientFilter&&"restore"===e||(this.element.trigger("filtered",{op:"apply",conditions:t,trigger:e}),this.saveUserSettings())},adjustPosLeft:function(t){var e=t.getBoundingClientRect(),i=window.innerWidth||document.documentElement.clientWidth;e.left<0&&e.right<=i?t.style.left="".concat(20,"px"):e.left>=0&&!(e.right<=i)&&(t.style.left="".concat(i-e.width-20,"px"))},setChildExpandOnMatch:function(){var t=this.settings;if(t.treeGrid){!function t(e){for(var i=!1,n=0,s=e.length;n<s;n++){var a,o,r=e[n];r._isFilteredOut&&!i&&null!==(a=r.children)&&void 0!==a&&(null==(o=a)?void 0:o.length)&&(r._isFilteredOut=!t(r.children)),"boolean"!=typeof r._isFilteredOut||r._isFilteredOut||(i=!0)}return i}(t.dataset),function e(i,n){for(var s=0,a=i.length;s<a;s++){var o=i[s],r=o.children,l=r?r.length:0;if(l){if(!o._isFilteredOut)if(t.allowChildExpandOnMatch&&!t.allowChildExpandOnMatchOnly)for(var c=0;c<l;c++)r[c]._isFilteredOut=!1;else{for(var d=!0,h=0;h<l;h++)r[h]._isFilteredOut||(d=!1);o.isAllChildrenFiltered=d}e(r)}}}(t.dataset)}},clearFilter:function(){this.settings.filterable&&(this.clearFilterFields(),this.applyFilter(),this.element.trigger("filtered",{op:"clear",conditions:[]}))},clearFilterFields:function(){this.settings.filterable&&(this.element.find(".datagrid-header input, select").each((function(){var t=$(this);t.val(""),t.is("select")&&t.find("option").each((function(){$(this).prop("selected",!1)})),t.trigger("updated")})),this.element.find(".datagrid-header .btn-filter").each((function(){var t=$(this),e=t.next(),i=e.find("li:first");t.find("svg:first > use").attr("href","#icon-filter-".concat(t.attr("data-default"))),e.find(".is-checked").removeClass("is-checked"),i.addClass("is-checked")})))},setFilterConditions:function(t){this.clearFilterFields();for(var e=0;e<t.length;e++){var i=this.element.find('.datagrid-header th[data-column-id="'.concat(t[e].columnId,'"]')),n=i.find("input, select"),s=i.find(".btn-filter"),a=s.next();if(void 0===t[e].value&&(t[e].value=""),n.val(t[e].value),n.is(".lookup")&&1===n.parent().find("svg.close").length&&t[e].value&&n.parent().find("svg.close").removeClass("is-empty"),n.is("select")){var o=t[e].value instanceof Array?t[e].value[0]:t[e].value;if(t[e].innerHTML&&(n[0].innerHTML=t[e].innerHTML),n.find('option[value="'.concat(o,'"]')).length)if(t[e].value instanceof Array&&!t[e].selectedOptions){var r=n[0].querySelectorAll("option");n.val("");for(var l=0;l<r.length;l++)r[l].selected=!1;for(var c=0;c<t[e].value.length;c++)n.find('option[value="'.concat(t[e].value[c],'"]')).prop("selected",!0)}else n.find('option[value="'.concat(t[e].value,'"]')).prop("selected",!0);else{var d=n.data("dropdown");d&&d.setCode(t[e].value)}n.trigger("updated")}s.find("svg:first > use").attr("href","#icon-filter-".concat(t[e].operator)),a.find(".is-checked").removeClass("is-checked"),a.find(".".concat(t[e].operator)).addClass("is-checked")}},filterConditions:function(){if(this.filterExpr&&1===this.filterExpr.length&&this.filterExpr[0].keywordSearch)return delete this.filterExpr[0].keywordSearch,this.filterExpr;var t=[],e=this;return this.element.find("th.is-filterable").each((function(){var i,n=$(this),s=n.find(".btn-filter"),a=n.find("input, select"),o=a.is("select"),r=s.find(".icon-dropdown:first"),l=null;if((s.length||o)&&"selected-notselected"!==(i=o?"equals":r.getIconName().replace("filter-",""))&&!(""===a.val()&&-1===["is-not-empty","is-empty","selected","not-selected"].indexOf(i)||a.val()instanceof Array&&0===a.val().length)){var c=a.val()?a.val():"";a.attr("data-mask-mode")&&"number"===a.attr("data-mask-mode")&&(c=H.parseNumber(c));var d={columnId:n.attr("data-column-id"),operator:i,value:c};a.data("datepicker")&&(l=a.data("datepicker").pattern,d.format=l),a.data("timepicker")&&(l=a.data("timepicker").settings.timeFormat,d.format=l),a.is("select")&&(d.innerHTML=a[0].innerHTML);var h=e.columnById(d.columnId);h&&h[0]&&(d.filterType=h[0].filterType),t.push(d)}})),t},getExtraTop:function(){var t=this.settings,e={default:{extraSmall:0,short:0,small:0,medium:0,normal:0,large:0},filterable:{extraSmall:0,short:0,small:0,medium:0,normal:0,large:0},group:{extraSmall:-22,short:-25,small:-25,medium:-30,normal:-39,large:-39},groupFilter:{extraSmall:-28,short:-29,small:-29,medium:-32,normal:-39,large:-39}},i="extra-small"===t.rowHeight?"extraSmall":t.rowHeight;return t.columnGroups?t.filterable?e.groupFilter[i]:e.group[i]:t.filterable?e.filterable[i]:e.default[i]},getTargetHeight:function(){var t=this.settings,e={default:{extraSmall:20,short:20,small:20,medium:28,normal:35,large:35},filterable:{extraSmall:47,short:53,small:53,medium:54,normal:60,large:60},group:{extraSmall:43,short:46,small:46,medium:56,normal:74,large:74},groupFilter:{extraSmall:80,small:80,short:83,medium:87,normal:103,large:103}},i="extra-small"===t.rowHeight?"extraSmall":t.rowHeight;return t.columnGroups?t.filterable&&this.filterRowRendered?e.groupFilter[i]:e.group[i]:t.filterable&&this.filterRowRendered?e.filterable[i]:e.default[i]},getCellPadding:function(){return{"extra-small":8,short:8,small:8,medium:16,normal:16,large:16}[this.settings.rowHeight]},createDraggableColumns:function(){var t=this,e=t.headerNodes().not('[data-column-id="selectionCheckbox"]').not(".datagrid-header-spacer-column"),i=$(".drag-target-arrows",t.element);i.length||(t.element.prepend('<span class="drag-target-arrows" style="height: '.concat(t.getTargetHeight(),'px;"></span>')),i=$(".drag-target-arrows",t.element)),e.not('[data-reorder="false"]').prepend('</span><span class="handle">'.concat($.createIcon({icon:"drag"}),"</span>")),e.prepend('<span class="is-draggable-target"></span>'),e.last().append('<span class="is-draggable-target last"></span>'),t.element.addClass("has-draggable-columns"),$(".handle",e).each((function(){var n=null,s=null,a=null,o=0,c=$(this),d=c.parent();c.on("mousedown.datagrid",(function(c){c.preventDefault(),t.editor&&t.editor.input&&t.commitCellEdit(),d.drag({clone:!0,cloneAppendTo:e.first().parent().parent(),clonePosIsFixed:!0}).off("dragstart.datagrid").on("dragstart.datagrid",(function(e,i,r){(n=r).removeAttr("id").addClass("is-dragging-clone").css({left:H.isRTL()?i.right-n.width():i.left,top:i.top,height:d.height(),border:0}),$(".is-draggable-target",n).remove(),t.setDraggableColumnTargets(),o=t.getExtraTop(),s=d.position(),a={top:i.top-s.top,left:i.left-s.left};var l=t.targetColumn(s);t.draggableStatus.startIndex=l,c.stopImmediatePropagation()})).off("drag.datagrid").on("drag.datagrid",(function(r,l){n[0].style.left="".concat(parseInt(H.isRTL()?l.left+l.offset.x-l.clone.width():l.left,10),"px"),n[0].style.top="".concat(parseInt(l.top,10),"px"),s={top:l.top-a.top,left:l.left-a.left},H.isRTL()&&(s.left=parseInt(n[0].style.left,10));var h=0,u=null,p=null,g=t.targetColumn(s);if($(".is-draggable-target",e).add(i).removeClass("is-over"),-1!==g)for(var f=0,m=t.draggableColumnTargets.length;f<m;f++)if(h=f+1,(u=t.draggableColumnTargets[f]).index===g&&u.index!==t.draggableStatus.startIndex){u.index>t.draggableStatus.startIndex&&h<m&&(u=t.draggableColumnTargets[h]),u.el.addClass("is-over"),i.addClass("is-over"),p=u.el[0].getBoundingClientRect(),i[0].style.left="".concat(parseInt(p.left+(H.isRTL()?2:1),10),"px");var v=0;u.el.hasClass("last")&&(v=-(d.height()-3)),i[0].style.top="".concat(parseInt(p.top,10)+1+o+v,"px")}c.stopImmediatePropagation()})).off("dragend.datagrid").on("dragend.datagrid",(function(o,c){H.isRTL()||(n[0].style.left="".concat(parseInt(c.left,10),"px"),n[0].style.top="".concat(parseInt(c.top,10),"px")),s={top:c.top-a.top,left:c.left-a.left},H.isRTL()&&(s.left=parseInt(n[0].style.left,10));var h,u,p,g,f=t.targetColumn(s),m=d.data("drag"),v=[];if(m&&m.destroy&&m.destroy(),t.draggableStatus.endIndex=f,$(".is-draggable-target",e).add(i).removeClass("is-over"),-1!==t.draggableStatus.endIndex&&t.draggableStatus.startIndex!==t.draggableStatus.endIndex){for(h=0,u=t.settings.columns.length;h<u;h++)t.settings.columns[h].hidden||"selectionCheckbox"===t.settings.columns[h].id||v.push(h);if(p=v[t.draggableStatus.startIndex]||0,g=v[t.draggableStatus.endIndex]||0,t.settings.showDirty&&t.dirtyArray){var b=function(e,i,n,s){for(var a=[],o=n;o<=s;o++)i[o]&&"object"===l(i[o])?t.setDirtyCell(e,o,i[o]):!1===i[o]&&a.push(o);a.forEach((function(i){return t.clearDirtyCell(e,i)}))},y=function(e,i,n,s,a){t.dirtyArray[e][a]&&t.dirtyArray[e][a].isDirty?n[a]=!0:(n[a]=r({},i[s]),n[a].cell=a,n[a].column=t.settings.columns[a])};p>g?t.dirtyArray.forEach((function(t,e){for(var i=Array(p+1).fill(!1),n=g;n<=p;n++)t&&t[n]&&t[n].isDirty&&(n===p?y(e,t,i,p,g):y(e,t,i,n,n+1));b(e,i,g,p)})):p<g&&t.dirtyArray.forEach((function(t,e){for(var i=Array(g+2).fill(!1),n=g;n>=p;n--)t&&t[n]&&t[n].isDirty&&(n===p?y(e,t,i,p,g):y(e,t,i,n,n-1));b(e,i,p,g)}))}t.updateGroupHeadersAfterColumnReorder(p,g),t.arrayIndexMove(t.settings.columns,p,g),t.updateColumns(t.settings.columns)}}))}))}))},setDraggableColumnTargets:function(){var t,e,i=this,n=i.headerNodes().not(".is-hidden").not('[data-column-id="selectionCheckbox"]').not(".datagrid-header-spacer-column");i.draggableColumnTargets=[],i.draggableStatus={},$(".is-draggable-target.last",n.last()).length||n.last().append($(".is-draggable-target.last",i.headerNodes())),$(".is-draggable-target",n).each((function(s){var a=$(this).is(".last")?s-1:s;t=n.eq(a),e=t.position(),20,i.draggableColumnTargets.push({el:$(this),index:a,pos:e,width:t.outerWidth(),height:t.outerHeight(),dropArea:{x1:e.left-20,x2:e.left+t.outerWidth()+20,y1:e.top-20+i.getExtraTop(),y2:e.top+t.outerHeight()+20}})}))},targetColumn:function(t){var e,i,n,s=-1;for(i=0,n=this.draggableColumnTargets.length-1;i<n;i++)if(e=this.draggableColumnTargets[i],t.left>e.dropArea.x1&&t.left<e.dropArea.x2&&t.top>e.dropArea.y1&&t.top<e.dropArea.y2&&(s=e.index,H.isRTL()))return s;return s},arrayIndexMove:function(t,e,i){t.splice(i,0,t.splice(e,1)[0])},resetGroupArray:function(){if(this.settings.groupable){this.groupArray=[];for(var t=0,e=this.settings.dataset.length;t<e;t++)for(var i=0,n=this.settings.dataset[t].values.length;i<n;i++)this.groupArray.push({group:t,node:i})}},resequenceGroupRows:function(){if(this.settings.groupable)for(var t=this.tableBody.find("tr:not(.datagrid-rowgroup-header)"),e=0;e<t.length;e++)t[e].setAttribute("aria-rowindex",this.pagingRowIndex(e+1))},createDraggableRowsGroupable:function(){var t=this,e=this,i=this.settings;if(i.rowReorder&&i.groupable){"rowReorder"===i.columns[0].id&&(i.columns[0].resizable=!1,this.tableBody.find('> [role="row"] td:first-child').removeClass("l-center-text").addClass("reorder-group-child-col"));var n,s="> tr:not(.is-dragging-clone)",a=function(t){for(var e=0,i=n.length;e<i;e++){var s=n[e].rect;if(t.top>=s.top&&t.top<s.bottom)return r({},n[e])}return null};this.tableBody.find(s).find(".datagrid-reorder-icon").each((function(){var o=null,r=null,l=null,c=!1,d=$(this);d.off("mousedown.datagrid").on("mousedown.datagrid",(function(h){h.preventDefault(),function(t){var e=t.buttons||t.which||t.button;return!(t.metaKey||t.ctrlKey||t.altKey||t.shiftKey)&&1===e}(h)&&d.closest("tr").drag({axis:"y",clone:!0,cloneAppendTo:d.closest("tbody"),clonePosIsFixed:!0}).off("dragstart.datagrid").on("dragstart.datagrid",(function(i,a,d){(o=d).removeAttr("id").addClass("is-dragging-clone groupable"),o.css({top:"".concat(a.top,"px")});var h=o.find("> td");$(this).find("> td").each((function(t,e){return h.eq(t).width($(e).width())})),function(){n=[];for(var e=t.tableBody.find(s),i=0,a=-1,o=-1,r=e.length;i<r;i++){var l=e[i];"rowgroup"===l.getAttribute("role")?(a++,o=-1):o++,n.push({group:a,node:o,row:l,idx:i,rect:l.getBoundingClientRect(),role:l.getAttribute("role")})}}(),(l={idx:e.tableBody.find(s).index(this),rect:this.getBoundingClientRect(),role:this.getAttribute("role"),row:this}).group=n[l.idx].group,l.node=n[l.idx].node;var u=$(this);"rowgroup"===l.role?r=u.add(u.nextUntil(".datagrid-rowgroup-header")):"row"===l.role&&(r=u),r.css("opacity",".4"),c=!0})).on("drag.datagrid",(function(t,e){if(o&&c){c=!1;var i=!1,n=a(e);null!==n&&("rowgroup"===l.role&&(i=l.group!==n.group),"row"===l.role&&(i=l.role===n.role&&l.group===n.group));var s=i?"":"not-allowed";o.add(o.find(".datagrid-reorder-icon")).css("cursor",s),c=!0}})).off("dragend.datagrid").on("dragend.datagrid",(function(t,n){var s=n.top>l.rect.top,o=a(n),h=!1;if(null!==o){var u;if("rowgroup"===l.role&&l.group!==o.group){var p=".datagrid-rowgroup-header";s?(u=$(o.row).nextUntil(p).last(),r.insertAfter(u)):(u="rowgroup"===o.role?o.row:$(o.row).prevUntil(p).last().prev(p),r.insertBefore(u)),e.arrayIndexMove(i.dataset,l.group,o.group),h=!0}else"row"===l.role&&l.role===o.role&&l.group===o.group&&(e.arrayIndexMove(i.dataset[l.group].values,l.node,o.node),u=o.row,r[s?"insertAfter":"insertBefore"](u),h=!0);if(h){e.resetGroupArray(),e.resequenceGroupRows(),e.syncSelectedRowsIdx();var g={start:{rowIndex:l.idx,group:l.group,node:-1!==l.node?l.node:null},end:{rowIndex:o.idx,group:o.group,node:-1!==l.node?o.node:null},rowreorderRows:r,role:l.role};e.element.trigger("rowreorder",[g])}}r.css("opacity",""),c=!1;var f=d.closest("tr").data("drag");null==f||null==f||f.destroy()}))}))}))}},createDraggableRows:function(){var t=this;this.settings.rowReorder&&(this.settings.groupable?this.createDraggableRowsGroupable():(this.tableBody.children().filter((function(){return $(this).find(".datagrid-reorder-icon").length<1})).attr("data-arrange-exclude",!0),this.tableBody.arrange({placeholder:'<tr class="datagrid-reorder-placeholder"><td colspan="'.concat(this.visibleColumns().length,'"></td></tr>'),handle:".datagrid-reorder-icon",isVisualItems:!0}).off("beforearrange.datagrid").on("beforearrange.datagrid",(function(e,i){t.isSafari&&i.start.css({display:"inline-block"})})).off("arrangeupdate.datagrid").on("arrangeupdate.datagrid",(function(e,i){t.isSafari&&i.end.css({display:""}),t.reorderRow(i.startIndex,i.endIndex,i)}))))},reorderRow:function(t,e,i){var n=e>t,s=this.tableBody.find("tr").eq(t),a=this.tableBody.find("tr").eq(e),o={start:this.dataRowIndex(i?i.start:a)};o.end=o.start+(e-t),this.arrayIndexMove(this.settings.dataset,o.start,o.end),i?(this.settings.rowTemplate||this.settings.expandableRow)&&(n?(this.tableBody.find("tr").eq(2*t).insertAfter(i.end),i.end.next().next().insertAfter(i.over)):this.tableBody.find("tr").eq(2*t).next().insertAfter(i.end)):s[n?"insertAfter":"insertBefore"](a);for(var r=this.tableBody.find("tr:not(.datagrid-expandable-row)"),l=0;l<r.length;l++)r[l].setAttribute("data-index",l),r[l].setAttribute("aria-rowindex",this.pagingRowIndex(l+1));this.element.trigger("rowreorder",[{endIndex:e,startIndex:t,over:a,start:s}]),this.syncSelectedRowsIdx()},fieldValue:function(t,e){if(!e||!t)return"";var i,n=(i=e.indexOf(".")>-1?e.split(".").reduce((function(t,e){return t?t[e]:""}),t):t[e])||0===i||!1===i?i:"";return n=g.escapeHTML(n)},setTreeRootNodes:function(){this.settings.treeGrid&&(this.settings.treeRootNodes=this.settings.treeDepth.filter((function(t){return 1===t.depth})))},setTreeDepth:function(t){if(this.settings.treeGrid){var e=this,i=0,n=function t(n,s){var a,o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[];i++,e.settings.treeDepth.push({idx:i,depth:s,parents:o.slice(),node:n});var r=null==n||null===(a=null==n?void 0:n.children)||void 0===a||null==a?void 0:a.length;if(r){o.push(i-1);for(var l=0;l<r;l++)t(n.children[l],s+1,o.slice())}};t=t||this.settings.dataset,e.settings.treeDepth=[];for(var s=0,a=t.length;s<a;s++)n(t[s],1)}},setRowGrouping:function(){var t=this.settings.groupable;t&&(this.originalDataset?(!this.settings.dataset||this.settings.dataset.length>0)&&(this.settings.dataset=this.originalDataset):this.originalDataset=this.settings.dataset.slice(),t.aggregator&&"none"!==t.aggregator?"sum"!==t.aggregator?"max"!==t.aggregator?"min"!==t.aggregator?"avg"!==t.aggregator?"count"!==t.aggregator?"list"!==t.aggregator?this.settings.dataset=Ro(this.settings.dataset,t.fields):this.settings.dataset=Ro.list(this.settings.dataset,t.fields,t.aggregatorOptions):this.settings.dataset=Ro.count(this.settings.dataset,t.fields,t.aggregate):this.settings.dataset=Ro.avg(this.settings.dataset,t.fields,t.aggregate):this.settings.dataset=Ro.min(this.settings.dataset,t.fields,t.aggregate):this.settings.dataset=Ro.max(this.settings.dataset,t.fields,t.aggregate):this.settings.dataset=Ro.sum(this.settings.dataset,t.fields,t.aggregate):this.settings.dataset=Ro.none(this.settings.dataset,t.fields))},renderRows:function(){var t="",e="",i="",n=0,s=this,a=s.settings,o=s.table.find("tbody"),r=1;if(s.pagerAPI){var l=s.pagerAPI.state;r=l.filteredActivePage?l.filteredActivePage:l.activePage}for(s.bodyColGroupHtmlLeft="<colgroup>",s.bodyColGroupHtml="<colgroup>",s.bodyColGroupHtmlRight="<colgroup>",s.triggerDestroyCell(),s.settings.columns&&0!==s.settings.columns.length||s.settings.columns.push({id:"blank",value:"",field:""}),this.currentPageRows=[],n=0;n<s.settings.columns.length;n++){var c=s.settings.columns[n],d=s.getContainer(c.id),h=s.columnWidth(c,n);switch(d){case"left":s.bodyColGroupHtmlLeft+="<col".concat(h).concat(c.hidden?' class="is-hidden"':"","></col>");break;case"right":s.bodyColGroupHtmlRight+="<col".concat(h).concat(c.hidden?' class="is-hidden"':"","></col>");break;default:s.bodyColGroupHtml+="<col".concat(h).concat(c.hidden?' class="is-hidden"':"","></col>")}c.colspan&&(s.hasColSpans=!0)}s.emptyMessageContainer&&s.emptyMessageContainer.hide(),0===o.length&&(s.hasLeftPane&&(s.tableBodyLeft=$('<tbody role="rowgroup"></tbody>'),s.tableLeft.append(s.tableBodyLeft)),s.tableBody=$('<tbody role="rowgroup"></tbody>'),s.table.append(s.tableBody),s.hasRightPane&&(s.tableBodyRight=$('<tbody role="rowgroup"></tbody>'),s.tableRight.append(s.tableBodyRight))),s.groupArray=[],s.recordCount=0,s.filteredCount=0,s.runningCount=0,a.treeGrid&&a.paging&&!a.source&&r>1&&(s.recordCount=a.treeRootNodes[a.pagesize*r-a.pagesize].idx-1),this.restoreSortOrder&&this.sortDataset();var u=!1;s.originalDataset&&!a.groupable&&a.dataset.length>0&&(a.dataset=s.originalDataset);for(var p=0;p<a.dataset.length;p++){var g,m,v,b;if(a.paging&&!a.source){if(1===r&&p-this.filteredCount>=a.pagesize){a.dataset[p]._isFilteredOut?this.filteredCount++:this.recordCount++;continue}if(r>1&&!(p-this.filteredCount>=a.pagesize*(r-1)&&p-this.filteredCount<a.pagesize*r)){a.dataset[p]._isFilteredOut?this.filteredCount++:this.filteredCount&&this.recordCount++;continue}}if(!a.virtualized||this.isRowVisible(this.recordCount))if((!a.treeGrid&&a.dataset[p])._isFilteredOut)this.filteredCount++;else if(this.settings.groupable){if(a.dataset[p].values){for(var y=a.dataset[p].values.length,w=0,x=0;x<y;x++)a.dataset[p].values[x]._isFilteredOut&&w++;if(w===y)continue}if(!this.settings.groupable.suppressGroupRow){var C=s.rowHtml(a.dataset[p],this.recordCount,p,!0);this.hasLeftPane&&C.left&&(e+=C.left),C.center&&(t+=C.center),this.hasRightPane&&C.right&&(i+=C.right)}if(this.settings.groupable.showOnlyGroupRow&&a.dataset[p].values[0]){var k=a.dataset[p].values[0];a.dataset[p].list&&(k.list=a.dataset[p].list),k.values=a.dataset[p].values;var T=s.rowHtml(k,this.recordCount,p);this.hasLeftPane&&T.left&&(e+=T.left),T.center&&(t+=T.center),this.hasRightPane&&T.right&&(i+=T.right),this.recordCount++,s.groupArray.push({group:p,node:0});continue}for(var A=0;A<a.dataset[p].values.length;A++)if(!a.dataset[p].values[A]._isFilteredOut){var S=s.rowHtml(a.dataset[p].values[A],this.recordCount,a.dataset[p].values[A].idx);s.hasLeftPane&&S.left&&(e+=S.left),S.center&&(t+=S.center),s.hasRightPane&&S.right&&(i+=S.right),this.recordCount++,s.groupArray.push({group:p,node:A})}if(this.settings.groupable.groupFooterRow){var E=s.rowHtml(a.dataset[p],this.recordCount,p,!0,!0);s.hasLeftPane&&E.left&&(e+=E.left),E.center&&(t+=E.center),s.hasRightPane&&E.right&&(i+=E.right)}}else{var D=p;a.treeGrid&&(D=this.recordCount),s.runningCount++;var I,L,P=s.rowHtml(a.dataset[p],D,p);if(s.hasLeftPane&&P.left&&(e+=P.left),P.center)t+=P.center,a.treeGrid&&null!==(I=this.filterExpr)&&void 0!==I&&(null==(L=I)?void 0:L.length)&&$("<table>".concat(P.center,"</table>")).find("tr:first").is(".is-hidden")&&this.filteredCount++;s.hasRightPane&&P.right&&(i+=P.right),null!==(g=a.dataset[p])&&void 0!==g&&(null==(m=g)?void 0:m._isFilteredOut)||this.recordCount++,this.visibleRowCount=D+1,null!==(v=a.dataset[p])&&void 0!==v&&(null==(b=v)?void 0:b.rowStatus)&&(u=!0)}else this.recordCount++}if(this.settings.summaryRow){s.element.addClass("has-summary-row");var M=s.calculateTotals(),B=s.rowHtml(M,this.recordCount,null,!1,!0);s.hasLeftPane&&B.left&&(e+=B.left),B.center&&(t+=B.center),s.hasRightPane&&B.right&&(i+=B.right)}"<colgroup>"!==s.bodyColGroupHtml&&(this.settings.spacerColumn&&(s.bodyColGroupHtml+='<col style="width: 100%">'),s.bodyColGroupHtmlLeft+="</colgroup>",s.bodyColGroupHtml+="</colgroup>",s.bodyColGroupHtmlRight+="</colgroup>",s.bodyColGroupLeft&&s.bodyColGroupLeft.remove(),s.bodyColGroup&&s.bodyColGroup.remove(),s.bodyColGroupRight&&s.bodyColGroupRight.remove(),s.hasLeftPane&&(s.bodyColGroupLeft=$(s.bodyColGroupHtmlLeft),(s.tableBodyLeft||s.headerRowLeft).before(s.bodyColGroupLeft)),s.bodyColGroup=$(s.bodyColGroupHtml),(s.headerRow||s.tableBody).before(s.bodyColGroup),s.hasRightPane&&(s.bodyColGroupRight=$(s.bodyColGroupHtmlRight),(s.tableBodyRight||s.headerRowRight).before(s.bodyColGroupRight))),s.hasLeftPane&&f.html(s.tableBodyLeft,e,"*"),f.html(s.tableBody,t,"*"),s.hasRightPane&&f.html(s.tableBodyRight,i,"*"),s.setVirtualHeight(),s.setScrollClass(),s.setupTooltips(u),s.afterRender()},afterRender:function(){var t=this,e=this;this.settings.fixedRowHeight&&"auto"===this.settings.fixedRowHeight&&this.settings.frozenColumns&&e.tableBody.find("tr").each((function(t){var i,n=0,s=0,a=$(this);i=a.height(),e.tableBodyLeft&&(n=e.tableBodyLeft.find("tr").eq(t).height()),e.tableBodyRight&&(s=e.tableBodyLeft.find("tr").eq(t).height());var o=Math.max(n,i,s);return a.css("height",o),e.tableBodyLeft&&(n=e.tableBodyLeft.find("tr").eq(t).css("height",o)),e.tableBodyRight&&(s=e.tableBodyLeft.find("tr").eq(t).css("height",o)),!0})),this.postRenderCell();var i=e.tableBody.find("select.dropdown");i.dropdown&&i.dropdown(),e.tableBody.find(".dropdown-wrapper.is-inline").prev("select").on("listclosed",(function(){var t=$(this),i=t.val(),n=t.closest("tr");e.updateCellNode(n.attr("aria-rowindex"),t.closest("td").index(),i,!1,!0)}));var n=e.tableBody.find(".spinbox");n.spinbox&&n.spinbox(),e.settings.source||e.displayCounts(),e.setAlternateRowShading(),e.createDraggableRows();var s=document.activeElement;(e.activeCell.isFocused&&(!s||s&&"body"===s.tagName.toLowerCase())&&e.setActiveCell(e.activeCell.row,e.activeCell.cell),e.settings.paging&&e.settings.source&&!e.settings.allowSelectAcrossPages&&(e.preventSelection||(e._selectedRows=[]),e.syncSelectedUI()),e.settings.paging&&e.settings.source&&e.settings.allowSelectAcrossPages&&(e.syncSelectedRows(),e.syncSelectedUI()),e.settings.paging&&!e.settings.source&&null===e.settings.allowSelectAcrossPages&&(e.syncSelectedRows(),e.syncSelectedUI()),this.stretchColumnDiff<0)&&(this.bodyColGroup.find("col").eq(e.getStretchColumnIdx())[0].style.width="".concat(this.stretchColumnWidth,"px"));this.element.find(".datagrid-img").off("error.datagrid").on("error.datagrid",(function(e){var i=$(e.target),n=i.parent(),s=$('<svg class="icon" focusable="false" aria-hidden="true" role="presentation"><use href="#icon-'.concat(t.settings.fallbackImage,'"></use></svg>'));s.tooltip(t.settings.fallbackTooltip),s.css(t.settings.fallbackSize),n.append(s),i.remove()})),setTimeout((function(){e.element.trigger("afterrender",{body:e.container.find("tbody"),header:e.container.find("thead"),pager:e.pagerAPI}),t.activateFirstCell()}))},postRenderCell:function(){var t=this;if(this.settings.onPostRenderCell)for(var e=function(){var e=t.settings.columns[i];e.component&&t.tableBody.find("tr").each((function(){var i,n,s=$(this),a=t.settings.treeGrid?t.actualPagingRowIndex(t.actualRowIndex(s)):t.dataRowIndex(s),o=s.attr("data-lineage"),r=t.rowData(a),l=t.columnIdxById(e.id)-((null===(i=t.settings.frozenColumns)||void 0===i||null===(n=null==i?void 0:i.left)||void 0===n||null==n?void 0:n.length)||0),c={row:o||a,cell:l,value:r,rowData:r,col:e,api:t};t.settings.onPostRenderCell(s.find("td").eq(l).find(".datagrid-cell-wrapper .content")[0],c)}))},i=0;i<this.settings.columns.length;i++)e()},activateFirstCell:function(){this.activeCell&&this.activeCell.node||(this.activeCell={node:this.cellNode(0,0).attr("tabindex","0"),isFocused:!1,cell:0,row:0})},triggerDestroyCell:function(){var t=this,e=this;if(e.tableBody&&this.settings.onPostRenderCell&&this.settings.onDestroyCell){var i=e.tableBody.find("tr");if(0===i.length)return;for(var n=function(){var n=t.settings.columns[s];n.component&&i.each((function(){var t=$(this),i=t.index(),s=e.columnIdxById(n.id),a={row:t.index(),cell:s,value:e.settings.dataset[i],col:n,api:e};e.settings.onDestroyCell(t.find("td").eq(s).find(".datagrid-cell-wrapper .content")[0],a)}))},s=0;s<this.settings.columns.length;s++)n()}},cacheVirtualStats:function(){var t=this.element[0].offsetHeight,e=this.bodyWrapperCenter[0].scrollTop,i="normal"===this.settings.rowHeight||"large"===this.settings.rowHeight?40:"medium"===this.settings.rowHeight?30:25,n=t-i,s="normal"===this.settings.rowHeight||"large"===this.settings.rowHeight?50:"medium"===this.settings.rowHeight?40:30;"extra-small"===this.settings.rowHeight&&(i=22,s=27),this.virtualRange={rowHeight:s,top:Math.max(e-(this.settings.virtualRowBuffer-1)*s,0),bottom:e+n+(this.settings.virtualRowBuffer-1)*s,totalHeight:s*this.settings.dataset.length,bodyHeight:n}},isRowVisible:function(t){if(!this.settings.virtualized)return!(this.settings.paging&&!this.settings.source&&t&&this.pagerAPI)||(this.pagerAPI.activePage-1)*this.settings.pagesize<=t&&this.pagerAPI.activePage*this.settings.pagesize>=t;0===t&&this.cacheVirtualStats();var e=t*this.virtualRange.rowHeight;return e>=this.virtualRange.top&&e<this.virtualRange.bottom},setVirtualHeight:function(){if(this.settings.virtualized&&this.virtualRange){var t=this.virtualRange.totalHeight-this.virtualRange.bottom,e=this.virtualRange.top;this.topSpacer=this.tableBody.find(".datagrid-virtual-row-top"),this.bottomSpacer=this.tableBody.find(".datagrid-virtual-row-bottom"),e>0&&!this.topSpacer.length&&(this.topSpacer=$('<tr class="datagrid-virtual-row-top" style="height: '.concat(e,'px"><td colspan="').concat(this.visibleColumns().length,'"></td></tr>')),this.tableBody.prepend(this.topSpacer)),e>0&&this.topSpacer.length&&this.topSpacer.css("height","".concat(e,"px")),0===e&&(this.topSpacer.length||this.virtualRange.topRow<=1)&&this.topSpacer.remove(),t>0&&!this.bottomSpacer.length&&(this.bottomSpacer=$('<tr class="datagrid-virtual-row-bottom" style="height: '.concat(t,'px"><td colspan="').concat(this.visibleColumns().length,'"></td></tr>')),this.tableBody.append(this.bottomSpacer)),t>0&&this.bottomSpacer.length&&this.bottomSpacer.css("height","".concat(t,"px")),t<=0&&(this.bottomSpacer.length||this.virtualRange.bottomRow>=this.settings.dataset.length)&&this.bottomSpacer.remove()}},setAlternateRowShading:function(){this.settings.alternateRowShading&&this.settings.treeGrid&&$('tr[role="row"]:visible',this.tableBody).removeClass("alt-shading").filter(":odd").addClass("alt-shading")},formatValue:function(t,e,i,n,s,a,o){var r=arguments.length>7&&void 0!==arguments[7]&&arguments[7];return o=o||this,void 0===t&&(t=this.defaultFormatter),"string"==typeof t?Io[t](e,i,n,s,a,o,r).toString():t(e,i,n,s,a,o,r).toString()},rowHtml:function(t,e,i,n,s,a,o){var r,l,c,d=arguments.length>7&&void 0!==arguments[7]&&arguments[7],h=this,u=this.settings.summaryRow&&!n&&s,p=h.pagerAPI?h.pagerAPI.activePage:1,g={left:"",center:"",right:""},f=h.settings.treeDepth?h.settings.treeDepth[e]:0,m=0,v=!1;if(!t)return"";var b=h.settings.enableTooltips,y=!1;(u&&(t.isSummaryRow=u),h.settings.isRowDisabled&&"function"==typeof h.settings.isRowDisabled)&&(h.settings.isRowDisabled(i,t)&&(y=!0));if(t.isRowDisabled&&(y=!0),l=f=f?f.depth:0,h.settings.treeDepth&&h.settings.treeDepth.length)if(t._isFilteredOut)v=!0;else{var w,x=h.settings.treeDepth[e];if(x&&x.depth>1&&null!==(w=x.parents)&&void 0!==w&&(null==w?void 0:w.length))for(var C=0,k=x.parents.length;C<k;C++){var T=x.parents[C],A=h.settings.treeDepth[T];if(A.node&&void 0!==A.node.expanded&&!A.node.expanded){v=!0;break}}}if(this.settings.groupable&&!s){var S=this.settings.groupable;v=void 0!==S.expanded&&!S.expanded,S.expanded&&"function"==typeof S.expanded&&(v=!S.expanded(e,0,null,null,t,this),this.isFooterHidden=v)}var $=this.settings.frozenColumns.left.length,E=this.settings.frozenColumns.right.length,D=this.visibleColumns().length-$-E;if(this.settings.groupable&&n&&!s){var I=Io.GroupRow(e,0,null,null,t,this);return g.left='<tr class="datagrid-rowgroup-header'.concat(v?"":" is-expanded",'" role="rowgroup"><td role="gridcell" colspan="').concat($,'">').concat(I.left||"<span>&nbsp;</span>","</td></tr>"),g.center='<tr class="datagrid-rowgroup-header'.concat(v?"":" is-expanded",'" role="rowgroup"><td role="gridcell" colspan="').concat(D,'">').concat(I.center||"<span>&nbsp;</span>","</td></tr>"),g.right='<tr class="datagrid-rowgroup-header'.concat(v?"":" is-expanded",'" role="rowgroup"><td role="gridcell" colspan="').concat(E,'">').concat(I.right||"<span>&nbsp;</span>","</td></tr>"),g}if(this.settings.groupable&&n&&s){var L=Io.GroupFooterRow(e,0,null,null,t,this);return g.left='<tr class="datagrid-row datagrid-rowgroup-footer'.concat(this.isFooterHidden?" is-hidden":"",'" role="rowgroup">').concat(L.left||"<span>&nbsp;</span>","</tr>"),g.center='<tr class="datagrid-row datagrid-rowgroup-footer'.concat(this.isFooterHidden?" is-hidden":"",'" role="rowgroup">').concat(L.center||"<span>&nbsp;</span>","</tr>"),g.right='<tr class="datagrid-row datagrid-rowgroup-footer'.concat(this.isFooterHidden?" is-hidden":"",'" role="rowgroup">').concat(L.right||"<span>&nbsp;</span>","</tr>"),this.isFooterHidden=!1,g}var P=(h.settings.treeGrid||u?e:i)+1+(h.settings.source&&!h.settings.indeterminate?(p-1)*h.settings.pagesize:0);r=this.recordCount%2==0;var M=this.isRowSelected(t),B=t._rowactivated,O={class:"",svg:""};t&&t.rowStatus&&("new"!==t.rowStatus.icon||h.settings.showNewRowIndicator)&&(O.show=!0,O.class=" rowstatus-row-".concat(t.rowStatus.icon),O.icon="success"===t.rowStatus.icon?"#icon-check":"#icon-exclamation",O.title=""!==t.rowStatus.tooltip?' title="'.concat(t.rowStatus.tooltip,'"'):"",O.svg='<svg class="icon icon-rowstatus" focusable="false" aria-hidden="true" role="presentation"'.concat(O.title,'><use href="').concat(O.icon,'"></use></svg>'));var R="";if(this.settings.fixedRowHeight&&"function"==typeof this.settings.fixedRowHeight&&(R=' style="height: '.concat(this.settings.fixedRowHeight(this.recordCount,P,i,t),'"px" ')),this.settings.fixedRowHeight&&"number"==typeof this.settings.fixedRowHeight&&(R=' style="height: '.concat(this.settings.fixedRowHeight,'px" ')),this.currentPageRows){var z={ariaRowindex:P,actualIndexLineage:a,dataIndex:i,isFilteredOut:t._isFilteredOut};this.currentPageRows.push(z)}for(g.center='<tr role="row" aria-rowindex="'.concat(P,'"')+' data-index="'.concat(i,'"').concat(a?' data-lineage="'.concat(a,'"'):"").concat(h.settings.treeGrid&&t.children?' aria-expanded="'.concat(t.expanded?'true"':'false"'):"").concat(h.settings.treeGrid?' aria-level="'.concat(l,'"'):"").concat(y?' aria-disabled="true"':"").concat(M?' aria-selected="true"':"",' class="datagrid-row').concat(O.class).concat(v?" is-hidden":"").concat(t._isFilteredOut?" is-filtered":"").concat(B?" is-rowactivated":"").concat(y?" is-rowdisabled":"").concat(b?" is-tooltips-enabled":"").concat(M?"mixed"===this.settings.selectable?" is-selected hide-selected-color":" is-selected":"").concat(h.settings.alternateRowShading&&!r?" alt-shading":"").concat(u?" datagrid-summary-row":"").concat(h.settings.treeGrid?t.children?" datagrid-tree-parent":l>1?" datagrid-tree-child":"":"",'"').concat(R,">"),g.left=g.center,g.right=g.center,m=0;m<h.settings.columns.length;m++){var F,N,H,q,V,W,j,U,G=h.settings.columns[m],_=this.getContainer(G.id),Y="",K=G.summaryRowFormatter||G.formatter||h.defaultFormatter,X=u?K:G.formatter||h.defaultFormatter,Q=h.formatValue(X,e,m,h.fieldValue(t,h.settings.columns[m].field),h.settings.columns[m],t,h,d);if(0===Q.indexOf('<span class="is-readonly">')&&(G.readonly=!0),(Q.indexOf("datagrid-checkbox")>-1||Q.indexOf("btn-actions")>-1)&&(Y+=" l-center-text"),Q.indexOf("trigger")>-1&&(Y+=" datagrid-trigger-cell"),-1===Q.indexOf("trigger")&&G&&G.editor){var J=this.getEditorName(G.editor);["colorpicker","dropdown","time","lookup","date"].indexOf(J)>=0&&(Y+=" datagrid-trigger-cell datagrid-no-default-formatter")}if(X.name&&"Fileupload"===X.name&&(Y+=" is-fileupload"),G.editor&&this.settings.editable&&(Y+=" has-editor"),G.expanded&&(h.treeExpansionField=G.expanded),G.align&&(Y+=" l-".concat(G.align,"-text")),"ellipsis"===G.textOverflow&&(Y+=" text-ellipsis"),G.uppercase&&(Y+=" uppercase-text"),Y+=G.readonly?" is-readonly":"",Y+=G.hidden?" is-hidden":"",G.isEditable&&!G.readonly){var Z=h.fieldValue(t,h.settings.columns[m].field);G.isEditable(P-1,m,Z,G,t)||(Y+=" is-readonly")}var tt="selectionCheckbox"===G.id||!G.readonly&&void 0!==G.editor?"":'aria-readonly="true"';if(Y.indexOf("is-readonly")>-1&&(tt='aria-readonly="true"'),G.isReadonly&&!G.readonly&&"selectionCheckbox"!==G.id){var et=h.fieldValue(t,h.settings.columns[m].field);G.isReadonly(this.recordCount,m,et,G,t)&&(Y+=" is-cell-readonly",tt='aria-readonly="true"')}var it=h.fieldValue(t,h.settings.columns[m].field);G.cssClass&&"function"==typeof G.cssClass&&(Y+=" ".concat(G.cssClass(this.recordCount,m,it,G,t))),G.cssClass&&"string"==typeof G.cssClass&&(Y+=" ".concat(G.cssClass)),Y+=G.focusable?" is-focusable":"",Y+="Actions"===X.name?" has-btn-actions":"";var nt=this.calculateRowspan(it,e,G);if(""!==nt){var st=0;if(c>0&&!G.hidden){var at,ot,rt,lt;c-=1,Y+=" is-spanned-hidden";var ct=null===(at=this.settings)||void 0===at||null===(ot=null==(rt=at)?void 0:rt.frozenColumns)||void 0===ot||null==(lt=ot)?void 0:lt.left.length;ct>0&&m===ct&&(Y=Y.replace(" is-spanned-hidden"," is-spanned-invisible"),(st=c+1)>0&&this.visibleColumns().length-m===st&&(Y+=" is-spanned-last")),ct>0&&m<ct&&(Y=Y.replace(" is-spanned-hidden"," is-spanned-last"))}if(G.colspan&&"function"==typeof G.colspan&&!c){var dt=h.fieldValue(t,h.settings.columns[m].field);if((st=G.colspan(P-1,m,dt,G,t,h))>0){var ht,ut,pt,gt;c=st-1;var ft=null===(ht=this.settings)||void 0===ht||null===(ut=null==(pt=ht)?void 0:pt.frozenColumns)||void 0===ut||null==(gt=ut)?void 0:gt.left.length;ft>0&&m<ft&&(Y+=" is-spanned-last")}if(st>0){var mt,vt,bt,yt;c=st-1;var wt=null===(mt=this.settings)||void 0===mt||null===(vt=null==(bt=mt)?void 0:bt.frozenColumns)||void 0===vt||null==(yt=vt)?void 0:yt.left.length;wt>0&&m<wt&&-1===Y.indexOf("is-spanned-last")&&(Y+=" is-spanned-last")}st>0&&-1===Y.indexOf("is-spanned-last")&&this.visibleColumns().length-1===st&&(Y+=" is-spanned-last")}0!==m&&(O.class="",O.svg=""),""!==O.class&&(Y+=" rowstatus-cell"),h.isCellDirty(h.settings.groupable?i:e,m)&&(Y+=" is-dirty-cell"),h.uniqueId().indexOf("inline")>0&&(Y+=" has-inline-editor"),""!==Y&&(Y=Y.replace(/^\s+|\s+$/g,"").replace(/\s+/g," "));var xt=null===(F=this.settings.attributes)||void 0===F||null==(N=F)?void 0:N.filter((function(t){return"id"===t.name})),Ct='aria-describedby="'.concat(1===(null==xt||null==xt?void 0:xt.length)?"".concat(xt[0].value,"-col-").concat(null===(H=G.id)||void 0===H||null==(q=H)?void 0:q.toLowerCase()):h.uniqueId("-header-".concat(m)),'"'),kt="";if(null!=G&&(null==G?void 0:G.ariaDescribedBy)&&(Ct="aria-describedBy=".concat(G.ariaDescribedBy(t.id,m))),0===(null===(V=G.formatter)||void 0===V||null==(W=V)?void 0:W.toString().indexOf("function Checkbox"))){var Tt=void 0;Tt=G.isChecked?G.isChecked(it):void 0!==it&&(!0===it||1===parseInt(it,10)),kt=' aria-checked="'.concat(Tt,'"')}if(0===(null===(j=G.formatter)||void 0===j||null==(U=j)?void 0:U.toString().indexOf("function SelectionCheckbox("))&&(kt=' aria-checked="'.concat(this.isRowSelected(t),'"')),g[_]+='<td role="gridcell" '.concat(tt,' aria-colindex="').concat(m+1,'"')+" ".concat(Ct).concat(kt).concat(M?' aria-selected="true"':"").concat(Y?' class="'.concat(Y,'"'):"").concat(st?' colspan="'.concat(st,'"'):"").concat(G.tooltip&&"string"==typeof G.tooltip?' title="'.concat(G.tooltip.replace("{{value}}",it),'"'):"").concat(h.settings.columnGroups?'headers = "'.concat(h.uniqueId("-header-".concat(m))," ").concat(h.getColumnGroup(m),'"'):"").concat(nt||"",">").concat(O.svg,'<div class="datagrid-cell-wrapper"\n        ').concat(G.whiteSpace?'style="white-space: '.concat(G.whiteSpace,'"'):"",">"),G.contentVisible)G.contentVisible(e+1,m,it,G,t)||(Q="");h.settings.onPostRenderCell&&G.component&&(g[_]+='<div class="content"></div>',Q=""),g[_]+="".concat(Q,"</div></td>")}}if(this.settings.spacerColumn&&(g.center+='<td class="datagrid-spacer-column"></td>'),g.left+="</tr>",g.center+="</tr>",g.right+="</tr>",h.settings.rowTemplate&&!u){var At=h.settings.rowTemplate,St=t,$t=h.settings.rowTemplateHeight||107,Et="";je&&St&&(Et=je.compile("{{#dataset}}".concat(At,"{{/dataset}}"),{dataset:St})),this.hasLeftPane&&(g.left+='<tr class="datagrid-expandable-row no-border"><td colspan="'.concat($,'">\n          <div class="datagrid-row-detail"><div style="height: ').concat($t,'px"></div></div>\n          </td></tr>')),g.center+='<tr class="datagrid-expandable-row '.concat(St.expanded?" is-expanded":"",'"><td colspan="').concat(D+(this.settings.spacerColumn?1:0),'">\n        <div class="datagrid-row-detail" ').concat(St.expanded?' style="height: auto"':"",'><div class="datagrid-row-detail-padding">').concat(Et,"</div></div>\n        </td></tr>"),this.hasRightPane&&(g.right+='<tr class="datagrid-expandable-row no-border"><td colspan="'.concat(E,'">\n          <div class="datagrid-row-detail"><div style="height: ').concat($t,'px"></div></div>\n          </td></tr>'))}if(h.settings.expandableRow&&(this.hasLeftPane&&(g.left+='<tr class="datagrid-expandable-row"><td colspan="'.concat($,'">\n          <div class="datagrid-row-detail"><div class="datagrid-row-detail-padding"></div></div>\n          </td></tr>')),g.center+='<tr class="datagrid-expandable-row"><td colspan="'.concat(D,'">\n        <div class="datagrid-row-detail"><div class="datagrid-row-detail-padding"></div></div>\n        </td></tr>'),this.hasRightPane&&(g.right+='<tr class="datagrid-expandable-row no-border"><td colspan="'.concat(E,'">\n          <div class="datagrid-row-detail"><div class="datagrid-row-detail-padding"></div></div>\n          </td></tr>'))),!t.children||o||t._isFilteredOut){if(this.settings.treeGrid&&!o&&t._isFilteredOut)if(t.children){var Dt=function t(e){var i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;i+=e.length;for(var n=0,s=e.length;n<s;n++)e[n].children&&(i=t(e[n].children,i));return i}(t.children);this.recordCount+=Dt+(t._isFilteredOut&&1===l?1:0)}else 1===l&&this.recordCount++}else for(var It=0,Lt=t.children.length;It<Lt;It++){var Pt=a?"".concat(a,".").concat(i):"".concat(i);this.recordCount++;var Mt=h.rowHtml(t.children[It],this.recordCount,It,!1,!1,Pt);g.left+=Mt.left,g.center+=Mt.center,g.right+=Mt.right}return g},getEditorName:function(t){if(!t)return"";var e=t.name;if(!function(){}.name){e=(t.toString().match(/function (.+?)\(/)||[,""])[1]}return e?e.toLowerCase():""},calculateTextWidth:function(t){var e=t.name||"",i=0,n=0,s=0,a="",o=!1,r=this;if(t.formatter===Io.Colorpicker)a="";else if(t.formatter===Io.Dropdown&&t.options&&"data"!==this.settings.columnSizing){for(var l="",c=0;c<t.options.length;c++)t.options[c].label.length>l.length&&(l=t.options[c].label);l=r.formatValue(t.formatter,0,0,l,t,null,r),a=l=g.stripHTML(l)}else{var d=0,h=this.settings.dataset;this.settings.groupable&&(h=this.originalDataset);for(var u=0;u<h.length;u++){var p=this.fieldValue(h[u],t.field),f=h[u];if(o=(p=r.formatValue(t.formatter,u,0,p,t,f,r)).toString().indexOf("btn-secondary")>-1,d=(p=g.stripHTML(p)).toString().length,this.settings.groupable&&f)for(var m=0;m<f.length;m++){var v=this.fieldValue(f[m],t.field);v=r.formatValue(t.formatter,u,0,v,t,f,r),(d=(v=g.stripHTML(v)).toString().length)>i&&(i=d,a=v)}d>i&&(i=d,a=p)}if(this.filterExpr&&this.filterExpr.length>0){var b=$.grep(this.filterExpr,(function(e){return e.columnId===t.id}));if(b&&1===b.length){var y=b[0].value;(d=y.toString().length)>i&&(i=d,a=y)}}if(""===a&&(t.formatter===Io.Date||t.formatter===Io.Time)){var w=new Date(9999,11,31,23,59,59,999);w=r.formatValue(t.formatter,0,0,w,t,null,r),a=w=g.stripHTML(w)}}var x=!!t.formatter&&t.formatter.toString().indexOf('<span class="tag')>-1,C=!!t.formatter&&t.formatter.toString().indexOf("datagrid-alert-icon")>-1,k=t.formatter===Io.Date||t.formatter===Io.Time||t.formatter===Io.Lookup||t.formatter===Io.Colorpicker||t.formatter===Io.Dropdown;s+=2*this.getCellPadding(),C&&(s+=20),x&&(s+=10),o&&(s+=50),this.settings.editable&&t.editor===jo.Spinbox&&(s+=46),!this.settings.editable||t.formatter!==Io.Dropdown&&t.formatter!==Io.Lookup&&t.editor!==jo.Time||(s+=10),this.settings.editable&&t.editor===jo.Date&&(s+=5),n=this.calculateTextRenderWidth(a)+s,t.formatter===Io.Colorpicker&&(n=150);var T=void 0===t.sortable||t.sortable?2*this.getCellPadding()+15:2*this.getCellPadding(),A=this.calculateTextRenderWidth(e,!0)+T;return t.filterType&&this.settings.filterable&&(A<2*this.getCellPadding()&&(A=5*this.getCellPadding()+n+36),"checkbox"!==t.filterType&&(""!==a?A<n+(2*this.getCellPadding()+32)&&""!==a&&(A=n+(2*this.getCellPadding()+32)):A<100&&(A=100)),"ellipsis"!==(null==t||null==t?void 0:t.textOverflow)||"small"!==this.settings.rowHeight&&"extra-small"!==this.settings.rowHeight||(A+=6)),"data"===this.settings.columnSizing&&(A=50+2*this.getCellPadding()+(k?25:0)),"header"===this.settings.columnSizing&&(n=0),Math.ceil(Math.max(n,A))},calculateTextRenderWidth:function(t,e){var i,n;if(this.canvas=this.canvas||(this.canvas=document.createElement("canvas")),!this.canvas||null===(i=this.canvas)||void 0===i||!(null==i?void 0:i.getContext))return 0;var s=null===(n=this.canvas)||void 0===n||null==n?void 0:n.getContext("2d"),a=ft.currentTheme.id.indexOf("uplift")>-1||ft.currentTheme.id.indexOf("new")>-1;return this.fontCached&&this.fontHeaderCached||(this.fontCached=ft.currentTheme.id&&a?"400 16px arial":"400 14px arial",this.fontHeaderCached=ft.currentTheme.id&&a?"600 14px arial":"700 12px arial","extra-small"===this.settings.rowHeight&&(this.fontCached=ft.currentTheme.id&&a?"400 14px arial":"400 12px arial",this.fontHeaderCached=ft.currentTheme.id&&a?"600 14px arial":"700 12px arial")),null!=s&&(null==s?void 0:s.font)?(s.font=this.fontCached,e&&(s.font=this.fontHeaderCached),s.measureText(t).width):0},setScrollClass:function(){if(this.bodyWrapperCenter[0]&&this.bodyWrapperCenter[0].offsetHeight){var t=parseInt(this.bodyWrapperCenter[0].offsetHeight,10),e=this.headerRow?this.headerRow[0].offsetHeight:0,i=parseInt(this.tableBody[0].offsetHeight,10);if(this.element.removeClass("has-vertical-scroll has-visible-last-row has-horizontal-scroll"),i<t-e&&this.element.addClass("has-visible-last-row"),this.hasLeftPane||this.hasRightPane){var n=parseInt(this.bodyWrapperCenter[0].scrollHeight,10)>t+2,s=parseInt(this.bodyWrapperCenter[0].offsetWidth,10),a=parseInt(this.bodyWrapperCenter[0].scrollWidth,10)>s;n&&this.element.addClass("has-vertical-scroll"),a&&this.element.addClass("has-horizontal-scroll")}}},clearCache:function(){this.totalWidths.left=0,this.totalWidths.center=0,this.totalWidths.right=0,this.elemWidth=0,this.stretchColumnWidth=0,this.stretchColumnDiff=0,this.stretchColumnIdx=-1,this.fontCached=null,this.fontHeaderCached=null,this.fixColumnIds()},fixColumnIds:function(){for(var t=0;t<this.settings.columns.length;t++){var e=this.settings.columns[t],i=this.columnById(e.id);if(i.length>1)for(var n=1;n<i.length;n++)i[n].id="".concat(i[n].id,"-").concat(n)}},getStretchColumnIdx:function(){var t=this,e=t.stretchColumnIdx;return-1===e&&"last"!==t.settings.stretchColumn&&t.headerNodes().each((function(i){$(this).attr("data-column-id")===t.settings.stretchColumn&&(e=i)})),e},columnWidth:function(t,e){return this.elemWidth||(this.element.parents(".modal").length>0?this.elemWidth=!1===this.element.parents(".modal").hasClass("is-visible")?void 0:this.element.outerWidth():this.elemWidth=this.element.outerWidth(),0===this.elemWidth&&(this.elemWidth=this.element.closest(".tab-container").outerWidth()),this.widthSpecified=!1,this.widthPixel=!1),this.calculateColumnWidth(t,e)},calculateColumnWidth:function(t,e){var i,n,s,a=this.visibleColumns(!0),o=e===this.visibleColumns().length-1,r=this.getContainer(t.id);t.width&&(this.widthSpecified=!0,this.widthPercent=!1),!this.widthPixel&&t.width&&(this.widthPixel="string"!=typeof t.width);var l=t.width;"string"==typeof t.width&&-1===t.width.indexOf("px")&&(this.widthPercent=!0,s=t.width.replace("%",""));var c=this.calculateTextWidth(t);if(this.widthSpecified&&l||(l=Math.max(c,l||0)),(!this.widthSpecified||void 0===t.width)&&this.settings.sizeColumnsEqually&&-1===["selectionCheckbox","expander","drilldown","rowStatus","favorite"].indexOf(t.id)){var d=Math.round(this.elemWidth/a.length);l=d-(o?2:0),d<c&&(l=c)}if("selectionCheckbox"!==t.id&&"selectionRadio"!==t.id&&"favorite"!==t.id||(l=43,t.width=l),"favorite"===t.id&&(l=62,t.width=l),!t.width&&(null===(i=t.formatter)||void 0===i||null===(n=null==i?void 0:i.toString())||void 0===n||null==n?void 0:n.indexOf("ProcessIndicator"))>-1&&(l=155,t.width=l),"expander"===t.id&&(l=55,t.width=l),"rowStatus"===t.id&&(l=62,t.width=l),"rowReorder"===t.id&&(l=this.settings.groupable?105:62,t.width=l),"drilldown"===t.id&&(l=62,t.width=l),t.minWidth&&l<t.minWidth&&(l=t.minWidth),t.minWidth&&l>t.maxWidth&&(l=t.maxWidth),this.totalWidths[r]+=t.hidden?0:l,o){var h=this.elemWidth-this.totalWidths[r];"last"!==this.settings.stretchColumn&&null!==this.settings.stretchColumn&&(this.stretchColumnDiff=h),"last"!==this.settings.stretchColumn&&this.settings.stretchColumn!==t.id||(this.stretchColumnIdx=e,h<l?this.stretchColumnWidth=l:(l=h,this.stretchColumnDiff=l)),this.hasLeftPane&&this.tableLeft.css("width",this.totalWidths.left),this.hasRightPane&&this.tableRight.css("width",this.totalWidths.right)}return"last"!==this.settings.stretchColumn&&null!==this.settings.stretchColumn&&this.settings.stretchColumn===t.id?(this.stretchColumnIdx=e,this.stretchColumnWidth=l,' style="width: 99%"'):this.widthPercent||void 0!==l?' style="width: '.concat(this.widthPercent?"".concat(s,"%"):"".concat(l,"px"),'"'):""},calculateRowspan:function(t,e,i){var n=0,s=null;if(!i.rowspan)return null;for(var a=0;a<this.settings.dataset.length;a++)t===this.settings.dataset[a][i.field]&&(n++,null===s&&(s=a));return e===s?' rowspan ="'.concat(n,'"'):""},calculateTotals:function(){return this.settings.totals=zo.aggregate(this.settings.dataset,this.settings.columns),this.settings.totals},setUnit:function(t){return t+(/(px|%)/i.test("".concat(t))?"":"px")},setupTooltips:function(t,e){var i=this;if(t||e||this.settings.enableTooltips){var n,s=this,a=s.element.find(".datagrid-header-icon").length>0,o={th:".datagrid-header th",td:'.datagrid-wrapper tbody tr.datagrid-row td[role="gridcell"]',rowstatus:'.datagrid-wrapper tbody tr.datagrid-row td[role="gridcell"] .icon-rowstatus',headerIcon:".datagrid-header th .datagrid-header-icon"};this.settings.filterable?(o.headerColumn="".concat(o.th," .datagrid-column-wrapper"),o.headerFilter="".concat(o.th," .datagrid-filter-wrapper .btn-menu"),o.header="".concat(o.headerColumn,", ").concat(o.headerFilter)):o.header=a?o.headerIcon:o.th,o.iconAlert="".concat(o.td," .icon-alert"),o.iconSuccess="".concat(o.td," .icon-success"),o.iconError="".concat(o.td," .icon-error"),o.iconInfo="".concat(o.td," .icon-info"),o.icons="".concat(o.iconAlert,", ").concat(o.iconSuccess,", ").concat(o.iconError,", ").concat(o.iconInfo),t&&this.settings.enableTooltips?o.str="".concat(o.header,", ").concat(o.td,", ").concat(o.icons,", ").concat(o.rowstatus):o.str=t?"".concat(o.header,", ").concat(o.rowstatus):"".concat(o.header,", ").concat(o.td,", ").concat(o.icons);var r=function(t,e){e=void 0===e?400:e,n=setTimeout((function(){var e=f.hasClass(t,"datagrid-column-wrapper"),i=f.hasClass(t.parentNode,"datagrid-filter-wrapper"),n=f.hasClass(t,"datagrid-header-icon"),a=!!i&&t.parentNode.querySelectorAll(".popupmenu.is-open").length>0,o=$(t).data("gridtooltip")||s.cacheTooltip(t),r=e||n?t.parentNode:t,l=s.getOuterWidth(r);o&&(o.forced||o.textwidth>l-35)&&!a&&s.showTooltip(o)}),e)},l=function(t,e){e=void 0===e?400:e,clearTimeout(n),setTimeout((function(){$(".grid-tooltip:hover").length>0||(s.hideTooltip(),f.hasClass(t.parentNode,"datagrid-filter-wrapper")&&s.removeTooltipData(t))}),e)};this.element.off("mouseenter.gridtooltip focus.gridtooltip",o.str).on("mouseenter.gridtooltip focus.gridtooltip",o.str,(function(){r(this)})).off("mouseleave.gridtooltip click.gridtooltip blur.gridtooltip",o.str).on("mouseleave.gridtooltip click.gridtooltip blur.gridtooltip",o.str,(function(){l(this)})).off("longpress.gridtooltip",o.str).on("longpress.gridtooltip",o.str,(function(){r(this,0)})).off("keydown.gridtooltip",o.str).on("keydown.gridtooltip",o.str,(function(t){var e=t.which||t.keyCode||t.charCode||0,i=!1;return t.shiftKey&&112===e?r(this,0):27===e&&(i=s.isGridtooltip(),l(this,0)),i&&t.preventDefault(),!i})).off("mousemove.gridtooltip").on("mousemove.gridtooltip",(function(t){$(t.target).parents(".datagrid").length>0||!$(".grid-tooltip").hasClass("is-hidden")&&!$(".grid-tooltip:hover").length>0&&l(i)})),this.toolbar&&this.toolbar.parent().find(".table-errors").length>0&&this.toolbar.parent().find(".table-errors").off("mouseenter.tableerrortooltip",".icon").on("mouseenter.tableerrortooltip",".icon",(function(){r(this)})).off("mouseleave.tableerrortooltip click.tableerrortooltip",".icon").on("mouseleave.tableerrortooltip click.tableerrortooltip",".icon",(function(){l(this)})).off("longpress.tableerrortooltip",".icon").on("longpress.tableerrortooltip",".icon",(function(){r(this,0)})),this.element.find(".datagrid-header-icon").length>0&&this.element.find(".datagrid-header-icon").off("mouseenter.headericon").on("mouseenter.headericon",(function(){r(this)})).off("mouseleave.headericon").on("mouseleave.headericon",(function(){l(this)}))}},getOuterWidth:function(t){var e=getComputedStyle(t),i=t.offsetWidth;return i+=parseInt(e.marginLeft,10)+parseInt(e.marginRight,10)},closest:function(t,e){return t&&(e(t)?t:this.closest(t.parentNode,e))},headerNodes:function(){return this.headerRow?this.element.find(".datagrid-header tr:not(.datagrid-header-groups) th"):$()},colGroupNodes:function(){return this.headerRow?this.element.find("colgroup col"):$()},updateRow:function(t,e){var i=this.settings,n=e;n||(n=i.treeGrid?i.treeDepth[t].node:this.getActiveDataset()[t]);for(var s=0;s<this.settings.columns.length;s++){var a=this.settings.columns[s];a.hidden||a.component&&this.settings.onPostRenderCell||this.updateCellNode(t,s,this.fieldValue(n,a.field),!0)}i.treeGrid||void 0===i.dataset[t]||(this.settings.dataset[t]=E.extend(!0,this.settings.dataset[t],e)),this.settings.rowReorder&&this.tableBody.data("arrange")&&this.tableBody.data("arrange").updated(),this.postRenderCell()},updateColumns:function(t,e){void 0===e?e=this.settings.columnGroups:null!==e&&0!==e.length||(this.settings.groupable=null);var i=[];this.settings.filterable&&this.filterRowRendered&&(i=this.filterConditions()),this.settings.columns=t,this.settings.columnGroups=e,this.rerender(),this.element.trigger("columnchange",[{type:"updatecolumns",columns:this.settings.columns}]),this.saveUserSettings(),this.settings.filterable&&this.filterRowRendered&&this.setFilterConditions(i)},saveUserSettings:function(){this.element.trigger("settingschanged",[{rowHeight:this.settings.rowHeight,columns:this.settings.columns,sortOrder:this.sortColumn,pagesize:this.settings.pagesize,showPageSizeSelector:this.settings.showPageSizeSelector,activePage:this.pagerAPI?this.pagerAPI.activePage:1,filter:this.filterConditions()}]);var t=this.settings.saveUserSettings;!$.isEmptyObject(t)&&this.canUseLocalStorage()&&(t.columns&&(localStorage[this.uniqueId("usersettings-columns")]=this.copyThenStringify(this.settings.columns)),t.rowHeight&&(localStorage[this.uniqueId("usersettings-row-height")]=this.settings.rowHeight),t.sortOrder&&(localStorage[this.uniqueId("usersettings-sort-order")]=JSON.stringify(this.sortColumn)),t.pagesize&&(localStorage[this.uniqueId("usersettings-pagesize")]=this.settings.pagesize),t.showPageSizeSelector&&(localStorage[this.uniqueId("usersettings-show-pagesize-selector")]=this.settings.showPageSizeSelector),t.activePage&&this.pagerAPI&&(localStorage[this.uniqueId("usersettings-active-page")]=this.pagerAPI.activePage),t.filter&&(localStorage[this.uniqueId("usersettings-filter")]=JSON.stringify(this.filterConditions())))},canUseLocalStorage:function(){try{if(localStorage.getItem)return!0}catch(t){return!1}return!1},setOriginalColumns:function(){this.originalColumns=E.deepCopy(this.settings.columns)},columnsFromString:function(t,e){if(!t)return[];var i=JSON.parse(t);if(!i)return[];for(var n=0;n<i.length;n++){var s=void 0,a=this.columnById(i[n].id),o=a.width;a&&(s=i[n].hidden,$.extend(i[n],a[0]),void 0!==s?i[n].hidden=s:delete i[n].hidden,e&&(i[n].width=o))}return i},restoreColumns:function(t){if(this.settings.saveColumns&&this.canUseLocalStorage())if(t)this.updateColumns(t);else{var e=localStorage[this.uniqueId("columns")];!t&&e&&(this.originalColumns=E.deepCopy(this.settings.columns),this.settings.columns=this.columnsFromString(e))}},restoreUserSettings:function(t){var e=this.settings.saveUserSettings;if(t||!$.isEmptyObject(e)&&this.canUseLocalStorage())if(t){if(t.columns&&this.updateColumns(t.columns),t.rowHeight&&this.rowHeight(t.rowHeight),t.sortOrder&&this.setSortColumn(t.sortOrder.sortId,t.sortOrder.sortAsc),t.pagesize&&(this.settings.pagesize=parseInt(t.pagesize,10),this.pagerAPI.settings.pagesize=parseInt(t.pagesize,10),t.activePage||this.pagerAPI.setActivePage(1,!0)),t.showPageSizeSelector&&(this.settings.showPageSizeSelector=t.showPageSizeSelector,this.pagerAPI.showPageSizeSelector(t.showPageSizeSelector)),t.filter&&this.applyFilter(t.filter,"restore"),t.activePage){var i=parseInt(t.activePage,10);this.pagerAPI.setActivePage(i,!0),this.restoreActivePage=!0}}else{if(e.columns){var n=localStorage[this.uniqueId("usersettings-columns")];n&&(this.settings.columns=this.columnsFromString(n))}if(e.rowHeight){var s=localStorage[this.uniqueId("usersettings-row-height")];s&&(this.settings.rowHeight=s)}if(e.sortOrder){var a=localStorage[this.uniqueId("usersettings-sort-order")];a&&(this.sortColumn=JSON.parse(a),this.restoreSortOrder=!0)}if(e.pagesize){var o=localStorage[this.uniqueId("usersettings-pagesize")];o&&(this.settings.pagesize=parseInt(o,10))}if(e.showPageSizeSelector){var r=localStorage[this.uniqueId("usersettings-show-pagesize-selector")];(r="true"===r.toString().toLowerCase())&&(this.settings.showPageSizeSelector=r)}if(e.activePage){var l=localStorage[this.uniqueId("usersettings-active-page")];l&&(this.savedActivePage=parseInt(l,10),this.restoreActivePage=!0)}if(e.filter){var c=localStorage[this.uniqueId("usersettings-filter")];c&&(this.savedFilter=JSON.parse(c),this.restoreFilter=!0)}}},copyThenStringify:function(t){if(!t)return JSON.stringify(t);var e=t.map((function(t){var e=E.extend({},t);return e.editorOptions&&delete e.editorOptions,e}));return JSON.stringify(e)},resetColumns:function(){if(this.canUseLocalStorage()&&localStorage.removeItem(this.uniqueId("columns")),this.originalColumns){var t=E.deepCopy(this.originalColumns),e=this.settings.columnGroups&&this.originalColGroups?this.originalColGroups:null;this.updateColumns(t,e)}this.clearFilter()},hideColumn:function(t){var e,i,n,s,a,o,r,l,c=this.columnIdxById(t);if(!(-1===c||null!==(e=this.settings.columns[c])&&void 0!==e&&(null==e?void 0:e.hidden))){this.settings.columns[c].hidden=!0;var d=this.headerNodes().eq(c);this.headerNodeCheckbox=d.find(".datagrid-checkbox").length>0?d:void 0,null!==(i=this.settings)&&void 0!==i&&null!==(n=null==i?void 0:i.frozenColumns)&&void 0!==n&&(null==n?void 0:n.left.length)||this.headerNodes().eq(c).addClass("is-hidden"),0===c&&"selectionCheckbox"===t&&null!==(s=this.settings)&&void 0!==s&&null!==(a=null==s?void 0:s.frozenColumns)&&void 0!==a&&(null==a?void 0:a.left.length)?this.headerNodes().eq(c).off().remove():this.headerNodes().eq(c).addClass("is-hidden"),this.colGroupNodes().eq(c).addClass("is-hidden");var h=(null===(o=this.settings)||void 0===o||null===(r=null==o?void 0:o.frozenColumns)||void 0===r||null==r?void 0:r.left.length)||0;if(this.tableBody.find("> tr > td:nth-child(".concat(c-h+1,")")).addClass("is-hidden"),this.bodyColGroup.find("col").eq(c-h).addClass("is-hidden"),null!==(l=this.tableBodyLeft)&&void 0!==l&&(null==l?void 0:l.find("> tr > td .datagrid-checkbox-wrapper"))&&(this.bodyNodeCheckboxes=this.tableBodyLeft.find("> tr > td:nth-child(".concat(c+1,")")),this.tableBodyLeft.find("> tr > td:nth-child(".concat(c+1,")")).remove()),this.updateColumnGroup(c,!1),this.hasColSpans){var u=this.element.find(".datagrid-header th").eq(c).attr("colspan");if(u&&u>0){u-=1;for(var p=0;p<u;p++)c+=u,this.tableBody.find("> tr > td:nth-child(".concat(c+1,")")).addClass("is-hidden")}}(this.settings.rowTemplate||this.settings.expandableRow)&&this.syncExpandableRowColspan(),this.element.trigger("columnchange",[{type:"hidecolumn",index:c,columns:this.settings.columns}]),this.saveUserSettings()}},syncExpandableRowColspan:function(){var t=this.visibleColumns().length;this.tableBody.find(".datagrid-expandable-row > td").attr("colspan",t)},showColumn:function(t){var e,i,n=this.columnIdxById(t);if(-1!==n&&this.settings.columns[n].hidden){this.settings.columns[n].hidden=!1,this.headerNodes().eq(n).removeClass("is-hidden"),this.colGroupNodes().eq(n).removeClass("is-hidden");var s=(null===(e=this.settings)||void 0===e||null===(i=null==e?void 0:e.frozenColumns)||void 0===i||null==i?void 0:i.left.length)||0;if(this.tableBody.find("> tr > td:nth-child(".concat(n-s+1,")")).removeClass("is-hidden"),this.bodyColGroup.find("col").eq(n-s).removeClass("is-hidden"),s){var a,o;if(this.tableBodyLeft.find("> tr > td:nth-child(".concat(n+1,")")).removeClass("is-hidden"),this.headerNodeCheckbox&&!this.headerRowLeft.find("> tr th:nth-child(".concat(n+1,") .datagrid-checkbox-wrapper")).length)null===(a=this.headerRowLeft)||void 0===a||null==a||a.find("> tr").prepend(this.headerNodeCheckbox);if(this.bodyNodeCheckboxes&&!this.tableBodyLeft.find("> tr td:nth-child(".concat(n+1,") .datagrid-checkbox-wrapper")).length)null===(o=this.tableBodyLeft)||void 0===o||null==o||o.find("> tr").prepend(this.bodyNodeCheckboxes[n])}if(this.updateColumnGroup(n,!0),this.hasColSpans){var r=this.element.find(".datagrid-header th").eq(n).attr("colspan");if(r&&r>0){r-=1;for(var l=0;l<r;l++)n+=r,this.tableBody.find("> tr > td:nth-child(".concat(n+1,")")).removeClass("is-hidden")}}(this.settings.rowTemplate||this.settings.expandableRow)&&this.syncExpandableRowColspan(),this.element.trigger("columnchange",[{type:"showcolumn",index:n,columns:this.settings.columns}]),this.saveUserSettings()}},exportToCsv:function(t,e,i){var n=arguments.length>3&&void 0!==arguments[3]&&arguments[3];Lo.exportToCsv(t,e,i,n,this)},exportToExcel:function(t,e,i){var n=arguments.length>3&&void 0!==arguments[3]&&arguments[3];Lo.exportToExcel(t,e,i,n,this)},copyToDataSet:function(t,e,i,n){Lo.copyToDataSet(t,e,i,n,this)},personalizeColumns:function(){var t=this,e='<div class="listview-search alternate-bg"><label class="audible" for="gridfilter">Search</label><input class="searchfield" placeholder="'.concat(H.translate("SearchColumnName"),'" name="searchfield" id="gridfilter"></div>');e+='<div class="listview alternate-bg" id="search-listview"><ul></ul></div>',$("body").modal({title:H.translate("PersonalizeColumns"),content:e,cssClass:"full-width datagrid-columns-dialog",buttons:[{text:H.translate("Close"),click:function(t,e){e.close(),$("body").off("beforeopen.datagrid")}}]}).off("beforeopen.datagrid").on("beforeopen.datagrid",(function(e,i){i&&(t.isColumnsChanged=!1,i.element.find(".searchfield").searchfield({clearable:!0,tabbable:!1}),i.element.find(".listview").listview({source:t.settings.columns,template:'\n            <ul>\n            {{#dataset}}\n              {{#name}}\n              <li>\n                <a href="#" target="_self" tabindex="-1">\n                  <label class="inline">\n                    <input tabindex="-1" type="checkbox" class="checkbox" {{^hideable}}disabled{{/hideable}} {{^hidden}}checked{{/hidden}} data-column-id="{{id}}"/>\n                    <span class="label-text">{{name}}</span>\n                  </label>\n                </a>\n              </li>\n              {{/name}}\n            {{/dataset}}\n            </ul>',searchable:!0,selectOnFocus:!1,listFilterSettings:{filterMode:"contains",searchableTextCallback:function(t){return t.name||t.id}}}).off("selected.datagrid").on("selected.datagrid",(function(e,i){var n=i.elem.find(".checkbox"),s=n.attr("data-column-id"),a=n.prop("checked");if(i.elem.removeClass("is-selected hide-selected-color"),!n.is(":disabled")){t.isColumnsChanged=!0;var o={node:{},api:$(this).data("listview")};if(o.api){var r=t.columnIdxById(s);-1!==r&&o.api.settings.dataset[r]&&(o.node=o.api.settings.dataset[r])}a?(t.hideColumn(s),n.prop("checked",!1),o.node.hidden=!0):(t.showColumn(s),n.prop("checked",!0),o.node.hidden=!1),t.settings.groupable&&t.rerender()}})),i.element.on("close.datagrid",(function(){t.isColumnsChanged=!1})),i.element.on("keydown.datagrid",(function(t){27===t.keyCode&&(i.close(),$("body").off("beforeopen.datagrid"))})))}))},setColumnWidth:function(t,e,i){var n=parseFloat(e),s=this.columnById(t)[0],a=-1;if(n&&t&&("string"==typeof t&&this.headerNodes().each((function(e){$(this).attr("data-column-id")===t&&(a=e)})),"number"!=typeof e&&(e=n/100*this.element.width()),!(Math.abs(e-s.width)<2))){if("number"==typeof s.colspan&&(e/=s.colspan),s&&(s.width=e),i)this.bodyColGroup.find("col").eq(a)[0].style.width="".concat(e,"px");this.element.trigger("columnchange",[{type:"resizecolumn",index:a,columns:this.settings.columns}]),this.saveUserSettings()}},isEllipsisActiveHeader:function(t){return(void 0===(t=t||{}).sortable||t.sortable)&&"ellipsis"===t.textOverflow},activeEllipsisHeaderAll:function(){for(var t=0,e=this.settings.columns.length;t<e;t++){var i=this.settings.columns[t].id,n=this.columnById(i)[0];if(this.isEllipsisActiveHeader(n)){var s=this.element[0].querySelector('.datagrid-header th[data-column-id="'.concat(i,'"]'));this.activeEllipsisHeader(s)}}},activeEllipsisHeader:function(t){if(t){var e=t.querySelector(".datagrid-column-wrapper .datagrid-header-text"),i=t.scrollWidth<e.scrollWidth+65;t.classList[i?"add":"remove"]("is-ellipsis-active")}},createResizeHandle:function(){var t=this,e=this;if(!this.resizeHandle){var i,n,s,a,o;this.resizeHandle=$('<div class="resize-handle" aria-hidden="true"></div>'),this.settings.columnGroups&&(this.resizeHandle[0].style.height="80px"),this.settings.filterable&&(this.resizeHandle[0].style.height="62px"),this.element.find("table").before(this.resizeHandle);var r=!1;this.resizeHandle.drag({axis:"x"}).on("dragstart.datagrid",(function(){e.currentHeader&&(e.dragging=!0,r=!1,n=e.currentHeader.attr("data-column-id"),i=e.columnById(n)[0],e.isEllipsisActiveHeader(i)&&e.currentHeader[0].classList.add("is-ellipsis-active"))})).on("drag.datagrid",(function(l,c){if(e.currentHeader&&i){var d=i.minWidth||12,h=i.maxWidth||2e3,u=e.currentHeader,p=u.index(),g="left"===e.getContainer(u.attr("data-column-id"));t.dragging=!0;var m=c.left+5,v=t.bodyColGroup.find("col").eq(p)[0];g&&(v=t.bodyColGroupLeft.find("col").eq(p)[0]);var b=parseInt(e.currentHeader.width(),10),y=parseInt(v.style.width||b,10);v.style.width.indexOf("%")>-1&&(y=b);var w=parseFloat(e.currentHeader.offsetParent().offset().left),x=parseFloat(e.currentHeader.offset().left),C=0===p?0:x-w-2;e.hasLeftPane&&!g&&0===p&&(C=x-w-2);var k=b-(m-C);if((s=y-k)<d||s>h)e.resizeHandle.css("cursor","inherit");else if(s!==y)if(v.style.width="".concat(s,"px"),(p===t.settings.frozenColumns.left.length-1||p<=t.settings.frozenColumns.left.length-1&&k>0)&&"left"===t.getContainer(n)&&(t.totalWidths.left+=k<0?Math.abs(k):-k,t.tableLeft.css("width",t.totalWidths.left)),et.pressedKeys.get("Shift")&&(r=!0),"fit"===t.settings.resizeMode&&!r||"flex"===t.settings.resizeMode&&r){var T=H.isRTL()?p+1:p-1,A=e.settings.columns[T];if(!A)return;var S=A.minWidth||12,$=A.maxWidth||1e3,E=H.isRTL()?f.getPreviousSibling(e.currentHeader,":not(.is-hidden)"):f.getNextSibling(e.currentHeader,":not(.is-hidden)");o=E.getAttribute("data-column-id");var D=H.isRTL()?f.getPreviousSibling(v,":not(.is-hidden)"):f.getNextSibling(v,":not(.is-hidden)"),I=parseInt(E.offsetWidth,10),L=parseInt(D.style.width||I,10);if(D.style.width.indexOf("%")>-1&&(L=I),(a=L+k)<S||a>$)return void e.resizeHandle.css("cursor","inherit");if(a===L||!D)return;D.style.width="".concat(a,"px")}}})).on("dragend.datagrid",(function(){t.dragging=!1,e.isEllipsisActiveHeader(i)&&e.activeEllipsisHeader(e.currentHeader[0]),e.setColumnWidth(n,s),o&&("fit"===t.settings.resizeMode&&!r||"flex"===t.settings.resizeMode&&r)&&e.setColumnWidth(o,a)}))}},displayCounts:function(t){var e,i,n,s,a,o=this,r=o.tableBody.find("tr:visible").length,l=0,c="",d=[],h=o.settings.paging&&!o.settings.source,u=function(t){return H.formatNumber(t,{style:"integer"})};(!h&&t||(this.recordCount=o.settings.dataset.length,r=o.settings.dataset.length),(this.settings.treeGrid&&h||!t)&&(this.filteredCount=o.settings.dataset.filter((function(t){return t._isFilteredOut})).length,r=o.settings.dataset.length,this.recordCount=o.settings.dataset.filter((function(t){return!t._isFilteredOut})).length),o.settings.groupable&&(l=r,r=o.originalDataset.length,c=(c="({0} ".concat(H.translate(1===l?"Group":"Groups"),")")).replace("{0}",u(l))),o.contextualToolbar&&o.contextualToolbar.length&&(o._selectedRows.forEach((function(t){d.push(t.idx);var e=d.filter((function(t,e){return d.indexOf(t)===e}));o.contextualToolbar.find(".selection-count").text("".concat(e.length," ").concat(H.translate("Selected")))})),o.settings.allowSelectAcrossPages&&o.contextualToolbar.find(".selection-count").text("".concat(o._selectedRows.length," ").concat(H.translate("Selected")))),t&&-1!==t&&(r=t),void 0===t&&this.settings.source&&(r=o.settings.dataset.length),void 0===t&&this.settings.source&&null!==(e=this.pagerAPI)&&void 0!==e&&null!==(i=null==e?void 0:e.state)&&void 0!==i&&(null==i?void 0:i.total))&&(r=null===(n=this.pagerAPI)||void 0===n||null===(s=null==n?void 0:n.state)||void 0===s||null==s?void 0:s.total);if(o.settings.showFilterTotal&&o.filteredCount>0)a=(a=(a="(".concat(H.translate(1===r?"ResultOf":"ResultsOf"),")")).replace("{0}",u(r-o.filteredCount))).replace("{1}",u(r));else{var p=H.translate(1===r?"Result":"Results");a=(a=-1===p.indexOf("{0}")?"({0} ".concat(p,")"):p).replace("{0}",u(r))}if(o.settings.resultsText)if("function"==typeof o.settings.resultsText)if(o.grandTotal)a=o.settings.resultsText(o,o.grandTotal,r===o.grandTotal?0:r);else{var g=0===o.filteredCount?0:r-o.filteredCount;a=o.settings.resultsText(o,r,g)}else a=o.settings.resultsText;if(o.toolbar){f.html(o.toolbar.find(".datagrid-result-count"),a,"<span>"),f.html(o.toolbar.find(".datagrid-group-count"),c,"<span>"),o.toolbar[0].setAttribute("aria-label",o.toolbar.find(".title").text()),o.toolbar.find(".datagrid-row-count").text(r),E.addAttributes(o.toolbar.find(".title"),this,this.settings.attributes,"title"),E.addAttributes(o.toolbar.find(".btn-actions"),this,this.settings.attributes,"actions");var m=o.toolbar.find("#gridfilter");E.addAttributes(m,this,this.settings.attributes,"search"),m.prev("label").attr("for",m.attr("id"))}f.html(o.element.closest(".modal").find(".datagrid-result-count"),a,"<span>"),this.lastCount=r,this.checkEmptyMessage()},setEmptyMessage:function(t){this.element["small"===(null==t||null==t?void 0:t.height)?"addClass":"removeClass"]("empty-message-height-small"),this.emptyMessage?(this.emptyMessage.settings=t,this.emptyMessage.updated()):(this.emptyMessageContainer=$('<div class="empty-message-container"><div></div></div>'),this.element.append(this.emptyMessageContainer).addClass("has-empty-message"),this.emptyMessage=this.emptyMessageContainer.find("div").emptymessage(t).data("emptymessage"),this.checkEmptyMessage())},checkEmptyMessage:function(){this.emptyMessage&&this.emptyMessageContainer&&(0===this.recordCount?(this.emptyMessageContainer.show(),this.element.addClass("is-empty")):(this.emptyMessageContainer.hide(),this.element.removeClass("is-empty")))},triggerRowEvent:function(t,e,i){var n=this,s=$(e.target),a=s.closest("tr"),o=s.closest("td"),r=o.index(),l=this.settings.treeGrid?this.actualRowIndex(a):this.dataRowIndex(a),c=!0;(s.is("a")||s.closest("a").length)&&(i=!1),i&&(e.stopPropagation(),e.preventDefault());var d=n.settings.dataset[l];if(this.settings.groupable&&(a.is(".datagrid-rowgroup-header, .datagrid-rowgroup-footer")?c=!1:(l=n.actualPagingRowIndex(n.actualRowIndex(a)),n.groupArray[l]&&(d=n.settings.dataset[n.groupArray[l].group])&&d.values&&(d=d.values[n.groupArray[l].node]))),c){var h={row:l,rowElem:a,cell:r,cellElem:o,item:d,originalEvent:e};n.element.trigger(t,[h])}return!1},verticalScrollToEnd:function(t){var e=t.currentTarget,i=(e.scrollTop+e.clientHeight)/e.scrollHeight*100,n=Math.round(i),s=e.scrollHeight-e.scrollTop-e.clientHeight<1&&100===n;this.element.trigger("scroll",{percent:n,percentScrolled:100}),this.isVerticalScrollToEnd=!1!==s},handleScrollY:function(t){var e=t.currentTarget.scrollTop;e!==this.scrollTop&&this.bodyWrapperCenter&&(this.bodyWrapperLeft||this.bodyWrapperRight)&&(this.scrollTop=e,this.bodyWrapperLeft&&(this.bodyWrapperLeft[0].scrollTop=this.scrollTop),this.bodyWrapperRight&&(this.bodyWrapperRight[0].scrollTop=this.scrollTop),this.bodyWrapperCenter[0].scrollTop=this.scrollTop)},rerender:function(){this.clearCache(),this.renderRows(),this.renderHeader()},handleEvents:function(){var t=this,e=this;if($("body").on("open.modal.datagrid",(function(e){var i=$(e.target);i.find(t.table).length>0&&t.table.width()>i.width()&&t.widthSpecified&&t.table.width(466)})),e.element.on("focus.datagrid","tbody > tr",(function(){if(!e.settings.cellNavigation&&e.settings.rowNavigation){var t=e.rowNodes($(this));if(!t.hasClass("is-active-row")){t.addClass("is-active-row");var i=e.actualRowIndex(t);e.settings.keyRowSelect&&e.selectRow(i)}}})).on("blur.datagrid","tbody > tr",(function(){if(!e.settings.cellNavigation&&e.settings.rowNavigation){var t=e.rowNodes($(this));t.hasClass("is-active-row")&&t.removeClass("is-active-row")}})),this.attachPagingEvents(),e.settings.showHoverState&&e.element.off("mouseenter.datagrid, mouseleave.datagrid").on("mouseenter.datagrid","tbody > tr",(function(){var t=e.rowNodes($(this));t.addClass("is-hover-row").removeClass("is-clickable"),e.settings.cellNavigation||!1===e.settings.selectable||t.addClass("is-clickable")})).on("mouseleave.datagrid","tbody > tr",(function(){e.rowNodes($(this)).removeClass("is-hover-row")})),(this.hasLeftPane||this.hasRightPane||this.hasVerticalScrollbar)&&(e.element.find(".datagrid-wrapper").on("scroll.table",(function(t){e.handleScrollY(t),e.verticalScrollToEnd(t)})),e.element.find(".datagrid-wrapper").on("wheel.table",(function(t){0!==t.originalEvent.deltaY&&(t.currentTarget.scrollTop+=t.originalEvent.deltaY,t.preventDefault(),e.handleScrollY(t),e.verticalScrollToEnd(t))}))),this.settings.virtualized){var i=0,n=0;e.bodyWrapperCenter.on("scroll.vtable",b((function(){var t=this.scrollTop,n=t>e.virtualRange.bottom-e.virtualRange.bodyHeight-25,s=t<e.virtualRange.top+25;t!==i&&(s||n)&&(i=this.scrollTop,e.clearCache(),e.renderRows())}),0)),$("body").on("resize.vtable",(function(){this.offsetHeight!==n&&(n=this.scrollTop,e.clearCache(),e.renderRows())}))}e.settings.frozenColumns.left.length&&$("body").on("resize.frozencolumns",(function(){e.setScrollClass()})),this.element.off("click.datagrid-header").on("click.datagrid-header",".datagrid-header th.is-sortable, .datagrid-header th.btn-filter",(function(t){var i=$(t.target).parent();return i.is(".datagrid-filter-wrapper")||i.is(".lookup-wrapper")||i.is(".has-close-icon-button")||(t.stopPropagation(),e.editor=null,e.setSortColumn($(this).attr("data-column-id"))),!1})),this.element.off("click.datagrid-links").on("click.datagrid-links","tbody .datagrid-row a",(function(t){var e=t.currentTarget.getAttribute("href");e&&"#"!==e||t.preventDefault()})),e.settings.allowPasteFromExcel&&e.settings.editable&&this.element.off("paste.datagrid").on("paste.datagrid",(function(t){var i;i=t.originalEvent.clipboardData&&t.originalEvent.clipboardData.getData?t.originalEvent.clipboardData.getData("text/plain"):!(!window.clipboardData||!window.clipboardData.getData)&&window.clipboardData.getData("Text");var n=/\n/.exec(i),s=/\r/.exec(i),a=/\r\n/.exec(i);if((!e.activeCell||!e.activeCell.node.hasClass("is-readonly"))&&(i&&s||n||a)){var o=n?i.split("\n"):i.split("\r");a&&(o=i.split("\r\n")),o.pop();var r=parseInt($(t.target)[0].parentElement.parentElement.parentElement.getAttribute("data-index"),10),l=parseInt($(t.target)[0].parentElement.parentElement.getAttribute("aria-colindex"),10)-1;e.editor&&e.editor.input&&e.commitCellEdit(),e.copyToDataSet(o,r,l,e.settings.dataset)}}));var s=e.settings.dblClickApply?"dblclick.datagrid, click.datagrid, select.datagrid":"click.datagrid, select.datagrid";this.element.off(s).on(s,"tbody td",(function(t){var i,n,s=null,a=null,o=$(t.target),r=o.closest("td"),l=r.closest("table").find("thead > tr > th:not([colspan])").eq(r.index()).attr("data-column-id"),c=e.settings.columns[e.columnIdxById(l)];if(("select"!==t.type||null!=c&&(null==c?void 0:c.inlineEditor))&&(1!==$(t.currentTarget).closest(".datagrid-expandable-row").length||"gridcell"===$(t.currentTarget).attr("role")))if(!r.is(".has-btn-actions")||o.is(".btn-actions")){o.parents("td").length>1&&(t.preventDefault(),t.stopPropagation());var d=$("#colorpicker-menu.is-open");if(e.settings.dblClickApply?e.triggerRowEvent(t.type,t,!0):e.triggerRowEvent("click",t,!0),e.setActiveCell(r),o.is(".datagrid-expand-btn"))return s=$(this).closest("tr"),a=e.actualRowIndex(s),e.toggleRowDetail(a),e.toggleGroupChildren(s),e.toggleChildren(t,a),!1;var h=o.is(".datagrid-selection-checkbox")||1===o.find(".datagrid-selection-checkbox").length,u=o.is(".datagrid-selection-radio")||1===o.find(".datagrid-selection-radio").length,p=!!e.settings.clickToSelect||(!!h||u);if(o.is(".datagrid-drilldown")&&(p=!1),"mixed"===e.settings.selectable&&((p=h)||(t.shiftKey&&e.activatedRow().length&&(e.selectRowsBetweenIndexes([e.activatedRow()[0].row,o.closest("tr").attr("aria-rowindex")-1]),t.preventDefault()),e.toggleRowActivation(o.closest("tr")))),p&&("multiple"===e.settings.selectable||"mixed"===e.settings.selectable)&&t.shiftKey)e.selectRowsBetweenIndexes([e.lastSelectedRow,o.closest("tr").attr("aria-rowindex")-1]),t.preventDefault();else if(p&&e.settings.dblClickApply){var g="dblclick"===t.type;e.toggleRowSelection(o.closest("tr"),g),"dblclick"!==t.type&&e.element.triggerHandler("ischanged")}else p&&e.toggleRowSelection(o.closest("tr"));var f=r.attr("aria-colindex")-1,m=e.columnSettings(f);if(!m||o.is(".datagrid-checkbox")||m.editor!==jo.Checkbox)if(1!==(null===(i=d)||void 0===i||null==i?void 0:i.length)){var v=e.makeCellEditable(e.activeCell.rowIndex,e.activeCell.cell,t);if(m.click&&"function"==typeof m.click&&o.is("button, input[checkbox], a, a.search-mode i, img")||o.parent().is("button:not(.trigger)")){var b=$(this).closest("tr"),y=e.actualRowIndex(b);a=e.dataRowIndex(b);var w=e.settings.treeGrid?e.settings.treeDepth[y].node:e.settings.dataset[a];if(r.hasClass("is-focusable")&&!o.is(e.buttonSelector)&&!o.parent("button").is(e.buttonSelector))return;e.settings.groupable&&(b.is(".datagrid-rowgroup-header, .datagrid-rowgroup-footer")||(y=e.actualPagingRowIndex(e.actualRowIndex(b)),(w=e.settings.dataset[e.groupArray[y].group])&&w.values&&(w=w.values[e.groupArray[y].node]))),(!r.hasClass("is-cell-readonly")&&!o.is("[disabled]")&&o.is("button, input[checkbox], a, a.search-mode i, img")||o.parent().is("button"))&&m.click(t,[{row:y,cell:e.activeCell.cell,item:w,originalEvent:t}]),o.is("[disabled]")&&m.formatter===Io.Hyperlink&&(t.stopImmediatePropagation(),t.preventDefault())}if(null!=c&&(null==c?void 0:c.inlineEditor)&&"click"===t.type&&"input"===t.target.nodeName.toLowerCase()){var x=t.target;x.focus(),x.select(),t.preventDefault(),t.stopPropagation()}if(m.menuId){e.closePrevPopupmenu();var C=$(this).find("button");C.popupmenu({attachToBody:!0,menuId:m.menuId,trigger:"immediate",offset:{y:5},returnFocus:function(){return r.focus()}}).off("close.gridpopupbtn").on("close.gridpopupbtn",(function(){var t=$(this);t.data("popupmenu")&&!t.data("tooltip")&&t.data("popupmenu").destroy()})),m.selected&&C.off("selected.gridpopupbtn").on("selected.gridpopupbtn",m.selected)}v&&null!==(n=e.settings)&&void 0!==n&&(null==n?void 0:n.actionableMode)&&setTimeout((function(){$("textarea, input",r).length&&!$(".dropdown,[type=file],[type=image],[type=button],[type=submit],[type=reset],[type=checkbox],[type=radio]",r).length&&(e.quickEditMode=!0)}),0)}else d=null}else e.setActiveCell(r)})),"last"!==this.stretchColumn&&($(window).on("orientationchange.datagrid",(function(){t.rerender()})),$(window).on("resize.datagrid",(function(){var i=0;for(t.clearCache(),i=0;i<e.settings.columns.length;i++){var n=e.settings.columns[i];e.columnWidth(n,i)}(e.stretchColumnDiff>0||e.stretchColumnWidth>0)&&(e.bodyColGroup.find("col").eq(e.getStretchColumnIdx())[0].style.width="".concat(e.stretchColumnDiff>0?"".concat(e.stretchColumnDiff,"px"):"".concat(e.stretchColumnWidth,"px")))}))),this.settings.dblClickApply||this.element.off("dblclick.datagrid").on("dblclick.datagrid","tbody tr",(function(t){e.triggerRowEvent("dblclick",t,!0)})),this.element.off("contextmenu.datagrid").on("contextmenu.datagrid","tbody td",(function(t){var i=$(t.currentTarget);t.stopPropagation(),e.closePrevPopupmenu(),e.triggerRowEvent("contextmenu",t,!!e.settings.menuId);var n=function(){return e.settings.menuId&&$("#".concat(e.settings.menuId)).length>0};return!n()||!E.isSubscribedTo(e.element[0],t,"contextmenu","datagrid")&&!n()||(t.preventDefault(),e.setActiveCell(parseInt(i.parent().attr("aria-rowindex"),10)-1,parseInt(i.attr("aria-colindex"),10)-1),e.activeCell.node.focus(),i.popupmenu({menuId:e.settings.menuId,eventObj:t,beforeOpen:e.settings.menuBeforeOpen,attachToBody:!0,trigger:"immediate"}).off("selected.gridpopuptr").on("selected.gridpopuptr",(function(t,i){e.settings.menuSelected&&e.settings.menuSelected(t,i)})).off("close.gridpopuptr").on("close.gridpopuptr",(function(){var t=$(this);t.data("popupmenu")&&t.data("popupmenu").destroy()})),!1)})),this.headerRow.add(this.headerRowLeft).off("mousemove.datagrid").on("mousemove.datagrid","th",(function(t){if(!e.dragging&&(e.currentHeader=$(t.target).closest("th"),e.currentHeader.hasClass("is-resizable")&&!e.currentHeader.hasClass("datagrid-header-spacer-column"))){var i=e.currentHeader.closest(".header-detail"),n=i.length?parseInt(i.css("margin-left"),10):0,s=parseInt(e.currentHeader.position().left,10)-(n||0)+e.element.scrollLeft(),a=s+e.currentHeader.outerWidth(),o=t.pageX-s>a-t.pageX,r=0;r=o?a-5:s-5;var l=e.currentHeader.parent().find("th:visible").index(e.currentHeader);(0===l&&(H.isRTL()?o:!o)||l===e.visibleColumns().length)&&(r="-999"),H.isRTL()||o||(e.currentHeader=e.currentHeader.prevAll(":visible").not(".is-hidden").first()),H.isRTL()&&!o&&(e.currentHeader=e.currentHeader.nextAll(":visible").not(".is-hidden").first()),e.currentHeader.hasClass("is-resizable")&&(e.createResizeHandle(),e.resizeHandle[0].style.left="".concat(r,"px"),e.resizeHandle[0].style.cursor="")}})).off("contextmenu.datagrid").on("contextmenu.datagrid","th",(function(t){return!e.settings.headerMenuId||(t.preventDefault(),e.closePrevPopupmenu(),$(t.currentTarget).popupmenu({menuId:e.settings.headerMenuId,eventObj:t,attachToBody:!0,beforeOpen:e.settings.headerMenuBeforeOpen,trigger:"immediate"}).off("selected.gridpopupth").on("selected.gridpopupth",(function(t,i){e.settings.headerMenuSelected(t,i)})).off("close.gridpopupth").on("close.gridpopupth",(function(){var t=$(this);t.data("popupmenu")&&t.data("popupmenu").destroy()})),!1)})),this.element.off("click.datagrid-header-select").on("click.datagrid-header-select","th .datagrid-checkbox",(function(){var t=$(this);t.hasClass("is-checked")?(t.removeClass("is-checked").attr("aria-checked","false"),e.settings.selectAllCurrentPage?e.unSelectAllRowsCurrentPage():e.unSelectAllRows()):(t.addClass("is-checked").attr("aria-checked","true"),e.settings.selectAllCurrentPage?e.selectAllRowsCurrentPage():e.selectAllRows())})),this.element.off("focusout.datagrid").on("focusout.datagrid","tbody td input, tbody td textarea, tbody div.dropdown",(function(t){var i=t.target;$(i).is("input.lookup, input.timepicker, input.datepicker, input.spinbox, input.colorpicker")?setTimeout((function(){var t=$("*:focus");if(!$(".lookup-modal.is-visible, #timepicker-popup, #monthview-popup, #colorpicker-menu").length&&e.editor){if(t.is(".spinbox, .trigger, .code-block-actions")||!$(i).is(":visible"))return;if(t&&e.editor.className&&t.closest(e.editor.className).length>0)return;e.commitCellEdit()}}),150):$("#dropdown-list, .autocomplete.popupmenu.is-open, #timepicker-popup, .is-editing .code-block").is(":visible")||e.editor&&e.editor.input&&e.commitCellEdit()}))},attachPagingEvents:function(){var t=this,e=this;this.settings.paging&&(this.pagerId=$(".pager-toolbar .btn-menu").attr("aria-controls"),this.tableBody.off("page.".concat(Uo)).on("page.".concat(Uo),(function(i,n){"filtered"===n.type&&t.settings.source||(e.closePopupmenuOnPaging(),e.saveUserSettings(),e.render(null,n),e.afterPaging(n))})).off("pagesizechange.".concat(Uo)).on("pagesizechange.".concat(Uo),(function(t,i){i.preserveSelected=!0,e.closePopupmenuOnPaging(),e.render(null,i),e.afterPaging(i)})))},closePopupmenuOnPaging:function(){var t=this.element.find(".datagrid-filter-wrapper .btn-filter.is-open"),e=null==t||null==t?void 0:t.data("popupmenu");$(".pager-toolbar .btn-menu").attr("aria-controls",this.pagerId).length&&(null==e||null==e||e.close(!0,!0))},closePrevPopupmenu:function(){[].slice.call(this.element[0].querySelectorAll(".is-open:not(.popupmenu)")).forEach((function(t){var e=$(t);e.data("popupmenu")&&e.trigger("close")}))},refreshSelectedRowHeight:function(){var t=this.element.parent().find(".toolbar:not(.contextual-toolbar)").find(".btn-actions").data("popupmenu");if(null!=t&&(null==t?void 0:t.wrapper)){var e=t.wrapper.find('[data-option="row-extra-small"]'),i=t.wrapper.find('[data-option="row-small"]'),n=t.wrapper.find('[data-option="row-short"]'),s=t.wrapper.find('[data-option="row-medium"]'),a=t.wrapper.find('[data-option="row-normal"]'),o=t.wrapper.find('[data-option="row-large"]');"extra-small"===this.settings.rowHeight&&(e.parent().addClass("is-checked"),i.parent().removeClass("is-checked"),n.parent().removeClass("is-checked"),s.parent().removeClass("is-checked"),a.parent().removeClass("is-checked"),o.parent().removeClass("is-checked")),"short"!==this.settings.rowHeight&&"small"!==this.settings.rowHeight||(e.parent().removeClass("is-checked"),i.parent().addClass("is-checked"),n.parent().addClass("is-checked"),s.parent().removeClass("is-checked"),a.parent().removeClass("is-checked"),o.parent().removeClass("is-checked")),"medium"===this.settings.rowHeight&&(e.parent().removeClass("is-checked"),i.parent().removeClass("is-checked"),n.parent().addClass("is-checked"),s.parent().addClass("is-checked"),a.parent().removeClass("is-checked"),o.parent().removeClass("is-checked")),"normal"!==this.settings.rowHeight&&"large"!==this.settings.rowHeight||(e.parent().removeClass("is-checked"),i.parent().removeClass("is-checked"),n.parent().addClass("is-checked"),s.parent().removeClass("is-checked"),a.parent().addClass("is-checked"),o.parent().addClass("is-checked")),$(".drag-target-arrows",this.element).css("height","".concat(this.getTargetHeight(),"px"))}},appendToolbar:function(){var t=null,e="",i=null,n=this.settings.useFlexToolbar,s=this;if(this.settings.toolbar){if(1===this.element.parent().find(".toolbar:not(.contextual-toolbar), .flex-toolbar:not(.contextual-toolbar)").length)t=this.element.parent().find(".toolbar:not(.contextual-toolbar), .flex-toolbar:not(.contextual-toolbar)"),this.refreshSelectedRowHeight();else{t=$('<div class="'.concat(n?"flex-toolbar":"toolbar",' datagrid-toolbar" role="toolbar"></div>')),this.removeToolbarOnDestroy=!0,this.settings.toolbar.title&&(e=$('<div class="'.concat(n?"toolbar-section ":"",'title">').concat(this.settings.toolbar.title,"  </div>"))),e||(e=t.find(".title")),t.append(e),this.settings.toolbar.results&&e.append('<span class="datagrid-result-count"></span>');var a=$('<div class="'.concat(n?"toolbar-section ":"",'buttonset"></div>')).appendTo(t),o=$('<div class="'.concat(n?"toolbar-section ":"",'search"></div>')).appendTo(t);if(this.settings.toolbar.keywordFilter){var r=$('<label class="audible" for="gridfilter">'.concat(H.translate("Keyword"),"</label>")),c=$('<input class="searchfield" name="searchfield" placeholder="'.concat(H.translate("Keyword"),'" id="gridfilter">'));this.settings.toolbar.collapsibleFilter||c.attr("data-options",'{ "collapsible": false }'),this.settings.useFlexToolbar?(o.append(r),o.append(c)):(a.append(r),a.append(c))}this.settings.toolbar.dateFilter&&a.append('<button class="btn" type="button">'.concat($.createIcon({icon:"calendar"}),"<span>").concat(H.translate("Date"),"</span></button>")),this.settings.toolbar.actions&&((i=$('<div class="'.concat(n?"toolbar-section ":"",'more"></div>'))).append('<button class="btn-actions" title="More" type="button">'.concat($.createIcon({icon:"more"}),'<span class="audible">Grid Features</span></button>')),t.addClass("has-more-button"));var d=$('<ul class="popupmenu"></ul>');if(this.settings.toolbar.personalize&&d.append('<li><a href="#" data-option="personalize-columns">'.concat(H.translate("PersonalizeColumns"),"</a></li>")),this.settings.toolbar.resetLayout&&d.append('<li><a href="#" data-option="reset-layout">'.concat(H.translate("ResetDefault"),"</a></li>")),this.settings.toolbar.exportToExcel&&d.append('<li><a href="#" data-option="export-to-excel">'.concat(H.translate("ExportToExcel"),"</a></li>")),this.settings.toolbar.exportToCsv&&d.append('<li><a href="#" data-option="export-to-csv">'.concat(H.translate("ExportToCsv"),"</a></li>")),this.settings.toolbar.advancedFilter&&d.append('<li><a href="#">'.concat(H.translate("AdvancedFilter"),"</a></li>")),this.settings.toolbar.views&&d.append('<li><a href="#">'.concat(H.translate("SaveCurrentView"),"</a></li> ")+'<li class="separator"></li> '+'<li class="heading">'.concat(H.translate("SavedViews"),"</li>")+'<li><a href="#">View One</a></li>'),this.settings.toolbar.rowHeight&&d.append("".concat('<li class="separator single-selectable-section"></li><li class="heading">').concat(H.translate("RowHeight"),"</li>")+'<li class="is-selectable'.concat("extra-small"===this.settings.rowHeight?" is-checked":"",'"><a href="#" data-option="row-extra-small">').concat(H.translate("ExtraSmall"),"</a></li>")+'<li class="is-selectable'.concat("short"===this.settings.rowHeight||"small"===this.settings.rowHeight?" is-checked":"",'"><a href="#" data-option="row-small">').concat(H.translate("Small"),"</a></li>")+'<li class="is-selectable'.concat("medium"===this.settings.rowHeight?" is-checked":"",'"><a href="#" data-option="row-medium">').concat(H.translate("Medium"),"</a></li>")+'<li class="is-selectable'.concat("normal"===this.settings.rowHeight||"large"===this.settings.rowHeight?" is-checked":"",'"><a href="#" data-option="row-large">').concat(H.translate("Large"),"</a></li>")),!0===this.settings.toolbar.filterRow&&d.append("".concat('<li class="separator"></li><li class="heading">').concat(H.translate("Filter"),"</li>")+'<li class="'.concat(this.settings.filterable?"is-checked ":"",'is-toggleable"><a href="#" data-option="show-filter-row">').concat(H.translate("ShowFilterRow"),"</a></li>")+'<li class="is-indented"><a href="#" data-option="run-filter">'.concat(H.translate("RunFilter"),"</a></li>")+'<li class="is-indented"><a href="#" data-option="clear-filter">'.concat(H.translate("ClearFilter"),"</a></li>")),"object"===l(this.settings.toolbar.filterRow)){var h='<li class="separator"></li>';this.settings.toolbar.filterRow.showFilter&&(h+='<li class="'.concat(this.settings.filterable?"is-checked ":"",'is-toggleable"><a href="#" data-option="show-filter-row">').concat(H.translate("ShowFilterRow"),"</a></li>")),this.settings.toolbar.filterRow.runFilter&&!this.settings.filterWhenTyping&&(h+='<li class="is-indented"><a href="#" data-option="run-filter">'.concat(H.translate("RunFilter"),"</a></li>")),this.settings.toolbar.filterRow.clearFilter&&(h+='<li class="is-indented"><a href="#" data-option="clear-filter">'.concat(H.translate("ClearFilter"),"</a></li>")),d.append(h)}this.settings.toolbar.actions&&i.append(d),this.settings.useFlexToolbar&&t.append(o),t.append(a),t.append(i),this.element.prev().is(".contextual-toolbar")?this.element.prev().before(t):this.element.before(t)}var u=t.find(".btn-actions"),p=t.is(".flex-toolbar");this.settings.stickyHeader&&t.addClass("is-sticky"),p||u.popupmenu();if((p?t:u).on("selected",(function(t,e,i){var n=null!=e&&(null==e?void 0:e.attr)?null==e||null==e?void 0:e.attr("data-option"):null==i||null==i?void 0:i.attr("data-option");n&&("row-extra-small"!==n&&"row-small"!==n&&"row-short"!==n&&"row-medium"!==n&&"row-normal"!==n&&"row-large"!==n||s.rowHeight(n.substr(4)),"personalize-columns"===n&&s.personalizeColumns(),"reset-layout"===n&&s.resetColumns(),"export-to-excel"===n&&s.exportToExcel(),"export-to-csv"===n&&s.exportToCsv(),"show-filter-row"===n&&s.toggleFilterRow(),"run-filter"===n&&s.applyFilter(null,"menu"),"clear-filter"===n&&s.clearFilter())})),this.settings.initializeToolbar&&!t.data("toolbar")&&!t.hasClass("flex-toolbar")){var g=$.fn.parseOptions(t);this.settings.toolbar.fullWidth&&(g.rightAligned=!0),t.toolbar(g)}if(this.settings.initializeToolbar&&t.hasClass("flex-toolbar")&&!t.data("toolbarFlex")){var f=$.fn.parseOptions(t);this.settings.toolbar.collapsibleFilter&&(f.collapsibleFilter=!0),t.toolbarflex(f)}if(this.settings.toolbar&&this.settings.toolbar.keywordFilter){var m=t.find(".searchfield"),v=m.parent().find(".close.icon");m.off("keypress.datagrid").on("keypress.datagrid",(function(t){13!==t.keyCode&&"change"!==t.type||(t.preventDefault(),s.keywordSearch(m.val()))})),v.off("click.datagrid").on("click.datagrid",(function(){s.keywordSearch(m.val())}))}if(this.settings.toolbar&&this.settings.toolbar.contextualToolbar){this.element.before('\n        <div class="contextual-toolbar datagrid-contextual-toolbar toolbar is-hidden">\n          <div class="title selection-count">1 Selected</div>\n        </div>'),this.contextualToolbar=this.element.prev(".contextual-toolbar")}this.toolbar=t,this.element.addClass("has-toolbar")}},rowHeight:function(t){var e=[];return t&&(this.settings.rowHeight!==t&&e.push({before:this.settings.rowHeight,after:t}),this.settings.rowHeight=t),this.setRowHeightClass(),this.virtualRange&&this.virtualRange.rowHeight&&(this.virtualRange.rowHeight="normal"===t||"large"===t?40:"medium"===t?30:25,"extra-small"===t&&(this.virtualRange.rowHeight=22)),this.saveUserSettings(),this.refreshSelectedRowHeight(),this.rerender(),e.length&&this.element.triggerHandler("rowheightchanged",e),this.settings.rowHeight},setRowHeightClass:function(){var t=this.settings.rowHeight;"short"===t&&(t="small"),"large"===t&&(t="normal"),this.element.add(this.table).add(this.tableLeft).add(this.tableRight).removeClass("extra-small-rowheight small-rowheight short-rowheight medium-rowheight normal-rowheight large-rowheight").addClass("".concat(t,"-rowheight")),this.refreshSelectedRowHeight()},keywordSearch:function(t){this.element.find('tr[role="row"]').removeClass("is-filtered").show(),this.filterExpr=[],this.element.find(".datagrid-expandable-row").each((function(){var t=$(this);t.prev().find(".datagrid-expand-btn").removeClass("is-expanded"),t.prev().find(".plus-minus").removeClass("active"),t.removeClass("is-expanded").css("display",""),t.find(".datagrid-row-detail").css("height","")})),this.element.find(".search-mode").each((function(){var t=$(this),e=t.text();t.text(e.replace("<i>","").replace("</i>",""))})),t=(t||"").toLowerCase(),this.filterExpr.push({column:"all",operator:"contains",value:t,keywordSearch:!0}),this.filterKeywordSearch(),this.clearCache(),this.renderRows(),this.setSearchActivePage({trigger:"searched",type:"filtered"}),this.settings.paging&&this.settings.source||this.highlightSearchRows(t)},setSearchActivePage:function(t){if(this.pagerAPI){var e=this;t||(t={});var i=!0;this.filterExpr&&1===this.filterExpr.length?""!==this.filterExpr[0].value?(t.activePage=this.pagerAPI.filteredActivePage||1,this.settings.source&&this.settings.allowSelectAcrossPages&&t.activePage!==this.activePage&&(i=!1)):""===this.filterExpr[0].value&&this.pagerAPI.filteredActivePage&&(t=n(t)):this.restoreActivePage||(t=n(t),this.restoreActivePage=!1),i&&(t.activePage=this.activePage),this.renderPager(t)}function n(t){return t.activePage=1,e.grandTotal&&(t.grandTotal=e.grandTotal),t}},filterKeywordSearch:function(){var t,e,i,n,s=this,a=s.filterExpr,o=function(t,e){var i=!1,n=function(i){var n=s.columnById(i)[0],o=s.fieldValue(t,n.field),r=s.settings.columns.indexOf(n);return s.formatValue(n.formatter,e,r,o,n,t,s).toLowerCase().replace(/(<([^>]+)>)|(amp;)|(&lt;([^>]+)&gt;)/gi,"").indexOf(g.escapeHTML(a.value))>-1};return"all"===a.column?s.element.find("th:visible").each((function(){var t=$(this).attr("data-column-id");if(t&&(i=n(t)))return!1})):a.columnId&&(i=n(a.columnId)),i};if(a&&1===a.length)if(a=a[0],s.settings.treeGrid)for(i=0,n=(t=s.settings.treeDepth).length;i<n;i++)e=""!==a.value&&!o(t[i].node,i),t[i].node._isFilteredOut=e;else for(i=0,n=(t=s.settings.dataset).length;i<n;i++)e=""!==a.value&&!o(t[i],i),t[i]._isFilteredOut=e},highlightSearchRows:function(t){var e=this,i=this;if(t&&""!==t){i.tableBody.find("tr").toArray().forEach((function(n){var s=i.rowNodes(n.getAttribute("aria-rowindex")-1),a=function(n){var s=!1;return n.toArray().forEach((function(n){[].slice.call(n.querySelectorAll("td")).forEach((function(n){var a=n.innerText.toLowerCase(),o=!!i.settings.searchExpandableRow||!f.hasClass(e,"datagrid-expandable-row");a.indexOf(t)>-1&&o&&(s=!0,[].slice.call(n.querySelectorAll("*")).forEach((function(e){[].slice.call(e.childNodes).forEach((function(e){var i=e.parentElement;if(3===e.nodeType&&"i"!==i.tagName.toLowerCase()&&g.unescapeHTML(i.innerHTML)===e.textContent){var n=e.textContent,s=new RegExp("(".concat(M.escapeRegExp(t),")"),"gi");f.addClass(i,"search-mode"),f.html(i,n.replace(s,"<i>$1</i>"))}}))})))}))})),s}(s);a||s.find(".datagrid-row-detail").length?i.settings.searchExpandableRow&&a&&s.is(".datagrid-expandable-row")&&""!==t&&(s.prev().show(),s.prev().find(".datagrid-expand-btn").addClass("is-expanded"),s.prev().find(".plus-minus").addClass("active"),s.addClass("is-expanded").css("display","table-row"),s.find(".datagrid-row-detail").css("height","auto")):s.addClass("is-filtered").hide()}))}},getActiveDataset:function(){var t=this.settings,e=t.treeGrid?t.treeDepth:t.dataset;return t.groupable&&(e=this.originalDataset),e},getDataset:function(){return this.settings.dataset.map((function(t){return delete t._isFilteredOut,t}))},selectAllRows:function(){for(var t=[],e=this.getActiveDataset(),i=0,n=e.length;i<n;i++){var s=this.settings.groupable?i:this.pagingRowIndex(i);this.filterRowRendered||this.filterExpr&&this.filterExpr[0]&&this.filterExpr[0].keywordSearch?e[i]._isFilteredOut||e[i]._selected||t.push(s):e[i]._selected||t.push(s)}this.dontSyncUi=!0,this.selectRows(t,!0,!0),this.dontSyncUi=!1,this.syncSelectedUI(),this.element.triggerHandler("selected",[this._selectedRows,"selectall"])},deSelectAllRows:function(){this.unSelectAllRows()},unSelectAllRows:function(t,e){if(this._selectedRows&&0!==this._selectedRows.length){this.dontSyncUi=!0;for(var i=this._selectedRows.length-1;i>=0;i--)if(this._selectedRows[i]){var n=this.settings.groupable?this._selectedRows[i].idx:this.pagingRowIndex(this._selectedRows[i].idx);this.unselectRow(n,!0,!0)}this.settings.dataset.map((function(t){delete t._selected})),this.dontSyncUi=!1;var s=this.element.closest(".lookup-modal");this._selectedRows=1===s.length?this._selectedRows:[],this.displayCounts(),t||this.syncSelectedUI(),e||this.element.triggerHandler("selected",[this._selectedRows,"deselectall"])}else this.settings.dataset.map((function(t){delete t._selected}))},unSelectAllRowsCurrentPage:function(t){var e,i,n=this,s=this.settings;this.dontSyncUi=!0,null===(e=this.currentPageRows)||void 0===e||null==e||e.forEach((function(t){var e=s.groupable?t.dataIndex:n.pagingRowIndex(t.dataIndex);n.unselectRow(e,!0,!0)}));var a=this.getActiveDataset(),o=(null===(i=this.currentPageRows)||void 0===i||null==i?void 0:i.map((function(t){return t.dataIndex})))||[];a.filter((function(t,e){return o.indexOf(e)>-1})).map((function(t){return delete t._selected})),this.dontSyncUi=!1,t||this.syncSelectedUI(),this.element.triggerHandler("selected",[this._selectedRows,"deselectall"])},selectAllRowsCurrentPage:function(t){var e;this.dontSyncUi=!0;var i=(null===(e=this.currentPageRows)||void 0===e||null==e?void 0:e.map((function(t){return t.dataIndex})))||[];i.length&&this.selectRowsBetweenIndexes([i[0],i[i.length-1]]),this.dontSyncUi=!1,t||this.syncSelectedUI()},isRowSelected:function(t){return!!t&&!0===t._selected},selectNode:function(t,e,i,n){var s=null,a=null,o="is-selected".concat("mixed"===this.settings.selectable?" hide-selected-color":"");(i&&!this.isRowSelected(i)||n)&&(t.addClass(o).attr("aria-selected","true"),(s=t.find(".datagrid-selection-checkbox").closest("td")).length&&(s.attr("aria-checked","true"),s.find(".datagrid-cell-wrapper .datagrid-checkbox").addClass("is-checked")),(a=t.find(".datagrid-selection-radio").closest("td")).length&&(this.unSelectAllRows(),a.attr("aria-checked","true"),a.find(".datagrid-cell-wrapper .datagrid-radio").addClass("is-checked")),i&&(i._selected=!0))},selectRow:function(t,e,i){var n,s=this,a=null,o=this,r=this.settings,l={};if(void 0!==t&&-1!==t&&r.selectable&&!this.isRowDisabled(t)&&(a=this.settings.groupable?this.rowNodesByDataIndex(t):this.rowNodes(t),n=this.dataRowIndex(a),isNaN(n)&&(n=t),a&&(a.length||!r.source))){if("function"==typeof r.onBeforeSelect&&!i)if(!1===r.onBeforeSelect({node:a,idx:n}))return;var c=function(t,e){return!!s._selectedRows.filter((function(i){return function(t,e,i){return i&&i.idx===t&&i.elem&&i.elem.is(e)}(t,e,i)})).length};if("single"===r.selectable){var d=-1;this._selectedRows.length>0?d=this._selectedRows[0].pagingIdx:a[0]&&a[0].classList.contains("is-selected")&&(d=n),-1!==d&&(this.unselectRow(d,!0,!0),!a.length&&this._selectedRows.length>0&&this._selectedRows.pop())}var h=r.source&&"multiple"===r.selectable&&r.allowSelectAcrossPages;if(!a.hasClass("is-selected")||h){var u;if(r.treeGrid&&r.treeDepth&&r.treeDepth.length>0){var p=parseInt(a.attr("aria-level"),10);if(u=r.treeDepth[o.pagerAPI&&r.source?a.index():t].node,a.is(".datagrid-tree-parent")&&"multiple"===r.selectable)a.add(a.nextUntil('[aria-level="'.concat(p,'"]'))).each((function(t){var e=$(this),i=e.attr("aria-rowindex")-1,n=o.actualPagingRowIndex(i),s=r.treeDepth[n].node;if(r.selectChildren||!r.selectChildren&&0===t){var d=!e.is(a)&&!e.hasClass("is-selected");if(o.selectNode(e,i,s),d&&!c(n,e)){if(l={idx:n,data:s,elem:e,page:o.pagerAPI?o.pagerAPI.activePage:1,pagingIdx:n,pagesize:r.pagesize},o.settings.source&&o.settings.allowSelectAcrossPages){var h=o.getUniqueRowID(s);h&&(l.uniqueRowID=h)}o._selectedRows.push(l)}}}));else if("siblings"===r.selectable){var g,f;o.settings.selectAllCurrentPage?o.unSelectAllRowsCurrentPage(!0):o.unSelectAllRows(!0,!0),1===p?g=a.parent().find('[aria-level="1"]'):p>1&&(g=a.nextUntil('[aria-level="'.concat(p-1,'"]')).filter('[aria-level="'.concat(p,'"]')),f=a.prevUntil('[aria-level="'.concat(p-1,'"]')).filter('[aria-level="'.concat(p,'"]'))),a.add(g).add(f).each((function(t){var e=$(this),i=e.attr("aria-rowindex")-1,n=o.actualPagingRowIndex(i),s=r.treeDepth[n].node;if(r.selectChildren||!r.selectChildren&&0===t){var d=!e.is(a)&&!e.hasClass("is-selected");if(o.selectNode(e,i,s),d&&!c(n,e)){if(l={idx:n,data:s,elem:e,page:o.pagerAPI?o.pagerAPI.activePage:1,pagingIdx:n,pagesize:r.pagesize},o.settings.source&&o.settings.allowSelectAcrossPages){var h=o.getUniqueRowID(s);h&&(l.uniqueRowID=h)}o._selectedRows.push(l)}}}))}else o.selectNode(a,t,u);o.setNodeStatus(a),o.lastSelectedRow=t}else{if(u=r.dataset[n],r.groupable){var m=o.actualPagingRowIndex(o.actualRowIndex(a));if(isNaN(m))return;var v=o.groupArray[m];if(u=o.settings.dataset[v.group].values[v.node],!c(u.idx,a)){if(l={idx:u.idx,data:u,elem:a,group:r.dataset[o.groupArray[m].group],page:o.pagerAPI?o.pagerAPI.activePage:1,pagingIdx:n,pagesize:o.settings.pagesize},o.settings.source&&o.settings.allowSelectAcrossPages){var b=o.getUniqueRowID(u);b&&(l.uniqueRowID=b)}this._selectedRows.push(l)}}o.selectNode(a,n,u),o.lastSelectedRow=t}if(!r.groupable){var y=o.actualPagingRowIndex(t);if(-1===y&&(y=t),!c(y,o.visualRowNode(y))){if(l={idx:y,data:u,elem:o.visualRowNode(y),page:o.pagerAPI?o.pagerAPI.activePage:1,pagingIdx:t,pagesize:o.settings.pagesize},o.settings.source&&o.settings.allowSelectAcrossPages){var w=o.getUniqueRowID(u);w&&(l.uniqueRowID=w)}this._selectedRows.push(l)}}}e||o.syncSelectedUI(),i||this.element.triggerHandler("selected",[this._selectedRows,"select"])}},selectRowsBetweenIndexes:function(t){t.sort((function(t,e){return t-e}));for(var e=t[0];e<=t[1];e++)this.selectRow(e);this.displayCounts()},syncHeaderCheckbox:function(t){if(0!==this.headerNodes().length){var e=this.settings,i=this.getActiveDataset(),n=this.headerNodes().find(".datagrid-checkbox"),s=t.length,a=this._selectedRows.length,o=n.data("selected");if(e.selectAllCurrentPage&&this.currentPageRows&&i.length!==t.length){var r,l=(null===(r=this.currentPageRows)||void 0===r||null==r?void 0:r.map((function(t){return t.dataIndex})))||[];a=i.filter((function(t,e){return l.indexOf(e)>-1})).filter((function(t){return t._selected})).length}e.source&&e.allowSelectAcrossPages&&(a=i.filter((function(t){return t._selected})).length),a!==s&&a>0&&"partial"===o||a===s&&"all"===o&&0!==a||0===a&&"none"===o||(a>0&&n.data("selected","partial").addClass("is-checked is-partial").attr("aria-checked","mixed"),a===s&&n.data("selected","all").addClass("is-checked").removeClass("is-partial").attr("aria-checked","true"),0===a&&n.data("selected","none").removeClass("is-checked is-partial").attr("aria-checked","false"))}},getUniqueRowID:function(t){var e=null;if(this.settings.columnIds.length>0){e="rowId";for(var i=0;i<this.settings.columnIds.length;i++)e+=" ".concat(t[this.settings.columnIds[i]]);e=e.replace(/([0-9A-Z]+)/g,(function(t,e){return" ".concat(e)})).replace(/([0-9a-z]+)/g,(function(t,e){return"".concat(e," ")})).trim(),e=g.toCamelCase(e)}return e},syncSelectedRows:function(){for(var t,e=this,i=this.settings,n=i.groupable&&this.originalDataset?this.originalDataset:i.dataset,s=null,a=function(t){var a=i.groupable?e.dataRowNode(s):e.visualRowNode(s);a[0]&&(e._selectedRows[t].elem=a,e.selectNode(a,s,n[s],!0))},o=0,r=this._selectedRows.length;o<r;o++){var l=(t=this._selectedRows[o])?t.uniqueRowID:null;if(this.settings.source&&this.settings.allowSelectAcrossPages&&l){for(var c=0,d=n.length;c<d;c++)if(l===this.getUniqueRowID(n[c])){var h=i.groupable?this.dataRowNode(c):this.visualRowNode(c);h[0]&&(this._selectedRows[o].elem=h,this.selectNode(h,c,n[c],!0))}}else this.pagerAPI&&this._selectedRows[o].page===this.pagerAPI.activePage&&(s=this._selectedRows[o].idx,a(o)),this._selectedRows[o].pagesize===i.pagesize||i.groupable||(s=this._selectedRows[o].pagingIdx,i.dataset[s]?(a(o),this._selectedRows[o].idx=s,this._selectedRows[o].page=this.pagerAPI.activePage,this._selectedRows[o].pagesize=i.pagesize):(this._selectedRows[o].idx=s%i.pagesize,this._selectedRows[o].page=Math.round(s/i.pagesize)+1,this._selectedRows[o].pagesize=i.pagesize))}},calculatePagerInfo:function(t,e,i){for(var n=this.settings,s=e||n.treeGrid?n.treeDepth:n.dataset,a={idx:t,page:1,pagesize:i="number"==typeof i&&i>0?i:n.pagesize,numOfPages:Math.ceil(s.length/i)},o=0;o<a.numOfPages;o++){if(t>=(a.page-1)*a.pagesize&&t<a.page*a.pagesize){a.pagingIdx=t-a.pagesize*o;break}a.page++}return a},syncSelectedRowsIdx:function(){var t=this.getActiveDataset();if(0!==this._selectedRows.length&&0!==t.length){this._selectedRows=[];for(var e=0;e<t.length;e++){var i=this.settings.treeGrid?t[e].node:t[e];if(i._selected){var n={idx:e,data:i,elem:this.dataRowNode(e),pagingIdx:e,pagesize:this.settings.pagesize};if(this.settings.groupable){var s=this.rowNodesByDataIndex(e);if(s.length){var a=this.actualPagingRowIndex(this.actualRowIndex(s)),o=this.groupArray[a].group;n.group=this.settings.dataset[o],n.page=this.calculatePagerInfo(o).page}}else n.page=this.calculatePagerInfo(e).page;this._selectedRows.push(n)}}}},syncSelectedUI:function(){var t,e,i,n,s,a,o=this,r=this.getActiveDataset(),l=r;if(this.settings.selectAllCurrentPage){var c,d=(null===(c=this.currentPageRows)||void 0===c||null==c?void 0:c.map((function(t){return t.dataIndex})))||[];l=r.filter((function(t,e){return d.indexOf(e)>-1}))}var h=l;if(this.filterRowRendered){h=[];for(var u=0,p=null===(g=l)||void 0===g||null==g?void 0:g.length;u<p;u++){var g;l[u]._isFilteredOut||h.push(u)}}if(this.syncHeaderCheckbox(h),1===(null===(t=this.contextualToolbar)||void 0===t||null==t?void 0:t.length)&&!this.dontSyncUi&&(0===(null===(e=this._selectedRows)||void 0===e||null==e?void 0:e.length)&&this.contextualToolbar.one("animateclosedcomplete.datagrid",(function(){o.contextualToolbar.css("display","none")})).animateClosed(),(null===(i=this._selectedRows)||void 0===i||null==i?void 0:i.length)>0&&(0===(null===(n=this.contextualToolbar)||void 0===n||null==n?void 0:n.height())||null===(s=this.contextualToolbar)||void 0===s||!(null==s?void 0:s.is(":visible"))||null===(a=this.contextualToolbar)||void 0===a||!(null==a?void 0:a.hasClass("is-hidden"))))){var f=this.contextualToolbar.hasClass("flex-toolbar")?"recalculate-buttonset":"recalculate-buttons",m=this.contextualToolbar.hasClass("flex-toolbar")?"flex":"block";this.contextualToolbar.find(".selection-count").text("".concat(this._selectedRows.length," ").concat(H.translate("Selected"))),this.contextualToolbar.removeClass("is-hidden").css("display",m).one("animateopencomplete.datagrid",(function(){$(this).removeClass("is-hidden").triggerHandler(f)})).animateOpen()}},activateRow:function(t){this.activatedRow()[0].row!==t&&this.toggleRowActivation(t)},deactivateRow:function(){var t=this.activatedRow()[0].row;t>=0&&this.toggleRowActivation(t)},activatedRow:function(){var t=[{row:-1,item:void 0,elem:void 0}];if(this.tableBody){var e=this.settings,i=e.treeGrid?e.treeDepth:e.dataset,n=this.tableBody.find("tr.is-rowactivated");if(n.length){var s=this.dataRowIndex(n),a=e.indeterminate?s:this.actualRowIndex(n);t=[{row:a,item:i[a],elem:n}]}else{t=null;for(var o=0;o<i.length;o++)if(i[o]._rowactivated){t=[{row:o,item:i[o],elem:void 0}];break}null===t&&(t=[{row:-1,item:void 0,elem:n}])}}return t},toggleRowActivation:function(t){var e,i,n,s=this.settings,a=s.treeGrid?s.treeDepth:s.dataset;if("number"==typeof t){if(e=this.tableBody[0].querySelector('tr[aria-rowindex="'.concat(t+1,'"]')),n=t,this.pagerAPI&&s.source&&s.indeterminate){var o=t+(this.pagerAPI.activePage-1)*s.pagesize;e=this.tableBody[0].querySelector('tr[aria-rowindex="'.concat(o+1,'"]'))}}else e=(i=t instanceof jQuery?t:$(t))[0],n=s.treeGrid||s.groupable?this.actualRowIndex(i):this.dataRowIndex(i);s.indeterminate&&!e&&(e=(i=this.actualRowNode(n))[0]),!!a[n]&&a[n]._rowactivated?this.deactivateRowNode(n,a):(this.deactivateAllRowNodes(a),this.activateAllRowNodes(n,a))},activateAllRowNodes:function(t,e){var i=this;if(!("number"!=typeof t||t<0)){var n=this.settings;void 0===e&&(e=n.treeGrid?n.treeDepth:n.dataset);var s=[{row:t,item:e[t]}];e[t]&&$.when(this.element.triggerHandler("beforerowactivated",s)).done((function(a){var o;"string"==typeof(o=a)&&"false"===o.toLowerCase()||"boolean"==typeof o&&!1===o||"number"==typeof o&&0===o||((n.paging&&n.source&&"mixed"===n.selectable?i.rowNodesByDataIndex(t).toArray():i.rowNodes(t).toArray()).forEach((function(t){t.classList.add("is-rowactivated")})),e[t]._rowactivated=!0,s=[{row:t,item:e[t]}],i.element.triggerHandler("rowactivated",s))}))}},deactivateRowNode:function(t,e){if(!("number"!=typeof t||t<0)){var i=this.settings;if(void 0===e&&(e=i.treeGrid?i.treeDepth:i.dataset),e[t]&&!i.disableRowDeactivation)this.rowNodes(t).toArray().forEach((function(t){t.classList.remove("is-rowactivated")})),delete e[t]._rowactivated,this.element.triggerHandler("rowdeactivated",[{row:t,item:e[t]}])}},deactivateAllRowNodes:function(t){var e=this.settings,i=null;void 0===t&&(t=e.treeGrid?e.treeDepth:e.dataset);var n=[].slice.call(this.element[0].querySelectorAll("tr.is-rowactivated"));if(n.length>0){n.forEach((function(t){t.classList.remove("is-rowactivated")}));var s=e.treeGrid||e.groupable?this.actualRowIndex($(n)):this.dataRowIndex($(n));i={row:s,item:t[s]},t[s]&&delete t[s]._rowactivated}else for(var a=0;a<t.length;a++){var o=t[a];if(o._rowactivated){delete o._rowactivated,i={row:a,item:o};break}}null!==i&&this.element.triggerHandler("rowdeactivated",[i])},toggleRowSelection:function(t,e){var i="number"==typeof t?this.tableBody.find('tr[aria-rowindex="'.concat(t+1,'"]')):t,n="number"==typeof t?t:this.actualRowIndex(i);if(this.settings.groupable&&(n=this.dataRowIndex(i)),!(!1===this.settings.selectable||this.editor&&i.hasClass("is-selected")))return this.settings.dblClickApply||"single"!==this.settings.selectable||!i.hasClass("is-selected")||this.settings.disableRowDeselection?(i.hasClass("is-selected")?this.settings.disableRowDeselection||this.unselectRow(n):this.settings.dblClickApply?this.selectRow(n,!1,!0):this.selectRow(n),this.settings.dblClickApply&&e&&this.selectRow(n),this.displayCounts(),this._selectedRows):(this.unselectRow(n),this.displayCounts(),this._selectedRows)},unselectRow:function(t,e,i){var n=this,s=n.settings,a=this.settings.groupable?this.rowNodesByDataIndex(t):this.rowNodes(t),o=s.source&&"multiple"===s.selectable&&s.allowSelectAcrossPages,r=null,l=null;if(a&&void 0!==t){var c=s.treeGrid?s.treeDepth[t].node:s.dataset[t],d=function(t){return t?t.uniqueRowID:null},h=function(t,e){var i=function(i,a){delete i._selected,void 0===a&&(a=e);for(var r=0;r<n._selectedRows.length;r++){var l=d(n._selectedRows[r]);if(s.source&&s.allowSelectAcrossPages&&l){if(l===n.getUniqueRowID(i)){n._selectedRows.splice(r,1);break}}else if(n._selectedRows[r].idx===a){if(o&&n._selectedRows[r].elem&&!n._selectedRows[r].elem.is(t))continue;n._selectedRows.splice(r,1);break}}};if(t.removeClass("is-selected hide-selected-color").removeAttr("aria-selected"),-1!==n.columnIdxById("selectionCheckbox")&&((r=n.cellNode(t,n.columnIdxById("selectionCheckbox"))).attr("aria-checked","false"),r.find(".datagrid-cell-wrapper .datagrid-checkbox").removeClass("is-checked no-animate").removeAttr("aria-label")),-1!==n.columnIdxById("selectionRadio")&&((l=n.cellNode(t,n.columnIdxById("selectionRadio"))).attr("aria-checked","false"),l.find(".datagrid-cell-wrapper .datagrid-radio").removeClass("is-checked no-animate").removeAttr("aria-label")),s.treeGrid)for(var a=0;a<s.treeDepth.length;a++)n.isRowSelected(s.treeDepth[a].node)&&(void 0!==e?e===s.treeDepth[a].idx-1&&i(s.treeDepth[a].node):i(s.treeDepth[a].node));else{var h=t.length?n.dataRowIndex(t):e;void 0!==h&&h>-1&&(c=s.dataset[h]),s.groupable&&(c=n.originalDataset[h]),void 0!==c&&(s.paging&&s.source?i(c,h):i(c))}};if(s.treeGrid){var u=parseInt(a.attr("aria-level"),10);if(a.is(".datagrid-tree-parent")&&"multiple"===s.selectable)a.add(a.nextUntil('[aria-level="'.concat(u,'"]'))).each((function(t){var e=$(this),i=e.attr("aria-rowindex")-1,a=n.actualPagingRowIndex(i);(s.selectChildren||!s.selectChildren&&0===t)&&h(e,a)}));else if("siblings"===s.selectable)a.parent().find(".is-selected").each((function(t){var e=$(this),i=e.attr("aria-rowindex")-1,a=n.actualPagingRowIndex(i);(s.selectChildren||!s.selectChildren&&0===t)&&h(e,a)}));else{var p=n.actualPagingRowIndex(t);h(a,p)}n.setNodeStatus(a)}else h(a,t);e||n.syncSelectedUI(),i||n.element.triggerHandler("selected",[n._selectedRows,"deselect",c])}},setNodeStatus:function(t){var e,i=this,n="multiple"===i.settings.selectable,s=i.settings,a=i.cellNode(t,i.columnIdxById("selectionCheckbox"));if(!n)return a.find(".datagrid-cell-wrapper .datagrid-checkbox").removeClass("is-checked is-partial"),a.attr("aria-checked","false"),void(t.is(".is-selected")&&(a.find(".datagrid-cell-wrapper .datagrid-checkbox").addClass("is-checked").attr("aria-checked","true"),a.attr("aria-checked","true")));var o=function(t,e){t.each((function(){var t=$(this),n=i.cellNode(t,i.columnIdxById("selectionCheckbox")),s=i.getSelectedStatus(t,e);n.find(".datagrid-cell-wrapper .datagrid-checkbox").removeClass("is-checked is-partial"),n.attr("aria-checked","false"),"mixed"===s?(n.find(".datagrid-cell-wrapper .datagrid-checkbox").addClass("is-checked is-partial").attr("aria-checked","mixed"),n.attr("aria-checked","mixed")):s&&(n.find(".datagrid-cell-wrapper .datagrid-checkbox").addClass("is-checked"),n.attr("aria-checked","true"))}))};e=t.add(t.nextUntil('[aria-level="1"]')).filter(".datagrid-tree-parent"),s.selectChildren&&o(e),e=t,+t.attr("aria-level")>1&&(e=e.add(t.prevUntil('[aria-level="1"]')).add(t.prevAll('[aria-level="1"]:first'))),e=e.filter(".datagrid-tree-parent"),s.selectChildren&&o(e)},getSelectedStatus:function(t){var e=this.settings,i=0,n=0,s=0,a=t.add(t.nextUntil('[aria-level="1"]'));if(e.treeGrid&&"multiple"===e.selectable){var o=t.attr("aria-level");a=t.add(t.nextUntil('[aria-level="'.concat(o,'"]')))}return a.each((function(){i++,$(this).is(".is-selected")?n++:s++})),i===n||i!==s&&"mixed"},selectedRows:function(){return this._selectedRows},selectRows:function(t,e,i){var n=-1,s=this.settings,a="siblings"===s.selectable,o=this.getActiveDataset();if("number"==typeof t&&(t=[t]),!t||0===t.length)return this._selectedRows;if("single"===s.selectable&&(this._selectedRows.length&&this.unselectRow(this._selectedRows[0].idx,!0,!0),n="[object Array]"===Object.prototype.toString.call(t)?t[0]:t.index(),this.selectRow(n,!0,!0)),"multiple"===s.selectable||"mixed"===s.selectable||a)if("[object Array]"===Object.prototype.toString.call(t)){for(var r=0;r<t.length;r++)this.selectRow(t[r],!0,!0);if(0===t.length)for(var l=0,c=o.length;l<c;l++)this.unselectRow(l,!0,!0)}else this.selectRow(t.index(),!0,!0);return this.displayCounts(),e||this.syncSelectedUI(),i||this.element.triggerHandler("selected",[this._selectedRows,"select"]),this._selectedRows},findRowsByValue:function(t,e){for(var i=this.settings,n=this.getActiveDataset(),s=-1,a=[],o=0;o<n.length;o++)if(i.groupable)for(var r=0;r<n[o].values.length;r++)s++,n[o].values[r][t]===e&&a.push(s);else(i.treeGrid?n[o].node:n[o])[t]===e&&a.push(o);return a},rowStatus:function(t,e,i){var n=this.getActiveDataset();if(!e)return delete n[t].rowStatus,this.settings.treeGrid&&delete n[t].node.rowStatus,void this.updateRow(t);if(!/dirty/.test(e)&&n[t]){n[t].rowStatus={};var s=n[t].rowStatus;s.icon=e,e=(e=e.charAt(0).toUpperCase()+e.slice(1)).replace("-progress","Progress"),s.text=H.translate(e),i=i?i.charAt(0).toUpperCase()+i.slice(1):s.text,s.tooltip=i,this.updateRow(t),this.setupTooltips(!0)}},columnById:function(t){return $.grep(this.settings.columns,(function(e){return e.id===t}))},columnIdxById:function(t){for(var e=this.settings.columns,i=-1,n=0;n<e.length;n++)if(e[n].id===t){i=n;break}return i},handleKeys:function(){var t=this,e=this,i=$("th .datagrid-checkbox",e.headerRow);e.element.on("keydown.datagrid","th",(function(t){var n,s,a=t.which||t.keyCode||t.charCode||0,o=$(this),r=e.columnIdxById(o.attr("data-column-id")),l=e.visibleColumns().length-1,c=$(t.target);if(!(c.closest(".popupmenu").length>0)){if(13===a||32===a){if(n="multiple"===e.settings.selectable&&0===r?$(".datagrid-checkbox",o):o,c.is(".btn-menu.btn-filter"))return void n.trigger("click.datagrid");var d=function(t){var e;return t&&"selectionCheckbox"===(null===(e=t[0])||void 0===e||null==e?void 0:e.dataset.columnId)},h=d(n)||d(n.prevObject);if(c.is("th.is-sortable")&&e.setSortColumn($(this).attr("data-column-id")),("multiple"===e.settings.selectable||"mixed"===e.settings.selectable)&&h){if(i.addClass("is-checked").removeClass("is-partial"),i.closest("th").attr("aria-checked","true"),e.recordCount===e._selectedRows.length)return void(e.settings.selectAllCurrentPage?e.unSelectAllRowsCurrentPage():e.unSelectAllRows());e.settings.selectAllCurrentPage?e.selectAllRowsCurrentPage():e.selectAllRows()}32===a&&t.preventDefault()}-1!==[35,36,37,39].indexOf(a)&&(s=r,s=/35|36/i.test(a)||(t.ctrlKey||t.metaKey)&&/37|39/i.test(a)?H.isRTL()?36===a||(t.ctrlKey||t.metaKey)&&37===a?l:0:35===a||(t.ctrlKey||t.metaKey)&&39===a?l:0:H.isRTL()?39===a?r>0?r-1:r:r<l?r+1:l:37===a?r>0?r-1:r:r<l?r+1:l,e.activeCell.cell=s,o.removeAttr("tabindex").removeClass("is-active"),t.preventDefault()),40===a&&(o.removeAttr("tabindex"),e.activeCell.node=e.cellNode(0,e.settings.groupable?0:e.activeCell.cell).attr("tabindex","0").focus(),t.preventDefault()),c.parents(".modal").length>0&&c.hasClass("btn-filter")&&(t.shiftKey&&9===a?c.parents("th").prev("th").length>0?(t.preventDefault(),c.parents("th").prev("th").find("[tabindex=0]").trigger("focus")):c.parents(".modal-content").find(".toolbar").length>0&&(t.preventDefault(),t.stopPropagation(),c.parents(".modal-content").find(".toolbar").find("button[tabindex=0]").trigger("focus")):9===a&&(t.preventDefault(),c.siblings("input").trigger("focus")))}})),e.element.on("keydown.datagrid","td, input",(function(i){var n=i.which||i.keyCode||i.charCode||0,s=!1;113!==n||t.inlineMode||(e.settings.actionableMode=!e.settings.actionableMode,s=!0);var a=e.element.parents(".modal");27===n&&0!==a.length&&a.data().modal.close();if(s)return i.preventDefault(),i.stopPropagation(),s})),e.element.on("keydown.datagrid","td",(function(t){var n=t.which||t.keyCode||t.charCode||0,s=!1,a=$(t.target),o=H.isRTL(),r=e.activeCell.node,l=$(this).parent(),c=l.prevAll(":not(.is-hidden, .datagrid-expandable-row)").first(),d=l.nextAll(":not(.is-hidden, .datagrid-expandable-row)").first(),h=e.activeCell.row,u=e.activeCell.cell,p=e.columnSettings(u),g=l.is(".datagrid-rowgroup-header, .datagrid-rowgroup-footer"),f=e.settings.dataset[e.dataRowIndex(l)],m=e.tableBody.find("tr:visible"),v=function(t){var i=m.filter('[aria-rowindex="'.concat(t+1,'"]'));return i.is(".datagrid-rowgroup-header")?i.index():e.dataRowIndex(i)};r.length||(e.activeCell.node=e.cellNode(h,u),r=e.activeCell.node);var b,y,w=function(t,i,n){if(g)return function(t,i,n){var s=e.activeCell.groupNode||r,a=t+(n?-1:1);return a>i?n?s.prevAll(":visible").last():s.nextAll(":visible").last():n?s.prevAll(":visible").first():s.nextAll(":visible").first()}(t,i,n);var s=t+(n?-1:1);if(s>i)return i;for(;e.settings.columns[s]&&e.settings.columns[s].hidden;)s=n?s-1:s+1;return s},x=!!$(".datagrid-selection-checkbox",r).length,C=m.last(),k=e.settings.columns.length-1;if(e.settings.onKeyDown){var T=function(e){e||(t.stopPropagation(),t.preventDefault())},A={activeCell:e.activeCell,row:h,cell:u};if(!1===e.settings.onKeyDown(t,A,T)||!T)return t.stopPropagation(),void t.preventDefault()}if(121===n&&t.shiftKey&&a.trigger("contextmenu"),-1!==[9,37,38,39,40].indexOf(n)){if(a.closest(".code-block").length&&(9!==n||!t.shiftKey||0!==e.getFocusables(r).index))return;if(9!==n){if(a.is(".code-block-actions"))return;if(a.closest(".popupmenu.is-open").closest(".popupmenu-wrapper").prev().is(".code-block-actions"))return}}if(-1!==[9,37,39].indexOf(n)){if(9===n&&!e.settings.actionableMode||!e.settings.cellNavigation)return;if(9!==n&&t.altKey)u=37===n&&!o||39===n&&o?0:k,e.setActiveCell(h,u);else if(!e.quickEditMode||9===n){if(9===n&&t.shiftKey&&a.is(".code-block-actions")?e.focusNextPrev("prev",r):u=!o&&(37===n||9===n&&t.shiftKey)||o&&(39===n||9===n)?w(u,k,!0):w(u,k),-1===u&&!e.settings.actionableMode||!e.settings.cellNavigation)return;if(-1===u&&e.settings.actionableMode&&(u=k,-1===--h))return;if(u===k&&k===e.activeCell.cell&&e.settings.actionableMode&&(u=0,++h===e.visibleRowCount))return;u instanceof jQuery?e.setActiveCell(u):e.setActiveCell(h,u),e.settings.actionableMode&&(e.makeCellEditable(e.activeCell.rowIndex,u,t),e.containsTextField(r)&&e.containsTriggerField(r)&&(e.quickEditMode=!0)),e.quickEditMode=!1,s=!0}}if(38===n&&!e.quickEditMode)if(t.altKey||t.metaKey){var S=v(0);e.setActiveCell(S,u),s=!0}else{if(0===h&&!c.is(".datagrid-rowgroup-header"))return r.removeAttr("tabindex"),void e.element.find("th").eq(u).attr("tabindex","0").focus();e.setActiveCell(c,u),s=!0}if(40!==n||e.quickEditMode||(t.altKey||t.metaKey?(e.setActiveCell(C,u),s=!0):(e.setActiveCell(d,u),s=!0)),32===n&&t.ctrlKey&&r){var E="";(h=r.closest("tr")).children().each((function(){var t=$(this);E+="".concat(t.text()," ")})),$("body").toast({title:"",audibleOnly:!0,message:E}),s=!0}if(36!==n||r.hasClass("is-editing")||(e.setActiveCell(h,0),s=!0),35!==n||r.hasClass("is-editing")||(e.setActiveCell(h,k),s=!0),34===n&&(e.setActiveCell(C,u),s=!0),33===n&&(e.setActiveCell(v(0),u),s=!0),32===n&&(!e.settings.editable||x)){if(h=r.closest("tr"),1===a.closest(".datagrid-row-detail").length)return;t.preventDefault();var D=a.find(".datagrid-expand-btn, .datagrid-drilldown");if(D&&D.length)return D.trigger("click"),void t.preventDefault();"multiple"!==e.settings.selectable&&"mixed"!==e.settings.selectable||!t.shiftKey?e.toggleRowSelection(h):e.selectRowsBetweenIndexes([e.lastSelectedRow,h.attr("aria-rowindex")-1])}if(e.settings.editable&&32===n){var I=null==a||null==a?void 0:a.find(".datagrid-expand-btn");I.length>0&&(I.trigger("click"),t.preventDefault()),e.editor||e.makeCellEditable(e.activeCell.rowIndex,u,t)}if(13===n&&r.is(".has-btn-actions")){var L=r.find(".btn-actions");L.length&&L.trigger("click")}if(13===n&&p.click&&"function"==typeof p.click&&(r.hasClass("is-cell-readonly")||p.click(t,[{row:h,cell:u,item:f,originalEvent:t}])),e.settings.editable&&13===n){if(a.is("textarea")&&t.shiftKey)return;if(a.is(".btn-menu")||a.closest(".popupmenu.is-open").length)return;e.editor?(e.quickEditMode=!1,e.commitCellEdit(),e.setNextActiveCell(t)):(e.makeCellEditable(e.activeCell.rowIndex,u,t),e.containsTextField(r)&&e.containsTriggerField(r)&&(e.quickEditMode=!0)),s=!0}-1!==[9,13,32,35,36,37,38,39,40,113].indexOf(n)||t.altKey||t.shiftKey||t.ctrlKey||t.metaKey||!e.settings.editable||(e.editor||null!==(b=t.currentTarget)&&void 0!==b&&null!==(y=null==b?void 0:b.parentElement)&&void 0!==y&&(null==y?void 0:y.classList.contains("datagrid-expandable-row"))||e.makeCellEditable(e.activeCell.rowIndex,u,t));if(("multiple"===e.settings.selectable||"mixed"===e.settings.selectable)&&!e.editor&&(t.ctrlKey||t.metaKey)&&65===n){if(i.addClass("is-checked").removeClass("is-partial"),i.closest("th").attr("aria-checked","true"),e.recordCount===e._selectedRows.length)return void(e.settings.selectAllCurrentPage?e.unSelectAllRowsCurrentPage():e.unSelectAllRows());e.settings.selectAllCurrentPage?e.selectAllRowsCurrentPage():e.selectAllRows(),s=!0}if(s)return t.preventDefault(),t.stopPropagation(),!1}))},getFocusables:function(t){var e=$(":focusable",t);return{elements:e,index:e.index($(":focus"))}},focusNextPrev:function(t,e){if(e&&"string"==typeof t){t=t.toLowerCase();var i=this.getFocusables(e),n=i.elements,s=n.length,a=i.index;/\b(next|prev)\b/g.test(t)&&(a="next"===t?a+1>=s?0:a+1:a-1<0?s:a-1,n.eq(a).focus())}},containsTextField:function(t){var e,i=["image","button","submit","reset","checkbox","radio"],n="textarea, input:not(.colorpicker)",s=i.length;for(n+=s?":not(":"",e=0;e<s;e++)n+="[type=".concat(i[e],"],");return n=s?"".concat(n.slice(0,-1),")"):"",!!$(n,t).length},containsTriggerField:function(t){return!$(".dropdown, .datepicker, .lookup",t).length},isRowDisabled:function(t){if(this.settings.isRowDisabled&&"function"==typeof this.settings.isRowDisabled&&"true"===this.rowNodes(t).attr("aria-disabled"))return!0;return!1},isCellEditable:function(t,e){if(!this.settings.editable)return!1;var i=this.columnSettings(e);if(i.readonly)return!1;if(this.isRowDisabled(t)||!this.activeCell.node)return!1;var n=this.activeCell.node.find(".datagrid-cell-wrapper"),s=n.text()?n.text():this.fieldValue(this.settings.dataset[t],i.field);if(i.isEditable){var a=this.settings.dataset[t];if(this.settings.treeDepth&&this.settings.treeDepth[t]&&(a=this.settings.treeDepth[t].node),!i.isEditable(t,e,s,i,a,this,"is-editable"))return!1}return!!i.editor},makeCellEditable:function(t,e,i){var n,s;if(!this.activeCell.node.closest("tr").hasClass("datagrid-summary-row")){var a=this.activeCell.node.find(".datagrid-cell-wrapper"),o=a.parent("td"),r=$(".datagrid-tree-node",a).length>0,l=$(".datagrid-expand-btn",a).length>0;if(o.hasClass("is-editing")||o.hasClass("is-editing-inline"))return!1;this.editor&&this.editor.input&&this.commitCellEdit();var c=this.columnSettings(e);if(!c.editor)return 32!==i.keyCode||$(i.currentTarget).find(".datagrid-selection-checkbox").length||this.toggleRowSelection(this.activeCell.node.closest("tr")),!1;var d=this.actualRowNode(t),h=this.settings.treeGrid?this.actualPagingRowIndex(this.actualRowIndex(d)):this.dataRowIndex(d);!this.settings.treeGrid&&this.settings.paging&&this.pagerAPI.activePage>1&&h<(this.pagerAPI.activePage-1)*this.settings.pagesize&&(h+=(this.pagerAPI.activePage-1)*this.settings.pagesize);var u=this.rowData(h),p=$(".is-editor",o).length>0,f=$(".is-placeholder",a).length>0,m=a.text()?a.text():this.fieldValue(u,c.field);if((p||f)&&(m=this.fieldValue(u,c.field)),!this.isCellEditable(h,e))return!1;(l||r)&&(l&&(m=$("> span",a).text()),"string"==typeof m&&(m=m.replace(/^\s/,""))),c.inlineEditor?o.addClass("is-editing-inline"):(p&&a.css({position:"static",height:a.outerHeight()}),o.addClass("is-editing"),c.doNotEmptyCellWhenEditing||o.hasClass("is-fileupload")||a.empty()),m=g.sanitizeConsoleMethods(m),this.element.triggerHandler("beforeentereditmode",[{row:h,cell:e,item:u,target:a,value:m,column:c,editor:this.editor}]),1===this.visibleColumns().length&&o.addClass("has-singlecolumn"),this.editor=new c.editor(h,e,m,a,c,i,this,u),this.editor.row=h,this.editor.cell=e;var v=(null===(n=c.formatter)||void 0===n||null==n?void 0:n.toString())||"";if(0!==v.indexOf("function Badge")&&0!==v.indexOf("function Alert")||(this.editor.useValue=!0),this.settings.onEditCell&&this.settings.onEditCell(this.editor),this.editor.useValue&&(m=this.fieldValue(u,c.field)),this.editor.val(m),this.settings.showDirty){var b=m;""===b&&/checkbox|favorite/i.test(this.getEditorName(this.editor))&&(b=!1);var y={originalVal:b,isDirty:!1};this.addToDirtyArray(h,e,y)}if("function"==typeof this.editor.focus&&"date"!==this.editor.name&&this.editor.focus(),null!==(s=this.editor)&&void 0!==s&&(null==s?void 0:s.input.is(".dropdown"))&&i.keyCode&&![9,13,32,37,38,39,40].includes(i.keyCode))this.editor.input.data("dropdown").handleAutoComplete(i);return this.element.triggerHandler("entereditmode",[{row:h,cell:e,item:u,target:a,value:m,column:c,editor:this.editor}]),!0}},rowData:function(t){if(this.settings.groupable)return this.originalDataset[t];var e=this.settings.treeGrid?this.settings.treeDepth[t].node:this.settings.dataset[t];return this.settings.treeGrid&&this.settings.treeDepth[t].rowStatus&&(this.settings.treeDepth[t].node.rowStatus=this.settings.treeDepth[t].rowStatus),e},commitCellEdit:function(t){var e=this;if(this.editor){var i,n=this.editor.input,s=this.getEditorName(this.editor),a="editor"===s,o="fileupload"===s,r=!n.is(".timepicker, .datepicker, .lookup, .spinbox, .colorpicker");if(o)"function"==typeof this.editor.val&&(i=this.editor.val()),Promise.resolve(i).then((function(i){e.commitCellEditUtil(n,i,a,o,r,t)}));else{if("function"==typeof this.editor.val){var l=this.editor.val();i=isNaN(Date.parse(l))&&"date"===this.editor.name?"":l}this.commitCellEditUtil(n,i,a,o,r,t)}}},commitCellEditUtil:function(t,e,i,n,s,a){if(this.editor){var o,r,l;i?o=this.editor.td:n?("clear"===this.editor.status?e="":"init"!==this.editor.status&&"cancel"!==this.editor.status||(e=this.editor.originalValue),o=this.tableBody.find("#".concat(t.attr("id"))).closest("td"),e=g.escapeHTML(e)):(o=t.closest("td"),e=g.escapeHTML(e)),null!==this.settings.source&&s?(r=this.activeCell.rowIndex,l=this.activeCell.dataRow):(r=this.actualRowIndex(o.parent()),l=this.dataRowIndex(o.parent()));var c=o.attr("aria-colindex")-1,d=this.columnSettings(c),h=this.settings.treeGrid?this.settings.treeDepth[l].node:this.getActiveDataset()[l],u=this.fieldValue(h,d.field);if(d.beforeCommitCellEdit&&(n||!a))if(!1===d.beforeCommitCellEdit({cell:c,row:l,item:h,editor:this.editor,api:this}))return;if(this.editor){u=g.sanitizeConsoleMethods(u),e=g.sanitizeConsoleMethods(e);var p=o.hasClass("is-editing-inline");o.removeClass("is-editing is-editing-inline has-singlecolumn"),this.editor.destroy(),this.editor=null,this.updateCellNode(this.settings.groupable?l:r,c,e,!1,p,u);var f=this.fieldValue(h,d.field);this.element.triggerHandler("exiteditmode",[{row:r,cell:c,item:h,target:o,value:f,oldValue:u,column:d,editor:this.editor}])}}},validateCell:function(t,e){var i,n,s=this,a=this,o=this.columnSettings(e);if(o.validate){var r,l,c=[],d=o.validate.split(" "),h=$.fn.validation,u=this.fieldValue(this.settings.dataset[t],o.field),p=[],g="";for(l=0;l<d.length;l++){var f=h.rules[d[l]],m={row:t,cell:e,item:this.settings.dataset[t],column:o,grid:a};r=$.Deferred(),f.async?f.check(u,$("<input>").val(u),m,v,r):v(f.check(u,$("<input>").val(u),m),!0,d[l],r),c.push(r)}(i=$).when.apply(i,c).then((function(){var i=$.fn.validation.ValidationTypes;for(var r in i)if(g="",n=i[r],p[n.type]&&(g=p[n.type]),""!==g){a.showCellError(t,e,g,n.type);var l=s.dataRowNode(t);a.element.trigger("cell".concat(n.type),{row:t,cell:e,message:g,target:s.cellNode(l,e),value:u,column:o})}else a.clearCellError(t,e,n.type)}))}function v(t,e,i,s){var a=h.rules[i];n=$.fn.validation.ValidationTypes[a.type]||$.fn.validation.ValidationTypes.error,g="",p[n.type]&&(g=p[n.type]),!t&&e&&(g?(g=(/^\u2022/.test(g)?"":"• ")+g,g+="<br/>".concat("• ",a.message)):g=a.message,p[n.type]=g),s.resolve()}},showCellError:function(t,e,i,n){var s=this.dataRowNode(t),a=this.cellNode(s,e);if(this.clearNonVisibleCellErrors(t,e,n),!a.length)return this.nonVisibleCellErrors.push({row:t,cell:e,message:i,type:n}),void this.showNonVisibleCellErrors();a.addClass(n).attr("data-".concat(n,"message"),i);var o=a.find(".icon-".concat(n));if(!o.length){a.find(".datagrid-cell-wrapper").append($($.createIcon({classes:["icon-".concat(n)],icon:n})));var r={forced:!0,placement:"bottom",content:i,isError:"error"===n||"dirtyerror"===n,wrapper:o=a.find(".icon-".concat(n)),y:5};this.cacheTooltip(o,r),this.setupTooltips(!1,!0)}},showNonVisibleCellErrors:function(){var t=this;if(this.toolbar||(this.settings.toolbar={title:" "},this.appendToolbar()),0===this.nonVisibleCellErrors.length)this.toolbar&&1===this.toolbar.parent().find(".table-errors").length&&this.toolbar.parent().find(".table-errors").remove(),this.removeToolbar();else{var e=function(){var e=$.fn.validation.ValidationTypes[i].type,n=$.grep(t.nonVisibleCellErrors,(function(t){return t.type===e}));t.showNonVisibleCellErrorType(n,e)};for(var i in $.fn.validation.ValidationTypes)e()}},removeToolbar:function(){this.toolbar.find(".buttonset").children().length<1&&this.toolbar.find(".title").text().trim().length<1&&(this.toolbar.destroy(),this.toolbar.remove(),delete this.toolbar)},showNonVisibleCellErrorType:function(t,e){var i,n,s,a,o=[],r=$.fn.validation.ValidationTypes[e]||$.fn.validation.ValidationTypes.error;if(1===this.toolbar.parent().find(".table-errors").length&&(n=this.toolbar.parent().find(".table-errors")),0!==t.length){for(a=0;a<t.length;a++){var l=Math.floor((t[a].row+this.settings.pagesize)/this.settings.pagesize);-1===$.inArray(l,o)&&o.push(l)}for(a=0;a<o.length;a++)i="".concat((i?"".concat(i,"<br>"):"")+H.translate(r.pagingMessageID)," ").concat(o[a]);0===this.toolbar.parent().find(".table-errors").length&&(n=$('<div class="table-errors"></div>')),(s=n.find(".icon-".concat(e))).length||(n.append($($.createIcon({classes:["icon-".concat(e)],icon:e}))),s=n.find(".icon-".concat(e))),this.element.hasClass("has-toolbar")&&this.toolbar.children(".title").append(n),this.cacheTooltip(s,{forced:!0,placement:"bottom",content:i,isError:"error"===e||"dirtyerror"===e,wrapper:s}),this.setupTooltips(!1,!0)}else n&&n.length&&(s=n.find(".icon-".concat(r.type)),this.hideTooltip(),n.find(".icon-".concat(r.type)).remove())},clearAllCellError:function(t,e){var i=$.fn.validation.ValidationTypes;for(var n in i){var s=i[n];this.clearCellError(t,e,s.type)}},clearCellError:function(t,e,i){this.clearNonVisibleCellErrors(t,e,i);var n=this.dataRowNode(t),s=this.cellNode(n,e);s.length&&this.clearNodeErrors(s,i)},clearNonVisibleCellErrors:function(t,e,i){if(this.nonVisibleCellErrors.length){if(this.toolbar&&this.toolbar.parent()&&this.toolbar.parent().find(".table-errors").length>0){var n=this.toolbar.parent().find(".table-errors").find(".icon-".concat(i));if(n.length)$.grep(this.nonVisibleCellErrors,(function(t){return t.type===i?t:""})).length||n.remove()}this.nonVisibleCellErrors=$.grep(this.nonVisibleCellErrors,(function(n){return n.row!==t||n.cell!==e||n.type!==i?n:""})),this.nonVisibleCellErrors.length||this.showNonVisibleCellErrors()}},clearRowError:function(t){this.dataRowNode(t).removeClass("error alert rowstatus-row-new rowstatus-row-error rowstatus-row-alert rowstatus-row-info rowstatus-row-in-progress rowstatus-row-success"),this.rowStatus(t,"","");for(var e=0;e<this.settings.columns.length;e++)this.clearAllCellError(t,e)},clearAllErrors:function(){for(var t=0,e=0;e<this.settings.dataset.length;e++){this.settings.dataset[e].rowStatus&&(delete this.settings.dataset[e].rowStatus,t++);for(var i=0;i<this.settings.columns.length;i++)this.clearAllCellError(e,i)}t>0&&this.render()},clearNodeErrors:function(t,e){var i;(t=t instanceof jQuery?t[0]:t).classList.remove(e),t.removeAttribute("data-".concat(e,"message"));var n=t.querySelector(".icon-".concat(e));null===(i=t)||void 0===i||null==i||i.classList.remove("rowstatus-cell"),n&&(n.parentNode.removeChild(n),this.hideTooltip())},resetRowStatus:function(){for(var t=this.settings.dataset.filter((function(t){return t.rowStatus})),e=0;e<t.length;e++){var i,n,s,a;if(delete t[e].rowStatus,null!==(i=t[e])&&void 0!==i&&null!==(n=null==(s=i)?void 0:s.children)&&void 0!==n&&(null==(a=n)?void 0:a.length))for(var o=0;o<t[e].children.length;o++)delete t[e].children[o].rowStatus}t.length>0&&this.render(),this.clearDirty()},clearDirtyClass:function(t){t instanceof jQuery&&t.length<0||t.each((function(t,e){[].slice.call(e.querySelectorAll(".is-dirty-cell")).forEach((function(t){t.classList.remove("is-dirty-cell")}))}))},clearDirty:function(){this.settings.showDirty&&(this.clearDirtyClass(this.element),this.dirtyArray=void 0)},clearDirtyRow:function(t){if(this.settings.showDirty&&"number"==typeof t){var e=this.rowNodes(t);this.clearDirtyClass(e),this.dirtyArray&&(this.dirtyArray[t]=void 0)}},clearDirtyCell:function(t,e){this.settings.showDirty&&this.dirtyArray&&"number"==typeof t&&"number"==typeof e&&(void 0!==this.dirtyArray[t]&&(this.cellNode(t,e).removeClass("is-dirty-cell"),this.dirtyArray[t][e]=void 0))},dirtyCells:function(){var t=this.settings,e=t.treeGrid?t.treeDepth:t.dataset,i=[];if(t.showDirty&&this.dirtyArray&&this.dirtyArray.length)for(var n=0,s=e.length;n<s;n++){var a=this.dirtyArray[n];if(void 0!==a)for(var o=0,r=a.length;o<r;o++){var l=a[o];void 0!==l&&l.isDirty&&i.push(t.treeGrid?e[n].node:e[n])}}return i},dirtyRows:function(){var t=this.settings,e=t.treeGrid?t.treeDepth:t.dataset,i=[];if(t.showDirty&&this.dirtyArray&&this.dirtyArray.length)for(var n=0,s=e.length;n<s;n++){var a=this.dirtyArray[n];if(void 0!==a)for(var o=0,r=a.length;o<r;o++){var l=a[o];if(void 0!==l&&l.isDirty){i.push(t.treeGrid?e[n].node:e[n]);break}}}return i},getModifiedRows:function(t){for(var e=this.settings,i=e.treeGrid?e.treeDepth:e.dataset,n=[],s=0;s<i.length;s++){var a=e.treeGrid?i[s].node:i[s],o={row:s,data:a,cells:[]};if(this.isRowDirty(s)){o.type="dirty";for(var r=0;!t&&r<this.dirtyArray[s].length;r++){var l=this.dirtyArray[s][r];void 0!==l&&l.isDirty&&o.cells.push({row:s,col:r,cellData:l})}}void 0===a.rowStatus||"error"!==a.rowStatus.icon&&"in-progress"!==a.rowStatus.icon||(o.type="dirty"===o.type?["dirty",a.rowStatus.icon]:a.rowStatus.icon),void 0!==o.type&&n.push(t?a:o)}return n},showRowError:function(t,e,i){var n=i||"error";this.dataRowNode(t).addClass(i),this.rowStatus(t,n,e)},validateRow:function(t){if(void 0!==t)for(var e=0;e<this.settings.columns.length;e++)this.validateCell(t,e)},validateAll:function(){for(var t=0;t<this.settings.dataset.length;t++)for(var e=0;e<this.settings.columns.length;e++)this.validateCell(t,e)},columnSettings:function(t){return this.settings.columns[t]||{}},coerceValue:function(t,e,i,n,s){var a;if(i.serialize){var o=this.settings,r=o.treeGrid?o.treeDepth:o.dataset;this.settings.groupable&&(r=this.originalDataset||r);var l=o.treeGrid?r[n].node:r[n];return a=i.serialize(t,e,i,n,s,l)}return i.sourceFormat?a=t instanceof Date?H.parseDate(t,i.sourceFormat):H.formatDate(t,{pattern:i.sourceFormat}):"number"==typeof e&&t&&"number"!=typeof t&&!/[A-Za-z!@#$%^&*()_+\-=\[\]{};':"\\|<>/?]/.test(t)&&(a=H.parseNumber(t)),a},updateCell:function(t,e,i){var n=this.columnSettings(e);void 0===i&&(i=this.fieldValue(this.settings.dataset[t],n.field)),this.updateCellNode(t,e,i,!0),this.postRenderCell()},updateColumn:function(t){if(t&&t.length){var e=this,i=e.columnIdxById(t);$.each(e.settings.dataset,(function(n,s){e.updateCell(n,i,s[t])}))}},updateCellNode:function(t,e,i,n,s){var a,o=this,r=this.settings.groupable?this.rowNodesByDataIndex(t):this.rowNodes(t),l=r.find("td").eq(e),c=this.settings.columns[e]||{},d="",h=c.formatter?c.formatter:this.defaultFormatter,u=$(".editor",l).length>0,p=this.settings.treeGrid,f=p?this.actualPagingRowIndex(this.actualRowIndex(r)):this.dataRowIndex(r);(null==f||isNaN(f))&&(f=t),!p&&this.settings.paging&&this.pagerAPI.activePage>1&&f<(this.pagerAPI.activePage-1)*this.settings.pagesize&&(f+=(this.pagerAPI.activePage-1)*this.settings.pagesize);var m=this.rowData(f);0===r.length&&this.settings.paging&&(l=(r=this.visualRowNode(t)).find("td").eq(e));var v=this.fieldValue(m,c.field);if((n||v===i||void 0===(a=this.coerceValue(i,v,c,t,e)))&&(a=i),r.length&&m&&!m.rowStatus){var b=r.find("svg.icon-rowstatus");b.length&&b.remove()}if(this.removeTooltipData(l),0===e&&m&&m.rowStatus){var y=l.find("svg.icon-rowstatus");if(r&&l[0]){for(var w=0;w<r.length;w++)r[w].classList.add("rowstatus-row-".concat(m.rowStatus.icon));if(l[0].classList.add("rowstatus-cell"),!y.length){var x="success"===m.rowStatus.icon?"#icon-check":"#icon-exclamation";l.prepend('<svg class="icon icon-rowstatus" focusable="false" aria-hidden="true" role="presentation"><use href="'.concat(x,'"></use></svg>'))}}if(m.rowStatus.tooltip){y=l.find("svg.icon-rowstatus");var C=m.rowStatus.icon;this.cacheTooltip(y,{forced:!0,placement:"right",content:m.rowStatus.tooltip,isError:"error"===C||"dirtyerror"===C,wrapper:l})}}if(c.field&&a!==v)if(c.field.indexOf(".")>-1){var k=m,T=c.field.split(".").length;c.field.split(".").forEach((function(t,e){e===T-1?k[t]=a:k=k[t]}))}else m[c.field]=a;var A,S=function(t,e){var i=("string"==typeof t?t.match(e):[""])||[""];return o.settings.trimSpaces||(i=i[0].replace(/\s/g,"&nbsp;")),i},E=g.escapeHTML(a),D=u?a:E;("string"==typeof D&&(D="".concat(S(D,/^\s*/)).concat(D.trim()).concat(S(D,/\s*$/))),d=this.formatValue(h,t,e,D,c,m),c.contentVisible)&&(c.contentVisible(t,e,E,c,m)||(d=""));if(!s){var I=l.find(".datagrid-cell-wrapper");I.html(d);var L=I.children();if(0===L.length)I.html(g.unescapeHTML(d));else for(var P=0;P<L.length;P++){var M=L[P];M.innerText&&(M.innerText=g.unescapeHTML(M.innerText))}}if(n||(this.validateCell(f,e),this.isDirtyCellUndefined(f,e)||(this.dirtyArray[f][e].value=i,this.dirtyArray[f][e].coercedVal=a,this.dirtyArray[f][e].escapedCoercedVal=g.escapeHTML(a),this.dirtyArray[f][e].cellNodeText=l.text(),this.dirtyArray[f][e].cell=e,this.dirtyArray[f][e].column=this.settings.columns[e],this.setDirtyCell(f,e))),this.settings.stretchColumnOnChange&&c){var B=this.calculateTextWidth(c);c.minWidth&&B<c.minWidth&&(B=c.minWidth),c.minWidth&&B>c.maxWidth&&(B=c.maxWidth),B>0&&B-this.stretchColumnWidth>this.stretchColumnDiff&&(this.stretchColumnWidth=B,this.stretchColumnDiff=0,this.setColumnWidth(c.id,B,!0))}if(this.settings.rowTemplate){var O=this.settings.rowTemplate,R=m,z="";if(je&&R&&(z=je.compile("{{#dataset}}".concat(O,"{{/dataset}}"),{dataset:R})),l.parent().next().is(".datagrid-expandable-row"))l.parent().next().find(".datagrid-row-detail-padding").html(z)}if(this.settings.summaryRow&&!this.settings.groupable&&this.updateSummaryRow(c,e),v=g.sanitizeConsoleMethods(v),a=g.sanitizeConsoleMethods(a),"string"==typeof v&&"string"==typeof a){var F={oldVal:"".concat(S(v,/^\s*/)).concat(v.trim()).concat(S(v,/\s*$/)),coercedVal:"".concat(S(a,/^\s*/)).concat(a.trim()).concat(S(a,/\s*$/))};A=F.coercedVal!==F.oldVal&&!n}else A=a!==v&&!n;if(A){var N={row:null!==this.settings.source?f:t,relativeRow:t,cell:e,target:l,value:a,oldValue:v,column:c,api:this};N.rowData=p&&this.settings.treeDepth[t]?this.settings.treeDepth[t].node:m,this.isCellDirty(t,e)||this.setActiveCell(t,e),this.element.trigger("cellchange",N),this.wasJustUpdated=!0}},updateSummaryRow:function(t,e){if(t.summaryRowFormatter){var i=this.calculateTotals(),n=this.rowNodes(this.settings.dataset.length).find("td").eq(e),s='<div class="datagrid-cell-wrapper">'.concat(i[t.id],"</div>");n.html(s)}},isRowDirty:function(t){var e=!1;if("number"==typeof t&&this.dirtyArray&&this.dirtyArray.length){var i=this.dirtyArray[t];if(void 0!==i)for(var n=0,s=i.length;n<s;n++){var a=i[n];if(void 0!==a&&a.isDirty){e=!0;break}}}return e},isCellDirty:function(t,e){return!this.isDirtyCellUndefined(t,e)&&this.dirtyArray[t][e].isDirty},addToDirtyArray:function(t,e,i){void 0===this.dirtyArray&&(this.dirtyArray=[]),void 0===this.dirtyArray[t]?(this.dirtyArray[t]=[],this.dirtyArray[t][e]=i):void 0===this.dirtyArray[t][e]&&(this.dirtyArray[t][e]=i)},setDirtyCell:function(t,e,i){var n=this.cellNode(t,e);i&&this.addToDirtyArray(t,e,i);var s=this.dirtyArray[t][e];!(s.originalVal===s.value||s.originalVal===s.coercedVal||s.originalVal===s.escapedCoercedVal||s.originalVal===s.cellNodeText)||s.originalVal.length!==s.value.length&&s.originalVal.length!==s.cellNodeText.length?(this.dirtyArray[t][e].isDirty=!0,n[0].classList.add("is-dirty-cell"),this.setDirtyIndicator(t,e,!0)):(this.dirtyArray[t][e].isDirty=!1,this.setDirtyIndicator(t,e,!1)),("string"==typeof s.originalVal||s.originalVal instanceof String)&&(null==s||null==s?void 0:s.originalVal)===(null==s||null==s?void 0:s.value)&&(this.dirtyArray[t][e].isDirty=!1,this.setDirtyIndicator(t,e,!1))},setDirtyIndicator:function(t,e,i,n){var s=this.cellNode(t,e);n&&this.addToDirtyArray(t,e,n),t<0||e<0||!s.length||(i?s[0].classList.add("is-dirty-cell"):s[0].classList.remove("is-dirty-cell"))},isDirtyCellUndefined:function(t,e){return!(this.settings.showDirty&&"number"==typeof t&&"number"==typeof e&&t>-1&&e>-1&&void 0!==this.dirtyArray&&void 0!==this.dirtyArray[t]&&void 0!==this.dirtyArray[t][e])},visualRowIndex:function(t){var e="tr:visible:not(.is-hidden, .datagrid-expandable-row)",i=this.tableBody.find(e).index(t);return-1===i&&this.tableBodyLeft&&(i=this.tableBodyLeft.find(e).index(t)),-1===i&&this.tableBodyRight&&(i=this.tableBodyRight.find(e).index(t)),i},visualRowNode:function(t){var e=t;return this.settings.paging&&this.settings.source&&(e+=(this.pagerAPI.activePage-1)*this.settings.pagesize),this.isRowVisible(t)?this.tableBody.find('tr[aria-rowindex="'.concat(e+1,'"]')):$([])},actualRowNode:function(t){return this.tableBody.find('tr[aria-rowindex="'.concat(t+1,'"]'))},rowNodes:function(t){var e=this.element;return t instanceof jQuery&&(e=t.closest(".datagrid-container"),t=t.attr("aria-rowindex")-1),e.find('> .datagrid-wrapper > table > tbody > tr[aria-rowindex="'.concat(t+1,'"]'))},rowNodesByDataIndex:function(t){t instanceof jQuery&&(t=t.attr("data-index"));var e=this.tableBodyLeft?this.tableBodyLeft.find('tr[data-index="'.concat(t,'"]')):$(),i=this.tableBody.find('tr[data-index="'.concat(t,'"]')),n=this.tableBodyRight?this.tableBodyRight.find('tr[data-index="'.concat(t,'"]')):$();return $(i).add(e).add(n)},cellNode:function(t,e){var i=this.settings.groupable?this.rowNodesByDataIndex(t).find("td"):this.rowNodes(t).find("td");return i.eq(e>=i.length?i.length-1:e)},actualRowIndex:function(t){return(t=t instanceof jQuery?t:$(t)).attr("aria-rowindex")-1},pagingRowIndex:function(t){var e=t;return this.settings.paging&&this.settings.source&&!this.settings.indeterminate&&(e+=(this.pagerAPI.activePage-1)*this.settings.pagesize),e},actualPagingRowIndex:function(t){var e=t;return this.settings.paging&&this.settings.source&&!this.settings.indeterminate&&(e-=(this.pagerAPI.activePage-1)*this.settings.pagesize),e},dataRowNode:function(t){return this.tableBody.find('tr[data-index="'.concat(t,'"]'))},dataRowIndex:function(t){return t=t instanceof jQuery?t:$(t),parseInt(t.attr("data-index"),10)},setActiveCell:function(t,e){var i,n,s,a=this,o=a.activeCell,r=t,l=t instanceof jQuery&&t.is(".datagrid-rowgroup-header, .datagrid-rowgroup-footer");if(!(t instanceof jQuery&&0===t.length||("number"==typeof t&&(i=t,r=this.tableBody.find("tr:visible").eq(t),n=this.actualRowIndex(r),s=this.dataRowIndex(r)),t instanceof jQuery&&t.is("td")&&((l=t.parent().is(".datagrid-rowgroup-header, .datagrid-rowgroup-footer"))&&(r=t.parent()),e=t.attr("aria-colindex")-1,i=this.visualRowIndex(t.parent()),n=this.actualRowIndex(t.parent()),s=this.dataRowIndex(t.parent()),r=t.parent()),t instanceof jQuery&&t.is("tr")&&(i=this.visualRowIndex(t),n=this.actualRowIndex(t),s=this.dataRowIndex(t),r=t),i<0||e<0))){o.node&&1===o.node.length&&a.activeCell.node.removeAttr("tabindex").removeClass("is-active"),o.cell===e&&o.row===t||a.hideTooltip(),a.activeCell.node=a.cellNode(l||r?r:n>-1?n:i,e).attr("tabindex","0"),a.activeCell.node&&1===o.node.length?(a.activeCell.row=i,a.activeCell.cell=e,n=this.actualRowIndex(a.activeCell.node.parent()),s=this.dataRowIndex(a.activeCell.node.parent())):a.activeCell=o,(!$("input:not(.colorpicker), select, button:not(.btn-secondary, .row-btn, .datagrid-expand-btn, .datagrid-drilldown, .btn-icon)",a.activeCell.node).length||a.activeCell.node.is(".has-btn-actions")&&a.activeCell.node.find(".btn-actions").length)&&(a.activeCell.node.focus(),l&&(a.activeCell.groupNode=a.activeCell.node)),a.activeCell.node.is(".is-focusable")&&a.activeCell.node.find("button").focus(),void 0!==s&&(a.activeCell.dataRow=s),void 0!==n&&(a.activeCell.rowIndex=n);var c=+r.find("td[colspan]").attr("colspan");if(l&&a.activeCell.node&&o.node&&!(t instanceof jQuery&&t.is("td"))&&(e<c?(r.find("td[colspan]").attr("tabindex","0").focus(),a.activeCell.groupNode=r.find("td[colspan]")):e>=c?(r.find("td").eq(e-c+1).attr("tabindex","0").focus(),a.activeCell.groupNode=r.find("td").eq(e-c+1)):(r.find("td").eq(e).attr("tabindex","0").focus(),a.activeCell.groupNode=r.find("td").eq(e))),l&&t instanceof jQuery&&t.is("td")&&(a.activeCell.cell=c-1+e,t.is("[colspan]")&&(a.activeCell.cell=e)),this.settings.cellNavigation){var d=a.headerNodes(),h=0;this.hasColSpans&&(h=0,d.eq(e).prevAll("[colspan]").each((function(t,e){var i=$(e).attr("colspan")-1;h+=i})),e-=h),this.settings.activeCheckboxSelection&&(d.removeClass("is-active"),d.eq(e).addClass("is-active"))}this.activeCell.isFocused=!0;var u=this.settings.columns[e];u&&u.expandOnActivate&&this.activeCell&&this.activeCell.node&&a.activeCell.node.addClass("is-active"),a.element.trigger("activecellchange",{node:this.activeCell.node,row:this.activeCell.row,cell:this.activeCell.cell,api:a})}},setNextActiveCell:function(t){var e=this;if("keydown"===t.type)if(this.settings.actionableMode){var i=13===t.keyCode?40:t.keyCode;if(32===i)return;setTimeout((function(){var t=$.Event("keydown.datagrid");t.keyCode=i,e.activeCell.node.trigger(t)}),0)}else this.setActiveCell(this.activeCell.row,this.activeCell.cell)},addChildren:function(t,e){if(!(!e||e&&!e.length||t<0)){var i=this.settings.treeDepth[t].node;i.children=i.children||[];for(var n=0,s=(e=JSON.parse(JSON.stringify(e))).length;n<s;n++)i.children.push(e[n]);this.updateDataset(this.settings.dataset)}},setExpandedInDataset:function(t,e){this.settings.treeDepth[t].node.expanded=e},toggleChildren:function(t,e){if(!this.settings.groupable){var i=this,n=this.settings,s=this.rowNodes(e),a=s.find(".datagrid-expand-btn"),o=parseInt(s.attr("aria-level"),10),r=a.hasClass("is-expanded"),l=[{grid:i,row:e,item:s}];if(i.settings.treeDepth&&i.settings.treeDepth[e]&&(l[0].rowData=i.settings.treeDepth[e].node),s.hasClass("datagrid-tree-parent")&&($(t.target).is(a)||!i.settings.editable&&!i.settings.selectable)){var c=function(){s=i.rowNodes(e),a=s.find(".datagrid-expand-btn");var t=s.nextUntil('[aria-level="'.concat(o,'"]')),n=i.settings.treeGrid&&i.settings.source&&i.settings.paging?i.dataRowIndex(s):e;r?(s.attr("aria-expanded",!1),a.removeClass("is-expanded").find(".plus-minus").removeClass("active")):(s.attr("aria-expanded",!0),a.addClass("is-expanded").find(".plus-minus").addClass("active")),i.setExpandedInDataset(n,!r);!function t(e,n,s){var a=e.nextUntil('[aria-level="'.concat(n,'"]'));s?a.each((function(){var t=$(this);parseInt(t.attr("aria-level"),10)>n&&t.addClass("is-hidden")})):a.each((function(){var e=$(this),s=parseInt(e.attr("aria-level"),10);if(s===n+1){if(!e.hasClass("is-filtered")&&(e.removeClass("is-hidden"),i.settings.frozenColumns)){var a=e.attr("aria-rowindex");i.tableBody.find('[aria-rowindex="'.concat(a,'"]')).removeClass("is-hidden")}if(e.is(".datagrid-tree-parent")){var o=e.find(".datagrid-expand-btn.is-expanded").length>0;o&&t(e,s,!o)}}}))}(s,o,r),i.setAlternateRowShading(),l.children=t},d=r?"collapserow":"expandrow";$.when(i.element.triggerHandler(d,l)).done((function(t){var e=function(t){return"string"==typeof t&&"false"===t.toLowerCase()||"boolean"==typeof t&&!1===t||"number"==typeof t&&0===t};e(t)||("function"!=typeof n.onExpandChildren||r?"function"==typeof n.onCollapseChildren&&r?$.when(n.onCollapseChildren(l[0])).done((function(t){e(t)||c()})):c():$.when(n.onExpandChildren(l[0])).done((function(t){e(t)||c()})))}))}}},toggleRowDetail:function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],i=this,n=i.rowNodes(t);!n.length&&i.settings.paging&&(i.settings.rowTemplate||i.settings.expandableRow)&&(t+=(i.pagerAPI.activePage-1)*i.settings.pagesize,n=i.dataRowNode(t));var s=n.next(),a=n.find(".datagrid-expand-btn"),o=s.find(".datagrid-row-detail"),r=i.settings.dataset[i.dataRowIndex(n)];if(!n.hasClass("datagrid-tree-parent")){if(i.settings.allowOneExpandedRow&&null===i.settings.groupable){var l=i.tableBody.add(i.tableBodyLeft).add(i.tableBodyRight).find("tr.is-expanded"),c=l.prev().find(".datagrid-expand-btn"),d=l.prev(),h=i.actualRowNode(d),u=i.dataRowIndex(d);if(l.length&&s.index()!==l.index()){var p=l.find(".datagrid-row-detail");l.add(c).removeClass("is-expanded"),d.removeClass("is-rowactivated"),d.find(".plus-minus").removeClass("active"),p.animateClosed().on("animateclosedcomplete",(function(){l.removeClass("is-expanded"),i.element.triggerHandler("collapserow",[{grid:i,row:h,detail:p,item:i.settings.dataset[u]}])}));var g=l.prev().find(".btn-primary");g.length&&g.attr("class",g.attr("class").replace("btn-primary","btn-secondary"))}var f=!s.hasClass("is-expanded"),m=s.prev().find(f?".btn-secondary":".btn-primary");if(r.expanded=!1,m.length>0&&d&&m){var v=m.attr("class")||"";m.attr("class",v.replace(f?"btn-secondary":"btn-primary",f?"btn-primary":"btn-secondary"))}}if(s.hasClass("is-expanded")&&!e)o.one("animateclosedcomplete",(function(){s.removeClass("is-expanded")})).animateClosed(),a.removeClass("is-expanded").find(".plus-minus").removeClass("active"),i.settings.allowOneExpandedRow&&n.removeClass("is-rowactivated"),r.expanded=!1,i.element.triggerHandler("collapserow",[{grid:i,row:t,detail:o,item:r}]);else if(r){s.addClass("is-expanded"),a.addClass("is-expanded").find(".plus-minus").addClass("active"),r.expanded=!0,s.find(".constrained-width").css("max-width",this.element.outerWidth());var b=[{grid:i,row:t,detail:o,item:r}];if(i.element.triggerHandler("expandrow",b),i.settings.allowOneExpandedRow&&"multiple"!==this.settings.selectable&&n.addClass("is-rowactivated"),i.settings.onExpandRow){i.settings.onExpandRow(b[0],(function(t){t&&o.find(".datagrid-row-detail-padding").empty().append(t),i.adjustExpandRowHeight(o),o.animateOpen()}))}else i.adjustExpandRowHeight(o),o.animateOpen();if(i.settings.frozenColumns.expandRowAcrossAllCells&&i.frozenExpandRowAcrossAllCells(),i.settings.frozenColumns.left.length||i.settings.frozenColumns.right.length){var y={left:o.eq(0)[0],center:o.eq(1)[0],right:o.eq(2)[0]},w={theme:"themechanged.".concat(Uo),rowheight:"rowheightchanged.".concat(Uo)};$("html").off(w.theme).on(w.theme,(function(){i.frozenExpandRowSetHeight(y)})),i.element.off(w.rowheight).on(w.rowheight,(function(){i.frozenExpandRowSetHeight(y)}))}}}},adjustExpandRowHeight:function(t){if(t.length&&(this.settings.frozenColumns.left.length||this.settings.frozenColumns.right.length)){var e={left:t.eq(0)[0]?t.eq(0)[0].children[0]:null,center:t.eq(1)[0],right:t.eq(2)[0]?t.eq(2)[0].children[0]:null},i=0;e.center&&(e.center.style.height="auto",i=e.center.offsetHeight-1,e.center.style.height=""),i>0&&(e.left&&(e.left.style.height="".concat(i,"px")),e.right&&(e.right.style.height="".concat(i,"px")))}},frozenExpandRowSetHeight:function(t){if(this.settings.frozenColumns.left.length||this.settings.frozenColumns.right.length){var e=0;if(t&&t.center&&(t.padding=t.center.querySelector(".datagrid-row-detail-padding"),e=t.padding.offsetHeight),e&&(t.center.style.height="".concat(e,"px"),t.left&&(t.left.style.height="".concat(e,"px")),t.right&&(t.right.style.height="".concat(e,"px")),this.settings.frozenColumns.expandRowAcrossAllCells)){var i={container:this.element[0].getBoundingClientRect(),centerEl:t.center.getBoundingClientRect()},n="".concat(i.centerEl.top-i.container.top,"px");t.padding.style.top=n}}},frozenExpandRowAcrossAllCells:function(){var t=this;if(this.settings.frozenColumns.left.length||this.settings.frozenColumns.right.length){var e={row:".datagrid-expandable-row.is-expanded"};e.detail="".concat(e.row," > td .datagrid-row-detail"),e.padding="".concat(e.detail," .datagrid-row-detail-padding");var i={left:this.tableBodyLeft?this.tableBodyLeft[0]:null,center:this.tableBody?this.tableBody[0]:null,right:this.tableBodyRight?this.tableBodyRight[0]:null},n={rows:{left:i.left?i.left.querySelector(e.row):null,center:i.center?i.center.querySelector(e.row):null,right:i.right?i.right.querySelector(e.row):null},details:{left:i.left?i.left.querySelector(e.detail):null,center:i.center?i.center.querySelector(e.detail):null,right:i.right?i.right.querySelector(e.detail):null},padding:this.tableBody[0].querySelector(e.padding)};if(n.padding&&(n.details.left||n.details.right)){var s="is-expanded-frozen";n.padding.style.opacity="0",n.rows.left&&n.rows.left.classList.add(s),n.rows.right&&n.rows.right.classList.add(s),$(n.details.center).one("animateopencomplete.datagrid.expandedfrozen",(function(){(n.details.left||n.details.right)&&setTimeout((function(){n.rows.center.classList.add(s),t.frozenExpandRowSetHeight(n.details),n.padding.style.opacity="",$(window).on("resize.datagrid.expandedfrozen",(function(){t.frozenExpandRowSetHeight(n.details)}))}),10)})).one("animateclosedstart.datagrid.expandedfrozen",(function(){$(window).off("resize.datagrid.expandedfrozen"),n.padding.style.opacity="0",n.padding.style.top="",n.rows.center.classList.remove(s),n.rows.left&&n.rows.left.classList.remove(s),n.rows.right&&n.rows.right.classList.remove(s)})).one("animateclosedcomplete.datagrid.expandedfrozen",(function(){n.padding.style.opacity=""}))}}},toggleGroupChildren:function(t){if(this.settings.groupable){var e=this,i=t.index(),n=$(),s=$(),a=$();this.hasLeftPane&&(n=this.tableLeft.find("tbody tr").eq(i).nextUntil(".datagrid-rowgroup-header")),s=this.table.find("tbody tr").eq(i).nextUntil(".datagrid-rowgroup-header"),this.hasRightPane&&(a=this.tableRight.find("tbody tr").eq(i).nextUntil(".datagrid-rowgroup-header"));var o=t.find(".datagrid-expand-btn");t.hasClass("is-expanded")?(o.removeClass("is-expanded").find(".plus-minus").removeClass("active"),n.hide(),n.addClass("is-hidden"),s.hide(),s.addClass("is-hidden"),a.hide(),a.addClass("is-hidden"),e.element.triggerHandler("collapserow",[{grid:e,row:t.index(),detail:s,item:{}}]),t.removeClass("is-expanded")):(o.addClass("is-expanded").find(".plus-minus").addClass("active"),n.show(),n.removeClass("is-hidden"),s.show(),s.removeClass("is-hidden"),a.show(),a.removeClass("is-hidden"),e.element.triggerHandler("expandrow",[{grid:e,row:t.index(),detail:s,item:{}}]),t.addClass("is-expanded"))}},setSortColumn:function(t,e){var i=!0;if(void 0!==e?(this.sortColumn.sortAsc===e&&this.sortColumn.sortId===t&&(i=!1),this.sortColumn.sortAsc=e):(this.sortColumn.sortId===t?this.sortColumn.sortAsc=!this.sortColumn.sortAsc:this.sortColumn.sortAsc=!0,e=this.sortColumn.sortAsc),this.sortColumn.sortId=t,this.sortColumn.sortField=this.columnById(t)[0]?this.columnById(t)[0].field:t,this.sortColumn.sortAsc=e,this.setSortIndicator(t,e),i&&this.sortDataset(),!this.settings.focusAfterSort&&this.activeCell&&this.activeCell.isFocused&&(this.activeCell.isFocused=!1),i){var n=this.activeCell.isFocused;this.settings.source||(this.setTreeDepth(),this.setRowGrouping(),this.setTreeRootNodes(),this.clearCache(),this.renderRows(),this.syncSelectedUI()),n&&1===this.activeCell.node.length&&this.setActiveCell(this.activeCell.row,this.activeCell.cell),this.filterExpr&&this.filterExpr[0]&&"all"===this.filterExpr[0].column&&this.highlightSearchRows(this.filterExpr[0].value),this.settings.source&&this.triggerSource({type:"sorted"})}this.tableBody.removeClass("is-loading"),this.saveUserSettings(),this.validateAll(),this.element.trigger("sorted",[this.sortColumn])},sortDataset:function(){if(this.settings.disableClientSort)this.restoreSortOrder=!0;else{this.settings.groupable&&this.originalDataset&&(this.settings.dataset=this.originalDataset);var t=this.sortFunction(this.sortColumn.sortId,this.sortColumn.sortAsc);this.saveDirtyRows(),this.settings.dataset.sort(t),this.setTreeDepth(),this.restoreDirtyRows(),this.settings.selectable&&this.syncDatasetWithSelectedRows()}},saveDirtyRows:function(){var t=this.settings,e=t.treeGrid?t.treeDepth:t.dataset;if(this.settings.groupable&&(e=this.originalDataset||e),t.showDirty&&!this.settings.source&&this.dirtyArray&&this.dirtyArray.length)for(var i=0,n=e.length;i<n;i++){if(void 0!==this.dirtyArray[i])(t.treeGrid?e[i].node:e[i]).tempNodeIndex=i}},restoreDirtyRows:function(){var t=this.settings,e=t.treeGrid?t.treeDepth:t.dataset;if(this.settings.groupable&&(e=this.originalDataset||e),t.showDirty&&this.dirtyArray&&this.dirtyArray.length){for(var i=[],n=0,s=e.length;n<s;n++){var a=t.treeGrid?e[n].node:e[n];void 0!==a.tempNodeIndex&&(i.push({newIdx:n,oldIdx:a.tempNodeIndex}),delete a.tempNodeIndex)}for(var o=[],r=0,l=i.length;r<l;r++)o[i[r].newIdx]=this.dirtyArray[i[r].oldIdx];this.dirtyArray=o}},syncDatasetWithSelectedRows:function(){var t,e=this,i=this.settings;if(!(i.source&&i.paging&&i.allowSelectAcrossPages&&null!==(t=i.columnIds)&&void 0!==t&&(null==t?void 0:t.length))){this._selectedRows=[];for(var n,s=i.treeGrid?i.treeDepth:i.dataset,a=-1,o=function(t,o){if(i.groupable&&!e.originalDataset)for(var r=0;r<Object.keys(s[o]).length;r++)a++,t=Object.keys(s[o]).map((function(t){return s[o][t]})),e.isRowSelected(t)&&e._selectedRows.push({idx:a,data:t,elem:e.dataRowNode(a),group:s[o],page:e.pagerAPI?e.pagerAPI.activePage:1,pagingIdx:a,pagesize:e.settings.pagesize});else t=i.treeGrid?s[o].node:s[o],e.isRowSelected(t)&&e._selectedRows.push({idx:o,data:t,elem:e.visualRowNode(o),pagesize:e.settings.pagesize,page:e.pagerAPI?e.pagerAPI.activePage:1,pagingIdx:o});n=t},r=0;r<s.length;r++)o(n,r)}},setSortIndicator:function(t,e){this.headerRow&&(this.sortColumn={sortId:t,sortAsc:e,sortField:this.columnById(t)[0]?this.columnById(t)[0].field:t},this.element.find(".is-sorted-asc, .is-sorted-desc").removeClass("is-sorted-asc is-sorted-desc"),this.element.find('[data-column-id="'.concat(t,'"]')).addClass(e?"is-sorted-asc":"is-sorted-desc").attr("aria-sort",e?"ascending":"descending"))},sortFunction:function(t,e){var i=this.columnById(t),n=0===i.length?null:i[0],s=null===n?t:n.field,a=this,o=function(t){return"string"==typeof(e=null==(e=a.fieldValue(t,s))?"":e)&&(e=e.toUpperCase(),$.isNumeric(e)&&(e=parseFloat(e))),e;var e};return n&&n.sortFunction&&(o=function(t){return n.sortFunction(a.fieldValue(t,s))}),e=e?1:-1,function(t,i){return t=o(t),i=o(i),"number"==typeof t&&"string"==typeof i&&""!==i?-1*e:"string"==typeof t&&"number"==typeof i&&""!==t?e:""===t?""===i?0:1:""===i?""===t?0:-1:(l(t)!==l(i)&&(t=t.toString().toLowerCase(),i=i.toString().toLowerCase()),e*((t>i)-(i>t)))}},defaultFormatter:function(t,e,i,n,s,a){if(arguments.length>6&&void 0!==arguments[6]&&arguments[6]){var o=parseFloat(i);if(isNaN(o)){var r=Date.parse(i);isNaN(r)||(i=H.formatDate(r))}else i=H.formatNumber(o,{style:Number.isInteger(o)?"integer":"decimal"})}return null==i||""===i?"":i.toString()},handlePaging:function(){this.settings.paging&&(this.element.addClass("paginated"),this.tableBody.pager({componentAPI:this,dataset:this.settings.dataset,hideOnOnePage:this.settings.hidePagerOnOnePage,source:this.settings.source,pagesize:this.settings.pagesize,indeterminate:this.settings.indeterminate,rowTemplate:this.settings.rowTemplate,pagesizes:this.settings.pagesizes,pageSizeSelectorText:this.settings.groupable?"GroupsPerPage":"RecordsPerPage",showPageSizeSelector:this.settings.showPageSizeSelector,activePage:this.restoreActivePage?parseInt(this.savedActivePage,10):1}),this.restoreActivePage&&(this.savedActivePage=null,this.restoreActivePage=!1))},renderPager:function(t,e,i){this.pagerAPI&&(this.settings.source||(this.pagerAPI.settings.dataset=this.settings.dataset,t.isFilteredClientside=!0),this.pagerAPI.updatePagingInfo(t,e),e||this.triggerSource(t,i),this.syncSelectedUI())},getPageableElements:function(){var t=this.element.children().not(".datagrid-expandable-row");return t.is("table")&&(t=t.find("tbody tr")),t},appendTooltip:function(t){var e=this,i="grid-tooltip",n=new RegExp("\\b".concat(i,"\\b"),"g");"string"==typeof t?n.test(t)||(t+=" ".concat(i)):t=i;var s=this.uniqueId("tooltip");if(this.tooltip=document.getElementById(s),!this.tooltip){var a=""+'<div id="'.concat(s,'" class="tooltip ').concat(t,' is-hidden">\n          <div class="arrow"></div>\n          <div class="tooltip-content"></div>\n        </div>');document.body.insertAdjacentHTML("beforeend",a),this.tooltip=document.getElementById(s),this.isTouch&&(this.tooltip.style.pointerEvents="auto",$(this.tooltip).on("touchend.gridtooltip",(function(){e.hideTooltip()})))}},cacheTooltip:function(t,e){if(void 0===e){var i,n,s,a,o=t.querySelector(".is-editor.content-tooltip"),r=t.querySelector("a[title]"),l=f.hasClass(t,"rowstatus-cell"),c="svg"===t.tagName.toLowerCase(),d=t.classList.contains("datagrid-header-icon"),h="th"===t.tagName.toLowerCase(),u=f.hasClass(t,"datagrid-column-wrapper"),p=f.hasClass(t.parentNode,"datagrid-filter-wrapper"),m=t.getAttribute("aria-colindex")-1,v=this.columnSettings(m);(e={content:"",wrapper:t.querySelector(".datagrid-cell-wrapper")}).extraClassList=[];var b=$(t).index();if(u){var y=$(t).attr("data-column-id");a=this.settings.columns[this.columnIdxById(y)]}else{var w=$(t).closest("table").find("thead > tr > th").eq(b).attr("data-column-id");a=this.settings.columns[this.columnIdxById(w)]}if((h||u||p||d)&&(e.wrapper=t,e.distance=p||d?15:null,e.placement=u||d?"top":"bottom"),l||c&&!d){var C=this.closest(t,(function(t){return f.hasClass(t,"datagrid-row")})),k=C?C.classList:[];e.isError=k.contains("rowstatus-row-error"),e.placement="right",c&&(e.wrapper=this.closest(t,(function(t){return"td"===t.tagName.toLowerCase()})))}if(o){var T=!(!v.editorOptions||!v.editorOptions.width)&&this.setUnit(v.editorOptions.width),A=$(o).clone()[0];A.style.width=T||"".concat(t.offsetWidth,"px");var S=A.outerHTML;""!==g.stripHTML(S)&&(e.content=S,e.extraClassList=["popover","alternate","content-tooltip"])}else if(r)e.content=r.getAttribute("title"),r.removeAttribute("title");else if(s=t.getAttribute("title")){var E=t.querySelector(".row-btn[disabled]");e.content=E?"":s,t.removeAttribute("title")}else if(h&&!p){var D=t.querySelector(".datagrid-header-text");e.content=D?g.stripHTML(D.textContent):""}else if(p){if(!(t.parentNode.querySelectorAll('.dropdown.is-disabled, input[type="text"][disabled], .btn-filter[disabled]').length>0)){var I=t.parentNode.querySelector(".is-checked");e.content=I?g.stripHTML(I.textContent):""}}else if(d){var L=$(t).parents("th").attr("data-column-id"),P=t.previousElementSibling.textContent,B=this.settings.columns[this.columnIdxById(L)].headerIconTooltip;e.content=g.stripHTML(B),e.headerText=g.stripHTML(P)}else e.wrapper.querySelector(".datagrid-expand-btn")?Array.prototype.filter.call(e.wrapper.children,(function(t){return!t.matches(".datagrid-expand-btn")})).forEach((function(t){e.content=t.textContent})):e.content=e.wrapper.textContent,e.content=g.stripHTML(e.content).trim();var O=t.getAttribute("tooltipClass");O&&e.extraClassList.push(O);var R=null===(i=e.wrapper)||void 0===i||null==i?void 0:i.querySelector("select");if(R&&R.selectedIndex&&R.options[R.selectedIndex]&&R.options[R.selectedIndex].innerHTML&&(e.content=x.features.touch?"":R.options[R.selectedIndex].innerHTML.trim()),h&&(e.content=e.content.trim()),""!==e.content){var z=f.hasClass(u?t.parentNode:t,"text-ellipsis"),F=[].slice.call(t.querySelectorAll(".icon")),N=z?8:0;if(F.forEach((function(t){var e="function"==typeof t.getBBox?t.getBBox():t.getBoundingClientRect();N+=e.width+8})),this.settings.treeGrid){var H=function(t){return!!e.wrapper.querySelector(t)};if(H(".datagrid-tree-node")||H(".datagrid-expand-btn")){var q=this.closest(e.wrapper,(function(t){return f.hasClass(t,"datagrid-row")})),V=parseInt(q.getAttribute("aria-level"),10);V&&(N+=30*V)}}if(z&&u){var W=t.querySelector(".datagrid-header-text");e.textwidth=W.scrollWidth+(R?0:N)}else e.textwidth=M.textWidth(e.content)+(R?0:N);if(h&&(e.textwidth=M.textWidth(e.content)),e.content=o?e.content:"<p>".concat(e.content,"</p>"),d){var j=t.previousElementSibling.textContent;e.content='<p class="align-text-left text-strong">'.concat(j,"</p>").concat(e.content)}(s||p||d)&&(e.forced=!0)}if("function"==typeof v.tooltip){var U,G,_=this.closest(t,(function(t){return f.hasClass(t,"datagrid-row")}));this.settings.treeGrid&&this.settings.treeDepth?(U=this.actualRowIndex(_),G=this.settings.treeDepth[U].node):(U=this.dataRowIndex(_),G=this.rowData(U));var Y=this.fieldValue(G,v.field);e.content=v.tooltip(U,m,Y,v,G,this),e.textwidth=M.textWidth(e.content)+20,void 0!==e.content&&null!==e.content&&""!==e.content&&(e.forced=!0)}null!==(n=a)&&void 0!==n&&(null==n?void 0:n.tooltipOptions)&&(a.tooltipOptions.placement&&(e.placement=a.tooltipOptions.placement),a.tooltipOptions.x&&(e.x=a.tooltipOptions.x),a.tooltipOptions.y&&(e.y=a.tooltipOptions.y))}return(t=t instanceof jQuery?t:$(t)).data("gridtooltip",e),e},showTooltip:function(t){var e=this;if(this.tooltip){var i=$(this.tooltip),n=this.tooltip.querySelector(".tooltip-content");if(n){var s;n.innerHTML=g.sanitizeHTML(t.content),this.tooltip.classList.remove("is-hidden","top","right","bottom","left"),this.tooltip.style.display="",this.tooltip.classList.add(t.placement||"top"),t.isError&&this.tooltip.classList.add("is-error"),t.extraClassList&&t.extraClassList.length>0&&t.extraClassList.map((function(t){return e.tooltip.classList.add(t)}));var a=null===(s=t.wrapper.classList)||void 0===s||null==s?void 0:s.contains("datagrid-header-icon");n.classList[a?"add":"remove"]("header-icon");var o="number"==typeof t.distance?t.distance:0,r={x:t.x||0,y:t.y||o,container:this.element.closest(".page-container.scrollable")||$("body"),containerOffsetX:t.wrapper.offsetLeft,containerOffsetY:t.wrapper.offsetTop,parent:$(null==t||null==t?void 0:t.wrapper),placement:t.placement||"top",strategies:["flip","nudge"]};"left"!==r.placement&&"right"!==r.placement||(r.x=o,r.y=0),i.one("afterplace.gridtooltip",(function(t,n){e.handleAfterPlaceTooltip(t,i,n)})).on("click.gridtooltip",(function(){e.hideTooltip()})),i.data("place")||i.place(r),i.data("place").place(r),i.data("gridtooltip",!0),$("body, .scrollable").off("scroll.gridtooltip").on("scroll.gridtooltip",(function(){e.hideTooltip()}))}}},handleAfterPlaceTooltip:function(t,e,i){var n=e||$("#tooltip");n[0]&&(n.data("place").setArrowPosition(t,i,n),n.triggerHandler("tooltipafterplace",[i]))},hideTooltip:function(){var t=this;this.tooltip&&(this.removeTooltipData(this.tooltip),this.tooltip.classList.add("is-hidden"),this.tooltip.classList.remove("is-error","popover","alternate","content-tooltip"),this.tooltip.style.left="-999px"),$("body, .scrollable").off("scroll.gridtooltip",(function(){t.hideTooltip()}))},isGridtooltip:function(){var t=!1;this.tooltip&&((this.tooltip instanceof jQuery?this.tooltip:$(this.tooltip)).data("gridtooltip")&&(t=!0));return t},removeTooltipData:function(t){(t=t instanceof jQuery?t:$(t)).data("gridtooltip")&&$.removeData(t[0],"gridtooltip")},removeTooltip:function(){var t=this;if(this.tooltip){var e=$(this.tooltip),i={th:".datagrid-header th",td:'.datagrid-wrapper tbody tr.datagrid-row td[role="gridcell"]:not(.rowstatus-cell)',rowstatus:'.datagrid-wrapper tbody tr.datagrid-row td[role="gridcell"] .icon-rowstatus'};i.str="".concat(i.th,", ").concat(i.td,", ").concat(i.rowstatus),$("body, .scrollable").off("scroll.gridtooltip"),e.off("touchend.gridtooltip"),this.element.off("mouseenter.gridtooltip mouseleave.gridtooltip click.gridtooltip longpress.gridtooltip keydown.gridtooltip",i.str),this.toolbar&&this.toolbar.parent().find(".table-errors").length>0&&this.toolbar.parent().find(".table-errors").off("mouseenter.tableerrortooltip",".icon").off("mouseleave.tableerrortooltip click.tableerrortooltip",".icon").off("longpress.tableerrortooltip",".icon");var n=e.data("place");n&&n.destroy(),[].slice.call(this.element[0].querySelectorAll(i.str)).forEach((function(e){return t.removeTooltipData(e)})),this.tooltip.parentNode&&this.tooltip.parentNode.removeChild(this.tooltip)}this.tooltip=void 0},destroy:function(){var t,e,i,n,s=this.element.prev(".toolbar, .contextual-toolbar");s=s.add(s.find("*"));var a=this.element.next(".pager-toolbar");a=a.add(a.find("*")),this.element.add(s).add(a).find("*").each((function(t,e){Object.values($(e).data()).forEach((function(t){"function"==typeof(null==t||null==t?void 0:t.destroy)&&t.destroy()}))})),null===(t=this.canvas)||void 0===t||null===(e=null==t?void 0:t.parentNode)||void 0===e||null===(i=null==e?void 0:e.removeChild)||void 0===i||null==(n=i.call)||n.call(i,e,this.canvas),this.canvas=null,this.removeTooltip(),$("html").off("themechanged.".concat(Uo)),this.element.find(".datagrid-img").off("error.datagrid"),this.element.add(this.element.find("*")).off("selected.gridpopupth close.gridpopupth selected.gridpopuptr close.gridpopuptr selected.gridpopupbtn close.gridpopupbtn"),this.pagerAPI&&(this.tableBody.off("page.".concat(Uo," pagesizechange.").concat(Uo)),this.pagerAPI.destroy()),this.element.off().empty().removeClass("datagrid-container");var o=this.element.prev(".toolbar");if(this.triggerDestroyCell(),this.removeToolbarOnDestroy&&this.settings.toolbar&&this.settings.toolbar.keywordFilter){var r=o.find(".searchfield"),l=r.data("searchfield"),c=r.parent().find(".close.icon");r.off("keypress.datagrid"),c.off("click.datagrid"),l&&"function"==typeof l.destroy&&l.destroy(),r.removeData("options")}return this.removeToolbarOnDestroy&&(o.data("toolbar")&&o.data("toolbar").destroy(),o.remove()),this.element.next(".pager-toolbar").remove(),$.removeData(this.element[0],Uo),this.element.off(),$(document).off("touchstart.datagrid touchend.datagrid touchcancel.datagrid click.datagrid touchmove.datagrid"),$("body").off("resize.vtable resize.datagrid resize.frozencolumns"),$("body").off("open.modal.datagrid"),$(window).off("orientationchange.datagrid"),$(window).off("resize.datagrid"),this},updated:function(t,e){var i=this.settings.paging!==(null==t||null==t?void 0:t.paging)&&(null==t||null==t?void 0:t.paging)&&!e;if(this.settings=E.mergeSettings(this.element,t,this.settings),null!=t&&(null==t?void 0:t.paging)||!this.pagerAPI||"function"!=typeof this.pagerAPI.destroy||this.pagerAPI.destroy(),t&&t.frozenColumns&&(this.headerRow=void 0,this.element.empty(),this.firstRender()),t&&t.dataset&&(this.settings.dataset=t.dataset),t&&t.columns&&(this.settings.columns=t.columns),t&&t.toolbar&&!this.toolbar&&this.appendToolbar(),t&&t.toolbar&&this.toolbar&&this.toolbar.hasClass("datagrid-toolbar")){var n=this.element.parent().find(".toolbar:not(.contextual-toolbar), .flex-toolbar:not(.contextual-toolbar)"),s=this.toolbar.data("toolbar")?this.toolbar.data("toolbar"):this.toolbar.data("toolbarFlex");s&&s.destroy(),n.remove(),this.appendToolbar()}return this.setRowHeightClass(),this.handlePaging(),this.attachPagingEvents(),this.render(null,e,i),this.renderHeader(),this}},$.fn.datagrid=function(t){return this.each((function(){var e=$.data(this,Uo);e?e.updated(t):e=$.data(this,Uo,new _o(this,t))}))};var Yo="formcompact",Ko={},Xo="is-focused";function Qo(t,e){if(!(t instanceof HTMLElement))throw new Error("Element used for `FormCompact` component must be an instance of HTMLElement");this.settings=E.mergeSettings(t,e,Ko),this.element=t,this.init()}Qo.prototype={init:function(){this.form=this.element.querySelector("form"),this.inputs=E.getArrayFromList(this.element.querySelectorAll("input")),this.renderProps(),this.handleEvents()},renderProps:function(){var t=this;if(this.inputs&&this.inputs.length){var e=["disabled","readonly"];this.inputs.forEach((function(i){e.forEach((function(e){t.setState(e,i)}))}))}},handleEvents:function(){var t=this;$(this.form).on("focusin.".concat(Yo),"input",(function(e){return t.handleFocusIn(e)})).on("focusout.".concat(Yo),"input",(function(e){return t.handleFocusOut(e)}));var e=["disabled","readonly"];this.inputsObserver=new MutationObserver((function(i){i.length&&i.forEach((function(i){"attributes"===i.type&&e.indexOf(i.attributeName)>-1&&t.setState(i.attributeName,i.target)}))}));var i={attributes:!0};this.inputs.forEach((function(e){t.inputsObserver.observe(e,i)}))},handleFocusIn:function(t){f.parents(t.target,".column, .columns",!0).classList.add(Xo)},handleFocusOut:function(t){f.parents(t.target,".column, .columns",!0).classList.remove(Xo)},setState:function(t,e){if("string"==typeof t&&e instanceof HTMLElement){if(!(["checkbox","radio"].indexOf(e.type)>-1)){"readonly"===t&&(t="readOnly");var i=!0===e[t]?"add":"remove";e.parentNode.classList[i]("is-".concat(t.toLowerCase()))}}},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown(),this.init()},teardown:function(){this.inputsObserver.disconnect(),delete this.inputsObserver,$(this.form).off(["focusin.".concat(Yo),"focusout.".concat(Yo)].join(" ")),delete this.form,delete this.inputs},destroy:function(){this.teardown(),$.removeData(this.element,Yo)}},$.fn.formcompact=function(t){return this.each((function(){var e=$.data(this,Yo);e?e.updated(t):e=$.data(this,Yo,new Qo(this,t))}))};var Jo="header",Zo={addScrollClass:!1,demoOptions:!0,tabs:null,toolbarSettings:void 0,useAlternate:!1,useBackButton:!0,useBreadcrumb:!1,useFlexToolbar:!1,usePopupmenu:!1,wizardTicks:null};function tr(t,e){this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,Zo),this.init()}tr.prototype={get toolbarAPI(){if(this.toolbarElem)return this.toolbarElem.data("toolbar-flex")||this.toolbarElem.data("toolbar")},get toolbar(){return V("toolbarAPI","toolbar"),this.toolbarAPI},get titleButton(){var t=".title > button";return this.toolbarElem&&this.toolbarElem.is(".flex-toolbar")&&(t=".toolbar-section:first-child > button"),this.element.find(t)},get hasTitleButton(){return this.titleButton.length>0},init:function(){this.setup().build().handleEvents(),this.settings.demoOptions&&this.initPageChanger()},setup:function(){return this.settings.demoOptions=this.element.attr("data-demo-options")?"true"===this.element.attr("data-demo-options"):this.settings.demoOptions,this.settings.useBackButton=this.element.attr("data-use-backbutton")?"true"===this.element.attr("data-use-backbutton"):this.settings.useBackButton,this.settings.useBreadcrumb=this.element.attr("data-use-breadcrumb")?"true"===this.element.attr("data-use-breadcrumb"):this.settings.useBreadcrumb,this.settings.useAlternate=this.element.attr("data-use-alternate")?"true"===this.element.attr("data-use-alternate"):this.settings.useAlternate,this.settings.tabs=$.isArray(this.settings.tabs)?this.settings.tabs:null,this.settings.wizardTicks=$.isArray(this.settings.wizardTicks)?this.settings.wizardTicks:null,this.titleText=this.element.find(".title > h1"),this.levelsDeep=[],this.levelsDeep.push("".concat(this.titleText.text())),this},build:function(){var t=!1,e=this.element.find(".toolbar, .flex-toolbar");if(!e.length)return this;e.is(".flex-toolbar")&&(t=!0),this.toolbarElem=e;var i=this.settings.toolbarSettings;if(this.toolbarElem[t?"toolbarflex":"toolbar"](i),!this.hasTitleButton&&this.levelsDeep.length>1&&this.buildTitleButton(),this.settings.tabs&&this.settings.tabs.length&&this.buildTabs(),this.settings.wizardTicks&&this.settings.wizardTicks.length&&this.buildWizard(),this.settings.usePopupmenu&&this.buildPopupmenu(),this.settings.addScrollClass){var n=$(this.element),s=$(this.element).next(".scrollable"),a=1===s.length?s:$(window),o=this.settings.scrollThreshold?this.settings.scrollThreshold:15;a.on("scroll.header",(function(){this.scrollTop>o?n.addClass("is-scrolled-down"):n.removeClass("is-scrolled-down")})),a.scrollTop()>o&&n.addClass("is-scrolled-down")}return this},personalizationColors:function(t){return Object.values(t).reduce((function(t,e){var i=e.id;return t[i]=i,t}),{})},removeClasses:function(t,e){if(!t||!e)return!1;var i=t.classList;if(i)if("string"==typeof e)i.remove(e);else{if(!Array.isArray(e))throw new TypeError("Classes parameter must be a string or an array of strings.");e.forEach((function(t){return i.remove(t)}))}return!0},buildTitleButton:function(){if(this.levelsDeep.length>1&&!this.hasTitleButton&&!this.titleButton.length){this.toolbarAPI.teardown();var t=$('<button class="btn-icon back-button" type="button">\n        <span class="audible">'.concat(H.translate("Drillup"),'</span>\n        <svg class="icon" focusable="false" aria-hidden="true" role="presentation">\n          <use href="#icon-arrow-left"></use>\n        </svg>\n      </button>')),e=this.toolbarElem.find(".title");if(this.settings.useFlexToolbar){var i=e.prev(".toolbar-section");i.length||(i=$('<div class="toolbar-section"></div>').insertBefore(e)),t.prependTo(i)}else t.prependTo(e);this.toolbarAPI.init()}this.toolbarElem.addClass("has-title-button"),this.titleButton.find(".icon.app-header").addClass("go-back");var n=$("#application-menu").data("applicationmenu");n?n.modifyTriggers([this.titleButton],null,!0):$("#application-menu").applicationmenu({triggers:[this.titleButton]}),this.handleTitleButtonEvents()},getCurrentBreadcrumbData:function(){var t=this;function e(){var e=this.index,i=t.breadcrumbAPI.list;if(!this.current)if(0!==e){var n,s=i.childNodes.length-1;if(e<s)for(n=s-e;n>0;)t.drillup(),t.breadcrumbAPI.remove(s,!1,!0),n-=1,s-=1;t.breadcrumbAPI.render()}else t.reset()}return this.levelsDeep.map((function(i,n){var s="header-breadcrumb-".concat(i.replace(" ","-").toLowerCase());return{callback:e,content:i,current:n+1===t.levelsDeep.length,id:s}}))},buildBreadcrumb:function(){var t="has-breadcrumb",e="default";this.settings.useAlternate&&(t="has-alternate-breadcrumb",e="alternate"),this.element.addClass(t),this.breadcrumb=this.element.find(".breadcrumb"),this.breadcrumb.length||(this.breadcrumb=$('<nav class="breadcrumb hidden" role="navigation"></nav>').appendTo(this.element),this.breadcrumb.breadcrumb({breadcrumbs:this.getCurrentBreadcrumbData(),style:e}),this.breadcrumbAPI=this.breadcrumb.data("breadcrumb")),this.adjustBreadcrumb()},adjustBreadcrumb:function(){this.breadcrumbAPI&&this.breadcrumbAPI.updated({breadcrumbs:this.getCurrentBreadcrumbData()})},buildTabs:function(){if(this.tabsContainer=this.element.find(".tab-container"),!this.tabsContainer.length){this.tabsContainer=$('<div class="tab-container"></div>').appendTo(this.element);var t=$('<ul class="tab-list" role="tablist"></ul>').appendTo(this.tabsContainer);$('<li class="tab"><a href="#header-tabs-home" role="tab">IDS Enterprise Controls | Patterns</a></li>').appendTo(t),$('<li class="tab"><a href="#header-tabs-level-1" role="tab">Level 1 Detail</a></li>').appendTo(t),$('<li class="tab"><a href="#header-tabs-level-2" role="tab">Level 2 Detail</a></li>').appendTo(t)}this.element.addClass(this.settings.useAlternate?"has-alternate-tabs":"has-tabs"),this.tabsContainer[this.settings.useAlternate?"addClass":"removeClass"]("alternate"),$("#header-tabs-level-1").removeAttr("style"),$("#header-tabs-level-2").removeAttr("style"),this.tabsContainer.tabs({containerElement:"#maincontent"})},buildWizard:function(){if(this.element.addClass("has-wizard"),this.wizard=this.element.find(".wizard"),!this.wizard.length){this.wizard=$('<div class="wizard"></div>').appendTo(this.element);var t=$('<div class="wizard-header"></div>').appendTo(this.wizard),e=$('<div class="bar"></div>').appendTo(t);$('<div class="completed-range"></div>').appendTo(e),$('<a href="#" class="tick current"><span class="label">Context Apps</span></a>').appendTo(e),$('<a href="#" class="tick"><span class="label">Utility Apps</span></a>').appendTo(e),$('<a href="#" class="tick"><span class="label">Inbound Configuration</span></a>').appendTo(e),$('<a href="#" class="tick"><span class="label">OID Mapping</span></a>').appendTo(e)}this.wizard[this.settings.useAlternate?"addClass":"removeClass"]("alternate"),this.wizard.wizard()},buildPopupmenu:function(){this.toolbarAPI.teardown();var t=this.toolbarElem.children(".title");if(this.titlePopup=t.find(".btn-menu"),!this.titlePopup.length){var e=t.find("h1");e.wrap('<button id="header-menu" type="button" class="btn-menu"></button>'),this.titlePopup=e.parent(".btn-menu")}this.titlePopupMenu=this.titlePopup.next(".popupmenu"),this.titlePopupMenu.length||(this.titlePopupMenu=$('<ul class="popupmenu is-selectable"></ul>').insertAfter(this.titlePopup),$('<li class="is-checked"><a href="#">Page One Title</a></li><li><a href="#">Page Two Title</a></li><li><a href="#">Page Three Title</a></li><li class="is-disabled"><a href="#">Page Four Title</a></li><li><a href="#">Page Five Title</a></li>').appendTo(this.titlePopupMenu)),this.titlePopupMenu.addClass("is-selectable");var i=this.titlePopupMenu.children(".is-checked").first().text();i||(i=this.titlePopupMenu.children().first().text()),this.titlePopup.children("h1").text(i),this.titlePopup.button().popupmenu(),this.element.addClass("has-popupmenu-title"),this.toolbarAPI.init()},handleEvents:function(){var t=this,e=this;return this.element.on("updated.".concat(Jo),(function(t,i){e.updated(i)})).on("reset.".concat(Jo),(function(){e.reset()})).on("drilldown.".concat(Jo),(function(t,i){e.drilldown(i)})).on("drillup.".concat(Jo),(function(t,i){e.drillup(i)})),$("html").on("themechanged.".concat(Jo),(function(){t.updatePageChanger()})),this.handleTitleButtonEvents(),this.titlePopup&&this.titlePopup.length&&this.titlePopup.on("selected.".concat(Jo),(function(t,e){var i;i=e instanceof $?e.text():$(e.element).text(),$(this).children("h1").text(i)})),this},handleTitleButtonEvents:function(){var t=this;this.titleButton&&this.titleButton.length&&this.titleButton.bindFirst("click.".concat(Jo),(function(e){t.levelsDeep.length>1&&(e.stopImmediatePropagation(),t.drillup(),e.returnValue=!1)}))},initPageChanger:function(){if(this.changer=this.element.find(".page-changer"),this.changer.length){var t=this.changer.data("popupmenu").menu,e=t.find("li.personalization-colors");if(e.length>0){var i=ft.personalizationColors(),n=e.parent().hasClass("popupmenu")?"":'<li class="heading" role="presentation">Personalization</li>';Object.keys(i).forEach((function(t){return n+='<li class="is-selectable'.concat("Default"===i[t].name?" is-checked is-default":"",'"><a href="#" data-rgbcolor="').concat(i[t].value,'">').concat(i[t].name,"</a></li>"),t})),e.replaceWith(n)}this.changer.on("selected.header",(function(e,i){e.preventDefault(),void 0!==i&&!(i instanceof $)&&i.element instanceof HTMLElement&&(i=$(i.element));var n=i.attr("data-theme-name"),s=i.attr("data-theme-mode");if(n||s){var a=t.find(".is-checked a[data-theme-name]").attr("data-theme-name"),o=t.find(".is-checked a[data-theme-mode]").attr("data-theme-mode");a&&o&&wt.setTheme("".concat(a,"-").concat(o))}else{var r=i.attr("data-theme");if(r)wt.setTheme(r);else if(i.parent().hasClass("is-default"))wt.setColorsToDefault();else{var l=i.attr("data-rgbcolor");wt.setColors(l)}}}));var s=ft.currentTheme;if("theme-new-light"!==s.id||"theme-uplift-light"!==s.id){var a=s.id.split("-");$("body").find(".popupmenu [data-theme-name]").parent().removeClass("is-checked"),$("body").find('.popupmenu [data-theme-name="'.concat(a[0],"-").concat(a[1],'"]')).parent().addClass("is-checked"),$("body").find(".popupmenu [data-theme-mode]").parent().removeClass("is-checked"),$("body").find('.popupmenu [data-theme-mode="'.concat(a[2],'"]')).parent().addClass("is-checked")}if(wt.settings.colors){var o="object"===l(wt.settings.colors)?wt.settings.colors.header:wt.settings.colors;o=o.replace("#",""),$("body").find(".popupmenu [data-rgbcolor]").parent().removeClass("is-checked"),$("body").find('.popupmenu [data-rgbcolor="#'.concat(o,'"]')).parent().addClass("is-checked")}}},updatePageChanger:function(){if(this.change&&0!==this.changer.length)for(var t=this.changer.data("popupmenu").menu.find("[data-rgbcolor]"),e=ft.personalizationColors(),i=Object.keys(e),n=0;n<t.length;n++)t[n].setAttribute("data-rgbcolor",e[i[n]].value)},drilldown:function(t){this.element.addClass("is-drilldown"),this.levelsDeep.push(t.toString()),this.titleText.text(this.levelsDeep[this.levelsDeep.length-1]),this.settings.useBackButton&&this.buildTitleButton(),this.settings.useBreadcrumb&&(this.breadcrumb&&this.breadcrumb.length?this.adjustBreadcrumb():(this.buildBreadcrumb(),this.breadcrumb.css({display:"block",height:"auto"}).removeClass("hidden")))},drillup:function(t){var e;if(this.element.removeClass("is-drilldown"),this.levelsDeep.length>1&&(this.levelsDeep.pop(),e=this.levelsDeep[this.levelsDeep.length-1]),void 0!==t&&(e=t),this.levelsDeep.length>1)return this.settings.useBreadcrumb&&this.adjustBreadcrumb(),void this.titleText.text(e);e=this.levelsDeep[0],this.settings.useBackButton&&this.removeButton(),this.settings.useBreadcrumb&&this.removeBreadcrumb(),this.settings.usePopupmenu&&this.removePopupmenu(),this.titleText.text(e),this.element.trigger("drillTop")},reset:function(){for(;this.levelsDeep.length>1;)this.levelsDeep.pop();return this.titleText.text(this.levelsDeep[0]),this.removeBreadcrumb(),this.removeTabs(),this.removeWizard(),this.removePopupmenu(),this.removeButton(),this.element.trigger("afterreset"),this},removeButton:function(){if(this.hasTitleButton)this.titleButton.find(".icon.app-header").removeClass("go-back");else if(this.titleButton&&this.titleButton.length){this.toolbarAPI.teardown();var t=$("#application-menu").data("applicationmenu");t&&t.modifyTriggers([this.titleButton],!0,!0),this.titleButton.off("click.".concat(Jo)).remove(),this.titleButton=$(),this.toolbarAPI.init()}},removeBackButton:function(){this.titleButton.length&&this.element.find(".go-back").removeClass("go-back")},removeBreadcrumb:function(){if(this.breadcrumb&&this.breadcrumb.length){var t,e=this,i=$.fn.transitionEndName();e.element.removeClass("has-breadcrumb").removeClass("has-alternate-breadcrumb"),this.breadcrumb.is(":not(:hidden)")?(this.element.one("".concat(i,".breadcrumb-header"),n),t=setTimeout(n,300)):n()}function n(){t&&(clearTimeout(t),t=null),e.element.off("".concat(i,".breadcrumb-header")),e.breadcrumbAPI.destroy(),e.breadcrumb.remove(),delete e.breadcrumbAPI,delete e.breadcrumb}},removeTabs:function(){if(this.tabsContainer&&this.tabsContainer.length){var t,e=this,i=$.fn.transitionEndName();this.element.removeClass("has-tabs").removeClass("has-alternate-tabs"),this.tabsContainer.is(":not(:hidden)")?(this.element.one("".concat(i,".tabs-header"),n),t=setTimeout(n,300)):n()}function n(){t&&(clearTimeout(t),t=null),e.element.off("".concat(i,".tabs-header")),e.tabsContainer.data("tabs").destroy(),e.tabsContainer.remove(),e.tabsContainer=null,$("#header-tabs-level-1").css("display","none"),$("#header-tabs-level-2").css("display","none")}},removeWizard:function(){if(this.wizard&&this.wizard.length){var t,e=this,i=$.fn.transitionEndName();this.element.removeClass("has-wizard"),this.wizard.is(":not(:hidden)")?(this.element.one("".concat(i,".wizard-header"),n),t=setTimeout(n,300)):n()}function n(){t&&(clearTimeout(t),t=null),e.element.off("".concat(i,".wizard-header")),e.wizard.data("wizard").destroy(),e.wizard.remove(),e.wizard=null}},removePopupmenu:function(){this.titlePopup&&this.titlePopup.length&&(this.toolbarAPI.teardown(),this.titlePopup.data("popupmenu")&&this.titlePopup.data("popupmenu").destroy(),this.titlePopup.data("popupmenu")&&this.titlePopup.data("button").destroy(),this.titlePopupMenu.remove(),this.titlePopup.children("h1").detach().insertBefore(this.titlePopup),this.titlePopup.remove(),this.titlePopup=void 0,this.titlePopupMenu=void 0,this.element.removeClass("has-popupmenu-title"),this.toolbarAPI.init())},unbind:function(){return this.titleButton&&this.titleButton.length&&this.titleButton.off("click.".concat(Jo)),this.titlePopup&&this.titlePopup.length&&this.titlePopup.off("updated.".concat(Jo)),this.element.off(["updated.".concat(Jo),"reset.".concat(Jo),"drilldown.".concat(Jo),"drillup.".concat(Jo)].join(" ")),$("html").off("themechanged.".concat(Jo)),this},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.reset().unbind().init()},destroy:function(){if(this.unbind(),this.hasTitleButton&&this.toolbarElem.removeClass("has-title-button"),this.changer){var t=this.changer.data("popupmenu");t&&"function"==typeof t.destroy&&t.destroy(),this.changer.remove(),delete this.changer}$.removeData(this.element[0],Jo)}},$.fn.header=function(t){return this.each((function(){var e=$.data(this,Jo);e?e.updated(t):e=$.data(this,Jo,new tr(this,t))}))};var er="multitabs",ir={tabContainers:[],sortable:!1},nr=["primary","secondary","tertiary"];function sr(t,e){return this.element=$(t),this.settings=E.mergeSettings(this.element[0],e,ir),this.tabContainers={},this.init()}sr.prototype={init:function(){var t=this,e=[];e=this.settings.tabContainers instanceof Array&&this.settings.tabContainers.length?this.settings.tabContainers:this.element.find(".tab-container").filter((function(){return!$(this).parents(".tab-panel-container").length})),$(e).each((function(){t.setupTabsInstance($(this))}))},setupTabsInstance:function(t){if(t instanceof $){var e=this,i=!1;if(nr.forEach((function(n){if(!i&&!Object.prototype.hasOwnProperty.call(e.tabContainers,n)){var s=t.data("tabs");s||(t.tabs(),s=t.data("tabs")),s.settings.sortable=e.settings.sortable,s.multitabsID=n,s.createSortable(),t.attr("data-multitabs",n),e.tabContainers[n]=t,i=!0}})),e.settings.sortable)$(t.siblings(".tab-panel-container")).prepend('<div class="tab-overlay"><div class="overlay-left"></div><div class="overlay-right"></div></div>');if(!i)throw new Error("all tab-container slots in MultiTabs component are taken, so a new tabs container was not invoked and stored")}},add:function(t,e,i,n){return(t=this.checkForValidTabContainer(t)).data("tabs").add(e,i,n)},remove:function(t,e,i){return(t=this.checkForValidTabContainer(t)).data("tabs").remove(e,i)},move:function(t,e,i){if(!t||!e)throw new Error("can't move a tab without both a tabId and a targetTabContainerName");var n,s,a;if(this.getFilterableTabContainers().each((function(){var e=$(this).data("tabs"),i=e.getTab(null,t);null!==i&&(a=e.multitabsID,n=i.clone(),s=e.getPanel(t).children())})),!n)throw new Error('No tab markup was found in any Multitabs container for href "'.concat(t,'"'));this.remove(a,t),this.add(e,t,{name:n.children("a").text().trim(),content:s,doActivate:i})},getTabContainer:function(t){var e,i=this;return nr.forEach((function(n){!e&&i.tabContainers[n]&&t===n&&(e=i.tabContainers[n])})),e},getFilterableTabContainers:function(){var t=$();return this.performOnAllContainers((function(e,i){t=t.add(i)})),t},checkForValidTabContainer:function(t){if(!t)throw new Error("Need to have a tabContainer defined to hide a tabs instance");if("string"==typeof t&&(t=this.getTabContainer(t)),!(t instanceof $&&t.length))throw new Error("No matching tabContainer could be found and hidden.");return t},performOnAllContainers:function(t,e){var i=this;Array.isArray(e)||(e=e?[e]:[]),nr.forEach((function(n){var s=i.tabContainers[n],a=[];s&&(a.push(n,s),a=a.concat(e),t.apply(this,a))}))},adjustModuleTabContainers:function(){this.performOnAllContainers((function(t,e){var i=e.data("tabs");i.isModuleTabs()&&i.adjustModuleTabs()}))},hideTabsInstance:function(t){(t=this.checkForValidTabContainer(t)).parent(".multitabs-section").addClass("hidden"),this.adjustModuleTabContainers()},showTabsInstance:function(t){(t=this.checkForValidTabContainer(t)).parent(".multitabs-section").removeClass("hidden"),this.adjustModuleTabContainers()},destroyTabsInstance:function(t){if(t instanceof $&&t.data("tabs")&&"function"==typeof t.data("tabs").destroy){var e=this,i=!1;if(nr.forEach((function(n){if(!i&&e.tabContainers[n].is(t)){var s=t.data("tabs");delete s.multitabsID,t.removeAttr("data-multitabs"),s.destroy(),delete e.tabContainers[n],i=!0}})),!i)throw new Error("could not destroy a tabContainer because it was not associated with this Multitabs instance")}},isHidden:function(t){return(t=this.checkForValidTabContainer(t)).parent(".multitabs-section").is(".hidden")},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings))},teardown:function(){var t=this;return Object.keys(this.tabContainers).forEach((function(e){t.destroyTabsInstance(e)})),this},destroy:function(){this.teardown(),$.removeData(this.element[0],er)}},$.fn.multitabs=function(t){return this.each((function(){var e=$.data(this,er);e?e.updated(t):e=$.data(this,er,new sr(this,t))}))};var ar="listdetail",or={backElement:void 0,edgeBleed:!1,edgeBleedBreakpoint:"phone",listElement:void 0,listCloseElement:void 0,toggleElement:void 0,detailElement:void 0},rr=["accordion","listview"];function lr(t){var e;if(t instanceof HTMLElement)return Object.keys($(t).data()).forEach((function(t){rr.indexOf(t)>-1&&(e=t)})),e}function cr(t,e){t instanceof HTMLElement&&(this.element=t,this.settings=E.mergeSettings(this.element,e,or),this.init())}cr.prototype={init:function(){var t=this;if(this.setInternalElementReference("listElement",(function(e){return(i=t.settings[e])instanceof HTMLElement&&void 0!==lr(i);var i})),this.setInternalElementReference("detailElement"),this.setInternalElementReference("backElement"),this.setInternalElementReference("listCloseElement"),this.setInternalElementReference("toggleElement"),this.showDetail=!1,this.element.classList.contains("show-detail")&&(this.showDetail=!0),this.showSidebar=!0,this.listElement){this.listComponentType=lr(this.listElement);var e=this.getListAPI();e&&(e.isControllingDetails=!0)}if(this.backElement){this.backElement.classList.add("list-detail-back-button");var i=this.backElement.querySelector(".icon");i&&(this.backElementIcon=i),this.listContainsBackElement&&(this.listContainsBackElement=this.listElement.contains(this.backElement)),this.detailContainsBackElement=this.detailElement.contains(this.backElement)}if(this.toggleElement){this.mainElement=this.element.querySelector(".main"),this.sidebarElement=this.element.querySelector(".sidebar"),this.toggleElement.classList.add("list-detail-toggle-button");var n=this.toggleElement.querySelector(".icon");n&&(this.toggleElementIcon=n,this.toggleElementIcon.classList.add("go-back"))}var s=this.element.querySelectorAll(".list-detail");s&&(this.childrenListDetailElements=s),this.settings.edgeBleed?(this.edgeBleed=!0,this.element.classList.add("bleeding-edge__".concat(this.settings.edgeBleedBreakpoint))):(this.edgeBleed=!1,this.element.classList.remove("bleeding-edge__phone","bleeding-edge__tablet")),this.setBreakpointChecks(),this.handleEvents()},setBreakpointChecks:function(){this.abovePhoneBreakpoint=m.isAbove("phone-to-tablet"),this.aboveTabletBreakpoint=m.isAbove("desktop")},setInternalElementReference:function(t,e){if("string"==typeof this.settings[t]&&this.settings[t].length){var i=document.querySelector(this.settings[t]);i&&(this.settings[t]=i)}e="function"!=typeof e||e(t),this.settings[t]instanceof HTMLElement&&e&&(this[t]=this.settings[t])},handleEvents:function(){var t=this;$(this.element).on("drilldown.".concat(ar),(function(e,i){e.stopPropagation(),t.drilldown(i,e.target)})).on("drillup.".concat(ar),(function(e){e.stopPropagation(),t.drillup()})),this.backElement&&this.backElement.addEventListener("click",this.handleBackClick.bind(this)),this.toggleElement&&this.toggleElement.addEventListener("click",this.handleToggleClick.bind(this)),this.listCloseElement&&this.listCloseElement.addEventListener("click",this.handleClose.bind(this)),$("body").off("resize.".concat(ar)).on("resize.".concat(ar),(function(){t.handleResize()}))},drilldown:function(t,e){this.showDetail||e&&e.innerHTML!==this.listElement.innerHTML||(this.showDetail=!0,this.element.classList.add("show-detail"),this.addBackElementIconContext(),this.isAboveBreakpoint()||this.getListAPI().disable(),this.detailArea&&$(this.detailElement).triggerHandler("activatescreen",[t]))},drillup:function(){this.showDetail&&(this.removeBackElementIconContext(),this.getListAPI().enable(),this.showDetail=!1,this.element.classList.remove("show-detail"))},addBackElementIconContext:function(){this.showDetail&&this.backElementIcon&&!this.backElementIcon.classList.contains("go-back")&&this.backElementIcon.classList.add("go-back")},removeBackElementIconContext:function(){this.backElementIcon&&this.backElementIcon.classList.contains("go-back")&&this.backElementIcon.classList.remove("go-back")},toggleSidebar:function(t){t?(this.sidebarElement.classList.add("hidden"),this.mainElement.style.width="100%",this.toggleElementIcon.classList.remove("go-back")):(this.sidebarElement.classList.remove("hidden"),this.mainElement.style.width="",this.toggleElementIcon.classList.add("go-back")),this.showSidebar=!t},handleBackClick:function(t){function e(){t.stopPropagation(),t.stopImmediatePropagation(),t.preventDefault()}if(!this.showDetail){if(this.edgeBleed){var i=this.getCurrentSelectedListItem();if(i)return this.drilldown(i),e(),!1}return!0}return this.drillup(),this.childrenListDetailElements&&this.childrenListDetailElements.forEach((function(t){var e=$(t).data(ar);e&&"function"==typeof e.drillup&&e.drillup()})),e(),!1},handleToggleClick:function(){this.toggleSidebar(this.showSidebar)},handleClose:function(){if(!this.showDetail){var t=this.getCurrentSelectedListItem();this.drilldown(t)}},handleResize:function(){this.setBreakpointChecks();var t=this.getListAPI();this.isAboveBreakpoint()?this.edgeBleed&&!this.showDetail&&this.listElement.classList.contains("is-disabled")&&t.enable():this.edgeBleed&&this.showDetail&&!this.listElement.classList.contains("is-disabled")&&t.disable(),this.isAboveBreakpoint()?this.removeBackElementIconContext():(this.addBackElementIconContext(),this.toggleElement&&this.toggleSidebar(!1)),m.isAbove("desktop")&&void 0!==t&&t.enable()},isAboveBreakpoint:function(){var t=this.settings.edgeBleedBreakpoint;return this["above".concat(M.capitalize(t),"Breakpoint")]},getListAPI:function(){if(this.listElement)return $(this.listElement).data(this.listComponentType)},getCurrentSelectedListItem:function(){if(!this.listElement)return"";var t,e=this.getListAPI();return this.listComponentType,t=e.getSelected(),t=f.convertToHTMLElement(t)},updated:function(t){t&&(this.settings=E.mergeSettings(this.element[0],t,this.settings)),this.teardown(),this.init()},teardown:function(){if($("body").off("resize.".concat(ar)),$(this.element).off("drilldown.".concat(ar," drillup.").concat(ar)),this.backElement&&this.backElement.removeEventListener("click",this.handleBackClick.bind(this)),this.listCloseElement&&this.listCloseElement.removeEventListener("click",this.handleClose.bind(this)),this.listComponentType){var t=this.getListAPI();t&&delete t.isControllingDetails}delete this.listElement,delete this.detailElement,delete this.backElement,delete this.backElementIcon,delete this.mainElement,delete this.sidebarElement,delete this.toggleElement,delete this.toggleElementIcon,delete this.edgeBleed,delete this.listContainsBackElement,delete this.detailContainsBackElement,delete this.childrenListDetailElements,delete this.showDetail,delete this.showSidebar},destroy:function(){this.teardown(),$.removeData(this.element,ar)}},$.fn.listdetail=function(t){return this.each((function(){var e=$.data(this,ar);e?e.updated(t):(e=$.data(this,ar,new cr(this,t))).destroy=function(){this.teardown(),$.removeData(this.element,ar),$.removeData(this.element,"options")}}))};var dr="stepprocess",hr={linearProgression:!1,folderIconOpen:"caret-up",folderIconClosed:"caret-down",stepList:"#step-list",stepLi:".js-step",stepLink:".js-step-link",stepFolder:".js-step-folder",btnPrev:".js-step-link-prev",btnNext:".js-step-link-next",btnSaveClose:".js-btn-save-changes",beforeSelectStep:null};function ur(t,e){this.settings=E.mergeSettings(t,e,hr),this.element=$(t),this.init(),W("Stepprocess")}ur.prototype={init:function(){this.stepListJq=$(this.settings.stepList),this.initStepprocess(),this.handleKeys(),this.setupEvents(),this.focusFirst()},initStepprocess:function(){for(var t=this.stepListJq.find(this.settings.stepLi),e=0,i=t.length;e<i;e++)this.decorateNode(t[e]);var n=$("".concat(this.settings.stepLi,".is-selected")).find(this.settings.stepLink);this.selectStep(n)},decorateNode:function(t){var e=$(t),i=e.children(this.settings.stepLink),n=e.children(this.settings.stepFolder),s=i.hasClass("is-disabled"),a=n.hasClass("is-open");if(s&&i.attr("aria-disabled","true"),n.length){if(e.addClass("folder"),n.attr("role","group"),s&&(n.addClass("disabled"),a))for(var o=n.children(),r=0,l=o.length;r<l;r++)$(o[r]).find(this.settings.stepLink).addClass("is-disabled").attr("aria-disabled","true");i.attr("aria-expanded",a)}var c=i.parentsUntil(this.stepListJq,"ul").length-1,d=e.index(),h=e.siblings().addBack().length;i.attr({role:"stepitem",tabindex:"-1","aria-selected":"false","aria-level":c+1,"aria-posinset":d+1,"aria-setsize":h,"aria-disabled":s}).addClass("hide-focus").hideFocus()},focusFirst:function(){this.stepListJq.find("".concat(this.settings.stepLi,":first")).attr("tabindex","0")},folderClose:function(t){var e=this,i=$(t),n=i.children(this.settings.stepLink),s=i.children(this.settings.stepFolder),a=n.closest(".folder").removeClass("is-open").end().find("svg.icon-tree");this.setIcon(a,this.settings.folderIconClosed),this.isAnimating=!0,s.one("animateclosedcomplete",(function(){s.removeClass("is-open"),e.isAnimating=!1})).animateClosed(),n.attr("aria-expanded","false")},folderOpen:function(t){var e=this,i=$(t);if(!this.isOpen(i)){var n=i.children(this.settings.stepLink),s=i.children(this.settings.stepFolder);i.addClass("is-open"),n.attr("aria-expanded","true");var a=n.find("svg.icon-tree");this.setIcon(a,this.settings.folderIconOpen),this.isAnimating=!0,s.one("animateopencomplete",(function(){e.isAnimating=!1})).addClass("is-open").css("height",0).animateOpen()}},folderToggle:function(t){var e=t.closest(this.settings.stepLi);if(this.isFolder(e)){var i=e.children(this.settings.stepFolder);this.isOpen(i)?this.folderClose(e):this.folderOpen(e)}},getSelectedStep:function(){return $("".concat(this.settings.stepLi,".is-selected"),this.stepListJq)},getNextNode:function(t){var e=this.settings,i=t.parent().next().find("".concat(e.stepLink,":first"));if(t.next().is(e.stepFolder)&&t.next().hasClass("is-open")&&(i=t.next().find("".concat(e.stepLink,":first"))),i.hasClass("is-disabled")&&(i=this.getNextNode(i)),0===i.length)for(var n=0,s=t;n<2&&!(i=(s=s.parent().closest(".folder")).next().find("".concat(e.stepLink,":first"))).length;n++);return i},getNextStep:function(t){var e=t.children(this.settings.stepLink),i=t.next(this.settings.stepFolder),n=this.getNextNode(e),s=n.next(this.settings.stepFolder),a=null,o=null;return(a=i.length?(o=i).children().length?o.find(this.settings.stepLink).first():n:s.length&&(o=s).children().length?o.find(this.settings.stepLink).first():n).hasClass("is-disabled")&&(a=this.getNextStep(a.parent())),a},getPreviousNode:function(t){var e=this.settings,i=t.parent().prev().find("".concat(e.stepLink,":first")),n=i.closest(e.stepLi);return n.is(".folder.is-open")&&n.find("ul.is-open a").length&&!n.find("ul.is-disabled").length&&(i=n.find("ul.is-open ".concat(e.stepLink,":last"))),i.hasClass("is-disabled")&&this.getPreviousNode(i),0===i.length&&(i=t.closest(e.stepFolder).prev(e.stepLink)),i},getPreviousStep:function(t){var e=this.settings,i=t.children(e.stepLink),n=this.getPreviousNode(i),s=n.closest(e.stepLi),a=n;if(this.isFolder(s))if(this.isOpen(s))a=this.getPreviousNode(n);else{var o=s.children(e.stepFolder);o.children().length&&(a=o.find(e.stepLink).last())}return a.hasClass("is-disabled")&&(a=this.getPreviousStep(a.parent())),a},goToNextStep:function(){var t=this,e=this.getSelectedStep(),i=t.getNextStep(e);if(i.length)t.selectStep(i,"next");else if("function"==typeof t.settings.beforeSelectStep){var n=t.settings.beforeSelectStep({isStepping:"next"});n.done&&"function"==typeof n.done?n.done((function(e,i){e&&i&&t.selectStepFinish(i)})):n&&t.selectStepFinish(i)}},goToPreviousStep:function(){var t=this.getSelectedStep(),e=this.getPreviousStep(t);e.length&&this.selectStep(e,"prev")},handleKeys:function(){var t=this,e=this.settings;this.stepListJq.on("focus.stepprocess",e.stepLink,(function(){var t=$(this);if(0===parseInt(t.attr("aria-level"),10)&&1===parseInt(t.attr("aria-posinset"),10)&&t.hasClass("is-disabled")){var e=$.Event("keydown.stepprocess");return e.keyCode=40,void t.trigger(e)}})),this.stepListJq.on("keydown.stepprocess",e.stepLink,(function(i){var n,s,a=i.charCode||i.keyCode,o=$(this);if(!t.isAnimating){if(40===a){var r=t.getNextNode(o);t.setFocus(r)}if(38===a){var l=t.getPreviousNode(o);t.setFocus(l)}if(32===i.keyCode&&o.trigger("click.stepprocess"),37===a)return H.isRTL()?o.next().hasClass("is-open")?(s=o.next().find("".concat(e.stepLink,":first")),t.setFocus(s)):t.folderToggle(o):o.next().hasClass("is-open")?t.folderToggle(o):(s=o.closest(".folder").find("".concat(e.stepLink,":first")),t.setFocus(s)),i.stopPropagation(),!1;if(39===a)return H.isRTL()?o.next().hasClass("is-open")?t.folderToggle(o):(n=o.closest(".folder").find("".concat(e.stepLink,":first")),t.setFocus(n)):o.next().hasClass("is-open")?(n=o.next().find("".concat(e.stepLink,":first")),t.setFocus(n)):(t.folderToggle(o),t.setFocus(o)),i.stopPropagation(),!1;36===a&&(n=t.stepListJq.find("".concat(e.stepLink,":first:visible")),t.setFocus(n)),35===a&&(n=t.stepListJq.find("".concat(e.stepLink,":last:visible")),t.setFocus(n))}})),this.stepListJq.on("keypress.stepprocess",e.stepLink,(function(i){var n=i.charCode||i.keyCode,s=$(this);if(n>=37&&n<=40||32===n)return i.stopPropagation(),!1;0!==i.which&&s.closest(e.stepLi).nextAll().find(".js-step-link:visible").each((function(){var e=$(this);if(e.text().substr(0,1).toLowerCase()===String.fromCharCode(i.which).toLowerCase())return t.setFocus(e),!1}))}))},isFolder:function(t){return $(t).hasClass("folder")},isInFolder:function(t){return $(t).closest(this.settings.stepFolder,this.stepListJq).length},isOpen:function(t){return $(t).hasClass("is-open")},setFocus:function(t){t.focus()},setIcon:function(t,e){var i=e.replace(/icon-|hide-focus|\s?/gi,"");t.changeIcon(i)},selectStep:function(t,e){var i,n=this;if(void 0===e&&(e="none"),"function"==typeof n.settings.beforeSelectStep){var s={stepLink:t,isStepping:e};(i=n.settings.beforeSelectStep(s)).done&&"function"==typeof i.done?i.done((function(i,s){i&&(s&&(t=s),n.selectStepFinish(t,e))})):i&&n.selectStepFinish(t,e)}else n.selectStepFinish(t,e)},selectStepFinish:function(t,e){var i=this,n=$(this.settings.stepLink,this.stepListJq),s=t.closest(this.settings.stepLi);if(this.isFolder(s)||(n.attr({tabindex:"-1","aria-selected":"false"}).parent().removeClass("is-selected"),t.attr({tabindex:"0","aria-selected":"true"}),s.addClass("is-selected")),this.isFolder(s))"none"===e?this.folderToggle(s):this.folderOpen(s);else{var a=s.closest(this.settings.stepFolder,this.stepListJq);a.length&&this.folderOpen(a.closest(this.settings.stepLi)),this.showStepPanel(t.attr("href"))}t.focus(),setTimeout((function(){i.element.triggerHandler("selected",t)}),0)},unSelectedNode:function(t){var e=$(this.settings.stepLink,this.stepListJq),i=$(t),n=i.children(this.settings.stepLink);e.attr("tabindex","-1"),n.attr("tabindex","0"),i.removeClass("is-selected"),n.attr("aria-selected","false")},setupEvents:function(){var t=this,e=this,i=this.settings;this.stepListJq.on("updated.stepprocess",(function(){t.initStepprocess()})).on("click.stepprocess","".concat(i.stepLink,":not(.is-clone)"),(function(t){if(t.preventDefault(),!i.linearProgression){var n=$(this);n.is(".is-disabled, .is-loading")||(e.selectStep(n),t.stopPropagation())}})),$(i.btnPrev).on("click",(function(i){i.preventDefault(),t.goToPreviousStep.call(e)})),$(i.btnNext).on("click",(function(i){i.preventDefault(),t.goToNextStep.call(e)})),$(i.contentScroll).scrollaction({scrollActionTarget:".main"}),$(i.stepListScroll).scrollaction({scrollActionTarget:".sidebar"}),$(".js-toggle-sidebar").click((function(e){e.preventDefault(),t.element.toggleClass("tablet-hide-steps").toggleClass("phone-hide-steps")}))},showStepPanel:function(t){$(".step-panel-active").removeClass("step-panel-active"),$(t).addClass("step-panel-active"),this.element.addClass("phone-hide-steps")},unbind:function(){return this.stepListJq.off("updated.stepprocess click.stepprocess focus.stepprocess keydown.stepprocess keypress.stepprocess").empty(),this},updated:function(t){return void 0!==t&&(this.settings=E.mergeSettings(this.element,t,hr)),this.unbind().init()},destroy:function(){this.unbind(),$.removeData(this.element[0],dr)}},$.fn.stepprocess=function(t){return this.each((function(){var e=$.data(this,dr);e?e.updated(t):e=$.data(this,dr,new ur(this,t))}))};var pr={locale:"en-US",language:null},gr=".no-init, [data-init]";function fr(t,e,i){return $(t)[e](i)}function mr(t,e){var i=t.find(e).not(gr);return t.filter(e).length&&(i=i.add(t)),i}var vr=[["texttranslations",'[data-translate="text"]',function(t,e,i){mr(t,i).each((function(t,e){var i=$(e);i.attr("data-translate-key",i.text()),i.text(H.translate(i.text(),!0))}))}],["texttranslations",'[data-translate="value"]',function(t,e,i){mr(t,i).each((function(t,e){var i=$(e);i.attr("data-translate-key",i.attr("value")),i.attr("value",H.translate(i.attr("value"),!0))}))}],["texttranslations",'[data-translate="placeholder"]',function(t,e,i){mr(t,i).each((function(t,e){var i=$(e);i.attr("data-translate-key",i.attr("placeholder")),i.attr("placeholder",H.translate(i.attr("placeholder"),!0))}))}],["applicationmenu","#application-menu",function(t,e,i){mr(t,i).each((function(e,i){$(i).applicationmenu({triggers:t.find(".application-menu-trigger")})}))}],["modulenav",".module-nav",function(t,e,i){mr(t,i).each((function(t,e){$(e).modulenav()}))}],["modulenavsettings",".module-nav-settings-btn",function(t,e,i){mr(t,i).each((function(t,e){$(e).modulenavsettings()}))}],["modulenavswitcher",".module-nav-switcher",function(t,e,i){mr(t,i).each((function(t,e){$(e).modulenavswitcher()}))}],["personalize",null,function(){$("html").personalize()}],["hyperlink"],["icon"],["radio",".radio-section input:radio.handle",function(t,e,i){mr(t,i).change((function(){if(this.checked){var t=$(this).closest(".option"),e=t.siblings(),i='button, select, input[type="text"]';$(i,t).removeAttr("disabled"),$(i,e).attr("disabled","disabled")}}))}],["splitter"],["tabs",".tab-container:not(.vertical)"],["verticaltabs",".tab-container.vertical"],["multitabs",".multitabs-container"],["dropdown","select.dropdown:not(.multiselect)"],["dropdown","select.dropdown-xs:not(.multiselect)"],["dropdown","select.dropdown-sm:not(.multiselect)"],["dropdown","select.dropdown-lg:not(.multiselect)"],["modal"],["revealText","input.input-password-reveal"],["slider",'input[type="range"], .slider'],["editor"],["tooltip","button[title], span[title], .hyperlink[title], .icon[title], input[title]"],["tree"],["rating"],["link"],["listbuilder"],["compositeform",".composite-form"],["progress",".progress-bar"],["mask","input[data-mask], .new-mask"],["autocomplete",".autocomplete:not([data-init])"],["cards",".card:not([data-init])"],["multiselect","select[multiple]:not(.dropdown), .multiselect:not([data-init])"],["button",[".btn",".btn-toggle",".btn-secondary",".btn-primary",".btn-modal-primary",".btn-tertiary",".btn-icon",".btn-actions",".btn-menu",".btn-split",".btn-secondary-border"].join(", "),function(t,e,i){mr(t,i).each((function(t,e){$(e).parents(".modal").length||$(e).button()}))}],["hideFocus","a.hide-focus, a.tick, .checkbox, .radio, .switch"],["circlepager"],["clearable",['[data-clearable="true"]'].join(", "),function(t,e,i){mr(t,i).each((function(t,e){var i=$(e).attr("data-tabbable");void 0===i?$(e).clearable({tabbable:!0}):$(e).clearable({tabbable:"true"===i})}))}],["textarea","textarea"],["spinbox"],["arrange"],["swaplist"],["colorpicker"],["datepicker"],["timepicker"],["tag"],["busyindicator",".busy, .busy-xs, .busy-sm"],["header"],["fileupload",'input.fileupload:not(.fileupload-background-transparent):not([type="text"])'],["fileuploadadvanced",".fileupload-advanced"],["fieldfilter",".field-filter"],["fieldoptions",".field-options"],["about"],["contextualactionpanel",".contextual-action-panel-trigger"],["expandablearea",".expandable-area"],["signin"],["homepage"],["lookup",".lookup:not([data-init])"],["wizard"],["popdown","[data-popdown]"],["stepchart",".step-chart"],["calendar",".calendar"],["calendartoolbar",".calendar-toolbar"],["monthview",".monthview"],["weekview",".week-view"],["listview"],["trackdirty",'[data-trackdirty="true"]'],["virtualscroll",".virtual-scroll"],["popupmenu","[data-popupmenu]:not(.btn-actions, .btn-filter, .btn-menu)",function(t,e,i){mr(t,i).each((function(t,i){var n=$(i),s=$.extend({},E.parseOptions(n)),a=n.attr("data-popupmenu");a&&(s.menuId=a),fr(n,e,s)}))}],["popupmenu",".btn-actions, .btn-filter, .btn-menu, .module-nav-settings-btn",function(t,e,i){mr(t,i).each((function(t,i){var n=$(i);n.parents(".toolbar, .flex-toolbar").length>0||fr(n,e)}))}],["popover","[data-popover]",function(t,e,i){mr(t,i).each((function(){var t=E.parseOptions(this),e=$(this),i=e.attr("data-trigger"),n=e.attr("data-title"),s=e.attr("data-placement");$.isEmptyObject(t)?e.popover({content:$("#".concat(e.attr("data-popover"))),popover:!0,trigger:i||"click",title:n||void 0,placement:s||"right",extraClass:t.extraClass||void 0}):e.popover({content:$(t.content),popover:!0,trigger:t.trigger||"click",title:t.title||void 0,placement:t.placement||"right",extraClass:t.extraClass||void 0})}))}],["searchfield",".searchfield",function(t,e,i){var n=mr(t,i),s=n.filter((function(){return $(this).parents(".toolbar").length}));(n=n.not(s)).each((function(){fr(this,"searchfield")}))}],["toolbar",".toolbar",function(t,e,i){mr(t,i).each((function(e,i){var n=$(i);n.parents(".header, .contextual-action-panel .modal-header").length&&!t.is(".toolbar")||fr(n,"toolbar")}))}],["toolbarflex",".flex-toolbar"],["accordion",".accordion",function(t,e,i){mr(t,i).each((function(t,e){var i=$(e);i.parents(".application-menu").length||fr(i,"accordion")}))}],["taglist",".tag-list"],["breadcrumb",".breadcrumb"],["swipeaction",".swipe-action"],["formcompact",".form-compact-container",function(t,e,i){mr(t,i).each((function(t,e){fr($(e),"formcompact")}))}],["listdetail",".list-detail",function(t,e,i){mr(t,i).each((function(t,e){fr($(e),"listdetail")}))}],["breadcrumblabels",".breadcrumb ol",function(t,e,i){mr(t,i).attr("aria-label",H.translate("Breadcrumb"))}],["validate","[data-validate]",function(t,e,i){mr(t,i).closest("form, body").validate()}],["validate",'form[data-validate-on="submit"]']];function br(t,e,i,n){($.fn[e]||"texttranslations"===e)&&(void 0===i&&(i=".".concat(e)),"function"!=typeof n?mr(t,i).each((function(){var t=$(this);t.is(gr)||t.parents(gr).length||fr(this,e)})):n(t,e,i))}function yr(t,e){var i=e;"string"==typeof e&&(i={locale:e}),this.element=$(t),H.currentLocale&&H.currentLocale.name&&!e&&(i={locale:H.currentLocale.name}),H.currentLanguage&&H.currentLanguage.name&&H.currentLocale.name.substr(0,2)!==H.currentLanguage.name&&!e&&(i.language=H.currentLanguage.name),this.settings=E.mergeSettings(this.element[0],i,pr),this.init()}yr.prototype={init:function(){var t=this,e=this;return H.set(this.settings.locale).done((function(){t.settings.language?H.setLanguage(t.settings.language).done((function(){e.initAll()})):e.initAll()})),this},initAll:function(){return this.element.each((function(){for(var t=0;t<vr.length;t++)br.apply(void 0,[$(this)].concat(u(vr[t])))})),this.element.triggerHandler("initialized",{locale:H.currentLocale}),$.fn.validate&&this.element.validate(),this}},$.fn.initialize=function(t){return this.each((function(){return new yr(this,t)}))};var wr="longpress",xr={delay:400,mouseEvents:!1};function Cr(t){return this.settings=E.mergeSettings(this.element,t,xr),this.init()}Cr.prototype={timer:null,init:function(){var t=this,e=this.getInputEventNames(),i=this;$(document).on("".concat(e.mousedown,".").concat(wr),(function(e){var n=e.target;t.timer=new X({id:"".concat(wr,"-timer"),duration:L.convertDelayToFPS(t.settings.delay),timeoutCallback:function(){i.fire(n,e)}}),J.register(t.timer)})),$(document).on(["".concat(e.mouseup,".").concat(wr),"".concat(e.mouseout,".").concat(wr),"".concat(e.mousemove,".").concat(wr)].join(" "),(function(){t.killTimer()}))},fire:function(t,e){$(t).trigger("".concat(wr),[e])},killTimer:function(){this.timer&&(this.timer.destroy(!0),this.timer=null)},getInputEventNames:function(){var t=x.features.touch,e=this.settings.mouseEvents,i=t||!e;return{mousedown:i?"touchstart":"mousedown",mouseout:i?"touchcancel":"mouseout",mouseup:i?"touchend":"mouseup",mousemove:i?"touchmove":"mousemove"}},updated:function(t){t&&(this.settings=E.mergeSettings(this.element,t,this.settings)),this.teardown(),this.init()},teardown:function(){var t=this.getInputEventNames();this.killTimer(),$(document).off(["".concat(t.mousedown,".").concat(wr),"".concat(t.mouseup,".").concat(wr),"".concat(t.mouseout,".").concat(wr),"".concat(t.mousemove,".").concat(wr)].join(" "))}};var kr=new Cr,Tr=Object.freeze({__proto__:null,About:ye,Accordion:Se,ActionSheet:Ie,ApplicationMenu:ei,Autocomplete:_e,Bar:Ni,Blockgrid:ai,Breadcrumb:hi,Bullet:xi,BusyIndicator:gi,Button:Qt,ButtonSet:te,Calendar:Pn,CalendarToolbar:Cn,Cards:vi,ColorPicker:hn,Column:Ri,CompletionChart:Ti,CompositeForm:Fn,ContextualActionPanel:Wn,Datagrid:_o,DatePicker:os,Drag:$t,Dropdown:ln,EmptyMessage:Ii,ExpandableArea:He,FontPicker:hs,FontPickerStyle:ls,FormCompact:Qo,Header:tr,Hyperlink:ne,Icon:Nt,Line:Mi,Link:Fs,ListBuilder:Ys,ListDetail:cr,ListFilter:We,ListView:Us,Lookup:Vo,MaskInput:pe,Message:ia,Modal:ta,ModuleNav:ba,ModuleNavSettings:xa,ModuleNavSwitcher:ga,MonthView:Sn,MultiSelect:aa,MultiTabs:sr,Notification:Ta,Pager:Vs,Pie:Vi,Popdown:Ia,PopupMenu:me,Radar:Ui,SearchField:Je,Slider:Na,Sparkline:$i,Spinbox:Ua,Splitter:Ya,StepChart:oo,Stepprocess:ur,SwipeAction:to,Tabs:ho,Tag:Qi,TagList:tn,Textarea:go,TimePicker:_n,Tmpl:je,Toast:Qn,Toolbar:gs,ToolbarFlex:yn,Tooltip:Vt,Tree:wo,Treemap:To,Validator:ts,VirtualScroll:sn,WeekView:Dn,Wizard:$o});return t.DOM=f,t.Editors=jo,t.Formatters=Io,t.GroupBy=Ro,t.HideFocus=C,t.Initialize=yr,t.Locale=H,t.Place=Rt,t.RenderLoopItem=X,t.Validation=Yn,t.base=G,t.breakpoints=m,t.colors=O,t.components=Tr,t.debug=n,t.defer=T,t.env=x,t.excel=Lo,t.getColorStyleSheet=function(t){return wt.getColorStyleSheet(t)},t.keyboard=et,t.longPress=kr,t.masks=se,t.modalManager=at,t.notificationManager=lt,t.number=B,t.object=P,t.personalization=wt,t.renderLoop=J,t.setColors=function(t){return wt.setColors(t)},t.setTheme=function(t){return wt.setTheme(t)},t.smoothScrollTo=k,t.string=M,t.theme=ft,t.uniqueIdCount=A,t.utils=E,t.version=s,t.xss=g,t}({});